window.VditorI18n = {
  'alignCenter': 'Centralizar',
  'alignLeft': 'Alinhar à esquerda',
  'alignRight': 'Alinhar à direita',
  'alternateText': 'Texto alternativo',
  'bold': 'Negrito',
  'both': 'Editor e visualização',
  'check': 'Lista de tarefas',
  'close': 'Fechar',
  'code': 'Bloco de código',
  'code-theme': 'Tema de bloco de código',
  'column': 'Coluna',
  'comment': 'Comentário',
  'confirm': 'Confirmar',
  'content-theme': 'Tema de conteúdo',
  'copied': 'Copiado',
  'copy': 'Copiar',
  'delete-column': 'Excluir coluna',
  'delete-row': 'Excluir linha',
  'devtools': 'Ferramentas de desenvolvimento',
  'down': 'Para baixo',
  'downloadTip': 'O navegador não suporta a função de download',
  'edit': 'Editar',
  'edit-mode': 'Alternar modo de edição',
  'emoji': 'Emoji',
  'export': 'Exportar',
  'fileTypeError': 'Tipo de arquivo incorreto',
  'footnoteRef': 'Referência de rodapé',
  'fullscreen': 'Alternar tela cheia',
  'generate': 'Gerando',
  'headings': 'Cabeçalhos',
  'heading1': 'Cabeçalho 1',
  'heading2': 'Cabeçalho 2',
  'heading3': 'Cabeçalho 3',
  'heading4': 'Cabeçalho 4',
  'heading5': 'Cabeçalho 5',
  'heading6': 'Cabeçalho 6',
  'help': 'Ajuda',
  'imageURL': 'URL da imagem',
  'indent': 'Recuar',
  'info': 'Informação',
  'inline-code': 'Código em linha',
  'insert-after': 'Inserir linha depois',
  'insert-before': 'Inserir linha antes',
  'insertColumnLeft': 'Inserir 1 à esquerda',
  'insertColumnRight': 'Inserir 1 à direita',
  'insertRowAbove': 'Inserir 1 acima',
  'insertRowBelow': 'Inserir 1 abaixo',
  'instantRendering': 'Renderização instantânea',
  'italic': 'Itálico',
  'language': 'Idioma',
  'line': 'Linha',
  'link': 'Link',
  'linkRef': 'Referência de link',
  'list': 'Lista',
  'more': 'Mais',
  'nameEmpty': 'Nome está vazio',
  'ordered-list': 'Lista ordenada',
  'outdent': 'Recuar para fora',
  'outline': 'Esboço',
  'over': 'sobre',
  'performanceTip': 'A visualização em tempo real requer ${x}ms, você pode fechá-la',
  'preview': 'Visualização',
  'quote': 'Citação',
  'record': 'Iniciar gravação/Encerrar gravação',
  'record-tip': 'O dispositivo não suporta gravação',
  'recording': 'gravando...',
  'redo': 'Refazer',
  'remove': 'Remover',
  'row': 'Linha',
  'spin': 'Girar',
  'splitView': 'Visão dividida',
  'strike': 'Riscado',
  'table': 'Tabela',
  'textIsNotEmpty': 'Texto (não vazio)',
  'title': 'Título',
  'tooltipText': 'Texto da dica de ferramenta',
  'undo': 'Desfazer',
  'up': 'Para cima',
  'update': 'Atualizar',
  'upload': 'Enviar imagem ou arquivo',
  'uploadError': 'Erro no envio',
  'uploading': 'Enviando...',
  'wysiwyg': 'WYSIWYG',
}
