window.VditorI18n = {
  'alignCenter': '置中',
  'alignLeft': '置左',
  'alignRight': '置右',
  'alternateText': '替代文字',
  'bold': '粗體',
  'both': '編輯 & 預覽',
  'check': '任務列表',
  'close': '關閉',
  'code': '代碼塊',
  'code-theme': '代碼塊主題預覽',
  'column': '欄',
  'comment': '評論',
  'confirm': '確定',
  'content-theme': '內容主題預覽',
  'copied': '已複製',
  'copy': '複製',
  'delete-column': '刪除欄',
  'delete-row': '刪除列',
  'devtools': '開發者工具',
  'down': '下',
  'downloadTip': '該瀏覽器不支持下載功能',
  'edit': '編輯',
  'edit-mode': '切換編輯模式',
  'emoji': '表情',
  'export': '匯出',
  'fileTypeError': '檔案類型不允許上傳',
  'footnoteRef': '腳註參考',
  'fullscreen': '全螢幕切換',
  'generate': '生成中',
  'headings': '標題',
  'heading1': '一級標題',
  'heading2': '二級標題',
  'heading3': '三級標題',
  'heading4': '四級標題',
  'heading5': '五級標題',
  'heading6': '六級標題',
  'help': '幫助',
  'imageURL': '圖片位址',
  'indent': '列表縮排',
  'info': '關於',
  'inline-code': '行內代碼',
  'insert-after': '末尾插入列',
  'insert-before': '起始插入列',
  'insertColumnLeft': '在左邊插入一欄',
  'insertColumnRight': '在右邊插入一欄',
  'insertRowAbove': '在上方插入一行',
  'insertRowBelow': '在下方插入一行',
  'instantRendering': '即時渲染',
  'italic': '斜體',
  'language': '語言',
  'line': '分隔線',
  'link': '連結',
  'linkRef': '連結參考',
  'list': '無序列表',
  'more': '更多',
  'nameEmpty': '文件名不能為空',
  'ordered-list': '有序列表',
  'outdent': '列表反向縮排',
  'outline': '大綱',
  'over': '超過',
  'performanceTip': '即時預覽需 ${x}ms，可點擊編輯 & 預覽按鈕進行關閉',
  'preview': '預覽',
  'quote': '引用',
  'record': '開始錄音/結束錄音',
  'record-tip': '該設備不支持錄音功能',
  'recording': '錄音中...',
  'redo': '重做',
  'remove': '刪除',
  'row': '列',
  'spin': '旋轉',
  'splitView': '分割預覽',
  'strike': '刪除線',
  'table': '表格',
  'textIsNotEmpty': '文字（不能為空）',
  'title': '標題',
  'tooltipText': '提示文字',
  'undo': '撤銷',
  'up': '上',
  'update': '更新',
  'upload': '上傳圖片或文件',
  'uploadError': '上傳錯誤',
  'uploading': '上傳中...',
  'wysiwyg': '所見即所得',
}
