window.VditorI18n = {
  'alignCenter': 'Выровнять по центру',
  'alignLeft': 'Выровнять по левому краю',
  'alignRight': 'Выровнять по правому краю',
  'alternateText': 'Альтернативный текст',
  'bold': 'Полужирный текст',
  'both': 'Редактор с предпросмотром',
  'check': 'Список задач',
  'close': 'Закрыть',
  'code': 'Блок кода',
  'code-theme': 'Тема отображения блока кода',
  'column': 'Количество столбцов',
  'comment': 'Комментарий',
  'confirm': 'Подтвердить',
  'content-theme': 'Тема отображения контента',
  'copied': 'Скопировано',
  'copy': 'Копировать',
  'delete-column': 'Удалить столбец',
  'delete-row': 'Удалить строку',
  'devtools': 'Граф документа',
  'down': 'Переместить вниз',
  'downloadTip': 'Браузер не поддерживает функции загрузки',
  'edit': 'Редактирование',
  'edit-mode': 'Переключить режим редактирования',
  'emoji': 'Смайлы',
  'export': 'Экспорт',
  'fileTypeError': 'Ошибка типа файла',
  'footnoteRef': 'Сноска',
  'fullscreen': 'Переключиться на полный экран',
  'generate': 'Генерация',
  'headings': 'Заголовки',
  'heading1': 'Заголовок первого уровня',
  'heading2': 'Заголовок второго уровня',
  'heading3': 'Трехуровневый заголовок',
  'heading4': 'Заголовок 4',
  'heading5': 'Пятиуровневый заголовок',
  'heading6': 'Шестиуровневый заголовок',
  'help': 'Помощь',
  'imageURL': 'Ссылка на картинку',
  'indent': 'Отступ',
  'info': 'Информация о редакторе',
  'inline-code': 'Встроенный код ',
  'insert-after': 'Вставить линию после этой строки',
  'insert-before': 'Вставить линию перед этой строкой',
  'insertColumnLeft': 'Вставить столбец слева',
  'insertColumnRight': 'Вставить столбец справа',
  'insertRowAbove': 'Вставить строку сверху',
  'insertRowBelow': 'Вставить строку снизу',
  'instantRendering': 'Мгновенный рендеринг',
  'italic': 'Курсив',
  'language': 'Синтаксис',
  'line': 'Линия',
  'link': 'Ссылка',
  'linkRef': 'Вставить ссылку',
  'list': 'Список',
  'more': 'Дополнительно',
  'nameEmpty': 'Имя пустое',
  'ordered-list': 'Упорядоченный список',
  'outdent': 'Выступ',
  'outline': 'Содержание',
  'over': 'Над',
  'performanceTip': 'Предварительный просмотр в реальном времени требует ${x}мс, вы можете закрыть его',
  'preview': 'Предпросмотр',
  'quote': 'Цитата',
  'record': 'Начать запись/Завершить запись',
  'record-tip': 'Устройство не поддерживает запись',
  'recording': 'Запись...',
  'redo': 'Вернуть',
  'remove': 'Удалить',
  'row': 'Количество строк',
  'spin': 'Прокрутка',
  'splitView': 'Разделенный просмотр',
  'strike': 'Зачеркнутый текст',
  'table': 'Таблица',
  'textIsNotEmpty': 'Текст (непустой)',
  'title': 'Заголовок',
  'tooltipText': 'Текст всплывающей подсказки',
  'undo': 'Отменить',
  'up': 'Переместить вверх',
  'update': 'Обновить',
  'upload': 'Загрузить изображение или файл',
  'uploadError': 'Ошибка загрузки',
  'uploading': 'Загрузка...',
  'wysiwyg': 'Редактор WYSIWYG',
}
