window.VditorI18n = {
  'alignCenter': '中央',
  'alignLeft': '左側',
  'alignRight': '右側',
  'alternateText': 'イメージタグ',
  'bold': '太く',
  'both': 'エディター & プレビュー',
  'check': 'チェックリスト',
  'close': '閉じる',
  'code': 'コードブロック挿入',
  'code-theme': 'コードブロックテーマ',
  'column': '行列',
  'comment': 'コメント',
  'confirm': '確認',
  'content-theme': 'コンテンツテーマ',
  'copied': 'コピー完了',
  'copy': 'コピー',
  'delete-column': '列 消去',
  'delete-row': '行 消去',
  'devtools': '開発ツール',
  'down': '下げる',
  'downloadTip': 'ブラウザがダウンロード機能をサポートしていません。',
  'edit': '修正',
  'edit-mode': '編集モード',
  'emoji': '絵文字',
  'export': 'エクスポート',
  'fileTypeError': 'サポートしていません。',
  'footnoteRef': '脚注参照',
  'fullscreen': 'フルスクリーン',
  'generate': '作成する',
  'headings': '見出しの大きさ',
  'heading1': '見出し1',
  'heading2': '見出し2',
  'heading3': '見出し3',
  'heading4': '見出し4',
  'heading5': '見出し5',
  'heading6': '見出し6',
  'help': 'ヘルプ',
  'imageURL': 'イメージ URL',
  'indent': '字下げ',
  'info': '情報',
  'inline-code': 'インラインコード',
  'insert-after': 'ブロックの後ろに入力',
  'insert-before': 'ブロックの前に入力',
  'insertColumnLeft': '左側に列を挿入',
  'insertColumnRight': '右側に列を挿入',
  'insertRowAbove': '上に行を挿入',
  'insertRowBelow': '下に行を挿入',
  'instantRendering': 'インスタントレンダリング',
  'italic': '斜体',
  'language': '言語',
  'line': '段落分割',
  'link': 'リンク',
  'linkRef': 'リンク参照',
  'list': 'リスト',
  'more': '詳しく見る',
  'nameEmpty': '名前が入力されていません。',
  'ordered-list': '順序のあるリスト',
  'outdent': 'ぶら下げインデント',
  'outline': '概要',
  'over': 'オーバー',
  'performanceTip': 'リアルタイムプレビューには、${x}msが必要でエディター/プレビューボタンをクリックして閉じる事が出来ます。',
  'preview': 'プレビュー',
  'quote': '引用段落',
  'record': '録音開始/録音終了',
  'record-tip': '録音がサポートされていません。',
  'recording': '録音中...',
  'redo': '戻る',
  'remove': '消去',
  'row': '列',
  'spin': 'スピン',
  'splitView': 'マークダウン',
  'strike': '取り消し線',
  'table': '表 挿入',
  'textIsNotEmpty': 'テキスト(空白は不可)',
  'title': '題名',
  'tooltipText': 'ツールチップ',
  'undo': '取り消す',
  'up': '上げる',
  'update': 'アップデート',
  'upload': 'イメージをアップロードする',
  'uploadError': 'アップロード失敗',
  'uploading': 'アップロード中',
  'wysiwyg': 'ウィジウィグ',
}
