window.VditorI18n = {
  'alignCenter': 'Centrera',
  'alignLeft': 'Vänsterställ',
  'alignRight': 'Högerställ',
  'alternateText': 'Alternativ text',
  'bold': 'Fet',
  'both': 'editera & granska',
  'check': 'Att göra lista',
  'close': 'Stäng',
  'code': 'Kodblock',
  'code-theme': 'Granska tema för kodblock',
  'column': 'Kolumn',
  'comment': 'Kommentera',
  'confirm': 'Bekräfta',
  'content-theme': 'Granska tema för innehåll',
  'copied': 'Kopierad',
  'copy': 'Kopiera',
  'delete-column': 'Radera Kolumn',
  'delete-row': 'Radera Rad',
  'devtools': 'Utveckingsverktyg',
  'down': 'Ner',
  'downloadTip': 'Webläsaren kan ej hantera funktionen nedladdning',
  'edit': 'Redigera',
  'edit-mode': 'Skfta Redigeringsläge',
  'emoji': 'Emoji',
  'export': 'Exportera',
  'fileTypeError': 'Fel filtyp',
  'footnoteRef': 'Fotnotreferens',
  'fullscreen': 'Skifta fullskärmsläge',
  'generate': 'Genererar',
  'headings': 'Rubriker',
  'heading1': 'Rubriknivå 1',
  'heading2': 'Rubriknivå 2',
  'heading3': 'Rubriknivå 3',
  'heading4': 'Rubriknivå 4',
  'heading5': 'Rubriknivå 5',
  'heading6': 'Rubriknivå 6',
  'help': 'Hjälp',
  'imageURL': 'Bildlänk (URL)',
  'indent': 'Dra in',
  'info': 'Info',
  'inline-code': 'Dra in kod',
  'insert-after': 'Infoga rad efter',
  'insert-before': 'Infoga rad innan',
  'insertColumnLeft': 'Infoga kolumn till vänster',
  'insertColumnRight': 'Infoga kolumn till höger',
  'insertRowAbove': 'Infoga rad ovan',
  'insertRowBelow': 'Infoga rad under',
  'instantRendering': 'Omedelbar Rendering',
  'italic': 'Kursiv',
  'language': 'Språk',
  'line': 'Linje',
  'link': 'Länk',
  'linkRef': 'Länk Ref',
  'list': 'Lista',
  'more': 'Mer',
  'nameEmpty': 'Namn saknas',
  'ordered-list': 'Ordnad Lista',
  'outdent': 'Dra ut',
  'outline': 'Disponera',
  'over': 'över',
  'performanceTip': 'Granskning i realtid kräver ${x}ms, du kan stänga det',
  'preview': 'Granska',
  'quote': 'Citera',
  'record': 'Börja inspelning/Avsluta inspelning',
  'record-tip': 'Enheten stödjer inte inspelning',
  'recording': 'spelar in...',
  'redo': 'Gör om',
  'remove': 'Ta bort',
  'row': 'Rad',
  'spin': 'Snurra',
  'splitView': 'Splittad Vy',
  'strike': 'Genomstruket',
  'table': 'Tabell',
  'textIsNotEmpty': 'text(ej tom)',
  'title': 'Titel',
  'tooltipText': 'Tooltip text',
  'undo': 'Ångra',
  'up': 'Upp',
  'update': 'Uppdatera',
  'upload': 'Ladda upp bild eller fil',
  'uploadError': 'uppladdningsfel',
  'uploading': 'laddar upp...',
  'wysiwyg': 'WYSIWYG',
}
