window.VditorI18n = {
  'alignCenter': '가운데',
  'alignLeft': '왼쪽',
  'alignRight': '오른쪽',
  'alternateText': '이미지 태그',
  'bold': '굵게',
  'both': '에디터 & 미리보기',
  'check': '체크박스',
  'close': '닫기',
  'code': '코드블럭삽입',
  'code-theme': '코드블럭테마',
  'column': '행',
  'comment': '코멘트',
  'confirm': '확인',
  'content-theme': '컨텐츠테마',
  'copied': '복사완료',
  'copy': '복사',
  'delete-column': '열 삭제',
  'delete-row': '행 삭제',
  'devtools': '개발툴',
  'down': '다운',
  'downloadTip': '브라우저가 다운로드 기능을 지원하지 않습니다',
  'edit': '수정',
  'edit-mode': '편집모드',
  'emoji': '이모지',
  'export': '내보내기',
  'fileTypeError': '지원하지않습니다.',
  'footnoteRef': '각주참조',
  'fullscreen': '전체화면',
  'generate': '생성',
  'headings': '제목크기',
  'heading1': '첫 번째 수준 제목',
  'heading2': '두 번째 수준 제목',
  'heading3': '3 단계 제목',
  'heading4': '제목 4',
  'heading5': '5 단계 제목',
  'heading6': '6 단계 제목',
  'help': '도움말',
  'imageURL': '이미지 URL',
  'indent': '들여쓰기',
  'info': '정보',
  'inline-code': '인라인코드',
  'insert-after': '블락 뒤로 입력',
  'insert-before': '블락 앞으로 입력',
  'insertColumnLeft': '왼쪽에 열 삽입',
  'insertColumnRight': '오른쪽에 열 삽입',
  'insertRowAbove': '위에 행 삽입',
  'insertRowBelow': '아래에 행 삽입',
  'instantRendering': '타이포라',
  'italic': '기울임꼴',
  'language': '언어',
  'line': '문단나눔',
  'link': '링크',
  'linkRef': '링크 참조',
  'list': '순서없는 목록',
  'more': '더보기',
  'nameEmpty': '이름이 비어있습니다.',
  'ordered-list': '순서있는 목록',
  'outdent': '내어쓰기',
  'outline': '개요',
  'over': '오버',
  'performanceTip': '실시간 미리보기에는 ${x}ms가 필요하며 에디터/미리보기 버튼을 클릭하여 닫을 수 있습니다.',
  'preview': '미리보기',
  'quote': '인용단락',
  'record': '녹음시작/녹음종료',
  'record-tip': '녹음을 지원하지 않습니다.',
  'recording': '녹음중...',
  'redo': '되돌리기',
  'remove': '삭제',
  'row': '열',
  'spin': '회전',
  'splitView': '마크다운',
  'strike': '취소선',
  'table': '표삽입',
  'textIsNotEmpty': '텍스트(no empty)',
  'title': '표제',
  'tooltipText': '툴팁',
  'undo': '취소하기',
  'up': '위로',
  'update': '업데이트',
  'upload': '이미지 업로드하기',
  'uploadError': '업로드 실패',
  'uploading': '업로드중...',
  'wysiwyg': '위지위그',
}
