window.VditorI18n = {
  'alignCenter': 'Center',
  'alignLeft': 'Left',
  'alignRight': 'Right',
  'alternateText': 'Alternate text',
  'bold': 'Bold',
  'both': 'editor & preview',
  'check': 'Task List',
  'close': 'Close',
  'code': 'Code Block',
  'code-theme': 'Code Block Theme Preview',
  'column': 'Column',
  'comment': 'Comment',
  'confirm': 'Confirm',
  'content-theme': 'Content Theme Preview',
  'copied': 'Copied',
  'copy': 'Copy',
  'delete-column': 'Delete Row',
  'delete-row': 'Delete Column',
  'devtools': 'DevTools',
  'down': 'Down',
  'downloadTip': 'The browser does not support the download function',
  'edit': 'Edit',
  'edit-mode': 'Toggle Edit Mode',
  'emoji': 'Emoji',
  'export': 'Export',
  'fileTypeError': 'file type is error',
  'footnoteRef': 'Footnote Ref',
  'fullscreen': 'Toggle Fullscreen',
  'generate': 'Generating',
  'headings': 'Headings',
  'heading1': 'Heading 1',
  'heading2': 'Heading 2',
  'heading3': 'Heading 3',
  'heading4': 'Heading 4',
  'heading5': 'Heading 5',
  'heading6': 'Heading 6',
  'help': 'Help',
  'imageURL': 'image URL',
  'indent': 'Indent',
  'info': 'Info',
  'inline-code': 'Inline Code',
  'insert-after': 'Insert line after',
  'insert-before': 'Insert line before',
  'insertColumnLeft': 'Insert 1 left',
  'insertColumnRight': 'Insert 1 right',
  'insertRowAbove': 'Insert 1 above',
  'insertRowBelow': 'Insert 1 below',
  'instantRendering': 'Instant Rendering',
  'italic': 'Italic',
  'language': 'Language',
  'line': 'Line',
  'link': 'Link',
  'linkRef': 'Link Ref',
  'list': 'List',
  'more': 'More',
  'nameEmpty': 'Name is empty',
  'ordered-list': 'Order List',
  'outdent': 'Outdent',
  'outline': 'Outline',
  'over': 'over',
  'performanceTip': 'Real-time preview requires ${x}ms, you can close it',
  'preview': 'Preview',
  'quote': 'Quote',
  'record': 'Start Record/End Record',
  'record-tip': 'The device does not support recording',
  'recording': 'recording...',
  'redo': 'Redo',
  'remove': 'Remove',
  'row': 'Row',
  'spin': 'Spin',
  'splitView': 'Split View',
  'strike': 'Strike',
  'table': 'Table',
  'textIsNotEmpty': 'text(no empty)',
  'title': 'Title',
  'tooltipText': 'Tooltip text',
  'undo': 'Undo',
  'up': 'Up',
  'update': 'Update',
  'upload': 'Upload image or file',
  'uploadError': 'upload error',
  'uploading': 'uploading...',
  'wysiwyg': 'WYSIWYG',
}
