window.VditorI18n = {
  'alignCenter': 'Centré',
  'alignLeft': 'Gauche',
  'alignRight': 'Droite',
  'alternateText': 'Texte de remplacement',
  'bold': 'Gras',
  'both': 'éditeur & prévisualisation',
  'check': 'Liste de tâches',
  'close': 'Fermer',
  'code': 'Bloc de code',
  'code-theme': 'Thème pour le bloc de code',
  'column': 'Colonne',
  'comment': 'Commentaire',
  'confirm': 'Confirmez',
  'content-theme': 'Thème pour le contenu',
  'copied': 'Copié',
  'copy': 'Copier',
  'delete-column': 'Enlever ligne',
  'delete-row': 'Enlever colonne',
  'devtools': 'Outils de dev',
  'down': 'Bas',
  'downloadTip': 'Le navigateur ne permet pas le téléversement',
  'edit': 'Editer',
  'edit-mode': 'Changer le mode d\'édition',
  'emoji': 'Emoji',
  'export': 'Export',
  'fileTypeError': 'erreur de type de fichier',
  'footnoteRef': 'Ref. notes de pied de page',
  'fullscreen': 'Bascule en plein écran',
  'generate': 'Création',
  'headings': 'Titres',
  'heading1': 'Titre 1',
  'heading2': 'Titre 2',
  'heading3': 'Titre 3',
  'heading4': 'Titre 4',
  'heading5': 'Titre 5',
  'heading6': 'Titre 6',
  'help': 'Aide',
  'imageURL': 'URL de l\'image',
  'indent': 'Augmenter le retrait',
  'info': 'Info',
  'inline-code': 'Code en ligne',
  'insert-after': 'Insérer une ligne après',
  'insert-before': 'Insérer une ligne avant',
  'insertColumnLeft': 'Insérer col. à gauche',
  'insertColumnRight': 'Insérer col. à droite',
  'insertRowAbove': 'Insérer 1 au dessus',
  'insertRowBelow': 'Insérer 1 en dessous',
  'instantRendering': 'Aperçu instantané',
  'italic': 'Italique',
  'language': 'Langage',
  'line': 'Ligne',
  'link': 'Lien',
  'linkRef': 'Réf. Lien',
  'list': 'Liste',
  'more': 'Plus',
  'nameEmpty': 'Nom est vide',
  'ordered-list': 'Liste ordonnée',
  'outdent': 'Réduire le retrait',
  'outline': 'Table des matières',
  'over': 'survol',
  'performanceTip': 'La prévisualisation temps réel prend ${x}ms, vous pouvez fermer',
  'preview': 'Prévisualisation',
  'quote': 'Citation',
  'record': 'Démarrer/Stopper l\'enregistrement',
  'record-tip': 'L\'appareil ne supporte pas l\'enregistrement',
  'recording': 'Enregistrement en cours...',
  'redo': 'Rétablir',
  'remove': 'Enlever',
  'row': 'Ligne',
  'spin': 'Rotation',
  'splitView': 'Vue scindée',
  'strike': 'Barré',
  'table': 'Table',
  'textIsNotEmpty': 'texte(pas vide)',
  'title': 'Titre',
  'tooltipText': 'Texte au survol',
  'undo': 'Annuler',
  'up': 'Haut',
  'update': 'Mettre à jour',
  'upload': 'Téléverser image ou fichier',
  'uploadError': 'erreur de téléversement',
  'uploading': 'Téléversement en cours...',
  'wysiwyg': 'WYSIWYG',
}
