sre.BrowserUtil.mapsForIE = {
  "en/functions/algebra.js":
    [{"locale": "en"}, {
      "category": "Algebra",
      "mappings": {"default": {"default": "degree"}},
      "key": "deg",
      "names": ["deg"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "determinant"}, "mathspeak": {"default": "det"}},
      "key": "det",
      "names": ["det"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "dimension"}},
      "key": "dim",
      "names": ["dim"]
    }, {
      "category": "Algebra",
      "mappings": {
        "default": {"default": "homomorphism"},
        "mathspeak": {"default": "hom"},
        "clearspeak": {"default": "hom"}
      },
      "key": "hom",
      "names": ["hom", "Hom"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "kernel"}},
      "key": "ker",
      "names": ["ker"]
    }, {"category": "Algebra", "mappings": {"default": {"default": "trace"}}, "key": "Tr", "names": ["Tr", "tr"]}]
  ,
  "en/functions/elementary.js":
    [{"locale": "en"}, {
      "category": "Logarithm",
      "mappings": {"default": {"default": "log"}},
      "key": "log",
      "names": ["log"]
    }, {
      "category": "Logarithm",
      "mappings": {
        "default": {"default": "natural log"},
        "mathspeak": {"default": "ln"},
        "clearspeak": {"default": "l n", "Log_LnAsNaturalLog": "natural log"}
      },
      "key": "ln",
      "names": ["ln"]
    }, {
      "category": "Logarithm",
      "mappings": {"default": {"default": "log base 10"}},
      "key": "lg",
      "names": ["lg"]
    }, {
      "category": "Elementary",
      "mappings": {
        "default": {"default": "exponential"},
        "mathspeak": {"default": "exp"},
        "clearspeak": {"default": "exp"}
      },
      "key": "exp",
      "names": ["exp", "expt"]
    }, {
      "category": "Elementary",
      "mappings": {
        "default": {"default": "greatest common divisor"},
        "mathspeak": {"default": "gcd"},
        "clearspeak": {"default": "gcd"}
      },
      "key": "gcd",
      "names": ["gcd"]
    }, {
      "category": "Elementary",
      "mappings": {
        "default": {"default": "least common multiple"},
        "mathspeak": {"default": "lcm"},
        "clearspeak": {"default": "lcm"}
      },
      "key": "lcm",
      "names": ["lcm"]
    }, {
      "category": "Complex",
      "mappings": {
        "default": {"default": "argument"},
        "mathspeak": {"default": "arg"},
        "clearspeak": {"default": "arg"}
      },
      "key": "arg",
      "names": ["arg"]
    }, {
      "category": "Complex",
      "mappings": {
        "default": {"default": "imaginary part"},
        "mathspeak": {"default": "im"},
        "clearspeak": {"default": "imaginary"}
      },
      "key": "im",
      "names": ["im"]
    }, {
      "category": "Complex",
      "mappings": {
        "default": {"default": "real part"},
        "mathspeak": {"default": "re"},
        "clearspeak": {"default": "real"}
      },
      "key": "re",
      "names": ["re"]
    }, {
      "category": "Limits",
      "mappings": {
        "default": {"default": "infimum"},
        "mathspeak": {"default": "inf"},
        "clearspeak": {"default": "inf"}
      },
      "key": "inf",
      "names": ["inf"]
    }, {
      "category": "Limits",
      "mappings": {
        "default": {"default": "limit"},
        "mathspeak": {"default": "limit"},
        "clearspeak": {"default": "lim"}
      },
      "key": "lim",
      "names": ["lim"]
    }, {
      "category": "Limits",
      "mappings": {
        "default": {"default": "limit inferior"},
        "mathspeak": {"default": "liminf"},
        "clearspeak": {"default": "liminf"}
      },
      "key": "liminf",
      "names": ["lim inf", "liminf"]
    }, {
      "category": "Limits",
      "mappings": {
        "default": {"default": "limit superior"},
        "mathspeak": {"default": "limsup"},
        "clearspeak": {"default": "limsup"}
      },
      "key": "limsup",
      "names": ["lim sup", "limsup"]
    }, {
      "category": "Limits",
      "mappings": {
        "default": {"default": "maximum"},
        "mathspeak": {"default": "max"},
        "clearspeak": {"default": "max"}
      },
      "key": "max",
      "names": ["max"]
    }, {
      "category": "Limits",
      "mappings": {
        "default": {"default": "minimum"},
        "mathspeak": {"default": "min"},
        "clearspeak": {"default": "min"}
      },
      "key": "min",
      "names": ["min"]
    }, {
      "category": "Limits",
      "mappings": {
        "default": {"default": "supremum"},
        "mathspeak": {"default": "sup"},
        "clearspeak": {"default": "sup"}
      },
      "key": "sup",
      "names": ["sup"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "colimit"}},
      "key": "injlim",
      "names": ["injlim", "inj lim"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "projective limit"}},
      "key": "projlim",
      "names": ["projlim", "proj lim"]
    }, {
      "category": "Elementary",
      "mappings": {"default": {"default": "modulo"}, "mathspeak": {"default": "mod"}, "clearspeak": {"default": "mod"}},
      "key": "mod",
      "names": ["mod"]
    }, {"category": "Probability", "mappings": {"default": {"default": "probability"}}, "key": "Pr", "names": ["Pr"]}]
  ,
  "en/functions/hyperbolic.js":
    [{"locale": "en"}, {
      "category": "Hyperbolic",
      "mappings": {"default": {"default": "hyperbolic cosine"}},
      "key": "cosh",
      "names": ["cosh"]
    }, {
      "category": "Hyperbolic",
      "mappings": {"default": {"default": "hyperbolic cotangent"}},
      "key": "coth",
      "names": ["coth"]
    }, {
      "category": "Hyperbolic",
      "mappings": {"default": {"default": "hyperbolic cosecant"}},
      "key": "csch",
      "names": ["csch"]
    }, {
      "category": "Hyperbolic",
      "mappings": {"default": {"default": "hyperbolic secant"}},
      "key": "sech",
      "names": ["sech"]
    }, {
      "category": "Hyperbolic",
      "mappings": {"default": {"default": "hyperbolic sine"}},
      "key": "sinh",
      "names": ["sinh"]
    }, {
      "category": "Hyperbolic",
      "mappings": {"default": {"default": "hyperbolic tangent"}},
      "key": "tanh",
      "names": ["tanh"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "area hyperbolic cosine"}},
      "key": "arcosh",
      "names": ["arcosh", "arccosh"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "area hyperbolic cotangent"}},
      "key": "arcoth",
      "names": ["arcoth", "arccoth"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "area hyperbolic cosecant"}},
      "key": "arcsch",
      "names": ["arcsch", "arccsch"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "area hyperbolic secant"}},
      "key": "arsech",
      "names": ["arsech", "arcsech"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "area hyperbolic sine"}},
      "key": "arsinh",
      "names": ["arsinh", "arcsinh"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "area hyperbolic tangent"}},
      "key": "artanh",
      "names": ["artanh", "arctanh"]
    }]
  ,
  "en/functions/trigonometry.js":
    [{"locale": "en"}, {
      "category": "Trigonometric",
      "mappings": {"default": {"default": "cosine"}},
      "key": "cos",
      "names": ["cos", "cosine"]
    }, {
      "category": "Trigonometric",
      "mappings": {"default": {"default": "cotangent"}},
      "key": "cot",
      "names": ["cot"]
    }, {
      "category": "Trigonometric",
      "mappings": {"default": {"default": "cosecant"}},
      "key": "csc",
      "names": ["csc"]
    }, {
      "category": "Trigonometric",
      "mappings": {"default": {"default": "secant"}},
      "key": "sec",
      "names": ["sec"]
    }, {
      "category": "Trigonometric",
      "mappings": {"default": {"default": "sine"}},
      "key": "sin",
      "names": ["sin", "sine"]
    }, {
      "category": "Trigonometric",
      "mappings": {"default": {"default": "tangent"}},
      "key": "tan",
      "names": ["tan"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "arc cosine"}, "clearspeak": {"Trig_TrigInverse": "cosine inverse"}},
      "key": "arccos",
      "names": ["arccos"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "arc cotangent"}, "clearspeak": {"Trig_TrigInverse": "cotangent inverse"}},
      "key": "arccot",
      "names": ["arccot"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "arc cosecant"}, "clearspeak": {"Trig_TrigInverse": "cosecant inverse"}},
      "key": "arccsc",
      "names": ["arccsc"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "arc secant"}, "clearspeak": {"Trig_TrigInverse": "secant inverse"}},
      "key": "arcsec",
      "names": ["arcsec"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "arc sine"}, "clearspeak": {"Trig_TrigInverse": "sine inverse"}},
      "key": "arcsin",
      "names": ["arcsin"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "arc tangent"}, "clearspeak": {"Trig_TrigInverse": "tangent inverse"}},
      "key": "arctan",
      "names": ["arctan"]
    }]
  ,
  "en/symbols/digits_rest.js":
    [{"locale": "en"}, {
      "category": "No",
      "mappings": {
        "default": {"default": "squared"},
        "mathspeak": {"default": "squared"},
        "clearspeak": {"default": "squared"}
      },
      "key": "00B2"
    }, {
      "category": "No",
      "mappings": {
        "default": {"default": "cubed"},
        "mathspeak": {"default": "cubed"},
        "clearspeak": {"default": "cubed"}
      },
      "key": "00B3"
    }, {"category": "No", "mappings": {"default": {"default": "one quarter"}}, "key": "00BC"}, {
      "category": "No",
      "mappings": {"default": {"default": "one half"}},
      "key": "00BD"
    }, {"category": "No", "mappings": {"default": {"default": "three quarters"}}, "key": "00BE"}, {
      "category": "No",
      "mappings": {"default": {"default": "one seventh"}},
      "key": "2150"
    }, {"category": "No", "mappings": {"default": {"default": "one ninth"}}, "key": "2151"}, {
      "category": "No",
      "mappings": {"default": {"default": "one tenth"}},
      "key": "2152"
    }, {"category": "No", "mappings": {"default": {"default": "one third"}}, "key": "2153"}, {
      "category": "No",
      "mappings": {"default": {"default": "two thirds"}},
      "key": "2154"
    }, {"category": "No", "mappings": {"default": {"default": "one fifth"}}, "key": "2155"}, {
      "category": "No",
      "mappings": {"default": {"default": "two fifths"}},
      "key": "2156"
    }, {"category": "No", "mappings": {"default": {"default": "three fifths"}}, "key": "2157"}, {
      "category": "No",
      "mappings": {"default": {"default": "four fifths"}},
      "key": "2158"
    }, {"category": "No", "mappings": {"default": {"default": "one sixth"}}, "key": "2159"}, {
      "category": "No",
      "mappings": {"default": {"default": "five sixths"}},
      "key": "215A"
    }, {"category": "No", "mappings": {"default": {"default": "one eighth"}}, "key": "215B"}, {
      "category": "No",
      "mappings": {"default": {"default": "three eighths"}},
      "key": "215C"
    }, {"category": "No", "mappings": {"default": {"default": "five eighths"}}, "key": "215D"}, {
      "category": "No",
      "mappings": {"default": {"default": "seven eighths"}},
      "key": "215E"
    }, {"category": "No", "mappings": {"default": {"default": "numerator one"}}, "key": "215F"}, {
      "category": "No",
      "mappings": {"default": {"default": "zero thirds"}},
      "key": "2189"
    }, {
      "category": "No",
      "mappings": {"default": {"default": "circled ten on black square"}},
      "key": "3248"
    }, {
      "category": "No",
      "mappings": {"default": {"default": "circled twenty on black square"}},
      "key": "3249"
    }, {
      "category": "No",
      "mappings": {"default": {"default": "circled thirty on black square"}},
      "key": "324A"
    }, {
      "category": "No",
      "mappings": {"default": {"default": "circled forty on black square"}},
      "key": "324B"
    }, {
      "category": "No",
      "mappings": {"default": {"default": "circled fifty on black square"}},
      "key": "324C"
    }, {
      "category": "No",
      "mappings": {"default": {"default": "circled sixty on black square"}},
      "key": "324D"
    }, {
      "category": "No",
      "mappings": {"default": {"default": "circled seventy on black square"}},
      "key": "324E"
    }, {"category": "No", "mappings": {"default": {"default": "circled eighty on black square"}}, "key": "324F"}]
  ,
  "en/symbols/greek-rest.js":
    [{"locale": "en"}, {
      "category": "Lu",
      "key": "0394",
      "mappings": {"clearspeak": {"default": "triangle", "TriangleSymbol_Delta": "cap Delta"}}
    }]
  ,
  "en/symbols/greek-scripts.js":
    [{"locale": "en"}, {
      "category": "Ll",
      "key": "1D26",
      "mappings": {"default": {"default": "small cap Gamma"}, "mathspeak": {"default": "small upper Gamma"}}
    }, {
      "category": "Ll",
      "key": "1D27",
      "mappings": {"default": {"default": "small cap Lamda"}, "mathspeak": {"default": "small upper Lamda"}}
    }, {
      "category": "Ll",
      "key": "1D28",
      "mappings": {"default": {"default": "small cap Pi"}, "mathspeak": {"default": "small upper Pi"}}
    }, {
      "category": "Ll",
      "key": "1D29",
      "mappings": {"default": {"default": "small cap Rho"}, "mathspeak": {"default": "small upper Rho"}}
    }, {
      "category": "Ll",
      "key": "1D2A",
      "mappings": {"default": {"default": "small cap Psi"}, "mathspeak": {"default": "small upper Psi"}}
    }, {"category": "Lm", "key": "1D5E", "mappings": {"default": {"default": "superscript gamma"}}}, {
      "category": "Lm",
      "key": "1D60",
      "mappings": {"default": {"default": "superscript phi"}}
    }, {"category": "Lm", "key": "1D66", "mappings": {"default": {"default": "subscript beta"}}}, {
      "category": "Lm",
      "key": "1D67",
      "mappings": {"default": {"default": "subscript gamma"}}
    }, {"category": "Lm", "key": "1D68", "mappings": {"default": {"default": "subscript rho"}}}, {
      "category": "Lm",
      "key": "1D69",
      "mappings": {"default": {"default": "subscript phi"}}
    }, {"category": "Lm", "key": "1D6A", "mappings": {"default": {"default": "subscript chi"}}}]
  ,
  "en/symbols/greek-symbols.js":
    [{"locale": "en"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "beta"}},
      "key": "03D0"
    }, {"category": "Ll", "mappings": {"default": {"default": "kai"}}, "key": "03D7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "reversed epsilon"}},
      "key": "03F6"
    }, {
      "category": "Lu",
      "mappings": {"default": {"default": "bold cap Digamma"}, "mathspeak": {"default": "bold upper Digamma"}},
      "key": "1D7CA"
    }, {"category": "Ll", "mappings": {"default": {"default": "bold digamma"}}, "key": "1D7CB"}]
  ,
  "en/symbols/hebrew_letters.js":
    [{"locale": "en"}, {
      "category": "Lo",
      "mappings": {"default": {"default": "first transfinite cardinal", "alternative": "alef"}},
      "key": "2135"
    }, {
      "category": "Lo",
      "mappings": {"default": {"default": "second transfinite cardinal", "alternative": "bet"}},
      "key": "2136"
    }, {
      "category": "Lo",
      "mappings": {"default": {"default": "third transfinite cardinal", "alternative": "gimel"}},
      "key": "2137"
    }, {
      "category": "Lo",
      "mappings": {"default": {"default": "fourth transfinite cardinal", "alternative": "dalet"}},
      "key": "2138"
    }]
  ,
  "en/symbols/latin-lower-double-accent.js":
    [{"locale": "en"}, {
      "category": "Ll",
      "key": "01D6",
      "mappings": {"default": {"default": "u double overdot overbar"}}
    }, {
      "category": "Ll",
      "key": "01D8",
      "mappings": {"default": {"default": "u double overdot acute"}}
    }, {
      "category": "Ll",
      "key": "01DA",
      "mappings": {"default": {"default": "u double overdot caron"}}
    }, {
      "category": "Ll",
      "key": "01DC",
      "mappings": {"default": {"default": "u double overdot grave"}}
    }, {
      "category": "Ll",
      "key": "01DF",
      "mappings": {"default": {"default": "a double overdot overbar"}}
    }, {"category": "Ll", "key": "01E1", "mappings": {"default": {"default": "a overdot overbar"}}}, {
      "category": "Ll",
      "key": "01ED",
      "mappings": {"default": {"default": "o ogonek overbar"}}
    }, {"category": "Ll", "key": "01FB", "mappings": {"default": {"default": "a ring above acute"}}}, {
      "category": "Ll",
      "key": "022B",
      "mappings": {"default": {"default": "o double overdot overbar"}}
    }, {"category": "Ll", "key": "022D", "mappings": {"default": {"default": "o tilde overbar"}}}, {
      "category": "Ll",
      "key": "0231",
      "mappings": {"default": {"default": "o overdot overbar"}}
    }, {"category": "Ll", "key": "1E09", "mappings": {"default": {"default": "c cedilla acute"}}}, {
      "category": "Ll",
      "key": "1E15",
      "mappings": {"default": {"default": "e overbar grave"}}
    }, {"category": "Ll", "key": "1E17", "mappings": {"default": {"default": "e overbar acute"}}}, {
      "category": "Ll",
      "key": "1E1D",
      "mappings": {"default": {"default": "e cedilla breve"}}
    }, {
      "category": "Ll",
      "key": "1E2F",
      "mappings": {"default": {"default": "i double overdot acute"}}
    }, {"category": "Ll", "key": "1E39", "mappings": {"default": {"default": "l underdot overbar"}}}, {
      "category": "Ll",
      "key": "1E4D",
      "mappings": {"default": {"default": "o tilde acute"}}
    }, {
      "category": "Ll",
      "key": "1E4F",
      "mappings": {"default": {"default": "o tilde double overdot"}}
    }, {"category": "Ll", "key": "1E51", "mappings": {"default": {"default": "o overbar grave"}}}, {
      "category": "Ll",
      "key": "1E53",
      "mappings": {"default": {"default": "o overbar acute"}}
    }, {"category": "Ll", "key": "1E5D", "mappings": {"default": {"default": "r underdot overbar"}}}, {
      "category": "Ll",
      "key": "1E65",
      "mappings": {"default": {"default": "s acute overdot"}}
    }, {"category": "Ll", "key": "1E67", "mappings": {"default": {"default": "s caron overdot"}}}, {
      "category": "Ll",
      "key": "1E69",
      "mappings": {"default": {"default": "s underdot overdot"}}
    }, {"category": "Ll", "key": "1E79", "mappings": {"default": {"default": "u tilde acute"}}}, {
      "category": "Ll",
      "key": "1E7B",
      "mappings": {"default": {"default": "u overbar double overdot"}}
    }, {"category": "Ll", "key": "1EA5", "mappings": {"default": {"default": "a hat acute"}}}, {
      "category": "Ll",
      "key": "1EA7",
      "mappings": {"default": {"default": "a hat grave"}}
    }, {"category": "Ll", "key": "1EA9", "mappings": {"default": {"default": "a hat hook above"}}}, {
      "category": "Ll",
      "key": "1EAB",
      "mappings": {"default": {"default": "a hat tilde"}}
    }, {"category": "Ll", "key": "1EAD", "mappings": {"default": {"default": "a hat underdot"}}}, {
      "category": "Ll",
      "key": "1EAF",
      "mappings": {"default": {"default": "a breve acute"}}
    }, {"category": "Ll", "key": "1EB1", "mappings": {"default": {"default": "a breve grave"}}}, {
      "category": "Ll",
      "key": "1EB3",
      "mappings": {"default": {"default": "a breve hook above"}}
    }, {"category": "Ll", "key": "1EB5", "mappings": {"default": {"default": "a breve tilde"}}}, {
      "category": "Ll",
      "key": "1EB7",
      "mappings": {"default": {"default": "a breve underdot"}}
    }, {"category": "Ll", "key": "1EBF", "mappings": {"default": {"default": "e hat acute"}}}, {
      "category": "Ll",
      "key": "1EC1",
      "mappings": {"default": {"default": "e hat grave"}}
    }, {"category": "Ll", "key": "1EC3", "mappings": {"default": {"default": "e hat hook above"}}}, {
      "category": "Ll",
      "key": "1EC5",
      "mappings": {"default": {"default": "e hat tilde"}}
    }, {"category": "Ll", "key": "1EC7", "mappings": {"default": {"default": "e hat underdot"}}}, {
      "category": "Ll",
      "key": "1ED1",
      "mappings": {"default": {"default": "o hat acute"}}
    }, {"category": "Ll", "key": "1ED3", "mappings": {"default": {"default": "o hat grave"}}}, {
      "category": "Ll",
      "key": "1ED5",
      "mappings": {"default": {"default": "o hat hook above"}}
    }, {"category": "Ll", "key": "1ED7", "mappings": {"default": {"default": "o hat tilde"}}}, {
      "category": "Ll",
      "key": "1ED9",
      "mappings": {"default": {"default": "o hat underdot"}}
    }, {"category": "Ll", "key": "1EDB", "mappings": {"default": {"default": "o acute prime"}}}, {
      "category": "Ll",
      "key": "1EDD",
      "mappings": {"default": {"default": "o grave prime"}}
    }, {"category": "Ll", "key": "1EDF", "mappings": {"default": {"default": "o hook above prime"}}}, {
      "category": "Ll",
      "key": "1EE1",
      "mappings": {"default": {"default": "o tilde prime"}}
    }, {"category": "Ll", "key": "1EE3", "mappings": {"default": {"default": "o underdot prime"}}}, {
      "category": "Ll",
      "key": "1EE9",
      "mappings": {"default": {"default": "u acute prime"}}
    }, {"category": "Ll", "key": "1EEB", "mappings": {"default": {"default": "u grave prime"}}}, {
      "category": "Ll",
      "key": "1EED",
      "mappings": {"default": {"default": "u hook above prime"}}
    }, {"category": "Ll", "key": "1EEF", "mappings": {"default": {"default": "u tilde prime"}}}, {
      "category": "Ll",
      "key": "1EF1",
      "mappings": {"default": {"default": "u underdot prime"}}
    }]
  ,
  "en/symbols/latin-lower-phonetic.js":
    [{"locale": "en"}, {
      "category": "Ll",
      "key": "00F8",
      "mappings": {"default": {"default": "o with stroke"}}
    }, {"category": "Ll", "key": "0111", "mappings": {"default": {"default": "d with stroke"}}}, {
      "category": "Ll",
      "key": "0127",
      "mappings": {"default": {"default": "h with stroke"}}
    }, {"category": "Ll", "key": "0142", "mappings": {"default": {"default": "l with stroke"}}}, {
      "category": "Ll",
      "key": "0167",
      "mappings": {"default": {"default": "t with stroke"}}
    }, {"category": "Ll", "key": "0180", "mappings": {"default": {"default": "b with stroke"}}}, {
      "category": "Ll",
      "key": "019B",
      "mappings": {"default": {"default": "lambda with stroke"}}
    }, {"category": "Ll", "key": "01B6", "mappings": {"default": {"default": "z with stroke"}}}, {
      "category": "Ll",
      "key": "01BE",
      "mappings": {"default": {"default": "latin letter inverted glottal stop with stroke"}}
    }, {"category": "Ll", "key": "01E5", "mappings": {"default": {"default": "g with stroke"}}}, {
      "category": "Ll",
      "key": "01FF",
      "mappings": {"default": {"default": "o with stroke and acute"}}
    }, {"category": "Ll", "key": "023C", "mappings": {"default": {"default": "c with stroke"}}}, {
      "category": "Ll",
      "key": "0247",
      "mappings": {"default": {"default": "e with stroke"}}
    }, {"category": "Ll", "key": "0249", "mappings": {"default": {"default": "j with stroke"}}}, {
      "category": "Ll",
      "key": "024D",
      "mappings": {"default": {"default": "r with stroke"}}
    }, {"category": "Ll", "key": "024F", "mappings": {"default": {"default": "y with stroke"}}}, {
      "category": "Ll",
      "key": "025F",
      "mappings": {"default": {"default": "dotless j with stroke"}}
    }, {"category": "Ll", "key": "0268", "mappings": {"default": {"default": "i with stroke"}}}, {
      "category": "Ll",
      "key": "0284",
      "mappings": {"default": {"default": "dotless j with stroke and hook"}}
    }, {
      "category": "Ll",
      "key": "02A1",
      "mappings": {"default": {"default": "latin letter glottal stop with stroke"}}
    }, {
      "category": "Ll",
      "key": "02A2",
      "mappings": {"default": {"default": "latin letter reversed glottal stop with stroke"}}
    }, {
      "category": "Ll",
      "key": "1D13",
      "mappings": {"default": {"default": "sideways o with stroke"}}
    }, {"category": "Ll", "key": "1D7C", "mappings": {"default": {"default": "iota with stroke"}}}, {
      "category": "Ll",
      "key": "1D7D",
      "mappings": {"default": {"default": "p with stroke"}}
    }, {
      "category": "Ll",
      "key": "1D7F",
      "mappings": {"default": {"default": "upsilon with stroke"}}
    }, {
      "category": "Ll",
      "key": "1E9C",
      "mappings": {"default": {"default": "long s with diagonal stroke"}}
    }, {
      "category": "Ll",
      "key": "1E9D",
      "mappings": {"default": {"default": "long s with high stroke"}}
    }, {"category": "Ll", "key": "018D", "mappings": {"default": {"default": "turned delta"}}}, {
      "category": "Ll",
      "key": "1E9B",
      "mappings": {"default": {"default": "long s with dot above"}}
    }, {"category": "Ll", "key": "1E9F", "mappings": {"default": {"default": "delta"}}}, {
      "category": "Ll",
      "key": "0138",
      "mappings": {"default": {"default": "kra"}}
    }, {"category": "Ll", "key": "017F", "mappings": {"default": {"default": "long s"}}}, {
      "category": "Ll",
      "key": "0183",
      "mappings": {"default": {"default": "b with topbar"}}
    }, {"category": "Ll", "key": "0185", "mappings": {"default": {"default": "tone six"}}}, {
      "category": "Ll",
      "key": "0188",
      "mappings": {"default": {"default": "c with hook"}}
    }, {"category": "Ll", "key": "018C", "mappings": {"default": {"default": "d with topbar"}}}, {
      "category": "Ll",
      "key": "0192",
      "mappings": {"default": {"default": "f with hook"}}
    }, {"category": "Ll", "key": "0195", "mappings": {"default": {"default": "hv"}}}, {
      "category": "Ll",
      "key": "0199",
      "mappings": {"default": {"default": "k with hook"}}
    }, {"category": "Ll", "key": "019A", "mappings": {"default": {"default": "l with bar"}}}, {
      "category": "Ll",
      "key": "019E",
      "mappings": {"default": {"default": "n with long right leg"}}
    }, {"category": "Ll", "key": "01A1", "mappings": {"default": {"default": "o with horn"}}}, {
      "category": "Ll",
      "key": "01A3",
      "mappings": {"default": {"default": "oi"}}
    }, {"category": "Ll", "key": "01A5", "mappings": {"default": {"default": "p with hook"}}}, {
      "category": "Ll",
      "key": "01A8",
      "mappings": {"default": {"default": "tone two"}}
    }, {
      "category": "Ll",
      "key": "01AA",
      "mappings": {"default": {"default": "latin letter reversed esh loop"}}
    }, {
      "category": "Ll",
      "key": "01AB",
      "mappings": {"default": {"default": "t with palatal hook"}}
    }, {"category": "Ll", "key": "01AD", "mappings": {"default": {"default": "t with hook"}}}, {
      "category": "Ll",
      "key": "01B0",
      "mappings": {"default": {"default": "u with horn"}}
    }, {"category": "Ll", "key": "01B4", "mappings": {"default": {"default": "y with hook"}}}, {
      "category": "Ll",
      "key": "01B9",
      "mappings": {"default": {"default": "ezh reversed"}}
    }, {"category": "Ll", "key": "01BA", "mappings": {"default": {"default": "ezh with tail"}}}, {
      "category": "Ll",
      "key": "01BD",
      "mappings": {"default": {"default": "tone five"}}
    }, {"category": "Ll", "key": "01BF", "mappings": {"default": {"default": "latin letter wynn"}}}, {
      "category": "Ll",
      "key": "01C6",
      "mappings": {"default": {"default": "dz with caron"}}
    }, {"category": "Ll", "key": "01C9", "mappings": {"default": {"default": "lj"}}}, {
      "category": "Ll",
      "key": "01CC",
      "mappings": {"default": {"default": "nj"}}
    }, {"category": "Ll", "key": "01E3", "mappings": {"default": {"default": "ae with macron"}}}, {
      "category": "Ll",
      "key": "01EF",
      "mappings": {"default": {"default": "ezh with caron"}}
    }, {"category": "Ll", "key": "01F3", "mappings": {"default": {"default": "dz"}}}, {
      "category": "Ll",
      "key": "021D",
      "mappings": {"default": {"default": "yogh"}}
    }, {"category": "Ll", "key": "026E", "mappings": {"default": {"default": "lezh"}}}, {
      "category": "Ll",
      "key": "0292",
      "mappings": {"default": {"default": "ezh"}}
    }, {"category": "Ll", "key": "0293", "mappings": {"default": {"default": "ezh with curl"}}}, {
      "category": "Ll",
      "key": "02A4",
      "mappings": {"default": {"default": "dezh digraph"}}
    }, {"category": "Ll", "key": "01DD", "mappings": {"default": {"default": "turned e"}}}, {
      "category": "Ll",
      "key": "01FD",
      "mappings": {"default": {"default": "ae with acute"}}
    }, {"category": "Ll", "key": "0221", "mappings": {"default": {"default": "d with curl"}}}, {
      "category": "Ll",
      "key": "0223",
      "mappings": {"default": {"default": "ou"}}
    }, {"category": "Ll", "key": "0225", "mappings": {"default": {"default": "z with hook"}}}, {
      "category": "Ll",
      "key": "0234",
      "mappings": {"default": {"default": "l with curl"}}
    }, {"category": "Ll", "key": "0235", "mappings": {"default": {"default": "n with curl"}}}, {
      "category": "Ll",
      "key": "0236",
      "mappings": {"default": {"default": "t with curl"}}
    }, {"category": "Ll", "key": "0238", "mappings": {"default": {"default": "db digraph"}}}, {
      "category": "Ll",
      "key": "0239",
      "mappings": {"default": {"default": "qp digraph"}}
    }, {"category": "Ll", "key": "023F", "mappings": {"default": {"default": "s with swash tail"}}}, {
      "category": "Ll",
      "key": "0240",
      "mappings": {"default": {"default": "z with swash tail"}}
    }, {"category": "Ll", "key": "0242", "mappings": {"default": {"default": "glottal stop"}}}, {
      "category": "Ll",
      "key": "024B",
      "mappings": {"default": {"default": "q with hook tail"}}
    }, {"category": "Ll", "key": "0250", "mappings": {"default": {"default": "turned a"}}}, {
      "category": "Ll",
      "key": "0251",
      "mappings": {"default": {"default": "alpha"}}
    }, {"category": "Ll", "key": "0252", "mappings": {"default": {"default": "turned alpha"}}}, {
      "category": "Ll",
      "key": "0253",
      "mappings": {"default": {"default": "b with hook"}}
    }, {"category": "Ll", "key": "0254", "mappings": {"default": {"default": "open o"}}}, {
      "category": "Ll",
      "key": "0255",
      "mappings": {"default": {"default": "c with curl"}}
    }, {"category": "Ll", "key": "0256", "mappings": {"default": {"default": "d with tail"}}}, {
      "category": "Ll",
      "key": "0257",
      "mappings": {"default": {"default": "d with hook"}}
    }, {"category": "Ll", "key": "0258", "mappings": {"default": {"default": "reversed e"}}}, {
      "category": "Ll",
      "key": "0259",
      "mappings": {"default": {"default": "schwa"}}
    }, {"category": "Ll", "key": "025A", "mappings": {"default": {"default": "schwa with hook"}}}, {
      "category": "Ll",
      "key": "025B",
      "mappings": {"default": {"default": "open e"}}
    }, {"category": "Ll", "key": "025C", "mappings": {"default": {"default": "reversed open e"}}}, {
      "category": "Ll",
      "key": "025D",
      "mappings": {"default": {"default": "reversed open e with hook"}}
    }, {
      "category": "Ll",
      "key": "025E",
      "mappings": {"default": {"default": "closed reversed open e"}}
    }, {"category": "Ll", "key": "0260", "mappings": {"default": {"default": "g with hook"}}}, {
      "category": "Ll",
      "key": "0261",
      "mappings": {"default": {"default": "script g"}}
    }, {"category": "Ll", "key": "0263", "mappings": {"default": {"default": "gamma"}}}, {
      "category": "Ll",
      "key": "0264",
      "mappings": {"default": {"default": "rams horn"}}
    }, {"category": "Ll", "key": "0265", "mappings": {"default": {"default": "turned h"}}}, {
      "category": "Ll",
      "key": "0266",
      "mappings": {"default": {"default": "h with hook"}}
    }, {"category": "Ll", "key": "0267", "mappings": {"default": {"default": "heng with hook"}}}, {
      "category": "Ll",
      "key": "0269",
      "mappings": {"default": {"default": "iota"}}
    }, {
      "category": "Ll",
      "key": "026B",
      "mappings": {"default": {"default": "l with middle tilde"}}
    }, {"category": "Ll", "key": "026C", "mappings": {"default": {"default": "l with belt"}}}, {
      "category": "Ll",
      "key": "026D",
      "mappings": {"default": {"default": "l with retroflex hook"}}
    }, {"category": "Ll", "key": "026F", "mappings": {"default": {"default": "turned m"}}}, {
      "category": "Ll",
      "key": "0270",
      "mappings": {"default": {"default": "turned m with long leg"}}
    }, {"category": "Ll", "key": "0271", "mappings": {"default": {"default": "m with hook"}}}, {
      "category": "Ll",
      "key": "0272",
      "mappings": {"default": {"default": "n with left hook"}}
    }, {
      "category": "Ll",
      "key": "0273",
      "mappings": {"default": {"default": "n with retroflex hook"}}
    }, {"category": "Ll", "key": "0275", "mappings": {"default": {"default": "barred o"}}}, {
      "category": "Ll",
      "key": "0277",
      "mappings": {"default": {"default": "closed omega"}}
    }, {"category": "Ll", "key": "0278", "mappings": {"default": {"default": "phi"}}}, {
      "category": "Ll",
      "key": "0279",
      "mappings": {"default": {"default": "turned r"}}
    }, {
      "category": "Ll",
      "key": "027A",
      "mappings": {"default": {"default": "turned r with long leg"}}
    }, {"category": "Ll", "key": "027B", "mappings": {"default": {"default": "turned r with hook"}}}, {
      "category": "Ll",
      "key": "027C",
      "mappings": {"default": {"default": "r with long leg"}}
    }, {"category": "Ll", "key": "027D", "mappings": {"default": {"default": "r with tail"}}}, {
      "category": "Ll",
      "key": "027E",
      "mappings": {"default": {"default": "r with fishhook"}}
    }, {
      "category": "Ll",
      "key": "027F",
      "mappings": {"default": {"default": "reversed r with fishhook"}}
    }, {"category": "Ll", "key": "0282", "mappings": {"default": {"default": "s with hook"}}}, {
      "category": "Ll",
      "key": "0283",
      "mappings": {"default": {"default": "esh"}}
    }, {"category": "Ll", "key": "0285", "mappings": {"default": {"default": "squat reversed esh"}}}, {
      "category": "Ll",
      "key": "0286",
      "mappings": {"default": {"default": "esh with curl"}}
    }, {"category": "Ll", "key": "0287", "mappings": {"default": {"default": "turned t"}}}, {
      "category": "Ll",
      "key": "0288",
      "mappings": {"default": {"default": "t with retroflex hook"}}
    }, {"category": "Ll", "key": "0289", "mappings": {"default": {"default": "u bar"}}}, {
      "category": "Ll",
      "key": "028A",
      "mappings": {"default": {"default": "upsilon"}}
    }, {"category": "Ll", "key": "028B", "mappings": {"default": {"default": "v with hook"}}}, {
      "category": "Ll",
      "key": "028C",
      "mappings": {"default": {"default": "turned v"}}
    }, {"category": "Ll", "key": "028D", "mappings": {"default": {"default": "turned w"}}}, {
      "category": "Ll",
      "key": "028E",
      "mappings": {"default": {"default": "turned y"}}
    }, {
      "category": "Ll",
      "key": "0290",
      "mappings": {"default": {"default": "z with retroflex hook"}}
    }, {"category": "Ll", "key": "0291", "mappings": {"default": {"default": "z with curl"}}}, {
      "category": "Ll",
      "key": "0295",
      "mappings": {"default": {"default": "latin letter pharyngeal voiced fricative"}}
    }, {
      "category": "Ll",
      "key": "0296",
      "mappings": {"default": {"default": "latin letter inverted glottal stop"}}
    }, {
      "category": "Ll",
      "key": "0297",
      "mappings": {"default": {"default": "latin letter stretched c"}}
    }, {
      "category": "Ll",
      "key": "0298",
      "mappings": {"default": {"default": "latin letter bilabial click"}}
    }, {"category": "Ll", "key": "029A", "mappings": {"default": {"default": "closed open e"}}}, {
      "category": "Ll",
      "key": "029E",
      "mappings": {"default": {"default": "turned k"}}
    }, {"category": "Ll", "key": "02A0", "mappings": {"default": {"default": "q with hook"}}}, {
      "category": "Ll",
      "key": "02A3",
      "mappings": {"default": {"default": "dz digraph"}}
    }, {
      "category": "Ll",
      "key": "02A5",
      "mappings": {"default": {"default": "dz digraph with curl"}}
    }, {"category": "Ll", "key": "02A6", "mappings": {"default": {"default": "ts digraph"}}}, {
      "category": "Ll",
      "key": "02A7",
      "mappings": {"default": {"default": "tesh digraph"}}
    }, {
      "category": "Ll",
      "key": "02A8",
      "mappings": {"default": {"default": "tc digraph with curl"}}
    }, {"category": "Ll", "key": "02A9", "mappings": {"default": {"default": "feng digraph"}}}, {
      "category": "Ll",
      "key": "02AA",
      "mappings": {"default": {"default": "ls digraph"}}
    }, {"category": "Ll", "key": "02AB", "mappings": {"default": {"default": "lz digraph"}}}, {
      "category": "Ll",
      "key": "02AC",
      "mappings": {"default": {"default": "latin letter bilabial percussive"}}
    }, {
      "category": "Ll",
      "key": "02AD",
      "mappings": {"default": {"default": "latin letter bidental percussive"}}
    }, {
      "category": "Ll",
      "key": "02AE",
      "mappings": {"default": {"default": "turned h with fishhook"}}
    }, {
      "category": "Ll",
      "key": "02AF",
      "mappings": {"default": {"default": "turned h with fishhook and tail"}}
    }, {"category": "Ll", "key": "1D02", "mappings": {"default": {"default": "turned ae"}}}, {
      "category": "Ll",
      "key": "1D08",
      "mappings": {"default": {"default": "turned open e"}}
    }, {"category": "Ll", "key": "1D09", "mappings": {"default": {"default": "turned i"}}}, {
      "category": "Ll",
      "key": "1D11",
      "mappings": {"default": {"default": "sideways o"}}
    }, {"category": "Ll", "key": "1D12", "mappings": {"default": {"default": "sideways open o"}}}, {
      "category": "Ll",
      "key": "1D14",
      "mappings": {"default": {"default": "turned oe"}}
    }, {"category": "Ll", "key": "1D16", "mappings": {"default": {"default": "top half o"}}}, {
      "category": "Ll",
      "key": "1D17",
      "mappings": {"default": {"default": "bottom half o"}}
    }, {"category": "Ll", "key": "1D1D", "mappings": {"default": {"default": "sideways u"}}}, {
      "category": "Ll",
      "key": "1D1E",
      "mappings": {"default": {"default": "sideways diaeresized u"}}
    }, {"category": "Ll", "key": "1D1F", "mappings": {"default": {"default": "sideways turned m"}}}, {
      "category": "Ll",
      "key": "1D24",
      "mappings": {"default": {"default": "latin letter voiced laryngeal spirant"}}
    }, {"category": "Ll", "key": "1D25", "mappings": {"default": {"default": "latin letter ain"}}}, {
      "category": "Ll",
      "key": "1D6B",
      "mappings": {"default": {"default": "ue"}}
    }, {
      "category": "Ll",
      "key": "1D6C",
      "mappings": {"default": {"default": "b with middle tilde"}}
    }, {
      "category": "Ll",
      "key": "1D6D",
      "mappings": {"default": {"default": "d with middle tilde"}}
    }, {
      "category": "Ll",
      "key": "1D6E",
      "mappings": {"default": {"default": "f with middle tilde"}}
    }, {
      "category": "Ll",
      "key": "1D6F",
      "mappings": {"default": {"default": "m with middle tilde"}}
    }, {
      "category": "Ll",
      "key": "1D70",
      "mappings": {"default": {"default": "n with middle tilde"}}
    }, {
      "category": "Ll",
      "key": "1D71",
      "mappings": {"default": {"default": "p with middle tilde"}}
    }, {
      "category": "Ll",
      "key": "1D72",
      "mappings": {"default": {"default": "r with middle tilde"}}
    }, {
      "category": "Ll",
      "key": "1D73",
      "mappings": {"default": {"default": "r with fishhook and middle tilde"}}
    }, {
      "category": "Ll",
      "key": "1D74",
      "mappings": {"default": {"default": "s with middle tilde"}}
    }, {
      "category": "Ll",
      "key": "1D75",
      "mappings": {"default": {"default": "t with middle tilde"}}
    }, {
      "category": "Ll",
      "key": "1D76",
      "mappings": {"default": {"default": "z with middle tilde"}}
    }, {"category": "Ll", "key": "1D77", "mappings": {"default": {"default": "turned g"}}}, {
      "category": "Ll",
      "key": "1D79",
      "mappings": {"default": {"default": "insular g"}}
    }, {
      "category": "Ll",
      "key": "1D7A",
      "mappings": {"default": {"default": "th with strikethrough"}}
    }, {
      "category": "Ll",
      "key": "1D80",
      "mappings": {"default": {"default": "b with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D81",
      "mappings": {"default": {"default": "d with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D82",
      "mappings": {"default": {"default": "f with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D83",
      "mappings": {"default": {"default": "g with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D84",
      "mappings": {"default": {"default": "k with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D85",
      "mappings": {"default": {"default": "l with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D86",
      "mappings": {"default": {"default": "m with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D87",
      "mappings": {"default": {"default": "n with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D88",
      "mappings": {"default": {"default": "p with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D89",
      "mappings": {"default": {"default": "r with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D8A",
      "mappings": {"default": {"default": "s with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D8B",
      "mappings": {"default": {"default": "esh with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D8C",
      "mappings": {"default": {"default": "v with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D8D",
      "mappings": {"default": {"default": "x with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D8E",
      "mappings": {"default": {"default": "z with palatal hook"}}
    }, {
      "category": "Ll",
      "key": "1D8F",
      "mappings": {"default": {"default": "a with retroflex hook"}}
    }, {
      "category": "Ll",
      "key": "1D90",
      "mappings": {"default": {"default": "alpha with retroflex hook"}}
    }, {
      "category": "Ll",
      "key": "1D91",
      "mappings": {"default": {"default": "d with hook and tail"}}
    }, {
      "category": "Ll",
      "key": "1D92",
      "mappings": {"default": {"default": "e with retroflex hook"}}
    }, {
      "category": "Ll",
      "key": "1D93",
      "mappings": {"default": {"default": "open e with retroflex hook"}}
    }, {
      "category": "Ll",
      "key": "1D94",
      "mappings": {"default": {"default": "reversed open e with retroflex hook"}}
    }, {
      "category": "Ll",
      "key": "1D95",
      "mappings": {"default": {"default": "schwa with retroflex hook"}}
    }, {
      "category": "Ll",
      "key": "1D96",
      "mappings": {"default": {"default": "i with retroflex hook"}}
    }, {
      "category": "Ll",
      "key": "1D97",
      "mappings": {"default": {"default": "open o with retroflex hook"}}
    }, {
      "category": "Ll",
      "key": "1D98",
      "mappings": {"default": {"default": "esh with retroflex hook"}}
    }, {
      "category": "Ll",
      "key": "1D99",
      "mappings": {"default": {"default": "u with retroflex hook"}}
    }, {
      "category": "Ll",
      "key": "1D9A",
      "mappings": {"default": {"default": "ezh with retroflex hook"}}
    }, {
      "category": "Ll",
      "key": "0149",
      "mappings": {"default": {"default": "n preceded by apostrophe"}}
    }, {"category": "Ll", "key": "014B", "mappings": {"default": {"default": "eng"}}}]
  ,
  "en/symbols/latin-lower-single-accent.js":
    [{"locale": "en"}, {
      "category": "Ll",
      "key": "00E0",
      "mappings": {
        "default": {"default": "a grave"},
        "mathspeak": {
          "default": "modifying above a with grave",
          "brief": "mod above a with grave",
          "sbrief": "mod above a with grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "00E1",
      "mappings": {
        "default": {"default": "a acute"},
        "mathspeak": {
          "default": "modifying above a with acute",
          "brief": "mod above a with acute",
          "sbrief": "mod above a with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "00E2",
      "mappings": {
        "default": {"default": "a hat"},
        "mathspeak": {
          "default": "modifying above a with caret",
          "brief": "mod above a with caret",
          "sbrief": "mod above a with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "00E3",
      "mappings": {
        "default": {"default": "a tilde"},
        "mathspeak": {"default": "a overtilde", "brief": "a overtilde", "sbrief": "a overtilde"}
      }
    }, {
      "category": "Ll",
      "key": "00E4",
      "mappings": {
        "default": {"default": "a double overdot"},
        "mathspeak": {
          "default": "modifying above a with double dot",
          "brief": "mod above a with double dot",
          "sbrief": "mod above a with double dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "00E5",
      "mappings": {
        "default": {"default": "a ring"},
        "mathspeak": {
          "default": "modifying above a with ring",
          "brief": "mod above a with ring",
          "sbrief": "mod above a with ring"
        }
      }
    }, {
      "category": "Ll",
      "key": "00E7",
      "mappings": {
        "default": {"default": "c cedilla"},
        "mathspeak": {
          "default": "modifying above c with cedilla",
          "brief": "mod above c with cedilla",
          "sbrief": "mod above c with cedilla"
        }
      }
    }, {
      "category": "Ll",
      "key": "00E8",
      "mappings": {
        "default": {"default": "e grave"},
        "mathspeak": {
          "default": "modifying above e with grave",
          "brief": "mod above e with grave",
          "sbrief": "mod above e with grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "00E9",
      "mappings": {
        "default": {"default": "e acute"},
        "mathspeak": {
          "default": "modifying above e with acute",
          "brief": "mod above e with acute",
          "sbrief": "mod above e with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "00EA",
      "mappings": {
        "default": {"default": "e hat"},
        "mathspeak": {
          "default": "modifying above e with caret",
          "brief": "mod above e with caret",
          "sbrief": "mod above e with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "00EB",
      "mappings": {
        "default": {"default": "e double overdot"},
        "mathspeak": {
          "default": "modifying above e with double dot",
          "brief": "mod above e with double dot",
          "sbrief": "mod above e with double dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "00EC",
      "mappings": {
        "default": {"default": "i grave"},
        "mathspeak": {
          "default": "modifying above i with grave",
          "brief": "mod above i with grave",
          "sbrief": "mod above i with grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "00ED",
      "mappings": {
        "default": {"default": "i acute"},
        "mathspeak": {
          "default": "modifying above i with acute",
          "brief": "mod above i with acute",
          "sbrief": "mod above i with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "00EE",
      "mappings": {
        "default": {"default": "i hat"},
        "mathspeak": {
          "default": "modifying above i with caret",
          "brief": "mod above i with caret",
          "sbrief": "mod above i with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "00EF",
      "mappings": {
        "default": {"default": "i double overdot"},
        "mathspeak": {
          "default": "modifying above i with double dot",
          "brief": "mod above i with double dot",
          "sbrief": "mod above i with double dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "00F1",
      "mappings": {
        "default": {"default": "n tilde"},
        "mathspeak": {"default": "n overtilde", "brief": "n overtilde", "sbrief": "n overtilde"}
      }
    }, {
      "category": "Ll",
      "key": "00F2",
      "mappings": {
        "default": {"default": "o grave"},
        "mathspeak": {
          "default": "modifying above o with grave",
          "brief": "mod above o with grave",
          "sbrief": "mod above o with grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "00F3",
      "mappings": {
        "default": {"default": "o acute"},
        "mathspeak": {
          "default": "modifying above o with acute",
          "brief": "mod above o with acute",
          "sbrief": "mod above o with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "00F4",
      "mappings": {
        "default": {"default": "o hat"},
        "mathspeak": {
          "default": "modifying above o with caret",
          "brief": "mod above o with caret",
          "sbrief": "mod above o with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "00F5",
      "mappings": {
        "default": {"default": "o tilde"},
        "mathspeak": {"default": "o overtilde", "brief": "o overtilde", "sbrief": "o overtilde"}
      }
    }, {
      "category": "Ll",
      "key": "00F6",
      "mappings": {
        "default": {"default": "o double overdot"},
        "mathspeak": {
          "default": "modifying above o with double dot",
          "brief": "mod above o with double dot",
          "sbrief": "mod above o with double dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "00F9",
      "mappings": {
        "default": {"default": "u grave"},
        "mathspeak": {
          "default": "modifying above u with grave",
          "brief": "mod above u with grave",
          "sbrief": "mod above u with grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "00FA",
      "mappings": {
        "default": {"default": "u acute"},
        "mathspeak": {
          "default": "modifying above u with acute",
          "brief": "mod above u with acute",
          "sbrief": "mod above u with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "00FB",
      "mappings": {
        "default": {"default": "u hat"},
        "mathspeak": {
          "default": "modifying above u with caret",
          "brief": "mod above u with caret",
          "sbrief": "mod above u with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "00FC",
      "mappings": {
        "default": {"default": "u double overdot"},
        "mathspeak": {
          "default": "modifying above u with double dot",
          "brief": "mod above u with double dot",
          "sbrief": "mod above u with double dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "00FD",
      "mappings": {
        "default": {"default": "y acute"},
        "mathspeak": {
          "default": "modifying above y with acute",
          "brief": "mod above y with acute",
          "sbrief": "mod above y with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "00FF",
      "mappings": {
        "default": {"default": "y double overdot"},
        "mathspeak": {
          "default": "modifying above y with double dot",
          "brief": "mod above y with double dot",
          "sbrief": "mod above y with double dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "0101",
      "mappings": {
        "default": {"default": "a overbar"},
        "mathspeak": {"default": "a overbar", "brief": "a overbar", "sbrief": "a overbar"}
      }
    }, {
      "category": "Ll",
      "key": "0103",
      "mappings": {
        "default": {"default": "a breve"},
        "mathspeak": {
          "default": "modifying above a with breve",
          "brief": "mod above a with breve",
          "sbrief": "mod above a with breve"
        }
      }
    }, {
      "category": "Ll",
      "key": "0105",
      "mappings": {
        "default": {"default": "a ogonek"},
        "mathspeak": {
          "default": "modifying above a with ogonek",
          "brief": "mod above a with ogonek",
          "sbrief": "mod above a with ogonek"
        }
      }
    }, {
      "category": "Ll",
      "key": "0107",
      "mappings": {
        "default": {"default": "c acute"},
        "mathspeak": {
          "default": "modifying above c with acute",
          "brief": "mod above c with acute",
          "sbrief": "mod above c with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "0109",
      "mappings": {
        "default": {"default": "c hat"},
        "mathspeak": {
          "default": "modifying above c with caret",
          "brief": "mod above c with caret",
          "sbrief": "mod above c with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "010B",
      "mappings": {
        "default": {"default": "c overdot"},
        "mathspeak": {
          "default": "modifying above c with dot",
          "brief": "mod above c with dot",
          "sbrief": "mod above c with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "010D",
      "mappings": {
        "default": {"default": "c caron"},
        "mathspeak": {
          "default": "modifying above c with caron",
          "brief": "mod above c with caron",
          "sbrief": "mod above c with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "010F",
      "mappings": {
        "default": {"default": "d caron"},
        "mathspeak": {
          "default": "modifying above d with caron",
          "brief": "mod above d with caron",
          "sbrief": "mod above d with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "0113",
      "mappings": {
        "default": {"default": "e overbar"},
        "mathspeak": {"default": "e overbar", "brief": "e overbar", "sbrief": "e overbar"}
      }
    }, {
      "category": "Ll",
      "key": "0115",
      "mappings": {
        "default": {"default": "e breve"},
        "mathspeak": {
          "default": "modifying above e with breve",
          "brief": "mod above e with breve",
          "sbrief": "mod above e with breve"
        }
      }
    }, {
      "category": "Ll",
      "key": "0117",
      "mappings": {
        "default": {"default": "e overdot"},
        "mathspeak": {
          "default": "modifying above e with dot",
          "brief": "mod above e with dot",
          "sbrief": "mod above e with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "0119",
      "mappings": {
        "default": {"default": "e ogonek"},
        "mathspeak": {
          "default": "modifying above e with ogonek",
          "brief": "mod above e with ogonek",
          "sbrief": "mod above e with ogonek"
        }
      }
    }, {
      "category": "Ll",
      "key": "011B",
      "mappings": {
        "default": {"default": "e caron"},
        "mathspeak": {
          "default": "modifying above e with caron",
          "brief": "mod above e with caron",
          "sbrief": "mod above e with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "011D",
      "mappings": {
        "default": {"default": "g hat"},
        "mathspeak": {
          "default": "modifying above g with caret",
          "brief": "mod above g with caret",
          "sbrief": "mod above g with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "011F",
      "mappings": {
        "default": {"default": "g breve"},
        "mathspeak": {
          "default": "modifying above g with breve",
          "brief": "mod above g with breve",
          "sbrief": "mod above g with breve"
        }
      }
    }, {
      "category": "Ll",
      "key": "0121",
      "mappings": {
        "default": {"default": "g overdot"},
        "mathspeak": {
          "default": "modifying above g with dot",
          "brief": "mod above g with dot",
          "sbrief": "mod above g with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "0123",
      "mappings": {
        "default": {"default": "g cedilla"},
        "mathspeak": {
          "default": "modifying above g with cedilla",
          "brief": "mod above g with cedilla",
          "sbrief": "mod above g with cedilla"
        }
      }
    }, {
      "category": "Ll",
      "key": "0125",
      "mappings": {
        "default": {"default": "h hat"},
        "mathspeak": {
          "default": "modifying above h with caret",
          "brief": "mod above h with caret",
          "sbrief": "mod above h with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "0129",
      "mappings": {
        "default": {"default": "i tilde"},
        "mathspeak": {"default": "i overtilde", "brief": "i overtilde", "sbrief": "i overtilde"}
      }
    }, {
      "category": "Ll",
      "key": "012B",
      "mappings": {
        "default": {"default": "i overbar"},
        "mathspeak": {"default": "i overbar", "brief": "i overbar", "sbrief": "i overbar"}
      }
    }, {
      "category": "Ll",
      "key": "012D",
      "mappings": {
        "default": {"default": "i breve"},
        "mathspeak": {
          "default": "modifying above i with breve",
          "brief": "mod above i with breve",
          "sbrief": "mod above i with breve"
        }
      }
    }, {
      "category": "Ll",
      "key": "012F",
      "mappings": {
        "default": {"default": "i ogonek"},
        "mathspeak": {
          "default": "modifying above i with ogonek",
          "brief": "mod above i with ogonek",
          "sbrief": "mod above i with ogonek"
        }
      }
    }, {
      "category": "Ll",
      "key": "0131",
      "mappings": {
        "default": {"default": "dotless i"},
        "mathspeak": {
          "default": "modifying above dotless i",
          "brief": "mod above dotless i",
          "sbrief": "mod above dotless i"
        }
      }
    }, {
      "category": "Ll",
      "key": "0135",
      "mappings": {
        "default": {"default": "j hat"},
        "mathspeak": {
          "default": "modifying above j with caret",
          "brief": "mod above j with caret",
          "sbrief": "mod above j with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "0137",
      "mappings": {
        "default": {"default": "k cedilla"},
        "mathspeak": {
          "default": "modifying above k with cedilla",
          "brief": "mod above k with cedilla",
          "sbrief": "mod above k with cedilla"
        }
      }
    }, {
      "category": "Ll",
      "key": "013A",
      "mappings": {
        "default": {"default": "l acute"},
        "mathspeak": {
          "default": "modifying above l with acute",
          "brief": "mod above l with acute",
          "sbrief": "mod above l with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "013C",
      "mappings": {
        "default": {"default": "l cedilla"},
        "mathspeak": {
          "default": "modifying above l with cedilla",
          "brief": "mod above l with cedilla",
          "sbrief": "mod above l with cedilla"
        }
      }
    }, {
      "category": "Ll",
      "key": "013E",
      "mappings": {
        "default": {"default": "l caron"},
        "mathspeak": {
          "default": "modifying above l with caron",
          "brief": "mod above l with caron",
          "sbrief": "mod above l with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "0140",
      "mappings": {
        "default": {"default": "l middle dot"},
        "mathspeak": {
          "default": "modifying above l with middle dot",
          "brief": "mod above l with middle dot",
          "sbrief": "mod above l with middle dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "0144",
      "mappings": {
        "default": {"default": "n acute"},
        "mathspeak": {
          "default": "modifying above n with acute",
          "brief": "mod above n with acute",
          "sbrief": "mod above n with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "0146",
      "mappings": {
        "default": {"default": "n cedilla"},
        "mathspeak": {
          "default": "modifying above n with cedilla",
          "brief": "mod above n with cedilla",
          "sbrief": "mod above n with cedilla"
        }
      }
    }, {
      "category": "Ll",
      "key": "0148",
      "mappings": {
        "default": {"default": "n caron"},
        "mathspeak": {
          "default": "modifying above n with caron",
          "brief": "mod above n with caron",
          "sbrief": "mod above n with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "014D",
      "mappings": {
        "default": {"default": "o overbar"},
        "mathspeak": {"default": "o overbar", "brief": "o overbar", "sbrief": "o overbar"}
      }
    }, {
      "category": "Ll",
      "key": "014F",
      "mappings": {
        "default": {"default": "o breve"},
        "mathspeak": {
          "default": "modifying above o with breve",
          "brief": "mod above o with breve",
          "sbrief": "mod above o with breve"
        }
      }
    }, {
      "category": "Ll",
      "key": "0151",
      "mappings": {
        "default": {"default": "o double acute"},
        "mathspeak": {
          "default": "modifying above o with double acute",
          "brief": "mod above o with double acute",
          "sbrief": "mod above o with double acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "0155",
      "mappings": {
        "default": {"default": "r acute"},
        "mathspeak": {
          "default": "modifying above r with acute",
          "brief": "mod above r with acute",
          "sbrief": "mod above r with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "0157",
      "mappings": {
        "default": {"default": "r cedilla"},
        "mathspeak": {
          "default": "modifying above r with cedilla",
          "brief": "mod above r with cedilla",
          "sbrief": "mod above r with cedilla"
        }
      }
    }, {
      "category": "Ll",
      "key": "0159",
      "mappings": {
        "default": {"default": "r caron"},
        "mathspeak": {
          "default": "modifying above r with caron",
          "brief": "mod above r with caron",
          "sbrief": "mod above r with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "015B",
      "mappings": {
        "default": {"default": "s acute"},
        "mathspeak": {
          "default": "modifying above s with acute",
          "brief": "mod above s with acute",
          "sbrief": "mod above s with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "015D",
      "mappings": {
        "default": {"default": "s hat"},
        "mathspeak": {
          "default": "modifying above s with caret",
          "brief": "mod above s with caret",
          "sbrief": "mod above s with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "015F",
      "mappings": {
        "default": {"default": "s cedilla"},
        "mathspeak": {
          "default": "modifying above s with cedilla",
          "brief": "mod above s with cedilla",
          "sbrief": "mod above s with cedilla"
        }
      }
    }, {
      "category": "Ll",
      "key": "0161",
      "mappings": {
        "default": {"default": "s caron"},
        "mathspeak": {
          "default": "modifying above s with caron",
          "brief": "mod above s with caron",
          "sbrief": "mod above s with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "0163",
      "mappings": {
        "default": {"default": "t cedilla"},
        "mathspeak": {
          "default": "modifying above t with cedilla",
          "brief": "mod above t with cedilla",
          "sbrief": "mod above t with cedilla"
        }
      }
    }, {
      "category": "Ll",
      "key": "0165",
      "mappings": {
        "default": {"default": "t caron"},
        "mathspeak": {
          "default": "modifying above t with caron",
          "brief": "mod above t with caron",
          "sbrief": "mod above t with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "0169",
      "mappings": {
        "default": {"default": "u tilde"},
        "mathspeak": {"default": "u overtilde", "brief": "u overtilde", "sbrief": "u overtilde"}
      }
    }, {
      "category": "Ll",
      "key": "016B",
      "mappings": {
        "default": {"default": "u overbar"},
        "mathspeak": {"default": "u overbar", "brief": "u overbar", "sbrief": "u overbar"}
      }
    }, {
      "category": "Ll",
      "key": "016D",
      "mappings": {
        "default": {"default": "u breve"},
        "mathspeak": {
          "default": "modifying above u with breve",
          "brief": "mod above u with breve",
          "sbrief": "mod above u with breve"
        }
      }
    }, {
      "category": "Ll",
      "key": "016F",
      "mappings": {
        "default": {"default": "u ring"},
        "mathspeak": {
          "default": "modifying above u with ring",
          "brief": "mod above u with ring",
          "sbrief": "mod above u with ring"
        }
      }
    }, {
      "category": "Ll",
      "key": "0171",
      "mappings": {
        "default": {"default": "u double acute"},
        "mathspeak": {
          "default": "modifying above u with double acute",
          "brief": "mod above u with double acute",
          "sbrief": "mod above u with double acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "0173",
      "mappings": {
        "default": {"default": "u ogonek"},
        "mathspeak": {
          "default": "modifying above u with ogonek",
          "brief": "mod above u with ogonek",
          "sbrief": "mod above u with ogonek"
        }
      }
    }, {
      "category": "Ll",
      "key": "0175",
      "mappings": {
        "default": {"default": "w hat"},
        "mathspeak": {
          "default": "modifying above w with caret",
          "brief": "mod above w with caret",
          "sbrief": "mod above w with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "0177",
      "mappings": {
        "default": {"default": "y hat"},
        "mathspeak": {
          "default": "modifying above y with caret",
          "brief": "mod above y with caret",
          "sbrief": "mod above y with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "017A",
      "mappings": {
        "default": {"default": "z acute"},
        "mathspeak": {
          "default": "modifying above z with acute",
          "brief": "mod above z with acute",
          "sbrief": "mod above z with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "017C",
      "mappings": {
        "default": {"default": "z overdot"},
        "mathspeak": {
          "default": "modifying above z with dot",
          "brief": "mod above z with dot",
          "sbrief": "mod above z with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "017E",
      "mappings": {
        "default": {"default": "z caron"},
        "mathspeak": {
          "default": "modifying above z with caron",
          "brief": "mod above z with caron",
          "sbrief": "mod above z with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "01CE",
      "mappings": {
        "default": {"default": "a caron"},
        "mathspeak": {
          "default": "modifying above a with caron",
          "brief": "mod above a with caron",
          "sbrief": "mod above a with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "01D0",
      "mappings": {
        "default": {"default": "i caron"},
        "mathspeak": {
          "default": "modifying above i with caron",
          "brief": "mod above i with caron",
          "sbrief": "mod above i with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "01D2",
      "mappings": {
        "default": {"default": "o caron"},
        "mathspeak": {
          "default": "modifying above o with caron",
          "brief": "mod above o with caron",
          "sbrief": "mod above o with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "01D4",
      "mappings": {
        "default": {"default": "u caron"},
        "mathspeak": {
          "default": "modifying above u with caron",
          "brief": "mod above u with caron",
          "sbrief": "mod above u with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "01E7",
      "mappings": {
        "default": {"default": "g caron"},
        "mathspeak": {
          "default": "modifying above g with caron",
          "brief": "mod above g with caron",
          "sbrief": "mod above g with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "01E9",
      "mappings": {
        "default": {"default": "k caron"},
        "mathspeak": {
          "default": "modifying above k with caron",
          "brief": "mod above k with caron",
          "sbrief": "mod above k with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "01EB",
      "mappings": {
        "default": {"default": "o ogonek"},
        "mathspeak": {
          "default": "modifying above o with ogonek",
          "brief": "mod above o with ogonek",
          "sbrief": "mod above o with ogonek"
        }
      }
    }, {
      "category": "Ll",
      "key": "01F0",
      "mappings": {
        "default": {"default": "j caron"},
        "mathspeak": {
          "default": "modifying above j with caron",
          "brief": "mod above j with caron",
          "sbrief": "mod above j with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "01F5",
      "mappings": {
        "default": {"default": "g acute"},
        "mathspeak": {
          "default": "modifying above g with acute",
          "brief": "mod above g with acute",
          "sbrief": "mod above g with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "01F9",
      "mappings": {
        "default": {"default": "n grave"},
        "mathspeak": {
          "default": "modifying above n with grave",
          "brief": "mod above n with grave",
          "sbrief": "mod above n with grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "0201",
      "mappings": {
        "default": {"default": "a double grave"},
        "mathspeak": {
          "default": "modifying above a with double grave",
          "brief": "mod above a with double grave",
          "sbrief": "mod above a with double grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "0203",
      "mappings": {
        "default": {"default": "a inverted breve"},
        "mathspeak": {
          "default": "modifying above a with inverted breve",
          "brief": "mod above a with inverted breve",
          "sbrief": "mod above a with inverted breve"
        }
      }
    }, {
      "category": "Ll",
      "key": "0205",
      "mappings": {
        "default": {"default": "e double grave"},
        "mathspeak": {
          "default": "modifying above e with double grave",
          "brief": "mod above e with double grave",
          "sbrief": "mod above e with double grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "0207",
      "mappings": {
        "default": {"default": "e inverted breve"},
        "mathspeak": {
          "default": "modifying above e with inverted breve",
          "brief": "mod above e with inverted breve",
          "sbrief": "mod above e with inverted breve"
        }
      }
    }, {
      "category": "Ll",
      "key": "0209",
      "mappings": {
        "default": {"default": "i double grave"},
        "mathspeak": {
          "default": "modifying above i with double grave",
          "brief": "mod above i with double grave",
          "sbrief": "mod above i with double grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "020B",
      "mappings": {
        "default": {"default": "i inverted breve"},
        "mathspeak": {
          "default": "modifying above i with inverted breve",
          "brief": "mod above i with inverted breve",
          "sbrief": "mod above i with inverted breve"
        }
      }
    }, {
      "category": "Ll",
      "key": "020D",
      "mappings": {
        "default": {"default": "o double grave"},
        "mathspeak": {
          "default": "modifying above o with double grave",
          "brief": "mod above o with double grave",
          "sbrief": "mod above o with double grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "020F",
      "mappings": {
        "default": {"default": "o inverted breve"},
        "mathspeak": {
          "default": "modifying above o with inverted breve",
          "brief": "mod above o with inverted breve",
          "sbrief": "mod above o with inverted breve"
        }
      }
    }, {
      "category": "Ll",
      "key": "0211",
      "mappings": {
        "default": {"default": "r double grave"},
        "mathspeak": {
          "default": "modifying above r with double grave",
          "brief": "mod above r with double grave",
          "sbrief": "mod above r with double grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "0213",
      "mappings": {
        "default": {"default": "r inverted breve"},
        "mathspeak": {
          "default": "modifying above r with inverted breve",
          "brief": "mod above r with inverted breve",
          "sbrief": "mod above r with inverted breve"
        }
      }
    }, {
      "category": "Ll",
      "key": "0215",
      "mappings": {
        "default": {"default": "u double grave"},
        "mathspeak": {
          "default": "modifying above u with double grave",
          "brief": "mod above u with double grave",
          "sbrief": "mod above u with double grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "0217",
      "mappings": {
        "default": {"default": "u inverted breve"},
        "mathspeak": {
          "default": "modifying above u with inverted breve",
          "brief": "mod above u with inverted breve",
          "sbrief": "mod above u with inverted breve"
        }
      }
    }, {
      "category": "Ll",
      "key": "0219",
      "mappings": {
        "default": {"default": "s comma below"},
        "mathspeak": {
          "default": "modifying below s with comma below",
          "brief": "mod below s with comma below",
          "sbrief": "mod below s with comma below"
        }
      }
    }, {
      "category": "Ll",
      "key": "021B",
      "mappings": {
        "default": {"default": "t comma below"},
        "mathspeak": {
          "default": "modifying below t with comma below",
          "brief": "mod below t with comma below",
          "sbrief": "mod below t with comma below"
        }
      }
    }, {
      "category": "Ll",
      "key": "021F",
      "mappings": {
        "default": {"default": "h caron"},
        "mathspeak": {
          "default": "modifying above h with caron",
          "brief": "mod above h with caron",
          "sbrief": "mod above h with caron"
        }
      }
    }, {
      "category": "Ll",
      "key": "0227",
      "mappings": {
        "default": {"default": "a overdot"},
        "mathspeak": {
          "default": "modifying above a with dot",
          "brief": "mod above a with dot",
          "sbrief": "mod above a with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "0229",
      "mappings": {
        "default": {"default": "e cedilla"},
        "mathspeak": {
          "default": "modifying above e with cedilla",
          "brief": "mod above e with cedilla",
          "sbrief": "mod above e with cedilla"
        }
      }
    }, {
      "category": "Ll",
      "key": "022F",
      "mappings": {
        "default": {"default": "o overdot"},
        "mathspeak": {
          "default": "modifying above o with dot",
          "brief": "mod above o with dot",
          "sbrief": "mod above o with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "0233",
      "mappings": {
        "default": {"default": "y overbar"},
        "mathspeak": {"default": "y overbar", "brief": "y overbar", "sbrief": "y overbar"}
      }
    }, {
      "category": "Ll",
      "key": "0237",
      "mappings": {
        "default": {"default": "dotless j"},
        "mathspeak": {
          "default": "modifying above dotless j",
          "brief": "mod above dotless j",
          "sbrief": "mod above dotless j"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E01",
      "mappings": {
        "default": {"default": "a ring below"},
        "mathspeak": {
          "default": "modifying below a with ring below",
          "brief": "mod below a with ring below",
          "sbrief": "mod below a with ring below"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E03",
      "mappings": {
        "default": {"default": "b overdot"},
        "mathspeak": {
          "default": "modifying above b with dot",
          "brief": "mod above b with dot",
          "sbrief": "mod above b with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E05",
      "mappings": {
        "default": {"default": "b underdot"},
        "mathspeak": {
          "default": "modifying below b with dot",
          "brief": "mod below b with dot",
          "sbrief": "mod below b with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E07",
      "mappings": {
        "default": {"default": "b underbar"},
        "mathspeak": {"default": "b underbar", "brief": "b underbar", "sbrief": "b underbar"}
      }
    }, {
      "category": "Ll",
      "key": "1E0B",
      "mappings": {
        "default": {"default": "d overdot"},
        "mathspeak": {
          "default": "modifying above d with dot",
          "brief": "mod above d with dot",
          "sbrief": "mod above d with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E0D",
      "mappings": {
        "default": {"default": "d underdot"},
        "mathspeak": {
          "default": "modifying below d with dot",
          "brief": "mod below d with dot",
          "sbrief": "mod below d with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E0F",
      "mappings": {
        "default": {"default": "d underbar"},
        "mathspeak": {"default": "d underbar", "brief": "d underbar", "sbrief": "d underbar"}
      }
    }, {
      "category": "Ll",
      "key": "1E11",
      "mappings": {
        "default": {"default": "d cedilla"},
        "mathspeak": {
          "default": "modifying above d with cedilla",
          "brief": "mod above d with cedilla",
          "sbrief": "mod above d with cedilla"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E13",
      "mappings": {
        "default": {"default": "d underhat"},
        "mathspeak": {
          "default": "modifying below d with caret",
          "brief": "mod below d with caret",
          "sbrief": "mod below d with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E19",
      "mappings": {
        "default": {"default": "e underhat"},
        "mathspeak": {
          "default": "modifying below e with caret",
          "brief": "mod below e with caret",
          "sbrief": "mod below e with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E1B",
      "mappings": {
        "default": {"default": "e tilde below"},
        "mathspeak": {"default": "e undertilde", "brief": "e undertilde", "sbrief": "e undertilde"}
      }
    }, {
      "category": "Ll",
      "key": "1E1F",
      "mappings": {
        "default": {"default": "f overdot"},
        "mathspeak": {
          "default": "modifying above f with dot",
          "brief": "mod above f with dot",
          "sbrief": "mod above f with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E21",
      "mappings": {
        "default": {"default": "g overbar"},
        "mathspeak": {"default": "g overbar", "brief": "g overbar", "sbrief": "g overbar"}
      }
    }, {
      "category": "Ll",
      "key": "1E23",
      "mappings": {
        "default": {"default": "h overdot"},
        "mathspeak": {
          "default": "modifying above h with dot",
          "brief": "mod above h with dot",
          "sbrief": "mod above h with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E25",
      "mappings": {
        "default": {"default": "h underdot"},
        "mathspeak": {
          "default": "modifying below h with dot",
          "brief": "mod below h with dot",
          "sbrief": "mod below h with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E27",
      "mappings": {
        "default": {"default": "h double overdot"},
        "mathspeak": {
          "default": "modifying above h with double dot",
          "brief": "mod above h with double dot",
          "sbrief": "mod above h with double dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E29",
      "mappings": {
        "default": {"default": "h cedilla"},
        "mathspeak": {
          "default": "modifying above h with cedilla",
          "brief": "mod above h with cedilla",
          "sbrief": "mod above h with cedilla"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E2B",
      "mappings": {
        "default": {"default": "h breve below"},
        "mathspeak": {
          "default": "modifying below h with breve below",
          "brief": "mod below h with breve below",
          "sbrief": "mod below h with breve below"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E2D",
      "mappings": {
        "default": {"default": "i tilde below"},
        "mathspeak": {"default": "i undertilde", "brief": "i undertilde", "sbrief": "i undertilde"}
      }
    }, {
      "category": "Ll",
      "key": "1E31",
      "mappings": {
        "default": {"default": "k acute"},
        "mathspeak": {
          "default": "modifying above k with acute",
          "brief": "mod above k with acute",
          "sbrief": "mod above k with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E33",
      "mappings": {
        "default": {"default": "k underdot"},
        "mathspeak": {
          "default": "modifying below k with dot",
          "brief": "mod below k with dot",
          "sbrief": "mod below k with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E35",
      "mappings": {
        "default": {"default": "k underbar"},
        "mathspeak": {"default": "k underbar", "brief": "k underbar", "sbrief": "k underbar"}
      }
    }, {
      "category": "Ll",
      "key": "1E37",
      "mappings": {
        "default": {"default": "l underdot"},
        "mathspeak": {
          "default": "modifying below l with dot",
          "brief": "mod below l with dot",
          "sbrief": "mod below l with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E3B",
      "mappings": {
        "default": {"default": "l underbar"},
        "mathspeak": {"default": "l underbar", "brief": "l underbar", "sbrief": "l underbar"}
      }
    }, {
      "category": "Ll",
      "key": "1E3D",
      "mappings": {
        "default": {"default": "l underhat"},
        "mathspeak": {
          "default": "modifying below l with caret",
          "brief": "mod below l with caret",
          "sbrief": "mod below l with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E3F",
      "mappings": {
        "default": {"default": "m acute"},
        "mathspeak": {
          "default": "modifying above m with acute",
          "brief": "mod above m with acute",
          "sbrief": "mod above m with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E41",
      "mappings": {
        "default": {"default": "m overdot"},
        "mathspeak": {
          "default": "modifying above m with dot",
          "brief": "mod above m with dot",
          "sbrief": "mod above m with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E43",
      "mappings": {
        "default": {"default": "m underdot"},
        "mathspeak": {
          "default": "modifying below m with dot",
          "brief": "mod below m with dot",
          "sbrief": "mod below m with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E45",
      "mappings": {
        "default": {"default": "n overdot"},
        "mathspeak": {
          "default": "modifying above n with dot",
          "brief": "mod above n with dot",
          "sbrief": "mod above n with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E47",
      "mappings": {
        "default": {"default": "n underdot"},
        "mathspeak": {
          "default": "modifying below n with dot",
          "brief": "mod below n with dot",
          "sbrief": "mod below n with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E49",
      "mappings": {
        "default": {"default": "n underbar"},
        "mathspeak": {"default": "n underbar", "brief": "n underbar", "sbrief": "n underbar"}
      }
    }, {
      "category": "Ll",
      "key": "1E4B",
      "mappings": {
        "default": {"default": "n underhat"},
        "mathspeak": {
          "default": "modifying below n with caret",
          "brief": "mod below n with caret",
          "sbrief": "mod below n with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E55",
      "mappings": {
        "default": {"default": "p acute"},
        "mathspeak": {
          "default": "modifying above p with acute",
          "brief": "mod above p with acute",
          "sbrief": "mod above p with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E57",
      "mappings": {
        "default": {"default": "p overdot"},
        "mathspeak": {
          "default": "modifying above p with dot",
          "brief": "mod above p with dot",
          "sbrief": "mod above p with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E59",
      "mappings": {
        "default": {"default": "r overdot"},
        "mathspeak": {
          "default": "modifying above r with dot",
          "brief": "mod above r with dot",
          "sbrief": "mod above r with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E5B",
      "mappings": {
        "default": {"default": "r underdot"},
        "mathspeak": {
          "default": "modifying below r with dot",
          "brief": "mod below r with dot",
          "sbrief": "mod below r with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E5F",
      "mappings": {
        "default": {"default": "r underbar"},
        "mathspeak": {"default": "r underbar", "brief": "r underbar", "sbrief": "r underbar"}
      }
    }, {
      "category": "Ll",
      "key": "1E61",
      "mappings": {
        "default": {"default": "s overdot"},
        "mathspeak": {
          "default": "modifying above s with dot",
          "brief": "mod above s with dot",
          "sbrief": "mod above s with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E63",
      "mappings": {
        "default": {"default": "s underdot"},
        "mathspeak": {
          "default": "modifying below s with dot",
          "brief": "mod below s with dot",
          "sbrief": "mod below s with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E6B",
      "mappings": {
        "default": {"default": "t overdot"},
        "mathspeak": {
          "default": "modifying above t with dot",
          "brief": "mod above t with dot",
          "sbrief": "mod above t with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E6D",
      "mappings": {
        "default": {"default": "t underdot"},
        "mathspeak": {
          "default": "modifying below t with dot",
          "brief": "mod below t with dot",
          "sbrief": "mod below t with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E6F",
      "mappings": {
        "default": {"default": "t underbar"},
        "mathspeak": {"default": "t underbar", "brief": "t underbar", "sbrief": "t underbar"}
      }
    }, {
      "category": "Ll",
      "key": "1E71",
      "mappings": {
        "default": {"default": "t underhat"},
        "mathspeak": {
          "default": "modifying below t with caret",
          "brief": "mod below t with caret",
          "sbrief": "mod below t with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E73",
      "mappings": {
        "default": {"default": "u double underdot"},
        "mathspeak": {
          "default": "modifying below u with double dot",
          "brief": "mod below u with double dot",
          "sbrief": "mod below u with double dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E75",
      "mappings": {
        "default": {"default": "u tilde below"},
        "mathspeak": {"default": "u undertilde", "brief": "u undertilde", "sbrief": "u undertilde"}
      }
    }, {
      "category": "Ll",
      "key": "1E77",
      "mappings": {
        "default": {"default": "u underhat"},
        "mathspeak": {
          "default": "modifying below u with caret",
          "brief": "mod below u with caret",
          "sbrief": "mod below u with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E7D",
      "mappings": {
        "default": {"default": "v tilde"},
        "mathspeak": {"default": "v overtilde", "brief": "v overtilde", "sbrief": "v overtilde"}
      }
    }, {
      "category": "Ll",
      "key": "1E7F",
      "mappings": {
        "default": {"default": "v underdot"},
        "mathspeak": {
          "default": "modifying below v with dot",
          "brief": "mod below v with dot",
          "sbrief": "mod below v with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E81",
      "mappings": {
        "default": {"default": "w grave"},
        "mathspeak": {
          "default": "modifying above w with grave",
          "brief": "mod above w with grave",
          "sbrief": "mod above w with grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E83",
      "mappings": {
        "default": {"default": "w acute"},
        "mathspeak": {
          "default": "modifying above w with acute",
          "brief": "mod above w with acute",
          "sbrief": "mod above w with acute"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E85",
      "mappings": {
        "default": {"default": "w double overdot"},
        "mathspeak": {
          "default": "modifying above w with double dot",
          "brief": "mod above w with double dot",
          "sbrief": "mod above w with double dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E87",
      "mappings": {
        "default": {"default": "w overdot"},
        "mathspeak": {
          "default": "modifying above w with dot",
          "brief": "mod above w with dot",
          "sbrief": "mod above w with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E89",
      "mappings": {
        "default": {"default": "w underdot"},
        "mathspeak": {
          "default": "modifying below w with dot",
          "brief": "mod below w with dot",
          "sbrief": "mod below w with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E8B",
      "mappings": {
        "default": {"default": "x overdot"},
        "mathspeak": {
          "default": "modifying above x with dot",
          "brief": "mod above x with dot",
          "sbrief": "mod above x with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E8D",
      "mappings": {
        "default": {"default": "x double overdot"},
        "mathspeak": {
          "default": "modifying above x with double dot",
          "brief": "mod above x with double dot",
          "sbrief": "mod above x with double dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E8F",
      "mappings": {
        "default": {"default": "y overdot"},
        "mathspeak": {
          "default": "modifying above y with dot",
          "brief": "mod above y with dot",
          "sbrief": "mod above y with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E91",
      "mappings": {
        "default": {"default": "z hat"},
        "mathspeak": {
          "default": "modifying above z with caret",
          "brief": "mod above z with caret",
          "sbrief": "mod above z with caret"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E93",
      "mappings": {
        "default": {"default": "z underdot"},
        "mathspeak": {
          "default": "modifying below z with dot",
          "brief": "mod below z with dot",
          "sbrief": "mod below z with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E95",
      "mappings": {
        "default": {"default": "z underbar"},
        "mathspeak": {"default": "z underbar", "brief": "z underbar", "sbrief": "z underbar"}
      }
    }, {
      "category": "Ll",
      "key": "1E96",
      "mappings": {
        "default": {"default": "h underbar"},
        "mathspeak": {"default": "h underbar", "brief": "h underbar", "sbrief": "h underbar"}
      }
    }, {
      "category": "Ll",
      "key": "1E97",
      "mappings": {
        "default": {"default": "t double overdot"},
        "mathspeak": {
          "default": "modifying above t with double dot",
          "brief": "mod above t with double dot",
          "sbrief": "mod above t with double dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E98",
      "mappings": {
        "default": {"default": "w ring"},
        "mathspeak": {
          "default": "modifying above w with ring",
          "brief": "mod above w with ring",
          "sbrief": "mod above w with ring"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E99",
      "mappings": {
        "default": {"default": "y ring"},
        "mathspeak": {
          "default": "modifying above y with ring",
          "brief": "mod above y with ring",
          "sbrief": "mod above y with ring"
        }
      }
    }, {
      "category": "Ll",
      "key": "1E9A",
      "mappings": {
        "default": {"default": "a right half ring"},
        "mathspeak": {
          "default": "modifying above a with right half ring",
          "brief": "mod above a with right half ring",
          "sbrief": "mod above a with right half ring"
        }
      }
    }, {
      "category": "Ll",
      "key": "1EA1",
      "mappings": {
        "default": {"default": "a underdot"},
        "mathspeak": {
          "default": "modifying below a with dot",
          "brief": "mod below a with dot",
          "sbrief": "mod below a with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1EA3",
      "mappings": {
        "default": {"default": "a hook"},
        "mathspeak": {
          "default": "modifying above a with hook",
          "brief": "mod above a with hook",
          "sbrief": "mod above a with hook"
        }
      }
    }, {
      "category": "Ll",
      "key": "1EB9",
      "mappings": {
        "default": {"default": "e underdot"},
        "mathspeak": {
          "default": "modifying below e with dot",
          "brief": "mod below e with dot",
          "sbrief": "mod below e with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1EBB",
      "mappings": {
        "default": {"default": "e hook"},
        "mathspeak": {
          "default": "modifying above e with hook",
          "brief": "mod above e with hook",
          "sbrief": "mod above e with hook"
        }
      }
    }, {
      "category": "Ll",
      "key": "1EBD",
      "mappings": {
        "default": {"default": "e tilde"},
        "mathspeak": {"default": "e overtilde", "brief": "e overtilde", "sbrief": "e overtilde"}
      }
    }, {
      "category": "Ll",
      "key": "1EC9",
      "mappings": {
        "default": {"default": "i hook"},
        "mathspeak": {
          "default": "modifying above i with hook",
          "brief": "mod above i with hook",
          "sbrief": "mod above i with hook"
        }
      }
    }, {
      "category": "Ll",
      "key": "1ECB",
      "mappings": {
        "default": {"default": "i underdot"},
        "mathspeak": {
          "default": "modifying below i with dot",
          "brief": "mod below i with dot",
          "sbrief": "mod below i with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1ECD",
      "mappings": {
        "default": {"default": "o underdot"},
        "mathspeak": {
          "default": "modifying below o with dot",
          "brief": "mod below o with dot",
          "sbrief": "mod below o with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1ECF",
      "mappings": {
        "default": {"default": "o hook"},
        "mathspeak": {
          "default": "modifying above o with hook",
          "brief": "mod above o with hook",
          "sbrief": "mod above o with hook"
        }
      }
    }, {
      "category": "Ll",
      "key": "1EE5",
      "mappings": {
        "default": {"default": "u underdot"},
        "mathspeak": {
          "default": "modifying below u with dot",
          "brief": "mod below u with dot",
          "sbrief": "mod below u with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1EE7",
      "mappings": {
        "default": {"default": "u hook"},
        "mathspeak": {
          "default": "modifying above u with hook",
          "brief": "mod above u with hook",
          "sbrief": "mod above u with hook"
        }
      }
    }, {
      "category": "Ll",
      "key": "1EF3",
      "mappings": {
        "default": {"default": "y grave"},
        "mathspeak": {
          "default": "modifying above y with grave",
          "brief": "mod above y with grave",
          "sbrief": "mod above y with grave"
        }
      }
    }, {
      "category": "Ll",
      "key": "1EF5",
      "mappings": {
        "default": {"default": "y underdot"},
        "mathspeak": {
          "default": "modifying below y with dot",
          "brief": "mod below y with dot",
          "sbrief": "mod below y with dot"
        }
      }
    }, {
      "category": "Ll",
      "key": "1EF7",
      "mappings": {
        "default": {"default": "y hook"},
        "mathspeak": {
          "default": "modifying above y with hook",
          "brief": "mod above y with hook",
          "sbrief": "mod above y with hook"
        }
      }
    }, {
      "category": "Ll",
      "key": "1EF9",
      "mappings": {
        "default": {"default": "y tilde"},
        "mathspeak": {"default": "y overtilde", "brief": "y overtilde", "sbrief": "y overtilde"}
      }
    }]
  ,
  "en/symbols/latin-rest.js":
    [{"locale": "en"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "italic h", "physics": "planck constant"}},
      "key": "210E"
    }, {"category": "Mn", "key": "0363", "mappings": {"default": {"default": "combining a"}}}, {
      "category": "Mn",
      "key": "0364",
      "mappings": {"default": {"default": "combining e"}}
    }, {"category": "Mn", "key": "0365", "mappings": {"default": {"default": "combining i"}}}, {
      "category": "Mn",
      "key": "0366",
      "mappings": {"default": {"default": "combining o"}}
    }, {"category": "Mn", "key": "0367", "mappings": {"default": {"default": "combining u"}}}, {
      "category": "Mn",
      "key": "0368",
      "mappings": {"default": {"default": "combining c"}}
    }, {"category": "Mn", "key": "0369", "mappings": {"default": {"default": "combining d"}}}, {
      "category": "Mn",
      "key": "036A",
      "mappings": {"default": {"default": "combining h"}}
    }, {"category": "Mn", "key": "036B", "mappings": {"default": {"default": "combining m"}}}, {
      "category": "Mn",
      "key": "036C",
      "mappings": {"default": {"default": "combining r"}}
    }, {"category": "Mn", "key": "036D", "mappings": {"default": {"default": "combining t"}}}, {
      "category": "Mn",
      "key": "036E",
      "mappings": {"default": {"default": "combining v"}}
    }, {"category": "Mn", "key": "036F", "mappings": {"default": {"default": "combining x"}}}, {
      "category": "Lm",
      "key": "1D62",
      "mappings": {"default": {"default": "subscript i"}}
    }, {"category": "Lm", "key": "1D63", "mappings": {"default": {"default": "subscript r"}}}, {
      "category": "Lm",
      "key": "1D64",
      "mappings": {"default": {"default": "subscript u"}}
    }, {"category": "Lm", "key": "1D65", "mappings": {"default": {"default": "subscript v"}}}, {
      "category": "Mn",
      "key": "1DCA",
      "mappings": {"default": {"default": "combining r below"}}
    }, {
      "category": "Mn",
      "key": "1DD3",
      "mappings": {"default": {"default": "combining flattened open a above"}}
    }, {"category": "Mn", "key": "1DD4", "mappings": {"default": {"default": "combining ae"}}}, {
      "category": "Mn",
      "key": "1DD5",
      "mappings": {"default": {"default": "combining ao"}}
    }, {"category": "Mn", "key": "1DD6", "mappings": {"default": {"default": "combining av"}}}, {
      "category": "Mn",
      "key": "1DD7",
      "mappings": {"default": {"default": "combining c cedilla"}}
    }, {
      "category": "Mn",
      "key": "1DD8",
      "mappings": {"default": {"default": "combining insular d"}}
    }, {"category": "Mn", "key": "1DD9", "mappings": {"default": {"default": "combining eth"}}}, {
      "category": "Mn",
      "key": "1DDA",
      "mappings": {"default": {"default": "combining g"}}
    }, {
      "category": "Mn",
      "key": "1DDB",
      "mappings": {"default": {"default": "combining small cap G"}, "mathspeak": {"default": "combining small upper G"}}
    }, {"category": "Mn", "key": "1DDC", "mappings": {"default": {"default": "combining k"}}}, {
      "category": "Mn",
      "key": "1DDD",
      "mappings": {"default": {"default": "combining l"}}
    }, {
      "category": "Mn",
      "key": "1DDE",
      "mappings": {"default": {"default": "combining small cap L"}, "mathspeak": {"default": "combining small upper L"}}
    }, {
      "category": "Mn",
      "key": "1DDF",
      "mappings": {"default": {"default": "combining small cap M"}, "mathspeak": {"default": "combining small upper M"}}
    }, {"category": "Mn", "key": "1DE0", "mappings": {"default": {"default": "combining n"}}}, {
      "category": "Mn",
      "key": "1DE1",
      "mappings": {"default": {"default": "combining small cap N"}, "mathspeak": {"default": "combining small upper N"}}
    }, {
      "category": "Mn",
      "key": "1DE2",
      "mappings": {"default": {"default": "combining small cap R"}, "mathspeak": {"default": "combining small upper R"}}
    }, {
      "category": "Mn",
      "key": "1DE3",
      "mappings": {"default": {"default": "combining r rotunda"}}
    }, {"category": "Mn", "key": "1DE4", "mappings": {"default": {"default": "combining s"}}}, {
      "category": "Mn",
      "key": "1DE5",
      "mappings": {"default": {"default": "combining long s"}}
    }, {"category": "Mn", "key": "1DE6", "mappings": {"default": {"default": "combining z"}}}, {
      "category": "Lm",
      "key": "2071",
      "mappings": {"default": {"default": "superscript i"}}
    }, {"category": "Lm", "key": "207F", "mappings": {"default": {"default": "superscript n"}}}, {
      "category": "Lm",
      "key": "2090",
      "mappings": {"default": {"default": "subscript a"}}
    }, {"category": "Lm", "key": "2091", "mappings": {"default": {"default": "subscript e"}}}, {
      "category": "Lm",
      "key": "2092",
      "mappings": {"default": {"default": "subscript o"}}
    }, {"category": "Lm", "key": "2093", "mappings": {"default": {"default": "subscript x"}}}, {
      "category": "Lm",
      "key": "2094",
      "mappings": {"default": {"default": "subscript schwa"}}
    }, {"category": "Lm", "key": "2095", "mappings": {"default": {"default": "subscript h"}}}, {
      "category": "Lm",
      "key": "2096",
      "mappings": {"default": {"default": "subscript k"}}
    }, {"category": "Lm", "key": "2097", "mappings": {"default": {"default": "subscript l"}}}, {
      "category": "Lm",
      "key": "2098",
      "mappings": {"default": {"default": "subscript m"}}
    }, {"category": "Lm", "key": "2099", "mappings": {"default": {"default": "subscript n"}}}, {
      "category": "Lm",
      "key": "209A",
      "mappings": {"default": {"default": "subscript p"}}
    }, {"category": "Lm", "key": "209B", "mappings": {"default": {"default": "subscript s"}}}, {
      "category": "Lm",
      "key": "209C",
      "mappings": {"default": {"default": "subscript t"}}
    }, {"category": "Lm", "key": "2C7C", "mappings": {"default": {"default": "subscript j"}}}, {
      "category": "So",
      "key": "1F12A",
      "mappings": {
        "default": {"default": "tortoise shell bracketed cap S"},
        "mathspeak": {"default": "tortoise shell bracketed upper S"}
      }
    }, {
      "category": "So",
      "key": "1F12B",
      "mappings": {"default": {"default": "circled italic cap C"}, "mathspeak": {"default": "circled italic upper C"}}
    }, {
      "category": "So",
      "key": "1F12C",
      "mappings": {"default": {"default": "circled italic cap R"}, "mathspeak": {"default": "circled italic upper R"}}
    }, {
      "category": "So",
      "key": "1F18A",
      "mappings": {
        "default": {"default": "crossed negative squared cap P"},
        "mathspeak": {"default": "crossed negative squared upper P"}
      }
    }]
  ,
  "en/symbols/latin-upper-double-accent.js":
    [{"locale": "en"}, {
      "category": "Lu",
      "key": "01D5",
      "mappings": {
        "default": {"default": "cap U double overdot overbar"},
        "mathspeak": {"default": "upper U double overdot overbar"}
      }
    }, {
      "category": "Lu",
      "key": "01D7",
      "mappings": {
        "default": {"default": "cap U double overdot acute"},
        "mathspeak": {"default": "upper U double overdot acute"}
      }
    }, {
      "category": "Lu",
      "key": "01D9",
      "mappings": {
        "default": {"default": "cap U double overdot caron"},
        "mathspeak": {"default": "upper U double overdot caron"}
      }
    }, {
      "category": "Lu",
      "key": "01DB",
      "mappings": {
        "default": {"default": "cap U double overdot grave"},
        "mathspeak": {"default": "upper U double overdot grave"}
      }
    }, {
      "category": "Lu",
      "key": "01DE",
      "mappings": {
        "default": {"default": "cap A double overdot overbar"},
        "mathspeak": {"default": "upper A double overdot overbar"}
      }
    }, {
      "category": "Lu",
      "key": "01E0",
      "mappings": {"default": {"default": "cap A overdot overbar"}, "mathspeak": {"default": "upper A overdot overbar"}}
    }, {
      "category": "Lu",
      "key": "01EC",
      "mappings": {"default": {"default": "cap O ogonek overbar"}, "mathspeak": {"default": "upper O ogonek overbar"}}
    }, {
      "category": "Lu",
      "key": "01FA",
      "mappings": {"default": {"default": "cap A ring acute"}, "mathspeak": {"default": "upper A ring acute"}}
    }, {
      "category": "Lu",
      "key": "022A",
      "mappings": {
        "default": {"default": "cap O double overdot overbar"},
        "mathspeak": {"default": "upper O double overdot overbar"}
      }
    }, {
      "category": "Lu",
      "key": "022C",
      "mappings": {"default": {"default": "cap O tilde overbar"}, "mathspeak": {"default": "upper O tilde overbar"}}
    }, {
      "category": "Lu",
      "key": "0230",
      "mappings": {"default": {"default": "cap O overdot overbar"}, "mathspeak": {"default": "upper O overdot overbar"}}
    }, {
      "category": "Lu",
      "key": "1E08",
      "mappings": {"default": {"default": "cap C cedilla acute"}, "mathspeak": {"default": "upper C cedilla acute"}}
    }, {
      "category": "Lu",
      "key": "1E14",
      "mappings": {"default": {"default": "cap E overbar grave"}, "mathspeak": {"default": "upper E overbar grave"}}
    }, {
      "category": "Lu",
      "key": "1E16",
      "mappings": {"default": {"default": "cap E overbar acute"}, "mathspeak": {"default": "upper E overbar acute"}}
    }, {
      "category": "Lu",
      "key": "1E1C",
      "mappings": {"default": {"default": "cap E cedilla breve"}, "mathspeak": {"default": "upper E cedilla breve"}}
    }, {
      "category": "Lu",
      "key": "1E2E",
      "mappings": {
        "default": {"default": "cap I double overdot acute"},
        "mathspeak": {"default": "upper I double overdot acute"}
      }
    }, {
      "category": "Lu",
      "key": "1E38",
      "mappings": {
        "default": {"default": "cap L underdot overbar"},
        "mathspeak": {"default": "upper L underdot overbar"}
      }
    }, {
      "category": "Lu",
      "key": "1E4C",
      "mappings": {"default": {"default": "cap O tilde acute"}, "mathspeak": {"default": "upper O tilde acute"}}
    }, {
      "category": "Lu",
      "key": "1E4E",
      "mappings": {
        "default": {"default": "cap O tilde double overdot"},
        "mathspeak": {"default": "upper O tilde double overdot"}
      }
    }, {
      "category": "Lu",
      "key": "1E50",
      "mappings": {"default": {"default": "cap O overbar grave"}, "mathspeak": {"default": "upper O overbar grave"}}
    }, {
      "category": "Lu",
      "key": "1E52",
      "mappings": {"default": {"default": "cap O overbar acute"}, "mathspeak": {"default": "upper O overbar acute"}}
    }, {
      "category": "Lu",
      "key": "1E5C",
      "mappings": {
        "default": {"default": "cap R overbar underdot"},
        "mathspeak": {"default": "upper R overbar underdot"}
      }
    }, {
      "category": "Lu",
      "key": "1E64",
      "mappings": {"default": {"default": "cap S acute overdot"}, "mathspeak": {"default": "upper S acute overdot"}}
    }, {
      "category": "Lu",
      "key": "1E66",
      "mappings": {"default": {"default": "cap S caron overdot"}, "mathspeak": {"default": "upper S caron overdot"}}
    }, {
      "category": "Lu",
      "key": "1E68",
      "mappings": {
        "default": {"default": "cap S underdot overdot"},
        "mathspeak": {"default": "upper S underdot overdot"}
      }
    }, {
      "category": "Lu",
      "key": "1E78",
      "mappings": {"default": {"default": "cap U tilde acute"}, "mathspeak": {"default": "upper U tilde acute"}}
    }, {
      "category": "Lu",
      "key": "1E7A",
      "mappings": {
        "default": {"default": "cap U overbar double overdot"},
        "mathspeak": {"default": "upper U overbar double overdot"}
      }
    }, {
      "category": "Lu",
      "key": "1EA4",
      "mappings": {"default": {"default": "cap A hat acute"}, "mathspeak": {"default": "upper A hat acute"}}
    }, {
      "category": "Lu",
      "key": "1EA6",
      "mappings": {"default": {"default": "cap A hat grave"}, "mathspeak": {"default": "upper A hat grave"}}
    }, {
      "category": "Lu",
      "key": "1EA8",
      "mappings": {"default": {"default": "cap A hat hook"}, "mathspeak": {"default": "upper A hat hook"}}
    }, {
      "category": "Lu",
      "key": "1EAA",
      "mappings": {"default": {"default": "cap A hat tilde"}, "mathspeak": {"default": "upper A hat tilde"}}
    }, {
      "category": "Lu",
      "key": "1EAC",
      "mappings": {"default": {"default": "cap A hat underdot"}, "mathspeak": {"default": "upper A hat underdot"}}
    }, {
      "category": "Lu",
      "key": "1EAE",
      "mappings": {"default": {"default": "cap A breve acute"}, "mathspeak": {"default": "upper A breve acute"}}
    }, {
      "category": "Lu",
      "key": "1EB0",
      "mappings": {"default": {"default": "cap A breve grave"}, "mathspeak": {"default": "upper A breve grave"}}
    }, {
      "category": "Lu",
      "key": "1EB2",
      "mappings": {"default": {"default": "cap A breve hook"}, "mathspeak": {"default": "upper A breve hook"}}
    }, {
      "category": "Lu",
      "key": "1EB4",
      "mappings": {"default": {"default": "cap A breve tilde"}, "mathspeak": {"default": "upper A breve tilde"}}
    }, {
      "category": "Lu",
      "key": "1EB6",
      "mappings": {"default": {"default": "cap A breve underdot"}, "mathspeak": {"default": "upper A breve underdot"}}
    }, {
      "category": "Lu",
      "key": "1EBE",
      "mappings": {"default": {"default": "cap E hat acute"}, "mathspeak": {"default": "upper E hat acute"}}
    }, {
      "category": "Lu",
      "key": "1EC0",
      "mappings": {"default": {"default": "cap E hat grave"}, "mathspeak": {"default": "upper E hat grave"}}
    }, {
      "category": "Lu",
      "key": "1EC2",
      "mappings": {"default": {"default": "cap E hat hook"}, "mathspeak": {"default": "upper E hat hook"}}
    }, {
      "category": "Lu",
      "key": "1EC4",
      "mappings": {"default": {"default": "cap E hat tilde"}, "mathspeak": {"default": "upper E hat tilde"}}
    }, {
      "category": "Lu",
      "key": "1EC6",
      "mappings": {"default": {"default": "cap E hat underdot"}, "mathspeak": {"default": "upper E hat underdot"}}
    }, {
      "category": "Lu",
      "key": "1ED0",
      "mappings": {"default": {"default": "cap O hat acute"}, "mathspeak": {"default": "upper O hat acute"}}
    }, {
      "category": "Lu",
      "key": "1ED2",
      "mappings": {"default": {"default": "cap O hat grave"}, "mathspeak": {"default": "upper O hat grave"}}
    }, {
      "category": "Lu",
      "key": "1ED4",
      "mappings": {"default": {"default": "cap O hat hook"}, "mathspeak": {"default": "upper O hat hook"}}
    }, {
      "category": "Lu",
      "key": "1ED6",
      "mappings": {"default": {"default": "cap O hat tilde"}, "mathspeak": {"default": "upper O hat tilde"}}
    }, {
      "category": "Lu",
      "key": "1ED8",
      "mappings": {"default": {"default": "cap O hat underdot"}, "mathspeak": {"default": "upper O hat underdot"}}
    }, {
      "category": "Lu",
      "key": "1EDA",
      "mappings": {"default": {"default": "cap O acute prime"}, "mathspeak": {"default": "upper O acute prime"}}
    }, {
      "category": "Lu",
      "key": "1EDC",
      "mappings": {"default": {"default": "cap O grave prime"}, "mathspeak": {"default": "upper O grave prime"}}
    }, {
      "category": "Lu",
      "key": "1EDE",
      "mappings": {"default": {"default": "cap O hook prime"}, "mathspeak": {"default": "upper O hook prime"}}
    }, {
      "category": "Lu",
      "key": "1EE0",
      "mappings": {"default": {"default": "cap O tilde prime"}, "mathspeak": {"default": "upper O tilde prime"}}
    }, {
      "category": "Lu",
      "key": "1EE2",
      "mappings": {"default": {"default": "cap O underdot prime"}, "mathspeak": {"default": "upper O underdot prime"}}
    }, {
      "category": "Lu",
      "key": "1EE8",
      "mappings": {"default": {"default": "cap U acute prime"}, "mathspeak": {"default": "upper U acute prime"}}
    }, {
      "category": "Lu",
      "key": "1EEA",
      "mappings": {"default": {"default": "cap U grave prime"}, "mathspeak": {"default": "upper U grave prime"}}
    }, {
      "category": "Lu",
      "key": "1EEC",
      "mappings": {"default": {"default": "cap U hook prime"}, "mathspeak": {"default": "upper U hook prime"}}
    }, {
      "category": "Lu",
      "key": "1EEE",
      "mappings": {"default": {"default": "cap U tilde prime"}, "mathspeak": {"default": "upper U tilde prime"}}
    }, {
      "category": "Lu",
      "key": "1EF0",
      "mappings": {"default": {"default": "cap U underdot prime"}, "mathspeak": {"default": "upper U underdot prime"}}
    }]
  ,
  "en/symbols/latin-upper-single-accent.js":
    [{"locale": "en"}, {
      "category": "Lu",
      "key": "00C0",
      "mappings": {
        "default": {"default": "cap A grave"},
        "mathspeak": {
          "default": "modifying above upper A with grave",
          "brief": "mod above upper A with grave",
          "sbrief": "mod above upper A with grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "00C1",
      "mappings": {
        "default": {"default": "cap A acute"},
        "mathspeak": {
          "default": "modifying above upper A with acute",
          "brief": "mod above upper A with acute",
          "sbrief": "mod above upper A with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "00C2",
      "mappings": {
        "default": {"default": "cap A hat"},
        "mathspeak": {
          "default": "modifying above upper A with caret",
          "brief": "mod above upper A with caret",
          "sbrief": "mod above upper A with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "00C3",
      "mappings": {
        "default": {"default": "cap A tilde"},
        "mathspeak": {"default": "upper A overtilde", "brief": "upper A overtilde", "sbrief": "upper A overtilde"}
      }
    }, {
      "category": "Lu",
      "key": "00C4",
      "mappings": {
        "default": {"default": "cap A double overdot"},
        "mathspeak": {
          "default": "modifying above upper A with double dot",
          "brief": "mod above upper A with double dot",
          "sbrief": "mod above upper A with double dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "00C5",
      "mappings": {
        "default": {"default": "cap A ring"},
        "mathspeak": {
          "default": "modifying above upper A with ring",
          "brief": "mod above upper A with ring",
          "sbrief": "mod above upper A with ring"
        }
      }
    }, {
      "category": "Lu",
      "key": "00C7",
      "mappings": {
        "default": {"default": "cap C cedilla"},
        "mathspeak": {
          "default": "modifying above upper C with cedilla",
          "brief": "mod above upper C with cedilla",
          "sbrief": "mod above upper C with cedilla"
        }
      }
    }, {
      "category": "Lu",
      "key": "00C8",
      "mappings": {
        "default": {"default": "cap E grave"},
        "mathspeak": {
          "default": "modifying above upper E with grave",
          "brief": "mod above upper E with grave",
          "sbrief": "mod above upper E with grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "00C9",
      "mappings": {
        "default": {"default": "cap E acute"},
        "mathspeak": {
          "default": "modifying above upper E with acute",
          "brief": "mod above upper E with acute",
          "sbrief": "mod above upper E with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "00CA",
      "mappings": {
        "default": {"default": "cap E hat"},
        "mathspeak": {
          "default": "modifying above upper E with caret",
          "brief": "mod above upper E with caret",
          "sbrief": "mod above upper E with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "00CB",
      "mappings": {
        "default": {"default": "cap E double overdot"},
        "mathspeak": {
          "default": "modifying above upper E with double dot",
          "brief": "mod above upper E with double dot",
          "sbrief": "mod above upper E with double dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "00CC",
      "mappings": {
        "default": {"default": "cap I grave"},
        "mathspeak": {
          "default": "modifying above upper I with grave",
          "brief": "mod above upper I with grave",
          "sbrief": "mod above upper I with grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "00CD",
      "mappings": {
        "default": {"default": "cap I acute"},
        "mathspeak": {
          "default": "modifying above upper I with acute",
          "brief": "mod above upper I with acute",
          "sbrief": "mod above upper I with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "00CE",
      "mappings": {
        "default": {"default": "cap I hat"},
        "mathspeak": {
          "default": "modifying above upper I with caret",
          "brief": "mod above upper I with caret",
          "sbrief": "mod above upper I with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "00CF",
      "mappings": {
        "default": {"default": "cap I double overdot"},
        "mathspeak": {
          "default": "modifying above upper I with double dot",
          "brief": "mod above upper I with double dot",
          "sbrief": "mod above upper I with double dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "00D1",
      "mappings": {
        "default": {"default": "cap N tilde"},
        "mathspeak": {"default": "upper N overtilde", "brief": "upper N overtilde", "sbrief": "upper N overtilde"}
      }
    }, {
      "category": "Lu",
      "key": "00D2",
      "mappings": {
        "default": {"default": "cap O grave"},
        "mathspeak": {
          "default": "modifying above upper O with grave",
          "brief": "mod above upper O with grave",
          "sbrief": "mod above upper O with grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "00D3",
      "mappings": {
        "default": {"default": "cap O acute"},
        "mathspeak": {
          "default": "modifying above upper O with acute",
          "brief": "mod above upper O with acute",
          "sbrief": "mod above upper O with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "00D4",
      "mappings": {
        "default": {"default": "cap O hat"},
        "mathspeak": {
          "default": "modifying above upper O with caret",
          "brief": "mod above upper O with caret",
          "sbrief": "mod above upper O with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "00D5",
      "mappings": {
        "default": {"default": "cap O tilde"},
        "mathspeak": {"default": "upper O overtilde", "brief": "upper O overtilde", "sbrief": "upper O overtilde"}
      }
    }, {
      "category": "Lu",
      "key": "00D6",
      "mappings": {
        "default": {"default": "cap O double overdot"},
        "mathspeak": {
          "default": "modifying above upper O with double dot",
          "brief": "mod above upper O with double dot",
          "sbrief": "mod above upper O with double dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "00D9",
      "mappings": {
        "default": {"default": "cap U grave"},
        "mathspeak": {
          "default": "modifying above upper U with grave",
          "brief": "mod above upper U with grave",
          "sbrief": "mod above upper U with grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "00DA",
      "mappings": {
        "default": {"default": "cap U acute"},
        "mathspeak": {
          "default": "modifying above upper U with acute",
          "brief": "mod above upper U with acute",
          "sbrief": "mod above upper U with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "00DB",
      "mappings": {
        "default": {"default": "cap U hat"},
        "mathspeak": {
          "default": "modifying above upper U with caret",
          "brief": "mod above upper U with caret",
          "sbrief": "mod above upper U with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "00DC",
      "mappings": {
        "default": {"default": "cap U double overdot"},
        "mathspeak": {
          "default": "modifying above upper U with double dot",
          "brief": "mod above upper U with double dot",
          "sbrief": "mod above upper U with double dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "00DD",
      "mappings": {
        "default": {"default": "cap Y acute"},
        "mathspeak": {
          "default": "modifying above upper Y with acute",
          "brief": "mod above upper Y with acute",
          "sbrief": "mod above upper Y with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "0100",
      "mappings": {
        "default": {"default": "cap A overbar"},
        "mathspeak": {"default": "upper A overbar", "brief": "upper A overbar", "sbrief": "upper A overbar"}
      }
    }, {
      "category": "Lu",
      "key": "0102",
      "mappings": {
        "default": {"default": "cap A breve"},
        "mathspeak": {
          "default": "modifying above upper A with breve",
          "brief": "mod above upper A with breve",
          "sbrief": "mod above upper A with breve"
        }
      }
    }, {
      "category": "Lu",
      "key": "0104",
      "mappings": {
        "default": {"default": "cap A ogonek"},
        "mathspeak": {
          "default": "modifying above upper A with ogonek",
          "brief": "mod above upper A with ogonek",
          "sbrief": "mod above upper A with ogonek"
        }
      }
    }, {
      "category": "Lu",
      "key": "0106",
      "mappings": {
        "default": {"default": "cap C acute"},
        "mathspeak": {
          "default": "modifying above upper C with acute",
          "brief": "mod above upper C with acute",
          "sbrief": "mod above upper C with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "0108",
      "mappings": {
        "default": {"default": "cap C hat"},
        "mathspeak": {
          "default": "modifying above upper C with caret",
          "brief": "mod above upper C with caret",
          "sbrief": "mod above upper C with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "010A",
      "mappings": {
        "default": {"default": "cap C overdot"},
        "mathspeak": {
          "default": "modifying above upper C with dot",
          "brief": "mod above upper C with dot",
          "sbrief": "mod above upper C with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "010C",
      "mappings": {
        "default": {"default": "cap C caron"},
        "mathspeak": {
          "default": "modifying above upper C with caron",
          "brief": "mod above upper C with caron",
          "sbrief": "mod above upper C with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "010E",
      "mappings": {
        "default": {"default": "cap D caron"},
        "mathspeak": {
          "default": "modifying above upper D with caron",
          "brief": "mod above upper D with caron",
          "sbrief": "mod above upper D with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "0112",
      "mappings": {
        "default": {"default": "cap E overbar"},
        "mathspeak": {"default": "upper E overbar", "brief": "upper E overbar", "sbrief": "upper E overbar"}
      }
    }, {
      "category": "Lu",
      "key": "0114",
      "mappings": {
        "default": {"default": "cap E breve"},
        "mathspeak": {
          "default": "modifying above upper E with breve",
          "brief": "mod above upper E with breve",
          "sbrief": "mod above upper E with breve"
        }
      }
    }, {
      "category": "Lu",
      "key": "0116",
      "mappings": {
        "default": {"default": "cap E overdot"},
        "mathspeak": {
          "default": "modifying above upper E with dot",
          "brief": "mod above upper E with dot",
          "sbrief": "mod above upper E with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "0118",
      "mappings": {
        "default": {"default": "cap E ogonek"},
        "mathspeak": {
          "default": "modifying above upper E with ogonek",
          "brief": "mod above upper E with ogonek",
          "sbrief": "mod above upper E with ogonek"
        }
      }
    }, {
      "category": "Lu",
      "key": "011A",
      "mappings": {
        "default": {"default": "cap E caron"},
        "mathspeak": {
          "default": "modifying above upper E with caron",
          "brief": "mod above upper E with caron",
          "sbrief": "mod above upper E with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "011C",
      "mappings": {
        "default": {"default": "cap G hat"},
        "mathspeak": {
          "default": "modifying above upper G with caret",
          "brief": "mod above upper G with caret",
          "sbrief": "mod above upper G with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "011E",
      "mappings": {
        "default": {"default": "cap G breve"},
        "mathspeak": {
          "default": "modifying above upper G with breve",
          "brief": "mod above upper G with breve",
          "sbrief": "mod above upper G with breve"
        }
      }
    }, {
      "category": "Lu",
      "key": "0120",
      "mappings": {
        "default": {"default": "cap G overdot"},
        "mathspeak": {
          "default": "modifying above upper G with dot",
          "brief": "mod above upper G with dot",
          "sbrief": "mod above upper G with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "0122",
      "mappings": {
        "default": {"default": "cap G cedilla"},
        "mathspeak": {
          "default": "modifying above upper G with cedilla",
          "brief": "mod above upper G with cedilla",
          "sbrief": "mod above upper G with cedilla"
        }
      }
    }, {
      "category": "Lu",
      "key": "0124",
      "mappings": {
        "default": {"default": "cap H hat"},
        "mathspeak": {
          "default": "modifying above upper H with caret",
          "brief": "mod above upper H with caret",
          "sbrief": "mod above upper H with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "0128",
      "mappings": {
        "default": {"default": "cap I tilde"},
        "mathspeak": {"default": "upper I overtilde", "brief": "upper I overtilde", "sbrief": "upper I overtilde"}
      }
    }, {
      "category": "Lu",
      "key": "012A",
      "mappings": {
        "default": {"default": "cap I overbar"},
        "mathspeak": {"default": "upper I overbar", "brief": "upper I overbar", "sbrief": "upper I overbar"}
      }
    }, {
      "category": "Lu",
      "key": "012C",
      "mappings": {
        "default": {"default": "cap I breve"},
        "mathspeak": {
          "default": "modifying above upper I with breve",
          "brief": "mod above upper I with breve",
          "sbrief": "mod above upper I with breve"
        }
      }
    }, {
      "category": "Lu",
      "key": "012E",
      "mappings": {
        "default": {"default": "cap I ogonek"},
        "mathspeak": {
          "default": "modifying above upper I with ogonek",
          "brief": "mod above upper I with ogonek",
          "sbrief": "mod above upper I with ogonek"
        }
      }
    }, {
      "category": "Lu",
      "key": "0130",
      "mappings": {
        "default": {"default": "cap I overdot"},
        "mathspeak": {
          "default": "modifying above upper I with dot",
          "brief": "mod above upper I with dot",
          "sbrief": "mod above upper I with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "0134",
      "mappings": {
        "default": {"default": "cap J hat"},
        "mathspeak": {
          "default": "modifying above upper J with caret",
          "brief": "mod above upper J with caret",
          "sbrief": "mod above upper J with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "0136",
      "mappings": {
        "default": {"default": "cap K cedilla"},
        "mathspeak": {
          "default": "modifying above upper K with cedilla",
          "brief": "mod above upper K with cedilla",
          "sbrief": "mod above upper K with cedilla"
        }
      }
    }, {
      "category": "Lu",
      "key": "0139",
      "mappings": {
        "default": {"default": "cap L acute"},
        "mathspeak": {
          "default": "modifying above upper L with acute",
          "brief": "mod above upper L with acute",
          "sbrief": "mod above upper L with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "013B",
      "mappings": {
        "default": {"default": "cap L cedilla"},
        "mathspeak": {
          "default": "modifying above upper L with cedilla",
          "brief": "mod above upper L with cedilla",
          "sbrief": "mod above upper L with cedilla"
        }
      }
    }, {
      "category": "Lu",
      "key": "013D",
      "mappings": {
        "default": {"default": "cap L caron"},
        "mathspeak": {
          "default": "modifying above upper L with caron",
          "brief": "mod above upper L with caron",
          "sbrief": "mod above upper L with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "013F",
      "mappings": {
        "default": {"default": "cap L middle dot"},
        "mathspeak": {
          "default": "modifying above upper L with middle dot",
          "brief": "mod above upper L with middle dot",
          "sbrief": "mod above upper L with middle dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "0143",
      "mappings": {
        "default": {"default": "cap N acute"},
        "mathspeak": {
          "default": "modifying above upper N with acute",
          "brief": "mod above upper N with acute",
          "sbrief": "mod above upper N with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "0145",
      "mappings": {
        "default": {"default": "cap N cedilla"},
        "mathspeak": {
          "default": "modifying above upper N with cedilla",
          "brief": "mod above upper N with cedilla",
          "sbrief": "mod above upper N with cedilla"
        }
      }
    }, {
      "category": "Lu",
      "key": "0147",
      "mappings": {
        "default": {"default": "cap N caron"},
        "mathspeak": {
          "default": "modifying above upper N with caron",
          "brief": "mod above upper N with caron",
          "sbrief": "mod above upper N with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "014C",
      "mappings": {
        "default": {"default": "cap O overbar"},
        "mathspeak": {"default": "upper O overbar", "brief": "upper O overbar", "sbrief": "upper O overbar"}
      }
    }, {
      "category": "Lu",
      "key": "014E",
      "mappings": {
        "default": {"default": "cap O breve"},
        "mathspeak": {
          "default": "modifying above upper O with breve",
          "brief": "mod above upper O with breve",
          "sbrief": "mod above upper O with breve"
        }
      }
    }, {
      "category": "Lu",
      "key": "0150",
      "mappings": {
        "default": {"default": "cap O double acute"},
        "mathspeak": {
          "default": "modifying above upper O with double acute",
          "brief": "mod above upper O with double acute",
          "sbrief": "mod above upper O with double acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "0154",
      "mappings": {
        "default": {"default": "cap R acute"},
        "mathspeak": {
          "default": "modifying above upper R with acute",
          "brief": "mod above upper R with acute",
          "sbrief": "mod above upper R with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "0156",
      "mappings": {
        "default": {"default": "cap R cedilla"},
        "mathspeak": {
          "default": "modifying above upper R with cedilla",
          "brief": "mod above upper R with cedilla",
          "sbrief": "mod above upper R with cedilla"
        }
      }
    }, {
      "category": "Lu",
      "key": "0158",
      "mappings": {
        "default": {"default": "cap R caron"},
        "mathspeak": {
          "default": "modifying above upper R with caron",
          "brief": "mod above upper R with caron",
          "sbrief": "mod above upper R with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "015A",
      "mappings": {
        "default": {"default": "cap S acute"},
        "mathspeak": {
          "default": "modifying above upper S with acute",
          "brief": "mod above upper S with acute",
          "sbrief": "mod above upper S with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "015C",
      "mappings": {
        "default": {"default": "cap S hat"},
        "mathspeak": {
          "default": "modifying above upper S with caret",
          "brief": "mod above upper S with caret",
          "sbrief": "mod above upper S with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "015E",
      "mappings": {
        "default": {"default": "cap S cedilla"},
        "mathspeak": {
          "default": "modifying above upper S with cedilla",
          "brief": "mod above upper S with cedilla",
          "sbrief": "mod above upper S with cedilla"
        }
      }
    }, {
      "category": "Lu",
      "key": "0160",
      "mappings": {
        "default": {"default": "cap S caron"},
        "mathspeak": {
          "default": "modifying above upper S with caron",
          "brief": "mod above upper S with caron",
          "sbrief": "mod above upper S with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "0162",
      "mappings": {
        "default": {"default": "cap T cedilla"},
        "mathspeak": {
          "default": "modifying above upper T with cedilla",
          "brief": "mod above upper T with cedilla",
          "sbrief": "mod above upper T with cedilla"
        }
      }
    }, {
      "category": "Lu",
      "key": "0164",
      "mappings": {
        "default": {"default": "cap T caron"},
        "mathspeak": {
          "default": "modifying above upper T with caron",
          "brief": "mod above upper T with caron",
          "sbrief": "mod above upper T with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "0168",
      "mappings": {
        "default": {"default": "cap U tilde"},
        "mathspeak": {"default": "upper U overtilde", "brief": "upper U overtilde", "sbrief": "upper U overtilde"}
      }
    }, {
      "category": "Lu",
      "key": "016A",
      "mappings": {
        "default": {"default": "cap U overbar"},
        "mathspeak": {"default": "upper U overbar", "brief": "upper U overbar", "sbrief": "upper U overbar"}
      }
    }, {
      "category": "Lu",
      "key": "016C",
      "mappings": {
        "default": {"default": "cap U breve"},
        "mathspeak": {
          "default": "modifying above upper U with breve",
          "brief": "mod above upper U with breve",
          "sbrief": "mod above upper U with breve"
        }
      }
    }, {
      "category": "Lu",
      "key": "016E",
      "mappings": {
        "default": {"default": "cap U ring"},
        "mathspeak": {
          "default": "modifying above upper U with ring",
          "brief": "mod above upper U with ring",
          "sbrief": "mod above upper U with ring"
        }
      }
    }, {
      "category": "Lu",
      "key": "0170",
      "mappings": {
        "default": {"default": "cap U double acute"},
        "mathspeak": {
          "default": "modifying above upper U with double acute",
          "brief": "mod above upper U with double acute",
          "sbrief": "mod above upper U with double acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "0172",
      "mappings": {
        "default": {"default": "cap U ogonek"},
        "mathspeak": {
          "default": "modifying above upper U with ogonek",
          "brief": "mod above upper U with ogonek",
          "sbrief": "mod above upper U with ogonek"
        }
      }
    }, {
      "category": "Lu",
      "key": "0174",
      "mappings": {
        "default": {"default": "cap W hat"},
        "mathspeak": {
          "default": "modifying above upper W with caret",
          "brief": "mod above upper W with caret",
          "sbrief": "mod above upper W with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "0176",
      "mappings": {
        "default": {"default": "cap Y hat"},
        "mathspeak": {
          "default": "modifying above upper Y with caret",
          "brief": "mod above upper Y with caret",
          "sbrief": "mod above upper Y with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "0178",
      "mappings": {
        "default": {"default": "cap Y double overdot"},
        "mathspeak": {
          "default": "modifying above upper Y with double dot",
          "brief": "mod above upper Y with double dot",
          "sbrief": "mod above upper Y with double dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "0179",
      "mappings": {
        "default": {"default": "cap Z acute"},
        "mathspeak": {
          "default": "modifying above upper Z with acute",
          "brief": "mod above upper Z with acute",
          "sbrief": "mod above upper Z with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "017B",
      "mappings": {
        "default": {"default": "cap Z overdot"},
        "mathspeak": {
          "default": "modifying above upper Z with dot",
          "brief": "mod above upper Z with dot",
          "sbrief": "mod above upper Z with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "017D",
      "mappings": {
        "default": {"default": "cap Z caron"},
        "mathspeak": {
          "default": "modifying above upper Z with caron",
          "brief": "mod above upper Z with caron",
          "sbrief": "mod above upper Z with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "01CD",
      "mappings": {
        "default": {"default": "cap A caron"},
        "mathspeak": {
          "default": "modifying above upper A with caron",
          "brief": "mod above upper A with caron",
          "sbrief": "mod above upper A with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "01CF",
      "mappings": {
        "default": {"default": "cap I caron"},
        "mathspeak": {
          "default": "modifying above upper I with caron",
          "brief": "mod above upper I with caron",
          "sbrief": "mod above upper I with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "01D1",
      "mappings": {
        "default": {"default": "cap O caron"},
        "mathspeak": {
          "default": "modifying above upper O with caron",
          "brief": "mod above upper O with caron",
          "sbrief": "mod above upper O with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "01D3",
      "mappings": {
        "default": {"default": "cap U caron"},
        "mathspeak": {
          "default": "modifying above upper U with caron",
          "brief": "mod above upper U with caron",
          "sbrief": "mod above upper U with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "01E6",
      "mappings": {
        "default": {"default": "cap G caron"},
        "mathspeak": {
          "default": "modifying above upper G with caron",
          "brief": "mod above upper G with caron",
          "sbrief": "mod above upper G with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "01E8",
      "mappings": {
        "default": {"default": "cap K caron"},
        "mathspeak": {
          "default": "modifying above upper K with caron",
          "brief": "mod above upper K with caron",
          "sbrief": "mod above upper K with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "01EA",
      "mappings": {
        "default": {"default": "cap O ogonek"},
        "mathspeak": {
          "default": "modifying above upper O with ogonek",
          "brief": "mod above upper O with ogonek",
          "sbrief": "mod above upper O with ogonek"
        }
      }
    }, {
      "category": "Lu",
      "key": "01F4",
      "mappings": {
        "default": {"default": "cap G acute"},
        "mathspeak": {
          "default": "modifying above upper G with acute",
          "brief": "mod above upper G with acute",
          "sbrief": "mod above upper G with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "01F8",
      "mappings": {
        "default": {"default": "cap N grave"},
        "mathspeak": {
          "default": "modifying above upper N with grave",
          "brief": "mod above upper N with grave",
          "sbrief": "mod above upper N with grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "0200",
      "mappings": {
        "default": {"default": "cap A double grave"},
        "mathspeak": {
          "default": "modifying above upper A with double grave",
          "brief": "mod above upper A with double grave",
          "sbrief": "mod above upper A with double grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "0202",
      "mappings": {
        "default": {"default": "cap A inverted breve"},
        "mathspeak": {
          "default": "modifying above upper A with inverted breve",
          "brief": "mod above upper A with inverted breve",
          "sbrief": "mod above upper A with inverted breve"
        }
      }
    }, {
      "category": "Lu",
      "key": "0204",
      "mappings": {
        "default": {"default": "cap E double grave"},
        "mathspeak": {
          "default": "modifying above upper E with double grave",
          "brief": "mod above upper E with double grave",
          "sbrief": "mod above upper E with double grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "0206",
      "mappings": {
        "default": {"default": "cap E inverted breve"},
        "mathspeak": {
          "default": "modifying above upper E with inverted breve",
          "brief": "mod above upper E with inverted breve",
          "sbrief": "mod above upper E with inverted breve"
        }
      }
    }, {
      "category": "Lu",
      "key": "0208",
      "mappings": {
        "default": {"default": "cap I double grave"},
        "mathspeak": {
          "default": "modifying above upper I with double grave",
          "brief": "mod above upper I with double grave",
          "sbrief": "mod above upper I with double grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "020A",
      "mappings": {
        "default": {"default": "cap I inverted breve"},
        "mathspeak": {
          "default": "modifying above upper I with inverted breve",
          "brief": "mod above upper I with inverted breve",
          "sbrief": "mod above upper I with inverted breve"
        }
      }
    }, {
      "category": "Lu",
      "key": "020C",
      "mappings": {
        "default": {"default": "cap O double grave"},
        "mathspeak": {
          "default": "modifying above upper O with double grave",
          "brief": "mod above upper O with double grave",
          "sbrief": "mod above upper O with double grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "020E",
      "mappings": {
        "default": {"default": "cap O inverted breve"},
        "mathspeak": {
          "default": "modifying above upper O with inverted breve",
          "brief": "mod above upper O with inverted breve",
          "sbrief": "mod above upper O with inverted breve"
        }
      }
    }, {
      "category": "Lu",
      "key": "0210",
      "mappings": {
        "default": {"default": "cap R double grave"},
        "mathspeak": {
          "default": "modifying above upper R with double grave",
          "brief": "mod above upper R with double grave",
          "sbrief": "mod above upper R with double grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "0212",
      "mappings": {
        "default": {"default": "cap R inverted breve"},
        "mathspeak": {
          "default": "modifying above upper R with inverted breve",
          "brief": "mod above upper R with inverted breve",
          "sbrief": "mod above upper R with inverted breve"
        }
      }
    }, {
      "category": "Lu",
      "key": "0214",
      "mappings": {
        "default": {"default": "cap U double grave"},
        "mathspeak": {
          "default": "modifying above upper U with double grave",
          "brief": "mod above upper U with double grave",
          "sbrief": "mod above upper U with double grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "0216",
      "mappings": {
        "default": {"default": "cap U inverted breve"},
        "mathspeak": {
          "default": "modifying above upper U with inverted breve",
          "brief": "mod above upper U with inverted breve",
          "sbrief": "mod above upper U with inverted breve"
        }
      }
    }, {
      "category": "Lu",
      "key": "0218",
      "mappings": {
        "default": {"default": "cap S comma below"},
        "mathspeak": {
          "default": "modifying below upper S with comma below",
          "brief": "mod below upper S with comma below",
          "sbrief": "mod below upper S with comma below"
        }
      }
    }, {
      "category": "Lu",
      "key": "021A",
      "mappings": {
        "default": {"default": "cap T comma below"},
        "mathspeak": {
          "default": "modifying below upper T with comma below",
          "brief": "mod below upper T with comma below",
          "sbrief": "mod below upper T with comma below"
        }
      }
    }, {
      "category": "Lu",
      "key": "021E",
      "mappings": {
        "default": {"default": "cap H caron"},
        "mathspeak": {
          "default": "modifying above upper H with caron",
          "brief": "mod above upper H with caron",
          "sbrief": "mod above upper H with caron"
        }
      }
    }, {
      "category": "Lu",
      "key": "0226",
      "mappings": {
        "default": {"default": "cap A overdot"},
        "mathspeak": {
          "default": "modifying above upper A with dot",
          "brief": "mod above upper A with dot",
          "sbrief": "mod above upper A with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "0228",
      "mappings": {
        "default": {"default": "cap E cedilla"},
        "mathspeak": {
          "default": "modifying above upper E with cedilla",
          "brief": "mod above upper E with cedilla",
          "sbrief": "mod above upper E with cedilla"
        }
      }
    }, {
      "category": "Lu",
      "key": "022E",
      "mappings": {
        "default": {"default": "cap O overdot"},
        "mathspeak": {
          "default": "modifying above upper O with dot",
          "brief": "mod above upper O with dot",
          "sbrief": "mod above upper O with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "0232",
      "mappings": {
        "default": {"default": "cap Y overbar"},
        "mathspeak": {"default": "upper Y overbar", "brief": "upper Y overbar", "sbrief": "upper Y overbar"}
      }
    }, {
      "category": "Lu",
      "key": "1E00",
      "mappings": {
        "default": {"default": "cap A ring below"},
        "mathspeak": {
          "default": "modifying below upper A with ring below",
          "brief": "mod below upper A with ring below",
          "sbrief": "mod below upper A with ring below"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E02",
      "mappings": {
        "default": {"default": "cap B overdot"},
        "mathspeak": {
          "default": "modifying above upper B with dot",
          "brief": "mod above upper B with dot",
          "sbrief": "mod above upper B with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E04",
      "mappings": {
        "default": {"default": "cap B underdot"},
        "mathspeak": {
          "default": "modifying below upper B with dot",
          "brief": "mod below upper B with dot",
          "sbrief": "mod below upper B with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E06",
      "mappings": {
        "default": {"default": "cap B underbar"},
        "mathspeak": {"default": "upper B underbar", "brief": "upper B underbar", "sbrief": "upper B underbar"}
      }
    }, {
      "category": "Lu",
      "key": "1E0A",
      "mappings": {
        "default": {"default": "cap D overdot"},
        "mathspeak": {
          "default": "modifying above upper D with dot",
          "brief": "mod above upper D with dot",
          "sbrief": "mod above upper D with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E0C",
      "mappings": {
        "default": {"default": "cap D underdot"},
        "mathspeak": {
          "default": "modifying below upper D with dot",
          "brief": "mod below upper D with dot",
          "sbrief": "mod below upper D with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E0E",
      "mappings": {
        "default": {"default": "cap D underbar"},
        "mathspeak": {"default": "upper D underbar", "brief": "upper D underbar", "sbrief": "upper D underbar"}
      }
    }, {
      "category": "Lu",
      "key": "1E10",
      "mappings": {
        "default": {"default": "cap D cedilla"},
        "mathspeak": {
          "default": "modifying above upper D with cedilla",
          "brief": "mod above upper D with cedilla",
          "sbrief": "mod above upper D with cedilla"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E12",
      "mappings": {
        "default": {"default": "cap D underhat"},
        "mathspeak": {
          "default": "modifying below upper D with caret",
          "brief": "mod below upper D with caret",
          "sbrief": "mod below upper D with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E18",
      "mappings": {
        "default": {"default": "cap E underhat"},
        "mathspeak": {
          "default": "modifying below upper E with caret",
          "brief": "mod below upper E with caret",
          "sbrief": "mod below upper E with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E1A",
      "mappings": {
        "default": {"default": "cap E tilde below"},
        "mathspeak": {"default": "upper E undertilde", "brief": "upper E undertilde", "sbrief": "upper E undertilde"}
      }
    }, {
      "category": "Lu",
      "key": "1E1E",
      "mappings": {
        "default": {"default": "cap F overdot"},
        "mathspeak": {
          "default": "modifying above upper F with dot",
          "brief": "mod above upper F with dot",
          "sbrief": "mod above upper F with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E20",
      "mappings": {
        "default": {"default": "cap G overbar"},
        "mathspeak": {"default": "upper G overbar", "brief": "upper G overbar", "sbrief": "upper G overbar"}
      }
    }, {
      "category": "Lu",
      "key": "1E22",
      "mappings": {
        "default": {"default": "cap H overdot"},
        "mathspeak": {
          "default": "modifying above upper H with dot",
          "brief": "mod above upper H with dot",
          "sbrief": "mod above upper H with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E24",
      "mappings": {
        "default": {"default": "cap H underdot"},
        "mathspeak": {
          "default": "modifying below upper H with dot",
          "brief": "mod below upper H with dot",
          "sbrief": "mod below upper H with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E26",
      "mappings": {
        "default": {"default": "cap H double overdot"},
        "mathspeak": {
          "default": "modifying above upper H with double dot",
          "brief": "mod above upper H with double dot",
          "sbrief": "mod above upper H with double dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E28",
      "mappings": {
        "default": {"default": "cap H cedilla"},
        "mathspeak": {
          "default": "modifying above upper H with cedilla",
          "brief": "mod above upper H with cedilla",
          "sbrief": "mod above upper H with cedilla"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E2A",
      "mappings": {
        "default": {"default": "cap H breve below"},
        "mathspeak": {
          "default": "modifying below upper H with breve below",
          "brief": "mod below upper H with breve below",
          "sbrief": "mod below upper H with breve below"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E2C",
      "mappings": {
        "default": {"default": "cap I tilde below"},
        "mathspeak": {"default": "upper I undertilde", "brief": "upper I undertilde", "sbrief": "upper I undertilde"}
      }
    }, {
      "category": "Lu",
      "key": "1E30",
      "mappings": {
        "default": {"default": "cap K acute"},
        "mathspeak": {
          "default": "modifying above upper K with acute",
          "brief": "mod above upper K with acute",
          "sbrief": "mod above upper K with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E32",
      "mappings": {
        "default": {"default": "cap K underdot"},
        "mathspeak": {
          "default": "modifying below upper K with dot",
          "brief": "mod below upper K with dot",
          "sbrief": "mod below upper K with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E34",
      "mappings": {
        "default": {"default": "cap K underbar"},
        "mathspeak": {"default": "upper K underbar", "brief": "upper K underbar", "sbrief": "upper K underbar"}
      }
    }, {
      "category": "Lu",
      "key": "1E36",
      "mappings": {
        "default": {"default": "cap L underdot"},
        "mathspeak": {
          "default": "modifying below upper L with dot",
          "brief": "mod below upper L with dot",
          "sbrief": "mod below upper L with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E3A",
      "mappings": {
        "default": {"default": "cap L underbar"},
        "mathspeak": {"default": "upper L underbar", "brief": "upper L underbar", "sbrief": "upper L underbar"}
      }
    }, {
      "category": "Lu",
      "key": "1E3C",
      "mappings": {
        "default": {"default": "cap L underhat"},
        "mathspeak": {
          "default": "modifying below upper L with caret",
          "brief": "mod below upper L with caret",
          "sbrief": "mod below upper L with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E3E",
      "mappings": {
        "default": {"default": "cap M acute"},
        "mathspeak": {
          "default": "modifying above upper M with acute",
          "brief": "mod above upper M with acute",
          "sbrief": "mod above upper M with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E40",
      "mappings": {
        "default": {"default": "cap M overdot"},
        "mathspeak": {
          "default": "modifying above upper M with dot",
          "brief": "mod above upper M with dot",
          "sbrief": "mod above upper M with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E42",
      "mappings": {
        "default": {"default": "cap M underdot"},
        "mathspeak": {
          "default": "modifying below upper M with dot",
          "brief": "mod below upper M with dot",
          "sbrief": "mod below upper M with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E44",
      "mappings": {
        "default": {"default": "cap N overdot"},
        "mathspeak": {
          "default": "modifying above upper N with dot",
          "brief": "mod above upper N with dot",
          "sbrief": "mod above upper N with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E46",
      "mappings": {
        "default": {"default": "cap N underdot"},
        "mathspeak": {
          "default": "modifying below upper N with dot",
          "brief": "mod below upper N with dot",
          "sbrief": "mod below upper N with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E48",
      "mappings": {
        "default": {"default": "cap N underbar"},
        "mathspeak": {"default": "upper N underbar", "brief": "upper N underbar", "sbrief": "upper N underbar"}
      }
    }, {
      "category": "Lu",
      "key": "1E4A",
      "mappings": {
        "default": {"default": "cap N underhat"},
        "mathspeak": {
          "default": "modifying below upper N with caret",
          "brief": "mod below upper N with caret",
          "sbrief": "mod below upper N with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E54",
      "mappings": {
        "default": {"default": "cap P acute"},
        "mathspeak": {
          "default": "modifying above upper P with acute",
          "brief": "mod above upper P with acute",
          "sbrief": "mod above upper P with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E56",
      "mappings": {
        "default": {"default": "cap P overdot"},
        "mathspeak": {
          "default": "modifying above upper P with dot",
          "brief": "mod above upper P with dot",
          "sbrief": "mod above upper P with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E58",
      "mappings": {
        "default": {"default": "cap R overdot"},
        "mathspeak": {
          "default": "modifying above upper R with dot",
          "brief": "mod above upper R with dot",
          "sbrief": "mod above upper R with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E5A",
      "mappings": {
        "default": {"default": "cap R underdot"},
        "mathspeak": {
          "default": "modifying below upper R with dot",
          "brief": "mod below upper R with dot",
          "sbrief": "mod below upper R with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E5E",
      "mappings": {
        "default": {"default": "cap R underbar"},
        "mathspeak": {"default": "upper R underbar", "brief": "upper R underbar", "sbrief": "upper R underbar"}
      }
    }, {
      "category": "Lu",
      "key": "1E60",
      "mappings": {
        "default": {"default": "cap S overdot"},
        "mathspeak": {
          "default": "modifying above upper S with dot",
          "brief": "mod above upper S with dot",
          "sbrief": "mod above upper S with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E62",
      "mappings": {
        "default": {"default": "cap S underdot"},
        "mathspeak": {
          "default": "modifying below upper S with dot",
          "brief": "mod below upper S with dot",
          "sbrief": "mod below upper S with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E6A",
      "mappings": {
        "default": {"default": "cap T overdot"},
        "mathspeak": {
          "default": "modifying above upper T with dot",
          "brief": "mod above upper T with dot",
          "sbrief": "mod above upper T with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E6C",
      "mappings": {
        "default": {"default": "cap T underdot"},
        "mathspeak": {
          "default": "modifying below upper T with dot",
          "brief": "mod below upper T with dot",
          "sbrief": "mod below upper T with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E6E",
      "mappings": {
        "default": {"default": "cap T underbar"},
        "mathspeak": {"default": "upper T underbar", "brief": "upper T underbar", "sbrief": "upper T underbar"}
      }
    }, {
      "category": "Lu",
      "key": "1E70",
      "mappings": {
        "default": {"default": "cap T underhat"},
        "mathspeak": {
          "default": "modifying below upper T with caret",
          "brief": "mod below upper T with caret",
          "sbrief": "mod below upper T with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E72",
      "mappings": {
        "default": {"default": "cap U double underdot"},
        "mathspeak": {
          "default": "modifying below upper U with double dot",
          "brief": "mod below upper U with double dot",
          "sbrief": "mod below upper U with double dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E74",
      "mappings": {
        "default": {"default": "cap U tilde below"},
        "mathspeak": {"default": "upper U undertilde", "brief": "upper U undertilde", "sbrief": "upper U undertilde"}
      }
    }, {
      "category": "Lu",
      "key": "1E76",
      "mappings": {
        "default": {"default": "cap U underhat"},
        "mathspeak": {
          "default": "modifying below upper U with caret",
          "brief": "mod below upper U with caret",
          "sbrief": "mod below upper U with caret"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E7C",
      "mappings": {
        "default": {"default": "cap V tilde"},
        "mathspeak": {"default": "upper V overtilde", "brief": "upper V overtilde", "sbrief": "upper V overtilde"}
      }
    }, {
      "category": "Lu",
      "key": "1E7E",
      "mappings": {
        "default": {"default": "cap V underdot"},
        "mathspeak": {
          "default": "modifying below upper V with dot",
          "brief": "mod below upper V with dot",
          "sbrief": "mod below upper V with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E80",
      "mappings": {
        "default": {"default": "cap W grave"},
        "mathspeak": {
          "default": "modifying above upper W with grave",
          "brief": "mod above upper W with grave",
          "sbrief": "mod above upper W with grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E82",
      "mappings": {
        "default": {"default": "cap W acute"},
        "mathspeak": {
          "default": "modifying above upper W with acute",
          "brief": "mod above upper W with acute",
          "sbrief": "mod above upper W with acute"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E84",
      "mappings": {
        "default": {"default": "cap W double overdot"},
        "mathspeak": {
          "default": "modifying above upper W with double dot",
          "brief": "mod above upper W with double dot",
          "sbrief": "mod above upper W with double dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E86",
      "mappings": {
        "default": {"default": "cap W overdot"},
        "mathspeak": {
          "default": "modifying above upper W with dot",
          "brief": "mod above upper W with dot",
          "sbrief": "mod above upper W with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E88",
      "mappings": {
        "default": {"default": "cap W underdot"},
        "mathspeak": {
          "default": "modifying below upper W with dot",
          "brief": "mod below upper W with dot",
          "sbrief": "mod below upper W with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E8A",
      "mappings": {
        "default": {"default": "cap X overdot"},
        "mathspeak": {
          "default": "modifying above upper X with dot",
          "brief": "mod above upper X with dot",
          "sbrief": "mod above upper X with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E8C",
      "mappings": {
        "default": {"default": "cap X double overdot"},
        "mathspeak": {
          "default": "modifying above upper X with double dot",
          "brief": "mod above upper X with double dot",
          "sbrief": "mod above upper X with double dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E8E",
      "mappings": {
        "default": {"default": "cap Y overdot"},
        "mathspeak": {
          "default": "modifying above upper Y with dot",
          "brief": "mod above upper Y with dot",
          "sbrief": "mod above upper Y with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E90",
      "mappings": {
        "default": {"default": "cap Z circumflex"},
        "mathspeak": {
          "default": "modifying above upper Z with circumflex",
          "brief": "mod above upper Z with circumflex",
          "sbrief": "mod above upper Z with circumflex"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E92",
      "mappings": {
        "default": {"default": "cap Z underdot"},
        "mathspeak": {
          "default": "modifying below upper Z with dot",
          "brief": "mod below upper Z with dot",
          "sbrief": "mod below upper Z with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1E94",
      "mappings": {
        "default": {"default": "cap Z underbar"},
        "mathspeak": {"default": "upper Z underbar", "brief": "upper Z underbar", "sbrief": "upper Z underbar"}
      }
    }, {
      "category": "Lu",
      "key": "1EA0",
      "mappings": {
        "default": {"default": "cap A underdot"},
        "mathspeak": {
          "default": "modifying below upper A with dot",
          "brief": "mod below upper A with dot",
          "sbrief": "mod below upper A with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1EA2",
      "mappings": {
        "default": {"default": "cap A hook"},
        "mathspeak": {
          "default": "modifying above upper A with hook",
          "brief": "mod above upper A with hook",
          "sbrief": "mod above upper A with hook"
        }
      }
    }, {
      "category": "Lu",
      "key": "1EB8",
      "mappings": {
        "default": {"default": "cap E underdot"},
        "mathspeak": {
          "default": "modifying below upper E with dot",
          "brief": "mod below upper E with dot",
          "sbrief": "mod below upper E with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1EBA",
      "mappings": {
        "default": {"default": "cap E hook"},
        "mathspeak": {
          "default": "modifying above upper E with hook",
          "brief": "mod above upper E with hook",
          "sbrief": "mod above upper E with hook"
        }
      }
    }, {
      "category": "Lu",
      "key": "1EBC",
      "mappings": {
        "default": {"default": "cap E tilde"},
        "mathspeak": {"default": "upper E overtilde", "brief": "upper E overtilde", "sbrief": "upper E overtilde"}
      }
    }, {
      "category": "Lu",
      "key": "1EC8",
      "mappings": {
        "default": {"default": "cap I hook"},
        "mathspeak": {
          "default": "modifying above upper I with hook",
          "brief": "mod above upper I with hook",
          "sbrief": "mod above upper I with hook"
        }
      }
    }, {
      "category": "Lu",
      "key": "1ECA",
      "mappings": {
        "default": {"default": "cap I underdot"},
        "mathspeak": {
          "default": "modifying below upper I with dot",
          "brief": "mod below upper I with dot",
          "sbrief": "mod below upper I with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1ECC",
      "mappings": {
        "default": {"default": "cap O underdot"},
        "mathspeak": {
          "default": "modifying below upper O with dot",
          "brief": "mod below upper O with dot",
          "sbrief": "mod below upper O with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1ECE",
      "mappings": {
        "default": {"default": "cap O hook"},
        "mathspeak": {
          "default": "modifying above upper O with hook",
          "brief": "mod above upper O with hook",
          "sbrief": "mod above upper O with hook"
        }
      }
    }, {
      "category": "Lu",
      "key": "1EE4",
      "mappings": {
        "default": {"default": "cap U underdot"},
        "mathspeak": {
          "default": "modifying below upper U with dot",
          "brief": "mod below upper U with dot",
          "sbrief": "mod below upper U with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1EE6",
      "mappings": {
        "default": {"default": "cap U hook"},
        "mathspeak": {
          "default": "modifying above upper U with hook",
          "brief": "mod above upper U with hook",
          "sbrief": "mod above upper U with hook"
        }
      }
    }, {
      "category": "Lu",
      "key": "1EF2",
      "mappings": {
        "default": {"default": "cap Y grave"},
        "mathspeak": {
          "default": "modifying above upper Y with grave",
          "brief": "mod above upper Y with grave",
          "sbrief": "mod above upper Y with grave"
        }
      }
    }, {
      "category": "Lu",
      "key": "1EF4",
      "mappings": {
        "default": {"default": "cap Y underdot"},
        "mathspeak": {
          "default": "modifying below upper Y with dot",
          "brief": "mod below upper Y with dot",
          "sbrief": "mod below upper Y with dot"
        }
      }
    }, {
      "category": "Lu",
      "key": "1EF6",
      "mappings": {
        "default": {"default": "cap Y hook"},
        "mathspeak": {
          "default": "modifying above upper Y with hook",
          "brief": "mod above upper Y with hook",
          "sbrief": "mod above upper Y with hook"
        }
      }
    }, {
      "category": "Lu",
      "key": "1EF8",
      "mappings": {
        "default": {"default": "cap Y tilde"},
        "mathspeak": {"default": "upper Y overtilde", "brief": "upper Y overtilde", "sbrief": "upper Y overtilde"}
      }
    }]
  ,
  "en/symbols/math_angles.js":
    [{"locale": "en"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "right angle with arc"}},
      "key": "22BE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right angle with downwards zigzag arrow"}},
      "key": "237C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "three dimensional angle"}},
      "key": "27C0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "measured angle opening left"}},
      "key": "299B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right angle variant with square"}},
      "key": "299C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "measured right angle with dot"}},
      "key": "299D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "angle with s inside"}},
      "key": "299E"
    }, {"category": "Sm", "mappings": {"default": {"default": "acute angle"}}, "key": "299F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "spherical angle opening left"}},
      "key": "29A0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "spherical angle opening up"}},
      "key": "29A1"
    }, {"category": "Sm", "mappings": {"default": {"default": "turned angle"}}, "key": "29A2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "reversed angle"}},
      "key": "29A3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "angle with underbar"}},
      "key": "29A4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "reversed angle with underbar"}},
      "key": "29A5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "oblique angle opening up"}},
      "key": "29A6"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "oblique angle opening down"}},
      "key": "29A7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing up and right"}},
      "key": "29A8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing up and left"}},
      "key": "29A9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing down and right"}},
      "key": "29AA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing down and left"}},
      "key": "29AB"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing right and up"}},
      "key": "29AC"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing left and up"}},
      "key": "29AD"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing right and down"}},
      "key": "29AE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing left and down"}},
      "key": "29AF"
    }]
  ,
  "en/symbols/math_arrows.js":
    [{"locale": "en"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "left arrow"}, "mathspeak": {"sbrief": "L arrow"}},
      "key": "2190"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up arrow"}, "mathspeak": {"sbrief": "U arrow"}},
      "key": "2191"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right arrow"}, "mathspeak": {"sbrief": "R arrow"}},
      "key": "2192"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down arrow"}, "mathspeak": {"sbrief": "D arrow"}},
      "key": "2193"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left right arrow"}, "mathspeak": {"sbrief": "L R arrow"}},
      "key": "2194"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up down arrow"}, "mathspeak": {"sbrief": "U D arrow"}},
      "key": "2195"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up left arrow"}, "mathspeak": {"sbrief": "U L arrow"}},
      "key": "2196"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up right arrow"}, "mathspeak": {"sbrief": "U R arrow"}},
      "key": "2197"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down right arrow"}, "mathspeak": {"sbrief": "D R arrow"}},
      "key": "2198"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down left arrow"}, "mathspeak": {"sbrief": "D L arrow"}},
      "key": "2199"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left arrow with stroke"}, "mathspeak": {"sbrief": "L arrow with stroke"}},
      "key": "219A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right arrow with stroke"}, "mathspeak": {"sbrief": "R arrow with stroke"}},
      "key": "219B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left wave arrow"}, "mathspeak": {"sbrief": "L wave arrow"}},
      "key": "219C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right wave arrow"}, "mathspeak": {"sbrief": "R wave arrow"}},
      "key": "219D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "two headed left arrow"}, "mathspeak": {"sbrief": "two headed L arrow"}},
      "key": "219E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "two headed up arrow"}, "mathspeak": {"sbrief": "two headed U arrow"}},
      "key": "219F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "two headed right arrow"}, "mathspeak": {"sbrief": "two headed R arrow"}},
      "key": "21A0"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "two headed down arrow"}, "mathspeak": {"sbrief": "two headed D arrow"}},
      "key": "21A1"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left arrow with tail"}, "mathspeak": {"sbrief": "L arrow with tail"}},
      "key": "21A2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right arrow with tail"}, "mathspeak": {"sbrief": "R arrow with tail"}},
      "key": "21A3"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left arrow from bar"}, "mathspeak": {"sbrief": "L arrow from bar"}},
      "key": "21A4"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up arrow from bar"}, "mathspeak": {"sbrief": "U arrow from bar"}},
      "key": "21A5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right arrow from bar"}, "mathspeak": {"sbrief": "R arrow from bar"}},
      "key": "21A6"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down arrow from bar"}, "mathspeak": {"sbrief": "D arrow from bar"}},
      "key": "21A7"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up down arrow with base"}, "mathspeak": {"sbrief": "U D arrow with base"}},
      "key": "21A8"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left arrow with hook"}, "mathspeak": {"sbrief": "L arrow with hook"}},
      "key": "21A9"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right arrow with hook"}, "mathspeak": {"sbrief": "R arrow with hook"}},
      "key": "21AA"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left arrow with loop"}, "mathspeak": {"sbrief": "L arrow with loop"}},
      "key": "21AB"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right arrow with loop"}, "mathspeak": {"sbrief": "R arrow with loop"}},
      "key": "21AC"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left right wave arrow"}, "mathspeak": {"sbrief": "L R wave arrow"}},
      "key": "21AD"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left right arrow with stroke"},
        "mathspeak": {"sbrief": "L R arrow with stroke"}
      },
      "key": "21AE"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down zigzag arrow"}, "mathspeak": {"sbrief": "d zigzag arrow"}},
      "key": "21AF"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up arrow with tip left"}, "mathspeak": {"sbrief": "U arrow with tip left"}},
      "key": "21B0"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "up arrow with tip right"},
        "mathspeak": {"sbrief": "U arrow with tip right"}
      },
      "key": "21B1"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "down arrow with tip left"},
        "mathspeak": {"sbrief": "D arrow with tip left"}
      },
      "key": "21B2"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "down arrow with tip right"},
        "mathspeak": {"sbrief": "D arrow with tip right"}
      },
      "key": "21B3"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "right arrow with corner down"},
        "mathspeak": {"sbrief": "R arrow with corner down"}
      },
      "key": "21B4"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "down arrow with corner left"},
        "mathspeak": {"sbrief": "D arrow with corner left"}
      },
      "key": "21B5"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "anticlockwise top semicircle arrow"}},
      "key": "21B6"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "clockwise top semicircle arrow"}},
      "key": "21B7"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "up left arrow to long bar"},
        "mathspeak": {"sbrief": "U L arrow to long bar"}
      },
      "key": "21B8"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "left arrow to bar over right arrow to bar"},
        "mathspeak": {"sbrief": "L arrow to bar over R arrow to bar"}
      },
      "key": "21B9"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "anticlockwise open circle arrow"}},
      "key": "21BA"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "clockwise open circle arrow"}},
      "key": "21BB"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "right arrow over left arrow"},
        "mathspeak": {"sbrief": "R arrow over L arrow"}
      },
      "key": "21C4"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "up arrow left of down arrow"},
        "mathspeak": {"sbrief": "U arrow L of D arrow"}
      },
      "key": "21C5"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "left arrow over right arrow"},
        "mathspeak": {"sbrief": "L arrow over R arrow"}
      },
      "key": "21C6"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left paired arrows"}, "mathspeak": {"sbrief": "L paired arrows"}},
      "key": "21C7"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up paired arrows"}, "mathspeak": {"sbrief": "U paired arrows"}},
      "key": "21C8"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right paired arrows"}, "mathspeak": {"sbrief": "R paired arrows"}},
      "key": "21C9"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down paired arrows"}, "mathspeak": {"sbrief": "D paired arrows"}},
      "key": "21CA"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "left double arrow with stroke"},
        "mathspeak": {"sbrief": "L double arrow with stroke"}
      },
      "key": "21CD"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left right double arrow with stroke"},
        "mathspeak": {"sbrief": "L R double arrow with stroke"}
      },
      "key": "21CE"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right double arrow with stroke"},
        "mathspeak": {"sbrief": "R double arrow with stroke"}
      },
      "key": "21CF"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left double arrow"}, "mathspeak": {"sbrief": "L double arrow"}},
      "key": "21D0"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up double arrow"}, "mathspeak": {"sbrief": "U double arrow"}},
      "key": "21D1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right double arrow"}, "mathspeak": {"sbrief": "R double arrow"}},
      "key": "21D2"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down double arrow"}, "mathspeak": {"sbrief": "d double arrow"}},
      "key": "21D3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left right double arrow"}, "mathspeak": {"sbrief": "L R double arrow"}},
      "key": "21D4"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up down double arrow"}, "mathspeak": {"sbrief": "U d double arrow"}},
      "key": "21D5"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up left double arrow"}, "mathspeak": {"sbrief": "U L double arrow"}},
      "key": "21D6"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up right double arrow"}, "mathspeak": {"sbrief": "U R double arrow"}},
      "key": "21D7"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down right double arrow"}, "mathspeak": {"sbrief": "d R double arrow"}},
      "key": "21D8"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down left double arrow"}, "mathspeak": {"sbrief": "d L double arrow"}},
      "key": "21D9"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left triple arrow"}, "mathspeak": {"sbrief": "L triple arrow"}},
      "key": "21DA"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right triple arrow"}, "mathspeak": {"sbrief": "r triple arrow"}},
      "key": "21DB"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left squiggle arrow"}, "mathspeak": {"sbrief": "L squiggle arrow"}},
      "key": "21DC"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right squiggle arrow"}, "mathspeak": {"sbrief": "r squiggle arrow"}},
      "key": "21DD"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "up arrow with double stroke"},
        "mathspeak": {"sbrief": "U arrow with double stroke"}
      },
      "key": "21DE"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "down arrow with double stroke"},
        "mathspeak": {"sbrief": "D arrow with double stroke"}
      },
      "key": "21DF"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left dasheD arrow"}, "mathspeak": {"sbrief": "L dasheD arrow"}},
      "key": "21E0"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up dasheD arrow"}, "mathspeak": {"sbrief": "U dasheD arrow"}},
      "key": "21E1"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right dasheD arrow"}, "mathspeak": {"sbrief": "r dasheD arrow"}},
      "key": "21E2"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down dasheD arrow"}, "mathspeak": {"sbrief": "d dasheD arrow"}},
      "key": "21E3"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left arrow to bar"}, "mathspeak": {"sbrief": "L arrow to bar"}},
      "key": "21E4"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right arrow to bar"}, "mathspeak": {"sbrief": "R arrow to bar"}},
      "key": "21E5"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white left arrow"}, "mathspeak": {"sbrief": "white L arrow"}},
      "key": "21E6"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white up arrow"}, "mathspeak": {"sbrief": "white U arrow"}},
      "key": "21E7"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white right arrow"}, "mathspeak": {"sbrief": "white R arrow"}},
      "key": "21E8"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white down arrow"}, "mathspeak": {"sbrief": "white D arrow"}},
      "key": "21E9"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "white up arrow from bar"},
        "mathspeak": {"sbrief": "white U arrow from bar"}
      },
      "key": "21EA"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "white up arrow on pedestal"},
        "mathspeak": {"sbrief": "white U arrow on pedestal"}
      },
      "key": "21EB"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "white up arrow on pedestal with horizontal bar"},
        "mathspeak": {"sbrief": "white U arrow on pedestal with horizontal bar"}
      },
      "key": "21EC"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "white up arrow on pedestal with vertical bar"},
        "mathspeak": {"sbrief": "white U arrow on pedestal with vertical bar"}
      },
      "key": "21ED"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white double up arrow"}, "mathspeak": {"sbrief": "white double U arrow"}},
      "key": "21EE"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "white double up arrow on pedestal"},
        "mathspeak": {"sbrief": "white double U arrow on pedestal"}
      },
      "key": "21EF"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "white right arrow from wall"},
        "mathspeak": {"sbrief": "white R arrow from wall"}
      },
      "key": "21F0"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "north west arrow to corner"}},
      "key": "21F1"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "south east arrow to corner"}},
      "key": "21F2"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up down white arrow"}, "mathspeak": {"sbrief": "U d white arrow"}},
      "key": "21F3"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow with small circle"},
        "mathspeak": {"sbrief": "R arrow with small circle"}
      },
      "key": "21F4"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "down arrow left of up arrow"},
        "mathspeak": {"sbrief": "D arrow l of U arrow"}
      },
      "key": "21F5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "three right arrows"}, "mathspeak": {"sbrief": "three R arrows"}},
      "key": "21F6"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow with vertical stroke"},
        "mathspeak": {"sbrief": "L arrow with vertical stroke"}
      },
      "key": "21F7"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow with vertical stroke"},
        "mathspeak": {"sbrief": "R arrow with vertical stroke"}
      },
      "key": "21F8"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left right arrow with vertical stroke"},
        "mathspeak": {"sbrief": "L R arrow with vertical stroke"}
      },
      "key": "21F9"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow with double vertical stroke"},
        "mathspeak": {"sbrief": "L arrow with double vertical stroke"}
      },
      "key": "21FA"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow with double vertical stroke"},
        "mathspeak": {"sbrief": "R arrow with double vertical stroke"}
      },
      "key": "21FB"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left right arrow with double vertical stroke"},
        "mathspeak": {"sbrief": "L R arrow with double vertical stroke"}
      },
      "key": "21FC"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left open headed arrow"}, "mathspeak": {"sbrief": "l open headed arrow"}},
      "key": "21FD"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right open headed arrow"}, "mathspeak": {"sbrief": "r open headed arrow"}},
      "key": "21FE"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left right open headed arrow"},
        "mathspeak": {"sbrief": "L R open headed arrow"}
      },
      "key": "21FF"
    }, {"category": "So", "mappings": {"default": {"default": "electric arrow"}}, "key": "2301"}, {
      "category": "So",
      "mappings": {"default": {"default": "up arrowhead"}, "mathspeak": {"sbrief": "U arrowhead"}},
      "key": "2303"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down arrowhead"}, "mathspeak": {"sbrief": "D arrowhead"}},
      "key": "2304"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "up arrowhead between two horizontal bars", "alternative": "enter key"},
        "mathspeak": {"sbrief": "U arrowhead between two horizontal bars"}
      },
      "key": "2324"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "broken circle with northwest arrow"}},
      "key": "238B"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy wide headed right arrow"},
        "mathspeak": {"sbrief": "heavy wide headed R arrow"}
      },
      "key": "2794"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "heavy down right arrow"}, "mathspeak": {"sbrief": "heavy d R arrow"}},
      "key": "2798"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "heavy right arrow"}, "mathspeak": {"sbrief": "heavy R arrow"}},
      "key": "2799"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "heavy up right arrow"}, "mathspeak": {"sbrief": "heavy U R arrow"}},
      "key": "279A"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "drafting point right arrow"},
        "mathspeak": {"sbrief": "drafting point R arrow"}
      },
      "key": "279B"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy round tipped right arrow"},
        "mathspeak": {"sbrief": "heavy round tipped R arrow"}
      },
      "key": "279C"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "triangle headed right arrow"},
        "mathspeak": {"sbrief": "triangle headed R arrow"}
      },
      "key": "279D"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy triangle headed right arrow"},
        "mathspeak": {"sbrief": "heavy triangle headed R arrow"}
      },
      "key": "279E"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "dashed triangle headed right arrow"},
        "mathspeak": {"sbrief": "dashed triangle headed R arrow"}
      },
      "key": "279F"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy dashed triangle headed right arrow"},
        "mathspeak": {"sbrief": "heavy dashed triangle headed R arrow"}
      },
      "key": "27A0"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black right arrow"}, "mathspeak": {"sbrief": "black R arrow"}},
      "key": "27A1"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "three d top lighted right arrowhead"},
        "mathspeak": {"sbrief": "three d top lighted R arrowhead"}
      },
      "key": "27A2"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "three d bottom lighted right arrowhead"},
        "mathspeak": {"sbrief": "three d bottom lighted R arrowhead"}
      },
      "key": "27A3"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black right arrowhead"}, "mathspeak": {"sbrief": "black R arrowhead"}},
      "key": "27A4"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy black curved down and right arrow"},
        "mathspeak": {"sbrief": "heavy black curved d and R arrow"}
      },
      "key": "27A5"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy black curved up and right arrow"},
        "mathspeak": {"sbrief": "heavy black curved U and R arrow"}
      },
      "key": "27A6"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "squat black right arrow"}, "mathspeak": {"sbrief": "squat black R arrow"}},
      "key": "27A7"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy concave pointed black right arrow"},
        "mathspeak": {"sbrief": "heavy concave pointed black R arrow"}
      },
      "key": "27A8"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "right shaded white right arrow"},
        "mathspeak": {"sbrief": "right shaded white R arrow"}
      },
      "key": "27A9"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "left shaded white right arrow"},
        "mathspeak": {"sbrief": "left shaded white R arrow"}
      },
      "key": "27AA"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "back tilted shadowed white right arrow"},
        "mathspeak": {"sbrief": "back tilted shadowed white R arrow"}
      },
      "key": "27AB"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "front tilted shadowed white right arrow"},
        "mathspeak": {"sbrief": "front tilted shadowed white R arrow"}
      },
      "key": "27AC"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy lower right shadowed white right arrow"},
        "mathspeak": {"sbrief": "heavy lower right shadowed white R arrow"}
      },
      "key": "27AD"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy upper right shadowed white right arrow"},
        "mathspeak": {"sbrief": "heavy upper right shadowed white R arrow"}
      },
      "key": "27AE"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "notched lower right shadowed white right arrow"},
        "mathspeak": {"sbrief": "notched lower right shadowed white R arrow"}
      },
      "key": "27AF"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "notched upper right shadowed white right arrow"},
        "mathspeak": {"sbrief": "notched upper right shadowed white R arrow"}
      },
      "key": "27B1"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "circled heavy white right arrow"},
        "mathspeak": {"sbrief": "circled heavy white R arrow"}
      },
      "key": "27B2"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "white feathered right arrow"},
        "mathspeak": {"sbrief": "white feathered R arrow"}
      },
      "key": "27B3"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "black feathered down right arrow"},
        "mathspeak": {"sbrief": "black feathered d R arrow"}
      },
      "key": "27B4"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "black feathered right arrow"},
        "mathspeak": {"sbrief": "black feathered R arrow"}
      },
      "key": "27B5"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "black feathered up right arrow"},
        "mathspeak": {"sbrief": "black feathered U R arrow"}
      },
      "key": "27B6"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy black feathered down right arrow"},
        "mathspeak": {"sbrief": "heavy black feathered d R arrow"}
      },
      "key": "27B7"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy black feathered right arrow"},
        "mathspeak": {"sbrief": "heavy black feathered R arrow"}
      },
      "key": "27B8"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy black feathered up right arrow"},
        "mathspeak": {"sbrief": "heavy black feathered U R arrow"}
      },
      "key": "27B9"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "teardrop barbed right arrow"},
        "mathspeak": {"sbrief": "teardrop barbed R arrow"}
      },
      "key": "27BA"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy teardrop shanked right arrow"},
        "mathspeak": {"sbrief": "heavy teardrop shanked R arrow"}
      },
      "key": "27BB"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "wedge tailed right arrow"}, "mathspeak": {"sbrief": "wedge tailed R arrow"}},
      "key": "27BC"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "heavy wedge tailed right arrow"},
        "mathspeak": {"sbrief": "heavy wedge tailed R arrow"}
      },
      "key": "27BD"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "open outlined right arrow"},
        "mathspeak": {"sbrief": "open outlined R arrow"}
      },
      "key": "27BE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up quadruple arrow"}, "mathspeak": {"sbrief": "U quadruple arrow"}},
      "key": "27F0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down quadruple arrow"}, "mathspeak": {"sbrief": "d quadrule arrow"}},
      "key": "27F1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "anticlockwise gapped circle arrow"}},
      "key": "27F2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "clockwise gapped circle arrow"}},
      "key": "27F3"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow with circled plus"},
        "mathspeak": {"sbrief": "R arrow with circled plus"}
      },
      "key": "27F4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "long left arrow"}, "mathspeak": {"sbrief": "long L arrow"}},
      "key": "27F5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "long right arrow"}, "mathspeak": {"sbrief": "long R arrow"}},
      "key": "27F6"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "long left right arrow"}, "mathspeak": {"sbrief": "long L R arrow"}},
      "key": "27F7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "long left double arrow"}, "mathspeak": {"sbrief": "long l double arrow"}},
      "key": "27F8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "long right double arrow"}, "mathspeak": {"sbrief": "long R double arrow"}},
      "key": "27F9"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "long left right double arrow"},
        "mathspeak": {"sbrief": "long L R double arrow"}
      },
      "key": "27FA"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "long left arrow from bar"},
        "mathspeak": {"sbrief": "long L arrow from bar"}
      },
      "key": "27FB"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "long right arrow from bar"},
        "mathspeak": {"sbrief": "long R arrow from bar"}
      },
      "key": "27FC"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "long left double arrow from bar"},
        "mathspeak": {"sbrief": "long l double arrow from bar"}
      },
      "key": "27FD"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "long right double arrow from bar"},
        "mathspeak": {"sbrief": "long R double arrow from bar"}
      },
      "key": "27FE"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "long right squiggle arrow"},
        "mathspeak": {"sbrief": "long r squiggle arrow"}
      },
      "key": "27FF"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "two headed right arrow with vertical stroke"},
        "mathspeak": {"sbrief": "two headed R arrow with vertical stroke"}
      },
      "key": "2900"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "two headed right arrow with double vertical stroke"},
        "mathspeak": {"sbrief": "two headed R arrow with double vertical stroke"}
      },
      "key": "2901"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "double left arrow with vertical stroke"},
        "mathspeak": {"sbrief": "double L arrow with vertical stroke"}
      },
      "key": "2902"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "double right arrow with vertical stroke"},
        "mathspeak": {"sbrief": "double R arrow with vertical stroke"}
      },
      "key": "2903"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "double left right arrow with vertical stroke"},
        "mathspeak": {"sbrief": "double L R arrow with vertical stroke"}
      },
      "key": "2904"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "two headed right arrow from bar"},
        "mathspeak": {"sbrief": "two headed R arrow from bar"}
      },
      "key": "2905"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "double left arrow from bar"},
        "mathspeak": {"sbrief": "double L arrow from bar"}
      },
      "key": "2906"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "double right arrow from bar"},
        "mathspeak": {"sbrief": "double R arrow from bar"}
      },
      "key": "2907"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "arrow down with horizontal stroke"}},
      "key": "2908"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "up arrow with horizontal stroke"},
        "mathspeak": {"sbrief": "U arrow with horizontal stroke"}
      },
      "key": "2909"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up triple arrow"}, "mathspeak": {"sbrief": "U triple arrow"}},
      "key": "290A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down triple arrow"}, "mathspeak": {"sbrief": "d triple arrow"}},
      "key": "290B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left double dash arrow"}, "mathspeak": {"sbrief": "l double dash arrow"}},
      "key": "290C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right double dash arrow"}, "mathspeak": {"sbrief": "R double dash arrow"}},
      "key": "290D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left triple dash arrow"}, "mathspeak": {"sbrief": "l triple dash arrow"}},
      "key": "290E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right triple dash arrow"}, "mathspeak": {"sbrief": "r triple dash arrow"}},
      "key": "290F"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right two headed triple dash arrow"},
        "mathspeak": {"sbrief": "r two headed triple dash arrow"}
      },
      "key": "2910"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow with dotted stem"},
        "mathspeak": {"sbrief": "R arrow with dotted stem"}
      },
      "key": "2911"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up arrow to bar"}, "mathspeak": {"sbrief": "U arrow to bar"}},
      "key": "2912"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down arrow to bar"}, "mathspeak": {"sbrief": "D arrow to bar"}},
      "key": "2913"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow with tail with vertical stroke"},
        "mathspeak": {"sbrief": "R arrow with tail with vertical stroke"}
      },
      "key": "2914"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow with tail with double vertical stroke"},
        "mathspeak": {"sbrief": "R arrow with tail with double vertical stroke"}
      },
      "key": "2915"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right two headed arrow with tail"},
        "mathspeak": {"sbrief": "r two headed arrow with tail"}
      },
      "key": "2916"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right two headed arrow with tail with vertical stroke"},
        "mathspeak": {"sbrief": "r two headed arrow with tail with vertical stroke"}
      },
      "key": "2917"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right two headed arrow with tail with double vertical stroke"},
        "mathspeak": {"sbrief": "r two headed arrow with tail with double vertical stroke"}
      },
      "key": "2918"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left arrow tail"}, "mathspeak": {"sbrief": "L arrow tail"}},
      "key": "2919"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right arrow tail"}, "mathspeak": {"sbrief": "R arrow tail"}},
      "key": "291A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left double arrow tail"}, "mathspeak": {"sbrief": "l double arrow tail"}},
      "key": "291B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right double arrow tail"}, "mathspeak": {"sbrief": "R double arrow tail"}},
      "key": "291C"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow to black diamond"},
        "mathspeak": {"sbrief": "L arrow to black diamond"}
      },
      "key": "291D"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow to black diamond"},
        "mathspeak": {"sbrief": "R arrow to black diamond"}
      },
      "key": "291E"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow from bar to black diamond"},
        "mathspeak": {"sbrief": "L arrow from bar to black diamond"}
      },
      "key": "291F"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow from bar to black diamond"},
        "mathspeak": {"sbrief": "R arrow from bar to black diamond"}
      },
      "key": "2920"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "north west and south east arrow"}},
      "key": "2921"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "north east and south west arrow"}},
      "key": "2922"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "north west arrow with hook"}},
      "key": "2923"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "north east arrow with hook"}},
      "key": "2924"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "south east arrow with hook"}},
      "key": "2925"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "south west arrow with hook"}},
      "key": "2926"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "north west arrow and north east arrow"}},
      "key": "2927"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "north east arrow and south east arrow"}},
      "key": "2928"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "south east arrow and south west arrow"}},
      "key": "2929"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "south west arrow and north west arrow"}},
      "key": "292A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "south east arrow crossing north east arrow"}},
      "key": "292D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "north east arrow crossing south east arrow"}},
      "key": "292E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "falling diagonal crossing north east arrow"}},
      "key": "292F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "rising diagonal crossing south east arrow"}},
      "key": "2930"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "north east arrow crossing north west arrow"}},
      "key": "2931"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "north west arrow crossing north east arrow"}},
      "key": "2932"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "wave arrow pointing directly right"}},
      "key": "2933"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "arrow pointing right then curving up"}},
      "key": "2934"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "arrow pointing right then curving down"}},
      "key": "2935"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "arrow pointing down then curving left"}},
      "key": "2936"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "arrow pointing down then curving right"}},
      "key": "2937"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right side arc clockwise arrow"},
        "mathspeak": {"sbrief": "r side arc clockwise arrow"}
      },
      "key": "2938"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left side arc anticlockwise arrow"},
        "mathspeak": {"sbrief": "l side arc anticlockwise arrow"}
      },
      "key": "2939"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "top arc anticlockwise arrow"}},
      "key": "293A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "bottom arc anticlockwise arrow"}},
      "key": "293B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "top arc clockwise arrow with minus"}},
      "key": "293C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "top arc anticlockwise arrow with plus"}},
      "key": "293D"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "down right semicircular clockwise arrow"},
        "mathspeak": {"sbrief": "d r semicircular clockwise arrow"}
      },
      "key": "293E"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "down left semicircular anticlockwise arrow"},
        "mathspeak": {"sbrief": "d l semicircular anticlockwise arrow"}
      },
      "key": "293F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "anticlockwise closed circle arrow"}},
      "key": "2940"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "clockwise closed circle arrow"}},
      "key": "2941"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow above short left arrow"},
        "mathspeak": {"sbrief": "R arrow above short L arrow"}
      },
      "key": "2942"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow above short right arrow"},
        "mathspeak": {"sbrief": "L arrow above short R arrow"}
      },
      "key": "2943"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "short right arrow above left arrow"},
        "mathspeak": {"sbrief": "short R arrow above L arrow"}
      },
      "key": "2944"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow with plus below"},
        "mathspeak": {"sbrief": "R arrow with plus below"}
      },
      "key": "2945"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow with plus below"},
        "mathspeak": {"sbrief": "L arrow with plus below"}
      },
      "key": "2946"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right arrow through x"}, "mathspeak": {"sbrief": "R arrow through x"}},
      "key": "2947"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left right arrow through small circle"},
        "mathspeak": {"sbrief": "L R arrow through small circle"}
      },
      "key": "2948"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "up two headed arrow from small circle"},
        "mathspeak": {"sbrief": "U two headed arrow from small circle"}
      },
      "key": "2949"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right double arrow with rounded head"},
        "mathspeak": {"sbrief": "R double arrow with rounded head"}
      },
      "key": "2970"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "equals sign above right arrow"},
        "mathspeak": {"sbrief": "equals sign above R arrow"}
      },
      "key": "2971"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "tilde operator above right arrow"},
        "mathspeak": {"sbrief": "tilde operator above R arrow"}
      },
      "key": "2972"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow above tilde operator"},
        "mathspeak": {"sbrief": "L arrow above tilde operator"}
      },
      "key": "2973"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow above tilde operator"},
        "mathspeak": {"sbrief": "R arrow above tilde operator"}
      },
      "key": "2974"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow above almost equals"},
        "mathspeak": {"sbrief": "R arrow above almost equals"}
      },
      "key": "2975"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "less than above left arrow"},
        "mathspeak": {"sbrief": "less than above L arrow"}
      },
      "key": "2976"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow through less than"},
        "mathspeak": {"sbrief": "L arrow through less than"}
      },
      "key": "2977"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "greater than above right arrow"},
        "mathspeak": {"sbrief": "greater than above R arrow"}
      },
      "key": "2978"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "subset above right arrow"}, "mathspeak": {"sbrief": "subset above R arrow"}},
      "key": "2979"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow through subset"},
        "mathspeak": {"sbrief": "L arrow through subset"}
      },
      "key": "297A"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "superset above left arrow"},
        "mathspeak": {"sbrief": "suerset above L arrow"}
      },
      "key": "297B"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "empty set with right arrow above"},
        "mathspeak": {"sbrief": "empty set with R arrow above"}
      },
      "key": "29B3"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "empty set with left arrow above"},
        "mathspeak": {"sbrief": "empty set with L arrow above"}
      },
      "key": "29B4"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "up arrow through circle"},
        "mathspeak": {"sbrief": "U arrow through circle"}
      },
      "key": "29BD"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "black diamond with down arrow"},
        "mathspeak": {"sbrief": "black diamond with D arrow"}
      },
      "key": "29EA"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "white circle with down arrow"},
        "mathspeak": {"sbrief": "white circle with D arrow"}
      },
      "key": "29EC"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "black circle with down arrow"},
        "mathspeak": {"sbrief": "black circle with D arrow"}
      },
      "key": "29ED"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "integral with left arrow with hook"},
        "mathspeak": {"sbrief": "integral with L arrow with hook"}
      },
      "key": "2A17"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "north east white arrow"}},
      "key": "2B00"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "north west white arrow"}},
      "key": "2B01"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "south east white arrow"}},
      "key": "2B02"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "south west white arrow"}},
      "key": "2B03"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left right white arrow"}, "mathspeak": {"sbrief": "L R white arrow"}},
      "key": "2B04"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left black arrow"}, "mathspeak": {"sbrief": "L black arrow"}},
      "key": "2B05"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up black arrow"}, "mathspeak": {"sbrief": "U black arrow"}},
      "key": "2B06"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down black arrow"}, "mathspeak": {"sbrief": "D black arrow"}},
      "key": "2B07"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "north east black arrow"}},
      "key": "2B08"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "north west black arrow"}},
      "key": "2B09"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "south east black arrow"}},
      "key": "2B0A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "south west black arrow"}},
      "key": "2B0B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left right black arrow"}, "mathspeak": {"sbrief": "L R black arrow"}},
      "key": "2B0C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up down black arrow"}, "mathspeak": {"sbrief": "U D black arrow"}},
      "key": "2B0D"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "right arrow with tip down"},
        "mathspeak": {"sbrief": "R arrow with tip down"}
      },
      "key": "2B0E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right arrow with tip up"}, "mathspeak": {"sbrief": "R arrow with tip up"}},
      "key": "2B0F"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "left arrow with tip down"},
        "mathspeak": {"sbrief": "L arrow with tip down"}
      },
      "key": "2B10"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left arrow with tip up"}, "mathspeak": {"sbrief": "L arrow with tip up"}},
      "key": "2B11"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow with small circle"},
        "mathspeak": {"sbrief": "L arrow with small circle"}
      },
      "key": "2B30"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "three left arrows"}, "mathspeak": {"sbrief": "three L arrows"}},
      "key": "2B31"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow with circled plus"},
        "mathspeak": {"sbrief": "L arrow with circled plus"}
      },
      "key": "2B32"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "long left squiggle arrow"},
        "mathspeak": {"sbrief": "long l squiggle arrow"}
      },
      "key": "2B33"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left two headed arrow with vertical stroke"},
        "mathspeak": {"sbrief": "l two headed arrow with vertical stroke"}
      },
      "key": "2B34"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left two headed arrow with double vertical stroke"},
        "mathspeak": {"sbrief": "l two headed arrow with double vertical stroke"}
      },
      "key": "2B35"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left two headed arrow from bar"},
        "mathspeak": {"sbrief": "l two headed arrow from bar"}
      },
      "key": "2B36"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left two headed triple dash arrow"},
        "mathspeak": {"sbrief": "l two headed triple dash arrow"}
      },
      "key": "2B37"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow with dotted stem"},
        "mathspeak": {"sbrief": "L arrow with dotted stem"}
      },
      "key": "2B38"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow with tail with vertical stroke"},
        "mathspeak": {"sbrief": "L arrow with tail with vertical stroke"}
      },
      "key": "2B39"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow with tail with double vertical stroke"},
        "mathspeak": {"sbrief": "L arrow with tail with double vertical stroke"}
      },
      "key": "2B3A"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left two headed arrow with tail"},
        "mathspeak": {"sbrief": "l two headed arrow with tail"}
      },
      "key": "2B3B"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left two headed arrow with tail with vertical stroke"},
        "mathspeak": {"sbrief": "l two headed arrow with tail with vertical stroke"}
      },
      "key": "2B3C"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left two headed arrow with tail with double vertical stroke"},
        "mathspeak": {"sbrief": "l two headed arrow with tail with double vertical stroke"}
      },
      "key": "2B3D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left arrow through x"}, "mathspeak": {"sbrief": "L arrow through x"}},
      "key": "2B3E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "wave arrow pointing directly left"}},
      "key": "2B3F"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "equals sign above left arrow"},
        "mathspeak": {"sbrief": "equals sign above L arrow"}
      },
      "key": "2B40"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "reverse tilde operator above left arrow"},
        "mathspeak": {"sbrief": "reverse tilde operator above L arrow"}
      },
      "key": "2B41"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow above reverse almost equals"},
        "mathspeak": {"sbrief": "L arrow above reverse almost equals"}
      },
      "key": "2B42"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow through greater than"},
        "mathspeak": {"sbrief": "R arrow through greater than"}
      },
      "key": "2B43"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow through superset"},
        "mathspeak": {"sbrief": "R arrow through superset"}
      },
      "key": "2B44"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left quadruple arrow"}, "mathspeak": {"sbrief": "l quadrule arrow"}},
      "key": "2B45"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right quadruple arrow"}, "mathspeak": {"sbrief": "r quadrule arrow"}},
      "key": "2B46"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "reverse tilde operator above right arrow"},
        "mathspeak": {"sbrief": "reverse tilde operator above R arrow"}
      },
      "key": "2B47"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow above reverse almost equals"},
        "mathspeak": {"sbrief": "R arrow above reverse almost equals"}
      },
      "key": "2B48"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "tilde operator above left arrow"},
        "mathspeak": {"sbrief": "tilde operator above L arrow"}
      },
      "key": "2B49"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow above almost equals"},
        "mathspeak": {"sbrief": "L arrow above almost equals"}
      },
      "key": "2B4A"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left arrow above reverse tilde operator"},
        "mathspeak": {"sbrief": "L arrow above reverse tilde operator"}
      },
      "key": "2B4B"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right arrow above reverse tilde operator"},
        "mathspeak": {"sbrief": "R arrow above reverse tilde operator"}
      },
      "key": "2B4C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "halfwidth left arrow"}, "mathspeak": {"sbrief": "halfwidth L arrow"}},
      "key": "FFE9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "halfwidth up arrow"}, "mathspeak": {"sbrief": "halfwidth U arrow"}},
      "key": "FFEA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "halfwidth right arrow"}, "mathspeak": {"sbrief": "halfwidth R arrow"}},
      "key": "FFEB"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "halfwidth down arrow"}, "mathspeak": {"sbrief": "halfwidth D arrow"}},
      "key": "FFEC"
    }]
  ,
  "en/symbols/math_characters.js":
    [{"locale": "en"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "script l"}},
      "key": "2113"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "script cap P"}, "mathspeak": {"default": "script upper P"}},
      "key": "2118"
    }, {"category": "Ll", "mappings": {"default": {"default": "double struck pi"}}, "key": "213C"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "double struck gamma"}},
      "key": "213D"
    }, {
      "category": "Lu",
      "mappings": {
        "default": {"default": "double struck cap Gamma"},
        "mathspeak": {"default": "double struck upper Gamma"}
      },
      "key": "213E"
    }, {
      "category": "Lu",
      "mappings": {"default": {"default": "double struck cap Pi"}, "mathspeak": {"default": "double struck upper Pi"}},
      "key": "213F"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "double struck sum"},
        "mathspeak": {"default": "double struck sigma summation"}
      },
      "key": "2140"
    }, {
      "category": "Lu",
      "mappings": {
        "default": {"default": "double struck italic cap D"},
        "mathspeak": {"default": "double struck italic upper D"}
      },
      "key": "2145"
    }, {
      "category": "Ll",
      "mappings": {"default": {"default": "double struck italic d"}},
      "key": "2146"
    }, {
      "category": "Ll",
      "mappings": {"default": {"default": "double struck italic e"}},
      "key": "2147"
    }, {
      "category": "Ll",
      "mappings": {"default": {"default": "double struck italic i"}},
      "key": "2148"
    }, {
      "category": "Ll",
      "mappings": {"default": {"default": "double struck italic j"}},
      "key": "2149"
    }, {"category": "Ll", "mappings": {"default": {"default": "italic dotless i"}}, "key": "1D6A4"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "italic dotless j"}},
      "key": "1D6A5"
    }]
  ,
  "en/symbols/math_delimiters.js":
    [{"locale": "en"}, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "left parenthesis"},
        "mathspeak": {"brief": "left p'ren", "sbrief": "L p'ren"},
        "clearspeak": {"default": "open paren"}
      },
      "key": "0028"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "right parenthesis"},
        "mathspeak": {"brief": "right p'ren", "sbrief": "R p'ren"},
        "clearspeak": {"default": "close paren"}
      },
      "key": "0029"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "left bracket"},
        "mathspeak": {"brief": "left brack", "sbrief": "L brack"},
        "clearspeak": {"default": "open bracket"}
      },
      "key": "005B"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "right bracket"},
        "mathspeak": {"brief": "right brack", "sbrief": "R brack"},
        "clearspeak": {"default": "close bracket"}
      },
      "key": "005D"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "left brace"},
        "mathspeak": {"sbrief": "L brace"},
        "clearspeak": {"default": "open brace"}
      },
      "key": "007B"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "right brace"},
        "mathspeak": {"sbrief": "R brace"},
        "clearspeak": {"default": "close brace"}
      },
      "key": "007D"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "left bracket with quill"},
        "mathspeak": {"brief": "left brack with quill", "sbrief": "L brack with quill"}
      },
      "key": "2045"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "right bracket with quill"},
        "mathspeak": {"brief": "right brack with quill", "sbrief": "R brack with quill"}
      },
      "key": "2046"
    }, {"category": "Sm", "mappings": {"default": {"default": "left ceiling"}}, "key": "2308"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "right ceiling"}},
      "key": "2309"
    }, {"category": "Sm", "mappings": {"default": {"default": "left floor"}}, "key": "230A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "right floor"}},
      "key": "230B"
    }, {"category": "So", "mappings": {"default": {"default": "bottom right crop"}}, "key": "230C"}, {
      "category": "So",
      "mappings": {"default": {"default": "bottom left crop"}},
      "key": "230D"
    }, {"category": "So", "mappings": {"default": {"default": "top right crop"}}, "key": "230E"}, {
      "category": "So",
      "mappings": {"default": {"default": "top left crop"}},
      "key": "230F"
    }, {"category": "So", "mappings": {"default": {"default": "top left corner"}}, "key": "231C"}, {
      "category": "So",
      "mappings": {"default": {"default": "top right corner"}},
      "key": "231D"
    }, {"category": "So", "mappings": {"default": {"default": "bottom left corner"}}, "key": "231E"}, {
      "category": "So",
      "mappings": {"default": {"default": "bottom right corner"}},
      "key": "231F"
    }, {"category": "Sm", "mappings": {"default": {"default": "top half integral"}}, "key": "2320"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "bottom half integral"}},
      "key": "2321"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left pointing angle bracket", "physics": "bra"}},
      "key": "2329"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right pointing angle bracket", "physics": "ket"}},
      "key": "232A"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left parenthesis upper hook"},
        "mathspeak": {"brief": "left p'ren upper hook", "sbrief": "L p'ren upper hook"}
      },
      "key": "239B"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left parenthesis extension"},
        "mathspeak": {"brief": "left p'ren extension", "sbrief": "L p'ren extension"}
      },
      "key": "239C"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left parenthesis lower hook"},
        "mathspeak": {"brief": "left p'ren lower hook", "sbrief": "L p'ren lower hook"}
      },
      "key": "239D"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right parenthesis upper hook"},
        "mathspeak": {"brief": "right p'ren upper hook", "sbrief": "R p'ren upper hook"}
      },
      "key": "239E"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right parenthesis extension"},
        "mathspeak": {"brief": "right p'ren extension", "sbrief": "R p'ren extension"}
      },
      "key": "239F"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right parenthesis lower hook"},
        "mathspeak": {"brief": "right p'ren lower hook", "sbrief": "R p'ren lower hook"}
      },
      "key": "23A0"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left bracket upper corner"},
        "mathspeak": {"brief": "left brack upper corner", "sbrief": "L brack upper corner"}
      },
      "key": "23A1"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left bracket extension"},
        "mathspeak": {"brief": "left brack extension", "sbrief": "L brack extension"}
      },
      "key": "23A2"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "left bracket lower corner"},
        "mathspeak": {"brief": "left brack lower corner", "sbrief": "L brack lower corner"}
      },
      "key": "23A3"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right bracket upper corner"},
        "mathspeak": {"brief": "right brack upper corner", "sbrief": "R brack upper corner"}
      },
      "key": "23A4"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right bracket extension"},
        "mathspeak": {"brief": "right brack extension", "sbrief": "R brack extension"}
      },
      "key": "23A5"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "right bracket lower corner"},
        "mathspeak": {"brief": "right brack lower corner", "sbrief": "R brack lower corner"}
      },
      "key": "23A6"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left brace upper hook"}, "mathspeak": {"sbrief": "L brace upper hook"}},
      "key": "23A7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left brace middle piece"}, "mathspeak": {"sbrief": "L brace middle piece"}},
      "key": "23A8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left brace lower hook"}, "mathspeak": {"sbrief": "L brace lower hook"}},
      "key": "23A9"
    }, {"category": "Sm", "mappings": {"default": {"default": "brace extension"}}, "key": "23AA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "right brace upper hook"}, "mathspeak": {"sbrief": "R brace upper hook"}},
      "key": "23AB"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right brace middle piece"}, "mathspeak": {"sbrief": "R brace middle piece"}},
      "key": "23AC"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right brace lower hook"}, "mathspeak": {"sbrief": "R brace lower hook"}},
      "key": "23AD"
    }, {"category": "Sm", "mappings": {"default": {"default": "integral extension"}}, "key": "23AE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "horizontal line extension"}},
      "key": "23AF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "upper left or lower right brace section"}},
      "key": "23B0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "upper right or lower left brace section"}},
      "key": "23B1"
    }, {"category": "Sm", "mappings": {"default": {"default": "summation top"}}, "key": "23B2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "summation bottom"}},
      "key": "23B3"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "top bracket"}, "mathspeak": {"brief": "top brack", "sbrief": "T brack"}},
      "key": "23B4"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "bottom bracket"},
        "mathspeak": {"brief": "bottom brack", "sbrief": "B brack"}
      },
      "key": "23B5"
    }, {
      "category": "So",
      "mappings": {
        "default": {"default": "bottom bracket over top bracket"},
        "mathspeak": {"brief": "bottom brack over top brack", "sbrief": "B brack over T brack"}
      },
      "key": "23B6"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "radical symbol bottom"}},
      "key": "23B7"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left vertical box line"}},
      "key": "23B8"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right vertical box line"}},
      "key": "23B9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "top parenthesis"}, "mathspeak": {"brief": "top p'ren", "sbrief": "t p'ren"}},
      "key": "23DC"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "bottom parenthesis"},
        "mathspeak": {"brief": "bottom p'ren", "sbrief": "b p'ren"}
      },
      "key": "23DD"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "top brace"}, "mathspeak": {"sbrief": "T brace"}},
      "key": "23DE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "bottom brace"}, "mathspeak": {"sbrief": "B brace"}},
      "key": "23DF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "top tortoise shell bracket"}},
      "key": "23E0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "bottom tortoise shell bracket"}},
      "key": "23E1"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "medium left parenthesis ornament"},
        "mathspeak": {"brief": "medium left p'ren ornament", "sbrief": "medium L p'ren ornament"}
      },
      "key": "2768"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "medium right parenthesis ornament"},
        "mathspeak": {"brief": "medium right p'ren ornament", "sbrief": "medium R p'ren ornament"}
      },
      "key": "2769"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "medium flattened left parenthesis ornament"},
        "mathspeak": {"brief": "medium flattened left p'ren ornament", "sbrief": "medium flattened L p'ren ornament"}
      },
      "key": "276A"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "medium flattened right parenthesis ornament"},
        "mathspeak": {"brief": "medium flattened right p'ren ornament", "sbrief": "medium flattened R p'ren ornament"}
      },
      "key": "276B"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "medium left pointing angle bracket ornament"}},
      "key": "276C"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "medium right pointing angle bracket ornament"}},
      "key": "276D"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "heavy left pointing angle quotation mark ornament"}},
      "key": "276E"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "heavy right pointing angle quotation mark ornament"}},
      "key": "276F"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "heavy left pointing angle bracket ornament"}},
      "key": "2770"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "heavy right pointing angle bracket ornament"}},
      "key": "2771"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "light left tortoise shell bracket ornament"}},
      "key": "2772"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "light right tortoise shell bracket ornament"}},
      "key": "2773"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "medium left brace ornament"},
        "mathspeak": {"sbrief": "medium L brace ornament"}
      },
      "key": "2774"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "medium right brace ornament"},
        "mathspeak": {"sbrief": "medium R brace ornament"}
      },
      "key": "2775"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left s shaped bag delimiter"}},
      "key": "27C5"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right s shaped bag delimiter"}},
      "key": "27C6"
    }, {"category": "Ps", "mappings": {"default": {"default": "left white bracket"}}, "key": "27E6"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "right white bracket"}},
      "key": "27E7"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left angle bracket"}, "mathspeak": {"sbrief": "l angle bracket"}},
      "key": "27E8"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right angle bracket"}, "mathspeak": {"sbrief": "r angle bracket"}},
      "key": "27E9"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left double angle bracket"}},
      "key": "27EA"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right double angle bracket"}},
      "key": "27EB"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left white tortoise shell bracket"}},
      "key": "27EC"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right white tortoise shell bracket"}},
      "key": "27ED"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "flattened left parenthesis"},
        "mathspeak": {"brief": "flattened left p'ren", "sbrief": "flattened L p'ren"}
      },
      "key": "27EE"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "flattened right parenthesis"},
        "mathspeak": {"brief": "flattened right p'ren", "sbrief": "flattened R p'ren"}
      },
      "key": "27EF"
    }, {"category": "Ps", "mappings": {"default": {"default": "left white brace"}}, "key": "2983"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "right white brace"}},
      "key": "2984"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "white left parenthesis"},
        "mathspeak": {"brief": "white left p'ren", "sbrief": "white L p'ren"}
      },
      "key": "2985"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "white right parenthesis"},
        "mathspeak": {"brief": "white right p'ren", "sbrief": "white R p'ren"}
      },
      "key": "2986"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "z notation left image bracket"}},
      "key": "2987"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "z notation right image bracket"}},
      "key": "2988"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "z notation left binding bracket"}},
      "key": "2989"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "z notation right binding bracket"}},
      "key": "298A"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "left bracket with underbar"},
        "mathspeak": {"brief": "left brack with underbar", "sbrief": "L brack with underbar"}
      },
      "key": "298B"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "right bracket with underbar"},
        "mathspeak": {"brief": "right brack with underbar", "sbrief": "R brack with underbar"}
      },
      "key": "298C"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "left bracket with tick in top corner"},
        "mathspeak": {"brief": "left brack with tick in top corner", "sbrief": "L brack with tick in top corner"}
      },
      "key": "298D"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "right bracket with tick in bottom corner"},
        "mathspeak": {"brief": "right brack with tick in bottom corner", "sbrief": "R brack with tick in bottom corner"}
      },
      "key": "298E"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "left bracket with tick in bottom corner"},
        "mathspeak": {"brief": "left brack with tick in bottom corner", "sbrief": "L brack with tick in bottom corner"}
      },
      "key": "298F"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "right bracket with tick in top corner"},
        "mathspeak": {"brief": "right brack with tick in top corner", "sbrief": "R brack with tick in top corner"}
      },
      "key": "2990"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "left angle bracket with dot"},
        "mathspeak": {"sbrief": "l angle bracket with dot"}
      },
      "key": "2991"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "right angle bracket with dot"},
        "mathspeak": {"sbrief": "r angle bracket with dot"}
      },
      "key": "2992"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left arc less than bracket"}},
      "key": "2993"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right arc greater than bracket"}},
      "key": "2994"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "double left arc greater than bracket"}},
      "key": "2995"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "double right arc less than bracket"}},
      "key": "2996"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left black tortoise shell bracket"}},
      "key": "2997"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right black tortoise shell bracket"}},
      "key": "2998"
    }, {"category": "Ps", "mappings": {"default": {"default": "left wiggly fence"}}, "key": "29D8"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "right wiggly fence"}},
      "key": "29D9"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left double wiggly fence"}},
      "key": "29DA"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right double wiggly fence"}},
      "key": "29DB"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left pointing curved angle bracket"}},
      "key": "29FC"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right pointing curved angle bracket"}},
      "key": "29FD"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "top half left bracket"},
        "mathspeak": {"brief": "top half left brack", "sbrief": "top half L brack"}
      },
      "key": "2E22"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "top half right bracket"},
        "mathspeak": {"brief": "top half right brack", "sbrief": "top half R brack"}
      },
      "key": "2E23"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "bottom half left bracket"},
        "mathspeak": {"brief": "bottom half left brack", "sbrief": "bottom half L brack"}
      },
      "key": "2E24"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "bottom half right bracket"},
        "mathspeak": {"brief": "bottom half right brack", "sbrief": "bottom half R brack"}
      },
      "key": "2E25"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left sideways U bracket"}},
      "key": "2E26"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right sideways U bracket"}},
      "key": "2E27"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "double left parenthesis"},
        "mathspeak": {"brief": "double left p'ren", "sbrief": "double L p'ren"}
      },
      "key": "2E28"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "double right parenthesis"},
        "mathspeak": {"brief": "double right p'ren", "sbrief": "double R p'ren"}
      },
      "key": "2E29"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left angle bracket"}, "mathspeak": {"sbrief": "l angle bracket"}},
      "key": "3008"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right angle bracket"}, "mathspeak": {"sbrief": "r angle bracket"}},
      "key": "3009"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left double angle bracket"}},
      "key": "300A"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right double angle bracket"}},
      "key": "300B"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left corner bracket"}},
      "key": "300C"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right corner bracket"}},
      "key": "300D"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left white corner bracket"}},
      "key": "300E"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right white corner bracket"}},
      "key": "300F"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left black lenticular bracket"}},
      "key": "3010"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right black lenticular bracket"}},
      "key": "3011"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left tortoise shell bracket"}},
      "key": "3014"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right tortoise shell bracket"}},
      "key": "3015"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left white lenticular bracket"}},
      "key": "3016"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right white lenticular bracket"}},
      "key": "3017"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "left white tortoise shell bracket"}},
      "key": "3018"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "right white tortoise shell bracket"}},
      "key": "3019"
    }, {"category": "Ps", "mappings": {"default": {"default": "left white bracket"}}, "key": "301A"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "right white bracket"}},
      "key": "301B"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "reversed double prime quotation mark"}},
      "key": "301D"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "double prime quotation mark"}},
      "key": "301E"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "low double prime quotation mark"}},
      "key": "301F"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "ornate left parenthesis"},
        "mathspeak": {"brief": "ornate left p'ren", "sbrief": "ornate L p'ren"}
      },
      "key": "FD3E"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "ornate right parenthesis"},
        "mathspeak": {"brief": "ornate right p'ren", "sbrief": "ornate R p'ren"}
      },
      "key": "FD3F"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "presentation form for vertical left white lenticular bracket"}},
      "key": "FE17"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "presentation form for vertical right white lenticular brakcet"}},
      "key": "FE18"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "presentation form for vertical left parenthesis"},
        "mathspeak": {
          "brief": "presentation form for vertical left p'ren",
          "sbrief": "presentation form for vertical L p'ren"
        }
      },
      "key": "FE35"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "presentation form for vertical right parenthesis"},
        "mathspeak": {
          "brief": "presentation form for vertical right p'ren",
          "sbrief": "presentation form for vertical R p'ren"
        }
      },
      "key": "FE36"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "presentation form for vertical left brace"},
        "mathspeak": {"sbrief": "presentation form for vertical L brace"}
      },
      "key": "FE37"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "presentation form for vertical right brace"},
        "mathspeak": {"sbrief": "presentation form for vertical r brace"}
      },
      "key": "FE38"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "presentation form for vertical left tortoise shell bracket"}},
      "key": "FE39"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "presentation form for vertical right tortoise shell bracket"}},
      "key": "FE3A"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "presentation form for vertical left black lenticular bracket"}},
      "key": "FE3B"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "presentation form for vertical right black lenticular bracket"}},
      "key": "FE3C"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "presentation form for vertical left double angle bracket"}},
      "key": "FE3D"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "presentation form for vertical right double angle bracket"}},
      "key": "FE3E"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "presentation form for vertical left angle bracket"},
        "mathspeak": {"sbrief": "presentation form for vertical l angle bracket"}
      },
      "key": "FE3F"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "presentation form for vertical right angle bracket"},
        "mathspeak": {"sbrief": "presentation form for vertical r angle bracket"}
      },
      "key": "FE40"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "presentation form for vertical left corner bracket"}},
      "key": "FE41"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "presentation form for vertical right corner bracket"}},
      "key": "FE42"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "presentation form for vertical left white corner bracket"}},
      "key": "FE43"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "presentation form for vertical right white corner bracket"}},
      "key": "FE44"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "presentation form for vertical left bracket"},
        "mathspeak": {
          "brief": "presentation form for vertical left brack",
          "sbrief": "presentation form for vertical L brack"
        }
      },
      "key": "FE47"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "presentation form for vertical right bracket"},
        "mathspeak": {
          "brief": "presentation form for vertical right brack",
          "sbrief": "presentation form for vertical r brack"
        }
      },
      "key": "FE48"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "small left parenthesis"},
        "mathspeak": {"brief": "small left p'ren", "sbrief": "small L p'ren"}
      },
      "key": "FE59"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "small right parenthesis"},
        "mathspeak": {"brief": "small right p'ren", "sbrief": "small R p'ren"}
      },
      "key": "FE5A"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "small left brace"}, "mathspeak": {"sbrief": "small L brace"}},
      "key": "FE5B"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "small right brace"}, "mathspeak": {"sbrief": "small r brace"}},
      "key": "FE5C"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "small left tortoise shell bracket"}},
      "key": "FE5D"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "small right tortoise shell bracket"}},
      "key": "FE5E"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "fullwidth left parenthesis"},
        "mathspeak": {"brief": "fullwidth left p'ren", "sbrief": "fullwidth L p'ren"}
      },
      "key": "FF08"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "fullwidth right parenthesis"},
        "mathspeak": {"brief": "fullwidth right p'ren", "sbrief": "fullwidth R p'ren"}
      },
      "key": "FF09"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "fullwidth left bracket"},
        "mathspeak": {"brief": "fullwidth left brack", "sbrief": "fullwidth L brack"}
      },
      "key": "FF3B"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "fullwidth right bracket"},
        "mathspeak": {"brief": "fullwidth right brack", "sbrief": "fullwidth r brack"}
      },
      "key": "FF3D"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "fullwidth left brace"}, "mathspeak": {"sbrief": "fullwidth L brace"}},
      "key": "FF5B"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "fullwidth right brace"}, "mathspeak": {"sbrief": "fullwidth r brace"}},
      "key": "FF5D"
    }, {
      "category": "Ps",
      "mappings": {
        "default": {"default": "fullwidth white left parenthesis"},
        "mathspeak": {"brief": "fullwidth white left p'ren", "sbrief": "fullwidth white L p'ren"}
      },
      "key": "FF5F"
    }, {
      "category": "Pe",
      "mappings": {
        "default": {"default": "fullwidth white right parenthesis"},
        "mathspeak": {"brief": "fullwidth white right p'ren", "sbrief": "fullwidth white R p'ren"}
      },
      "key": "FF60"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "halfwidth left corner bracket"}},
      "key": "FF62"
    }, {"category": "Pe", "mappings": {"default": {"default": "halfwidth right corner bracket"}}, "key": "FF63"}]
  ,
  "en/symbols/math_geometry.js":
    [{"locale": "en"}, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light horizontal"}},
      "key": "2500"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy horizontal"}},
      "key": "2501"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light vertical"}},
      "key": "2502"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy vertical"}},
      "key": "2503"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light triple dash horizontal"}},
      "key": "2504"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy triple dash horizontal"}},
      "key": "2505"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light triple dash vertical"}},
      "key": "2506"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy triple dash vertical"}},
      "key": "2507"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light quadruple dash horizontal"}},
      "key": "2508"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy quadruple dash horizontal"}},
      "key": "2509"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light quadruple dash vertical"}},
      "key": "250A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy quadruple dash vertical"}},
      "key": "250B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light down and right"}},
      "key": "250C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down light and right heavy"}},
      "key": "250D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down heavy and right light"}},
      "key": "250E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy down and right"}},
      "key": "250F"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light down and left"}},
      "key": "2510"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down light and left heavy"}},
      "key": "2511"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down heavy and left light"}},
      "key": "2512"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy down and left"}},
      "key": "2513"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light up and right"}},
      "key": "2514"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up light and right heavy"}},
      "key": "2515"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up heavy and right light"}},
      "key": "2516"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy up and right"}},
      "key": "2517"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light up and left"}},
      "key": "2518"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up light and left heavy"}},
      "key": "2519"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up heavy and left light"}},
      "key": "251A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy up and left"}},
      "key": "251B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light vertical and right"}},
      "key": "251C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings vertical light and right heavy"}},
      "key": "251D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up heavy and right down light"}},
      "key": "251E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down heavy and right up light"}},
      "key": "251F"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings vertical heavy and right light"}},
      "key": "2520"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down light and right up heavy"}},
      "key": "2521"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up light and right down heavy"}},
      "key": "2522"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy vertical and right"}},
      "key": "2523"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light vertical and left"}},
      "key": "2524"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings vertical light and left heavy"}},
      "key": "2525"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up heavy and left down light"}},
      "key": "2526"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down heavy and left up light"}},
      "key": "2527"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings vertical heavy and left light"}},
      "key": "2528"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down light and left up heavy"}},
      "key": "2529"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up light and left down heavy"}},
      "key": "252A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy vertical and left"}},
      "key": "252B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light down and horizontal"}},
      "key": "252C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings left heavy and right down light"}},
      "key": "252D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings right heavy and left down light"}},
      "key": "252E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down light and horizontal heavy"}},
      "key": "252F"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down heavy and horizontal light"}},
      "key": "2530"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings right light and left down heavy"}},
      "key": "2531"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings left light and right down heavy"}},
      "key": "2532"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy down and horizontal"}},
      "key": "2533"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light up and horizontal"}},
      "key": "2534"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings left heavy and right up light"}},
      "key": "2535"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings right heavy and left up light"}},
      "key": "2536"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up light and horizontal heavy"}},
      "key": "2537"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up heavy and horizontal light"}},
      "key": "2538"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings right light and left up heavy"}},
      "key": "2539"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings left light and right up heavy"}},
      "key": "253A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy up and horizontal"}},
      "key": "253B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light vertical and horizontal"}},
      "key": "253C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings left heavy and right vertical light"}},
      "key": "253D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings right heavy and left vertical light"}},
      "key": "253E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings vertical light and horizontal heavy"}},
      "key": "253F"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up heavy and down horizontal light"}},
      "key": "2540"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down heavy and up horizontal light"}},
      "key": "2541"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings vertical heavy and horizontal light"}},
      "key": "2542"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings left up heavy and right down light"}},
      "key": "2543"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings right up heavy and left down light"}},
      "key": "2544"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings left down heavy and right up light"}},
      "key": "2545"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings right down heavy and left up light"}},
      "key": "2546"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down light and up horizontal heavy"}},
      "key": "2547"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up light and down horizontal heavy"}},
      "key": "2548"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings right light and left vertical heavy"}},
      "key": "2549"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings left light and right vertical heavy"}},
      "key": "254A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy vertical and horizontal"}},
      "key": "254B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light double dash horizontal"}},
      "key": "254C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy double dash horizontal"}},
      "key": "254D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light double dash vertical"}},
      "key": "254E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy double dash vertical"}},
      "key": "254F"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings double horizontal"}},
      "key": "2550"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings double vertical"}},
      "key": "2551"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down single and right double"}},
      "key": "2552"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down double and right single"}},
      "key": "2553"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings double down and right"}},
      "key": "2554"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down single and left double"}},
      "key": "2555"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down double and left single"}},
      "key": "2556"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings double down and left"}},
      "key": "2557"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up single and right double"}},
      "key": "2558"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up double and right single"}},
      "key": "2559"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings double up and right"}},
      "key": "255A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up single and left double"}},
      "key": "255B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up double and left single"}},
      "key": "255C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings double up and left"}},
      "key": "255D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings vertical single and right double"}},
      "key": "255E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings vertical double and right single"}},
      "key": "255F"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings double vertical and right"}},
      "key": "2560"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings vertical single and left double"}},
      "key": "2561"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings vertical double and left single"}},
      "key": "2562"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings double vertical and left"}},
      "key": "2563"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down single and horizontal double"}},
      "key": "2564"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings down double and horizontal single"}},
      "key": "2565"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings double down and horizontal"}},
      "key": "2566"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up single and horizontal double"}},
      "key": "2567"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings up double and horizontal single"}},
      "key": "2568"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings double up and horizontal"}},
      "key": "2569"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings vertical single and horizontal double"}},
      "key": "256A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings vertical double and horizontal single"}},
      "key": "256B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings double vertical and horizontal"}},
      "key": "256C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light arc down and right"}},
      "key": "256D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light arc down and left"}},
      "key": "256E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light arc up and left"}},
      "key": "256F"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light arc up and right"}},
      "key": "2570"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light diagonal upper right to lower left"}},
      "key": "2571"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light diagonal upper left to lower right"}},
      "key": "2572"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light diagonal cross"}},
      "key": "2573"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light left"}},
      "key": "2574"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light up"}},
      "key": "2575"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light right"}},
      "key": "2576"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light down"}},
      "key": "2577"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy left"}},
      "key": "2578"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy up"}},
      "key": "2579"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy right"}},
      "key": "257A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy down"}},
      "key": "257B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light left and heavy right"}},
      "key": "257C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings light up and heavy down"}},
      "key": "257D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy left and light right"}},
      "key": "257E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "box drawings heavy up and light down"}},
      "key": "257F"
    }, {"category": "So", "mappings": {"default": {"default": "upper half block"}}, "key": "2580"}, {
      "category": "So",
      "mappings": {"default": {"default": "lower one eighth block"}},
      "key": "2581"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "lower one quarter block"}},
      "key": "2582"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "lower three eighths block"}},
      "key": "2583"
    }, {"category": "So", "mappings": {"default": {"default": "lower half block"}}, "key": "2584"}, {
      "category": "So",
      "mappings": {"default": {"default": "lower five eighths block"}},
      "key": "2585"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "lower three quarters block"}},
      "key": "2586"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "lower seven eighths block"}},
      "key": "2587"
    }, {"category": "So", "mappings": {"default": {"default": "full block"}}, "key": "2588"}, {
      "category": "So",
      "mappings": {"default": {"default": "left seven eighths block"}},
      "key": "2589"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left three quarters block"}},
      "key": "258A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left five eighths block"}},
      "key": "258B"
    }, {"category": "So", "mappings": {"default": {"default": "left half block"}}, "key": "258C"}, {
      "category": "So",
      "mappings": {"default": {"default": "left three eighths block"}},
      "key": "258D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left one quarter block"}},
      "key": "258E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left one eighth block"}},
      "key": "258F"
    }, {"category": "So", "mappings": {"default": {"default": "right half block"}}, "key": "2590"}, {
      "category": "So",
      "mappings": {"default": {"default": "light shade"}},
      "key": "2591"
    }, {"category": "So", "mappings": {"default": {"default": "medium shade"}}, "key": "2592"}, {
      "category": "So",
      "mappings": {"default": {"default": "dark shade"}},
      "key": "2593"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "upper one eighth block"}},
      "key": "2594"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right one eighth block"}},
      "key": "2595"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "quadrant lower left"}},
      "key": "2596"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "quadrant lower right"}},
      "key": "2597"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "quadrant upper left"}},
      "key": "2598"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "quadrant upper left and lower left and lower right"}},
      "key": "2599"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "quadrant upper left and lower right"}},
      "key": "259A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "quadrant upper left and upper right and lower left"}},
      "key": "259B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "quadrant upper left and upper right and lower right"}},
      "key": "259C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "quadrant upper right"}},
      "key": "259D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "quadrant upper right and lower left"}},
      "key": "259E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "quadrant upper right and lower left and lower right"}},
      "key": "259F"
    }, {"category": "So", "mappings": {"default": {"default": "black square"}}, "key": "25A0"}, {
      "category": "So",
      "mappings": {"default": {"default": "white square"}},
      "key": "25A1"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white square with rounded corners"}},
      "key": "25A2"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white square containing black small square"}},
      "key": "25A3"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with horizontal fill"}},
      "key": "25A4"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with vertical fill"}},
      "key": "25A5"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with orthogonal crosshatch fill"}},
      "key": "25A6"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with upper left to lower right fill"}},
      "key": "25A7"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with upper right to lower left fill"}},
      "key": "25A8"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with diagonal crosshatch fill"}},
      "key": "25A9"
    }, {"category": "So", "mappings": {"default": {"default": "black small square"}}, "key": "25AA"}, {
      "category": "So",
      "mappings": {"default": {"default": "white small square"}},
      "key": "25AB"
    }, {"category": "So", "mappings": {"default": {"default": "black rectangle"}}, "key": "25AC"}, {
      "category": "So",
      "mappings": {"default": {"default": "white rectangle"}},
      "key": "25AD"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black vertical rectangle"}},
      "key": "25AE"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white vertical rectangle"}},
      "key": "25AF"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black parallelogram"}},
      "key": "25B0"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white parallelogram"}},
      "key": "25B1"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black up pointing triangle"}},
      "key": "25B2"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white up pointing triangle"}},
      "key": "25B3"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black up pointing small triangle"}},
      "key": "25B4"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white up pointing small triangle"}},
      "key": "25B5"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black right pointing triangle"}},
      "key": "25B6"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "white right pointing triangle"}},
      "key": "25B7"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black right pointing small triangle"}},
      "key": "25B8"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white right pointing small triangle"}},
      "key": "25B9"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black right pointing pointer"}},
      "key": "25BA"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white right pointing pointer"}},
      "key": "25BB"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black down pointing triangle"}},
      "key": "25BC"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white down pointing triangle"}},
      "key": "25BD"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black down pointing small triangle"}},
      "key": "25BE"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white down pointing small triangle"}},
      "key": "25BF"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black left pointing triangle"}},
      "key": "25C0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "white left pointing triangle"}},
      "key": "25C1"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black left pointing small triangle"}},
      "key": "25C2"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white left pointing small triangle"}},
      "key": "25C3"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black left pointing pointer"}},
      "key": "25C4"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white left pointing pointer"}},
      "key": "25C5"
    }, {"category": "So", "mappings": {"default": {"default": "black diamond"}}, "key": "25C6"}, {
      "category": "So",
      "mappings": {"default": {"default": "white diamond"}},
      "key": "25C7"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white diamond containing black small diamond"}},
      "key": "25C8"
    }, {"category": "So", "mappings": {"default": {"default": "fisheye"}}, "key": "25C9"}, {
      "category": "So",
      "mappings": {"default": {"default": "lozenge"}},
      "key": "25CA"
    }, {"category": "So", "mappings": {"default": {"default": "white circle"}}, "key": "25CB"}, {
      "category": "So",
      "mappings": {"default": {"default": "dotted circle"}},
      "key": "25CC"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "circle with vertical fill"}},
      "key": "25CD"
    }, {"category": "So", "mappings": {"default": {"default": "bullseye"}}, "key": "25CE"}, {
      "category": "So",
      "mappings": {"default": {"default": "black circle"}},
      "key": "25CF"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "circle with left half black"}},
      "key": "25D0"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "circle with right half black"}},
      "key": "25D1"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "circle with lower half black"}},
      "key": "25D2"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "circle with upper half black"}},
      "key": "25D3"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "circle with upper right quadrant black"}},
      "key": "25D4"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "circle with all but upper left quadrant black"}},
      "key": "25D5"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left half black circle"}},
      "key": "25D6"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right half black circle"}},
      "key": "25D7"
    }, {"category": "So", "mappings": {"default": {"default": "inverse bullet"}}, "key": "25D8"}, {
      "category": "So",
      "mappings": {"default": {"default": "inverse white circle"}},
      "key": "25D9"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "upper half inverse white circle"}},
      "key": "25DA"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "lower half inverse white circle"}},
      "key": "25DB"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "upper left quadrant circular arc"}},
      "key": "25DC"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "upper right quadrant circular arc"}},
      "key": "25DD"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "lower right quadrant circular arc"}},
      "key": "25DE"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "lower left quadrant circular arc"}},
      "key": "25DF"
    }, {"category": "So", "mappings": {"default": {"default": "upper half circle"}}, "key": "25E0"}, {
      "category": "So",
      "mappings": {"default": {"default": "lower half circle"}},
      "key": "25E1"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black lower right triangle"}},
      "key": "25E2"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black lower left triangle"}},
      "key": "25E3"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black upper left triangle"}},
      "key": "25E4"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black upper right triangle"}},
      "key": "25E5"
    }, {"category": "So", "mappings": {"default": {"default": "white bullet"}}, "key": "25E6"}, {
      "category": "So",
      "mappings": {"default": {"default": "square with left half black"}},
      "key": "25E7"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with right half black"}},
      "key": "25E8"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with upper left diagonal half black"}},
      "key": "25E9"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with lower right diagonal half black"}},
      "key": "25EA"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white square with vertical bisecting line"}},
      "key": "25EB"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white up pointing triangle with dot"}},
      "key": "25EC"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up pointing triangle with left half black"}},
      "key": "25ED"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up pointing triangle with right half black"}},
      "key": "25EE"
    }, {"category": "So", "mappings": {"default": {"default": "large circle"}}, "key": "25EF"}, {
      "category": "So",
      "mappings": {"default": {"default": "white square with upper left quadrant"}},
      "key": "25F0"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white square with lower left quadrant"}},
      "key": "25F1"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white square with lower right quadrant"}},
      "key": "25F2"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white square with upper right quadrant"}},
      "key": "25F3"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white circle with upper left quadrant"}},
      "key": "25F4"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white circle with lower left quadrant"}},
      "key": "25F5"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white circle with lower right quadrant"}},
      "key": "25F6"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white circle with upper right quadrant"}},
      "key": "25F7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "upper left triangle"}},
      "key": "25F8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "upper right triangle"}},
      "key": "25F9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "lower left triangle"}},
      "key": "25FA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "white medium square"}},
      "key": "25FB"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "black medium square"}},
      "key": "25FC"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "white medium small square"}},
      "key": "25FD"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "black medium small square"}},
      "key": "25FE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "lower right triangle"}},
      "key": "25FF"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with top half black"}},
      "key": "2B12"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with bottom half black"}},
      "key": "2B13"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with upper right diagonal half black"}},
      "key": "2B14"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "square with lower left diagonal half black"}},
      "key": "2B15"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "diamond with left half black"}},
      "key": "2B16"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "diamond with right half black"}},
      "key": "2B17"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "diamond with top half black"}},
      "key": "2B18"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "diamond with bottom half black"}},
      "key": "2B19"
    }, {"category": "So", "mappings": {"default": {"default": "dotted square"}}, "key": "2B1A"}, {
      "category": "So",
      "mappings": {"default": {"default": "black large square"}},
      "key": "2B1B"
    }, {"category": "So", "mappings": {"default": {"default": "white large square"}}, "key": "2B1C"}, {
      "category": "So",
      "mappings": {"default": {"default": "black very small square"}},
      "key": "2B1D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white very small square"}},
      "key": "2B1E"
    }, {"category": "So", "mappings": {"default": {"default": "black pentagon"}}, "key": "2B1F"}, {
      "category": "So",
      "mappings": {"default": {"default": "white pentagon"}},
      "key": "2B20"
    }, {"category": "So", "mappings": {"default": {"default": "white hexagon"}}, "key": "2B21"}, {
      "category": "So",
      "mappings": {"default": {"default": "black hexagon"}},
      "key": "2B22"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "horizontal black hexagon"}},
      "key": "2B23"
    }, {"category": "So", "mappings": {"default": {"default": "black large circle"}}, "key": "2B24"}, {
      "category": "So",
      "mappings": {"default": {"default": "black medium diamond"}},
      "key": "2B25"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white medium diamond"}},
      "key": "2B26"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black medium lozenge"}},
      "key": "2B27"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white medium lozenge"}},
      "key": "2B28"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black small diamond"}},
      "key": "2B29"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black small lozenge"}},
      "key": "2B2A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white small lozenge"}},
      "key": "2B2B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black horizontal ellipse"}},
      "key": "2B2C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white horizontal ellipse"}},
      "key": "2B2D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black vertical ellipse"}},
      "key": "2B2E"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white vertical ellipse"}},
      "key": "2B2F"
    }, {"category": "So", "mappings": {"default": {"default": "white medium star"}}, "key": "2B50"}, {
      "category": "So",
      "mappings": {"default": {"default": "black small star"}},
      "key": "2B51"
    }, {"category": "So", "mappings": {"default": {"default": "white small star"}}, "key": "2B52"}, {
      "category": "So",
      "mappings": {"default": {"default": "black right pointing pentagon"}},
      "key": "2B53"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white right pointing pentagon"}},
      "key": "2B54"
    }, {"category": "So", "mappings": {"default": {"default": "heavy large circle"}}, "key": "2B55"}, {
      "category": "So",
      "mappings": {"default": {"default": "heavy oval with oval inside"}},
      "key": "2B56"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "heavy circle with circle inside"}},
      "key": "2B57"
    }, {"category": "So", "mappings": {"default": {"default": "heavy circle"}}, "key": "2B58"}, {
      "category": "So",
      "mappings": {"default": {"default": "heavy circled saltire"}},
      "key": "2B59"
    }]
  ,
  "en/symbols/math_harpoons.js":
    [{"locale": "en"}, {
      "category": "So",
      "mappings": {"default": {"default": "left harpoon with barb up"}},
      "key": "21BC"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left harpoon with barb down"}},
      "key": "21BD"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up harpoon with barb right"}},
      "key": "21BE"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "up harpoon with barb left"}},
      "key": "21BF"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right harpoon with barb up"}},
      "key": "21C0"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right harpoon with barb down"}},
      "key": "21C1"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down harpoon with barb right"}},
      "key": "21C2"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "down harpoon with barb left"}},
      "key": "21C3"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "left harpoon over right harpoon"}},
      "key": "21CB"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "right harpoon over left harpoon"}},
      "key": "21CC"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left barb up right barb down harpoon"}},
      "key": "294A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left barb down right barb up harpoon"}},
      "key": "294B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up barb right down barb left harpoon"}},
      "key": "294C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up barb left down barb right harpoon"}},
      "key": "294D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left barb up right barb up harpoon"}},
      "key": "294E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up barb right down barb right harpoon"}},
      "key": "294F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left barb down right barb down harpoon"}},
      "key": "2950"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up barb left down barb left harpoon"}},
      "key": "2951"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left harpoon with barb up to bar"}},
      "key": "2952"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right harpoon with barb up to bar"}},
      "key": "2953"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up harpoon with barb right to bar"}},
      "key": "2954"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down harpoon with barb right to bar"}},
      "key": "2955"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left harpoon with barb down to bar"}},
      "key": "2956"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right harpoon with barb down to bar"}},
      "key": "2957"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up harpoon with barb left to bar"}},
      "key": "2958"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down harpoon with barb left to bar"}},
      "key": "2959"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left harpoon with barb up from bar"}},
      "key": "295A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right harpoon with barb up from bar"}},
      "key": "295B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up harpoon with barb right from bar"}},
      "key": "295C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down harpoon with barb right from bar"}},
      "key": "295D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left harpoon with barb down from bar"}},
      "key": "295E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right harpoon with barb down from bar"}},
      "key": "295F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up harpoon with barb left from bar"}},
      "key": "2960"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down harpoon with barb left from bar"}},
      "key": "2961"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left harpoon with barb up above left harpoon with barb down"}},
      "key": "2962"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up harpoon with barb left beside up harpoon with barb right"}},
      "key": "2963"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right harpoon with barb up above right harpoon with barb down"}},
      "key": "2964"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down harpoon with barb left beside down harpoon with barb right"}},
      "key": "2965"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left harpoon with barb up above right harpoon with barb up"}},
      "key": "2966"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left harpoon with barb down above right harpoon with barb down"}},
      "key": "2967"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right harpoon with barb up above left harpoon with barb up"}},
      "key": "2968"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right harpoon with barb down above left harpoon with barb down"}},
      "key": "2969"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left harpoon with barb up above long dash"}},
      "key": "296A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left harpoon with barb down below long dash"}},
      "key": "296B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right harpoon with barb up above long dash"}},
      "key": "296C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right harpoon with barb down below long dash"}},
      "key": "296D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up harpoon with barb left beside down harpoon with barb right"}},
      "key": "296E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down harpoon with barb left beside up harpoon with barb right"}},
      "key": "296F"
    }, {"category": "Sm", "mappings": {"default": {"default": "left fish tail"}}, "key": "297C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "right fish tail"}},
      "key": "297D"
    }, {"category": "Sm", "mappings": {"default": {"default": "up fish tail"}}, "key": "297E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "down fish tail"}},
      "key": "297F"
    }]
  ,
  "en/symbols/math_non_characters.js":
    [{"locale": "en"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "italic h over two pi", "physics": "planck constant over two pi"}},
      "key": "210F"
    }, {"category": "So", "mappings": {"default": {"default": "l b bar"}}, "key": "2114"}, {
      "category": "So",
      "mappings": {"default": {"default": "numero"}},
      "key": "2116"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "sound recording copyright"}},
      "key": "2117"
    }, {"category": "So", "mappings": {"default": {"default": "prescription take"}}, "key": "211E"}, {
      "category": "So",
      "mappings": {"default": {"default": "response"}},
      "key": "211F"
    }, {"category": "So", "mappings": {"default": {"default": "service mark"}}, "key": "2120"}, {
      "category": "So",
      "mappings": {"default": {"default": "telephone sign", "alternative": "t e l symbol"}},
      "key": "2121"
    }, {"category": "So", "mappings": {"default": {"default": "trade mark"}}, "key": "2122"}, {
      "category": "So",
      "mappings": {"default": {"default": "versicle"}},
      "key": "2123"
    }, {"category": "So", "mappings": {"default": {"default": "ounce"}}, "key": "2125"}, {
      "category": "Lu",
      "mappings": {"default": {"default": "ohm"}},
      "key": "2126"
    }, {"category": "So", "mappings": {"default": {"default": "inverted ohm"}}, "key": "2127"}, {
      "category": "Lu",
      "mappings": {"default": {"default": "kelvin"}},
      "key": "212A"
    }, {"category": "Lu", "mappings": {"default": {"default": "angstrom"}}, "key": "212B"}, {
      "category": "So",
      "mappings": {"default": {"default": "estimated"}},
      "key": "212E"
    }, {
      "category": "Lu",
      "mappings": {"default": {"default": "turned cap F"}, "mathspeak": {"default": "turned upper F"}},
      "key": "2132"
    }, {"category": "Ll", "mappings": {"default": {"default": "information source"}}, "key": "2139"}, {
      "category": "So",
      "mappings": {"default": {"default": "rotated cap Q"}, "mathspeak": {"default": "rotated upper Q"}},
      "key": "213A"
    }, {"category": "So", "mappings": {"default": {"default": "facsimile sign"}}, "key": "213B"}, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "turned sans serif cap G"},
        "mathspeak": {"default": "turned sans serif upper G"}
      },
      "key": "2141"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "turned sans serif cap L"},
        "mathspeak": {"default": "turned sans serif upper L"}
      },
      "key": "2142"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "reversed sans serif cap L"},
        "mathspeak": {"default": "reversed sans serif upper L"}
      },
      "key": "2143"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "turned sans serif cap Y"},
        "mathspeak": {"default": "turned sans serif upper Y"}
      },
      "key": "2144"
    }]
  ,
  "en/symbols/math_symbols.js":
    [{"locale": "en"}, {
      "category": "Po",
      "mappings": {"default": {"default": "exclamation mark"}},
      "key": "0021"
    }, {"category": "Po", "mappings": {"default": {"default": "quotation mark"}}, "key": "0022"}, {
      "category": "Po",
      "mappings": {
        "default": {"default": "number sign", "alternative": "hash"},
        "mathspeak": {"brief": "num sign", "sbrief": "num sign"}
      },
      "key": "0023"
    }, {"category": "Sc", "mappings": {"default": {"default": "dollar sign"}}, "key": "0024"}, {
      "category": "Po",
      "mappings": {"default": {"default": "percent sign"}},
      "key": "0025"
    }, {"category": "Po", "mappings": {"default": {"default": "ampersand"}}, "key": "0026"}, {
      "category": "Po",
      "mappings": {"default": {"default": "prime"}},
      "key": "0027"
    }, {"category": "Po", "mappings": {"default": {"default": "asterisk"}}, "key": "002A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "plus"}},
      "key": "002B"
    }, {"category": "Po", "mappings": {"default": {"default": "comma"}}, "key": "002C"}, {
      "category": "Pd",
      "mappings": {"default": {"default": "minus"}, "mathspeak": {"default": "hyphen"}},
      "key": "002D"
    }, {"category": "Po", "mappings": {"default": {"default": "period"}}, "key": "002E"}, {
      "category": "Po",
      "mappings": {"default": {"default": "slash", "alternative": "solidus"}, "emacspeak": {"default": "slash"}},
      "key": "002F"
    }, {"category": "Po", "mappings": {"default": {"default": "colon"}}, "key": "003A"}, {
      "category": "Po",
      "mappings": {"default": {"default": "semicolon"}},
      "key": "003B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than"}, "clearspeak": {"default": "is less than"}},
      "key": "003C"
    }, {"category": "Sm", "mappings": {"default": {"default": "equals"}}, "key": "003D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than"}, "clearspeak": {"default": "is greater than"}},
      "key": "003E"
    }, {"category": "Po", "mappings": {"default": {"default": "question mark"}}, "key": "003F"}, {
      "category": "Po",
      "mappings": {"default": {"default": "at"}},
      "key": "0040"
    }, {"category": "Po", "mappings": {"default": {"default": "backslash"}}, "key": "005C"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "hat"}, "mathspeak": {"default": "caret"}},
      "key": "005E"
    }, {
      "category": "Pc",
      "mappings": {"default": {"default": "bar", "alternative": "underline"}},
      "key": "005F"
    }, {
      "category": "Sk",
      "mappings": {"default": {"default": "grave"}, "mathspeak": {"default": "grave"}},
      "key": "0060"
    }, {"category": "Sm", "mappings": {"default": {"default": "vertical bar"}}, "key": "007C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "tilde"}},
      "key": "007E"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "inverted exclamation mark"}},
      "key": "00A1"
    }, {"category": "Sc", "mappings": {"default": {"default": "cent sign"}}, "key": "00A2"}, {
      "category": "Sc",
      "mappings": {"default": {"default": "pound sign"}},
      "key": "00A3"
    }, {"category": "Sc", "mappings": {"default": {"default": "currency sign"}}, "key": "00A4"}, {
      "category": "Sc",
      "mappings": {"default": {"default": "yen sign"}},
      "key": "00A5"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "broken vertical bar"}},
      "key": "00A6"
    }, {"category": "Po", "mappings": {"default": {"default": "section sign"}}, "key": "00A7"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "two dots"}},
      "key": "00A8"
    }, {"category": "So", "mappings": {"default": {"default": "copyright sign"}}, "key": "00A9"}, {
      "category": "Lo",
      "mappings": {"default": {"default": "feminine ordinal indicator"}},
      "key": "00AA"
    }, {
      "category": "Pi",
      "mappings": {"default": {"default": "left pointing guillemet"}},
      "key": "00AB"
    }, {"category": "Sm", "mappings": {"default": {"default": "not sign"}}, "key": "00AC"}, {
      "category": "So",
      "mappings": {
        "default": {"default": "registered sign"},
        "mathspeak": {"default": "registered trade mark sign"},
        "clearspeak": {"default": "trade mark sign"}
      },
      "key": "00AE"
    }, {"category": "Sk", "mappings": {"default": {"default": "bar"}}, "key": "00AF"}, {
      "category": "So",
      "mappings": {"default": {"default": "degree"}, "clearspeak": {"default": "degrees"}},
      "key": "00B0"
    }, {"category": "Sm", "mappings": {"default": {"default": "plus or minus"}}, "key": "00B1"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "acute"}},
      "key": "00B4"
    }, {"category": "Ll", "mappings": {"default": {"default": "micro sign"}}, "key": "00B5"}, {
      "category": "Po",
      "mappings": {"default": {"default": "paragraph sign"}},
      "key": "00B6"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "dot"}, "clearspeak": {"default": "times"}},
      "key": "00B7"
    }, {"category": "Sk", "mappings": {"default": {"default": "cedilla"}}, "key": "00B8"}, {
      "category": "Lo",
      "mappings": {"default": {"default": "masculine ordinal indicator"}},
      "key": "00BA"
    }, {
      "category": "Pf",
      "mappings": {"default": {"default": "right pointing guillemet"}},
      "key": "00BB"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "inverted question mark"}},
      "key": "00BF"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "times"},
        "clearspeak": {"MultsymbolX_By": "by", "MultsymbolX_Cross": "cross"}
      },
      "key": "00D7"
    }, {"category": "Sm", "mappings": {"default": {"default": "division sign"}}, "key": "00F7"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "breve"}},
      "key": "02D8"
    }, {
      "category": "Sk",
      "mappings": {"default": {"default": "dot above"}, "mathspeak": {"default": "dot"}},
      "key": "02D9"
    }, {"category": "Sk", "mappings": {"default": {"default": "ring above"}}, "key": "02DA"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "ogonek"}},
      "key": "02DB"
    }, {"category": "Sk", "mappings": {"default": {"default": "tilde"}}, "key": "02DC"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "double acute"}},
      "key": "02DD"
    }, {"category": "Pd", "mappings": {"default": {"default": "hyphen"}}, "key": "2010"}, {
      "category": "Pd",
      "mappings": {"default": {"default": "non breaking hyphen"}},
      "key": "2011"
    }, {"category": "Pd", "mappings": {"default": {"default": "figure dash"}}, "key": "2012"}, {
      "category": "Pd",
      "mappings": {"default": {"default": "en dash"}},
      "key": "2013"
    }, {"category": "Pd", "mappings": {"default": {"default": "em dash"}}, "key": "2014"}, {
      "category": "Pd",
      "mappings": {"default": {"default": "horizontal bar"}, "mathspeak": {"default": "quotation dash"}},
      "key": "2015"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "double vertical bar"}},
      "key": "2016"
    }, {"category": "Po", "mappings": {"default": {"default": "double underline"}}, "key": "2017"}, {
      "category": "Pi",
      "mappings": {"default": {"default": "left single quotation mark"}},
      "key": "2018"
    }, {
      "category": "Pf",
      "mappings": {"default": {"default": "right single quotation mark"}},
      "key": "2019"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "low right single quotation mark"}},
      "key": "201A"
    }, {
      "category": "Pi",
      "mappings": {"default": {"default": "left reversed single quotation mark"}},
      "key": "201B"
    }, {
      "category": "Pi",
      "mappings": {"default": {"default": "left double quotation mark"}},
      "key": "201C"
    }, {
      "category": "Pf",
      "mappings": {"default": {"default": "right double quotation mark"}},
      "key": "201D"
    }, {
      "category": "Ps",
      "mappings": {"default": {"default": "low right double quotation mark"}},
      "key": "201E"
    }, {
      "category": "Pi",
      "mappings": {"default": {"default": "left reversed double quotation mark"}},
      "key": "201F"
    }, {"category": "Po", "mappings": {"default": {"default": "dagger"}}, "key": "2020"}, {
      "category": "Po",
      "mappings": {"default": {"default": "double dagger"}},
      "key": "2021"
    }, {"category": "Po", "mappings": {"default": {"default": "bullet"}}, "key": "2022"}, {
      "category": "Po",
      "mappings": {"default": {"default": "triangular bullet"}},
      "key": "2023"
    }, {"category": "Po", "mappings": {"default": {"default": "one dot leader"}}, "key": "2024"}, {
      "category": "Po",
      "mappings": {"default": {"default": "two dot leader"}},
      "key": "2025"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "ellipsis"}, "clearspeak": {"default": "dot dot dot"}},
      "key": "2026"
    }, {"category": "Po", "mappings": {"default": {"default": "hyphenation point"}}, "key": "2027"}, {
      "category": "Po",
      "mappings": {"default": {"default": "per mille"}},
      "key": "2030"
    }, {"category": "Po", "mappings": {"default": {"default": "per ten thousand"}}, "key": "2031"}, {
      "category": "Po",
      "mappings": {"default": {"default": "prime"}},
      "key": "2032"
    }, {"category": "Po", "mappings": {"default": {"default": "double prime"}}, "key": "2033"}, {
      "category": "Po",
      "mappings": {"default": {"default": "triple prime"}},
      "key": "2034"
    }, {"category": "Po", "mappings": {"default": {"default": "reversed prime"}}, "key": "2035"}, {
      "category": "Po",
      "mappings": {"default": {"default": "reversed double prime"}},
      "key": "2036"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "reversed triple prime"}},
      "key": "2037"
    }, {"category": "Po", "mappings": {"default": {"default": "caret"}}, "key": "2038"}, {
      "category": "Pi",
      "mappings": {"default": {"default": "left pointing single guillemet"}},
      "key": "2039"
    }, {
      "category": "Pf",
      "mappings": {"default": {"default": "right pointing single guillemet"}},
      "key": "203A"
    }, {"category": "Po", "mappings": {"default": {"default": "reference mark"}}, "key": "203B"}, {
      "category": "Po",
      "mappings": {"default": {"default": "double exclamation mark"}},
      "key": "203C"
    }, {"category": "Po", "mappings": {"default": {"default": "interrobang"}}, "key": "203D"}, {
      "category": "Po",
      "mappings": {"default": {"default": "overline"}, "mathspeak": {"default": "bar"}},
      "key": "203E"
    }, {"category": "Pc", "mappings": {"default": {"default": "undertie"}}, "key": "203F"}, {
      "category": "Pc",
      "mappings": {"default": {"default": "character tie"}},
      "key": "2040"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "caret insertion point"}},
      "key": "2041"
    }, {"category": "Po", "mappings": {"default": {"default": "asterism"}}, "key": "2042"}, {
      "category": "Po",
      "mappings": {"default": {"default": "hyphen bullet"}},
      "key": "2043"
    }, {"category": "Sm", "mappings": {"default": {"default": "fraction slash"}}, "key": "2044"}, {
      "category": "Po",
      "mappings": {"default": {"default": "double question mark"}},
      "key": "2047"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "question exclamation mark"}},
      "key": "2048"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "exclamation question mark"}},
      "key": "2049"
    }, {"category": "Po", "mappings": {"default": {"default": "reversed pilcrow"}}, "key": "204B"}, {
      "category": "Po",
      "mappings": {"default": {"default": "black leftwards bullet"}},
      "key": "204C"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "black rightwards bullet"}},
      "key": "204D"
    }, {"category": "Po", "mappings": {"default": {"default": "low asterisk"}}, "key": "204E"}, {
      "category": "Po",
      "mappings": {"default": {"default": "reversed semicolon"}},
      "key": "204F"
    }, {"category": "Po", "mappings": {"default": {"default": "close up"}}, "key": "2050"}, {
      "category": "Po",
      "mappings": {"default": {"default": "two asterisks aligned vertically"}},
      "key": "2051"
    }, {"category": "Sm", "mappings": {"default": {"default": "commercial minus"}}, "key": "2052"}, {
      "category": "Po",
      "mappings": {"default": {"default": "swung dash"}},
      "key": "2053"
    }, {"category": "Pc", "mappings": {"default": {"default": "inverted undertie"}}, "key": "2054"}, {
      "category": "Po",
      "mappings": {"default": {"default": "flower punctuation mark"}},
      "key": "2055"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "three dot punctuation"}},
      "key": "2056"
    }, {"category": "Po", "mappings": {"default": {"default": "quadruple prime"}}, "key": "2057"}, {
      "category": "Po",
      "mappings": {"default": {"default": "four dot punctuation"}},
      "key": "2058"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "five dot punctuation"}},
      "key": "2059"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "two dot punctuation"}},
      "key": "205A"
    }, {"category": "Po", "mappings": {"default": {"default": "four dot mark"}}, "key": "205B"}, {
      "category": "Po",
      "mappings": {"default": {"default": "dotted cross"}},
      "key": "205C"
    }, {"category": "Po", "mappings": {"default": {"default": "tricolon"}}, "key": "205D"}, {
      "category": "Po",
      "mappings": {"default": {"default": "vertical four dots"}},
      "key": "205E"
    }, {"category": "Sm", "mappings": {"default": {"default": "superscript plus"}}, "key": "207A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "superscript minus"}},
      "key": "207B"
    }, {"category": "Sm", "mappings": {"default": {"default": "superscript equals"}}, "key": "207C"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "superscript left parenthesis"}},
      "key": "207D"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "superscript right parenthesis"}},
      "key": "207E"
    }, {"category": "Sm", "mappings": {"default": {"default": "subscript plus"}}, "key": "208A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "subscript minus"}},
      "key": "208B"
    }, {"category": "Sm", "mappings": {"default": {"default": "subscript equals"}}, "key": "208C"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "subscript left parenthesis"}},
      "key": "208D"
    }, {
      "category": "Pe",
      "mappings": {"default": {"default": "subscript right parenthesis"}},
      "key": "208E"
    }, {"category": "So", "mappings": {"default": {"default": "property line"}}, "key": "214A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "turned ampersand"}},
      "key": "214B"
    }, {"category": "So", "mappings": {"default": {"default": "per"}}, "key": "214C"}, {
      "category": "So",
      "mappings": {"default": {"default": "aktieselskab"}},
      "key": "214D"
    }, {"category": "Ll", "mappings": {"default": {"default": "turned small f"}}, "key": "214E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "for all"}},
      "key": "2200"
    }, {"category": "Sm", "mappings": {"default": {"default": "complement"}}, "key": "2201"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "there exists"}},
      "key": "2203"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "there does not exist"}},
      "key": "2204"
    }, {"category": "Sm", "mappings": {"default": {"default": "empty set"}}, "key": "2205"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "increment"}},
      "key": "2206"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "element of"},
        "clearspeak": {
          "default": "is a member of",
          "SetMemberSymbol_Member": "is a member of",
          "SetMemberSymbol_Element": "is an element of",
          "SetMemberSymbol_In": "is in",
          "SetMemberSymbol_Belongs": "belongs to"
        }
      },
      "key": "2208"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "not an element of"},
        "clearspeak": {
          "default": "is not a member of",
          "SetMemberSymbol_Member": "is not a member of",
          "SetMemberSymbol_Element": "is not an element of",
          "SetMemberSymbol_In": "is not in",
          "SetMemberSymbol_Belongs": "does not belong to"
        }
      },
      "key": "2209"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "small element of"},
        "clearspeak": {
          "default": "is a member of",
          "SetMemberSymbol_Member": "is a member of",
          "SetMemberSymbol_Element": "is an element of",
          "SetMemberSymbol_In": "is in",
          "SetMemberSymbol_Belongs": "belongs to"
        }
      },
      "key": "220A"
    }, {"category": "Sm", "mappings": {"default": {"default": "contains as member"}}, "key": "220B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "does not contain as member"}},
      "key": "220C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "small contains as member"}},
      "key": "220D"
    }, {"category": "Sm", "mappings": {"default": {"default": "end of proof"}}, "key": "220E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "product"}},
      "key": "220F"
    }, {"category": "Sm", "mappings": {"default": {"default": "coproduct"}}, "key": "2210"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "sum"}, "mathspeak": {"default": "sigma summation"}},
      "key": "2211"
    }, {"category": "Sm", "mappings": {"default": {"default": "minus"}}, "key": "2212"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "minus or plus"}},
      "key": "2213"
    }, {"category": "Sm", "mappings": {"default": {"default": "dot plus"}}, "key": "2214"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "division slash"}},
      "key": "2215"
    }, {"category": "Sm", "mappings": {"default": {"default": "set minus"}}, "key": "2216"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "asterisk"}},
      "key": "2217"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "ring"}, "clearspeak": {"default": "composed with"}},
      "key": "2218"
    }, {"category": "Sm", "mappings": {"default": {"default": "bullet"}}, "key": "2219"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "square root"}},
      "key": "221A"
    }, {"category": "Sm", "mappings": {"default": {"default": "cube root"}}, "key": "221B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "fourth root"}},
      "key": "221C"
    }, {"category": "Sm", "mappings": {"default": {"default": "proportional to"}}, "key": "221D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "infinity"}},
      "key": "221E"
    }, {"category": "Sm", "mappings": {"default": {"default": "right angle"}}, "key": "221F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "angle"}},
      "key": "2220"
    }, {"category": "Sm", "mappings": {"default": {"default": "measured angle"}}, "key": "2221"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "spherical angle"}},
      "key": "2222"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "vertical bar", "alternative": "divides"}},
      "key": "2223"
    }, {"category": "Sm", "mappings": {"default": {"default": "does not divide"}}, "key": "2224"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "parallel to"}},
      "key": "2225"
    }, {"category": "Sm", "mappings": {"default": {"default": "not parallel to"}}, "key": "2226"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "and"}},
      "key": "2227"
    }, {"category": "Sm", "mappings": {"default": {"default": "or"}}, "key": "2228"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "intersection"}},
      "key": "2229"
    }, {"category": "Sm", "mappings": {"default": {"default": "union"}}, "key": "222A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "integral"}},
      "key": "222B"
    }, {"category": "Sm", "mappings": {"default": {"default": "double integral"}}, "key": "222C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "triple integral"}},
      "key": "222D"
    }, {"category": "Sm", "mappings": {"default": {"default": "contour integral"}}, "key": "222E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "surface integral"}},
      "key": "222F"
    }, {"category": "Sm", "mappings": {"default": {"default": "volume integral"}}, "key": "2230"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "clockwise integral"}},
      "key": "2231"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "clockwise contour integral"}},
      "key": "2232"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "anticlockwise contour integral"}},
      "key": "2233"
    }, {"category": "Sm", "mappings": {"default": {"default": "therefore"}}, "key": "2234"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "because"}},
      "key": "2235"
    }, {"category": "Sm", "mappings": {"default": {"default": "ratio"}}, "key": "2236"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "proportion"}},
      "key": "2237"
    }, {"category": "Sm", "mappings": {"default": {"default": "dot minus"}}, "key": "2238"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "excess"}},
      "key": "2239"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "geometric proportion"}},
      "key": "223A"
    }, {"category": "Sm", "mappings": {"default": {"default": "homothetic"}}, "key": "223B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "tilde"}},
      "key": "223C"
    }, {"category": "Sm", "mappings": {"default": {"default": "reversed tilde"}}, "key": "223D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "inverted lazy s"}},
      "key": "223E"
    }, {"category": "Sm", "mappings": {"default": {"default": "sine wave"}}, "key": "223F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "wreath product"}},
      "key": "2240"
    }, {"category": "Sm", "mappings": {"default": {"default": "not tilde"}}, "key": "2241"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "minus tilde"}},
      "key": "2242"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "asymptotically equals"}},
      "key": "2243"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "not asymptotically equals"}},
      "key": "2244"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "approximately equals"}},
      "key": "2245"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "approximately but not actually equals"}},
      "key": "2246"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "neither approximately nor actually equals"}},
      "key": "2247"
    }, {"category": "Sm", "mappings": {"default": {"default": "almost equals"}}, "key": "2248"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "not almost equals"}},
      "key": "2249"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "almost equal or equals"}},
      "key": "224A"
    }, {"category": "Sm", "mappings": {"default": {"default": "triple tilde"}}, "key": "224B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "all equals"}},
      "key": "224C"
    }, {"category": "Sm", "mappings": {"default": {"default": "equivalent to"}}, "key": "224D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "geometrically equivalent to"}},
      "key": "224E"
    }, {"category": "Sm", "mappings": {"default": {"default": "difference between"}}, "key": "224F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "approaches the limit"}},
      "key": "2250"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "geometrically equals"}},
      "key": "2251"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "approximately equals or the image of"}},
      "key": "2252"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "image of or approximately equals"}},
      "key": "2253"
    }, {"category": "Sm", "mappings": {"default": {"default": "colon equals"}}, "key": "2254"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "equals colon"}},
      "key": "2255"
    }, {"category": "Sm", "mappings": {"default": {"default": "ring in equals"}}, "key": "2256"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "ring equals"}},
      "key": "2257"
    }, {"category": "Sm", "mappings": {"default": {"default": "corresponds to"}}, "key": "2258"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "estimates"}},
      "key": "2259"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "equiangular to"}, "clearspeak": {"default": "is equiangular to"}},
      "key": "225A"
    }, {"category": "Sm", "mappings": {"default": {"default": "star equals"}}, "key": "225B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "delta equals"}},
      "key": "225C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "equals by definition"}, "clearspeak": {"default": "is defined to be"}},
      "key": "225D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "measured by"}, "clearspeak": {"default": "is measured by"}},
      "key": "225E"
    }, {"category": "Sm", "mappings": {"default": {"default": "questioned equals"}}, "key": "225F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "not equals"}, "clearspeak": {"default": "is not equal to"}},
      "key": "2260"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "identical to"}, "clearspeak": {"default": "is identical to"}},
      "key": "2261"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "not identical to"}, "clearspeak": {"default": "is not identical to"}},
      "key": "2262"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "strictly equivalent to"},
        "clearspeak": {"default": "is strictly equivalent to"}
      },
      "key": "2263"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "less than or equals"},
        "clearspeak": {"default": "is less than or equal to"}
      },
      "key": "2264"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "greater than or equals"},
        "clearspeak": {"default": "is greater than or equal to"}
      },
      "key": "2265"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "less than over equals"},
        "clearspeak": {"default": "is less than over equals"}
      },
      "key": "2266"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "greater than over equals"},
        "clearspeak": {"default": "is greater than over equals"}
      },
      "key": "2267"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "less than but not equals"},
        "clearspeak": {"default": "is less than but not equal to"}
      },
      "key": "2268"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "greater than but not equals"},
        "clearspeak": {"default": "is greater than but not equal to"}
      },
      "key": "2269"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "much less than"}, "clearspeak": {"default": "is much less than"}},
      "key": "226A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "much greater than"}, "clearspeak": {"default": "is much greater than"}},
      "key": "226B"
    }, {"category": "Sm", "mappings": {"default": {"default": "between"}}, "key": "226C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "not equivalent to"}, "clearspeak": {"default": "is not equivalent to"}},
      "key": "226D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "not less than"}, "clearspeak": {"default": "is not less than"}},
      "key": "226E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "not greater than"}, "clearspeak": {"default": "is not greater than"}},
      "key": "226F"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "neither less than nor equals"},
        "clearspeak": {"default": "is neither less than nor equal to"}
      },
      "key": "2270"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "neither greater than nor equals"},
        "clearspeak": {"default": "is neither greater than nor equal to"}
      },
      "key": "2271"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "less than or equivalent to"},
        "clearspeak": {"default": "is less than or equivalent to"}
      },
      "key": "2272"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "greater than or equivalent to"},
        "clearspeak": {"default": "is greater than or equivalent to"}
      },
      "key": "2273"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "neither less than nor equivalent to"},
        "clearspeak": {"default": "is neither less than nor equivalent to"}
      },
      "key": "2274"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "neither greater than nor equivalent to"},
        "clearspeak": {"default": "is neither greater than nor equivalent to"}
      },
      "key": "2275"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "less than or greater than"},
        "clearspeak": {"default": "is less than or greater than"}
      },
      "key": "2276"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "greater than or less than"},
        "clearspeak": {"default": "is greater than or less than"}
      },
      "key": "2277"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "neither less than nor greater than"},
        "clearspeak": {"default": "is neither less than nor greater than"}
      },
      "key": "2278"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "neither greater than nor less than"},
        "clearspeak": {"default": "is neither greater than nor less than"}
      },
      "key": "2279"
    }, {"category": "Sm", "mappings": {"default": {"default": "precedes"}}, "key": "227A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "succeeds"}},
      "key": "227B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "precedes or equal to"}},
      "key": "227C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "succeeds or equal to"}},
      "key": "227D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "precedes or equivalent to"}},
      "key": "227E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "succeeds or equivalent to"}},
      "key": "227F"
    }, {"category": "Sm", "mappings": {"default": {"default": "does not precede"}}, "key": "2280"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "does not succeed"}},
      "key": "2281"
    }, {"category": "Sm", "mappings": {"default": {"default": "subset of"}}, "key": "2282"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset of"}},
      "key": "2283"
    }, {"category": "Sm", "mappings": {"default": {"default": "not a subset of"}}, "key": "2284"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "not a superset of"}},
      "key": "2285"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "subset of or equal to"}},
      "key": "2286"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset of or equal to"}},
      "key": "2287"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "neither a subset of nor equal to"}},
      "key": "2288"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "neither a superset of nor equal to"}},
      "key": "2289"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "subset of or not equals"}},
      "key": "228A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset of or not equals"}},
      "key": "228B"
    }, {"category": "Sm", "mappings": {"default": {"default": "multiset"}}, "key": "228C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "multiset multiplication"}},
      "key": "228D"
    }, {"category": "Sm", "mappings": {"default": {"default": "multiset union"}}, "key": "228E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "square image of"}},
      "key": "228F"
    }, {"category": "Sm", "mappings": {"default": {"default": "square original of"}}, "key": "2290"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "square image of or equal to"}},
      "key": "2291"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "square original of or equal to"}},
      "key": "2292"
    }, {"category": "Sm", "mappings": {"default": {"default": "square cap"}}, "key": "2293"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "square cup"}},
      "key": "2294"
    }, {"category": "Sm", "mappings": {"default": {"default": "circled plus"}}, "key": "2295"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "circled minus"}},
      "key": "2296"
    }, {"category": "Sm", "mappings": {"default": {"default": "circled times"}}, "key": "2297"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "circled division slash"}},
      "key": "2298"
    }, {"category": "Sm", "mappings": {"default": {"default": "circled dot"}}, "key": "2299"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "circled ring"}},
      "key": "229A"
    }, {"category": "Sm", "mappings": {"default": {"default": "circled asterisk"}}, "key": "229B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "circled equals"}},
      "key": "229C"
    }, {"category": "Sm", "mappings": {"default": {"default": "circled dash"}}, "key": "229D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "squared plus"}},
      "key": "229E"
    }, {"category": "Sm", "mappings": {"default": {"default": "squared minus"}}, "key": "229F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "squared times"}},
      "key": "22A0"
    }, {"category": "Sm", "mappings": {"default": {"default": "squared dot"}}, "key": "22A1"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "right tack"}},
      "key": "22A2"
    }, {"category": "Sm", "mappings": {"default": {"default": "left tack"}}, "key": "22A3"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "down tack"}},
      "key": "22A4"
    }, {"category": "Sm", "mappings": {"default": {"default": "up tack"}}, "key": "22A5"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "assertion"}},
      "key": "22A6"
    }, {"category": "Sm", "mappings": {"default": {"default": "models"}}, "key": "22A7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "true"}},
      "key": "22A8"
    }, {"category": "Sm", "mappings": {"default": {"default": "forces"}}, "key": "22A9"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "triple vertical bar right turnstile"}},
      "key": "22AA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double vertical bar double right turnstile"}},
      "key": "22AB"
    }, {"category": "Sm", "mappings": {"default": {"default": "does not prove"}}, "key": "22AC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "not true"}},
      "key": "22AD"
    }, {"category": "Sm", "mappings": {"default": {"default": "does not force"}}, "key": "22AE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "negated double vertical bar double right turnstile"}},
      "key": "22AF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "precedes under relation"}},
      "key": "22B0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "succeeds under relation"}},
      "key": "22B1"
    }, {"category": "Sm", "mappings": {"default": {"default": "normal subgroup of"}}, "key": "22B2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "contains as normal subgroup"}},
      "key": "22B3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "normal subgroup of or equal to"}},
      "key": "22B4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "contains as normal subgroup or equal to"}},
      "key": "22B5"
    }, {"category": "Sm", "mappings": {"default": {"default": "original of"}}, "key": "22B6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "image of"}},
      "key": "22B7"
    }, {"category": "Sm", "mappings": {"default": {"default": "multimap"}}, "key": "22B8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "hermitian conjugate matrix"}},
      "key": "22B9"
    }, {"category": "Sm", "mappings": {"default": {"default": "intercalate"}}, "key": "22BA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "xor"}},
      "key": "22BB"
    }, {"category": "Sm", "mappings": {"default": {"default": "nand"}}, "key": "22BC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "nor"}},
      "key": "22BD"
    }, {"category": "Sm", "mappings": {"default": {"default": "right triangle"}}, "key": "22BF"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "n ary and"}, "mathspeak": {"default": "and"}},
      "key": "22C0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "n ary or"}, "mathspeak": {"default": "or"}},
      "key": "22C1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "n ary intersection"}, "mathspeak": {"default": "intersection"}},
      "key": "22C2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "n ary union"}, "mathspeak": {"default": "union"}},
      "key": "22C3"
    }, {"category": "Sm", "mappings": {"default": {"default": "diamond"}}, "key": "22C4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "dot"}, "clearspeak": {"default": "times", "MultsymbolDot_Dot": "dot"}},
      "key": "22C5"
    }, {"category": "Sm", "mappings": {"default": {"default": "star"}}, "key": "22C6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "division times"}},
      "key": "22C7"
    }, {"category": "Sm", "mappings": {"default": {"default": "bowtie"}}, "key": "22C8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "left normal factor semidirect product"}},
      "key": "22C9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right normal factor semidirect product"}},
      "key": "22CA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left semidirect product"}},
      "key": "22CB"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right semidirect product"}},
      "key": "22CC"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "reversed tilde equals"}},
      "key": "22CD"
    }, {"category": "Sm", "mappings": {"default": {"default": "curly or"}}, "key": "22CE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "curly and"}},
      "key": "22CF"
    }, {"category": "Sm", "mappings": {"default": {"default": "double subset"}}, "key": "22D0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "double superset"}},
      "key": "22D1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double intersection"}},
      "key": "22D2"
    }, {"category": "Sm", "mappings": {"default": {"default": "double union"}}, "key": "22D3"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "pitchfork"}},
      "key": "22D4"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "equal and parallel to"},
        "clearspeak": {"default": "is equal and parallel to"}
      },
      "key": "22D5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than dot"}, "clearspeak": {"default": "is less than dot"}},
      "key": "22D6"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than dot"}, "clearspeak": {"default": "is greater than dot"}},
      "key": "22D7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "very much less than"}, "clearspeak": {"default": "is very much less than"}},
      "key": "22D8"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "very much greater than"},
        "clearspeak": {"default": "is very much greater than"}
      },
      "key": "22D9"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "less than equals or greater than"},
        "clearspeak": {"default": "is less than equal to or greater than"}
      },
      "key": "22DA"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "greater than equals or less than"},
        "clearspeak": {"default": "is greater than equal to or less than"}
      },
      "key": "22DB"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "equals or less than"},
        "clearspeak": {"default": "is equal to or less than"}
      },
      "key": "22DC"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "equals or greater than"},
        "clearspeak": {"default": "is equal to or greater than"}
      },
      "key": "22DD"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "equals or precedes"}, "clearspeak": {"default": "is equal to or precedes"}},
      "key": "22DE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "equals or succeeds"}, "clearspeak": {"default": "is equal to or succeeds"}},
      "key": "22DF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "does not precede or equal"}},
      "key": "22E0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "does not succeed or equal"}},
      "key": "22E1"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "not square image of or equals"},
        "clearspeak": {"default": "is not square image of or equal to"}
      },
      "key": "22E2"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "not square original of or equals"},
        "clearspeak": {"default": "is not square original of or equal to"}
      },
      "key": "22E3"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "square image of or not equals"},
        "clearspeak": {"default": "is square image of or not equal to"}
      },
      "key": "22E4"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "square original of or not equals"},
        "clearspeak": {"default": "is square original of or not equal to"}
      },
      "key": "22E5"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "less than but not equivalent to"},
        "clearspeak": {"default": "is less than but not equivalent to"}
      },
      "key": "22E6"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "greater than but not equivalent to"},
        "clearspeak": {"default": "is greater than but not equivalent to"}
      },
      "key": "22E7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "precedes but not equivalent to"}},
      "key": "22E8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "succeeds but not equivalent to"}},
      "key": "22E9"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "not normal subgroup of"},
        "clearspeak": {"default": "is not normal subgroup of"}
      },
      "key": "22EA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "does not contain as normal subgroup"}},
      "key": "22EB"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "not normal subgroup of or equals"},
        "clearspeak": {"default": "is not normal subgroup of or equal to"}
      },
      "key": "22EC"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "does not contain as normal subgroup or equal"}},
      "key": "22ED"
    }, {"category": "Sm", "mappings": {"default": {"default": "vertical ellipsis"}}, "key": "22EE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "midline horizontal ellipsis"}, "clearspeak": {"default": "dot dot dot"}},
      "key": "22EF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "up right diagonal ellipsis"}},
      "key": "22F0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down right diagonal ellipsis"}},
      "key": "22F1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "element of with long horizontal stroke"}},
      "key": "22F2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "element of with vertical bar at end of horizontal stroke"}},
      "key": "22F3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "small element of with vertical bar at end of horizontal stroke"}},
      "key": "22F4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "element of with dot above"}},
      "key": "22F5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "element of with overbar"}},
      "key": "22F6"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "small element of with overbar"}},
      "key": "22F7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "element of with underbar"}},
      "key": "22F8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "element of with two horizontal strokes"}},
      "key": "22F9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "contains with long horizontal stroke"}},
      "key": "22FA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "contains with vertical bar at end of horizontal stroke"}},
      "key": "22FB"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "small contains with vertical bar at end of horizontal stroke"}},
      "key": "22FC"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "contains with overbar"}},
      "key": "22FD"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "small contains with overbar"}},
      "key": "22FE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "z notation bag membership"}},
      "key": "22FF"
    }, {"category": "So", "mappings": {"default": {"default": "diameter sign"}}, "key": "2300"}, {
      "category": "So",
      "mappings": {"default": {"default": "house"}},
      "key": "2302"
    }, {"category": "So", "mappings": {"default": {"default": "projective"}}, "key": "2305"}, {
      "category": "So",
      "mappings": {"default": {"default": "perspective"}},
      "key": "2306"
    }, {"category": "So", "mappings": {"default": {"default": "wavy line"}}, "key": "2307"}, {
      "category": "So",
      "mappings": {"default": {"default": "reversed not"}},
      "key": "2310"
    }, {"category": "So", "mappings": {"default": {"default": "square lozenge"}}, "key": "2311"}, {
      "category": "So",
      "mappings": {"default": {"default": "arc"}},
      "key": "2312"
    }, {"category": "So", "mappings": {"default": {"default": "segment"}}, "key": "2313"}, {
      "category": "So",
      "mappings": {"default": {"default": "sector"}},
      "key": "2314"
    }, {"category": "So", "mappings": {"default": {"default": "bold plus"}}, "key": "2795"}, {
      "category": "So",
      "mappings": {"default": {"default": "bold minus"}},
      "key": "2796"
    }, {"category": "So", "mappings": {"default": {"default": "bold division"}}, "key": "2797"}, {
      "category": "So",
      "mappings": {"default": {"default": "curly loop"}},
      "key": "27B0"
    }, {"category": "So", "mappings": {"default": {"default": "double curly loop"}}, "key": "27BF"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "white triangle containing small white triangle"}},
      "key": "27C1"
    }, {"category": "Sm", "mappings": {"default": {"default": "perpendicular"}}, "key": "27C2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "open subset"}},
      "key": "27C3"
    }, {"category": "Sm", "mappings": {"default": {"default": "open superset"}}, "key": "27C4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "or with dot inside"}},
      "key": "27C7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "backslash preceding subset"}},
      "key": "27C8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset preceding solidus"}},
      "key": "27C9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "vertical bar with horizontal stroke"}},
      "key": "27CA"
    }, {"category": "Sm", "mappings": {"default": {"default": "rising diagonal"}}, "key": "27CB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "long division"}},
      "key": "27CC"
    }, {"category": "Sm", "mappings": {"default": {"default": "falling diagonal"}}, "key": "27CD"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "squared and"}},
      "key": "27CE"
    }, {"category": "Sm", "mappings": {"default": {"default": "squared or"}}, "key": "27CF"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "white diamond with centered dot"}},
      "key": "27D0"
    }, {"category": "Sm", "mappings": {"default": {"default": "and with dot"}}, "key": "27D1"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "element of opening upwards"}},
      "key": "27D2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "lower right corner with dot"}},
      "key": "27D3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "upper left corner with dot"}},
      "key": "27D4"
    }, {"category": "Sm", "mappings": {"default": {"default": "left outer join"}}, "key": "27D5"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "right outer join"}},
      "key": "27D6"
    }, {"category": "Sm", "mappings": {"default": {"default": "full outer join"}}, "key": "27D7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "large up tack"}},
      "key": "27D8"
    }, {"category": "Sm", "mappings": {"default": {"default": "large down tack"}}, "key": "27D9"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "left and right double turnstile"}},
      "key": "27DA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left and right tack"}},
      "key": "27DB"
    }, {"category": "Sm", "mappings": {"default": {"default": "left multimap"}}, "key": "27DC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "long right tack"}},
      "key": "27DD"
    }, {"category": "Sm", "mappings": {"default": {"default": "long left tack"}}, "key": "27DE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "up tack with circle above"}},
      "key": "27DF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "lozenge divided by horizontal rule"}},
      "key": "27E0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "white concave sided diamond"}},
      "key": "27E1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "white concave sided diamond with leftwards tick"}},
      "key": "27E2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "white concave sided diamond with rightwards tick"}},
      "key": "27E3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "white square with leftwards tick"}},
      "key": "27E4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "white square with rightwards tick"}},
      "key": "27E5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "rising diagonal crossing falling diagonal"}},
      "key": "292B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "falling diagonal crossing rising diagonal"}},
      "key": "292C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "triple vertical bar delimiter"}},
      "key": "2980"
    }, {"category": "Sm", "mappings": {"default": {"default": "z notation spot"}}, "key": "2981"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "z notation type colon"}},
      "key": "2982"
    }, {"category": "Sm", "mappings": {"default": {"default": "dotted fence"}}, "key": "2999"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "vertical zigzag line"}},
      "key": "299A"
    }, {"category": "Sm", "mappings": {"default": {"default": "reversed empty set"}}, "key": "29B0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "empty set with overbar"}},
      "key": "29B1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "empty set with small circle above"}},
      "key": "29B2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "circle with horizontal bar"}},
      "key": "29B5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "circled vertical bar"}},
      "key": "29B6"
    }, {"category": "Sm", "mappings": {"default": {"default": "circled parallel"}}, "key": "29B7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "circled backslash"}},
      "key": "29B8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "circled perpendicular"}},
      "key": "29B9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "circle divided by horizontal bar and top half divided by vertical bar"}},
      "key": "29BA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "circle with superimposed x"}},
      "key": "29BB"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "circled anticlockwise rotated division"}},
      "key": "29BC"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "circled white bullet"}},
      "key": "29BE"
    }, {"category": "Sm", "mappings": {"default": {"default": "circled bullet"}}, "key": "29BF"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "circled less than"}},
      "key": "29C0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "circled greater than"}},
      "key": "29C1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "circle with small circle to the right"}},
      "key": "29C2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "circle with two horizontal strokes to the right"}},
      "key": "29C3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "squared rising diagonal slash"}},
      "key": "29C4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "squared falling diagonal slash"}},
      "key": "29C5"
    }, {"category": "Sm", "mappings": {"default": {"default": "squared asterisk"}}, "key": "29C6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "squared small circle"}},
      "key": "29C7"
    }, {"category": "Sm", "mappings": {"default": {"default": "squared square"}}, "key": "29C8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "two joined squares"}},
      "key": "29C9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "triangle with dot above"}},
      "key": "29CA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "triangle with underbar"}},
      "key": "29CB"
    }, {"category": "Sm", "mappings": {"default": {"default": "s in triangle"}}, "key": "29CC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "triangle with serifs at bottom"}},
      "key": "29CD"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "right triangle above left triangle"}},
      "key": "29CE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "left triangle beside vertical bar"}},
      "key": "29CF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "vertical bar beside right triangle"}},
      "key": "29D0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "bowtie with left half black"}},
      "key": "29D1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "bowtie with right half black"}},
      "key": "29D2"
    }, {"category": "Sm", "mappings": {"default": {"default": "black bowtie"}}, "key": "29D3"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "times with left half black"}},
      "key": "29D4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "times with right half black"}},
      "key": "29D5"
    }, {"category": "Sm", "mappings": {"default": {"default": "white hourglass"}}, "key": "29D6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "black hourglass"}},
      "key": "29D7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "incomplete infinity"}},
      "key": "29DC"
    }, {"category": "Sm", "mappings": {"default": {"default": "tie over infinity"}}, "key": "29DD"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "infinity negated with vertical bar"}},
      "key": "29DE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double ended multimap"}},
      "key": "29DF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "square with contoured outline"}},
      "key": "29E0"
    }, {"category": "Sm", "mappings": {"default": {"default": "increases as"}}, "key": "29E1"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "shuffle product"}},
      "key": "29E2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "equals and slanted parallel"}},
      "key": "29E3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "equals and slanted parallel with tilde above"}},
      "key": "29E4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "identical to and slanted parallel"}},
      "key": "29E5"
    }, {"category": "Sm", "mappings": {"default": {"default": "gleich stark"}}, "key": "29E6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "thermodynamic"}},
      "key": "29E7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down pointing triangle with left half black"}},
      "key": "29E8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down pointing triangle with right half black"}},
      "key": "29E9"
    }, {"category": "Sm", "mappings": {"default": {"default": "black lozenge"}}, "key": "29EB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "error barred white square"}},
      "key": "29EE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "error barred black square"}},
      "key": "29EF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "error barred white diamond"}},
      "key": "29F0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "error barred black diamond"}},
      "key": "29F1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "error barred white circle"}},
      "key": "29F2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "error barred black circle"}},
      "key": "29F3"
    }, {"category": "Sm", "mappings": {"default": {"default": "rule delayed"}}, "key": "29F4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "backslash"}},
      "key": "29F5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "solidus with overbar"}},
      "key": "29F6"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "backslash with horizontal stroke"}},
      "key": "29F7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "big solidus"}, "mathspeak": {"default": "solidus"}},
      "key": "29F8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "big backslash"}, "mathspeak": {"default": "backslash"}},
      "key": "29F9"
    }, {"category": "Sm", "mappings": {"default": {"default": "double plus"}}, "key": "29FA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "triple plus"}},
      "key": "29FB"
    }, {"category": "Sm", "mappings": {"default": {"default": "tiny"}}, "key": "29FE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "miny"}},
      "key": "29FF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "n ary circled dot"}, "mathspeak": {"default": "circled dot"}},
      "key": "2A00"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "n ary circled plus"}, "mathspeak": {"default": "circled plus"}},
      "key": "2A01"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "n ary circled times"}, "mathspeak": {"default": "circled times"}},
      "key": "2A02"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "n ary union with dot"}, "mathspeak": {"default": "union with dot"}},
      "key": "2A03"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "n ary union operator with plus"},
        "mathspeak": {"default": "union with plus"}
      },
      "key": "2A04"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "n ary square intersection"},
        "mathspeak": {"default": "square intersection"}
      },
      "key": "2A05"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "n ary square union"}, "mathspeak": {"default": "square union"}},
      "key": "2A06"
    }, {"category": "Sm", "mappings": {"default": {"default": "two and"}}, "key": "2A07"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "two or"}},
      "key": "2A08"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "n ary times"}, "mathspeak": {"default": "times"}},
      "key": "2A09"
    }, {"category": "Sm", "mappings": {"default": {"default": "modulo two sum"}}, "key": "2A0A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "summation with integral"}},
      "key": "2A0B"
    }, {"category": "Sm", "mappings": {"default": {"default": "quadruple integral"}}, "key": "2A0C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "finite part integral"}},
      "key": "2A0D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "integral with double stroke"}},
      "key": "2A0E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "integral average with slash"}},
      "key": "2A0F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "circulation function"}},
      "key": "2A10"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "anticlockwise integration"}},
      "key": "2A11"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "line integration with rectangular path around pole"}},
      "key": "2A12"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "line integration with semicircular path around pole"}},
      "key": "2A13"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "line integration not including the pole"}},
      "key": "2A14"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "integral around a point"}},
      "key": "2A15"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "quaternion integral"}},
      "key": "2A16"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "integral with times"}},
      "key": "2A18"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "integral with intersection"}},
      "key": "2A19"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "integral with union"}},
      "key": "2A1A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "integral with overbar"}},
      "key": "2A1B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "integral with underbar"}},
      "key": "2A1C"
    }, {"category": "Sm", "mappings": {"default": {"default": "join"}}, "key": "2A1D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "large left triangle"}},
      "key": "2A1E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "z notation schema composition"}},
      "key": "2A1F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "z notation schema piping"}},
      "key": "2A20"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "z notation schema projection"}},
      "key": "2A21"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "plus with circle above"}},
      "key": "2A22"
    }, {"category": "Sm", "mappings": {"default": {"default": "plus hat"}}, "key": "2A23"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "plus tilde"}},
      "key": "2A24"
    }, {"category": "Sm", "mappings": {"default": {"default": "plus underdot"}}, "key": "2A25"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "plus sign with tilde below"}},
      "key": "2A26"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "plus sign with subscript two"}},
      "key": "2A27"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "plus sign with black triangle"}},
      "key": "2A28"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "minus sign with comma above"}},
      "key": "2A29"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "minus sign with dot below"}},
      "key": "2A2A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "minus sign with falling dots"}},
      "key": "2A2B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "minus sign with rising dots"}},
      "key": "2A2C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "plus sign in left half circle"}},
      "key": "2A2D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "plus sign in right half circle"}},
      "key": "2A2E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "vector or cross product"}},
      "key": "2A2F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "multiplication sign with dot above"}},
      "key": "2A30"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "multiplication sign with underbar"}},
      "key": "2A31"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "semidirect product with bottom closed"}},
      "key": "2A32"
    }, {"category": "Sm", "mappings": {"default": {"default": "smash product"}}, "key": "2A33"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "multiplication sign in left half circle"}},
      "key": "2A34"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "multiplication sign in right half circle"}},
      "key": "2A35"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "circled multiplication sign with circumflex accent"}},
      "key": "2A36"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "multiplication sign in double circle"}},
      "key": "2A37"
    }, {"category": "Sm", "mappings": {"default": {"default": "circled division"}}, "key": "2A38"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "plus sign in triangle"}},
      "key": "2A39"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "minus sign in triangle"}},
      "key": "2A3A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "multiplication sign in triangle"}},
      "key": "2A3B"
    }, {"category": "Sm", "mappings": {"default": {"default": "interior product"}}, "key": "2A3C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "righthand interior product"}},
      "key": "2A3D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "z notation relational composition"}},
      "key": "2A3E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "amalgamation or coproduct"}},
      "key": "2A3F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "intersection with dot"}},
      "key": "2A40"
    }, {"category": "Sm", "mappings": {"default": {"default": "union with minus"}}, "key": "2A41"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "union with overbar"}},
      "key": "2A42"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "intersection with overbar"}},
      "key": "2A43"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "intersection with and"}},
      "key": "2A44"
    }, {"category": "Sm", "mappings": {"default": {"default": "union with or"}}, "key": "2A45"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "union above intersection"}},
      "key": "2A46"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "intersection above union"}},
      "key": "2A47"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "union above bar above intersection"}},
      "key": "2A48"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "intersection above bar above union"}},
      "key": "2A49"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "union beside and joined with union"}},
      "key": "2A4A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "intersection beside and joined with intersection"}},
      "key": "2A4B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "closed union with serifs"}},
      "key": "2A4C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "closed intersection with serifs"}},
      "key": "2A4D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double square intersection"}},
      "key": "2A4E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double square union"}},
      "key": "2A4F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "closed union with serifs and smash product"}},
      "key": "2A50"
    }, {"category": "Sm", "mappings": {"default": {"default": "and with dot above"}}, "key": "2A51"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "or with dot above"}},
      "key": "2A52"
    }, {"category": "Sm", "mappings": {"default": {"default": "double and"}}, "key": "2A53"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "double or"}},
      "key": "2A54"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "two intersecting and"}},
      "key": "2A55"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "two intersecting or"}},
      "key": "2A56"
    }, {"category": "Sm", "mappings": {"default": {"default": "sloping large or"}}, "key": "2A57"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "sloping large and"}},
      "key": "2A58"
    }, {"category": "Sm", "mappings": {"default": {"default": "or overlapping and"}}, "key": "2A59"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "and with middle stem"}},
      "key": "2A5A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "or with middle stem"}},
      "key": "2A5B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "and with horizontal dash"}},
      "key": "2A5C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "or with horizontal dash"}},
      "key": "2A5D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "and with double overbar"}},
      "key": "2A5E"
    }, {"category": "Sm", "mappings": {"default": {"default": "and with underbar"}}, "key": "2A5F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "and with double underbar"}},
      "key": "2A60"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "small vee with underbar"}},
      "key": "2A61"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "or with double overbar"}},
      "key": "2A62"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "or with double underbar"}},
      "key": "2A63"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "z notation domain antirestriction"}},
      "key": "2A64"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "z notation range antirestriction"}},
      "key": "2A65"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "equals sign with dot below"}},
      "key": "2A66"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "identical with dot above"}},
      "key": "2A67"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "triple horizontal bar with double vertical stroke"}},
      "key": "2A68"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "triple horizontal bar with triple vertical stroke"}},
      "key": "2A69"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "tilde with dot above"}},
      "key": "2A6A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "tilde with rising dots"}},
      "key": "2A6B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "similar minus similar"}},
      "key": "2A6C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "congruent with dot above"}},
      "key": "2A6D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "equals with asterisk"}},
      "key": "2A6E"
    }, {"category": "Sm", "mappings": {"default": {"default": "almost equal hat"}}, "key": "2A6F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "approximately equal or equal to"}},
      "key": "2A70"
    }, {"category": "Sm", "mappings": {"default": {"default": "equals above plus"}}, "key": "2A71"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "plus above equals"}},
      "key": "2A72"
    }, {"category": "Sm", "mappings": {"default": {"default": "equals above tilde"}}, "key": "2A73"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "double colon equal"}},
      "key": "2A74"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "two consecutive equals"}},
      "key": "2A75"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "three consecutive equals"}},
      "key": "2A76"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "equals sign with two dots above and two dots below"}},
      "key": "2A77"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "equivalent with four dots above"}},
      "key": "2A78"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than with circle inside"}},
      "key": "2A79"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than with circle inside"}},
      "key": "2A7A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than with question mark above"}},
      "key": "2A7B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than with question mark above"}},
      "key": "2A7C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than or slanted equals"}},
      "key": "2A7D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than or slanted equals"}},
      "key": "2A7E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than or slanted equals with dot inside"}},
      "key": "2A7F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than or slanted equals with dot inside"}},
      "key": "2A80"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than or slanted equals with dot above"}},
      "key": "2A81"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than or slanted equals with dot above"}},
      "key": "2A82"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than or slanted equals with dot above right"}},
      "key": "2A83"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than or slanted equals with dot above left"}},
      "key": "2A84"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than or approximate"}},
      "key": "2A85"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than or approximate"}},
      "key": "2A86"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than and single line not equals"}},
      "key": "2A87"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than and single line not equals"}},
      "key": "2A88"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than and not approximate"}},
      "key": "2A89"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than and not approximate"}},
      "key": "2A8A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than above double line equal above greater than"}},
      "key": "2A8B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than above double line equal above less than"}},
      "key": "2A8C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than above similar or equal"}},
      "key": "2A8D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than above similar or equal"}},
      "key": "2A8E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than above similar above greater than"}},
      "key": "2A8F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than above similar above less than"}},
      "key": "2A90"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than above greater than above double line equal"}},
      "key": "2A91"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than above less than above double line equal"}},
      "key": "2A92"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than above slanted equal above greater than above slanted equal"}},
      "key": "2A93"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than above slanted equal above less than above slanted equal"}},
      "key": "2A94"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "slanted equals or less than"}},
      "key": "2A95"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "slanted equals or greater than"}},
      "key": "2A96"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "slanted equals or less than with dot inside"}},
      "key": "2A97"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "slanted equals or greater than with dot inside"}},
      "key": "2A98"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double line equals or less than"}},
      "key": "2A99"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double line equals or greater than"}},
      "key": "2A9A"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double line slanted equals or less than"}},
      "key": "2A9B"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double line slanted equals or greater than"}},
      "key": "2A9C"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "similar or less than"}},
      "key": "2A9D"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "similar or greater than"}},
      "key": "2A9E"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "similar above less than above equals sign"}},
      "key": "2A9F"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "similar above greater than above equals sign"}},
      "key": "2AA0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double nested less than"}},
      "key": "2AA1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double nested greater than"}},
      "key": "2AA2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double nested less than with underbar"}},
      "key": "2AA3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than overlapping less than"}},
      "key": "2AA4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than beside less than"}},
      "key": "2AA5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than closed by curve"}},
      "key": "2AA6"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than closed by curve"}},
      "key": "2AA7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "less than closed by curve above slanted equal"}},
      "key": "2AA8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "greater than closed by curve above slanted equal"}},
      "key": "2AA9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "smaller than"}, "clearspeak": {"default": "is smaller than"}},
      "key": "2AAA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "larger than"}, "clearspeak": {"default": "is larger than"}},
      "key": "2AAB"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "smaller than or equals"},
        "clearspeak": {"default": "is smaller than or equal to"}
      },
      "key": "2AAC"
    }, {
      "category": "Sm",
      "mappings": {
        "default": {"default": "larger than or equal"},
        "clearspeak": {"default": "is larger than or equal to"}
      },
      "key": "2AAD"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "equals with bumpy above"}},
      "key": "2AAE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "precedes above single line equals sign"}},
      "key": "2AAF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "succeeds above single line equals sign"}},
      "key": "2AB0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "precedes above single line not equals"}},
      "key": "2AB1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "succeeds above single line not equals"}},
      "key": "2AB2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "precedes above equals"}},
      "key": "2AB3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "succeeds above equals"}},
      "key": "2AB4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "precedes above not equals"}},
      "key": "2AB5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "succeeds above not equals"}},
      "key": "2AB6"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "precedes above almost equals"}},
      "key": "2AB7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "succeeds above almost equals"}},
      "key": "2AB8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "precedes above not almost equals"}},
      "key": "2AB9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "succeeds above not almost equals"}},
      "key": "2ABA"
    }, {"category": "Sm", "mappings": {"default": {"default": "double precedes"}}, "key": "2ABB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "double succeeds"}},
      "key": "2ABC"
    }, {"category": "Sm", "mappings": {"default": {"default": "subset with dot"}}, "key": "2ABD"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset with dot"}},
      "key": "2ABE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "subset with plus sign below"}},
      "key": "2ABF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset with plus sign below"}},
      "key": "2AC0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "subset with multiplication sign below"}},
      "key": "2AC1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset with multiplication sign below"}},
      "key": "2AC2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "subset of or equals with dot above"}},
      "key": "2AC3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset of or equals with dot above"}},
      "key": "2AC4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "subset of above equals sign"}},
      "key": "2AC5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset of above equals sign"}},
      "key": "2AC6"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "subset of above tilde"}},
      "key": "2AC7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset of above tilde"}},
      "key": "2AC8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "subset of above almost equals"}},
      "key": "2AC9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset of above almost equals"}},
      "key": "2ACA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "subset of above not equals"}},
      "key": "2ACB"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset of above not equals"}},
      "key": "2ACC"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "square left open box"}},
      "key": "2ACD"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "square right open box"}},
      "key": "2ACE"
    }, {"category": "Sm", "mappings": {"default": {"default": "closed subset"}}, "key": "2ACF"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "closed superset"}},
      "key": "2AD0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "closed subset or equals"}},
      "key": "2AD1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "closed superset or equals"}},
      "key": "2AD2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "subset above superset"}},
      "key": "2AD3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset above subset"}},
      "key": "2AD4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "subset above subset"}},
      "key": "2AD5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset above superset"}},
      "key": "2AD6"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset beside subset"}},
      "key": "2AD7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "superset beside and joined by dash with subset"}},
      "key": "2AD8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "element of opening downwards"}},
      "key": "2AD9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "pitchfork with tee top"}},
      "key": "2ADA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "transversal intersection"}},
      "key": "2ADB"
    }, {"category": "Sm", "mappings": {"default": {"default": "forking"}}, "key": "2ADC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "nonforking"}},
      "key": "2ADD"
    }, {"category": "Sm", "mappings": {"default": {"default": "short left tack"}}, "key": "2ADE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "short down tack"}},
      "key": "2ADF"
    }, {"category": "Sm", "mappings": {"default": {"default": "short up tack"}}, "key": "2AE0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "perpendicular with s"}},
      "key": "2AE1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "vertical bar triple right turnstile"}},
      "key": "2AE2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double vertical bar left turnstile"}},
      "key": "2AE3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "vertical bar double left turnstile"}},
      "key": "2AE4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double vertical bar double left turnstile"}},
      "key": "2AE5"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "long dash from left member of double vertical"}},
      "key": "2AE6"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "short down tack with overbar"}},
      "key": "2AE7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "short up tack with underbar"}},
      "key": "2AE8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "short up tack above short down tack"}},
      "key": "2AE9"
    }, {"category": "Sm", "mappings": {"default": {"default": "double down tack"}}, "key": "2AEA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "double up tack"}},
      "key": "2AEB"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double stroke not sign"}},
      "key": "2AEC"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "reversed double stroke not sign"}},
      "key": "2AED"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "does not divide with reversed negation slash"}},
      "key": "2AEE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "vertical line with circle above"}},
      "key": "2AEF"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "vertical line with circle below"}},
      "key": "2AF0"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "down tack with circle below"}},
      "key": "2AF1"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "parallel with horizontal stroke"}},
      "key": "2AF2"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "parallel with tilde"}},
      "key": "2AF3"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "triple vertical bar binary relation"}},
      "key": "2AF4"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "triple vertical bar with horizontal stroke"}},
      "key": "2AF5"
    }, {"category": "Sm", "mappings": {"default": {"default": "triple colon"}}, "key": "2AF6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "triple nested less than"}},
      "key": "2AF7"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "triple nested greater than"}},
      "key": "2AF8"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double line slanted less than or equals"}},
      "key": "2AF9"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "double line slanted greater than or equals"}},
      "key": "2AFA"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "triple solidus binary relation"}},
      "key": "2AFB"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "large triple vertical bar"}},
      "key": "2AFC"
    }, {"category": "Sm", "mappings": {"default": {"default": "double solidus"}}, "key": "2AFD"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "white vertical bar"}},
      "key": "2AFE"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "n ary white vertical bar"}, "mathspeak": {"default": "white vertical bar"}},
      "key": "2AFF"
    }, {"category": "Pd", "mappings": {"default": {"default": "wave dash"}}, "key": "301C"}, {
      "category": "Po",
      "mappings": {"default": {"default": "presentation form for vertical comma"}},
      "key": "FE10"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "presentation form for vertical colon"}},
      "key": "FE13"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "presentation form for vertical semicolon"}},
      "key": "FE14"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "presentation form for vertical exclamation mark"}},
      "key": "FE15"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "presentation form for vertical question mark"}},
      "key": "FE16"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "presentation form for vertical horizontal ellipsis"}},
      "key": "FE19"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "glyph for vertical two dot leader"}},
      "key": "FE30"
    }, {
      "category": "Pd",
      "mappings": {"default": {"default": "glyph for vertical em dash"}},
      "key": "FE31"
    }, {
      "category": "Pd",
      "mappings": {"default": {"default": "glyph for vertical en dash"}},
      "key": "FE32"
    }, {
      "category": "Pc",
      "mappings": {"default": {"default": "glyph for vertical underline"}},
      "key": "FE33"
    }, {
      "category": "Pc",
      "mappings": {"default": {"default": "glyph for vertical wavy underline"}},
      "key": "FE34"
    }, {"category": "Po", "mappings": {"default": {"default": "sesame dot"}}, "key": "FE45"}, {
      "category": "Po",
      "mappings": {"default": {"default": "white sesame dot"}},
      "key": "FE46"
    }, {"category": "Po", "mappings": {"default": {"default": "dashed overline"}}, "key": "FE49"}, {
      "category": "Po",
      "mappings": {"default": {"default": "dash dot overline"}},
      "key": "FE4A"
    }, {"category": "Po", "mappings": {"default": {"default": "wavy overline"}}, "key": "FE4B"}, {
      "category": "Po",
      "mappings": {"default": {"default": "double wavy overline"}},
      "key": "FE4C"
    }, {"category": "Pc", "mappings": {"default": {"default": "dashed underline"}}, "key": "FE4D"}, {
      "category": "Pc",
      "mappings": {"default": {"default": "dash dot underline"}},
      "key": "FE4E"
    }, {"category": "Pc", "mappings": {"default": {"default": "wavy underline"}}, "key": "FE4F"}, {
      "category": "Po",
      "mappings": {"default": {"default": "small comma"}},
      "key": "FE50"
    }, {"category": "Po", "mappings": {"default": {"default": "small period"}}, "key": "FE52"}, {
      "category": "Po",
      "mappings": {"default": {"default": "small semicolon"}},
      "key": "FE54"
    }, {"category": "Po", "mappings": {"default": {"default": "small colon"}}, "key": "FE55"}, {
      "category": "Po",
      "mappings": {"default": {"default": "small question mark"}},
      "key": "FE56"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "small exclamation mark"}},
      "key": "FE57"
    }, {"category": "Pd", "mappings": {"default": {"default": "small em dash"}}, "key": "FE58"}, {
      "category": "Po",
      "mappings": {"default": {"default": "small number sign"}},
      "key": "FE5F"
    }, {"category": "Po", "mappings": {"default": {"default": "small ampersand"}}, "key": "FE60"}, {
      "category": "Po",
      "mappings": {"default": {"default": "small asterisk"}},
      "key": "FE61"
    }, {"category": "Sm", "mappings": {"default": {"default": "small plus sign"}}, "key": "FE62"}, {
      "category": "Pd",
      "mappings": {"default": {"default": "small hyphen minus"}},
      "key": "FE63"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "small less than sign"}},
      "key": "FE64"
    }, {
      "category": "Sm",
      "mappings": {"default": {"default": "small greater than sign"}},
      "key": "FE65"
    }, {"category": "Sm", "mappings": {"default": {"default": "small equals"}}, "key": "FE66"}, {
      "category": "Po",
      "mappings": {"default": {"default": "small backslash", "alternative": "small reverse solidus"}},
      "key": "FE68"
    }, {"category": "Sc", "mappings": {"default": {"default": "small dollar sign"}}, "key": "FE69"}, {
      "category": "Po",
      "mappings": {"default": {"default": "small percent sign"}},
      "key": "FE6A"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "small commercial at"}},
      "key": "FE6B"
    }, {"category": "Po", "mappings": {"default": {"default": "exclamation mark"}}, "key": "FF01"}, {
      "category": "Po",
      "mappings": {"default": {"default": "quotation mark"}},
      "key": "FF02"
    }, {"category": "Po", "mappings": {"default": {"default": "number sign"}}, "key": "FF03"}, {
      "category": "Sc",
      "mappings": {"default": {"default": "dollar sign"}},
      "key": "FF04"
    }, {"category": "Po", "mappings": {"default": {"default": "percent sign"}}, "key": "FF05"}, {
      "category": "Po",
      "mappings": {"default": {"default": "ampersand"}},
      "key": "FF06"
    }, {"category": "Po", "mappings": {"default": {"default": "apostrophe"}}, "key": "FF07"}, {
      "category": "Po",
      "mappings": {"default": {"default": "asterisk"}},
      "key": "FF0A"
    }, {"category": "Sm", "mappings": {"default": {"default": "plus sign"}}, "key": "FF0B"}, {
      "category": "Po",
      "mappings": {"default": {"default": "comma"}},
      "key": "FF0C"
    }, {"category": "Pd", "mappings": {"default": {"default": "hyphen minus"}}, "key": "FF0D"}, {
      "category": "Po",
      "mappings": {"default": {"default": "period"}},
      "key": "FF0E"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "slash", "alternative": "solidus"}},
      "key": "FF0F"
    }, {"category": "Po", "mappings": {"default": {"default": "colon"}}, "key": "FF1A"}, {
      "category": "Po",
      "mappings": {"default": {"default": "semicolon"}},
      "key": "FF1B"
    }, {"category": "Sm", "mappings": {"default": {"default": "less than"}}, "key": "FF1C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "equals"}},
      "key": "FF1D"
    }, {"category": "Sm", "mappings": {"default": {"default": "greater than"}}, "key": "FF1E"}, {
      "category": "Po",
      "mappings": {"default": {"default": "question mark"}},
      "key": "FF1F"
    }, {"category": "Po", "mappings": {"default": {"default": "commercial at"}}, "key": "FF20"}, {
      "category": "Po",
      "mappings": {"default": {"default": "backslash", "alternative": "reverse solidus"}},
      "key": "FF3C"
    }, {
      "category": "Sk",
      "mappings": {"default": {"default": "caret", "alternative": "circumflex"}},
      "key": "FF3E"
    }, {
      "category": "Pc",
      "mappings": {"default": {"default": "bar", "alternative": "underline"}},
      "key": "FF3F"
    }, {"category": "Sk", "mappings": {"default": {"default": "grave"}}, "key": "FF40"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "vertical bar"}},
      "key": "FF5C"
    }, {"category": "Sm", "mappings": {"default": {"default": "tilde"}}, "key": "FF5E"}, {
      "category": "Sc",
      "mappings": {"default": {"default": "cent sign"}},
      "key": "FFE0"
    }, {"category": "Sc", "mappings": {"default": {"default": "pound sign"}}, "key": "FFE1"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "not sign"}},
      "key": "FFE2"
    }, {
      "category": "Sk",
      "mappings": {"default": {"default": "macron"}, "mathspeak": {"default": "bar"}},
      "key": "FFE3"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "broken vertical bar"}},
      "key": "FFE4"
    }, {"category": "Sc", "mappings": {"default": {"default": "yen sign"}}, "key": "FFE5"}, {
      "category": "Sc",
      "mappings": {"default": {"default": "won sign"}},
      "key": "FFE6"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "halfwidth forms light vertical"}},
      "key": "FFE8"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "halfwidth black square"}},
      "key": "FFED"
    }, {"category": "So", "mappings": {"default": {"default": "halfwidth white circle"}}, "key": "FFEE"}]
  ,
  "en/symbols/math_whitespace.js":
    [{"locale": "en"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "space"}},
      "key": "0020"
    }, {
      "category": "Zs",
      "mappings": {"default": {"default": "no break space", "alternative": "non breaking space"}},
      "key": "00A0"
    }, {"category": "Cf", "mappings": {"default": {"default": "soft hyphen"}}, "key": "00AD"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "en quad"}},
      "key": "2000"
    }, {"category": "Zs", "mappings": {"default": {"default": "em quad"}}, "key": "2001"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "en space"}},
      "key": "2002"
    }, {"category": "Zs", "mappings": {"default": {"default": "em space"}}, "key": "2003"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "three per em space"}},
      "key": "2004"
    }, {"category": "Zs", "mappings": {"default": {"default": "four per em space"}}, "key": "2005"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "six per em space"}},
      "key": "2006"
    }, {"category": "Zs", "mappings": {"default": {"default": "figure space"}}, "key": "2007"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "punctuation space"}},
      "key": "2008"
    }, {"category": "Zs", "mappings": {"default": {"default": "thin space"}}, "key": "2009"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "hair space"}},
      "key": "200A"
    }, {"category": "Cf", "mappings": {"default": {"default": "zero width space"}}, "key": "200B"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "zero width non joiner"}},
      "key": "200C"
    }, {"category": "Cf", "mappings": {"default": {"default": "zero width joiner"}}, "key": "200D"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "left to right mark"}},
      "key": "200E"
    }, {"category": "Cf", "mappings": {"default": {"default": "right to left mark"}}, "key": "200F"}, {
      "category": "Zl",
      "mappings": {"default": {"default": "line separator"}},
      "key": "2028"
    }, {
      "category": "Zp",
      "mappings": {"default": {"default": "paragraph separator"}},
      "key": "2029"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "left to right embedding"}},
      "key": "202A"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "right to left embedding"}},
      "key": "202B"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "pop directional formatting"}},
      "key": "202C"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "left to right override"}},
      "key": "202D"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "right to left override"}},
      "key": "202E"
    }, {
      "category": "Zs",
      "mappings": {"default": {"default": "narrow no break space"}},
      "key": "202F"
    }, {
      "category": "Zs",
      "mappings": {"default": {"default": "medium mathematical space"}},
      "key": "205F"
    }, {"category": "Cf", "mappings": {"default": {"default": "word joiner"}}, "key": "2060"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "of", "alternative": "function application"}},
      "key": "2061"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "times", "alternative": "invisible times"}},
      "key": "2062"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "separator", "alternative": "invisible separator"}},
      "key": "2063"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "plus", "alternative": "invisible plus"}},
      "key": "2064"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "inhibit symmetric swapping"}},
      "key": "206A"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "activate symmetric swapping"}},
      "key": "206B"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "national digit shapes"}},
      "key": "206E"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "nominal digit shapes"}},
      "key": "206F"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "zero width no break space", "alternative": "byte order mark"}},
      "key": "FEFF"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "interlinear annotation anchor"}},
      "key": "FFF9"
    }, {
      "category": "Cf",
      "mappings": {"default": {"default": "interlinear annotation separator"}},
      "key": "FFFA"
    }, {"category": "Cf", "mappings": {"default": {"default": "interlinear annotation terminator"}}, "key": "FFFB"}]
  ,
  "en/symbols/other_stars.js":
    [{"locale": "en"}, {
      "category": "So",
      "mappings": {"default": {"default": "decimal exponent symbol"}},
      "key": "23E8"
    }, {"category": "So", "mappings": {"default": {"default": "black star"}}, "key": "2605"}, {
      "category": "So",
      "mappings": {"default": {"default": "white star"}},
      "key": "2606"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white circle", "alternative": "medium white circle"}},
      "key": "26AA"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black circle", "alternative": "medium black circle"}},
      "key": "26AB"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white check mark", "alternative": "white heavy check mark"}},
      "key": "2705"
    }, {"category": "So", "mappings": {"default": {"default": "check mark"}}, "key": "2713"}, {
      "category": "So",
      "mappings": {"default": {"default": "heavy check mark"}},
      "key": "2714"
    }, {"category": "So", "mappings": {"default": {"default": "multiplication x"}}, "key": "2715"}, {
      "category": "So",
      "mappings": {"default": {"default": "heavy multiplication x"}},
      "key": "2716"
    }, {"category": "So", "mappings": {"default": {"default": "ballot x"}}, "key": "2717"}, {
      "category": "So",
      "mappings": {"default": {"default": "heavy ballot x"}},
      "key": "2718"
    }, {"category": "So", "mappings": {"default": {"default": "open center cross"}}, "key": "271B"}, {
      "category": "So",
      "mappings": {"default": {"default": "heavy open center cross"}},
      "key": "271C"
    }, {"category": "So", "mappings": {"default": {"default": "maltese cross"}}, "key": "2720"}, {
      "category": "So",
      "mappings": {"default": {"default": "star of david"}},
      "key": "2721"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "four teardrop spoked asterisk"}},
      "key": "2722"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "four balloon spoked asterisk"}},
      "key": "2723"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "heavy four balloon spoked asterisk"}},
      "key": "2724"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "four club spoked asterisk"}},
      "key": "2725"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black four pointed star"}},
      "key": "2726"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "white four pointed star"}},
      "key": "2727"
    }, {"category": "So", "mappings": {"default": {"default": "sparkles"}}, "key": "2728"}, {
      "category": "So",
      "mappings": {"default": {"default": "stress outlined white star"}},
      "key": "2729"
    }, {"category": "So", "mappings": {"default": {"default": "circled white star"}}, "key": "272A"}, {
      "category": "So",
      "mappings": {"default": {"default": "open center black star"}},
      "key": "272B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "black center white star"}},
      "key": "272C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "outlined black star"}},
      "key": "272D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "heavy outlined black star"}},
      "key": "272E"
    }, {"category": "So", "mappings": {"default": {"default": "pinwheel star"}}, "key": "272F"}, {
      "category": "So",
      "mappings": {"default": {"default": "shadowed white star"}},
      "key": "2730"
    }, {"category": "So", "mappings": {"default": {"default": "heavy asterisk"}}, "key": "2731"}, {
      "category": "So",
      "mappings": {"default": {"default": "open center asterisk"}},
      "key": "2732"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "eight spoked asterisk"}},
      "key": "2733"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "eight pointed black star"}},
      "key": "2734"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "eight pointed pinwheel star"}},
      "key": "2735"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "six pointed black star"}},
      "key": "2736"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "twelve pointed black star"}},
      "key": "2739"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "sixteen pointed asterisk"}},
      "key": "273A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "teardrop spoked asterisk"}},
      "key": "273B"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "open center teardrop spoked asterisk"}},
      "key": "273C"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "heavy teardrop spoked asterisk"}},
      "key": "273D"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "six petalled black and white florette"}},
      "key": "273E"
    }, {"category": "So", "mappings": {"default": {"default": "black florette"}}, "key": "273F"}, {
      "category": "So",
      "mappings": {"default": {"default": "white florette"}},
      "key": "2740"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "eight petalled outlined black florette"}},
      "key": "2741"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "circled open center eight pointed star"}},
      "key": "2742"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "heavy teardrop spoked pinwheel asterisk"}},
      "key": "2743"
    }, {"category": "So", "mappings": {"default": {"default": "snowflake"}}, "key": "2744"}, {
      "category": "So",
      "mappings": {"default": {"default": "tight trifoliate snowflake"}},
      "key": "2745"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "heavy chevron snowflake"}},
      "key": "2746"
    }, {"category": "So", "mappings": {"default": {"default": "sparkle"}}, "key": "2747"}, {
      "category": "So",
      "mappings": {"default": {"default": "heavy sparkle"}},
      "key": "2748"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "balloon spoked asterisk"}},
      "key": "2749"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "eight teardrop spoked propeller asterisk"}},
      "key": "274A"
    }, {
      "category": "So",
      "mappings": {"default": {"default": "heavy eight teardrop spoked propeller asterisk"}},
      "key": "274B"
    }, {"category": "So", "mappings": {"default": {"default": "cross mark"}}, "key": "274C"}, {
      "category": "So",
      "mappings": {"default": {"default": "shadowed white circle"}},
      "key": "274D"
    }]
  ,
  "en/units/area.js":
    [{"locale": "en"}, {
      "category": "other",
      "mappings": {"default": {"default": "square"}},
      "key": "sq",
      "names": ["sq", "sq."]
    }, {
      "category": "area",
      "mappings": {"default": {"default": "square inch"}},
      "key": "sq inch",
      "names": ["sq inch", "sq. inch."]
    }, {
      "category": "area",
      "mappings": {"default": {"default": "square rod"}},
      "key": "sq rd",
      "names": ["sq rd", "sq. rd."]
    }, {
      "category": "area",
      "mappings": {"default": {"default": "square foot", "plural": "square feet"}},
      "key": "sq ft",
      "names": ["sq ft", "sq. ft."]
    }, {
      "category": "area",
      "mappings": {"default": {"default": "square yard"}},
      "key": "sq yd",
      "names": ["sq yd", "sq. yd."]
    }, {
      "category": "area",
      "mappings": {"default": {"default": "square mile"}},
      "key": "sq mi",
      "names": ["sq mi", "sq. mi."]
    }, {
      "category": "area",
      "mappings": {"default": {"default": "acre"}},
      "key": "acr",
      "names": ["ac", "ac.", "acr", "acr."]
    }, {"category": "area", "mappings": {"default": {"default": "hectare"}}, "key": "ha", "names": ["ha"]}]
  ,
  "en/units/currency.js":
    [{"locale": "en"}, {
      "category": "currency",
      "key": "$",
      "mappings": {"default": {"default": "dollar"}},
      "names": ["$", "💲", "＄", "﹩", "USD"]
    }, {
      "category": "currency",
      "key": "£",
      "mappings": {"default": {"default": "pound"}},
      "names": ["£", "￡", "GBP"]
    }, {
      "category": "currency",
      "key": "¥",
      "mappings": {"default": {"default": "yen"}},
      "names": ["¥", "￥", "JPY"]
    }, {
      "category": "currency",
      "key": "€",
      "mappings": {"default": {"default": "euro"}},
      "names": ["€", "EUR"]
    }, {
      "category": "currency",
      "key": "₡",
      "mappings": {"default": {"default": "colon"}},
      "names": ["₡", "CRC"]
    }, {
      "category": "currency",
      "key": "₢",
      "mappings": {"default": {"default": "cruzeiro"}},
      "names": ["₢"]
    }, {
      "category": "currency",
      "key": "₣",
      "mappings": {"default": {"default": "franc"}},
      "names": ["₣"]
    }, {
      "category": "currency",
      "key": "₤",
      "mappings": {"default": {"default": "lira"}},
      "names": ["₤"]
    }, {
      "category": "currency",
      "key": "₥",
      "mappings": {"default": {"default": "mill"}},
      "names": ["₥"]
    }, {
      "category": "currency",
      "key": "₦",
      "mappings": {"default": {"default": "naira"}},
      "names": ["₦", "NGN"]
    }, {
      "category": "currency",
      "key": "₧",
      "mappings": {"default": {"default": "peseta"}},
      "names": ["₧"]
    }, {
      "category": "currency",
      "key": "₨",
      "mappings": {"default": {"default": "rupee"}},
      "names": ["₨", "₹", "INR", "NPR", "PKR", "LKR"]
    }, {
      "category": "currency",
      "key": "₩",
      "mappings": {"default": {"default": "won"}},
      "names": ["₩", "￦", "KRW"]
    }, {
      "category": "currency",
      "key": "₪",
      "mappings": {"default": {"default": "sheqel"}},
      "names": ["₪"]
    }, {
      "category": "currency",
      "key": "₫",
      "mappings": {"default": {"default": "dong"}},
      "names": ["₫"]
    }, {
      "category": "currency",
      "key": "₭",
      "mappings": {"default": {"default": "kip"}},
      "names": ["₭"]
    }, {
      "category": "currency",
      "key": "₮",
      "mappings": {"default": {"default": "tugrik"}},
      "names": ["₮"]
    }, {
      "category": "currency",
      "key": "₯",
      "mappings": {"default": {"default": "drachma"}},
      "names": ["₯"]
    }, {
      "category": "currency",
      "key": "₰",
      "mappings": {"default": {"default": "Pfennig"}},
      "names": ["₰"]
    }, {
      "category": "currency",
      "key": "₱",
      "mappings": {"default": {"default": "peso"}},
      "names": ["₱"]
    }, {
      "category": "currency",
      "key": "₲",
      "mappings": {"default": {"default": "guaranis"}},
      "names": ["₲"]
    }, {
      "category": "currency",
      "key": "₳",
      "mappings": {"default": {"default": "austral"}},
      "names": ["₳"]
    }, {
      "category": "currency",
      "key": "₴",
      "mappings": {"default": {"default": "hryvnia"}},
      "names": ["₴", "UAH"]
    }, {
      "category": "currency",
      "key": "₵",
      "mappings": {"default": {"default": "cedis"}},
      "names": ["₵", "GHS"]
    }, {
      "category": "currency",
      "key": "₶",
      "mappings": {"default": {"default": "livre tournois"}},
      "names": ["₶"]
    }, {
      "category": "currency",
      "key": "₷",
      "mappings": {"default": {"default": "spesmilo"}},
      "names": ["₷"]
    }, {
      "category": "currency",
      "key": "₸",
      "mappings": {"default": {"default": "tenge"}},
      "names": ["₸", "KZT"]
    }, {
      "category": "currency",
      "key": "₺",
      "mappings": {"default": {"default": "turkish lira"}},
      "names": ["₺", "TRY"]
    }, {
      "category": "currency",
      "key": "元",
      "mappings": {"default": {"default": "yuan"}},
      "names": ["元"]
    }, {"category": "currency", "key": "¢", "mappings": {"default": {"default": "cent"}}, "names": ["￠", "¢"]}]
  ,
  "en/units/energy.js":
    [{"locale": "en"}, {
      "category": "energy",
      "mappings": {"default": {"default": "watt"}},
      "key": "W",
      "names": ["W", "w"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "kilowatt"}},
      "key": "kW",
      "names": ["kw", "kW"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "milliwatt"}},
      "key": "mW",
      "names": ["mw", "mW"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "kilowatt hour"}},
      "key": "kwh",
      "names": ["kwh", "kWh"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "joule"}},
      "key": "J",
      "names": ["J"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "Newton"}},
      "key": "N",
      "names": ["N"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "ampere"}},
      "key": "A",
      "names": ["A"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "volt"}},
      "key": "V",
      "names": ["V"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "microohm"}},
      "key": "µΩ",
      "names": ["µΩ"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "milliohm"}},
      "key": "mΩ",
      "names": ["mΩ"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "ohm"}},
      "key": "Ω",
      "names": ["Ω", "Ohm"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "kilohm"}},
      "key": "kΩ",
      "names": ["kΩ", "KΩ"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "megaohm"}},
      "key": "MΩ",
      "names": ["MΩ"]
    }, {"category": "energy", "mappings": {"default": {"default": "gigaohm"}}, "key": "GΩ", "names": ["GΩ"]}]
  ,
  "en/units/length.js":
    [{"locale": "en"}, {
      "category": "length",
      "mappings": {"default": {"default": "foot"}},
      "key": "ft",
      "names": ["ft", "ft."]
    }, {
      "category": "length",
      "mappings": {"default": {"default": "inch"}},
      "key": "in",
      "names": ["in", "in."]
    }, {
      "category": "length",
      "mappings": {"default": {"default": "mile"}},
      "key": "mi",
      "names": ["mi", "mi."]
    }, {
      "category": "length",
      "mappings": {"default": {"default": "yard"}},
      "key": "yd",
      "names": ["yd", "yd."]
    }, {
      "category": "length",
      "mappings": {"default": {"default": "link"}},
      "key": "link",
      "names": ["li", "li."]
    }, {
      "category": "length",
      "mappings": {"default": {"default": "rod"}},
      "key": "rod",
      "names": ["rd", "rd."]
    }, {
      "category": "length",
      "mappings": {"default": {"default": "chain"}},
      "key": "chain",
      "names": ["ch", "ch."]
    }, {
      "category": "length",
      "mappings": {"default": {"default": "furlong"}},
      "key": "furlong",
      "names": ["fur", "fur."]
    }, {
      "category": "length",
      "mappings": {"default": {"default": "nautical mile"}},
      "key": "n.m.",
      "names": ["n.m."]
    }, {
      "category": "length",
      "mappings": {"default": {"default": "millimeter"}},
      "key": "mm",
      "names": ["mm"]
    }, {
      "category": "length",
      "mappings": {"default": {"default": "centimeter"}},
      "key": "cm",
      "names": ["cm"]
    }, {
      "category": "length",
      "mappings": {"default": {"default": "meter"}},
      "key": "m",
      "names": ["m"]
    }, {"category": "length", "mappings": {"default": {"default": "kilometer"}}, "key": "km", "names": ["km"]}]
  ,
  "en/units/memory.js":
    [{"locale": "en"}, {
      "category": "memory",
      "mappings": {"default": {"default": "bit"}},
      "key": "b",
      "names": ["b"]
    }, {
      "category": "memory",
      "mappings": {"default": {"default": "byte"}},
      "key": "B",
      "names": ["B"]
    }, {
      "category": "memory",
      "mappings": {"default": {"default": "kilobyte"}},
      "key": "KB",
      "names": ["KB"]
    }, {
      "category": "memory",
      "mappings": {"default": {"default": "megabyte"}},
      "key": "MB",
      "names": ["MB"]
    }, {
      "category": "memory",
      "mappings": {"default": {"default": "gigabyte"}},
      "key": "GB",
      "names": ["GB"]
    }, {"category": "memory", "mappings": {"default": {"default": "terabyte"}}, "key": "TB", "names": ["TB"]}]
  ,
  "en/units/other.js":
    [{"locale": "en"}, {
      "category": "other",
      "mappings": {"default": {"default": "dozen"}},
      "key": "doz",
      "names": ["doz", "doz.", "dz", "dz."]
    }]
  ,
  "en/units/speed.js":
    [{"locale": "en"}, {
      "category": "speed",
      "mappings": {"default": {"default": "knot"}},
      "key": "kt",
      "names": ["kt", "kt."]
    }, {
      "category": "speed",
      "mappings": {"default": {"default": "miles per hour"}},
      "key": "mph",
      "names": ["mph"]
    }, {
      "category": "speed",
      "mappings": {"default": {"default": "revolutions per minute"}},
      "key": "rpm",
      "names": ["rpm"]
    }, {
      "category": "speed",
      "mappings": {"default": {"default": "kilometers per hour"}},
      "key": "kmh",
      "names": ["kmh"]
    }]
  ,
  "en/units/temperature.js":
    [{"locale": "en"}, {
      "category": "temperature",
      "mappings": {"default": {"default": "Fahrenheit"}},
      "key": "F",
      "names": ["F", "F.", "°F"]
    }, {
      "category": "temperature",
      "mappings": {"default": {"default": "Celsius", "alternative": "Centigrade"}},
      "key": "C",
      "names": ["C", "°C"]
    }, {"category": "temperature", "mappings": {"default": {"default": "Kelvin"}}, "key": "K", "names": ["K", "°K"]}]
  ,
  "en/units/time.js":
    [{"locale": "en"}, {
      "category": "time",
      "mappings": {"default": {"default": "nanosecond"}},
      "key": "ns",
      "names": ["ns"]
    }, {
      "category": "time",
      "mappings": {"default": {"default": "microsecond"}},
      "key": "µs",
      "names": ["µs"]
    }, {
      "category": "time",
      "mappings": {"default": {"default": "millisecond"}},
      "key": "ms",
      "names": ["ms"]
    }, {
      "category": "time",
      "mappings": {"default": {"default": "second"}},
      "key": "s",
      "names": ["s"]
    }, {
      "category": "time",
      "mappings": {"default": {"default": "minute"}},
      "key": "min",
      "names": ["min"]
    }, {"category": "time", "mappings": {"default": {"default": "hour"}}, "key": "h", "names": ["h", "hr"]}]
  ,
  "en/units/volume.js":
    [{"locale": "en"}, {
      "category": "volume",
      "mappings": {"default": {"default": "cubic"}},
      "key": "cu",
      "names": ["cu", "cu."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "cubic inch"}},
      "key": "cu inch",
      "names": ["cu in", "cu. in."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "cubic foot", "plural": "cubic feet"}},
      "key": "cu ft",
      "names": ["cu ft", "cu. ft."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "cubic yard"}},
      "key": "cu yd",
      "names": ["cu yd", "cu. yd."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "barrel"}},
      "key": "bbl",
      "names": ["bbl.", "bbl"]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "fluid ounce"}},
      "key": "fl. oz.",
      "names": ["fl. oz.", "fl oz"]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "gallon"}},
      "key": "gal",
      "names": ["gal", "gal."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "pint"}},
      "key": "pt",
      "names": ["pt", "pt."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "quart"}},
      "key": "qt",
      "names": ["qt", "qt."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "fluid dram"}},
      "key": "fluid dram",
      "names": ["fl dr", "fl. dr."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "tablespoon"}},
      "key": "tbsp",
      "names": ["tbsp", "tbsp.", "Tbsp", "Tbsp."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "teaspoon"}},
      "key": "tsp",
      "names": ["tsp", "tsp."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "cup"}},
      "key": "cup",
      "names": ["cp", "cp."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "cubic centimeter"}},
      "key": "cc",
      "names": ["cc", "ccm"]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "kiloliter"}},
      "key": "kl",
      "names": ["kl"]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "liter"}},
      "key": "l",
      "names": ["l"]
    }, {"category": "volume", "mappings": {"default": {"default": "milliliter"}}, "key": "ml", "names": ["ml"]}]
  ,
  "en/units/weight.js":
    [{"locale": "en"}, {
      "category": "weight",
      "mappings": {"default": {"default": "dram"}},
      "key": "dram",
      "names": ["dr", "dr."]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "ounce"}},
      "key": "oz",
      "names": ["oz", "oz."]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "pound"}},
      "key": "lb",
      "names": ["lb", "lb."]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "stone"}},
      "key": "st",
      "names": ["st", "st."]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "quarter"}},
      "key": "qtr",
      "names": ["qtr", "qtr."]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "hundredweight"}},
      "key": "cwt",
      "names": ["cwt", "cwt."]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "long ton"}},
      "key": "LT",
      "names": ["LT", "L.T."]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "gram"}},
      "key": "gr",
      "names": ["g", "gr"]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "kilogram"}},
      "key": "kg",
      "names": ["kg"]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "microgram"}},
      "key": "mcg",
      "names": ["mcg", "µg"]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "milligram"}},
      "key": "mg",
      "names": ["mg"]
    }, {"category": "weight", "mappings": {"default": {"default": "ton"}}, "key": "t", "names": ["t", "T"]}]
  ,
  "de/functions/algebra.js":
    [{"locale": "de"}, {
      "category": "Algebra",
      "mappings": {"default": {"default": "Grad"}},
      "key": "deg",
      "names": ["deg"]
    }, {
      "key": "det",
      "mappings": {"default": {"default": "Determinante"}},
      "category": "Algebra",
      "names": ["det"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "Dimension"}},
      "key": "dim",
      "names": ["dim"]
    }, {
      "category": "Algebra",
      "mappings": {
        "default": {"default": "Homomorphismus"},
        "mathspeak": {"default": "hom"},
        "clearspeak": {"default": "hom"}
      },
      "key": "hom",
      "names": ["hom", "Hom"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "Kern"}},
      "key": "ker",
      "names": ["ker"]
    }, {"category": "Algebra", "mappings": {"default": {"default": "Spur"}}, "key": "Tr", "names": ["Tr", "tr"]}]
  ,
  "de/functions/elementary.js":
    [{"locale": "de"}, {
      "key": "log",
      "mappings": {"default": {"default": "Logarithmus"}},
      "category": "Logarithm",
      "names": ["log"]
    }, {
      "key": "ln",
      "mappings": {"default": {"default": "natürlicher Logarithmus"}},
      "category": "Logarithm",
      "names": ["ln"]
    }, {
      "category": "Logarithm",
      "mappings": {"default": {"default": "Logarithmus zur Basis 10"}},
      "key": "lg",
      "names": ["lg"]
    }, {
      "category": "Elementary",
      "mappings": {
        "default": {"default": "Exponent"},
        "mathspeak": {"default": "exp"},
        "clearspeak": {"default": "exp"}
      },
      "key": "exp",
      "names": ["exp", "expt"]
    }, {
      "category": "Elementary",
      "mappings": {
        "default": {"default": "größter gemeinsamer Teiler"},
        "mathspeak": {"default": "ggt"},
        "clearspeak": {"default": "ggt"}
      },
      "key": "gcd",
      "names": ["gcd", "ggt", "ggT"]
    }, {
      "category": "Elementary",
      "mappings": {
        "default": {"default": "kleinstes gemeinsames Vielfaches"},
        "mathspeak": {"default": "kgv"},
        "clearspeak": {"default": "kgv"}
      },
      "key": "lcm",
      "names": ["lcm", "kgv", "kgV"]
    }, {
      "category": "Complex",
      "mappings": {"default": {"default": "Argument"}},
      "key": "arg",
      "names": ["arg"]
    }, {
      "key": "im",
      "mappings": {"default": {"default": "der Imaginärteil der komplexen Zahl"}},
      "category": "Complex",
      "names": ["im"]
    }, {
      "key": "re",
      "mappings": {"default": {"default": "der Realteil der komplexen Zahl"}},
      "category": "Complex",
      "names": ["re"]
    }, {
      "key": "mod",
      "mappings": {"default": {"default": "modulo"}},
      "category": "Elementary",
      "names": ["mod"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "Infimum"}},
      "key": "inf",
      "names": ["inf"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "Limes"}},
      "key": "lim",
      "names": ["lim"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "Limes inferior"}},
      "key": "liminf",
      "names": ["lim inf", "liminf"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "Limes superior"}},
      "key": "limsup",
      "names": ["lim sup", "limsup"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "Maximum"}},
      "key": "max",
      "names": ["max"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "Minimum"}},
      "key": "min",
      "names": ["min"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "Supremum"}},
      "key": "sup",
      "names": ["sup"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "Colimes"}},
      "key": "injlim",
      "names": ["injlim", "inj lim"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "projektiver Limes"}},
      "key": "projlim",
      "names": ["projlim", "proj lim"]
    }, {
      "category": "Probability",
      "mappings": {"default": {"default": "Wahrscheinlichkeit"}},
      "key": "Pr",
      "names": ["Pr"]
    }]
  ,
  "de/functions/hyperbolic.js":
    [{"locale": "de"}, {
      "key": "cosh",
      "mappings": {"default": {"default": "Kosinus hyperbolicus"}},
      "category": "Hyperbolic",
      "names": ["cosh"]
    }, {
      "key": "coth",
      "mappings": {"default": {"default": "Kotangens hyperbolicus"}},
      "category": "Hyperbolic",
      "names": ["coth"]
    }, {
      "key": "csch",
      "mappings": {"default": {"default": "Kosekans hyperbolicus"}},
      "category": "Hyperbolic",
      "names": ["csch"]
    }, {
      "key": "sech",
      "mappings": {"default": {"default": "Sekans hyperbolicus"}},
      "category": "Hyperbolic",
      "names": ["sech"]
    }, {
      "key": "sinh",
      "mappings": {"default": {"default": "Sinus hyperbolicus"}},
      "category": "Hyperbolic",
      "names": ["sinh"]
    }, {
      "key": "tanh",
      "mappings": {"default": {"default": "Tangens hyperbolicus"}},
      "category": "Hyperbolic",
      "names": ["tanh"]
    }, {
      "key": "arcosh",
      "mappings": {"default": {"default": "Areakosinus hyperbolicus"}},
      "category": "Area",
      "names": ["arcosh", "arccosh"]
    }, {
      "key": "arcoth",
      "mappings": {"default": {"default": "Areakotangens hyperbolicus"}},
      "category": "Area",
      "names": ["arcoth", "arccoth"]
    }, {
      "key": "arcsch",
      "mappings": {"default": {"default": "Areakosekans hyperbolicus"}},
      "category": "Area",
      "names": ["arcsch", "arccsch"]
    }, {
      "key": "arsech",
      "mappings": {"default": {"default": "Areasekans hyperbolicus"}},
      "category": "Area",
      "names": ["arsech", "arcsech"]
    }, {
      "key": "arsinh",
      "mappings": {"default": {"default": "Areasinus hyperbolicus"}},
      "category": "Area",
      "names": ["arsinh", "arcsinh"]
    }, {
      "key": "artanh",
      "mappings": {"default": {"default": "Areatangens hyperbolicus "}},
      "category": "Area",
      "names": ["artanh", "arctanh"]
    }]
  ,
  "de/functions/trigonometry.js":
    [{"locale": "de"}, {
      "key": "cos",
      "mappings": {"default": {"default": "Kosinus"}},
      "category": "Trigonometric",
      "names": ["cos", "cosine"]
    }, {
      "key": "cot",
      "mappings": {"default": {"default": "Kotangens"}},
      "category": "Trigonometric",
      "names": ["cot"]
    }, {
      "key": "csc",
      "mappings": {"default": {"default": "Kosekans"}},
      "category": "Trigonometric",
      "names": ["csc"]
    }, {
      "key": "sec",
      "mappings": {"default": {"default": "Sekans"}},
      "category": "Trigonometric",
      "names": ["sec"]
    }, {
      "key": "sin",
      "mappings": {"default": {"default": "Sinus"}},
      "category": "Trigonometric",
      "names": ["sin", "sine"]
    }, {
      "key": "tan",
      "mappings": {"default": {"default": "Tangens"}},
      "category": "Trigonometric",
      "names": ["tan"]
    }, {
      "key": "arccos",
      "mappings": {"default": {"default": "Arkuskosinus"}, "clearspeak": {"Trig_TrigInverse": "inverser Kosinus"}},
      "category": "Cyclometric",
      "names": ["arccos"]
    }, {
      "key": "arccot",
      "mappings": {"default": {"default": "Arkuskotangens"}, "clearspeak": {"Trig_TrigInverse": "inverser Kotangens"}},
      "category": "Cyclometric",
      "names": ["arccot"]
    }, {
      "key": "arccsc",
      "mappings": {"default": {"default": "Arkuskosekans"}, "clearspeak": {"Trig_TrigInverse": "inverser Kosekans"}},
      "category": "Cyclometric",
      "names": ["arccsc"]
    }, {
      "key": "arcsec",
      "mappings": {"default": {"default": "Arkussekans"}, "clearspeak": {"Trig_TrigInverse": "inverser Sekans"}},
      "category": "Cyclometric",
      "names": ["arcsec"]
    }, {
      "key": "arcsin",
      "mappings": {"default": {"default": "Arkussinus"}, "clearspeak": {"Trig_TrigInverse": "inverser Sinus"}},
      "category": "Cyclometric",
      "names": ["arcsin"]
    }, {
      "key": "arctan",
      "mappings": {"default": {"default": "Arkustangens"}, "clearspeak": {"Trig_TrigInverse": "inverser Tangens"}},
      "category": "Cyclometric",
      "names": ["arctan"]
    }]
  ,
  "de/symbols/digits_rest.js":
    [{"locale": "de"}, {
      "key": "00B2",
      "mappings": {
        "default": {"default": "Quadrat"},
        "clearspeak": {"Exponent_Exponent": "mit Exponent 2", "Exponent_OrdinalPower": "zur zweiten Potenz"}
      },
      "category": "No"
    }, {
      "key": "00B3",
      "mappings": {
        "default": {"default": "Kubik"},
        "clearspeak": {"Exponent_Exponent": "mit Exponent 3", "Exponent_OrdinalPower": "zur dritten Potenz"}
      },
      "category": "No"
    }, {"key": "00BC", "mappings": {"default": {"default": "ein Viertel"}}, "category": "No"}, {
      "key": "00BD",
      "mappings": {"default": {"default": "ein Halb"}},
      "category": "No"
    }, {"key": "00BE", "mappings": {"default": {"default": "drei Viertel"}}, "category": "No"}, {
      "category": "No",
      "key": "2150",
      "mappings": {"default": {"default": "ein siebtel"}}
    }, {"category": "No", "key": "2151", "mappings": {"default": {"default": "ein neuntel"}}}, {
      "category": "No",
      "key": "2152",
      "mappings": {"default": {"default": "ein zehntel"}}
    }, {"category": "No", "key": "2153", "mappings": {"default": {"default": "ein drittel"}}}, {
      "category": "No",
      "key": "2154",
      "mappings": {"default": {"default": "zwei drittel"}}
    }, {"category": "No", "key": "2155", "mappings": {"default": {"default": "ein fünftel"}}}, {
      "category": "No",
      "key": "2156",
      "mappings": {"default": {"default": "zwei fünftel"}}
    }, {"category": "No", "key": "2157", "mappings": {"default": {"default": "drei fünftel"}}}, {
      "category": "No",
      "key": "2158",
      "mappings": {"default": {"default": "vier fünftel"}}
    }, {"category": "No", "key": "2159", "mappings": {"default": {"default": "ein sechstel"}}}, {
      "category": "No",
      "key": "215A",
      "mappings": {"default": {"default": "fünf sechstel"}}
    }, {"category": "No", "key": "215B", "mappings": {"default": {"default": "ein achtel"}}}, {
      "category": "No",
      "key": "215C",
      "mappings": {"default": {"default": "drei achtel"}}
    }, {"category": "No", "key": "215D", "mappings": {"default": {"default": "fünf achtel"}}}, {
      "category": "No",
      "key": "215E",
      "mappings": {"default": {"default": "sieben achtel"}}
    }, {"category": "No", "key": "215F", "mappings": {"default": {"default": "Bruchzähler eins"}}}, {
      "category": "No",
      "key": "2189",
      "mappings": {"default": {"default": "null drittel"}}
    }, {
      "category": "No",
      "key": "3248",
      "mappings": {"default": {"default": "eingekreiste Zehn auf schwarzem Quadrat"}}
    }, {
      "category": "No",
      "key": "3249",
      "mappings": {"default": {"default": "eingekreiste Zwanzig auf schwarzem Quadrat"}}
    }, {
      "category": "No",
      "key": "324A",
      "mappings": {"default": {"default": "eingekreiste Dreißig auf schwarzem Quadrat"}}
    }, {
      "category": "No",
      "key": "324B",
      "mappings": {"default": {"default": "eingekreiste Vierzig auf schwarzem Quadrat"}}
    }, {
      "category": "No",
      "key": "324C",
      "mappings": {"default": {"default": "eingekreiste Fünfzig auf schwarzem Quadrat"}}
    }, {
      "category": "No",
      "key": "324D",
      "mappings": {"default": {"default": "eingekreiste Sechzig auf schwarzem Quadrat"}}
    }, {
      "category": "No",
      "key": "324E",
      "mappings": {"default": {"default": "eingekreiste Siebzig auf schwarzem Quadrat"}}
    }, {
      "category": "No",
      "key": "324F",
      "mappings": {"default": {"default": "eingekreiste Achtzig auf schwarzem Quadrat"}}
    }]
  ,
  "de/symbols/greek-rest.js":
    [{"locale": "de"}, {
      "key": "0394",
      "mappings": {"clearspeak": {"default": "Dreieck", "TriangleSymbol_Delta": "großes Delta"}},
      "category": "Lu"
    }]
  ,
  "de/symbols/greek-scripts.js":
    [{"locale": "de"}, {
      "category": "Ll",
      "key": "1D26",
      "mappings": {"default": {"default": "Kapitälchen Gamma"}}
    }, {"category": "Ll", "key": "1D27", "mappings": {"default": {"default": "Kapitälchen Lamda"}}}, {
      "category": "Ll",
      "key": "1D28",
      "mappings": {"default": {"default": "Kapitälchen Pi"}}
    }, {"category": "Ll", "key": "1D29", "mappings": {"default": {"default": "Kapitälchen Rho"}}}, {
      "category": "Ll",
      "key": "1D2A",
      "mappings": {"default": {"default": "Kapitälchen Psi"}}
    }, {
      "category": "Lm",
      "key": "1D5E",
      "mappings": {"default": {"default": "hochgestelltes gamma"}}
    }, {"category": "Lm", "key": "1D60", "mappings": {"default": {"default": "hochgestelltes phi"}}}, {
      "category": "Lm",
      "key": "1D66",
      "mappings": {"default": {"default": "tiefgestelltes beta"}}
    }, {
      "category": "Lm",
      "key": "1D67",
      "mappings": {"default": {"default": "tiefgestelltes gamma"}}
    }, {"category": "Lm", "key": "1D68", "mappings": {"default": {"default": "tiefgestelltes rho"}}}, {
      "category": "Lm",
      "key": "1D69",
      "mappings": {"default": {"default": "tiefgestelltes phi"}}
    }, {"category": "Lm", "key": "1D6A", "mappings": {"default": {"default": "tiefgestelltes chi"}}}]
  ,
  "de/symbols/greek-symbols.js":
    [{"locale": "de"}, {
      "category": "Ll",
      "key": "03D0",
      "mappings": {"default": {"default": "beta"}}
    }, {"category": "Ll", "key": "03D7", "mappings": {"default": {"default": "kai"}}}, {
      "category": "Sm",
      "key": "03F6",
      "mappings": {"default": {"default": "umgekehrtes epsilon"}}
    }, {
      "category": "Lu",
      "key": "1D7CA",
      "mappings": {"default": {"default": "fettes großes Digamma"}}
    }, {"category": "Ll", "key": "1D7CB", "mappings": {"default": {"default": "fettes digamma"}}}]
  ,
  "de/symbols/hebrew_letters.js":
    [{"locale": "de"}, {
      "category": "Lo",
      "key": "2135",
      "mappings": {"default": {"default": "Alef-Symbol"}}
    }, {"category": "Lo", "key": "2136", "mappings": {"default": {"default": "Bet-Symbol"}}}, {
      "category": "Lo",
      "key": "2137",
      "mappings": {"default": {"default": "Gimel-Symbol"}}
    }, {"category": "Lo", "key": "2138", "mappings": {"default": {"default": "Dalet-Symbol"}}}]
  ,
  "de/symbols/latin-lower-double-accent.js":
    [{"locale": "de"}, {
      "category": "Ll",
      "key": "01D6",
      "mappings": {"default": {"default": "u mit Diärese und Makron"}}
    }, {
      "category": "Ll",
      "key": "01D8",
      "mappings": {"default": {"default": "u mit Diärese und Akut"}}
    }, {
      "category": "Ll",
      "key": "01DA",
      "mappings": {"default": {"default": "u mit Diärese und Caron"}}
    }, {
      "category": "Ll",
      "key": "01DC",
      "mappings": {"default": {"default": "u mit Diärese und Grab"}}
    }, {
      "category": "Ll",
      "key": "01DF",
      "mappings": {"default": {"default": "a mit Diärese und Makron"}}
    }, {
      "category": "Ll",
      "key": "01E1",
      "mappings": {"default": {"default": "a mit Dot Above und Makron"}}
    }, {
      "category": "Ll",
      "key": "01ED",
      "mappings": {"default": {"default": "o mit Ogonek und Makron"}}
    }, {
      "category": "Ll",
      "key": "01FB",
      "mappings": {"default": {"default": "a mit Ring oben und akut"}}
    }, {
      "category": "Ll",
      "key": "022B",
      "mappings": {"default": {"default": "o mit Diärese und Makron"}}
    }, {
      "category": "Ll",
      "key": "022D",
      "mappings": {"default": {"default": "o mit Tilde und Makron"}}
    }, {
      "category": "Ll",
      "key": "0231",
      "mappings": {"default": {"default": "o mit Punkt oben und Makron"}}
    }, {
      "category": "Ll",
      "key": "1E09",
      "mappings": {"default": {"default": "c mit Cedilla und Akut"}}
    }, {
      "category": "Ll",
      "key": "1E15",
      "mappings": {"default": {"default": "e mit Makron und Grab"}}
    }, {
      "category": "Ll",
      "key": "1E17",
      "mappings": {"default": {"default": "e mit Makron und Akut"}}
    }, {
      "category": "Ll",
      "key": "1E1D",
      "mappings": {"default": {"default": "e mit Cedilla und Breve"}}
    }, {
      "category": "Ll",
      "key": "1E2F",
      "mappings": {"default": {"default": "i mit Diärese und Akut"}}
    }, {
      "category": "Ll",
      "key": "1E39",
      "mappings": {"default": {"default": "l mit Punkt unten und Makron"}}
    }, {
      "category": "Ll",
      "key": "1E4D",
      "mappings": {"default": {"default": "o mit Tilde und Akut"}}
    }, {
      "category": "Ll",
      "key": "1E4F",
      "mappings": {"default": {"default": "o mit Tilde und Diärese"}}
    }, {
      "category": "Ll",
      "key": "1E51",
      "mappings": {"default": {"default": "o mit Makron und Grab"}}
    }, {
      "category": "Ll",
      "key": "1E53",
      "mappings": {"default": {"default": "o mit Makron und Akut"}}
    }, {
      "category": "Ll",
      "key": "1E5D",
      "mappings": {"default": {"default": "r mit Punkt unten und Makron"}}
    }, {
      "category": "Ll",
      "key": "1E65",
      "mappings": {"default": {"default": "s mit Akute und Punkt oben"}}
    }, {
      "category": "Ll",
      "key": "1E67",
      "mappings": {"default": {"default": "s mit Caron und Punkt oben"}}
    }, {
      "category": "Ll",
      "key": "1E69",
      "mappings": {"default": {"default": "s mit Punkt unten und Punkt oben"}}
    }, {
      "category": "Ll",
      "key": "1E79",
      "mappings": {"default": {"default": "u mit Tilde und Akut"}}
    }, {
      "category": "Ll",
      "key": "1E7B",
      "mappings": {"default": {"default": "u mit Makron und Diärese"}}
    }, {
      "category": "Ll",
      "key": "1EA5",
      "mappings": {"default": {"default": "a mit Zirkumflex und Akut"}}
    }, {
      "category": "Ll",
      "key": "1EA7",
      "mappings": {"default": {"default": "a mit Zirkumflex und Grab"}}
    }, {
      "category": "Ll",
      "key": "1EA9",
      "mappings": {"default": {"default": "a mit Zirkumflex und Haken oben"}}
    }, {
      "category": "Ll",
      "key": "1EAB",
      "mappings": {"default": {"default": "a mit Zirkumflex und Tilde"}}
    }, {
      "category": "Ll",
      "key": "1EAD",
      "mappings": {"default": {"default": "a mit Zirkumflex und Punkt darunter"}}
    }, {
      "category": "Ll",
      "key": "1EAF",
      "mappings": {"default": {"default": "a mit Breve und Akut"}}
    }, {
      "category": "Ll",
      "key": "1EB1",
      "mappings": {"default": {"default": "a mit Breve und Grab"}}
    }, {
      "category": "Ll",
      "key": "1EB3",
      "mappings": {"default": {"default": "a mit Breve und Haken oben"}}
    }, {
      "category": "Ll",
      "key": "1EB5",
      "mappings": {"default": {"default": "a mit Breve und Tilde"}}
    }, {
      "category": "Ll",
      "key": "1EB7",
      "mappings": {"default": {"default": "a mit Breve und Dot Below"}}
    }, {
      "category": "Ll",
      "key": "1EBF",
      "mappings": {"default": {"default": "e mit Zirkumflex und Akut"}}
    }, {
      "category": "Ll",
      "key": "1EC1",
      "mappings": {"default": {"default": "e mit Zirkumflex und Grab"}}
    }, {
      "category": "Ll",
      "key": "1EC3",
      "mappings": {"default": {"default": "e mit Zirkumflex und Haken oben"}}
    }, {
      "category": "Ll",
      "key": "1EC5",
      "mappings": {"default": {"default": "e mit Zirkumflex und Tilde"}}
    }, {
      "category": "Ll",
      "key": "1EC7",
      "mappings": {"default": {"default": "e mit Zirkumflex und Punkt darunter"}}
    }, {
      "category": "Ll",
      "key": "1ED1",
      "mappings": {"default": {"default": "o mit Zirkumflex und Akut"}}
    }, {
      "category": "Ll",
      "key": "1ED3",
      "mappings": {"default": {"default": "o mit Zirkumflex und Grab"}}
    }, {
      "category": "Ll",
      "key": "1ED5",
      "mappings": {"default": {"default": "o mit Zirkumflex und Haken oben"}}
    }, {
      "category": "Ll",
      "key": "1ED7",
      "mappings": {"default": {"default": "o mit Zirkumflex und Tilde"}}
    }, {
      "category": "Ll",
      "key": "1ED9",
      "mappings": {"default": {"default": "o mit Zirkumflex und Punkt darunter"}}
    }, {
      "category": "Ll",
      "key": "1EDB",
      "mappings": {"default": {"default": "o mit Horn und Akut"}}
    }, {
      "category": "Ll",
      "key": "1EDD",
      "mappings": {"default": {"default": "o mit Horn und Grab"}}
    }, {
      "category": "Ll",
      "key": "1EDF",
      "mappings": {"default": {"default": "o mit Horn und Haken oben"}}
    }, {
      "category": "Ll",
      "key": "1EE1",
      "mappings": {"default": {"default": "o mit Horn und Tilde"}}
    }, {
      "category": "Ll",
      "key": "1EE3",
      "mappings": {"default": {"default": "o mit Horn und Punkt"}}
    }, {
      "category": "Ll",
      "key": "1EE9",
      "mappings": {"default": {"default": "u mit Horn und Akut"}}
    }, {
      "category": "Ll",
      "key": "1EEB",
      "mappings": {"default": {"default": "u mit Horn und Grab"}}
    }, {
      "category": "Ll",
      "key": "1EED",
      "mappings": {"default": {"default": "u mit Horn und Haken oben"}}
    }, {
      "category": "Ll",
      "key": "1EEF",
      "mappings": {"default": {"default": "u mit Horn und Tilde"}}
    }, {"category": "Ll", "key": "1EF1", "mappings": {"default": {"default": "u mit Horn und Punkt"}}}]
  ,
  "de/symbols/latin-lower-phonetic.js":
    [{"locale": "de"}, {
      "category": "Ll",
      "key": "00F8",
      "mappings": {"default": {"default": "o mit Strich"}}
    }, {"category": "Ll", "key": "0111", "mappings": {"default": {"default": "d mit Strich"}}}, {
      "category": "Ll",
      "key": "0127",
      "mappings": {"default": {"default": "h mit Strich"}}
    }, {"category": "Ll", "key": "0142", "mappings": {"default": {"default": "l mit Strich"}}}, {
      "category": "Ll",
      "key": "0167",
      "mappings": {"default": {"default": "t mit Strich"}}
    }, {"category": "Ll", "key": "0180", "mappings": {"default": {"default": "b mit Strich"}}}, {
      "category": "Ll",
      "key": "019B",
      "mappings": {"default": {"default": "lambda mit Strich"}}
    }, {"category": "Ll", "key": "01B6", "mappings": {"default": {"default": "z mit Strich"}}}, {
      "category": "Ll",
      "key": "01BE",
      "mappings": {"default": {"default": "umgekehrter Glottalanschlag mit Strich"}}
    }, {"category": "Ll", "key": "01E5", "mappings": {"default": {"default": "g mit Strich"}}}, {
      "category": "Ll",
      "key": "01FF",
      "mappings": {"default": {"default": "o mit Strich und Akut"}}
    }, {"category": "Ll", "key": "023C", "mappings": {"default": {"default": "c mit Strich"}}}, {
      "category": "Ll",
      "key": "0247",
      "mappings": {"default": {"default": "e mit Strich"}}
    }, {"category": "Ll", "key": "0249", "mappings": {"default": {"default": "j mit Strich"}}}, {
      "category": "Ll",
      "key": "024D",
      "mappings": {"default": {"default": "r mit Strich"}}
    }, {"category": "Ll", "key": "024F", "mappings": {"default": {"default": "y mit Strich"}}}, {
      "category": "Ll",
      "key": "025F",
      "mappings": {"default": {"default": "punktloses j mit Strich"}}
    }, {"category": "Ll", "key": "0268", "mappings": {"default": {"default": "i mit Strich"}}}, {
      "category": "Ll",
      "key": "0284",
      "mappings": {"default": {"default": "punktloses j mit Strich und Haken"}}
    }, {
      "category": "Ll",
      "key": "02A1",
      "mappings": {"default": {"default": "Glottalanschlag mit Strich"}}
    }, {
      "category": "Ll",
      "key": "02A2",
      "mappings": {"default": {"default": "umgekehrter Glottalanschlag mit Strich"}}
    }, {
      "category": "Ll",
      "key": "1D13",
      "mappings": {"default": {"default": "liegendes O mit Strich"}}
    }, {"category": "Ll", "key": "1D7C", "mappings": {"default": {"default": "iota mit Strich"}}}, {
      "category": "Ll",
      "key": "1D7D",
      "mappings": {"default": {"default": "p mit Strich"}}
    }, {"category": "Ll", "key": "1D7F", "mappings": {"default": {"default": "upsilon mit Strich"}}}, {
      "category": "Ll",
      "key": "1E9C",
      "mappings": {"default": {"default": "lang mit diagonalem Anschlag"}}
    }, {
      "category": "Ll",
      "key": "1E9D",
      "mappings": {"default": {"default": "lang S mit hohem Hub"}}
    }, {
      "category": "Ll",
      "key": "018D",
      "mappings": {"default": {"default": "wurde Delta gedreht"}}
    }, {
      "category": "Ll",
      "key": "1E9B",
      "mappings": {"default": {"default": "lang mit Punkt oben"}}
    }, {"category": "Ll", "key": "1E9F", "mappings": {"default": {"default": "delta"}}}, {
      "key": "0138",
      "mappings": {"default": {"default": "kra"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "017F", "mappings": {"default": {"default": "lang S"}}}, {
      "category": "Ll",
      "key": "0183",
      "mappings": {"default": {"default": "b mit Oberleiste"}}
    }, {"category": "Ll", "key": "0185", "mappings": {"default": {"default": "ton sechs"}}}, {
      "category": "Ll",
      "key": "0188",
      "mappings": {"default": {"default": "c mit Haken"}}
    }, {"category": "Ll", "key": "018C", "mappings": {"default": {"default": "d mit Oberleiste"}}}, {
      "key": "0192",
      "mappings": {"default": {"default": "f mit Haken"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0195", "mappings": {"default": {"default": "hv"}}}, {
      "category": "Ll",
      "key": "0199",
      "mappings": {"default": {"default": "k mit Haken"}}
    }, {"key": "019A", "mappings": {"default": {"default": "l mit Strich"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "019E",
      "mappings": {"default": {"default": "n mit langem rechtem Bein"}}
    }, {"category": "Ll", "key": "01A1", "mappings": {"default": {"default": "o mit Horn"}}}, {
      "category": "Ll",
      "key": "01A3",
      "mappings": {"default": {"default": "oi"}}
    }, {"category": "Ll", "key": "01A5", "mappings": {"default": {"default": "p mit Haken"}}}, {
      "category": "Ll",
      "key": "01A8",
      "mappings": {"default": {"default": "ton zwei"}}
    }, {
      "category": "Ll",
      "key": "01AA",
      "mappings": {"default": {"default": "umgedrehte esh Schleife"}}
    }, {
      "category": "Ll",
      "key": "01AB",
      "mappings": {"default": {"default": "t mit palatal Haken"}}
    }, {"category": "Ll", "key": "01AD", "mappings": {"default": {"default": "t mit Haken"}}}, {
      "category": "Ll",
      "key": "01B0",
      "mappings": {"default": {"default": "u mit Horn"}}
    }, {"category": "Ll", "key": "01B4", "mappings": {"default": {"default": "y mit Haken"}}}, {
      "category": "Ll",
      "key": "01B9",
      "mappings": {"default": {"default": "umgekehrtes ezh"}}
    }, {"category": "Ll", "key": "01BA", "mappings": {"default": {"default": "ezh mit Schwanz"}}}, {
      "category": "Ll",
      "key": "01BD",
      "mappings": {"default": {"default": "ton fünf"}}
    }, {"category": "Ll", "key": "01BF", "mappings": {"default": {"default": "wynn"}}}, {
      "category": "Ll",
      "key": "01C6",
      "mappings": {"default": {"default": "dz mit Caron"}}
    }, {"category": "Ll", "key": "01C9", "mappings": {"default": {"default": "lj"}}}, {
      "category": "Ll",
      "key": "01CC",
      "mappings": {"default": {"default": "nj"}}
    }, {"category": "Ll", "key": "01E3", "mappings": {"default": {"default": "ae mit Makron"}}}, {
      "category": "Ll",
      "key": "01EF",
      "mappings": {"default": {"default": "ezh mit Caron"}}
    }, {"category": "Ll", "key": "01F3", "mappings": {"default": {"default": "dz"}}}, {
      "category": "Ll",
      "key": "021D",
      "mappings": {"default": {"default": "yogh"}}
    }, {"category": "Ll", "key": "026E", "mappings": {"default": {"default": "lezh"}}}, {
      "category": "Ll",
      "key": "0292",
      "mappings": {"default": {"default": "ezh"}}
    }, {"category": "Ll", "key": "0293", "mappings": {"default": {"default": "ezh mit Kringel"}}}, {
      "category": "Ll",
      "key": "02A4",
      "mappings": {"default": {"default": "dezh Digraph"}}
    }, {"category": "Ll", "key": "01DD", "mappings": {"default": {"default": "gedreht E"}}}, {
      "category": "Ll",
      "key": "01FD",
      "mappings": {"default": {"default": "ae mit Akut"}}
    }, {"category": "Ll", "key": "0221", "mappings": {"default": {"default": "d mit Kringel"}}}, {
      "category": "Ll",
      "key": "0223",
      "mappings": {"default": {"default": "ou"}}
    }, {"category": "Ll", "key": "0225", "mappings": {"default": {"default": "z mit Haken"}}}, {
      "category": "Ll",
      "key": "0234",
      "mappings": {"default": {"default": "l mit Kringel"}}
    }, {"category": "Ll", "key": "0235", "mappings": {"default": {"default": "n mit Kringel"}}}, {
      "category": "Ll",
      "key": "0236",
      "mappings": {"default": {"default": "t mit Kringel"}}
    }, {"category": "Ll", "key": "0238", "mappings": {"default": {"default": "db"}}}, {
      "category": "Ll",
      "key": "0239",
      "mappings": {"default": {"default": "qp Digraph"}}
    }, {"category": "Ll", "key": "023F", "mappings": {"default": {"default": "s mit Swash Tail"}}}, {
      "category": "Ll",
      "key": "0240",
      "mappings": {"default": {"default": "z mit Swash Tail"}}
    }, {"category": "Ll", "key": "0242", "mappings": {"default": {"default": "Glottalanschlag"}}}, {
      "category": "Ll",
      "key": "024B",
      "mappings": {"default": {"default": "q mit Hakenschwanz"}}
    }, {"category": "Ll", "key": "0250", "mappings": {"default": {"default": "gedreht A"}}}, {
      "category": "Ll",
      "key": "0251",
      "mappings": {"default": {"default": "alpha"}}
    }, {"category": "Ll", "key": "0252", "mappings": {"default": {"default": "gedrehtes Alpha"}}}, {
      "category": "Ll",
      "key": "0253",
      "mappings": {"default": {"default": "b mit Haken"}}
    }, {"category": "Ll", "key": "0254", "mappings": {"default": {"default": "open O"}}}, {
      "category": "Ll",
      "key": "0255",
      "mappings": {"default": {"default": "c mit Kringel"}}
    }, {"category": "Ll", "key": "0256", "mappings": {"default": {"default": "d mit Schwanz"}}}, {
      "category": "Ll",
      "key": "0257",
      "mappings": {"default": {"default": "d mit Haken"}}
    }, {"category": "Ll", "key": "0258", "mappings": {"default": {"default": "umgedreht E"}}}, {
      "category": "Ll",
      "key": "0259",
      "mappings": {"default": {"default": "schwa"}}
    }, {"category": "Ll", "key": "025A", "mappings": {"default": {"default": "schwa mit Haken"}}}, {
      "key": "025B",
      "mappings": {"default": {"default": "offenes e"}},
      "category": "Ll"
    }, {
      "category": "Ll",
      "key": "025C",
      "mappings": {"default": {"default": "umgedrehtes offenes e"}}
    }, {
      "category": "Ll",
      "key": "025D",
      "mappings": {"default": {"default": "umgekehrtes offenes e mit Haken"}}
    }, {
      "category": "Ll",
      "key": "025E",
      "mappings": {"default": {"default": "umgekehrtes geschlossenes epsilon"}}
    }, {"category": "Ll", "key": "0260", "mappings": {"default": {"default": "g mit Haken"}}}, {
      "category": "Ll",
      "key": "0261",
      "mappings": {"default": {"default": "g"}}
    }, {"category": "Ll", "key": "0263", "mappings": {"default": {"default": "gamma"}}}, {
      "category": "Ll",
      "key": "0264",
      "mappings": {"default": {"default": "mini gamma"}}
    }, {"category": "Ll", "key": "0265", "mappings": {"default": {"default": "gedrehtes h"}}}, {
      "category": "Ll",
      "key": "0266",
      "mappings": {"default": {"default": "h mit Haken"}}
    }, {"category": "Ll", "key": "0267", "mappings": {"default": {"default": "h mit Haken"}}}, {
      "category": "Ll",
      "key": "0269",
      "mappings": {"default": {"default": "iota"}}
    }, {
      "category": "Ll",
      "key": "026B",
      "mappings": {"default": {"default": "l mit mittlerer Tilde"}}
    }, {"category": "Ll", "key": "026C", "mappings": {"default": {"default": "l mit Gurt"}}}, {
      "category": "Ll",
      "key": "026D",
      "mappings": {"default": {"default": "l mit Retroflexhaken"}}
    }, {"category": "Ll", "key": "026F", "mappings": {"default": {"default": "gedreht M"}}}, {
      "category": "Ll",
      "key": "0270",
      "mappings": {"default": {"default": "gedrehtes M mit langem Bein"}}
    }, {"category": "Ll", "key": "0271", "mappings": {"default": {"default": "m mit Haken"}}}, {
      "category": "Ll",
      "key": "0272",
      "mappings": {"default": {"default": "n mit linkem Haken"}}
    }, {
      "category": "Ll",
      "key": "0273",
      "mappings": {"default": {"default": "n mit Retroflexhaken"}}
    }, {"category": "Ll", "key": "0275", "mappings": {"default": {"default": "o mit balken"}}}, {
      "category": "Ll",
      "key": "0277",
      "mappings": {"default": {"default": "geschlossenes omega"}}
    }, {"category": "Ll", "key": "0278", "mappings": {"default": {"default": "phi"}}}, {
      "category": "Ll",
      "key": "0279",
      "mappings": {"default": {"default": "gedrehtes r"}}
    }, {
      "category": "Ll",
      "key": "027A",
      "mappings": {"default": {"default": "gedrehtes r mit langem Bein"}}
    }, {
      "category": "Ll",
      "key": "027B",
      "mappings": {"default": {"default": "gedrehtes r mit langem Bein und Haken"}}
    }, {"category": "Ll", "key": "027C", "mappings": {"default": {"default": "r mit langem Bein"}}}, {
      "category": "Ll",
      "key": "027D",
      "mappings": {"default": {"default": "r mit Endstück"}}
    }, {"category": "Ll", "key": "027E", "mappings": {"default": {"default": "r mit Angelhaken"}}}, {
      "category": "Ll",
      "key": "027F",
      "mappings": {"default": {"default": "umgedrehtes R mit Angelhaken"}}
    }, {"category": "Ll", "key": "0282", "mappings": {"default": {"default": "s mit Haken"}}}, {
      "category": "Ll",
      "key": "0283",
      "mappings": {"default": {"default": "esh"}}
    }, {"category": "Ll", "key": "0285", "mappings": {"default": {"default": "squat umgekehrt"}}}, {
      "category": "Ll",
      "key": "0286",
      "mappings": {"default": {"default": "esh mit Kringel"}}
    }, {"category": "Ll", "key": "0287", "mappings": {"default": {"default": "gedreht T"}}}, {
      "category": "Ll",
      "key": "0288",
      "mappings": {"default": {"default": "t mit Retroflexhaken"}}
    }, {"category": "Ll", "key": "0289", "mappings": {"default": {"default": "u Bar"}}}, {
      "category": "Ll",
      "key": "028A",
      "mappings": {"default": {"default": "upsilon"}}
    }, {"category": "Ll", "key": "028B", "mappings": {"default": {"default": "v mit Haken"}}}, {
      "category": "Ll",
      "key": "028C",
      "mappings": {"default": {"default": "gedrehtes v"}}
    }, {"category": "Ll", "key": "028D", "mappings": {"default": {"default": "gedrehtes w"}}}, {
      "category": "Ll",
      "key": "028E",
      "mappings": {"default": {"default": "gedrehtes y"}}
    }, {
      "category": "Ll",
      "key": "0290",
      "mappings": {"default": {"default": "z mit Retroflexhaken"}}
    }, {"category": "Ll", "key": "0291", "mappings": {"default": {"default": "z mit Kringel"}}}, {
      "category": "Ll",
      "key": "0295",
      "mappings": {"default": {"default": "pharyngales stimmhaft Frikativ"}}
    }, {
      "category": "Ll",
      "key": "0296",
      "mappings": {"default": {"default": "invertierter Glottalanschlag"}}
    }, {"category": "Ll", "key": "0297", "mappings": {"default": {"default": "gestrecktes C"}}}, {
      "category": "Ll",
      "key": "0298",
      "mappings": {"default": {"default": "bilabialer Klick"}}
    }, {
      "category": "Ll",
      "key": "029A",
      "mappings": {"default": {"default": "geschlossenes offenes e"}}
    }, {"category": "Ll", "key": "029E", "mappings": {"default": {"default": "gedrehtes k"}}}, {
      "category": "Ll",
      "key": "02A0",
      "mappings": {"default": {"default": "q mit Haken"}}
    }, {"category": "Ll", "key": "02A3", "mappings": {"default": {"default": "dz Digraph"}}}, {
      "category": "Ll",
      "key": "02A5",
      "mappings": {"default": {"default": "dz Digraph mit Kringel"}}
    }, {"category": "Ll", "key": "02A6", "mappings": {"default": {"default": "ts Digraph"}}}, {
      "category": "Ll",
      "key": "02A7",
      "mappings": {"default": {"default": "tesh Digraph"}}
    }, {
      "category": "Ll",
      "key": "02A8",
      "mappings": {"default": {"default": "tc Digraph mit Kringel"}}
    }, {"category": "Ll", "key": "02A9", "mappings": {"default": {"default": "feng Digraph"}}}, {
      "category": "Ll",
      "key": "02AA",
      "mappings": {"default": {"default": "ls Digraph"}}
    }, {"category": "Ll", "key": "02AB", "mappings": {"default": {"default": "lz Digraph"}}}, {
      "category": "Ll",
      "key": "02AC",
      "mappings": {"default": {"default": "bilabiales Perkussiv"}}
    }, {
      "category": "Ll",
      "key": "02AD",
      "mappings": {"default": {"default": "bidentales Perkussiv"}}
    }, {
      "category": "Ll",
      "key": "02AE",
      "mappings": {"default": {"default": "gedrehtes h mit Angelhaken"}}
    }, {
      "category": "Ll",
      "key": "02AF",
      "mappings": {"default": {"default": "gedrehtes h mit Angelhaken und Schwanz"}}
    }, {"category": "Ll", "key": "1D02", "mappings": {"default": {"default": "gedrehtes ae"}}}, {
      "category": "Ll",
      "key": "1D08",
      "mappings": {"default": {"default": "gedrehtes e"}}
    }, {"category": "Ll", "key": "1D09", "mappings": {"default": {"default": "gedrehtes i"}}}, {
      "category": "Ll",
      "key": "1D11",
      "mappings": {"default": {"default": "liegendes O"}}
    }, {
      "category": "Ll",
      "key": "1D12",
      "mappings": {"default": {"default": "liegendes geöffnet O"}}
    }, {"category": "Ll", "key": "1D14", "mappings": {"default": {"default": "gedreht Oe"}}}, {
      "category": "Ll",
      "key": "1D16",
      "mappings": {"default": {"default": "obere Hälfte O"}}
    }, {"category": "Ll", "key": "1D17", "mappings": {"default": {"default": "untere Hälfte O"}}}, {
      "category": "Ll",
      "key": "1D1D",
      "mappings": {"default": {"default": "liegendes U"}}
    }, {"category": "Ll", "key": "1D1E", "mappings": {"default": {"default": "liegendes u Umlaut"}}}, {
      "category": "Ll",
      "key": "1D1F",
      "mappings": {"default": {"default": "liegendes m"}}
    }, {
      "category": "Ll",
      "key": "1D24",
      "mappings": {"default": {"default": "stimmhaft laryngaler Frikativ"}}
    }, {"category": "Ll", "key": "1D25", "mappings": {"default": {"default": "Ain"}}}, {
      "category": "Ll",
      "key": "1D6B",
      "mappings": {"default": {"default": "ue"}}
    }, {
      "category": "Ll",
      "key": "1D6C",
      "mappings": {"default": {"default": "b mit mittlerer Tilde"}}
    }, {
      "category": "Ll",
      "key": "1D6D",
      "mappings": {"default": {"default": "d mit mittlerer Tilde"}}
    }, {
      "category": "Ll",
      "key": "1D6E",
      "mappings": {"default": {"default": "f mit mittlerer Tilde"}}
    }, {
      "category": "Ll",
      "key": "1D6F",
      "mappings": {"default": {"default": "m mit mittlerer Tilde"}}
    }, {
      "category": "Ll",
      "key": "1D70",
      "mappings": {"default": {"default": "n mit mittlerer Tilde"}}
    }, {
      "category": "Ll",
      "key": "1D71",
      "mappings": {"default": {"default": "p mit mittlerer Tilde"}}
    }, {
      "category": "Ll",
      "key": "1D72",
      "mappings": {"default": {"default": "r mit mittlerer Tilde"}}
    }, {
      "category": "Ll",
      "key": "1D73",
      "mappings": {"default": {"default": "r mit Angelhaken und mittlerer Tilde"}}
    }, {
      "category": "Ll",
      "key": "1D74",
      "mappings": {"default": {"default": "s mit mittlerer Tilde"}}
    }, {
      "category": "Ll",
      "key": "1D75",
      "mappings": {"default": {"default": "t mit mittlerer Tilde"}}
    }, {
      "category": "Ll",
      "key": "1D76",
      "mappings": {"default": {"default": "z mit mittlerer Tilde"}}
    }, {"category": "Ll", "key": "1D77", "mappings": {"default": {"default": "gedreht G"}}}, {
      "category": "Ll",
      "key": "1D79",
      "mappings": {"default": {"default": "insular G"}}
    }, {
      "category": "Ll",
      "key": "1D7A",
      "mappings": {"default": {"default": "th mit Durchgestrichen"}}
    }, {"category": "Ll", "key": "1D80", "mappings": {"default": {"default": "b mit Palatalhaken"}}}, {
      "category": "Ll",
      "key": "1D81",
      "mappings": {"default": {"default": "d mit Palatalhaken"}}
    }, {"category": "Ll", "key": "1D82", "mappings": {"default": {"default": "f mit Palatalhaken"}}}, {
      "category": "Ll",
      "key": "1D83",
      "mappings": {"default": {"default": "g mit Palatalhaken"}}
    }, {"category": "Ll", "key": "1D84", "mappings": {"default": {"default": "k mit Palatalhaken"}}}, {
      "category": "Ll",
      "key": "1D85",
      "mappings": {"default": {"default": "l mit Palatalhaken"}}
    }, {"category": "Ll", "key": "1D86", "mappings": {"default": {"default": "m mit Palatalhaken"}}}, {
      "category": "Ll",
      "key": "1D87",
      "mappings": {"default": {"default": "n mit Palatalhaken"}}
    }, {"category": "Ll", "key": "1D88", "mappings": {"default": {"default": "p mit Palatalhaken"}}}, {
      "category": "Ll",
      "key": "1D89",
      "mappings": {"default": {"default": "r mit Palatalhaken"}}
    }, {"category": "Ll", "key": "1D8A", "mappings": {"default": {"default": "s mit Palatalhaken"}}}, {
      "category": "Ll",
      "key": "1D8B",
      "mappings": {"default": {"default": "esh mit Palatalhaken"}}
    }, {"category": "Ll", "key": "1D8C", "mappings": {"default": {"default": "v mit Palatalhaken"}}}, {
      "category": "Ll",
      "key": "1D8D",
      "mappings": {"default": {"default": "x mit Palatalhaken"}}
    }, {"category": "Ll", "key": "1D8E", "mappings": {"default": {"default": "z mit Palatalhaken"}}}, {
      "category": "Ll",
      "key": "1D8F",
      "mappings": {"default": {"default": "a mit Retroflexhaken"}}
    }, {
      "category": "Ll",
      "key": "1D90",
      "mappings": {"default": {"default": "alpha mit Retroflexhaken"}}
    }, {
      "category": "Ll",
      "key": "1D91",
      "mappings": {"default": {"default": "d mit Haken und Schwanz"}}
    }, {
      "category": "Ll",
      "key": "1D92",
      "mappings": {"default": {"default": "e mit Retroflexhaken"}}
    }, {
      "category": "Ll",
      "key": "1D93",
      "mappings": {"default": {"default": "offenes e mit Retroflexhaken"}}
    }, {
      "category": "Ll",
      "key": "1D94",
      "mappings": {"default": {"default": "umgedrehtes offenes e mit Retroflexhaken"}}
    }, {
      "category": "Ll",
      "key": "1D95",
      "mappings": {"default": {"default": "schwa mit Retroflexhaken"}}
    }, {
      "category": "Ll",
      "key": "1D96",
      "mappings": {"default": {"default": "i mit Retroflexhaken"}}
    }, {
      "category": "Ll",
      "key": "1D97",
      "mappings": {"default": {"default": "offenes o mit Retroflexhaken"}}
    }, {
      "category": "Ll",
      "key": "1D98",
      "mappings": {"default": {"default": "esh mit Retroflexhaken"}}
    }, {
      "category": "Ll",
      "key": "1D99",
      "mappings": {"default": {"default": "u mit Retroflexhaken"}}
    }, {
      "category": "Ll",
      "key": "1D9A",
      "mappings": {"default": {"default": "ezh mit Retroflexhaken"}}
    }, {"key": "0149", "mappings": {"default": {"default": "n mit Apostroph davor"}}, "category": "Ll"}, {
      "key": "014B",
      "mappings": {"default": {"default": "eng"}},
      "category": "Ll"
    }]
  ,
  "de/symbols/latin-lower-single-accent.js":
    [{"locale": "de"}, {
      "key": "00E0",
      "mappings": {"default": {"default": "a mit Grave"}},
      "category": "Ll"
    }, {"key": "00E1", "mappings": {"default": {"default": "a mit Aigu"}}, "category": "Ll"}, {
      "key": "00E2",
      "mappings": {"default": {"default": "a mit Zirkumflex"}},
      "category": "Ll"
    }, {"key": "00E3", "mappings": {"default": {"default": "a mit Tilde"}}, "category": "Ll"}, {
      "key": "00E4",
      "mappings": {"default": {"default": "a mit Diärese"}},
      "category": "Ll"
    }, {"key": "00E5", "mappings": {"default": {"default": "a mit Ring darüber"}}, "category": "Ll"}, {
      "key": "00E7",
      "mappings": {"default": {"default": "c mit Cedille"}},
      "category": "Ll"
    }, {"key": "00E8", "mappings": {"default": {"default": "e mit Grave"}}, "category": "Ll"}, {
      "key": "00E9",
      "mappings": {"default": {"default": "e mit Aigu"}},
      "category": "Ll"
    }, {"key": "00EA", "mappings": {"default": {"default": "e mit Zirkumflex"}}, "category": "Ll"}, {
      "key": "00EB",
      "mappings": {"default": {"default": "e mit Diärese"}},
      "category": "Ll"
    }, {"key": "00EC", "mappings": {"default": {"default": "i mit Grave"}}, "category": "Ll"}, {
      "key": "00ED",
      "mappings": {"default": {"default": "i mit Aigu"}},
      "category": "Ll"
    }, {"key": "00EE", "mappings": {"default": {"default": "i mit Zirkumflex"}}, "category": "Ll"}, {
      "key": "00EF",
      "mappings": {"default": {"default": "i mit Diärese"}},
      "category": "Ll"
    }, {"key": "00F1", "mappings": {"default": {"default": "n mit Tilde"}}, "category": "Ll"}, {
      "key": "00F2",
      "mappings": {"default": {"default": "o mit Grave"}},
      "category": "Ll"
    }, {"key": "00F3", "mappings": {"default": {"default": "o mit Aigu"}}, "category": "Ll"}, {
      "key": "00F4",
      "mappings": {"default": {"default": "o mit Zirkumflex"}},
      "category": "Ll"
    }, {"key": "00F5", "mappings": {"default": {"default": "o mit Tilde"}}, "category": "Ll"}, {
      "key": "00F6",
      "mappings": {"default": {"default": "o mit Diärese"}},
      "category": "Ll"
    }, {"key": "00F9", "mappings": {"default": {"default": "u mit Grave"}}, "category": "Ll"}, {
      "key": "00FA",
      "mappings": {"default": {"default": "u mit Aigu"}},
      "category": "Ll"
    }, {"key": "00FB", "mappings": {"default": {"default": "u mit Zirkumflex"}}, "category": "Ll"}, {
      "key": "00FC",
      "mappings": {"default": {"default": "u mit Diärese"}},
      "category": "Ll"
    }, {"key": "00FD", "mappings": {"default": {"default": "y mit Aigu"}}, "category": "Ll"}, {
      "key": "00FF",
      "mappings": {"default": {"default": "y mit Diärese"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0101", "mappings": {"default": {"default": "a mit Makron"}}}, {
      "category": "Ll",
      "key": "0103",
      "mappings": {"default": {"default": "a mit Breve"}}
    }, {"category": "Ll", "key": "0105", "mappings": {"default": {"default": "a mit Ogonek"}}}, {
      "key": "0107",
      "mappings": {"default": {"default": "c mit Aigu"}},
      "category": "Ll"
    }, {"key": "0109", "mappings": {"default": {"default": "c mit Zirkumflex"}}, "category": "Ll"}, {
      "key": "010B",
      "mappings": {"default": {"default": "c mit Punkt darüber"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "010D", "mappings": {"default": {"default": "c mit Caron"}}}, {
      "category": "Ll",
      "key": "010F",
      "mappings": {"default": {"default": "d mit Caron"}}
    }, {"category": "Ll", "key": "0113", "mappings": {"default": {"default": "e mit Makron"}}}, {
      "category": "Ll",
      "key": "0115",
      "mappings": {"default": {"default": "e mit Breve"}}
    }, {
      "key": "0117",
      "mappings": {"default": {"default": "e mit Punkt darüber"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0119", "mappings": {"default": {"default": "e mit Ogonek"}}}, {
      "category": "Ll",
      "key": "011B",
      "mappings": {"default": {"default": "e mit Caron"}}
    }, {"key": "011D", "mappings": {"default": {"default": "g mit Zirkumflex"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "011F",
      "mappings": {"default": {"default": "g mit Breve"}}
    }, {
      "key": "0121",
      "mappings": {"default": {"default": "g mit Punkt darüber"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0123", "mappings": {"default": {"default": "g mit Cedilla"}}}, {
      "key": "0125",
      "mappings": {"default": {"default": "h mit Zirkumflex"}},
      "category": "Ll"
    }, {"key": "0129", "mappings": {"default": {"default": "i mit Tilde"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "012B",
      "mappings": {"default": {"default": "i mit Makron"}}
    }, {"category": "Ll", "key": "012D", "mappings": {"default": {"default": "i mit Breve"}}}, {
      "category": "Ll",
      "key": "012F",
      "mappings": {"default": {"default": "i mit Ogonek"}}
    }, {"key": "0131", "mappings": {"default": {"default": "i ohne punkt"}}, "category": "Ll"}, {
      "key": "0135",
      "mappings": {"default": {"default": "j mit Zirkumflex"}},
      "category": "Ll"
    }, {"key": "0137", "mappings": {"default": {"default": "k mit Cedille"}}, "category": "Ll"}, {
      "key": "013A",
      "mappings": {"default": {"default": "l mit Aigu"}},
      "category": "Ll"
    }, {"key": "013C", "mappings": {"default": {"default": "l mit Cedille"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "013E",
      "mappings": {"default": {"default": "l mit Caron"}}
    }, {
      "key": "0140",
      "mappings": {"default": {"default": "l mit Punkt in der Mitte"}},
      "category": "Ll"
    }, {"key": "0144", "mappings": {"default": {"default": "n mit Aigu"}}, "category": "Ll"}, {
      "key": "0146",
      "mappings": {"default": {"default": "n mit Cedille"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0148", "mappings": {"default": {"default": "n mit Caron"}}}, {
      "category": "Ll",
      "key": "014D",
      "mappings": {"default": {"default": "o mit Makron"}}
    }, {"category": "Ll", "key": "014F", "mappings": {"default": {"default": "o mit Breve"}}}, {
      "key": "0151",
      "mappings": {"default": {"default": "o mit doppeltem Aigu"}},
      "category": "Ll"
    }, {"key": "0155", "mappings": {"default": {"default": "r mit Aigu"}}, "category": "Ll"}, {
      "key": "0157",
      "mappings": {"default": {"default": "r mit Cedille"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0159", "mappings": {"default": {"default": "r mit Caron"}}}, {
      "key": "015B",
      "mappings": {"default": {"default": "s mit Aigu"}},
      "category": "Ll"
    }, {"key": "015D", "mappings": {"default": {"default": "s mit Zirkumflex"}}, "category": "Ll"}, {
      "key": "015F",
      "mappings": {"default": {"default": "s mit Cedille"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0161", "mappings": {"default": {"default": "s mit Caron"}}}, {
      "key": "0163",
      "mappings": {"default": {"default": "t mit Cedille"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0165", "mappings": {"default": {"default": "t mit Caron"}}}, {
      "key": "0169",
      "mappings": {"default": {"default": "u mit Tilde"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "016B", "mappings": {"default": {"default": "u mit Makron"}}}, {
      "category": "Ll",
      "key": "016D",
      "mappings": {"default": {"default": "u mit Breve"}}
    }, {"key": "016F", "mappings": {"default": {"default": "u mit Ring darüber"}}, "category": "Ll"}, {
      "key": "0171",
      "mappings": {"default": {"default": "u mit doppeltem Aigu"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0173", "mappings": {"default": {"default": "u mit Ogonek"}}}, {
      "key": "0175",
      "mappings": {"default": {"default": "w mit Zirkumflex"}},
      "category": "Ll"
    }, {"key": "0177", "mappings": {"default": {"default": "y mit Zirkumflex"}}, "category": "Ll"}, {
      "key": "017A",
      "mappings": {"default": {"default": "z mit Aigu"}},
      "category": "Ll"
    }, {
      "key": "017C",
      "mappings": {"default": {"default": "z mit Punkt darüber"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "017E", "mappings": {"default": {"default": "z mit Caron"}}}, {
      "category": "Ll",
      "key": "01CE",
      "mappings": {"default": {"default": "a mit Caron"}}
    }, {"category": "Ll", "key": "01D0", "mappings": {"default": {"default": "i mit Caron"}}}, {
      "category": "Ll",
      "key": "01D2",
      "mappings": {"default": {"default": "o mit Caron"}}
    }, {"category": "Ll", "key": "01D4", "mappings": {"default": {"default": "u mit Caron"}}}, {
      "category": "Ll",
      "key": "01E7",
      "mappings": {"default": {"default": "g mit Caron"}}
    }, {"category": "Ll", "key": "01E9", "mappings": {"default": {"default": "k mit Caron"}}}, {
      "category": "Ll",
      "key": "01EB",
      "mappings": {"default": {"default": "o mit Ogonek"}}
    }, {"category": "Ll", "key": "01F0", "mappings": {"default": {"default": "j mit Caron"}}}, {
      "key": "01F5",
      "mappings": {"default": {"default": "g mit Aigu"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "01F9", "mappings": {"default": {"default": "n mit Grab"}}}, {
      "category": "Ll",
      "key": "0201",
      "mappings": {"default": {"default": "a mit Doppelgrab"}}
    }, {
      "category": "Ll",
      "key": "0203",
      "mappings": {"default": {"default": "a mit umgekehrter Breve"}}
    }, {"category": "Ll", "key": "0205", "mappings": {"default": {"default": "e mit Doppelgrab"}}}, {
      "category": "Ll",
      "key": "0207",
      "mappings": {"default": {"default": "e mit umgekehrter Breve"}}
    }, {"category": "Ll", "key": "0209", "mappings": {"default": {"default": "i mit Doppelgrab"}}}, {
      "category": "Ll",
      "key": "020B",
      "mappings": {"default": {"default": "i mit umgekehrter Breve"}}
    }, {"category": "Ll", "key": "020D", "mappings": {"default": {"default": "o mit Doppelgrab"}}}, {
      "category": "Ll",
      "key": "020F",
      "mappings": {"default": {"default": "o mit umgekehrter Breve"}}
    }, {"category": "Ll", "key": "0211", "mappings": {"default": {"default": "r mit Doppelgrab"}}}, {
      "category": "Ll",
      "key": "0213",
      "mappings": {"default": {"default": "r mit umgekehrter Breve"}}
    }, {"category": "Ll", "key": "0215", "mappings": {"default": {"default": "u mit Doppelgrab"}}}, {
      "category": "Ll",
      "key": "0217",
      "mappings": {"default": {"default": "u mit umgekehrter Breve"}}
    }, {
      "category": "Ll",
      "key": "0219",
      "mappings": {"default": {"default": "s mit untenstehendem Komma"}}
    }, {
      "category": "Ll",
      "key": "021B",
      "mappings": {"default": {"default": "t mit untenstehendem Komma"}}
    }, {"category": "Ll", "key": "021F", "mappings": {"default": {"default": "h mit Caron"}}}, {
      "category": "Ll",
      "key": "0227",
      "mappings": {"default": {"default": "a mit Punkt oben"}}
    }, {"category": "Ll", "key": "0229", "mappings": {"default": {"default": "e mit Cedilla"}}}, {
      "category": "Ll",
      "key": "022F",
      "mappings": {"default": {"default": "o mit Punkt oben"}}
    }, {"category": "Ll", "key": "0233", "mappings": {"default": {"default": "y mit Makron"}}}, {
      "category": "Ll",
      "key": "0237",
      "mappings": {"default": {"default": "dotless J"}}
    }, {"category": "Ll", "key": "1E01", "mappings": {"default": {"default": "a mit Ring unten"}}}, {
      "category": "Ll",
      "key": "1E03",
      "mappings": {"default": {"default": "b mit Punkt oben"}}
    }, {
      "category": "Ll",
      "key": "1E05",
      "mappings": {"default": {"default": "b mit untenstehendem Punkt"}}
    }, {
      "category": "Ll",
      "key": "1E07",
      "mappings": {"default": {"default": "b mit Zeile darunter"}}
    }, {"category": "Ll", "key": "1E0B", "mappings": {"default": {"default": "d mit Punkt oben"}}}, {
      "category": "Ll",
      "key": "1E0D",
      "mappings": {"default": {"default": "d mit untenstehendem Punkt"}}
    }, {
      "category": "Ll",
      "key": "1E0F",
      "mappings": {"default": {"default": "d mit Zeile darunter"}}
    }, {"category": "Ll", "key": "1E11", "mappings": {"default": {"default": "d mit Cedilla"}}}, {
      "category": "Ll",
      "key": "1E13",
      "mappings": {"default": {"default": "d mit darunterem Zirkumflex"}}
    }, {
      "category": "Ll",
      "key": "1E19",
      "mappings": {"default": {"default": "e mit darunterem Zirkumflex"}}
    }, {
      "category": "Ll",
      "key": "1E1B",
      "mappings": {"default": {"default": "e mit darunterliegender Tilde"}}
    }, {"category": "Ll", "key": "1E1F", "mappings": {"default": {"default": "f mit Punkt oben"}}}, {
      "category": "Ll",
      "key": "1E21",
      "mappings": {"default": {"default": "g mit Makron"}}
    }, {"category": "Ll", "key": "1E23", "mappings": {"default": {"default": "h mit Punkt oben"}}}, {
      "category": "Ll",
      "key": "1E25",
      "mappings": {"default": {"default": "h mit untenstehendem Punkt"}}
    }, {"category": "Ll", "key": "1E27", "mappings": {"default": {"default": "h mit Diärese"}}}, {
      "category": "Ll",
      "key": "1E29",
      "mappings": {"default": {"default": "h mit Cedilla"}}
    }, {"category": "Ll", "key": "1E2B", "mappings": {"default": {"default": "h mit Breve unten"}}}, {
      "category": "Ll",
      "key": "1E2D",
      "mappings": {"default": {"default": "i mit darunterliegender Tilde"}}
    }, {"category": "Ll", "key": "1E31", "mappings": {"default": {"default": "k mit Akut"}}}, {
      "category": "Ll",
      "key": "1E33",
      "mappings": {"default": {"default": "k mit untenstehendem Punkt"}}
    }, {
      "category": "Ll",
      "key": "1E35",
      "mappings": {"default": {"default": "k mit Zeile darunter"}}
    }, {"category": "Ll", "key": "1E37", "mappings": {"default": {"default": "l mit Punkt unten"}}}, {
      "category": "Ll",
      "key": "1E3B",
      "mappings": {"default": {"default": "l mit Zeile darunter"}}
    }, {
      "category": "Ll",
      "key": "1E3D",
      "mappings": {"default": {"default": "l mit darunterem Zirkumflex"}}
    }, {"category": "Ll", "key": "1E3F", "mappings": {"default": {"default": "m mit Akut"}}}, {
      "category": "Ll",
      "key": "1E41",
      "mappings": {"default": {"default": "m mit Punkt oben"}}
    }, {
      "category": "Ll",
      "key": "1E43",
      "mappings": {"default": {"default": "m mit untenstehendem Punkt"}}
    }, {"category": "Ll", "key": "1E45", "mappings": {"default": {"default": "n mit Punkt oben"}}}, {
      "category": "Ll",
      "key": "1E47",
      "mappings": {"default": {"default": "n mit untenstehendem Punkt"}}
    }, {
      "category": "Ll",
      "key": "1E49",
      "mappings": {"default": {"default": "n mit Zeile darunter"}}
    }, {
      "category": "Ll",
      "key": "1E4B",
      "mappings": {"default": {"default": "n mit darunterem Zirkumflex"}}
    }, {"category": "Ll", "key": "1E55", "mappings": {"default": {"default": "p mit Akut"}}}, {
      "category": "Ll",
      "key": "1E57",
      "mappings": {"default": {"default": "p mit Punkt oben"}}
    }, {"category": "Ll", "key": "1E59", "mappings": {"default": {"default": "r mit Punkt oben"}}}, {
      "category": "Ll",
      "key": "1E5B",
      "mappings": {"default": {"default": "r mit untenstehendem Punkt"}}
    }, {
      "category": "Ll",
      "key": "1E5F",
      "mappings": {"default": {"default": "r mit Zeile darunter"}}
    }, {"category": "Ll", "key": "1E61", "mappings": {"default": {"default": "s mit Punkt oben"}}}, {
      "category": "Ll",
      "key": "1E63",
      "mappings": {"default": {"default": "s mit untenstehendem Punkt"}}
    }, {"category": "Ll", "key": "1E6B", "mappings": {"default": {"default": "t mit Punkt oben"}}}, {
      "category": "Ll",
      "key": "1E6D",
      "mappings": {"default": {"default": "t mit untenstehendem Punkt"}}
    }, {
      "category": "Ll",
      "key": "1E6F",
      "mappings": {"default": {"default": "t mit Zeile darunter"}}
    }, {
      "category": "Ll",
      "key": "1E71",
      "mappings": {"default": {"default": "t mit darunterem Zirkumflex"}}
    }, {
      "category": "Ll",
      "key": "1E73",
      "mappings": {"default": {"default": "u mit untenstehender Diärese"}}
    }, {
      "category": "Ll",
      "key": "1E75",
      "mappings": {"default": {"default": "u mit darunterliegender Tilde"}}
    }, {
      "category": "Ll",
      "key": "1E77",
      "mappings": {"default": {"default": "u mit darunterem Zirkumflex"}}
    }, {"category": "Ll", "key": "1E7D", "mappings": {"default": {"default": "v mit Tilde"}}}, {
      "category": "Ll",
      "key": "1E7F",
      "mappings": {"default": {"default": "v mit untenstehendem Punkt"}}
    }, {"category": "Ll", "key": "1E81", "mappings": {"default": {"default": "w mit Grab"}}}, {
      "category": "Ll",
      "key": "1E83",
      "mappings": {"default": {"default": "w mit Akut"}}
    }, {"category": "Ll", "key": "1E85", "mappings": {"default": {"default": "w mit Diärese"}}}, {
      "category": "Ll",
      "key": "1E87",
      "mappings": {"default": {"default": "w mit Punkt oben"}}
    }, {"category": "Ll", "key": "1E89", "mappings": {"default": {"default": "w mit Punkt unten"}}}, {
      "category": "Ll",
      "key": "1E8B",
      "mappings": {"default": {"default": "x mit Punkt oben"}}
    }, {"category": "Ll", "key": "1E8D", "mappings": {"default": {"default": "x mit Diärese"}}}, {
      "category": "Ll",
      "key": "1E8F",
      "mappings": {"default": {"default": "y mit Punkt oben"}}
    }, {"category": "Ll", "key": "1E91", "mappings": {"default": {"default": "z mit Zirkumflex"}}}, {
      "category": "Ll",
      "key": "1E93",
      "mappings": {"default": {"default": "z mit untenstehendem Punkt"}}
    }, {
      "category": "Ll",
      "key": "1E95",
      "mappings": {"default": {"default": "z mit Zeile darunter"}}
    }, {
      "category": "Ll",
      "key": "1E96",
      "mappings": {"default": {"default": "h mit Zeile darunter"}}
    }, {"category": "Ll", "key": "1E97", "mappings": {"default": {"default": "t mit Diärese"}}}, {
      "category": "Ll",
      "key": "1E98",
      "mappings": {"default": {"default": "w mit Ring oben"}}
    }, {"category": "Ll", "key": "1E99", "mappings": {"default": {"default": "y mit Ring oben"}}}, {
      "category": "Ll",
      "key": "1E9A",
      "mappings": {"default": {"default": "a mit rechtem Halbring"}}
    }, {"category": "Ll", "key": "1EA1", "mappings": {"default": {"default": "a mit Punkt unten"}}}, {
      "category": "Ll",
      "key": "1EA3",
      "mappings": {"default": {"default": "a mit Haken oben"}}
    }, {
      "category": "Ll",
      "key": "1EB9",
      "mappings": {"default": {"default": "e mit untenstehendem Punkt"}}
    }, {"category": "Ll", "key": "1EBB", "mappings": {"default": {"default": "e mit Haken oben"}}}, {
      "category": "Ll",
      "key": "1EBD",
      "mappings": {"default": {"default": "e mit Tilde"}}
    }, {"category": "Ll", "key": "1EC9", "mappings": {"default": {"default": "i mit Haken oben"}}}, {
      "category": "Ll",
      "key": "1ECB",
      "mappings": {"default": {"default": "i mit untenstehendem Punkt"}}
    }, {
      "category": "Ll",
      "key": "1ECD",
      "mappings": {"default": {"default": "o mit untenstehendem Punkt"}}
    }, {"category": "Ll", "key": "1ECF", "mappings": {"default": {"default": "o mit Haken oben"}}}, {
      "category": "Ll",
      "key": "1EE5",
      "mappings": {"default": {"default": "u mit untenstehendem Punkt"}}
    }, {"category": "Ll", "key": "1EE7", "mappings": {"default": {"default": "u mit Haken oben"}}}, {
      "category": "Ll",
      "key": "1EF3",
      "mappings": {"default": {"default": "y mit Grab"}}
    }, {
      "category": "Ll",
      "key": "1EF5",
      "mappings": {"default": {"default": "y mit untenstehendem Punkt"}}
    }, {"category": "Ll", "key": "1EF7", "mappings": {"default": {"default": "y mit Haken oben"}}}, {
      "category": "Ll",
      "key": "1EF9",
      "mappings": {"default": {"default": "y mit Tilde"}}
    }]
  ,
  "de/symbols/latin-rest.js":
    [{"locale": "de"}, {
      "category": "Ll",
      "key": "210E",
      "mappings": {"default": {"default": "Planck-Konstante"}}
    }, {"category": "Mn", "key": "0363", "mappings": {"default": {"default": "kombinierendes a"}}}, {
      "category": "Mn",
      "key": "0364",
      "mappings": {"default": {"default": "kombinierendes e"}}
    }, {"category": "Mn", "key": "0365", "mappings": {"default": {"default": "kombinierendes i"}}}, {
      "category": "Mn",
      "key": "0366",
      "mappings": {"default": {"default": "kombinierendes o"}}
    }, {"category": "Mn", "key": "0367", "mappings": {"default": {"default": "kombinierendes u"}}}, {
      "category": "Mn",
      "key": "0368",
      "mappings": {"default": {"default": "kombinierendes c"}}
    }, {"category": "Mn", "key": "0369", "mappings": {"default": {"default": "kombinierendes d"}}}, {
      "category": "Mn",
      "key": "036A",
      "mappings": {"default": {"default": "kombinierendes h"}}
    }, {"category": "Mn", "key": "036B", "mappings": {"default": {"default": "kombinierendes m"}}}, {
      "category": "Mn",
      "key": "036C",
      "mappings": {"default": {"default": "kombinierendes r"}}
    }, {"category": "Mn", "key": "036D", "mappings": {"default": {"default": "kombinierendes t"}}}, {
      "category": "Mn",
      "key": "036E",
      "mappings": {"default": {"default": "kombinierendes v"}}
    }, {"category": "Mn", "key": "036F", "mappings": {"default": {"default": "kombinierendes x"}}}, {
      "category": "Lm",
      "key": "1D62",
      "mappings": {"default": {"default": "tiefgestelltes i"}}
    }, {"category": "Lm", "key": "1D63", "mappings": {"default": {"default": "tiefgestelltes r"}}}, {
      "category": "Lm",
      "key": "1D64",
      "mappings": {"default": {"default": "tiefgestelltes u"}}
    }, {"category": "Lm", "key": "1D65", "mappings": {"default": {"default": "tiefgestelltes v"}}}, {
      "category": "Mn",
      "key": "1DCA",
      "mappings": {"default": {"default": "tiefgestelltes r"}}
    }, {
      "category": "Mn",
      "key": "1DD3",
      "mappings": {"default": {"default": "kombinierendes oberes w"}}
    }, {"category": "Mn", "key": "1DD4", "mappings": {"default": {"default": "kombinierendes ae"}}}, {
      "category": "Mn",
      "key": "1DD5",
      "mappings": {"default": {"default": "kombinierendes ao"}}
    }, {"category": "Mn", "key": "1DD6", "mappings": {"default": {"default": "kombinierendes av"}}}, {
      "category": "Mn",
      "key": "1DD7",
      "mappings": {"default": {"default": "kombinierendes c mit Cedilla"}}
    }, {
      "category": "Mn",
      "key": "1DD8",
      "mappings": {"default": {"default": "kombinierendes insulares d"}}
    }, {"category": "Mn", "key": "1DD9", "mappings": {"default": {"default": "kombinierendes eth"}}}, {
      "category": "Mn",
      "key": "1DDA",
      "mappings": {"default": {"default": "kombinierendes g"}}
    }, {
      "category": "Mn",
      "key": "1DDB",
      "mappings": {"default": {"default": "kombinierendes Kapitälchen g"}}
    }, {"category": "Mn", "key": "1DDC", "mappings": {"default": {"default": "kombinierendes k"}}}, {
      "category": "Mn",
      "key": "1DDD",
      "mappings": {"default": {"default": "kombinierendes l"}}
    }, {
      "category": "Mn",
      "key": "1DDE",
      "mappings": {"default": {"default": "kombinierendes Kapitälchen L"}}
    }, {
      "category": "Mn",
      "key": "1DDF",
      "mappings": {"default": {"default": "kombinierendes Kapitälchen M"}}
    }, {"category": "Mn", "key": "1DE0", "mappings": {"default": {"default": "kombinierendes n"}}}, {
      "category": "Mn",
      "key": "1DE1",
      "mappings": {"default": {"default": "kombinierendes Kapitälchen N"}}
    }, {
      "category": "Mn",
      "key": "1DE2",
      "mappings": {"default": {"default": "kombinierendes lateinisches Kapitälchen R"}}
    }, {
      "category": "Mn",
      "key": "1DE3",
      "mappings": {"default": {"default": "kombinierendes r rotunda"}}
    }, {"category": "Mn", "key": "1DE4", "mappings": {"default": {"default": "kombinierendes s"}}}, {
      "category": "Mn",
      "key": "1DE5",
      "mappings": {"default": {"default": "kombinierendes langes s"}}
    }, {"category": "Mn", "key": "1DE6", "mappings": {"default": {"default": "kombinierendes z"}}}, {
      "category": "Lm",
      "key": "2071",
      "mappings": {"default": {"default": "hochgestelltes i"}}
    }, {"category": "Lm", "key": "207F", "mappings": {"default": {"default": "hochgestelltes n"}}}, {
      "category": "Lm",
      "key": "2090",
      "mappings": {"default": {"default": "tiefgestelltes a"}}
    }, {"category": "Lm", "key": "2091", "mappings": {"default": {"default": "tiefgestelltes e"}}}, {
      "category": "Lm",
      "key": "2092",
      "mappings": {"default": {"default": "tiefgestelltes o"}}
    }, {"category": "Lm", "key": "2093", "mappings": {"default": {"default": "tiefgestelltes x"}}}, {
      "category": "Lm",
      "key": "2094",
      "mappings": {"default": {"default": "tiefgestelltes schwa"}}
    }, {"category": "Lm", "key": "2095", "mappings": {"default": {"default": "tiefgestelltes h"}}}, {
      "category": "Lm",
      "key": "2096",
      "mappings": {"default": {"default": "tiefgestelltes k"}}
    }, {"category": "Lm", "key": "2097", "mappings": {"default": {"default": "tiefgestelltes l"}}}, {
      "category": "Lm",
      "key": "2098",
      "mappings": {"default": {"default": "tiefgestelltes m"}}
    }, {"category": "Lm", "key": "2099", "mappings": {"default": {"default": "tiefgestelltes n"}}}, {
      "category": "Lm",
      "key": "209A",
      "mappings": {"default": {"default": "tiefgestelltes p"}}
    }, {"category": "Lm", "key": "209B", "mappings": {"default": {"default": "tiefgestelltes s"}}}, {
      "category": "Lm",
      "key": "209C",
      "mappings": {"default": {"default": "tiefgestelltes t"}}
    }, {"category": "Lm", "key": "2C7C", "mappings": {"default": {"default": "tiefgestelltes j"}}}, {
      "category": "So",
      "key": "1F12A",
      "mappings": {"default": {"default": "groß S in stumpfen Klammern"}}
    }, {
      "category": "So",
      "key": "1F12B",
      "mappings": {"default": {"default": "eingekreistes kursives groß C"}}
    }, {
      "category": "So",
      "key": "1F12C",
      "mappings": {"default": {"default": "eingekreistes kursives groß R"}}
    }, {
      "category": "So",
      "key": "1F18A",
      "mappings": {"default": {"default": "durchgestrichenes P auf schwarzen Quadrat"}}
    }]
  ,
  "de/symbols/latin-upper-double-accent.js":
    [{"locale": "de"}, {
      "category": "Lu",
      "key": "01D5",
      "mappings": {"default": {"default": "groß U mit Diärese und Makron"}}
    }, {
      "category": "Lu",
      "key": "01D7",
      "mappings": {"default": {"default": "groß U mit Diärese und Akut"}}
    }, {
      "category": "Lu",
      "key": "01D9",
      "mappings": {"default": {"default": "groß U mit Diärese und Caron"}}
    }, {
      "category": "Lu",
      "key": "01DB",
      "mappings": {"default": {"default": "groß U mit Diärese und Grab"}}
    }, {
      "category": "Lu",
      "key": "01DE",
      "mappings": {"default": {"default": "groß A mit Diärese und Makron"}}
    }, {
      "category": "Lu",
      "key": "01E0",
      "mappings": {"default": {"default": "groß A mit Dot Above und Makron"}}
    }, {
      "category": "Lu",
      "key": "01EC",
      "mappings": {"default": {"default": "groß O mit Ogonek und Makron"}}
    }, {
      "category": "Lu",
      "key": "01FA",
      "mappings": {"default": {"default": "groß A mit Ring oben und akut"}}
    }, {
      "category": "Lu",
      "key": "022A",
      "mappings": {"default": {"default": "groß O mit Diärese und Makron"}}
    }, {
      "category": "Lu",
      "key": "022C",
      "mappings": {"default": {"default": "groß O mit Tilde und Makron"}}
    }, {
      "category": "Lu",
      "key": "0230",
      "mappings": {"default": {"default": "groß O mit Punkt oben und Makron"}}
    }, {
      "category": "Lu",
      "key": "1E08",
      "mappings": {"default": {"default": "groß C mit Cedilla und Akut"}}
    }, {
      "category": "Lu",
      "key": "1E14",
      "mappings": {"default": {"default": "groß E mit Makron und Grab"}}
    }, {
      "category": "Lu",
      "key": "1E16",
      "mappings": {"default": {"default": "groß E mit Makron und Akut"}}
    }, {
      "category": "Lu",
      "key": "1E1C",
      "mappings": {"default": {"default": "groß E mit Cedilla und Breve"}}
    }, {
      "category": "Lu",
      "key": "1E2E",
      "mappings": {"default": {"default": "groß I mit Diärese und Akut"}}
    }, {
      "category": "Lu",
      "key": "1E38",
      "mappings": {"default": {"default": "groß L mit Dot Below und Makron"}}
    }, {
      "category": "Lu",
      "key": "1E4C",
      "mappings": {"default": {"default": "groß O mit Tilde und Akut"}}
    }, {
      "category": "Lu",
      "key": "1E4E",
      "mappings": {"default": {"default": "groß O mit Tilde und Diärese"}}
    }, {
      "category": "Lu",
      "key": "1E50",
      "mappings": {"default": {"default": "groß O mit Makron und Grab"}}
    }, {
      "category": "Lu",
      "key": "1E52",
      "mappings": {"default": {"default": "groß O mit Makron und Akut"}}
    }, {
      "category": "Lu",
      "key": "1E5C",
      "mappings": {"default": {"default": "groß R mit Punkt unten und Makron"}}
    }, {
      "category": "Lu",
      "key": "1E64",
      "mappings": {"default": {"default": "groß S mit Akute und Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E66",
      "mappings": {"default": {"default": "groß S mit Caron und Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E68",
      "mappings": {"default": {"default": "groß S mit untenstehendem Punkt und oben stehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1E78",
      "mappings": {"default": {"default": "groß U mit Tilde und Akut"}}
    }, {
      "category": "Lu",
      "key": "1E7A",
      "mappings": {"default": {"default": "groß U mit Makron und Diärese"}}
    }, {
      "category": "Lu",
      "key": "1EA4",
      "mappings": {"default": {"default": "groß A mit Zirkumflex und Akut"}}
    }, {
      "category": "Lu",
      "key": "1EA6",
      "mappings": {"default": {"default": "groß A mit Zirkumflex und Grab"}}
    }, {
      "category": "Lu",
      "key": "1EA8",
      "mappings": {"default": {"default": "groß A mit Zirkumflex und Haken oben"}}
    }, {
      "category": "Lu",
      "key": "1EAA",
      "mappings": {"default": {"default": "groß A mit Circumflex und Tilde"}}
    }, {
      "category": "Lu",
      "key": "1EAC",
      "mappings": {"default": {"default": "groß A mit Zirkumflex und Punkt darunter"}}
    }, {
      "category": "Lu",
      "key": "1EAE",
      "mappings": {"default": {"default": "groß A mit Breve und Akut"}}
    }, {
      "category": "Lu",
      "key": "1EB0",
      "mappings": {"default": {"default": "groß A mit Breve und Grab"}}
    }, {
      "category": "Lu",
      "key": "1EB2",
      "mappings": {"default": {"default": "groß A mit Breve und Hook oben"}}
    }, {
      "category": "Lu",
      "key": "1EB4",
      "mappings": {"default": {"default": "groß A mit Breve und Tilde"}}
    }, {
      "category": "Lu",
      "key": "1EB6",
      "mappings": {"default": {"default": "groß A mit Breve und Dot Below"}}
    }, {
      "category": "Lu",
      "key": "1EBE",
      "mappings": {"default": {"default": "groß E mit Zirkumflex und Akut"}}
    }, {
      "category": "Lu",
      "key": "1EC0",
      "mappings": {"default": {"default": "groß E mit Zirkumflex und Grab"}}
    }, {
      "category": "Lu",
      "key": "1EC2",
      "mappings": {"default": {"default": "groß E mit Zirkumflex und Haken oben"}}
    }, {
      "category": "Lu",
      "key": "1EC4",
      "mappings": {"default": {"default": "groß E mit Zirkumflex und Tilde"}}
    }, {
      "category": "Lu",
      "key": "1EC6",
      "mappings": {"default": {"default": "groß E mit Zirkumflex und Punkt darunter"}}
    }, {
      "category": "Lu",
      "key": "1ED0",
      "mappings": {"default": {"default": "groß O mit Circumflex und Akut"}}
    }, {
      "category": "Lu",
      "key": "1ED2",
      "mappings": {"default": {"default": "groß O mit Zirkumflex und Grab"}}
    }, {
      "category": "Lu",
      "key": "1ED4",
      "mappings": {"default": {"default": "groß O mit Zirkumflex und Haken oben"}}
    }, {
      "category": "Lu",
      "key": "1ED6",
      "mappings": {"default": {"default": "groß O mit Zirkumflex und Tilde"}}
    }, {
      "category": "Lu",
      "key": "1ED8",
      "mappings": {"default": {"default": "groß O mit Zirkumflex und Punkt darunter"}}
    }, {
      "category": "Lu",
      "key": "1EDA",
      "mappings": {"default": {"default": "groß O mit Horn und Akut"}}
    }, {
      "category": "Lu",
      "key": "1EDC",
      "mappings": {"default": {"default": "groß O mit Horn und Grab"}}
    }, {
      "category": "Lu",
      "key": "1EDE",
      "mappings": {"default": {"default": "groß O mit Horn und Haken oben"}}
    }, {
      "category": "Lu",
      "key": "1EE0",
      "mappings": {"default": {"default": "groß O mit Horn und Tilde"}}
    }, {
      "category": "Lu",
      "key": "1EE2",
      "mappings": {"default": {"default": "groß O mit Horn und Punkt"}}
    }, {
      "category": "Lu",
      "key": "1EE8",
      "mappings": {"default": {"default": "groß U mit Horn und Akut"}}
    }, {
      "category": "Lu",
      "key": "1EEA",
      "mappings": {"default": {"default": "groß U mit Horn und Grab"}}
    }, {
      "category": "Lu",
      "key": "1EEC",
      "mappings": {"default": {"default": "groß U mit Horn und Haken oben"}}
    }, {
      "category": "Lu",
      "key": "1EEE",
      "mappings": {"default": {"default": "groß U mit Horn und Tilde"}}
    }, {"category": "Lu", "key": "1EF0", "mappings": {"default": {"default": "groß U mit Horn und Punkt"}}}]
  ,
  "de/symbols/latin-upper-single-accent.js":
    [{"locale": "de"}, {
      "key": "00C0",
      "mappings": {"default": {"default": "groß A mit Gravis"}},
      "category": "Lu"
    }, {"key": "00C1", "mappings": {"default": {"default": "groß A mit Aigu"}}, "category": "Lu"}, {
      "key": "00C2",
      "mappings": {"default": {"default": "groß A mit Zirkumflex"}},
      "category": "Lu"
    }, {"key": "00C3", "mappings": {"default": {"default": "groß A mit Tilde"}}, "category": "Lu"}, {
      "key": "00C4",
      "mappings": {"default": {"default": "groß A mit Diärese"}},
      "category": "Lu"
    }, {
      "key": "00C5",
      "mappings": {"default": {"default": "groß A mit Ring darüber"}},
      "category": "Lu"
    }, {"key": "00C7", "mappings": {"default": {"default": "groß C mit Cedille"}}, "category": "Lu"}, {
      "key": "00C8",
      "mappings": {"default": {"default": "groß E mit Gravis"}},
      "category": "Lu"
    }, {"key": "00C9", "mappings": {"default": {"default": "groß E mit Aigu"}}, "category": "Lu"}, {
      "key": "00CA",
      "mappings": {"default": {"default": "groß E mit Zirkumflex"}},
      "category": "Lu"
    }, {"key": "00CB", "mappings": {"default": {"default": "groß E mit Diärese"}}, "category": "Lu"}, {
      "key": "00CC",
      "mappings": {"default": {"default": "groß I mit Gravis"}},
      "category": "Lu"
    }, {"key": "00CD", "mappings": {"default": {"default": "groß I mit Aigu"}}, "category": "Lu"}, {
      "key": "00CE",
      "mappings": {"default": {"default": "groß I mit Zirkumflex"}},
      "category": "Lu"
    }, {"key": "00CF", "mappings": {"default": {"default": "groß I mit Diärese"}}, "category": "Lu"}, {
      "key": "00D1",
      "mappings": {"default": {"default": "großes n mit Tilde"}},
      "category": "Lu"
    }, {"key": "00D2", "mappings": {"default": {"default": "großes o mit Gravis"}}, "category": "Lu"}, {
      "key": "00D3",
      "mappings": {"default": {"default": "groß O mit Aigu"}},
      "category": "Lu"
    }, {"key": "00D4", "mappings": {"default": {"default": "groß O mit Zirkumflex"}}, "category": "Lu"}, {
      "key": "00D5",
      "mappings": {"default": {"default": "groß O mit Tilde"}},
      "category": "Lu"
    }, {"key": "00D6", "mappings": {"default": {"default": "groß O mit Diärese"}}, "category": "Lu"}, {
      "key": "00D9",
      "mappings": {"default": {"default": "groß U mit Gravis"}},
      "category": "Lu"
    }, {"key": "00DA", "mappings": {"default": {"default": "groß U mit Aigu"}}, "category": "Lu"}, {
      "key": "00DB",
      "mappings": {"default": {"default": "groß U mit Zirkumflex"}},
      "category": "Lu"
    }, {"key": "00DC", "mappings": {"default": {"default": "groß U mit Diärese"}}, "category": "Lu"}, {
      "key": "00DD",
      "mappings": {"default": {"default": "groß Y mit Aigu"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "0100", "mappings": {"default": {"default": "groß A mit Makron"}}}, {
      "category": "Lu",
      "key": "0102",
      "mappings": {"default": {"default": "groß A mit Breve"}}
    }, {"category": "Lu", "key": "0104", "mappings": {"default": {"default": "groß A mit Ogonek"}}}, {
      "key": "0106",
      "mappings": {"default": {"default": "groß C mit Aigu"}},
      "category": "Lu"
    }, {"key": "0108", "mappings": {"default": {"default": "groß C mit Zirkumflex"}}, "category": "Lu"}, {
      "key": "010A",
      "mappings": {"default": {"default": "groß C mit Punkt darüber"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "010C", "mappings": {"default": {"default": "groß C mit Caron"}}}, {
      "category": "Lu",
      "key": "010E",
      "mappings": {"default": {"default": "groß D mit Caron"}}
    }, {"category": "Lu", "key": "0112", "mappings": {"default": {"default": "groß E mit Makron"}}}, {
      "category": "Lu",
      "key": "0114",
      "mappings": {"default": {"default": "groß E mit Breve"}}
    }, {
      "key": "0116",
      "mappings": {"default": {"default": "groß E mit Punkt darüber"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "0118", "mappings": {"default": {"default": "groß E mit Ogonek"}}}, {
      "category": "Lu",
      "key": "011A",
      "mappings": {"default": {"default": "groß E mit Caron"}}
    }, {
      "key": "011C",
      "mappings": {"default": {"default": "groß G mit Zirkumflex"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "011E", "mappings": {"default": {"default": "groß G mit Breve"}}}, {
      "key": "0120",
      "mappings": {"default": {"default": "groß G mit Punkt darüber"}},
      "category": "Lu"
    }, {"key": "0122", "mappings": {"default": {"default": "groß G mit Cedille"}}, "category": "Lu"}, {
      "key": "0124",
      "mappings": {"default": {"default": "groß H mit Zirkumflex"}},
      "category": "Lu"
    }, {"key": "0128", "mappings": {"default": {"default": "groß I mit Tilde"}}, "category": "Lu"}, {
      "category": "Lu",
      "key": "012A",
      "mappings": {"default": {"default": "groß I mit Makron"}}
    }, {"category": "Lu", "key": "012C", "mappings": {"default": {"default": "groß I mit Breve"}}}, {
      "category": "Lu",
      "key": "012E",
      "mappings": {"default": {"default": "groß I mit Ogonek"}}
    }, {
      "key": "0130",
      "mappings": {"default": {"default": "groß I mit Punkt darüber"}},
      "category": "Lu"
    }, {"key": "0134", "mappings": {"default": {"default": "groß J mit Zirkumflex"}}, "category": "Lu"}, {
      "key": "0136",
      "mappings": {"default": {"default": "groß K mit Cedille"}},
      "category": "Lu"
    }, {"key": "0139", "mappings": {"default": {"default": "groß L mit Aigu"}}, "category": "Lu"}, {
      "key": "013B",
      "mappings": {"default": {"default": "groß L mit Cedille"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "013D", "mappings": {"default": {"default": "groß L mit Caron"}}}, {
      "key": "013F",
      "mappings": {"default": {"default": "groß L mit Punkt in der Mitte"}},
      "category": "Lu"
    }, {"key": "0143", "mappings": {"default": {"default": "groß N mit Aigu"}}, "category": "Lu"}, {
      "key": "0145",
      "mappings": {"default": {"default": "groß N mit Cedille"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "0147", "mappings": {"default": {"default": "groß N mit Caron"}}}, {
      "category": "Lu",
      "key": "014C",
      "mappings": {"default": {"default": "groß O mit Makron"}}
    }, {"category": "Lu", "key": "014E", "mappings": {"default": {"default": "groß O mit Breve"}}}, {
      "key": "0150",
      "mappings": {"default": {"default": "groß O mit doppeltem Aigu"}},
      "category": "Lu"
    }, {"key": "0154", "mappings": {"default": {"default": "groß R mit Aigu"}}, "category": "Lu"}, {
      "key": "0156",
      "mappings": {"default": {"default": "groß R mit Cedille"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "0158", "mappings": {"default": {"default": "groß R mit Caron"}}}, {
      "key": "015A",
      "mappings": {"default": {"default": "groß S mit Aigu"}},
      "category": "Lu"
    }, {"key": "015C", "mappings": {"default": {"default": "groß S mit Zirkumflex"}}, "category": "Lu"}, {
      "key": "015E",
      "mappings": {"default": {"default": "groß S mit Cedille"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "0160", "mappings": {"default": {"default": "groß S mit Caron"}}}, {
      "key": "0162",
      "mappings": {"default": {"default": "groß T mit Cedille"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "0164", "mappings": {"default": {"default": "groß T mit Caron"}}}, {
      "key": "0168",
      "mappings": {"default": {"default": "groß U mit Tilde"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "016A", "mappings": {"default": {"default": "groß U mit Makron"}}}, {
      "category": "Lu",
      "key": "016C",
      "mappings": {"default": {"default": "groß U mit Breve"}}
    }, {
      "key": "016E",
      "mappings": {"default": {"default": "groß U mit Ring darüber"}},
      "category": "Lu"
    }, {
      "key": "0170",
      "mappings": {"default": {"default": "groß U mit dopppeltem Aigu"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "0172", "mappings": {"default": {"default": "groß U mit Ogonek"}}}, {
      "key": "0174",
      "mappings": {"default": {"default": "groß W mit Zirkumflex"}},
      "category": "Lu"
    }, {"key": "0176", "mappings": {"default": {"default": "groß Y mit Zirkumflex"}}, "category": "Lu"}, {
      "key": "0178",
      "mappings": {"default": {"default": "groß Y mit Diärese"}},
      "category": "Lu"
    }, {"key": "0179", "mappings": {"default": {"default": "groß Z mit Aigu"}}, "category": "Lu"}, {
      "key": "017B",
      "mappings": {"default": {"default": "groß Z mit Punkt darüber"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "017D", "mappings": {"default": {"default": "groß Z mit Caron"}}}, {
      "category": "Lu",
      "key": "01CD",
      "mappings": {"default": {"default": "groß A mit Caron"}}
    }, {"category": "Lu", "key": "01CF", "mappings": {"default": {"default": "groß I mit Caron"}}}, {
      "category": "Lu",
      "key": "01D1",
      "mappings": {"default": {"default": "groß O mit Caron"}}
    }, {"category": "Lu", "key": "01D3", "mappings": {"default": {"default": "groß U mit Caron"}}}, {
      "category": "Lu",
      "key": "01E6",
      "mappings": {"default": {"default": "groß G mit Caron"}}
    }, {"category": "Lu", "key": "01E8", "mappings": {"default": {"default": "groß K mit Caron"}}}, {
      "category": "Lu",
      "key": "01EA",
      "mappings": {"default": {"default": "groß O mit Ogonek"}}
    }, {"category": "Lu", "key": "01F4", "mappings": {"default": {"default": "groß G mit Akut"}}}, {
      "category": "Lu",
      "key": "01F8",
      "mappings": {"default": {"default": "groß N mit Gravis"}}
    }, {
      "category": "Lu",
      "key": "0200",
      "mappings": {"default": {"default": "groß A mit Doppelgravis"}}
    }, {
      "category": "Lu",
      "key": "0202",
      "mappings": {"default": {"default": "groß A mit umgekehrter Breve"}}
    }, {
      "category": "Lu",
      "key": "0204",
      "mappings": {"default": {"default": "groß E mit Doppelgravis"}}
    }, {
      "category": "Lu",
      "key": "0206",
      "mappings": {"default": {"default": "groß E mit umgekehrter Breve"}}
    }, {
      "category": "Lu",
      "key": "0208",
      "mappings": {"default": {"default": "groß I mit Doppelgravis"}}
    }, {
      "category": "Lu",
      "key": "020A",
      "mappings": {"default": {"default": "groß I mit umgekehrter Breve"}}
    }, {
      "category": "Lu",
      "key": "020C",
      "mappings": {"default": {"default": "groß O mit Doppelgravis"}}
    }, {
      "category": "Lu",
      "key": "020E",
      "mappings": {"default": {"default": "groß O mit umgekehrter Breve"}}
    }, {
      "category": "Lu",
      "key": "0210",
      "mappings": {"default": {"default": "groß R mit Doppelgravis"}}
    }, {
      "category": "Lu",
      "key": "0212",
      "mappings": {"default": {"default": "groß R mit umgekehrter Breve"}}
    }, {
      "category": "Lu",
      "key": "0214",
      "mappings": {"default": {"default": "groß U mit Doppelgravis"}}
    }, {
      "category": "Lu",
      "key": "0216",
      "mappings": {"default": {"default": "groß U mit umgekehrter Breve"}}
    }, {
      "category": "Lu",
      "key": "0218",
      "mappings": {"default": {"default": "groß S mit untenstehendem Komma"}}
    }, {
      "category": "Lu",
      "key": "021A",
      "mappings": {"default": {"default": "groß T mit untenstehendem Komma"}}
    }, {"category": "Lu", "key": "021E", "mappings": {"default": {"default": "groß H mit Caron"}}}, {
      "category": "Lu",
      "key": "0226",
      "mappings": {"default": {"default": "groß A mit Punkt oben"}}
    }, {"category": "Lu", "key": "0228", "mappings": {"default": {"default": "groß E mit Cedilla"}}}, {
      "category": "Lu",
      "key": "022E",
      "mappings": {"default": {"default": "groß O mit Punkt oben"}}
    }, {"category": "Lu", "key": "0232", "mappings": {"default": {"default": "groß Y mit Makron"}}}, {
      "category": "Lu",
      "key": "1E00",
      "mappings": {"default": {"default": "groß A mit Ring unten"}}
    }, {
      "category": "Lu",
      "key": "1E02",
      "mappings": {"default": {"default": "groß B mit Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E04",
      "mappings": {"default": {"default": "groß B mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1E06",
      "mappings": {"default": {"default": "groß B mit Zeile darunter"}}
    }, {
      "category": "Lu",
      "key": "1E0A",
      "mappings": {"default": {"default": "groß D mit Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E0C",
      "mappings": {"default": {"default": "groß D mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1E0E",
      "mappings": {"default": {"default": "groß D mit Zeile darunter"}}
    }, {"category": "Lu", "key": "1E10", "mappings": {"default": {"default": "groß D mit Cedilla"}}}, {
      "category": "Lu",
      "key": "1E12",
      "mappings": {"default": {"default": "groß D mit darunterem Zirkumflex"}}
    }, {
      "category": "Lu",
      "key": "1E18",
      "mappings": {"default": {"default": "groß E mit darunterem Zirkumflex"}}
    }, {
      "category": "Lu",
      "key": "1E1A",
      "mappings": {"default": {"default": "groß E mit darunterliegender Tilde"}}
    }, {
      "category": "Lu",
      "key": "1E1E",
      "mappings": {"default": {"default": "groß F mit Punkt oben"}}
    }, {"category": "Lu", "key": "1E20", "mappings": {"default": {"default": "groß G mit Makron"}}}, {
      "category": "Lu",
      "key": "1E22",
      "mappings": {"default": {"default": "groß H mit Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E24",
      "mappings": {"default": {"default": "groß H mit untenstehendem Punkt"}}
    }, {"category": "Lu", "key": "1E26", "mappings": {"default": {"default": "groß H mit Diärese"}}}, {
      "category": "Lu",
      "key": "1E28",
      "mappings": {"default": {"default": "groß H mit Cedilla"}}
    }, {
      "category": "Lu",
      "key": "1E2A",
      "mappings": {"default": {"default": "groß H mit Breve unten"}}
    }, {
      "category": "Lu",
      "key": "1E2C",
      "mappings": {"default": {"default": "groß I mit unten stehender Tilde"}}
    }, {"category": "Lu", "key": "1E30", "mappings": {"default": {"default": "groß K mit Akut"}}}, {
      "category": "Lu",
      "key": "1E32",
      "mappings": {"default": {"default": "groß K mit untenstehendem Punkt"}}
    }, {"category": "Lu", "key": "1E34", "mappings": {"default": {"default": "groß K mit Zeile"}}}, {
      "category": "Lu",
      "key": "1E36",
      "mappings": {"default": {"default": "groß L mit Punkt unten"}}
    }, {
      "category": "Lu",
      "key": "1E3A",
      "mappings": {"default": {"default": "groß L mit Zeile darunter"}}
    }, {
      "category": "Lu",
      "key": "1E3C",
      "mappings": {"default": {"default": "groß L mit darunterem Zirkumflex"}}
    }, {"category": "Lu", "key": "1E3E", "mappings": {"default": {"default": "groß M mit Akut"}}}, {
      "category": "Lu",
      "key": "1E40",
      "mappings": {"default": {"default": "groß M mit Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E42",
      "mappings": {"default": {"default": "groß M mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1E44",
      "mappings": {"default": {"default": "groß N mit Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E46",
      "mappings": {"default": {"default": "groß N mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1E48",
      "mappings": {"default": {"default": "groß N mit Zeile darunter"}}
    }, {
      "category": "Lu",
      "key": "1E4A",
      "mappings": {"default": {"default": "groß N mit darunterem Zirkumflex"}}
    }, {"category": "Lu", "key": "1E54", "mappings": {"default": {"default": "groß P mit Akut"}}}, {
      "category": "Lu",
      "key": "1E56",
      "mappings": {"default": {"default": "groß P mit Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E58",
      "mappings": {"default": {"default": "groß R mit Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E5A",
      "mappings": {"default": {"default": "groß R mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1E5E",
      "mappings": {"default": {"default": "groß R mit Zeile darunter"}}
    }, {
      "category": "Lu",
      "key": "1E60",
      "mappings": {"default": {"default": "groß S mit Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E62",
      "mappings": {"default": {"default": "groß S mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1E6A",
      "mappings": {"default": {"default": "groß T mit Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E6C",
      "mappings": {"default": {"default": "groß T mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1E6E",
      "mappings": {"default": {"default": "groß T mit Zeile darunter"}}
    }, {
      "category": "Lu",
      "key": "1E70",
      "mappings": {"default": {"default": "groß T mit darunterem Zirkumflex"}}
    }, {
      "category": "Lu",
      "key": "1E72",
      "mappings": {"default": {"default": "groß U mit untenstehender Diärese"}}
    }, {
      "category": "Lu",
      "key": "1E74",
      "mappings": {"default": {"default": "groß U mit darunterliegender Tilde"}}
    }, {
      "category": "Lu",
      "key": "1E76",
      "mappings": {"default": {"default": "groß U mit darunterem Zirkumflex"}}
    }, {"category": "Lu", "key": "1E7C", "mappings": {"default": {"default": "groß V mit Tilde"}}}, {
      "category": "Lu",
      "key": "1E7E",
      "mappings": {"default": {"default": "groß V mit untenstehendem Punkt"}}
    }, {"category": "Lu", "key": "1E80", "mappings": {"default": {"default": "groß W mit Gravis"}}}, {
      "category": "Lu",
      "key": "1E82",
      "mappings": {"default": {"default": "groß W mit Akut"}}
    }, {"category": "Lu", "key": "1E84", "mappings": {"default": {"default": "groß W mit Diärese"}}}, {
      "category": "Lu",
      "key": "1E86",
      "mappings": {"default": {"default": "groß W mit Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E88",
      "mappings": {"default": {"default": "groß W mit Punkt unten"}}
    }, {
      "category": "Lu",
      "key": "1E8A",
      "mappings": {"default": {"default": "groß X mit Punkt oben"}}
    }, {"category": "Lu", "key": "1E8C", "mappings": {"default": {"default": "groß X mit Diärese"}}}, {
      "category": "Lu",
      "key": "1E8E",
      "mappings": {"default": {"default": "groß Y mit Punkt oben"}}
    }, {
      "category": "Lu",
      "key": "1E90",
      "mappings": {"default": {"default": "groß Z mit Zirkumflex"}}
    }, {
      "category": "Lu",
      "key": "1E92",
      "mappings": {"default": {"default": "groß Z mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1E94",
      "mappings": {"default": {"default": "groß Z mit Zeile darunter"}}
    }, {
      "category": "Lu",
      "key": "1EA0",
      "mappings": {"default": {"default": "groß A mit Punkt unten"}}
    }, {
      "category": "Lu",
      "key": "1EA2",
      "mappings": {"default": {"default": "groß A mit Haken oben"}}
    }, {
      "category": "Lu",
      "key": "1EB8",
      "mappings": {"default": {"default": "groß E mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1EBA",
      "mappings": {"default": {"default": "groß E mit Haken oben"}}
    }, {"category": "Lu", "key": "1EBC", "mappings": {"default": {"default": "groß E mit Tilde"}}}, {
      "category": "Lu",
      "key": "1EC8",
      "mappings": {"default": {"default": "groß I mit Haken oben"}}
    }, {
      "category": "Lu",
      "key": "1ECA",
      "mappings": {"default": {"default": "groß I mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1ECC",
      "mappings": {"default": {"default": "groß O mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1ECE",
      "mappings": {"default": {"default": "groß O mit Haken oben"}}
    }, {
      "category": "Lu",
      "key": "1EE4",
      "mappings": {"default": {"default": "groß U mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1EE6",
      "mappings": {"default": {"default": "groß U mit Haken oben"}}
    }, {"category": "Lu", "key": "1EF2", "mappings": {"default": {"default": "groß Y mit Gravis"}}}, {
      "category": "Lu",
      "key": "1EF4",
      "mappings": {"default": {"default": "groß Y mit untenstehendem Punkt"}}
    }, {
      "category": "Lu",
      "key": "1EF6",
      "mappings": {"default": {"default": "groß Y mit Haken oben"}}
    }, {"category": "Lu", "key": "1EF8", "mappings": {"default": {"default": "groß Y mit Tilde"}}}]
  ,
  "de/symbols/math_angles.js":
    [{"locale": "de"}, {
      "category": "Sm",
      "key": "22BE",
      "mappings": {"default": {"default": "rechter Winkel mit Bogen"}}
    }, {
      "category": "Sm",
      "key": "237C",
      "mappings": {"default": {"default": "rechter Winkel mit nach unten gerichtetem Zickzackpfeil"}}
    }, {
      "category": "Sm",
      "key": "27C0",
      "mappings": {"default": {"default": "Dreidimensionaler Winkel"}}
    }, {
      "category": "Sm",
      "key": "299B",
      "mappings": {"default": {"default": "Gemessene Winkelöffnung links"}}
    }, {
      "category": "Sm",
      "key": "299C",
      "mappings": {"default": {"default": "Rechtwinklige Variante mit Quadrat"}}
    }, {
      "category": "Sm",
      "key": "299D",
      "mappings": {"default": {"default": "rechter Winkel mit Punkt gemessen"}}
    }, {
      "category": "Sm",
      "key": "299E",
      "mappings": {"default": {"default": "Winkel mit S nach innen"}}
    }, {"category": "Sm", "key": "299F", "mappings": {"default": {"default": "Spitzer Winkel"}}}, {
      "category": "Sm",
      "key": "29A0",
      "mappings": {"default": {"default": "Sphärischer Öffnungswinkel nach links"}}
    }, {
      "category": "Sm",
      "key": "29A1",
      "mappings": {"default": {"default": "Sphärischer Winkel, der sich öffnet"}}
    }, {"category": "Sm", "key": "29A2", "mappings": {"default": {"default": "Gedrehter Winkel"}}}, {
      "category": "Sm",
      "key": "29A3",
      "mappings": {"default": {"default": "Umgekehrter Winkel"}}
    }, {
      "category": "Sm",
      "key": "29A4",
      "mappings": {"default": {"default": "Winkel mit Underbar"}}
    }, {
      "category": "Sm",
      "key": "29A5",
      "mappings": {"default": {"default": "Umgekehrter Winkel mit Unterleiste"}}
    }, {
      "category": "Sm",
      "key": "29A6",
      "mappings": {"default": {"default": "Schräger Winkel, der sich öffnet"}}
    }, {
      "category": "Sm",
      "key": "29A7",
      "mappings": {"default": {"default": "Schräger Winkel, der sich nach unten öffnet"}}
    }, {
      "category": "Sm",
      "key": "29A8",
      "mappings": {"default": {"default": "Gemessener Winkel mit offenem Arm endet im Pfeil nach oben und rechts"}}
    }, {
      "category": "Sm",
      "key": "29A9",
      "mappings": {"default": {"default": "Gemessener Winkel mit offenem Arm und Pfeil nach oben und links"}}
    }, {
      "category": "Sm",
      "key": "29AA",
      "mappings": {"default": {"default": "Gemessener Winkel mit offenem Arm endet im nach unten zeigenden Pfeil und rechts"}}
    }, {
      "category": "Sm",
      "key": "29AB",
      "mappings": {"default": {"default": "Gemessener Winkel mit offenem Arm endet im nach unten zeigenden Pfeil"}}
    }, {
      "category": "Sm",
      "key": "29AC",
      "mappings": {"default": {"default": "Gemessener Winkel mit offenem Arm endet im Pfeil nach rechts und oben"}}
    }, {
      "category": "Sm",
      "key": "29AD",
      "mappings": {"default": {"default": "Gemessener Winkel mit offenem Arm endet im Pfeil nach links und oben"}}
    }, {
      "category": "Sm",
      "key": "29AE",
      "mappings": {"default": {"default": "Gemessener Winkel mit offenem Arm endet im nach rechts und unten zeigenden Pfeil"}}
    }, {
      "category": "Sm",
      "key": "29AF",
      "mappings": {"default": {"default": "Gemessener Winkel mit offenem Arm und Pfeil nach links und unten"}}
    }]
  ,
  "de/symbols/math_arrows.js":
    [{"locale": "de"}, {
      "category": "Sm",
      "key": "2190",
      "mappings": {"default": {"default": "Pfeil nach links"}}
    }, {"category": "Sm", "key": "2191", "mappings": {"default": {"default": "Aufwärtspfeil"}}}, {
      "category": "Sm",
      "key": "2192",
      "mappings": {"default": {"default": "Pfeil nach rechts"}}
    }, {"category": "Sm", "key": "2193", "mappings": {"default": {"default": "Pfeil nach unten"}}}, {
      "category": "Sm",
      "key": "2194",
      "mappings": {"default": {"default": "Linker rechter Pfeil"}}
    }, {"category": "So", "key": "2195", "mappings": {"default": {"default": "Pfeil nach oben"}}}, {
      "category": "So",
      "key": "2196",
      "mappings": {"default": {"default": "Nordwestpfeil"}}
    }, {"category": "So", "key": "2197", "mappings": {"default": {"default": "Nordostpfeil"}}}, {
      "category": "So",
      "key": "2198",
      "mappings": {"default": {"default": "Südostpfeil"}}
    }, {"category": "So", "key": "2199", "mappings": {"default": {"default": "Südwestpfeil"}}}, {
      "category": "Sm",
      "key": "219A",
      "mappings": {"default": {"default": "Pfeil nach links mit Strich"}}
    }, {
      "category": "Sm",
      "key": "219B",
      "mappings": {"default": {"default": "Pfeil nach rechts mit Strich"}}
    }, {
      "category": "So",
      "key": "219C",
      "mappings": {"default": {"default": "Wellenpfeil nach links"}}
    }, {
      "category": "So",
      "key": "219D",
      "mappings": {"default": {"default": "Wellenpfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "219E",
      "mappings": {"default": {"default": "Pfeil nach links mit zwei Spitzen"}}
    }, {
      "category": "So",
      "key": "219F",
      "mappings": {"default": {"default": "Aufwärts zwei Pfeil nach unten"}}
    }, {
      "category": "Sm",
      "key": "21A0",
      "mappings": {"default": {"default": "Pfeil nach rechts mit zwei Köpfen"}}
    }, {
      "category": "So",
      "key": "21A1",
      "mappings": {"default": {"default": "Abwärts zwei Pfeil nach unten"}}
    }, {
      "category": "So",
      "key": "21A2",
      "mappings": {"default": {"default": "Pfeil nach links mit Schwanz"}}
    }, {
      "category": "Sm",
      "key": "21A3",
      "mappings": {"default": {"default": "Pfeil nach rechts mit Schwanz"}}
    }, {
      "category": "So",
      "key": "21A4",
      "mappings": {"default": {"default": "Pfeil nach links von der Bar"}}
    }, {
      "category": "So",
      "key": "21A5",
      "mappings": {"default": {"default": "Pfeil von der Bar nach oben"}}
    }, {
      "category": "Sm",
      "key": "21A6",
      "mappings": {"default": {"default": "Pfeil nach rechts von der Bar"}}
    }, {
      "category": "So",
      "key": "21A7",
      "mappings": {"default": {"default": "Pfeil von der Bar nach unten"}}
    }, {
      "category": "So",
      "key": "21A8",
      "mappings": {"default": {"default": "Pfeil nach unten mit Basis"}}
    }, {
      "category": "So",
      "key": "21A9",
      "mappings": {"default": {"default": "Pfeil nach links mit Haken"}}
    }, {
      "category": "So",
      "key": "21AA",
      "mappings": {"default": {"default": "Pfeil nach rechts mit Haken"}}
    }, {
      "category": "So",
      "key": "21AB",
      "mappings": {"default": {"default": "Pfeil nach links mit Schleife"}}
    }, {
      "category": "So",
      "key": "21AC",
      "mappings": {"default": {"default": "Pfeil nach rechts mit Schleife"}}
    }, {
      "category": "So",
      "key": "21AD",
      "mappings": {"default": {"default": "Linker rechter Wellenpfeil"}}
    }, {
      "category": "Sm",
      "key": "21AE",
      "mappings": {"default": {"default": "Linker rechter Pfeil mit Strich"}}
    }, {
      "category": "So",
      "key": "21AF",
      "mappings": {"default": {"default": "Zickzack-Pfeil nach unten"}}
    }, {
      "category": "So",
      "key": "21B0",
      "mappings": {"default": {"default": "Aufwärtspfeil mit Spitze nach links"}}
    }, {
      "category": "So",
      "key": "21B1",
      "mappings": {"default": {"default": "Aufwärtspfeil mit Spitze nach rechts"}}
    }, {
      "category": "So",
      "key": "21B2",
      "mappings": {"default": {"default": "Pfeil nach unten mit Spitze nach links"}}
    }, {
      "category": "So",
      "key": "21B3",
      "mappings": {"default": {"default": "Abwärtspfeil mit Spitze nach rechts"}}
    }, {
      "category": "So",
      "key": "21B4",
      "mappings": {"default": {"default": "Pfeil nach rechts mit Ecke nach unten"}}
    }, {
      "category": "So",
      "key": "21B5",
      "mappings": {"default": {"default": "Pfeil nach unten mit Ecke nach links"}}
    }, {
      "category": "So",
      "key": "21B6",
      "mappings": {"default": {"default": "Gegen den Uhrzeigersinn oberer halbkreisförmiger Pfeil"}}
    }, {
      "category": "So",
      "key": "21B7",
      "mappings": {"default": {"default": "Halbkreis im Uhrzeigersinn"}}
    }, {
      "category": "So",
      "key": "21B8",
      "mappings": {"default": {"default": "Nordwestpfeil zur langen Bar"}}
    }, {
      "category": "So",
      "key": "21B9",
      "mappings": {"default": {"default": "Pfeil nach links zum Überfahren Pfeil nach rechts zum Überfahren"}}
    }, {
      "category": "So",
      "key": "21BA",
      "mappings": {"default": {"default": "Offener Kreispfeil gegen den Uhrzeigersinn"}}
    }, {
      "category": "So",
      "key": "21BB",
      "mappings": {"default": {"default": "Kreispfeil im Uhrzeigersinn öffnen"}}
    }, {
      "category": "So",
      "key": "21C4",
      "mappings": {"default": {"default": "Pfeil nach rechts über Pfeil nach links"}}
    }, {
      "category": "So",
      "key": "21C5",
      "mappings": {"default": {"default": "Aufwärtspfeil nach links von Abwärtspfeil"}}
    }, {
      "category": "So",
      "key": "21C6",
      "mappings": {"default": {"default": "Pfeil nach links über Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "21C7",
      "mappings": {"default": {"default": "Links gepaarte Pfeile"}}
    }, {
      "category": "So",
      "key": "21C8",
      "mappings": {"default": {"default": "Aufwärts gepaarte Pfeile"}}
    }, {
      "category": "So",
      "key": "21C9",
      "mappings": {"default": {"default": "Rechts gepaarte Pfeile"}}
    }, {
      "category": "So",
      "key": "21CA",
      "mappings": {"default": {"default": "Abwärts gepaarte Pfeile"}}
    }, {
      "category": "So",
      "key": "21CD",
      "mappings": {"default": {"default": "Doppelpfeil nach links mit Strich"}}
    }, {
      "category": "Sm",
      "key": "21CE",
      "mappings": {"default": {"default": "Links Rechts Doppelpfeil mit Strich"}}
    }, {
      "category": "Sm",
      "key": "21CF",
      "mappings": {"default": {"default": "Doppelpfeil nach rechts mit Strich"}}
    }, {
      "category": "So",
      "key": "21D0",
      "mappings": {"default": {"default": "Doppelpfeil nach links"}}
    }, {
      "category": "So",
      "key": "21D1",
      "mappings": {"default": {"default": "Doppelpfeil nach oben"}}
    }, {
      "category": "Sm",
      "key": "21D2",
      "mappings": {"default": {"default": "Doppelpfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "21D3",
      "mappings": {"default": {"default": "Doppelpfeil nach unten"}}
    }, {
      "category": "Sm",
      "key": "21D4",
      "mappings": {"default": {"default": "Linker rechter Doppelpfeil"}}
    }, {
      "category": "So",
      "key": "21D5",
      "mappings": {"default": {"default": "Doppelpfeil nach unten"}}
    }, {
      "category": "So",
      "key": "21D6",
      "mappings": {"default": {"default": "Nordwest-Doppelpfeil"}}
    }, {
      "category": "So",
      "key": "21D7",
      "mappings": {"default": {"default": "Nordost-Doppelpfeil"}}
    }, {"category": "So", "key": "21D8", "mappings": {"default": {"default": "Südost Doppelpfeil"}}}, {
      "category": "So",
      "key": "21D9",
      "mappings": {"default": {"default": "Südwest-Doppelpfeil"}}
    }, {
      "category": "So",
      "key": "21DA",
      "mappings": {"default": {"default": "Dreifacher Pfeil nach links"}}
    }, {
      "category": "So",
      "key": "21DB",
      "mappings": {"default": {"default": "Dreifacher Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "21DC",
      "mappings": {"default": {"default": "Linker Squiggle-Pfeil"}}
    }, {
      "category": "So",
      "key": "21DD",
      "mappings": {"default": {"default": "Nach rechts gekrümmter Pfeil"}}
    }, {
      "category": "So",
      "key": "21DE",
      "mappings": {"default": {"default": "Aufwärtspfeil mit Doppelstrich"}}
    }, {
      "category": "So",
      "key": "21DF",
      "mappings": {"default": {"default": "Abwärtspfeil mit Doppelstrich"}}
    }, {
      "category": "So",
      "key": "21E0",
      "mappings": {"default": {"default": "Pfeil nach links nach links"}}
    }, {
      "category": "So",
      "key": "21E1",
      "mappings": {"default": {"default": "Pfeil nach oben gestrichen"}}
    }, {
      "category": "So",
      "key": "21E2",
      "mappings": {"default": {"default": "Rechts gestrichener Pfeil"}}
    }, {
      "category": "So",
      "key": "21E3",
      "mappings": {"default": {"default": "Abwärts gestrichelter Pfeil"}}
    }, {
      "category": "So",
      "key": "21E4",
      "mappings": {"default": {"default": "Pfeil nach links zur Bar"}}
    }, {
      "category": "So",
      "key": "21E5",
      "mappings": {"default": {"default": "Pfeil nach rechts zur Bar"}}
    }, {
      "category": "So",
      "key": "21E6",
      "mappings": {"default": {"default": "Weißer Pfeil nach links"}}
    }, {
      "category": "So",
      "key": "21E7",
      "mappings": {"default": {"default": "Aufwärts weißer Pfeil"}}
    }, {
      "category": "So",
      "key": "21E8",
      "mappings": {"default": {"default": "Rechts weißer Pfeil"}}
    }, {
      "category": "So",
      "key": "21E9",
      "mappings": {"default": {"default": "Abwärts weißer Pfeil"}}
    }, {
      "category": "So",
      "key": "21EA",
      "mappings": {"default": {"default": "Aufwärts weißer Pfeil von der Stange"}}
    }, {
      "category": "So",
      "key": "21EB",
      "mappings": {"default": {"default": "Aufwärts weißer Pfeil auf Sockel"}}
    }, {
      "category": "So",
      "key": "21EC",
      "mappings": {"default": {"default": "Aufwärts weißer Pfeil auf Sockel mit horizontaler Stange"}}
    }, {
      "category": "So",
      "key": "21ED",
      "mappings": {"default": {"default": "Aufwärts weißer Pfeil auf Sockel mit vertikaler Stange"}}
    }, {
      "category": "So",
      "key": "21EE",
      "mappings": {"default": {"default": "Aufwärts weißer Doppelpfeil"}}
    }, {
      "category": "So",
      "key": "21EF",
      "mappings": {"default": {"default": "Aufwärts weißer doppelter Pfeil auf Sockel"}}
    }, {
      "category": "So",
      "key": "21F0",
      "mappings": {"default": {"default": "Rechts weißer Pfeil von der Wand"}}
    }, {
      "category": "So",
      "key": "21F1",
      "mappings": {"default": {"default": "Nordwestpfeil zur Ecke"}}
    }, {
      "category": "So",
      "key": "21F2",
      "mappings": {"default": {"default": "Südostpfeil zur Ecke"}}
    }, {
      "category": "So",
      "key": "21F3",
      "mappings": {"default": {"default": "Weißer Pfeil nach unten"}}
    }, {
      "category": "Sm",
      "key": "21F4",
      "mappings": {"default": {"default": "rechter Pfeil mit kleinem Kreis"}}
    }, {
      "category": "Sm",
      "key": "21F5",
      "mappings": {"default": {"default": "Abwärtspfeil nach links von Aufwärtspfeil"}}
    }, {
      "category": "Sm",
      "key": "21F6",
      "mappings": {"default": {"default": "Drei nach rechts gerichtete Pfeile"}}
    }, {
      "category": "Sm",
      "key": "21F7",
      "mappings": {"default": {"default": "Pfeil nach links mit vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "21F8",
      "mappings": {"default": {"default": "Pfeil nach rechts mit vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "21F9",
      "mappings": {"default": {"default": "Linker rechter Pfeil mit vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "21FA",
      "mappings": {"default": {"default": "Pfeil nach links mit doppeltem vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "21FB",
      "mappings": {"default": {"default": "Pfeil nach rechts mit doppeltem vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "21FC",
      "mappings": {"default": {"default": "Linker rechter Pfeil mit doppeltem vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "21FD",
      "mappings": {"default": {"default": "Pfeil nach links mit offenem Kopf"}}
    }, {
      "category": "Sm",
      "key": "21FE",
      "mappings": {"default": {"default": "Pfeil nach rechts mit offenem Kopf"}}
    }, {
      "category": "Sm",
      "key": "21FF",
      "mappings": {"default": {"default": "Linker rechter Pfeil mit offenem Kopf"}}
    }, {"category": "So", "key": "2301", "mappings": {"default": {"default": "Elektrischer Pfeil"}}}, {
      "category": "So",
      "key": "2303",
      "mappings": {"default": {"default": "Pfeil nach oben"}}
    }, {"category": "So", "key": "2304", "mappings": {"default": {"default": "Pfeil nach unten"}}}, {
      "category": "So",
      "key": "2324",
      "mappings": {"default": {"default": "Pfeilspitze zwischen zwei horizontalen Balken"}}
    }, {
      "category": "So",
      "key": "238B",
      "mappings": {"default": {"default": "Gebrochener Kreis mit Nordwestpfeil"}}
    }, {
      "category": "So",
      "key": "2794",
      "mappings": {"default": {"default": "Schwerer, nach rechts gerichteter Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "2798",
      "mappings": {"default": {"default": "Schwerer Südostpfeil"}}
    }, {
      "category": "So",
      "key": "2799",
      "mappings": {"default": {"default": "Schwerer Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "279A",
      "mappings": {"default": {"default": "Schwerer Nordostpfeil"}}
    }, {
      "category": "So",
      "key": "279B",
      "mappings": {"default": {"default": "Zeichnungspunkt Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "279C",
      "mappings": {"default": {"default": "Schwerer runder Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "279D",
      "mappings": {"default": {"default": "Dreieckköpfiger Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "279E",
      "mappings": {"default": {"default": "Schwerer Dreieckspfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "279F",
      "mappings": {"default": {"default": "Dreieckköpfiger Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27A0",
      "mappings": {"default": {"default": "Schwerer, gestrichelter Dreieckspfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27A1",
      "mappings": {"default": {"default": "Schwarzer Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27A2",
      "mappings": {"default": {"default": "Dreidimensionale, nach oben beleuchtete Pfeilspitze"}}
    }, {
      "category": "So",
      "key": "27A3",
      "mappings": {"default": {"default": "Dreidimensionale, von unten beleuchtete Pfeilspitze nach rechts"}}
    }, {
      "category": "So",
      "key": "27A4",
      "mappings": {"default": {"default": "Schwarze Pfeilspitze nach rechts"}}
    }, {
      "category": "So",
      "key": "27A5",
      "mappings": {"default": {"default": "Schwerer schwarzer Pfeil nach unten und rechts gebogen"}}
    }, {
      "category": "So",
      "key": "27A6",
      "mappings": {"default": {"default": "Schwerer schwarzer Pfeil nach oben und rechts gebogen"}}
    }, {
      "category": "So",
      "key": "27A7",
      "mappings": {"default": {"default": "Schwarzer nach rechts gerichteter Pfeil"}}
    }, {
      "category": "So",
      "key": "27A8",
      "mappings": {"default": {"default": "Schwerer konkaver Spitze-Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27A9",
      "mappings": {"default": {"default": "Rechts geschatteter weißer Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27AA",
      "mappings": {"default": {"default": "Links schattierter weißer Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27AB",
      "mappings": {"default": {"default": "Zurückgekippter, beschatteter weißer Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27AC",
      "mappings": {"default": {"default": "Vorne geneigter, beschatteter weißer Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27AD",
      "mappings": {"default": {"default": "Schwerer unterer rechter Schatten mit weißem Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27AE",
      "mappings": {"default": {"default": "Schwerer oberer rechter Schatten mit weißem Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27AF",
      "mappings": {"default": {"default": "Gekerbter Pfeil nach rechts unten rechts"}}
    }, {
      "category": "So",
      "key": "27B1",
      "mappings": {"default": {"default": "Weißer Pfeil nach rechts oben gekerbt"}}
    }, {
      "category": "So",
      "key": "27B2",
      "mappings": {"default": {"default": "Eingekreister schwerer weißer Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27B3",
      "mappings": {"default": {"default": "Pfeil mit weißer Feder nach rechts"}}
    }, {
      "category": "So",
      "key": "27B4",
      "mappings": {"default": {"default": "Schwarz-gefiederter Südostpfeil"}}
    }, {
      "category": "So",
      "key": "27B5",
      "mappings": {"default": {"default": "Pfeil mit schwarzer Feder nach rechts"}}
    }, {
      "category": "So",
      "key": "27B6",
      "mappings": {"default": {"default": "Schwarzgefiederter Nordostpfeil"}}
    }, {
      "category": "So",
      "key": "27B7",
      "mappings": {"default": {"default": "Schwerer schwarzgefiederter Südostpfeil"}}
    }, {
      "category": "So",
      "key": "27B8",
      "mappings": {"default": {"default": "Schwerer schwarzgefiederter Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27B9",
      "mappings": {"default": {"default": "Schwerer schwarzgefiederter Nordostpfeil"}}
    }, {
      "category": "So",
      "key": "27BA",
      "mappings": {"default": {"default": "Teardrop-Barbed-Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27BB",
      "mappings": {"default": {"default": "Schwerer Tropfen-Pfeil nach rechts"}}
    }, {
      "category": "So",
      "key": "27BC",
      "mappings": {"default": {"default": "Keil-Schwanz nach rechts Pfeil"}}
    }, {
      "category": "So",
      "key": "27BD",
      "mappings": {"default": {"default": "Schwerer, nach rechts gekeilter Pfeil"}}
    }, {
      "category": "So",
      "key": "27BE",
      "mappings": {"default": {"default": "Offener Pfeil nach rechts"}}
    }, {
      "category": "Sm",
      "key": "27F0",
      "mappings": {"default": {"default": "Vierfachpfeil nach oben"}}
    }, {
      "category": "Sm",
      "key": "27F1",
      "mappings": {"default": {"default": "Vierfacher Pfeil nach unten"}}
    }, {
      "category": "Sm",
      "key": "27F2",
      "mappings": {"default": {"default": "Kreispfeil gegen den Uhrzeigersinn"}}
    }, {
      "category": "Sm",
      "key": "27F3",
      "mappings": {"default": {"default": "Kreispfeil im Uhrzeigersinn"}}
    }, {
      "category": "Sm",
      "key": "27F4",
      "mappings": {"default": {"default": "rechter Pfeil mit Circled Plus"}}
    }, {
      "category": "Sm",
      "key": "27F5",
      "mappings": {"default": {"default": "Langer Pfeil nach links"}}
    }, {
      "category": "Sm",
      "key": "27F6",
      "mappings": {"default": {"default": "Langer Pfeil nach rechts"}}
    }, {
      "category": "Sm",
      "key": "27F7",
      "mappings": {"default": {"default": "Langer linker rechter Pfeil"}}
    }, {
      "category": "Sm",
      "key": "27F8",
      "mappings": {"default": {"default": "Langer Doppelpfeil nach links"}}
    }, {
      "category": "Sm",
      "key": "27F9",
      "mappings": {"default": {"default": "Langer Doppelpfeil nach rechts"}}
    }, {
      "category": "Sm",
      "key": "27FA",
      "mappings": {"default": {"default": "Langer linker rechter Doppelpfeil"}}
    }, {
      "category": "Sm",
      "key": "27FB",
      "mappings": {"default": {"default": "Langer Pfeil nach links von der Bar"}}
    }, {
      "category": "Sm",
      "key": "27FC",
      "mappings": {"default": {"default": "Langer Rechtspfeil von der Bar"}}
    }, {
      "category": "Sm",
      "key": "27FD",
      "mappings": {"default": {"default": "Langer Doppelpfeil nach links von der Bar"}}
    }, {
      "category": "Sm",
      "key": "27FE",
      "mappings": {"default": {"default": "Langer Doppelpfeil nach rechts von der Bar"}}
    }, {
      "category": "Sm",
      "key": "27FF",
      "mappings": {"default": {"default": "Langer nach rechts gerichteter Squiggle-Pfeil"}}
    }, {
      "category": "Sm",
      "key": "2900",
      "mappings": {"default": {"default": "Pfeil nach rechts mit senkrechtem Strich"}}
    }, {
      "category": "Sm",
      "key": "2901",
      "mappings": {"default": {"default": "Zwei Pfeil nach rechts mit doppeltem vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "2902",
      "mappings": {"default": {"default": "Doppelpfeil nach links mit vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "2903",
      "mappings": {"default": {"default": "Doppelpfeil nach rechts mit vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "2904",
      "mappings": {"default": {"default": "Linker rechter Doppelpfeil mit vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "2905",
      "mappings": {"default": {"default": "Zwei Pfeil nach rechts von der Bar"}}
    }, {
      "category": "Sm",
      "key": "2906",
      "mappings": {"default": {"default": "Doppelpfeil nach links von der Bar"}}
    }, {
      "category": "Sm",
      "key": "2907",
      "mappings": {"default": {"default": "Doppelter Pfeil nach rechts von der Bar"}}
    }, {
      "category": "Sm",
      "key": "2908",
      "mappings": {"default": {"default": "Abwärtspfeil mit horizontalem Strich"}}
    }, {
      "category": "Sm",
      "key": "2909",
      "mappings": {"default": {"default": "Aufwärtspfeil mit horizontalem Anschlag"}}
    }, {
      "category": "Sm",
      "key": "290A",
      "mappings": {"default": {"default": "Dreifachpfeil nach oben"}}
    }, {
      "category": "Sm",
      "key": "290B",
      "mappings": {"default": {"default": "Dreifacher Pfeil nach unten"}}
    }, {
      "category": "Sm",
      "key": "290C",
      "mappings": {"default": {"default": "Doppelpfeil nach links"}}
    }, {
      "category": "Sm",
      "key": "290D",
      "mappings": {"default": {"default": "Doppelpfeil nach rechts"}}
    }, {
      "category": "Sm",
      "key": "290E",
      "mappings": {"default": {"default": "Dreifacher Strichpfeil nach links"}}
    }, {
      "category": "Sm",
      "key": "290F",
      "mappings": {"default": {"default": "Dreifacher Bindestrich nach rechts"}}
    }, {
      "category": "Sm",
      "key": "2910",
      "mappings": {"default": {"default": "Dreipoliger Pfeil nach rechts"}}
    }, {
      "category": "Sm",
      "key": "2911",
      "mappings": {"default": {"default": "Pfeil nach rechts mit gepunktetem Stamm"}}
    }, {
      "category": "Sm",
      "key": "2912",
      "mappings": {"default": {"default": "Aufwärtspfeil zur Bar"}}
    }, {
      "category": "Sm",
      "key": "2913",
      "mappings": {"default": {"default": "Abwärtspfeil zur Bar"}}
    }, {
      "category": "Sm",
      "key": "2914",
      "mappings": {"default": {"default": "Pfeil nach rechts mit Schwanz mit senkrechtem Anschlag"}}
    }, {
      "category": "Sm",
      "key": "2915",
      "mappings": {"default": {"default": "Pfeil nach rechts mit Schwanz mit doppeltem vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "2916",
      "mappings": {"default": {"default": "Nach rechts gerichteter zweiköpfiger Pfeil mit Schwanz"}}
    }, {
      "category": "Sm",
      "key": "2917",
      "mappings": {"default": {"default": "Nach rechts gerichteter Doppelpfeil mit Schwanz mit senkrechtem Anschlag"}}
    }, {
      "category": "Sm",
      "key": "2918",
      "mappings": {"default": {"default": "Nach rechts gerichteter Doppelpfeil mit Schwanz mit doppeltem vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "2919",
      "mappings": {"default": {"default": "Pfeil nach links nach links"}}
    }, {
      "category": "Sm",
      "key": "291A",
      "mappings": {"default": {"default": "Pfeil nach rechts nach rechts"}}
    }, {
      "category": "Sm",
      "key": "291B",
      "mappings": {"default": {"default": "Doppelter Pfeilschwanz nach links"}}
    }, {
      "category": "Sm",
      "key": "291C",
      "mappings": {"default": {"default": "Doppelter Pfeilschwanz nach rechts"}}
    }, {
      "category": "Sm",
      "key": "291D",
      "mappings": {"default": {"default": "Pfeil nach links zu Black Diamond"}}
    }, {
      "category": "Sm",
      "key": "291E",
      "mappings": {"default": {"default": "Pfeil nach rechts zu Black Diamond"}}
    }, {
      "category": "Sm",
      "key": "291F",
      "mappings": {"default": {"default": "Pfeil nach links von Bar zu Black Diamond"}}
    }, {
      "category": "Sm",
      "key": "2920",
      "mappings": {"default": {"default": "Pfeil nach rechts von Bar zu Black Diamond"}}
    }, {
      "category": "Sm",
      "key": "2921",
      "mappings": {"default": {"default": "Nordwest- und Südostpfeil"}}
    }, {
      "category": "Sm",
      "key": "2922",
      "mappings": {"default": {"default": "Nordost- und Südwestpfeil"}}
    }, {
      "category": "Sm",
      "key": "2923",
      "mappings": {"default": {"default": "Nordwestpfeil mit Haken"}}
    }, {
      "category": "Sm",
      "key": "2924",
      "mappings": {"default": {"default": "Nordostpfeil mit Haken"}}
    }, {
      "category": "Sm",
      "key": "2925",
      "mappings": {"default": {"default": "Südostpfeil mit Haken"}}
    }, {
      "category": "Sm",
      "key": "2926",
      "mappings": {"default": {"default": "Südwestpfeil mit Haken"}}
    }, {
      "category": "Sm",
      "key": "2927",
      "mappings": {"default": {"default": "Nordwestpfeil und Nordostpfeil"}}
    }, {
      "category": "Sm",
      "key": "2928",
      "mappings": {"default": {"default": "Nordostpfeil und Südostpfeil"}}
    }, {
      "category": "Sm",
      "key": "2929",
      "mappings": {"default": {"default": "Südostpfeil und Südwestpfeil"}}
    }, {
      "category": "Sm",
      "key": "292A",
      "mappings": {"default": {"default": "Südwestpfeil und Nordwestpfeil"}}
    }, {
      "category": "Sm",
      "key": "292D",
      "mappings": {"default": {"default": "Südostpfeil Kreuzung Nordostpfeil"}}
    }, {
      "category": "Sm",
      "key": "292E",
      "mappings": {"default": {"default": "Nordostpfeil Kreuzung Südostpfeil"}}
    }, {
      "category": "Sm",
      "key": "292F",
      "mappings": {"default": {"default": "Fallende Diagonale, die Nordostpfeil kreuzt"}}
    }, {
      "category": "Sm",
      "key": "2930",
      "mappings": {"default": {"default": "Steigende Diagonale, die Südostpfeil kreuzt"}}
    }, {
      "category": "Sm",
      "key": "2931",
      "mappings": {"default": {"default": "Nordostpfeilüberquerung Nordwestpfeil"}}
    }, {
      "category": "Sm",
      "key": "2932",
      "mappings": {"default": {"default": "Nordwestpfeilüberquerung Nordostpfeil"}}
    }, {
      "category": "Sm",
      "key": "2933",
      "mappings": {"default": {"default": "Wellenpfeil, der direkt nach rechts zeigt"}}
    }, {
      "category": "Sm",
      "key": "2934",
      "mappings": {"default": {"default": "Pfeil nach rechts, dann nach oben gekrümmt"}}
    }, {
      "category": "Sm",
      "key": "2935",
      "mappings": {"default": {"default": "Pfeil nach rechts, dann nach unten gekrümmt"}}
    }, {
      "category": "Sm",
      "key": "2936",
      "mappings": {"default": {"default": "Pfeil nach unten, dann nach links gekrümmt"}}
    }, {
      "category": "Sm",
      "key": "2937",
      "mappings": {"default": {"default": "Pfeil nach unten, dann nach rechts gekrümmt"}}
    }, {
      "category": "Sm",
      "key": "2938",
      "mappings": {"default": {"default": "Rechtsseitiger Bogen im Uhrzeigersinn"}}
    }, {
      "category": "Sm",
      "key": "2939",
      "mappings": {"default": {"default": "Bogen der linken Seite gegen den Uhrzeigersinn"}}
    }, {
      "category": "Sm",
      "key": "293A",
      "mappings": {"default": {"default": "Pfeil oben gegen den Uhrzeigersinn"}}
    }, {
      "category": "Sm",
      "key": "293B",
      "mappings": {"default": {"default": "Pfeil unten gegen den Uhrzeigersinn"}}
    }, {
      "category": "Sm",
      "key": "293C",
      "mappings": {"default": {"default": "Pfeil oben im Uhrzeigersinn mit Minus"}}
    }, {
      "category": "Sm",
      "key": "293D",
      "mappings": {"default": {"default": "Pfeil oben gegen den Uhrzeigersinn mit Plus"}}
    }, {
      "category": "Sm",
      "key": "293E",
      "mappings": {"default": {"default": "Rechts unten halbkreisförmig im Uhrzeigersinn"}}
    }, {
      "category": "Sm",
      "key": "293F",
      "mappings": {"default": {"default": "Linker halbkreisförmiger Pfeil gegen den Uhrzeigersinn"}}
    }, {
      "category": "Sm",
      "key": "2940",
      "mappings": {"default": {"default": "Geschlossener Kreispfeil gegen den Uhrzeigersinn"}}
    }, {
      "category": "Sm",
      "key": "2941",
      "mappings": {"default": {"default": "Geschlossener Kreispfeil im Uhrzeigersinn"}}
    }, {
      "category": "Sm",
      "key": "2942",
      "mappings": {"default": {"default": "Pfeil nach rechts über Kurzer Pfeil nach links"}}
    }, {
      "category": "Sm",
      "key": "2943",
      "mappings": {"default": {"default": "Pfeil nach links über Kurzer Pfeil nach rechts"}}
    }, {
      "category": "Sm",
      "key": "2944",
      "mappings": {"default": {"default": "Kurzer Pfeil nach rechts Über Pfeil nach links"}}
    }, {
      "category": "Sm",
      "key": "2945",
      "mappings": {"default": {"default": "Pfeil nach rechts mit Plus unten"}}
    }, {
      "category": "Sm",
      "key": "2946",
      "mappings": {"default": {"default": "Pfeil nach links mit Plus unten"}}
    }, {
      "category": "Sm",
      "key": "2947",
      "mappings": {"default": {"default": "Pfeil nach rechts durch X"}}
    }, {
      "category": "Sm",
      "key": "2948",
      "mappings": {"default": {"default": "Linker rechter Pfeil durch kleinen Kreis"}}
    }, {
      "category": "Sm",
      "key": "2949",
      "mappings": {"default": {"default": "Aufwärts zweiköpfiger Pfeil vom kleinen Kreis"}}
    }, {"key": "2970", "mappings": {"default": {"default": "round implies"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "2971",
      "mappings": {"default": {"default": "Gleichheitszeichen über dem Pfeil nach rechts"}}
    }, {
      "category": "Sm",
      "key": "2972",
      "mappings": {"default": {"default": "Tilde-Operator über dem Pfeil nach rechts"}}
    }, {
      "category": "Sm",
      "key": "2973",
      "mappings": {"default": {"default": "Pfeil nach links über Tilde-Operator"}}
    }, {
      "category": "Sm",
      "key": "2974",
      "mappings": {"default": {"default": "Pfeil nach rechts über Tilde-Operator"}}
    }, {
      "category": "Sm",
      "key": "2975",
      "mappings": {"default": {"default": "Pfeil nach rechts fast gleich"}}
    }, {
      "category": "Sm",
      "key": "2976",
      "mappings": {"default": {"default": "Weniger als oben Pfeil nach links"}}
    }, {
      "category": "Sm",
      "key": "2977",
      "mappings": {"default": {"default": "Pfeil nach links durch weniger als"}}
    }, {
      "category": "Sm",
      "key": "2978",
      "mappings": {"default": {"default": "Größer als über dem Pfeil nach rechts"}}
    }, {
      "category": "Sm",
      "key": "2979",
      "mappings": {"default": {"default": "Untersatz über dem Pfeil nach rechts"}}
    }, {
      "category": "Sm",
      "key": "297A",
      "mappings": {"default": {"default": "Pfeil nach links durch Teilmenge"}}
    }, {
      "category": "Sm",
      "key": "297B",
      "mappings": {"default": {"default": "Superset über dem Pfeil nach links"}}
    }, {
      "category": "Sm",
      "key": "29B3",
      "mappings": {"default": {"default": "Leeres Set mit Pfeil rechts oben"}}
    }, {
      "category": "Sm",
      "key": "29B4",
      "mappings": {"default": {"default": "Leeres Set mit linkem Pfeil oben"}}
    }, {
      "category": "Sm",
      "key": "29BD",
      "mappings": {"default": {"default": "Pfeil nach oben durch Kreis"}}
    }, {
      "category": "Sm",
      "key": "29EA",
      "mappings": {"default": {"default": "Schwarzer Diamant mit Pfeil nach unten"}}
    }, {
      "category": "Sm",
      "key": "29EC",
      "mappings": {"default": {"default": "Weißer Kreis mit Pfeil nach unten"}}
    }, {
      "category": "Sm",
      "key": "29ED",
      "mappings": {"default": {"default": "Schwarzer Kreis mit Pfeil nach unten"}}
    }, {
      "category": "Sm",
      "key": "2A17",
      "mappings": {"default": {"default": "Integral mit Pfeil nach links mit Haken"}}
    }, {
      "category": "So",
      "key": "2B00",
      "mappings": {"default": {"default": "North East White Arrow"}}
    }, {
      "category": "So",
      "key": "2B01",
      "mappings": {"default": {"default": "Nordwestlicher weißer Pfeil"}}
    }, {"category": "So", "key": "2B02", "mappings": {"default": {"default": "Südostweißer Pfeil"}}}, {
      "category": "So",
      "key": "2B03",
      "mappings": {"default": {"default": "Südwestweißer Pfeil"}}
    }, {
      "category": "So",
      "key": "2B04",
      "mappings": {"default": {"default": "Linker rechter weißer Pfeil"}}
    }, {
      "category": "So",
      "key": "2B05",
      "mappings": {"default": {"default": "Schwarzer Pfeil nach links"}}
    }, {
      "category": "So",
      "key": "2B06",
      "mappings": {"default": {"default": "Aufwärts schwarzer Pfeil"}}
    }, {
      "category": "So",
      "key": "2B07",
      "mappings": {"default": {"default": "Abwärts schwarzer Pfeil"}}
    }, {
      "category": "So",
      "key": "2B08",
      "mappings": {"default": {"default": "Schwarzer Nordostpfeil"}}
    }, {
      "category": "So",
      "key": "2B09",
      "mappings": {"default": {"default": "Nordwestlicher schwarzer Pfeil"}}
    }, {
      "category": "So",
      "key": "2B0A",
      "mappings": {"default": {"default": "Schwarzer Südostpfeil"}}
    }, {
      "category": "So",
      "key": "2B0B",
      "mappings": {"default": {"default": "Südwestlicher schwarzer Pfeil"}}
    }, {
      "category": "So",
      "key": "2B0C",
      "mappings": {"default": {"default": "Linker rechter schwarzer Pfeil"}}
    }, {
      "category": "So",
      "key": "2B0D",
      "mappings": {"default": {"default": "Schwarzer Pfeil nach unten"}}
    }, {
      "category": "So",
      "key": "2B0E",
      "mappings": {"default": {"default": "Pfeil nach rechts mit Spitze nach unten"}}
    }, {
      "category": "So",
      "key": "2B0F",
      "mappings": {"default": {"default": "Pfeil nach rechts mit Spitze nach oben"}}
    }, {
      "category": "So",
      "key": "2B10",
      "mappings": {"default": {"default": "Pfeil nach links mit Spitze nach unten"}}
    }, {
      "category": "So",
      "key": "2B11",
      "mappings": {"default": {"default": "Pfeil nach links mit Spitze nach oben"}}
    }, {
      "category": "Sm",
      "key": "2B30",
      "mappings": {"default": {"default": "Linker Pfeil mit kleinem Kreis"}}
    }, {
      "category": "Sm",
      "key": "2B31",
      "mappings": {"default": {"default": "Drei nach links gerichtete Pfeile"}}
    }, {
      "category": "Sm",
      "key": "2B32",
      "mappings": {"default": {"default": "Linker Pfeil mit Circled Plus"}}
    }, {
      "category": "Sm",
      "key": "2B33",
      "mappings": {"default": {"default": "Langer nach links gerichteter Squiggle-Pfeil"}}
    }, {
      "category": "Sm",
      "key": "2B34",
      "mappings": {"default": {"default": "Pfeil nach links mit senkrechtem Strich"}}
    }, {
      "category": "Sm",
      "key": "2B35",
      "mappings": {"default": {"default": "Doppelpfeil nach links mit doppeltem vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "2B36",
      "mappings": {"default": {"default": "Doppelpfeil nach links von der Bar"}}
    }, {
      "category": "Sm",
      "key": "2B37",
      "mappings": {"default": {"default": "Pfeil nach links mit zwei Köpfen"}}
    }, {
      "category": "Sm",
      "key": "2B38",
      "mappings": {"default": {"default": "Pfeil nach links mit gepunktetem Stamm"}}
    }, {
      "category": "Sm",
      "key": "2B39",
      "mappings": {"default": {"default": "Pfeil nach links mit Schwanz mit vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "2B3A",
      "mappings": {"default": {"default": "Pfeil nach links mit Schwanz mit doppeltem vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "2B3B",
      "mappings": {"default": {"default": "Nach links gerichteter zweiköpfiger Pfeil mit Schwanz"}}
    }, {
      "category": "Sm",
      "key": "2B3C",
      "mappings": {"default": {"default": "Nach links gerichteter zweiköpfiger Pfeil mit Schwanz mit vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "2B3D",
      "mappings": {"default": {"default": "Nach links gerichteter zweiköpfiger Pfeil mit Schwanz mit doppeltem vertikalem Strich"}}
    }, {
      "category": "Sm",
      "key": "2B3E",
      "mappings": {"default": {"default": "Pfeil nach links durch X"}}
    }, {
      "category": "Sm",
      "key": "2B3F",
      "mappings": {"default": {"default": "Wellenpfeil, der direkt nach links zeigt"}}
    }, {
      "category": "Sm",
      "key": "2B40",
      "mappings": {"default": {"default": "Gleichheitszeichen über dem Pfeil nach links"}}
    }, {
      "category": "Sm",
      "key": "2B41",
      "mappings": {"default": {"default": "Reverse-Tilde-Operator Über Linkspfeil"}}
    }, {
      "category": "Sm",
      "key": "2B42",
      "mappings": {"default": {"default": "Pfeil nach links über der Rückseite fast gleich"}}
    }, {
      "category": "Sm",
      "key": "2B43",
      "mappings": {"default": {"default": "Pfeil nach rechts durch mehr als"}}
    }, {
      "category": "Sm",
      "key": "2B44",
      "mappings": {"default": {"default": "Pfeil nach rechts durch Superset"}}
    }, {
      "category": "So",
      "key": "2B45",
      "mappings": {"default": {"default": "Vierfachpfeil nach links"}}
    }, {
      "category": "So",
      "key": "2B46",
      "mappings": {"default": {"default": "Vierfachpfeil nach rechts"}}
    }, {
      "category": "Sm",
      "key": "2B47",
      "mappings": {"default": {"default": "Reverse-Tilde-Operator über dem Pfeil nach rechts"}}
    }, {
      "category": "Sm",
      "key": "2B48",
      "mappings": {"default": {"default": "Pfeil nach rechts oben fast gleich"}}
    }, {
      "category": "Sm",
      "key": "2B49",
      "mappings": {"default": {"default": "Tilde-Operator über dem Pfeil nach links"}}
    }, {
      "category": "Sm",
      "key": "2B4A",
      "mappings": {"default": {"default": "Pfeil nach links oben fast gleich"}}
    }, {
      "category": "Sm",
      "key": "2B4B",
      "mappings": {"default": {"default": "Pfeil nach links über Reverse-Tilde-Operator"}}
    }, {
      "category": "Sm",
      "key": "2B4C",
      "mappings": {"default": {"default": "Pfeil nach rechts über Reverse-Tilde-Operator"}}
    }, {
      "category": "Sm",
      "key": "FFE9",
      "mappings": {"default": {"default": "Halbbreiter Pfeil nach links"}}
    }, {
      "category": "Sm",
      "key": "FFEA",
      "mappings": {"default": {"default": "Halbbreiter Pfeil nach oben"}}
    }, {
      "category": "Sm",
      "key": "FFEB",
      "mappings": {"default": {"default": "Halbbreiter Pfeil nach rechts"}}
    }, {"category": "Sm", "key": "FFEC", "mappings": {"default": {"default": "Halbbreiter Pfeil nach unten"}}}]
  ,
  "de/symbols/math_characters.js":
    [{"locale": "de"}, {
      "category": "Ll",
      "key": "2113",
      "mappings": {"default": {"default": "Skript klein L"}}
    }, {"category": "Sm", "key": "2118", "mappings": {"default": {"default": "Skript groß P"}}}, {
      "category": "Ll",
      "key": "213C",
      "mappings": {"default": {"default": "pi mit Doppelstrich"}}
    }, {
      "category": "Ll",
      "key": "213D",
      "mappings": {"default": {"default": "gamma mit Doppelstrich"}}
    }, {
      "category": "Lu",
      "key": "213E",
      "mappings": {"default": {"default": "groß Gamma mit Doppelstrich"}}
    }, {
      "category": "Lu",
      "key": "213F",
      "mappings": {"default": {"default": "groß Pi mit Doppelstrich"}}
    }, {
      "category": "Sm",
      "key": "2140",
      "mappings": {"default": {"default": "Summenzeichen mit Doppelstrich"}}
    }, {
      "category": "Lu",
      "key": "2145",
      "mappings": {"default": {"default": "groß D kursiv mit Doppelstrich"}}
    }, {
      "category": "Ll",
      "mappings": {"default": {"default": "d kursiv mit Doppelstrich"}},
      "key": "2146"
    }, {
      "category": "Ll",
      "mappings": {"default": {"default": "e kursiv mit Doppelstrich"}},
      "key": "2147"
    }, {
      "category": "Ll",
      "mappings": {"default": {"default": "i kursiv mit Doppelstrich"}},
      "key": "2148"
    }, {
      "category": "Ll",
      "key": "2149",
      "mappings": {"default": {"default": "j kursiv mit Doppelstrich"}}
    }, {
      "category": "Ll",
      "key": "1D6A4",
      "mappings": {"default": {"default": "punktloses i kursiv"}}
    }, {"category": "Ll", "key": "1D6A5", "mappings": {"default": {"default": "punktloses j kursiv"}}}]
  ,
  "de/symbols/math_delimiters.js":
    [{"locale": "de"}, {
      "key": "0028",
      "mappings": {"default": {"default": "Klammer auf"}},
      "category": "Ps"
    }, {"key": "0029", "mappings": {"default": {"default": "Klammer zu"}}, "category": "Pe"}, {
      "key": "005B",
      "mappings": {"default": {"default": "eckige Klammer auf"}},
      "category": "Ps"
    }, {"key": "005D", "mappings": {"default": {"default": "eckige Klammer zu"}}, "category": "Pe"}, {
      "key": "007B",
      "mappings": {"default": {"default": "geschwungene Klammer auf"}},
      "category": "Ps"
    }, {
      "key": "007D",
      "mappings": {"default": {"default": "geschwungene Klammer zu"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "2045",
      "mappings": {"default": {"default": "Linke eckige Klammer mit Stachel"}}
    }, {
      "category": "Pe",
      "key": "2046",
      "mappings": {"default": {"default": "Rechte eckige Klammer mit Stachel"}}
    }, {
      "category": "Sm",
      "key": "2308",
      "mappings": {"default": {"default": "linke Aufrundungsklammer"}}
    }, {
      "category": "Sm",
      "key": "2309",
      "mappings": {"default": {"default": "rechte Aufrundungsklammer"}}
    }, {
      "category": "Sm",
      "key": "230A",
      "mappings": {"default": {"default": "linke Abrundungsklammer"}}
    }, {
      "category": "Sm",
      "key": "230B",
      "mappings": {"default": {"default": "rechter Abrundungsklammer"}}
    }, {
      "category": "So",
      "key": "230C",
      "mappings": {"default": {"default": "unterer rechter Abschnitt"}}
    }, {
      "category": "So",
      "key": "230D",
      "mappings": {"default": {"default": "Unterer linker Abschnitt"}}
    }, {
      "category": "So",
      "key": "230E",
      "mappings": {"default": {"default": "Oberer rechter Abschnitt"}}
    }, {
      "category": "So",
      "key": "230F",
      "mappings": {"default": {"default": "Oberer linker Abschnitt"}}
    }, {"category": "So", "key": "231C", "mappings": {"default": {"default": "Obere linke Ecke"}}}, {
      "category": "So",
      "key": "231D",
      "mappings": {"default": {"default": "Obere rechte Ecke"}}
    }, {"category": "So", "key": "231E", "mappings": {"default": {"default": "Untere linke Ecke"}}}, {
      "category": "So",
      "key": "231F",
      "mappings": {"default": {"default": "Rechte untere Ecke"}}
    }, {
      "category": "Sm",
      "key": "2320",
      "mappings": {"default": {"default": "Obere Hälfte eines Integrals"}}
    }, {
      "category": "Sm",
      "key": "2321",
      "mappings": {"default": {"default": "Untere Hälfte eines Integrals"}}
    }, {
      "category": "Ps",
      "key": "2329",
      "mappings": {"default": {"default": "Linke spitze Klammer"}}
    }, {
      "category": "Pe",
      "key": "232A",
      "mappings": {"default": {"default": "Rechte spitze Klammer"}}
    }, {
      "category": "Sm",
      "key": "239B",
      "mappings": {"default": {"default": "Oberer Teil der linken Klammer"}}
    }, {
      "category": "Sm",
      "key": "239C",
      "mappings": {"default": {"default": "Linke Klammerverlängerung"}}
    }, {
      "category": "Sm",
      "key": "239D",
      "mappings": {"default": {"default": "Unterer Teil der linken Klammer"}}
    }, {
      "category": "Sm",
      "key": "239E",
      "mappings": {"default": {"default": "Oberer Teil der rechten Klammer"}}
    }, {
      "category": "Sm",
      "key": "239F",
      "mappings": {"default": {"default": "Rechte Klammerverlängerung"}}
    }, {
      "category": "Sm",
      "key": "23A0",
      "mappings": {"default": {"default": "Unterer Teil der rechten Klammer"}}
    }, {
      "category": "Sm",
      "key": "23A1",
      "mappings": {"default": {"default": "Oberer Teil der linken eckigen Klammer"}}
    }, {
      "category": "Sm",
      "key": "23A2",
      "mappings": {"default": {"default": "Linke eckige Klammerverlängerung"}}
    }, {
      "category": "Sm",
      "key": "23A3",
      "mappings": {"default": {"default": "unterer Teil der linken eckigen Klammer"}}
    }, {
      "category": "Sm",
      "key": "23A4",
      "mappings": {"default": {"default": "Oberer Teil der rechten eckigen Klammer"}}
    }, {
      "category": "Sm",
      "key": "23A5",
      "mappings": {"default": {"default": "Rechte eckige Klammerverlängerung"}}
    }, {
      "category": "Sm",
      "key": "23A6",
      "mappings": {"default": {"default": "unterer Teil der rechten eckigen Klammer"}}
    }, {
      "category": "Sm",
      "key": "23A7",
      "mappings": {"default": {"default": "oberer Teil der linken geschweiften Klammer"}}
    }, {
      "category": "Sm",
      "key": "23A8",
      "mappings": {"default": {"default": "Mittelstück der linken geschweiften Klammer"}}
    }, {
      "category": "Sm",
      "key": "23A9",
      "mappings": {"default": {"default": "unterer Teil der linken geschweiften Klammer"}}
    }, {
      "category": "Sm",
      "key": "23AA",
      "mappings": {"default": {"default": "Zwischenteil einer geschweiften Klammer"}}
    }, {
      "category": "Sm",
      "key": "23AB",
      "mappings": {"default": {"default": "oberer Teil der rechten geschweiften Klammer"}}
    }, {
      "category": "Sm",
      "key": "23AC",
      "mappings": {"default": {"default": "Mittelstück der rechten geschweiften Klammer"}}
    }, {
      "category": "Sm",
      "key": "23AD",
      "mappings": {"default": {"default": "unterer Teil der rechten geschweiften Klammer"}}
    }, {
      "category": "Sm",
      "key": "23AE",
      "mappings": {"default": {"default": "Mittelteil eines Integrals"}}
    }, {
      "category": "Sm",
      "key": "23AF",
      "mappings": {"default": {"default": "Erweiterung einer Querlinie"}}
    }, {
      "category": "Sm",
      "key": "23B0",
      "mappings": {"default": {"default": "Geschweiftes Klammerteil oben links oder rechts unten"}}
    }, {
      "category": "Sm",
      "key": "23B1",
      "mappings": {"default": {"default": "Geschweiftes Klammerteil oben rechts oder unten links"}}
    }, {
      "category": "Sm",
      "key": "23B2",
      "mappings": {"default": {"default": "Oberer Teil einer Summe"}}
    }, {
      "category": "Sm",
      "key": "23B3",
      "mappings": {"default": {"default": "Unterer Teil einer Summe"}}
    }, {"key": "23B4", "mappings": {"default": {"default": "Obere eckige Klammer"}}, "category": "So"}, {
      "key": "23B5",
      "mappings": {"default": {"default": "Untere eckige Klammer"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "23B6",
      "mappings": {"default": {"default": "Untere eckige Klammer auf oberer eckiger Klammer"}}
    }, {
      "category": "So",
      "key": "23B7",
      "mappings": {"default": {"default": "Unterer Teil des Radikalsymbols"}}
    }, {
      "category": "So",
      "key": "23B8",
      "mappings": {"default": {"default": "Linke vertikale Boxlinie"}}
    }, {
      "category": "So",
      "key": "23B9",
      "mappings": {"default": {"default": "Rechte vertikale Boxlinie"}}
    }, {"category": "Sm", "key": "23DC", "mappings": {"default": {"default": "Obere Klammer"}}}, {
      "category": "Sm",
      "key": "23DD",
      "mappings": {"default": {"default": "Untere Klammer"}}
    }, {
      "category": "Sm",
      "key": "23DE",
      "mappings": {"default": {"default": "Obere geschweifte Klammer"}}
    }, {
      "category": "Sm",
      "key": "23DF",
      "mappings": {"default": {"default": "Untere geschweifte Klammer"}}
    }, {
      "category": "Sm",
      "key": "23E0",
      "mappings": {"default": {"default": "Obere eckige Klammer"}}
    }, {
      "category": "Sm",
      "key": "23E1",
      "mappings": {"default": {"default": "Untere eckige Klammer"}}
    }, {
      "category": "Ps",
      "key": "2768",
      "mappings": {"default": {"default": "Mittlere linke Klammerverzierung"}}
    }, {
      "category": "Pe",
      "key": "2769",
      "mappings": {"default": {"default": "Mittlere rechte Klammerverzierung"}}
    }, {
      "category": "Ps",
      "key": "276A",
      "mappings": {"default": {"default": "Mittlere abgeflachte linke Klammerverzierung"}}
    }, {
      "category": "Pe",
      "key": "276B",
      "mappings": {"default": {"default": "Mittlere abgeflachte rechte Klammerverzierung"}}
    }, {
      "category": "Ps",
      "key": "276C",
      "mappings": {"default": {"default": "Mittlere nach links weisende Winkelhalterung"}}
    }, {
      "category": "Pe",
      "key": "276D",
      "mappings": {"default": {"default": "Mittlere, rechtwinklige Winkelhalterung"}}
    }, {
      "category": "Ps",
      "key": "276E",
      "mappings": {"default": {"default": "Schwere nach links weisende Anführungszeichen-Verzierung"}}
    }, {
      "category": "Pe",
      "key": "276F",
      "mappings": {"default": {"default": "Schwere nach rechts zeigenden Winkel-Anführungszeichen-Verzierung"}}
    }, {
      "category": "Ps",
      "key": "2770",
      "mappings": {"default": {"default": "Schwere nach links weisende Winkelhalterung"}}
    }, {
      "category": "Pe",
      "key": "2771",
      "mappings": {"default": {"default": "Schwere, nach rechts weisende Winkelhalterung"}}
    }, {
      "category": "Ps",
      "key": "2772",
      "mappings": {"default": {"default": "Leichte linke Schildpatt Bracket Ornament"}}
    }, {
      "category": "Pe",
      "key": "2773",
      "mappings": {"default": {"default": "Leichte rechte Schildpatt-Klammerverzierung"}}
    }, {
      "category": "Ps",
      "key": "2774",
      "mappings": {"default": {"default": "Mittlere linke geschweifte Klammerverzierung"}}
    }, {
      "category": "Pe",
      "key": "2775",
      "mappings": {"default": {"default": "Mittlere rechte geschweifte Klammerverzierung"}}
    }, {
      "category": "Ps",
      "key": "27C5",
      "mappings": {"default": {"default": "Linker S-förmiger Taschenbegrenzer"}}
    }, {
      "category": "Pe",
      "key": "27C6",
      "mappings": {"default": {"default": "rechter S-förmiger Taschenbegrenzer"}}
    }, {
      "category": "Ps",
      "key": "27E6",
      "mappings": {"default": {"default": "Mathematische linke weiße eckige Klammer"}}
    }, {
      "category": "Pe",
      "key": "27E7",
      "mappings": {"default": {"default": "Mathematische rechte weiße eckige Klammer"}}
    }, {
      "category": "Ps",
      "key": "27E8",
      "mappings": {"default": {"default": "Mathematische linke Winkelklammer"}}
    }, {
      "category": "Pe",
      "key": "27E9",
      "mappings": {"default": {"default": "Mathematische rechtwinklige Klammer"}}
    }, {
      "category": "Ps",
      "key": "27EA",
      "mappings": {"default": {"default": "Mathematische linke doppelte Winkelklammer"}}
    }, {
      "category": "Pe",
      "key": "27EB",
      "mappings": {"default": {"default": "Mathematische rechtwinklige eckige Klammer"}}
    }, {
      "category": "Ps",
      "key": "27EC",
      "mappings": {"default": {"default": "Mathematische linke Schildkrötenpanzerhalterung"}}
    }, {
      "category": "Pe",
      "key": "27ED",
      "mappings": {"default": {"default": "Mathematische rechte weiße Schildpatt-Klammer"}}
    }, {
      "category": "Ps",
      "key": "27EE",
      "mappings": {"default": {"default": "Mathematische linke abgeflachte Klammer"}}
    }, {
      "category": "Pe",
      "key": "27EF",
      "mappings": {"default": {"default": "Mathematische rechte abgeflachte Klammer"}}
    }, {
      "category": "Ps",
      "key": "2983",
      "mappings": {"default": {"default": "Linke weiße geschweifte Klammer"}}
    }, {
      "category": "Pe",
      "key": "2984",
      "mappings": {"default": {"default": "Rechte weiße geschweifte Klammer"}}
    }, {
      "category": "Ps",
      "key": "2985",
      "mappings": {"default": {"default": "Linke weiße Klammer"}}
    }, {
      "category": "Pe",
      "key": "2986",
      "mappings": {"default": {"default": "Rechte weiße Klammer"}}
    }, {
      "category": "Ps",
      "key": "2987",
      "mappings": {"default": {"default": "Z-Notation Linke Bildklammer"}}
    }, {
      "category": "Pe",
      "key": "2988",
      "mappings": {"default": {"default": "Z-Notation rechte Bildklammer"}}
    }, {
      "category": "Ps",
      "key": "2989",
      "mappings": {"default": {"default": "Z-Notation Linke Klammer"}}
    }, {
      "category": "Pe",
      "key": "298A",
      "mappings": {"default": {"default": "Z Notation Right Binding Bracket"}}
    }, {
      "category": "Ps",
      "key": "298B",
      "mappings": {"default": {"default": "Linke eckige Klammer mit Unterlenker"}}
    }, {
      "category": "Pe",
      "key": "298C",
      "mappings": {"default": {"default": "Rechte eckige Klammer mit Unterlenker"}}
    }, {
      "category": "Ps",
      "key": "298D",
      "mappings": {"default": {"default": "Linke eckige Klammer mit Häkchen in der oberen Ecke"}}
    }, {
      "category": "Pe",
      "key": "298E",
      "mappings": {"default": {"default": "Rechte eckige Klammer mit Tick in der unteren Ecke"}}
    }, {
      "category": "Ps",
      "key": "298F",
      "mappings": {"default": {"default": "Linke eckige Klammer mit Tick in der unteren Ecke"}}
    }, {
      "category": "Pe",
      "key": "2990",
      "mappings": {"default": {"default": "Rechte eckige Klammer mit Tick in der oberen Ecke"}}
    }, {
      "category": "Ps",
      "key": "2991",
      "mappings": {"default": {"default": "Linke Winkelklammer mit Punkt"}}
    }, {
      "category": "Pe",
      "key": "2992",
      "mappings": {"default": {"default": "Rechtwinklige Klammer mit Punkt"}}
    }, {
      "category": "Ps",
      "key": "2993",
      "mappings": {"default": {"default": "Linker Bogen weniger als Halterung"}}
    }, {
      "category": "Pe",
      "key": "2994",
      "mappings": {"default": {"default": "rechter Bogen größer als Halterung"}}
    }, {
      "category": "Ps",
      "key": "2995",
      "mappings": {"default": {"default": "Doppelter linker Bogen größer als Halterung"}}
    }, {
      "category": "Pe",
      "key": "2996",
      "mappings": {"default": {"default": "Doppelter rechter Bogen weniger als Halterung"}}
    }, {
      "category": "Ps",
      "key": "2997",
      "mappings": {"default": {"default": "Linke schwarze Schildkrötenhalterung"}}
    }, {
      "category": "Pe",
      "key": "2998",
      "mappings": {"default": {"default": "Rechte schwarze Schildpatt-Klammer"}}
    }, {
      "category": "Ps",
      "key": "29D8",
      "mappings": {"default": {"default": "Linker wackeliger Zaun"}}
    }, {
      "category": "Pe",
      "key": "29D9",
      "mappings": {"default": {"default": "rechter wackeliger Zaun"}}
    }, {
      "category": "Ps",
      "key": "29DA",
      "mappings": {"default": {"default": "Linker doppelter wackeliger Zaun"}}
    }, {
      "category": "Pe",
      "key": "29DB",
      "mappings": {"default": {"default": "rechter doppelter wackeliger Zaun"}}
    }, {
      "category": "Ps",
      "key": "29FC",
      "mappings": {"default": {"default": "Nach links zeigender gebogener Winkelhalter"}}
    }, {
      "category": "Pe",
      "key": "29FD",
      "mappings": {"default": {"default": "Nach rechts zeigender gebogener Winkelhalter"}}
    }, {
      "category": "Ps",
      "key": "2E22",
      "mappings": {"default": {"default": "Obere linke halbe Halterung"}}
    }, {
      "category": "Pe",
      "key": "2E23",
      "mappings": {"default": {"default": "Obere rechte halbe Klammer"}}
    }, {
      "category": "Ps",
      "key": "2E24",
      "mappings": {"default": {"default": "Linke halbe halterung unten"}}
    }, {
      "category": "Pe",
      "key": "2E25",
      "mappings": {"default": {"default": "Untere rechte halbe Halterung"}}
    }, {
      "category": "Ps",
      "key": "2E26",
      "mappings": {"default": {"default": "U-Halterung links seitlich"}}
    }, {
      "category": "Pe",
      "key": "2E27",
      "mappings": {"default": {"default": "U-Halterung rechts seitlich"}}
    }, {
      "category": "Ps",
      "key": "2E28",
      "mappings": {"default": {"default": "Linke doppelte Klammer"}}
    }, {
      "category": "Pe",
      "key": "2E29",
      "mappings": {"default": {"default": "Rechte doppelte Klammer"}}
    }, {
      "category": "Ps",
      "key": "3008",
      "mappings": {"default": {"default": "Linke Winkelklammer"}}
    }, {
      "category": "Pe",
      "key": "3009",
      "mappings": {"default": {"default": "Rechtwinklige Halterung"}}
    }, {
      "category": "Ps",
      "key": "300A",
      "mappings": {"default": {"default": "Linke Doppelwinkelhalterung"}}
    }, {
      "category": "Pe",
      "key": "300B",
      "mappings": {"default": {"default": "Rechte Doppelwinkelhalterung"}}
    }, {"category": "Ps", "key": "300C", "mappings": {"default": {"default": "Linke Eckklammer"}}}, {
      "category": "Pe",
      "key": "300D",
      "mappings": {"default": {"default": "Rechte Eckhalterung"}}
    }, {
      "category": "Ps",
      "key": "300E",
      "mappings": {"default": {"default": "Linke weiße Eckhalterung"}}
    }, {
      "category": "Pe",
      "key": "300F",
      "mappings": {"default": {"default": "Rechte weiße Eckhalterung"}}
    }, {
      "category": "Ps",
      "key": "3010",
      "mappings": {"default": {"default": "Linke schwarze linsenförmige Halterung"}}
    }, {
      "category": "Pe",
      "key": "3011",
      "mappings": {"default": {"default": "Rechte schwarze linsenförmige Halterung"}}
    }, {
      "category": "Ps",
      "key": "3014",
      "mappings": {"default": {"default": "Linke Schildpatt-Klammer"}}
    }, {
      "category": "Pe",
      "key": "3015",
      "mappings": {"default": {"default": "Rechte Schildpattlasche"}}
    }, {
      "category": "Ps",
      "key": "3016",
      "mappings": {"default": {"default": "Linke weiße linsenförmige Halterung"}}
    }, {
      "category": "Pe",
      "key": "3017",
      "mappings": {"default": {"default": "Rechte weiße linsenförmige Halterung"}}
    }, {
      "category": "Ps",
      "key": "3018",
      "mappings": {"default": {"default": "Linke weiße Schildpatt-Klammer"}}
    }, {
      "category": "Pe",
      "key": "3019",
      "mappings": {"default": {"default": "Rechte weiße Schildpatt-Klammer"}}
    }, {
      "category": "Ps",
      "key": "301A",
      "mappings": {"default": {"default": "Linke weiße eckige Klammer"}}
    }, {
      "category": "Pe",
      "key": "301B",
      "mappings": {"default": {"default": "Rechte weiße eckige Klammer"}}
    }, {
      "category": "Ps",
      "key": "301D",
      "mappings": {"default": {"default": "Doppelte Anführungszeichen in umgekehrter Reihenfolge"}}
    }, {
      "category": "Pe",
      "key": "301E",
      "mappings": {"default": {"default": "Doppelte Anführungszeichen"}}
    }, {
      "category": "Pe",
      "key": "301F",
      "mappings": {"default": {"default": "Niedriges doppeltes Prime-Anführungszeichen"}}
    }, {
      "category": "Ps",
      "key": "FD3E",
      "mappings": {"default": {"default": "Verzierte linke Klammer"}}
    }, {
      "category": "Pe",
      "key": "FD3F",
      "mappings": {"default": {"default": "Verzierte rechte Klammer"}}
    }, {
      "category": "Ps",
      "key": "FE17",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale linke weiße linsenförmige Halterung"}}
    }, {
      "category": "Pe",
      "key": "FE18",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale rechtwinklige linsenförmige Halterung"}}
    }, {
      "category": "Ps",
      "key": "FE35",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale linke Klammer"}}
    }, {
      "category": "Pe",
      "key": "FE36",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale rechte Klammer"}}
    }, {
      "category": "Ps",
      "key": "FE37",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale linke geschweifte Klammer"}}
    }, {
      "category": "Pe",
      "key": "FE38",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale, recht geschweifte Klammer"}}
    }, {
      "category": "Ps",
      "key": "FE39",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale linke Schildpattklammer"}}
    }, {
      "category": "Pe",
      "key": "FE3A",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale rechte Schildpatt-Klammer"}}
    }, {
      "category": "Ps",
      "key": "FE3B",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale linke schwarze linsenförmige Halterung"}}
    }, {
      "category": "Pe",
      "key": "FE3C",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale rechtwinklige schwarze linsenförmige Halterung"}}
    }, {
      "category": "Ps",
      "key": "FE3D",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale linke Doppelwinkelhalterung"}}
    }, {
      "category": "Pe",
      "key": "FE3E",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale rechtwinklige, rechtwinklige Halterung"}}
    }, {
      "category": "Ps",
      "key": "FE3F",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale linke Winkelhalterung"}}
    }, {
      "category": "Pe",
      "key": "FE40",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale rechtwinklige Halterung"}}
    }, {
      "category": "Ps",
      "key": "FE41",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale linke Eckhalterung"}}
    }, {
      "category": "Pe",
      "key": "FE42",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale rechte Eckhalterung"}}
    }, {
      "category": "Ps",
      "key": "FE43",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale linke weiße Eckhalterung"}}
    }, {
      "category": "Pe",
      "key": "FE44",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale rechte weiße Eckhalterung"}}
    }, {
      "category": "Ps",
      "key": "FE47",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale linke eckige Klammer"}}
    }, {
      "category": "Pe",
      "key": "FE48",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale eckige Klammer"}}
    }, {
      "category": "Ps",
      "key": "FE59",
      "mappings": {"default": {"default": "Kleine linke Klammer"}}
    }, {
      "category": "Pe",
      "key": "FE5A",
      "mappings": {"default": {"default": "Kleine rechte Klammer"}}
    }, {
      "category": "Ps",
      "key": "FE5B",
      "mappings": {"default": {"default": "Kleine linke geschweifte Klammer"}}
    }, {
      "category": "Pe",
      "key": "FE5C",
      "mappings": {"default": {"default": "Kleine rechte geschweifte Klammer"}}
    }, {
      "category": "Ps",
      "key": "FE5D",
      "mappings": {"default": {"default": "Kleine linke Schildpatt-Klammer"}}
    }, {
      "category": "Pe",
      "key": "FE5E",
      "mappings": {"default": {"default": "Kleine rechte Schildpatt-Klammer"}}
    }, {
      "category": "Ps",
      "key": "FF08",
      "mappings": {"default": {"default": "Linke Klammer in voller Breite"}}
    }, {
      "category": "Pe",
      "key": "FF09",
      "mappings": {"default": {"default": "Rechte Klammer mit voller Breite"}}
    }, {
      "category": "Ps",
      "key": "FF3B",
      "mappings": {"default": {"default": "Linke eckige Klammer mit voller Breite"}}
    }, {
      "category": "Pe",
      "key": "FF3D",
      "mappings": {"default": {"default": "Rechteckige Klammer mit voller Breite"}}
    }, {
      "category": "Ps",
      "key": "FF5B",
      "mappings": {"default": {"default": "Linke geschweifte Klammer mit voller Breite"}}
    }, {
      "category": "Pe",
      "key": "FF5D",
      "mappings": {"default": {"default": "Rechtwinklige Klammer mit voller Breite"}}
    }, {
      "category": "Ps",
      "key": "FF5F",
      "mappings": {"default": {"default": "Linke weiße Klammer mit voller Breite"}}
    }, {
      "category": "Pe",
      "key": "FF60",
      "mappings": {"default": {"default": "Rechte breite weiße Klammer"}}
    }, {
      "category": "Ps",
      "key": "FF62",
      "mappings": {"default": {"default": "Halbe breite linke eckklammer"}}
    }, {"category": "Pe", "key": "FF63", "mappings": {"default": {"default": "Halbe breite rechte eckklammer"}}}]
  ,
  "de/symbols/math_geometry.js":
    [{"locale": "de"}, {
      "category": "So",
      "key": "2500",
      "mappings": {"default": {"default": "Box Drawings Light Horizontal"}}
    }, {
      "category": "So",
      "key": "2501",
      "mappings": {"default": {"default": "Box Drawings Schwere Horizontale"}}
    }, {
      "category": "So",
      "key": "2502",
      "mappings": {"default": {"default": "Box Drawings Light Vertical"}}
    }, {
      "category": "So",
      "key": "2503",
      "mappings": {"default": {"default": "Box Drawings Schwere Vertikale"}}
    }, {
      "category": "So",
      "key": "2504",
      "mappings": {"default": {"default": "Box-Zeichnungen Licht Triple Dash Horizontal"}}
    }, {
      "category": "So",
      "key": "2505",
      "mappings": {"default": {"default": "Box Drawings Heavy Triple Dash Horizontal"}}
    }, {
      "category": "So",
      "key": "2506",
      "mappings": {"default": {"default": "Box Drawings Light Triple Dash Vertical"}}
    }, {
      "category": "So",
      "key": "2507",
      "mappings": {"default": {"default": "Box Drawings Heavy Triple Dash Vertical"}}
    }, {
      "category": "So",
      "key": "2508",
      "mappings": {"default": {"default": "Box Drawings Light Quadruple Dash Horizontal"}}
    }, {
      "category": "So",
      "key": "2509",
      "mappings": {"default": {"default": "Box Drawings Heavy Quadruple Dash Horizontal"}}
    }, {
      "category": "So",
      "key": "250A",
      "mappings": {"default": {"default": "Box Drawings Light Quadruple Dash Vertical"}}
    }, {
      "category": "So",
      "key": "250B",
      "mappings": {"default": {"default": "Box Drawings Heavy Quadruple Dash Vertical"}}
    }, {
      "category": "So",
      "key": "250C",
      "mappings": {"default": {"default": "Box Drawings Light Down und Right"}}
    }, {
      "category": "So",
      "key": "250D",
      "mappings": {"default": {"default": "Box Drawings Down Light und Right Heavy"}}
    }, {
      "category": "So",
      "key": "250E",
      "mappings": {"default": {"default": "Box Drawings Down Heavy und Right Light"}}
    }, {
      "category": "So",
      "key": "250F",
      "mappings": {"default": {"default": "Box Drawings Heavy Down und Right"}}
    }, {
      "category": "So",
      "key": "2510",
      "mappings": {"default": {"default": "Box Drawings Light Down und Links"}}
    }, {
      "category": "So",
      "key": "2511",
      "mappings": {"default": {"default": "Box Drawings Down Light und Left Heavy"}}
    }, {
      "category": "So",
      "key": "2512",
      "mappings": {"default": {"default": "Box Drawings Down Heavy und Left Light"}}
    }, {
      "category": "So",
      "key": "2513",
      "mappings": {"default": {"default": "Box Drawings Heavy Down und Left"}}
    }, {
      "category": "So",
      "key": "2514",
      "mappings": {"default": {"default": "Box-Zeichnungen leuchten auf und richtig"}}
    }, {
      "category": "So",
      "key": "2515",
      "mappings": {"default": {"default": "Box Drawings Up Light und Right Heavy"}}
    }, {
      "category": "So",
      "key": "2516",
      "mappings": {"default": {"default": "Box Drawings Up Heavy und Right Light"}}
    }, {
      "category": "So",
      "key": "2517",
      "mappings": {"default": {"default": "Box Drawings Heavy Up und Right"}}
    }, {
      "category": "So",
      "key": "2518",
      "mappings": {"default": {"default": "Box-Zeichnungen leuchten und links"}}
    }, {
      "category": "So",
      "key": "2519",
      "mappings": {"default": {"default": "Box Drawings Up Light und Left Heavy"}}
    }, {
      "category": "So",
      "key": "251A",
      "mappings": {"default": {"default": "Box Drawings Up Heavy und Left Light"}}
    }, {
      "category": "So",
      "key": "251B",
      "mappings": {"default": {"default": "Box Drawings Heavy Up und Links"}}
    }, {
      "category": "So",
      "key": "251C",
      "mappings": {"default": {"default": "Box Drawings Light Vertical und Right"}}
    }, {
      "category": "So",
      "key": "251D",
      "mappings": {"default": {"default": "Box Drawings Vertical Light und Right Heavy"}}
    }, {
      "category": "So",
      "key": "251E",
      "mappings": {"default": {"default": "Box Drawings Up Heavy und Right Down Light"}}
    }, {
      "category": "So",
      "key": "251F",
      "mappings": {"default": {"default": "Box Drawings Down Heavy und Right Up Light"}}
    }, {
      "category": "So",
      "key": "2520",
      "mappings": {"default": {"default": "Box Drawings Vertical Heavy und Right Light"}}
    }, {
      "category": "So",
      "key": "2521",
      "mappings": {"default": {"default": "Box Drawings Down Light und Right Up Heavy"}}
    }, {
      "category": "So",
      "key": "2522",
      "mappings": {"default": {"default": "Box Drawings Up Light und Right Down Heavy"}}
    }, {
      "category": "So",
      "key": "2523",
      "mappings": {"default": {"default": "Box Drawings Heavy Vertical und Right"}}
    }, {
      "category": "So",
      "key": "2524",
      "mappings": {"default": {"default": "Box Drawings Light Vertical und Left"}}
    }, {
      "category": "So",
      "key": "2525",
      "mappings": {"default": {"default": "Box Drawings Vertical Light und Left Heavy"}}
    }, {
      "category": "So",
      "key": "2526",
      "mappings": {"default": {"default": "Box Drawings Up Heavy und Left Down Light"}}
    }, {
      "category": "So",
      "key": "2527",
      "mappings": {"default": {"default": "Box Drawings Down Heavy und Left Up Light"}}
    }, {
      "category": "So",
      "key": "2528",
      "mappings": {"default": {"default": "Box Drawings Vertical Heavy und Left Light"}}
    }, {
      "category": "So",
      "key": "2529",
      "mappings": {"default": {"default": "Box Drawings Down Light und Left Up Heavy"}}
    }, {
      "category": "So",
      "key": "252A",
      "mappings": {"default": {"default": "Box Drawings Up Light und Left Down Heavy"}}
    }, {
      "category": "So",
      "key": "252B",
      "mappings": {"default": {"default": "Box Drawings Heavy Vertical und Left"}}
    }, {
      "category": "So",
      "key": "252C",
      "mappings": {"default": {"default": "Box-Zeichnungen leicht und horizontal"}}
    }, {
      "category": "So",
      "key": "252D",
      "mappings": {"default": {"default": "Box Drawings Left Heavy und Right Down Light"}}
    }, {
      "category": "So",
      "key": "252E",
      "mappings": {"default": {"default": "Box Drawings Right Heavy und Left Down Light"}}
    }, {
      "category": "So",
      "key": "252F",
      "mappings": {"default": {"default": "Box Drawings Down Light und Horizontal Heavy"}}
    }, {
      "category": "So",
      "key": "2530",
      "mappings": {"default": {"default": "Box-Zeichnungen für schweres und horizontales Licht"}}
    }, {
      "category": "So",
      "key": "2531",
      "mappings": {"default": {"default": "Box Drawings Right Light und Left Down Heavy"}}
    }, {
      "category": "So",
      "key": "2532",
      "mappings": {"default": {"default": "Box Drawings Left Light und Right Down Heavy"}}
    }, {
      "category": "So",
      "key": "2533",
      "mappings": {"default": {"default": "Box-Zeichnungen nach unten und horizontal"}}
    }, {
      "category": "So",
      "key": "2534",
      "mappings": {"default": {"default": "Box-Zeichnungen leuchten und horizontal"}}
    }, {
      "category": "So",
      "key": "2535",
      "mappings": {"default": {"default": "Box Drawings Left Heavy und Right Up Light"}}
    }, {
      "category": "So",
      "key": "2536",
      "mappings": {"default": {"default": "Box Drawings Right Heavy und Left Up Light"}}
    }, {
      "category": "So",
      "key": "2537",
      "mappings": {"default": {"default": "Box Drawings Up Light und Horizontal Heavy"}}
    }, {
      "category": "So",
      "key": "2538",
      "mappings": {"default": {"default": "Box-Zeichnungen für schweres und horizontales Licht"}}
    }, {
      "category": "So",
      "key": "2539",
      "mappings": {"default": {"default": "Box Drawings Right Light und Left Up Heavy"}}
    }, {
      "category": "So",
      "key": "253A",
      "mappings": {"default": {"default": "Box Drawings Left Light und Right Up Heavy"}}
    }, {
      "category": "So",
      "key": "253B",
      "mappings": {"default": {"default": "Box-Zeichnungen schwer und horizontal"}}
    }, {
      "category": "So",
      "key": "253C",
      "mappings": {"default": {"default": "Box Drawings Light vertikal und horizontal"}}
    }, {
      "category": "So",
      "key": "253D",
      "mappings": {"default": {"default": "Kastenzeichnungen links schweres und rechtes vertikales Licht"}}
    }, {
      "category": "So",
      "key": "253E",
      "mappings": {"default": {"default": "Box Drawings Right Heavy und Left Vertical Light"}}
    }, {
      "category": "So",
      "key": "253F",
      "mappings": {"default": {"default": "Box Drawings Vertical Light und Horizontal Heavy"}}
    }, {
      "category": "So",
      "key": "2540",
      "mappings": {"default": {"default": "Box Drawings Up Heavy und Down Horizontal Light"}}
    }, {
      "category": "So",
      "key": "2541",
      "mappings": {"default": {"default": "Box Drawings Down Heavy und Up Horizontal Light"}}
    }, {
      "category": "So",
      "key": "2542",
      "mappings": {"default": {"default": "Kastenzeichnungen Vertikales schweres und horizontales Licht"}}
    }, {
      "category": "So",
      "key": "2543",
      "mappings": {"default": {"default": "Box Drawings Left Up Heavy und Right Down Light"}}
    }, {
      "category": "So",
      "key": "2544",
      "mappings": {"default": {"default": "Box Drawings Right Up Heavy und Left Down Light"}}
    }, {
      "category": "So",
      "key": "2545",
      "mappings": {"default": {"default": "Box Drawings Left Down Heavy und Right Up Light"}}
    }, {
      "category": "So",
      "key": "2546",
      "mappings": {"default": {"default": "Box Drawings Right Down Heavy und Left Up Light"}}
    }, {
      "category": "So",
      "key": "2547",
      "mappings": {"default": {"default": "Box Drawings Down Light und Up Horizontal Heavy"}}
    }, {
      "category": "So",
      "key": "2548",
      "mappings": {"default": {"default": "Box Drawings Up Light und Down Horizontal Heavy"}}
    }, {
      "category": "So",
      "key": "2549",
      "mappings": {"default": {"default": "Box Drawings Right Light und Left Vertical Heavy"}}
    }, {
      "category": "So",
      "key": "254A",
      "mappings": {"default": {"default": "Box Drawings Left Light und Right Vertical Heavy"}}
    }, {
      "category": "So",
      "key": "254B",
      "mappings": {"default": {"default": "Box Drawings Heavy vertikal und horizontal"}}
    }, {
      "category": "So",
      "key": "254C",
      "mappings": {"default": {"default": "Box Drawings Light Double Dash Horizontal"}}
    }, {
      "category": "So",
      "key": "254D",
      "mappings": {"default": {"default": "Box Drawings Heavy Double Dash Horizontal"}}
    }, {
      "category": "So",
      "key": "254E",
      "mappings": {"default": {"default": "Box Drawings Light Double Dash Vertical"}}
    }, {
      "category": "So",
      "key": "254F",
      "mappings": {"default": {"default": "Box Drawings Heavy Double Dash Vertical"}}
    }, {
      "category": "So",
      "key": "2550",
      "mappings": {"default": {"default": "Box-Zeichnungen doppelt horizontal"}}
    }, {
      "category": "So",
      "key": "2551",
      "mappings": {"default": {"default": "Box Drawings Double Vertical"}}
    }, {
      "category": "So",
      "key": "2552",
      "mappings": {"default": {"default": "Box Drawings Down Single und Right Double"}}
    }, {
      "category": "So",
      "key": "2553",
      "mappings": {"default": {"default": "Box Drawings Down Double und Right Single"}}
    }, {
      "category": "So",
      "key": "2554",
      "mappings": {"default": {"default": "Box Drawings Double Down und Right"}}
    }, {
      "category": "So",
      "key": "2555",
      "mappings": {"default": {"default": "Box Drawings Down Single und Left Double"}}
    }, {
      "category": "So",
      "key": "2556",
      "mappings": {"default": {"default": "Box Drawings Down Double und Left Single"}}
    }, {
      "category": "So",
      "key": "2557",
      "mappings": {"default": {"default": "Box Drawings Double Down und Links"}}
    }, {
      "category": "So",
      "key": "2558",
      "mappings": {"default": {"default": "Box Drawings Up Single und Right Double"}}
    }, {
      "category": "So",
      "key": "2559",
      "mappings": {"default": {"default": "Box Drawings Up Double und Right Single"}}
    }, {
      "category": "So",
      "key": "255A",
      "mappings": {"default": {"default": "Box-Zeichnungen verdoppeln und rechts"}}
    }, {
      "category": "So",
      "key": "255B",
      "mappings": {"default": {"default": "Box Drawings Up Single und Left Double"}}
    }, {
      "category": "So",
      "key": "255C",
      "mappings": {"default": {"default": "Box Drawings Up Double und Left Single"}}
    }, {
      "category": "So",
      "key": "255D",
      "mappings": {"default": {"default": "Box-Zeichnungen verdoppeln und links"}}
    }, {
      "category": "So",
      "key": "255E",
      "mappings": {"default": {"default": "Box Drawings Vertical Single und Right Double"}}
    }, {
      "category": "So",
      "key": "255F",
      "mappings": {"default": {"default": "Box Drawings Vertical Double und Right Single"}}
    }, {
      "category": "So",
      "key": "2560",
      "mappings": {"default": {"default": "Box Drawings Double Vertical und Right"}}
    }, {
      "category": "So",
      "key": "2561",
      "mappings": {"default": {"default": "Box Drawings Vertical Single und Left Double"}}
    }, {
      "category": "So",
      "key": "2562",
      "mappings": {"default": {"default": "Box Drawings Vertical Double und Left Single"}}
    }, {
      "category": "So",
      "key": "2563",
      "mappings": {"default": {"default": "Box Drawings Double Vertical und Left"}}
    }, {
      "category": "So",
      "key": "2564",
      "mappings": {"default": {"default": "Box Drawings Down Single und Horizontal Double"}}
    }, {
      "category": "So",
      "key": "2565",
      "mappings": {"default": {"default": "Box Drawings Down Double und Horizontal Single"}}
    }, {
      "category": "So",
      "key": "2566",
      "mappings": {"default": {"default": "Box Drawings Double Down und Horizontal"}}
    }, {
      "category": "So",
      "key": "2567",
      "mappings": {"default": {"default": "Box Drawings Up Single und Horizontal Double"}}
    }, {
      "category": "So",
      "key": "2568",
      "mappings": {"default": {"default": "Box Drawings Up Double und Horizontal Single"}}
    }, {
      "category": "So",
      "key": "2569",
      "mappings": {"default": {"default": "Box-Zeichnungen verdoppeln und horizontal"}}
    }, {
      "category": "So",
      "key": "256A",
      "mappings": {"default": {"default": "Box Drawings Vertical Single und Horizontal Double"}}
    }, {
      "category": "So",
      "key": "256B",
      "mappings": {"default": {"default": "Box Drawings Vertical Double und Horizontal Single"}}
    }, {
      "category": "So",
      "key": "256C",
      "mappings": {"default": {"default": "Box-Zeichnungen doppelt vertikal und horizontal"}}
    }, {
      "category": "So",
      "key": "256D",
      "mappings": {"default": {"default": "Box Drawings Light Arc Down und Right"}}
    }, {
      "category": "So",
      "key": "256E",
      "mappings": {"default": {"default": "Box Drawings Light Arc Down und Links"}}
    }, {
      "category": "So",
      "key": "256F",
      "mappings": {"default": {"default": "Box Drawings Light Arc Up und links"}}
    }, {
      "category": "So",
      "key": "2570",
      "mappings": {"default": {"default": "Box Drawings Light Arc Up und rechts"}}
    }, {
      "category": "So",
      "key": "2571",
      "mappings": {"default": {"default": "Box Drawings Light Diagonal Oben Rechts Nach Unten Links"}}
    }, {
      "category": "So",
      "key": "2572",
      "mappings": {"default": {"default": "Box Drawings Light Diagonal von oben links nach rechts unten"}}
    }, {
      "category": "So",
      "key": "2573",
      "mappings": {"default": {"default": "Box Zeichnungen Light Diagonal Cross"}}
    }, {
      "category": "So",
      "key": "2574",
      "mappings": {"default": {"default": "Box-Zeichnungen Light Left"}}
    }, {
      "category": "So",
      "key": "2575",
      "mappings": {"default": {"default": "Box-Zeichnungen leuchten auf"}}
    }, {
      "category": "So",
      "key": "2576",
      "mappings": {"default": {"default": "Box-Zeichnungen Light Right"}}
    }, {
      "category": "So",
      "key": "2577",
      "mappings": {"default": {"default": "Box Drawings Light Down"}}
    }, {
      "category": "So",
      "key": "2578",
      "mappings": {"default": {"default": "Box Drawings Heavy Left"}}
    }, {
      "category": "So",
      "key": "2579",
      "mappings": {"default": {"default": "Box Drawings Heavy Up"}}
    }, {
      "category": "So",
      "key": "257A",
      "mappings": {"default": {"default": "Box Drawings Heavy Right"}}
    }, {
      "category": "So",
      "key": "257B",
      "mappings": {"default": {"default": "Box Drawings Heavy Down"}}
    }, {
      "category": "So",
      "key": "257C",
      "mappings": {"default": {"default": "Box Drawings Light Left und Heavy Right"}}
    }, {
      "category": "So",
      "key": "257D",
      "mappings": {"default": {"default": "Box Drawings Light Up und Heavy Down"}}
    }, {
      "category": "So",
      "key": "257E",
      "mappings": {"default": {"default": "Box Drawings Heavy Left und Light Right"}}
    }, {
      "category": "So",
      "key": "257F",
      "mappings": {"default": {"default": "Box Drawings Heavy Up und Light Down"}}
    }, {
      "category": "So",
      "key": "2580",
      "mappings": {"default": {"default": "Block der oberen Hälfte"}}
    }, {
      "category": "So",
      "key": "2581",
      "mappings": {"default": {"default": "Unterer achter Block"}}
    }, {
      "category": "So",
      "key": "2582",
      "mappings": {"default": {"default": "Unterer Viertelblock"}}
    }, {
      "category": "So",
      "key": "2583",
      "mappings": {"default": {"default": "Drei Achtel-Block senken"}}
    }, {
      "category": "So",
      "key": "2584",
      "mappings": {"default": {"default": "Block der unteren Hälfte"}}
    }, {"category": "So", "key": "2585", "mappings": {"default": {"default": "Fünf Achtel Block"}}}, {
      "category": "So",
      "key": "2586",
      "mappings": {"default": {"default": "Unterer Dreiviertelblock"}}
    }, {
      "category": "So",
      "key": "2587",
      "mappings": {"default": {"default": "Unterer Sieben Achtel-Block"}}
    }, {"category": "So", "key": "2588", "mappings": {"default": {"default": "Voller Block"}}}, {
      "category": "So",
      "key": "2589",
      "mappings": {"default": {"default": "Linker Sieben Achtel Block"}}
    }, {
      "category": "So",
      "key": "258A",
      "mappings": {"default": {"default": "Linker Dreiviertelblock"}}
    }, {
      "category": "So",
      "key": "258B",
      "mappings": {"default": {"default": "Linker Block mit fünf Achteln"}}
    }, {"category": "So", "key": "258C", "mappings": {"default": {"default": "Linke halbe Block"}}}, {
      "category": "So",
      "key": "258D",
      "mappings": {"default": {"default": "Block drei Achtel"}}
    }, {
      "category": "So",
      "key": "258E",
      "mappings": {"default": {"default": "Linke einen Viertelblock"}}
    }, {
      "category": "So",
      "key": "258F",
      "mappings": {"default": {"default": "Linker achter Block"}}
    }, {"category": "So", "key": "2590", "mappings": {"default": {"default": "Rechte halbe Block"}}}, {
      "category": "So",
      "key": "2591",
      "mappings": {"default": {"default": "Lichtschatten"}}
    }, {"category": "So", "key": "2592", "mappings": {"default": {"default": "Mittlere Schatten"}}}, {
      "category": "So",
      "key": "2593",
      "mappings": {"default": {"default": "Dunkler Schatten"}}
    }, {
      "category": "So",
      "key": "2594",
      "mappings": {"default": {"default": "Oberer achter Block"}}
    }, {
      "category": "So",
      "key": "2595",
      "mappings": {"default": {"default": "rechter achter Block"}}
    }, {
      "category": "So",
      "key": "2596",
      "mappings": {"default": {"default": "Quadrant unten links"}}
    }, {
      "category": "So",
      "key": "2597",
      "mappings": {"default": {"default": "Quadrant rechts unten"}}
    }, {
      "category": "So",
      "key": "2598",
      "mappings": {"default": {"default": "Quadrant oben links"}}
    }, {
      "category": "So",
      "key": "2599",
      "mappings": {"default": {"default": "Quadrant oben links und unten links und rechts unten"}}
    }, {
      "category": "So",
      "key": "259A",
      "mappings": {"default": {"default": "Quadrant oben links und rechts unten"}}
    }, {
      "category": "So",
      "key": "259B",
      "mappings": {"default": {"default": "Quadrant oben links und oben rechts und unten links"}}
    }, {
      "category": "So",
      "key": "259C",
      "mappings": {"default": {"default": "Quadrant oben links und oben rechts und unten rechts"}}
    }, {
      "category": "So",
      "key": "259D",
      "mappings": {"default": {"default": "Quadrant oben rechts"}}
    }, {
      "category": "So",
      "key": "259E",
      "mappings": {"default": {"default": "Quadrant oben rechts und unten links"}}
    }, {
      "category": "So",
      "key": "259F",
      "mappings": {"default": {"default": "Quadrant oben rechts und unten links und unten rechts"}}
    }, {"category": "So", "key": "25A0", "mappings": {"default": {"default": "Schwarzes Quadrat"}}}, {
      "category": "So",
      "key": "25A1",
      "mappings": {"default": {"default": "Weißes Quadrat"}}
    }, {
      "category": "So",
      "key": "25A2",
      "mappings": {"default": {"default": "Weißes Quadrat mit abgerundeten Ecken"}}
    }, {
      "category": "So",
      "key": "25A3",
      "mappings": {"default": {"default": "Weißes Quadrat, das schwarzes kleines Quadrat enthält"}}
    }, {
      "category": "So",
      "key": "25A4",
      "mappings": {"default": {"default": "Quadrat mit horizontaler Füllung"}}
    }, {
      "category": "So",
      "key": "25A5",
      "mappings": {"default": {"default": "Quadrat mit vertikaler Füllung"}}
    }, {
      "category": "So",
      "key": "25A6",
      "mappings": {"default": {"default": "Quadrat mit orthogonaler Schraffurfüllung"}}
    }, {
      "category": "So",
      "key": "25A7",
      "mappings": {"default": {"default": "Quadrat mit Oben links nach rechts unten füllen"}}
    }, {
      "category": "So",
      "key": "25A8",
      "mappings": {"default": {"default": "Quadrat mit oberer rechter bis unterer linker Füllung"}}
    }, {
      "category": "So",
      "key": "25A9",
      "mappings": {"default": {"default": "Quadrat mit diagonaler Schraffurfüllung"}}
    }, {
      "category": "So",
      "key": "25AA",
      "mappings": {"default": {"default": "Schwarzes kleines Quadrat"}}
    }, {
      "category": "So",
      "key": "25AB",
      "mappings": {"default": {"default": "Weißes kleines Quadrat"}}
    }, {"category": "So", "key": "25AC", "mappings": {"default": {"default": "Schwarzes Rechteck"}}}, {
      "category": "So",
      "key": "25AD",
      "mappings": {"default": {"default": "Weißes Rechteck"}}
    }, {
      "category": "So",
      "key": "25AE",
      "mappings": {"default": {"default": "Schwarzes vertikales Rechteck"}}
    }, {
      "category": "So",
      "key": "25AF",
      "mappings": {"default": {"default": "Weißes vertikales Rechteck"}}
    }, {
      "category": "So",
      "key": "25B0",
      "mappings": {"default": {"default": "Schwarzes Parallelogramm"}}
    }, {
      "category": "So",
      "key": "25B1",
      "mappings": {"default": {"default": "Weißes Parallelogramm"}}
    }, {
      "category": "So",
      "key": "25B2",
      "mappings": {"default": {"default": "Schwarzes nach oben zeigendes Dreieck"}}
    }, {
      "category": "So",
      "key": "25B3",
      "mappings": {"default": {"default": "Weißes nach oben zeigendes Dreieck"}}
    }, {
      "category": "So",
      "key": "25B4",
      "mappings": {"default": {"default": "Schwarzes nach oben zeigendes kleines Dreieck"}}
    }, {
      "category": "So",
      "key": "25B5",
      "mappings": {"default": {"default": "Weißes nach oben zeigendes kleines Dreieck"}}
    }, {
      "category": "So",
      "key": "25B6",
      "mappings": {"default": {"default": "Schwarzes rechtwinkliges Dreieck"}}
    }, {
      "category": "Sm",
      "key": "25B7",
      "mappings": {"default": {"default": "Weißes rechtwinkliges Dreieck"}}
    }, {
      "category": "So",
      "key": "25B8",
      "mappings": {"default": {"default": "Schwarzes nach rechts zeigendes kleines Dreieck"}}
    }, {
      "category": "So",
      "key": "25B9",
      "mappings": {"default": {"default": "Weißes nach rechts zeigendes kleines Dreieck"}}
    }, {
      "category": "So",
      "key": "25BA",
      "mappings": {"default": {"default": "Schwarzer nach rechts zeigender Zeiger"}}
    }, {
      "category": "So",
      "key": "25BB",
      "mappings": {"default": {"default": "Weißer nach rechts zeigender Zeiger"}}
    }, {
      "category": "So",
      "key": "25BC",
      "mappings": {"default": {"default": "Schwarzes Abwärtsdreieck"}}
    }, {
      "category": "So",
      "key": "25BD",
      "mappings": {"default": {"default": "Weißes nach unten zeigendes Dreieck"}}
    }, {
      "category": "So",
      "key": "25BE",
      "mappings": {"default": {"default": "Schwarzes, nach unten zeigendes kleines Dreieck"}}
    }, {
      "category": "So",
      "key": "25BF",
      "mappings": {"default": {"default": "Weißes kleines Dreieck"}}
    }, {
      "category": "So",
      "key": "25C0",
      "mappings": {"default": {"default": "Schwarzes nach links zeigendes Dreieck"}}
    }, {
      "category": "Sm",
      "key": "25C1",
      "mappings": {"default": {"default": "Weißes nach links zeigendes Dreieck"}}
    }, {
      "category": "So",
      "key": "25C2",
      "mappings": {"default": {"default": "Schwarzes nach links zeigendes kleines Dreieck"}}
    }, {
      "category": "So",
      "key": "25C3",
      "mappings": {"default": {"default": "Weißes nach links zeigendes kleines Dreieck"}}
    }, {
      "category": "So",
      "key": "25C4",
      "mappings": {"default": {"default": "Schwarzer nach links zeigender Zeiger"}}
    }, {
      "category": "So",
      "key": "25C5",
      "mappings": {"default": {"default": "Weißer nach links zeigender Zeiger"}}
    }, {"category": "So", "key": "25C6", "mappings": {"default": {"default": "Schwarzer Diamant"}}}, {
      "category": "So",
      "key": "25C7",
      "mappings": {"default": {"default": "Weißer Diamant"}}
    }, {
      "category": "So",
      "key": "25C8",
      "mappings": {"default": {"default": "Weißer Diamant mit schwarzem kleinem Diamant"}}
    }, {"category": "So", "key": "25C9", "mappings": {"default": {"default": "Fischauge"}}}, {
      "category": "So",
      "key": "25CA",
      "mappings": {"default": {"default": "Raute"}}
    }, {"category": "So", "key": "25CB", "mappings": {"default": {"default": "Weißer Kreis"}}}, {
      "category": "So",
      "key": "25CC",
      "mappings": {"default": {"default": "Gepunkteter Kreis"}}
    }, {
      "category": "So",
      "key": "25CD",
      "mappings": {"default": {"default": "Kreis mit vertikaler Füllung"}}
    }, {"category": "So", "key": "25CE", "mappings": {"default": {"default": "Bullseye"}}}, {
      "category": "So",
      "key": "25CF",
      "mappings": {"default": {"default": "Schwarzer Kreis"}}
    }, {
      "category": "So",
      "key": "25D0",
      "mappings": {"default": {"default": "Kreis mit linker Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "25D1",
      "mappings": {"default": {"default": "Kreis mit rechter Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "25D2",
      "mappings": {"default": {"default": "Kreis mit unterer Hälfte Schwarz"}}
    }, {
      "category": "So",
      "key": "25D3",
      "mappings": {"default": {"default": "Kreis mit oberer Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "25D4",
      "mappings": {"default": {"default": "Kreis mit oberem rechten Quadranten schwarz"}}
    }, {
      "category": "So",
      "key": "25D5",
      "mappings": {"default": {"default": "Kreis mit ganzem oberen linken Quadranten schwarz"}}
    }, {
      "category": "So",
      "key": "25D6",
      "mappings": {"default": {"default": "Linker halber schwarzer Kreis"}}
    }, {
      "category": "So",
      "key": "25D7",
      "mappings": {"default": {"default": "rechter halber schwarzer Kreis"}}
    }, {"category": "So", "key": "25D8", "mappings": {"default": {"default": "Inverse Kugel"}}}, {
      "category": "So",
      "key": "25D9",
      "mappings": {"default": {"default": "Inverse weißer Kreis"}}
    }, {
      "category": "So",
      "key": "25DA",
      "mappings": {"default": {"default": "Umgekehrter weißer Kreis der oberen Hälfte"}}
    }, {
      "category": "So",
      "key": "25DB",
      "mappings": {"default": {"default": "Inverser weißer Kreis der unteren Hälfte"}}
    }, {
      "category": "So",
      "key": "25DC",
      "mappings": {"default": {"default": "Kreisbogen des oberen linken Quadranten"}}
    }, {
      "category": "So",
      "key": "25DD",
      "mappings": {"default": {"default": "Kreisbogen im oberen rechten Quadranten"}}
    }, {
      "category": "So",
      "key": "25DE",
      "mappings": {"default": {"default": "Kreisbogen des rechten unteren Quadranten"}}
    }, {
      "category": "So",
      "key": "25DF",
      "mappings": {"default": {"default": "Kreisbogen des unteren linken Quadranten"}}
    }, {"category": "So", "key": "25E0", "mappings": {"default": {"default": "Oberer Halbkreis"}}}, {
      "category": "So",
      "key": "25E1",
      "mappings": {"default": {"default": "Unterer Halbkreis"}}
    }, {
      "category": "So",
      "key": "25E2",
      "mappings": {"default": {"default": "Schwarzes rechtes Dreieck"}}
    }, {
      "category": "So",
      "key": "25E3",
      "mappings": {"default": {"default": "Schwarzes unteres linkes Dreieck"}}
    }, {
      "category": "So",
      "key": "25E4",
      "mappings": {"default": {"default": "Schwarzes oberes linkes Dreieck"}}
    }, {
      "category": "So",
      "key": "25E5",
      "mappings": {"default": {"default": "Schwarzes rechtes Dreieck"}}
    }, {"category": "So", "key": "25E6", "mappings": {"default": {"default": "Weiße Kugel"}}}, {
      "category": "So",
      "key": "25E7",
      "mappings": {"default": {"default": "Quadrat mit linker Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "25E8",
      "mappings": {"default": {"default": "Quadrat mit rechter Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "25E9",
      "mappings": {"default": {"default": "Quadrat mit der oberen linken Diagonalenhälfte schwarz"}}
    }, {
      "category": "So",
      "key": "25EA",
      "mappings": {"default": {"default": "Quadrat mit unterer rechter diagonaler Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "25EB",
      "mappings": {"default": {"default": "Weißes Quadrat mit senkrechter Linie"}}
    }, {
      "category": "So",
      "key": "25EC",
      "mappings": {"default": {"default": "Weißes nach oben zeigendes Dreieck mit Punkt"}}
    }, {
      "category": "So",
      "key": "25ED",
      "mappings": {"default": {"default": "Aufwärtszeigendes Dreieck mit linker Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "25EE",
      "mappings": {"default": {"default": "Aufwärtszeigendes Dreieck mit rechter Hälfte schwarz"}}
    }, {"category": "So", "key": "25EF", "mappings": {"default": {"default": "Großer Kreis"}}}, {
      "category": "So",
      "key": "25F0",
      "mappings": {"default": {"default": "Weißes Quadrat mit oberem linken Quadranten"}}
    }, {
      "category": "So",
      "key": "25F1",
      "mappings": {"default": {"default": "Weißes Quadrat mit unterem linken Quadranten"}}
    }, {
      "category": "So",
      "key": "25F2",
      "mappings": {"default": {"default": "Weißes Quadrat mit unterem rechten Quadranten"}}
    }, {
      "category": "So",
      "key": "25F3",
      "mappings": {"default": {"default": "Weißes Quadrat mit rechtem oberen Quadranten"}}
    }, {
      "category": "So",
      "key": "25F4",
      "mappings": {"default": {"default": "Weißer Kreis mit oberem linkem Quadranten"}}
    }, {
      "category": "So",
      "key": "25F5",
      "mappings": {"default": {"default": "Weißer Kreis mit unterem linken Quadranten"}}
    }, {
      "category": "So",
      "key": "25F6",
      "mappings": {"default": {"default": "Weißer Kreis mit unterem rechten Quadranten"}}
    }, {
      "category": "So",
      "key": "25F7",
      "mappings": {"default": {"default": "Weißer Kreis mit rechtem oberen Quadranten"}}
    }, {
      "category": "Sm",
      "key": "25F8",
      "mappings": {"default": {"default": "Oberes linkes Dreieck"}}
    }, {
      "category": "Sm",
      "key": "25F9",
      "mappings": {"default": {"default": "Oberes rechtes Dreieck"}}
    }, {
      "category": "Sm",
      "key": "25FA",
      "mappings": {"default": {"default": "Unteres linkes Dreieck"}}
    }, {
      "category": "Sm",
      "key": "25FB",
      "mappings": {"default": {"default": "Weißes mittleres Quadrat"}}
    }, {
      "category": "Sm",
      "key": "25FC",
      "mappings": {"default": {"default": "Schwarzes mittleres Quadrat"}}
    }, {
      "category": "Sm",
      "key": "25FD",
      "mappings": {"default": {"default": "Weißes mittleres kleines Quadrat"}}
    }, {
      "category": "Sm",
      "key": "25FE",
      "mappings": {"default": {"default": "Schwarzes mittleres kleines Quadrat"}}
    }, {
      "category": "Sm",
      "key": "25FF",
      "mappings": {"default": {"default": "Unteres rechtes Dreieck"}}
    }, {
      "category": "So",
      "key": "2B12",
      "mappings": {"default": {"default": "Quadrat mit oberer Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "2B13",
      "mappings": {"default": {"default": "Quadrat mit unterer Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "2B14",
      "mappings": {"default": {"default": "Quadrat mit oberer rechter diagonaler Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "2B15",
      "mappings": {"default": {"default": "Quadrat mit unterer linker diagonaler Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "2B16",
      "mappings": {"default": {"default": "Diamant mit linker Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "2B17",
      "mappings": {"default": {"default": "Diamant mit rechter Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "2B18",
      "mappings": {"default": {"default": "Diamant mit oberer Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "2B19",
      "mappings": {"default": {"default": "Diamant mit unterer Hälfte schwarz"}}
    }, {
      "category": "So",
      "key": "2B1A",
      "mappings": {"default": {"default": "Gepunktetes Quadrat"}}
    }, {
      "category": "So",
      "key": "2B1B",
      "mappings": {"default": {"default": "Schwarzes großes Quadrat"}}
    }, {
      "category": "So",
      "key": "2B1C",
      "mappings": {"default": {"default": "Weißes großes Quadrat"}}
    }, {
      "category": "So",
      "key": "2B1D",
      "mappings": {"default": {"default": "Schwarzes sehr kleines Quadrat"}}
    }, {
      "category": "So",
      "key": "2B1E",
      "mappings": {"default": {"default": "Weißes sehr kleines Quadrat"}}
    }, {"category": "So", "key": "2B1F", "mappings": {"default": {"default": "Schwarzes Pentagon"}}}, {
      "category": "So",
      "key": "2B20",
      "mappings": {"default": {"default": "Weißes Pentagon"}}
    }, {"category": "So", "key": "2B21", "mappings": {"default": {"default": "Weißes Sechseck"}}}, {
      "category": "So",
      "key": "2B22",
      "mappings": {"default": {"default": "Schwarzes Sechseck"}}
    }, {
      "category": "So",
      "key": "2B23",
      "mappings": {"default": {"default": "Horizontales schwarzes Sechseck"}}
    }, {
      "category": "So",
      "key": "2B24",
      "mappings": {"default": {"default": "Schwarzer großer Kreis"}}
    }, {
      "category": "So",
      "key": "2B25",
      "mappings": {"default": {"default": "Schwarzer mittlerer Diamant"}}
    }, {
      "category": "So",
      "key": "2B26",
      "mappings": {"default": {"default": "Weißer mittlerer Diamant"}}
    }, {
      "category": "So",
      "key": "2B27",
      "mappings": {"default": {"default": "Schwarze mittlere Raute"}}
    }, {
      "category": "So",
      "key": "2B28",
      "mappings": {"default": {"default": "Weiße mittlere Raute"}}
    }, {
      "category": "So",
      "key": "2B29",
      "mappings": {"default": {"default": "Schwarzer kleiner Diamant"}}
    }, {
      "category": "So",
      "key": "2B2A",
      "mappings": {"default": {"default": "Schwarze kleine Raute"}}
    }, {"category": "So", "key": "2B2B", "mappings": {"default": {"default": "Weiße kleine Raute"}}}, {
      "category": "So",
      "key": "2B2C",
      "mappings": {"default": {"default": "Schwarze horizontale Ellipse"}}
    }, {
      "category": "So",
      "key": "2B2D",
      "mappings": {"default": {"default": "Weiße horizontale Ellipse"}}
    }, {
      "category": "So",
      "key": "2B2E",
      "mappings": {"default": {"default": "Schwarze vertikale Ellipse"}}
    }, {
      "category": "So",
      "key": "2B2F",
      "mappings": {"default": {"default": "Weiße vertikale Ellipse"}}
    }, {
      "category": "So",
      "key": "2B50",
      "mappings": {"default": {"default": "Weißer mittlerer Stern"}}
    }, {
      "category": "So",
      "key": "2B51",
      "mappings": {"default": {"default": "Schwarzer kleiner Stern"}}
    }, {
      "category": "So",
      "key": "2B52",
      "mappings": {"default": {"default": "Weißer kleiner Stern"}}
    }, {
      "category": "So",
      "key": "2B53",
      "mappings": {"default": {"default": "Schwarzes nach rechts zeigendes Pentagon"}}
    }, {
      "category": "So",
      "key": "2B54",
      "mappings": {"default": {"default": "Weißes nach rechts zeigendes Pentagon"}}
    }, {
      "category": "So",
      "key": "2B55",
      "mappings": {"default": {"default": "Schwerer großer Kreis"}}
    }, {
      "category": "So",
      "key": "2B56",
      "mappings": {"default": {"default": "Schweres Oval mit innenliegendem Oval"}}
    }, {
      "category": "So",
      "key": "2B57",
      "mappings": {"default": {"default": "Schwerer Kreis mit Kreis nach innen"}}
    }, {"category": "So", "key": "2B58", "mappings": {"default": {"default": "Schwerer Kreis"}}}, {
      "category": "So",
      "key": "2B59",
      "mappings": {"default": {"default": "Schwerer eingekreister Saltire"}}
    }]
  ,
  "de/symbols/math_harpoons.js":
    [{"locale": "de"}, {
      "category": "So",
      "key": "21BC",
      "mappings": {"default": {"default": "Linke Harpune mit Barb nach oben"}}
    }, {
      "category": "So",
      "key": "21BD",
      "mappings": {"default": {"default": "Harpune nach links mit Barb nach unten"}}
    }, {
      "category": "So",
      "key": "21BE",
      "mappings": {"default": {"default": "Harpune nach oben mit Barb nach rechts"}}
    }, {
      "category": "So",
      "key": "21BF",
      "mappings": {"default": {"default": "Harpune nach oben mit Barb nach links"}}
    }, {
      "category": "So",
      "key": "21C0",
      "mappings": {"default": {"default": "Harpune nach rechts mit Barb nach oben"}}
    }, {
      "category": "So",
      "key": "21C1",
      "mappings": {"default": {"default": "Harpune nach rechts mit Barb nach unten"}}
    }, {
      "category": "So",
      "key": "21C2",
      "mappings": {"default": {"default": "Harpune nach unten mit Barb nach rechts"}}
    }, {
      "category": "So",
      "key": "21C3",
      "mappings": {"default": {"default": "Harpune nach unten mit Barb nach links"}}
    }, {
      "category": "So",
      "key": "21CB",
      "mappings": {"default": {"default": "Harpune nach links über Harpune nach rechts"}}
    }, {
      "category": "So",
      "key": "21CC",
      "mappings": {"default": {"default": "Harpune nach rechts über Harpune nach links"}}
    }, {
      "category": "Sm",
      "key": "294A",
      "mappings": {"default": {"default": "Linke Barb Up Rechte Barb Down Harpoon"}}
    }, {
      "category": "Sm",
      "key": "294B",
      "mappings": {"default": {"default": "Linke Barb Down Rechte Barb Up Harpoon"}}
    }, {
      "category": "Sm",
      "key": "294C",
      "mappings": {"default": {"default": "Nach oben Barb Rechts Nach links Barb Harpoon"}}
    }, {
      "category": "Sm",
      "key": "294D",
      "mappings": {"default": {"default": "Up Barb Left Abwärts Barb Right Harpoon"}}
    }, {
      "category": "Sm",
      "key": "294E",
      "mappings": {"default": {"default": "Linke Barb Up Rechte Barb Up Harpoon"}}
    }, {
      "category": "Sm",
      "key": "294F",
      "mappings": {"default": {"default": "Barb nach rechts Barb nach rechts Harpoon"}}
    }, {
      "category": "Sm",
      "key": "2950",
      "mappings": {"default": {"default": "Linke Barb Down Rechte Barb Down Harpoon"}}
    }, {
      "category": "Sm",
      "key": "2951",
      "mappings": {"default": {"default": "Up Barb Left Abwärts Barb Left Harpoon"}}
    }, {
      "category": "Sm",
      "key": "2952",
      "mappings": {"default": {"default": "Linke Harpune mit Barb Up To Bar"}}
    }, {
      "category": "Sm",
      "key": "2953",
      "mappings": {"default": {"default": "Harpune nach rechts mit Barb bis zur Bar"}}
    }, {
      "category": "Sm",
      "key": "2954",
      "mappings": {"default": {"default": "Harpune nach oben mit Widerhaken rechts"}}
    }, {
      "category": "Sm",
      "key": "2955",
      "mappings": {"default": {"default": "Harpune nach unten mit Barb Right to Bar"}}
    }, {
      "category": "Sm",
      "key": "2956",
      "mappings": {"default": {"default": "Linke Harpune mit Widerhaken nach unten"}}
    }, {
      "category": "Sm",
      "key": "2957",
      "mappings": {"default": {"default": "Harpune nach rechts mit Widerhaken nach unten"}}
    }, {
      "category": "Sm",
      "key": "2958",
      "mappings": {"default": {"default": "Harpune nach oben mit Barb nach links"}}
    }, {
      "category": "Sm",
      "key": "2959",
      "mappings": {"default": {"default": "Harpune nach unten mit Barb nach links"}}
    }, {
      "category": "Sm",
      "key": "295A",
      "mappings": {"default": {"default": "Linke Harpune mit Barb Up von der Bar"}}
    }, {
      "category": "Sm",
      "key": "295B",
      "mappings": {"default": {"default": "Harpune nach rechts mit Barb Up von der Bar"}}
    }, {
      "category": "Sm",
      "key": "295C",
      "mappings": {"default": {"default": "Harpune nach oben mit Barb rechts von der Bar"}}
    }, {
      "category": "Sm",
      "key": "295D",
      "mappings": {"default": {"default": "Harpune nach unten mit Barb rechts von der Bar"}}
    }, {
      "category": "Sm",
      "key": "295E",
      "mappings": {"default": {"default": "Linke Harpune mit Barb Down von der Bar"}}
    }, {
      "category": "Sm",
      "key": "295F",
      "mappings": {"default": {"default": "Rechts Harpoon mit Barb Down von der Bar"}}
    }, {
      "category": "Sm",
      "key": "2960",
      "mappings": {"default": {"default": "Harpune nach oben mit Barb links von der Bar"}}
    }, {
      "category": "Sm",
      "key": "2961",
      "mappings": {"default": {"default": "Harpune nach unten mit Barb links von der Bar"}}
    }, {
      "category": "Sm",
      "key": "2962",
      "mappings": {"default": {"default": "Harpune nach links mit Barb nach oben Harpune nach links mit Barb nach unten"}}
    }, {
      "category": "Sm",
      "key": "2963",
      "mappings": {"default": {"default": "Harpune nach oben mit Barb links neben Harpune nach oben mit Barb rechts"}}
    }, {
      "category": "Sm",
      "key": "2964",
      "mappings": {"default": {"default": "Harpune nach rechts mit Barb nach oben Harpoon nach rechts mit Barb nach unten"}}
    }, {
      "category": "Sm",
      "key": "2965",
      "mappings": {"default": {"default": "Harpune nach unten mit Barb nach unten Harpune nach unten mit Barb nach rechts"}}
    }, {
      "category": "Sm",
      "key": "2966",
      "mappings": {"default": {"default": "Linke Harpune mit Barb Up Oben Rechts Harpoon mit Barb Up"}}
    }, {
      "category": "Sm",
      "key": "2967",
      "mappings": {"default": {"default": "Linke Harpune mit Widerhaken nach unten Rechts Harpune mit Widerhaken nach unten"}}
    }, {
      "category": "Sm",
      "key": "2968",
      "mappings": {"default": {"default": "Harpune nach rechts mit Barb nach oben Harpune nach links mit Barb nach oben"}}
    }, {
      "category": "Sm",
      "key": "2969",
      "mappings": {"default": {"default": "Harpune nach rechts mit Barb nach unten Harpune nach links mit Barb nach unten"}}
    }, {
      "category": "Sm",
      "key": "296A",
      "mappings": {"default": {"default": "Linke Harpune mit Barb Up Above Long Dash"}}
    }, {
      "category": "Sm",
      "key": "296B",
      "mappings": {"default": {"default": "Linke Harpune mit Barb Down Under Long Dash"}}
    }, {
      "category": "Sm",
      "key": "296C",
      "mappings": {"default": {"default": "Harpune nach rechts mit Barb Up Long Long Dash"}}
    }, {
      "category": "Sm",
      "key": "296D",
      "mappings": {"default": {"default": "Harpune nach rechts mit Widerhaken nach unten"}}
    }, {
      "category": "Sm",
      "key": "296E",
      "mappings": {"default": {"default": "Harpune nach oben mit Barb links neben Harpune nach unten mit Barb rechts"}}
    }, {
      "category": "Sm",
      "key": "296F",
      "mappings": {"default": {"default": "Harpune nach unten mit Barb links neben Harpune nach oben mit Barb rechts"}}
    }, {
      "category": "Sm",
      "key": "297C",
      "mappings": {"default": {"default": "linker Fischschwanz"}}
    }, {
      "category": "Sm",
      "key": "297D",
      "mappings": {"default": {"default": "rechter Fischschwanz"}}
    }, {"category": "Sm", "key": "297E", "mappings": {"default": {"default": "Fischschwanz"}}}, {
      "category": "Sm",
      "key": "297F",
      "mappings": {"default": {"default": "Fischschwanz"}}
    }]
  ,
  "de/symbols/math_non_characters.js":
    [{"locale": "de"}, {
      "category": "Ll",
      "key": "210F",
      "mappings": {"default": {"default": "Planck-Konstante über zwei Pi"}}
    }, {"category": "So", "key": "2114", "mappings": {"default": {"default": "L B Balkensymbol"}}}, {
      "category": "So",
      "key": "2116",
      "mappings": {"default": {"default": "Numero-Zeichen"}}
    }, {
      "category": "So",
      "key": "2117",
      "mappings": {"default": {"default": "Tonaufnahme Copyright"}}
    }, {"category": "So", "key": "211E", "mappings": {"default": {"default": "Rezept nehmen"}}}, {
      "category": "So",
      "key": "211F",
      "mappings": {"default": {"default": "Antwort"}}
    }, {
      "category": "So",
      "key": "2120",
      "mappings": {"default": {"default": "Dienstleistungsmarke"}}
    }, {"category": "So", "key": "2121", "mappings": {"default": {"default": "Telefon Zeichen"}}}, {
      "category": "So",
      "key": "2122",
      "mappings": {"default": {"default": "Markenzeichen"}}
    }, {"category": "So", "key": "2123", "mappings": {"default": {"default": "Versicle"}}}, {
      "category": "So",
      "key": "2125",
      "mappings": {"default": {"default": "Unze-Zeichen"}}
    }, {"category": "Lu", "key": "2126", "mappings": {"default": {"default": "Ohm-Zeichen"}}}, {
      "category": "So",
      "key": "2127",
      "mappings": {"default": {"default": "Umgekehrtes Ohm-Zeichen"}}
    }, {"category": "Lu", "key": "212A", "mappings": {"default": {"default": "Kelvin-Zeichen"}}}, {
      "category": "Lu",
      "key": "212B",
      "mappings": {"default": {"default": "Angstrom-Zeichen"}}
    }, {"category": "So", "key": "212E", "mappings": {"default": {"default": "Geschätztes Symbol"}}}, {
      "category": "Lu",
      "key": "2132",
      "mappings": {"default": {"default": "Umgeschlagenes Kapital F"}}
    }, {"category": "Ll", "key": "2139", "mappings": {"default": {"default": "Informationsquelle"}}}, {
      "category": "So",
      "key": "213A",
      "mappings": {"default": {"default": "Gedrehtes Kapital Q"}}
    }, {"category": "So", "key": "213B", "mappings": {"default": {"default": "Faksimile-Zeichen"}}}, {
      "category": "Sm",
      "key": "2141",
      "mappings": {"default": {"default": "Sans-Serif Capital G"}}
    }, {
      "category": "Sm",
      "key": "2142",
      "mappings": {"default": {"default": "Sans-Serif Capital L"}}
    }, {
      "category": "Sm",
      "key": "2143",
      "mappings": {"default": {"default": "Aufgehobenes Sans-Serif-Kapital L"}}
    }, {"category": "Sm", "key": "2144", "mappings": {"default": {"default": "Sans-Serif Capital Y"}}}]
  ,
  "de/symbols/math_symbols.js":
    [{"locale": "de"}, {
      "key": "0021",
      "mappings": {"default": {"default": "Fakultät"}},
      "category": "Po"
    }, {"key": "0022", "mappings": {"default": {"default": "Anführungszeichen"}}, "category": "Po"}, {
      "key": "0023",
      "mappings": {"default": {"default": "Nummernzeichen"}},
      "category": "Po"
    }, {"key": "0024", "mappings": {"default": {"default": "Dollar"}}, "category": "Sc"}, {
      "key": "0025",
      "mappings": {"default": {"default": "Prozent"}},
      "category": "Po"
    }, {"key": "0026", "mappings": {"default": {"default": "und"}}, "category": "Po"}, {
      "key": "0027",
      "mappings": {"default": {"default": "Hochkomma"}},
      "category": "Po"
    }, {"key": "002A", "mappings": {"default": {"default": "times"}}, "category": "Po"}, {
      "key": "002B",
      "mappings": {"default": {"default": "plus"}},
      "category": "Sm"
    }, {"key": "002C", "mappings": {"default": {"default": "Komma"}}, "category": "Po"}, {
      "key": "002D",
      "mappings": {"default": {"default": "minus"}},
      "category": "Pd"
    }, {"category": "Po", "key": "002E", "mappings": {"default": {"default": "Punkt"}}}, {
      "key": "002F",
      "mappings": {"default": {"default": "Schrägstrich"}},
      "category": "Po"
    }, {"key": "003A", "mappings": {"default": {"default": "Doppelpunkt"}}, "category": "Po"}, {
      "key": "003B",
      "mappings": {"default": {"default": "Strichpunkt"}},
      "category": "Po"
    }, {"key": "003C", "mappings": {"default": {"default": "kleiner als"}}, "category": "Sm"}, {
      "key": "003D",
      "mappings": {"default": {"default": "ist gleich"}},
      "category": "Sm"
    }, {"key": "003E", "mappings": {"default": {"default": "größer als"}}, "category": "Sm"}, {
      "key": "003F",
      "mappings": {"default": {"default": "Fragezeichen"}},
      "category": "Po"
    }, {"key": "0040", "mappings": {"default": {"default": "Klammeraffe"}}, "category": "Po"}, {
      "key": "005C",
      "mappings": {"default": {"default": "Umgekehrter Schrägstrich"}},
      "category": "Po"
    }, {"key": "005E", "mappings": {"default": {"default": "circumflex"}}, "category": "Sk"}, {
      "key": "005F",
      "mappings": {"default": {"default": "Unterstrich"}},
      "category": "Pc"
    }, {"key": "0060", "mappings": {"default": {"default": "Gravis Akzent"}}, "category": "Sk"}, {
      "key": "007C",
      "mappings": {"default": {"default": "senkrechter Strich"}},
      "category": "Sm"
    }, {"key": "007E", "mappings": {"default": {"default": "tilde"}}, "category": "Sm"}, {
      "key": "00A1",
      "mappings": {"default": {"default": "Umgekehrtes Ausrufezeichen"}},
      "category": "Po"
    }, {"key": "00A2", "mappings": {"default": {"default": "Cent"}}, "category": "Sc"}, {
      "key": "00A3",
      "mappings": {"default": {"default": "Pfund"}},
      "category": "Sc"
    }, {"key": "00A4", "mappings": {"default": {"default": "währungszeichen"}}, "category": "Sc"}, {
      "key": "00A5",
      "mappings": {"default": {"default": "yen"}},
      "category": "Sc"
    }, {"key": "00A6", "mappings": {"default": {"default": "gebrochene Linie"}}, "category": "So"}, {
      "key": "00A7",
      "mappings": {"default": {"default": "Abschnitt"}},
      "category": "Po"
    }, {"key": "00A8", "mappings": {"default": {"default": "diaeresis"}}, "category": "Sk"}, {
      "key": "00A9",
      "mappings": {"default": {"default": "Copyright"}},
      "category": "So"
    }, {
      "key": "00AA",
      "mappings": {"default": {"default": "weibliches Ordnungszeichen"}},
      "category": "Lo"
    }, {
      "category": "Pi",
      "key": "00AB",
      "mappings": {"default": {"default": "Nach links zeigendes doppeltes spitzes Anführungszeichen"}}
    }, {"key": "00AC", "mappings": {"default": {"default": "nicht"}}, "category": "Sm"}, {
      "key": "00AE",
      "mappings": {"default": {"default": "registered sign"}},
      "category": "So"
    }, {
      "key": "00AF",
      "mappings": {"default": {"default": "Überstrich", "alternative": "Makron"}},
      "category": "Sk"
    }, {"key": "00B0", "mappings": {"default": {"default": "Grad"}}, "category": "So"}, {
      "key": "00B1",
      "mappings": {"default": {"default": "plus minus"}},
      "category": "Sm"
    }, {"key": "00B4", "mappings": {"default": {"default": "acute accent"}}, "category": "Sk"}, {
      "key": "00B5",
      "mappings": {"default": {"default": "micro sign"}},
      "category": "Ll"
    }, {"key": "00B6", "mappings": {"default": {"default": "Paragraf"}}, "category": "Po"}, {
      "key": "00B7",
      "mappings": {"default": {"default": "mal", "alternative": "Punkt mittig"}},
      "category": "Po"
    }, {"key": "00B8", "mappings": {"default": {"default": "Cedille"}}, "category": "Sk"}, {
      "key": "00BA",
      "mappings": {"default": {"default": "Männliches Ordnungszeichen"}},
      "category": "Lo"
    }, {
      "category": "Pf",
      "key": "00BB",
      "mappings": {"default": {"default": "Nach rechts zeigendes doppeltes spitzes Anführungszeichen"}}
    }, {
      "key": "00BF",
      "mappings": {"default": {"default": "umgekehrtes Fragezeichen"}},
      "category": "Po"
    }, {"category": "Sm", "key": "00D7", "mappings": {"default": {"default": "mal"}}}, {
      "category": "Sm",
      "key": "00F7",
      "mappings": {"default": {"default": "geteilt durch"}}
    }, {"key": "02D8", "mappings": {"default": {"default": "Breve"}}, "category": "Sk"}, {
      "key": "02D9",
      "mappings": {"default": {"default": "Überpunkt"}},
      "category": "Sk"
    }, {"key": "02DA", "mappings": {"default": {"default": "Überring"}}, "category": "Sk"}, {
      "key": "02DB",
      "mappings": {"default": {"default": "Ogonek"}},
      "category": "Sk"
    }, {"key": "02DC", "mappings": {"default": {"default": "Tilde"}}, "category": "Sk"}, {
      "key": "02DD",
      "mappings": {"default": {"default": "Doppleakutakzent"}},
      "category": "Sk"
    }, {"category": "Pd", "key": "2010", "mappings": {"default": {"default": "Bindestrich"}}}, {
      "category": "Pd",
      "key": "2011",
      "mappings": {"default": {"default": "Nicht brechender Bindestrich"}}
    }, {
      "category": "Pd",
      "key": "2012",
      "mappings": {"default": {"default": "Abbildung Bindestrich"}}
    }, {"category": "Pd", "key": "2013", "mappings": {"default": {"default": "En Dash"}}}, {
      "category": "Pd",
      "key": "2014",
      "mappings": {"default": {"default": "EM Dash"}}
    }, {"category": "Pd", "key": "2015", "mappings": {"default": {"default": "Horizontale Linie"}}}, {
      "category": "Po",
      "key": "2016",
      "mappings": {"default": {"default": "Doppelte vertikale Linie"}}
    }, {
      "category": "Po",
      "key": "2017",
      "mappings": {"default": {"default": "Doppelter Unterstrich"}}
    }, {
      "category": "Pi",
      "key": "2018",
      "mappings": {"default": {"default": "Linkes einfaches Anführungszeichen"}}
    }, {
      "category": "Pf",
      "key": "2019",
      "mappings": {"default": {"default": "Rechtes einfaches Anführungszeichen"}}
    }, {
      "category": "Ps",
      "key": "201A",
      "mappings": {"default": {"default": "Einzelnes rechtes Anführungszeichen"}}
    }, {
      "category": "Pi",
      "key": "201B",
      "mappings": {"default": {"default": "Einzelnes linkes Anführungszeichen"}}
    }, {
      "category": "Pi",
      "key": "201C",
      "mappings": {"default": {"default": "Linkes doppeltes Anführungszeichen"}}
    }, {
      "category": "Pf",
      "key": "201D",
      "mappings": {"default": {"default": "Rechtes doppeltes Anführungszeichen"}}
    }, {
      "category": "Ps",
      "key": "201E",
      "mappings": {"default": {"default": "Doppeltes rechtes Anführungszeichen"}}
    }, {
      "category": "Pi",
      "key": "201F",
      "mappings": {"default": {"default": "Doppeltes linkes Anführungszeichen"}}
    }, {"category": "Po", "key": "2020", "mappings": {"default": {"default": "Dolch"}}}, {
      "category": "Po",
      "key": "2021",
      "mappings": {"default": {"default": "Doppelter Dolch"}}
    }, {"category": "Po", "key": "2022", "mappings": {"default": {"default": "Kugel"}}}, {
      "category": "Po",
      "key": "2023",
      "mappings": {"default": {"default": "Dreieckige Kugel"}}
    }, {"category": "Po", "key": "2024", "mappings": {"default": {"default": "Einzelpunkt"}}}, {
      "category": "Po",
      "key": "2025",
      "mappings": {"default": {"default": "zwei Punkte"}}
    }, {
      "category": "Po",
      "key": "2026",
      "mappings": {"default": {"default": "horizontale Ellipsis"}}
    }, {"category": "Po", "key": "2027", "mappings": {"default": {"default": "Trennungspunkt"}}}, {
      "category": "Po",
      "key": "2030",
      "mappings": {"default": {"default": "promille"}}
    }, {"category": "Po", "key": "2031", "mappings": {"default": {"default": "pro Zehntausend"}}}, {
      "key": "2032",
      "mappings": {"default": {"default": "Strich"}},
      "category": "Po"
    }, {"key": "2033", "mappings": {"default": {"default": "zwei Strich"}}, "category": "Po"}, {
      "key": "2034",
      "mappings": {"default": {"default": "drei Strich"}},
      "category": "Po"
    }, {"category": "Po", "key": "2035", "mappings": {"default": {"default": "strich invertiert"}}}, {
      "category": "Po",
      "key": "2036",
      "mappings": {"default": {"default": "zwei Strich invertiert"}}
    }, {
      "category": "Po",
      "key": "2037",
      "mappings": {"default": {"default": "drei Strich invertiert"}}
    }, {"category": "Po", "key": "2038", "mappings": {"default": {"default": "Caret"}}}, {
      "category": "Pi",
      "key": "2039",
      "mappings": {"default": {"default": "Einfacher nach links zeigender Winkel Anführungszeichen"}}
    }, {
      "category": "Pf",
      "key": "203A",
      "mappings": {"default": {"default": "Einzelne Nach-rechts-Winkel-Anführungszeichen"}}
    }, {"category": "Po", "key": "203B", "mappings": {"default": {"default": "Referenzmarke"}}}, {
      "category": "Po",
      "key": "203C",
      "mappings": {"default": {"default": "Doppelausrufezeichen"}}
    }, {"category": "Po", "key": "203D", "mappings": {"default": {"default": "Interrobang"}}}, {
      "category": "Po",
      "key": "203E",
      "mappings": {"default": {"default": "Überschrift"}}
    }, {"category": "Pc", "key": "203F", "mappings": {"default": {"default": "Undertie"}}}, {
      "category": "Pc",
      "key": "2040",
      "mappings": {"default": {"default": "Zeichen Krawatte"}}
    }, {
      "category": "Po",
      "key": "2041",
      "mappings": {"default": {"default": "Caret-Einfügungspunkt"}}
    }, {"category": "Po", "key": "2042", "mappings": {"default": {"default": "Asterism"}}}, {
      "category": "Po",
      "key": "2043",
      "mappings": {"default": {"default": "Bindestrich Kugel"}}
    }, {
      "category": "Sm",
      "key": "2044",
      "mappings": {"default": {"default": "Fraktions-Schrägstrich"}}
    }, {
      "category": "Po",
      "key": "2047",
      "mappings": {"default": {"default": "Doppel-Fragezeichen"}}
    }, {
      "category": "Po",
      "key": "2048",
      "mappings": {"default": {"default": "Fragezeichen für Ausrufezeichen"}}
    }, {"category": "Po", "key": "2049", "mappings": {"default": {"default": "Ausrufezeichen"}}}, {
      "category": "Po",
      "key": "204B",
      "mappings": {"default": {"default": "Umgekehrtes Pilcrow-Zeichen"}}
    }, {
      "category": "Po",
      "key": "204C",
      "mappings": {"default": {"default": "Schwarze Kugel nach links"}}
    }, {
      "category": "Po",
      "key": "204D",
      "mappings": {"default": {"default": "Schwarze Kugel nach rechts"}}
    }, {
      "category": "Po",
      "key": "204E",
      "mappings": {"default": {"default": "Niedriges Sternchen"}}
    }, {
      "category": "Po",
      "key": "204F",
      "mappings": {"default": {"default": "Umgekehrtes Semikolon"}}
    }, {"category": "Po", "key": "2050", "mappings": {"default": {"default": "Nahansicht"}}}, {
      "category": "Po",
      "key": "2051",
      "mappings": {"default": {"default": "Zwei Sternchen vertikal ausgerichtet"}}
    }, {
      "category": "Sm",
      "key": "2052",
      "mappings": {"default": {"default": "Kommerzielles Minuszeichen"}}
    }, {"category": "Po", "key": "2053", "mappings": {"default": {"default": "Swing Dash"}}}, {
      "category": "Pc",
      "key": "2054",
      "mappings": {"default": {"default": "Inverted Undertie"}}
    }, {
      "category": "Po",
      "key": "2055",
      "mappings": {"default": {"default": "Blume Interpunktionszeichen"}}
    }, {
      "category": "Po",
      "key": "2056",
      "mappings": {"default": {"default": "Drei Punkt-Interpunktion"}}
    }, {"key": "2057", "mappings": {"default": {"default": "quadruple prime"}}, "category": "Po"}, {
      "category": "Po",
      "key": "2058",
      "mappings": {"default": {"default": "Vierpunkt-Interpunktion"}}
    }, {
      "category": "Po",
      "key": "2059",
      "mappings": {"default": {"default": "Fünf Punkt-Interpunktion"}}
    }, {
      "category": "Po",
      "key": "205A",
      "mappings": {"default": {"default": "Zwei Punkt-Interpunktion"}}
    }, {
      "category": "Po",
      "key": "205B",
      "mappings": {"default": {"default": "Vier-Punkt-Markierung"}}
    }, {"category": "Po", "key": "205C", "mappings": {"default": {"default": "Gepunktetes Kreuz"}}}, {
      "category": "Po",
      "key": "205D",
      "mappings": {"default": {"default": "Tricolon"}}
    }, {
      "category": "Po",
      "key": "205E",
      "mappings": {"default": {"default": "Vertikale vier Punkte"}}
    }, {
      "category": "Sm",
      "key": "207A",
      "mappings": {"default": {"default": "Hochgestelltes Pluszeichen"}}
    }, {
      "category": "Sm",
      "key": "207B",
      "mappings": {"default": {"default": "Hochgestelltes Minus"}}
    }, {
      "category": "Sm",
      "key": "207C",
      "mappings": {"default": {"default": "Hochgestelltes Gleichheitszeichen"}}
    }, {
      "category": "Ps",
      "key": "207D",
      "mappings": {"default": {"default": "Hochgestellte linke Klammer"}}
    }, {
      "category": "Pe",
      "key": "207E",
      "mappings": {"default": {"default": "Hochgestellte rechte Klammer"}}
    }, {"category": "Sm", "key": "208A", "mappings": {"default": {"default": "Index Pluszeichen"}}}, {
      "category": "Sm",
      "key": "208B",
      "mappings": {"default": {"default": "Tiefgestelltes Minus"}}
    }, {
      "category": "Sm",
      "key": "208C",
      "mappings": {"default": {"default": "Tiefgestelltes Gleichheitszeichen"}}
    }, {
      "category": "Ps",
      "key": "208D",
      "mappings": {"default": {"default": "Tiefgestellte linke Klammer"}}
    }, {
      "category": "Pe",
      "key": "208E",
      "mappings": {"default": {"default": "Tiefgestellte rechte Klammer"}}
    }, {"category": "So", "key": "214A", "mappings": {"default": {"default": "Grundstücksgrenze"}}}, {
      "category": "Sm",
      "key": "214B",
      "mappings": {"default": {"default": "Ampersand gedreht"}}
    }, {"category": "So", "key": "214C", "mappings": {"default": {"default": "Pro Zeichen"}}}, {
      "category": "So",
      "key": "214D",
      "mappings": {"default": {"default": "Aktieselskab"}}
    }, {"category": "Ll", "key": "214E", "mappings": {"default": {"default": "Gedreht klein F"}}}, {
      "key": "2200",
      "mappings": {"default": {"default": "für alle"}},
      "category": "Sm"
    }, {"key": "2201", "mappings": {"default": {"default": "Komplement"}}, "category": "Sm"}, {
      "key": "2203",
      "mappings": {"default": {"default": "es gibt"}},
      "category": "Sm"
    }, {"key": "2204", "mappings": {"default": {"default": "es gibt nicht"}}, "category": "Sm"}, {
      "key": "2205",
      "mappings": {"default": {"default": "leere Menge"}},
      "category": "Sm"
    }, {"key": "2206", "mappings": {"default": {"default": "Differenz"}}, "category": "Sm"}, {
      "key": "2208",
      "mappings": {"default": {"default": "Element von"}},
      "category": "Sm"
    }, {"key": "2209", "mappings": {"default": {"default": "nicht Element von"}}, "category": "Sm"}, {
      "key": "220A",
      "mappings": {"default": {"default": "Element von"}},
      "category": "Sm"
    }, {"key": "220B", "mappings": {"default": {"default": "hat als Element"}}, "category": "Sm"}, {
      "key": "220C",
      "mappings": {"default": {"default": "hat nicht als Element"}},
      "category": "Sm"
    }, {"key": "220D", "mappings": {"default": {"default": "hat als Element"}}, "category": "Sm"}, {
      "key": "220E",
      "mappings": {"default": {"default": "Ende Beweis"}},
      "category": "Sm"
    }, {"key": "220F", "mappings": {"default": {"default": "Produkt"}}, "category": "Sm"}, {
      "key": "2210",
      "mappings": {"default": {"default": "Koprodukt"}},
      "category": "Sm"
    }, {"key": "2211", "mappings": {"default": {"default": "Summe"}}, "category": "Sm"}, {
      "key": "2212",
      "mappings": {"default": {"default": "minus"}},
      "category": "Sm"
    }, {"key": "2213", "mappings": {"default": {"default": "minus plus"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "2214",
      "mappings": {"default": {"default": "Plus mit Punkt"}}
    }, {"key": "2215", "mappings": {"default": {"default": "Division"}}, "category": "Sm"}, {
      "key": "2216",
      "mappings": {"default": {"default": "Mengendifferenz"}},
      "category": "Sm"
    }, {"key": "2217", "mappings": {"default": {"default": "Sternoperator"}}, "category": "Sm"}, {
      "key": "2218",
      "mappings": {"default": {"default": "verknüpft mit"}},
      "category": "Sm"
    }, {"key": "2219", "mappings": {"default": {"default": "Kreisoperator"}}, "category": "Sm"}, {
      "key": "221A",
      "mappings": {"default": {"default": "Quadratwurzelzeichen"}},
      "category": "Sm"
    }, {"key": "221B", "mappings": {"default": {"default": "Kubikwurzelzeichen"}}, "category": "Sm"}, {
      "key": "221C",
      "mappings": {"default": {"default": "vierte Wurzel"}},
      "category": "Sm"
    }, {"key": "221D", "mappings": {"default": {"default": "proportional zu"}}, "category": "Sm"}, {
      "key": "221E",
      "mappings": {"default": {"default": "unendlich"}},
      "category": "Sm"
    }, {"key": "221F", "mappings": {"default": {"default": "rechter Winkel"}}, "category": "Sm"}, {
      "key": "2220",
      "mappings": {"default": {"default": "Winkel"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "2221", "mappings": {"default": {"default": "gerichteter Winkel"}}}, {
      "category": "Sm",
      "key": "2222",
      "mappings": {"default": {"default": "Raumwinkel"}}
    }, {"key": "2223", "mappings": {"default": {"default": "teilt"}}, "category": "Sm"}, {
      "key": "2224",
      "mappings": {"default": {"default": "teilt nicht"}},
      "category": "Sm"
    }, {"key": "2225", "mappings": {"default": {"default": "parallel zu"}}, "category": "Sm"}, {
      "key": "2226",
      "mappings": {"default": {"default": "nicht parallel zu"}},
      "category": "Sm"
    }, {"key": "2227", "mappings": {"default": {"default": "und"}}, "category": "Sm"}, {
      "key": "2228",
      "mappings": {"default": {"default": "oder"}},
      "category": "Sm"
    }, {"key": "2229", "mappings": {"default": {"default": "Durchschnitt"}}, "category": "Sm"}, {
      "key": "222A",
      "mappings": {"default": {"default": "Vereinigung"}},
      "category": "Sm"
    }, {"key": "222B", "mappings": {"default": {"default": "Integral"}}, "category": "Sm"}, {
      "key": "222C",
      "mappings": {"default": {"default": "Doppelintegral"}},
      "category": "Sm"
    }, {"key": "222D", "mappings": {"default": {"default": "Dreifachintegral"}}, "category": "Sm"}, {
      "key": "222E",
      "mappings": {"default": {"default": "Randintegral"}},
      "category": "Sm"
    }, {"key": "222F", "mappings": {"default": {"default": "Oberflächenintegral"}}, "category": "Sm"}, {
      "key": "2230",
      "mappings": {"default": {"default": "Volumenintegral"}},
      "category": "Sm"
    }, {
      "key": "2231",
      "mappings": {"default": {"default": "Integral im Uhrzeigersinn"}},
      "category": "Sm"
    }, {
      "key": "2232",
      "mappings": {"default": {"default": "Kurvenintegral im Uhrzeigersinn"}},
      "category": "Sm"
    }, {
      "key": "2233",
      "mappings": {"default": {"default": "Kurvenintegral im Gegenuhrzeigersinn"}},
      "category": "Sm"
    }, {"key": "2234", "mappings": {"default": {"default": "folglich"}}, "category": "Sm"}, {
      "key": "2235",
      "mappings": {"default": {"default": "weil"}},
      "category": "Sm"
    }, {"key": "2236", "mappings": {"default": {"default": "Verhältnis"}}, "category": "Sm"}, {
      "key": "2237",
      "mappings": {"default": {"default": "Proportion"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "2238", "mappings": {"default": {"default": "Minus mit Punkt"}}}, {
      "category": "Sm",
      "key": "2239",
      "mappings": {"default": {"default": "Überschuss"}}
    }, {
      "key": "223A",
      "mappings": {"default": {"default": "geometrische Proportion"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "223B", "mappings": {"default": {"default": "homothetisch"}}}, {
      "category": "Sm",
      "key": "223C",
      "mappings": {"default": {"default": "Tildeoperator"}}
    }, {"category": "Sm", "key": "223D", "mappings": {"default": {"default": "Umgekehrte Tilde"}}}, {
      "category": "Sm",
      "key": "223E",
      "mappings": {"default": {"default": "Umgekehrtes stummes S"}}
    }, {"category": "Sm", "key": "223F", "mappings": {"default": {"default": "Sinuswelle"}}}, {
      "category": "Sm",
      "key": "2240",
      "mappings": {"default": {"default": "Kranzprodukt"}}
    }, {
      "category": "Sm",
      "key": "2241",
      "mappings": {"default": {"default": "durchgestrichene Tilde"}}
    }, {"category": "Sm", "key": "2242", "mappings": {"default": {"default": "Minus über Tilde"}}}, {
      "key": "2243",
      "mappings": {"default": {"default": "asymptotisch gleich"}},
      "category": "Sm"
    }, {
      "key": "2244",
      "mappings": {"default": {"default": "nicht asymptotisch gleich"}},
      "category": "Sm"
    }, {"key": "2245", "mappings": {"default": {"default": "ungefähr gleich"}}, "category": "Sm"}, {
      "key": "2246",
      "mappings": {"default": {"default": "ungefähr, aber nicht ganz gleich"}},
      "category": "Sm"
    }, {
      "key": "2247",
      "mappings": {"default": {"default": "weder ungefähr noch ganz gleich"}},
      "category": "Sm"
    }, {"key": "2248", "mappings": {"default": {"default": "beinahe gleich"}}, "category": "Sm"}, {
      "key": "2249",
      "mappings": {"default": {"default": "nicht beinahe gleich"}},
      "category": "Sm"
    }, {
      "key": "224A",
      "mappings": {"default": {"default": "beinahe gleich oder gleich"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "224B", "mappings": {"default": {"default": "dreifache Tilde"}}}, {
      "category": "Sm",
      "key": "224C",
      "mappings": {"default": {"default": "alles gleich"}}
    }, {"key": "224D", "mappings": {"default": {"default": "äquivalent zu"}}, "category": "Sm"}, {
      "key": "224E",
      "mappings": {"default": {"default": "geometrisch äquivalent zu"}},
      "category": "Sm"
    }, {"key": "224F", "mappings": {"default": {"default": "Differenz zwischen"}}, "category": "Sm"}, {
      "key": "2250",
      "mappings": {"default": {"default": "nähert sich der Grenze"}},
      "category": "Sm"
    }, {"key": "2251", "mappings": {"default": {"default": "geometrisch gleich"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "2252",
      "mappings": {"default": {"default": "Ungefähr gleich oder das Bild von"}}
    }, {
      "category": "Sm",
      "key": "2253",
      "mappings": {"default": {"default": "Bild von oder ungefähr gleich"}}
    }, {"key": "2254", "mappings": {"default": {"default": "definiert als"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "2255",
      "mappings": {"default": {"default": "definiert als von rechts"}}
    }, {"category": "Sm", "key": "2256", "mappings": {"default": {"default": "ungefähr gleich"}}}, {
      "category": "Sm",
      "key": "2257",
      "mappings": {"default": {"default": "ungefähr gleich"}}
    }, {"key": "2258", "mappings": {"default": {"default": "entspricht"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "2259",
      "mappings": {"default": {"default": "entspricht"}}
    }, {"category": "Sm", "key": "225A", "mappings": {"default": {"default": "gleichwinklig zu"}}}, {
      "category": "Sm",
      "key": "225B",
      "mappings": {"default": {"default": "gleich mit Stern"}}
    }, {"category": "Sm", "key": "225C", "mappings": {"default": {"default": "gleich mit Delta"}}}, {
      "key": "225D",
      "mappings": {"default": {"default": "definitionsgemäß gleich"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "225E", "mappings": {"default": {"default": "gemessen mit"}}}, {
      "category": "Sm",
      "key": "225F",
      "mappings": {"default": {"default": "vielleicht gleich"}}
    }, {"key": "2260", "mappings": {"default": {"default": "ungleich"}}, "category": "Sm"}, {
      "key": "2261",
      "mappings": {"default": {"default": "kongruent mit"}},
      "category": "Sm"
    }, {"key": "2262", "mappings": {"default": {"default": "nicht kongruent mit"}}, "category": "Sm"}, {
      "key": "2263",
      "mappings": {"default": {"default": "genau äquivalent mit"}},
      "category": "Sm"
    }, {"key": "2264", "mappings": {"default": {"default": "kleiner oder gleich"}}, "category": "Sm"}, {
      "key": "2265",
      "mappings": {"default": {"default": "größer oder gleich"}},
      "category": "Sm"
    }, {
      "key": "2266",
      "mappings": {"default": {"default": "kleiner als über gleich"}},
      "category": "Sm"
    }, {
      "key": "2267",
      "mappings": {"default": {"default": "größer als über gleich"}},
      "category": "Sm"
    }, {
      "key": "2268",
      "mappings": {"default": {"default": "kleiner als, aber nicht gleich"}},
      "category": "Sm"
    }, {
      "key": "2269",
      "mappings": {"default": {"default": "größer als, aber nicht gleich"}},
      "category": "Sm"
    }, {"key": "226A", "mappings": {"default": {"default": "viel kleiner als"}}, "category": "Sm"}, {
      "key": "226B",
      "mappings": {"default": {"default": "viel größer als"}},
      "category": "Sm"
    }, {"key": "226C", "mappings": {"default": {"default": "zwischen"}}, "category": "Sm"}, {
      "key": "226D",
      "mappings": {"default": {"default": "nicht äquivalent mit"}},
      "category": "Sm"
    }, {"key": "226E", "mappings": {"default": {"default": "nicht kleiner als"}}, "category": "Sm"}, {
      "key": "226F",
      "mappings": {"default": {"default": "nicht größer als"}},
      "category": "Sm"
    }, {
      "key": "2270",
      "mappings": {"default": {"default": "weder kleiner als noch gleich"}},
      "category": "Sm"
    }, {
      "key": "2271",
      "mappings": {"default": {"default": "weder größer als noch gleich"}},
      "category": "Sm"
    }, {
      "key": "2272",
      "mappings": {"default": {"default": "kleiner als oder äquivalent"}},
      "category": "Sm"
    }, {
      "key": "2273",
      "mappings": {"default": {"default": "größer als oder äquivalent"}},
      "category": "Sm"
    }, {
      "key": "2274",
      "mappings": {"default": {"default": "weder kleiner als noch äquivalent"}},
      "category": "Sm"
    }, {
      "key": "2275",
      "mappings": {"default": {"default": "weder größer als noch äquivalent"}},
      "category": "Sm"
    }, {
      "key": "2276",
      "mappings": {"default": {"default": "kleiner oder größer als"}},
      "category": "Sm"
    }, {
      "key": "2277",
      "mappings": {"default": {"default": "größer oder kleiner als"}},
      "category": "Sm"
    }, {
      "key": "2278",
      "mappings": {"default": {"default": "weder kleiner noch größer als"}},
      "category": "Sm"
    }, {
      "key": "2279",
      "mappings": {"default": {"default": "weder größer noch kleiner als"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "227A", "mappings": {"default": {"default": "vorangehend"}}}, {
      "category": "Sm",
      "key": "227B",
      "mappings": {"default": {"default": "nachfolgend"}}
    }, {
      "category": "Sm",
      "key": "227C",
      "mappings": {"default": {"default": "vorangehend oder gleich"}}
    }, {
      "category": "Sm",
      "key": "227D",
      "mappings": {"default": {"default": "nachfolgend oder gleich"}}
    }, {
      "category": "Sm",
      "key": "227E",
      "mappings": {"default": {"default": "vorangehend oder äquivalent"}}
    }, {
      "category": "Sm",
      "key": "227F",
      "mappings": {"default": {"default": "nachfolgend oder äquivalent"}}
    }, {"category": "Sm", "key": "2280", "mappings": {"default": {"default": "nicht vorangehend"}}}, {
      "category": "Sm",
      "key": "2281",
      "mappings": {"default": {"default": "nicht nachfolgend"}}
    }, {
      "category": "Sm",
      "key": "2282",
      "mappings": {"default": {"default": "echte Teilmenge von"}}
    }, {
      "category": "Sm",
      "key": "2283",
      "mappings": {"default": {"default": "echte Obermenge von"}}
    }, {
      "category": "Sm",
      "key": "2284",
      "mappings": {"default": {"default": "keine echte Teilmenge von"}}
    }, {
      "category": "Sm",
      "key": "2285",
      "mappings": {"default": {"default": "kein echte Obermenge von"}}
    }, {
      "category": "Sm",
      "key": "2286",
      "mappings": {"default": {"default": "Teilmenge oder gleich"}}
    }, {
      "category": "Sm",
      "key": "2287",
      "mappings": {"default": {"default": "Obermenge oder gleich"}}
    }, {
      "category": "Sm",
      "key": "2288",
      "mappings": {"default": {"default": "weder Teilmenge noch gleich"}}
    }, {
      "category": "Sm",
      "key": "2289",
      "mappings": {"default": {"default": "weder Obermenge noch gleich"}}
    }, {
      "category": "Sm",
      "key": "228A",
      "mappings": {"default": {"default": "Teilmenge aber nicht gleich"}}
    }, {
      "category": "Sm",
      "key": "228B",
      "mappings": {"default": {"default": "Obermenge aber nicht gleich"}}
    }, {"category": "Sm", "key": "228C", "mappings": {"default": {"default": "Multimenge"}}}, {
      "category": "Sm",
      "key": "228D",
      "mappings": {"default": {"default": "Multimengenmultiplikation"}}
    }, {
      "category": "Sm",
      "key": "228E",
      "mappings": {"default": {"default": "Multimengenvereinigung"}}
    }, {
      "category": "Sm",
      "key": "228F",
      "mappings": {"default": {"default": "quadratisches Bild von"}}
    }, {
      "category": "Sm",
      "key": "2290",
      "mappings": {"default": {"default": "quadratisches Original von"}}
    }, {
      "category": "Sm",
      "key": "2291",
      "mappings": {"default": {"default": "quadratisches Bild oder gleich"}}
    }, {
      "category": "Sm",
      "key": "2292",
      "mappings": {"default": {"default": "quadratisches Original oder gleich"}}
    }, {
      "category": "Sm",
      "key": "2293",
      "mappings": {"default": {"default": "quadratische Schnittmenge"}}
    }, {
      "category": "Sm",
      "key": "2294",
      "mappings": {"default": {"default": "quadratische VereinigungTasse"}}
    }, {"category": "Sm", "key": "2295", "mappings": {"default": {"default": "eingekreistes Plus"}}}, {
      "category": "Sm",
      "key": "2296",
      "mappings": {"default": {"default": "eingekreistes Minus"}}
    }, {
      "category": "Sm",
      "key": "2297",
      "mappings": {"default": {"default": "eingekreiste Multiplikation"}}
    }, {
      "category": "Sm",
      "key": "2298",
      "mappings": {"default": {"default": "eingekreiste Division"}}
    }, {
      "category": "Sm",
      "key": "2299",
      "mappings": {"default": {"default": "eingekreister Punktoperator"}}
    }, {
      "category": "Sm",
      "key": "229A",
      "mappings": {"default": {"default": "eingekreister Ringoperator"}}
    }, {
      "category": "Sm",
      "key": "229B",
      "mappings": {"default": {"default": "eingekreister Sternoperator"}}
    }, {
      "category": "Sm",
      "key": "229C",
      "mappings": {"default": {"default": "eingekreistes Gleich"}}
    }, {
      "category": "Sm",
      "key": "229D",
      "mappings": {"default": {"default": "eingekreister Gedankenstrich"}}
    }, {"category": "Sm", "key": "229E", "mappings": {"default": {"default": "eingerahmtes Plus"}}}, {
      "category": "Sm",
      "key": "229F",
      "mappings": {"default": {"default": "eingerahmtes Minus"}}
    }, {
      "category": "Sm",
      "key": "22A0",
      "mappings": {"default": {"default": "eingerahmte Multiplikation"}}
    }, {
      "category": "Sm",
      "key": "22A1",
      "mappings": {"default": {"default": "eingerahmter Punktoperator"}}
    }, {"category": "Sm", "key": "22A2", "mappings": {"default": {"default": "ergibt"}}}, {
      "category": "Sm",
      "key": "22A3",
      "mappings": {"default": {"default": "ergibt nicht"}}
    }, {"category": "Sm", "key": "22A4", "mappings": {"default": {"default": "senkrecht von"}}}, {
      "category": "Sm",
      "key": "22A5",
      "mappings": {"default": {"default": "senkrecht auf"}}
    }, {"category": "Sm", "key": "22A6", "mappings": {"default": {"default": "Behauptung"}}}, {
      "category": "Sm",
      "key": "22A7",
      "mappings": {"default": {"default": "Model"}}
    }, {"category": "Sm", "key": "22A8", "mappings": {"default": {"default": "wahr"}}}, {
      "category": "Sm",
      "key": "22A9",
      "mappings": {"default": {"default": "erzwingen"}}
    }, {
      "category": "Sm",
      "key": "22AA",
      "mappings": {"default": {"default": "dreifache vertikale Leiste mit rechtem Drehkreuz"}}
    }, {
      "category": "Sm",
      "key": "22AB",
      "mappings": {"default": {"default": "doppelte vertikale Leiste mit doppeltem rechtem Drehkreuz"}}
    }, {"category": "Sm", "key": "22AC", "mappings": {"default": {"default": "beweist nicht"}}}, {
      "category": "Sm",
      "key": "22AD",
      "mappings": {"default": {"default": "nicht wahr"}}
    }, {"category": "Sm", "key": "22AE", "mappings": {"default": {"default": "nicht erzwingen"}}}, {
      "category": "Sm",
      "key": "22AF",
      "mappings": {"default": {"default": "negierte doppelte vertikale Leiste mit doppeltem rechten Drehkreuz"}}
    }, {
      "category": "Sm",
      "key": "22B0",
      "mappings": {"default": {"default": "vorangehend in Relation"}}
    }, {
      "category": "Sm",
      "key": "22B1",
      "mappings": {"default": {"default": "nachfolgend in Relation"}}
    }, {
      "category": "Sm",
      "key": "22B2",
      "mappings": {"default": {"default": "normale Untergruppe"}}
    }, {
      "category": "Sm",
      "key": "22B3",
      "mappings": {"default": {"default": "enthält normale Untergruppe"}}
    }, {
      "category": "Sm",
      "key": "22B4",
      "mappings": {"default": {"default": "normale Untergruppe von oder gleich"}}
    }, {
      "category": "Sm",
      "key": "22B5",
      "mappings": {"default": {"default": "enthält normale Untergruppe oder gleich"}}
    }, {"category": "Sm", "key": "22B6", "mappings": {"default": {"default": "Original von"}}}, {
      "category": "Sm",
      "key": "22B7",
      "mappings": {"default": {"default": "Bild von"}}
    }, {"category": "Sm", "key": "22B8", "mappings": {"default": {"default": "Mehrfachzuordnung"}}}, {
      "category": "Sm",
      "key": "22B9",
      "mappings": {"default": {"default": "hermitesch konjugierte Matrix"}}
    }, {"category": "Sm", "key": "22BA", "mappings": {"default": {"default": "einschalten"}}}, {
      "category": "Sm",
      "key": "22BB",
      "mappings": {"default": {"default": "Ausschließendes Oder"}}
    }, {"category": "Sm", "key": "22BC", "mappings": {"default": {"default": "Nand verknüpft mit"}}}, {
      "category": "Sm",
      "key": "22BD",
      "mappings": {"default": {"default": "Nor verknüpft mit"}}
    }, {
      "category": "Sm",
      "key": "22BF",
      "mappings": {"default": {"default": "rechtwinkliges Dreieck"}}
    }, {
      "category": "Sm",
      "key": "22C0",
      "mappings": {"default": {"default": "N-stufiges logisches Und"}}
    }, {
      "category": "Sm",
      "key": "22C1",
      "mappings": {"default": {"default": "N-stufiges logisches Oder"}}
    }, {
      "category": "Sm",
      "key": "22C2",
      "mappings": {"default": {"default": "N-stufiger Durchschnitt"}}
    }, {
      "category": "Sm",
      "key": "22C3",
      "mappings": {"default": {"default": "N-stufige Vereinigung"}}
    }, {"category": "Sm", "key": "22C4", "mappings": {"default": {"default": "Rautenoperator"}}}, {
      "category": "Sm",
      "key": "22C5",
      "mappings": {"default": {"default": "mal", "alternative": "Multiplikationspunkt"}}
    }, {"category": "Sm", "key": "22C6", "mappings": {"default": {"default": "Sternoperator"}}}, {
      "category": "Sm",
      "key": "22C7",
      "mappings": {"default": {"default": "Divisionsanzahl"}}
    }, {"category": "Sm", "key": "22C8", "mappings": {"default": {"default": "Schleife"}}}, {
      "category": "Sm",
      "key": "22C9",
      "mappings": {"default": {"default": "linkes halbdirektes Produkt"}}
    }, {
      "category": "Sm",
      "key": "22CA",
      "mappings": {"default": {"default": "rechtes halbdirektes Produkt"}}
    }, {
      "category": "Sm",
      "key": "22CB",
      "mappings": {"default": {"default": "linkes halbdirektes Produkt"}}
    }, {
      "category": "Sm",
      "key": "22CC",
      "mappings": {"default": {"default": "rechtes halbdirektes Produkt"}}
    }, {
      "category": "Sm",
      "key": "22CD",
      "mappings": {"default": {"default": "umgekehrte Tilde gleich"}}
    }, {
      "category": "Sm",
      "key": "22CE",
      "mappings": {"default": {"default": "geschweiftes logisches Oder"}}
    }, {
      "category": "Sm",
      "key": "22CF",
      "mappings": {"default": {"default": "geschweiftes logisches Und"}}
    }, {"category": "Sm", "key": "22D0", "mappings": {"default": {"default": "doppelte Teilmenge"}}}, {
      "category": "Sm",
      "key": "22D1",
      "mappings": {"default": {"default": "doppelte Obermenge"}}
    }, {
      "category": "Sm",
      "key": "22D2",
      "mappings": {"default": {"default": "doppelter Durchschnitt"}}
    }, {
      "category": "Sm",
      "key": "22D3",
      "mappings": {"default": {"default": "doppelte Vereinigung"}}
    }, {
      "category": "Sm",
      "key": "22D4",
      "mappings": {"default": {"default": "echter Durchschnitt"}}
    }, {
      "category": "Sm",
      "key": "22D5",
      "mappings": {"default": {"default": "gleich und parallel"}}
    }, {
      "category": "Sm",
      "key": "22D6",
      "mappings": {"default": {"default": "kleiner als mit Punkt"}}
    }, {
      "category": "Sm",
      "key": "22D7",
      "mappings": {"default": {"default": "größer als mit Punkt"}}
    }, {
      "category": "Sm",
      "key": "22D8",
      "mappings": {"default": {"default": "sehr viel kleiner als"}}
    }, {
      "category": "Sm",
      "key": "22D9",
      "mappings": {"default": {"default": "sehr viel größer als"}}
    }, {
      "category": "Sm",
      "key": "22DA",
      "mappings": {"default": {"default": "kleiner als, gleich oder größer als"}}
    }, {
      "category": "Sm",
      "key": "22DB",
      "mappings": {"default": {"default": "größer als, gleich oder kleiner als"}}
    }, {
      "category": "Sm",
      "key": "22DC",
      "mappings": {"default": {"default": "gleich oder kleiner als"}}
    }, {
      "category": "Sm",
      "key": "22DD",
      "mappings": {"default": {"default": "gleich oder größer als"}}
    }, {
      "category": "Sm",
      "key": "22DE",
      "mappings": {"default": {"default": "gleich oder vorangehend"}}
    }, {
      "category": "Sm",
      "key": "22DF",
      "mappings": {"default": {"default": "gleich oder nachfolgend"}}
    }, {
      "category": "Sm",
      "key": "22E0",
      "mappings": {"default": {"default": "weder vorangehend oder gleich"}}
    }, {
      "category": "Sm",
      "key": "22E1",
      "mappings": {"default": {"default": "weder nachfolgend oder gleich"}}
    }, {
      "category": "Sm",
      "key": "22E2",
      "mappings": {"default": {"default": "kein quadratisches Bild oder gleich"}}
    }, {
      "category": "Sm",
      "key": "22E3",
      "mappings": {"default": {"default": "kein quadratisches Original oder gleich"}}
    }, {
      "category": "Sm",
      "key": "22E4",
      "mappings": {"default": {"default": "quadratisches Bild oder nicht gleich"}}
    }, {
      "category": "Sm",
      "key": "22E5",
      "mappings": {"default": {"default": "quadratisches Original oder nicht gleich"}}
    }, {
      "category": "Sm",
      "key": "22E6",
      "mappings": {"default": {"default": "kleiner als, aber nicht äquivalent"}}
    }, {
      "category": "Sm",
      "key": "22E7",
      "mappings": {"default": {"default": "größer als, aber nicht äquivalent"}}
    }, {
      "category": "Sm",
      "key": "22E8",
      "mappings": {"default": {"default": "vorangehend, aber nicht äquivalent"}}
    }, {
      "category": "Sm",
      "key": "22E9",
      "mappings": {"default": {"default": "nachfolgend, aber nicht äquivalent"}}
    }, {
      "category": "Sm",
      "key": "22EA",
      "mappings": {"default": {"default": "nicht normale Untergruppe von"}}
    }, {
      "category": "Sm",
      "key": "22EB",
      "mappings": {"default": {"default": "enthält keine normale Untergruppe"}}
    }, {
      "category": "Sm",
      "key": "22EC",
      "mappings": {"default": {"default": "keine normale Untergruppe von oder gleich"}}
    }, {
      "category": "Sm",
      "key": "22ED",
      "mappings": {"default": {"default": "enthält nicht als normale Untergruppe oder gleich"}}
    }, {"category": "Sm", "key": "22EE", "mappings": {"default": {"default": "Vertikale Ellipsis"}}}, {
      "category": "Sm",
      "key": "22EF",
      "mappings": {"default": {"default": "Zentrierte horizontale Ellipse"}}
    }, {
      "category": "Sm",
      "key": "22F0",
      "mappings": {"default": {"default": "Diagonale Ellipse unten links nach oben rechts"}}
    }, {
      "category": "Sm",
      "key": "22F1",
      "mappings": {"default": {"default": "Diagonale Ellipse, oben links nach unten rechts"}}
    }, {
      "category": "Sm",
      "key": "22F2",
      "mappings": {"default": {"default": "Element mit langem horizontalen Strich"}}
    }, {
      "category": "Sm",
      "key": "22F3",
      "mappings": {"default": {"default": "Element mit vertikalem Strich am Ende des horizontalen Strichs"}}
    }, {
      "category": "Sm",
      "key": "22F4",
      "mappings": {"default": {"default": "kleines Element mit vertikalem Strich am Ende des horizontalen Strichs"}}
    }, {"category": "Sm", "key": "22F5", "mappings": {"default": {"default": "Element mit Punkt"}}}, {
      "category": "Sm",
      "key": "22F6",
      "mappings": {"default": {"default": "Element mit Überstrich"}}
    }, {
      "key": "22F7",
      "mappings": {"default": {"default": "kleines Element mit Überstrich"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "22F8",
      "mappings": {"default": {"default": "Element mit Unterstrich"}}
    }, {
      "category": "Sm",
      "key": "22F9",
      "mappings": {"default": {"default": "Element mit 2 horizontalen Strichen"}}
    }, {
      "category": "Sm",
      "key": "22FA",
      "mappings": {"default": {"default": "umgekehrtes Elementzeichen mit langem horizontalen Strich"}}
    }, {
      "category": "Sm",
      "key": "22FB",
      "mappings": {"default": {"default": "umgekehrtes Elementzeichen mit vertikalem Strich am Ende des horizontalen Strichs"}}
    }, {
      "category": "Sm",
      "key": "22FC",
      "mappings": {"default": {"default": "kleines umgekehrtes Elementzeichen mit vertikalem Strich am Ende des horizontalen Strichs"}}
    }, {
      "key": "22FD",
      "mappings": {"default": {"default": "umgekehrtes Elementzeichen mit Überstrich"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "22FE",
      "mappings": {"default": {"default": "kleines umgekehrtes Elementzeichen mit Überstrich"}}
    }, {
      "category": "Sm",
      "key": "22FF",
      "mappings": {"default": {"default": "Z-Notation-Bag-Mitgliedschaft"}}
    }, {
      "category": "So",
      "key": "2300",
      "mappings": {"default": {"default": "Durchmesser-Zeichen"}}
    }, {"category": "So", "key": "2302", "mappings": {"default": {"default": "Haus"}}}, {
      "category": "So",
      "key": "2305",
      "mappings": {"default": {"default": "Projektiv"}}
    }, {"category": "So", "key": "2306", "mappings": {"default": {"default": "Perspektive"}}}, {
      "category": "So",
      "key": "2307",
      "mappings": {"default": {"default": "Schlangenlinie"}}
    }, {
      "category": "So",
      "key": "2310",
      "mappings": {"default": {"default": "Umgekehrtes Nichtzeichen"}}
    }, {"category": "So", "key": "2311", "mappings": {"default": {"default": "Quadratische Raute"}}}, {
      "category": "So",
      "key": "2312",
      "mappings": {"default": {"default": "Bogen"}}
    }, {"category": "So", "key": "2313", "mappings": {"default": {"default": "Segment"}}}, {
      "category": "So",
      "key": "2314",
      "mappings": {"default": {"default": "Sektor"}}
    }, {
      "category": "So",
      "key": "2795",
      "mappings": {"default": {"default": "Schweres Pluszeichen"}}
    }, {
      "category": "So",
      "key": "2796",
      "mappings": {"default": {"default": "Schweres Minuszeichen"}}
    }, {
      "category": "So",
      "key": "2797",
      "mappings": {"default": {"default": "Heavy Division Sign"}}
    }, {"category": "So", "key": "27B0", "mappings": {"default": {"default": "Curly Loop"}}}, {
      "category": "So",
      "key": "27BF",
      "mappings": {"default": {"default": "Doppelte geschweifte Schleife"}}
    }, {
      "category": "Sm",
      "key": "27C1",
      "mappings": {"default": {"default": "Weißes Dreieck, das kleines weißes Dreieck enthält"}}
    }, {"category": "Sm", "key": "27C2", "mappings": {"default": {"default": "Aufrecht"}}}, {
      "category": "Sm",
      "key": "27C3",
      "mappings": {"default": {"default": "Subset öffnen"}}
    }, {"category": "Sm", "key": "27C4", "mappings": {"default": {"default": "Superset öffnen"}}}, {
      "category": "Sm",
      "key": "27C7",
      "mappings": {"default": {"default": "Oder mit innerem Punkt"}}
    }, {
      "category": "Sm",
      "key": "27C8",
      "mappings": {"default": {"default": "Umgekehrter Schrägstrich Vorhergehender Teilsatz"}}
    }, {
      "category": "Sm",
      "key": "27C9",
      "mappings": {"default": {"default": "Superset Vorgänger Schrägstrich"}}
    }, {
      "category": "Sm",
      "key": "27CA",
      "mappings": {"default": {"default": "Vertikaler Balken mit horizontalem Hub"}}
    }, {
      "category": "Sm",
      "key": "27CB",
      "mappings": {"default": {"default": "Mathematische aufsteigende Diagonale"}}
    }, {"category": "Sm", "key": "27CC", "mappings": {"default": {"default": "Lange Abteilung"}}}, {
      "category": "Sm",
      "key": "27CD",
      "mappings": {"default": {"default": "Mathematische fallende Diagonale"}}
    }, {"category": "Sm", "key": "27CE", "mappings": {"default": {"default": "quadratisches und"}}}, {
      "category": "Sm",
      "key": "27CF",
      "mappings": {"default": {"default": "quadratisches oder"}}
    }, {
      "category": "Sm",
      "key": "27D0",
      "mappings": {"default": {"default": "Weißer Diamant mit zentriertem Punkt"}}
    }, {"category": "Sm", "key": "27D1", "mappings": {"default": {"default": "Und mit Dot"}}}, {
      "category": "Sm",
      "key": "27D2",
      "mappings": {"default": {"default": "Element der Öffnung nach oben"}}
    }, {
      "category": "Sm",
      "key": "27D3",
      "mappings": {"default": {"default": "Untere rechte Ecke mit Punkt"}}
    }, {
      "category": "Sm",
      "key": "27D4",
      "mappings": {"default": {"default": "Obere linke Ecke mit Punkt"}}
    }, {
      "category": "Sm",
      "key": "27D5",
      "mappings": {"default": {"default": "Linke äußere Verbindung"}}
    }, {
      "category": "Sm",
      "key": "27D6",
      "mappings": {"default": {"default": "rechter äußerer Join"}}
    }, {
      "category": "Sm",
      "key": "27D7",
      "mappings": {"default": {"default": "Volle äußere Verbindung"}}
    }, {"category": "Sm", "key": "27D8", "mappings": {"default": {"default": "Große Up Tack"}}}, {
      "category": "Sm",
      "key": "27D9",
      "mappings": {"default": {"default": "Großes Down Tack"}}
    }, {
      "category": "Sm",
      "key": "27DA",
      "mappings": {"default": {"default": "Linkes und rechtes doppeltes Drehkreuz"}}
    }, {
      "category": "Sm",
      "key": "27DB",
      "mappings": {"default": {"default": "Linker und rechter Tack"}}
    }, {"category": "Sm", "key": "27DC", "mappings": {"default": {"default": "Linke Multimap"}}}, {
      "category": "Sm",
      "key": "27DD",
      "mappings": {"default": {"default": "Long Right Tack"}}
    }, {"category": "Sm", "key": "27DE", "mappings": {"default": {"default": "Langer linker Tack"}}}, {
      "category": "Sm",
      "key": "27DF",
      "mappings": {"default": {"default": "Up Tack mit Kreis oben"}}
    }, {
      "category": "Sm",
      "key": "27E0",
      "mappings": {"default": {"default": "Raute durch horizontale Regel geteilt"}}
    }, {
      "category": "Sm",
      "key": "27E1",
      "mappings": {"default": {"default": "Weißer konkavseitiger Diamant"}}
    }, {
      "category": "Sm",
      "key": "27E2",
      "mappings": {"default": {"default": "Weißer konkavseitiger Diamant mit Zecke nach links"}}
    }, {
      "category": "Sm",
      "key": "27E3",
      "mappings": {"default": {"default": "Weißer konkavseitiger Diamant mit Zecke nach rechts"}}
    }, {
      "category": "Sm",
      "key": "27E4",
      "mappings": {"default": {"default": "Weißes Quadrat mit Zecke nach links"}}
    }, {
      "category": "Sm",
      "key": "27E5",
      "mappings": {"default": {"default": "Weißes Quadrat mit Tick nach rechts"}}
    }, {
      "category": "Sm",
      "key": "292B",
      "mappings": {"default": {"default": "Steigende diagonale Kreuzung fallende Diagonale"}}
    }, {
      "category": "Sm",
      "key": "292C",
      "mappings": {"default": {"default": "Fallende diagonale Kreuzung steigende Diagonale"}}
    }, {
      "category": "Sm",
      "key": "2980",
      "mappings": {"default": {"default": "Dreifacher vertikaler Balkenbegrenzer"}}
    }, {"category": "Sm", "key": "2981", "mappings": {"default": {"default": "Z Notationspunkt"}}}, {
      "category": "Sm",
      "key": "2982",
      "mappings": {"default": {"default": "Z Notationstyp Doppelpunkt"}}
    }, {"category": "Sm", "key": "2999", "mappings": {"default": {"default": "Gepunkteter Zaun"}}}, {
      "category": "Sm",
      "key": "299A",
      "mappings": {"default": {"default": "Vertikale Zickzacklinie"}}
    }, {
      "category": "Sm",
      "key": "29B0",
      "mappings": {"default": {"default": "Umgekehrter leerer Satz"}}
    }, {
      "category": "Sm",
      "key": "29B1",
      "mappings": {"default": {"default": "Leeres Set mit Overbar"}}
    }, {
      "category": "Sm",
      "key": "29B2",
      "mappings": {"default": {"default": "Leeres Set mit kleinem Kreis oben"}}
    }, {
      "category": "Sm",
      "key": "29B5",
      "mappings": {"default": {"default": "Kreis mit horizontaler Leiste"}}
    }, {
      "key": "29B6",
      "mappings": {"default": {"default": "circled vertical bar"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29B7",
      "mappings": {"default": {"default": "eingekreiste Parallele"}}
    }, {
      "category": "Sm",
      "key": "29B8",
      "mappings": {"default": {"default": "eingekreister umgekehrter Schrägstrich"}}
    }, {
      "category": "Sm",
      "key": "29B9",
      "mappings": {"default": {"default": "eingekreistes Senkrecht"}}
    }, {
      "category": "Sm",
      "key": "29BA",
      "mappings": {"default": {"default": "Kreis geteilt durch horizontale Leiste und obere Hälfte geteilt durch vertikale Leiste"}}
    }, {
      "category": "Sm",
      "key": "29BB",
      "mappings": {"default": {"default": "Kreis mit überlagertem X"}}
    }, {
      "category": "Sm",
      "key": "29BC",
      "mappings": {"default": {"default": "eingekreistes, gegen den Uhrzeigersinn gedrehtes Divisionszeichen"}}
    }, {
      "category": "Sm",
      "key": "29BE",
      "mappings": {"default": {"default": "eingekreiste weiße Kugel"}}
    }, {"category": "Sm", "key": "29BF", "mappings": {"default": {"default": "eingekreiste Kugel"}}}, {
      "category": "Sm",
      "key": "29C0",
      "mappings": {"default": {"default": "eingekreist weniger als"}}
    }, {
      "category": "Sm",
      "key": "29C1",
      "mappings": {"default": {"default": "eingekreist größer als"}}
    }, {
      "category": "Sm",
      "key": "29C2",
      "mappings": {"default": {"default": "Kreis mit kleinem Kreis nach rechts"}}
    }, {
      "category": "Sm",
      "key": "29C3",
      "mappings": {"default": {"default": "Kreis mit zwei horizontalen Strichen nach rechts"}}
    }, {
      "category": "Sm",
      "key": "29C4",
      "mappings": {"default": {"default": "Quadrat mit Diagonale links unten nach rechts oben"}}
    }, {
      "category": "Sm",
      "key": "29C5",
      "mappings": {"default": {"default": "Quadrat mit Diagonale links oben nach rechts unten"}}
    }, {"category": "Sm", "key": "29C6", "mappings": {"default": {"default": "Quadrat Sternchen"}}}, {
      "category": "Sm",
      "key": "29C7",
      "mappings": {"default": {"default": "Quadratischer kleiner Kreis"}}
    }, {"category": "Sm", "key": "29C8", "mappings": {"default": {"default": "Doppelquadrat"}}}, {
      "category": "Sm",
      "key": "29C9",
      "mappings": {"default": {"default": "Zwei verbundene Quadrate"}}
    }, {
      "category": "Sm",
      "key": "29CA",
      "mappings": {"default": {"default": "Dreieck mit Punkt oben"}}
    }, {
      "category": "Sm",
      "key": "29CB",
      "mappings": {"default": {"default": "Dreieck mit Unterleiste"}}
    }, {"category": "Sm", "key": "29CC", "mappings": {"default": {"default": "S im Dreieck"}}}, {
      "category": "Sm",
      "key": "29CD",
      "mappings": {"default": {"default": "Dreieck mit Serifen unten"}}
    }, {
      "category": "Sm",
      "key": "29CE",
      "mappings": {"default": {"default": "Rechtes Dreieck über linkem Dreieck"}}
    }, {
      "category": "Sm",
      "key": "29CF",
      "mappings": {"default": {"default": "Linkes Dreieck neben vertikaler Leiste"}}
    }, {
      "category": "Sm",
      "key": "29D0",
      "mappings": {"default": {"default": "Vertikaler Balken neben dem rechten Dreieck"}}
    }, {
      "category": "Sm",
      "key": "29D1",
      "mappings": {"default": {"default": "Fliege mit linker Hälfte schwarz"}}
    }, {
      "category": "Sm",
      "key": "29D2",
      "mappings": {"default": {"default": "Fliege mit rechter Hälfte schwarz"}}
    }, {"category": "Sm", "key": "29D3", "mappings": {"default": {"default": "Schwarze Fliege"}}}, {
      "category": "Sm",
      "key": "29D4",
      "mappings": {"default": {"default": "Zeiten mit linker Hälfte schwarz"}}
    }, {
      "category": "Sm",
      "key": "29D5",
      "mappings": {"default": {"default": "Mal mit rechter Hälfte Schwarz"}}
    }, {"category": "Sm", "key": "29D6", "mappings": {"default": {"default": "Weiße Sanduhr"}}}, {
      "category": "Sm",
      "key": "29D7",
      "mappings": {"default": {"default": "Schwarze Sanduhr"}}
    }, {
      "category": "Sm",
      "key": "29DC",
      "mappings": {"default": {"default": "Unvollständige Unendlichkeit"}}
    }, {
      "category": "Sm",
      "key": "29DD",
      "mappings": {"default": {"default": "Krawatte über Unendlichkeit"}}
    }, {
      "category": "Sm",
      "key": "29DE",
      "mappings": {"default": {"default": "Unendlichkeit mit vertikaler Leiste negiert"}}
    }, {"category": "Sm", "key": "29DF", "mappings": {"default": {"default": "Doppelter Multimap"}}}, {
      "category": "Sm",
      "key": "29E0",
      "mappings": {"default": {"default": "Quadrat mit konturierter Kontur"}}
    }, {"category": "Sm", "key": "29E1", "mappings": {"default": {"default": "Erhöht als"}}}, {
      "category": "Sm",
      "key": "29E2",
      "mappings": {"default": {"default": "Produkt mischen"}}
    }, {
      "category": "Sm",
      "key": "29E3",
      "mappings": {"default": {"default": "Gleichheitszeichen und geneigte Parallele"}}
    }, {
      "category": "Sm",
      "key": "29E4",
      "mappings": {"default": {"default": "Gleichheitszeichen und schräge Parallele zu Tilde oben"}}
    }, {
      "category": "Sm",
      "key": "29E5",
      "mappings": {"default": {"default": "Identisch und schräg parallel"}}
    }, {"category": "Sm", "key": "29E6", "mappings": {"default": {"default": "Gleich Stark"}}}, {
      "category": "Sm",
      "key": "29E7",
      "mappings": {"default": {"default": "Thermodynamisch"}}
    }, {
      "category": "Sm",
      "key": "29E8",
      "mappings": {"default": {"default": "Abwärtszeigendes Dreieck mit linker Hälfte schwarz"}}
    }, {
      "category": "Sm",
      "key": "29E9",
      "mappings": {"default": {"default": "Abwärtsdreieck mit rechter Hälfte schwarz"}}
    }, {"category": "Sm", "key": "29EB", "mappings": {"default": {"default": "Schwarze Raute"}}}, {
      "category": "Sm",
      "key": "29EE",
      "mappings": {"default": {"default": "Fehlergesperrtes weißes Quadrat"}}
    }, {
      "category": "Sm",
      "key": "29EF",
      "mappings": {"default": {"default": "Fehlerfreies schwarzes Quadrat"}}
    }, {
      "category": "Sm",
      "key": "29F0",
      "mappings": {"default": {"default": "Fehlergesperrter weißer Diamant"}}
    }, {
      "category": "Sm",
      "key": "29F1",
      "mappings": {"default": {"default": "Error-Barred Black Diamond"}}
    }, {
      "category": "Sm",
      "key": "29F2",
      "mappings": {"default": {"default": "Fehlergesperrter weißer Kreis"}}
    }, {
      "category": "Sm",
      "key": "29F3",
      "mappings": {"default": {"default": "Fehlergesperrter schwarzer Kreis"}}
    }, {"key": "29F4", "mappings": {"default": {"default": "rule delayed"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "29F5",
      "mappings": {"default": {"default": "Umgekehrter Schrägstrich Operator"}}
    }, {
      "category": "Sm",
      "key": "29F6",
      "mappings": {"default": {"default": "Schrägstrich mit Overbar"}}
    }, {
      "category": "Sm",
      "key": "29F7",
      "mappings": {"default": {"default": "Umgekehrter Schrägstrich mit horizontalem Hub"}}
    }, {
      "category": "Sm",
      "key": "29F8",
      "mappings": {"default": {"default": "Großer Schrägstrich"}}
    }, {
      "category": "Sm",
      "key": "29F9",
      "mappings": {"default": {"default": "Großer umgekehrter Schrägstrich"}}
    }, {"category": "Sm", "key": "29FA", "mappings": {"default": {"default": "Doppel plus"}}}, {
      "category": "Sm",
      "key": "29FB",
      "mappings": {"default": {"default": "Triple Plus"}}
    }, {"category": "Sm", "key": "29FE", "mappings": {"default": {"default": "Winzig"}}}, {
      "category": "Sm",
      "key": "29FF",
      "mappings": {"default": {"default": "Miny"}}
    }, {
      "category": "Sm",
      "key": "2A00",
      "mappings": {"default": {"default": "N-Ary Circled-Punkt-Operator"}}
    }, {
      "category": "Sm",
      "key": "2A01",
      "mappings": {"default": {"default": "N-Ary Circled Plus Operator"}}
    }, {
      "category": "Sm",
      "key": "2A02",
      "mappings": {"default": {"default": "N-Ary Circled-Times-Operator"}}
    }, {
      "category": "Sm",
      "key": "2A03",
      "mappings": {"default": {"default": "N-Ary Union-Operator mit Punkt"}}
    }, {
      "category": "Sm",
      "key": "2A04",
      "mappings": {"default": {"default": "N-Ary Union-Operator mit Plus"}}
    }, {
      "category": "Sm",
      "key": "2A05",
      "mappings": {"default": {"default": "N-Ary Square-Kreuzungsoperator"}}
    }, {
      "category": "Sm",
      "key": "2A06",
      "mappings": {"default": {"default": "N-Ary Square Union-Betreiber"}}
    }, {
      "category": "Sm",
      "key": "2A07",
      "mappings": {"default": {"default": "Zwei logische und Operator"}}
    }, {
      "category": "Sm",
      "key": "2A08",
      "mappings": {"default": {"default": "Zwei logisch oder Operator"}}
    }, {
      "category": "Sm",
      "key": "2A09",
      "mappings": {"default": {"default": "N-Ary Times-Operator"}}
    }, {"category": "Sm", "key": "2A0A", "mappings": {"default": {"default": "Modulo Zwei Summe"}}}, {
      "category": "Sm",
      "key": "2A0B",
      "mappings": {"default": {"default": "Summation mit Integral"}}
    }, {
      "category": "Sm",
      "key": "2A0C",
      "mappings": {"default": {"default": "Vierfach-Integral-Operator"}}
    }, {
      "category": "Sm",
      "key": "2A0D",
      "mappings": {"default": {"default": "Finite Part Integral"}}
    }, {
      "category": "Sm",
      "key": "2A0E",
      "mappings": {"default": {"default": "Integral mit Doppelhub"}}
    }, {
      "category": "Sm",
      "key": "2A0F",
      "mappings": {"default": {"default": "Integraler Durchschnitt mit Schrägstrich"}}
    }, {
      "category": "Sm",
      "key": "2A10",
      "mappings": {"default": {"default": "Zirkulationsfunktion"}}
    }, {
      "category": "Sm",
      "key": "2A11",
      "mappings": {"default": {"default": "Integration gegen den Uhrzeigersinn"}}
    }, {
      "category": "Sm",
      "key": "2A12",
      "mappings": {"default": {"default": "Linienintegration mit rechteckigem Pfad um die Pole"}}
    }, {
      "category": "Sm",
      "key": "2A13",
      "mappings": {"default": {"default": "Linienintegration mit halbkreisförmigem Pfad um den Pol"}}
    }, {
      "category": "Sm",
      "key": "2A14",
      "mappings": {"default": {"default": "Leitungsintegration ohne Pole"}}
    }, {
      "category": "Sm",
      "key": "2A15",
      "mappings": {"default": {"default": "Integral um einen Punktoperator"}}
    }, {
      "category": "Sm",
      "key": "2A16",
      "mappings": {"default": {"default": "Quaternion Integral Operator"}}
    }, {
      "category": "Sm",
      "key": "2A18",
      "mappings": {"default": {"default": "Integriert mit Times Sign"}}
    }, {
      "category": "Sm",
      "key": "2A19",
      "mappings": {"default": {"default": "Integral mit der Kreuzung"}}
    }, {"category": "Sm", "key": "2A1A", "mappings": {"default": {"default": "Integral mit Union"}}}, {
      "category": "Sm",
      "key": "2A1B",
      "mappings": {"default": {"default": "Integral mit Overbar"}}
    }, {
      "category": "Sm",
      "key": "2A1C",
      "mappings": {"default": {"default": "Integral mit Underbar"}}
    }, {"category": "Sm", "key": "2A1D", "mappings": {"default": {"default": "Beitreten"}}}, {
      "category": "Sm",
      "key": "2A1E",
      "mappings": {"default": {"default": "Großes linkes Dreieckbediener"}}
    }, {
      "category": "Sm",
      "key": "2A1F",
      "mappings": {"default": {"default": "Z-Notationsschema-Zusammensetzung"}}
    }, {
      "category": "Sm",
      "key": "2A20",
      "mappings": {"default": {"default": "Z-Notationsschema-Piping"}}
    }, {
      "category": "Sm",
      "key": "2A21",
      "mappings": {"default": {"default": "Z-Notationsschema-Projektion"}}
    }, {
      "category": "Sm",
      "key": "2A22",
      "mappings": {"default": {"default": "Pluszeichen mit kleinem Kreis oben"}}
    }, {
      "category": "Sm",
      "key": "2A23",
      "mappings": {"default": {"default": "Pluszeichen mit Zirkumflex-Akzent oben"}}
    }, {
      "category": "Sm",
      "key": "2A24",
      "mappings": {"default": {"default": "Pluszeichen mit Tilde oben"}}
    }, {
      "category": "Sm",
      "key": "2A25",
      "mappings": {"default": {"default": "Pluszeichen mit Punkt unten"}}
    }, {
      "category": "Sm",
      "key": "2A26",
      "mappings": {"default": {"default": "Pluszeichen mit Tilde unter"}}
    }, {
      "category": "Sm",
      "key": "2A27",
      "mappings": {"default": {"default": "Pluszeichen mit Index Zwei"}}
    }, {
      "category": "Sm",
      "key": "2A28",
      "mappings": {"default": {"default": "Pluszeichen mit schwarzem Dreieck"}}
    }, {
      "category": "Sm",
      "key": "2A29",
      "mappings": {"default": {"default": "Minuszeichen mit Komma oben"}}
    }, {
      "category": "Sm",
      "key": "2A2A",
      "mappings": {"default": {"default": "Minuszeichen mit Punkt unten"}}
    }, {
      "category": "Sm",
      "key": "2A2B",
      "mappings": {"default": {"default": "Minuszeichen mit fallenden Punkten"}}
    }, {
      "category": "Sm",
      "key": "2A2C",
      "mappings": {"default": {"default": "Minuszeichen mit steigenden Punkten"}}
    }, {
      "category": "Sm",
      "key": "2A2D",
      "mappings": {"default": {"default": "Pluszeichen Im linken Halbkreis"}}
    }, {
      "category": "Sm",
      "key": "2A2E",
      "mappings": {"default": {"default": "Pluszeichen Im rechten Halbkreis"}}
    }, {
      "category": "Sm",
      "key": "2A2F",
      "mappings": {"default": {"default": "Vektor- oder Kreuzprodukt"}}
    }, {
      "category": "Sm",
      "key": "2A30",
      "mappings": {"default": {"default": "Multiplikationszeichen mit Punkt oben"}}
    }, {
      "category": "Sm",
      "key": "2A31",
      "mappings": {"default": {"default": "Multiplikationszeichen mit Unterleiste"}}
    }, {
      "category": "Sm",
      "key": "2A32",
      "mappings": {"default": {"default": "Semidirektprodukt mit geschlossenem Boden"}}
    }, {"key": "2A33", "mappings": {"default": {"default": "smash product"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "2A34",
      "mappings": {"default": {"default": "Multiplikationszeichen im linken Halbkreis"}}
    }, {
      "category": "Sm",
      "key": "2A35",
      "mappings": {"default": {"default": "Multiplikationszeichen im rechten Halbkreis"}}
    }, {
      "category": "Sm",
      "key": "2A36",
      "mappings": {"default": {"default": "eingekreistes Multiplikationszeichen mit Circumflex-Akzent"}}
    }, {
      "category": "Sm",
      "key": "2A37",
      "mappings": {"default": {"default": "Multiplikationszeichen im doppelten Kreis"}}
    }, {
      "key": "2A38",
      "mappings": {"default": {"default": "circled division sign"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A39",
      "mappings": {"default": {"default": "Pluszeichen im Dreieck"}}
    }, {
      "category": "Sm",
      "key": "2A3A",
      "mappings": {"default": {"default": "Minuszeichen im Dreieck"}}
    }, {
      "category": "Sm",
      "key": "2A3B",
      "mappings": {"default": {"default": "Multiplikationszeichen im Dreieck"}}
    }, {"category": "Sm", "key": "2A3C", "mappings": {"default": {"default": "Innenprodukt"}}}, {
      "category": "Sm",
      "key": "2A3D",
      "mappings": {"default": {"default": "Produkt für den rechten Innenraum"}}
    }, {
      "category": "Sm",
      "key": "2A3E",
      "mappings": {"default": {"default": "Z-Notation relationaler Aufbau"}}
    }, {
      "category": "Sm",
      "key": "2A3F",
      "mappings": {"default": {"default": "Verschmelzung oder Koprodukt"}}
    }, {
      "category": "Sm",
      "key": "2A40",
      "mappings": {"default": {"default": "Schnittpunkt mit Punkt"}}
    }, {
      "category": "Sm",
      "key": "2A41",
      "mappings": {"default": {"default": "Union mit Minuszeichen"}}
    }, {"category": "Sm", "key": "2A42", "mappings": {"default": {"default": "Union mit Overbar"}}}, {
      "category": "Sm",
      "key": "2A43",
      "mappings": {"default": {"default": "Kreuzung mit Overbar"}}
    }, {
      "category": "Sm",
      "key": "2A44",
      "mappings": {"default": {"default": "Überschneidung mit logischem und"}}
    }, {
      "category": "Sm",
      "key": "2A45",
      "mappings": {"default": {"default": "Vereinigung mit logischem Or"}}
    }, {
      "category": "Sm",
      "key": "2A46",
      "mappings": {"default": {"default": "Vereinigung über dem Schnittpunkt"}}
    }, {
      "category": "Sm",
      "key": "2A47",
      "mappings": {"default": {"default": "Schnittpunkt über Union"}}
    }, {
      "category": "Sm",
      "key": "2A48",
      "mappings": {"default": {"default": "Vereinigung über Stab über Kreuzung"}}
    }, {
      "category": "Sm",
      "key": "2A49",
      "mappings": {"default": {"default": "Schnittpunkt oberhalb der Stange oberhalb der Union"}}
    }, {
      "category": "Sm",
      "key": "2A4A",
      "mappings": {"default": {"default": "Union daneben und mit Union verbunden"}}
    }, {
      "category": "Sm",
      "key": "2A4B",
      "mappings": {"default": {"default": "Kreuzung neben und mit Kreuzung verbunden"}}
    }, {
      "category": "Sm",
      "key": "2A4C",
      "mappings": {"default": {"default": "Geschlossene Vereinigung mit Serifen"}}
    }, {
      "category": "Sm",
      "key": "2A4D",
      "mappings": {"default": {"default": "Geschlossener Schnittpunkt mit Serifen"}}
    }, {
      "category": "Sm",
      "key": "2A4E",
      "mappings": {"default": {"default": "Doppelter Quadratschnitt"}}
    }, {
      "category": "Sm",
      "key": "2A4F",
      "mappings": {"default": {"default": "Double Square Union"}}
    }, {
      "category": "Sm",
      "key": "2A50",
      "mappings": {"default": {"default": "Closed Union mit Serifen und Smash-Produkten"}}
    }, {
      "category": "Sm",
      "key": "2A51",
      "mappings": {"default": {"default": "Logisch und mit Punkt oben"}}
    }, {
      "category": "Sm",
      "key": "2A52",
      "mappings": {"default": {"default": "Logisch oder mit Punkt oben"}}
    }, {"key": "2A53", "mappings": {"default": {"default": "double logical and"}}, "category": "Sm"}, {
      "key": "2A54",
      "mappings": {"default": {"default": "double logical or"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A55",
      "mappings": {"default": {"default": "Zwei durchschneiden logisch und"}}
    }, {
      "category": "Sm",
      "key": "2A56",
      "mappings": {"default": {"default": "Zwei sich überschneidende logische Oder"}}
    }, {
      "category": "Sm",
      "key": "2A57",
      "mappings": {"default": {"default": "Abfallend groß oder"}}
    }, {"category": "Sm", "key": "2A58", "mappings": {"default": {"default": "Abfallend groß und"}}}, {
      "category": "Sm",
      "key": "2A59",
      "mappings": {"default": {"default": "Logisch oder Überlappend Logisch Und"}}
    }, {
      "category": "Sm",
      "key": "2A5A",
      "mappings": {"default": {"default": "Logisch und mit Middle Stem"}}
    }, {
      "category": "Sm",
      "key": "2A5B",
      "mappings": {"default": {"default": "Logisch oder mit Middle Stem"}}
    }, {
      "category": "Sm",
      "key": "2A5C",
      "mappings": {"default": {"default": "Logisch und mit Horizontalstrich"}}
    }, {
      "category": "Sm",
      "key": "2A5D",
      "mappings": {"default": {"default": "Logisch oder mit Horizontalstrich"}}
    }, {
      "category": "Sm",
      "key": "2A5E",
      "mappings": {"default": {"default": "Logisch und mit Double Overbar"}}
    }, {
      "category": "Sm",
      "key": "2A5F",
      "mappings": {"default": {"default": "Logisch und mit Underbar"}}
    }, {
      "category": "Sm",
      "key": "2A60",
      "mappings": {"default": {"default": "Logisch und mit Double Underbar"}}
    }, {
      "category": "Sm",
      "key": "2A61",
      "mappings": {"default": {"default": "kleines Vee mit Underbar"}}
    }, {
      "category": "Sm",
      "key": "2A62",
      "mappings": {"default": {"default": "Logisch oder mit Double Overbar"}}
    }, {
      "category": "Sm",
      "key": "2A63",
      "mappings": {"default": {"default": "Logisch oder mit doppelter Unterleiste"}}
    }, {
      "category": "Sm",
      "key": "2A64",
      "mappings": {"default": {"default": "Z-Notation Domain Antirestriction"}}
    }, {
      "category": "Sm",
      "key": "2A65",
      "mappings": {"default": {"default": "Z Notationsbereich Antirestriction"}}
    }, {
      "category": "Sm",
      "key": "2A66",
      "mappings": {"default": {"default": "Gleichheitszeichen mit Punkt unten"}}
    }, {
      "category": "Sm",
      "key": "2A67",
      "mappings": {"default": {"default": "Identisch mit dem obigen Punkt"}}
    }, {
      "category": "Sm",
      "key": "2A68",
      "mappings": {"default": {"default": "Dreifacher Horizontalbalken mit doppeltem vertikalem Hub"}}
    }, {
      "category": "Sm",
      "key": "2A69",
      "mappings": {"default": {"default": "Dreifacher Horizontalbalken mit dreifachem vertikalem Hub"}}
    }, {
      "category": "Sm",
      "key": "2A6A",
      "mappings": {"default": {"default": "Tilde-Operator mit Punkt oben"}}
    }, {
      "category": "Sm",
      "key": "2A6B",
      "mappings": {"default": {"default": "Tilde-Operator mit steigenden Punkten"}}
    }, {
      "category": "Sm",
      "key": "2A6C",
      "mappings": {"default": {"default": "Ähnlich Minus Ähnlich"}}
    }, {
      "category": "Sm",
      "key": "2A6D",
      "mappings": {"default": {"default": "Kongruent mit Punkt oben"}}
    }, {
      "category": "Sm",
      "key": "2A6E",
      "mappings": {"default": {"default": "Entspricht Asterisk"}}
    }, {
      "category": "Sm",
      "key": "2A6F",
      "mappings": {"default": {"default": "Fast gleichwertig mit dem Circumflex Accent"}}
    }, {
      "category": "Sm",
      "key": "2A70",
      "mappings": {"default": {"default": "Ungefähr gleich oder gleich zu"}}
    }, {
      "category": "Sm",
      "key": "2A71",
      "mappings": {"default": {"default": "Gleichheitszeichen über Pluszeichen"}}
    }, {
      "category": "Sm",
      "key": "2A72",
      "mappings": {"default": {"default": "Pluszeichen über Gleichheitszeichen"}}
    }, {
      "category": "Sm",
      "key": "2A73",
      "mappings": {"default": {"default": "Gleichheitszeichen über Tilde-Operator"}}
    }, {"category": "Sm", "key": "2A74", "mappings": {"default": {"default": "Doppelpunkt gleich"}}}, {
      "key": "2A75",
      "mappings": {"default": {"default": "double equal"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A76",
      "mappings": {"default": {"default": "Drei aufeinander folgende Gleichheitszeichen"}}
    }, {
      "category": "Sm",
      "key": "2A77",
      "mappings": {"default": {"default": "Gleichheitszeichen mit zwei Punkten oben und zwei Punkten unten"}}
    }, {
      "category": "Sm",
      "key": "2A78",
      "mappings": {"default": {"default": "Gleichwertig mit vier Punkten oben"}}
    }, {
      "category": "Sm",
      "key": "2A79",
      "mappings": {"default": {"default": "Weniger als mit Circle Inside"}}
    }, {
      "category": "Sm",
      "key": "2A7A",
      "mappings": {"default": {"default": "Größer als mit Kreis nach innen"}}
    }, {
      "category": "Sm",
      "key": "2A7B",
      "mappings": {"default": {"default": "Weniger als mit Fragezeichen oben"}}
    }, {
      "category": "Sm",
      "key": "2A7C",
      "mappings": {"default": {"default": "Größer als mit Fragezeichen oben"}}
    }, {
      "category": "Sm",
      "key": "2A7D",
      "mappings": {"default": {"default": "Weniger als oder schräg gleich"}}
    }, {
      "category": "Sm",
      "key": "2A7E",
      "mappings": {"default": {"default": "Größer als oder schräg gleich"}}
    }, {
      "category": "Sm",
      "key": "2A7F",
      "mappings": {"default": {"default": "Weniger als oder schräg gleich mit Punkt nach innen"}}
    }, {
      "category": "Sm",
      "key": "2A80",
      "mappings": {"default": {"default": "Größer als oder schräg gleich mit Punkt nach innen"}}
    }, {
      "category": "Sm",
      "key": "2A81",
      "mappings": {"default": {"default": "Weniger als oder schräg gleich mit Punkt oben"}}
    }, {
      "category": "Sm",
      "key": "2A82",
      "mappings": {"default": {"default": "Größer als oder schräg gleich mit Punkt oben"}}
    }, {
      "category": "Sm",
      "key": "2A83",
      "mappings": {"default": {"default": "Weniger als oder schräg gleich mit Punkt oben rechts"}}
    }, {
      "category": "Sm",
      "key": "2A84",
      "mappings": {"default": {"default": "Größer als oder schräg gleich mit Punkt oben links"}}
    }, {
      "category": "Sm",
      "key": "2A85",
      "mappings": {"default": {"default": "Weniger als oder ungefähr"}}
    }, {
      "category": "Sm",
      "key": "2A86",
      "mappings": {"default": {"default": "Größer als oder ungefähr"}}
    }, {
      "category": "Sm",
      "key": "2A87",
      "mappings": {"default": {"default": "Kleiner als aber nicht gleich"}}
    }, {
      "category": "Sm",
      "key": "2A88",
      "mappings": {"default": {"default": "Größer als aber nicht gleich"}}
    }, {
      "key": "2A89",
      "mappings": {"default": {"default": "Kleiner als aber nicht ungefähr gleich"}},
      "category": "Sm"
    }, {
      "key": "2A8A",
      "mappings": {"default": {"default": "Größer als aber nicht ungefähr gleich"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A8B",
      "mappings": {"default": {"default": "Weniger als aber größer als oben"}}
    }, {
      "category": "Sm",
      "key": "2A8C",
      "mappings": {"default": {"default": "Größer als aber weniger als unten und größer als oben"}}
    }, {
      "category": "Sm",
      "key": "2A8D",
      "mappings": {"default": {"default": "Weniger als oben ähnlich oder gleich"}}
    }, {
      "category": "Sm",
      "key": "2A8E",
      "mappings": {"default": {"default": "Größer als oben ähnlich oder gleich"}}
    }, {
      "category": "Sm",
      "key": "2A8F",
      "mappings": {"default": {"default": "Weniger als oben ähnlich oben größer als"}}
    }, {
      "category": "Sm",
      "key": "2A90",
      "mappings": {"default": {"default": "Größer als oben Ähnlich oben"}}
    }, {
      "key": "2A91",
      "mappings": {"default": {"default": "less than greater than or equal to"}},
      "category": "Sm"
    }, {
      "key": "2A92",
      "mappings": {"default": {"default": "greater than less than or equal to"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A93",
      "mappings": {"default": {"default": "Weniger als oben schräg gleich oben Größer als oben schräg gleich"}}
    }, {
      "category": "Sm",
      "key": "2A94",
      "mappings": {"default": {"default": "Größer als oben schräg gleich oben kleiner als oben schräg gleich groß"}}
    }, {
      "category": "Sm",
      "key": "2A95",
      "mappings": {"default": {"default": "Schräg gleich oder kleiner als"}}
    }, {
      "category": "Sm",
      "key": "2A96",
      "mappings": {"default": {"default": "Schräg gleich oder größer als"}}
    }, {
      "category": "Sm",
      "key": "2A97",
      "mappings": {"default": {"default": "Schräg gleich oder kleiner als mit innenliegendem Punkt"}}
    }, {
      "category": "Sm",
      "key": "2A98",
      "mappings": {"default": {"default": "Schräg gleich oder größer als mit Punkt nach innen"}}
    }, {
      "key": "2A99",
      "mappings": {"default": {"default": "Gleich oder kleiner als"}},
      "category": "Sm"
    }, {
      "key": "2A9A",
      "mappings": {"default": {"default": "Gleich oder größer als"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A9B",
      "mappings": {"default": {"default": "Doppelte Linie schräg gleich oder kleiner als"}}
    }, {
      "category": "Sm",
      "key": "2A9C",
      "mappings": {"default": {"default": "Doppelte Linie schräg gleich oder größer als"}}
    }, {
      "key": "2A9D",
      "mappings": {"default": {"default": "Äquivalent oder kleiner als"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A9E",
      "mappings": {"default": {"default": "Ähnlich oder größer als"}}
    }, {
      "category": "Sm",
      "key": "2A9F",
      "mappings": {"default": {"default": "Ähnlich oben Weniger als Gleiches Gleichheitszeichen"}}
    }, {
      "category": "Sm",
      "key": "2AA0",
      "mappings": {"default": {"default": "Ähnlich oben größer als oben Gleichheitszeichen"}}
    }, {
      "key": "2AA1",
      "mappings": {"default": {"default": "Verschachteltes kleiner als"}},
      "category": "Sm"
    }, {
      "key": "2AA2",
      "mappings": {"default": {"default": "Verschachteltes größer als"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AA3",
      "mappings": {"default": {"default": "Doppelt verschachtelt, weniger als mit Unterleiste"}}
    }, {
      "key": "2AA4",
      "mappings": {"default": {"default": "less than greater than overlay"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AA5",
      "mappings": {"default": {"default": "Größer als weniger als"}}
    }, {
      "category": "Sm",
      "key": "2AA6",
      "mappings": {"default": {"default": "Weniger als durch Kurve geschlossen"}}
    }, {
      "category": "Sm",
      "key": "2AA7",
      "mappings": {"default": {"default": "Größer als durch Kurve geschlossen"}}
    }, {
      "category": "Sm",
      "key": "2AA8",
      "mappings": {"default": {"default": "Weniger als dann geschlossen durch Kurve oberhalb der gleichen Neigung"}}
    }, {
      "category": "Sm",
      "key": "2AA9",
      "mappings": {"default": {"default": "Größer als geschlossen durch Kurve oberhalb der gleichen Neigung"}}
    }, {"category": "Sm", "key": "2AAA", "mappings": {"default": {"default": "kleiner als"}}}, {
      "category": "Sm",
      "key": "2AAB",
      "mappings": {"default": {"default": "Größer als"}}
    }, {
      "category": "Sm",
      "key": "2AAC",
      "mappings": {"default": {"default": "kleiner als oder gleich"}}
    }, {
      "category": "Sm",
      "key": "2AAD",
      "mappings": {"default": {"default": "Größer als oder gleich"}}
    }, {
      "key": "2AAE",
      "mappings": {"default": {"default": "difference between (variant"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AAF",
      "mappings": {"default": {"default": "Stellt über dem einzeiligen Gleichheitszeichen ein"}}
    }, {
      "category": "Sm",
      "key": "2AB0",
      "mappings": {"default": {"default": "Erfolg über dem einzeiligen Gleichheitszeichen"}}
    }, {
      "category": "Sm",
      "key": "2AB1",
      "mappings": {"default": {"default": "Vorangehende Single-Line ist nicht gleich"}}
    }, {
      "category": "Sm",
      "key": "2AB2",
      "mappings": {"default": {"default": "Erfolg über Single-Line nicht gleich"}}
    }, {
      "category": "Sm",
      "key": "2AB3",
      "mappings": {"default": {"default": "Vorangegangenes Gleichheitszeichen"}}
    }, {
      "category": "Sm",
      "key": "2AB4",
      "mappings": {"default": {"default": "Erfolgreich über Gleichheitszeichen"}}
    }, {
      "key": "2AB5",
      "mappings": {"default": {"default": "precedes but not equal to"}},
      "category": "Sm"
    }, {
      "key": "2AB6",
      "mappings": {"default": {"default": "succeeds but not equal to"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AB7",
      "mappings": {"default": {"default": "Vorangegangenes fast gleich zu"}}
    }, {
      "category": "Sm",
      "key": "2AB8",
      "mappings": {"default": {"default": "Erreicht über fast gleich viel"}}
    }, {
      "category": "Sm",
      "key": "2AB9",
      "mappings": {"default": {"default": "Vorangegangene nicht annähernd gleich"}}
    }, {
      "category": "Sm",
      "key": "2ABA",
      "mappings": {"default": {"default": "Erreicht oben nicht annähernd gleich"}}
    }, {
      "category": "Sm",
      "key": "2ABB",
      "mappings": {"default": {"default": "Doppelter Vorgänger"}}
    }, {"category": "Sm", "key": "2ABC", "mappings": {"default": {"default": "Doppelter Erfolg"}}}, {
      "category": "Sm",
      "key": "2ABD",
      "mappings": {"default": {"default": "Teilmenge mit Punkt"}}
    }, {"category": "Sm", "key": "2ABE", "mappings": {"default": {"default": "Superset mit Punkt"}}}, {
      "category": "Sm",
      "key": "2ABF",
      "mappings": {"default": {"default": "Teilmenge mit Pluszeichen unten"}}
    }, {
      "category": "Sm",
      "key": "2AC0",
      "mappings": {"default": {"default": "Superset mit Pluszeichen unten"}}
    }, {
      "category": "Sm",
      "key": "2AC1",
      "mappings": {"default": {"default": "Teilmenge mit Multiplikationszeichen unten"}}
    }, {
      "category": "Sm",
      "key": "2AC2",
      "mappings": {"default": {"default": "Superset mit Multiplikationszeichen unten"}}
    }, {
      "category": "Sm",
      "key": "2AC3",
      "mappings": {"default": {"default": "Teilmenge oder gleich mit dem obigen Punkt"}}
    }, {
      "category": "Sm",
      "key": "2AC4",
      "mappings": {"default": {"default": "Superset von oder gleich mit Punkt oben"}}
    }, {
      "category": "Sm",
      "key": "2AC5",
      "mappings": {"default": {"default": "Teilmenge des obigen Gleichheitszeichens"}}
    }, {
      "category": "Sm",
      "key": "2AC6",
      "mappings": {"default": {"default": "Superset des obigen Gleichheitszeichens"}}
    }, {"key": "2AC7", "mappings": {"default": {"default": "approximate subset of"}}, "category": "Sm"}, {
      "key": "2AC8",
      "mappings": {"default": {"default": "approximate superset of"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AC9",
      "mappings": {"default": {"default": "Teilmenge von oben fast gleich"}}
    }, {
      "category": "Sm",
      "key": "2ACA",
      "mappings": {"default": {"default": "Superset von oben fast gleichwertig"}}
    }, {
      "category": "Sm",
      "key": "2ACB",
      "mappings": {"default": {"default": "Teilmenge von oben nicht gleich"}}
    }, {
      "category": "Sm",
      "key": "2ACC",
      "mappings": {"default": {"default": "Superset von oben nicht gleich"}}
    }, {
      "category": "Sm",
      "key": "2ACD",
      "mappings": {"default": {"default": "Platz links Feldbetreiber geöffnet"}}
    }, {
      "category": "Sm",
      "key": "2ACE",
      "mappings": {"default": {"default": "Rechteckige Box mit offener Box"}}
    }, {
      "category": "Sm",
      "key": "2ACF",
      "mappings": {"default": {"default": "Geschlossener Teilsatz"}}
    }, {
      "category": "Sm",
      "key": "2AD0",
      "mappings": {"default": {"default": "Superset geschlossen"}}
    }, {
      "category": "Sm",
      "key": "2AD1",
      "mappings": {"default": {"default": "Geschlossene Untermenge oder gleich"}}
    }, {
      "category": "Sm",
      "key": "2AD2",
      "mappings": {"default": {"default": "Superset geschlossen oder gleich"}}
    }, {"key": "2AD3", "mappings": {"default": {"default": "subset over superset"}}, "category": "Sm"}, {
      "key": "2AD4",
      "mappings": {"default": {"default": "superset over subset"}},
      "category": "Sm"
    }, {"key": "2AD5", "mappings": {"default": {"default": "subset over subset"}}, "category": "Sm"}, {
      "key": "2AD6",
      "mappings": {"default": {"default": "superset over superset"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AD7",
      "mappings": {"default": {"default": "Superset neben dem Subset"}}
    }, {
      "category": "Sm",
      "key": "2AD8",
      "mappings": {"default": {"default": "Superset Neben und Joined von Dash mit Subset"}}
    }, {
      "category": "Sm",
      "key": "2AD9",
      "mappings": {"default": {"default": "Element der Öffnung nach unten"}}
    }, {
      "category": "Sm",
      "key": "2ADA",
      "mappings": {"default": {"default": "Heugabel mit T-Stück"}}
    }, {"category": "Sm", "key": "2ADB", "mappings": {"default": {"default": "Transversalschnitt"}}}, {
      "category": "Sm",
      "key": "2ADC",
      "mappings": {"default": {"default": "Gabelung"}}
    }, {"category": "Sm", "key": "2ADD", "mappings": {"default": {"default": "Nicht Gabeln"}}}, {
      "category": "Sm",
      "key": "2ADE",
      "mappings": {"default": {"default": "Kurzer Linker Tack"}}
    }, {"category": "Sm", "key": "2ADF", "mappings": {"default": {"default": "Kurzer Down Tack"}}}, {
      "category": "Sm",
      "key": "2AE0",
      "mappings": {"default": {"default": "Short Up Tack"}}
    }, {"category": "Sm", "key": "2AE1", "mappings": {"default": {"default": "Rechtwinklig zu S"}}}, {
      "category": "Sm",
      "key": "2AE2",
      "mappings": {"default": {"default": "Vertikales Dreifach-Drehkreuz"}}
    }, {
      "category": "Sm",
      "key": "2AE3",
      "mappings": {"default": {"default": "Doppelter vertikaler Balken linkes Drehkreuz"}}
    }, {
      "key": "2AE4",
      "mappings": {"default": {"default": "double left turnstile vertical bar"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AE5",
      "mappings": {"default": {"default": "Doppelter vertikaler Balken Doppelter linker Drehkreuz"}}
    }, {
      "category": "Sm",
      "key": "2AE6",
      "mappings": {"default": {"default": "Long Dash von Left Member von Double Vertical"}}
    }, {
      "category": "Sm",
      "key": "2AE7",
      "mappings": {"default": {"default": "Short Down Tack mit Overbar"}}
    }, {
      "key": "2AE8",
      "mappings": {"default": {"default": "perpendicular over bar"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AE9",
      "mappings": {"default": {"default": "Short Up Tack Über Short Down Tack"}}
    }, {"category": "Sm", "key": "2AEA", "mappings": {"default": {"default": "Double Down Tack"}}}, {
      "category": "Sm",
      "key": "2AEB",
      "mappings": {"default": {"default": "Verdoppeln Sie Tack"}}
    }, {
      "category": "Sm",
      "key": "2AEC",
      "mappings": {"default": {"default": "Doppelstrich nicht unterschreiben"}}
    }, {
      "category": "Sm",
      "key": "2AED",
      "mappings": {"default": {"default": "Umgekehrter Doppelstrich nicht unterschrieben"}}
    }, {
      "category": "Sm",
      "key": "2AEE",
      "mappings": {"default": {"default": "Teilt sich nicht mit umgekehrten Negationsschrägstrich"}}
    }, {
      "category": "Sm",
      "key": "2AEF",
      "mappings": {"default": {"default": "Vertikale Linie mit Kreis oben"}}
    }, {
      "category": "Sm",
      "key": "2AF0",
      "mappings": {"default": {"default": "Vertikale Linie mit Kreis unten"}}
    }, {
      "category": "Sm",
      "key": "2AF1",
      "mappings": {"default": {"default": "Down Tack mit Kreis unten"}}
    }, {
      "category": "Sm",
      "key": "2AF2",
      "mappings": {"default": {"default": "Parallel zum horizontalen Anschlag"}}
    }, {
      "category": "Sm",
      "key": "2AF3",
      "mappings": {"default": {"default": "Parallel zu Tilde Operator"}}
    }, {
      "category": "Sm",
      "key": "2AF4",
      "mappings": {"default": {"default": "Dreifache vertikale Balken-Binärbeziehung"}}
    }, {
      "category": "Sm",
      "key": "2AF5",
      "mappings": {"default": {"default": "Dreifacher vertikaler Strich mit horizontalem Hub"}}
    }, {
      "category": "Sm",
      "key": "2AF6",
      "mappings": {"default": {"default": "Triple Colon Operator"}}
    }, {
      "category": "Sm",
      "key": "2AF7",
      "mappings": {"default": {"default": "Dreifach verschachtelt weniger als"}}
    }, {
      "category": "Sm",
      "key": "2AF8",
      "mappings": {"default": {"default": "Dreifach verschachteltes Größeres als"}}
    }, {
      "category": "Sm",
      "key": "2AF9",
      "mappings": {"default": {"default": "Doppellinie schräg, weniger als oder gleich"}}
    }, {
      "category": "Sm",
      "key": "2AFA",
      "mappings": {"default": {"default": "Doppelte Linie schräg größer als oder gleich"}}
    }, {
      "category": "Sm",
      "key": "2AFB",
      "mappings": {"default": {"default": "Dreifache Schrägstrich-Binärbeziehung"}}
    }, {
      "category": "Sm",
      "key": "2AFC",
      "mappings": {"default": {"default": "Großer dreifacher vertikaler Barbediener"}}
    }, {
      "category": "Sm",
      "key": "2AFD",
      "mappings": {"default": {"default": "Doppelter Schrägstrich-Operator"}}
    }, {
      "category": "Sm",
      "key": "2AFE",
      "mappings": {"default": {"default": "Weiße vertikale Leiste"}}
    }, {
      "category": "Sm",
      "key": "2AFF",
      "mappings": {"default": {"default": "N-Ary White Vertical Bar"}}
    }, {"category": "Pd", "key": "301C", "mappings": {"default": {"default": "Wave Dash"}}}, {
      "category": "Po",
      "key": "FE10",
      "mappings": {"default": {"default": "Präsentationsformular für vertikales Komma"}}
    }, {
      "category": "Po",
      "key": "FE13",
      "mappings": {"default": {"default": "Präsentationsformular für vertikalen Doppelpunkt"}}
    }, {
      "category": "Po",
      "key": "FE14",
      "mappings": {"default": {"default": "Präsentationsformular für vertikales Semikolon"}}
    }, {
      "category": "Po",
      "key": "FE15",
      "mappings": {"default": {"default": "Präsentationsformular für vertikales Ausrufezeichen"}}
    }, {
      "category": "Po",
      "key": "FE16",
      "mappings": {"default": {"default": "Präsentationsformular für vertikales Fragezeichen"}}
    }, {
      "category": "Po",
      "key": "FE19",
      "mappings": {"default": {"default": "Präsentationsformular für vertikale horizontale Ellipsen"}}
    }, {
      "category": "Po",
      "key": "FE30",
      "mappings": {"default": {"default": "Präsentationsformular für vertikalen Zwei-Punkt-Leader"}}
    }, {
      "category": "Pd",
      "key": "FE31",
      "mappings": {"default": {"default": "Präsentationsformular für Vertical Em Dash"}}
    }, {
      "category": "Pd",
      "key": "FE32",
      "mappings": {"default": {"default": "Präsentationsformular für Vertical En Dash"}}
    }, {
      "category": "Pc",
      "key": "FE33",
      "mappings": {"default": {"default": "Präsentationsformular für Vertical Low Line"}}
    }, {
      "category": "Pc",
      "key": "FE34",
      "mappings": {"default": {"default": "Präsentationsformular für Vertical Wavy Low Line"}}
    }, {"category": "Po", "key": "FE45", "mappings": {"default": {"default": "Sesampunkt"}}}, {
      "category": "Po",
      "key": "FE46",
      "mappings": {"default": {"default": "Weißer Sesam Punkt"}}
    }, {
      "category": "Po",
      "key": "FE49",
      "mappings": {"default": {"default": "Gestrichelte Überlänge"}}
    }, {
      "category": "Po",
      "key": "FE4A",
      "mappings": {"default": {"default": "Centreline Overline"}}
    }, {
      "category": "Po",
      "key": "FE4B",
      "mappings": {"default": {"default": "Wellenförmige Überlänge"}}
    }, {
      "category": "Po",
      "key": "FE4C",
      "mappings": {"default": {"default": "Doppelte Wellenlinie"}}
    }, {
      "category": "Pc",
      "key": "FE4D",
      "mappings": {"default": {"default": "Gestrichelte Low Line"}}
    }, {
      "category": "Pc",
      "key": "FE4E",
      "mappings": {"default": {"default": "Mittellinie Low Line"}}
    }, {
      "category": "Pc",
      "key": "FE4F",
      "mappings": {"default": {"default": "Wellenförmige Low Line"}}
    }, {"category": "Po", "key": "FE50", "mappings": {"default": {"default": "kleines Komma"}}}, {
      "category": "Po",
      "key": "FE52",
      "mappings": {"default": {"default": "kleiner Punkt"}}
    }, {"category": "Po", "key": "FE54", "mappings": {"default": {"default": "kleines Semikolon"}}}, {
      "category": "Po",
      "key": "FE55",
      "mappings": {"default": {"default": "kleiner Doppelpunkt"}}
    }, {
      "category": "Po",
      "key": "FE56",
      "mappings": {"default": {"default": "kleines Fragezeichen"}}
    }, {
      "category": "Po",
      "key": "FE57",
      "mappings": {"default": {"default": "kleines Ausrufezeichen"}}
    }, {"category": "Pd", "key": "FE58", "mappings": {"default": {"default": "kleiner Em Dash"}}}, {
      "category": "Po",
      "key": "FE5F",
      "mappings": {"default": {"default": "kleines Nummernzeichen"}}
    }, {
      "category": "Po",
      "key": "FE60",
      "mappings": {"default": {"default": "kleines kaufmännisches"}}
    }, {"category": "Po", "key": "FE61", "mappings": {"default": {"default": "kleines Sternchen"}}}, {
      "category": "Sm",
      "key": "FE62",
      "mappings": {"default": {"default": "kleines Pluszeichen"}}
    }, {
      "category": "Pd",
      "key": "FE63",
      "mappings": {"default": {"default": "kleiner Bindestrich-Minus"}}
    }, {
      "category": "Sm",
      "key": "FE64",
      "mappings": {"default": {"default": "kleines weniger als Zeichen"}}
    }, {
      "category": "Sm",
      "key": "FE65",
      "mappings": {"default": {"default": "kleines Größeres als Zeichen"}}
    }, {
      "category": "Sm",
      "key": "FE66",
      "mappings": {"default": {"default": "kleines Gleichheitszeichen"}}
    }, {
      "category": "Po",
      "key": "FE68",
      "mappings": {"default": {"default": "kleiner umgekehrter Schrägstrich"}}
    }, {
      "category": "Sc",
      "key": "FE69",
      "mappings": {"default": {"default": "kleines Dollarzeichen"}}
    }, {
      "category": "Po",
      "key": "FE6A",
      "mappings": {"default": {"default": "kleines Prozentzeichen"}}
    }, {
      "category": "Po",
      "key": "FE6B",
      "mappings": {"default": {"default": "kleine kommerzielle bei"}}
    }, {
      "category": "Po",
      "key": "FF01",
      "mappings": {"default": {"default": "vollbreites Ausrufezeichen"}}
    }, {
      "category": "Po",
      "key": "FF02",
      "mappings": {"default": {"default": "vollbreites Anführungszeichen"}}
    }, {
      "category": "Po",
      "key": "FF03",
      "mappings": {"default": {"default": "vollbreites Nummernzeichen"}}
    }, {
      "category": "Sc",
      "key": "FF04",
      "mappings": {"default": {"default": "vollbreites Dollarzeichen"}}
    }, {
      "category": "Po",
      "key": "FF05",
      "mappings": {"default": {"default": "vollbreites Prozentzeichen"}}
    }, {
      "category": "Po",
      "key": "FF06",
      "mappings": {"default": {"default": "vollbreites kaufmännisches und"}}
    }, {
      "category": "Po",
      "key": "FF07",
      "mappings": {"default": {"default": "vollbreites Apostroph"}}
    }, {"category": "Po", "key": "FF0A", "mappings": {"default": {"default": "vollbreiter Stern"}}}, {
      "category": "Sm",
      "key": "FF0B",
      "mappings": {"default": {"default": "vollbreites Plus"}}
    }, {"category": "Po", "key": "FF0C", "mappings": {"default": {"default": "vollbreites Komma"}}}, {
      "category": "Pd",
      "key": "FF0D",
      "mappings": {"default": {"default": "vollbreites Minus"}}
    }, {"category": "Po", "key": "FF0E", "mappings": {"default": {"default": "vollbreiter Punkt"}}}, {
      "category": "Po",
      "key": "FF0F",
      "mappings": {"default": {"default": "Schrägstrich mit voller Breite"}}
    }, {
      "category": "Po",
      "key": "FF1A",
      "mappings": {"default": {"default": "vollbreiter Doppelpunkt"}}
    }, {
      "category": "Po",
      "key": "FF1B",
      "mappings": {"default": {"default": "vollbreites Semikolon"}}
    }, {
      "category": "Sm",
      "key": "FF1C",
      "mappings": {"default": {"default": "Weniger als das gesamte Zeichen"}}
    }, {
      "category": "Sm",
      "key": "FF1D",
      "mappings": {"default": {"default": "vollbreites Gleichspannungszeichen"}}
    }, {
      "category": "Sm",
      "key": "FF1E",
      "mappings": {"default": {"default": "Fullwidth Mehr als Zeichen"}}
    }, {
      "category": "Po",
      "key": "FF1F",
      "mappings": {"default": {"default": "vollbreites Fragezeichen"}}
    }, {
      "category": "Po",
      "key": "FF20",
      "mappings": {"default": {"default": "Fullwidth Commercial at"}}
    }, {
      "category": "Po",
      "key": "FF3C",
      "mappings": {"default": {"default": "Umgekehrter Schrägstrich mit voller Breite"}}
    }, {
      "category": "Sk",
      "key": "FF3E",
      "mappings": {"default": {"default": "Circumflex-Akzent mit voller Breite"}}
    }, {
      "category": "Pc",
      "key": "FF3F",
      "mappings": {"default": {"default": "vollbreites Low Line"}}
    }, {
      "category": "Sk",
      "key": "FF40",
      "mappings": {"default": {"default": "vollbreiter Gravis Akzent"}}
    }, {
      "category": "Sm",
      "key": "FF5C",
      "mappings": {"default": {"default": "vollbreites Vertikale Linie"}}
    }, {"category": "Sm", "key": "FF5E", "mappings": {"default": {"default": "vollbreite Tilde"}}}, {
      "category": "Sc",
      "key": "FFE0",
      "mappings": {"default": {"default": "vollbreites Cent-Zeichen"}}
    }, {
      "category": "Sc",
      "key": "FFE1",
      "mappings": {"default": {"default": "vollbreites Pfundzeichen"}}
    }, {
      "category": "Sm",
      "key": "FFE2",
      "mappings": {"default": {"default": "vollbreite nicht Symbol"}}
    }, {
      "category": "Sk",
      "key": "FFE3",
      "mappings": {"default": {"default": "vollbreiter Überstrich", "alternative": "vollbreites Makron"}}
    }, {
      "category": "So",
      "key": "FFE4",
      "mappings": {"default": {"default": "vollbreite gestrichelte Linie"}}
    }, {
      "category": "Sc",
      "key": "FFE5",
      "mappings": {"default": {"default": "vollbreites Yen-Zeichen"}}
    }, {
      "category": "Sc",
      "key": "FFE6",
      "mappings": {"default": {"default": "vollbreites Won-Zeichen"}}
    }, {
      "category": "So",
      "key": "FFE8",
      "mappings": {"default": {"default": "Halbbreite bildet leicht vertikal"}}
    }, {
      "category": "So",
      "key": "FFED",
      "mappings": {"default": {"default": "Schwarzes Quadrat mit halber Breite"}}
    }, {"category": "So", "key": "FFEE", "mappings": {"default": {"default": "Weißer Kreis mit halber Breite"}}}]
  ,
  "de/symbols/math_whitespace.js":
    [{"locale": "de"}, {
      "category": "Zs",
      "key": "0020",
      "mappings": {"default": {"default": "Leerzeichen"}}
    }, {"key": "00A0", "mappings": {"default": {"default": " "}}, "category": "Zs"}, {
      "key": "00AD",
      "mappings": {"default": {"default": "weicher Trennstrich"}},
      "category": "Cf"
    }, {"category": "Zs", "key": "2000", "mappings": {"default": {"default": "En Quad"}}}, {
      "category": "Zs",
      "key": "2001",
      "mappings": {"default": {"default": "Em Quad"}}
    }, {"category": "Zs", "key": "2002", "mappings": {"default": {"default": "En Space"}}}, {
      "key": "2003",
      "mappings": {"default": {"default": ""}},
      "category": "Zs"
    }, {
      "category": "Zs",
      "key": "2004",
      "mappings": {"default": {"default": "Raum für drei Per-Em"}}
    }, {
      "category": "Zs",
      "key": "2005",
      "mappings": {"default": {"default": "Raum für vier pro Em"}}
    }, {"category": "Zs", "key": "2006", "mappings": {"default": {"default": "Six-Per-Em Space"}}}, {
      "category": "Zs",
      "key": "2007",
      "mappings": {"default": {"default": "Raum darstellen"}}
    }, {"category": "Zs", "key": "2008", "mappings": {"default": {"default": "Interpunktionsraum"}}}, {
      "key": "2009",
      "mappings": {"default": {"default": ""}},
      "category": "Zs"
    }, {"key": "200A", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
      "key": "200B",
      "mappings": {"default": {"default": ""}},
      "category": "Cf"
    }, {
      "category": "Cf",
      "key": "200C",
      "mappings": {"default": {"default": "Nullbreite ohne Joiner"}}
    }, {"category": "Cf", "key": "200D", "mappings": {"default": {"default": "Zero Width Joiner"}}}, {
      "category": "Cf",
      "key": "200E",
      "mappings": {"default": {"default": "Markierung von links nach rechts"}}
    }, {
      "category": "Cf",
      "key": "200F",
      "mappings": {"default": {"default": "Markierung von rechts nach links"}}
    }, {"category": "Zl", "key": "2028", "mappings": {"default": {"default": "Trennzeichen"}}}, {
      "category": "Zp",
      "key": "2029",
      "mappings": {"default": {"default": "Absatztrennzeichen"}}
    }, {
      "category": "Cf",
      "key": "202A",
      "mappings": {"default": {"default": "Einbetten von links nach rechts"}}
    }, {
      "category": "Cf",
      "key": "202B",
      "mappings": {"default": {"default": "Einbetten von rechts nach links"}}
    }, {
      "category": "Cf",
      "key": "202C",
      "mappings": {"default": {"default": "Pop Richtungsformatierung"}}
    }, {
      "category": "Cf",
      "key": "202D",
      "mappings": {"default": {"default": "Von links nach rechts überschreiben"}}
    }, {
      "category": "Cf",
      "key": "202E",
      "mappings": {"default": {"default": "Von rechts nach links überschreiben"}}
    }, {
      "category": "Zs",
      "key": "202F",
      "mappings": {"default": {"default": "Schmaler No-Break-Bereich"}}
    }, {"key": "205F", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
      "category": "Cf",
      "key": "2060",
      "mappings": {"default": {"default": "Word Joiner"}}
    }, {"key": "2061", "mappings": {"default": {"default": "von"}}, "category": "Cf"}, {
      "key": "2062",
      "mappings": {"default": {"default": " "}},
      "category": "Cf"
    }, {"key": "2063", "mappings": {"default": {"default": ""}}, "category": "Cf"}, {
      "category": "Cf",
      "key": "2064",
      "mappings": {"default": {"default": "Unsichtbares Plus"}}
    }, {
      "category": "Cf",
      "key": "206A",
      "mappings": {"default": {"default": "Symmetrisches Austauschen verhindern"}}
    }, {
      "category": "Cf",
      "key": "206B",
      "mappings": {"default": {"default": "Aktivieren Sie das symmetrische Austauschen"}}
    }, {
      "category": "Cf",
      "key": "206E",
      "mappings": {"default": {"default": "Nationale Ziffernformen"}}
    }, {
      "category": "Cf",
      "key": "206F",
      "mappings": {"default": {"default": "Nominale Ziffernformen"}}
    }, {
      "category": "Cf",
      "key": "FEFF",
      "mappings": {"default": {"default": "Nullbreite Leerzeichen"}}
    }, {
      "category": "Cf",
      "key": "FFF9",
      "mappings": {"default": {"default": "Interlinearer Anmerkungsanker"}}
    }, {
      "category": "Cf",
      "key": "FFFA",
      "mappings": {"default": {"default": "Separator für interlineare Anmerkungen"}}
    }, {"category": "Cf", "key": "FFFB", "mappings": {"default": {"default": "Interlinear Annotation Terminator"}}}]
  ,
  "de/symbols/other_stars.js":
    [{"locale": "de"}, {
      "category": "So",
      "key": "23E8",
      "mappings": {"default": {"default": "Dezimalexponentensymbol"}}
    }, {"category": "So", "key": "2605", "mappings": {"default": {"default": "Schwarzer Stern"}}}, {
      "category": "So",
      "key": "2606",
      "mappings": {"default": {"default": "weißer Stern"}}
    }, {
      "category": "So",
      "key": "26AA",
      "mappings": {"default": {"default": "Mittlerer weißer Kreis"}}
    }, {
      "category": "So",
      "key": "26AB",
      "mappings": {"default": {"default": "Mittlerer schwarzer Kreis"}}
    }, {
      "category": "So",
      "key": "2705",
      "mappings": {"default": {"default": "Weißes schweres Häkchen"}}
    }, {"category": "So", "key": "2713", "mappings": {"default": {"default": "Häkchen"}}}, {
      "category": "So",
      "key": "2714",
      "mappings": {"default": {"default": "Schweres Häkchen"}}
    }, {"category": "So", "key": "2715", "mappings": {"default": {"default": "Multiplikation X"}}}, {
      "category": "So",
      "key": "2716",
      "mappings": {"default": {"default": "Schwere Multiplikation X"}}
    }, {"category": "So", "key": "2717", "mappings": {"default": {"default": "Stimmzettel X"}}}, {
      "category": "So",
      "key": "2718",
      "mappings": {"default": {"default": "Schwerer Stimmzettel X"}}
    }, {
      "category": "So",
      "key": "271B",
      "mappings": {"default": {"default": "Center Cross öffnen"}}
    }, {
      "category": "So",
      "key": "271C",
      "mappings": {"default": {"default": "Schweres offenes Mittelkreuz"}}
    }, {"category": "So", "key": "2720", "mappings": {"default": {"default": "Malteserkreuz"}}}, {
      "category": "So",
      "key": "2721",
      "mappings": {"default": {"default": "Davidstern"}}
    }, {
      "category": "So",
      "key": "2722",
      "mappings": {"default": {"default": "Vier Teardrop-Spoked Asterisk"}}
    }, {
      "category": "So",
      "key": "2723",
      "mappings": {"default": {"default": "Vier Ballon-Speichen-Sternchen"}}
    }, {
      "category": "So",
      "key": "2724",
      "mappings": {"default": {"default": "Schwere Vier-Ballon-Speichen-Sternchen"}}
    }, {
      "category": "So",
      "key": "2725",
      "mappings": {"default": {"default": "Vier Club-Spoked Asterisk"}}
    }, {
      "category": "So",
      "key": "2726",
      "mappings": {"default": {"default": "Schwarzer vierzackiger Stern"}}
    }, {
      "category": "So",
      "key": "2727",
      "mappings": {"default": {"default": "Weißer vierzackiger Stern"}}
    }, {"category": "So", "key": "2728", "mappings": {"default": {"default": "Funkelt"}}}, {
      "category": "So",
      "key": "2729",
      "mappings": {"default": {"default": "Stress umrissener weißer Stern"}}
    }, {
      "category": "So",
      "key": "272A",
      "mappings": {"default": {"default": "Eingekreister weißer Stern"}}
    }, {
      "category": "So",
      "key": "272B",
      "mappings": {"default": {"default": "Open Center Black Star"}}
    }, {
      "category": "So",
      "key": "272C",
      "mappings": {"default": {"default": "Schwarzer Center White Star"}}
    }, {
      "category": "So",
      "key": "272D",
      "mappings": {"default": {"default": "Schwarzer Stern umrissen"}}
    }, {
      "category": "So",
      "key": "272E",
      "mappings": {"default": {"default": "Schwerer umrissener schwarzer Stern"}}
    }, {"category": "So", "key": "272F", "mappings": {"default": {"default": "Sternrad Stern"}}}, {
      "category": "So",
      "key": "2730",
      "mappings": {"default": {"default": "Beschatteter weißer Stern"}}
    }, {"category": "So", "key": "2731", "mappings": {"default": {"default": "Schwerer Stern"}}}, {
      "category": "So",
      "key": "2732",
      "mappings": {"default": {"default": "Öffnen Sie Center Asterisk"}}
    }, {
      "category": "So",
      "key": "2733",
      "mappings": {"default": {"default": "Acht Speichen-Sternchen"}}
    }, {
      "category": "So",
      "key": "2734",
      "mappings": {"default": {"default": "Achtzackiger schwarzer Stern"}}
    }, {
      "category": "So",
      "key": "2735",
      "mappings": {"default": {"default": "Achtzackiger Sternradstern"}}
    }, {
      "category": "So",
      "key": "2736",
      "mappings": {"default": {"default": "Sechszackiger schwarzer Stern"}}
    }, {
      "category": "So",
      "key": "2739",
      "mappings": {"default": {"default": "Zwölf schwarzer Stern"}}
    }, {
      "category": "So",
      "key": "273A",
      "mappings": {"default": {"default": "Sechzehn Spitzen Sternchen"}}
    }, {
      "category": "So",
      "key": "273B",
      "mappings": {"default": {"default": "Teardrop-Spoked Sternchen"}}
    }, {
      "category": "So",
      "key": "273C",
      "mappings": {"default": {"default": "Open Center Teardrop-Spoked Sternchen"}}
    }, {
      "category": "So",
      "key": "273D",
      "mappings": {"default": {"default": "Schwerer Tropfen-Speichen-Sternchen"}}
    }, {
      "category": "So",
      "key": "273E",
      "mappings": {"default": {"default": "Sechs Petalled Schwarzweiss-Florette"}}
    }, {"category": "So", "key": "273F", "mappings": {"default": {"default": "Schwarze Florette"}}}, {
      "category": "So",
      "key": "2740",
      "mappings": {"default": {"default": "Weiße Florette"}}
    }, {
      "category": "So",
      "key": "2741",
      "mappings": {"default": {"default": "Acht Petalled umrissene schwarze Florette"}}
    }, {
      "category": "So",
      "key": "2742",
      "mappings": {"default": {"default": "Eingekreistes offenes Zentrum Achtstern"}}
    }, {
      "category": "So",
      "key": "2743",
      "mappings": {"default": {"default": "Schweres, tropfenbesetztes Nadelrad-Sternchen"}}
    }, {"category": "So", "key": "2744", "mappings": {"default": {"default": "Schneeflocke"}}}, {
      "category": "So",
      "key": "2745",
      "mappings": {"default": {"default": "Feste Trifoliate-Schneeflocke"}}
    }, {
      "category": "So",
      "key": "2746",
      "mappings": {"default": {"default": "Schwere Chevron-Schneeflocke"}}
    }, {"category": "So", "key": "2747", "mappings": {"default": {"default": "Funkeln"}}}, {
      "category": "So",
      "key": "2748",
      "mappings": {"default": {"default": "Schweres Funkeln"}}
    }, {
      "category": "So",
      "key": "2749",
      "mappings": {"default": {"default": "Ballon-Speichen-Sternchen"}}
    }, {
      "category": "So",
      "key": "274A",
      "mappings": {"default": {"default": "Acht Teardrop-Spoked Propeller Sternchen"}}
    }, {
      "category": "So",
      "key": "274B",
      "mappings": {"default": {"default": "Schwere Acht Teardrop-Spoked Propeller Sternchen"}}
    }, {"category": "So", "key": "274C", "mappings": {"default": {"default": "Cross Mark"}}}, {
      "category": "So",
      "key": "274D",
      "mappings": {"default": {"default": "Beschatteter weißer Kreis"}}
    }]
  ,
  "de/units/energy.js":
    [{"locale": "de"}, {
      "key": "W",
      "mappings": {"default": {"default": "Watt"}},
      "category": "energy",
      "names": ["W", "w"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "Kilowatt"}},
      "key": "kW",
      "names": ["kw", "kW"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "Milliwatt"}},
      "key": "mW",
      "names": ["mw", "mW"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "Kilowattstunde"}},
      "key": "kwh",
      "names": ["kwh", "kWh"]
    }, {
      "key": "J",
      "mappings": {"default": {"default": "Joule", "plural": "Joules"}},
      "category": "energy",
      "names": ["J"]
    }, {"key": "N", "mappings": {"default": {"default": "Newton"}}, "category": "energy", "names": ["N"]}, {
      "key": "A",
      "mappings": {"default": {"default": "Ampere"}},
      "category": "energy",
      "names": ["A"]
    }, {
      "key": "V",
      "mappings": {"default": {"default": "Volt"}},
      "category": "energy",
      "names": ["V"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "Mikroohm"}},
      "key": "µΩ",
      "names": ["µΩ"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "Milliohm"}},
      "key": "mΩ",
      "names": ["mΩ"]
    }, {
      "key": "Ω",
      "mappings": {"default": {"default": "Ohm"}},
      "category": "energy",
      "names": ["Ω", "Ohm"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "Kiloohm"}},
      "key": "kΩ",
      "names": ["kΩ", "KΩ"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "Megaohm"}},
      "key": "MΩ",
      "names": ["MΩ"]
    }, {"category": "energy", "mappings": {"default": {"default": "Gigaohm"}}, "key": "GΩ", "names": ["GΩ"]}]
  ,
  "de/units/length.js":
    [{"locale": "de"}, {
      "key": "mm",
      "names": ["mm"],
      "mappings": {"default": {"default": "Millimeter"}},
      "category": "length"
    }, {
      "key": "cm",
      "names": ["cm"],
      "mappings": {"default": {"default": "Zentimeter"}},
      "category": "length"
    }, {
      "key": "m",
      "mappings": {"default": {"default": "Meter"}},
      "category": "length",
      "names": ["m"]
    }, {
      "category": "length",
      "mappings": {"default": {"default": "Kilometer"}},
      "key": "km",
      "names": ["km"]
    }, {
      "key": "ft",
      "mappings": {"default": {"default": "Fuß"}},
      "category": "length",
      "names": ["ft", "ft."]
    }, {
      "key": "in",
      "mappings": {"default": {"default": "Zoll"}},
      "category": "length",
      "names": ["in", "in."]
    }, {
      "key": "mi",
      "mappings": {"default": {"plural": "Meilen", "default": "Meile"}},
      "category": "length",
      "names": ["mi", "mi."]
    }, {
      "key": "yd",
      "mappings": {"default": {"plural": "Yards", "default": "Yard"}},
      "category": "length",
      "names": ["yd", "yd."]
    }, {
      "key": "n.m.",
      "names": ["n.m."],
      "mappings": {"default": {"default": "nautische Meile"}},
      "category": "length"
    }]
  ,
  "de/units/memory.js":
    [{"locale": "de"}, {
      "key": "b",
      "mappings": {"default": {"plural": "Bits", "default": "Bit"}},
      "category": "memory",
      "names": ["b"]
    }, {
      "key": "B",
      "mappings": {"default": {"plural": "Bytes", "default": "Byte"}},
      "category": "memory",
      "names": ["B"]
    }, {
      "key": "KB",
      "mappings": {"default": {"plural": "Kilobytes", "default": "Kilobyte"}},
      "category": "memory",
      "names": ["KB"]
    }, {
      "key": "MB",
      "mappings": {"default": {"plural": "Megabytes", "default": "Megabyte"}},
      "category": "memory",
      "names": ["MB"]
    }, {
      "key": "GB",
      "mappings": {"default": {"plural": "Gigabytes", "default": "Gigabyte"}},
      "category": "memory",
      "names": ["GB"]
    }, {
      "key": "TB",
      "mappings": {"default": {"plural": "Terabytes", "default": "Terabyte"}},
      "category": "memory",
      "names": ["TB"]
    }]
  ,
  "de/units/other.js":
    [{"locale": "de"}, {
      "key": "ha",
      "mappings": {"default": {"default": "Hektar"}},
      "category": "other",
      "names": ["ha"]
    }, {
      "category": "other",
      "mappings": {"default": {"default": "Dutzend"}},
      "key": "doz",
      "names": ["doz", "doz.", "dz", "dz."]
    }]
  ,
  "de/units/speed.js":
    [{"locale": "de"}, {
      "category": "speed",
      "mappings": {"default": {"default": "Knoten"}},
      "key": "kt",
      "names": ["kt", "kt."]
    }, {
      "key": "mph",
      "mappings": {"default": {"default": "Meilen pro Stunde", "singular": "Meile pro Stunde"}},
      "category": "speed",
      "names": ["mph"]
    }, {
      "key": "kmh",
      "mappings": {"default": {"default": "Kilometer pro Stunde"}},
      "category": "speed",
      "names": ["kmh"]
    }, {
      "key": "rpm",
      "mappings": {"default": {"default": "Umdrehungen pro Minute", "singular": "Umdrehung pro Minute"}},
      "category": "speed",
      "names": ["rpm"]
    }]
  ,
  "de/units/temperature.js":
    [{"locale": "de"}, {
      "key": "F",
      "mappings": {"default": {"default": "Farad"}},
      "category": "temperature",
      "names": ["F", "F.", "°F"]
    }, {
      "key": "C",
      "mappings": {"default": {"default": "Celsius"}},
      "category": "temperature",
      "names": ["C", "°C"]
    }, {"key": "K", "mappings": {"default": {"default": "Kelvin"}}, "category": "temperature", "names": ["K", "°K"]}]
  ,
  "de/units/time.js":
    [{"locale": "de"}, {
      "category": "time",
      "mappings": {"default": {"default": "Nanosekunde"}},
      "key": "ns",
      "names": ["ns"]
    }, {
      "category": "time",
      "mappings": {"default": {"default": "Microsekunde"}},
      "key": "µs",
      "names": ["µs"]
    }, {
      "category": "time",
      "mappings": {"default": {"default": "Millisekunde"}},
      "key": "ms",
      "names": ["ms"]
    }, {
      "category": "time",
      "mappings": {"default": {"default": "Sekunde"}},
      "key": "s",
      "names": ["s"]
    }, {
      "category": "time",
      "mappings": {"default": {"default": "Minute"}},
      "key": "min",
      "names": ["min"]
    }, {"category": "time", "mappings": {"default": {"default": "Stunde"}}, "key": "h", "names": ["h", "hr"]}]
  ,
  "de/units/volume.js":
    [{"locale": "de"}, {
      "category": "volume",
      "mappings": {"default": {"default": "Kubik"}},
      "key": "cu",
      "names": ["cu", "cu."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "Kubikinch"}},
      "key": "cu inch",
      "names": ["cu in", "cu. in."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "Kubikfuß"}},
      "key": "cu ft",
      "names": ["cu ft", "cu. ft."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "Kubikyard"}},
      "key": "cu yd",
      "names": ["cu yd", "cu. yd."]
    }, {
      "key": "bbl",
      "mappings": {"default": {"default": "Fass"}},
      "category": "volume",
      "names": ["bbl.", "bbl"]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "Fluid ounce"}},
      "key": "fl. oz.",
      "names": ["fl. oz.", "fl oz"]
    }, {
      "key": "gal",
      "mappings": {"default": {"plural": "Gallonen", "default": "Gallone"}},
      "category": "volume",
      "names": ["gal", "gal."]
    }, {
      "key": "pt",
      "mappings": {"default": {"default": "Pint"}},
      "category": "volume",
      "names": ["pt", "pt."]
    }, {
      "key": "qt",
      "mappings": {"default": {"plural": "Quarts", "default": "Quart"}},
      "category": "volume",
      "names": ["qt", "qt."]
    }, {
      "key": "tbsp",
      "mappings": {"default": {"default": "Esslöffel"}},
      "category": "volume",
      "names": ["tbsp", "tbsp.", "Tbsp", "Tbsp."]
    }, {
      "key": "tsp",
      "mappings": {"default": {"default": "Teelöffel"}},
      "category": "volume",
      "names": ["tsp", "tsp."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "Kubikzentimeter"}},
      "key": "cc",
      "names": ["cc", "ccm"]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "Kiloliter"}},
      "key": "kl",
      "names": ["kl"]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "Liter"}},
      "key": "l",
      "names": ["l"]
    }, {"category": "volume", "mappings": {"default": {"default": "Milliliter"}}, "key": "ml", "names": ["ml"]}]
  ,
  "de/units/weight.js":
    [{"locale": "de"}, {
      "key": "lb",
      "mappings": {"default": {"default": "Pfund"}},
      "category": "weight",
      "names": ["lb", "lb."]
    }, {
      "key": "oz",
      "mappings": {"default": {"plural": "Unzen", "default": "Unze"}},
      "category": "weight",
      "names": ["oz", "oz."]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "Long ton"}},
      "key": "LT",
      "names": ["LT", "L.T."]
    }, {
      "key": "gr",
      "mappings": {"default": {"default": "Gramm"}},
      "category": "weight",
      "names": ["g", "gr"]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "Kilogramm"}},
      "key": "kg",
      "names": ["kg"]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "Mikrogramm"}},
      "key": "mcg",
      "names": ["mcg", "µg"]
    }, {
      "category": "weight",
      "mappings": {"default": {"default": "Milligramm"}},
      "key": "mg",
      "names": ["mg"]
    }, {
      "key": "t",
      "mappings": {"default": {"plural": "Tonnen", "default": "Tonne"}},
      "category": "weight",
      "names": ["t", "T"]
    }]
  ,
  "es/functions/algebra.js":
    [{"locale": "es"}, {
      "key": "deg",
      "names": ["deg"],
      "mappings": {"default": {"default": "grados"}},
      "category": "Algebra"
    }, {
      "key": "det",
      "names": ["det"],
      "mappings": {"default": {"default": "determinante"}},
      "category": "Algebra"
    }, {
      "key": "dim",
      "names": ["dim"],
      "mappings": {"default": {"default": "dimensión"}},
      "category": "Algebra"
    }, {
      "key": "hom",
      "names": ["hom", "Hom"],
      "mappings": {"default": {"default": "homomorfismo"}},
      "category": "Algebra"
    }, {
      "key": "ker",
      "names": ["ker"],
      "mappings": {"default": {"default": "kernel"}},
      "category": "Algebra"
    }, {"key": "Tr", "names": ["Tr", "tr"], "mappings": {"default": {"default": "traza"}}, "category": "Algebra"}]
  ,
  "es/functions/elementary.js":
    [{"locale": "es"}, {
      "key": "log",
      "names": ["log"],
      "mappings": {"default": {"default": "logaritmo"}},
      "category": "Logarithm"
    }, {
      "key": "ln",
      "names": ["ln"],
      "mappings": {"default": {"default": "logaritmo neperiano"}},
      "category": "Logarithm"
    }, {
      "key": "lg",
      "names": ["lg"],
      "mappings": {"default": {"default": "logaritmo base 10"}},
      "category": "Logarithm"
    }, {
      "key": "exp",
      "names": ["exp", "expt"],
      "mappings": {"default": {"default": "exponente"}},
      "category": "Elementary"
    }, {
      "key": "gcd",
      "names": ["gcd", "mcd", "MCD"],
      "mappings": {"default": {"default": "MCD"}},
      "category": "Elementary"
    }, {
      "key": "lcm",
      "names": ["lcm", "mcm", "MCM"],
      "mappings": {"default": {"default": "mcm"}},
      "category": "Elementary"
    }, {
      "key": "arg",
      "names": ["arg"],
      "mappings": {"default": {"default": "argumento"}},
      "category": "Complex"
    }, {
      "key": "im",
      "names": ["im"],
      "mappings": {"default": {"default": "parte imaginaria"}},
      "category": "Complex"
    }, {
      "key": "re",
      "names": ["re"],
      "mappings": {"default": {"default": "residuo"}},
      "category": "Complex"
    }, {
      "key": "inf",
      "names": ["inf"],
      "mappings": {"default": {"default": "extremo inferior"}},
      "category": "Limits"
    }, {
      "key": "lim",
      "names": ["lim"],
      "mappings": {"default": {"default": "límite"}},
      "category": "Limits"
    }, {
      "key": "max",
      "names": ["max"],
      "mappings": {"default": {"default": "máximo"}},
      "category": "Limits"
    }, {
      "key": "min",
      "names": ["min"],
      "mappings": {"default": {"default": "mínimo"}},
      "category": "Limits"
    }, {
      "key": "sup",
      "names": ["sup"],
      "mappings": {"default": {"default": "superior"}},
      "category": "Limits"
    }, {
      "key": "lim inf",
      "names": ["lim inf", "liminf"],
      "mappings": {"default": {"default": "límite inferior"}},
      "category": "Limits"
    }, {
      "key": "lim sup",
      "names": ["lim sup", "limsup"],
      "mappings": {"default": {"default": "límite superior"}},
      "category": "Limits"
    }, {
      "key": "injlim",
      "names": ["injlim", "inj lim"],
      "mappings": {"default": {"default": "límite directo"}},
      "category": "Limits"
    }, {
      "key": "projlim",
      "names": ["projlim", "proj lim"],
      "mappings": {"default": {"default": "límite inverso"}},
      "category": "Limits"
    }, {
      "key": "mod",
      "names": ["mod"],
      "mappings": {"default": {"default": "módulo"}},
      "category": "Elementary"
    }, {"key": "Pr", "names": ["Pr"], "mappings": {"default": {"default": "probabilidad"}}, "category": "Probability"}]
  ,
  "es/functions/hyperbolic.js":
    [{"locale": "es"}, {
      "key": "cosh",
      "names": ["cosh"],
      "mappings": {"default": {"default": "coseno hiperbólico"}},
      "category": "Hyperbolic"
    }, {
      "key": "coth",
      "names": ["coth"],
      "mappings": {"default": {"default": "cotangente hiperbólica"}},
      "category": "Hyperbolic"
    }, {
      "key": "csch",
      "names": ["csch"],
      "mappings": {"default": {"default": "cosecante hiperbólica"}},
      "category": "Hyperbolic"
    }, {
      "key": "sech",
      "names": ["sech"],
      "mappings": {"default": {"default": "secante hiperbólica"}},
      "category": "Hyperbolic"
    }, {
      "key": "sinh",
      "names": ["sinh"],
      "mappings": {"default": {"default": "seno hiperbólico"}},
      "category": "Hyperbolic"
    }, {
      "key": "tanh",
      "names": ["tanh"],
      "mappings": {"default": {"default": "tangente hiperbólica"}},
      "category": "Hyperbolic"
    }, {
      "key": "arcosh",
      "names": ["arcosh", "arccosh"],
      "mappings": {"default": {"default": "area coseno hiperbólico"}},
      "category": "Area"
    }, {
      "key": "arcoth",
      "names": ["arcoth", "arccoth"],
      "mappings": {"default": {"default": "area cotangente hiperbólica"}},
      "category": "Area"
    }, {
      "key": "arcsch",
      "names": ["arcsch", "arccsch"],
      "mappings": {"default": {"default": "area cosecante hiperbólica"}},
      "category": "Area"
    }, {
      "key": "arsech",
      "names": ["arsech", "arcsech"],
      "mappings": {"default": {"default": "area secante hiperbólica"}},
      "category": "Area"
    }, {
      "key": "arsinh",
      "names": ["arsinh", "arcsinh"],
      "mappings": {"default": {"default": "area seno hiperbólico"}},
      "category": "Area"
    }, {
      "key": "artanh",
      "names": ["artanh", "arctanh"],
      "mappings": {"default": {"default": "area tangente hiperbólica"}},
      "category": "Area"
    }]
  ,
  "es/functions/trigonometry.js":
    [{"locale": "es"}, {
      "key": "cos",
      "names": ["cos", "cosine"],
      "mappings": {"default": {"default": "coseno"}},
      "category": "Trigonometric"
    }, {
      "key": "cot",
      "names": ["cot"],
      "mappings": {"default": {"default": "cotangente"}},
      "category": "Trigonometric"
    }, {
      "key": "csc",
      "names": ["csc"],
      "mappings": {"default": {"default": "cosecante"}},
      "category": "Trigonometric"
    }, {
      "key": "sec",
      "names": ["sec"],
      "mappings": {"default": {"default": "secant"}},
      "category": "Trigonometric"
    }, {
      "key": "sin",
      "names": ["sin", "sine", "sen"],
      "mappings": {"default": {"default": "seno"}},
      "category": "Trigonometric"
    }, {
      "key": "tan",
      "names": ["tan"],
      "mappings": {"default": {"default": "tangente"}},
      "category": "Trigonometric"
    }, {
      "key": "arccos",
      "names": ["arccos"],
      "mappings": {"default": {"default": "arco coseno"}},
      "category": "Cyclometric"
    }, {
      "key": "arccot",
      "names": ["arccot"],
      "mappings": {"default": {"default": "arco cotangente"}},
      "category": "Cyclometric"
    }, {
      "key": "arccsc",
      "names": ["arccsc"],
      "mappings": {"default": {"default": "arco cosecante"}},
      "category": "Cyclometric"
    }, {
      "key": "arcsec",
      "names": ["arcsec"],
      "mappings": {"default": {"default": "arco secante"}},
      "category": "Cyclometric"
    }, {
      "key": "arcsin",
      "names": ["arcsin"],
      "mappings": {"default": {"default": "arco seno"}},
      "category": "Cyclometric"
    }, {
      "key": "arctan",
      "names": ["arctan"],
      "mappings": {"default": {"default": "arco tangente"}},
      "category": "Cyclometric"
    }]
  ,
  "es/symbols/digits_rest.js":
    [{"locale": "es"}, {
      "category": "No",
      "mappings": {
        "default": {"default": "al cuadrado"},
        "mathspeak": {"default": "al cuadrado"},
        "clearspeak": {"default": "al cuadrado"}
      },
      "key": "00B2"
    }, {
      "category": "No",
      "mappings": {
        "default": {"default": "al cubo"},
        "mathspeak": {"default": "al cubo"},
        "clearspeak": {"default": "al cubo"}
      },
      "key": "00B3"
    }, {"category": "No", "mappings": {"default": {"default": "un cuarto"}}, "key": "00BC"}, {
      "category": "No",
      "mappings": {"default": {"default": "un medio"}},
      "key": "00BD"
    }, {"category": "No", "mappings": {"default": {"default": "tres cuartos"}}, "key": "00BE"}, {
      "category": "No",
      "key": "2150",
      "mappings": {"default": {"default": "Vulgar Fracción Un Séptimo"}}
    }, {
      "category": "No",
      "key": "2151",
      "mappings": {"default": {"default": "Vulgar Fracción Un Noveno"}}
    }, {
      "category": "No",
      "key": "2152",
      "mappings": {"default": {"default": "Vulgar Fracción Un Décimo"}}
    }, {
      "category": "No",
      "key": "2153",
      "mappings": {"default": {"default": "Vulgar Fracción Un Tercero"}}
    }, {
      "category": "No",
      "key": "2154",
      "mappings": {"default": {"default": "Fracción vulgar dos tercios"}}
    }, {
      "category": "No",
      "key": "2155",
      "mappings": {"default": {"default": "Vulgar Fraction One Fifth"}}
    }, {
      "category": "No",
      "key": "2156",
      "mappings": {"default": {"default": "Fracción vulgar dos quintos"}}
    }, {
      "category": "No",
      "key": "2157",
      "mappings": {"default": {"default": "Fracción vulgar tres quintos"}}
    }, {
      "category": "No",
      "key": "2158",
      "mappings": {"default": {"default": "Fracción vulgar cuatro quintos"}}
    }, {
      "category": "No",
      "key": "2159",
      "mappings": {"default": {"default": "Vulgar Fracción Un Sexto"}}
    }, {
      "category": "No",
      "key": "215A",
      "mappings": {"default": {"default": "Vulgar fracción cinco sextos"}}
    }, {
      "category": "No",
      "key": "215B",
      "mappings": {"default": {"default": "Vulgar Fracción Un Octavo"}}
    }, {
      "category": "No",
      "key": "215C",
      "mappings": {"default": {"default": "Fracción vulgar tres octavos"}}
    }, {
      "category": "No",
      "key": "215D",
      "mappings": {"default": {"default": "Fracción vulgar cinco octavos"}}
    }, {
      "category": "No",
      "key": "215E",
      "mappings": {"default": {"default": "Fracción vulgar siete octavos"}}
    }, {
      "category": "No",
      "key": "215F",
      "mappings": {"default": {"default": "Numerador de fracciones uno"}}
    }, {
      "category": "No",
      "key": "2189",
      "mappings": {"default": {"default": "Vulgar Fraction Zero Thirds"}}
    }, {
      "category": "No",
      "key": "3248",
      "mappings": {"default": {"default": "Número diez en un círculo en la Plaza Negra"}}
    }, {
      "category": "No",
      "key": "3249",
      "mappings": {"default": {"default": "Número veinte en un círculo en la Plaza Negra"}}
    }, {
      "category": "No",
      "key": "324A",
      "mappings": {"default": {"default": "Número circundado treinta en cuadrado negro"}}
    }, {
      "category": "No",
      "key": "324B",
      "mappings": {"default": {"default": "Número circundado Cuarenta en cuadrado negro"}}
    }, {
      "category": "No",
      "key": "324C",
      "mappings": {"default": {"default": "Círculo número cincuenta en la plaza negra"}}
    }, {
      "category": "No",
      "key": "324D",
      "mappings": {"default": {"default": "Círculo número sesenta en la plaza negra"}}
    }, {
      "category": "No",
      "key": "324E",
      "mappings": {"default": {"default": "Número setenta en un círculo en la Plaza Negra"}}
    }, {
      "category": "No",
      "key": "324F",
      "mappings": {"default": {"default": "Círculo número ochenta en cuadrado negro"}}
    }]
  ,
  "es/symbols/greek-rest.js":
    [{"locale": "es"}, {
      "category": "Lu",
      "key": "0394",
      "mappings": {"clearspeak": {"default": "triangle", "TriangleSymbol_Delta": "cap Delta"}}
    }]
  ,
  "es/symbols/greek-scripts.js":
    [{"locale": "es"}, {
      "category": "Ll",
      "key": "1D26",
      "mappings": {"default": {"default": "pequeño mayúscula gamma"}}
    }, {
      "category": "Ll",
      "key": "1D27",
      "mappings": {"default": {"default": "pequeña mayúscula lamda"}}
    }, {
      "category": "Ll",
      "key": "1D28",
      "mappings": {"default": {"default": "pequeña mayúscula Pi"}}
    }, {
      "category": "Ll",
      "key": "1D29",
      "mappings": {"default": {"default": "pequeña mayúscula Rho"}}
    }, {
      "category": "Ll",
      "key": "1D2A",
      "mappings": {"default": {"default": "pequeña mayúscula Psi"}}
    }, {
      "category": "Lm",
      "key": "1D5E",
      "mappings": {"default": {"default": "modificador de pequeño gamma"}}
    }, {
      "category": "Lm",
      "key": "1D60",
      "mappings": {"default": {"default": "modificador de pequeño phi"}}
    }, {
      "category": "Lm",
      "key": "1D66",
      "mappings": {"default": {"default": "subscripción pequeña beta"}}
    }, {
      "category": "Lm",
      "key": "1D67",
      "mappings": {"default": {"default": "subíndice pequeña gamma"}}
    }, {"category": "Lm", "key": "1D68", "mappings": {"default": {"default": "subíndice rho"}}}, {
      "category": "Lm",
      "key": "1D69",
      "mappings": {"default": {"default": "subíndice phi"}}
    }, {"category": "Lm", "key": "1D6A", "mappings": {"default": {"default": "subscripción chi"}}}]
  ,
  "es/symbols/greek-symbols.js":
    [{"locale": "es"}, {
      "key": "03D0",
      "mappings": {"default": {"default": "símbolo beta"}},
      "category": "Ll"
    }, {"key": "03D7", "mappings": {"default": {"default": "símbolo kai griego"}}, "category": "Ll"}, {
      "key": "03F6",
      "mappings": {"default": {"default": "épsilon invertido"}},
      "category": "Sm"
    }, {
      "category": "Lu",
      "key": "1D7CA",
      "mappings": {"default": {"default": "Matemática Bold Capital Digamma"}}
    }, {"category": "Ll", "key": "1D7CB", "mappings": {"default": {"default": "Matemática negrita pequeño digamma"}}}]
  ,
  "es/symbols/hebrew_letters.js":
    [{"locale": "es"}, {"key": "2135", "mappings": {"default": {"default": "alef"}}, "category": "Lo"}, {
      "key": "2136",
      "mappings": {"default": {"default": "bet"}},
      "category": "Lo"
    }, {"key": "2137", "mappings": {"default": {"default": "guímel"}}, "category": "Lo"}, {
      "key": "2138",
      "mappings": {"default": {"default": "dálet"}},
      "category": "Lo"
    }]
  ,
  "es/symbols/latin-lower-double-accent.js":
    [{"locale": "es"}, {
      "category": "Ll",
      "key": "01D6",
      "mappings": {"default": {"default": "u con diéresis y macron"}}
    }, {
      "category": "Ll",
      "key": "01D8",
      "mappings": {"default": {"default": "u con diéresis y acento agudo"}}
    }, {
      "category": "Ll",
      "key": "01DA",
      "mappings": {"default": {"default": "u con diéresis y acento"}}
    }, {
      "category": "Ll",
      "key": "01DC",
      "mappings": {"default": {"default": "u con diéresis y grave"}}
    }, {
      "category": "Ll",
      "key": "01DF",
      "mappings": {"default": {"default": "a con diéresis y macron"}}
    }, {
      "category": "Ll",
      "key": "01E1",
      "mappings": {"default": {"default": "a con punto arriba y macron"}}
    }, {
      "category": "Ll",
      "key": "01ED",
      "mappings": {"default": {"default": "o con pecíolo y macron"}}
    }, {
      "category": "Ll",
      "key": "01FB",
      "mappings": {"default": {"default": "a con ring above y acento agudo"}}
    }, {
      "category": "Ll",
      "key": "022B",
      "mappings": {"default": {"default": "o con diéresis y macron"}}
    }, {
      "category": "Ll",
      "key": "022D",
      "mappings": {"default": {"default": "o con tilde y macron"}}
    }, {
      "category": "Ll",
      "key": "0231",
      "mappings": {"default": {"default": "o con Dot Above y Macron"}}
    }, {"key": "1E09", "mappings": {"default": {"default": "c con cedilla y agudo"}}, "category": "Ll"}, {
      "key": "1E15",
      "mappings": {"default": {"default": "e con macrón y grave"}},
      "category": "Ll"
    }, {"key": "1E17", "mappings": {"default": {"default": "e con macrón y agudo"}}, "category": "Ll"}, {
      "key": "1E1D",
      "mappings": {"default": {"default": "e con cedilla y breve"}},
      "category": "Ll"
    }, {
      "key": "1E2F",
      "mappings": {"default": {"default": "i con diéresis y agudo"}},
      "category": "Ll"
    }, {
      "key": "1E39",
      "mappings": {"default": {"default": "l con punto debajo y macrón"}},
      "category": "Ll"
    }, {"key": "1E4D", "mappings": {"default": {"default": "o con tilde y acute"}}, "category": "Ll"}, {
      "key": "1E4F",
      "mappings": {"default": {"default": "o con tilde y diéresis"}},
      "category": "Ll"
    }, {"key": "1E51", "mappings": {"default": {"default": "o con macrón y grave"}}, "category": "Ll"}, {
      "key": "1E53",
      "mappings": {"default": {"default": "o con macrón y agudo"}},
      "category": "Ll"
    }, {
      "key": "1E5D",
      "mappings": {"default": {"default": "r con punto debajo y macrón"}},
      "category": "Ll"
    }, {
      "key": "1E65",
      "mappings": {"default": {"default": "s con agudo y punto arriba"}},
      "category": "Ll"
    }, {
      "key": "1E67",
      "mappings": {"default": {"default": "s con carón y punto arriba"}},
      "category": "Ll"
    }, {
      "key": "1E69",
      "mappings": {"default": {"default": "s con punto debajo y punto arriba"}},
      "category": "Ll"
    }, {"key": "1E79", "mappings": {"default": {"default": "u con tilde y agudo"}}, "category": "Ll"}, {
      "key": "1E7B",
      "mappings": {"default": {"default": "u con macrón y diéresis"}},
      "category": "Ll"
    }, {
      "key": "1EA5",
      "mappings": {"default": {"default": "a con acento circunflejo y agudo"}},
      "category": "Ll"
    }, {
      "key": "1EA7",
      "mappings": {"default": {"default": "a con acento circunflejo y grave"}},
      "category": "Ll"
    }, {
      "key": "1EA9",
      "mappings": {"default": {"default": "a con acento circunflejo y gancho arriba"}},
      "category": "Ll"
    }, {
      "key": "1EAB",
      "mappings": {"default": {"default": "a con acento circunflejo y tilde"}},
      "category": "Ll"
    }, {
      "key": "1EAD",
      "mappings": {"default": {"default": "a con acento circunflejo y punto debajo"}},
      "category": "Ll"
    }, {"key": "1EAF", "mappings": {"default": {"default": "a con breve y agudo"}}, "category": "Ll"}, {
      "key": "1EB1",
      "mappings": {"default": {"default": "a con breve y grave"}},
      "category": "Ll"
    }, {
      "key": "1EB3",
      "mappings": {"default": {"default": "a con breve y gancho arriba"}},
      "category": "Ll"
    }, {"key": "1EB5", "mappings": {"default": {"default": "a con breve y tilde"}}, "category": "Ll"}, {
      "key": "1EB7",
      "mappings": {"default": {"default": "a con breve y punto debajo"}},
      "category": "Ll"
    }, {
      "key": "1EBF",
      "mappings": {"default": {"default": "e con acento circunflejo y agudo"}},
      "category": "Ll"
    }, {
      "key": "1EC1",
      "mappings": {"default": {"default": "e con acento circunflejo y grave"}},
      "category": "Ll"
    }, {
      "key": "1EC3",
      "mappings": {"default": {"default": "e con acento circunflejo y gancho arriba"}},
      "category": "Ll"
    }, {
      "key": "1EC5",
      "mappings": {"default": {"default": "e con acento circunflejo y tilde"}},
      "category": "Ll"
    }, {
      "key": "1EC7",
      "mappings": {"default": {"default": "e con acento circunflejo y punto debajo"}},
      "category": "Ll"
    }, {
      "key": "1ED1",
      "mappings": {"default": {"default": "o con acento circunflejo y agudo"}},
      "category": "Ll"
    }, {
      "key": "1ED3",
      "mappings": {"default": {"default": "o con acento circunflejo y grave"}},
      "category": "Ll"
    }, {
      "key": "1ED5",
      "mappings": {"default": {"default": "o con acento circunflejo y gancho arriba"}},
      "category": "Ll"
    }, {
      "key": "1ED7",
      "mappings": {"default": {"default": "o con acento circunflejo y tilde"}},
      "category": "Ll"
    }, {
      "key": "1ED9",
      "mappings": {"default": {"default": "o con acento circunflejo y punto debajo"}},
      "category": "Ll"
    }, {"key": "1EDB", "mappings": {"default": {"default": "o with horn and acute"}}, "category": "Ll"}, {
      "key": "1EDD",
      "mappings": {"default": {"default": "o with horn and grave"}},
      "category": "Ll"
    }, {
      "key": "1EDF",
      "mappings": {"default": {"default": "o with horn and hook above"}},
      "category": "Ll"
    }, {"key": "1EE1", "mappings": {"default": {"default": "o with horn and tilde"}}, "category": "Ll"}, {
      "key": "1EE3",
      "mappings": {"default": {"default": "o con cuerno y punto debajo"}},
      "category": "Ll"
    }, {"key": "1EE9", "mappings": {"default": {"default": "u con cuerno y agudo"}}, "category": "Ll"}, {
      "key": "1EEB",
      "mappings": {"default": {"default": "u con cuerno y grave"}},
      "category": "Ll"
    }, {
      "key": "1EED",
      "mappings": {"default": {"default": "u con cuerno y gancho arriba"}},
      "category": "Ll"
    }, {"key": "1EEF", "mappings": {"default": {"default": "u con cuerno y tilde"}}, "category": "Ll"}, {
      "key": "1EF1",
      "mappings": {"default": {"default": "u con cuerno y punto debajo"}},
      "category": "Ll"
    }]
  ,
  "es/symbols/latin-lower-phonetic.js":
    [{"locale": "es"}, {
      "key": "00F8",
      "mappings": {"default": {"default": "o barrada"}},
      "category": "Ll"
    }, {"key": "0111", "mappings": {"default": {"default": "d con barra"}}, "category": "Ll"}, {
      "key": "0127",
      "mappings": {"default": {"default": "h con guión"}},
      "category": "Ll"
    }, {"key": "0142", "mappings": {"default": {"default": "l con guión"}}, "category": "Ll"}, {
      "key": "0167",
      "mappings": {"default": {"default": "t con guión"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0180", "mappings": {"default": {"default": "b con línea"}}}, {
      "key": "019B",
      "mappings": {"default": {"default": "lambda tachada"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "01B6", "mappings": {"default": {"default": "z con línea"}}}, {
      "category": "Ll",
      "key": "01BE",
      "mappings": {"default": {"default": "parada glotal invertida con línea"}}
    }, {"category": "Ll", "key": "01E5", "mappings": {"default": {"default": "g con línea"}}}, {
      "category": "Ll",
      "key": "01FF",
      "mappings": {"default": {"default": "o con línea y acento agudo"}}
    }, {"category": "Ll", "key": "023C", "mappings": {"default": {"default": "c con trazo"}}}, {
      "category": "Ll",
      "key": "0247",
      "mappings": {"default": {"default": "pequeña e con trazo"}}
    }, {"category": "Ll", "key": "0249", "mappings": {"default": {"default": "j con trazo"}}}, {
      "category": "Ll",
      "key": "024D",
      "mappings": {"default": {"default": "con trazo"}}
    }, {"category": "Ll", "key": "024F", "mappings": {"default": {"default": "y con trazo"}}}, {
      "category": "Ll",
      "key": "025F",
      "mappings": {"default": {"default": "pequeña sin puntos j con trazo"}}
    }, {"category": "Ll", "key": "0268", "mappings": {"default": {"default": "i con trazo"}}}, {
      "category": "Ll",
      "key": "0284",
      "mappings": {"default": {"default": "pequeña sin puntos j con trazo y gancho"}}
    }, {
      "category": "Ll",
      "key": "02A1",
      "mappings": {"default": {"default": "parada glotal con trazo"}}
    }, {
      "category": "Ll",
      "key": "02A2",
      "mappings": {"default": {"default": "parada invertida glotal con trazo"}}
    }, {
      "category": "Ll",
      "key": "1D13",
      "mappings": {"default": {"default": "de lado O con trazo"}}
    }, {"category": "Ll", "key": "1D7C", "mappings": {"default": {"default": "iota con trazo"}}}, {
      "category": "Ll",
      "key": "1D7D",
      "mappings": {"default": {"default": "p con trazo"}}
    }, {"category": "Ll", "key": "1D7F", "mappings": {"default": {"default": "upsilon con trazo"}}}, {
      "category": "Ll",
      "key": "1E9C",
      "mappings": {"default": {"default": "larga s con trazo diagonal"}}
    }, {
      "category": "Ll",
      "key": "1E9D",
      "mappings": {"default": {"default": "pequeña s larga con trazo alto"}}
    }, {"category": "Ll", "key": "018D", "mappings": {"default": {"default": "delta convertido"}}}, {
      "key": "1E9B",
      "mappings": {"default": {"default": "s larga con punto arriba"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "1E9F", "mappings": {"default": {"default": "delta pequeña latina"}}}, {
      "key": "0138",
      "mappings": {"default": {"default": "kra"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "017F", "mappings": {"default": {"default": "s larga"}}}, {
      "category": "Ll",
      "key": "0183",
      "mappings": {"default": {"default": "b con barra encima"}}
    }, {"category": "Ll", "key": "0185", "mappings": {"default": {"default": "tono seis"}}}, {
      "category": "Ll",
      "key": "0188",
      "mappings": {"default": {"default": "c con gancho"}}
    }, {"category": "Ll", "key": "018C", "mappings": {"default": {"default": "d con barra encima"}}}, {
      "key": "0192",
      "mappings": {"default": {"default": "f con gancho"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0195", "mappings": {"default": {"default": "hv"}}}, {
      "category": "Ll",
      "key": "0199",
      "mappings": {"default": {"default": "k con gancho"}}
    }, {"key": "019A", "mappings": {"default": {"default": "l con barra"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "019E",
      "mappings": {"default": {"default": "n con pierna derecha larga"}}
    }, {"category": "Ll", "key": "01A1", "mappings": {"default": {"default": "o con cuerno"}}}, {
      "category": "Ll",
      "key": "01A3",
      "mappings": {"default": {"default": "oi"}}
    }, {"category": "Ll", "key": "01A5", "mappings": {"default": {"default": "p con gancho"}}}, {
      "category": "Ll",
      "key": "01A8",
      "mappings": {"default": {"default": "dos tonos"}}
    }, {
      "category": "Ll",
      "key": "01AA",
      "mappings": {"default": {"default": "latin letter invertido Esh bucle"}}
    }, {
      "category": "Ll",
      "key": "01AB",
      "mappings": {"default": {"default": "t con palatal gancho"}}
    }, {"category": "Ll", "key": "01AD", "mappings": {"default": {"default": "t con gancho"}}}, {
      "category": "Ll",
      "key": "01B0",
      "mappings": {"default": {"default": "u con horn"}}
    }, {"category": "Ll", "key": "01B4", "mappings": {"default": {"default": "y con gancho"}}}, {
      "category": "Ll",
      "key": "01B9",
      "mappings": {"default": {"default": "ezh invertido"}}
    }, {"category": "Ll", "key": "01BA", "mappings": {"default": {"default": "ezh con cola"}}}, {
      "category": "Ll",
      "key": "01BD",
      "mappings": {"default": {"default": "tono cinco"}}
    }, {"category": "Ll", "key": "01BF", "mappings": {"default": {"default": "wynn"}}}, {
      "category": "Ll",
      "key": "01C6",
      "mappings": {"default": {"default": "dz con acento"}}
    }, {"category": "Ll", "key": "01C9", "mappings": {"default": {"default": "lj"}}}, {
      "category": "Ll",
      "key": "01CC",
      "mappings": {"default": {"default": "nj"}}
    }, {"category": "Ll", "key": "01E3", "mappings": {"default": {"default": "ae con macron"}}}, {
      "category": "Ll",
      "key": "01EF",
      "mappings": {"default": {"default": "ezh con acento"}}
    }, {"category": "Ll", "key": "01F3", "mappings": {"default": {"default": "dz"}}}, {
      "category": "Ll",
      "key": "021D",
      "mappings": {"default": {"default": "latín letra pequeña yogh"}}
    }, {
      "category": "Ll",
      "key": "026E",
      "mappings": {"default": {"default": "lezh letra pequeña latina"}}
    }, {"category": "Ll", "key": "0292", "mappings": {"default": {"default": "ezh"}}}, {
      "category": "Ll",
      "key": "0293",
      "mappings": {"default": {"default": "pequeña Ezh con Curl"}}
    }, {
      "category": "Ll",
      "key": "02A4",
      "mappings": {"default": {"default": "latín letra pequeña Dezh Digraph"}}
    }, {"category": "Ll", "key": "01DD", "mappings": {"default": {"default": "e convertido"}}}, {
      "category": "Ll",
      "key": "01FD",
      "mappings": {"default": {"default": "ae con acento agudo"}}
    }, {"category": "Ll", "key": "0221", "mappings": {"default": {"default": "d con rizo"}}}, {
      "category": "Ll",
      "key": "0223",
      "mappings": {"default": {"default": "ou"}}
    }, {"category": "Ll", "key": "0225", "mappings": {"default": {"default": "z con gancho"}}}, {
      "category": "Ll",
      "key": "0234",
      "mappings": {"default": {"default": "l con rizo"}}
    }, {"category": "Ll", "key": "0235", "mappings": {"default": {"default": "n con rizo"}}}, {
      "category": "Ll",
      "key": "0236",
      "mappings": {"default": {"default": "t con rizo"}}
    }, {
      "category": "Ll",
      "key": "0238",
      "mappings": {"default": {"default": "letra pequeña latina db digraph"}}
    }, {
      "category": "Ll",
      "key": "0239",
      "mappings": {"default": {"default": "latín letra pequeña Qp Digraph"}}
    }, {"category": "Ll", "key": "023F", "mappings": {"default": {"default": "s con cola cruzada"}}}, {
      "category": "Ll",
      "key": "0240",
      "mappings": {"default": {"default": "z con cola swash"}}
    }, {
      "category": "Ll",
      "key": "0242",
      "mappings": {"default": {"default": "latín letra pequeña parada glotal"}}
    }, {
      "category": "Ll",
      "key": "024B",
      "mappings": {"default": {"default": "q con cola de gancho"}}
    }, {"category": "Ll", "key": "0250", "mappings": {"default": {"default": "convertida en una"}}}, {
      "category": "Ll",
      "key": "0251",
      "mappings": {"default": {"default": "alfa minúscula latina"}}
    }, {"category": "Ll", "key": "0252", "mappings": {"default": {"default": "convertida en alfa"}}}, {
      "category": "Ll",
      "key": "0253",
      "mappings": {"default": {"default": "b con gancho"}}
    }, {"category": "Ll", "key": "0254", "mappings": {"default": {"default": "abierta o"}}}, {
      "category": "Ll",
      "key": "0255",
      "mappings": {"default": {"default": "c con rizo"}}
    }, {"category": "Ll", "key": "0256", "mappings": {"default": {"default": "d con cola"}}}, {
      "category": "Ll",
      "key": "0257",
      "mappings": {"default": {"default": "d con gancho"}}
    }, {"category": "Ll", "key": "0258", "mappings": {"default": {"default": "invertida E"}}}, {
      "category": "Ll",
      "key": "0259",
      "mappings": {"default": {"default": "schwa"}}
    }, {
      "category": "Ll",
      "key": "025A",
      "mappings": {"default": {"default": "pequeña Schwa con gancho"}}
    }, {"key": "025B", "mappings": {"default": {"default": "e abierta"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "025C",
      "mappings": {"default": {"default": "invertida abierta E"}}
    }, {
      "category": "Ll",
      "key": "025D",
      "mappings": {"default": {"default": "letra pequeña latina invertida E abierta con gancho"}}
    }, {
      "category": "Ll",
      "key": "025E",
      "mappings": {"default": {"default": "cerrada invertida abierta E"}}
    }, {"category": "Ll", "key": "0260", "mappings": {"default": {"default": "g con gancho"}}}, {
      "category": "Ll",
      "key": "0261",
      "mappings": {"default": {"default": "escritura latina de letras pequeñas G"}}
    }, {
      "category": "Ll",
      "key": "0263",
      "mappings": {"default": {"default": "latín letra pequeña gamma"}}
    }, {
      "category": "Ll",
      "key": "0264",
      "mappings": {"default": {"default": "cuerno de carnero de letra pequeña latina"}}
    }, {"category": "Ll", "key": "0265", "mappings": {"default": {"default": "girada h"}}}, {
      "category": "Ll",
      "key": "0266",
      "mappings": {"default": {"default": "h con gancho"}}
    }, {
      "category": "Ll",
      "key": "0267",
      "mappings": {"default": {"default": "pequeña letra latina Heng con gancho"}}
    }, {
      "category": "Ll",
      "key": "0269",
      "mappings": {"default": {"default": "latín letra minúscula Iota"}}
    }, {"category": "Ll", "key": "026B", "mappings": {"default": {"default": "l con tilde medio"}}}, {
      "category": "Ll",
      "key": "026C",
      "mappings": {"default": {"default": "l con cinturón"}}
    }, {
      "category": "Ll",
      "key": "026D",
      "mappings": {"default": {"default": "letra L latina pequeña con gancho Retroflex"}}
    }, {"category": "Ll", "key": "026F", "mappings": {"default": {"default": "girada m"}}}, {
      "category": "Ll",
      "key": "0270",
      "mappings": {"default": {"default": "vuelta M con pierna larga"}}
    }, {"category": "Ll", "key": "0271", "mappings": {"default": {"default": "m con gancho"}}}, {
      "category": "Ll",
      "key": "0272",
      "mappings": {"default": {"default": "n con gancho izquierdo"}}
    }, {
      "category": "Ll",
      "key": "0273",
      "mappings": {"default": {"default": "n con gancho retroflex"}}
    }, {
      "category": "Ll",
      "key": "0275",
      "mappings": {"default": {"default": "letra minúscula omega"}}
    }, {
      "category": "Ll",
      "key": "0277",
      "mappings": {"default": {"default": "latin Small Letter Closed Omega"}}
    }, {
      "category": "Ll",
      "key": "0278",
      "mappings": {"default": {"default": "Letra pequeña latina"}}
    }, {"category": "Ll", "key": "0279", "mappings": {"default": {"default": "vuelta r"}}}, {
      "category": "Ll",
      "key": "027A",
      "mappings": {"default": {"default": "vuelta R con pierna larga"}}
    }, {
      "category": "Ll",
      "key": "027B",
      "mappings": {"default": {"default": "vuelta R con gancho"}}
    }, {"category": "Ll", "key": "027C", "mappings": {"default": {"default": "R con pierna larga"}}}, {
      "category": "Ll",
      "key": "027D",
      "mappings": {"default": {"default": "R con cola"}}
    }, {"category": "Ll", "key": "027E", "mappings": {"default": {"default": "R con anzuelo"}}}, {
      "category": "Ll",
      "key": "027F",
      "mappings": {"default": {"default": "invertida R con anzuelo"}}
    }, {"category": "Ll", "key": "0282", "mappings": {"default": {"default": "S con gancho"}}}, {
      "category": "Ll",
      "key": "0283",
      "mappings": {"default": {"default": "Pequeña letra latina esh"}}
    }, {
      "category": "Ll",
      "key": "0285",
      "mappings": {"default": {"default": "Latín letra pequeña en cuclillas invertida Esh"}}
    }, {
      "category": "Ll",
      "key": "0286",
      "mappings": {"default": {"default": "Esh minúscula latina con rizo"}}
    }, {"category": "Ll", "key": "0287", "mappings": {"default": {"default": "vuelta t"}}}, {
      "category": "Ll",
      "key": "0288",
      "mappings": {"default": {"default": "T con gancho retroflex"}}
    }, {
      "category": "Ll",
      "key": "0289",
      "mappings": {"default": {"default": "Letra de letra pequeña latina U"}}
    }, {"category": "Ll", "key": "028A", "mappings": {"default": {"default": "Upsilon"}}}, {
      "category": "Ll",
      "key": "028B",
      "mappings": {"default": {"default": "V con gancho"}}
    }, {"category": "Ll", "key": "028C", "mappings": {"default": {"default": "convertida V"}}}, {
      "category": "Ll",
      "key": "028D",
      "mappings": {"default": {"default": "girada w"}}
    }, {"category": "Ll", "key": "028E", "mappings": {"default": {"default": "vuelta y"}}}, {
      "category": "Ll",
      "key": "0290",
      "mappings": {"default": {"default": "Z con gancho retroflex"}}
    }, {"category": "Ll", "key": "0291", "mappings": {"default": {"default": "z con rizo"}}}, {
      "category": "Ll",
      "key": "0295",
      "mappings": {"default": {"default": "faríngea con voz fricativa"}}
    }, {
      "category": "Ll",
      "key": "0296",
      "mappings": {"default": {"default": "invertida parada glotal"}}
    }, {"category": "Ll", "key": "0297", "mappings": {"default": {"default": "estirada c"}}}, {
      "category": "Ll",
      "key": "0298",
      "mappings": {"default": {"default": "Carta Bilabial Click"}}
    }, {"category": "Ll", "key": "029A", "mappings": {"default": {"default": "cerrada abierta e"}}}, {
      "category": "Ll",
      "key": "029E",
      "mappings": {"default": {"default": "vuelta k"}}
    }, {"category": "Ll", "key": "02A0", "mappings": {"default": {"default": "q con gancho"}}}, {
      "category": "Ll",
      "key": "02A3",
      "mappings": {"default": {"default": "pequeña letra Dz Digraph"}}
    }, {
      "category": "Ll",
      "key": "02A5",
      "mappings": {"default": {"default": "Letra pequeña latina Dz Digraph con Curl"}}
    }, {
      "category": "Ll",
      "key": "02A6",
      "mappings": {"default": {"default": "Latín letra pequeña Ts Digraph"}}
    }, {
      "category": "Ll",
      "key": "02A7",
      "mappings": {"default": {"default": "Letra pequeña latina Tesh Digraph"}}
    }, {
      "category": "Ll",
      "key": "02A8",
      "mappings": {"default": {"default": "Latín letra pequeña Tc Digraph con Curl"}}
    }, {
      "category": "Ll",
      "key": "02A9",
      "mappings": {"default": {"default": "Letra pequeña latina Feng Digraph"}}
    }, {
      "category": "Ll",
      "key": "02AA",
      "mappings": {"default": {"default": "pequeña letra Ls Digraph"}}
    }, {
      "category": "Ll",
      "key": "02AB",
      "mappings": {"default": {"default": "pequeña letra Lz Digraph"}}
    }, {"category": "Ll", "key": "02AC", "mappings": {"default": {"default": "percusion bilabial"}}}, {
      "category": "Ll",
      "key": "02AD",
      "mappings": {"default": {"default": "de percusión bidental"}}
    }, {
      "category": "Ll",
      "key": "02AE",
      "mappings": {"default": {"default": "convertida en H con anzuelo"}}
    }, {
      "category": "Ll",
      "key": "02AF",
      "mappings": {"default": {"default": "convertida en H con anzuelo y cola"}}
    }, {"category": "Ll", "key": "1D02", "mappings": {"default": {"default": "vuelta ae"}}}, {
      "category": "Ll",
      "key": "1D08",
      "mappings": {"default": {"default": "convertida abierta E"}}
    }, {"category": "Ll", "key": "1D09", "mappings": {"default": {"default": "convertida i"}}}, {
      "category": "Ll",
      "key": "1D11",
      "mappings": {"default": {"default": "de lado O"}}
    }, {"category": "Ll", "key": "1D12", "mappings": {"default": {"default": "de lado abierto O"}}}, {
      "category": "Ll",
      "key": "1D14",
      "mappings": {"default": {"default": "vuelta oe"}}
    }, {"category": "Ll", "key": "1D16", "mappings": {"default": {"default": "mitad superior O"}}}, {
      "category": "Ll",
      "key": "1D17",
      "mappings": {"default": {"default": "Letra latina, letra inferior, mitad inferior O"}}
    }, {"category": "Ll", "key": "1D1D", "mappings": {"default": {"default": "hacia los lados U"}}}, {
      "category": "Ll",
      "key": "1D1E",
      "mappings": {"default": {"default": "de lado diaeresizada U"}}
    }, {
      "category": "Ll",
      "key": "1D1F",
      "mappings": {"default": {"default": "girada hacia los lados M"}}
    }, {
      "category": "Ll",
      "key": "1D24",
      "mappings": {"default": {"default": "Carta latina expresada espirante laríngeo"}}
    }, {"category": "Ll", "key": "1D25", "mappings": {"default": {"default": "ain"}}}, {
      "category": "Ll",
      "key": "1D6B",
      "mappings": {"default": {"default": "ue"}}
    }, {"category": "Ll", "key": "1D6C", "mappings": {"default": {"default": "B con tilde medio"}}}, {
      "category": "Ll",
      "key": "1D6D",
      "mappings": {"default": {"default": "D con tilde medio"}}
    }, {"category": "Ll", "key": "1D6E", "mappings": {"default": {"default": "F con tilde medio"}}}, {
      "category": "Ll",
      "key": "1D6F",
      "mappings": {"default": {"default": "M con tilde medio"}}
    }, {"category": "Ll", "key": "1D70", "mappings": {"default": {"default": "N con tilde medio"}}}, {
      "category": "Ll",
      "key": "1D71",
      "mappings": {"default": {"default": "P con tilde medio"}}
    }, {"category": "Ll", "key": "1D72", "mappings": {"default": {"default": "R con tilde medio"}}}, {
      "category": "Ll",
      "key": "1D73",
      "mappings": {"default": {"default": "R con anzuelo y tilde medio"}}
    }, {"category": "Ll", "key": "1D74", "mappings": {"default": {"default": "S con tilde medio"}}}, {
      "category": "Ll",
      "key": "1D75",
      "mappings": {"default": {"default": "T con tilde medio"}}
    }, {"category": "Ll", "key": "1D76", "mappings": {"default": {"default": "z con tilde media"}}}, {
      "category": "Ll",
      "key": "1D77",
      "mappings": {"default": {"default": "girada g"}}
    }, {"category": "Ll", "key": "1D79", "mappings": {"default": {"default": "insular g"}}}, {
      "category": "Ll",
      "key": "1D7A",
      "mappings": {"default": {"default": "th con tachado"}}
    }, {
      "category": "Ll",
      "key": "1D80",
      "mappings": {"default": {"default": "B con gancho palatal"}}
    }, {
      "category": "Ll",
      "key": "1D81",
      "mappings": {"default": {"default": "D con gancho palatal"}}
    }, {
      "category": "Ll",
      "key": "1D82",
      "mappings": {"default": {"default": "F con gancho palatino"}}
    }, {
      "category": "Ll",
      "key": "1D83",
      "mappings": {"default": {"default": "G con gancho palatal"}}
    }, {
      "category": "Ll",
      "key": "1D84",
      "mappings": {"default": {"default": "K con gancho palatino"}}
    }, {
      "category": "Ll",
      "key": "1D85",
      "mappings": {"default": {"default": "L con gancho palatino"}}
    }, {
      "category": "Ll",
      "key": "1D86",
      "mappings": {"default": {"default": "M con gancho palatino"}}
    }, {
      "category": "Ll",
      "key": "1D87",
      "mappings": {"default": {"default": "N con gancho palatal"}}
    }, {
      "category": "Ll",
      "key": "1D88",
      "mappings": {"default": {"default": "P con gancho palatino"}}
    }, {
      "category": "Ll",
      "key": "1D89",
      "mappings": {"default": {"default": "R con gancho palatal"}}
    }, {
      "category": "Ll",
      "key": "1D8A",
      "mappings": {"default": {"default": "S con gancho palatal"}}
    }, {
      "category": "Ll",
      "key": "1D8B",
      "mappings": {"default": {"default": "Pequeña letra latina Esh con gancho palatal"}}
    }, {
      "category": "Ll",
      "key": "1D8C",
      "mappings": {"default": {"default": "V con gancho palatal"}}
    }, {
      "category": "Ll",
      "key": "1D8D",
      "mappings": {"default": {"default": "X con gancho palatal"}}
    }, {
      "category": "Ll",
      "key": "1D8E",
      "mappings": {"default": {"default": "Z con gancho palatal"}}
    }, {
      "category": "Ll",
      "key": "1D8F",
      "mappings": {"default": {"default": "a con gancho retroflex"}}
    }, {
      "category": "Ll",
      "key": "1D90",
      "mappings": {"default": {"default": "Alfa latina pequeña letra con gancho retroflex"}}
    }, {
      "category": "Ll",
      "key": "1D91",
      "mappings": {"default": {"default": "D con gancho y cola"}}
    }, {
      "category": "Ll",
      "key": "1D92",
      "mappings": {"default": {"default": "pequeña E con gancho retroflex"}}
    }, {
      "category": "Ll",
      "key": "1D93",
      "mappings": {"default": {"default": "E Pequeña Open con Gancho Retroflex"}}
    }, {
      "category": "Ll",
      "key": "1D94",
      "mappings": {"default": {"default": "E abierta invertido con gancho retroflex"}}
    }, {
      "category": "Ll",
      "key": "1D95",
      "mappings": {"default": {"default": "pequeña Schwa con gancho retroflex"}}
    }, {
      "category": "Ll",
      "key": "1D96",
      "mappings": {"default": {"default": "I con gancho retroflex"}}
    }, {
      "category": "Ll",
      "key": "1D97",
      "mappings": {"default": {"default": "pequeña latina Open O con gancho retroflex"}}
    }, {
      "category": "Ll",
      "key": "1D98",
      "mappings": {"default": {"default": "Esh de letra pequeña latina con gancho retroflex"}}
    }, {
      "category": "Ll",
      "key": "1D99",
      "mappings": {"default": {"default": "U con gancho retroflex"}}
    }, {
      "category": "Ll",
      "key": "1D9A",
      "mappings": {"default": {"default": "pequeña Ezh con gancho retroflex"}}
    }, {
      "key": "0149",
      "mappings": {"default": {"default": "n precedida por apóstrofe"}},
      "category": "Ll"
    }, {"key": "014B", "mappings": {"default": {"default": "eng"}}, "category": "Ll"}]
  ,
  "es/symbols/latin-lower-single-accent.js":
    [{"locale": "es"}, {
      "key": "00E0",
      "mappings": {"default": {"default": "a grave"}},
      "category": "Ll"
    }, {"key": "00E1", "mappings": {"default": {"default": "a aguda"}}, "category": "Ll"}, {
      "key": "00E2",
      "mappings": {"default": {"default": "a acento circunflejo"}},
      "category": "Ll"
    }, {"key": "00E3", "mappings": {"default": {"default": "a con tilde"}}, "category": "Ll"}, {
      "key": "00E4",
      "mappings": {"default": {"default": "a diéresis"}},
      "category": "Ll"
    }, {"key": "00E5", "mappings": {"default": {"default": "a con anillo"}}, "category": "Ll"}, {
      "key": "00E7",
      "mappings": {"default": {"default": "c cedilla"}},
      "category": "Ll"
    }, {"key": "00E8", "mappings": {"default": {"default": "e grave"}}, "category": "Ll"}, {
      "key": "00E9",
      "mappings": {"default": {"default": "e aguda"}},
      "category": "Ll"
    }, {"key": "00EA", "mappings": {"default": {"default": "e acento circunflejo"}}, "category": "Ll"}, {
      "key": "00EB",
      "mappings": {"default": {"default": "e diéresis"}},
      "category": "Ll"
    }, {"key": "00EC", "mappings": {"default": {"default": "i grave"}}, "category": "Ll"}, {
      "key": "00ED",
      "mappings": {"default": {"default": "i aguda"}},
      "category": "Ll"
    }, {"key": "00EE", "mappings": {"default": {"default": "i acento circunflejo"}}, "category": "Ll"}, {
      "key": "00EF",
      "mappings": {"default": {"default": "i diéresis"}},
      "category": "Ll"
    }, {"key": "00F1", "mappings": {"default": {"default": "eñe"}}, "category": "Ll"}, {
      "key": "00F2",
      "mappings": {"default": {"default": "o grave"}},
      "category": "Ll"
    }, {"key": "00F3", "mappings": {"default": {"default": "o aguda"}}, "category": "Ll"}, {
      "key": "00F4",
      "mappings": {"default": {"default": "o acento circunflejo"}},
      "category": "Ll"
    }, {"key": "00F5", "mappings": {"default": {"default": "o con tilde"}}, "category": "Ll"}, {
      "key": "00F6",
      "mappings": {"default": {"default": "o diéresis"}},
      "category": "Ll"
    }, {"key": "00F9", "mappings": {"default": {"default": "u grave"}}, "category": "Ll"}, {
      "key": "00FA",
      "mappings": {"default": {"default": "u aguda"}},
      "category": "Ll"
    }, {"key": "00FB", "mappings": {"default": {"default": "u acento circunflejo"}}, "category": "Ll"}, {
      "key": "00FC",
      "mappings": {"default": {"default": "u diéresis"}},
      "category": "Ll"
    }, {
      "key": "00FD",
      "mappings": {"default": {"default": "Y aguda", "defaultMP": "ye aguda"}},
      "category": "Ll"
    }, {
      "key": "00FF",
      "mappings": {"default": {"default": "y diéresis", "defaultMP": "ye diéresis"}},
      "category": "Ll"
    }, {"key": "0101", "mappings": {"default": {"default": "a barra"}}, "category": "Ll"}, {
      "key": "0103",
      "mappings": {"default": {"default": "a con breve"}},
      "category": "Ll"
    }, {"key": "0105", "mappings": {"default": {"default": "a con ogonek"}}, "category": "Ll"}, {
      "key": "0107",
      "mappings": {"default": {"default": "c con agudo"}},
      "category": "Ll"
    }, {
      "key": "0109",
      "mappings": {"default": {"default": "c con acento circunflejo"}},
      "category": "Ll"
    }, {"key": "010B", "mappings": {"default": {"default": "c con punto arriba"}}, "category": "Ll"}, {
      "key": "010D",
      "mappings": {"default": {"default": "c con carón"}},
      "category": "Ll"
    }, {"key": "010F", "mappings": {"default": {"default": "d con carón"}}, "category": "Ll"}, {
      "key": "0113",
      "mappings": {"default": {"default": "e barra"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0115", "mappings": {"default": {"default": "e con breve"}}}, {
      "key": "0117",
      "mappings": {"default": {"default": "e con punto arriba"}},
      "category": "Ll"
    }, {"key": "0119", "mappings": {"default": {"default": "e con ogonek"}}, "category": "Ll"}, {
      "key": "011B",
      "mappings": {"default": {"default": "e con carón"}},
      "category": "Ll"
    }, {
      "key": "011D",
      "mappings": {"default": {"default": "g con acento circunflejo"}},
      "category": "Ll"
    }, {"key": "011F", "mappings": {"default": {"default": "g con breve"}}, "category": "Ll"}, {
      "key": "0121",
      "mappings": {"default": {"default": "g con punto arriba"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0123", "mappings": {"default": {"default": "g con cedilla"}}}, {
      "key": "0125",
      "mappings": {"default": {"default": "h con acento circunflejo"}},
      "category": "Ll"
    }, {"key": "0129", "mappings": {"default": {"default": "i con tilde"}}, "category": "Ll"}, {
      "key": "012B",
      "mappings": {"default": {"default": "i barra"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "012D", "mappings": {"default": {"default": "i con breve"}}}, {
      "key": "012F",
      "mappings": {"default": {"default": "i con ogonek"}},
      "category": "Ll"
    }, {"key": "0131", "mappings": {"default": {"default": "i sin punto"}}, "category": "Ll"}, {
      "key": "0135",
      "mappings": {"default": {"default": "j con acento circunflejo"}},
      "category": "Ll"
    }, {"key": "0137", "mappings": {"default": {"default": "k con cedilla"}}, "category": "Ll"}, {
      "key": "013A",
      "mappings": {"default": {"default": "l con agudo"}},
      "category": "Ll"
    }, {"key": "013C", "mappings": {"default": {"default": "l con cedilla"}}, "category": "Ll"}, {
      "key": "013E",
      "mappings": {"default": {"default": "l con carón"}},
      "category": "Ll"
    }, {"key": "0140", "mappings": {"default": {"default": "l con punto mediano"}}, "category": "Ll"}, {
      "key": "0144",
      "mappings": {"default": {"default": "n con agudo"}},
      "category": "Ll"
    }, {"key": "0146", "mappings": {"default": {"default": "n con cedilla"}}, "category": "Ll"}, {
      "key": "0148",
      "mappings": {"default": {"default": "n con carón"}},
      "category": "Ll"
    }, {"key": "014D", "mappings": {"default": {"default": "o barra"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "014F",
      "mappings": {"default": {"default": "o con breve"}}
    }, {
      "key": "0151",
      "mappings": {"default": {"default": "o con doble acento agudo"}},
      "category": "Ll"
    }, {"key": "0155", "mappings": {"default": {"default": "r con agudo"}}, "category": "Ll"}, {
      "key": "0157",
      "mappings": {"default": {"default": "r con cedilla"}},
      "category": "Ll"
    }, {"key": "0159", "mappings": {"default": {"default": "r con carón"}}, "category": "Ll"}, {
      "key": "015B",
      "mappings": {"default": {"default": "s con agudo"}},
      "category": "Ll"
    }, {
      "key": "015D",
      "mappings": {"default": {"default": "s con acento circunflejo"}},
      "category": "Ll"
    }, {"key": "015F", "mappings": {"default": {"default": "s con cedilla"}}, "category": "Ll"}, {
      "key": "0161",
      "mappings": {"default": {"default": "s con carón"}},
      "category": "Ll"
    }, {"key": "0163", "mappings": {"default": {"default": "t con cedilla"}}, "category": "Ll"}, {
      "key": "0165",
      "mappings": {"default": {"default": "t con carón"}},
      "category": "Ll"
    }, {"key": "0169", "mappings": {"default": {"default": "u con tilde"}}, "category": "Ll"}, {
      "key": "016B",
      "mappings": {"default": {"default": "u barra"}},
      "category": "Ll"
    }, {"key": "016D", "mappings": {"default": {"default": "u con breve"}}, "category": "Ll"}, {
      "key": "016F",
      "mappings": {"default": {"default": "u con anillo arriba"}},
      "category": "Ll"
    }, {
      "key": "0171",
      "mappings": {"default": {"default": "u con doble acento agudo"}},
      "category": "Ll"
    }, {"key": "0173", "mappings": {"default": {"default": "u con ogonek"}}, "category": "Ll"}, {
      "key": "0175",
      "mappings": {"default": {"default": "w con circunflejo"}},
      "category": "Ll"
    }, {"key": "0177", "mappings": {"default": {"default": "y con circunflejo"}}, "category": "Ll"}, {
      "key": "017A",
      "mappings": {"default": {"default": "z con agudo"}},
      "category": "Ll"
    }, {"key": "017C", "mappings": {"default": {"default": "z con punto"}}, "category": "Ll"}, {
      "key": "017E",
      "mappings": {"default": {"default": "z con carón"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "01CE", "mappings": {"default": {"default": "a con acento"}}}, {
      "category": "Ll",
      "key": "01D0",
      "mappings": {"default": {"default": "i con acento"}}
    }, {"category": "Ll", "key": "01D2", "mappings": {"default": {"default": "o con acento"}}}, {
      "category": "Ll",
      "key": "01D4",
      "mappings": {"default": {"default": "u con acento"}}
    }, {"category": "Ll", "key": "01E7", "mappings": {"default": {"default": "g con acento"}}}, {
      "category": "Ll",
      "key": "01E9",
      "mappings": {"default": {"default": "k con acento"}}
    }, {"category": "Ll", "key": "01EB", "mappings": {"default": {"default": "o con pecíolo"}}}, {
      "category": "Ll",
      "key": "01F0",
      "mappings": {"default": {"default": "j con acento"}}
    }, {"key": "01F5", "mappings": {"default": {"default": "g con agudo"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "01F9",
      "mappings": {"default": {"default": "n con grave"}}
    }, {"category": "Ll", "key": "0201", "mappings": {"default": {"default": "a con doble tumba"}}}, {
      "category": "Ll",
      "key": "0203",
      "mappings": {"default": {"default": "a con letras invertidas"}}
    }, {"category": "Ll", "key": "0205", "mappings": {"default": {"default": "e con doble tumba"}}}, {
      "category": "Ll",
      "key": "0207",
      "mappings": {"default": {"default": "e con letras invertidas"}}
    }, {"category": "Ll", "key": "0209", "mappings": {"default": {"default": "i con doble tumba"}}}, {
      "category": "Ll",
      "key": "020B",
      "mappings": {"default": {"default": "i con Breve invertida"}}
    }, {"category": "Ll", "key": "020D", "mappings": {"default": {"default": "o con doble tumba"}}}, {
      "category": "Ll",
      "key": "020F",
      "mappings": {"default": {"default": "o con letras invertidas"}}
    }, {
      "category": "Ll",
      "key": "0211",
      "mappings": {"default": {"default": "r con doble sepulcro"}}
    }, {
      "category": "Ll",
      "key": "0213",
      "mappings": {"default": {"default": "r con letras invertidas"}}
    }, {"category": "Ll", "key": "0215", "mappings": {"default": {"default": "u con doble tumba"}}}, {
      "category": "Ll",
      "key": "0217",
      "mappings": {"default": {"default": "u con letras invertidas"}}
    }, {"category": "Ll", "key": "0219", "mappings": {"default": {"default": "s con coma debajo"}}}, {
      "category": "Ll",
      "key": "021B",
      "mappings": {"default": {"default": "t con coma debajo"}}
    }, {"category": "Ll", "key": "021F", "mappings": {"default": {"default": "h con caron"}}}, {
      "category": "Ll",
      "key": "0227",
      "mappings": {"default": {"default": "con un punto arriba"}}
    }, {"category": "Ll", "key": "0229", "mappings": {"default": {"default": "e con cedilla"}}}, {
      "category": "Ll",
      "key": "022F",
      "mappings": {"default": {"default": "o con punto arriba"}}
    }, {"category": "Ll", "key": "0233", "mappings": {"default": {"default": "y con Macron"}}}, {
      "category": "Ll",
      "key": "0237",
      "mappings": {"default": {"default": "s in punto j"}}
    }, {"key": "1E01", "mappings": {"default": {"default": "a con anillo debajo"}}, "category": "Ll"}, {
      "key": "1E03",
      "mappings": {"default": {"default": "b con punto arriba"}},
      "category": "Ll"
    }, {"key": "1E05", "mappings": {"default": {"default": "b con punto debajo"}}, "category": "Ll"}, {
      "key": "1E07",
      "mappings": {"default": {"default": "b con línea debajo"}},
      "category": "Ll"
    }, {"key": "1E0B", "mappings": {"default": {"default": "d con punto arriba"}}, "category": "Ll"}, {
      "key": "1E0D",
      "mappings": {"default": {"default": "d con punto debajo"}},
      "category": "Ll"
    }, {"key": "1E0F", "mappings": {"default": {"default": "d con línea debajo"}}, "category": "Ll"}, {
      "key": "1E11",
      "mappings": {"default": {"default": "d con cedilla"}},
      "category": "Ll"
    }, {
      "key": "1E13",
      "mappings": {"default": {"default": "d con acento circunflejo debajo"}},
      "category": "Ll"
    }, {
      "key": "1E19",
      "mappings": {"default": {"default": "e con acento circunflejo debajo"}},
      "category": "Ll"
    }, {"key": "1E1B", "mappings": {"default": {"default": "e con tilde debajo"}}, "category": "Ll"}, {
      "key": "1E1F",
      "mappings": {"default": {"default": "f con punto arriba"}},
      "category": "Ll"
    }, {"key": "1E21", "mappings": {"default": {"default": "g con macrón"}}, "category": "Ll"}, {
      "key": "1E23",
      "mappings": {"default": {"default": "h con punto arriba"}},
      "category": "Ll"
    }, {"key": "1E25", "mappings": {"default": {"default": "h con punto debajo"}}, "category": "Ll"}, {
      "key": "1E27",
      "mappings": {"default": {"default": "h con diéresis"}},
      "category": "Ll"
    }, {"key": "1E29", "mappings": {"default": {"default": "h con cedilla"}}, "category": "Ll"}, {
      "key": "1E2B",
      "mappings": {"default": {"default": "h con breve debajo"}},
      "category": "Ll"
    }, {"key": "1E2D", "mappings": {"default": {"default": "i con tilde debajo"}}, "category": "Ll"}, {
      "key": "1E31",
      "mappings": {"default": {"default": "k con agudo"}},
      "category": "Ll"
    }, {"key": "1E33", "mappings": {"default": {"default": "k con punto debajo"}}, "category": "Ll"}, {
      "key": "1E35",
      "mappings": {"default": {"default": "k con línea debajo"}},
      "category": "Ll"
    }, {"key": "1E37", "mappings": {"default": {"default": "l con punto debajo"}}, "category": "Ll"}, {
      "key": "1E3B",
      "mappings": {"default": {"default": "l con línea debajo"}},
      "category": "Ll"
    }, {
      "key": "1E3D",
      "mappings": {"default": {"default": "l con acento circunflejo debajo"}},
      "category": "Ll"
    }, {"key": "1E3F", "mappings": {"default": {"default": "m con agudo"}}, "category": "Ll"}, {
      "key": "1E41",
      "mappings": {"default": {"default": "m con punto arriba"}},
      "category": "Ll"
    }, {"key": "1E43", "mappings": {"default": {"default": "m con punto debajo"}}, "category": "Ll"}, {
      "key": "1E45",
      "mappings": {"default": {"default": "n con punto arriba"}},
      "category": "Ll"
    }, {"key": "1E47", "mappings": {"default": {"default": "n con punto debajo"}}, "category": "Ll"}, {
      "key": "1E49",
      "mappings": {"default": {"default": "n con línea debajo"}},
      "category": "Ll"
    }, {
      "key": "1E4B",
      "mappings": {"default": {"default": "n con acento circunflejo debajo"}},
      "category": "Ll"
    }, {"key": "1E55", "mappings": {"default": {"default": "p con agudo"}}, "category": "Ll"}, {
      "key": "1E57",
      "mappings": {"default": {"default": "p con punto arriba"}},
      "category": "Ll"
    }, {"key": "1E59", "mappings": {"default": {"default": "r con punto arriba"}}, "category": "Ll"}, {
      "key": "1E5B",
      "mappings": {"default": {"default": "r con punto debajo"}},
      "category": "Ll"
    }, {"key": "1E5F", "mappings": {"default": {"default": "r con línea debajo"}}, "category": "Ll"}, {
      "key": "1E61",
      "mappings": {"default": {"default": "s con punto arriba"}},
      "category": "Ll"
    }, {"key": "1E63", "mappings": {"default": {"default": "s con punto debajo"}}, "category": "Ll"}, {
      "key": "1E6B",
      "mappings": {"default": {"default": "t con punto arriba"}},
      "category": "Ll"
    }, {"key": "1E6D", "mappings": {"default": {"default": "t con punto debajo"}}, "category": "Ll"}, {
      "key": "1E6F",
      "mappings": {"default": {"default": "t con línea debajo"}},
      "category": "Ll"
    }, {
      "key": "1E71",
      "mappings": {"default": {"default": "t con acento circunflejo debajo"}},
      "category": "Ll"
    }, {"key": "1E73", "mappings": {"default": {"default": "u con diéresis debajo"}}, "category": "Ll"}, {
      "key": "1E75",
      "mappings": {"default": {"default": "u con tilde debajo"}},
      "category": "Ll"
    }, {
      "key": "1E77",
      "mappings": {"default": {"default": "u con acento circunflejo debajo"}},
      "category": "Ll"
    }, {"key": "1E7D", "mappings": {"default": {"default": "v con tilde"}}, "category": "Ll"}, {
      "key": "1E7F",
      "mappings": {"default": {"default": "v con punto debajo"}},
      "category": "Ll"
    }, {"key": "1E81", "mappings": {"default": {"default": "w con grave"}}, "category": "Ll"}, {
      "key": "1E83",
      "mappings": {"default": {"default": "w con agudo"}},
      "category": "Ll"
    }, {"key": "1E85", "mappings": {"default": {"default": "w con diéresis"}}, "category": "Ll"}, {
      "key": "1E87",
      "mappings": {"default": {"default": "w con punto arriba"}},
      "category": "Ll"
    }, {"key": "1E89", "mappings": {"default": {"default": "w con punto debajo"}}, "category": "Ll"}, {
      "key": "1E8B",
      "mappings": {"default": {"default": "x con punto arriba"}},
      "category": "Ll"
    }, {"key": "1E8D", "mappings": {"default": {"default": "x con diéresis"}}, "category": "Ll"}, {
      "key": "1E8F",
      "mappings": {"default": {"default": "y con punto arriba"}},
      "category": "Ll"
    }, {
      "key": "1E91",
      "mappings": {"default": {"default": "z con acento circunflejo"}},
      "category": "Ll"
    }, {"key": "1E93", "mappings": {"default": {"default": "z con punto debajo"}}, "category": "Ll"}, {
      "key": "1E95",
      "mappings": {"default": {"default": "z con línea debajo"}},
      "category": "Ll"
    }, {"key": "1E96", "mappings": {"default": {"default": "h con línea debajo"}}, "category": "Ll"}, {
      "key": "1E97",
      "mappings": {"default": {"default": "t con diéresis"}},
      "category": "Ll"
    }, {"key": "1E98", "mappings": {"default": {"default": "w con anillo arriba"}}, "category": "Ll"}, {
      "key": "1E99",
      "mappings": {"default": {"default": "y con anillo arriba"}},
      "category": "Ll"
    }, {
      "key": "1E9A",
      "mappings": {"default": {"default": "a con medio derecho de anillo"}},
      "category": "Ll"
    }, {"key": "1EA1", "mappings": {"default": {"default": "a con punto debajo"}}, "category": "Ll"}, {
      "key": "1EA3",
      "mappings": {"default": {"default": "a con gancho arriba"}},
      "category": "Ll"
    }, {"key": "1EB9", "mappings": {"default": {"default": "e con punto debajo"}}, "category": "Ll"}, {
      "key": "1EBB",
      "mappings": {"default": {"default": "e con gancho arriba"}},
      "category": "Ll"
    }, {"key": "1EBD", "mappings": {"default": {"default": "e con tilde"}}, "category": "Ll"}, {
      "key": "1EC9",
      "mappings": {"default": {"default": "i con gancho arriba"}},
      "category": "Ll"
    }, {"key": "1ECB", "mappings": {"default": {"default": "i con punto debajo"}}, "category": "Ll"}, {
      "key": "1ECD",
      "mappings": {"default": {"default": "o con punto debajo"}},
      "category": "Ll"
    }, {"key": "1ECF", "mappings": {"default": {"default": "o con gancho arriba"}}, "category": "Ll"}, {
      "key": "1EE5",
      "mappings": {"default": {"default": "u con punto debajo"}},
      "category": "Ll"
    }, {"key": "1EE7", "mappings": {"default": {"default": "u con gancho arriba"}}, "category": "Ll"}, {
      "key": "1EF3",
      "mappings": {"default": {"default": "y con grave"}},
      "category": "Ll"
    }, {"key": "1EF5", "mappings": {"default": {"default": "y con punto debajo"}}, "category": "Ll"}, {
      "key": "1EF7",
      "mappings": {"default": {"default": "y con gancho arriba"}},
      "category": "Ll"
    }, {"key": "1EF9", "mappings": {"default": {"default": "y con tilde"}}, "category": "Ll"}]
  ,
  "es/symbols/latin-rest.js":
    [{"locale": "es"}, {
      "category": "Ll",
      "key": "210E",
      "mappings": {"default": {"default": "constante de planck"}}
    }, {"category": "Mn", "key": "0363", "mappings": {"default": {"default": "combinando a"}}}, {
      "category": "Mn",
      "key": "0364",
      "mappings": {"default": {"default": "combinando e"}}
    }, {"category": "Mn", "key": "0365", "mappings": {"default": {"default": "combinando i"}}}, {
      "category": "Mn",
      "key": "0366",
      "mappings": {"default": {"default": "combinando o"}}
    }, {"category": "Mn", "key": "0367", "mappings": {"default": {"default": "combinando u"}}}, {
      "category": "Mn",
      "key": "0368",
      "mappings": {"default": {"default": "combinando c"}}
    }, {"category": "Mn", "key": "0369", "mappings": {"default": {"default": "combinando d"}}}, {
      "category": "Mn",
      "key": "036A",
      "mappings": {"default": {"default": "combinando h"}}
    }, {"category": "Mn", "key": "036B", "mappings": {"default": {"default": "combinando m"}}}, {
      "category": "Mn",
      "key": "036C",
      "mappings": {"default": {"default": "combinando r"}}
    }, {"category": "Mn", "key": "036D", "mappings": {"default": {"default": "combinando t"}}}, {
      "category": "Mn",
      "key": "036E",
      "mappings": {"default": {"default": "combinando v"}}
    }, {"category": "Mn", "key": "036F", "mappings": {"default": {"default": "combinando x"}}}, {
      "category": "Lm",
      "key": "1D62",
      "mappings": {"default": {"default": "subíndice i"}}
    }, {"category": "Lm", "key": "1D63", "mappings": {"default": {"default": "subíndice r"}}}, {
      "category": "Lm",
      "key": "1D64",
      "mappings": {"default": {"default": "subíndice u"}}
    }, {
      "category": "Lm",
      "key": "1D65",
      "mappings": {"default": {"default": "subíndice latino letra pequeña V"}}
    }, {"category": "Mn", "key": "1DCA", "mappings": {"default": {"default": "combinando r abajo"}}}, {
      "category": "Mn",
      "key": "1DD3",
      "mappings": {"default": {"default": "combinando letra minúscula en latín, aplanado, abierto arriba"}}
    }, {"category": "Mn", "key": "1DD4", "mappings": {"default": {"default": "combinando ae"}}}, {
      "category": "Mn",
      "key": "1DD5",
      "mappings": {"default": {"default": "combinando ao"}}
    }, {"category": "Mn", "key": "1DD6", "mappings": {"default": {"default": "combinando av"}}}, {
      "category": "Mn",
      "key": "1DD7",
      "mappings": {"default": {"default": "combinando Letra Pequeña Latina C Cedilla"}}
    }, {
      "category": "Mn",
      "key": "1DD8",
      "mappings": {"default": {"default": "combinando insular d"}}
    }, {"category": "Mn", "key": "1DD9", "mappings": {"default": {"default": "combinando la eth"}}}, {
      "category": "Mn",
      "key": "1DDA",
      "mappings": {"default": {"default": "combinando g"}}
    }, {
      "category": "Mn",
      "key": "1DDB",
      "mappings": {"default": {"default": "combinando pequeña mayúscula G"}}
    }, {"category": "Mn", "key": "1DDC", "mappings": {"default": {"default": "combinando k"}}}, {
      "category": "Mn",
      "key": "1DDD",
      "mappings": {"default": {"default": "combinando l"}}
    }, {
      "category": "Mn",
      "key": "1DDE",
      "mappings": {"default": {"default": "combinando pequeña mayúscula L"}}
    }, {
      "category": "Mn",
      "key": "1DDF",
      "mappings": {"default": {"default": "combinando pequeña mayúscula M"}}
    }, {"category": "Mn", "key": "1DE0", "mappings": {"default": {"default": "combinando n"}}}, {
      "category": "Mn",
      "key": "1DE1",
      "mappings": {"default": {"default": "combinando pequeña mayúscula N"}}
    }, {
      "category": "Mn",
      "key": "1DE2",
      "mappings": {"default": {"default": "combinando pequeña mayúscula R"}}
    }, {
      "category": "Mn",
      "key": "1DE3",
      "mappings": {"default": {"default": "combinando pequeña r rotunda"}}
    }, {"category": "Mn", "key": "1DE4", "mappings": {"default": {"default": "combinando s"}}}, {
      "category": "Mn",
      "key": "1DE5",
      "mappings": {"default": {"default": "combinando larga s"}}
    }, {"category": "Mn", "key": "1DE6", "mappings": {"default": {"default": "combinando z"}}}, {
      "category": "Lm",
      "key": "2071",
      "mappings": {"default": {"default": "superíndice i"}}
    }, {"category": "Lm", "key": "207F", "mappings": {"default": {"default": "superíndice n"}}}, {
      "category": "Lm",
      "key": "2090",
      "mappings": {"default": {"default": "subíndice a"}}
    }, {"category": "Lm", "key": "2091", "mappings": {"default": {"default": "subíndice e"}}}, {
      "category": "Lm",
      "key": "2092",
      "mappings": {"default": {"default": "subíndice o"}}
    }, {"category": "Lm", "key": "2093", "mappings": {"default": {"default": "subíndice x"}}}, {
      "category": "Lm",
      "key": "2094",
      "mappings": {"default": {"default": "subíndice schwa"}}
    }, {"category": "Lm", "key": "2095", "mappings": {"default": {"default": "subíndice h"}}}, {
      "category": "Lm",
      "key": "2096",
      "mappings": {"default": {"default": "subíndice k"}}
    }, {"category": "Lm", "key": "2097", "mappings": {"default": {"default": "subíndice l"}}}, {
      "category": "Lm",
      "key": "2098",
      "mappings": {"default": {"default": "subíndice m"}}
    }, {"category": "Lm", "key": "2099", "mappings": {"default": {"default": "subíndice n"}}}, {
      "category": "Lm",
      "key": "209A",
      "mappings": {"default": {"default": "subíndice p"}}
    }, {"category": "Lm", "key": "209B", "mappings": {"default": {"default": "subíndice s"}}}, {
      "category": "Lm",
      "key": "209C",
      "mappings": {"default": {"default": "subíndice t"}}
    }, {"category": "Lm", "key": "2C7C", "mappings": {"default": {"default": "subíndice j"}}}, {
      "category": "So",
      "key": "1F12A",
      "mappings": {"default": {"default": "concha de tortuga con corchetes mayúscula S"}}
    }, {
      "category": "So",
      "key": "1F12B",
      "mappings": {"default": {"default": "círculo itálico mayúscula C"}}
    }, {
      "category": "So",
      "key": "1F12C",
      "mappings": {"default": {"default": "círculo itálico mayúscula R"}}
    }, {
      "category": "So",
      "key": "1F18A",
      "mappings": {"default": {"default": "cruzada negativa cuadrada Mayúscula P"}}
    }]
  ,
  "es/symbols/latin-upper-double-accent.js":
    [{"locale": "es"}, {
      "category": "Lu",
      "key": "01D5",
      "mappings": {"default": {"default": "mayúscula U con diéresis y macron"}}
    }, {
      "category": "Lu",
      "key": "01D7",
      "mappings": {"default": {"default": "mayúscula U con diéresis y acento agudo"}}
    }, {
      "category": "Lu",
      "key": "01D9",
      "mappings": {"default": {"default": "mayúscula U con diéresis y acento"}}
    }, {
      "category": "Lu",
      "key": "01DB",
      "mappings": {"default": {"default": "mayúscula U con diéresis y grave"}}
    }, {
      "category": "Lu",
      "key": "01DE",
      "mappings": {"default": {"default": "mayúscula A con diéresis y macron"}}
    }, {
      "category": "Lu",
      "key": "01E0",
      "mappings": {"default": {"default": "mayúscula A con punto arriba y macron"}}
    }, {
      "category": "Lu",
      "key": "01EC",
      "mappings": {"default": {"default": "mayúscula O con pecíolo y macron"}}
    }, {
      "category": "Lu",
      "key": "01FA",
      "mappings": {"default": {"default": "mayúscula A con ring above y acento agudo"}}
    }, {
      "category": "Lu",
      "key": "022A",
      "mappings": {"default": {"default": "mayúscula O con diaéresis y macron"}}
    }, {
      "category": "Lu",
      "key": "022C",
      "mappings": {"default": {"default": "mayúscula O con tilde y macron"}}
    }, {
      "category": "Lu",
      "key": "0230",
      "mappings": {"default": {"default": "mayúscula O con dot above y macron"}}
    }, {
      "key": "1E08",
      "mappings": {"default": {"default": "mayúscula C con cedilla y agudo"}},
      "category": "Lu"
    }, {
      "key": "1E14",
      "mappings": {"default": {"default": "mayúscula E con macrón y grave"}},
      "category": "Lu"
    }, {
      "key": "1E16",
      "mappings": {"default": {"default": "mayúscula E con macrón y agudo"}},
      "category": "Lu"
    }, {
      "key": "1E1C",
      "mappings": {"default": {"default": "mayúscula E with cedilla and breve"}},
      "category": "Lu"
    }, {
      "key": "1E2E",
      "mappings": {"default": {"default": "mayúscula I con diéresis y agudo"}},
      "category": "Lu"
    }, {
      "key": "1E38",
      "mappings": {"default": {"default": "mayúscula L con punto debajo y macrón"}},
      "category": "Lu"
    }, {
      "key": "1E4C",
      "mappings": {"default": {"default": "mayúscula O con tilde y acute"}},
      "category": "Lu"
    }, {
      "key": "1E4E",
      "mappings": {"default": {"default": "mayúscula O con tilde y diéresis"}},
      "category": "Lu"
    }, {
      "key": "1E50",
      "mappings": {"default": {"default": "mayúscula O con macrón y grave"}},
      "category": "Lu"
    }, {
      "key": "1E52",
      "mappings": {"default": {"default": "mayúscula O con macrón y agudo"}},
      "category": "Lu"
    }, {
      "key": "1E5C",
      "mappings": {"default": {"default": "mayúscula R con punto debajo y macrón"}},
      "category": "Lu"
    }, {
      "key": "1E64",
      "mappings": {"default": {"default": "mayúscula S con agudo y punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E66",
      "mappings": {"default": {"default": "mayúscula S con carón y punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E68",
      "mappings": {"default": {"default": "mayúscula S con punto debajo y punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E78",
      "mappings": {"default": {"default": "mayúscula U con tilde y agudo"}},
      "category": "Lu"
    }, {
      "key": "1E7A",
      "mappings": {"default": {"default": "mayúscula U con macrón y diéresis"}},
      "category": "Lu"
    }, {
      "key": "1EA4",
      "mappings": {"default": {"default": "mayúscula A con acento circunflejo y agudo"}},
      "category": "Lu"
    }, {
      "key": "1EA6",
      "mappings": {"default": {"default": "mayúscula A con acento circunflejo y grave"}},
      "category": "Lu"
    }, {
      "key": "1EA8",
      "mappings": {"default": {"default": "mayúscula A con acento circunflejo y gancho arriba"}},
      "category": "Lu"
    }, {
      "key": "1EAA",
      "mappings": {"default": {"default": "mayúscula A con acento circunflejo y tilde"}},
      "category": "Lu"
    }, {
      "key": "1EAC",
      "mappings": {"default": {"default": "mayúscula A con acento circunflejo y punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1EAE",
      "mappings": {"default": {"default": "mayúscula A con breve y agudo"}},
      "category": "Lu"
    }, {
      "key": "1EB0",
      "mappings": {"default": {"default": "mayúscula A con breve y grave"}},
      "category": "Lu"
    }, {
      "key": "1EB2",
      "mappings": {"default": {"default": "mayúscula A con breve y gancho arriba"}},
      "category": "Lu"
    }, {
      "key": "1EB4",
      "mappings": {"default": {"default": "mayúscula A con breve y tilde"}},
      "category": "Lu"
    }, {
      "key": "1EB6",
      "mappings": {"default": {"default": "mayúscula A con breve y punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1EBE",
      "mappings": {"default": {"default": "mayúscula E con acento circunflejo y agudo"}},
      "category": "Lu"
    }, {
      "key": "1EC0",
      "mappings": {"default": {"default": "mayúscula E con acento circunflejo y grave"}},
      "category": "Lu"
    }, {
      "key": "1EC2",
      "mappings": {"default": {"default": "mayúscula E con acento circunflejo y gancho arriba"}},
      "category": "Lu"
    }, {
      "key": "1EC4",
      "mappings": {"default": {"default": "mayúscula E con acento circunflejo y tilde"}},
      "category": "Lu"
    }, {
      "key": "1EC6",
      "mappings": {"default": {"default": "mayúscula E con acento circunflejo y punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1ED0",
      "mappings": {"default": {"default": "mayúscula O con acento circunflejo y agudo"}},
      "category": "Lu"
    }, {
      "key": "1ED2",
      "mappings": {"default": {"default": "mayúscula O con acento circunflejo y grave"}},
      "category": "Lu"
    }, {
      "key": "1ED4",
      "mappings": {"default": {"default": "mayúscula O con acento circunflejo y gancho arriba"}},
      "category": "Lu"
    }, {
      "key": "1ED6",
      "mappings": {"default": {"default": "mayúscula O con acento circunflejo y tilde"}},
      "category": "Lu"
    }, {
      "key": "1ED8",
      "mappings": {"default": {"default": "mayúscula O con acento circunflejo y punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1EDA",
      "mappings": {"default": {"default": "mayúscula O with horn and acute"}},
      "category": "Lu"
    }, {
      "key": "1EDC",
      "mappings": {"default": {"default": "mayúscula O with horn and grave"}},
      "category": "Lu"
    }, {
      "key": "1EDE",
      "mappings": {"default": {"default": "mayúscula O with horn and hook above"}},
      "category": "Lu"
    }, {
      "key": "1EE0",
      "mappings": {"default": {"default": "mayúscula O with horn and tilde"}},
      "category": "Lu"
    }, {
      "key": "1EE2",
      "mappings": {"default": {"default": "mayúscula O con cuerno y punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1EE8",
      "mappings": {"default": {"default": "mayúscula U con cuerno y agudo"}},
      "category": "Lu"
    }, {
      "key": "1EEA",
      "mappings": {"default": {"default": "mayúscula U con cuerno y grave"}},
      "category": "Lu"
    }, {
      "key": "1EEC",
      "mappings": {"default": {"default": "mayúscula U con cuerno y gancho arriba"}},
      "category": "Lu"
    }, {
      "key": "1EEE",
      "mappings": {"default": {"default": "mayúscula U con cuerno y tilde"}},
      "category": "Lu"
    }, {"key": "1EF0", "mappings": {"default": {"default": "mayúscula U con cuerno y punto debajo"}}, "category": "Lu"}]
  ,
  "es/symbols/latin-upper-single-accent.js":
    [{"locale": "es"}, {
      "key": "00C0",
      "mappings": {"default": {"default": "mayúscula A grave"}},
      "category": "Lu"
    }, {"key": "00C1", "mappings": {"default": {"default": "mayúscula A aguda"}}, "category": "Lu"}, {
      "key": "00C2",
      "mappings": {"default": {"default": "mayúscula A acento circunflejo"}},
      "category": "Lu"
    }, {"key": "00C3", "mappings": {"default": {"default": "mayúscula A con tilde"}}, "category": "Lu"}, {
      "key": "00C4",
      "mappings": {"default": {"default": "mayúscula A diéresis"}},
      "category": "Lu"
    }, {
      "key": "00C5",
      "mappings": {"default": {"default": "mayúscula A con anillo"}},
      "category": "Lu"
    }, {"key": "00C7", "mappings": {"default": {"default": "mayúscula C cedilla"}}, "category": "Lu"}, {
      "key": "00C8",
      "mappings": {"default": {"default": "mayúscula E grave"}},
      "category": "Lu"
    }, {"key": "00C9", "mappings": {"default": {"default": "mayúscula E aguda"}}, "category": "Lu"}, {
      "key": "00CA",
      "mappings": {"default": {"default": "mayúscula E acento circunflejo"}},
      "category": "Lu"
    }, {"key": "00CB", "mappings": {"default": {"default": "mayúscula E diéresis"}}, "category": "Lu"}, {
      "key": "00CC",
      "mappings": {"default": {"default": "mayúscula I grave"}},
      "category": "Lu"
    }, {"key": "00CD", "mappings": {"default": {"default": "mayúscula I aguda"}}, "category": "Lu"}, {
      "key": "00CE",
      "mappings": {"default": {"default": "mayúscula I acento circumflejo"}},
      "category": "Lu"
    }, {"key": "00CF", "mappings": {"default": {"default": "mayúscula I diéresis"}}, "category": "Lu"}, {
      "key": "00D1",
      "mappings": {"default": {"default": "mayúscula Ñ"}},
      "category": "Lu"
    }, {"key": "00D2", "mappings": {"default": {"default": "mayúscula O grave"}}, "category": "Lu"}, {
      "key": "00D3",
      "mappings": {"default": {"default": "mayúscula O aguda"}},
      "category": "Lu"
    }, {
      "key": "00D4",
      "mappings": {"default": {"default": "mayúscula O acento circunflejo"}},
      "category": "Lu"
    }, {"key": "00D5", "mappings": {"default": {"default": "mayúscula O con tilde"}}, "category": "Lu"}, {
      "key": "00D6",
      "mappings": {"default": {"default": "mayúscula O diéresis"}},
      "category": "Lu"
    }, {"key": "00D9", "mappings": {"default": {"default": "mayúscula U grave"}}, "category": "Lu"}, {
      "key": "00DA",
      "mappings": {"default": {"default": "mayúscula U aguda"}},
      "category": "Lu"
    }, {
      "key": "00DB",
      "mappings": {"default": {"default": "mayúscula U acento circunflejo"}},
      "category": "Lu"
    }, {"key": "00DC", "mappings": {"default": {"default": "mayúscula U diéresis"}}, "category": "Lu"}, {
      "key": "00DD",
      "mappings": {"default": {"default": "mayúscula Y aguda"}},
      "category": "Lu"
    }, {"key": "0100", "mappings": {"default": {"default": "mayúscula A barra"}}, "category": "Lu"}, {
      "key": "0102",
      "mappings": {"default": {"default": "mayúscula A con breve"}},
      "category": "Lu"
    }, {
      "key": "0104",
      "mappings": {"default": {"default": "mayúscula A con ogonek"}},
      "category": "Lu"
    }, {"key": "0106", "mappings": {"default": {"default": "mayúscula C con agudo"}}, "category": "Lu"}, {
      "key": "0108",
      "mappings": {"default": {"default": "mayúscula C con acento circunflejo"}},
      "category": "Lu"
    }, {
      "key": "010A",
      "mappings": {"default": {"default": "mayúscula C con punto arriba"}},
      "category": "Lu"
    }, {"key": "010C", "mappings": {"default": {"default": "mayúscula C con carón"}}, "category": "Lu"}, {
      "key": "010E",
      "mappings": {"default": {"default": "mayúscula D con carón"}},
      "category": "Lu"
    }, {"key": "0112", "mappings": {"default": {"default": "mayúscula E barra"}}, "category": "Lu"}, {
      "category": "Lu",
      "key": "0114",
      "mappings": {"default": {"default": "mayúscula E con breve"}}
    }, {
      "key": "0116",
      "mappings": {"default": {"default": "mayúscula E con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "0118",
      "mappings": {"default": {"default": "mayúscula E con ogonek"}},
      "category": "Lu"
    }, {"key": "011A", "mappings": {"default": {"default": "mayúscula E con carón"}}, "category": "Lu"}, {
      "key": "011C",
      "mappings": {"default": {"default": "mayúscula G con acento circunflejo"}},
      "category": "Lu"
    }, {"key": "011E", "mappings": {"default": {"default": "mayúscula G con breve"}}, "category": "Lu"}, {
      "key": "0120",
      "mappings": {"default": {"default": "mayúscula G con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "0122",
      "mappings": {"default": {"default": "mayúscula G con cedilla"}},
      "category": "Lu"
    }, {
      "key": "0124",
      "mappings": {"default": {"default": "mayúscula H con acento circunflejo"}},
      "category": "Lu"
    }, {"key": "0128", "mappings": {"default": {"default": "mayúscula I con tilde"}}, "category": "Lu"}, {
      "key": "012A",
      "mappings": {"default": {"default": "mayúscula I barra"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "012C", "mappings": {"default": {"default": "mayúscula I con breve"}}}, {
      "key": "012E",
      "mappings": {"default": {"default": "mayúscula I con ogonek"}},
      "category": "Lu"
    }, {
      "key": "0130",
      "mappings": {"default": {"default": "mayúscula I con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "0134",
      "mappings": {"default": {"default": "mayúscula J con acento circunflejo"}},
      "category": "Lu"
    }, {
      "key": "0136",
      "mappings": {"default": {"default": "mayúscula K con cedilla"}},
      "category": "Lu"
    }, {"key": "0139", "mappings": {"default": {"default": "mayúscula L con agudo"}}, "category": "Lu"}, {
      "key": "013B",
      "mappings": {"default": {"default": "mayúscula L con cedilla"}},
      "category": "Lu"
    }, {"key": "013D", "mappings": {"default": {"default": "mayúscula L con carón"}}, "category": "Lu"}, {
      "key": "013F",
      "mappings": {"default": {"default": "mayúscula L con punto mediano"}},
      "category": "Lu"
    }, {"key": "0143", "mappings": {"default": {"default": "mayúscula N con agudo"}}, "category": "Lu"}, {
      "key": "0145",
      "mappings": {"default": {"default": "mayúscula N con cedilla"}},
      "category": "Lu"
    }, {"key": "0147", "mappings": {"default": {"default": "mayúscula n con carón"}}, "category": "Lu"}, {
      "key": "014C",
      "mappings": {"default": {"default": "mayúscula O barra"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "014E", "mappings": {"default": {"default": "mayúscula O con breve"}}}, {
      "key": "0150",
      "mappings": {"default": {"default": "mayúscula O con doble acento agudo "}},
      "category": "Lu"
    }, {"key": "0154", "mappings": {"default": {"default": "mayúscula R con agudo"}}, "category": "Lu"}, {
      "key": "0156",
      "mappings": {"default": {"default": "mayúscula R con cedilla"}},
      "category": "Lu"
    }, {"key": "0158", "mappings": {"default": {"default": "mayúscula R con carón"}}, "category": "Lu"}, {
      "key": "015A",
      "mappings": {"default": {"default": "mayúscula S con agudo"}},
      "category": "Lu"
    }, {
      "key": "015C",
      "mappings": {"default": {"default": "mayúscula S con acento circunflejo"}},
      "category": "Lu"
    }, {
      "key": "015E",
      "mappings": {"default": {"default": "mayúscula S con cedilla"}},
      "category": "Lu"
    }, {"key": "0160", "mappings": {"default": {"default": "mayúscula S con carón"}}, "category": "Lu"}, {
      "key": "0162",
      "mappings": {"default": {"default": "mayúscula T con cedilla"}},
      "category": "Lu"
    }, {"key": "0164", "mappings": {"default": {"default": "mayúscula T con carón"}}, "category": "Lu"}, {
      "key": "0168",
      "mappings": {"default": {"default": "mayúscula U con tilde"}},
      "category": "Lu"
    }, {"key": "016A", "mappings": {"default": {"default": "mayúscula U barra"}}, "category": "Lu"}, {
      "key": "016C",
      "mappings": {"default": {"default": "mayúscula U con breve"}},
      "category": "Lu"
    }, {
      "key": "016E",
      "mappings": {"default": {"default": "mayúscula U con anillo arriba"}},
      "category": "Lu"
    }, {
      "key": "0170",
      "mappings": {"default": {"default": "mayúscula U con doble acento agudo"}},
      "category": "Lu"
    }, {
      "key": "0172",
      "mappings": {"default": {"default": "mayúscula U con ogonek"}},
      "category": "Lu"
    }, {
      "key": "0174",
      "mappings": {"default": {"default": "mayúscula W con acento circunflejo"}},
      "category": "Lu"
    }, {
      "key": "0176",
      "mappings": {"default": {"default": "mayúscula Y con circunflejo"}},
      "category": "Lu"
    }, {"key": "0178", "mappings": {"default": {"default": "mayúscula Y diéresis"}}, "category": "Lu"}, {
      "key": "0179",
      "mappings": {"default": {"default": "mayúscula Z con agudo"}},
      "category": "Lu"
    }, {"key": "017B", "mappings": {"default": {"default": "mayúscula Z con punto"}}, "category": "Lu"}, {
      "key": "017D",
      "mappings": {"default": {"default": "mayúscula Z con carón"}},
      "category": "Lu"
    }, {
      "category": "Lu",
      "key": "01CD",
      "mappings": {"default": {"default": "mayúscula A con acento"}}
    }, {
      "category": "Lu",
      "key": "01CF",
      "mappings": {"default": {"default": "mayúscula I con acento"}}
    }, {
      "category": "Lu",
      "key": "01D1",
      "mappings": {"default": {"default": "mayúscula O con acento"}}
    }, {
      "category": "Lu",
      "key": "01D3",
      "mappings": {"default": {"default": "mayúscula U con acento"}}
    }, {
      "category": "Lu",
      "key": "01E6",
      "mappings": {"default": {"default": "mayúscula G con acento"}}
    }, {
      "category": "Lu",
      "key": "01E8",
      "mappings": {"default": {"default": "mayúscula K con acento"}}
    }, {
      "category": "Lu",
      "key": "01EA",
      "mappings": {"default": {"default": "mayúscula O con pecíolo"}}
    }, {
      "category": "Lu",
      "key": "01F4",
      "mappings": {"default": {"default": "mayúscula G con acento agudo"}}
    }, {
      "category": "Lu",
      "key": "01F8",
      "mappings": {"default": {"default": "mayúscula N con grave"}}
    }, {
      "category": "Lu",
      "key": "0200",
      "mappings": {"default": {"default": "mayúscula con doble tumba"}}
    }, {
      "category": "Lu",
      "key": "0202",
      "mappings": {"default": {"default": "mayúscula a con breve invertido"}}
    }, {
      "category": "Lu",
      "key": "0204",
      "mappings": {"default": {"default": "mayúscula E con doble tumba"}}
    }, {
      "category": "Lu",
      "key": "0206",
      "mappings": {"default": {"default": "mayúscula E con Breve invertido"}}
    }, {
      "category": "Lu",
      "key": "0208",
      "mappings": {"default": {"default": "mayúscula I con doble tumba"}}
    }, {
      "category": "Lu",
      "key": "020A",
      "mappings": {"default": {"default": "mayúscula I con Breve invertido"}}
    }, {
      "category": "Lu",
      "key": "020C",
      "mappings": {"default": {"default": "mayúscula O con doble tumba"}}
    }, {
      "category": "Lu",
      "key": "020E",
      "mappings": {"default": {"default": "mayúscula O con Breve invertido"}}
    }, {
      "category": "Lu",
      "key": "0210",
      "mappings": {"default": {"default": "mayúscula R con doble tumba"}}
    }, {
      "category": "Lu",
      "key": "0212",
      "mappings": {"default": {"default": "mayúscula R con breve invertido"}}
    }, {
      "category": "Lu",
      "key": "0214",
      "mappings": {"default": {"default": "mayúscula U con doble tumba"}}
    }, {
      "category": "Lu",
      "key": "0216",
      "mappings": {"default": {"default": "mayúscula U con breve invertido"}}
    }, {
      "category": "Lu",
      "key": "0218",
      "mappings": {"default": {"default": "mayúscula S con coma debajo"}}
    }, {
      "category": "Lu",
      "key": "021A",
      "mappings": {"default": {"default": "mayúscula T con coma debajo"}}
    }, {
      "category": "Lu",
      "key": "021E",
      "mappings": {"default": {"default": "mayúscula H con Caron"}}
    }, {
      "category": "Lu",
      "key": "0226",
      "mappings": {"default": {"default": "mayúscula con un punto arriba"}}
    }, {
      "category": "Lu",
      "key": "0228",
      "mappings": {"default": {"default": "mayúscula E con cedilla"}}
    }, {
      "category": "Lu",
      "key": "022E",
      "mappings": {"default": {"default": "mayúscula O con punto arriba"}}
    }, {
      "category": "Lu",
      "key": "0232",
      "mappings": {"default": {"default": "Mayúscula Y con Macron"}}
    }, {
      "key": "1E00",
      "mappings": {"default": {"default": "mayúscula A con anillo debajo"}},
      "category": "Lu"
    }, {
      "key": "1E02",
      "mappings": {"default": {"default": "mayúscula B con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E04",
      "mappings": {"default": {"default": "mayúscula B con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1E06",
      "mappings": {"default": {"default": "mayúscula B con línea debajo"}},
      "category": "Lu"
    }, {
      "key": "1E0A",
      "mappings": {"default": {"default": "mayúscula D con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E0C",
      "mappings": {"default": {"default": "mayúscula D con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1E0E",
      "mappings": {"default": {"default": "mayúscula D con línea debajo"}},
      "category": "Lu"
    }, {
      "key": "1E10",
      "mappings": {"default": {"default": "mayúscula D con cedilla"}},
      "category": "Lu"
    }, {
      "key": "1E12",
      "mappings": {"default": {"default": "mayúscula D con acento circunflejo debajo"}},
      "category": "Lu"
    }, {
      "key": "1E18",
      "mappings": {"default": {"default": "mayúscula E con acento circunflejo debajo"}},
      "category": "Lu"
    }, {
      "key": "1E1A",
      "mappings": {"default": {"default": "mayúscula E con tilde debajo"}},
      "category": "Lu"
    }, {
      "key": "1E1E",
      "mappings": {"default": {"default": "mayúscula F con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E20",
      "mappings": {"default": {"default": "mayúscula G con macrón"}},
      "category": "Lu"
    }, {
      "key": "1E22",
      "mappings": {"default": {"default": "mayúscula H con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E24",
      "mappings": {"default": {"default": "mayúscula H con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1E26",
      "mappings": {"default": {"default": "mayúscula H con diéresis"}},
      "category": "Lu"
    }, {
      "key": "1E28",
      "mappings": {"default": {"default": "mayúscula H con cedilla"}},
      "category": "Lu"
    }, {
      "key": "1E2A",
      "mappings": {"default": {"default": "mayúscula H con breve debajo"}},
      "category": "Lu"
    }, {
      "key": "1E2C",
      "mappings": {"default": {"default": "mayúscula I con tilde debajo"}},
      "category": "Lu"
    }, {"key": "1E30", "mappings": {"default": {"default": "mayúscula K con agudo"}}, "category": "Lu"}, {
      "key": "1E32",
      "mappings": {"default": {"default": "mayúscula K con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1E34",
      "mappings": {"default": {"default": "mayúscula K con línea debajo"}},
      "category": "Lu"
    }, {
      "key": "1E36",
      "mappings": {"default": {"default": "mayúscula L con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1E3A",
      "mappings": {"default": {"default": "mayúscula L con línea debajo"}},
      "category": "Lu"
    }, {
      "key": "1E3C",
      "mappings": {"default": {"default": "mayúscula L con acento circunflejo debajo"}},
      "category": "Lu"
    }, {"key": "1E3E", "mappings": {"default": {"default": "mayúscula M con agudo"}}, "category": "Lu"}, {
      "key": "1E40",
      "mappings": {"default": {"default": "mayúscula M con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E42",
      "mappings": {"default": {"default": "mayúscula M con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1E44",
      "mappings": {"default": {"default": "mayúscula N con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E46",
      "mappings": {"default": {"default": "mayúscula N con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1E48",
      "mappings": {"default": {"default": "mayúscula N con línea debajo"}},
      "category": "Lu"
    }, {
      "key": "1E4A",
      "mappings": {"default": {"default": "mayúscula N con acento circunflejo debajo"}},
      "category": "Lu"
    }, {"key": "1E54", "mappings": {"default": {"default": "mayúscula P con agudo"}}, "category": "Lu"}, {
      "key": "1E56",
      "mappings": {"default": {"default": "mayúscula P con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E58",
      "mappings": {"default": {"default": "mayúscula R con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E5A",
      "mappings": {"default": {"default": "mayúscula R con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1E5E",
      "mappings": {"default": {"default": "mayúscula R con línea debajo"}},
      "category": "Lu"
    }, {
      "key": "1E60",
      "mappings": {"default": {"default": "mayúscula S con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E62",
      "mappings": {"default": {"default": "mayúscula S con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1E6A",
      "mappings": {"default": {"default": "mayúscula T con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E6C",
      "mappings": {"default": {"default": "mayúscula T con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1E6E",
      "mappings": {"default": {"default": "mayúscula T con línea debajo"}},
      "category": "Lu"
    }, {
      "key": "1E70",
      "mappings": {"default": {"default": "mayúscula T con acento circunflejo debajo"}},
      "category": "Lu"
    }, {
      "key": "1E72",
      "mappings": {"default": {"default": "mayúscula U con diéresis debajo"}},
      "category": "Lu"
    }, {
      "key": "1E74",
      "mappings": {"default": {"default": "mayúscula U con tilde debajo"}},
      "category": "Lu"
    }, {
      "key": "1E76",
      "mappings": {"default": {"default": "mayúscula U con acento circunflejo debajo"}},
      "category": "Lu"
    }, {"key": "1E7C", "mappings": {"default": {"default": "mayúscula V con tilde"}}, "category": "Lu"}, {
      "key": "1E7E",
      "mappings": {"default": {"default": "mayúscula V con punto debajo"}},
      "category": "Lu"
    }, {"key": "1E80", "mappings": {"default": {"default": "mayúscula W con grave"}}, "category": "Lu"}, {
      "key": "1E82",
      "mappings": {"default": {"default": "mayúscula W con agudo"}},
      "category": "Lu"
    }, {
      "key": "1E84",
      "mappings": {"default": {"default": "mayúscula W con diéresis"}},
      "category": "Lu"
    }, {
      "key": "1E86",
      "mappings": {"default": {"default": "mayúscula W con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E88",
      "mappings": {"default": {"default": "mayúscula W con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1E8A",
      "mappings": {"default": {"default": "mayúscula X con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E8C",
      "mappings": {"default": {"default": "mayúscula X con diéresis"}},
      "category": "Lu"
    }, {
      "key": "1E8E",
      "mappings": {"default": {"default": "mayúscula Y con punto arriba"}},
      "category": "Lu"
    }, {
      "key": "1E90",
      "mappings": {"default": {"default": "mayúscula Z con acento circunflejo"}},
      "category": "Lu"
    }, {
      "key": "1E92",
      "mappings": {"default": {"default": "mayúscula Z con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1E94",
      "mappings": {"default": {"default": "mayúscula Z with line below"}},
      "category": "Lu"
    }, {
      "key": "1EA0",
      "mappings": {"default": {"default": "mayúscula A con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1EA2",
      "mappings": {"default": {"default": "mayúscula A con gancho arriba"}},
      "category": "Lu"
    }, {
      "key": "1EB8",
      "mappings": {"default": {"default": "mayúscula E con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1EBA",
      "mappings": {"default": {"default": "mayúscula E con gancho arriba"}},
      "category": "Lu"
    }, {"key": "1EBC", "mappings": {"default": {"default": "mayúscula E con tilde"}}, "category": "Lu"}, {
      "key": "1EC8",
      "mappings": {"default": {"default": "mayúscula I con gancho arriba"}},
      "category": "Lu"
    }, {
      "key": "1ECA",
      "mappings": {"default": {"default": "mayúscula I con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1ECC",
      "mappings": {"default": {"default": "mayúscula O con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1ECE",
      "mappings": {"default": {"default": "mayúscula O con gancho arriba"}},
      "category": "Lu"
    }, {
      "key": "1EE4",
      "mappings": {"default": {"default": "mayúscula U con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1EE6",
      "mappings": {"default": {"default": "mayúscula U con gancho arriba"}},
      "category": "Lu"
    }, {"key": "1EF2", "mappings": {"default": {"default": "mayúscula Y con grave"}}, "category": "Lu"}, {
      "key": "1EF4",
      "mappings": {"default": {"default": "mayúscula Y con punto debajo"}},
      "category": "Lu"
    }, {
      "key": "1EF6",
      "mappings": {"default": {"default": "mayúscula Y con gancho arriba"}},
      "category": "Lu"
    }, {"key": "1EF8", "mappings": {"default": {"default": "mayúscula Y con tilde"}}, "category": "Lu"}]
  ,
  "es/symbols/math_angles.js":
    [{"locale": "es"}, {
      "key": "22BE",
      "mappings": {"default": {"default": "ángulo recto con arco"}},
      "category": "Sm"
    }, {
      "key": "237C",
      "mappings": {"default": {"default": "ángulo recto con flecha zigzag hacia abajo"}},
      "category": "Sm"
    }, {"key": "27C0", "mappings": {"default": {"default": "ángulo sólido"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "299B",
      "mappings": {"default": {"default": "Ángulo medido de apertura a la izquierda"}}
    }, {
      "category": "Sm",
      "key": "299C",
      "mappings": {"default": {"default": "Variante de ángulo recto con cuadrado"}}
    }, {
      "key": "299D",
      "mappings": {"default": {"default": "ángulo recto medido con punto"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "299E",
      "mappings": {"default": {"default": "Ángulo con S dentro"}}
    }, {"category": "Sm", "key": "299F", "mappings": {"default": {"default": "Ángulo agudo"}}}, {
      "category": "Sm",
      "key": "29A0",
      "mappings": {"default": {"default": "Ángulo esférico de apertura a la izquierda"}}
    }, {
      "category": "Sm",
      "key": "29A1",
      "mappings": {"default": {"default": "Apertura de ángulo esférico"}}
    }, {"category": "Sm", "key": "29A2", "mappings": {"default": {"default": "Ángulo de giro"}}}, {
      "category": "Sm",
      "key": "29A3",
      "mappings": {"default": {"default": "Ángulo invertido"}}
    }, {
      "key": "29A4",
      "mappings": {"default": {"default": "ángulo con barra inferior"}},
      "category": "Sm"
    }, {
      "key": "29A5",
      "mappings": {"default": {"default": "ángulo inverso con barra inferior"}},
      "category": "Sm"
    }, {
      "key": "29A6",
      "mappings": {"default": {"default": "ángulo oblicuo hacia arriba"}},
      "category": "Sm"
    }, {
      "key": "29A7",
      "mappings": {"default": {"default": "ángulo oblicuo hacia abajo"}},
      "category": "Sm"
    }, {
      "key": "29A8",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing up and to the right"}},
      "category": "Sm"
    }, {
      "key": "29A9",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing up and to the left"}},
      "category": "Sm"
    }, {
      "key": "29AA",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing down and to the right"}},
      "category": "Sm"
    }, {
      "key": "29AB",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing down and to the left"}},
      "category": "Sm"
    }, {
      "key": "29AC",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing right and up"}},
      "category": "Sm"
    }, {
      "key": "29AD",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing left and up"}},
      "category": "Sm"
    }, {
      "key": "29AE",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing right and down"}},
      "category": "Sm"
    }, {
      "key": "29AF",
      "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing left and down"}},
      "category": "Sm"
    }]
  ,
  "es/symbols/math_arrows.js":
    [{"locale": "es"}, {
      "key": "2190",
      "mappings": {"default": {"default": "flecha izquierda"}},
      "category": "Sm"
    }, {"key": "2191", "mappings": {"default": {"default": "flecha arriba"}}, "category": "Sm"}, {
      "key": "2192",
      "mappings": {"default": {"default": "flecha derecha", "defaultMP": "flecha"}},
      "category": "Sm"
    }, {"key": "2193", "mappings": {"default": {"default": "flecha abajo"}}, "category": "Sm"}, {
      "key": "2194",
      "mappings": {"default": {"default": "flecha izquierda y derecha"}},
      "category": "Sm"
    }, {"key": "2195", "mappings": {"default": {"default": "flecha arriba y abajo"}}, "category": "So"}, {
      "key": "2196",
      "mappings": {"default": {"default": "flecha a noroeste"}},
      "category": "So"
    }, {"key": "2197", "mappings": {"default": {"default": "flecha a nordeste"}}, "category": "So"}, {
      "key": "2198",
      "mappings": {"default": {"default": "flecha a sureste"}},
      "category": "So"
    }, {"key": "2199", "mappings": {"default": {"default": "flecha a suroeste"}}, "category": "So"}, {
      "key": "219A",
      "mappings": {"default": {"default": "flecha izquierda tachada"}},
      "category": "Sm"
    }, {"key": "219B", "mappings": {"default": {"default": "flecha tachada"}}, "category": "Sm"}, {
      "key": "219C",
      "mappings": {"default": {"default": "flecha ondulada izquierda"}},
      "category": "So"
    }, {"key": "219D", "mappings": {"default": {"default": "flecha ondulada"}}, "category": "So"}, {
      "key": "219E",
      "mappings": {"default": {"default": "flecha izquierda con doble punta"}},
      "category": "So"
    }, {
      "key": "219F",
      "mappings": {"default": {"default": "flecha con doble punta hacia arriba"}},
      "category": "So"
    }, {
      "key": "21A0",
      "mappings": {"default": {"default": "flecha derechaa con doble punta"}},
      "category": "Sm"
    }, {
      "key": "21A1",
      "mappings": {"default": {"default": "flecha con doble punta hacia abajo"}},
      "category": "So"
    }, {
      "key": "21A2",
      "mappings": {"default": {"default": "flecha izquierda con cola"}},
      "category": "So"
    }, {
      "key": "21A3",
      "mappings": {"default": {"default": "flecha derecha con cola"}},
      "category": "Sm"
    }, {
      "key": "21A4",
      "mappings": {"default": {"default": "flecha izquierda de barra"}},
      "category": "So"
    }, {"key": "21A5", "mappings": {"default": {"default": "barra con flecha"}}, "category": "So"}, {
      "key": "21A6",
      "mappings": {"default": {"default": "flecha derecha de barra"}},
      "category": "Sm"
    }, {"key": "21A7", "mappings": {"default": {"default": "flecha con barra"}}, "category": "So"}, {
      "key": "21A8",
      "mappings": {"default": {"default": "flecha arriba y abajo con base"}},
      "category": "So"
    }, {
      "key": "21A9",
      "mappings": {"default": {"default": "flecha izquierda con gancho"}},
      "category": "So"
    }, {"key": "21AA", "mappings": {"default": {"default": "flecha con gancho"}}, "category": "So"}, {
      "key": "21AB",
      "mappings": {"default": {"default": "flecha izquierda con lazo"}},
      "category": "So"
    }, {"key": "21AC", "mappings": {"default": {"default": "flecha con lazo"}}, "category": "So"}, {
      "key": "21AD",
      "mappings": {"default": {"default": "flecha ondulada izquierda y derecha"}},
      "category": "So"
    }, {
      "key": "21AE",
      "mappings": {"default": {"default": "flecha izquierda y derecha tachada"}},
      "category": "Sm"
    }, {"key": "21AF", "mappings": {"default": {"default": "flecha zigzag abajo"}}, "category": "So"}, {
      "key": "21B0",
      "mappings": {"default": {"default": "flecha arriba con punta izquierda"}},
      "category": "So"
    }, {
      "key": "21B1",
      "mappings": {"default": {"default": "flecha arriba con punta derecha"}},
      "category": "So"
    }, {
      "key": "21B2",
      "mappings": {"default": {"default": "flecha abajo con punta izquierda"}},
      "category": "So"
    }, {
      "key": "21B3",
      "mappings": {"default": {"default": "flecha abajo con punta derecha"}},
      "category": "So"
    }, {
      "key": "21B4",
      "mappings": {"default": {"default": "flecha derecha desde esquina inferior"}},
      "category": "So"
    }, {
      "key": "21B5",
      "mappings": {"default": {"default": "flecha hacia abajo desde esquina izquierda"}},
      "category": "So"
    }, {
      "key": "21B6",
      "mappings": {"default": {"default": "flecha semicircular superior en sentido antihorario"}},
      "category": "So"
    }, {
      "key": "21B7",
      "mappings": {"default": {"default": "flecha semicircular superior en sentido horario"}},
      "category": "So"
    }, {
      "key": "21B8",
      "mappings": {"default": {"default": "flecha noroeste a barra larga"}},
      "category": "So"
    }, {
      "key": "21B9",
      "mappings": {"default": {"default": "flecha izquierda a barra arriba de flecha derecha a barra"}},
      "category": "So"
    }, {
      "key": "21BA",
      "mappings": {"default": {"default": "flecha circular abierta en sentido antihorario"}},
      "category": "So"
    }, {
      "key": "21BB",
      "mappings": {"default": {"default": "flecha circular abierta en sentido horario"}},
      "category": "So"
    }, {
      "key": "21C4",
      "mappings": {"default": {"default": "flecha derecha arriba de flecha izquierda"}},
      "category": "So"
    }, {
      "key": "21C5",
      "mappings": {"default": {"default": "flecha arriba a la izquierda de flecha abajo"}},
      "category": "So"
    }, {
      "key": "21C6",
      "mappings": {"default": {"default": "flecha izquierda arriba de flecha derecha"}},
      "category": "So"
    }, {
      "key": "21C7",
      "mappings": {"default": {"default": "dos flechas hacia la izquierda"}},
      "category": "So"
    }, {
      "key": "21C8",
      "mappings": {"default": {"default": "dos flechas hacia arriba"}},
      "category": "So"
    }, {"key": "21C9", "mappings": {"default": {"default": "dos flechas"}}, "category": "So"}, {
      "key": "21CA",
      "mappings": {"default": {"default": "dos flechas hacia abajo"}},
      "category": "So"
    }, {
      "key": "21CD",
      "mappings": {"default": {"default": "flecha doble izquierda tachada"}},
      "category": "So"
    }, {
      "key": "21CE",
      "mappings": {"default": {"default": "flecha doble izquierda y derecha tachada"}},
      "category": "Sm"
    }, {"key": "21CF", "mappings": {"default": {"default": "flecha doble tachada"}}, "category": "Sm"}, {
      "key": "21D0",
      "mappings": {"default": {"default": "flecha doble hacia la izquierda"}},
      "category": "So"
    }, {
      "key": "21D1",
      "mappings": {"default": {"default": "flecha doble hacia arriba"}},
      "category": "So"
    }, {"key": "21D2", "mappings": {"default": {"default": "flecha doble"}}, "category": "Sm"}, {
      "key": "21D3",
      "mappings": {"default": {"default": "flecha doble hacia abajo"}},
      "category": "So"
    }, {
      "key": "21D4",
      "mappings": {"default": {"default": "flecha doble izquierda-derecha"}},
      "category": "Sm"
    }, {"key": "21D5", "mappings": {"default": {"default": "flecha doble vertical"}}, "category": "So"}, {
      "key": "21D6",
      "mappings": {"default": {"default": "flecha doble hacia noroeste"}},
      "category": "So"
    }, {
      "key": "21D7",
      "mappings": {"default": {"default": "flecha doble hacia nordeste"}},
      "category": "So"
    }, {
      "key": "21D8",
      "mappings": {"default": {"default": "flecha doble hacia sudeste"}},
      "category": "So"
    }, {
      "key": "21D9",
      "mappings": {"default": {"default": "flecha doble hacia sudoeste"}},
      "category": "So"
    }, {
      "key": "21DA",
      "mappings": {"default": {"default": "flecha triple izquierda"}},
      "category": "So"
    }, {"key": "21DB", "mappings": {"default": {"default": "flecha triple"}}, "category": "So"}, {
      "key": "21DC",
      "mappings": {"default": {"default": "flecha ondulada izquierda"}},
      "category": "So"
    }, {"key": "21DD", "mappings": {"default": {"default": "flecha ondulada"}}, "category": "So"}, {
      "key": "21DE",
      "mappings": {"default": {"default": "flecha hacia arriba con doble tachado"}},
      "category": "So"
    }, {
      "key": "21DF",
      "mappings": {"default": {"default": "flecha hacia abajo con doble tachado"}},
      "category": "So"
    }, {
      "key": "21E0",
      "mappings": {"default": {"default": "flecha de puntos hacia la izquierda"}},
      "category": "So"
    }, {
      "key": "21E1",
      "mappings": {"default": {"default": "flecha de puntos hacia arriba"}},
      "category": "So"
    }, {"key": "21E2", "mappings": {"default": {"default": "flecha de puntos"}}, "category": "So"}, {
      "key": "21E3",
      "mappings": {"default": {"default": "flecha de puntos hacia abajo"}},
      "category": "So"
    }, {
      "key": "21E4",
      "mappings": {"default": {"default": "flecha izquierda a barra"}},
      "category": "So"
    }, {
      "key": "21E5",
      "mappings": {"default": {"default": "flecha derecha a barra"}},
      "category": "So"
    }, {
      "key": "21E6",
      "mappings": {"default": {"default": "flecha vacía hacia la izquierda"}},
      "category": "So"
    }, {
      "key": "21E7",
      "mappings": {"default": {"default": "flecha vacía hacia arriba"}},
      "category": "So"
    }, {"key": "21E8", "mappings": {"default": {"default": "flecha vacía"}}, "category": "So"}, {
      "key": "21E9",
      "mappings": {"default": {"default": "flecha vacía hacia abajo"}},
      "category": "So"
    }, {
      "key": "21EA",
      "mappings": {"default": {"default": "flecha vacía hacia arriba desde barra"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "21EB",
      "mappings": {"default": {"default": "flecha blanca hacia arriba en el pedestal"}}
    }, {
      "category": "So",
      "key": "21EC",
      "mappings": {"default": {"default": "flecha blanca hacia arriba sobre pedestal con barra horizontal"}}
    }, {
      "category": "So",
      "key": "21ED",
      "mappings": {"default": {"default": "flecha blanca hacia arriba sobre pedestal con barra vertical"}}
    }, {
      "category": "So",
      "key": "21EE",
      "mappings": {"default": {"default": "flecha doble hacia arriba blanca"}}
    }, {
      "category": "So",
      "key": "21EF",
      "mappings": {"default": {"default": "flecha doble hacia arriba blanca en pedestal"}}
    }, {
      "category": "So",
      "key": "21F0",
      "mappings": {"default": {"default": "flecha blanca hacia la derecha de la pared"}}
    }, {
      "category": "So",
      "key": "21F1",
      "mappings": {"default": {"default": "flecha del noroeste a la esquina"}}
    }, {
      "category": "So",
      "key": "21F2",
      "mappings": {"default": {"default": "flecha del sudeste a la esquina"}}
    }, {
      "category": "So",
      "key": "21F3",
      "mappings": {"default": {"default": "arriba abajo flecha blanca"}}
    }, {
      "category": "Sm",
      "key": "21F4",
      "mappings": {"default": {"default": "flecha derecha con círculo pequeño"}}
    }, {
      "key": "21F5",
      "mappings": {"default": {"default": "upwards arrow to the right of downwards arrow"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "21F6",
      "mappings": {"default": {"default": "tres flechas a la derecha"}}
    }, {
      "category": "Sm",
      "key": "21F7",
      "mappings": {"default": {"default": "flecha hacia la izquierda con trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "21F8",
      "mappings": {"default": {"default": "flecha hacia la derecha con trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "21F9",
      "mappings": {"default": {"default": "flecha izquierda derecha con trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "21FA",
      "mappings": {"default": {"default": "flecha hacia la izquierda con doble trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "21FB",
      "mappings": {"default": {"default": "flecha hacia la derecha con doble trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "21FC",
      "mappings": {"default": {"default": "flecha izquierda derecha con doble trazo vertical"}}
    }, {
      "key": "21FD",
      "mappings": {"default": {"default": "flecha izquierda con punta vacía"}},
      "category": "Sm"
    }, {
      "key": "21FE",
      "mappings": {"default": {"default": "flecha izquierda con punta vacía"}},
      "category": "Sm"
    }, {
      "key": "21FF",
      "mappings": {"default": {"default": "flecha izquierda derecha con punta vacía"}},
      "category": "Sm"
    }, {"category": "So", "key": "2301", "mappings": {"default": {"default": "flecha electrica"}}}, {
      "category": "So",
      "key": "2303",
      "mappings": {"default": {"default": "flecha arriba"}}
    }, {"category": "So", "key": "2304", "mappings": {"default": {"default": "flecha hacia abajo"}}}, {
      "category": "So",
      "key": "2324",
      "mappings": {"default": {"default": "flecha arriba entre dos barras horizontales"}}
    }, {
      "category": "So",
      "key": "238B",
      "mappings": {"default": {"default": "círculo roto con la flecha del noroeste"}}
    }, {
      "category": "So",
      "key": "2794",
      "mappings": {"default": {"default": "flecha hacia la derecha con cabeza ancha"}}
    }, {
      "category": "So",
      "key": "2798",
      "mappings": {"default": {"default": "flecha Sureste pesada"}}
    }, {
      "category": "So",
      "key": "2799",
      "mappings": {"default": {"default": "flecha hacia la derecha pesada"}}
    }, {
      "category": "So",
      "key": "279A",
      "mappings": {"default": {"default": "flecha pesada del noreste"}}
    }, {
      "category": "So",
      "key": "279B",
      "mappings": {"default": {"default": "punto de dibujo Flecha hacia la derecha"}}
    }, {
      "category": "So",
      "key": "279C",
      "mappings": {"default": {"default": "flecha hacia la derecha con punta redonda redonda"}}
    }, {
      "category": "So",
      "key": "279D",
      "mappings": {"default": {"default": "flecha hacia la derecha con forma de triángulo"}}
    }, {
      "category": "So",
      "key": "279E",
      "mappings": {"default": {"default": "flecha hacia la derecha con forma de triángulo pesado"}}
    }, {
      "category": "So",
      "key": "279F",
      "mappings": {"default": {"default": "triángulo punteado hacia la flecha hacia la derecha"}}
    }, {
      "category": "So",
      "key": "27A0",
      "mappings": {"default": {"default": "flecha hacia la derecha con forma de triángulo de trazo pesado"}}
    }, {
      "category": "So",
      "key": "27A1",
      "mappings": {"default": {"default": "flecha hacia la derecha negra"}}
    }, {
      "category": "So",
      "key": "27A2",
      "mappings": {"default": {"default": "punta de flecha tridimensional hacia la derecha hacia arriba"}}
    }, {
      "category": "So",
      "key": "27A3",
      "mappings": {"default": {"default": "punta de flecha hacia la derecha con iluminación inferior en tres dimensiones"}}
    }, {
      "category": "So",
      "key": "27A4",
      "mappings": {"default": {"default": "punta de flecha negra hacia la derecha"}}
    }, {
      "category": "So",
      "key": "27A5",
      "mappings": {"default": {"default": "pesado negro curvado hacia abajo y hacia la derecha flecha"}}
    }, {
      "category": "So",
      "key": "27A6",
      "mappings": {"default": {"default": "pesado negro curvado hacia arriba y hacia la derecha flecha"}}
    }, {
      "category": "So",
      "key": "27A7",
      "mappings": {"default": {"default": "squat Black Arrow hacia la derecha"}}
    }, {
      "category": "So",
      "key": "27A8",
      "mappings": {"default": {"default": "flecha hacia la derecha de color negro cóncava puntiaguda pesada"}}
    }, {
      "category": "So",
      "key": "27A9",
      "mappings": {"default": {"default": "sombra derecha flecha blanca hacia la derecha"}}
    }, {
      "category": "So",
      "key": "27AA",
      "mappings": {"default": {"default": "sombra hacia la izquierda blanca a la izquierda"}}
    }, {
      "category": "So",
      "key": "27AB",
      "mappings": {"default": {"default": "back-Tilted Shadowed White Right Arrow"}}
    }, {
      "category": "So",
      "key": "27AC",
      "mappings": {"default": {"default": "flecha hacia la derecha blanca sombreada inclinada hacia delante"}}
    }, {
      "category": "So",
      "key": "27AD",
      "mappings": {"default": {"default": "flecha hacia la derecha pesada inferior, sombreada a la derecha, blanca"}}
    }, {
      "category": "So",
      "key": "27AE",
      "mappings": {"default": {"default": "flecha hacia la derecha blanca superior derecha sombreada pesada"}}
    }, {
      "category": "So",
      "key": "27AF",
      "mappings": {"default": {"default": "con muesca, flecha derecha hacia abajo, a la derecha, hacia la derecha"}}
    }, {
      "category": "So",
      "key": "27B1",
      "mappings": {"default": {"default": "con muescas, flecha derecha hacia arriba, a la derecha, hacia la derecha"}}
    }, {
      "category": "So",
      "key": "27B2",
      "mappings": {"default": {"default": "círculo pesado blanco flecha hacia la derecha"}}
    }, {
      "category": "So",
      "key": "27B3",
      "mappings": {"default": {"default": "flecha hacia la derecha con plumas blancas"}}
    }, {
      "category": "So",
      "key": "27B4",
      "mappings": {"default": {"default": "flecha suroriental de plumas negras"}}
    }, {
      "category": "So",
      "key": "27B5",
      "mappings": {"default": {"default": "flecha hacia la derecha con plumas negras"}}
    }, {
      "category": "So",
      "key": "27B6",
      "mappings": {"default": {"default": "flecha del noreste con plumas negras"}}
    }, {
      "category": "So",
      "key": "27B7",
      "mappings": {"default": {"default": "flecha suroriental con plumas negras pesadas"}}
    }, {
      "category": "So",
      "key": "27B8",
      "mappings": {"default": {"default": "flecha derecha pesada con plumas negras"}}
    }, {
      "category": "So",
      "key": "27B9",
      "mappings": {"default": {"default": "flecha del noreste con plumas negras pesadas"}}
    }, {
      "category": "So",
      "key": "27BA",
      "mappings": {"default": {"default": "lágrima-flecha de flecha hacia la derecha"}}
    }, {
      "category": "So",
      "key": "27BB",
      "mappings": {"default": {"default": "flecha hacia la derecha con forma de lágrima pesada"}}
    }, {
      "category": "So",
      "key": "27BC",
      "mappings": {"default": {"default": "flecha hacia la derecha de cola de cuña"}}
    }, {
      "category": "So",
      "key": "27BD",
      "mappings": {"default": {"default": "flecha hacia la derecha de cola de cuña pesada"}}
    }, {
      "category": "So",
      "key": "27BE",
      "mappings": {"default": {"default": "open-Outlined Outward Arrow"}}
    }, {
      "key": "27F0",
      "mappings": {"default": {"default": "flecha cuádruple hacia arriba"}},
      "category": "Sm"
    }, {
      "key": "27F1",
      "mappings": {"default": {"default": "flecha cuádruple hacia abajo"}},
      "category": "Sm"
    }, {
      "key": "27F2",
      "mappings": {"default": {"default": "flecha hacia la izquierda desde círculo"}},
      "category": "Sm"
    }, {
      "key": "27F3",
      "mappings": {"default": {"default": "flecha hacia la derecha desde círculo"}},
      "category": "Sm"
    }, {
      "key": "27F4",
      "mappings": {"default": {"default": "flecha con más en círculo"}},
      "category": "Sm"
    }, {
      "key": "27F5",
      "mappings": {"default": {"default": "flecha larga hacia la izquierda"}},
      "category": "Sm"
    }, {"key": "27F6", "mappings": {"default": {"default": "flecha larga"}}, "category": "Sm"}, {
      "key": "27F7",
      "mappings": {"default": {"default": "flecha larga hacia izquierda y derecha"}},
      "category": "Sm"
    }, {
      "key": "27F8",
      "mappings": {"default": {"default": "doble flecha larga hacia la izquierda"}},
      "category": "Sm"
    }, {"key": "27F9", "mappings": {"default": {"default": "doble flecha larga"}}, "category": "Sm"}, {
      "key": "27FA",
      "mappings": {"default": {"default": "doble flecha larga hacia izquierda y derecha"}},
      "category": "Sm"
    }, {
      "key": "27FB",
      "mappings": {"default": {"default": "flecha larga hacia la izquierda, desde barra"}},
      "category": "Sm"
    }, {
      "key": "27FC",
      "mappings": {"default": {"default": "flecha larga desde barra"}},
      "category": "Sm"
    }, {
      "key": "27FD",
      "mappings": {"default": {"default": "doble flecha larga hacia la izquierda desde barra"}},
      "category": "Sm"
    }, {
      "key": "27FE",
      "mappings": {"default": {"default": "doble flecha larga desde barra"}},
      "category": "Sm"
    }, {
      "key": "27FF",
      "mappings": {"default": {"default": "flecha larga en zigzag"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2900",
      "mappings": {"default": {"default": "flecha de dos cabezas hacia la derecha con trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "2901",
      "mappings": {"default": {"default": "flecha de dos cabezas hacia la derecha con doble movimiento vertical"}}
    }, {
      "category": "Sm",
      "key": "2902",
      "mappings": {"default": {"default": "doble flecha hacia la izquierda con trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "2903",
      "mappings": {"default": {"default": "doble flecha hacia la derecha con trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "2904",
      "mappings": {"default": {"default": "izquierda Derecha Doble Flecha con Trazo Vertical"}}
    }, {
      "key": "2905",
      "mappings": {"default": {"default": "rightwards two headed arrow from bar"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2906",
      "mappings": {"default": {"default": "doble flecha hacia la izquierda de la barra"}}
    }, {
      "category": "Sm",
      "key": "2907",
      "mappings": {"default": {"default": "flecha doble hacia la derecha de la barra"}}
    }, {
      "category": "Sm",
      "key": "2908",
      "mappings": {"default": {"default": "flecha hacia abajo con trazo horizontal"}}
    }, {
      "category": "Sm",
      "key": "2909",
      "mappings": {"default": {"default": "flecha hacia arriba con trazo horizontal"}}
    }, {
      "category": "Sm",
      "key": "290A",
      "mappings": {"default": {"default": "flecha triple hacia arriba"}}
    }, {
      "category": "Sm",
      "key": "290B",
      "mappings": {"default": {"default": "triple flecha hacia abajo"}}
    }, {
      "key": "290C",
      "mappings": {"default": {"default": "leftwards double dash arrow"}},
      "category": "Sm"
    }, {
      "key": "290D",
      "mappings": {"default": {"default": "rightwards double dash arrow"}},
      "category": "Sm"
    }, {
      "key": "290E",
      "mappings": {"default": {"default": "leftwards triple dash arrow"}},
      "category": "Sm"
    }, {
      "key": "290F",
      "mappings": {"default": {"default": "rightwards triple dash arrow"}},
      "category": "Sm"
    }, {
      "key": "2910",
      "mappings": {"default": {"default": "rightwards two headed triple dash arrow"}},
      "category": "Sm"
    }, {
      "key": "2911",
      "mappings": {"default": {"default": "rightwards arrow with dotted stem"}},
      "category": "Sm"
    }, {"key": "2912", "mappings": {"default": {"default": "upwards arrow to bar"}}, "category": "Sm"}, {
      "key": "2913",
      "mappings": {"default": {"default": "downwards arrow to bar"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2914",
      "mappings": {"default": {"default": "flecha hacia la derecha con cola con trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "2915",
      "mappings": {"default": {"default": "flecha hacia la derecha con la cola con doble trazo vertical"}}
    }, {
      "key": "2916",
      "mappings": {"default": {"default": "rightwards two headed arrow with tail"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2917",
      "mappings": {"default": {"default": "flecha de dos cabezas hacia la derecha con cola con trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "2918",
      "mappings": {"default": {"default": "flecha de dos cabezas hacia la derecha con cola con doble movimiento vertical"}}
    }, {
      "key": "2919",
      "mappings": {"default": {"default": "cola de flecha hacia la izquierda"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "291A",
      "mappings": {"default": {"default": "flecha hacia la derecha"}}
    }, {
      "key": "291B",
      "mappings": {"default": {"default": "cola de doble flecha hacia la izquierda"}},
      "category": "Sm"
    }, {
      "key": "291C",
      "mappings": {"default": {"default": "cola de doble flecha hacia la derecha"}},
      "category": "Sm"
    }, {
      "key": "291D",
      "mappings": {"default": {"default": "flecha hacia diamante relleno a la izquierda"}},
      "category": "Sm"
    }, {
      "key": "291E",
      "mappings": {"default": {"default": "flecha hacia diamante a la derecha"}},
      "category": "Sm"
    }, {
      "key": "291F",
      "mappings": {"default": {"default": "flecha desde barra hacia diamante a la izquierda"}},
      "category": "Sm"
    }, {
      "key": "2920",
      "mappings": {"default": {"default": "flecha desde barra hacia diamante a la derecha"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2921",
      "mappings": {"default": {"default": "flechas del noroeste y sureste"}}
    }, {
      "category": "Sm",
      "key": "2922",
      "mappings": {"default": {"default": "flecha noreste y sudoeste"}}
    }, {
      "key": "2923",
      "mappings": {"default": {"default": "flecha a noroeste con garfio"}},
      "category": "Sm"
    }, {
      "key": "2924",
      "mappings": {"default": {"default": "flecha a nordeste con garfio"}},
      "category": "Sm"
    }, {
      "key": "2925",
      "mappings": {"default": {"default": "flecha a sureste con garfio"}},
      "category": "Sm"
    }, {
      "key": "2926",
      "mappings": {"default": {"default": "flecha a suroeste con garfio"}},
      "category": "Sm"
    }, {
      "key": "2927",
      "mappings": {"default": {"default": "flechas a noroeste y nordeste"}},
      "category": "Sm"
    }, {
      "key": "2928",
      "mappings": {"default": {"default": "flechas a nordeste y sudeste"}},
      "category": "Sm"
    }, {
      "key": "2929",
      "mappings": {"default": {"default": "flechas a suddeste y sudoeste"}},
      "category": "Sm"
    }, {
      "key": "292A",
      "mappings": {"default": {"default": "flechas a sudoeste y noroeste"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "292D",
      "mappings": {"default": {"default": "flecha Sureste Cruzando Flecha Noreste"}}
    }, {
      "category": "Sm",
      "key": "292E",
      "mappings": {"default": {"default": "north East Arrow Crossing South East Arrow"}}
    }, {
      "category": "Sm",
      "key": "292F",
      "mappings": {"default": {"default": "caída en diagonal que cruza la flecha noreste"}}
    }, {
      "category": "Sm",
      "key": "2930",
      "mappings": {"default": {"default": "creciente Cruce de la Cruz del Sureste de la Flecha"}}
    }, {
      "category": "Sm",
      "key": "2931",
      "mappings": {"default": {"default": "north East Arrow Crossing North West Arrow"}}
    }, {
      "category": "Sm",
      "key": "2932",
      "mappings": {"default": {"default": "north West Arrow Crossing North East Arrow"}}
    }, {"key": "2933", "mappings": {"default": {"default": "flecha ondulada"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "2934",
      "mappings": {"default": {"default": "flecha que apunta hacia la derecha y luego curva hacia arriba"}}
    }, {
      "key": "2935",
      "mappings": {"default": {"default": "flecha hacia la derecha que gira hacia abajo"}},
      "category": "Sm"
    }, {
      "key": "2936",
      "mappings": {"default": {"default": "flecha hacia abajo que gira hacia la izquierda"}},
      "category": "Sm"
    }, {
      "key": "2937",
      "mappings": {"default": {"default": "flecha hacia abajo que gira hacia la derecha"}},
      "category": "Sm"
    }, {
      "key": "2938",
      "mappings": {"default": {"default": "flecha semicircular a la derecha en sentido horario"}},
      "category": "Sm"
    }, {
      "key": "2939",
      "mappings": {"default": {"default": "flecha semicircular a la izquierda en sentido antihorario"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "293A",
      "mappings": {"default": {"default": "arco superior flecha hacia la izquierda"}}
    }, {
      "category": "Sm",
      "key": "293B",
      "mappings": {"default": {"default": "arco inferior flecha hacia la izquierda"}}
    }, {"key": "293C", "mappings": {"default": {"default": "giro negativo"}}, "category": "Sm"}, {
      "key": "293D",
      "mappings": {"default": {"default": "giro positivo"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "293E",
      "mappings": {"default": {"default": "flecha hacia la derecha semicircular inferior derecha"}}
    }, {
      "category": "Sm",
      "key": "293F",
      "mappings": {"default": {"default": "flecha inferior izquierda semicircular en sentido antihorario"}}
    }, {
      "category": "Sm",
      "key": "2940",
      "mappings": {"default": {"default": "círculo cerrado en sentido contrario a las agujas del reloj"}}
    }, {
      "category": "Sm",
      "key": "2941",
      "mappings": {"default": {"default": "círculo cerrado en el sentido de las agujas del reloj"}}
    }, {
      "category": "Sm",
      "key": "2942",
      "mappings": {"default": {"default": "flecha hacia la derecha sobre la flecha corta hacia la izquierda"}}
    }, {
      "category": "Sm",
      "key": "2943",
      "mappings": {"default": {"default": "flecha hacia la izquierda sobre la flecha corta hacia la derecha"}}
    }, {
      "category": "Sm",
      "key": "2944",
      "mappings": {"default": {"default": "flecha corta hacia la derecha arriba flecha hacia la izquierda"}}
    }, {
      "key": "2945",
      "mappings": {"default": {"default": "flecha con más suscrito"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2946",
      "mappings": {"default": {"default": "flecha hacia la izquierda con más abajo"}}
    }, {
      "category": "Sm",
      "key": "2947",
      "mappings": {"default": {"default": "flecha hacia la derecha a través de X"}}
    }, {
      "key": "2948",
      "mappings": {"default": {"default": "flecha hacia la izquierda a través de círculo"}},
      "category": "Sm"
    }, {
      "key": "2949",
      "mappings": {"default": {"default": "dos cabezas de flecha hacia arriba desde círculo"}},
      "category": "Sm"
    }, {"key": "2970", "mappings": {"default": {"default": "round implies"}}, "category": "Sm"}, {
      "key": "2971",
      "mappings": {"default": {"default": "flecha con igual"}},
      "category": "Sm"
    }, {"key": "2972", "mappings": {"default": {"default": "flecha con tilde"}}, "category": "Sm"}, {
      "key": "2973",
      "mappings": {"default": {"default": "tilde con flecha hacia la izquierda"}},
      "category": "Sm"
    }, {"key": "2974", "mappings": {"default": {"default": "tilde con flecha"}}, "category": "Sm"}, {
      "key": "2975",
      "mappings": {"default": {"default": "flecha con casi igual a"}},
      "category": "Sm"
    }, {
      "key": "2976",
      "mappings": {"default": {"default": "flecha hacia la izquierda con menor que"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2977",
      "mappings": {"default": {"default": "flecha hacia la izquierda a través de menos de"}}
    }, {"key": "2978", "mappings": {"default": {"default": "flecha con mayor que"}}, "category": "Sm"}, {
      "key": "2979",
      "mappings": {"default": {"default": "flecha hacia la izquierda con incluido"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "297A",
      "mappings": {"default": {"default": "flecha hacia la izquierda a través del subconjunto"}}
    }, {
      "key": "297B",
      "mappings": {"default": {"default": "flecha hacia la izquierda con contiene"}},
      "category": "Sm"
    }, {
      "key": "29B3",
      "mappings": {"default": {"default": "conjunto vacío con flecha"}},
      "category": "Sm"
    }, {
      "key": "29B4",
      "mappings": {"default": {"default": "conjunto vacío con flecha inversa"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29BD",
      "mappings": {"default": {"default": "flecha arriba a través del círculo"}}
    }, {
      "category": "Sm",
      "key": "29EA",
      "mappings": {"default": {"default": "diamante negro con flecha hacia abajo"}}
    }, {
      "category": "Sm",
      "key": "29EC",
      "mappings": {"default": {"default": "círculo blanco con flecha hacia abajo"}}
    }, {
      "category": "Sm",
      "key": "29ED",
      "mappings": {"default": {"default": "círculo negro con flecha hacia abajo"}}
    }, {
      "key": "2A17",
      "mappings": {"default": {"default": "integral con flecha hacia la izquierda con garfio"}},
      "category": "Sm"
    }, {
      "category": "So",
      "key": "2B00",
      "mappings": {"default": {"default": "flecha blanca del noreste"}}
    }, {
      "category": "So",
      "key": "2B01",
      "mappings": {"default": {"default": "flecha blanca del noroeste"}}
    }, {
      "category": "So",
      "key": "2B02",
      "mappings": {"default": {"default": "flecha blanca del sureste"}}
    }, {
      "category": "So",
      "key": "2B03",
      "mappings": {"default": {"default": "sudoeste flecha blanca"}}
    }, {
      "category": "So",
      "key": "2B04",
      "mappings": {"default": {"default": "izquierda derecha flecha blanca"}}
    }, {
      "category": "So",
      "key": "2B05",
      "mappings": {"default": {"default": "flecha negra hacia la izquierda"}}
    }, {
      "category": "So",
      "key": "2B06",
      "mappings": {"default": {"default": "flecha negra hacia arriba"}}
    }, {
      "category": "So",
      "key": "2B07",
      "mappings": {"default": {"default": "flecha negra hacia abajo"}}
    }, {
      "category": "So",
      "key": "2B08",
      "mappings": {"default": {"default": "flecha negra del noreste"}}
    }, {
      "category": "So",
      "key": "2B09",
      "mappings": {"default": {"default": "flecha negra del noroeste"}}
    }, {
      "category": "So",
      "key": "2B0A",
      "mappings": {"default": {"default": "flecha negra sur este"}}
    }, {
      "category": "So",
      "key": "2B0B",
      "mappings": {"default": {"default": "sudoeste flecha negra"}}
    }, {
      "category": "So",
      "key": "2B0C",
      "mappings": {"default": {"default": "izquierda derecha flecha negra"}}
    }, {
      "category": "So",
      "key": "2B0D",
      "mappings": {"default": {"default": "arriba abajo flecha negra"}}
    }, {
      "category": "So",
      "key": "2B0E",
      "mappings": {"default": {"default": "flecha hacia la derecha con la punta hacia abajo"}}
    }, {
      "category": "So",
      "key": "2B0F",
      "mappings": {"default": {"default": "flecha hacia la derecha con la punta hacia arriba"}}
    }, {
      "category": "So",
      "key": "2B10",
      "mappings": {"default": {"default": "flecha hacia la izquierda con la punta hacia abajo"}}
    }, {
      "category": "So",
      "key": "2B11",
      "mappings": {"default": {"default": "flecha hacia la izquierda con la punta hacia arriba"}}
    }, {
      "category": "Sm",
      "key": "2B30",
      "mappings": {"default": {"default": "flecha izquierda con círculo pequeño"}}
    }, {
      "category": "Sm",
      "key": "2B31",
      "mappings": {"default": {"default": "tres flechas a la izquierda"}}
    }, {
      "category": "Sm",
      "key": "2B32",
      "mappings": {"default": {"default": "flecha izquierda con un círculo más"}}
    }, {
      "category": "Sm",
      "key": "2B33",
      "mappings": {"default": {"default": "flecha de flecha larga hacia la izquierda"}}
    }, {
      "category": "Sm",
      "key": "2B34",
      "mappings": {"default": {"default": "flecha de dos cabezas hacia la izquierda con trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "2B35",
      "mappings": {"default": {"default": "flecha de dos cabezas hacia la izquierda con doble movimiento vertical"}}
    }, {
      "category": "Sm",
      "key": "2B36",
      "mappings": {"default": {"default": "flecha de dos cabezas hacia la izquierda de la barra"}}
    }, {
      "category": "Sm",
      "key": "2B37",
      "mappings": {"default": {"default": "flecha triple de dos cabezas hacia la izquierda"}}
    }, {
      "category": "Sm",
      "key": "2B38",
      "mappings": {"default": {"default": "flecha hacia la izquierda con tallo punteado"}}
    }, {
      "category": "Sm",
      "key": "2B39",
      "mappings": {"default": {"default": "flecha hacia la izquierda con cola con trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "2B3A",
      "mappings": {"default": {"default": "flecha hacia la izquierda con cola con doble trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "2B3B",
      "mappings": {"default": {"default": "flecha de dos cabezas hacia la izquierda con cola"}}
    }, {
      "category": "Sm",
      "key": "2B3C",
      "mappings": {"default": {"default": "flecha de dos cabezas hacia la izquierda con cola con trazo vertical"}}
    }, {
      "category": "Sm",
      "key": "2B3D",
      "mappings": {"default": {"default": "flecha de dos cabezas hacia la izquierda con cola con doble movimiento vertical"}}
    }, {
      "category": "Sm",
      "key": "2B3E",
      "mappings": {"default": {"default": "flecha hacia la izquierda a través de X"}}
    }, {
      "category": "Sm",
      "key": "2B3F",
      "mappings": {"default": {"default": "ola flecha apuntando directamente a la izquierda"}}
    }, {
      "category": "Sm",
      "key": "2B40",
      "mappings": {"default": {"default": "signo de igual flecha arriba a la izquierda"}}
    }, {
      "category": "Sm",
      "key": "2B41",
      "mappings": {"default": {"default": "operador de tilde inverso sobre la flecha hacia la izquierda"}}
    }, {
      "category": "Sm",
      "key": "2B42",
      "mappings": {"default": {"default": "flecha hacia la izquierda sobre el reverso casi igual a"}}
    }, {
      "category": "Sm",
      "key": "2B43",
      "mappings": {"default": {"default": "flecha hacia la derecha a través de mayor-que"}}
    }, {
      "category": "Sm",
      "key": "2B44",
      "mappings": {"default": {"default": "flecha hacia la derecha a través de Superset"}}
    }, {
      "category": "So",
      "key": "2B45",
      "mappings": {"default": {"default": "flecha cuádruple hacia la izquierda"}}
    }, {
      "category": "So",
      "key": "2B46",
      "mappings": {"default": {"default": "flecha cuádruple hacia la derecha"}}
    }, {
      "category": "Sm",
      "key": "2B47",
      "mappings": {"default": {"default": "operador de tilde inverso sobre la flecha hacia la derecha"}}
    }, {
      "category": "Sm",
      "key": "2B48",
      "mappings": {"default": {"default": "flecha hacia la derecha sobre el reverso casi igual a"}}
    }, {
      "category": "Sm",
      "key": "2B49",
      "mappings": {"default": {"default": "operador de tilde sobre la flecha hacia la izquierda"}}
    }, {
      "category": "Sm",
      "key": "2B4A",
      "mappings": {"default": {"default": "flecha hacia la izquierda por encima de casi igual a"}}
    }, {
      "category": "Sm",
      "key": "2B4B",
      "mappings": {"default": {"default": "flecha hacia la izquierda sobre el operador de tilde inverso"}}
    }, {
      "category": "Sm",
      "key": "2B4C",
      "mappings": {"default": {"default": "flecha hacia la derecha sobre el operador de tilde inverso"}}
    }, {
      "category": "Sm",
      "key": "FFE9",
      "mappings": {"default": {"default": "flecha hacia la izquierda de medio ancho"}}
    }, {
      "category": "Sm",
      "key": "FFEA",
      "mappings": {"default": {"default": "flecha ascendente de medio ancho"}}
    }, {
      "category": "Sm",
      "key": "FFEB",
      "mappings": {"default": {"default": "flecha de medio ancho hacia la derecha"}}
    }, {"category": "Sm", "key": "FFEC", "mappings": {"default": {"default": "flecha hacia abajo de medio ancho"}}}]
  ,
  "es/symbols/math_characters.js":
    [{"locale": "es"}, {
      "key": "2113",
      "mappings": {"default": {"default": "script l"}},
      "category": "Ll"
    }, {"key": "2118", "mappings": {"default": {"default": "p de weierstrass"}}, "category": "Sm"}, {
      "category": "Ll",
      "key": "213C",
      "mappings": {"default": {"default": "negrita de pizarra pi"}}
    }, {
      "category": "Ll",
      "key": "213D",
      "mappings": {"default": {"default": "negrita de pizarra gamma"}}
    }, {
      "category": "Lu",
      "key": "213E",
      "mappings": {"default": {"default": "negrita de pizarra mayúscula Gamma"}}
    }, {
      "category": "Lu",
      "key": "213F",
      "mappings": {"default": {"default": "negrita de pizarra mayúscula Pi"}}
    }, {
      "category": "Sm",
      "key": "2140",
      "mappings": {"default": {"default": "negrita de pizarra suma de n-ary"}}
    }, {"key": "2145", "mappings": {"default": {"default": "mayúscula D"}}, "category": "Lu"}, {
      "key": "2146",
      "mappings": {"default": {"default": "d"}},
      "category": "Ll"
    }, {"key": "2147", "mappings": {"default": {"default": "e"}}, "category": "Ll"}, {
      "key": "2148",
      "mappings": {"default": {"default": "i"}},
      "category": "Ll"
    }, {
      "category": "Ll",
      "key": "2149",
      "mappings": {"default": {"default": "negrita de pizarra cursiva j"}}
    }, {
      "category": "Ll",
      "mappings": {"default": {"default": "cursiva sin punto i"}},
      "key": "1D6A4"
    }, {"category": "Ll", "mappings": {"default": {"default": "cursiva sin punto j"}}, "key": "1D6A5"}]
  ,
  "es/symbols/math_delimiters.js":
    [{"locale": "es"}, {
      "key": "0028",
      "mappings": {"default": {"default": "paréntesis izquierdo", "defaultMP": "paréntesis"}},
      "category": "Ps"
    }, {
      "key": "0029",
      "mappings": {"default": {"default": "paréntesis derecho", "defaultMP": "cierra paréntesis"}},
      "category": "Pe"
    }, {
      "key": "005B",
      "mappings": {"default": {"default": "corchete izquierdo", "defaultMP": "abre corchetes"}},
      "category": "Ps"
    }, {
      "key": "005D",
      "mappings": {"default": {"default": "corchete derecho", "defaultMP": "cierra corchetes"}},
      "category": "Pe"
    }, {
      "key": "007B",
      "mappings": {"default": {"default": "llave izquierda", "defaultMP": "abre llaves"}},
      "category": "Ps"
    }, {
      "key": "007D",
      "mappings": {"default": {"default": "llaves derecha", "defaultMP": "cierra llaves"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "2045",
      "mappings": {"default": {"default": "soporte cuadrado izquierdo con pluma"}}
    }, {
      "category": "Pe",
      "key": "2046",
      "mappings": {"default": {"default": "soporte cuadrado derecho con pluma"}}
    }, {
      "key": "2308",
      "mappings": {"default": {"default": "esquina superior izquierda"}},
      "category": "Sm"
    }, {
      "key": "2309",
      "mappings": {"default": {"default": "esquina superior derecha"}},
      "category": "Sm"
    }, {
      "key": "230A",
      "mappings": {"default": {"default": "esquina inferior izquierda"}},
      "category": "Sm"
    }, {
      "key": "230B",
      "mappings": {"default": {"default": "esquina inferior derecha"}},
      "category": "Sm"
    }, {
      "key": "230C",
      "mappings": {"default": {"default": "corte inferior derecho"}},
      "category": "So"
    }, {
      "key": "230D",
      "mappings": {"default": {"default": "corte inferior izquierdo"}},
      "category": "So"
    }, {
      "key": "230E",
      "mappings": {"default": {"default": "corte superior derecho "}},
      "category": "So"
    }, {
      "key": "230F",
      "mappings": {"default": {"default": "corte superior izquierdo"}},
      "category": "So"
    }, {
      "key": "231C",
      "mappings": {"default": {"default": "ángulo superior izquierdo"}},
      "category": "So"
    }, {
      "key": "231D",
      "mappings": {"default": {"default": "ángulo superior derecho"}},
      "category": "So"
    }, {
      "key": "231E",
      "mappings": {"default": {"default": "ángulo inferior izquierdo"}},
      "category": "So"
    }, {
      "key": "231F",
      "mappings": {"default": {"default": "ángulo inferior derecho"}},
      "category": "So"
    }, {
      "key": "2320",
      "mappings": {"default": {"default": "mitad superior de integral"}},
      "category": "Sm"
    }, {
      "key": "2321",
      "mappings": {"default": {"default": "mitad inferior de integral"}},
      "category": "Sm"
    }, {
      "key": "2329",
      "mappings": {"default": {"default": "paréntesis angular izquierdo"}},
      "category": "Ps"
    }, {
      "key": "232A",
      "mappings": {"default": {"default": "paréntesis angular derecho"}},
      "category": "Pe"
    }, {
      "category": "Sm",
      "key": "239B",
      "mappings": {"default": {"default": "paréntesis izquierdo gancho superior"}}
    }, {
      "category": "Sm",
      "key": "239C",
      "mappings": {"default": {"default": "extensión del paréntesis izquierdo"}}
    }, {
      "category": "Sm",
      "key": "239D",
      "mappings": {"default": {"default": "paréntesis inferior gancho inferior"}}
    }, {
      "category": "Sm",
      "key": "239E",
      "mappings": {"default": {"default": "paréntesis derecho gancho superior"}}
    }, {
      "category": "Sm",
      "key": "239F",
      "mappings": {"default": {"default": "extensión del paréntesis derecho"}}
    }, {
      "category": "Sm",
      "key": "23A0",
      "mappings": {"default": {"default": "paréntesis inferior gancho inferior"}}
    }, {
      "category": "Sm",
      "key": "23A1",
      "mappings": {"default": {"default": "esquina cuadrada izquierda esquina superior"}}
    }, {
      "category": "Sm",
      "key": "23A2",
      "mappings": {"default": {"default": "extensión del soporte cuadrado izquierdo"}}
    }, {
      "category": "Sm",
      "key": "23A3",
      "mappings": {"default": {"default": "esquina cuadrada izquierda esquina inferior"}}
    }, {
      "category": "Sm",
      "key": "23A4",
      "mappings": {"default": {"default": "esquina cuadrada derecha esquina superior"}}
    }, {
      "category": "Sm",
      "key": "23A5",
      "mappings": {"default": {"default": "extensión de soporte cuadrado derecho"}}
    }, {
      "category": "Sm",
      "key": "23A6",
      "mappings": {"default": {"default": "esquina cuadrada derecha esquina inferior"}}
    }, {
      "category": "Sm",
      "key": "23A7",
      "mappings": {"default": {"default": "gancho superior rizado izquierdo"}}
    }, {
      "category": "Sm",
      "key": "23A8",
      "mappings": {"default": {"default": "pedazo medio rizado izquierdo"}}
    }, {
      "category": "Sm",
      "key": "23A9",
      "mappings": {"default": {"default": "soporte izquierdo rizado gancho inferior"}}
    }, {
      "category": "Sm",
      "key": "23AA",
      "mappings": {"default": {"default": "extensión de soporte rizado"}}
    }, {
      "category": "Sm",
      "key": "23AB",
      "mappings": {"default": {"default": "gancho superior rizado derecho"}}
    }, {
      "category": "Sm",
      "key": "23AC",
      "mappings": {"default": {"default": "pedazo medio de soporte rizado derecho"}}
    }, {
      "category": "Sm",
      "key": "23AD",
      "mappings": {"default": {"default": "abrazadera inferior derecha rizado"}}
    }, {"category": "Sm", "key": "23AE", "mappings": {"default": {"default": "extensión integral"}}}, {
      "category": "Sm",
      "key": "23AF",
      "mappings": {"default": {"default": "extensión de línea horizontal"}}
    }, {
      "key": "23B0",
      "mappings": {"default": {"default": "izquierda arriba o derecho debajo sección llave"}},
      "category": "Sm"
    }, {
      "key": "23B1",
      "mappings": {"default": {"default": "derecha arriba o izquierdo debajo sección llave"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "23B2", "mappings": {"default": {"default": "suma superior"}}}, {
      "category": "Sm",
      "key": "23B3",
      "mappings": {"default": {"default": "parte inferior de la suma"}}
    }, {"key": "23B4", "mappings": {"default": {"default": "corchete superior"}}, "category": "So"}, {
      "key": "23B5",
      "mappings": {"default": {"default": "corchete inferior"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "23B6",
      "mappings": {"default": {"default": "soporte cuadrado inferior sobre soporte cuadrado superior"}}
    }, {
      "category": "So",
      "key": "23B7",
      "mappings": {"default": {"default": "parte inferior del símbolo radical"}}
    }, {
      "category": "So",
      "key": "23B8",
      "mappings": {"default": {"default": "línea de caja vertical izquierda"}}
    }, {
      "category": "So",
      "key": "23B9",
      "mappings": {"default": {"default": "línea de caja vertical derecha"}}
    }, {"key": "23DC", "mappings": {"default": {"default": "paréntesis superior"}}, "category": "Sm"}, {
      "key": "23DD",
      "mappings": {"default": {"default": "paréntesis inferior"}},
      "category": "Sm"
    }, {
      "key": "23DE",
      "mappings": {"default": {"default": "corchete rizado superior"}},
      "category": "Sm"
    }, {
      "key": "23DF",
      "mappings": {"default": {"default": "corchete rizado inferior"}},
      "category": "Sm"
    }, {
      "key": "23E0",
      "mappings": {"default": {"default": "corchete superior en tortuga"}},
      "category": "Sm"
    }, {
      "key": "23E1",
      "mappings": {"default": {"default": "corchete inferior en tortuga"}},
      "category": "Sm"
    }, {
      "category": "Ps",
      "key": "2768",
      "mappings": {"default": {"default": "ornamento de paréntesis izquierdo medio"}}
    }, {
      "category": "Pe",
      "key": "2769",
      "mappings": {"default": {"default": "adorno de paréntesis medio derecho"}}
    }, {
      "category": "Ps",
      "key": "276A",
      "mappings": {"default": {"default": "adorno paréntesis izquierdo aplanado medio"}}
    }, {
      "category": "Pe",
      "key": "276B",
      "mappings": {"default": {"default": "adorno paréntesis derecho medio aplanado"}}
    }, {
      "category": "Ps",
      "key": "276C",
      "mappings": {"default": {"default": "adorno de soporte de ángulo medio apuntando hacia la izquierda"}}
    }, {
      "category": "Pe",
      "key": "276D",
      "mappings": {"default": {"default": "adorno de soporte de ángulo medio apuntando hacia la derecha"}}
    }, {
      "category": "Ps",
      "key": "276E",
      "mappings": {"default": {"default": "ornamento de comillas en el ángulo que apunta hacia la izquierda"}}
    }, {
      "category": "Pe",
      "key": "276F",
      "mappings": {"default": {"default": "ornamento de comillas en ángulo recto que apunta hacia la derecha"}}
    }, {
      "category": "Ps",
      "key": "2770",
      "mappings": {"default": {"default": "ornamento pesado del soporte del ángulo que señala a la izquierda"}}
    }, {
      "category": "Pe",
      "key": "2771",
      "mappings": {"default": {"default": "ornamento pesado del soporte del ángulo que señala hacia la derecha"}}
    }, {
      "category": "Ps",
      "key": "2772",
      "mappings": {"default": {"default": "ornamento de soporte de concha de tortuga izquierda ligera"}}
    }, {
      "category": "Pe",
      "key": "2773",
      "mappings": {"default": {"default": "ornamento de corchete derecho tortuga derecha"}}
    }, {
      "category": "Ps",
      "key": "2774",
      "mappings": {"default": {"default": "ornamento de soporte rizado izquierdo medio"}}
    }, {
      "category": "Pe",
      "key": "2775",
      "mappings": {"default": {"default": "ornamento de soporte rizado derecho medio"}}
    }, {
      "key": "27C5",
      "mappings": {"default": {"default": "símbolo delimitador izquierdo de bolsa de s-formó"}},
      "category": "Ps"
    }, {
      "key": "27C6",
      "mappings": {"default": {"default": "símbolo delimitador derecho de bolsa de s-formó"}},
      "category": "Pe"
    }, {
      "key": "27E6",
      "mappings": {"default": {"default": "corchete vacío izquierdo"}},
      "category": "Ps"
    }, {
      "key": "27E7",
      "mappings": {"default": {"default": "corchete vacío derecho"}},
      "category": "Pe"
    }, {
      "key": "27E8",
      "mappings": {"default": {"default": "corchete agudo izquierdo"}},
      "category": "Ps"
    }, {
      "key": "27E9",
      "mappings": {"default": {"default": "corchete agudo derecho"}},
      "category": "Pe"
    }, {
      "key": "27EA",
      "mappings": {"default": {"default": "doble corchete agudo izquierdo"}},
      "category": "Ps"
    }, {
      "key": "27EB",
      "mappings": {"default": {"default": "doble corchete agudo derecho"}},
      "category": "Pe"
    }, {
      "key": "27EC",
      "mappings": {"default": {"default": "corchete de tortuga vacío izquierdo"}},
      "category": "Ps"
    }, {
      "key": "27ED",
      "mappings": {"default": {"default": "corchete de tortuga vacío derecho "}},
      "category": "Pe"
    }, {
      "key": "27EE",
      "mappings": {"default": {"default": "paréntesis plano izquierdo"}},
      "category": "Ps"
    }, {
      "key": "27EF",
      "mappings": {"default": {"default": "paréntesis plano derecho"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "2983",
      "mappings": {"default": {"default": "soporte rizado blanco izquierdo"}}
    }, {
      "category": "Pe",
      "key": "2984",
      "mappings": {"default": {"default": "derecha rizado blanco soporte"}}
    }, {"key": "2985", "mappings": {"default": {"default": "left white paren"}}, "category": "Ps"}, {
      "key": "2986",
      "mappings": {"default": {"default": "right white paren"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "2987",
      "mappings": {"default": {"default": "notación Z izquierda soporte de imagen"}}
    }, {
      "category": "Pe",
      "key": "2988",
      "mappings": {"default": {"default": "corchete de imagen derecha de notación Z"}}
    }, {
      "key": "2989",
      "mappings": {"default": {"default": "z notation left binding bracket"}},
      "category": "Ps"
    }, {
      "key": "298A",
      "mappings": {"default": {"default": "z notation right binding bracket"}},
      "category": "Pe"
    }, {
      "key": "298B",
      "mappings": {"default": {"default": "left bracket with underbar"}},
      "category": "Ps"
    }, {
      "key": "298C",
      "mappings": {"default": {"default": "right bracket with underbar"}},
      "category": "Pe"
    }, {
      "key": "298D",
      "mappings": {"default": {"default": "left bracket with tick in top corner"}},
      "category": "Ps"
    }, {
      "key": "298E",
      "mappings": {"default": {"default": "right bracket with tick in bottom corner"}},
      "category": "Pe"
    }, {
      "key": "298F",
      "mappings": {"default": {"default": "left bracket with tick in bottom corner"}},
      "category": "Ps"
    }, {
      "key": "2990",
      "mappings": {"default": {"default": "right bracket with tick in top corner"}},
      "category": "Pe"
    }, {
      "key": "2991",
      "mappings": {"default": {"default": "left angle bracket with dot"}},
      "category": "Ps"
    }, {
      "key": "2992",
      "mappings": {"default": {"default": "right angle bracket with dot"}},
      "category": "Pe"
    }, {
      "key": "2993",
      "mappings": {"default": {"default": "left arc less than bracket"}},
      "category": "Ps"
    }, {
      "key": "2994",
      "mappings": {"default": {"default": "right arc greater than bracket"}},
      "category": "Pe"
    }, {
      "key": "2995",
      "mappings": {"default": {"default": "double left arc greater than bracket"}},
      "category": "Ps"
    }, {
      "key": "2996",
      "mappings": {"default": {"default": "double right arc less than bracket"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "2997",
      "mappings": {"default": {"default": "soporte de concha de tortuga negra izquierda"}}
    }, {
      "category": "Pe",
      "key": "2998",
      "mappings": {"default": {"default": "abrazadera de concha de tortuga negra derecha"}}
    }, {
      "category": "Ps",
      "key": "29D8",
      "mappings": {"default": {"default": "valla izquierda ondulada"}}
    }, {
      "category": "Pe",
      "key": "29D9",
      "mappings": {"default": {"default": "valla derecha ondulada"}}
    }, {
      "key": "29DA",
      "mappings": {"default": {"default": "left double wiggly fence"}},
      "category": "Ps"
    }, {
      "key": "29DB",
      "mappings": {"default": {"default": "right double wiggly fence"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "29FC",
      "mappings": {"default": {"default": "abrazadera de ángulo curvo a la izquierda"}}
    }, {
      "category": "Pe",
      "key": "29FD",
      "mappings": {"default": {"default": "abrazadera de ángulo curvo orientada hacia la derecha"}}
    }, {
      "category": "Ps",
      "key": "2E22",
      "mappings": {"default": {"default": "soporte superior izquierdo superior"}}
    }, {
      "category": "Pe",
      "key": "2E23",
      "mappings": {"default": {"default": "medio soporte superior derecho"}}
    }, {
      "category": "Ps",
      "key": "2E24",
      "mappings": {"default": {"default": "abrazadera inferior izquierda"}}
    }, {
      "category": "Pe",
      "key": "2E25",
      "mappings": {"default": {"default": "medio soporte inferior derecho"}}
    }, {
      "category": "Ps",
      "key": "2E26",
      "mappings": {"default": {"default": "soporte U lateral izquierdo"}}
    }, {
      "category": "Pe",
      "key": "2E27",
      "mappings": {"default": {"default": "lado derecho U soporte"}}
    }, {
      "category": "Ps",
      "key": "2E28",
      "mappings": {"default": {"default": "doble paréntesis izquierdo"}}
    }, {
      "category": "Pe",
      "key": "2E29",
      "mappings": {"default": {"default": "paréntesis doble derecho"}}
    }, {
      "category": "Ps",
      "key": "3008",
      "mappings": {"default": {"default": "soporte de ángulo izquierdo"}}
    }, {
      "category": "Pe",
      "key": "3009",
      "mappings": {"default": {"default": "soporte de ángulo recto"}}
    }, {
      "key": "300A",
      "mappings": {"default": {"default": "paréntesis angular doble"}},
      "category": "Ps"
    }, {
      "key": "300B",
      "mappings": {
        "default": {
          "default": "paréntesis angular doble derecho",
          "defaultMP": "cierra paréntesis angular doble"
        }
      },
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "300C",
      "mappings": {"default": {"default": "soporte de la esquina izquierda"}}
    }, {
      "category": "Pe",
      "key": "300D",
      "mappings": {"default": {"default": "soporte de esquina derecha"}}
    }, {
      "category": "Ps",
      "key": "300E",
      "mappings": {"default": {"default": "soporte de esquina blanco izquierdo"}}
    }, {
      "category": "Pe",
      "key": "300F",
      "mappings": {"default": {"default": "soporte de esquina blanco derecho"}}
    }, {
      "category": "Ps",
      "key": "3010",
      "mappings": {"default": {"default": "soporte Lenticular Izquierdo Negro"}}
    }, {
      "category": "Pe",
      "key": "3011",
      "mappings": {"default": {"default": "soporte Lenticular Negro Derecho"}}
    }, {
      "key": "3014",
      "mappings": {"default": {"default": "left tortoise shell bracket"}},
      "category": "Ps"
    }, {
      "key": "3015",
      "mappings": {"default": {"default": "right tortoise shell bracket"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "3016",
      "mappings": {"default": {"default": "soporte lenticular blanco izquierdo"}}
    }, {
      "category": "Pe",
      "key": "3017",
      "mappings": {"default": {"default": "soporte lenticular blanco derecho"}}
    }, {
      "category": "Ps",
      "key": "3018",
      "mappings": {"default": {"default": "soporte de caparazón de tortuga blanca izquierda"}}
    }, {
      "category": "Pe",
      "key": "3019",
      "mappings": {"default": {"default": "soporte de concha de tortuga blanca derecha"}}
    }, {"key": "301A", "mappings": {"default": {"default": "corchete vacío"}}, "category": "Ps"}, {
      "key": "301B",
      "mappings": {"default": {"default": "corchete vacío derecho", "defaultMP": "cierra corchete vacío"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "301D",
      "mappings": {"default": {"default": "comilla doble invertida"}}
    }, {"key": "301E", "mappings": {"default": {"default": "segunda"}}, "category": "Pe"}, {
      "category": "Pe",
      "key": "301F",
      "mappings": {"default": {"default": "comillas bajas dobles"}}
    }, {
      "category": "Ps",
      "key": "FD3E",
      "mappings": {"default": {"default": "paréntesis izquierdo adornado"}}
    }, {
      "category": "Pe",
      "key": "FD3F",
      "mappings": {"default": {"default": "paréntesis derecho adornado"}}
    }, {
      "category": "Ps",
      "key": "FE17",
      "mappings": {"default": {"default": "formulario de presentación para soporte lenticular blanco izquierdo vertical"}}
    }, {
      "category": "Pe",
      "key": "FE18",
      "mappings": {"default": {"default": "formulario de presentación para soporte lenticular blanco vertical derecho"}}
    }, {"key": "FE35", "mappings": {"default": {"default": "paréntesis superior"}}, "category": "Ps"}, {
      "key": "FE36",
      "mappings": {"default": {"default": "paréntesis inferior"}},
      "category": "Pe"
    }, {"key": "FE37", "mappings": {"default": {"default": "llave superior"}}, "category": "Ps"}, {
      "key": "FE38",
      "mappings": {"default": {"default": "llave inferior"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "FE39",
      "mappings": {"default": {"default": "formulario de presentación para el soporte vertical de concha de tortuga izquierda"}}
    }, {
      "category": "Pe",
      "key": "FE3A",
      "mappings": {"default": {"default": "formulario de presentación para el soporte vertical de concha de tortuga derecha"}}
    }, {
      "category": "Ps",
      "key": "FE3B",
      "mappings": {"default": {"default": "formulario de presentación para el soporte lenticular negro izquierdo vertical"}}
    }, {
      "category": "Pe",
      "key": "FE3C",
      "mappings": {"default": {"default": "formulario de presentación para el soporte lenticular negro derecho vertical"}}
    }, {
      "category": "Ps",
      "key": "FE3D",
      "mappings": {"default": {"default": "formulario de presentación para soporte vertical doble ángulo izquierdo"}}
    }, {
      "category": "Pe",
      "key": "FE3E",
      "mappings": {"default": {"default": "formulario de presentación para el soporte de ángulo recto doble vertical"}}
    }, {
      "key": "FE3F",
      "mappings": {"default": {"default": "paréntesis angular superior"}},
      "category": "Ps"
    }, {
      "key": "FE40",
      "mappings": {"default": {"default": "paréntesis angular inferior"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "FE41",
      "mappings": {"default": {"default": "formulario de presentación para soporte de esquina vertical izquierda"}}
    }, {
      "category": "Pe",
      "key": "FE42",
      "mappings": {"default": {"default": "formulario de presentación para soporte de esquina vertical derecha"}}
    }, {
      "category": "Ps",
      "key": "FE43",
      "mappings": {"default": {"default": "formulario de presentación para soporte de esquina blanco vertical izquierdo"}}
    }, {
      "category": "Pe",
      "key": "FE44",
      "mappings": {"default": {"default": "formulario de presentación para soporte de esquina vertical derecha blanca"}}
    }, {
      "category": "Ps",
      "key": "FE47",
      "mappings": {"default": {"default": "formulario de presentación para soporte cuadrado vertical izquierdo"}}
    }, {
      "category": "Pe",
      "key": "FE48",
      "mappings": {"default": {"default": "formulario de presentación para soporte cuadrado vertical derecho"}}
    }, {
      "category": "Ps",
      "key": "FE59",
      "mappings": {"default": {"default": "pequeño paréntesis izquierdo"}}
    }, {
      "category": "Pe",
      "key": "FE5A",
      "mappings": {"default": {"default": "paréntesis derecho pequeño"}}
    }, {
      "category": "Ps",
      "key": "FE5B",
      "mappings": {"default": {"default": "corchete Rizado Izquierdo Pequeño"}}
    }, {
      "category": "Pe",
      "key": "FE5C",
      "mappings": {"default": {"default": "corchete derecho rizado pequeño"}}
    }, {
      "category": "Ps",
      "key": "FE5D",
      "mappings": {"default": {"default": "soporte de concha pequeña tortuga izquierda"}}
    }, {
      "category": "Pe",
      "key": "FE5E",
      "mappings": {"default": {"default": "pequeño corchete derecho de concha de tortuga"}}
    }, {
      "category": "Ps",
      "key": "FF08",
      "mappings": {"default": {"default": "paréntesis izquierdo de ancho completo"}}
    }, {
      "category": "Pe",
      "key": "FF09",
      "mappings": {"default": {"default": "paréntesis derecho de ancho completo"}}
    }, {
      "category": "Ps",
      "key": "FF3B",
      "mappings": {"default": {"default": "soporte cuadrado izquierdo de ancho completo"}}
    }, {
      "category": "Pe",
      "key": "FF3D",
      "mappings": {"default": {"default": "soporte cuadrado derecho de ancho completo"}}
    }, {
      "category": "Ps",
      "key": "FF5B",
      "mappings": {"default": {"default": "corchete rizado izquierdo de ancho completo"}}
    }, {
      "category": "Pe",
      "key": "FF5D",
      "mappings": {"default": {"default": "corchete derecho de ancho completo"}}
    }, {
      "category": "Ps",
      "key": "FF5F",
      "mappings": {"default": {"default": "paréntesis blanco izquierdo de ancho completo"}}
    }, {
      "category": "Pe",
      "key": "FF60",
      "mappings": {"default": {"default": "paréntesis blanco derecho de ancho completo"}}
    }, {
      "category": "Ps",
      "key": "FF62",
      "mappings": {"default": {"default": "corchete de esquina izquierda de medio ancho"}}
    }, {
      "category": "Pe",
      "key": "FF63",
      "mappings": {"default": {"default": "soporte de esquina derecha de medio ancho"}}
    }]
  ,
  "es/symbols/math_geometry.js":
    [{"locale": "es"}, {
      "key": "2500",
      "mappings": {"default": {"default": "delimitador horizontal"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2501",
      "mappings": {"default": {"default": "dibujos de cajas pesadas horizontales"}}
    }, {
      "key": "2502",
      "mappings": {"default": {"default": "delimitador vertical"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2503",
      "mappings": {"default": {"default": "dibujos de cajas Pesado Vertical"}}
    }, {
      "category": "So",
      "key": "2504",
      "mappings": {"default": {"default": "dibujos de cajas Light Triple Dash Horizontal"}}
    }, {
      "category": "So",
      "key": "2505",
      "mappings": {"default": {"default": "dibujos de cajas Heavy Triple Dash Horizontal"}}
    }, {
      "category": "So",
      "key": "2506",
      "mappings": {"default": {"default": "dibujos de cajas Light Triple Dash Vertical"}}
    }, {
      "category": "So",
      "key": "2507",
      "mappings": {"default": {"default": "dibujos de cajas Heavy Triple Dash Vertical"}}
    }, {
      "category": "So",
      "key": "2508",
      "mappings": {"default": {"default": "caja Dibujos Ligero Cuadruple Dash Horizontal"}}
    }, {
      "category": "So",
      "key": "2509",
      "mappings": {"default": {"default": "dibujos de cajas Heavy Quadruple Dash Horizontal"}}
    }, {
      "category": "So",
      "key": "250A",
      "mappings": {"default": {"default": "caja Dibujos Luz Cuadruple Tablero Vertical"}}
    }, {
      "category": "So",
      "key": "250B",
      "mappings": {"default": {"default": "dibujos de cajas Heavy Quadruple Dash Vertical"}}
    }, {
      "key": "250C",
      "mappings": {"default": {"default": "esquina inferior derecha"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "250D",
      "mappings": {"default": {"default": "dibujos de cajas Abajo Ligero y Derecho Pesado"}}
    }, {
      "category": "So",
      "key": "250E",
      "mappings": {"default": {"default": "dibujos de cajas abajo pesados ​​y rectos ligeros"}}
    }, {
      "category": "So",
      "key": "250F",
      "mappings": {"default": {"default": "dibujos de cajas pesados ​​abajo y derecha"}}
    }, {
      "key": "2510",
      "mappings": {"default": {"default": "esquina inferior izquierda"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2511",
      "mappings": {"default": {"default": "dibujos de cajas abajo ligeros y pesados ​​a la izquierda"}}
    }, {
      "category": "So",
      "key": "2512",
      "mappings": {"default": {"default": "dibujos de cajas abajo pesados ​​y luz izquierda"}}
    }, {
      "category": "So",
      "key": "2513",
      "mappings": {"default": {"default": "dibujos de cajas pesados ​​hacia abajo y hacia la izquierda"}}
    }, {
      "key": "2514",
      "mappings": {"default": {"default": "esquina superior derecha"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2515",
      "mappings": {"default": {"default": "dibujos de cajas hasta ligero y derecho pesado"}}
    }, {
      "category": "So",
      "key": "2516",
      "mappings": {"default": {"default": "dibujos de cajas hasta Pesadas y Ligeras"}}
    }, {
      "category": "So",
      "key": "2517",
      "mappings": {"default": {"default": "dibujos de cajas pesados ​​y derechos"}}
    }, {
      "key": "2518",
      "mappings": {"default": {"default": "esquina superior izquierda"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2519",
      "mappings": {"default": {"default": "dibujos de cajas hasta livianos y pesados ​​a la izquierda"}}
    }, {
      "category": "So",
      "key": "251A",
      "mappings": {"default": {"default": "dibujos de cajas hasta pesado y luz izquierda"}}
    }, {
      "category": "So",
      "key": "251B",
      "mappings": {"default": {"default": "dibujos de cajas pesados ​​hacia arriba y hacia la izquierda"}}
    }, {
      "key": "251C",
      "mappings": {"default": {"default": "delimitador izquierdo y separador horizontal"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "251D",
      "mappings": {"default": {"default": "cuadros de caja Vertical ligero y derecho pesado"}}
    }, {
      "category": "So",
      "key": "251E",
      "mappings": {"default": {"default": "dibujos de cajas hasta pesados ​​y rectos hacia abajo"}}
    }, {
      "category": "So",
      "key": "251F",
      "mappings": {"default": {"default": "dibujos de cajas abajo pesados ​​y rectos arriba ligeros"}}
    }, {
      "category": "So",
      "key": "2520",
      "mappings": {"default": {"default": "dibujos de cajas verticales pesadas y rectas ligeras"}}
    }, {
      "category": "So",
      "key": "2521",
      "mappings": {"default": {"default": "dibujos de cajas abajo ligeros y rectos arriba pesados"}}
    }, {
      "category": "So",
      "key": "2522",
      "mappings": {"default": {"default": "dibujos de cajas para arriba livianos y rectos para abajo pesados"}}
    }, {
      "category": "So",
      "key": "2523",
      "mappings": {"default": {"default": "dibujos de cajas pesadas verticales y derechas"}}
    }, {
      "key": "2524",
      "mappings": {"default": {"default": "delimitador derecho y separador horizontal"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2525",
      "mappings": {"default": {"default": "cuadros de caja Vertical ligero y izquierdo pesado"}}
    }, {
      "category": "So",
      "key": "2526",
      "mappings": {"default": {"default": "dibujos de cajas para arriba Pesado y Izquierda Abajo Luz"}}
    }, {
      "category": "So",
      "key": "2527",
      "mappings": {"default": {"default": "dibujos de cajas abajo pesados ​​y luz izquierda arriba"}}
    }, {
      "category": "So",
      "key": "2528",
      "mappings": {"default": {"default": "dibujos de cajas verticales pesadas y luz izquierda"}}
    }, {
      "category": "So",
      "key": "2529",
      "mappings": {"default": {"default": "dibujos de cajas abajo ligeros e izquierdos arriba pesados"}}
    }, {
      "category": "So",
      "key": "252A",
      "mappings": {"default": {"default": "dibujos de cajas para arriba Ligero y Izquierda Abajo Pesado"}}
    }, {
      "category": "So",
      "key": "252B",
      "mappings": {"default": {"default": "dibujos de cajas pesadas verticales e izquierdas"}}
    }, {
      "key": "252C",
      "mappings": {"default": {"default": "delimitador superior y separador vertical"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "252D",
      "mappings": {"default": {"default": "dibujos de cajas izquierda pesados ​​y derecha abajo luz"}}
    }, {
      "category": "So",
      "key": "252E",
      "mappings": {"default": {"default": "dibujos de cajas, derecha, pesada y izquierda abajo, luz"}}
    }, {
      "category": "So",
      "key": "252F",
      "mappings": {"default": {"default": "dibujos de cajas abajo ligeros y horizontales pesados"}}
    }, {
      "category": "So",
      "key": "2530",
      "mappings": {"default": {"default": "dibujos de cajas de luz pesada y horizontal"}}
    }, {
      "category": "So",
      "key": "2531",
      "mappings": {"default": {"default": "dibujos de la caja de luz derecha y izquierda abajo pesado"}}
    }, {
      "category": "So",
      "key": "2532",
      "mappings": {"default": {"default": "cuadros de caja izquierda ligera y derecha abajo pesada"}}
    }, {
      "category": "So",
      "key": "2533",
      "mappings": {"default": {"default": "dibujos de cajas pesados ​​y horizontales"}}
    }, {
      "key": "2534",
      "mappings": {"default": {"default": "delimitador inferior y separador vertical"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2535",
      "mappings": {"default": {"default": "dibujos de cajas izquierda pesada y derecha arriba ligera"}}
    }, {
      "category": "So",
      "key": "2536",
      "mappings": {"default": {"default": "dibujos de caja derecha pesada y luz izquierda arriba"}}
    }, {
      "category": "So",
      "key": "2537",
      "mappings": {"default": {"default": "dibujos de cajas hasta ligero y horizontal pesado"}}
    }, {
      "category": "So",
      "key": "2538",
      "mappings": {"default": {"default": "dibujos de cajas hasta luz pesada y horizontal"}}
    }, {
      "category": "So",
      "key": "2539",
      "mappings": {"default": {"default": "dibujos de cajas a la derecha, livianos y a la izquierda, pesados"}}
    }, {
      "category": "So",
      "key": "253A",
      "mappings": {"default": {"default": "dibujos de cajas Izquierda Ligera y Derecha Arriba Pesada"}}
    }, {
      "category": "So",
      "key": "253B",
      "mappings": {"default": {"default": "dibujos de cajas pesados ​​y horizontales"}}
    }, {
      "key": "253C",
      "mappings": {"default": {"default": "cruce de separadores sencillos"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "253D",
      "mappings": {"default": {"default": "dibujos de cajas Izquierda Pesada y Derecha Vertical Luz"}}
    }, {
      "category": "So",
      "key": "253E",
      "mappings": {"default": {"default": "dibujos de caja derecha pesada y luz vertical izquierda"}}
    }, {
      "category": "So",
      "key": "253F",
      "mappings": {"default": {"default": "dibujos de cajas Vertical Ligera y Horizontal Pesada"}}
    }, {
      "category": "So",
      "key": "2540",
      "mappings": {"default": {"default": "dibujos de cajas para arriba pesados ​​y abajo luz horizontal"}}
    }, {
      "category": "So",
      "key": "2541",
      "mappings": {"default": {"default": "dibujos de cajas hacia abajo pesados ​​y hacia arriba luz horizontal"}}
    }, {
      "category": "So",
      "key": "2542",
      "mappings": {"default": {"default": "dibujos de cajas verticales pesadas y horizontales ligeras"}}
    }, {
      "category": "So",
      "key": "2543",
      "mappings": {"default": {"default": "dibujos de cajas Izquierda Arriba Pesada y Derecha abajo Luz"}}
    }, {
      "category": "So",
      "key": "2544",
      "mappings": {"default": {"default": "dibujos de caja derecha arriba pesada y izquierda abajo luz"}}
    }, {
      "category": "So",
      "key": "2545",
      "mappings": {"default": {"default": "dibujos de cajas, izquierda abajo, pesada y derecha arriba, ligera"}}
    }, {
      "category": "So",
      "key": "2546",
      "mappings": {"default": {"default": "dibujos de la caja derecha abajo pesada y izquierda arriba ligera"}}
    }, {
      "category": "So",
      "key": "2547",
      "mappings": {"default": {"default": "dibujos de cajas abajo ligeros y ascendentes horizontales pesados"}}
    }, {
      "category": "So",
      "key": "2548",
      "mappings": {"default": {"default": "dibujos de cajas para arriba Ligero y para abajo horizontal pesado"}}
    }, {
      "category": "So",
      "key": "2549",
      "mappings": {"default": {"default": "dibujos de cajas a la derecha, ligeros y a la izquierda, verticales, pesados"}}
    }, {
      "category": "So",
      "key": "254A",
      "mappings": {"default": {"default": "dibujos de cajas Izquierda Ligera y Derecha Vertical Pesada"}}
    }, {
      "category": "So",
      "key": "254B",
      "mappings": {"default": {"default": "dibujos de cajas pesadas verticales y horizontales"}}
    }, {
      "category": "So",
      "key": "254C",
      "mappings": {"default": {"default": "dibujos de cajas Light Double Dash Horizontal"}}
    }, {
      "category": "So",
      "key": "254D",
      "mappings": {"default": {"default": "dibujos de cajas Heavy Double Dash Horizontal"}}
    }, {
      "category": "So",
      "key": "254E",
      "mappings": {"default": {"default": "dibujos de cajas Light Double Dash Vertical"}}
    }, {
      "category": "So",
      "key": "254F",
      "mappings": {"default": {"default": "dibujos de cajas Heavy Double Dash Vertical"}}
    }, {
      "key": "2550",
      "mappings": {"default": {"default": "delimitador horizontal doble"}},
      "category": "So"
    }, {
      "key": "2551",
      "mappings": {"default": {"default": "delimitador vertical doble"}},
      "category": "So"
    }, {
      "key": "2552",
      "mappings": {"default": {"default": "esquina inferior derecha, vertical doble"}},
      "category": "So"
    }, {
      "key": "2553",
      "mappings": {"default": {"default": "esquina inferior derecha, horizontal doble"}},
      "category": "So"
    }, {
      "key": "2554",
      "mappings": {"default": {"default": "esquina inferior derecha doble"}},
      "category": "So"
    }, {
      "key": "2555",
      "mappings": {"default": {"default": "esquina inferior izquierda, vertical doble"}},
      "category": "So"
    }, {
      "key": "2556",
      "mappings": {"default": {"default": "esquina inferior izquierda, horizontal doble"}},
      "category": "So"
    }, {
      "key": "2557",
      "mappings": {"default": {"default": "esquina inferior izquierda doble"}},
      "category": "So"
    }, {
      "key": "2558",
      "mappings": {"default": {"default": "esquina superior derecha, vertical doble"}},
      "category": "So"
    }, {
      "key": "2559",
      "mappings": {"default": {"default": "esquina superior derecha, horizontal doble"}},
      "category": "So"
    }, {
      "key": "255A",
      "mappings": {"default": {"default": "esquina superior derecha doble"}},
      "category": "So"
    }, {
      "key": "255B",
      "mappings": {"default": {"default": "esquina superior izquierda, vertical doble"}},
      "category": "So"
    }, {
      "key": "255C",
      "mappings": {"default": {"default": "esquina superior izquierda, horizontal doble"}},
      "category": "So"
    }, {
      "key": "255D",
      "mappings": {"default": {"default": "esquina superior izquierda doble"}},
      "category": "So"
    }, {
      "key": "255E",
      "mappings": {"default": {"default": "delimitador izquierdo con separador doble"}},
      "category": "So"
    }, {
      "key": "255F",
      "mappings": {"default": {"default": "delimitador izquierdo doble con separador horizontal"}},
      "category": "So"
    }, {
      "key": "2560",
      "mappings": {"default": {"default": "delimitador izquierdo y separador horizontal dobles"}},
      "category": "So"
    }, {
      "key": "2561",
      "mappings": {"default": {"default": "delimitador derecho con separador doble"}},
      "category": "So"
    }, {
      "key": "2562",
      "mappings": {"default": {"default": "delimitador derecho doble con separador horizontal"}},
      "category": "So"
    }, {
      "key": "2563",
      "mappings": {"default": {"default": "delimitador derecho y separador horizontal dobles"}},
      "category": "So"
    }, {
      "key": "2564",
      "mappings": {"default": {"default": "delimitador superior doble con separador vertical"}},
      "category": "So"
    }, {
      "key": "2565",
      "mappings": {"default": {"default": "delimitador superior con separador vertical doble"}},
      "category": "So"
    }, {
      "key": "2566",
      "mappings": {"default": {"default": "delimitador superior y separador vertical dobles"}},
      "category": "So"
    }, {
      "key": "2567",
      "mappings": {"default": {"default": "delimitador inferior doble con separador vertical"}},
      "category": "So"
    }, {
      "key": "2568",
      "mappings": {"default": {"default": "delimitador inferior con separador vertical doble"}},
      "category": "So"
    }, {
      "key": "2569",
      "mappings": {"default": {"default": "delimitador inferior y separador vertical dobles"}},
      "category": "So"
    }, {
      "key": "256A",
      "mappings": {"default": {"default": "cruce de separadores, vertical sencillo y horizontal doble"}},
      "category": "So"
    }, {
      "key": "256B",
      "mappings": {"default": {"default": "cruce de separadores, vertical doble y horizontal sencillo"}},
      "category": "So"
    }, {
      "key": "256C",
      "mappings": {"default": {"default": "cruce de separadores dobles"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "256D",
      "mappings": {"default": {"default": "dibujos de cajas de luz arco abajo y derecha"}}
    }, {
      "category": "So",
      "key": "256E",
      "mappings": {"default": {"default": "dibujos de cajas de luz arco hacia abajo y hacia la izquierda"}}
    }, {
      "category": "So",
      "key": "256F",
      "mappings": {"default": {"default": "dibujos de cajas de luz arco arriba y a la izquierda"}}
    }, {
      "category": "So",
      "key": "2570",
      "mappings": {"default": {"default": "caja de dibujos de luz arco arriba y derecha"}}
    }, {
      "category": "So",
      "key": "2571",
      "mappings": {"default": {"default": "dibujos de cajas Luz Diagonal Superior derecha a inferior izquierda"}}
    }, {
      "category": "So",
      "key": "2572",
      "mappings": {"default": {"default": "dibujos de cajas Luz Diagonal Superior izquierda a inferior derecha"}}
    }, {
      "category": "So",
      "key": "2573",
      "mappings": {"default": {"default": "dibujos de cajas de luz diagonal cruz"}}
    }, {
      "category": "So",
      "key": "2574",
      "mappings": {"default": {"default": "dibujos de cajas luz izquierda"}}
    }, {
      "category": "So",
      "key": "2575",
      "mappings": {"default": {"default": "dibujos de cajas se iluminan"}}
    }, {
      "category": "So",
      "key": "2576",
      "mappings": {"default": {"default": "dibujos de cajas a la derecha"}}
    }, {
      "category": "So",
      "key": "2577",
      "mappings": {"default": {"default": "dibujos de cajas de luz hacia abajo"}}
    }, {
      "category": "So",
      "key": "2578",
      "mappings": {"default": {"default": "dibujos de cajas pesados ​​a la izquierda"}}
    }, {
      "category": "So",
      "key": "2579",
      "mappings": {"default": {"default": "dibujos de cajas pesados"}}
    }, {
      "category": "So",
      "key": "257A",
      "mappings": {"default": {"default": "caja de dibujos pesado derecho"}}
    }, {
      "category": "So",
      "key": "257B",
      "mappings": {"default": {"default": "dibujos de cajas pesados"}}
    }, {
      "category": "So",
      "key": "257C",
      "mappings": {"default": {"default": "dibujos de cajas Ligero Izquierda y Pesada Derecha"}}
    }, {
      "category": "So",
      "key": "257D",
      "mappings": {"default": {"default": "dibujos de cajas iluminados y pesados"}}
    }, {
      "category": "So",
      "key": "257E",
      "mappings": {"default": {"default": "dibujos de cajas Pesado Izquierda y Luz Derecha"}}
    }, {
      "category": "So",
      "key": "257F",
      "mappings": {"default": {"default": "dibujos de cajas pesados ​​y ligeros"}}
    }, {
      "key": "2580",
      "mappings": {"default": {"default": "bloque mitad superior"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2581",
      "mappings": {"default": {"default": "bajar un octavo bloque"}}
    }, {
      "category": "So",
      "key": "2582",
      "mappings": {"default": {"default": "bloque de un cuarto inferior"}}
    }, {
      "category": "So",
      "key": "2583",
      "mappings": {"default": {"default": "bloque de tres octavos más bajo"}}
    }, {
      "key": "2584",
      "mappings": {"default": {"default": "bloque mitad inferior"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2585",
      "mappings": {"default": {"default": "bloque inferior de cinco octavos"}}
    }, {
      "category": "So",
      "key": "2586",
      "mappings": {"default": {"default": "bloque de tres cuartos más bajo"}}
    }, {
      "category": "So",
      "key": "2587",
      "mappings": {"default": {"default": "lower Seven Eighths Block"}}
    }, {"key": "2588", "mappings": {"default": {"default": "bloque"}}, "category": "So"}, {
      "category": "So",
      "key": "2589",
      "mappings": {"default": {"default": "bloque de siete octavos a la izquierda"}}
    }, {
      "category": "So",
      "key": "258A",
      "mappings": {"default": {"default": "bloque de tres cuartos a la izquierda"}}
    }, {
      "category": "So",
      "key": "258B",
      "mappings": {"default": {"default": "bloque de cinco octavos a la izquierda"}}
    }, {
      "category": "So",
      "key": "258C",
      "mappings": {"default": {"default": "media cuadra izquierda"}}
    }, {
      "category": "So",
      "key": "258D",
      "mappings": {"default": {"default": "bloque de tres octavos a la izquierda"}}
    }, {
      "category": "So",
      "key": "258E",
      "mappings": {"default": {"default": "bloque de un cuarto a la izquierda"}}
    }, {
      "category": "So",
      "key": "258F",
      "mappings": {"default": {"default": "izquierda un octavo bloque"}}
    }, {"category": "So", "key": "2590", "mappings": {"default": {"default": "media cuadra derecha"}}}, {
      "key": "2591",
      "mappings": {"default": {"default": "sombra ligera"}},
      "category": "So"
    }, {"key": "2592", "mappings": {"default": {"default": "sombra media"}}, "category": "So"}, {
      "key": "2593",
      "mappings": {"default": {"default": "sombra intensa"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2594",
      "mappings": {"default": {"default": "upper One Eighth Block"}}
    }, {
      "category": "So",
      "key": "2595",
      "mappings": {"default": {"default": "a la derecha un octavo bloque"}}
    }, {
      "category": "So",
      "key": "2596",
      "mappings": {"default": {"default": "cuadrante inferior izquierdo"}}
    }, {
      "category": "So",
      "key": "2597",
      "mappings": {"default": {"default": "cuadrante inferior derecho"}}
    }, {
      "category": "So",
      "key": "2598",
      "mappings": {"default": {"default": "cuadrante superior izquierdo"}}
    }, {
      "category": "So",
      "key": "2599",
      "mappings": {"default": {"default": "cuadrante superior izquierdo e inferior izquierdo e inferior derecho"}}
    }, {
      "category": "So",
      "key": "259A",
      "mappings": {"default": {"default": "cuadrante superior izquierdo e inferior derecho"}}
    }, {
      "category": "So",
      "key": "259B",
      "mappings": {"default": {"default": "cuadrante superior izquierdo y superior derecho e inferior izquierdo"}}
    }, {
      "category": "So",
      "key": "259C",
      "mappings": {"default": {"default": "cuadrante superior izquierdo y superior derecho e inferior derecho"}}
    }, {
      "category": "So",
      "key": "259D",
      "mappings": {"default": {"default": "cuadrante superior derecho"}}
    }, {
      "category": "So",
      "key": "259E",
      "mappings": {"default": {"default": "cuadrante superior derecho e inferior izquierdo"}}
    }, {
      "category": "So",
      "key": "259F",
      "mappings": {"default": {"default": "cuadrante superior derecho e inferior izquierdo e inferior derecho"}}
    }, {"key": "25A0", "mappings": {"default": {"default": "cuadrado relleno"}}, "category": "So"}, {
      "key": "25A1",
      "mappings": {"default": {"default": "cuadrado"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "25A2",
      "mappings": {"default": {"default": "cuadrado blanco con esquinas redondeadas"}}
    }, {
      "category": "So",
      "key": "25A3",
      "mappings": {"default": {"default": "cuadrado blanco que contiene cuadrado pequeño negro"}}
    }, {
      "category": "So",
      "key": "25A4",
      "mappings": {"default": {"default": "cuadrado con relleno horizontal"}}
    }, {
      "category": "So",
      "key": "25A5",
      "mappings": {"default": {"default": "cuadrado con relleno vertical"}}
    }, {
      "category": "So",
      "key": "25A6",
      "mappings": {"default": {"default": "cuadrado con relleno de rayado ortogonal"}}
    }, {
      "category": "So",
      "key": "25A7",
      "mappings": {"default": {"default": "cuadrado con la esquina superior izquierda a la derecha inferior"}}
    }, {
      "category": "So",
      "key": "25A8",
      "mappings": {"default": {"default": "cuadrado con la parte superior derecha hacia abajo izquierda Relleno"}}
    }, {
      "category": "So",
      "key": "25A9",
      "mappings": {"default": {"default": "cuadrado con relleno de rayado diagonal"}}
    }, {"key": "25AA", "mappings": {"default": {"default": "cuadrado relleno"}}, "category": "So"}, {
      "key": "25AB",
      "mappings": {"default": {"default": "cuadrado"}},
      "category": "So"
    }, {"category": "So", "key": "25AC", "mappings": {"default": {"default": "rectángulo negro"}}}, {
      "key": "25AD",
      "mappings": {"default": {"default": "rectángulo"}},
      "category": "So"
    }, {
      "key": "25AE",
      "mappings": {"default": {"default": "rectángulo relleno vertical"}},
      "category": "So"
    }, {
      "key": "25AF",
      "mappings": {"default": {"default": "rectángulo vertical"}},
      "category": "So"
    }, {"category": "So", "key": "25B0", "mappings": {"default": {"default": "paralelogramo negro"}}}, {
      "key": "25B1",
      "mappings": {"default": {"default": "paralelogramo"}},
      "category": "So"
    }, {"key": "25B2", "mappings": {"default": {"default": "triángulo relleno"}}, "category": "So"}, {
      "key": "25B3",
      "mappings": {"default": {"default": "triángulo"}},
      "category": "So"
    }, {"key": "25B4", "mappings": {"default": {"default": "triángulo relleno"}}, "category": "So"}, {
      "key": "25B5",
      "mappings": {"default": {"default": "triángulo"}},
      "category": "So"
    }, {
      "key": "25B6",
      "mappings": {"default": {"default": "triángulo relleno señala derecha"}},
      "category": "So"
    }, {
      "key": "25B7",
      "mappings": {"default": {"default": "triángulo señala derecha"}},
      "category": "Sm"
    }, {
      "key": "25B8",
      "mappings": {"default": {"default": "triángulo relleno señala derecha"}},
      "category": "So"
    }, {
      "key": "25B9",
      "mappings": {"default": {"default": "triángulo señala derecha"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "25BA",
      "mappings": {"default": {"default": "indicador de apuntar hacia la derecha negro"}}
    }, {
      "category": "So",
      "key": "25BB",
      "mappings": {"default": {"default": "puntero de apuntar hacia la derecha blanco"}}
    }, {
      "key": "25BC",
      "mappings": {"default": {"default": "triángulo relleno señala abajo"}},
      "category": "So"
    }, {
      "key": "25BD",
      "mappings": {"default": {"default": "triángulo señala abajo"}},
      "category": "So"
    }, {
      "key": "25BE",
      "mappings": {"default": {"default": "triángulo relleno señala abajo"}},
      "category": "So"
    }, {
      "key": "25BF",
      "mappings": {"default": {"default": "triángulo señala abajo"}},
      "category": "So"
    }, {
      "key": "25C0",
      "mappings": {"default": {"default": "triángulo relleno señala izquierda"}},
      "category": "So"
    }, {
      "key": "25C1",
      "mappings": {"default": {"default": "triángulo señala izquierda"}},
      "category": "Sm"
    }, {
      "key": "25C2",
      "mappings": {"default": {"default": "triángulo relleno señala izquierda"}},
      "category": "So"
    }, {
      "key": "25C3",
      "mappings": {"default": {"default": "triángulo señala izquierda"}},
      "category": "So"
    }, {
      "key": "25C4",
      "mappings": {"default": {"default": "puntero relleno señala izquierda"}},
      "category": "So"
    }, {
      "key": "25C5",
      "mappings": {"default": {"default": "puntero señala izquierda"}},
      "category": "So"
    }, {"key": "25C6", "mappings": {"default": {"default": "diamante relleno"}}, "category": "So"}, {
      "key": "25C7",
      "mappings": {"default": {"default": "diamante"}},
      "category": "So"
    }, {
      "key": "25C8",
      "mappings": {"default": {"default": "diamante vacío contiene diamante relleno"}},
      "category": "So"
    }, {"key": "25C9", "mappings": {"default": {"default": "ojo de pez"}}, "category": "So"}, {
      "key": "25CA",
      "mappings": {"default": {"default": "rombo"}},
      "category": "So"
    }, {"key": "25CB", "mappings": {"default": {"default": "círculo vacío"}}, "category": "So"}, {
      "key": "25CC",
      "mappings": {"default": {"default": "círculo punteado"}},
      "category": "So"
    }, {
      "key": "25CD",
      "mappings": {"default": {"default": "círculo con relleno vertical"}},
      "category": "So"
    }, {"key": "25CE", "mappings": {"default": {"default": "blanco"}}, "category": "So"}, {
      "key": "25CF",
      "mappings": {"default": {"default": "círculo relleno"}},
      "category": "So"
    }, {
      "key": "25D0",
      "mappings": {"default": {"default": "círculo con mitad izquierda rellena"}},
      "category": "So"
    }, {
      "key": "25D1",
      "mappings": {"default": {"default": "círculo con mitad derecha rellena"}},
      "category": "So"
    }, {
      "key": "25D2",
      "mappings": {"default": {"default": "círculo con mitad inferior rellena"}},
      "category": "So"
    }, {
      "key": "25D3",
      "mappings": {"default": {"default": "círculo con mitad superior rellena"}},
      "category": "So"
    }, {
      "key": "25D4",
      "mappings": {"default": {"default": "círculo con cuadrante superior derecho relleno"}},
      "category": "So"
    }, {
      "key": "25D5",
      "mappings": {"default": {"default": "círculo con cuadrante superior izquierdo relleno"}},
      "category": "So"
    }, {
      "key": "25D6",
      "mappings": {"default": {"default": "semicírculo izquierdo relleno"}},
      "category": "So"
    }, {
      "key": "25D7",
      "mappings": {"default": {"default": "semicírculo derecho relleno"}},
      "category": "So"
    }, {"key": "25D8", "mappings": {"default": {"default": "bullet inversa"}}, "category": "So"}, {
      "key": "25D9",
      "mappings": {"default": {"default": "círculo vacío inverso"}},
      "category": "So"
    }, {
      "key": "25DA",
      "mappings": {"default": {"default": "semicírculo superior vacío inverso"}},
      "category": "So"
    }, {
      "key": "25DB",
      "mappings": {"default": {"default": "semicírculo superior vacío inverso"}},
      "category": "So"
    }, {
      "key": "25DC",
      "mappings": {"default": {"default": "arco superior izquierda"}},
      "category": "So"
    }, {"key": "25DD", "mappings": {"default": {"default": "arco superior derecha"}}, "category": "So"}, {
      "key": "25DE",
      "mappings": {"default": {"default": "arco inferior derecha"}},
      "category": "So"
    }, {
      "key": "25DF",
      "mappings": {"default": {"default": "arco inferior izquierda"}},
      "category": "So"
    }, {"key": "25E0", "mappings": {"default": {"default": "semicírculo superior"}}, "category": "So"}, {
      "key": "25E1",
      "mappings": {"default": {"default": "semicírculo inferior"}},
      "category": "So"
    }, {
      "key": "25E2",
      "mappings": {"default": {"default": "triángulo inferior derecha relleno"}},
      "category": "So"
    }, {
      "key": "25E3",
      "mappings": {"default": {"default": "triángulo inferior izquierda relleno"}},
      "category": "So"
    }, {
      "key": "25E4",
      "mappings": {"default": {"default": "triángulo superior izquierda relleno"}},
      "category": "So"
    }, {
      "key": "25E5",
      "mappings": {"default": {"default": "triángulo superior derecha relleno"}},
      "category": "So"
    }, {"key": "25E6", "mappings": {"default": {"default": "composición"}}, "category": "So"}, {
      "key": "25E7",
      "mappings": {"default": {"default": "cuadrado con mitad izquierda rellena"}},
      "category": "So"
    }, {
      "key": "25E8",
      "mappings": {"default": {"default": "cuadrado con mitad derecha rellena"}},
      "category": "So"
    }, {
      "key": "25E9",
      "mappings": {"default": {"default": "cuadrado con mitad superior izquierda rellena"}},
      "category": "So"
    }, {
      "key": "25EA",
      "mappings": {"default": {"default": "cuadrado con mitad inferior derecha rellena"}},
      "category": "So"
    }, {
      "key": "25EB",
      "mappings": {"default": {"default": "cuadrado con bisectriz"}},
      "category": "So"
    }, {
      "key": "25EC",
      "mappings": {"default": {"default": "triángulo vacío con punto"}},
      "category": "So"
    }, {
      "key": "25ED",
      "mappings": {"default": {"default": "triángulo apuntando arriba con mitad izquierda rellena"}},
      "category": "So"
    }, {
      "key": "25EE",
      "mappings": {"default": {"default": "triángulo apuntando arriba con mitad derecha rellena"}},
      "category": "So"
    }, {"key": "25EF", "mappings": {"default": {"default": "círculo grande"}}, "category": "So"}, {
      "category": "So",
      "key": "25F0",
      "mappings": {"default": {"default": "cuadrado blanco con cuadrante superior izquierdo"}}
    }, {
      "category": "So",
      "key": "25F1",
      "mappings": {"default": {"default": "cuadrado blanco con cuadrante inferior izquierdo"}}
    }, {
      "category": "So",
      "key": "25F2",
      "mappings": {"default": {"default": "cuadrado blanco con cuadrante inferior derecho"}}
    }, {
      "category": "So",
      "key": "25F3",
      "mappings": {"default": {"default": "cuadrado blanco con cuadrante superior derecho"}}
    }, {
      "category": "So",
      "key": "25F4",
      "mappings": {"default": {"default": "círculo blanco con cuadrante superior izquierdo"}}
    }, {
      "category": "So",
      "key": "25F5",
      "mappings": {"default": {"default": "círculo blanco con cuadrante inferior izquierdo"}}
    }, {
      "category": "So",
      "key": "25F6",
      "mappings": {"default": {"default": "círculo blanco con cuadrante inferior derecho"}}
    }, {
      "category": "So",
      "key": "25F7",
      "mappings": {"default": {"default": "círculo blanco con cuadrante superior derecho"}}
    }, {
      "key": "25F8",
      "mappings": {"default": {"default": "triángulo superior izquierda"}},
      "category": "Sm"
    }, {
      "key": "25F9",
      "mappings": {"default": {"default": "triángulo superior derecha"}},
      "category": "Sm"
    }, {
      "key": "25FA",
      "mappings": {"default": {"default": "triángulo inferior izquierda"}},
      "category": "Sm"
    }, {
      "key": "25FB",
      "mappings": {"default": {"default": "cuadrado mediano vacío"}},
      "category": "Sm"
    }, {
      "key": "25FC",
      "mappings": {"default": {"default": "cuadrado mediano relleno"}},
      "category": "Sm"
    }, {
      "key": "25FD",
      "mappings": {"default": {"default": "cuadrado mediano vacío"}},
      "category": "Sm"
    }, {
      "key": "25FE",
      "mappings": {"default": {"default": "cuadrado mediano relleno"}},
      "category": "Sm"
    }, {
      "key": "25FF",
      "mappings": {"default": {"default": "triángulo inferior derecha"}},
      "category": "Sm"
    }, {
      "category": "So",
      "key": "2B12",
      "mappings": {"default": {"default": "cuadrado con mitad superior negra"}}
    }, {
      "category": "So",
      "key": "2B13",
      "mappings": {"default": {"default": "cuadrado con mitad inferior negra"}}
    }, {
      "category": "So",
      "key": "2B14",
      "mappings": {"default": {"default": "cuadrado con mitad derecha superior diagonal negra"}}
    }, {
      "category": "So",
      "key": "2B15",
      "mappings": {"default": {"default": "cuadrado con mitad diagonal izquierda inferior negra"}}
    }, {
      "category": "So",
      "key": "2B16",
      "mappings": {"default": {"default": "diamante con Mitad Izquierda Negro"}}
    }, {
      "category": "So",
      "key": "2B17",
      "mappings": {"default": {"default": "diamante con mitad derecha negra"}}
    }, {
      "category": "So",
      "key": "2B18",
      "mappings": {"default": {"default": "diamante con mitad superior negra"}}
    }, {
      "category": "So",
      "key": "2B19",
      "mappings": {"default": {"default": "diamante con mitad inferior negra"}}
    }, {"category": "So", "key": "2B1A", "mappings": {"default": {"default": "cuadrado punteado"}}}, {
      "category": "So",
      "key": "2B1B",
      "mappings": {"default": {"default": "cuadrado grande negro"}}
    }, {"category": "So", "key": "2B1C", "mappings": {"default": {"default": "gran Plaza Blanca"}}}, {
      "category": "So",
      "key": "2B1D",
      "mappings": {"default": {"default": "cuadrado muy pequeño negro"}}
    }, {
      "category": "So",
      "key": "2B1E",
      "mappings": {"default": {"default": "cuadrado muy pequeño blanco"}}
    }, {"category": "So", "key": "2B1F", "mappings": {"default": {"default": "pentágono negro"}}}, {
      "category": "So",
      "key": "2B20",
      "mappings": {"default": {"default": "pentágono blanco"}}
    }, {"category": "So", "key": "2B21", "mappings": {"default": {"default": "hexágono blanco"}}}, {
      "category": "So",
      "key": "2B22",
      "mappings": {"default": {"default": "hexágono negro"}}
    }, {
      "category": "So",
      "key": "2B23",
      "mappings": {"default": {"default": "hexagonal negro horizontal"}}
    }, {
      "category": "So",
      "key": "2B24",
      "mappings": {"default": {"default": "circulo grande negro"}}
    }, {
      "category": "So",
      "key": "2B25",
      "mappings": {"default": {"default": "diamante Mediano Negro"}}
    }, {
      "category": "So",
      "key": "2B26",
      "mappings": {"default": {"default": "diamante blanco medio"}}
    }, {
      "category": "So",
      "key": "2B27",
      "mappings": {"default": {"default": "lozenge Mediano Negro"}}
    }, {
      "category": "So",
      "key": "2B28",
      "mappings": {"default": {"default": "pastilla blanca mediana"}}
    }, {
      "category": "So",
      "key": "2B29",
      "mappings": {"default": {"default": "pequeño diamante negro"}}
    }, {
      "category": "So",
      "key": "2B2A",
      "mappings": {"default": {"default": "lozenge pequeño negro"}}
    }, {
      "category": "So",
      "key": "2B2B",
      "mappings": {"default": {"default": "lozenge pequeño blanco"}}
    }, {
      "category": "So",
      "key": "2B2C",
      "mappings": {"default": {"default": "elipse horizontal negro"}}
    }, {
      "category": "So",
      "key": "2B2D",
      "mappings": {"default": {"default": "elipse horizontal blanco"}}
    }, {
      "category": "So",
      "key": "2B2E",
      "mappings": {"default": {"default": "elipse Vertical Negro"}}
    }, {
      "category": "So",
      "key": "2B2F",
      "mappings": {"default": {"default": "elipse Vertical Blanco"}}
    }, {"key": "2B50", "mappings": {"default": {"default": "estrella media vacía"}}, "category": "So"}, {
      "key": "2B51",
      "mappings": {"default": {"default": "estrella pequeña rellena"}},
      "category": "So"
    }, {
      "key": "2B52",
      "mappings": {"default": {"default": "estrella pequeña vacía"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2B53",
      "mappings": {"default": {"default": "pentágono negro que señala hacia la derecha"}}
    }, {
      "category": "So",
      "key": "2B54",
      "mappings": {"default": {"default": "pentágono blanco que señala hacia la derecha"}}
    }, {
      "category": "So",
      "key": "2B55",
      "mappings": {"default": {"default": "círculo grande pesado"}}
    }, {
      "category": "So",
      "key": "2B56",
      "mappings": {"default": {"default": "óvalo pesado con interior oval"}}
    }, {
      "category": "So",
      "key": "2B57",
      "mappings": {"default": {"default": "círculo pesado con círculo dentro"}}
    }, {"category": "So", "key": "2B58", "mappings": {"default": {"default": "círculo pesado"}}}, {
      "category": "So",
      "key": "2B59",
      "mappings": {"default": {"default": "saltire en círculo pesado"}}
    }]
  ,
  "es/symbols/math_harpoons.js":
    [{"locale": "es"}, {
      "key": "21BC",
      "mappings": {"default": {"default": "flecha izquierda con arpón arriba"}},
      "category": "So"
    }, {
      "key": "21BD",
      "mappings": {"default": {"default": "flecha izquierda con arpón debajo"}},
      "category": "So"
    }, {
      "key": "21BE",
      "mappings": {"default": {"default": "flecha arriba con arpón derecho"}},
      "category": "So"
    }, {
      "key": "21BF",
      "mappings": {"default": {"default": "flecha arriba con arpón izquierdo"}},
      "category": "So"
    }, {
      "key": "21C0",
      "mappings": {"default": {"default": "flecha derecha con arpón arriba"}},
      "category": "So"
    }, {
      "key": "21C1",
      "mappings": {"default": {"default": "flecha derecha con arpón debajo"}},
      "category": "So"
    }, {
      "key": "21C2",
      "mappings": {"default": {"default": "flecha abajo con arpón derecho"}},
      "category": "So"
    }, {
      "key": "21C3",
      "mappings": {"default": {"default": "flecha abajo con arpón izquierdo"}},
      "category": "So"
    }, {
      "key": "21CB",
      "mappings": {"default": {"default": "arpón izquierdo arriba de arpón derecho"}},
      "category": "So"
    }, {
      "key": "21CC",
      "mappings": {"default": {"default": "arpón derecho arriba de arpón izquierdo"}},
      "category": "So"
    }, {
      "category": "Sm",
      "key": "294A",
      "mappings": {"default": {"default": "barra izquierda arriba barra derecha abajo arpón"}}
    }, {
      "category": "Sm",
      "key": "294B",
      "mappings": {"default": {"default": "barra izquierda abajo barra derecha arriba arpón"}}
    }, {
      "category": "Sm",
      "key": "294C",
      "mappings": {"default": {"default": "up barb right down barb left harpoon"}}
    }, {
      "category": "Sm",
      "key": "294D",
      "mappings": {"default": {"default": "arriba barb izquierda abajo barb derecha arpón"}}
    }, {
      "category": "Sm",
      "key": "294E",
      "mappings": {"default": {"default": "barra izquierda arriba barra derecha arriba arpón"}}
    }, {
      "key": "294F",
      "mappings": {"default": {"default": "arpón a la derecha hacia arriba y abajo"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2950",
      "mappings": {"default": {"default": "barra izquierda abajo barra derecha abajo arpón"}}
    }, {
      "key": "2951",
      "mappings": {"default": {"default": "arpón a la izquierda hacia arriba y abajo"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2952",
      "mappings": {"default": {"default": "arpón a la izquierda con púa hasta la barra"}}
    }, {
      "category": "Sm",
      "key": "2953",
      "mappings": {"default": {"default": "arpón a la derecha con púa hasta la barra"}}
    }, {
      "category": "Sm",
      "key": "2954",
      "mappings": {"default": {"default": "arpón ascendente con púa derecha a barra"}}
    }, {
      "category": "Sm",
      "key": "2955",
      "mappings": {"default": {"default": "arpón hacia abajo con púa derecha a barra"}}
    }, {
      "category": "Sm",
      "key": "2956",
      "mappings": {"default": {"default": "arpón a la izquierda con púas abajo a la barra"}}
    }, {
      "category": "Sm",
      "key": "2957",
      "mappings": {"default": {"default": "arpón a la derecha con púa abajo a la barra"}}
    }, {
      "category": "Sm",
      "key": "2958",
      "mappings": {"default": {"default": "arpón ascendente con púa izquierda a barra"}}
    }, {
      "category": "Sm",
      "key": "2959",
      "mappings": {"default": {"default": "arpón hacia abajo con la barra izquierda a la barra"}}
    }, {
      "category": "Sm",
      "key": "295A",
      "mappings": {"default": {"default": "arpón a la izquierda con púa arriba de la barra"}}
    }, {
      "category": "Sm",
      "key": "295B",
      "mappings": {"default": {"default": "arpón a la derecha con púa arriba del bar"}}
    }, {
      "category": "Sm",
      "key": "295C",
      "mappings": {"default": {"default": "arpón ascendente con púa derecha de barra"}}
    }, {
      "category": "Sm",
      "key": "295D",
      "mappings": {"default": {"default": "arpón hacia abajo con púa derecha de barra"}}
    }, {
      "category": "Sm",
      "key": "295E",
      "mappings": {"default": {"default": "arpón a la izquierda con púa abajo de barra"}}
    }, {
      "category": "Sm",
      "key": "295F",
      "mappings": {"default": {"default": "arpón a la derecha con púa abajo de barra"}}
    }, {
      "category": "Sm",
      "key": "2960",
      "mappings": {"default": {"default": "arpón ascendente con púa izquierda de barra"}}
    }, {
      "category": "Sm",
      "key": "2961",
      "mappings": {"default": {"default": "arpón hacia abajo con barra izquierda de barra"}}
    }, {
      "category": "Sm",
      "key": "2962",
      "mappings": {"default": {"default": "arpón a la izquierda con púas arriba arriba arpón a la izquierda con púas abajo"}}
    }, {
      "category": "Sm",
      "key": "2963",
      "mappings": {"default": {"default": "arpón ascendente con púa izquierda al lado arpón ascendente con púa derecha"}}
    }, {
      "category": "Sm",
      "key": "2964",
      "mappings": {"default": {"default": "arpón a la derecha con púas arriba arriba arpón a la derecha con púas abajo"}}
    }, {
      "category": "Sm",
      "key": "2965",
      "mappings": {"default": {"default": "arpón hacia abajo con lengüeta a la izquierda al lado arpón hacia abajo con lengüeta a la derecha"}}
    }, {
      "category": "Sm",
      "key": "2966",
      "mappings": {"default": {"default": "arpón a la izquierda con púa arriba arriba arpón a la derecha con púa arriba"}}
    }, {
      "category": "Sm",
      "key": "2967",
      "mappings": {"default": {"default": "arpón a la izquierda con púa abajo arriba arpón a la derecha con púa abajo"}}
    }, {
      "category": "Sm",
      "key": "2968",
      "mappings": {"default": {"default": "arpón a la derecha con púas arriba arriba arpón a la izquierda con púas arriba"}}
    }, {
      "category": "Sm",
      "key": "2969",
      "mappings": {"default": {"default": "arpón a la derecha con púa abajo arriba arpón a la izquierda con púa abajo"}}
    }, {
      "key": "296A",
      "mappings": {"default": {"default": "barra con arpón hacia la izquierda"}},
      "category": "Sm"
    }, {
      "key": "296B",
      "mappings": {"default": {"default": "arpón hacia la izquierda con barra"}},
      "category": "Sm"
    }, {
      "key": "296C",
      "mappings": {"default": {"default": "barra con arpón hacia la derecha"}},
      "category": "Sm"
    }, {
      "key": "296D",
      "mappings": {"default": {"default": "arpón hacia la derecha con barra"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "296E",
      "mappings": {"default": {"default": "arpón ascendente con púa izquierda al lado arpón hacia abajo con púa derecha"}}
    }, {
      "category": "Sm",
      "key": "296F",
      "mappings": {"default": {"default": "arpón hacia abajo con lengüeta a la izquierda al lado arpón hacia arriba con pica a la derecha"}}
    }, {
      "category": "Sm",
      "key": "297C",
      "mappings": {"default": {"default": "cola de pescado izquierda"}}
    }, {
      "category": "Sm",
      "key": "297D",
      "mappings": {"default": {"default": "cola de pescado derecha"}}
    }, {
      "key": "297E",
      "mappings": {"default": {"default": "barra con doble gancho"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "297F", "mappings": {"default": {"default": "cola de pez abajo"}}}]
  ,
  "es/symbols/math_non_characters.js":
    [{"locale": "es"}, {
      "key": "210F",
      "mappings": {"default": {"default": "h barra"}},
      "category": "Ll"
    }, {"category": "So", "key": "2114", "mappings": {"default": {"default": "L B símbolo de barra"}}}, {
      "key": "2116",
      "mappings": {"default": {"default": "signo de número"}},
      "category": "So"
    }, {
      "key": "2117",
      "mappings": {"default": {"default": "derechos de autor de sonido"}},
      "category": "So"
    }, {"key": "211E", "mappings": {"default": {"default": "receta"}}, "category": "So"}, {
      "category": "So",
      "key": "211F",
      "mappings": {"default": {"default": "respuesta"}}
    }, {"category": "So", "key": "2120", "mappings": {"default": {"default": "marca de servicio"}}}, {
      "category": "So",
      "key": "2121",
      "mappings": {"default": {"default": "señal de telefono"}}
    }, {"key": "2122", "mappings": {"default": {"default": "registrado"}}, "category": "So"}, {
      "category": "So",
      "key": "2123",
      "mappings": {"default": {"default": "versículo"}}
    }, {"key": "2125", "mappings": {"default": {"default": "onza"}}, "category": "So"}, {
      "key": "2126",
      "mappings": {"default": {"default": "ohmio"}},
      "category": "Lu"
    }, {"key": "2127", "mappings": {"default": {"default": "ohmio invertido"}}, "category": "So"}, {
      "key": "212A",
      "mappings": {"default": {"default": "grados kelvin"}},
      "category": "Lu"
    }, {"key": "212B", "mappings": {"default": {"default": "angstrom"}}, "category": "Lu"}, {
      "category": "So",
      "key": "212E",
      "mappings": {"default": {"default": "Símbolo estimado"}}
    }, {
      "key": "2132",
      "mappings": {"default": {"default": "mayúscula F invertida"}},
      "category": "Lu"
    }, {
      "category": "Ll",
      "key": "2139",
      "mappings": {"default": {"default": "fuente de información"}}
    }, {"category": "So", "key": "213A", "mappings": {"default": {"default": "capital rotado Q"}}}, {
      "category": "So",
      "key": "213B",
      "mappings": {"default": {"default": "signo de fax"}}
    }, {
      "category": "Sm",
      "key": "2141",
      "mappings": {"default": {"default": "convertido sans serif mayúscula G"}}
    }, {
      "category": "Sm",
      "key": "2142",
      "mappings": {"default": {"default": "convertido sans serif mayúscula L"}}
    }, {
      "category": "Sm",
      "key": "2143",
      "mappings": {"default": {"default": "invertido sans serif mayúscula L"}}
    }, {"category": "Sm", "key": "2144", "mappings": {"default": {"default": "convertido en sans serif mayúscula Y"}}}]
  ,
  "es/symbols/math_symbols.js":
    [{"locale": "es"}, {
      "key": "0021",
      "mappings": {"default": {"default": "factorial"}},
      "category": "Po"
    }, {"key": "0022", "mappings": {"default": {"default": "comillas"}}, "category": "Po"}, {
      "key": "0023",
      "mappings": {"default": {"default": "almuhadilla", "defaultMP": "signo de número"}},
      "category": "Po"
    }, {"key": "0024", "mappings": {"default": {"default": "dólar"}}, "category": "Sc"}, {
      "key": "0025",
      "mappings": {"default": {"default": "porciento"}},
      "category": "Po"
    }, {"key": "0026", "mappings": {"default": {"default": "ampersand"}}, "category": "Po"}, {
      "key": "0027",
      "mappings": {"default": {"default": "prima", "defaultMP": "apóstrofo"}},
      "category": "Po"
    }, {"key": "002A", "mappings": {"default": {"default": "por"}}, "category": "Po"}, {
      "key": "002B",
      "mappings": {"default": {"default": "más"}},
      "category": "Sm"
    }, {"key": "002C", "mappings": {"default": {"default": "coma"}}, "category": "Po"}, {
      "key": "002D",
      "mappings": {"default": {"default": "menos"}},
      "category": "Pd"
    }, {"key": "002E", "mappings": {"default": {"default": "punto"}}, "category": "Po"}, {
      "key": "002F",
      "mappings": {"default": {"default": "barra oblicua"}},
      "category": "Po"
    }, {"key": "003A", "mappings": {"default": {"default": "dos puntos"}}, "category": "Po"}, {
      "key": "003B",
      "mappings": {"default": {"default": "punto y coma"}},
      "category": "Po"
    }, {"key": "003C", "mappings": {"default": {"default": "menor que"}}, "category": "Sm"}, {
      "key": "003D",
      "mappings": {"default": {"default": "igual"}},
      "category": "Sm"
    }, {"key": "003E", "mappings": {"default": {"default": "mayor que"}}, "category": "Sm"}, {
      "key": "003F",
      "mappings": {"default": {"default": "interrogación derecho", "defaultMP": "cierra interrogación"}},
      "category": "Po"
    }, {"key": "0040", "mappings": {"default": {"default": "arroba"}}, "category": "Po"}, {
      "key": "005C",
      "mappings": {"default": {"default": "barra inversa"}},
      "category": "Po"
    }, {"key": "005E", "mappings": {"default": {"default": "circunflejo"}}, "category": "Sk"}, {
      "key": "005F",
      "mappings": {"default": {"default": "subrayado"}},
      "category": "Pc"
    }, {"key": "0060", "mappings": {"default": {"default": "acento grave"}}, "category": "Sk"}, {
      "key": "007C",
      "mappings": {"default": {"default": "barra vertical"}},
      "category": "Sm"
    }, {"key": "007E", "mappings": {"default": {"default": "tilde"}}, "category": "Sm"}, {
      "key": "00A1",
      "mappings": {"default": {"default": "admiración izquierdo", "defaultMP": "abre admiración"}},
      "category": "Po"
    }, {"key": "00A2", "mappings": {"default": {"default": "centavo"}}, "category": "Sc"}, {
      "key": "00A3",
      "mappings": {"default": {"default": "libra esterlina"}},
      "category": "Sc"
    }, {"key": "00A4", "mappings": {"default": {"default": "moneda"}}, "category": "Sc"}, {
      "key": "00A5",
      "mappings": {"default": {"default": "yen"}},
      "category": "Sc"
    }, {
      "key": "00A6",
      "mappings": {"default": {"default": "barra vertical partida"}},
      "category": "So"
    }, {"key": "00A7", "mappings": {"default": {"default": "sección"}}, "category": "Po"}, {
      "key": "00A8",
      "mappings": {"default": {"default": "diéresis"}},
      "category": "Sk"
    }, {"key": "00A9", "mappings": {"default": {"default": "copyright"}}, "category": "So"}, {
      "key": "00AA",
      "mappings": {"default": {"default": "ordinal femenino"}},
      "category": "Lo"
    }, {
      "key": "00AB",
      "mappings": {"default": {"default": "comillas angulares izquierda", "defaultMP": "abre comillas angulares"}},
      "category": "Pi"
    }, {"key": "00AC", "mappings": {"default": {"default": "negación"}}, "category": "Sm"}, {
      "key": "00AE",
      "mappings": {"default": {"default": "registrado"}},
      "category": "So"
    }, {"key": "00AF", "mappings": {"default": {"default": "barra"}}, "category": "Sk"}, {
      "key": "00B0",
      "mappings": {"default": {"default": "grado"}},
      "category": "So"
    }, {"key": "00B1", "mappings": {"default": {"default": "más menos"}}, "category": "Sm"}, {
      "key": "00B4",
      "mappings": {"default": {"default": "acento agudo"}},
      "category": "Sk"
    }, {"key": "00B5", "mappings": {"default": {"default": "micro"}}, "category": "Ll"}, {
      "key": "00B6",
      "mappings": {"default": {"default": "párrafo"}},
      "category": "Po"
    }, {"key": "00B7", "mappings": {"default": {"default": "punto medio"}}, "category": "Po"}, {
      "key": "00B8",
      "mappings": {"default": {"default": "cedilla"}},
      "category": "Sk"
    }, {"key": "00BA", "mappings": {"default": {"default": "ordinal masculino"}}, "category": "Lo"}, {
      "key": "00BB",
      "mappings": {"default": {"default": "comillas angulares derecha", "defaultMP": "cierra comillas angulares"}},
      "category": "Pf"
    }, {
      "key": "00BF",
      "mappings": {"default": {"default": "interrogación izquierdo", "defaultMP": "abre interrogación"}},
      "category": "Po"
    }, {"key": "00D7", "mappings": {"default": {"default": "por"}}, "category": "Sm"}, {
      "key": "00F7",
      "mappings": {"default": {"default": "dividido"}},
      "category": "Sm"
    }, {"key": "02D8", "mappings": {"default": {"default": "breve"}}, "category": "Sk"}, {
      "key": "02D9",
      "mappings": {"default": {"default": "punto en superescrito"}},
      "category": "Sk"
    }, {
      "key": "02DA",
      "mappings": {"default": {"default": "anillo en superescrito"}},
      "category": "Sk"
    }, {"key": "02DB", "mappings": {"default": {"default": "ogonek"}}, "category": "Sk"}, {
      "key": "02DC",
      "mappings": {"default": {"default": "tilde"}},
      "category": "Sk"
    }, {"key": "02DD", "mappings": {"default": {"default": "doble acento agudo"}}, "category": "Sk"}, {
      "key": "2010",
      "mappings": {"default": {"default": "menos"}},
      "category": "Pd"
    }, {"category": "Pd", "key": "2011", "mappings": {"default": {"default": "guión no rompible"}}}, {
      "category": "Pd",
      "key": "2012",
      "mappings": {"default": {"default": "figura Dash"}}
    }, {"key": "2013", "mappings": {"default": {"default": "en raya"}}, "category": "Pd"}, {
      "key": "2014",
      "mappings": {"default": {"default": "em raya"}},
      "category": "Pd"
    }, {"key": "2015", "mappings": {"default": {"default": "barra horizontal"}}, "category": "Pd"}, {
      "key": "2016",
      "mappings": {"default": {"default": "doble barra vertical"}},
      "category": "Po"
    }, {"category": "Po", "key": "2017", "mappings": {"default": {"default": "línea baja doble"}}}, {
      "key": "2018",
      "mappings": {"default": {"default": "comilla izquierda"}},
      "category": "Pi"
    }, {"key": "2019", "mappings": {"default": {"default": "comilla derecha"}}, "category": "Pf"}, {
      "key": "201A",
      "mappings": {"default": {"default": "comilla bajo 9"}},
      "category": "Ps"
    }, {
      "category": "Pi",
      "key": "201B",
      "mappings": {"default": {"default": "marca de comillas simple con inversión alta-9"}}
    }, {"key": "201C", "mappings": {"default": {"default": "doble comilla"}}, "category": "Pi"}, {
      "key": "201D",
      "mappings": {"default": {"default": "doble comilla derecha"}},
      "category": "Pf"
    }, {
      "key": "201E",
      "mappings": {"default": {"default": "doble comilla bajo 9"}},
      "category": "Ps"
    }, {
      "category": "Pi",
      "key": "201F",
      "mappings": {"default": {"default": "doble alto-invertido-9 comillas"}}
    }, {"key": "2020", "mappings": {"default": {"default": "daga"}}, "category": "Po"}, {
      "key": "2021",
      "mappings": {"default": {"default": "doble daga"}},
      "category": "Po"
    }, {"key": "2022", "mappings": {"default": {"default": "bala"}}, "category": "Po"}, {
      "category": "Po",
      "key": "2023",
      "mappings": {"default": {"default": "bala triangular"}}
    }, {"category": "Po", "key": "2024", "mappings": {"default": {"default": "líder de un punto"}}}, {
      "key": "2025",
      "mappings": {"default": {"default": "dos puntos líder"}},
      "category": "Po"
    }, {"key": "2026", "mappings": {"default": {"default": "puntos suspensivos"}}, "category": "Po"}, {
      "category": "Po",
      "key": "2027",
      "mappings": {"default": {"default": "punto de separación"}}
    }, {"key": "2030", "mappings": {"default": {"default": "tanto por millar"}}, "category": "Po"}, {
      "key": "2031",
      "mappings": {"default": {"default": "tanto por diez mil"}},
      "category": "Po"
    }, {"key": "2032", "mappings": {"default": {"default": "prima"}}, "category": "Po"}, {
      "key": "2033",
      "mappings": {"default": {"default": "segunda"}},
      "category": "Po"
    }, {"key": "2034", "mappings": {"default": {"default": "tercera"}}, "category": "Po"}, {
      "key": "2035",
      "mappings": {"default": {"default": "acento grave"}},
      "category": "Po"
    }, {"key": "2036", "mappings": {"default": {"default": "doble acento grave"}}, "category": "Po"}, {
      "category": "Po",
      "key": "2037",
      "mappings": {"default": {"default": "prime Prime invertido"}}
    }, {
      "category": "Po",
      "key": "2038",
      "mappings": {"default": {"default": "signo de intercalación"}}
    }, {
      "key": "2039",
      "mappings": {"default": {"default": "comillas izquierda", "defaultMP": "abre comillas"}},
      "category": "Pi"
    }, {
      "key": "203A",
      "mappings": {"default": {"default": "comillas derecha", "defaultMP": "cierra comillas"}},
      "category": "Pf"
    }, {
      "category": "Po",
      "key": "203B",
      "mappings": {"default": {"default": "marca de referencia"}}
    }, {
      "category": "Po",
      "key": "203C",
      "mappings": {"default": {"default": "doble signo de exclamación"}}
    }, {"category": "Po", "key": "203D", "mappings": {"default": {"default": "interrobang"}}}, {
      "key": "203E",
      "mappings": {"default": {"default": "barra superior"}},
      "category": "Po"
    }, {"category": "Pc", "key": "203F", "mappings": {"default": {"default": "undertie"}}}, {
      "key": "2040",
      "mappings": {"default": {"default": "ligadura"}},
      "category": "Pc"
    }, {"key": "2041", "mappings": {"default": {"default": "punto de inserción"}}, "category": "Po"}, {
      "category": "Po",
      "key": "2042",
      "mappings": {"default": {"default": "asterismo"}}
    }, {"key": "2043", "mappings": {"default": {"default": "guión bala"}}, "category": "Po"}, {
      "key": "2044",
      "mappings": {"default": {"default": "barra de fracción"}},
      "category": "Sm"
    }, {
      "category": "Po",
      "key": "2047",
      "mappings": {"default": {"default": "doble signo de interrogación"}}
    }, {
      "category": "Po",
      "key": "2048",
      "mappings": {"default": {"default": "pregunta Signo de exclamación"}}
    }, {
      "category": "Po",
      "key": "2049",
      "mappings": {"default": {"default": "signo de interrogación de exclamación"}}
    }, {
      "category": "Po",
      "key": "204B",
      "mappings": {"default": {"default": "signo de Pilcrow invertido"}}
    }, {
      "category": "Po",
      "key": "204C",
      "mappings": {"default": {"default": "bala negra hacia la izquierda"}}
    }, {
      "category": "Po",
      "key": "204D",
      "mappings": {"default": {"default": "bala negra hacia la derecha"}}
    }, {"category": "Po", "key": "204E", "mappings": {"default": {"default": "bajo asterisco"}}}, {
      "key": "204F",
      "mappings": {"default": {"default": "punto y coma invertido"}},
      "category": "Po"
    }, {"key": "2050", "mappings": {"default": {"default": "cierre superior"}}, "category": "Po"}, {
      "category": "Po",
      "key": "2051",
      "mappings": {"default": {"default": "dos asteriscos alineados verticalmente"}}
    }, {
      "category": "Sm",
      "key": "2052",
      "mappings": {"default": {"default": "signo de menos comercial"}}
    }, {"category": "Po", "key": "2053", "mappings": {"default": {"default": "swung Dash"}}}, {
      "category": "Pc",
      "key": "2054",
      "mappings": {"default": {"default": "ropa interior invertida"}}
    }, {
      "category": "Po",
      "key": "2055",
      "mappings": {"default": {"default": "signo de puntuación de la flor"}}
    }, {
      "category": "Po",
      "key": "2056",
      "mappings": {"default": {"default": "puntuación de tres puntos"}}
    }, {"key": "2057", "mappings": {"default": {"default": "cuarta"}}, "category": "Po"}, {
      "category": "Po",
      "key": "2058",
      "mappings": {"default": {"default": "puntuación de cuatro puntos"}}
    }, {
      "category": "Po",
      "key": "2059",
      "mappings": {"default": {"default": "puntuación de cinco puntos"}}
    }, {
      "category": "Po",
      "key": "205A",
      "mappings": {"default": {"default": "puntuación de dos puntos"}}
    }, {
      "category": "Po",
      "key": "205B",
      "mappings": {"default": {"default": "marca de cuatro puntos"}}
    }, {"category": "Po", "key": "205C", "mappings": {"default": {"default": "cruz punteada"}}}, {
      "category": "Po",
      "key": "205D",
      "mappings": {"default": {"default": "tricolon"}}
    }, {
      "category": "Po",
      "key": "205E",
      "mappings": {"default": {"default": "cuatro puntos verticales"}}
    }, {
      "category": "Sm",
      "key": "207A",
      "mappings": {"default": {"default": "superscript Plus Sign"}}
    }, {"category": "Sm", "key": "207B", "mappings": {"default": {"default": "superíndice menos"}}}, {
      "category": "Sm",
      "key": "207C",
      "mappings": {"default": {"default": "superscript Equals Sign"}}
    }, {
      "category": "Ps",
      "key": "207D",
      "mappings": {"default": {"default": "paréntesis izquierdo superíndice"}}
    }, {
      "category": "Pe",
      "key": "207E",
      "mappings": {"default": {"default": "paréntesis derecho superíndice"}}
    }, {
      "category": "Sm",
      "key": "208A",
      "mappings": {"default": {"default": "signo de subíndice más"}}
    }, {"category": "Sm", "key": "208B", "mappings": {"default": {"default": "subíndice menos"}}}, {
      "category": "Sm",
      "key": "208C",
      "mappings": {"default": {"default": "signo de subíndice es igual"}}
    }, {
      "category": "Ps",
      "key": "208D",
      "mappings": {"default": {"default": "subíndice paréntesis izquierdo"}}
    }, {
      "category": "Pe",
      "key": "208E",
      "mappings": {"default": {"default": "subíndice de paréntesis derecho"}}
    }, {"category": "So", "key": "214A", "mappings": {"default": {"default": "línea de propiedad"}}}, {
      "category": "Sm",
      "key": "214B",
      "mappings": {"default": {"default": "ampersand convertido"}}
    }, {"category": "So", "key": "214C", "mappings": {"default": {"default": "por signo"}}}, {
      "category": "So",
      "key": "214D",
      "mappings": {"default": {"default": "aktieselskab"}}
    }, {"category": "Ll", "key": "214E", "mappings": {"default": {"default": "torneado pequeño f"}}}, {
      "key": "2200",
      "mappings": {"default": {"default": "para todo"}},
      "category": "Sm"
    }, {"key": "2201", "mappings": {"default": {"default": "complementario"}}, "category": "Sm"}, {
      "key": "2203",
      "mappings": {"default": {"default": "existe"}},
      "category": "Sm"
    }, {"key": "2204", "mappings": {"default": {"default": "no existe"}}, "category": "Sm"}, {
      "key": "2205",
      "mappings": {"default": {"default": "conjunto vacío"}},
      "category": "Sm"
    }, {"key": "2206", "mappings": {"default": {"default": "incremento"}}, "category": "Sm"}, {
      "key": "2208",
      "mappings": {"default": {"default": "perteneciente a"}},
      "category": "Sm"
    }, {"key": "2209", "mappings": {"default": {"default": "no perteneciente a"}}, "category": "Sm"}, {
      "key": "220A",
      "mappings": {"default": {"default": "perteneciente a"}},
      "category": "Sm"
    }, {"key": "220B", "mappings": {"default": {"default": "comprende a"}}, "category": "Sm"}, {
      "key": "220C",
      "mappings": {"default": {"default": "no comprende a"}},
      "category": "Sm"
    }, {"key": "220D", "mappings": {"default": {"default": "comprende a"}}, "category": "Sm"}, {
      "key": "220E",
      "mappings": {"default": {"default": "fin de demostración"}},
      "category": "Sm"
    }, {"key": "220F", "mappings": {"default": {"default": "producto"}}, "category": "Sm"}, {
      "key": "2210",
      "mappings": {"default": {"default": "coproducto"}},
      "category": "Sm"
    }, {"key": "2211", "mappings": {"default": {"default": "sumatorio"}}, "category": "Sm"}, {
      "key": "2212",
      "mappings": {"default": {"default": "menos"}},
      "category": "Sm"
    }, {"key": "2213", "mappings": {"default": {"default": "menos más"}}, "category": "Sm"}, {
      "key": "2214",
      "mappings": {"default": {"default": "punto más"}},
      "category": "Sm"
    }, {"key": "2215", "mappings": {"default": {"default": "barra de división"}}, "category": "Sm"}, {
      "key": "2216",
      "mappings": {"default": {"default": "menos"}},
      "category": "Sm"
    }, {"key": "2217", "mappings": {"default": {"default": "asterisco"}}, "category": "Sm"}, {
      "key": "2218",
      "mappings": {"default": {"default": "composición"}},
      "category": "Sm"
    }, {"key": "2219", "mappings": {"default": {"default": "bala"}}, "category": "Sm"}, {
      "key": "221A",
      "mappings": {"default": {"default": "raíz"}},
      "category": "Sm"
    }, {"key": "221B", "mappings": {"default": {"default": "raíz cúbica"}}, "category": "Sm"}, {
      "key": "221C",
      "mappings": {"default": {"default": "raíz cuarta"}},
      "category": "Sm"
    }, {"key": "221D", "mappings": {"default": {"default": "proporcional a"}}, "category": "Sm"}, {
      "key": "221E",
      "mappings": {"default": {"default": "infinito"}},
      "category": "Sm"
    }, {"key": "221F", "mappings": {"default": {"default": "ángulo recto"}}, "category": "Sm"}, {
      "key": "2220",
      "mappings": {"default": {"default": "ángulo"}},
      "category": "Sm"
    }, {"key": "2221", "mappings": {"default": {"default": "ángulo medido"}}, "category": "Sm"}, {
      "key": "2222",
      "mappings": {"default": {"default": "ángulo esférico"}},
      "category": "Sm"
    }, {"key": "2223", "mappings": {"default": {"default": "divide a"}}, "category": "Sm"}, {
      "key": "2224",
      "mappings": {"default": {"default": "no divide a"}},
      "category": "Sm"
    }, {"key": "2225", "mappings": {"default": {"default": "paralela a"}}, "category": "Sm"}, {
      "key": "2226",
      "mappings": {"default": {"default": "no paralela a"}},
      "category": "Sm"
    }, {"key": "2227", "mappings": {"default": {"default": "y"}}, "category": "Sm"}, {
      "key": "2228",
      "mappings": {"default": {"default": "o"}},
      "category": "Sm"
    }, {"key": "2229", "mappings": {"default": {"default": "intersección"}}, "category": "Sm"}, {
      "key": "222A",
      "mappings": {"default": {"default": "unión"}},
      "category": "Sm"
    }, {"key": "222B", "mappings": {"default": {"default": "integral"}}, "category": "Sm"}, {
      "key": "222C",
      "mappings": {"default": {"default": "integral doble"}},
      "category": "Sm"
    }, {"key": "222D", "mappings": {"default": {"default": "integral triple"}}, "category": "Sm"}, {
      "key": "222E",
      "mappings": {"default": {"default": "integral de contorno"}},
      "category": "Sm"
    }, {
      "key": "222F",
      "mappings": {"default": {"default": "integral de superficie"}},
      "category": "Sm"
    }, {"key": "2230", "mappings": {"default": {"default": "integral de volumen"}}, "category": "Sm"}, {
      "key": "2231",
      "mappings": {"default": {"default": "integral de contorno en sentido horario"}},
      "category": "Sm"
    }, {
      "key": "2232",
      "mappings": {"default": {"default": "integral de contorno en sentido horario"}},
      "category": "Sm"
    }, {
      "key": "2233",
      "mappings": {"default": {"default": "integral de contorno en sentido antihorario"}},
      "category": "Sm"
    }, {"key": "2234", "mappings": {"default": {"default": "por lo tanto"}}, "category": "Sm"}, {
      "key": "2235",
      "mappings": {"default": {"default": "porque"}},
      "category": "Sm"
    }, {"key": "2236", "mappings": {"default": {"default": "razón"}}, "category": "Sm"}, {
      "key": "2237",
      "mappings": {"default": {"default": "proporción"}},
      "category": "Sm"
    }, {"key": "2238", "mappings": {"default": {"default": "punto menos"}}, "category": "Sm"}, {
      "key": "2239",
      "mappings": {"default": {"default": "exceso"}},
      "category": "Sm"
    }, {"key": "223A", "mappings": {"default": {"default": "proporción geométrica"}}, "category": "Sm"}, {
      "key": "223B",
      "mappings": {"default": {"default": "homotecia"}},
      "category": "Sm"
    }, {"key": "223C", "mappings": {"default": {"default": "tilde"}}, "category": "Sm"}, {
      "key": "223D",
      "mappings": {"default": {"default": "tilde invertido"}},
      "category": "Sm"
    }, {
      "key": "223E",
      "mappings": {"default": {"default": "s horizontal invertida"}},
      "category": "Sm"
    }, {"key": "223F", "mappings": {"default": {"default": "sinusoide"}}, "category": "Sm"}, {
      "key": "2240",
      "mappings": {"default": {"default": "producto de guirnalda"}},
      "category": "Sm"
    }, {"key": "2241", "mappings": {"default": {"default": "no tilde"}}, "category": "Sm"}, {
      "key": "2242",
      "mappings": {"default": {"default": "menos tilde"}},
      "category": "Sm"
    }, {
      "key": "2243",
      "mappings": {"default": {"default": "asintóticamente igual a"}},
      "category": "Sm"
    }, {
      "key": "2244",
      "mappings": {"default": {"default": "no asintóticamente igual a"}},
      "category": "Sm"
    }, {
      "key": "2245",
      "mappings": {"default": {"default": "aproximadamente igual a"}},
      "category": "Sm"
    }, {
      "key": "2246",
      "mappings": {"default": {"default": "aproximado pero no igual a"}},
      "category": "Sm"
    }, {"key": "2247", "mappings": {"default": {"default": "no aproximado a"}}, "category": "Sm"}, {
      "key": "2248",
      "mappings": {"default": {"default": "aproximado", "defaultMP": "casi igual a"}},
      "category": "Sm"
    }, {"key": "2249", "mappings": {"default": {"default": "no es casi igual a"}}, "category": "Sm"}, {
      "key": "224A",
      "mappings": {"default": {"default": "igual o casi igual a"}},
      "category": "Sm"
    }, {"key": "224B", "mappings": {"default": {"default": "triple tilde"}}, "category": "Sm"}, {
      "key": "224C",
      "mappings": {"default": {"default": "todo igual a"}},
      "category": "Sm"
    }, {"key": "224D", "mappings": {"default": {"default": "equivalente a"}}, "category": "Sm"}, {
      "key": "224E",
      "mappings": {"default": {"default": "geométricamente equivalente a"}},
      "category": "Sm"
    }, {"key": "224F", "mappings": {"default": {"default": "diferencia entre"}}, "category": "Sm"}, {
      "key": "2250",
      "mappings": {"default": {"default": "se acerca al límite"}},
      "category": "Sm"
    }, {
      "key": "2251",
      "mappings": {"default": {"default": "geométricamente igual a"}},
      "category": "Sm"
    }, {
      "key": "2252",
      "mappings": {"default": {"default": "aproximadamente igual a o imagen de"}},
      "category": "Sm"
    }, {
      "key": "2253",
      "mappings": {"default": {"default": "imagen de o aproximadamente igual a"}},
      "category": "Sm"
    }, {"key": "2254", "mappings": {"default": {"default": "dos puntos igual"}}, "category": "Sm"}, {
      "key": "2255",
      "mappings": {"default": {"default": "igual dos puntos"}},
      "category": "Sm"
    }, {
      "key": "2256",
      "mappings": {"default": {"default": "igual incluyendo anillo"}},
      "category": "Sm"
    }, {"key": "2257", "mappings": {"default": {"default": "igual anillo a"}}, "category": "Sm"}, {
      "key": "2258",
      "mappings": {"default": {"default": "corresponde a"}},
      "category": "Sm"
    }, {"key": "2259", "mappings": {"default": {"default": "estima a"}}, "category": "Sm"}, {
      "key": "225A",
      "mappings": {"default": {"default": "equiangular con"}},
      "category": "Sm"
    }, {"key": "225B", "mappings": {"default": {"default": "igual con estrella"}}, "category": "Sm"}, {
      "key": "225C",
      "mappings": {"default": {"default": "igual con delta"}},
      "category": "Sm"
    }, {
      "key": "225D",
      "mappings": {"default": {"default": "igual por definición a"}},
      "category": "Sm"
    }, {"key": "225E", "mappings": {"default": {"default": "medido por"}}, "category": "Sm"}, {
      "key": "225F",
      "mappings": {"default": {"default": "igual con interrogante"}},
      "category": "Sm"
    }, {
      "key": "2260",
      "mappings": {"default": {"default": "no es igual a", "defaultMP": "distinto de"}},
      "category": "Sm"
    }, {"key": "2261", "mappings": {"default": {"default": "idéntico a"}}, "category": "Sm"}, {
      "key": "2262",
      "mappings": {"default": {"default": "no es idéntico a"}},
      "category": "Sm"
    }, {
      "key": "2263",
      "mappings": {"default": {"default": "estrictamente equivalente a"}},
      "category": "Sm"
    }, {"key": "2264", "mappings": {"default": {"default": "menor o igual que"}}, "category": "Sm"}, {
      "key": "2265",
      "mappings": {"default": {"default": "mayor o igual que"}},
      "category": "Sm"
    }, {"key": "2266", "mappings": {"default": {"default": "menor o igual que"}}, "category": "Sm"}, {
      "key": "2267",
      "mappings": {"default": {"default": "mayor o igual que"}},
      "category": "Sm"
    }, {
      "key": "2268",
      "mappings": {"default": {"default": "estrictamente menor que"}},
      "category": "Sm"
    }, {
      "key": "2269",
      "mappings": {"default": {"default": "estrictamente mayor que"}},
      "category": "Sm"
    }, {"key": "226A", "mappings": {"default": {"default": "mucho menor que"}}, "category": "Sm"}, {
      "key": "226B",
      "mappings": {"default": {"default": "mucho mayor que"}},
      "category": "Sm"
    }, {"key": "226C", "mappings": {"default": {"default": "entre"}}, "category": "Sm"}, {
      "key": "226D",
      "mappings": {"default": {"default": "no equivalente a"}},
      "category": "Sm"
    }, {"key": "226E", "mappings": {"default": {"default": "no menor que"}}, "category": "Sm"}, {
      "key": "226F",
      "mappings": {"default": {"default": "no mayor que"}},
      "category": "Sm"
    }, {"key": "2270", "mappings": {"default": {"default": "no menor ni igual a"}}, "category": "Sm"}, {
      "key": "2271",
      "mappings": {"default": {"default": "no mayor ni igual a"}},
      "category": "Sm"
    }, {"key": "2272", "mappings": {"default": {"default": "menor o equivalente a"}}, "category": "Sm"}, {
      "key": "2273",
      "mappings": {"default": {"default": "mayor o equivalente a"}},
      "category": "Sm"
    }, {
      "key": "2274",
      "mappings": {"default": {"default": "no menor ni equivalente a"}},
      "category": "Sm"
    }, {
      "key": "2275",
      "mappings": {"default": {"default": "no mayor ni equivalente a"}},
      "category": "Sm"
    }, {"key": "2276", "mappings": {"default": {"default": "menor o mayor que"}}, "category": "Sm"}, {
      "key": "2277",
      "mappings": {"default": {"default": "mayor o menor que"}},
      "category": "Sm"
    }, {"key": "2278", "mappings": {"default": {"default": "no menor ni mayor que"}}, "category": "Sm"}, {
      "key": "2279",
      "mappings": {"default": {"default": "no mayor ni menor que"}},
      "category": "Sm"
    }, {"key": "227A", "mappings": {"default": {"default": "precede a"}}, "category": "Sm"}, {
      "key": "227B",
      "mappings": {"default": {"default": "sigue a"}},
      "category": "Sm"
    }, {"key": "227C", "mappings": {"default": {"default": "precede o es igual a"}}, "category": "Sm"}, {
      "key": "227D",
      "mappings": {"default": {"default": "sigue o es igual a"}},
      "category": "Sm"
    }, {
      "key": "227E",
      "mappings": {"default": {"default": "precede o es equivalente a"}},
      "category": "Sm"
    }, {
      "key": "227F",
      "mappings": {"default": {"default": "sigue o es equivalente a"}},
      "category": "Sm"
    }, {"key": "2280", "mappings": {"default": {"default": "no precede a"}}, "category": "Sm"}, {
      "key": "2281",
      "mappings": {"default": {"default": "no sigue a"}},
      "category": "Sm"
    }, {"key": "2282", "mappings": {"default": {"default": "incluido en"}}, "category": "Sm"}, {
      "key": "2283",
      "mappings": {"default": {"default": "contiene a"}},
      "category": "Sm"
    }, {"key": "2284", "mappings": {"default": {"default": "no incluido en"}}, "category": "Sm"}, {
      "key": "2285",
      "mappings": {"default": {"default": "no contiene a"}},
      "category": "Sm"
    }, {"key": "2286", "mappings": {"default": {"default": "incluido o igual a"}}, "category": "Sm"}, {
      "key": "2287",
      "mappings": {"default": {"default": "contiene o es igual a"}},
      "category": "Sm"
    }, {
      "key": "2288",
      "mappings": {"default": {"default": "no incluido ni igual a"}},
      "category": "Sm"
    }, {
      "key": "2289",
      "mappings": {"default": {"default": "no contiene ni es igual a"}},
      "category": "Sm"
    }, {
      "key": "228A",
      "mappings": {"default": {"default": "incluido estrictamente en"}},
      "category": "Sm"
    }, {
      "key": "228B",
      "mappings": {"default": {"default": "contiene estrictamente a"}},
      "category": "Sm"
    }, {"key": "228C", "mappings": {"default": {"default": "familia de conjuntos"}}, "category": "Sm"}, {
      "key": "228D",
      "mappings": {"default": {"default": "producto de familia de conjuntos"}},
      "category": "Sm"
    }, {
      "key": "228E",
      "mappings": {"default": {"default": "unión de familia de conjuntos"}},
      "category": "Sm"
    }, {"key": "228F", "mappings": {"default": {"default": "imagen cuadrada de"}}, "category": "Sm"}, {
      "key": "2290",
      "mappings": {"default": {"default": "original cuadrado de"}},
      "category": "Sm"
    }, {
      "key": "2291",
      "mappings": {"default": {"default": "imagen cuadrada de o igual a"}},
      "category": "Sm"
    }, {
      "key": "2292",
      "mappings": {"default": {"default": "original cuadrado de o igual a"}},
      "category": "Sm"
    }, {"key": "2293", "mappings": {"default": {"default": "intersección cuadrada"}}, "category": "Sm"}, {
      "key": "2294",
      "mappings": {"default": {"default": "unión cuadrada"}},
      "category": "Sm"
    }, {"key": "2295", "mappings": {"default": {"default": "más en círculo"}}, "category": "Sm"}, {
      "key": "2296",
      "mappings": {"default": {"default": "menos en círculo"}},
      "category": "Sm"
    }, {"key": "2297", "mappings": {"default": {"default": "por en círculo"}}, "category": "Sm"}, {
      "key": "2298",
      "mappings": {"default": {"default": "barra en círculo"}},
      "category": "Sm"
    }, {"key": "2299", "mappings": {"default": {"default": "punto en círculo"}}, "category": "Sm"}, {
      "key": "229A",
      "mappings": {"default": {"default": "anillo en círculo"}},
      "category": "Sm"
    }, {"key": "229B", "mappings": {"default": {"default": "asterisco en círculo"}}, "category": "Sm"}, {
      "key": "229C",
      "mappings": {"default": {"default": "igual en círculo"}},
      "category": "Sm"
    }, {"key": "229D", "mappings": {"default": {"default": "menos en círculo"}}, "category": "Sm"}, {
      "key": "229E",
      "mappings": {"default": {"default": "más en cuadrado"}},
      "category": "Sm"
    }, {"key": "229F", "mappings": {"default": {"default": "menos en cuadrado"}}, "category": "Sm"}, {
      "key": "22A0",
      "mappings": {"default": {"default": "veces en cuadrado"}},
      "category": "Sm"
    }, {"key": "22A1", "mappings": {"default": {"default": "punto en cuadrado"}}, "category": "Sm"}, {
      "key": "22A2",
      "mappings": {"default": {"default": "t horizontal hacia la izquierda"}},
      "category": "Sm"
    }, {
      "key": "22A3",
      "mappings": {"default": {"default": "t horizontal hacia la derecha"}},
      "category": "Sm"
    }, {"key": "22A4", "mappings": {"default": {"default": "perpendicular"}}, "category": "Sm"}, {
      "key": "22A5",
      "mappings": {"default": {"default": "perpendicular invertida"}},
      "category": "Sm"
    }, {"key": "22A6", "mappings": {"default": {"default": "afirmación"}}, "category": "Sm"}, {
      "key": "22A7",
      "mappings": {"default": {"default": "modela"}},
      "category": "Sm"
    }, {"key": "22A8", "mappings": {"default": {"default": "verdadero"}}, "category": "Sm"}, {
      "key": "22A9",
      "mappings": {"default": {"default": "obliga"}},
      "category": "Sm"
    }, {
      "key": "22AA",
      "mappings": {"default": {"default": "triple barra vertical torniquete derecho"}},
      "category": "Sm"
    }, {
      "key": "22AB",
      "mappings": {"default": {"default": "doble barra vertical doble torniquete derecho"}},
      "category": "Sm"
    }, {"key": "22AC", "mappings": {"default": {"default": "no prueba"}}, "category": "Sm"}, {
      "key": "22AD",
      "mappings": {"default": {"default": "falso"}},
      "category": "Sm"
    }, {"key": "22AE", "mappings": {"default": {"default": "no obliga"}}, "category": "Sm"}, {
      "key": "22AF",
      "mappings": {"default": {"default": "doble barra vertical doble torniquete derecho negada"}},
      "category": "Sm"
    }, {"key": "22B0", "mappings": {"default": {"default": "precede respecto de"}}, "category": "Sm"}, {
      "key": "22B1",
      "mappings": {"default": {"default": "sigue respecto de"}},
      "category": "Sm"
    }, {"key": "22B2", "mappings": {"default": {"default": "subgrupo normal de"}}, "category": "Sm"}, {
      "key": "22B3",
      "mappings": {"default": {"default": "contiene como subgrupo normal a"}},
      "category": "Sm"
    }, {
      "key": "22B4",
      "mappings": {"default": {"default": "subgrupo normal o es igual a"}},
      "category": "Sm"
    }, {
      "key": "22B5",
      "mappings": {"default": {"default": "contiene como subgrupo normal o es igual a"}},
      "category": "Sm"
    }, {"key": "22B6", "mappings": {"default": {"default": "original de"}}, "category": "Sm"}, {
      "key": "22B7",
      "mappings": {"default": {"default": "imagen de"}},
      "category": "Sm"
    }, {"key": "22B8", "mappings": {"default": {"default": "multifunción"}}, "category": "Sm"}, {
      "key": "22B9",
      "mappings": {"default": {"default": "matriz hermitiana conjugada"}},
      "category": "Sm"
    }, {"key": "22BA", "mappings": {"default": {"default": "interpola"}}, "category": "Sm"}, {
      "key": "22BB",
      "mappings": {"default": {"default": "o excluyente"}},
      "category": "Sm"
    }, {"key": "22BC", "mappings": {"default": {"default": "no y"}}, "category": "Sm"}, {
      "key": "22BD",
      "mappings": {"default": {"default": "no o"}},
      "category": "Sm"
    }, {"key": "22BF", "mappings": {"default": {"default": "triángulo rectángulo"}}, "category": "Sm"}, {
      "key": "22C0",
      "mappings": {"default": {"default": "y"}},
      "category": "Sm"
    }, {"key": "22C1", "mappings": {"default": {"default": "o"}}, "category": "Sm"}, {
      "key": "22C2",
      "mappings": {"default": {"default": "intersección"}},
      "category": "Sm"
    }, {"key": "22C3", "mappings": {"default": {"default": "unión"}}, "category": "Sm"}, {
      "key": "22C4",
      "mappings": {"default": {"default": "operador diamante"}},
      "category": "Sm"
    }, {"key": "22C5", "mappings": {"default": {"default": "punto"}}, "category": "Sm"}, {
      "key": "22C6",
      "mappings": {"default": {"default": "estrella"}},
      "category": "Sm"
    }, {"key": "22C7", "mappings": {"default": {"default": "dividido por"}}, "category": "Sm"}, {
      "key": "22C8",
      "mappings": {"default": {"default": "moño"}},
      "category": "Sm"
    }, {
      "key": "22C9",
      "mappings": {"default": {"default": "factor normal izquierdo producto semidirecto"}},
      "category": "Sm"
    }, {
      "key": "22CA",
      "mappings": {"default": {"default": "factor normal derecho producto semidirecto"}},
      "category": "Sm"
    }, {
      "key": "22CB",
      "mappings": {"default": {"default": "producto semidirecto izquierdo"}},
      "category": "Sm"
    }, {
      "key": "22CC",
      "mappings": {"default": {"default": "producto semidirecto derecho"}},
      "category": "Sm"
    }, {
      "key": "22CD",
      "mappings": {"default": {"default": "tilde invertida igual a"}},
      "category": "Sm"
    }, {"key": "22CE", "mappings": {"default": {"default": "o rizada"}}, "category": "Sm"}, {
      "key": "22CF",
      "mappings": {"default": {"default": "y rizada"}},
      "category": "Sm"
    }, {"key": "22D0", "mappings": {"default": {"default": "doble incluido"}}, "category": "Sm"}, {
      "key": "22D1",
      "mappings": {"default": {"default": "doble contiene"}},
      "category": "Sm"
    }, {"key": "22D2", "mappings": {"default": {"default": "doble intersección"}}, "category": "Sm"}, {
      "key": "22D3",
      "mappings": {"default": {"default": "doble unión"}},
      "category": "Sm"
    }, {"key": "22D4", "mappings": {"default": {"default": "bieldo"}}, "category": "Sm"}, {
      "key": "22D5",
      "mappings": {"default": {"default": "paralela o igual a"}},
      "category": "Sm"
    }, {"key": "22D6", "mappings": {"default": {"default": "menor que con punto"}}, "category": "Sm"}, {
      "key": "22D7",
      "mappings": {"default": {"default": "mayor que con punto"}},
      "category": "Sm"
    }, {"key": "22D8", "mappings": {"default": {"default": "mucho menor que"}}, "category": "Sm"}, {
      "key": "22D9",
      "mappings": {"default": {"default": "mucho mayor que"}},
      "category": "Sm"
    }, {
      "key": "22DA",
      "mappings": {"default": {"default": "menor igual o mayor que"}},
      "category": "Sm"
    }, {
      "key": "22DB",
      "mappings": {"default": {"default": "mayor igual o menor que"}},
      "category": "Sm"
    }, {"key": "22DC", "mappings": {"default": {"default": "menor o igual que"}}, "category": "Sm"}, {
      "key": "22DD",
      "mappings": {"default": {"default": "mayor o igual que"}},
      "category": "Sm"
    }, {"key": "22DE", "mappings": {"default": {"default": "precede o es igual a"}}, "category": "Sm"}, {
      "key": "22DF",
      "mappings": {"default": {"default": "sigue o es igual a"}},
      "category": "Sm"
    }, {
      "key": "22E0",
      "mappings": {"default": {"default": "no precede ni es igual a"}},
      "category": "Sm"
    }, {
      "key": "22E1",
      "mappings": {"default": {"default": "no sigue ni es igual a"}},
      "category": "Sm"
    }, {
      "key": "22E2",
      "mappings": {"default": {"default": "no es imagen cuadrada ni igual a"}},
      "category": "Sm"
    }, {
      "key": "22E3",
      "mappings": {"default": {"default": "no es original cuadrada ni igual a"}},
      "category": "Sm"
    }, {
      "key": "22E4",
      "mappings": {"default": {"default": "imagen cuadrada estricta"}},
      "category": "Sm"
    }, {
      "key": "22E5",
      "mappings": {"default": {"default": "original cuadrada estricta"}},
      "category": "Sm"
    }, {
      "key": "22E6",
      "mappings": {"default": {"default": "menor que no equivalente a"}},
      "category": "Sm"
    }, {
      "key": "22E7",
      "mappings": {"default": {"default": "mayor que no equivalente a"}},
      "category": "Sm"
    }, {
      "key": "22E8",
      "mappings": {"default": {"default": "precede no equivalente a"}},
      "category": "Sm"
    }, {
      "key": "22E9",
      "mappings": {"default": {"default": "sigue no equivalente a"}},
      "category": "Sm"
    }, {"key": "22EA", "mappings": {"default": {"default": "no subgrupo normal de"}}, "category": "Sm"}, {
      "key": "22EB",
      "mappings": {"default": {"default": "no contiene como subgrupo normal a"}},
      "category": "Sm"
    }, {
      "key": "22EC",
      "mappings": {"default": {"default": "no es subgrupo normal ni igual a"}},
      "category": "Sm"
    }, {
      "key": "22ED",
      "mappings": {"default": {"default": "no contiene como subgrupo normal ni es igual a"}},
      "category": "Sm"
    }, {
      "key": "22EE",
      "mappings": {"default": {"default": "puntos suspensivos verticales"}},
      "category": "Sm"
    }, {
      "key": "22EF",
      "mappings": {"default": {"default": "puntos suspensivos altos"}},
      "category": "Sm"
    }, {
      "key": "22F0",
      "mappings": {"default": {"default": "puntos suspensivos diagonales subiendo"}},
      "category": "Sm"
    }, {
      "key": "22F1",
      "mappings": {"default": {"default": "puntos suspensivos diagonales bajando"}},
      "category": "Sm"
    }, {
      "key": "22F2",
      "mappings": {"default": {"default": "pertenece con trazo horizontal"}},
      "category": "Sm"
    }, {
      "key": "22F3",
      "mappings": {"default": {"default": "pertenece con barra vertical al fin de raya horizontal"}},
      "category": "Sm"
    }, {
      "key": "22F4",
      "mappings": {"default": {"default": "pertenece en pequeño con trazo vertical"}},
      "category": "Sm"
    }, {
      "key": "22F5",
      "mappings": {"default": {"default": "pertenece con punto arriba"}},
      "category": "Sm"
    }, {
      "key": "22F6",
      "mappings": {"default": {"default": "pertenece con barra arriba"}},
      "category": "Sm"
    }, {
      "key": "22F7",
      "mappings": {"default": {"default": "perteneciente subrayado"}},
      "category": "Sm"
    }, {
      "key": "22F8",
      "mappings": {"default": {"default": "perteneciente subrayado"}},
      "category": "Sm"
    }, {
      "key": "22F9",
      "mappings": {"default": {"default": "pertenece con dos trazos horizontales"}},
      "category": "Sm"
    }, {"key": "22FA", "mappings": {"default": {"default": "contiene o es igual a"}}, "category": "Sm"}, {
      "key": "22FB",
      "mappings": {"default": {"default": "contiene con marca"}},
      "category": "Sm"
    }, {"key": "22FC", "mappings": {"default": {"default": "contiene con marca"}}, "category": "Sm"}, {
      "key": "22FD",
      "mappings": {"default": {"default": "contiene con barra"}},
      "category": "Sm"
    }, {
      "key": "22FE",
      "mappings": {"default": {"default": "perteneciente con barra"}},
      "category": "Sm"
    }, {
      "key": "22FF",
      "mappings": {"default": {"default": "z anotación asociación de bolsa"}},
      "category": "Sm"
    }, {"key": "2300", "mappings": {"default": {"default": "diámetro"}}, "category": "So"}, {
      "key": "2302",
      "mappings": {"default": {"default": "casa"}},
      "category": "So"
    }, {"key": "2305", "mappings": {"default": {"default": "en proyectividad con"}}, "category": "So"}, {
      "key": "2306",
      "mappings": {"default": {"default": "en perspectividad con"}},
      "category": "So"
    }, {"category": "So", "key": "2307", "mappings": {"default": {"default": "línea ondulada"}}}, {
      "key": "2310",
      "mappings": {"default": {"default": "no invertido"}},
      "category": "So"
    }, {"category": "So", "key": "2311", "mappings": {"default": {"default": "pastillas cuadradas"}}}, {
      "key": "2312",
      "mappings": {"default": {"default": "arco"}},
      "category": "So"
    }, {"key": "2313", "mappings": {"default": {"default": "segmento"}}, "category": "So"}, {
      "category": "So",
      "key": "2314",
      "mappings": {"default": {"default": "sector"}}
    }, {"category": "So", "key": "2795", "mappings": {"default": {"default": "signo más pesado"}}}, {
      "category": "So",
      "key": "2796",
      "mappings": {"default": {"default": "signo menos pesado"}}
    }, {
      "category": "So",
      "key": "2797",
      "mappings": {"default": {"default": "signo de la división pesada"}}
    }, {"category": "So", "key": "27B0", "mappings": {"default": {"default": "bucle rizado"}}}, {
      "category": "So",
      "key": "27BF",
      "mappings": {"default": {"default": "doble bucle rizado"}}
    }, {
      "key": "27C1",
      "mappings": {"default": {"default": "triángulo encerrando triángulo"}},
      "category": "Sm"
    }, {"key": "27C2", "mappings": {"default": {"default": "perpendicular"}}, "category": "Sm"}, {
      "key": "27C3",
      "mappings": {"default": {"default": "incluido en"}},
      "category": "Sm"
    }, {"key": "27C4", "mappings": {"default": {"default": "contiene a"}}, "category": "Sm"}, {
      "key": "27C7",
      "mappings": {"default": {"default": "punto en círculo"}},
      "category": "Sm"
    }, {
      "key": "27C8",
      "mappings": {"default": {"default": "atrás tajo antes de subconjunto"}},
      "category": "Sm"
    }, {
      "key": "27C9",
      "mappings": {"default": {"default": "sobreconjunto antes de tajo"}},
      "category": "Sm"
    }, {
      "key": "27CA",
      "mappings": {"default": {"default": "barra vertical con golpe horizontal"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "27CB",
      "mappings": {"default": {"default": "diagonal matemático de levantamiento"}}
    }, {"key": "27CC", "mappings": {"default": {"default": "división larga"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "27CD",
      "mappings": {"default": {"default": "diagonal descendente matemática"}}
    }, {"category": "Sm", "key": "27CE", "mappings": {"default": {"default": "cuadrado Lógico Y"}}}, {
      "category": "Sm",
      "key": "27CF",
      "mappings": {"default": {"default": "cuadrado Lógico O"}}
    }, {
      "category": "Sm",
      "key": "27D0",
      "mappings": {"default": {"default": "diamante blanco con punto centrado"}}
    }, {"key": "27D1", "mappings": {"default": {"default": "y con punto interior"}}, "category": "Sm"}, {
      "key": "27D2",
      "mappings": {"default": {"default": "elemento de apertura hacia arriba"}},
      "category": "Sm"
    }, {
      "key": "27D3",
      "mappings": {"default": {"default": "ángulo inferior derecho con punto"}},
      "category": "Sm"
    }, {
      "key": "27D4",
      "mappings": {"default": {"default": "ángulo inferior izquierdo con punto"}},
      "category": "Sm"
    }, {
      "key": "27D5",
      "mappings": {"default": {"default": "izquierdo exterior une"}},
      "category": "Sm"
    }, {"key": "27D6", "mappings": {"default": {"default": "derecho exterior une"}}, "category": "Sm"}, {
      "key": "27D7",
      "mappings": {"default": {"default": "lleno exterior une"}},
      "category": "Sm"
    }, {"key": "27D8", "mappings": {"default": {"default": "perpendicular inversa"}}, "category": "Sm"}, {
      "key": "27D9",
      "mappings": {"default": {"default": "perpendicular"}},
      "category": "Sm"
    }, {
      "key": "27DA",
      "mappings": {"default": {"default": "izquierdo y derecho doble torniquete"}},
      "category": "Sm"
    }, {"key": "27DB", "mappings": {"default": {"default": "doble T horizontal"}}, "category": "Sm"}, {
      "key": "27DC",
      "mappings": {"default": {"default": "multifunción por la izquierda"}},
      "category": "Sm"
    }, {
      "key": "27DD",
      "mappings": {"default": {"default": "t horizontal derecha larga"}},
      "category": "Sm"
    }, {
      "key": "27DE",
      "mappings": {"default": {"default": "t horizontal izquierda larga"}},
      "category": "Sm"
    }, {
      "key": "27DF",
      "mappings": {"default": {"default": "perpendicular inversa con círculo arriba"}},
      "category": "Sm"
    }, {
      "key": "27E0",
      "mappings": {"default": {"default": "pastilla dividida por regla horizontal"}},
      "category": "Sm"
    }, {
      "key": "27E1",
      "mappings": {"default": {"default": "diamante cóncavo vacío"}},
      "category": "Sm"
    }, {
      "key": "27E2",
      "mappings": {"default": {"default": "diamante cóncavo con tictac hacia izquierda"}},
      "category": "Sm"
    }, {
      "key": "27E3",
      "mappings": {"default": {"default": "diamante cóncavo con tictac hacia derecha"}},
      "category": "Sm"
    }, {
      "key": "27E4",
      "mappings": {"default": {"default": "cuadrado con tictac hacia izquierda"}},
      "category": "Sm"
    }, {
      "key": "27E5",
      "mappings": {"default": {"default": "cuadrado con tictac hacia derecha"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "292B",
      "mappings": {"default": {"default": "cruce en diagonal Cruce en diagonal Caída en diagonal"}}
    }, {
      "category": "Sm",
      "key": "292C",
      "mappings": {"default": {"default": "diagonal descendente Cruce diagonal ascendente"}}
    }, {
      "category": "Sm",
      "key": "2980",
      "mappings": {"default": {"default": "delimitador de barra vertical triple"}}
    }, {
      "category": "Sm",
      "key": "2981",
      "mappings": {"default": {"default": "punto de notación Z"}}
    }, {
      "category": "Sm",
      "key": "2982",
      "mappings": {"default": {"default": "tipo de notación Z Colon"}}
    }, {"category": "Sm", "key": "2999", "mappings": {"default": {"default": "valla punteada"}}}, {
      "key": "299A",
      "mappings": {"default": {"default": "zigzag en vertical"}},
      "category": "Sm"
    }, {
      "key": "29B0",
      "mappings": {"default": {"default": "conjunto vacío invertido"}},
      "category": "Sm"
    }, {
      "key": "29B1",
      "mappings": {"default": {"default": "conjunto vacío con barra"}},
      "category": "Sm"
    }, {
      "key": "29B2",
      "mappings": {"default": {"default": "conjunto vacío con círculo"}},
      "category": "Sm"
    }, {"key": "29B5", "mappings": {"default": {"default": "círculo con barra"}}, "category": "Sm"}, {
      "key": "29B6",
      "mappings": {"default": {"default": "barra en círculo"}},
      "category": "Sm"
    }, {
      "key": "29B7",
      "mappings": {"default": {"default": "paralelas en círculo"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29B8",
      "mappings": {"default": {"default": "círculo inverso solidus"}}
    }, {
      "key": "29B9",
      "mappings": {"default": {"default": "perpendicular en círculo"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29BA",
      "mappings": {"default": {"default": "círculo dividido por barra horizontal y mitad superior dividido por barra vertical"}}
    }, {
      "key": "29BB",
      "mappings": {"default": {"default": "x superimpresa en círculo"}},
      "category": "Sm"
    }, {
      "key": "29BC",
      "mappings": {"default": {"default": "circled anticlockwise rotated division"}},
      "category": "Sm"
    }, {
      "key": "29BE",
      "mappings": {"default": {"default": "bullet vacía en círculo"}},
      "category": "Sm"
    }, {"key": "29BF", "mappings": {"default": {"default": "bullet en círculo"}}, "category": "Sm"}, {
      "key": "29C0",
      "mappings": {"default": {"default": "menor que en círculo"}},
      "category": "Sm"
    }, {"key": "29C1", "mappings": {"default": {"default": "mayor que en círculo"}}, "category": "Sm"}, {
      "key": "29C2",
      "mappings": {"default": {"default": "círculo con círculo a la derecha"}},
      "category": "Sm"
    }, {
      "key": "29C3",
      "mappings": {"default": {"default": "circle with two horizontal strokes to the right"}},
      "category": "Sm"
    }, {
      "key": "29C4",
      "mappings": {"default": {"default": "cuadrado con diagonal ascendente"}},
      "category": "Sm"
    }, {
      "key": "29C5",
      "mappings": {"default": {"default": "cuadrado con diagonal descendente"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "29C6", "mappings": {"default": {"default": "asterisco cuadrado"}}}, {
      "category": "Sm",
      "key": "29C7",
      "mappings": {"default": {"default": "pequeño círculo cuadrado"}}
    }, {"category": "Sm", "key": "29C8", "mappings": {"default": {"default": "cuadrado cuadrado"}}}, {
      "key": "29C9",
      "mappings": {"default": {"default": "dos cuadrados unidos"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29CA",
      "mappings": {"default": {"default": "triángulo con punto arriba"}}
    }, {
      "category": "Sm",
      "key": "29CB",
      "mappings": {"default": {"default": "triángulo con barra inferior"}}
    }, {"category": "Sm", "key": "29CC", "mappings": {"default": {"default": "s en triangulo"}}}, {
      "key": "29CD",
      "mappings": {"default": {"default": "triangle with serifs at bottom"}},
      "category": "Sm"
    }, {
      "key": "29CE",
      "mappings": {"default": {"default": "triángulo hacia la derecha sobre triángulo hacia la izquierda"}},
      "category": "Sm"
    }, {
      "key": "29CF",
      "mappings": {"default": {"default": "subgrupo normal o igual a"}},
      "category": "Sm"
    }, {
      "key": "29D0",
      "mappings": {"default": {"default": "contiene como subgrupo normal o es igual a"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29D1",
      "mappings": {"default": {"default": "pajarita Con Mitad Izquierda Negra"}}
    }, {
      "category": "Sm",
      "key": "29D2",
      "mappings": {"default": {"default": "pajarita Con Mitad Derecha Negra"}}
    }, {"category": "Sm", "key": "29D3", "mappings": {"default": {"default": "pajarita Negra"}}}, {
      "category": "Sm",
      "key": "29D4",
      "mappings": {"default": {"default": "tiempos con Mitad Izquierda Negra"}}
    }, {
      "category": "Sm",
      "key": "29D5",
      "mappings": {"default": {"default": "tiempos con la mitad derecha negra"}}
    }, {
      "category": "Sm",
      "key": "29D6",
      "mappings": {"default": {"default": "reloj de arena blanco"}}
    }, {"category": "Sm", "key": "29D7", "mappings": {"default": {"default": "reloj de arena negro"}}}, {
      "key": "29DC",
      "mappings": {"default": {"default": "infinito incompleto"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29DD",
      "mappings": {"default": {"default": "empate sobre el infinito"}}
    }, {"key": "29DE", "mappings": {"default": {"default": "no infinito"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "29DF",
      "mappings": {"default": {"default": "multimapa de doble extremo"}}
    }, {
      "category": "Sm",
      "key": "29E0",
      "mappings": {"default": {"default": "cuadrado con contorno contorneado"}}
    }, {"category": "Sm", "key": "29E1", "mappings": {"default": {"default": "aumenta como"}}}, {
      "category": "Sm",
      "key": "29E2",
      "mappings": {"default": {"default": "shuffle Product"}}
    }, {"key": "29E3", "mappings": {"default": {"default": "igual tachado doble "}}, "category": "Sm"}, {
      "key": "29E4",
      "mappings": {"default": {"default": "igual con tilde, con doble tachado"}},
      "category": "Sm"
    }, {
      "key": "29E5",
      "mappings": {"default": {"default": "idéntico a con doble tachado"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "29E6", "mappings": {"default": {"default": "gleich Stark"}}}, {
      "category": "Sm",
      "key": "29E7",
      "mappings": {"default": {"default": "termodinamica"}}
    }, {
      "category": "Sm",
      "key": "29E8",
      "mappings": {"default": {"default": "triángulo que apunta hacia abajo con la mitad izquierda negra"}}
    }, {
      "category": "Sm",
      "key": "29E9",
      "mappings": {"default": {"default": "triángulo que apunta hacia abajo con la mitad derecha negra"}}
    }, {"key": "29EB", "mappings": {"default": {"default": "rombo relleno"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "29EE",
      "mappings": {"default": {"default": "plaza blanca con error"}}
    }, {
      "category": "Sm",
      "key": "29EF",
      "mappings": {"default": {"default": "error en la Plaza Negra"}}
    }, {
      "category": "Sm",
      "key": "29F0",
      "mappings": {"default": {"default": "error White Barred Diamond"}}
    }, {
      "category": "Sm",
      "key": "29F1",
      "mappings": {"default": {"default": "error Black Barred Diamond"}}
    }, {
      "category": "Sm",
      "key": "29F2",
      "mappings": {"default": {"default": "círculo blanco con error"}}
    }, {
      "category": "Sm",
      "key": "29F3",
      "mappings": {"default": {"default": "círculo negro con barras de error"}}
    }, {"key": "29F4", "mappings": {"default": {"default": "rule delayed"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "29F5",
      "mappings": {"default": {"default": "operador de Solidus Inverso"}}
    }, {
      "key": "29F6",
      "mappings": {"default": {"default": "solidus with overbar"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29F7",
      "mappings": {"default": {"default": "solidus inverso con trazo horizontal"}}
    }, {"category": "Sm", "key": "29F8", "mappings": {"default": {"default": "sólido grande"}}}, {
      "category": "Sm",
      "key": "29F9",
      "mappings": {"default": {"default": "sólido inverso grande"}}
    }, {"category": "Sm", "key": "29FA", "mappings": {"default": {"default": "doble Plus"}}}, {
      "category": "Sm",
      "key": "29FB",
      "mappings": {"default": {"default": "triple Plus"}}
    }, {"category": "Sm", "key": "29FE", "mappings": {"default": {"default": "minúsculo"}}}, {
      "category": "Sm",
      "key": "29FF",
      "mappings": {"default": {"default": "miny"}}
    }, {
      "category": "Sm",
      "key": "2A00",
      "mappings": {"default": {"default": "operador de puntos circulares N-Ary"}}
    }, {
      "category": "Sm",
      "key": "2A01",
      "mappings": {"default": {"default": "operador N-Ary Circled Plus"}}
    }, {
      "category": "Sm",
      "key": "2A02",
      "mappings": {"default": {"default": "operador de los tiempos en círculo de N-Ary"}}
    }, {
      "category": "Sm",
      "key": "2A03",
      "mappings": {"default": {"default": "operador de la unión N-Ary con punto"}}
    }, {
      "category": "Sm",
      "key": "2A04",
      "mappings": {"default": {"default": "n-Ary Union Operator con Plus"}}
    }, {
      "category": "Sm",
      "key": "2A05",
      "mappings": {"default": {"default": "operador de intersecciones cuadradas N-Ary"}}
    }, {
      "category": "Sm",
      "key": "2A06",
      "mappings": {"default": {"default": "n-Ary Square Union Operator"}}
    }, {
      "category": "Sm",
      "key": "2A07",
      "mappings": {"default": {"default": "dos lógico y operador"}}
    }, {
      "category": "Sm",
      "key": "2A08",
      "mappings": {"default": {"default": "dos lógico u operador"}}
    }, {
      "category": "Sm",
      "key": "2A09",
      "mappings": {"default": {"default": "operador de N-Ary Times"}}
    }, {"category": "Sm", "key": "2A0A", "mappings": {"default": {"default": "modulo dos sumas"}}}, {
      "category": "Sm",
      "key": "2A0B",
      "mappings": {"default": {"default": "suma con Integral"}}
    }, {
      "key": "2A0C",
      "mappings": {"default": {"default": "operador integral cuádruple"}},
      "category": "Sm"
    }, {
      "key": "2A0D",
      "mappings": {"default": {"default": "finite part integral"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A0E",
      "mappings": {"default": {"default": "integral con Doble Trazo"}}
    }, {
      "category": "Sm",
      "key": "2A0F",
      "mappings": {"default": {"default": "promedio Integral con Slash"}}
    }, {"key": "2A10", "mappings": {"default": {"default": "circulation function"}}, "category": "Sm"}, {
      "key": "2A11",
      "mappings": {"default": {"default": "integral en sentido antihorario"}},
      "category": "Sm"
    }, {
      "key": "2A12",
      "mappings": {"default": {"default": "integral lineal para rectángulo en polo"}},
      "category": "Sm"
    }, {
      "key": "2A13",
      "mappings": {"default": {"default": "integral lineal para semicírculo en polo"}},
      "category": "Sm"
    }, {
      "key": "2A14",
      "mappings": {"default": {"default": "integral lineal excluyendo el polo"}},
      "category": "Sm"
    }, {
      "key": "2A15",
      "mappings": {"default": {"default": "integral en torno de un punto operador"}},
      "category": "Sm"
    }, {
      "key": "2A16",
      "mappings": {"default": {"default": "operador integral para cuaterniones"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A18",
      "mappings": {"default": {"default": "integral con Times Sign"}}
    }, {
      "category": "Sm",
      "key": "2A19",
      "mappings": {"default": {"default": "integral con Intersección"}}
    }, {"category": "Sm", "key": "2A1A", "mappings": {"default": {"default": "integral con union"}}}, {
      "category": "Sm",
      "key": "2A1B",
      "mappings": {"default": {"default": "integral con Overbar"}}
    }, {
      "category": "Sm",
      "key": "2A1C",
      "mappings": {"default": {"default": "integral con barra inferior"}}
    }, {"category": "Sm", "key": "2A1D", "mappings": {"default": {"default": "unirse"}}}, {
      "category": "Sm",
      "key": "2A1E",
      "mappings": {"default": {"default": "operador de triángulo izquierdo grande"}}
    }, {
      "category": "Sm",
      "key": "2A1F",
      "mappings": {"default": {"default": "composición de esquemas de notación Z"}}
    }, {
      "category": "Sm",
      "key": "2A20",
      "mappings": {"default": {"default": "esquema de notación Z"}}
    }, {
      "category": "Sm",
      "key": "2A21",
      "mappings": {"default": {"default": "proyección del esquema de notación Z"}}
    }, {"key": "2A22", "mappings": {"default": {"default": "más con círculo"}}, "category": "Sm"}, {
      "key": "2A23",
      "mappings": {"default": {"default": "más con ángulo"}},
      "category": "Sm"
    }, {
      "key": "2A24",
      "mappings": {"default": {"default": "tilde con más suscrito"}},
      "category": "Sm"
    }, {
      "key": "2A25",
      "mappings": {"default": {"default": "más con punto inferior"}},
      "category": "Sm"
    }, {"key": "2A26", "mappings": {"default": {"default": "tilde con más"}}, "category": "Sm"}, {
      "key": "2A27",
      "mappings": {"default": {"default": "más con 2 suscrito"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A28",
      "mappings": {"default": {"default": "signo más con triángulo negro"}}
    }, {"key": "2A29", "mappings": {"default": {"default": "menos con coma"}}, "category": "Sm"}, {
      "key": "2A2A",
      "mappings": {"default": {"default": "menos con punto inferior"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A2B",
      "mappings": {"default": {"default": "signo menos con puntos que caen"}}
    }, {
      "category": "Sm",
      "key": "2A2C",
      "mappings": {"default": {"default": "signo menos con puntos crecientes"}}
    }, {
      "key": "2A2D",
      "mappings": {"default": {"default": "más en semicírculo izquierdo"}},
      "category": "Sm"
    }, {
      "key": "2A2E",
      "mappings": {"default": {"default": "más en semicírculo derecho"}},
      "category": "Sm"
    }, {"key": "2A2F", "mappings": {"default": {"default": "producto vectorial"}}, "category": "Sm"}, {
      "key": "2A30",
      "mappings": {"default": {"default": "por con punto"}},
      "category": "Sm"
    }, {"key": "2A31", "mappings": {"default": {"default": "por con subrayado"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "2A32",
      "mappings": {"default": {"default": "producto semidirecto con fondo cerrado"}}
    }, {"key": "2A33", "mappings": {"default": {"default": "smash product"}}, "category": "Sm"}, {
      "key": "2A34",
      "mappings": {"default": {"default": "por en semicírculo izquierdo"}},
      "category": "Sm"
    }, {
      "key": "2A35",
      "mappings": {"default": {"default": "por en semicírculo derecho"}},
      "category": "Sm"
    }, {
      "key": "2A36",
      "mappings": {"default": {"default": "por en círculo con ángulo"}},
      "category": "Sm"
    }, {"key": "2A37", "mappings": {"default": {"default": "por en anillo"}}, "category": "Sm"}, {
      "key": "2A38",
      "mappings": {"default": {"default": "división en círculo"}},
      "category": "Sm"
    }, {"key": "2A39", "mappings": {"default": {"default": "más en triángulo"}}, "category": "Sm"}, {
      "key": "2A3A",
      "mappings": {"default": {"default": "menos en triángulo"}},
      "category": "Sm"
    }, {"key": "2A3B", "mappings": {"default": {"default": "por en triángulo"}}, "category": "Sm"}, {
      "key": "2A3C",
      "mappings": {"default": {"default": "producto interior"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A3D",
      "mappings": {"default": {"default": "producto interior derecho"}}
    }, {
      "category": "Sm",
      "key": "2A3E",
      "mappings": {"default": {"default": "notación Z Composición Relacional"}}
    }, {"key": "2A3F", "mappings": {"default": {"default": "coproducto"}}, "category": "Sm"}, {
      "key": "2A40",
      "mappings": {"default": {"default": "intersección con punto"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A41",
      "mappings": {"default": {"default": "unión con signo de menos"}}
    }, {"key": "2A42", "mappings": {"default": {"default": "unión con barra"}}, "category": "Sm"}, {
      "key": "2A43",
      "mappings": {"default": {"default": "intersección con barra"}},
      "category": "Sm"
    }, {"key": "2A44", "mappings": {"default": {"default": "intersección con and"}}, "category": "Sm"}, {
      "key": "2A45",
      "mappings": {"default": {"default": "unión con or"}},
      "category": "Sm"
    }, {
      "key": "2A46",
      "mappings": {"default": {"default": "unión sobre intersección"}},
      "category": "Sm"
    }, {
      "key": "2A47",
      "mappings": {"default": {"default": "intersección sobre unión"}},
      "category": "Sm"
    }, {
      "key": "2A48",
      "mappings": {"default": {"default": "unión sobre barra, sobre intersección"}},
      "category": "Sm"
    }, {
      "key": "2A49",
      "mappings": {"default": {"default": "intersección sobre barra, sobre unión "}},
      "category": "Sm"
    }, {"key": "2A4A", "mappings": {"default": {"default": "unión y unión"}}, "category": "Sm"}, {
      "key": "2A4B",
      "mappings": {"default": {"default": "intersección e intersección"}},
      "category": "Sm"
    }, {"key": "2A4C", "mappings": {"default": {"default": "unión cerrada"}}, "category": "Sm"}, {
      "key": "2A4D",
      "mappings": {"default": {"default": "intersección cerrada"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A4E",
      "mappings": {"default": {"default": "intersección cuadrada doble"}}
    }, {"category": "Sm", "key": "2A4F", "mappings": {"default": {"default": "unión cuadrada doble"}}}, {
      "key": "2A50",
      "mappings": {"default": {"default": "unión cerrada con por"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A51",
      "mappings": {"default": {"default": "lógico y con punto arriba."}}
    }, {
      "category": "Sm",
      "key": "2A52",
      "mappings": {"default": {"default": "lógico o con punto arriba"}}
    }, {"key": "2A53", "mappings": {"default": {"default": "y"}}, "category": "Sm"}, {
      "key": "2A54",
      "mappings": {"default": {"default": "o"}},
      "category": "Sm"
    }, {
      "key": "2A55",
      "mappings": {"default": {"default": "two intersecting logical and"}},
      "category": "Sm"
    }, {
      "key": "2A56",
      "mappings": {"default": {"default": "two intersecting logical or"}},
      "category": "Sm"
    }, {"key": "2A57", "mappings": {"default": {"default": "sloping large or"}}, "category": "Sm"}, {
      "key": "2A58",
      "mappings": {"default": {"default": "sloping large and"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A59",
      "mappings": {"default": {"default": "lógica o superpuesta, lógica y"}}
    }, {
      "key": "2A5A",
      "mappings": {"default": {"default": "logical and with middle stem"}},
      "category": "Sm"
    }, {
      "key": "2A5B",
      "mappings": {"default": {"default": "logical or with middle stem"}},
      "category": "Sm"
    }, {
      "key": "2A5C",
      "mappings": {"default": {"default": "logical and with horizontal dash"}},
      "category": "Sm"
    }, {
      "key": "2A5D",
      "mappings": {"default": {"default": "logical or with horizontal dash"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A5E",
      "mappings": {"default": {"default": "lógica y con doble Overbar"}}
    }, {
      "key": "2A5F",
      "mappings": {"default": {"default": "logical and with underbar"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A60",
      "mappings": {"default": {"default": "lógica y con doble barra inferior."}}
    }, {
      "category": "Sm",
      "key": "2A61",
      "mappings": {"default": {"default": "pequeño Vee con Underbar"}}
    }, {
      "category": "Sm",
      "key": "2A62",
      "mappings": {"default": {"default": "lógica o con doble barra."}}
    }, {
      "category": "Sm",
      "key": "2A63",
      "mappings": {"default": {"default": "lógica o con doble barra inferior"}}
    }, {
      "category": "Sm",
      "key": "2A64",
      "mappings": {"default": {"default": "notación Z Dominio Antirestricción"}}
    }, {
      "category": "Sm",
      "key": "2A65",
      "mappings": {"default": {"default": "rango de notación Z Antirestricción"}}
    }, {
      "key": "2A66",
      "mappings": {"default": {"default": "igual con punto suscrito"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A67",
      "mappings": {"default": {"default": "idéntico con punto arriba"}}
    }, {
      "category": "Sm",
      "key": "2A68",
      "mappings": {"default": {"default": "triple barra horizontal con doble movimiento vertical"}}
    }, {
      "category": "Sm",
      "key": "2A69",
      "mappings": {"default": {"default": "triple barra horizontal con triple movimiento vertical"}}
    }, {"key": "2A6A", "mappings": {"default": {"default": "tilde con punto"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "2A6B",
      "mappings": {"default": {"default": "operador de tilde con puntos crecientes"}}
    }, {
      "category": "Sm",
      "key": "2A6C",
      "mappings": {"default": {"default": "similares menos similares"}}
    }, {
      "key": "2A6D",
      "mappings": {"default": {"default": "congruente con punto"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "2A6E", "mappings": {"default": {"default": "igual a Asterisco"}}}, {
      "key": "2A6F",
      "mappings": {"default": {"default": "casi igual con ángulo"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A70",
      "mappings": {"default": {"default": "aproximadamente igual o igual a"}}
    }, {
      "key": "2A71",
      "mappings": {"default": {"default": "igual con más suscrito"}},
      "category": "Sm"
    }, {"key": "2A72", "mappings": {"default": {"default": "igual con más"}}, "category": "Sm"}, {
      "key": "2A73",
      "mappings": {"default": {"default": "igual con operador tilde"}},
      "category": "Sm"
    }, {"key": "2A74", "mappings": {"default": {"default": "igual con dos puntos"}}, "category": "Sm"}, {
      "key": "2A75",
      "mappings": {"default": {"default": "igual doble"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A76",
      "mappings": {"default": {"default": "tres signos de igualdad consecutivos"}}
    }, {
      "key": "2A77",
      "mappings": {"default": {"default": "igual entre cuatro puntos"}},
      "category": "Sm"
    }, {
      "key": "2A78",
      "mappings": {"default": {"default": "equivalente con cuatro puntos"}},
      "category": "Sm"
    }, {
      "key": "2A79",
      "mappings": {"default": {"default": "menor que con círculo interior"}},
      "category": "Sm"
    }, {
      "key": "2A7A",
      "mappings": {"default": {"default": "mayor que con círculo interior"}},
      "category": "Sm"
    }, {
      "key": "2A7B",
      "mappings": {"default": {"default": "menor que con interrogante"}},
      "category": "Sm"
    }, {
      "key": "2A7C",
      "mappings": {"default": {"default": "mayor que con interrogante"}},
      "category": "Sm"
    }, {
      "key": "2A7D",
      "mappings": {"default": {"default": "estrictamente menor que"}},
      "category": "Sm"
    }, {
      "key": "2A7E",
      "mappings": {"default": {"default": "estrictamente mayor que"}},
      "category": "Sm"
    }, {
      "key": "2A7F",
      "mappings": {"default": {"default": "estrictamente menor que con punto interior"}},
      "category": "Sm"
    }, {
      "key": "2A80",
      "mappings": {"default": {"default": "estrictamente mayor que con punto interior"}},
      "category": "Sm"
    }, {
      "key": "2A81",
      "mappings": {"default": {"default": "estrictamente menor que con punto"}},
      "category": "Sm"
    }, {
      "key": "2A82",
      "mappings": {"default": {"default": "estrictamente mayor que con punto"}},
      "category": "Sm"
    }, {
      "key": "2A83",
      "mappings": {"default": {"default": "estrictamente menor que con punto"}},
      "category": "Sm"
    }, {
      "key": "2A84",
      "mappings": {"default": {"default": "estrictamente mayor que con punto"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A85",
      "mappings": {"default": {"default": "menor que o aproximado"}}
    }, {
      "category": "Sm",
      "key": "2A86",
      "mappings": {"default": {"default": "mayor que o aproximado"}}
    }, {
      "category": "Sm",
      "key": "2A87",
      "mappings": {"default": {"default": "menor que una línea no es igual a"}}
    }, {
      "category": "Sm",
      "key": "2A88",
      "mappings": {"default": {"default": "mayores y una sola línea no son iguales a"}}
    }, {
      "key": "2A89",
      "mappings": {"default": {"default": "menor que, pero no aproximado a"}},
      "category": "Sm"
    }, {
      "key": "2A8A",
      "mappings": {"default": {"default": "mayor que, pero no aproximado a"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A8B",
      "mappings": {"default": {"default": "menos que arriba de la doble línea igual por encima de mayor que"}}
    }, {
      "category": "Sm",
      "key": "2A8C",
      "mappings": {"default": {"default": "más que arriba de la línea doble igual sobre menos que"}}
    }, {
      "key": "2A8D",
      "mappings": {"default": {"default": "menor, similar o igual a"}},
      "category": "Sm"
    }, {
      "key": "2A8E",
      "mappings": {"default": {"default": "mayor, similar o igual a"}},
      "category": "Sm"
    }, {
      "key": "2A8F",
      "mappings": {"default": {"default": "menor, similar o mayor que"}},
      "category": "Sm"
    }, {
      "key": "2A90",
      "mappings": {"default": {"default": "mayor, similar o menor que"}},
      "category": "Sm"
    }, {
      "key": "2A91",
      "mappings": {"default": {"default": "menor, mayor o igual a"}},
      "category": "Sm"
    }, {
      "key": "2A92",
      "mappings": {"default": {"default": "mayor, menor o igual a"}},
      "category": "Sm"
    }, {
      "key": "2A93",
      "mappings": {"default": {"default": "estrictamente menor o mayor que"}},
      "category": "Sm"
    }, {
      "key": "2A94",
      "mappings": {"default": {"default": "estrictamente mayor o menor que"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A95",
      "mappings": {"default": {"default": "inclinado igual o menor que"}}
    }, {
      "category": "Sm",
      "key": "2A96",
      "mappings": {"default": {"default": "inclinado igual o mayor que"}}
    }, {
      "key": "2A97",
      "mappings": {"default": {"default": "estrictamente menor que con punto interior"}},
      "category": "Sm"
    }, {
      "key": "2A98",
      "mappings": {"default": {"default": "estrictamente mayor que con punto interior"}},
      "category": "Sm"
    }, {"key": "2A99", "mappings": {"default": {"default": "menor o igual a"}}, "category": "Sm"}, {
      "key": "2A9A",
      "mappings": {"default": {"default": "mayor o igual a"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A9B",
      "mappings": {"default": {"default": "doble línea inclinada igual o menor que"}}
    }, {
      "category": "Sm",
      "key": "2A9C",
      "mappings": {"default": {"default": "doble línea inclinada igual o mayor que"}}
    }, {
      "key": "2A9D",
      "mappings": {"default": {"default": "equivalente o menor que"}},
      "category": "Sm"
    }, {"key": "2A9E", "mappings": {"default": {"default": "mayor o similar a"}}, "category": "Sm"}, {
      "key": "2A9F",
      "mappings": {"default": {"default": "similar, menor o igual a"}},
      "category": "Sm"
    }, {
      "key": "2AA0",
      "mappings": {"default": {"default": "similar, mayor o igual a"}},
      "category": "Sm"
    }, {"key": "2AA1", "mappings": {"default": {"default": "mucho menor que"}}, "category": "Sm"}, {
      "key": "2AA2",
      "mappings": {"default": {"default": "mucho mayor que"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AA3",
      "mappings": {"default": {"default": "doble anidada menor que con barra inferior"}}
    }, {"key": "2AA4", "mappings": {"default": {"default": "menor que, mayor que"}}, "category": "Sm"}, {
      "key": "2AA5",
      "mappings": {"default": {"default": "mayor o menor que"}},
      "category": "Sm"
    }, {
      "key": "2AA6",
      "mappings": {"default": {"default": "menor que cerrado con curva"}},
      "category": "Sm"
    }, {
      "key": "2AA7",
      "mappings": {"default": {"default": "mayor que cerrado con curva"}},
      "category": "Sm"
    }, {
      "key": "2AA8",
      "mappings": {"default": {"default": "estrictamente menor que cerrado con curva"}},
      "category": "Sm"
    }, {
      "key": "2AA9",
      "mappings": {"default": {"default": "estrictamente mayor que cerrado con curva"}},
      "category": "Sm"
    }, {"key": "2AAA", "mappings": {"default": {"default": "más reducido que"}}, "category": "Sm"}, {
      "key": "2AAB",
      "mappings": {"default": {"default": "más amplio que"}},
      "category": "Sm"
    }, {
      "key": "2AAC",
      "mappings": {"default": {"default": "más reducido o igual a"}},
      "category": "Sm"
    }, {"key": "2AAD", "mappings": {"default": {"default": "más amplio o igual a"}}, "category": "Sm"}, {
      "key": "2AAE",
      "mappings": {"default": {"default": "diferencia entre"}},
      "category": "Sm"
    }, {
      "key": "2AAF",
      "mappings": {"default": {"default": "precede o es igual a"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AB0",
      "mappings": {"default": {"default": "sucede por encima de la línea de igual signo"}}
    }, {
      "category": "Sm",
      "key": "2AB1",
      "mappings": {"default": {"default": "precede por encima de una línea no es igual a"}}
    }, {
      "category": "Sm",
      "key": "2AB2",
      "mappings": {"default": {"default": "tiene éxito por encima de una línea no es igual a"}}
    }, {
      "category": "Sm",
      "key": "2AB3",
      "mappings": {"default": {"default": "precede arriba del signo igual"}}
    }, {
      "category": "Sm",
      "key": "2AB4",
      "mappings": {"default": {"default": "sucede por encima del signo igual"}}
    }, {
      "key": "2AB5",
      "mappings": {"default": {"default": "precede estrictamente a"}},
      "category": "Sm"
    }, {
      "key": "2AB6",
      "mappings": {"default": {"default": "sigue estrictamente a"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AB7",
      "mappings": {"default": {"default": "precede por encima de casi igual a"}}
    }, {
      "category": "Sm",
      "key": "2AB8",
      "mappings": {"default": {"default": "tiene éxito por encima de casi igual a"}}
    }, {
      "category": "Sm",
      "key": "2AB9",
      "mappings": {"default": {"default": "precede arriba no es igual a"}}
    }, {
      "category": "Sm",
      "key": "2ABA",
      "mappings": {"default": {"default": "tiene éxito por encima de no casi igual a"}}
    }, {"key": "2ABB", "mappings": {"default": {"default": "precede mucho antes"}}, "category": "Sm"}, {
      "key": "2ABC",
      "mappings": {"default": {"default": "sigue mucho después"}},
      "category": "Sm"
    }, {"key": "2ABD", "mappings": {"default": {"default": "subrelación de"}}, "category": "Sm"}, {
      "key": "2ABE",
      "mappings": {"default": {"default": "contiene como subrelación a"}},
      "category": "Sm"
    }, {
      "key": "2ABF",
      "mappings": {"default": {"default": "incluido con más suscrito"}},
      "category": "Sm"
    }, {
      "key": "2AC0",
      "mappings": {"default": {"default": "contiene con más suscrito"}},
      "category": "Sm"
    }, {
      "key": "2AC1",
      "mappings": {"default": {"default": "incluido con por suscrito"}},
      "category": "Sm"
    }, {
      "key": "2AC2",
      "mappings": {"default": {"default": "contiene con por suscrito"}},
      "category": "Sm"
    }, {
      "key": "2AC3",
      "mappings": {"default": {"default": "incluido con punto o es igual a"}},
      "category": "Sm"
    }, {
      "key": "2AC4",
      "mappings": {"default": {"default": "contiene con punto o es igual a"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AC5",
      "mappings": {"default": {"default": "subconjunto de signo igual arriba"}}
    }, {
      "category": "Sm",
      "key": "2AC6",
      "mappings": {"default": {"default": "superconjunto de signo igual arriba"}}
    }, {
      "key": "2AC7",
      "mappings": {"default": {"default": "incluido aproximadamente en"}},
      "category": "Sm"
    }, {
      "key": "2AC8",
      "mappings": {"default": {"default": "contiene aproximadamente a"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AC9",
      "mappings": {"default": {"default": "subconjunto de Arriba Casi Igual a"}}
    }, {
      "category": "Sm",
      "key": "2ACA",
      "mappings": {"default": {"default": "superconjunto de arriba casi igual a"}}
    }, {
      "category": "Sm",
      "key": "2ACB",
      "mappings": {"default": {"default": "subconjunto de lo anterior no es igual a"}}
    }, {
      "category": "Sm",
      "key": "2ACC",
      "mappings": {"default": {"default": "superconjunto de arriba no es igual a"}}
    }, {
      "category": "Sm",
      "key": "2ACD",
      "mappings": {"default": {"default": "operador de caja abierta a la izquierda cuadrada"}}
    }, {
      "category": "Sm",
      "key": "2ACE",
      "mappings": {"default": {"default": "operador de caja abierta a la derecha cuadrada"}}
    }, {
      "key": "2ACF",
      "mappings": {"default": {"default": "incluido con cierre en"}},
      "category": "Sm"
    }, {
      "key": "2AD0",
      "mappings": {"default": {"default": "contiene al cierre del subconjunto"}},
      "category": "Sm"
    }, {
      "key": "2AD1",
      "mappings": {"default": {"default": "incluido el cierre o es igual a"}},
      "category": "Sm"
    }, {
      "key": "2AD2",
      "mappings": {"default": {"default": "contiene al cierre del subconjunto o es igual a"}},
      "category": "Sm"
    }, {
      "key": "2AD3",
      "mappings": {"default": {"default": "está incluido o contiene a"}},
      "category": "Sm"
    }, {
      "key": "2AD4",
      "mappings": {"default": {"default": "contiene o está incluido en"}},
      "category": "Sm"
    }, {
      "key": "2AD5",
      "mappings": {"default": {"default": "incluido doblemente en"}},
      "category": "Sm"
    }, {"key": "2AD6", "mappings": {"default": {"default": "contiene doblemente a"}}, "category": "Sm"}, {
      "key": "2AD7",
      "mappings": {"default": {"default": "contiene o está incluido en"}},
      "category": "Sm"
    }, {
      "key": "2AD8",
      "mappings": {"default": {"default": "contiene o está incluido en"}},
      "category": "Sm"
    }, {
      "key": "2AD9",
      "mappings": {"default": {"default": "perteneciente dirigido hacia abajo"}},
      "category": "Sm"
    }, {
      "key": "2ADA",
      "mappings": {"default": {"default": "pitchfork with tee top"}},
      "category": "Sm"
    }, {
      "key": "2ADB",
      "mappings": {"default": {"default": "transversal intersection"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "2ADC", "mappings": {"default": {"default": "bifurcación"}}}, {
      "category": "Sm",
      "key": "2ADD",
      "mappings": {"default": {"default": "nonforking"}}
    }, {
      "category": "Sm",
      "key": "2ADE",
      "mappings": {"default": {"default": "tachuela Izquierda Corta"}}
    }, {
      "category": "Sm",
      "key": "2ADF",
      "mappings": {"default": {"default": "tachuela corta hacia abajo"}}
    }, {"category": "Sm", "key": "2AE0", "mappings": {"default": {"default": "short Up Tack"}}}, {
      "category": "Sm",
      "key": "2AE1",
      "mappings": {"default": {"default": "perpendicular con s"}}
    }, {
      "category": "Sm",
      "key": "2AE2",
      "mappings": {"default": {"default": "barra vertical Torniquete a la derecha"}}
    }, {
      "category": "Sm",
      "key": "2AE3",
      "mappings": {"default": {"default": "barra vertical doble Torniquete izquierdo"}}
    }, {
      "key": "2AE4",
      "mappings": {"default": {"default": "double left turnstile vertical bar"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AE5",
      "mappings": {"default": {"default": "doble barra vertical doble torniquete izquierdo"}}
    }, {
      "key": "2AE6",
      "mappings": {"default": {"default": "long dash from left member of double vertical"}},
      "category": "Sm"
    }, {
      "key": "2AE7",
      "mappings": {"default": {"default": "short down tack with overbar"}},
      "category": "Sm"
    }, {
      "key": "2AE8",
      "mappings": {"default": {"default": "barra con perpendicular"}},
      "category": "Sm"
    }, {
      "key": "2AE9",
      "mappings": {"default": {"default": "short up tack above short down tack"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "2AEA", "mappings": {"default": {"default": "tachuela doble"}}}, {
      "key": "2AEB",
      "mappings": {"default": {"default": "double up tack"}},
      "category": "Sm"
    }, {
      "key": "2AEC",
      "mappings": {"default": {"default": "double stroke not sign"}},
      "category": "Sm"
    }, {
      "key": "2AED",
      "mappings": {"default": {"default": "reversed double stroke not sign"}},
      "category": "Sm"
    }, {
      "key": "2AEE",
      "mappings": {"default": {"default": "does not divide with reversed negation slash"}},
      "category": "Sm"
    }, {
      "key": "2AEF",
      "mappings": {"default": {"default": "barra vertical con círculo"}},
      "category": "Sm"
    }, {
      "key": "2AF0",
      "mappings": {"default": {"default": "barra con círculo inferior"}},
      "category": "Sm"
    }, {
      "key": "2AF1",
      "mappings": {"default": {"default": "down tack with circle below"}},
      "category": "Sm"
    }, {"key": "2AF2", "mappings": {"default": {"default": "paralela con marca"}}, "category": "Sm"}, {
      "key": "2AF3",
      "mappings": {"default": {"default": "paralela con tilde"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AF4",
      "mappings": {"default": {"default": "relación binaria de barra vertical triple"}}
    }, {
      "category": "Sm",
      "key": "2AF5",
      "mappings": {"default": {"default": "barra vertical triple con trazo horizontal"}}
    }, {
      "category": "Sm",
      "key": "2AF6",
      "mappings": {"default": {"default": "operador Triple Colon"}}
    }, {
      "category": "Sm",
      "key": "2AF7",
      "mappings": {"default": {"default": "triple anidada menos que"}}
    }, {
      "category": "Sm",
      "key": "2AF8",
      "mappings": {"default": {"default": "triple anidado mayor que"}}
    }, {
      "category": "Sm",
      "key": "2AF9",
      "mappings": {"default": {"default": "doble línea inclinada menor o igual a"}}
    }, {
      "category": "Sm",
      "key": "2AFA",
      "mappings": {"default": {"default": "doble línea inclinada mayor o igual a"}}
    }, {
      "category": "Sm",
      "key": "2AFB",
      "mappings": {"default": {"default": "relación Binaria Triple Sólida"}}
    }, {
      "category": "Sm",
      "key": "2AFC",
      "mappings": {"default": {"default": "operador de barra vertical triple grande"}}
    }, {
      "category": "Sm",
      "key": "2AFD",
      "mappings": {"default": {"default": "operador Doble Sólido"}}
    }, {
      "category": "Sm",
      "key": "2AFE",
      "mappings": {"default": {"default": "barra vertical blanca"}}
    }, {
      "category": "Sm",
      "key": "2AFF",
      "mappings": {"default": {"default": "barra vertical blanca N-Ary"}}
    }, {"category": "Pd", "key": "301C", "mappings": {"default": {"default": "wave Dash"}}}, {
      "category": "Po",
      "key": "FE10",
      "mappings": {"default": {"default": "formulario de presentación para coma vertical"}}
    }, {
      "category": "Po",
      "key": "FE13",
      "mappings": {"default": {"default": "formulario de presentación para colon vertical"}}
    }, {
      "category": "Po",
      "key": "FE14",
      "mappings": {"default": {"default": "formulario de presentación para punto y coma vertical"}}
    }, {
      "category": "Po",
      "key": "FE15",
      "mappings": {"default": {"default": "formulario de presentación para el signo de exclamación vertical"}}
    }, {
      "category": "Po",
      "key": "FE16",
      "mappings": {"default": {"default": "formulario de presentación para el signo de interrogación vertical"}}
    }, {
      "category": "Po",
      "key": "FE19",
      "mappings": {"default": {"default": "formulario de presentación para elipsis horizontal vertical"}}
    }, {
      "category": "Po",
      "key": "FE30",
      "mappings": {"default": {"default": "formulario de presentación para líder vertical de dos puntos"}}
    }, {
      "category": "Pd",
      "key": "FE31",
      "mappings": {"default": {"default": "formulario de presentación para Vertical Em Dash"}}
    }, {
      "category": "Pd",
      "key": "FE32",
      "mappings": {"default": {"default": "formulario de presentación para Vertical En Dash"}}
    }, {
      "category": "Pc",
      "key": "FE33",
      "mappings": {"default": {"default": "formulario de presentación para línea vertical baja"}}
    }, {
      "category": "Pc",
      "key": "FE34",
      "mappings": {"default": {"default": "formulario de presentación para línea baja ondulada vertical"}}
    }, {"category": "Po", "key": "FE45", "mappings": {"default": {"default": "punto de sésamo"}}}, {
      "category": "Po",
      "key": "FE46",
      "mappings": {"default": {"default": "punto de sésamo blanco"}}
    }, {"category": "Po", "key": "FE49", "mappings": {"default": {"default": "línea discontinua"}}}, {
      "category": "Po",
      "key": "FE4A",
      "mappings": {"default": {"default": "centinela Overline"}}
    }, {"category": "Po", "key": "FE4B", "mappings": {"default": {"default": "ondulado Overline"}}}, {
      "category": "Po",
      "key": "FE4C",
      "mappings": {"default": {"default": "doble Ondulado Ondulado"}}
    }, {
      "category": "Pc",
      "key": "FE4D",
      "mappings": {"default": {"default": "línea baja discontinua"}}
    }, {
      "category": "Pc",
      "key": "FE4E",
      "mappings": {"default": {"default": "línea baja de línea central"}}
    }, {
      "category": "Pc",
      "key": "FE4F",
      "mappings": {"default": {"default": "línea baja ondulada"}}
    }, {"category": "Po", "key": "FE50", "mappings": {"default": {"default": "comas pequeñas"}}}, {
      "category": "Po",
      "key": "FE52",
      "mappings": {"default": {"default": "parada completa pequeña"}}
    }, {
      "category": "Po",
      "key": "FE54",
      "mappings": {"default": {"default": "pequeño punto y coma"}}
    }, {"category": "Po", "key": "FE55", "mappings": {"default": {"default": "pequeño colon"}}}, {
      "category": "Po",
      "key": "FE56",
      "mappings": {"default": {"default": "pequeño signo de interrogación"}}
    }, {
      "category": "Po",
      "key": "FE57",
      "mappings": {"default": {"default": "pequeño signo de exclamación"}}
    }, {"category": "Pd", "key": "FE58", "mappings": {"default": {"default": "pequeño Em Dash"}}}, {
      "category": "Po",
      "key": "FE5F",
      "mappings": {"default": {"default": "signo de número pequeño"}}
    }, {"category": "Po", "key": "FE60", "mappings": {"default": {"default": "ampersand pequeño"}}}, {
      "category": "Po",
      "key": "FE61",
      "mappings": {"default": {"default": "pequeño asterisco"}}
    }, {"category": "Sm", "key": "FE62", "mappings": {"default": {"default": "signo más pequeño"}}}, {
      "category": "Pd",
      "key": "FE63",
      "mappings": {"default": {"default": "pequeño guión-menos"}}
    }, {
      "category": "Sm",
      "key": "FE64",
      "mappings": {"default": {"default": "signo menor que pequeño"}}
    }, {
      "category": "Sm",
      "key": "FE65",
      "mappings": {"default": {"default": "signo mayor que pequeño"}}
    }, {"category": "Sm", "key": "FE66", "mappings": {"default": {"default": "signo pequeño igual"}}}, {
      "key": "FE68",
      "mappings": {"default": {"default": "división entera"}},
      "category": "Po"
    }, {
      "category": "Sc",
      "key": "FE69",
      "mappings": {"default": {"default": "pequeño signo de dólar"}}
    }, {
      "category": "Po",
      "key": "FE6A",
      "mappings": {"default": {"default": "signo de porcentaje pequeño"}}
    }, {
      "category": "Po",
      "key": "FE6B",
      "mappings": {"default": {"default": "pequeño comercial en"}}
    }, {
      "category": "Po",
      "key": "FF01",
      "mappings": {"default": {"default": "signo de exclamación de ancho completo"}}
    }, {
      "category": "Po",
      "key": "FF02",
      "mappings": {"default": {"default": "comillas de ancho completo"}}
    }, {
      "category": "Po",
      "key": "FF03",
      "mappings": {"default": {"default": "número de signo de ancho completo"}}
    }, {
      "category": "Sc",
      "key": "FF04",
      "mappings": {"default": {"default": "signo de dólar de ancho completo"}}
    }, {
      "category": "Po",
      "key": "FF05",
      "mappings": {"default": {"default": "signo de porcentaje de ancho completo"}}
    }, {
      "category": "Po",
      "key": "FF06",
      "mappings": {"default": {"default": "ampersand de ancho completo"}}
    }, {
      "category": "Po",
      "key": "FF07",
      "mappings": {"default": {"default": "apóstrofe de ancho completo"}}
    }, {
      "category": "Po",
      "key": "FF0A",
      "mappings": {"default": {"default": "asterisco de ancho completo"}}
    }, {
      "category": "Sm",
      "key": "FF0B",
      "mappings": {"default": {"default": "signo de ancho completo"}}
    }, {
      "category": "Po",
      "key": "FF0C",
      "mappings": {"default": {"default": "coma de ancho completo"}}
    }, {
      "category": "Pd",
      "key": "FF0D",
      "mappings": {"default": {"default": "ancho de ancho completo"}}
    }, {
      "category": "Po",
      "key": "FF0E",
      "mappings": {"default": {"default": "fullwidth Full Stop"}}
    }, {
      "category": "Po",
      "key": "FF0F",
      "mappings": {"default": {"default": "sólido de ancho completo"}}
    }, {
      "category": "Po",
      "key": "FF1A",
      "mappings": {"default": {"default": "ancho completo de colon"}}
    }, {
      "category": "Po",
      "key": "FF1B",
      "mappings": {"default": {"default": "punto y coma de ancho completo"}}
    }, {
      "category": "Sm",
      "key": "FF1C",
      "mappings": {"default": {"default": "ancho completo menos de la señal"}}
    }, {
      "category": "Sm",
      "key": "FF1D",
      "mappings": {"default": {"default": "signo de igual ancho completo"}}
    }, {
      "category": "Sm",
      "key": "FF1E",
      "mappings": {"default": {"default": "ancho completo mayor que el signo"}}
    }, {
      "category": "Po",
      "key": "FF1F",
      "mappings": {"default": {"default": "signo de interrogación de ancho completo"}}
    }, {
      "category": "Po",
      "key": "FF20",
      "mappings": {"default": {"default": "comercial de ancho completo en"}}
    }, {
      "category": "Po",
      "key": "FF3C",
      "mappings": {"default": {"default": "sólido inverso de ancho completo"}}
    }, {
      "category": "Sk",
      "key": "FF3E",
      "mappings": {"default": {"default": "acento circunflejo de ancho completo"}}
    }, {
      "category": "Pc",
      "key": "FF3F",
      "mappings": {"default": {"default": "línea baja de ancho completo"}}
    }, {
      "category": "Sk",
      "key": "FF40",
      "mappings": {"default": {"default": "acento grave de ancho completo"}}
    }, {"key": "FF5C", "mappings": {"default": {"default": "barra vertical"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "FF5E",
      "mappings": {"default": {"default": "tilde de ancho completo"}}
    }, {
      "category": "Sc",
      "key": "FFE0",
      "mappings": {"default": {"default": "signo de centavo de ancho completo"}}
    }, {
      "category": "Sc",
      "key": "FFE1",
      "mappings": {"default": {"default": "signo de libra de ancho completo"}}
    }, {
      "category": "Sm",
      "key": "FFE2",
      "mappings": {"default": {"default": "fullwidth no firmar"}}
    }, {
      "category": "Sk",
      "key": "FFE3",
      "mappings": {"default": {"default": "ancho completo macron"}}
    }, {
      "category": "So",
      "key": "FFE4",
      "mappings": {"default": {"default": "barra rota de ancho completo"}}
    }, {
      "category": "Sc",
      "key": "FFE5",
      "mappings": {"default": {"default": "signo de yen de ancho completo"}}
    }, {
      "category": "Sc",
      "key": "FFE6",
      "mappings": {"default": {"default": "signo de ancho completo ganado"}}
    }, {
      "category": "So",
      "key": "FFE8",
      "mappings": {"default": {"default": "formas de ancho medio luz vertical"}}
    }, {
      "category": "So",
      "key": "FFED",
      "mappings": {"default": {"default": "medio ancho cuadrado negro"}}
    }, {"category": "So", "key": "FFEE", "mappings": {"default": {"default": "círculo blanco de medio ancho"}}}]
  ,
  "es/symbols/math_whitespace.js":
    [{"locale": "es"}, {
      "category": "Zs",
      "key": "0020",
      "mappings": {"default": {"default": "espacio"}}
    }, {"key": "00A0", "mappings": {"default": {"default": " "}}, "category": "Zs"}, {
      "key": "00AD",
      "mappings": {"default": {"default": "guión corto"}},
      "category": "Cf"
    }, {"category": "Zs", "key": "2000", "mappings": {"default": {"default": "en quad"}}}, {
      "category": "Zs",
      "key": "2001",
      "mappings": {"default": {"default": "em Quad"}}
    }, {"key": "2002", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
      "key": "2003",
      "mappings": {"default": {"default": ""}},
      "category": "Zs"
    }, {"key": "2004", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
      "key": "2005",
      "mappings": {"default": {"default": ""}},
      "category": "Zs"
    }, {"category": "Zs", "key": "2006", "mappings": {"default": {"default": "espacio Six-Per-Em"}}}, {
      "key": "2007",
      "mappings": {"default": {"default": ""}},
      "category": "Zs"
    }, {"key": "2008", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
      "key": "2009",
      "mappings": {"default": {"default": ""}},
      "category": "Zs"
    }, {"key": "200A", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
      "key": "200B",
      "mappings": {"default": {"default": ""}},
      "category": "Cf"
    }, {
      "key": "200C",
      "mappings": {"default": {"default": "ancho de cero no ensamblador"}},
      "category": "Cf"
    }, {
      "key": "200D",
      "mappings": {"default": {"default": "ancho de cero ensamblador"}},
      "category": "Cf"
    }, {
      "key": "200E",
      "mappings": {"default": {"default": "marca de izquierda a derecha"}},
      "category": "Cf"
    }, {
      "key": "200F",
      "mappings": {"default": {"default": "marca de derecha a izquierda"}},
      "category": "Cf"
    }, {"category": "Zl", "key": "2028", "mappings": {"default": {"default": "separador de linea"}}}, {
      "category": "Zp",
      "key": "2029",
      "mappings": {"default": {"default": "separador de párrafo"}}
    }, {
      "category": "Cf",
      "key": "202A",
      "mappings": {"default": {"default": "incrustación de izquierda a derecha"}}
    }, {
      "category": "Cf",
      "key": "202B",
      "mappings": {"default": {"default": "incrustación de derecha a izquierda"}}
    }, {
      "category": "Cf",
      "key": "202C",
      "mappings": {"default": {"default": "formato direccional pop"}}
    }, {
      "category": "Cf",
      "key": "202D",
      "mappings": {"default": {"default": "anulación de izquierda a derecha"}}
    }, {
      "category": "Cf",
      "key": "202E",
      "mappings": {"default": {"default": "anulación de derecha a izquierda"}}
    }, {
      "category": "Zs",
      "key": "202F",
      "mappings": {"default": {"default": "espacio estrecho sin descanso"}}
    }, {"key": "205F", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
      "key": "2060",
      "mappings": {"default": {"default": ""}},
      "category": "Cf"
    }, {"key": "2061", "mappings": {"default": {"default": "de"}}, "category": "Cf"}, {
      "key": "2062",
      "mappings": {"default": {"default": "por"}},
      "category": "Cf"
    }, {"key": "2063", "mappings": {"default": {"default": "coma"}}, "category": "Cf"}, {
      "key": "2064",
      "mappings": {"default": {"default": "más"}},
      "category": "Cf"
    }, {
      "category": "Cf",
      "key": "206A",
      "mappings": {"default": {"default": "inhibir el intercambio simétrico"}}
    }, {
      "category": "Cf",
      "key": "206B",
      "mappings": {"default": {"default": "activar el intercambio simétrico"}}
    }, {
      "category": "Cf",
      "key": "206E",
      "mappings": {"default": {"default": "formas de dígitos nacionales"}}
    }, {
      "category": "Cf",
      "key": "206F",
      "mappings": {"default": {"default": "formas de dígitos nominales"}}
    }, {"key": "FEFF", "mappings": {"default": {"default": ""}}, "category": "Cf"}, {
      "category": "Cf",
      "key": "FFF9",
      "mappings": {"default": {"default": "ancla de anotación interlinear"}}
    }, {
      "category": "Cf",
      "key": "FFFA",
      "mappings": {"default": {"default": "separador de anotación interlinear"}}
    }, {"category": "Cf", "key": "FFFB", "mappings": {"default": {"default": "terminador de anotación interlinear"}}}]
  ,
  "es/symbols/other_stars.js":
    [{"locale": "es"}, {
      "category": "So",
      "key": "23E8",
      "mappings": {"default": {"default": "símbolo exponente decimal"}}
    }, {"key": "2605", "mappings": {"default": {"default": "estrella rellena"}}, "category": "So"}, {
      "key": "2606",
      "mappings": {"default": {"default": "estrella vacía"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "26AA",
      "mappings": {"default": {"default": "círculo blanco medio"}}
    }, {
      "category": "So",
      "key": "26AB",
      "mappings": {"default": {"default": "círculo negro medio"}}
    }, {
      "category": "So",
      "key": "2705",
      "mappings": {"default": {"default": "marca de verificación pesada blanca"}}
    }, {"key": "2713", "mappings": {"default": {"default": "check mark"}}, "category": "So"}, {
      "category": "So",
      "key": "2714",
      "mappings": {"default": {"default": "marca de verificación pesada"}}
    }, {"category": "So", "key": "2715", "mappings": {"default": {"default": "multiplicación x"}}}, {
      "category": "So",
      "key": "2716",
      "mappings": {"default": {"default": "multiplicación pesada X"}}
    }, {"key": "2717", "mappings": {"default": {"default": "ballot x"}}, "category": "So"}, {
      "category": "So",
      "key": "2718",
      "mappings": {"default": {"default": "boleta pesada X"}}
    }, {
      "category": "So",
      "key": "271B",
      "mappings": {"default": {"default": "centro abierto de la cruz"}}
    }, {
      "category": "So",
      "key": "271C",
      "mappings": {"default": {"default": "pesado centro abierto cruz"}}
    }, {"key": "2720", "mappings": {"default": {"default": "cruz de Malta"}}, "category": "So"}, {
      "category": "So",
      "key": "2721",
      "mappings": {"default": {"default": "estrella de David"}}
    }, {
      "category": "So",
      "key": "2722",
      "mappings": {"default": {"default": "cuatro asterisco de lágrima"}}
    }, {
      "category": "So",
      "key": "2723",
      "mappings": {"default": {"default": "cuatro asterisco de globo"}}
    }, {
      "category": "So",
      "key": "2724",
      "mappings": {"default": {"default": "pesado Cuatro asterisco impulsado por globo"}}
    }, {
      "category": "So",
      "key": "2725",
      "mappings": {"default": {"default": "cuatro asterisco de club-spoked"}}
    }, {
      "category": "So",
      "key": "2726",
      "mappings": {"default": {"default": "estrella de cuatro puntas negra"}}
    }, {
      "category": "So",
      "key": "2727",
      "mappings": {"default": {"default": "estrella blanca de cuatro puntas"}}
    }, {"category": "So", "key": "2728", "mappings": {"default": {"default": "destellos"}}}, {
      "category": "So",
      "key": "2729",
      "mappings": {"default": {"default": "estrés subrayado estrella blanca"}}
    }, {
      "category": "So",
      "key": "272A",
      "mappings": {"default": {"default": "estrella blanca circulada"}}
    }, {
      "category": "So",
      "key": "272B",
      "mappings": {"default": {"default": "centro Abierto Estrella Negra"}}
    }, {
      "category": "So",
      "key": "272C",
      "mappings": {"default": {"default": "centro negro estrella blanca"}}
    }, {
      "category": "So",
      "key": "272D",
      "mappings": {"default": {"default": "contorneado negro estrella"}}
    }, {
      "category": "So",
      "key": "272E",
      "mappings": {"default": {"default": "estrella negra contorneada pesada"}}
    }, {"category": "So", "key": "272F", "mappings": {"default": {"default": "pinwheel Star"}}}, {
      "category": "So",
      "key": "2730",
      "mappings": {"default": {"default": "estrella blanca sombreada"}}
    }, {"category": "So", "key": "2731", "mappings": {"default": {"default": "asterisco pesado"}}}, {
      "category": "So",
      "key": "2732",
      "mappings": {"default": {"default": "centro abierto de asterisco"}}
    }, {
      "category": "So",
      "key": "2733",
      "mappings": {"default": {"default": "ocho asterisco hablado"}}
    }, {
      "category": "So",
      "key": "2734",
      "mappings": {"default": {"default": "ocho estrellas negras puntiagudas"}}
    }, {
      "category": "So",
      "key": "2735",
      "mappings": {"default": {"default": "ocho estrellas de molinete en punta"}}
    }, {
      "key": "2736",
      "mappings": {"default": {"default": "estrella rellena de seis puntas"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2739",
      "mappings": {"default": {"default": "doce estrellas negras puntiagudas"}}
    }, {
      "category": "So",
      "key": "273A",
      "mappings": {"default": {"default": "dieciséis asteriscos en punta"}}
    }, {
      "category": "So",
      "key": "273B",
      "mappings": {"default": {"default": "asterisco con efecto de lágrima"}}
    }, {
      "category": "So",
      "key": "273C",
      "mappings": {"default": {"default": "centro abierto con asterisco con forma de lágrima"}}
    }, {
      "key": "273D",
      "mappings": {"default": {"default": "heavy teardrop spoked asterisk"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "273E",
      "mappings": {"default": {"default": "seis pétalos de florette en blanco y negro"}}
    }, {"category": "So", "key": "273F", "mappings": {"default": {"default": "florette negro"}}}, {
      "category": "So",
      "key": "2740",
      "mappings": {"default": {"default": "florette blanco"}}
    }, {
      "category": "So",
      "key": "2741",
      "mappings": {"default": {"default": "ocho Petalled Outlined Black Florette"}}
    }, {
      "category": "So",
      "key": "2742",
      "mappings": {"default": {"default": "círculo abierto centro ocho estrellas señaladas"}}
    }, {
      "category": "So",
      "key": "2743",
      "mappings": {"default": {"default": "asterisco pesado del molinillo de viento"}}
    }, {"category": "So", "key": "2744", "mappings": {"default": {"default": "copo de nieve"}}}, {
      "category": "So",
      "key": "2745",
      "mappings": {"default": {"default": "copo de nieve trifolio apretado"}}
    }, {
      "category": "So",
      "key": "2746",
      "mappings": {"default": {"default": "copo de nieve pesado de Chevron"}}
    }, {"category": "So", "key": "2747", "mappings": {"default": {"default": "brillar"}}}, {
      "category": "So",
      "key": "2748",
      "mappings": {"default": {"default": "chispa pesada"}}
    }, {"category": "So", "key": "2749", "mappings": {"default": {"default": "asterisco de globo"}}}, {
      "category": "So",
      "key": "274A",
      "mappings": {"default": {"default": "ocho asterisco de hélice con efecto de lágrima"}}
    }, {
      "category": "So",
      "key": "274B",
      "mappings": {"default": {"default": "heavy eight asterisk de hélice en forma de lágrima"}}
    }, {"category": "So", "key": "274C", "mappings": {"default": {"default": "marca de la cruz"}}}, {
      "category": "So",
      "key": "274D",
      "mappings": {"default": {"default": "círculo blanco sombreado"}}
    }]
  ,
  "es/units/energy.js":
    [{"locale": "es"}, {
      "key": "W",
      "names": ["W", "w"],
      "mappings": {"default": {"default": "watt"}},
      "category": "energy"
    }, {
      "key": "kW",
      "names": ["kw", "kW"],
      "mappings": {"default": {"default": "kilowatt"}},
      "category": "energy"
    }, {
      "key": "mW",
      "names": ["mw", "mW"],
      "mappings": {"default": {"default": "miliwatt"}},
      "category": "energy"
    }, {
      "key": "kwh",
      "names": ["kwh", "kWh"],
      "mappings": {"default": {"default": "kilowatt hora"}},
      "category": "energy"
    }, {"key": "J", "names": ["J"], "mappings": {"default": {"default": "Joule"}}, "category": "energy"}, {
      "key": "N",
      "names": ["N"],
      "mappings": {"default": {"default": "Newton"}},
      "category": "energy"
    }, {"key": "A", "names": ["A"], "mappings": {"default": {"default": "Ampere"}}, "category": "energy"}, {
      "key": "V",
      "names": ["V"],
      "mappings": {"default": {"default": "Voltio"}},
      "category": "energy"
    }, {
      "key": "µΩ",
      "names": ["µΩ"],
      "mappings": {"default": {"default": "microohmio"}},
      "category": "energy"
    }, {
      "key": "mΩ",
      "names": ["mΩ"],
      "mappings": {"default": {"default": "miliohmio"}},
      "category": "energy"
    }, {
      "key": "Ω",
      "names": ["Ω", "Ohm"],
      "mappings": {"default": {"default": "ohmio"}},
      "category": "energy"
    }, {
      "key": "kΩ",
      "names": ["kΩ", "KΩ"],
      "mappings": {"default": {"default": "kiloohmio"}},
      "category": "energy"
    }, {
      "key": "MΩ",
      "names": ["MΩ"],
      "mappings": {"default": {"default": "megaohmio"}},
      "category": "energy"
    }, {"key": "GΩ", "names": ["GΩ"], "mappings": {"default": {"default": "gigaohmio"}}, "category": "energy"}]
  ,
  "es/units/length.js":
    [{"locale": "es"}, {
      "key": "mm",
      "names": ["mm"],
      "mappings": {"default": {"default": "milímetro"}},
      "category": "length"
    }, {
      "key": "cm",
      "names": ["cm"],
      "mappings": {"default": {"default": "centímetro"}},
      "category": "length"
    }, {"key": "m", "names": ["m"], "mappings": {"default": {"default": "metro"}}, "category": "length"}, {
      "key": "km",
      "names": ["km"],
      "mappings": {"default": {"default": "kilómetro"}},
      "category": "length"
    }, {
      "key": "ft",
      "names": ["ft", "ft."],
      "mappings": {"default": {"default": "pies"}},
      "category": "length"
    }, {
      "key": "in",
      "names": ["in", "in."],
      "mappings": {"default": {"default": "pulgada"}},
      "category": "length"
    }, {
      "key": "mi",
      "names": ["mi", "mi."],
      "mappings": {"default": {"default": "milla"}},
      "category": "length"
    }, {
      "key": "yd",
      "names": ["yd", "yd."],
      "mappings": {"default": {"default": "yarda"}},
      "category": "length"
    }, {
      "key": "n.m.",
      "names": ["n.m."],
      "mappings": {"default": {"default": "millas náuticas"}},
      "category": "length"
    }]
  ,
  "es/units/memory.js":
    [{"locale": "es"}, {
      "key": "b",
      "names": ["b"],
      "mappings": {"default": {"default": "bit"}},
      "category": "memory"
    }, {"key": "B", "names": ["B"], "mappings": {"default": {"default": "byte"}}, "category": "memory"}, {
      "key": "KB",
      "names": ["KB"],
      "mappings": {"default": {"default": "kilobyte"}},
      "category": "memory"
    }, {
      "key": "MB",
      "names": ["MB"],
      "mappings": {"default": {"default": "megabyte"}},
      "category": "memory"
    }, {
      "key": "GB",
      "names": ["GB"],
      "mappings": {"default": {"default": "gigabyte"}},
      "category": "memory"
    }, {"key": "TB", "names": ["TB"], "mappings": {"default": {"default": "terabyte"}}, "category": "memory"}]
  ,
  "es/units/other.js":
    [{"locale": "es"}, {
      "key": "doz",
      "names": ["doz", "doz.", "dz", "dz."],
      "mappings": {"default": {"default": "docena"}},
      "category": "other"
    }, {
      "key": "sq",
      "names": ["sq", "sq."],
      "mappings": {"default": {"default": "al cuadrado"}},
      "category": "other"
    }, {"key": "ha", "names": ["ha"], "mappings": {"default": {"default": "hectarea"}}, "category": "other"}]
  ,
  "es/units/speed.js":
    [{"locale": "es"}, {
      "key": "kt",
      "names": ["kt", "kt."],
      "mappings": {"default": {"default": "nudo"}},
      "category": "speed"
    }, {
      "key": "mph",
      "names": ["mph"],
      "mappings": {"default": {"default": "millas por hora"}},
      "category": "speed"
    }, {
      "key": "rpm",
      "names": ["rpm"],
      "mappings": {"default": {"default": "revoluciones por minuto"}},
      "category": "speed"
    }, {
      "key": "kmh",
      "names": ["kmh"],
      "mappings": {"default": {"default": "kilómetros por hora"}},
      "category": "speed"
    }]
  ,
  "es/units/temperature.js":
    [{"locale": "es"}, {
      "key": "F",
      "names": ["F", "F.", "°F"],
      "mappings": {"default": {"default": "Fahrenheit"}},
      "category": "temperature"
    }, {
      "key": "C",
      "names": ["C", "°C"],
      "mappings": {"default": {"default": "centígrados"}},
      "category": "temperature"
    }, {"key": "K", "names": ["K", "°K"], "mappings": {"default": {"default": "Kelvin"}}, "category": "temperature"}]
  ,
  "es/units/time.js":
    [{"locale": "es"}, {
      "key": "ns",
      "names": ["ns"],
      "mappings": {"default": {"default": "nanosegundos"}},
      "category": "time"
    }, {
      "key": "µs",
      "names": ["µs"],
      "mappings": {"default": {"default": "microsegundos"}},
      "category": "time"
    }, {
      "key": "ms",
      "names": ["ms"],
      "mappings": {"default": {"default": "milisegundos"}},
      "category": "time"
    }, {
      "key": "s",
      "names": ["s"],
      "mappings": {"default": {"default": "segundos"}},
      "category": "time"
    }, {
      "key": "min",
      "names": ["min"],
      "mappings": {"default": {"default": "minutos"}},
      "category": "time"
    }, {"key": "h", "names": ["h", "hr"], "mappings": {"default": {"default": "hora"}}, "category": "time"}]
  ,
  "es/units/volume.js":
    [{"locale": "es"}, {
      "key": "bbl",
      "names": ["bbl.", "bbl"],
      "mappings": {"default": {"default": "barrile"}},
      "category": "volume"
    }, {
      "key": "cu",
      "names": ["cu", "cu."],
      "mappings": {"default": {"default": "cúbico"}},
      "category": "volume"
    }, {
      "key": "fl. oz.",
      "names": ["fl. oz."],
      "mappings": {"default": {"default": "onzas líquidas"}},
      "category": "volume"
    }, {
      "key": "gal",
      "names": ["gal", "gal."],
      "mappings": {"default": {"default": "galone"}},
      "category": "volume"
    }, {
      "key": "pt",
      "names": ["pt", "pt."],
      "mappings": {"default": {"default": "pinta"}},
      "category": "volume"
    }, {
      "key": "qt",
      "names": ["qt", "qt."],
      "mappings": {"default": {"default": "quart"}},
      "category": "volume"
    }, {
      "key": "tbsp",
      "names": ["tbsp", "tbsp."],
      "mappings": {"default": {"default": "cuchara"}},
      "category": "volume"
    }, {
      "key": "tsp",
      "names": ["tsp", "tsp."],
      "mappings": {"default": {"default": "cucharilla"}},
      "category": "volume"
    }, {
      "key": "cc",
      "names": ["cc"],
      "mappings": {"default": {"default": "centímetros cúbicos"}},
      "category": "volume"
    }, {
      "key": "kl",
      "names": ["kl"],
      "mappings": {"default": {"default": "kilolitro"}},
      "category": "volume"
    }, {"key": "l", "names": ["l"], "mappings": {"default": {"default": "litro"}}, "category": "volume"}, {
      "key": "ml",
      "names": ["ml"],
      "mappings": {"default": {"default": "mililitro"}},
      "category": "volume"
    }]
  ,
  "es/units/weight.js":
    [{"locale": "es"}, {
      "key": "lb",
      "names": ["lb", "lb."],
      "mappings": {"default": {"default": "libra"}},
      "category": "weight"
    }, {
      "key": "LT",
      "names": ["LT", "L.T."],
      "mappings": {"default": {"default": "libras imperiales"}},
      "category": "weight"
    }, {
      "key": "oz",
      "names": ["oz", "oz."],
      "mappings": {"default": {"default": "onza"}},
      "category": "weight"
    }, {
      "key": "µg",
      "names": ["µg", "mcg"],
      "mappings": {"default": {"default": "microgramo"}},
      "category": "weight"
    }, {
      "key": "gr",
      "names": ["g", "gr"],
      "mappings": {"default": {"default": "gramo"}},
      "category": "weight"
    }, {
      "key": "kg",
      "names": ["kg"],
      "mappings": {"default": {"default": "kilogramo"}},
      "category": "weight"
    }, {
      "key": "mg",
      "names": ["mg"],
      "mappings": {"default": {"default": "miligramo"}},
      "category": "weight"
    }, {"key": "t", "names": ["t", "T"], "mappings": {"default": {"default": "tonelada"}}, "category": "weight"}]
  ,
  "nemeth/functions/algebra.js":
    [{"locale": "nemeth"}, {
      "category": "Algebra",
      "mappings": {"default": {"default": "⠙⠑⠛"}},
      "key": "deg",
      "names": ["deg"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "⠙⠑⠞"}},
      "key": "det",
      "names": ["det"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "⠙⠊⠍"}},
      "key": "dim",
      "names": ["dim"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "⠓⠕⠍"}},
      "key": "hom",
      "names": ["hom", "Hom"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "⠅⠑⠗"}},
      "key": "ker",
      "names": ["ker"]
    }, {"category": "Algebra", "mappings": {"default": {"default": "⠠⠞⠗"}}, "key": "Tr", "names": ["Tr", "tr"]}]
  ,
  "nemeth/functions/elementary.js":
    [{"locale": "nemeth"}, {
      "category": "Logarithm",
      "mappings": {"default": {"default": "⠇⠕⠏⠀"}},
      "key": "log",
      "names": ["log"]
    }, {
      "category": "Logarithm",
      "mappings": {"default": {"default": "⠇⠝⠀"}},
      "key": "ln",
      "names": ["ln"]
    }, {
      "category": "Logarithm",
      "mappings": {"default": {"default": "⠇⠛"}},
      "key": "lg",
      "names": ["lg"]
    }, {
      "category": "Elementary",
      "mappings": {"default": {"default": "⠑⠭⠏"}},
      "key": "exp",
      "names": ["exp", "expt"]
    }, {
      "category": "Elementary",
      "mappings": {"default": {"default": "⠛⠉⠙"}},
      "key": "gcd",
      "names": ["gcd"]
    }, {
      "category": "Elementary",
      "mappings": {"default": {"default": "⠇⠉⠍"}},
      "key": "lcm",
      "names": ["lcm"]
    }, {
      "category": "Complex",
      "mappings": {"default": {"default": "⠁⠗⠛⠀"}},
      "key": "arg",
      "names": ["arg"]
    }, {
      "category": "Complex",
      "mappings": {"default": {"default": "⠊⠍"}},
      "key": "im",
      "names": ["im"]
    }, {
      "category": "Complex",
      "mappings": {"default": {"default": "⠗⠑"}},
      "key": "re",
      "names": ["re"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "⠊⠝⠋⠀"}},
      "key": "inf",
      "names": ["inf"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "⠇⠊⠍"}},
      "key": "lim",
      "names": ["lim"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "⠇⠊⠍⠊⠝⠀⠋"}},
      "key": "liminf",
      "names": ["lim inf", "liminf"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "⠇⠊⠍⠎⠥⠏"}},
      "key": "limsup",
      "names": ["lim sup", "limsup"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "⠍⠁⠭⠀"}},
      "key": "max",
      "names": ["max"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "⠍⠊⠝⠀"}},
      "key": "min",
      "names": ["min"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "⠎⠥⠏"}},
      "key": "sup",
      "names": ["sup"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "⠊⠝⠚⠇⠊⠍"}},
      "key": "injlim",
      "names": ["injlim", "inj lim"]
    }, {
      "category": "Limits",
      "mappings": {"default": {"default": "⠏⠗⠕⠚⠇⠊⠍"}},
      "key": "projlim",
      "names": ["projlim", "proj lim"]
    }, {
      "category": "Elementary",
      "mappings": {"default": {"default": "⠍⠕⠙"}},
      "key": "mod",
      "names": ["mod"]
    }, {"category": "Probability", "mappings": {"default": {"default": "⠠⠏⠗"}}, "key": "Pr", "names": ["Pr"]}]
  ,
  "nemeth/functions/hyperbolic.js":
    [{"locale": "nemeth"}, {
      "category": "Hyperbolic",
      "mappings": {"default": {"default": "⠉⠕⠎⠓⠀"}},
      "key": "cosh",
      "names": ["cosh"]
    }, {
      "category": "Hyperbolic",
      "mappings": {"default": {"default": "⠉⠕⠞⠓⠀"}},
      "key": "coth",
      "names": ["coth"]
    }, {
      "category": "Hyperbolic",
      "mappings": {"default": {"default": "⠉⠎⠉⠓⠀"}},
      "key": "csch",
      "names": ["csch"]
    }, {
      "category": "Hyperbolic",
      "mappings": {"default": {"default": "⠎⠑⠉⠓⠀"}},
      "key": "sech",
      "names": ["sech"]
    }, {
      "category": "Hyperbolic",
      "mappings": {"default": {"default": "⠎⠊⠝⠓⠀"}},
      "key": "sinh",
      "names": ["sinh"]
    }, {
      "category": "Hyperbolic",
      "mappings": {"default": {"default": "⠞⠁⠕⠓⠀"}},
      "key": "tanh",
      "names": ["tanh"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "⠁⠗⠉⠀⠕⠎⠓"}},
      "key": "arcosh",
      "names": ["arcosh", "arccosh"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "⠁⠗⠉⠀⠕⠞⠓"}},
      "key": "arcoth",
      "names": ["arcoth", "arccoth"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "⠁⠗⠉⠀⠎⠉⠓"}},
      "key": "arcsch",
      "names": ["arcsch", "arccsch"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "⠁⠗⠎⠑⠉⠓⠀"}},
      "key": "arsech",
      "names": ["arsech", "arcsech"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "⠁⠗⠎⠊⠝⠓⠀"}},
      "key": "arsinh",
      "names": ["arsinh", "arcsinh"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "⠁⠗⠞⠁⠕⠓⠀"}},
      "key": "artanh",
      "names": ["artanh", "arctanh"]
    }]
  ,
  "nemeth/functions/trigonometry.js":
    [{"locale": "nemeth"}, {
      "category": "Trigonometric",
      "mappings": {"default": {"default": "⠉⠕⠎⠀"}},
      "key": "cos",
      "names": ["cos", "cosine"]
    }, {
      "category": "Trigonometric",
      "mappings": {"default": {"default": "⠉⠕⠞⠀"}},
      "key": "cot",
      "names": ["cot"]
    }, {
      "category": "Trigonometric",
      "mappings": {"default": {"default": "⠉⠎⠉⠀"}},
      "key": "csc",
      "names": ["csc"]
    }, {
      "category": "Trigonometric",
      "mappings": {"default": {"default": "⠎⠑⠉⠀"}},
      "key": "sec",
      "names": ["sec"]
    }, {
      "category": "Trigonometric",
      "mappings": {"default": {"default": "⠎⠊⠝⠀"}},
      "key": "sin",
      "names": ["sin", "sine"]
    }, {
      "category": "Trigonometric",
      "mappings": {"default": {"default": "⠞⠁⠝⠀"}},
      "key": "tan",
      "names": ["tan"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "⠁⠗⠉⠉⠕⠎⠀"}},
      "key": "arccos",
      "names": ["arccos"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "⠁⠗⠉⠀⠉⠕⠞⠀"}},
      "key": "arccot",
      "names": ["arccot"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "⠁⠗⠉⠀⠉⠎⠉⠀"}},
      "key": "arccsc",
      "names": ["arccsc"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "⠁⠗⠉⠀⠎⠑⠉⠀"}},
      "key": "arcsec",
      "names": ["arcsec"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "⠁⠗⠉⠀⠎⠊⠝⠀"}},
      "key": "arcsin",
      "names": ["arcsin"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "⠁⠗⠉⠀⠞⠁⠝⠀"}},
      "key": "arctan",
      "names": ["arctan"]
    }]
  ,
  "nemeth/symbols/digits_rest.js":
    [{"locale": "nemeth"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠹⠂⠌⠲⠼"}},
      "key": "00BC"
    }, {"category": "No", "mappings": {"default": {"default": "⠹⠂⠌⠆⠼"}}, "key": "00BD"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠹⠒⠌⠲⠼"}},
      "key": "00BE"
    }, {"category": "No", "mappings": {"default": {"default": "⠹⠂⠌⠶⠼"}}, "key": "2150"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠹⠂⠌⠔⠼"}},
      "key": "2151"
    }, {"category": "No", "mappings": {"default": {"default": "⠹⠂⠌⠂⠴⠼"}}, "key": "2152"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠹⠂⠌⠒⠼"}},
      "key": "2153"
    }, {"category": "No", "mappings": {"default": {"default": "⠹⠆⠌⠒⠼"}}, "key": "2154"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠹⠂⠌⠢⠼"}},
      "key": "2155"
    }, {"category": "No", "mappings": {"default": {"default": "⠹⠆⠌⠢⠼"}}, "key": "2156"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠹⠒⠌⠢⠼"}},
      "key": "2157"
    }, {"category": "No", "mappings": {"default": {"default": "⠹⠲⠌⠢⠼"}}, "key": "2158"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠹⠂⠌⠖⠼"}},
      "key": "2159"
    }, {"category": "No", "mappings": {"default": {"default": "⠹⠢⠌⠖⠼"}}, "key": "215A"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠹⠂⠌⠦⠼"}},
      "key": "215B"
    }, {"category": "No", "mappings": {"default": {"default": "⠹⠒⠌⠦⠼"}}, "key": "215C"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠹⠢⠌⠦⠼"}},
      "key": "215D"
    }, {"category": "No", "mappings": {"default": {"default": "⠹⠶⠌⠦⠼"}}, "key": "215E"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠹⠂⠌⠼"}},
      "key": "215F"
    }, {"category": "No", "mappings": {"default": {"default": "⠹⠴⠌⠒⠼"}}, "key": "2189"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠫⠸⠲⠸⠫⠫⠉⠸⠫⠼⠂⠴⠻⠻"}},
      "key": "3248"
    }, {"category": "No", "mappings": {"default": {"default": "⠫⠸⠲⠸⠫⠫⠉⠸⠫⠼⠆⠴⠻⠻"}}, "key": "3249"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠫⠸⠲⠸⠫⠫⠉⠸⠫⠼⠒⠴⠻⠻"}},
      "key": "324A"
    }, {"category": "No", "mappings": {"default": {"default": "⠫⠸⠲⠸⠫⠫⠉⠸⠫⠼⠲⠴⠻⠻"}}, "key": "324B"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠫⠸⠲⠸⠫⠫⠉⠸⠫⠼⠢⠴⠻⠻"}},
      "key": "324C"
    }, {"category": "No", "mappings": {"default": {"default": "⠫⠸⠲⠸⠫⠫⠉⠸⠫⠼⠖⠴⠻⠻"}}, "key": "324D"}, {
      "category": "No",
      "mappings": {"default": {"default": "⠫⠸⠲⠸⠫⠫⠉⠸⠫⠼⠶⠴⠻⠻"}},
      "key": "324E"
    }, {"category": "No", "mappings": {"default": {"default": "⠫⠸⠲⠸⠫⠫⠉⠸⠫⠼⠦⠴⠻⠻"}}, "key": "324F"}]
  ,
  "nemeth/symbols/greek-rest.js":
    [{"locale": "nemeth"}]
  ,
  "nemeth/symbols/greek-scripts.js":
    [{"locale": "nemeth"}, {
      "category": "Ll",
      "key": "1D26",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠆⠖⠄"}}
    }, {"category": "Ll", "key": "1D27", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠆⠶⠄"}}}, {
      "category": "Ll",
      "key": "1D28",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠆⠦⠄"}}
    }, {"category": "Ll", "key": "1D29", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠆⠔⠄"}}}, {
      "category": "Ll",
      "key": "1D2A",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠆⠁⠄"}}
    }, {"category": "Lm", "key": "1D5E", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠢⠑⠄"}}}, {
      "category": "Lm",
      "key": "1D60",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠴⠄"}}
    }, {"category": "Lm", "key": "1D66", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠖⠄"}}}, {
      "category": "Lm",
      "key": "1D67",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠶⠄"}}
    }, {"category": "Lm", "key": "1D68", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠦⠄"}}}, {
      "category": "Lm",
      "key": "1D69",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠔⠄"}}
    }, {"category": "Lm", "key": "1D6A", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠁⠄"}}}]
  ,
  "nemeth/symbols/greek-symbols.js":
    [{"locale": "nemeth"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠙⠴⠄"}},
      "key": "03D0"
    }, {"category": "Ll", "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠙⠶⠄"}}, "key": "03D7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠋⠖⠄"}},
      "key": "03F6"
    }, {"category": "Lu", "mappings": {"default": {"default": "⠄⡳⠽⠂⠙⠶⠉⠁⠄"}}, "key": "1D7CA"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "⠄⡳⠽⠂⠙⠶⠉⠃⠄"}},
      "key": "1D7CB"
    }]
  ,
  "nemeth/symbols/hebrew_letters.js":
    [{"locale": "nemeth"}, {
      "category": "Lo",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠒⠢⠄"}},
      "key": "2135"
    }, {"category": "Lo", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠒⠖⠄"}}, "key": "2136"}, {
      "category": "Lo",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠒⠶⠄"}},
      "key": "2137"
    }, {"category": "Lo", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠒⠦⠄"}}, "key": "2138"}]
  ,
  "nemeth/symbols/latin-lower-double-accent.js":
    [{"locale": "nemeth"}, {
      "category": "Ll",
      "key": "01D6",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠖⠄"}}
    }, {"category": "Ll", "key": "01D8", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠦⠄"}}}, {
      "category": "Ll",
      "key": "01DA",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠁⠄"}}
    }, {"category": "Ll", "key": "01DC", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠉⠄"}}}, {
      "category": "Ll",
      "key": "01DF",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠋⠄"}}
    }, {"category": "Ll", "key": "01E1", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠂⠄"}}}, {
      "category": "Ll",
      "key": "01ED",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠙⠄"}}
    }, {"category": "Ll", "key": "01FB", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠋⠃⠄"}}}, {
      "category": "Ll",
      "key": "022B",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠃⠄"}}
    }, {"category": "Ll", "key": "022D", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠙⠄"}}}, {
      "category": "Ll",
      "key": "0231",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠒⠂⠄"}}
    }, {"category": "Ll", "key": "1E09", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠔⠄"}}}, {
      "category": "Ll",
      "key": "1E15",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠢⠄"}}
    }, {"category": "Ll", "key": "1E17", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠶⠄"}}}, {
      "category": "Ll",
      "key": "1E1D",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠙⠄"}}
    }, {"category": "Ll", "key": "1E2F", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠋⠄"}}}, {
      "category": "Ll",
      "key": "1E39",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠔⠄"}}
    }, {"category": "Ll", "key": "1E4D", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠙⠄"}}}, {
      "category": "Ll",
      "key": "1E4F",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠋⠄"}}
    }, {"category": "Ll", "key": "1E51", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠂⠄"}}}, {
      "category": "Ll",
      "key": "1E53",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠒⠄"}}
    }, {"category": "Ll", "key": "1E5D", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠙⠄"}}}, {
      "category": "Ll",
      "key": "1E65",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠢⠄"}}
    }, {"category": "Ll", "key": "1E67", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠶⠄"}}}, {
      "category": "Ll",
      "key": "1E69",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠔⠄"}}
    }, {"category": "Ll", "key": "1E79", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠔⠄"}}}, {
      "category": "Ll",
      "key": "1E7B",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠃⠄"}}
    }, {"category": "Ll", "key": "1EA5", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠢⠄"}}}, {
      "category": "Ll",
      "key": "1EA7",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠶⠄"}}
    }, {"category": "Ll", "key": "1EA9", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠔⠄"}}}, {
      "category": "Ll",
      "key": "1EAB",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠃⠄"}}
    }, {"category": "Ll", "key": "1EAD", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠙⠄"}}}, {
      "category": "Ll",
      "key": "1EAF",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠋⠄"}}
    }, {"category": "Ll", "key": "1EB1", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠂⠄"}}}, {
      "category": "Ll",
      "key": "1EB3",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠒⠄"}}
    }, {"category": "Ll", "key": "1EB5", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠢⠄"}}}, {
      "category": "Ll",
      "key": "1EB7",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠶⠄"}}
    }, {"category": "Ll", "key": "1EBF", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠋⠄"}}}, {
      "category": "Ll",
      "key": "1EC1",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠂⠄"}}
    }, {"category": "Ll", "key": "1EC3", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠒⠄"}}}, {
      "category": "Ll",
      "key": "1EC5",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠢⠄"}}
    }, {"category": "Ll", "key": "1EC7", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠶⠄"}}}, {
      "category": "Ll",
      "key": "1ED1",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠂⠄"}}
    }, {"category": "Ll", "key": "1ED3", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠒⠄"}}}, {
      "category": "Ll",
      "key": "1ED5",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠢⠄"}}
    }, {"category": "Ll", "key": "1ED7", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠶⠄"}}}, {
      "category": "Ll",
      "key": "1ED9",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠔⠄"}}
    }, {"category": "Ll", "key": "1EDB", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠃⠄"}}}, {
      "category": "Ll",
      "key": "1EDD",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠙⠄"}}
    }, {"category": "Ll", "key": "1EDF", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠋⠄"}}}, {
      "category": "Ll",
      "key": "1EE1",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠂⠄"}}
    }, {"category": "Ll", "key": "1EE3", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠒⠄"}}}, {
      "category": "Ll",
      "key": "1EE9",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠔⠄"}}
    }, {"category": "Ll", "key": "1EEB", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠃⠄"}}}, {
      "category": "Ll",
      "key": "1EED",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠙⠄"}}
    }, {"category": "Ll", "key": "1EEF", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠋⠄"}}}, {
      "category": "Ll",
      "key": "1EF1",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠋⠂⠄"}}
    }]
  ,
  "nemeth/symbols/latin-lower-phonetic.js":
    [{"locale": "nemeth"}, {
      "category": "Ll",
      "key": "00F8",
      "mappings": {"default": {"default": "⠈⠕"}}
    }, {"category": "Ll", "key": "0111", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠂⠄"}}}, {
      "category": "Ll",
      "key": "0127",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠶⠄"}}
    }, {"category": "Ll", "key": "0142", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠆⠄"}}}, {
      "category": "Ll",
      "key": "0167",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠶⠄"}}
    }, {"category": "Ll", "key": "0180", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠦⠴⠄"}}}, {
      "category": "Ll",
      "key": "019B",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠔⠃⠄"}}
    }, {"category": "Ll", "key": "01B6", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠃⠖⠄"}}}, {
      "category": "Ll",
      "key": "01BE",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠃⠑⠄"}}
    }, {"category": "Ll", "key": "01E5", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠢⠄"}}}, {
      "category": "Ll",
      "key": "01FF",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠋⠋⠄"}}
    }, {"category": "Ll", "key": "023C", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠒⠉⠄"}}}, {
      "category": "Ll",
      "key": "0247",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠲⠶⠄"}}
    }, {"category": "Ll", "key": "0249", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠲⠔⠄"}}}, {
      "category": "Ll",
      "key": "024D",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠲⠙⠄"}}
    }, {"category": "Ll", "key": "024F", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠲⠋⠄"}}}, {
      "category": "Ll",
      "key": "025F",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠋⠄"}}
    }, {"category": "Ll", "key": "0268", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠦⠄"}}}, {
      "category": "Ll",
      "key": "0284",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠲⠄"}}
    }, {"category": "Ll", "key": "02A1", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠂⠄"}}}, {
      "category": "Ll",
      "key": "02A2",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠆⠄"}}
    }, {"category": "Ll", "key": "1D13", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠂⠒⠄"}}}, {
      "category": "Ll",
      "key": "1D7C",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠉⠄"}}
    }, {"category": "Ll", "key": "1D7D", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠙⠄"}}}, {
      "category": "Ll",
      "key": "1D7F",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠋⠄"}}
    }, {"category": "Ll", "key": "1E9C", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠉⠄"}}}, {
      "category": "Ll",
      "key": "1E9D",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠙⠄"}}
    }, {"category": "Ll", "key": "018D", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠦⠙⠄"}}}, {
      "category": "Ll",
      "key": "1E9B",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠃⠄"}}
    }, {"category": "Ll", "key": "1E9F", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠋⠄"}}}, {
      "category": "Ll",
      "key": "0138",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠦⠄"}}
    }, {"category": "Ll", "key": "017F", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠋⠄"}}}, {
      "category": "Ll",
      "key": "0183",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠦⠒⠄"}}
    }, {"category": "Ll", "key": "0185", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠦⠢⠄"}}}, {
      "category": "Ll",
      "key": "0188",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠦⠦⠄"}}
    }, {"category": "Ll", "key": "018C", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠦⠉⠄"}}}, {
      "category": "Ll",
      "key": "0192",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠔⠆⠄"}}
    }, {"category": "Ll", "key": "0195", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠔⠢⠄"}}}, {
      "category": "Ll",
      "key": "0199",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠔⠔⠄"}}
    }, {"category": "Ll", "key": "019A", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠔⠁⠄"}}}, {
      "category": "Ll",
      "key": "019E",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠔⠑⠄"}}
    }, {"category": "Ll", "key": "01A1", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠁⠂⠄"}}}, {
      "category": "Ll",
      "key": "01A3",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠁⠒⠄"}}
    }, {"category": "Ll", "key": "01A5", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠁⠢⠄"}}}, {
      "category": "Ll",
      "key": "01A8",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠁⠦⠄"}}
    }, {"category": "Ll", "key": "01AA", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠁⠁⠄"}}}, {
      "category": "Ll",
      "key": "01AB",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠁⠃⠄"}}
    }, {"category": "Ll", "key": "01AD", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠁⠙⠄"}}}, {
      "category": "Ll",
      "key": "01B0",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠃⠴⠄"}}
    }, {"category": "Ll", "key": "01B4", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠃⠲⠄"}}}, {
      "category": "Ll",
      "key": "01B9",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠃⠔⠄"}}
    }, {"category": "Ll", "key": "01BA", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠃⠁⠄"}}}, {
      "category": "Ll",
      "key": "01BD",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠃⠙⠄"}}
    }, {"category": "Ll", "key": "01BF", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠃⠋⠄"}}}, {
      "category": "Ll",
      "key": "01C6",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠉⠖⠄"}}
    }, {"category": "Ll", "key": "01C9", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠉⠔⠄"}}}, {
      "category": "Ll",
      "key": "01CC",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠉⠉⠄"}}
    }, {"category": "Ll", "key": "01E3", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠒⠄"}}}, {
      "category": "Ll",
      "key": "01EF",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠋⠄"}}
    }, {"category": "Ll", "key": "01F3", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠋⠒⠄"}}}, {
      "category": "Ll",
      "key": "01DD",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠙⠄"}}
    }, {"category": "Ll", "key": "01FD", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠋⠙⠄"}}}, {
      "category": "Ll",
      "key": "021D",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠙⠄"}}
    }, {"category": "Ll", "key": "0221", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠂⠄"}}}, {
      "category": "Ll",
      "key": "0223",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠒⠄"}}
    }, {"category": "Ll", "key": "0225", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠢⠄"}}}, {
      "category": "Ll",
      "key": "0234",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠒⠲⠄"}}
    }, {"category": "Ll", "key": "0235", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠒⠢⠄"}}}, {
      "category": "Ll",
      "key": "0236",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠒⠖⠄"}}
    }, {"category": "Ll", "key": "0238", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠒⠦⠄"}}}, {
      "category": "Ll",
      "key": "0239",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠒⠔⠄"}}
    }, {"category": "Ll", "key": "023F", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠒⠋⠄"}}}, {
      "category": "Ll",
      "key": "0240",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠲⠴⠄"}}
    }, {"category": "Ll", "key": "0242", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠲⠆⠄"}}}, {
      "category": "Ll",
      "key": "024B",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠲⠃⠄"}}
    }, {"category": "Ll", "key": "0250", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠴⠄"}}}, {
      "category": "Ll",
      "key": "0251",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠂⠄"}}
    }, {"category": "Ll", "key": "0252", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠆⠄"}}}, {
      "category": "Ll",
      "key": "0253",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠒⠄"}}
    }, {"category": "Ll", "key": "0254", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠲⠄"}}}, {
      "category": "Ll",
      "key": "0255",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠢⠄"}}
    }, {"category": "Ll", "key": "0256", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠖⠄"}}}, {
      "category": "Ll",
      "key": "0257",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠶⠄"}}
    }, {"category": "Ll", "key": "0258", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠦⠄"}}}, {
      "category": "Ll",
      "key": "0259",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠔⠄"}}
    }, {"category": "Ll", "key": "025A", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠁⠄"}}}, {
      "category": "Ll",
      "key": "025B",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠃⠄"}}
    }, {"category": "Ll", "key": "025C", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠉⠄"}}}, {
      "category": "Ll",
      "key": "025D",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠙⠄"}}
    }, {"category": "Ll", "key": "025E", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠢⠑⠄"}}}, {
      "category": "Ll",
      "key": "0260",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠴⠄"}}
    }, {"category": "Ll", "key": "0261", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠂⠄"}}}, {
      "category": "Ll",
      "key": "0263",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠒⠄"}}
    }, {"category": "Ll", "key": "0264", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠲⠄"}}}, {
      "category": "Ll",
      "key": "0265",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠢⠄"}}
    }, {"category": "Ll", "key": "0266", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠖⠄"}}}, {
      "category": "Ll",
      "key": "0267",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠶⠄"}}
    }, {"category": "Ll", "key": "0269", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠔⠄"}}}, {
      "category": "Ll",
      "key": "026B",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠃⠄"}}
    }, {"category": "Ll", "key": "026C", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠉⠄"}}}, {
      "category": "Ll",
      "key": "026D",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠙⠄"}}
    }, {"category": "Ll", "key": "026E", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠑⠄"}}}, {
      "category": "Ll",
      "key": "026F",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠖⠋⠄"}}
    }, {"category": "Ll", "key": "0270", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠴⠄"}}}, {
      "category": "Ll",
      "key": "0271",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠂⠄"}}
    }, {"category": "Ll", "key": "0272", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠆⠄"}}}, {
      "category": "Ll",
      "key": "0273",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠒⠄"}}
    }, {"category": "Ll", "key": "0275", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠢⠄"}}}, {
      "category": "Ll",
      "key": "0277",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠶⠄"}}
    }, {"category": "Ll", "key": "0278", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠦⠄"}}}, {
      "category": "Ll",
      "key": "0279",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠔⠄"}}
    }, {"category": "Ll", "key": "027A", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠁⠄"}}}, {
      "category": "Ll",
      "key": "027B",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠃⠄"}}
    }, {"category": "Ll", "key": "027C", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠉⠄"}}}, {
      "category": "Ll",
      "key": "027D",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠙⠄"}}
    }, {"category": "Ll", "key": "027E", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠑⠄"}}}, {
      "category": "Ll",
      "key": "027F",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠶⠋⠄"}}
    }, {"category": "Ll", "key": "0282", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠆⠄"}}}, {
      "category": "Ll",
      "key": "0283",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠒⠄"}}
    }, {"category": "Ll", "key": "0285", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠢⠄"}}}, {
      "category": "Ll",
      "key": "0286",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠖⠄"}}
    }, {"category": "Ll", "key": "0287", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠶⠄"}}}, {
      "category": "Ll",
      "key": "0288",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠦⠄"}}
    }, {"category": "Ll", "key": "0289", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠔⠄"}}}, {
      "category": "Ll",
      "key": "028A",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠁⠄"}}
    }, {"category": "Ll", "key": "028B", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠃⠄"}}}, {
      "category": "Ll",
      "key": "028C",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠉⠄"}}
    }, {"category": "Ll", "key": "028D", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠙⠄"}}}, {
      "category": "Ll",
      "key": "028E",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠦⠑⠄"}}
    }, {"category": "Ll", "key": "0290", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠔⠴⠄"}}}, {
      "category": "Ll",
      "key": "0291",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠔⠂⠄"}}
    }, {"category": "Ll", "key": "0292", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠔⠆⠄"}}}, {
      "category": "Ll",
      "key": "0293",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠔⠒⠄"}}
    }, {"category": "Ll", "key": "0295", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠔⠢⠄"}}}, {
      "category": "Ll",
      "key": "0296",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠔⠖⠄"}}
    }, {"category": "Ll", "key": "0297", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠔⠶⠄"}}}, {
      "category": "Ll",
      "key": "0298",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠔⠦⠄"}}
    }, {"category": "Ll", "key": "029A", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠔⠁⠄"}}}, {
      "category": "Ll",
      "key": "029E",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠔⠑⠄"}}
    }, {"category": "Ll", "key": "02A0", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠴⠄"}}}, {
      "category": "Ll",
      "key": "02A3",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠒⠄"}}
    }, {"category": "Ll", "key": "02A4", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠲⠄"}}}, {
      "category": "Ll",
      "key": "02A5",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠢⠄"}}
    }, {"category": "Ll", "key": "02A6", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠖⠄"}}}, {
      "category": "Ll",
      "key": "02A7",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠶⠄"}}
    }, {"category": "Ll", "key": "02A8", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠦⠄"}}}, {
      "category": "Ll",
      "key": "02A9",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠔⠄"}}
    }, {"category": "Ll", "key": "02AA", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠁⠄"}}}, {
      "category": "Ll",
      "key": "02AB",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠃⠄"}}
    }, {"category": "Ll", "key": "02AC", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠉⠄"}}}, {
      "category": "Ll",
      "key": "02AD",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠙⠄"}}
    }, {"category": "Ll", "key": "02AE", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠑⠄"}}}, {
      "category": "Ll",
      "key": "02AF",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠁⠋⠄"}}
    }, {"category": "Ll", "key": "1D02", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠴⠆⠄"}}}, {
      "category": "Ll",
      "key": "1D08",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠴⠦⠄"}}
    }, {"category": "Ll", "key": "1D09", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠴⠔⠄"}}}, {
      "category": "Ll",
      "key": "1D11",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠂⠂⠄"}}
    }, {"category": "Ll", "key": "1D12", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠂⠆⠄"}}}, {
      "category": "Ll",
      "key": "1D14",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠂⠲⠄"}}
    }, {"category": "Ll", "key": "1D16", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠂⠖⠄"}}}, {
      "category": "Ll",
      "key": "1D17",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠂⠶⠄"}}
    }, {"category": "Ll", "key": "1D1D", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠂⠙⠄"}}}, {
      "category": "Ll",
      "key": "1D1E",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠂⠑⠄"}}
    }, {"category": "Ll", "key": "1D1F", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠂⠋⠄"}}}, {
      "category": "Ll",
      "key": "1D24",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠆⠲⠄"}}
    }, {"category": "Ll", "key": "1D25", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠆⠢⠄"}}}, {
      "category": "Ll",
      "key": "1D6B",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠃⠄"}}
    }, {"category": "Ll", "key": "1D6C", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠉⠄"}}}, {
      "category": "Ll",
      "key": "1D6D",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠙⠄"}}
    }, {"category": "Ll", "key": "1D6E", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠑⠄"}}}, {
      "category": "Ll",
      "key": "1D6F",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠋⠄"}}
    }, {"category": "Ll", "key": "1D70", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠴⠄"}}}, {
      "category": "Ll",
      "key": "1D71",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠂⠄"}}
    }, {"category": "Ll", "key": "1D72", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠆⠄"}}}, {
      "category": "Ll",
      "key": "1D73",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠒⠄"}}
    }, {"category": "Ll", "key": "1D74", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠲⠄"}}}, {
      "category": "Ll",
      "key": "1D75",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠢⠄"}}
    }, {"category": "Ll", "key": "1D76", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠖⠄"}}}, {
      "category": "Ll",
      "key": "1D77",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠶⠄"}}
    }, {"category": "Ll", "key": "1D79", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠔⠄"}}}, {
      "category": "Ll",
      "key": "1D7A",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠶⠁⠄"}}
    }, {"category": "Ll", "key": "1D80", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠴⠄"}}}, {
      "category": "Ll",
      "key": "1D81",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠂⠄"}}
    }, {"category": "Ll", "key": "1D82", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠆⠄"}}}, {
      "category": "Ll",
      "key": "1D83",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠒⠄"}}
    }, {"category": "Ll", "key": "1D84", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠲⠄"}}}, {
      "category": "Ll",
      "key": "1D85",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠢⠄"}}
    }, {"category": "Ll", "key": "1D86", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠖⠄"}}}, {
      "category": "Ll",
      "key": "1D87",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠶⠄"}}
    }, {"category": "Ll", "key": "1D88", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠦⠄"}}}, {
      "category": "Ll",
      "key": "1D89",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠔⠄"}}
    }, {"category": "Ll", "key": "1D8A", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠁⠄"}}}, {
      "category": "Ll",
      "key": "1D8B",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠃⠄"}}
    }, {"category": "Ll", "key": "1D8C", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠉⠄"}}}, {
      "category": "Ll",
      "key": "1D8D",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠙⠄"}}
    }, {"category": "Ll", "key": "1D8E", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠑⠄"}}}, {
      "category": "Ll",
      "key": "1D8F",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠦⠋⠄"}}
    }, {"category": "Ll", "key": "1D90", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠔⠴⠄"}}}, {
      "category": "Ll",
      "key": "1D91",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠔⠂⠄"}}
    }, {"category": "Ll", "key": "1D92", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠔⠆⠄"}}}, {
      "category": "Ll",
      "key": "1D93",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠔⠒⠄"}}
    }, {"category": "Ll", "key": "1D94", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠔⠲⠄"}}}, {
      "category": "Ll",
      "key": "1D95",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠔⠢⠄"}}
    }, {"category": "Ll", "key": "1D96", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠔⠖⠄"}}}, {
      "category": "Ll",
      "key": "1D97",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠔⠶⠄"}}
    }, {"category": "Ll", "key": "1D98", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠔⠦⠄"}}}, {
      "category": "Ll",
      "key": "1D99",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠔⠔⠄"}}
    }, {"category": "Ll", "key": "1D9A", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠔⠁⠄"}}}, {
      "category": "Ll",
      "key": "0149",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠔⠄"}}
    }, {"category": "Ll", "key": "014B", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠃⠄"}}}]
  ,
  "nemeth/symbols/latin-lower-single-accent.js":
    [{"locale": "nemeth"}, {
      "category": "Ll",
      "key": "00E0",
      "mappings": {"default": {"default": "⠈⠁"}}
    }, {"category": "Ll", "key": "00E1", "mappings": {"default": {"default": "⠈⠁"}}}, {
      "category": "Ll",
      "key": "00E2",
      "mappings": {"default": {"default": "⠈⠁"}}
    }, {"category": "Ll", "key": "00E3", "mappings": {"default": {"default": "⠈⠁"}}}, {
      "category": "Ll",
      "key": "00E4",
      "mappings": {"default": {"default": "⠈⠁"}}
    }, {"category": "Ll", "key": "00E5", "mappings": {"default": {"default": "⠈⠁"}}}, {
      "category": "Ll",
      "key": "00E7",
      "mappings": {"default": {"default": "⠈⠉"}}
    }, {"category": "Ll", "key": "00E8", "mappings": {"default": {"default": "⠈⠑"}}}, {
      "category": "Ll",
      "key": "00E9",
      "mappings": {"default": {"default": "⠈⠑"}}
    }, {"category": "Ll", "key": "00EA", "mappings": {"default": {"default": "⠈⠑"}}}, {
      "category": "Ll",
      "key": "00EB",
      "mappings": {"default": {"default": "⠈⠑"}}
    }, {"category": "Ll", "key": "00EC", "mappings": {"default": {"default": "⠈⠊"}}}, {
      "category": "Ll",
      "key": "00ED",
      "mappings": {"default": {"default": "⠈⠊"}}
    }, {"category": "Ll", "key": "00EE", "mappings": {"default": {"default": "⠈⠊"}}}, {
      "category": "Ll",
      "key": "00EF",
      "mappings": {"default": {"default": "⠈⠊"}}
    }, {"category": "Ll", "key": "00F1", "mappings": {"default": {"default": "⠈⠝"}}}, {
      "category": "Ll",
      "key": "00F2",
      "mappings": {"default": {"default": "⠈⠕"}}
    }, {"category": "Ll", "key": "00F3", "mappings": {"default": {"default": "⠈⠕"}}}, {
      "category": "Ll",
      "key": "00F4",
      "mappings": {"default": {"default": "⠈⠕"}}
    }, {"category": "Ll", "key": "00F5", "mappings": {"default": {"default": "⠈⠕"}}}, {
      "category": "Ll",
      "key": "00F6",
      "mappings": {"default": {"default": "⠈⠕"}}
    }, {"category": "Ll", "key": "00F9", "mappings": {"default": {"default": "⠈⠥"}}}, {
      "category": "Ll",
      "key": "00FA",
      "mappings": {"default": {"default": "⠈⠥"}}
    }, {"category": "Ll", "key": "00FB", "mappings": {"default": {"default": "⠈⠥"}}}, {
      "category": "Ll",
      "key": "00FC",
      "mappings": {"default": {"default": "⠈⠥"}}
    }, {"category": "Ll", "key": "00FD", "mappings": {"default": {"default": "⠈⠽"}}}, {
      "category": "Ll",
      "key": "00FF",
      "mappings": {"default": {"default": "⠈⠽"}}
    }, {"category": "Ll", "key": "0101", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠂⠄"}}}, {
      "category": "Ll",
      "key": "0103",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠒⠄"}}
    }, {"category": "Ll", "key": "0105", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠢⠄"}}}, {
      "category": "Ll",
      "key": "0107",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠶⠄"}}
    }, {"category": "Ll", "key": "0109", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠔⠄"}}}, {
      "category": "Ll",
      "key": "010B",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠃⠄"}}
    }, {"category": "Ll", "key": "010D", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠙⠄"}}}, {
      "category": "Ll",
      "key": "010F",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠋⠄"}}
    }, {"category": "Ll", "key": "0113", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠒⠄"}}}, {
      "category": "Ll",
      "key": "0115",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠢⠄"}}
    }, {"category": "Ll", "key": "0117", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠶⠄"}}}, {
      "category": "Ll",
      "key": "0119",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠔⠄"}}
    }, {"category": "Ll", "key": "011B", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠃⠄"}}}, {
      "category": "Ll",
      "key": "011D",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠙⠄"}}
    }, {"category": "Ll", "key": "011F", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠋⠄"}}}, {
      "category": "Ll",
      "key": "0121",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠂⠄"}}
    }, {"category": "Ll", "key": "0123", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠒⠄"}}}, {
      "category": "Ll",
      "key": "0125",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠢⠄"}}
    }, {"category": "Ll", "key": "0129", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠔⠄"}}}, {
      "category": "Ll",
      "key": "012B",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠃⠄"}}
    }, {"category": "Ll", "key": "012D", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠙⠄"}}}, {
      "category": "Ll",
      "key": "012F",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠋⠄"}}
    }, {"category": "Ll", "key": "0131", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠂⠄"}}}, {
      "category": "Ll",
      "key": "0135",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠢⠄"}}
    }, {"category": "Ll", "key": "0137", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠶⠄"}}}, {
      "category": "Ll",
      "key": "013A",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠁⠄"}}
    }, {"category": "Ll", "key": "013C", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠉⠄"}}}, {
      "category": "Ll",
      "key": "013E",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠑⠄"}}
    }, {"category": "Ll", "key": "0140", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠴⠄"}}}, {
      "category": "Ll",
      "key": "0144",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠲⠄"}}
    }, {"category": "Ll", "key": "0146", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠖⠄"}}}, {
      "category": "Ll",
      "key": "0148",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠦⠄"}}
    }, {"category": "Ll", "key": "014D", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠙⠄"}}}, {
      "category": "Ll",
      "key": "014F",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠋⠄"}}
    }, {"category": "Ll", "key": "0151", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠂⠄"}}}, {
      "category": "Ll",
      "key": "0155",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠢⠄"}}
    }, {"category": "Ll", "key": "0157", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠶⠄"}}}, {
      "category": "Ll",
      "key": "0159",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠔⠄"}}
    }, {"category": "Ll", "key": "015B", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠃⠄"}}}, {
      "category": "Ll",
      "key": "015D",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠙⠄"}}
    }, {"category": "Ll", "key": "015F", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠋⠄"}}}, {
      "category": "Ll",
      "key": "0161",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠂⠄"}}
    }, {"category": "Ll", "key": "0163", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠒⠄"}}}, {
      "category": "Ll",
      "key": "0165",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠢⠄"}}
    }, {"category": "Ll", "key": "0169", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠔⠄"}}}, {
      "category": "Ll",
      "key": "016B",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠃⠄"}}
    }, {"category": "Ll", "key": "016D", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠙⠄"}}}, {
      "category": "Ll",
      "key": "016F",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠋⠄"}}
    }, {"category": "Ll", "key": "0171", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠂⠄"}}}, {
      "category": "Ll",
      "key": "0173",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠒⠄"}}
    }, {"category": "Ll", "key": "0175", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠢⠄"}}}, {
      "category": "Ll",
      "key": "0177",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠶⠄"}}
    }, {"category": "Ll", "key": "017A", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠁⠄"}}}, {
      "category": "Ll",
      "key": "017C",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠉⠄"}}
    }, {"category": "Ll", "key": "017E", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠑⠄"}}}, {
      "category": "Ll",
      "key": "01CE",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠉⠑⠄"}}
    }, {"category": "Ll", "key": "01D0", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠴⠄"}}}, {
      "category": "Ll",
      "key": "01D2",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠆⠄"}}
    }, {"category": "Ll", "key": "01D4", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠲⠄"}}}, {
      "category": "Ll",
      "key": "01E7",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠶⠄"}}
    }, {"category": "Ll", "key": "01E9", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠔⠄"}}}, {
      "category": "Ll",
      "key": "01EB",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠃⠄"}}
    }, {"category": "Ll", "key": "01F0", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠋⠴⠄"}}}, {
      "category": "Ll",
      "key": "01F5",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠋⠢⠄"}}
    }, {"category": "Ll", "key": "01F9", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠋⠔⠄"}}}, {
      "category": "Ll",
      "key": "0201",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠂⠄"}}
    }, {"category": "Ll", "key": "0203", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠒⠄"}}}, {
      "category": "Ll",
      "key": "0205",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠢⠄"}}
    }, {"category": "Ll", "key": "0207", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠶⠄"}}}, {
      "category": "Ll",
      "key": "0209",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠔⠄"}}
    }, {"category": "Ll", "key": "020B", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠃⠄"}}}, {
      "category": "Ll",
      "key": "020D",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠙⠄"}}
    }, {"category": "Ll", "key": "020F", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠋⠄"}}}, {
      "category": "Ll",
      "key": "0211",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠂⠄"}}
    }, {"category": "Ll", "key": "0213", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠒⠄"}}}, {
      "category": "Ll",
      "key": "0215",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠢⠄"}}
    }, {"category": "Ll", "key": "0217", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠶⠄"}}}, {
      "category": "Ll",
      "key": "0219",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠔⠄"}}
    }, {"category": "Ll", "key": "021B", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠃⠄"}}}, {
      "category": "Ll",
      "key": "021F",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠋⠄"}}
    }, {"category": "Ll", "key": "0227", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠶⠄"}}}, {
      "category": "Ll",
      "key": "0229",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠔⠄"}}
    }, {"category": "Ll", "key": "022F", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠋⠄"}}}, {
      "category": "Ll",
      "key": "0233",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠒⠒⠄"}}
    }, {"category": "Ll", "key": "0237", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠒⠶⠄"}}}, {
      "category": "Ll",
      "key": "1E01",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠂⠄"}}
    }, {"category": "Ll", "key": "1E03", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠒⠄"}}}, {
      "category": "Ll",
      "key": "1E05",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠢⠄"}}
    }, {"category": "Ll", "key": "1E07", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠶⠄"}}}, {
      "category": "Ll",
      "key": "1E0B",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠃⠄"}}
    }, {"category": "Ll", "key": "1E0D", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠙⠄"}}}, {
      "category": "Ll",
      "key": "1E0F",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠋⠄"}}
    }, {"category": "Ll", "key": "1E11", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠂⠄"}}}, {
      "category": "Ll",
      "key": "1E13",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠒⠄"}}
    }, {"category": "Ll", "key": "1E19", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠔⠄"}}}, {
      "category": "Ll",
      "key": "1E1B",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠃⠄"}}
    }, {"category": "Ll", "key": "1E1F", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠋⠄"}}}, {
      "category": "Ll",
      "key": "1E21",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠂⠄"}}
    }, {"category": "Ll", "key": "1E23", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠒⠄"}}}, {
      "category": "Ll",
      "key": "1E25",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠢⠄"}}
    }, {"category": "Ll", "key": "1E27", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠶⠄"}}}, {
      "category": "Ll",
      "key": "1E29",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠔⠄"}}
    }, {"category": "Ll", "key": "1E2B", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠃⠄"}}}, {
      "category": "Ll",
      "key": "1E2D",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠙⠄"}}
    }, {"category": "Ll", "key": "1E31", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠂⠄"}}}, {
      "category": "Ll",
      "key": "1E33",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠒⠄"}}
    }, {"category": "Ll", "key": "1E35", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠢⠄"}}}, {
      "category": "Ll",
      "key": "1E37",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠶⠄"}}
    }, {"category": "Ll", "key": "1E3B", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠃⠄"}}}, {
      "category": "Ll",
      "key": "1E3D",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠙⠄"}}
    }, {"category": "Ll", "key": "1E3F", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠋⠄"}}}, {
      "category": "Ll",
      "key": "1E41",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠂⠄"}}
    }, {"category": "Ll", "key": "1E43", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠒⠄"}}}, {
      "category": "Ll",
      "key": "1E45",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠢⠄"}}
    }, {"category": "Ll", "key": "1E47", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠶⠄"}}}, {
      "category": "Ll",
      "key": "1E49",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠔⠄"}}
    }, {"category": "Ll", "key": "1E4B", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠃⠄"}}}, {
      "category": "Ll",
      "key": "1E55",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠢⠄"}}
    }, {"category": "Ll", "key": "1E57", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠶⠄"}}}, {
      "category": "Ll",
      "key": "1E59",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠔⠄"}}
    }, {"category": "Ll", "key": "1E5B", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠃⠄"}}}, {
      "category": "Ll",
      "key": "1E5F",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠋⠄"}}
    }, {"category": "Ll", "key": "1E61", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠂⠄"}}}, {
      "category": "Ll",
      "key": "1E63",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠒⠄"}}
    }, {"category": "Ll", "key": "1E6B", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠃⠄"}}}, {
      "category": "Ll",
      "key": "1E6D",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠙⠄"}}
    }, {"category": "Ll", "key": "1E6F", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠋⠄"}}}, {
      "category": "Ll",
      "key": "1E71",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠂⠄"}}
    }, {"category": "Ll", "key": "1E73", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠒⠄"}}}, {
      "category": "Ll",
      "key": "1E75",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠢⠄"}}
    }, {"category": "Ll", "key": "1E77", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠶⠄"}}}, {
      "category": "Ll",
      "key": "1E7D",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠙⠄"}}
    }, {"category": "Ll", "key": "1E7F", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠋⠄"}}}, {
      "category": "Ll",
      "key": "1E81",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠂⠄"}}
    }, {"category": "Ll", "key": "1E83", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠒⠄"}}}, {
      "category": "Ll",
      "key": "1E85",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠢⠄"}}
    }, {"category": "Ll", "key": "1E87", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠶⠄"}}}, {
      "category": "Ll",
      "key": "1E89",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠔⠄"}}
    }, {"category": "Ll", "key": "1E8B", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠃⠄"}}}, {
      "category": "Ll",
      "key": "1E8D",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠙⠄"}}
    }, {"category": "Ll", "key": "1E8F", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠋⠄"}}}, {
      "category": "Ll",
      "key": "1E91",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠂⠄"}}
    }, {"category": "Ll", "key": "1E93", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠒⠄"}}}, {
      "category": "Ll",
      "key": "1E95",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠢⠄"}}
    }, {"category": "Ll", "key": "1E96", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠖⠄"}}}, {
      "category": "Ll",
      "key": "1E97",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠶⠄"}}
    }, {"category": "Ll", "key": "1E98", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠦⠄"}}}, {
      "category": "Ll",
      "key": "1E99",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠔⠄"}}
    }, {"category": "Ll", "key": "1E9A", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠁⠄"}}}, {
      "category": "Ll",
      "key": "1EA1",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠂⠄"}}
    }, {"category": "Ll", "key": "1EA3", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠒⠄"}}}, {
      "category": "Ll",
      "key": "1EB9",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠔⠄"}}
    }, {"category": "Ll", "key": "1EBB", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠃⠄"}}}, {
      "category": "Ll",
      "key": "1EBD",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠙⠄"}}
    }, {"category": "Ll", "key": "1EC9", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠔⠄"}}}, {
      "category": "Ll",
      "key": "1ECB",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠃⠄"}}
    }, {"category": "Ll", "key": "1ECD", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠙⠄"}}}, {
      "category": "Ll",
      "key": "1ECF",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠋⠄"}}
    }, {"category": "Ll", "key": "1EE5", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠢⠄"}}}, {
      "category": "Ll",
      "key": "1EE7",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠶⠄"}}
    }, {"category": "Ll", "key": "1EF3", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠋⠒⠄"}}}, {
      "category": "Ll",
      "key": "1EF5",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠋⠢⠄"}}
    }, {"category": "Ll", "key": "1EF7", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠋⠶⠄"}}}, {
      "category": "Ll",
      "key": "1EF9",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠋⠔⠄"}}
    }]
  ,
  "nemeth/symbols/latin-rest.js":
    [{"locale": "nemeth"}, {
      "category": "Mn",
      "key": "0363",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠒⠄"}}
    }, {"category": "Mn", "key": "0364", "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠲⠄"}}}, {
      "category": "Mn",
      "key": "0365",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠢⠄"}}
    }, {"category": "Mn", "key": "0366", "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠖⠄"}}}, {
      "category": "Mn",
      "key": "0367",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠶⠄"}}
    }, {"category": "Mn", "key": "0368", "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠦⠄"}}}, {
      "category": "Mn",
      "key": "0369",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠔⠄"}}
    }, {"category": "Mn", "key": "036A", "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠁⠄"}}}, {
      "category": "Mn",
      "key": "036B",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠃⠄"}}
    }, {"category": "Mn", "key": "036C", "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠉⠄"}}}, {
      "category": "Mn",
      "key": "036D",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠙⠄"}}
    }, {"category": "Mn", "key": "036E", "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠑⠄"}}}, {
      "category": "Mn",
      "key": "036F",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠒⠖⠋⠄"}}
    }, {"category": "Lm", "key": "1D62", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠆⠄"}}}, {
      "category": "Lm",
      "key": "1D63",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠒⠄"}}
    }, {"category": "Lm", "key": "1D64", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠲⠄"}}}, {
      "category": "Lm",
      "key": "1D65",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠖⠢⠄"}}
    }, {"category": "Mn", "key": "1DCA", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠉⠁⠄"}}}, {
      "category": "Mn",
      "key": "1DD3",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠒⠄"}}
    }, {"category": "Mn", "key": "1DD4", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠲⠄"}}}, {
      "category": "Mn",
      "key": "1DD5",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠢⠄"}}
    }, {"category": "Mn", "key": "1DD6", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠖⠄"}}}, {
      "category": "Mn",
      "key": "1DD7",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠶⠄"}}
    }, {"category": "Mn", "key": "1DD8", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠦⠄"}}}, {
      "category": "Mn",
      "key": "1DD9",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠔⠄"}}
    }, {"category": "Mn", "key": "1DDA", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠁⠄"}}}, {
      "category": "Mn",
      "key": "1DDB",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠃⠄"}}
    }, {"category": "Mn", "key": "1DDC", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠉⠄"}}}, {
      "category": "Mn",
      "key": "1DDD",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠙⠄"}}
    }, {"category": "Mn", "key": "1DDE", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠑⠄"}}}, {
      "category": "Mn",
      "key": "1DDF",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠙⠋⠄"}}
    }, {"category": "Mn", "key": "1DE0", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠑⠴⠄"}}}, {
      "category": "Mn",
      "key": "1DE1",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠑⠂⠄"}}
    }, {"category": "Mn", "key": "1DE2", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠑⠆⠄"}}}, {
      "category": "Mn",
      "key": "1DE3",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠑⠒⠄"}}
    }, {"category": "Mn", "key": "1DE4", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠑⠲⠄"}}}, {
      "category": "Mn",
      "key": "1DE5",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠑⠢⠄"}}
    }, {"category": "Mn", "key": "1DE6", "mappings": {"default": {"default": "⠄⡳⠭⠂⠙⠑⠖⠄"}}}, {
      "category": "Lm",
      "key": "2071",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠶⠂⠄"}}
    }, {"category": "Lm", "key": "207F", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠶⠋⠄"}}}, {
      "category": "Lm",
      "key": "2090",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠴⠄"}}
    }, {"category": "Lm", "key": "2091", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠂⠄"}}}, {
      "category": "Lm",
      "key": "2092",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠆⠄"}}
    }, {"category": "Lm", "key": "2093", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠒⠄"}}}, {
      "category": "Lm",
      "key": "2094",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠲⠄"}}
    }, {"category": "Lm", "key": "2095", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠢⠄"}}}, {
      "category": "Lm",
      "key": "2096",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠖⠄"}}
    }, {"category": "Lm", "key": "2097", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠶⠄"}}}, {
      "category": "Lm",
      "key": "2098",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠦⠄"}}
    }, {"category": "Lm", "key": "2099", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠔⠄"}}}, {
      "category": "Lm",
      "key": "209A",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠁⠄"}}
    }, {"category": "Lm", "key": "209B", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠃⠄"}}}, {
      "category": "Lm",
      "key": "209C",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠔⠉⠄"}}
    }, {"category": "Lm", "key": "2C7C", "mappings": {"default": {"default": "⠄⡳⠭⠆⠉⠶⠉⠄"}}}, {
      "category": "So",
      "key": "1F12A",
      "mappings": {"default": {"default": "⠄⡳⠽⠂⠋⠂⠆⠁⠄"}}
    }, {"category": "So", "key": "1F12B", "mappings": {"default": {"default": "⠄⡳⠽⠂⠋⠂⠆⠃⠄"}}}, {
      "category": "So",
      "key": "1F12C",
      "mappings": {"default": {"default": "⠄⡳⠽⠂⠋⠂⠆⠉⠄"}}
    }, {"category": "So", "key": "1F18A", "mappings": {"default": {"default": "⠄⡳⠽⠂⠋⠂⠦⠁⠄"}}}]
  ,
  "nemeth/symbols/latin-upper-double-accent.js":
    [{"locale": "nemeth"}, {
      "category": "Lu",
      "key": "01D5",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠢⠄"}}
    }, {"category": "Lu", "key": "01D7", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠶⠄"}}}, {
      "category": "Lu",
      "key": "01D9",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠔⠄"}}
    }, {"category": "Lu", "key": "01DB", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠃⠄"}}}, {
      "category": "Lu",
      "key": "01DE",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠑⠄"}}
    }, {"category": "Lu", "key": "01E0", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠴⠄"}}}, {
      "category": "Lu",
      "key": "01EC",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠉⠄"}}
    }, {"category": "Lu", "key": "01FA", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠋⠁⠄"}}}, {
      "category": "Lu",
      "key": "022A",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠁⠄"}}
    }, {"category": "Lu", "key": "022C", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠉⠄"}}}, {
      "category": "Lu",
      "key": "0230",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠒⠴⠄"}}
    }, {"category": "Lu", "key": "1E08", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠦⠄"}}}, {
      "category": "Lu",
      "key": "1E14",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠲⠄"}}
    }, {"category": "Lu", "key": "1E16", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠖⠄"}}}, {
      "category": "Lu",
      "key": "1E1C",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠉⠄"}}
    }, {"category": "Lu", "key": "1E2E", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠑⠄"}}}, {
      "category": "Lu",
      "key": "1E38",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠦⠄"}}
    }, {"category": "Lu", "key": "1E4C", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠉⠄"}}}, {
      "category": "Lu",
      "key": "1E4E",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠑⠄"}}
    }, {"category": "Lu", "key": "1E50", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠴⠄"}}}, {
      "category": "Lu",
      "key": "1E52",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠆⠄"}}
    }, {"category": "Lu", "key": "1E5C", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠉⠄"}}}, {
      "category": "Lu",
      "key": "1E64",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠲⠄"}}
    }, {"category": "Lu", "key": "1E66", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠖⠄"}}}, {
      "category": "Lu",
      "key": "1E68",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠦⠄"}}
    }, {"category": "Lu", "key": "1E78", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠦⠄"}}}, {
      "category": "Lu",
      "key": "1E7A",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠁⠄"}}
    }, {"category": "Lu", "key": "1EA4", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠲⠄"}}}, {
      "category": "Lu",
      "key": "1EA6",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠖⠄"}}
    }, {"category": "Lu", "key": "1EA8", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠦⠄"}}}, {
      "category": "Lu",
      "key": "1EAA",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠁⠄"}}
    }, {"category": "Lu", "key": "1EAC", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠉⠄"}}}, {
      "category": "Lu",
      "key": "1EAE",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠑⠄"}}
    }, {"category": "Lu", "key": "1EB0", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠴⠄"}}}, {
      "category": "Lu",
      "key": "1EB2",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠆⠄"}}
    }, {"category": "Lu", "key": "1EB4", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠲⠄"}}}, {
      "category": "Lu",
      "key": "1EB6",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠖⠄"}}
    }, {"category": "Lu", "key": "1EBE", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠑⠄"}}}, {
      "category": "Lu",
      "key": "1EC0",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠴⠄"}}
    }, {"category": "Lu", "key": "1EC2", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠆⠄"}}}, {
      "category": "Lu",
      "key": "1EC4",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠲⠄"}}
    }, {"category": "Lu", "key": "1EC6", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠖⠄"}}}, {
      "category": "Lu",
      "key": "1ED0",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠴⠄"}}
    }, {"category": "Lu", "key": "1ED2", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠆⠄"}}}, {
      "category": "Lu",
      "key": "1ED4",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠲⠄"}}
    }, {"category": "Lu", "key": "1ED6", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠖⠄"}}}, {
      "category": "Lu",
      "key": "1ED8",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠦⠄"}}
    }, {"category": "Lu", "key": "1EDA", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠁⠄"}}}, {
      "category": "Lu",
      "key": "1EDC",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠉⠄"}}
    }, {"category": "Lu", "key": "1EDE", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠙⠑⠄"}}}, {
      "category": "Lu",
      "key": "1EE0",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠴⠄"}}
    }, {"category": "Lu", "key": "1EE2", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠆⠄"}}}, {
      "category": "Lu",
      "key": "1EE8",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠦⠄"}}
    }, {"category": "Lu", "key": "1EEA", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠁⠄"}}}, {
      "category": "Lu",
      "key": "1EEC",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠉⠄"}}
    }, {"category": "Lu", "key": "1EEE", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠑⠄"}}}, {
      "category": "Lu",
      "key": "1EF0",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠋⠴⠄"}}
    }]
  ,
  "nemeth/symbols/latin-upper-single-accent.js":
    [{"locale": "nemeth"}, {
      "category": "Lu",
      "key": "00C0",
      "mappings": {"default": {"default": "⠠⠈⠁"}}
    }, {"category": "Lu", "key": "00C1", "mappings": {"default": {"default": "⠠⠈⠁"}}}, {
      "category": "Lu",
      "key": "00C2",
      "mappings": {"default": {"default": "⠠⠈⠁"}}
    }, {"category": "Lu", "key": "00C3", "mappings": {"default": {"default": "⠠⠈⠁"}}}, {
      "category": "Lu",
      "key": "00C4",
      "mappings": {"default": {"default": "⠠⠈⠁"}}
    }, {"category": "Lu", "key": "00C5", "mappings": {"default": {"default": "⠠⠈⠁"}}}, {
      "category": "Lu",
      "key": "00C7",
      "mappings": {"default": {"default": "⠠⠈⠉"}}
    }, {"category": "Lu", "key": "00C8", "mappings": {"default": {"default": "⠠⠈⠑"}}}, {
      "category": "Lu",
      "key": "00C9",
      "mappings": {"default": {"default": "⠠⠈⠑"}}
    }, {"category": "Lu", "key": "00CA", "mappings": {"default": {"default": "⠠⠈⠑"}}}, {
      "category": "Lu",
      "key": "00CB",
      "mappings": {"default": {"default": "⠠⠈⠑"}}
    }, {"category": "Lu", "key": "00CC", "mappings": {"default": {"default": "⠠⠈⠊"}}}, {
      "category": "Lu",
      "key": "00CD",
      "mappings": {"default": {"default": "⠠⠈⠊"}}
    }, {"category": "Lu", "key": "00CE", "mappings": {"default": {"default": "⠠⠈⠊"}}}, {
      "category": "Lu",
      "key": "00CF",
      "mappings": {"default": {"default": "⠠⠈⠊"}}
    }, {"category": "Lu", "key": "00D1", "mappings": {"default": {"default": "⠠⠈⠝"}}}, {
      "category": "Lu",
      "key": "00D2",
      "mappings": {"default": {"default": "⠠⠈⠕"}}
    }, {"category": "Lu", "key": "00D3", "mappings": {"default": {"default": "⠠⠈⠕"}}}, {
      "category": "Lu",
      "key": "00D4",
      "mappings": {"default": {"default": "⠠⠈⠕"}}
    }, {"category": "Lu", "key": "00D5", "mappings": {"default": {"default": "⠄⡳⠭⠴⠴⠙⠢⠄"}}}, {
      "category": "Lu",
      "key": "00D6",
      "mappings": {"default": {"default": "⠠⠈⠕"}}
    }, {"category": "Lu", "key": "00D9", "mappings": {"default": {"default": "⠠⠈⠥"}}}, {
      "category": "Lu",
      "key": "00DA",
      "mappings": {"default": {"default": "⠠⠈⠥"}}
    }, {"category": "Lu", "key": "00DB", "mappings": {"default": {"default": "⠠⠈⠥"}}}, {
      "category": "Lu",
      "key": "00DC",
      "mappings": {"default": {"default": "⠠⠈⠥"}}
    }, {"category": "Lu", "key": "00DD", "mappings": {"default": {"default": "⠠⠈⠽"}}}, {
      "category": "Lu",
      "key": "0100",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠴⠄"}}
    }, {"category": "Lu", "key": "0102", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠆⠄"}}}, {
      "category": "Lu",
      "key": "0104",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠲⠄"}}
    }, {"category": "Lu", "key": "0106", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠖⠄"}}}, {
      "category": "Lu",
      "key": "0108",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠦⠄"}}
    }, {"category": "Lu", "key": "010A", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠁⠄"}}}, {
      "category": "Lu",
      "key": "010C",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠉⠄"}}
    }, {"category": "Lu", "key": "010E", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠴⠑⠄"}}}, {
      "category": "Lu",
      "key": "0112",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠆⠄"}}
    }, {"category": "Lu", "key": "0114", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠲⠄"}}}, {
      "category": "Lu",
      "key": "0116",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠖⠄"}}
    }, {"category": "Lu", "key": "0118", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠦⠄"}}}, {
      "category": "Lu",
      "key": "011A",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠁⠄"}}
    }, {"category": "Lu", "key": "011C", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠉⠄"}}}, {
      "category": "Lu",
      "key": "011E",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠂⠑⠄"}}
    }, {"category": "Lu", "key": "0120", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠴⠄"}}}, {
      "category": "Lu",
      "key": "0122",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠆⠄"}}
    }, {"category": "Lu", "key": "0124", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠲⠄"}}}, {
      "category": "Lu",
      "key": "0128",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠦⠄"}}
    }, {"category": "Lu", "key": "012A", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠁⠄"}}}, {
      "category": "Lu",
      "key": "012C",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠉⠄"}}
    }, {"category": "Lu", "key": "012E", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠆⠑⠄"}}}, {
      "category": "Lu",
      "key": "0130",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠴⠄"}}
    }, {"category": "Lu", "key": "0134", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠲⠄"}}}, {
      "category": "Lu",
      "key": "0136",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠖⠄"}}
    }, {"category": "Lu", "key": "0139", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠔⠄"}}}, {
      "category": "Lu",
      "key": "013B",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠃⠄"}}
    }, {"category": "Lu", "key": "013D", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠙⠄"}}}, {
      "category": "Lu",
      "key": "013F",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠒⠋⠄"}}
    }, {"category": "Lu", "key": "0143", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠒⠄"}}}, {
      "category": "Lu",
      "key": "0145",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠢⠄"}}
    }, {"category": "Lu", "key": "0147", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠶⠄"}}}, {
      "category": "Lu",
      "key": "014C",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠉⠄"}}
    }, {"category": "Lu", "key": "014E", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠲⠑⠄"}}}, {
      "category": "Lu",
      "key": "0150",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠴⠄"}}
    }, {"category": "Lu", "key": "0154", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠲⠄"}}}, {
      "category": "Lu",
      "key": "0156",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠖⠄"}}
    }, {"category": "Lu", "key": "0158", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠦⠄"}}}, {
      "category": "Lu",
      "key": "015A",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠁⠄"}}
    }, {"category": "Lu", "key": "015C", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠉⠄"}}}, {
      "category": "Lu",
      "key": "015E",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠢⠑⠄"}}
    }, {"category": "Lu", "key": "0160", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠴⠄"}}}, {
      "category": "Lu",
      "key": "0162",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠆⠄"}}
    }, {"category": "Lu", "key": "0164", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠲⠄"}}}, {
      "category": "Lu",
      "key": "0168",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠦⠄"}}
    }, {"category": "Lu", "key": "016A", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠁⠄"}}}, {
      "category": "Lu",
      "key": "016C",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠉⠄"}}
    }, {"category": "Lu", "key": "016E", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠖⠑⠄"}}}, {
      "category": "Lu",
      "key": "0170",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠴⠄"}}
    }, {"category": "Lu", "key": "0172", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠆⠄"}}}, {
      "category": "Lu",
      "key": "0174",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠲⠄"}}
    }, {"category": "Lu", "key": "0176", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠖⠄"}}}, {
      "category": "Lu",
      "key": "0178",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠦⠄"}}
    }, {"category": "Lu", "key": "0179", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠔⠄"}}}, {
      "category": "Lu",
      "key": "017B",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠃⠄"}}
    }, {"category": "Lu", "key": "017D", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠶⠙⠄"}}}, {
      "category": "Lu",
      "key": "01CD",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠉⠙⠄"}}
    }, {"category": "Lu", "key": "01CF", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠉⠋⠄"}}}, {
      "category": "Lu",
      "key": "01D1",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠂⠄"}}
    }, {"category": "Lu", "key": "01D3", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠙⠒⠄"}}}, {
      "category": "Lu",
      "key": "01E6",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠖⠄"}}
    }, {"category": "Lu", "key": "01E8", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠦⠄"}}}, {
      "category": "Lu",
      "key": "01EA",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠑⠁⠄"}}
    }, {"category": "Lu", "key": "01F4", "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠋⠲⠄"}}}, {
      "category": "Lu",
      "key": "01F8",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠂⠋⠦⠄"}}
    }, {"category": "Lu", "key": "0200", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠴⠄"}}}, {
      "category": "Lu",
      "key": "0202",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠆⠄"}}
    }, {"category": "Lu", "key": "0204", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠲⠄"}}}, {
      "category": "Lu",
      "key": "0206",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠖⠄"}}
    }, {"category": "Lu", "key": "0208", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠦⠄"}}}, {
      "category": "Lu",
      "key": "020A",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠁⠄"}}
    }, {"category": "Lu", "key": "020C", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠉⠄"}}}, {
      "category": "Lu",
      "key": "020E",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠴⠑⠄"}}
    }, {"category": "Lu", "key": "0210", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠴⠄"}}}, {
      "category": "Lu",
      "key": "0212",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠆⠄"}}
    }, {"category": "Lu", "key": "0214", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠲⠄"}}}, {
      "category": "Lu",
      "key": "0216",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠖⠄"}}
    }, {"category": "Lu", "key": "0218", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠦⠄"}}}, {
      "category": "Lu",
      "key": "021A",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠁⠄"}}
    }, {"category": "Lu", "key": "021E", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠂⠑⠄"}}}, {
      "category": "Lu",
      "key": "0226",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠖⠄"}}
    }, {"category": "Lu", "key": "0228", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠦⠄"}}}, {
      "category": "Lu",
      "key": "022E",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠆⠑⠄"}}
    }, {"category": "Lu", "key": "0232", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠒⠆⠄"}}}, {
      "category": "Lu",
      "key": "1E00",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠴⠄"}}
    }, {"category": "Lu", "key": "1E02", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠆⠄"}}}, {
      "category": "Lu",
      "key": "1E04",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠲⠄"}}
    }, {"category": "Lu", "key": "1E06", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠖⠄"}}}, {
      "category": "Lu",
      "key": "1E0A",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠁⠄"}}
    }, {"category": "Lu", "key": "1E0C", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠉⠄"}}}, {
      "category": "Lu",
      "key": "1E0E",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠴⠑⠄"}}
    }, {"category": "Lu", "key": "1E10", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠴⠄"}}}, {
      "category": "Lu",
      "key": "1E12",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠆⠄"}}
    }, {"category": "Lu", "key": "1E18", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠦⠄"}}}, {
      "category": "Lu",
      "key": "1E1A",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠁⠄"}}
    }, {"category": "Lu", "key": "1E1E", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠂⠑⠄"}}}, {
      "category": "Lu",
      "key": "1E20",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠴⠄"}}
    }, {"category": "Lu", "key": "1E22", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠆⠄"}}}, {
      "category": "Lu",
      "key": "1E24",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠲⠄"}}
    }, {"category": "Lu", "key": "1E26", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠖⠄"}}}, {
      "category": "Lu",
      "key": "1E28",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠦⠄"}}
    }, {"category": "Lu", "key": "1E2A", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠁⠄"}}}, {
      "category": "Lu",
      "key": "1E2C",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠆⠉⠄"}}
    }, {"category": "Lu", "key": "1E30", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠴⠄"}}}, {
      "category": "Lu",
      "key": "1E32",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠆⠄"}}
    }, {"category": "Lu", "key": "1E34", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠲⠄"}}}, {
      "category": "Lu",
      "key": "1E36",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠖⠄"}}
    }, {"category": "Lu", "key": "1E3A", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠁⠄"}}}, {
      "category": "Lu",
      "key": "1E3C",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠉⠄"}}
    }, {"category": "Lu", "key": "1E3E", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠒⠑⠄"}}}, {
      "category": "Lu",
      "key": "1E40",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠴⠄"}}
    }, {"category": "Lu", "key": "1E42", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠆⠄"}}}, {
      "category": "Lu",
      "key": "1E44",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠲⠄"}}
    }, {"category": "Lu", "key": "1E46", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠖⠄"}}}, {
      "category": "Lu",
      "key": "1E48",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠦⠄"}}
    }, {"category": "Lu", "key": "1E4A", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠲⠁⠄"}}}, {
      "category": "Lu",
      "key": "1E54",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠲⠄"}}
    }, {"category": "Lu", "key": "1E56", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠖⠄"}}}, {
      "category": "Lu",
      "key": "1E58",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠦⠄"}}
    }, {"category": "Lu", "key": "1E5A", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠁⠄"}}}, {
      "category": "Lu",
      "key": "1E5E",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠢⠑⠄"}}
    }, {"category": "Lu", "key": "1E60", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠴⠄"}}}, {
      "category": "Lu",
      "key": "1E62",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠆⠄"}}
    }, {"category": "Lu", "key": "1E6A", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠁⠄"}}}, {
      "category": "Lu",
      "key": "1E6C",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠉⠄"}}
    }, {"category": "Lu", "key": "1E6E", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠖⠑⠄"}}}, {
      "category": "Lu",
      "key": "1E70",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠴⠄"}}
    }, {"category": "Lu", "key": "1E72", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠆⠄"}}}, {
      "category": "Lu",
      "key": "1E74",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠲⠄"}}
    }, {"category": "Lu", "key": "1E76", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠖⠄"}}}, {
      "category": "Lu",
      "key": "1E7C",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠉⠄"}}
    }, {"category": "Lu", "key": "1E7E", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠶⠑⠄"}}}, {
      "category": "Lu",
      "key": "1E80",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠴⠄"}}
    }, {"category": "Lu", "key": "1E82", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠆⠄"}}}, {
      "category": "Lu",
      "key": "1E84",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠲⠄"}}
    }, {"category": "Lu", "key": "1E86", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠖⠄"}}}, {
      "category": "Lu",
      "key": "1E88",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠦⠄"}}
    }, {"category": "Lu", "key": "1E8A", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠁⠄"}}}, {
      "category": "Lu",
      "key": "1E8C",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠉⠄"}}
    }, {"category": "Lu", "key": "1E8E", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠦⠑⠄"}}}, {
      "category": "Lu",
      "key": "1E90",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠴⠄"}}
    }, {"category": "Lu", "key": "1E92", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠆⠄"}}}, {
      "category": "Lu",
      "key": "1E94",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠔⠲⠄"}}
    }, {"category": "Lu", "key": "1EA0", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠴⠄"}}}, {
      "category": "Lu",
      "key": "1EA2",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠁⠆⠄"}}
    }, {"category": "Lu", "key": "1EB8", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠦⠄"}}}, {
      "category": "Lu",
      "key": "1EBA",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠁⠄"}}
    }, {"category": "Lu", "key": "1EBC", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠃⠉⠄"}}}, {
      "category": "Lu",
      "key": "1EC8",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠦⠄"}}
    }, {"category": "Lu", "key": "1ECA", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠁⠄"}}}, {
      "category": "Lu",
      "key": "1ECC",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠉⠄"}}
    }, {"category": "Lu", "key": "1ECE", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠉⠑⠄"}}}, {
      "category": "Lu",
      "key": "1EE4",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠲⠄"}}
    }, {"category": "Lu", "key": "1EE6", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠑⠖⠄"}}}, {
      "category": "Lu",
      "key": "1EF2",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠋⠆⠄"}}
    }, {"category": "Lu", "key": "1EF4", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠋⠲⠄"}}}, {
      "category": "Lu",
      "key": "1EF6",
      "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠋⠖⠄"}}
    }, {"category": "Lu", "key": "1EF8", "mappings": {"default": {"default": "⠄⡳⠭⠂⠑⠋⠦⠄"}}}]
  ,
  "nemeth/symbols/math_angles.js":
    [{"locale": "nemeth"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠑⠄"}},
      "key": "22BE"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠶⠉⠄"}}, "key": "237C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠴⠄"}},
      "key": "27C0"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠃⠄"}}, "key": "299B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠉⠄"}},
      "key": "299C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠙⠄"}}, "key": "299D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠑⠄"}},
      "key": "299E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠋⠄"}}, "key": "299F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠴⠄"}},
      "key": "29A0"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠂⠄"}}, "key": "29A1"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠆⠄"}},
      "key": "29A2"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠒⠄"}}, "key": "29A3"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠲⠄"}},
      "key": "29A4"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠢⠄"}}, "key": "29A5"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠖⠄"}},
      "key": "29A6"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠶⠄"}}, "key": "29A7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠦⠄"}},
      "key": "29A8"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠔⠄"}}, "key": "29A9"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠁⠄"}},
      "key": "29AA"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠃⠄"}}, "key": "29AB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠉⠄"}},
      "key": "29AC"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠙⠄"}}, "key": "29AD"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠑⠄"}},
      "key": "29AE"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠁⠋⠄"}}, "key": "29AF"}]
  ,
  "nemeth/symbols/math_arrows.js":
    [{"locale": "nemeth"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠫⠪⠒⠒⠀"}},
      "key": "2190"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠫⠣⠒⠒⠕⠀"}}, "key": "2191"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠫⠕⠀"}},
      "key": "2192"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠫⠩⠒⠒⠕⠀"}}, "key": "2193"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠫⠪⠒⠒⠕⠀"}},
      "key": "2194"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠣⠩⠪⠒⠒⠕⠀"}}, "key": "2195"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠘⠪⠒⠒⠀"}},
      "key": "2196"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠘⠒⠒⠕⠀"}}, "key": "2197"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠰⠒⠒⠕⠀"}},
      "key": "2198"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠰⠪⠒⠒⠀"}}, "key": "2199"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠳⠈⠫⠪⠒⠒⠻⠀"}},
      "key": "219A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠳⠈⠫⠒⠒⠕⠻⠀"}}, "key": "219B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠪⠔⠒⠢⠀"}},
      "key": "219C"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠔⠒⠢⠕⠀"}}, "key": "219D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠪⠪⠒⠒⠀"}},
      "key": "219E"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠣⠒⠒⠕⠕⠀"}}, "key": "219F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠫⠒⠒⠕⠕⠀"}},
      "key": "21A0"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠩⠒⠒⠕⠕⠀"}}, "key": "21A1"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠪⠒⠒⠠⠽⠀"}},
      "key": "21A2"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠫⠠⠯⠒⠒⠕⠀"}}, "key": "21A3"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠪⠒⠒⠳⠀"}},
      "key": "21A4"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠣⠳⠒⠒⠕⠀"}}, "key": "21A5"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠫⠳⠒⠒⠕⠀"}},
      "key": "21A6"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠩⠳⠒⠒⠕⠀"}}, "key": "21A7"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠪⠒⠳⠒⠕⠀"}},
      "key": "21A8"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠪⠒⠒⠠⠕⠀"}}, "key": "21A9"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠠⠪⠒⠒⠕⠀"}},
      "key": "21AA"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠪⠒⠒⠨⠡⠀"}}, "key": "21AB"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠨⠡⠒⠒⠕⠀"}},
      "key": "21AC"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠪⠔⠒⠢⠕⠀"}}, "key": "21AD"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠳⠈⠫⠪⠒⠒⠕⠀"}},
      "key": "21AE"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠩⠔⠢⠔⠀"}}, "key": "21AF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠃⠴⠄"}},
      "key": "21B0"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠃⠂⠄"}}, "key": "21B1"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠃⠆⠄"}},
      "key": "21B2"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠃⠒⠄"}}, "key": "21B3"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠠⠳⠒⠒⠕⠀"}},
      "key": "21B4"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠩⠠⠳⠒⠒⠕⠀"}}, "key": "21B5"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠢⠔⠀⠕⠀"}},
      "key": "21B6"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠪⠢⠀⠔⠀"}}, "key": "21B7"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠃⠦⠄"}},
      "key": "21B8"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠃⠔⠄"}}, "key": "21B9"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠢⠔⠕⠀"}},
      "key": "21BA"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠪⠢⠔⠀"}}, "key": "21BB"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠒⠒⠕⠫⠪⠒⠒⠀"}},
      "key": "21C4"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠣⠒⠒⠕⠐⠫⠩⠒⠒⠕⠀"}}, "key": "21C5"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠪⠒⠒⠫⠒⠒⠕⠀"}},
      "key": "21C6"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠚⠒⠒⠫⠚⠒⠒⠀"}}, "key": "21C7"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠣⠒⠒⠕⠐⠫⠣⠒⠒⠕⠀"}},
      "key": "21C8"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠒⠒⠕⠫⠒⠒⠕⠀"}}, "key": "21C9"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠩⠒⠒⠕⠐⠫⠩⠒⠒⠕⠀"}},
      "key": "21CA"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠳⠈⠫⠪⠪⠒⠒⠀"}}, "key": "21CD"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠳⠈⠫⠪⠪⠒⠒⠕⠕⠀"}},
      "key": "21CE"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠳⠈⠫⠒⠒⠕⠕⠀"}}, "key": "21CF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠪⠪⠒⠒⠀"}},
      "key": "21D0"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠣⠒⠒⠕⠕⠀"}}, "key": "21D1"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠫⠒⠒⠕⠕⠀"}},
      "key": "21D2"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠣⠒⠒⠕⠕⠀"}}, "key": "21D3"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠫⠪⠪⠒⠒⠕⠕⠀"}},
      "key": "21D4"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠣⠪⠪⠒⠒⠕⠕⠀"}}, "key": "21D5"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠘⠪⠪⠒⠒⠀"}},
      "key": "21D6"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠘⠒⠒⠕⠕⠀"}}, "key": "21D7"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠰⠒⠒⠕⠕⠀"}},
      "key": "21D8"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠰⠪⠪⠒⠒⠀"}}, "key": "21D9"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠪⠪⠪⠒⠒⠀"}},
      "key": "21DA"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠒⠒⠕⠕⠕⠀"}}, "key": "21DB"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠪⠢⠤⠔⠒⠢⠀"}},
      "key": "21DC"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠢⠤⠔⠒⠢⠕⠀"}}, "key": "21DD"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠳⠳⠈⠫⠣⠒⠒⠕⠻⠀"}},
      "key": "21DE"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠳⠳⠈⠫⠩⠒⠒⠕⠻⠀"}}, "key": "21DF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠪⠒⠀⠒⠀"}},
      "key": "21E0"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠣⠒⠀⠒⠕⠀"}}, "key": "21E1"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠒⠀⠒⠕⠀"}},
      "key": "21E2"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠩⠒⠀⠒⠕⠀"}}, "key": "21E3"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠳⠪⠒⠒⠀"}},
      "key": "21E4"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠒⠒⠕⠳⠀"}}, "key": "21E5"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠸⠪⠒⠒⠀"}},
      "key": "21E6"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠣⠸⠒⠒⠕⠀"}}, "key": "21E7"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠸⠒⠒⠕⠀"}},
      "key": "21E8"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠩⠸⠒⠒⠕⠀"}}, "key": "21E9"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠑⠁⠄"}},
      "key": "21EA"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠑⠃⠄"}}, "key": "21EB"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠑⠉⠄"}},
      "key": "21EC"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠑⠙⠄"}}, "key": "21ED"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠑⠑⠄"}},
      "key": "21EE"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠑⠋⠄"}}, "key": "21EF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠋⠴⠄"}},
      "key": "21F0"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠋⠂⠄"}}, "key": "21F1"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠋⠆⠄"}},
      "key": "21F2"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠣⠸⠪⠒⠒⠕⠀"}}, "key": "21F3"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠫⠒⠒⠕⠨⠡⠀"}},
      "key": "21F4"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠫⠩⠒⠒⠕⠐⠫⠣⠒⠒⠕⠀"}}, "key": "21F5"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠫⠒⠒⠕⠫⠒⠒⠕⠫⠒⠒⠕⠀"}},
      "key": "21F6"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠳⠈⠫⠪⠒⠒⠻⠀"}}, "key": "21F7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠳⠈⠫⠒⠒⠕⠻⠀"}},
      "key": "21F8"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠳⠈⠫⠪⠒⠒⠕⠀"}}, "key": "21F9"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠳⠳⠈⠫⠪⠒⠒⠻⠀"}},
      "key": "21FA"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠳⠳⠈⠫⠒⠒⠕⠻⠀"}}, "key": "21FB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠳⠳⠈⠫⠪⠒⠒⠕⠀"}},
      "key": "21FC"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠫⠳⠒⠒⠀"}}, "key": "21FD"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠫⠒⠒⠳⠀"}},
      "key": "21FE"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠫⠳⠒⠒⠳⠀"}}, "key": "21FF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠴⠂⠄"}},
      "key": "2301"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠴⠒⠄"}}, "key": "2303"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠴⠲⠄"}},
      "key": "2304"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠆⠲⠄"}}, "key": "2324"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠦⠃⠄"}},
      "key": "238B"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠔⠲⠄"}}, "key": "2794"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠔⠦⠄"}},
      "key": "2798"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠔⠔⠄"}}, "key": "2799"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠔⠁⠄"}},
      "key": "279A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠔⠃⠄"}}, "key": "279B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠔⠉⠄"}},
      "key": "279C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠔⠙⠄"}}, "key": "279D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠔⠑⠄"}},
      "key": "279E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠔⠋⠄"}}, "key": "279F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠴⠄"}},
      "key": "27A0"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠂⠄"}}, "key": "27A1"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠆⠄"}},
      "key": "27A2"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠒⠄"}}, "key": "27A3"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠲⠄"}},
      "key": "27A4"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠢⠄"}}, "key": "27A5"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠖⠄"}},
      "key": "27A6"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠶⠄"}}, "key": "27A7"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠦⠄"}},
      "key": "27A8"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠔⠄"}}, "key": "27A9"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠁⠄"}},
      "key": "27AA"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠃⠄"}}, "key": "27AB"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠉⠄"}},
      "key": "27AC"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠙⠄"}}, "key": "27AD"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠑⠄"}},
      "key": "27AE"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠁⠋⠄"}}, "key": "27AF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠂⠄"}},
      "key": "27B1"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠆⠄"}}, "key": "27B2"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠒⠄"}},
      "key": "27B3"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠲⠄"}}, "key": "27B4"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠢⠄"}},
      "key": "27B5"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠖⠄"}}, "key": "27B6"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠶⠄"}},
      "key": "27B7"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠦⠄"}}, "key": "27B8"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠔⠄"}},
      "key": "27B9"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠁⠄"}}, "key": "27BA"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠃⠄"}},
      "key": "27BB"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠉⠄"}}, "key": "27BC"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠙⠄"}},
      "key": "27BD"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠑⠄"}}, "key": "27BE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠴⠄"}},
      "key": "27F0"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠂⠄"}}, "key": "27F1"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠆⠄"}},
      "key": "27F2"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠒⠄"}}, "key": "27F3"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠲⠄"}},
      "key": "27F4"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠢⠄"}}, "key": "27F5"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠖⠄"}},
      "key": "27F6"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠶⠄"}}, "key": "27F7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠦⠄"}},
      "key": "27F8"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠔⠄"}}, "key": "27F9"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠁⠄"}},
      "key": "27FA"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠃⠄"}}, "key": "27FB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠉⠄"}},
      "key": "27FC"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠙⠄"}}, "key": "27FD"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠑⠄"}},
      "key": "27FE"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠋⠋⠄"}}, "key": "27FF"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠴⠄"}},
      "key": "2900"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠂⠄"}}, "key": "2901"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠆⠄"}},
      "key": "2902"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠒⠄"}}, "key": "2903"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠲⠄"}},
      "key": "2904"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠢⠄"}}, "key": "2905"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠖⠄"}},
      "key": "2906"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠶⠄"}}, "key": "2907"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠦⠄"}},
      "key": "2908"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠔⠄"}}, "key": "2909"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠁⠄"}},
      "key": "290A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠃⠄"}}, "key": "290B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠉⠄"}},
      "key": "290C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠙⠄"}}, "key": "290D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠑⠄"}},
      "key": "290E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠴⠋⠄"}}, "key": "290F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠴⠄"}},
      "key": "2910"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠂⠄"}}, "key": "2911"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠆⠄"}},
      "key": "2912"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠒⠄"}}, "key": "2913"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠲⠄"}},
      "key": "2914"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠢⠄"}}, "key": "2915"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠖⠄"}},
      "key": "2916"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠶⠄"}}, "key": "2917"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠦⠄"}},
      "key": "2918"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠔⠄"}}, "key": "2919"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠁⠄"}},
      "key": "291A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠃⠄"}}, "key": "291B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠉⠄"}},
      "key": "291C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠙⠄"}}, "key": "291D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠑⠄"}},
      "key": "291E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠂⠋⠄"}}, "key": "291F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠴⠄"}},
      "key": "2920"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠂⠄"}}, "key": "2921"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠆⠄"}},
      "key": "2922"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠒⠄"}}, "key": "2923"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠲⠄"}},
      "key": "2924"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠢⠄"}}, "key": "2925"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠖⠄"}},
      "key": "2926"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠶⠄"}}, "key": "2927"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠦⠄"}},
      "key": "2928"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠔⠄"}}, "key": "2929"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠁⠄"}},
      "key": "292A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠙⠄"}}, "key": "292D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠑⠄"}},
      "key": "292E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠋⠄"}}, "key": "292F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠴⠄"}},
      "key": "2930"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠂⠄"}}, "key": "2931"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠆⠄"}},
      "key": "2932"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠒⠄"}}, "key": "2933"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠲⠄"}},
      "key": "2934"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠢⠄"}}, "key": "2935"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠖⠄"}},
      "key": "2936"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠶⠄"}}, "key": "2937"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠦⠄"}},
      "key": "2938"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠔⠄"}}, "key": "2939"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠁⠄"}},
      "key": "293A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠃⠄"}}, "key": "293B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠉⠄"}},
      "key": "293C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠙⠄"}}, "key": "293D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠑⠄"}},
      "key": "293E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠒⠋⠄"}}, "key": "293F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠴⠄"}},
      "key": "2940"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠂⠄"}}, "key": "2941"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠆⠄"}},
      "key": "2942"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠒⠄"}}, "key": "2943"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠲⠄"}},
      "key": "2944"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠢⠄"}}, "key": "2945"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠖⠄"}},
      "key": "2946"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠶⠄"}}, "key": "2947"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠦⠄"}},
      "key": "2948"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠔⠄"}}, "key": "2949"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠴⠄"}},
      "key": "2970"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠂⠄"}}, "key": "2971"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠆⠄"}},
      "key": "2972"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠒⠄"}}, "key": "2973"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠲⠄"}},
      "key": "2974"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠢⠄"}}, "key": "2975"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠖⠄"}},
      "key": "2976"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠶⠄"}}, "key": "2977"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠦⠄"}},
      "key": "2978"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠔⠄"}}, "key": "2979"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠁⠄"}},
      "key": "297A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠃⠄"}}, "key": "297B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠃⠒⠄"}},
      "key": "29B3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠃⠲⠄"}}, "key": "29B4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠃⠙⠄"}},
      "key": "29BD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠁⠄"}}, "key": "29EA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠉⠄"}},
      "key": "29EC"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠙⠄"}}, "key": "29ED"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠶⠄"}},
      "key": "2A17"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠴⠄"}}, "key": "2B00"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠂⠄"}},
      "key": "2B01"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠆⠄"}}, "key": "2B02"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠒⠄"}},
      "key": "2B03"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠲⠄"}}, "key": "2B04"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠢⠄"}},
      "key": "2B05"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠖⠄"}}, "key": "2B06"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠶⠄"}},
      "key": "2B07"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠦⠄"}}, "key": "2B08"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠔⠄"}},
      "key": "2B09"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠁⠄"}}, "key": "2B0A"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠃⠄"}},
      "key": "2B0B"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠉⠄"}}, "key": "2B0C"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠙⠄"}},
      "key": "2B0D"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠑⠄"}}, "key": "2B0E"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠴⠋⠄"}},
      "key": "2B0F"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠴⠄"}}, "key": "2B10"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠂⠄"}},
      "key": "2B11"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠴⠄"}}, "key": "2B30"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠂⠄"}},
      "key": "2B31"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠆⠄"}}, "key": "2B32"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠒⠄"}},
      "key": "2B33"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠲⠄"}}, "key": "2B34"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠢⠄"}},
      "key": "2B35"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠖⠄"}}, "key": "2B36"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠶⠄"}},
      "key": "2B37"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠦⠄"}}, "key": "2B38"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠔⠄"}},
      "key": "2B39"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠁⠄"}}, "key": "2B3A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠃⠄"}},
      "key": "2B3B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠉⠄"}}, "key": "2B3C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠙⠄"}},
      "key": "2B3D"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠑⠄"}}, "key": "2B3E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠒⠋⠄"}},
      "key": "2B3F"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠴⠄"}}, "key": "2B40"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠂⠄"}},
      "key": "2B41"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠆⠄"}}, "key": "2B42"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠒⠄"}},
      "key": "2B43"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠲⠄"}}, "key": "2B44"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠢⠄"}},
      "key": "2B45"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠖⠄"}}, "key": "2B46"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠶⠄"}},
      "key": "2B47"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠦⠄"}}, "key": "2B48"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠔⠄"}},
      "key": "2B49"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠁⠄"}}, "key": "2B4A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠃⠄"}},
      "key": "2B4B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠲⠉⠄"}}, "key": "2B4C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠔⠄"}},
      "key": "FFE9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠁⠄"}}, "key": "FFEA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠃⠄"}},
      "key": "FFEB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠉⠄"}}, "key": "FFEC"}]
  ,
  "nemeth/symbols/math_characters.js":
    [{"locale": "nemeth"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "⠈⠇"}},
      "key": "2113"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠂⠦⠄"}}, "key": "2118"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠒⠉⠄"}},
      "key": "213C"
    }, {"category": "Ll", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠒⠙⠄"}}, "key": "213D"}, {
      "category": "Lu",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠒⠑⠄"}},
      "key": "213E"
    }, {"category": "Lu", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠒⠋⠄"}}, "key": "213F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠴⠄"}},
      "key": "2140"
    }, {"category": "Lu", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠢⠄"}}, "key": "2145"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "⠙"}},
      "key": "2146"
    }, {"category": "Ll", "mappings": {"default": {"default": "⠑"}}, "key": "2147"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠦⠄"}},
      "key": "2148"
    }, {"category": "Ll", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠔⠄"}}, "key": "2149"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "⠄⡳⠽⠂⠙⠖⠁⠲⠄"}},
      "key": "1D6A4"
    }, {"category": "Ll", "mappings": {"default": {"default": "⠄⡳⠽⠂⠙⠖⠁⠢⠄"}}, "key": "1D6A5"}]
  ,
  "nemeth/symbols/math_delimiters.js":
    [{"locale": "nemeth"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠷"}},
      "key": "0028"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠾"}}, "key": "0029"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠈⠷"}},
      "key": "005B"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠈⠾"}}, "key": "005D"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠨⠷"}},
      "key": "007B"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠨⠾"}}, "key": "007D"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠲⠢⠄"}},
      "key": "2045"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠲⠖⠄"}}, "key": "2046"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠈⠘⠷"}},
      "key": "2308"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠈⠘⠾"}}, "key": "2309"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠈⠰⠷"}},
      "key": "230A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠈⠰⠾"}}, "key": "230B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠴⠉⠄"}},
      "key": "230C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠴⠙⠄"}}, "key": "230D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠴⠑⠄"}},
      "key": "230E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠴⠋⠄"}}, "key": "230F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠈⠘⠷"}},
      "key": "231C"
    }, {"category": "So", "mappings": {"default": {"default": "⠈⠘⠾"}}, "key": "231D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠈⠰⠷"}},
      "key": "231E"
    }, {"category": "So", "mappings": {"default": {"default": "⠈⠰⠾"}}, "key": "231F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠆⠴⠄"}},
      "key": "2320"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠆⠂⠄"}}, "key": "2321"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠨⠨⠷"}},
      "key": "2329"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠨⠨⠾"}}, "key": "232A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠔⠃⠄"}},
      "key": "239B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠔⠉⠄"}}, "key": "239C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠔⠙⠄"}},
      "key": "239D"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠔⠑⠄"}}, "key": "239E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠔⠋⠄"}},
      "key": "239F"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠴⠄"}}, "key": "23A0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠂⠄"}},
      "key": "23A1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠆⠄"}}, "key": "23A2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠒⠄"}},
      "key": "23A3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠲⠄"}}, "key": "23A4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠢⠄"}},
      "key": "23A5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠖⠄"}}, "key": "23A6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠶⠄"}},
      "key": "23A7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠦⠄"}}, "key": "23A8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠔⠄"}},
      "key": "23A9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠁⠄"}}, "key": "23AA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠃⠄"}},
      "key": "23AB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠉⠄"}}, "key": "23AC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠙⠄"}},
      "key": "23AD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠑⠄"}}, "key": "23AE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠁⠋⠄"}},
      "key": "23AF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠃⠴⠄"}}, "key": "23B0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠃⠂⠄"}},
      "key": "23B1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠃⠆⠄"}}, "key": "23B2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠃⠒⠄"}},
      "key": "23B3"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠃⠲⠄"}}, "key": "23B4"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠃⠢⠄"}},
      "key": "23B5"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠃⠖⠄"}}, "key": "23B6"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠃⠶⠄"}},
      "key": "23B7"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠃⠦⠄"}}, "key": "23B8"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠃⠔⠄"}},
      "key": "23B9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠙⠉⠄"}}, "key": "23DC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠙⠙⠄"}},
      "key": "23DD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠣⠨⠷"}}, "key": "23DE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠣⠨⠾"}},
      "key": "23DF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠑⠴⠄"}}, "key": "23E0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠑⠂⠄"}},
      "key": "23E1"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠖⠦⠄"}}, "key": "2768"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠖⠔⠄"}},
      "key": "2769"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠖⠁⠄"}}, "key": "276A"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠖⠃⠄"}},
      "key": "276B"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠖⠉⠄"}}, "key": "276C"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠖⠙⠄"}},
      "key": "276D"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠖⠑⠄"}}, "key": "276E"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠖⠋⠄"}},
      "key": "276F"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠶⠴⠄"}}, "key": "2770"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠶⠂⠄"}},
      "key": "2771"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠶⠆⠄"}}, "key": "2772"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠶⠒⠄"}},
      "key": "2773"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠶⠲⠄"}}, "key": "2774"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠶⠢⠄"}},
      "key": "2775"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠢⠄"}}, "key": "27C5"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠖⠄"}},
      "key": "27C6"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠈⠸⠷"}}, "key": "27E6"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠈⠸⠾"}},
      "key": "27E7"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠨⠨⠷"}}, "key": "27E8"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠨⠨⠾"}},
      "key": "27E9"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠨⠨⠨⠷"}}, "key": "27EA"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠨⠨⠨⠾"}},
      "key": "27EB"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠑⠉⠄"}}, "key": "27EC"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠑⠙⠄"}},
      "key": "27ED"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠑⠑⠄"}}, "key": "27EE"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠑⠋⠄"}},
      "key": "27EF"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠨⠸⠷"}}, "key": "2983"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠨⠸⠾"}},
      "key": "2984"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠢⠄"}}, "key": "2985"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠖⠄"}},
      "key": "2986"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠶⠄"}}, "key": "2987"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠦⠄"}},
      "key": "2988"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠔⠄"}}, "key": "2989"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠁⠄"}},
      "key": "298A"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠃⠄"}}, "key": "298B"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠉⠄"}},
      "key": "298C"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠙⠄"}}, "key": "298D"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠑⠄"}},
      "key": "298E"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠋⠄"}}, "key": "298F"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠴⠄"}},
      "key": "2990"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠂⠄"}}, "key": "2991"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠆⠄"}},
      "key": "2992"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠒⠄"}}, "key": "2993"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠲⠄"}},
      "key": "2994"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠢⠄"}}, "key": "2995"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠖⠄"}},
      "key": "2996"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠶⠄"}}, "key": "2997"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠦⠄"}},
      "key": "2998"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠦⠄"}}, "key": "29D8"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠔⠄"}},
      "key": "29D9"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠁⠄"}}, "key": "29DA"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠃⠄"}},
      "key": "29DB"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠉⠄"}}, "key": "29FC"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠙⠄"}},
      "key": "29FD"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠑⠆⠆⠄"}}, "key": "2E22"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠑⠆⠒⠄"}},
      "key": "2E23"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠑⠆⠲⠄"}}, "key": "2E24"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠑⠆⠢⠄"}},
      "key": "2E25"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠑⠆⠖⠄"}}, "key": "2E26"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠑⠆⠶⠄"}},
      "key": "2E27"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠑⠆⠦⠄"}}, "key": "2E28"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠑⠆⠔⠄"}},
      "key": "2E29"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠨⠨⠷"}}, "key": "3008"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠨⠨⠾"}},
      "key": "3009"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠨⠨⠨⠷"}}, "key": "300A"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠨⠨⠨⠾"}},
      "key": "300B"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠈⠘⠷"}}, "key": "300C"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠈⠘⠾"}},
      "key": "300D"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠴⠑⠄"}}, "key": "300E"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠴⠋⠄"}},
      "key": "300F"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠴⠄"}}, "key": "3010"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠂⠄"}},
      "key": "3011"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠲⠄"}}, "key": "3014"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠢⠄"}},
      "key": "3015"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠖⠄"}}, "key": "3016"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠶⠄"}},
      "key": "3017"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠦⠄"}}, "key": "3018"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠔⠄"}},
      "key": "3019"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠁⠄"}}, "key": "301A"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠃⠄"}},
      "key": "301B"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠙⠄"}}, "key": "301D"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠑⠄"}},
      "key": "301E"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠋⠄"}}, "key": "301F"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠙⠒⠑⠄"}},
      "key": "FD3E"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠙⠒⠋⠄"}}, "key": "FD3F"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠂⠶⠄"}},
      "key": "FE17"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠂⠦⠄"}}, "key": "FE18"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠢⠄"}},
      "key": "FE35"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠖⠄"}}, "key": "FE36"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠶⠄"}},
      "key": "FE37"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠦⠄"}}, "key": "FE38"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠔⠄"}},
      "key": "FE39"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠁⠄"}}, "key": "FE3A"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠃⠄"}},
      "key": "FE3B"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠉⠄"}}, "key": "FE3C"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠙⠄"}},
      "key": "FE3D"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠑⠄"}}, "key": "FE3E"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠋⠄"}},
      "key": "FE3F"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠴⠄"}}, "key": "FE40"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠂⠄"}},
      "key": "FE41"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠆⠄"}}, "key": "FE42"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠒⠄"}},
      "key": "FE43"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠲⠄"}}, "key": "FE44"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠶⠄"}},
      "key": "FE47"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠦⠄"}}, "key": "FE48"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠔⠄"}},
      "key": "FE59"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠁⠄"}}, "key": "FE5A"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠃⠄"}},
      "key": "FE5B"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠉⠄"}}, "key": "FE5C"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠙⠄"}},
      "key": "FE5D"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠑⠄"}}, "key": "FE5E"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠦⠄"}},
      "key": "FF08"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠔⠄"}}, "key": "FF09"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠒⠃⠄"}},
      "key": "FF3B"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠒⠙⠄"}}, "key": "FF3D"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠢⠃⠄"}},
      "key": "FF5B"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠢⠙⠄"}}, "key": "FF5D"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠢⠋⠄"}},
      "key": "FF5F"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠖⠴⠄"}}, "key": "FF60"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠖⠆⠄"}},
      "key": "FF62"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠖⠒⠄"}}, "key": "FF63"}]
  ,
  "nemeth/symbols/math_geometry.js":
    [{"locale": "nemeth"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠴⠄"}},
      "key": "2500"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠂⠄"}}, "key": "2501"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠆⠄"}},
      "key": "2502"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠒⠄"}}, "key": "2503"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠲⠄"}},
      "key": "2504"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠢⠄"}}, "key": "2505"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠖⠄"}},
      "key": "2506"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠶⠄"}}, "key": "2507"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠦⠄"}},
      "key": "2508"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠔⠄"}}, "key": "2509"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠁⠄"}},
      "key": "250A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠃⠄"}}, "key": "250B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠉⠄"}},
      "key": "250C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠙⠄"}}, "key": "250D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠑⠄"}},
      "key": "250E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠴⠋⠄"}}, "key": "250F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠴⠄"}},
      "key": "2510"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠂⠄"}}, "key": "2511"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠆⠄"}},
      "key": "2512"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠒⠄"}}, "key": "2513"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠲⠄"}},
      "key": "2514"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠢⠄"}}, "key": "2515"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠖⠄"}},
      "key": "2516"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠶⠄"}}, "key": "2517"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠦⠄"}},
      "key": "2518"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠔⠄"}}, "key": "2519"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠁⠄"}},
      "key": "251A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠃⠄"}}, "key": "251B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠉⠄"}},
      "key": "251C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠙⠄"}}, "key": "251D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠑⠄"}},
      "key": "251E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠂⠋⠄"}}, "key": "251F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠴⠄"}},
      "key": "2520"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠂⠄"}}, "key": "2521"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠆⠄"}},
      "key": "2522"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠒⠄"}}, "key": "2523"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠲⠄"}},
      "key": "2524"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠢⠄"}}, "key": "2525"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠖⠄"}},
      "key": "2526"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠶⠄"}}, "key": "2527"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠦⠄"}},
      "key": "2528"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠔⠄"}}, "key": "2529"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠁⠄"}},
      "key": "252A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠃⠄"}}, "key": "252B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠉⠄"}},
      "key": "252C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠙⠄"}}, "key": "252D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠑⠄"}},
      "key": "252E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠆⠋⠄"}}, "key": "252F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠴⠄"}},
      "key": "2530"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠂⠄"}}, "key": "2531"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠆⠄"}},
      "key": "2532"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠒⠄"}}, "key": "2533"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠲⠄"}},
      "key": "2534"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠢⠄"}}, "key": "2535"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠖⠄"}},
      "key": "2536"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠶⠄"}}, "key": "2537"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠦⠄"}},
      "key": "2538"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠔⠄"}}, "key": "2539"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠁⠄"}},
      "key": "253A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠃⠄"}}, "key": "253B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠉⠄"}},
      "key": "253C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠙⠄"}}, "key": "253D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠑⠄"}},
      "key": "253E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠒⠋⠄"}}, "key": "253F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠴⠄"}},
      "key": "2540"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠂⠄"}}, "key": "2541"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠆⠄"}},
      "key": "2542"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠒⠄"}}, "key": "2543"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠲⠄"}},
      "key": "2544"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠢⠄"}}, "key": "2545"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠖⠄"}},
      "key": "2546"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠶⠄"}}, "key": "2547"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠦⠄"}},
      "key": "2548"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠔⠄"}}, "key": "2549"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠁⠄"}},
      "key": "254A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠃⠄"}}, "key": "254B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠉⠄"}},
      "key": "254C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠙⠄"}}, "key": "254D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠑⠄"}},
      "key": "254E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠲⠋⠄"}}, "key": "254F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠴⠄"}},
      "key": "2550"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠂⠄"}}, "key": "2551"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠆⠄"}},
      "key": "2552"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠒⠄"}}, "key": "2553"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠲⠄"}},
      "key": "2554"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠢⠄"}}, "key": "2555"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠖⠄"}},
      "key": "2556"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠶⠄"}}, "key": "2557"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠦⠄"}},
      "key": "2558"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠔⠄"}}, "key": "2559"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠁⠄"}},
      "key": "255A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠃⠄"}}, "key": "255B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠉⠄"}},
      "key": "255C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠙⠄"}}, "key": "255D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠑⠄"}},
      "key": "255E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠢⠋⠄"}}, "key": "255F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠴⠄"}},
      "key": "2560"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠂⠄"}}, "key": "2561"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠆⠄"}},
      "key": "2562"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠒⠄"}}, "key": "2563"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠲⠄"}},
      "key": "2564"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠢⠄"}}, "key": "2565"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠖⠄"}},
      "key": "2566"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠶⠄"}}, "key": "2567"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠦⠄"}},
      "key": "2568"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠔⠄"}}, "key": "2569"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠁⠄"}},
      "key": "256A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠃⠄"}}, "key": "256B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠉⠄"}},
      "key": "256C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠙⠄"}}, "key": "256D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠑⠄"}},
      "key": "256E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠖⠋⠄"}}, "key": "256F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠴⠄"}},
      "key": "2570"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠂⠄"}}, "key": "2571"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠆⠄"}},
      "key": "2572"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠒⠄"}}, "key": "2573"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠲⠄"}},
      "key": "2574"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠢⠄"}}, "key": "2575"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠖⠄"}},
      "key": "2576"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠶⠄"}}, "key": "2577"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠦⠄"}},
      "key": "2578"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠔⠄"}}, "key": "2579"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠁⠄"}},
      "key": "257A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠃⠄"}}, "key": "257B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠉⠄"}},
      "key": "257C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠙⠄"}}, "key": "257D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠑⠄"}},
      "key": "257E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠶⠋⠄"}}, "key": "257F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠴⠄"}},
      "key": "2580"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠂⠄"}}, "key": "2581"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠆⠄"}},
      "key": "2582"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠒⠄"}}, "key": "2583"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠲⠄"}},
      "key": "2584"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠢⠄"}}, "key": "2585"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠖⠄"}},
      "key": "2586"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠶⠄"}}, "key": "2587"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠦⠄"}},
      "key": "2588"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠔⠄"}}, "key": "2589"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠁⠄"}},
      "key": "258A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠃⠄"}}, "key": "258B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠉⠄"}},
      "key": "258C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠙⠄"}}, "key": "258D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠑⠄"}},
      "key": "258E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠦⠋⠄"}}, "key": "258F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠴⠄"}},
      "key": "2590"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠂⠄"}}, "key": "2591"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠆⠄"}},
      "key": "2592"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠒⠄"}}, "key": "2593"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠲⠄"}},
      "key": "2594"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠢⠄"}}, "key": "2595"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠖⠄"}},
      "key": "2596"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠶⠄"}}, "key": "2597"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠦⠄"}},
      "key": "2598"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠔⠄"}}, "key": "2599"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠁⠄"}},
      "key": "259A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠃⠄"}}, "key": "259B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠉⠄"}},
      "key": "259C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠙⠄"}}, "key": "259D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠑⠄"}},
      "key": "259E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠔⠋⠄"}}, "key": "259F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠫⠸⠲"}},
      "key": "25A0"
    }, {"category": "So", "mappings": {"default": {"default": "⠫⠲"}}, "key": "25A1"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠁⠆⠄"}},
      "key": "25A2"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠁⠒⠄"}}, "key": "25A3"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠁⠲⠄"}},
      "key": "25A4"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠁⠢⠄"}}, "key": "25A5"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠁⠖⠄"}},
      "key": "25A6"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠁⠶⠄"}}, "key": "25A7"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠁⠦⠄"}},
      "key": "25A8"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠁⠔⠄"}}, "key": "25A9"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠁⠁⠄"}},
      "key": "25AA"
    }, {"category": "So", "mappings": {"default": {"default": "⠫⠸⠗"}}, "key": "25AB"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠫⠗"}},
      "key": "25AC"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠁⠙⠄"}}, "key": "25AD"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠁⠑⠄"}},
      "key": "25AE"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠁⠋⠄"}}, "key": "25AF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠫⠸⠛"}},
      "key": "25B0"
    }, {"category": "So", "mappings": {"default": {"default": "⠫⠛"}}, "key": "25B1"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠫⠸⠞"}},
      "key": "25B2"
    }, {"category": "So", "mappings": {"default": {"default": "⠫⠞"}}, "key": "25B3"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠃⠲⠄"}},
      "key": "25B4"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠃⠢⠄"}}, "key": "25B5"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠃⠖⠄"}},
      "key": "25B6"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠃⠶⠄"}}, "key": "25B7"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠃⠦⠄"}},
      "key": "25B8"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠃⠔⠄"}}, "key": "25B9"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠃⠁⠄"}},
      "key": "25BA"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠃⠃⠄"}}, "key": "25BB"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠸⠨⠫"}},
      "key": "25BC"
    }, {"category": "So", "mappings": {"default": {"default": "⠨⠫"}}, "key": "25BD"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠃⠑⠄"}},
      "key": "25BE"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠃⠋⠄"}}, "key": "25BF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠴⠄"}},
      "key": "25C0"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠂⠄"}}, "key": "25C1"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠆⠄"}},
      "key": "25C2"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠒⠄"}}, "key": "25C3"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠲⠄"}},
      "key": "25C4"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠢⠄"}}, "key": "25C5"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠖⠄"}},
      "key": "25C6"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠶⠄"}}, "key": "25C7"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠦⠄"}},
      "key": "25C8"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠔⠄"}}, "key": "25C9"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠫⠸⠉"}},
      "key": "25CA"
    }, {"category": "So", "mappings": {"default": {"default": "⠫⠉"}}, "key": "25CB"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠉⠄"}},
      "key": "25CC"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠙⠄"}}, "key": "25CD"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠑⠄"}},
      "key": "25CE"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠉⠋⠄"}}, "key": "25CF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠴⠄"}},
      "key": "25D0"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠂⠄"}}, "key": "25D1"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠆⠄"}},
      "key": "25D2"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠒⠄"}}, "key": "25D3"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠲⠄"}},
      "key": "25D4"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠢⠄"}}, "key": "25D5"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠖⠄"}},
      "key": "25D6"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠶⠄"}}, "key": "25D7"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠦⠄"}},
      "key": "25D8"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠔⠄"}}, "key": "25D9"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠁⠄"}},
      "key": "25DA"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠃⠄"}}, "key": "25DB"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠉⠄"}},
      "key": "25DC"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠙⠄"}}, "key": "25DD"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠑⠄"}},
      "key": "25DE"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠙⠋⠄"}}, "key": "25DF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠴⠄"}},
      "key": "25E0"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠂⠄"}}, "key": "25E1"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠆⠄"}},
      "key": "25E2"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠒⠄"}}, "key": "25E3"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠲⠄"}},
      "key": "25E4"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠢⠄"}}, "key": "25E5"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠖⠄"}},
      "key": "25E6"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠶⠄"}}, "key": "25E7"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠦⠄"}},
      "key": "25E8"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠔⠄"}}, "key": "25E9"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠁⠄"}},
      "key": "25EA"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠃⠄"}}, "key": "25EB"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠉⠄"}},
      "key": "25EC"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠙⠄"}}, "key": "25ED"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠑⠄"}},
      "key": "25EE"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠑⠋⠄"}}, "key": "25EF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠴⠄"}},
      "key": "25F0"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠂⠄"}}, "key": "25F1"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠆⠄"}},
      "key": "25F2"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠒⠄"}}, "key": "25F3"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠲⠄"}},
      "key": "25F4"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠢⠄"}}, "key": "25F5"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠖⠄"}},
      "key": "25F6"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠶⠄"}}, "key": "25F7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠦⠄"}},
      "key": "25F8"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠔⠄"}}, "key": "25F9"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠁⠄"}},
      "key": "25FA"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠃⠄"}}, "key": "25FB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠉⠄"}},
      "key": "25FC"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠙⠄"}}, "key": "25FD"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠑⠄"}},
      "key": "25FE"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠢⠋⠋⠄"}}, "key": "25FF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠆⠄"}},
      "key": "2B12"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠒⠄"}}, "key": "2B13"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠲⠄"}},
      "key": "2B14"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠢⠄"}}, "key": "2B15"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠖⠄"}},
      "key": "2B16"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠶⠄"}}, "key": "2B17"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠦⠄"}},
      "key": "2B18"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠔⠄"}}, "key": "2B19"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠁⠄"}},
      "key": "2B1A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠃⠄"}}, "key": "2B1B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠉⠄"}},
      "key": "2B1C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠙⠄"}}, "key": "2B1D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠑⠄"}},
      "key": "2B1E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠂⠋⠄"}}, "key": "2B1F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠴⠄"}},
      "key": "2B20"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠂⠄"}}, "key": "2B21"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠆⠄"}},
      "key": "2B22"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠒⠄"}}, "key": "2B23"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠲⠄"}},
      "key": "2B24"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠢⠄"}}, "key": "2B25"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠖⠄"}},
      "key": "2B26"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠶⠄"}}, "key": "2B27"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠦⠄"}},
      "key": "2B28"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠔⠄"}}, "key": "2B29"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠁⠄"}},
      "key": "2B2A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠃⠄"}}, "key": "2B2B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠉⠄"}},
      "key": "2B2C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠙⠄"}}, "key": "2B2D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠑⠄"}},
      "key": "2B2E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠆⠋⠄"}}, "key": "2B2F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠢⠴⠄"}},
      "key": "2B50"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠢⠂⠄"}}, "key": "2B51"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠢⠆⠄"}},
      "key": "2B52"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠢⠒⠄"}}, "key": "2B53"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠢⠲⠄"}},
      "key": "2B54"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠢⠢⠄"}}, "key": "2B55"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠢⠖⠄"}},
      "key": "2B56"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠢⠶⠄"}}, "key": "2B57"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠢⠦⠄"}},
      "key": "2B58"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠃⠢⠔⠄"}}, "key": "2B59"}]
  ,
  "nemeth/symbols/math_harpoons.js":
    [{"locale": "nemeth"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠈⠪⠒⠒⠀"}},
      "key": "21BC"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠠⠪⠒⠒⠀"}}, "key": "21BD"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠣⠒⠒⠠⠕⠀"}},
      "key": "21BE"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠣⠒⠒⠈⠕⠀"}}, "key": "21BF"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠒⠒⠈⠕⠀"}},
      "key": "21C0"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠒⠒⠠⠕⠀"}}, "key": "21C1"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠪⠒⠒⠈⠕⠀"}},
      "key": "21C2"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠩⠒⠒⠠⠕⠀"}}, "key": "21C3"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠀⠫⠈⠪⠒⠒⠫⠒⠒⠈⠕⠀"}},
      "key": "21CB"
    }, {"category": "So", "mappings": {"default": {"default": "⠀⠫⠒⠒⠈⠕⠫⠈⠪⠒⠒⠀"}}, "key": "21CC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠁⠄"}},
      "key": "294A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠃⠄"}}, "key": "294B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠉⠄"}},
      "key": "294C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠙⠄"}}, "key": "294D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠑⠄"}},
      "key": "294E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠲⠋⠄"}}, "key": "294F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠴⠄"}},
      "key": "2950"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠂⠄"}}, "key": "2951"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠆⠄"}},
      "key": "2952"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠒⠄"}}, "key": "2953"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠲⠄"}},
      "key": "2954"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠢⠄"}}, "key": "2955"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠖⠄"}},
      "key": "2956"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠶⠄"}}, "key": "2957"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠦⠄"}},
      "key": "2958"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠔⠄"}}, "key": "2959"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠁⠄"}},
      "key": "295A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠃⠄"}}, "key": "295B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠉⠄"}},
      "key": "295C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠙⠄"}}, "key": "295D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠑⠄"}},
      "key": "295E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠢⠋⠄"}}, "key": "295F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠴⠄"}},
      "key": "2960"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠂⠄"}}, "key": "2961"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠆⠄"}},
      "key": "2962"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠒⠄"}}, "key": "2963"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠲⠄"}},
      "key": "2964"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠢⠄"}}, "key": "2965"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠖⠄"}},
      "key": "2966"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠶⠄"}}, "key": "2967"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠦⠄"}},
      "key": "2968"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠔⠄"}}, "key": "2969"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠁⠄"}},
      "key": "296A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠃⠄"}}, "key": "296B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠉⠄"}},
      "key": "296C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠙⠄"}}, "key": "296D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠑⠄"}},
      "key": "296E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠖⠋⠄"}}, "key": "296F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠉⠄"}},
      "key": "297C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠙⠄"}}, "key": "297D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠑⠄"}},
      "key": "297E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠶⠋⠄"}}, "key": "297F"}]
  ,
  "nemeth/symbols/math_non_characters.js":
    [{"locale": "nemeth"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠴⠋⠄"}},
      "key": "210F"
    }, {"category": "So", "mappings": {"default": {"default": "⠳"}}, "key": "2114"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠂⠖⠄"}},
      "key": "2116"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠂⠶⠄"}}, "key": "2117"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠂⠑⠄"}},
      "key": "211E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠂⠋⠄"}}, "key": "211F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠆⠴⠄"}},
      "key": "2120"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠆⠂⠄"}}, "key": "2121"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠘⠞"}},
      "key": "2122"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠆⠒⠄"}}, "key": "2123"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠆⠢⠄"}},
      "key": "2125"
    }, {"category": "Lu", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠆⠖⠄"}}, "key": "2126"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠆⠶⠄"}},
      "key": "2127"
    }, {"category": "Lu", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠆⠁⠄"}}, "key": "212A"}, {
      "category": "Lu",
      "mappings": {"default": {"default": "⠀⠈⠠⠁"}},
      "key": "212B"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠆⠑⠄"}}, "key": "212E"}, {
      "category": "Lu",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠒⠆⠄"}},
      "key": "2132"
    }, {"category": "Ll", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠒⠔⠄"}}, "key": "2139"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠒⠁⠄"}},
      "key": "213A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠒⠃⠄"}}, "key": "213B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠂⠄"}},
      "key": "2141"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠆⠄"}}, "key": "2142"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠒⠄"}},
      "key": "2143"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠲⠄"}}, "key": "2144"}]
  ,
  "nemeth/symbols/math_symbols.js":
    [{"locale": "nemeth"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠖"}},
      "key": "0021"
    }, {"category": "Po", "mappings": {"default": {"default": "⠹"}}, "key": "0022"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠨⠼"}},
      "key": "0023"
    }, {"category": "Sc", "mappings": {"default": {"default": "⠈⠎"}}, "key": "0024"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠈⠴"}},
      "key": "0025"
    }, {"category": "Po", "mappings": {"default": {"default": "⠸⠯"}}, "key": "0026"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄"}},
      "key": "0027"
    }, {"category": "Po", "mappings": {"default": {"default": "⠈⠼"}}, "key": "002A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠬"}},
      "key": "002B"
    }, {
      "category": "Po",
      "mappings": {"default": {"default": "⠠", "defaultLibLouis": "⠠⠀"}},
      "key": "002C"
    }, {"category": "Pd", "mappings": {"default": {"default": "⠤"}}, "key": "002D"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠨", "other": "⠲"}},
      "key": "002E"
    }, {"category": "Po", "mappings": {"default": {"default": "⠸⠌"}}, "key": "002F"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠒"}},
      "key": "003A"
    }, {"category": "Po", "mappings": {"default": {"default": "⠆"}}, "key": "003B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠐⠅⠀"}},
      "key": "003C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠅⠀"}}, "key": "003D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠨⠂⠀"}},
      "key": "003E"
    }, {"category": "Po", "mappings": {"default": {"default": "⠸⠦"}}, "key": "003F"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠈⠁"}},
      "key": "0040"
    }, {"category": "Po", "mappings": {"default": {"default": "⠸⠡"}}, "key": "005C"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "⡘"}},
      "key": "005E"
    }, {"category": "Pc", "mappings": {"default": {"default": "⠸"}}, "key": "005F"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "⠈"}},
      "key": "0060"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠳"}}, "key": "007C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠈⠱"}},
      "key": "007E"
    }, {"category": "Po", "mappings": {"default": {"default": "⠖"}}, "key": "00A1"}, {
      "category": "Sc",
      "mappings": {"default": {"default": "⠈⠉"}},
      "key": "00A2"
    }, {"category": "Sc", "mappings": {"default": {"default": "⠼"}}, "key": "00A3"}, {
      "category": "Sc",
      "mappings": {"default": {"default": "⠫"}},
      "key": "00A4"
    }, {"category": "Sc", "mappings": {"default": {"default": "⠄⡳⠭⠴⠴⠁⠢⠄"}}, "key": "00A5"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠸⠳"}},
      "key": "00A6"
    }, {"category": "Po", "mappings": {"default": {"default": "⠈⠠⠎"}}, "key": "00A7"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠴⠁⠦⠄"}},
      "key": "00A8"
    }, {"category": "So", "mappings": {"default": {"default": "⠘⠉"}}, "key": "00A9"}, {
      "category": "Lo",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠴⠁⠁⠄"}},
      "key": "00AA"
    }, {"category": "Pi", "mappings": {"default": {"default": "⠄⡳⠭⠴⠴⠁⠃⠄"}}, "key": "00AB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠴⠁⠉⠄"}},
      "key": "00AC"
    }, {"category": "So", "mappings": {"default": {"default": "⠘⠗"}}, "key": "00AE"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "⠱"}},
      "key": "00AF"
    }, {"category": "So", "mappings": {"default": {"default": "⠘⠨⠡"}}, "key": "00B0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠬⠤"}},
      "key": "00B1"
    }, {"category": "Sk", "mappings": {"default": {"default": "⠈"}}, "key": "00B4"}, {
      "category": "Ll",
      "mappings": {"default": {"default": "⠨⠍"}},
      "key": "00B5"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠴⠴⠃⠖⠄"}}, "key": "00B6"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠡"}},
      "key": "00B7"
    }, {"category": "Sk", "mappings": {"default": {"default": "⠄⡳⠭⠴⠴⠃⠦⠄"}}, "key": "00B8"}, {
      "category": "Lo",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠴⠃⠁⠄"}},
      "key": "00BA"
    }, {"category": "Pf", "mappings": {"default": {"default": "⠄⡳⠭⠴⠴⠃⠃⠄"}}, "key": "00BB"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠦"}},
      "key": "00BF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠈⠡"}}, "key": "00D7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠈⠌"}},
      "key": "00F7"
    }, {"category": "Sk", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠙⠦⠄"}}, "key": "02D8"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠙⠔⠄"}},
      "key": "02D9"
    }, {"category": "Sk", "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠙⠁⠄"}}, "key": "02DA"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠙⠃⠄"}},
      "key": "02DB"
    }, {"category": "Sk", "mappings": {"default": {"default": "⠈⠱"}}, "key": "02DC"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "⠄⡳⠭⠴⠆⠙⠙⠄"}},
      "key": "02DD"
    }, {"category": "Pd", "mappings": {"default": {"default": "⠤"}}, "key": "2010"}, {
      "category": "Pd",
      "mappings": {"default": {"default": "⠤"}},
      "key": "2011"
    }, {"category": "Pd", "mappings": {"default": {"default": "⠤⠤"}}, "key": "2012"}, {
      "category": "Pd",
      "mappings": {"default": {"default": "⠤⠤"}},
      "key": "2013"
    }, {"category": "Pd", "mappings": {"default": {"default": "⠤⠤"}}, "key": "2014"}, {
      "category": "Pd",
      "mappings": {"default": {"default": "⠤⠤"}},
      "key": "2015"
    }, {"category": "Po", "mappings": {"default": {"default": "⠳⠳"}}, "key": "2016"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠂⠶⠄"}},
      "key": "2017"
    }, {"category": "Pi", "mappings": {"default": {"default": "⠠⠦"}}, "key": "2018"}, {
      "category": "Pf",
      "mappings": {"default": {"default": "⠄"}},
      "key": "2019"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠂⠁⠄"}}, "key": "201A"}, {
      "category": "Pi",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠂⠃⠄"}},
      "key": "201B"
    }, {"category": "Pi", "mappings": {"default": {"default": "⠦"}}, "key": "201C"}, {
      "category": "Pf",
      "mappings": {"default": {"default": "⠴"}},
      "key": "201D"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠂⠑⠄"}}, "key": "201E"}, {
      "category": "Pi",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠂⠋⠄"}},
      "key": "201F"
    }, {"category": "Po", "mappings": {"default": {"default": "⠸⠻"}}, "key": "2020"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠸⠸⠻"}},
      "key": "2021"
    }, {"category": "Po", "mappings": {"default": {"default": "⠔⠔"}}, "key": "2022"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠆⠒⠄"}},
      "key": "2023"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄"}}, "key": "2024"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⠄"}},
      "key": "2025"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⠄⠄"}}, "key": "2026"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠆⠶⠄"}},
      "key": "2027"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠒⠴⠄"}}, "key": "2030"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠒⠂⠄"}},
      "key": "2031"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄"}}, "key": "2032"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⠄"}},
      "key": "2033"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⠄⠄"}}, "key": "2034"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠒⠢⠄"}},
      "key": "2035"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠒⠖⠄"}}, "key": "2036"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠒⠶⠄"}},
      "key": "2037"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠒⠦⠄"}}, "key": "2038"}, {
      "category": "Pi",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠒⠔⠄"}},
      "key": "2039"
    }, {"category": "Pf", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠒⠁⠄"}}, "key": "203A"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠒⠃⠄"}},
      "key": "203B"
    }, {"category": "Po", "mappings": {"default": {"default": "⠖⠖"}}, "key": "203C"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠒⠙⠄"}},
      "key": "203D"
    }, {"category": "Po", "mappings": {"default": {"default": "⠱"}}, "key": "203E"}, {
      "category": "Pc",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠒⠋⠄"}},
      "key": "203F"
    }, {"category": "Pc", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠲⠴⠄"}}, "key": "2040"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠲⠂⠄"}},
      "key": "2041"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠲⠆⠄"}}, "key": "2042"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠲⠒⠄"}},
      "key": "2043"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠲⠲⠄"}}, "key": "2044"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠹⠹"}},
      "key": "2047"
    }, {"category": "Po", "mappings": {"default": {"default": "⠹⠖"}}, "key": "2048"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠖⠹"}},
      "key": "2049"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠲⠃⠄"}}, "key": "204B"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠲⠉⠄"}},
      "key": "204C"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠲⠙⠄"}}, "key": "204D"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠲⠑⠄"}},
      "key": "204E"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠲⠋⠄"}}, "key": "204F"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠴⠄"}},
      "key": "2050"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠂⠄"}}, "key": "2051"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠆⠄"}},
      "key": "2052"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠒⠄"}}, "key": "2053"}, {
      "category": "Pc",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠲⠄"}},
      "key": "2054"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠢⠄"}}, "key": "2055"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠖⠄"}},
      "key": "2056"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠶⠄"}}, "key": "2057"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠦⠄"}},
      "key": "2058"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠔⠄"}}, "key": "2059"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠁⠄"}},
      "key": "205A"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠃⠄"}}, "key": "205B"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠉⠄"}},
      "key": "205C"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠙⠄"}}, "key": "205D"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠢⠑⠄"}},
      "key": "205E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠶⠁⠄"}}, "key": "207A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠶⠃⠄"}},
      "key": "207B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠶⠉⠄"}}, "key": "207C"}, {
      "category": "Ps",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠶⠙⠄"}},
      "key": "207D"
    }, {"category": "Pe", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠶⠑⠄"}}, "key": "207E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠦⠁⠄"}},
      "key": "208A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠦⠃⠄"}}, "key": "208B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠦⠉⠄"}},
      "key": "208C"
    }, {"category": "Ps", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠦⠙⠄"}}, "key": "208D"}, {
      "category": "Pe",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠦⠑⠄"}},
      "key": "208E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠁⠄"}}, "key": "214A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠃⠄"}},
      "key": "214B"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠉⠄"}}, "key": "214C"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠙⠄"}},
      "key": "214D"
    }, {"category": "Ll", "mappings": {"default": {"default": "⠄⡳⠭⠆⠂⠲⠑⠄"}}, "key": "214E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠈⠯"}},
      "key": "2200"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠴⠂⠄"}}, "key": "2201"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠈⠿"}},
      "key": "2203"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠌⠈⠿"}}, "key": "2204"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠸⠴"}},
      "key": "2205"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠴⠖⠄"}}, "key": "2206"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠈⠑⠀"}},
      "key": "2208"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠌⠈⠑⠀"}}, "key": "2209"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠈⠑⠀"}},
      "key": "220A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠸⠨⠁⠀"}}, "key": "220B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠌⠸⠨⠁⠀"}},
      "key": "220C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠈⠢⠀"}}, "key": "220D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠸⠳⠀"}},
      "key": "220E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠴⠋⠄"}}, "key": "220F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠂⠴⠄"}},
      "key": "2210"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠨⠠⠎"}}, "key": "2211"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠤"}},
      "key": "2212"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠤⠬"}}, "key": "2213"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠂⠲⠄"}},
      "key": "2214"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠸⠌"}}, "key": "2215"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠸⠡"}},
      "key": "2216"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠈⠼"}}, "key": "2217"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠘⠨⠡"}},
      "key": "2218"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠂⠔⠄"}}, "key": "2219"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠂⠁⠄"}},
      "key": "221A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠂⠃⠄"}}, "key": "221B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠂⠉⠄"}},
      "key": "221C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠰⠆⠀"}}, "key": "221D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠠⠿"}},
      "key": "221E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠪⠨⠗⠻"}}, "key": "221F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠪"}},
      "key": "2220"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠪⠸⠫⠫⠁⠻"}}, "key": "2221"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠪⠸⠫⠫⠁⠻"}},
      "key": "2222"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠳"}}, "key": "2223"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠌⠳"}},
      "key": "2224"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠫⠇⠀"}}, "key": "2225"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠌⠫⠇⠀"}},
      "key": "2226"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠈⠩"}}, "key": "2227"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠈⠬"}},
      "key": "2228"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠨⠩"}}, "key": "2229"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠨⠬"}},
      "key": "222A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠮"}}, "key": "222B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠮⠮"}},
      "key": "222C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠮⠮⠮"}}, "key": "222D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠮⠈⠫⠉⠻"}},
      "key": "222E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠆⠋⠄"}}, "key": "222F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠒⠴⠄"}},
      "key": "2230"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠒⠂⠄"}}, "key": "2231"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠮⠈⠫⠪⠢⠔⠻"}},
      "key": "2232"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠮⠈⠫⠢⠔⠕⠻"}}, "key": "2233"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠠⠡⠀"}},
      "key": "2234"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠈⠌⠀"}}, "key": "2235"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠐⠂⠀"}},
      "key": "2236"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠒⠶⠄"}}, "key": "2237"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠒⠦⠄"}},
      "key": "2238"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠒⠔⠄"}}, "key": "2239"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠒⠁⠄"}},
      "key": "223A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠒⠃⠄"}}, "key": "223B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠈⠱"}},
      "key": "223C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠒⠙⠄"}}, "key": "223D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠒⠑⠄"}},
      "key": "223E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠒⠋⠄"}}, "key": "223F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠴⠄"}},
      "key": "2240"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠂⠄"}}, "key": "2241"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠆⠄"}},
      "key": "2242"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠒⠄"}}, "key": "2243"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠲⠄"}},
      "key": "2244"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠈⠱⠈⠱⠀"}}, "key": "2245"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠖⠄"}},
      "key": "2246"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠶⠄"}}, "key": "2247"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠦⠄"}},
      "key": "2248"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠔⠄"}}, "key": "2249"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠁⠄"}},
      "key": "224A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠃⠄"}}, "key": "224B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠉⠄"}},
      "key": "224C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠈⠣⠠⠣⠀"}}, "key": "224D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠑⠄"}},
      "key": "224E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠲⠋⠄"}}, "key": "224F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠴⠄"}},
      "key": "2250"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠂⠄"}}, "key": "2251"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠆⠄"}},
      "key": "2252"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠒⠄"}}, "key": "2253"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠲⠄"}},
      "key": "2254"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠢⠄"}}, "key": "2255"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠖⠄"}},
      "key": "2256"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠶⠄"}}, "key": "2257"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠦⠄"}},
      "key": "2258"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠔⠄"}}, "key": "2259"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠁⠄"}},
      "key": "225A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠃⠄"}}, "key": "225B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠉⠄"}},
      "key": "225C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠙⠄"}}, "key": "225D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠑⠄"}},
      "key": "225E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠢⠋⠄"}}, "key": "225F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠌⠨⠅⠀"}},
      "key": "2260"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠸⠇⠀"}}, "key": "2261"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠌⠸⠇⠀"}},
      "key": "2262"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠖⠒⠄"}}, "key": "2263"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠐⠅⠱⠀"}},
      "key": "2264"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠂⠱⠀"}}, "key": "2265"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠐⠅⠨⠅⠀"}},
      "key": "2266"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠂⠨⠅⠀"}}, "key": "2267"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠐⠅⠌⠨⠅⠀"}},
      "key": "2268"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠂⠌⠨⠅⠀"}}, "key": "2269"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠐⠅⠈⠐⠅⠻⠀"}},
      "key": "226A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠂⠈⠨⠂⠻⠀"}}, "key": "226B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠖⠉⠄"}},
      "key": "226C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠌⠈⠣⠠⠣⠀"}}, "key": "226D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠌⠐⠅⠀"}},
      "key": "226E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠌⠨⠂⠀"}}, "key": "226F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠌⠐⠅⠱⠀"}},
      "key": "2270"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠌⠨⠂⠱⠀"}}, "key": "2271"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠐⠅⠈⠱⠀"}},
      "key": "2272"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠂⠈⠱⠀"}}, "key": "2273"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠌⠐⠅⠈⠱⠀"}},
      "key": "2274"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠌⠨⠂⠈⠱⠀"}}, "key": "2275"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠐⠅⠨⠂⠀"}},
      "key": "2276"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠂⠐⠅⠀"}}, "key": "2277"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠌⠐⠅⠨⠂⠀"}},
      "key": "2278"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠌⠨⠂⠐⠅⠀"}}, "key": "2279"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠨⠐⠅⠀"}},
      "key": "227A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠨⠐⠀"}}, "key": "227B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠨⠐⠅⠱⠀"}},
      "key": "227C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠨⠐⠱⠀"}}, "key": "227D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠨⠐⠅⠈⠱⠀"}},
      "key": "227E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠨⠐⠈⠱⠀"}}, "key": "227F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠌⠨⠐⠅⠀"}},
      "key": "2280"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠌⠨⠐⠅⠀"}}, "key": "2281"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠸⠐⠅⠀"}},
      "key": "2282"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠸⠨⠂⠀"}}, "key": "2283"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠌⠸⠐⠅⠀"}},
      "key": "2284"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠌⠸⠨⠂⠀"}}, "key": "2285"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠸⠀⠅⠱"}},
      "key": "2286"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠂⠱⠀"}}, "key": "2287"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠌⠀⠅⠱⠀"}},
      "key": "2288"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠌⠨⠂⠱⠀"}}, "key": "2289"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠸⠐⠅⠌⠨⠅⠀"}},
      "key": "228A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠸⠨⠂⠌⠨⠅⠀"}}, "key": "228B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠦⠉⠄"}},
      "key": "228C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠦⠙⠄"}}, "key": "228D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠦⠑⠄"}},
      "key": "228E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠦⠋⠄"}}, "key": "228F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠔⠴⠄"}},
      "key": "2290"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠔⠂⠄"}}, "key": "2291"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠔⠆⠄"}},
      "key": "2292"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠔⠒⠄"}}, "key": "2293"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠔⠲⠄"}},
      "key": "2294"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠉⠸⠫⠬⠻"}}, "key": "2295"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠉⠸⠫⠤⠻"}},
      "key": "2296"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠉⠸⠫⠈⠡⠻"}}, "key": "2297"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠉⠸⠫⠸⠌⠻"}},
      "key": "2298"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠉⠸⠫⠡⠻"}}, "key": "2299"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠉⠸⠫⠘⠨⠡⠻"}},
      "key": "229A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠉⠸⠫⠈⠼⠻"}}, "key": "229B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠉⠸⠫⠨⠅⠻"}},
      "key": "229C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠉⠸⠫⠤⠤⠻"}}, "key": "229D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠲⠸⠫⠬⠻"}},
      "key": "229E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠲⠸⠫⠤⠻"}}, "key": "229F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠲⠸⠫⠈⠡⠻"}},
      "key": "22A0"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠲⠸⠫⠡⠻"}}, "key": "22A1"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠆⠄"}},
      "key": "22A2"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠒⠄"}}, "key": "22A3"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠲⠄"}},
      "key": "22A4"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠫⠏⠀"}}, "key": "22A5"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠖⠄"}},
      "key": "22A6"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠶⠄"}}, "key": "22A7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠦⠄"}},
      "key": "22A8"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠔⠄"}}, "key": "22A9"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠁⠄"}},
      "key": "22AA"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠃⠄"}}, "key": "22AB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠉⠄"}},
      "key": "22AC"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠙⠄"}}, "key": "22AD"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠑⠄"}},
      "key": "22AE"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠁⠋⠄"}}, "key": "22AF"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠴⠄"}},
      "key": "22B0"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠂⠄"}}, "key": "22B1"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠆⠄"}},
      "key": "22B2"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠒⠄"}}, "key": "22B3"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠲⠄"}},
      "key": "22B4"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠢⠄"}}, "key": "22B5"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠖⠄"}},
      "key": "22B6"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠶⠄"}}, "key": "22B7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠦⠄"}},
      "key": "22B8"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠔⠄"}}, "key": "22B9"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠁⠄"}},
      "key": "22BA"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠃⠄"}}, "key": "22BB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠉⠄"}},
      "key": "22BC"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠙⠄"}}, "key": "22BD"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠃⠋⠄"}},
      "key": "22BF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠈⠩"}}, "key": "22C0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠈⠬"}},
      "key": "22C1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠨⠩"}}, "key": "22C2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠨⠬"}},
      "key": "22C3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠉⠲⠄"}}, "key": "22C4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠡"}},
      "key": "22C5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠉⠖⠄"}}, "key": "22C6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠉⠶⠄"}},
      "key": "22C7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠉⠦⠄"}}, "key": "22C8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠉⠔⠄"}},
      "key": "22C9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠉⠁⠄"}}, "key": "22CA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠉⠃⠄"}},
      "key": "22CB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠉⠉⠄"}}, "key": "22CC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠉⠙⠄"}},
      "key": "22CD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠉⠑⠄"}}, "key": "22CE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠉⠋⠄"}},
      "key": "22CF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠴⠄"}}, "key": "22D0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠂⠄"}},
      "key": "22D1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠆⠄"}}, "key": "22D2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠒⠄"}},
      "key": "22D3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠲⠄"}}, "key": "22D4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠢⠄"}},
      "key": "22D5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠖⠄"}}, "key": "22D6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠶⠄"}},
      "key": "22D7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠦⠄"}}, "key": "22D8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠔⠄"}},
      "key": "22D9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠁⠄"}}, "key": "22DA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠃⠄"}},
      "key": "22DB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠱⠐⠅⠀"}}, "key": "22DC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠱⠨⠂⠀"}},
      "key": "22DD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠑⠄"}}, "key": "22DE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠙⠋⠄"}},
      "key": "22DF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠴⠄"}}, "key": "22E0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠂⠄"}},
      "key": "22E1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠆⠄"}}, "key": "22E2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠒⠄"}},
      "key": "22E3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠲⠄"}}, "key": "22E4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠢⠄"}},
      "key": "22E5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠖⠄"}}, "key": "22E6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠶⠄"}},
      "key": "22E7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠦⠄"}}, "key": "22E8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠔⠄"}},
      "key": "22E9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠁⠄"}}, "key": "22EA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠃⠄"}},
      "key": "22EB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠉⠄"}}, "key": "22EC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠑⠙⠄"}},
      "key": "22ED"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠣⠄⠄⠄"}}, "key": "22EE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⠄⠄"}},
      "key": "22EF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠘⠄⠄⠄"}}, "key": "22F0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠰⠄⠄⠄"}},
      "key": "22F1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠆⠄"}}, "key": "22F2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠒⠄"}},
      "key": "22F3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠲⠄"}}, "key": "22F4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠢⠄"}},
      "key": "22F5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠖⠄"}}, "key": "22F6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠶⠄"}},
      "key": "22F7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠦⠄"}}, "key": "22F8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠔⠄"}},
      "key": "22F9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠁⠄"}}, "key": "22FA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠃⠄"}},
      "key": "22FB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠉⠄"}}, "key": "22FC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠙⠄"}},
      "key": "22FD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠑⠄"}}, "key": "22FE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠆⠋⠋⠄"}},
      "key": "22FF"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠴⠴⠄"}}, "key": "2300"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠴⠆⠄"}},
      "key": "2302"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠴⠢⠄"}}, "key": "2305"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠴⠖⠄"}},
      "key": "2306"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠴⠶⠄"}}, "key": "2307"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠂⠴⠄"}},
      "key": "2310"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠂⠂⠄"}}, "key": "2311"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠂⠆⠄"}},
      "key": "2312"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠂⠒⠄"}}, "key": "2313"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠂⠲⠄"}},
      "key": "2314"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠔⠢⠄"}}, "key": "2795"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠔⠖⠄"}},
      "key": "2796"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠔⠶⠄"}}, "key": "2797"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠴⠄"}},
      "key": "27B0"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠃⠋⠄"}}, "key": "27BF"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠂⠄"}},
      "key": "27C1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠏"}}, "key": "27C2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠒⠄"}},
      "key": "27C3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠲⠄"}}, "key": "27C4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠶⠄"}},
      "key": "27C7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠦⠄"}}, "key": "27C8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠔⠄"}},
      "key": "27C9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠁⠄"}}, "key": "27CA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠃⠄"}},
      "key": "27CB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠉⠄"}}, "key": "27CC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠙⠄"}},
      "key": "27CD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠑⠄"}}, "key": "27CE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠉⠋⠄"}},
      "key": "27CF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠴⠄"}}, "key": "27D0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠂⠄"}},
      "key": "27D1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠆⠄"}}, "key": "27D2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠒⠄"}},
      "key": "27D3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠲⠄"}}, "key": "27D4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠢⠄"}},
      "key": "27D5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠖⠄"}}, "key": "27D6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠶⠄"}},
      "key": "27D7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠦⠄"}}, "key": "27D8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠔⠄"}},
      "key": "27D9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠁⠄"}}, "key": "27DA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠃⠄"}},
      "key": "27DB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠉⠄"}}, "key": "27DC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠙⠄"}},
      "key": "27DD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠑⠄"}}, "key": "27DE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠙⠋⠄"}},
      "key": "27DF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠑⠴⠄"}}, "key": "27E0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠑⠂⠄"}},
      "key": "27E1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠑⠆⠄"}}, "key": "27E2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠑⠒⠄"}},
      "key": "27E3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠑⠲⠄"}}, "key": "27E4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠑⠢⠄"}},
      "key": "27E5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠃⠄"}}, "key": "292B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠆⠉⠄"}},
      "key": "292C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠳⠳⠳"}}, "key": "2980"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠂⠄"}},
      "key": "2981"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠦⠆⠄"}}, "key": "2982"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠔⠄"}},
      "key": "2999"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠔⠁⠄"}}, "key": "299A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠃⠴⠄"}},
      "key": "29B0"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠃⠂⠄"}}, "key": "29B1"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠃⠆⠄"}},
      "key": "29B2"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠃⠢⠄"}}, "key": "29B5"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠉⠸⠫⠳⠻"}},
      "key": "29B6"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠉⠸⠫⠫⠇⠻"}}, "key": "29B7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠉⠸⠫⠸⠡⠻"}},
      "key": "29B8"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠉⠸⠫⠫⠏⠻"}}, "key": "29B9"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠃⠁⠄"}},
      "key": "29BA"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠃⠃⠄"}}, "key": "29BB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠃⠉⠄"}},
      "key": "29BC"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠃⠑⠄"}}, "key": "29BE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠉⠸⠫⠔⠔⠻"}},
      "key": "29BF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠉⠸⠫⠐⠅⠻"}}, "key": "29C0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠉⠸⠫⠨⠂⠻"}},
      "key": "29C1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠆⠄"}}, "key": "29C2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠒⠄"}},
      "key": "29C3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠲⠄"}}, "key": "29C4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠢⠄"}},
      "key": "29C5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠖⠄"}}, "key": "29C6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠶⠄"}},
      "key": "29C7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠦⠄"}}, "key": "29C8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠔⠄"}},
      "key": "29C9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠁⠄"}}, "key": "29CA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠃⠄"}},
      "key": "29CB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠉⠄"}}, "key": "29CC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠙⠄"}},
      "key": "29CD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠑⠄"}}, "key": "29CE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠉⠋⠄"}},
      "key": "29CF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠴⠄"}}, "key": "29D0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠂⠄"}},
      "key": "29D1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠆⠄"}}, "key": "29D2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠒⠄"}},
      "key": "29D3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠲⠄"}}, "key": "29D4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠢⠄"}},
      "key": "29D5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠖⠄"}}, "key": "29D6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠶⠄"}},
      "key": "29D7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠉⠄"}}, "key": "29DC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠙⠄"}},
      "key": "29DD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠑⠄"}}, "key": "29DE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠙⠋⠄"}},
      "key": "29DF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠴⠄"}}, "key": "29E0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠂⠄"}},
      "key": "29E1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠆⠄"}}, "key": "29E2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠒⠄"}},
      "key": "29E3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠲⠄"}}, "key": "29E4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠢⠄"}},
      "key": "29E5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠖⠄"}}, "key": "29E6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠶⠄"}},
      "key": "29E7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠦⠄"}}, "key": "29E8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠔⠄"}},
      "key": "29E9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠃⠄"}}, "key": "29EB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠑⠄"}},
      "key": "29EE"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠑⠋⠄"}}, "key": "29EF"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠴⠄"}},
      "key": "29F0"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠂⠄"}}, "key": "29F1"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠆⠄"}},
      "key": "29F2"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠒⠄"}}, "key": "29F3"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠲⠄"}},
      "key": "29F4"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠢⠄"}}, "key": "29F5"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠖⠄"}},
      "key": "29F6"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠶⠄"}}, "key": "29F7"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠦⠄"}},
      "key": "29F8"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠔⠄"}}, "key": "29F9"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠁⠄"}},
      "key": "29FA"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠃⠄"}}, "key": "29FB"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠑⠄"}},
      "key": "29FE"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠔⠋⠋⠄"}}, "key": "29FF"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠉⠸⠫⠡⠻"}},
      "key": "2A00"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠉⠸⠫⠬⠻"}}, "key": "2A01"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠫⠉⠸⠫⠈⠡⠻"}},
      "key": "2A02"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠒⠄"}}, "key": "2A03"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠲⠄"}},
      "key": "2A04"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠢⠄"}}, "key": "2A05"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠖⠄"}},
      "key": "2A06"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠶⠄"}}, "key": "2A07"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠦⠄"}},
      "key": "2A08"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠔⠄"}}, "key": "2A09"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠁⠄"}},
      "key": "2A0A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠃⠄"}}, "key": "2A0B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠉⠄"}},
      "key": "2A0C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠙⠄"}}, "key": "2A0D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠑⠄"}},
      "key": "2A0E"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠴⠋⠄"}}, "key": "2A0F"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠴⠄"}},
      "key": "2A10"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠂⠄"}}, "key": "2A11"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠆⠄"}},
      "key": "2A12"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠒⠄"}}, "key": "2A13"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠲⠄"}},
      "key": "2A14"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠢⠄"}}, "key": "2A15"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠖⠄"}},
      "key": "2A16"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠦⠄"}}, "key": "2A18"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠔⠄"}},
      "key": "2A19"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠁⠄"}}, "key": "2A1A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠃⠄"}},
      "key": "2A1B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠉⠄"}}, "key": "2A1C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠙⠄"}},
      "key": "2A1D"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠑⠄"}}, "key": "2A1E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠂⠋⠄"}},
      "key": "2A1F"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠴⠄"}}, "key": "2A20"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠂⠄"}},
      "key": "2A21"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠆⠄"}}, "key": "2A22"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠒⠄"}},
      "key": "2A23"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠲⠄"}}, "key": "2A24"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠢⠄"}},
      "key": "2A25"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠖⠄"}}, "key": "2A26"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠶⠄"}},
      "key": "2A27"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠦⠄"}}, "key": "2A28"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠔⠄"}},
      "key": "2A29"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠁⠄"}}, "key": "2A2A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠃⠄"}},
      "key": "2A2B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠉⠄"}}, "key": "2A2C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠙⠄"}},
      "key": "2A2D"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠑⠄"}}, "key": "2A2E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠆⠋⠄"}},
      "key": "2A2F"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠴⠄"}}, "key": "2A30"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠂⠄"}},
      "key": "2A31"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠆⠄"}}, "key": "2A32"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠒⠄"}},
      "key": "2A33"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠲⠄"}}, "key": "2A34"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠢⠄"}},
      "key": "2A35"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠖⠄"}}, "key": "2A36"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠶⠄"}},
      "key": "2A37"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠫⠉⠸⠫⠈⠌⠻"}}, "key": "2A38"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠔⠄"}},
      "key": "2A39"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠁⠄"}}, "key": "2A3A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠃⠄"}},
      "key": "2A3B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠉⠄"}}, "key": "2A3C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠙⠄"}},
      "key": "2A3D"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠑⠄"}}, "key": "2A3E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠒⠋⠄"}},
      "key": "2A3F"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠴⠄"}}, "key": "2A40"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠂⠄"}},
      "key": "2A41"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠆⠄"}}, "key": "2A42"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠒⠄"}},
      "key": "2A43"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠲⠄"}}, "key": "2A44"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠢⠄"}},
      "key": "2A45"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠖⠄"}}, "key": "2A46"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠶⠄"}},
      "key": "2A47"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠦⠄"}}, "key": "2A48"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠔⠄"}},
      "key": "2A49"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠁⠄"}}, "key": "2A4A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠃⠄"}},
      "key": "2A4B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠉⠄"}}, "key": "2A4C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠙⠄"}},
      "key": "2A4D"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠑⠄"}}, "key": "2A4E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠲⠋⠄"}},
      "key": "2A4F"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠴⠄"}}, "key": "2A50"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠂⠄"}},
      "key": "2A51"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠆⠄"}}, "key": "2A52"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠒⠄"}},
      "key": "2A53"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠲⠄"}}, "key": "2A54"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠢⠄"}},
      "key": "2A55"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠖⠄"}}, "key": "2A56"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠶⠄"}},
      "key": "2A57"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠦⠄"}}, "key": "2A58"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠔⠄"}},
      "key": "2A59"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠁⠄"}}, "key": "2A5A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠃⠄"}},
      "key": "2A5B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠉⠄"}}, "key": "2A5C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠙⠄"}},
      "key": "2A5D"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠑⠄"}}, "key": "2A5E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠢⠋⠄"}},
      "key": "2A5F"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠴⠄"}}, "key": "2A60"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠂⠄"}},
      "key": "2A61"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠆⠄"}}, "key": "2A62"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠒⠄"}},
      "key": "2A63"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠲⠄"}}, "key": "2A64"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠢⠄"}},
      "key": "2A65"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠖⠄"}}, "key": "2A66"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠶⠄"}},
      "key": "2A67"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠦⠄"}}, "key": "2A68"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠔⠄"}},
      "key": "2A69"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠁⠄"}}, "key": "2A6A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠃⠄"}},
      "key": "2A6B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠉⠄"}}, "key": "2A6C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠙⠄"}},
      "key": "2A6D"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠑⠄"}}, "key": "2A6E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠖⠋⠄"}},
      "key": "2A6F"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠴⠄"}}, "key": "2A70"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠂⠄"}},
      "key": "2A71"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠆⠄"}}, "key": "2A72"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠒⠄"}},
      "key": "2A73"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠲⠄"}}, "key": "2A74"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠢⠄"}},
      "key": "2A75"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠖⠄"}}, "key": "2A76"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠶⠄"}},
      "key": "2A77"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠦⠄"}}, "key": "2A78"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠔⠄"}},
      "key": "2A79"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠁⠄"}}, "key": "2A7A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠃⠄"}},
      "key": "2A7B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠉⠄"}}, "key": "2A7C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠙⠄"}},
      "key": "2A7D"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠑⠄"}}, "key": "2A7E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠶⠋⠄"}},
      "key": "2A7F"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠴⠄"}}, "key": "2A80"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠂⠄"}},
      "key": "2A81"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠆⠄"}}, "key": "2A82"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠒⠄"}},
      "key": "2A83"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠲⠄"}}, "key": "2A84"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠢⠄"}},
      "key": "2A85"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠖⠄"}}, "key": "2A86"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠶⠄"}},
      "key": "2A87"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠦⠄"}}, "key": "2A88"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠔⠄"}},
      "key": "2A89"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠁⠄"}}, "key": "2A8A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠃⠄"}},
      "key": "2A8B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠉⠄"}}, "key": "2A8C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠙⠄"}},
      "key": "2A8D"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠑⠄"}}, "key": "2A8E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠦⠋⠄"}},
      "key": "2A8F"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠴⠄"}}, "key": "2A90"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠂⠄"}},
      "key": "2A91"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠆⠄"}}, "key": "2A92"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠒⠄"}},
      "key": "2A93"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠲⠄"}}, "key": "2A94"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠢⠄"}},
      "key": "2A95"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠖⠄"}}, "key": "2A96"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠶⠄"}},
      "key": "2A97"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠦⠄"}}, "key": "2A98"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠔⠄"}},
      "key": "2A99"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠁⠄"}}, "key": "2A9A"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠃⠄"}},
      "key": "2A9B"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠉⠄"}}, "key": "2A9C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠙⠄"}},
      "key": "2A9D"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠑⠄"}}, "key": "2A9E"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠔⠋⠄"}},
      "key": "2A9F"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠴⠄"}}, "key": "2AA0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠂⠄"}},
      "key": "2AA1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠆⠄"}}, "key": "2AA2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠒⠄"}},
      "key": "2AA3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠲⠄"}}, "key": "2AA4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠢⠄"}},
      "key": "2AA5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠖⠄"}}, "key": "2AA6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠶⠄"}},
      "key": "2AA7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠦⠄"}}, "key": "2AA8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠔⠄"}},
      "key": "2AA9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠁⠄"}}, "key": "2AAA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠃⠄"}},
      "key": "2AAB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠉⠄"}}, "key": "2AAC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠙⠄"}},
      "key": "2AAD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠁⠑⠄"}}, "key": "2AAE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠨⠐⠅⠱⠀"}},
      "key": "2AAF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠨⠐⠱⠀"}}, "key": "2AB0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠨⠐⠅⠌⠱⠀"}},
      "key": "2AB1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠨⠐⠌⠱⠀"}}, "key": "2AB2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠨⠐⠅⠨⠅⠀"}},
      "key": "2AB3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠨⠐⠨⠅⠀"}}, "key": "2AB4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠨⠐⠅⠌⠨⠅⠀"}},
      "key": "2AB5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠨⠐⠌⠨⠅⠀"}}, "key": "2AB6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠨⠐⠅⠈⠱⠈⠱⠀"}},
      "key": "2AB7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠨⠐⠈⠱⠈⠱⠀"}}, "key": "2AB8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠨⠐⠅⠌⠈⠱⠈⠱⠀"}},
      "key": "2AB9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠨⠐⠌⠈⠱⠈⠱⠀"}}, "key": "2ABA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠀⠨⠐⠅⠈⠨⠐⠅⠻⠀"}},
      "key": "2ABB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠀⠨⠨⠐⠈⠨⠨⠐⠻⠀"}}, "key": "2ABC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠃⠙⠄"}},
      "key": "2ABD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠃⠑⠄"}}, "key": "2ABE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠃⠋⠄"}},
      "key": "2ABF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠴⠄"}}, "key": "2AC0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠂⠄"}},
      "key": "2AC1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠆⠄"}}, "key": "2AC2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠒⠄"}},
      "key": "2AC3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠲⠄"}}, "key": "2AC4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠢⠄"}},
      "key": "2AC5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠖⠄"}}, "key": "2AC6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠶⠄"}},
      "key": "2AC7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠦⠄"}}, "key": "2AC8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠔⠄"}},
      "key": "2AC9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠁⠄"}}, "key": "2ACA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠃⠄"}},
      "key": "2ACB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠉⠄"}}, "key": "2ACC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠙⠄"}},
      "key": "2ACD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠑⠄"}}, "key": "2ACE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠉⠋⠄"}},
      "key": "2ACF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠴⠄"}}, "key": "2AD0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠂⠄"}},
      "key": "2AD1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠆⠄"}}, "key": "2AD2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠒⠄"}},
      "key": "2AD3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠲⠄"}}, "key": "2AD4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠢⠄"}},
      "key": "2AD5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠖⠄"}}, "key": "2AD6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠶⠄"}},
      "key": "2AD7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠦⠄"}}, "key": "2AD8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠔⠄"}},
      "key": "2AD9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠁⠄"}}, "key": "2ADA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠃⠄"}},
      "key": "2ADB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠉⠄"}}, "key": "2ADC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠙⠄"}},
      "key": "2ADD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠑⠄"}}, "key": "2ADE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠙⠋⠄"}},
      "key": "2ADF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠴⠄"}}, "key": "2AE0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠂⠄"}},
      "key": "2AE1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠆⠄"}}, "key": "2AE2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠒⠄"}},
      "key": "2AE3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠲⠄"}}, "key": "2AE4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠢⠄"}},
      "key": "2AE5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠖⠄"}}, "key": "2AE6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠶⠄"}},
      "key": "2AE7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠦⠄"}}, "key": "2AE8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠔⠄"}},
      "key": "2AE9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠁⠄"}}, "key": "2AEA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠃⠄"}},
      "key": "2AEB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠉⠄"}}, "key": "2AEC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠙⠄"}},
      "key": "2AED"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠑⠄"}}, "key": "2AEE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠑⠋⠄"}},
      "key": "2AEF"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠴⠄"}}, "key": "2AF0"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠂⠄"}},
      "key": "2AF1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠆⠄"}}, "key": "2AF2"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠒⠄"}},
      "key": "2AF3"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠲⠄"}}, "key": "2AF4"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠢⠄"}},
      "key": "2AF5"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠖⠄"}}, "key": "2AF6"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠶⠄"}},
      "key": "2AF7"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠦⠄"}}, "key": "2AF8"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠔⠄"}},
      "key": "2AF9"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠁⠄"}}, "key": "2AFA"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠃⠄"}},
      "key": "2AFB"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠉⠄"}}, "key": "2AFC"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠙⠄"}},
      "key": "2AFD"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠑⠄"}}, "key": "2AFE"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠁⠋⠋⠄"}},
      "key": "2AFF"
    }, {"category": "Pd", "mappings": {"default": {"default": "⠄⡳⠭⠒⠴⠂⠉⠄"}}, "key": "301C"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠂⠴⠄"}},
      "key": "FE10"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠂⠒⠄"}}, "key": "FE13"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠂⠲⠄"}},
      "key": "FE14"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠂⠢⠄"}}, "key": "FE15"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠂⠖⠄"}},
      "key": "FE16"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠂⠔⠄"}}, "key": "FE19"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠴⠄"}},
      "key": "FE30"
    }, {"category": "Pd", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠂⠄"}}, "key": "FE31"}, {
      "category": "Pd",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠆⠄"}},
      "key": "FE32"
    }, {"category": "Pc", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠒⠄"}}, "key": "FE33"}, {
      "category": "Pc",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠒⠲⠄"}},
      "key": "FE34"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠢⠄"}}, "key": "FE45"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠖⠄"}},
      "key": "FE46"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠔⠄"}}, "key": "FE49"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠁⠄"}},
      "key": "FE4A"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠃⠄"}}, "key": "FE4B"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠉⠄"}},
      "key": "FE4C"
    }, {"category": "Pc", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠙⠄"}}, "key": "FE4D"}, {
      "category": "Pc",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠑⠄"}},
      "key": "FE4E"
    }, {"category": "Pc", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠲⠋⠄"}}, "key": "FE4F"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠴⠄"}},
      "key": "FE50"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠆⠄"}}, "key": "FE52"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠲⠄"}},
      "key": "FE54"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠢⠄"}}, "key": "FE55"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠖⠄"}},
      "key": "FE56"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠶⠄"}}, "key": "FE57"}, {
      "category": "Pd",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠦⠄"}},
      "key": "FE58"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠢⠋⠄"}}, "key": "FE5F"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠖⠴⠄"}},
      "key": "FE60"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠖⠂⠄"}}, "key": "FE61"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠖⠆⠄"}},
      "key": "FE62"
    }, {"category": "Pd", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠖⠒⠄"}}, "key": "FE63"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠖⠲⠄"}},
      "key": "FE64"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠖⠢⠄"}}, "key": "FE65"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠖⠖⠄"}},
      "key": "FE66"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠖⠦⠄"}}, "key": "FE68"}, {
      "category": "Sc",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠖⠔⠄"}},
      "key": "FE69"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠖⠁⠄"}}, "key": "FE6A"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠖⠃⠄"}},
      "key": "FE6B"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠂⠄"}}, "key": "FF01"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠆⠄"}},
      "key": "FF02"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠒⠄"}}, "key": "FF03"}, {
      "category": "Sc",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠲⠄"}},
      "key": "FF04"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠢⠄"}}, "key": "FF05"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠖⠄"}},
      "key": "FF06"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠶⠄"}}, "key": "FF07"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠁⠄"}},
      "key": "FF0A"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠃⠄"}}, "key": "FF0B"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠉⠄"}},
      "key": "FF0C"
    }, {"category": "Pd", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠙⠄"}}, "key": "FF0D"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠑⠄"}},
      "key": "FF0E"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠴⠋⠄"}}, "key": "FF0F"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠂⠁⠄"}},
      "key": "FF1A"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠂⠃⠄"}}, "key": "FF1B"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠂⠉⠄"}},
      "key": "FF1C"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠂⠙⠄"}}, "key": "FF1D"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠂⠑⠄"}},
      "key": "FF1E"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠂⠋⠄"}}, "key": "FF1F"}, {
      "category": "Po",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠆⠴⠄"}},
      "key": "FF20"
    }, {"category": "Po", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠒⠉⠄"}}, "key": "FF3C"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠒⠑⠄"}},
      "key": "FF3E"
    }, {"category": "Pc", "mappings": {"default": {"default": "⠱"}}, "key": "FF3F"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠲⠴⠄"}},
      "key": "FF40"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠢⠉⠄"}}, "key": "FF5C"}, {
      "category": "Sm",
      "mappings": {"default": {"default": "⠈⠱"}},
      "key": "FF5E"
    }, {"category": "Sc", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠴⠄"}}, "key": "FFE0"}, {
      "category": "Sc",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠂⠄"}},
      "key": "FFE1"
    }, {"category": "Sm", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠆⠄"}}, "key": "FFE2"}, {
      "category": "Sk",
      "mappings": {"default": {"default": "⠱"}},
      "key": "FFE3"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠲⠄"}}, "key": "FFE4"}, {
      "category": "Sc",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠢⠄"}},
      "key": "FFE5"
    }, {"category": "Sc", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠖⠄"}}, "key": "FFE6"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠦⠄"}},
      "key": "FFE8"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠙⠄"}}, "key": "FFED"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠑⠑⠄"}},
      "key": "FFEE"
    }]
  ,
  "nemeth/symbols/math_whitespace.js":
    [{"locale": "nemeth"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "⠀"}},
      "key": "0020"
    }, {"category": "Zs", "mappings": {"default": {"default": "⠀"}}, "key": "00A0"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "⠤"}},
      "key": "00AD"
    }, {"category": "Zs", "mappings": {"default": {"default": "⠀"}}, "key": "2000"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "⠀"}},
      "key": "2001"
    }, {"category": "Zs", "mappings": {"default": {"default": "⠀"}}, "key": "2002"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "⠀"}},
      "key": "2003"
    }, {"category": "Zs", "mappings": {"default": {"default": "⠀"}}, "key": "2004"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "⠀"}},
      "key": "2005"
    }, {"category": "Zs", "mappings": {"default": {"default": "⠀"}}, "key": "2006"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "⠀"}},
      "key": "2007"
    }, {"category": "Zs", "mappings": {"default": {"default": "⠀"}}, "key": "2008"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "⠀"}},
      "key": "2009"
    }, {"category": "Zs", "mappings": {"default": {"default": "⠀"}}, "key": "200A"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "⠀"}},
      "key": "200B"
    }, {"category": "Cf", "mappings": {"default": {"default": ""}}, "key": "200C"}, {
      "category": "Cf",
      "mappings": {"default": {"default": ""}},
      "key": "200D"
    }, {"category": "Cf", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠴⠑⠄"}}, "key": "200E"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠴⠋⠄"}},
      "key": "200F"
    }, {"category": "Zl", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠆⠦⠄"}}, "key": "2028"}, {
      "category": "Zp",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠆⠔⠄"}},
      "key": "2029"
    }, {"category": "Cf", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠆⠁⠄"}}, "key": "202A"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠆⠃⠄"}},
      "key": "202B"
    }, {"category": "Cf", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠆⠉⠄"}}, "key": "202C"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠆⠙⠄"}},
      "key": "202D"
    }, {"category": "Cf", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠆⠑⠄"}}, "key": "202E"}, {
      "category": "Zs",
      "mappings": {"default": {"default": "⠀"}},
      "key": "202F"
    }, {"category": "Zs", "mappings": {"default": {"default": "⠀"}}, "key": "205F"}, {
      "category": "Cf",
      "mappings": {"default": {"default": ""}},
      "key": "2060"
    }, {"category": "Cf", "mappings": {"default": {"default": "⠀"}}, "key": "2061"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "⠈⠡"}},
      "key": "2062"
    }, {"category": "Cf", "mappings": {"default": {"default": "⠠"}}, "key": "2063"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "⠬"}},
      "key": "2064"
    }, {"category": "Cf", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠖⠁⠄"}}, "key": "206A"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠖⠃⠄"}},
      "key": "206B"
    }, {"category": "Cf", "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠖⠑⠄"}}, "key": "206E"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠴⠖⠋⠄"}},
      "key": "206F"
    }, {"category": "Cf", "mappings": {"default": {"default": "⠄⡳⠭⠋⠑⠋⠋⠄"}}, "key": "FEFF"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠋⠔⠄"}},
      "key": "FFF9"
    }, {"category": "Cf", "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠋⠁⠄"}}, "key": "FFFA"}, {
      "category": "Cf",
      "mappings": {"default": {"default": "⠄⡳⠭⠋⠋⠋⠃⠄"}},
      "key": "FFFB"
    }]
  ,
  "nemeth/symbols/other_stars.js":
    [{"locale": "nemeth"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠒⠑⠦⠄"}},
      "key": "23E8"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠖⠴⠢⠄"}}, "key": "2605"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠫⠎"}},
      "key": "2606"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠖⠁⠁⠄"}}, "key": "26AA"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠖⠁⠃⠄"}},
      "key": "26AB"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠴⠢⠄"}}, "key": "2705"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠂⠒⠄"}},
      "key": "2713"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠂⠲⠄"}}, "key": "2714"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠈⠡"}},
      "key": "2715"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠂⠖⠄"}}, "key": "2716"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠂⠶⠄"}},
      "key": "2717"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠂⠦⠄"}}, "key": "2718"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠂⠃⠄"}},
      "key": "271B"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠂⠉⠄"}}, "key": "271C"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠴⠄"}},
      "key": "2720"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠂⠄"}}, "key": "2721"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠆⠄"}},
      "key": "2722"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠒⠄"}}, "key": "2723"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠲⠄"}},
      "key": "2724"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠢⠄"}}, "key": "2725"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠖⠄"}},
      "key": "2726"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠶⠄"}}, "key": "2727"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠦⠄"}},
      "key": "2728"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠔⠄"}}, "key": "2729"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠁⠄"}},
      "key": "272A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠃⠄"}}, "key": "272B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠉⠄"}},
      "key": "272C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠙⠄"}}, "key": "272D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠑⠄"}},
      "key": "272E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠆⠋⠄"}}, "key": "272F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠴⠄"}},
      "key": "2730"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠂⠄"}}, "key": "2731"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠆⠄"}},
      "key": "2732"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠒⠄"}}, "key": "2733"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠲⠄"}},
      "key": "2734"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠢⠄"}}, "key": "2735"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠖⠄"}},
      "key": "2736"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠔⠄"}}, "key": "2739"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠁⠄"}},
      "key": "273A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠃⠄"}}, "key": "273B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠉⠄"}},
      "key": "273C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠙⠄"}}, "key": "273D"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠑⠄"}},
      "key": "273E"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠒⠋⠄"}}, "key": "273F"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠴⠄"}},
      "key": "2740"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠂⠄"}}, "key": "2741"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠆⠄"}},
      "key": "2742"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠒⠄"}}, "key": "2743"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠲⠄"}},
      "key": "2744"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠢⠄"}}, "key": "2745"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠖⠄"}},
      "key": "2746"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠶⠄"}}, "key": "2747"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠦⠄"}},
      "key": "2748"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠔⠄"}}, "key": "2749"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠁⠄"}},
      "key": "274A"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠃⠄"}}, "key": "274B"}, {
      "category": "So",
      "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠉⠄"}},
      "key": "274C"
    }, {"category": "So", "mappings": {"default": {"default": "⠄⡳⠭⠆⠶⠲⠙⠄"}}, "key": "274D"}]
  ,
  "nemeth/units/energy.js":
    [{"locale": "nemeth"}, {
      "category": "energy",
      "mappings": {"default": {"default": "⠠⠺"}},
      "key": "W",
      "names": ["W", "w"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠅⠠⠺"}},
      "key": "kW",
      "names": ["kw", "kW"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠍⠠⠺"}},
      "key": "mW",
      "names": ["mw", "mW"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠅⠺⠓"}},
      "key": "kwh",
      "names": ["kwh", "kWh"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠠⠚"}},
      "key": "J",
      "names": ["J"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠠⠝"}},
      "key": "N",
      "names": ["N"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠠⠁"}},
      "key": "A",
      "names": ["A"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠠⠧"}},
      "key": "V",
      "names": ["V"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠨⠍⠨⠠⠺"}},
      "key": "µΩ",
      "names": ["µΩ"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠍⠨⠠⠺"}},
      "key": "mΩ",
      "names": ["mΩ"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠨⠠⠺"}},
      "key": "Ω",
      "names": ["Ω", "Ohm"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠅⠨⠠⠺"}},
      "key": "kΩ",
      "names": ["kΩ", "KΩ"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠨⠠⠺"}},
      "key": "Ω",
      "names": ["Ω"]
    }, {
      "category": "energy",
      "mappings": {"default": {"default": "⠍⠨⠠⠺"}},
      "key": "MΩ",
      "names": ["MΩ"]
    }, {"category": "energy", "mappings": {"default": {"default": "⠛⠨⠠⠺"}}, "key": "GΩ", "names": ["GΩ"]}]
  ,
  "nemeth/units/length.js":
    [{"locale": "nemeth"}, {
      "category": "Length",
      "mappings": {"default": {"default": "⠍⠍"}},
      "key": "mm",
      "names": ["mm"]
    }, {
      "category": "Length",
      "mappings": {"default": {"default": "⠉⠍"}},
      "key": "cm",
      "names": ["cm"]
    }, {
      "category": "Length",
      "mappings": {"default": {"default": "⠍"}},
      "key": "m",
      "names": ["m"]
    }, {
      "category": "Length",
      "mappings": {"default": {"default": "⠅⠍"}},
      "key": "km",
      "names": ["km"]
    }, {
      "category": "Length",
      "mappings": {"default": {"default": "⠋⠞"}},
      "key": "ft",
      "names": ["ft", "ft."]
    }, {
      "category": "Length",
      "mappings": {"default": {"default": "⠊⠝"}},
      "key": "in",
      "names": ["in", "in."]
    }, {
      "category": "Length",
      "mappings": {"default": {"default": "⠍⠊"}},
      "key": "mi",
      "names": ["mi", "mi."]
    }, {
      "category": "Length",
      "mappings": {"default": {"default": "⠽⠙"}},
      "key": "yd",
      "names": ["yd", "yd."]
    }, {"category": "", "mappings": {"default": {"default": "⠝⠲⠍⠸⠲"}}, "key": "n.m.", "names": ["n.m."]}]
  ,
  "nemeth/units/memory.js":
    [{"locale": "nemeth"}, {
      "category": "",
      "mappings": {"default": {"default": "⠃"}},
      "key": "b",
      "names": ["b"]
    }, {"category": "", "mappings": {"default": {"default": "⠠⠃"}}, "key": "B", "names": ["B"]}, {
      "category": "",
      "mappings": {"default": {"default": "⠅⠃"}},
      "key": "KB",
      "names": ["KB"]
    }, {"category": "", "mappings": {"default": {"default": "⠍⠃"}}, "key": "MB", "names": ["MB"]}, {
      "category": "",
      "mappings": {"default": {"default": "⠛⠃"}},
      "key": "GB",
      "names": ["GB"]
    }, {"category": "", "mappings": {"default": {"default": "⠞⠃"}}, "key": "TB", "names": ["TB"]}]
  ,
  "nemeth/units/other.js":
    [{"locale": "nemeth"}, {
      "category": "",
      "mappings": {"default": {"default": "⠙⠕⠵"}},
      "key": "doz",
      "names": ["doz", "doz.", "dz", "dz."]
    }, {
      "category": "",
      "mappings": {"default": {"default": "⠎⠟"}},
      "key": "sq",
      "names": ["sq", "sq."]
    }, {"category": "", "mappings": {"default": {"default": "⠓⠁"}}, "key": "ha", "names": ["ha"]}]
  ,
  "nemeth/units/speed.js":
    [{"locale": "nemeth"}, {
      "category": "",
      "mappings": {"default": {"default": "⠅⠞"}},
      "key": "kt",
      "names": ["kt", "kt."]
    }, {"category": "", "mappings": {"default": {"default": "⠍⠏⠓"}}, "key": "mph", "names": ["mph"]}, {
      "category": "",
      "mappings": {"default": {"default": "⠗⠏⠍"}},
      "key": "rpm",
      "names": ["rpm"]
    }, {"category": "", "mappings": {"default": {"default": "⠅⠍⠓"}}, "key": "kmh", "names": ["kmh"]}]
  ,
  "nemeth/units/temperature.js":
    [{"locale": "nemeth"}, {
      "category": "",
      "mappings": {"default": {"default": "⠠⠋"}},
      "key": "F",
      "names": ["F", "F.", "°F"]
    }, {"category": "", "mappings": {"default": {"default": "⠠⠉"}}, "key": "C", "names": ["C", "°C"]}, {
      "category": "",
      "mappings": {"default": {"default": "⠠⠅"}},
      "key": "K",
      "names": ["K", "°K"]
    }]
  ,
  "nemeth/units/time.js":
    [{"locale": "nemeth"}, {
      "category": "time",
      "mappings": {"default": {"default": "⠝⠎"}},
      "key": "ns",
      "names": ["ns"]
    }, {
      "category": "time",
      "mappings": {"default": {"default": "⠨⠍⠎"}},
      "key": "µs",
      "names": ["µs"]
    }, {
      "category": "time",
      "mappings": {"default": {"default": "⠍⠎"}},
      "key": "ms",
      "names": ["ms"]
    }, {"category": "time", "mappings": {"default": {"default": "⠎"}}, "key": "s", "names": ["s"]}, {
      "category": "time",
      "mappings": {"default": {"default": "⠍⠊⠝⠀"}},
      "key": "min",
      "names": ["min"]
    }, {"category": "time", "mappings": {"default": {"default": "⠓"}}, "key": "h", "names": ["h", "hr"]}]
  ,
  "nemeth/units/volume.js":
    [{"locale": "nemeth"}, {
      "category": "volume",
      "mappings": {"default": {"default": "⠃⠃⠇"}},
      "key": "bbl",
      "names": ["bbl.", "bbl"]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "⠉⠥"}},
      "key": "cu",
      "names": ["cu", "cu."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "⠋⠇⠸⠲⠀⠕⠵⠸⠲"}},
      "key": "fl. oz.",
      "names": ["fl. oz."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "⠛⠁⠇"}},
      "key": "gal",
      "names": ["gal", "gal."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "⠏⠞"}},
      "key": "pt",
      "names": ["pt", "pt."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "⠟⠞"}},
      "key": "qt",
      "names": ["qt", "qt."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "⠞⠃⠎⠏"}},
      "key": "tbsp",
      "names": ["tbsp", "tbsp."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "⠞⠎⠏"}},
      "key": "tsp",
      "names": ["tsp", "tsp."]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "⠉⠉"}},
      "key": "cc",
      "names": ["cc"]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "⠅⠇"}},
      "key": "kl",
      "names": ["kl"]
    }, {
      "category": "volume",
      "mappings": {"default": {"default": "⠇"}},
      "key": "l",
      "names": ["l"]
    }, {"category": "volume", "mappings": {"default": {"default": "⠍⠇"}}, "key": "ml", "names": ["ml"]}]
  ,
  "nemeth/units/weight.js":
    [{"locale": "nemeth"}, {
      "category": "",
      "mappings": {"default": {"default": "⠇⠃"}},
      "key": "lb",
      "names": ["lb", "lb."]
    }, {
      "category": "",
      "mappings": {"default": {"default": "⠇⠞"}},
      "key": "LT",
      "names": ["LT", "L.T."]
    }, {"category": "", "mappings": {"default": {"default": "⠕⠵"}}, "key": "oz", "names": ["oz", "oz."]}, {
      "key": "µg",
      "names": ["µg", "mcg"],
      "mappings": {"default": {"default": "⠨⠍⠛"}}
    }, {"category": "", "mappings": {"default": {"default": "⠛⠗"}}, "key": "gr", "names": ["g", "gr"]}, {
      "category": "",
      "mappings": {"default": {"default": "⠅⠛"}},
      "key": "kg",
      "names": ["kg"]
    }, {
      "category": "",
      "mappings": {"default": {"default": "⠍⠉⠛"}},
      "key": "mcg",
      "names": ["mg", "µg"]
    }, {"category": "", "mappings": {"default": {"default": "⠍⠛"}}, "key": "mg", "names": ["mg"]}, {
      "category": "",
      "mappings": {"default": {"default": "⠞"}},
      "key": "t",
      "names": ["t", "T"]
    }]
  ,
  "fr/functions/algebra.js":
    [{"locale": "fr"}, {
      "category": "Algebra",
      "mappings": {"default": {"default": "degré"}},
      "key": "deg",
      "names": ["deg"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "déterminant"}},
      "key": "det",
      "names": ["det"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "dimension"}},
      "key": "dim",
      "names": ["dim"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "homomorphisme"}},
      "key": "hom",
      "names": ["hom", "Hom"]
    }, {
      "category": "Algebra",
      "mappings": {"default": {"default": "noyau"}},
      "key": "ker",
      "names": ["ker"]
    }, {"category": "Algebra", "mappings": {"default": {"default": "trace"}}, "key": "Tr", "names": ["Tr", "tr"]}]
  ,
  "fr/functions/elementary.js":
    [{"locale": "fr"}, {
      "key": "log",
      "mappings": {"default": {"default": "log"}},
      "category": "Logarithm",
      "names": ["log"]
    }, {
      "key": "ln",
      "mappings": {"default": {"default": "logarithme népérien"}},
      "category": "Logarithm",
      "names": ["ln"]
    }, {
      "key": "lg",
      "names": ["lg"],
      "mappings": {"default": {"default": "logarithme décimal"}},
      "category": "Logarithm"
    }, {
      "key": "exp",
      "names": ["exp", "expt"],
      "mappings": {"default": {"default": "exponentielle"}},
      "category": "Elementary"
    }, {
      "key": "gcd",
      "names": ["gcd", "PGCD", "pgcd"],
      "mappings": {
        "default": {"default": "plus grand commun diviseur"},
        "mathspeak": {"default": "pgcd"},
        "clearspeak": {"default": "pgcd"}
      },
      "category": "Elementary"
    }, {
      "key": "lcm",
      "names": ["lcm", "ppcm", "PPCM", "PPMC", "ppmc"],
      "mappings": {
        "default": {"default": "plus petit commun multiple"},
        "mathspeak": {"default": "ppcm"},
        "clearspeak": {"default": "ppcm"}
      },
      "category": "Elementary"
    }, {
      "key": "arg",
      "names": ["arg"],
      "mappings": {"default": {"default": "argument"}},
      "category": "Complex"
    }, {
      "key": "im",
      "names": ["im"],
      "mappings": {"default": {"default": "partie imaginaire"}},
      "category": "Complex"
    }, {
      "key": "re",
      "names": ["re"],
      "mappings": {"default": {"default": "partie réelle"}},
      "category": "Complex"
    }, {
      "key": "inf",
      "names": ["inf"],
      "mappings": {"default": {"default": "borne inférieure"}},
      "category": "Limits"
    }, {
      "key": "lim",
      "mappings": {"default": {"default": "limite"}},
      "category": "Limits",
      "names": ["lim"]
    }, {
      "key": "max",
      "names": ["max"],
      "mappings": {"default": {"default": "maximum"}},
      "category": "Limits"
    }, {
      "key": "min",
      "names": ["min"],
      "mappings": {"default": {"default": "minimum"}},
      "category": "Limits"
    }, {
      "key": "sup",
      "names": ["sup"],
      "mappings": {"default": {"default": "borne supérieure"}},
      "category": "Limits"
    }, {
      "key": "lim inf",
      "names": ["lim inf", "liminf"],
      "mappings": {"default": {"default": "limite inferior"}},
      "category": "Limits"
    }, {
      "key": "lim sup",
      "names": ["lim sup", "limsup"],
      "mappings": {"default": {"default": "limite superior"}},
      "category": "Limits"
    }, {
      "key": "injlim",
      "names": ["injlim", "inj lim"],
      "mappings": {"default": {"default": "limite inductive"}},
      "category": "Limits"
    }, {
      "key": "projlim",
      "names": ["projlim", "proj lim"],
      "mappings": {"default": {"default": "limite projective"}},
      "category": "Limits"
    }, {
      "key": "mod",
      "names": ["mod"],
      "mappings": {"default": {"default": "modulo"}},
      "category": "Elementary"
    }, {"key": "Pr", "names": ["Pr"], "mappings": {"default": {"default": "probabilité"}}, "category": "Probability"}]
  ,
  "fr/functions/hyperbolic.js":
    [{"locale": "fr"}, {
      "key": "cosh",
      "mappings": {"default": {"default": "cosinus hyperbolique"}},
      "category": "Hyperbolic",
      "names": ["cosh"]
    }, {
      "key": "coth",
      "mappings": {"default": {"default": "cotangente hyperbolique"}},
      "category": "Hyperbolic",
      "names": ["coth"]
    }, {
      "key": "csch",
      "mappings": {"default": {"default": "cosécante hyperbolique"}},
      "category": "Hyperbolic",
      "names": ["csch"]
    }, {
      "key": "sech",
      "mappings": {"default": {"default": "sécante hyperbolique"}},
      "category": "Hyperbolic",
      "names": ["sech"]
    }, {
      "key": "sinh",
      "mappings": {"default": {"default": "sinus hyperbolique"}},
      "category": "Hyperbolic",
      "names": ["sinh"]
    }, {
      "key": "tanh",
      "mappings": {"default": {"default": "tangente hyperbolique"}},
      "category": "Hyperbolic",
      "names": ["tanh"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "argument cosinus hyperbolique"}},
      "key": "arcosh",
      "names": ["arcosh", "arccosh", "argch"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "argument cotangente hyperbolique"}},
      "key": "arcoth",
      "names": ["arcoth", "arccoth", "argcoth"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "argument cosécante hyperbolique"}},
      "key": "arcsch",
      "names": ["arcsch", "arccsch", "argcsch"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "argument sécante hyperbolique"}},
      "key": "arsech",
      "names": ["arsech", "arcsech", "argsech"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "argument sinus hyperbolique"}},
      "key": "arsinh",
      "names": ["arsinh", "arcsinh", "argsh"]
    }, {
      "category": "Area",
      "mappings": {"default": {"default": "argument tangente hyperbolique"}},
      "key": "artanh",
      "names": ["artanh", "arctanh", "argth"]
    }]
  ,
  "fr/functions/trigonometry.js":
    [{"locale": "fr"}, {
      "key": "cos",
      "mappings": {"default": {"default": "cosinus"}},
      "category": "Trigonometric",
      "names": ["cos", "cosine"]
    }, {
      "key": "cot",
      "mappings": {"default": {"default": "cotangente"}},
      "category": "Trigonometric",
      "names": ["cot"]
    }, {
      "key": "csc",
      "mappings": {"default": {"default": "cosécante"}},
      "category": "Trigonometric",
      "names": ["csc"]
    }, {
      "key": "sec",
      "mappings": {"default": {"default": "sécante"}},
      "category": "Trigonometric",
      "names": ["sec"]
    }, {
      "key": "sin",
      "mappings": {"default": {"default": "sinus"}},
      "category": "Trigonometric",
      "names": ["sin", "sine"]
    }, {
      "key": "tan",
      "mappings": {"default": {"default": "tangente"}},
      "category": "Trigonometric",
      "names": ["tan"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "arc cosinus"}},
      "key": "arccos",
      "names": ["arccos"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "arc cotangente"}},
      "key": "arccot",
      "names": ["arccot"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "arc cosécante"}},
      "key": "arccsc",
      "names": ["arccsc"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "arc sécante"}},
      "key": "arcsec",
      "names": ["arcsec"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "arc sinus"}},
      "key": "arcsin",
      "names": ["arcsin"]
    }, {
      "category": "Cyclometric",
      "mappings": {"default": {"default": "arc tangente"}},
      "key": "arctan",
      "names": ["arctan"]
    }]
  ,
  "fr/symbols/digits_rest.js":
    [{"locale": "fr"}, {
      "key": "00B2",
      "mappings": {
        "default": {"default": "au carré"},
        "mathspeak": {"default": "au carré"},
        "clearspeak": {"default": "au carré"}
      },
      "category": "No"
    }, {
      "key": "00B3",
      "mappings": {
        "default": {"default": "cubique"},
        "mathspeak": {"default": "cubique"},
        "clearspeak": {"default": "cubique"}
      },
      "category": "No"
    }, {"key": "00BC", "mappings": {"default": {"default": "un quart"}}, "category": "No"}, {
      "key": "00BD",
      "mappings": {"default": {"default": "un demi"}},
      "category": "No"
    }, {"key": "00BE", "mappings": {"default": {"default": "trois quarts"}}, "category": "No"}, {
      "key": "2150",
      "mappings": {"default": {"default": "un septième"}},
      "category": "No"
    }, {"key": "2151", "mappings": {"default": {"default": "un neuvième"}}, "category": "No"}, {
      "key": "2152",
      "mappings": {"default": {"default": "un dixième"}},
      "category": "No"
    }, {"key": "2153", "mappings": {"default": {"default": "un tiers"}}, "category": "No"}, {
      "key": "2154",
      "mappings": {"default": {"default": "deux tiers"}},
      "category": "No"
    }, {"key": "2155", "mappings": {"default": {"default": "un cinquième"}}, "category": "No"}, {
      "key": "2156",
      "mappings": {"default": {"default": "deux cinquièmes"}},
      "category": "No"
    }, {"key": "2157", "mappings": {"default": {"default": "trois cinquièmes"}}, "category": "No"}, {
      "key": "2158",
      "mappings": {"default": {"default": "quatre cinquièmes"}},
      "category": "No"
    }, {"key": "2159", "mappings": {"default": {"default": "un sixième"}}, "category": "No"}, {
      "key": "215A",
      "mappings": {"default": {"default": "cinq sixièmes"}},
      "category": "No"
    }, {"key": "215B", "mappings": {"default": {"default": "un huitième"}}, "category": "No"}, {
      "key": "215C",
      "mappings": {"default": {"default": "trois huitièmes"}},
      "category": "No"
    }, {"key": "215D", "mappings": {"default": {"default": "cinq huitièmes"}}, "category": "No"}, {
      "key": "215E",
      "mappings": {"default": {"default": "sept huitièmes"}},
      "category": "No"
    }, {"key": "215F", "mappings": {"default": {"default": "numérateur un"}}, "category": "No"}, {
      "key": "2189",
      "mappings": {"default": {"default": "zero tiers"}},
      "category": "No"
    }, {
      "key": "3248",
      "mappings": {"default": {"default": "dix cerclé sur carré noir"}},
      "category": "No"
    }, {
      "key": "3249",
      "mappings": {"default": {"default": "vingt cerclé sur carré noir"}},
      "category": "No"
    }, {
      "key": "324A",
      "mappings": {"default": {"default": "trente cerclé sur carré noir"}},
      "category": "No"
    }, {
      "key": "324B",
      "mappings": {"default": {"default": "quarante cerclé sur carré noir"}},
      "category": "No"
    }, {
      "key": "324C",
      "mappings": {"default": {"default": "cinquante cerclé sur carré noir"}},
      "category": "No"
    }, {
      "key": "324D",
      "mappings": {"default": {"default": "soixante cerclé sur carré noir"}},
      "category": "No"
    }, {
      "key": "324E",
      "mappings": {"default": {"default": "soixante dix cerclé sur carré noir"}},
      "category": "No"
    }, {"key": "324F", "mappings": {"default": {"default": "quatre vingts cerclé sur carré noir"}}, "category": "No"}]
  ,
  "fr/symbols/greek-rest.js":
    [{"locale": "fr"}, {
      "category": "Lu",
      "key": "0394",
      "mappings": {"clearspeak": {"default": "triangle", "TriangleSymbol_Delta": "Delta majuscule"}}
    }]
  ,
  "fr/symbols/greek-scripts.js":
    [{"locale": "fr"}, {
      "category": "Ll",
      "key": "1D26",
      "mappings": {"default": {"default": "petite gamma majuscule"}}
    }, {
      "category": "Ll",
      "key": "1D27",
      "mappings": {"default": {"default": "petite lambda majuscule"}}
    }, {
      "category": "Ll",
      "key": "1D28",
      "mappings": {"default": {"default": "petite pi majuscule"}}
    }, {
      "category": "Ll",
      "key": "1D29",
      "mappings": {"default": {"default": "petite rhô majuscule"}}
    }, {
      "category": "Ll",
      "key": "1D2A",
      "mappings": {"default": {"default": "petite psi majuscule"}}
    }, {"category": "Lm", "key": "1D5E", "mappings": {"default": {"default": "gamma suscript"}}}, {
      "category": "Lm",
      "key": "1D60",
      "mappings": {"default": {"default": "phi suscript"}}
    }, {"category": "Lm", "key": "1D66", "mappings": {"default": {"default": "bêta souscrit"}}}, {
      "category": "Lm",
      "key": "1D67",
      "mappings": {"default": {"default": "gamma souscrit"}}
    }, {"category": "Lm", "key": "1D68", "mappings": {"default": {"default": "rhô souscrit"}}}, {
      "category": "Lm",
      "key": "1D69",
      "mappings": {"default": {"default": "phi souscrit"}}
    }, {"category": "Lm", "key": "1D6A", "mappings": {"default": {"default": "khi souscrit"}}}]
  ,
  "fr/symbols/greek-symbols.js":
    [{"locale": "fr"}, {
      "key": "03D0",
      "mappings": {"default": {"default": "bêta grec"}},
      "category": "Ll"
    }, {"key": "03D7", "mappings": {"default": {"default": "ligature kai"}}, "category": "Ll"}, {
      "key": "03F6",
      "mappings": {"default": {"default": "epsilon lunaire réfléchi"}},
      "category": "Sm"
    }, {"category": "Lu", "key": "1D7CA", "mappings": {"default": {"default": "Digamma en gras"}}}, {
      "category": "Ll",
      "key": "1D7CB",
      "mappings": {"default": {"default": "digamma en gras"}}
    }]
  ,
  "fr/symbols/hebrew_letters.js":
    [{"locale": "fr"}, {"key": "2135", "mappings": {"default": {"default": "alef"}}, "category": "Lo"}, {
      "key": "2136",
      "mappings": {"default": {"default": "bèt"}},
      "category": "Lo"
    }, {"key": "2137", "mappings": {"default": {"default": "guimel"}}, "category": "Lo"}, {
      "key": "2138",
      "mappings": {"default": {"default": "dalèt"}},
      "category": "Lo"
    }]
  ,
  "fr/symbols/latin-lower-double-accent.js":
    [{"locale": "fr"}, {
      "category": "Ll",
      "key": "01D6",
      "mappings": {"default": {"default": "u trémaa et macron"}}
    }, {
      "category": "Ll",
      "key": "01D8",
      "mappings": {"default": {"default": "u trémaa et accent aigu"}}
    }, {"category": "Ll", "key": "01DA", "mappings": {"default": {"default": "u trémaa et caron"}}}, {
      "category": "Ll",
      "key": "01DC",
      "mappings": {"default": {"default": "u trémaa et accent grave"}}
    }, {"category": "Ll", "key": "01DF", "mappings": {"default": {"default": "a trémaa et macron"}}}, {
      "category": "Ll",
      "key": "01E1",
      "mappings": {"default": {"default": "a point en chef et macron"}}
    }, {"category": "Ll", "key": "01ED", "mappings": {"default": {"default": "o ogonek et macron"}}}, {
      "category": "Ll",
      "key": "01FB",
      "mappings": {"default": {"default": "a rond en chef et accent aigu"}}
    }, {"category": "Ll", "key": "022B", "mappings": {"default": {"default": "o tréma et macron"}}}, {
      "category": "Ll",
      "key": "022D",
      "mappings": {"default": {"default": "o tilde et macron"}}
    }, {
      "category": "Ll",
      "key": "0231",
      "mappings": {"default": {"default": "o point en chef et macron"}}
    }, {
      "key": "1E09",
      "mappings": {"default": {"default": "c cédille et accent aigu"}},
      "category": "Ll"
    }, {
      "key": "1E15",
      "mappings": {"default": {"default": "e macron et accent grave"}},
      "category": "Ll"
    }, {
      "key": "1E17",
      "mappings": {"default": {"default": "e macron et accent aigu"}},
      "category": "Ll"
    }, {"key": "1E1D", "mappings": {"default": {"default": "e cédille et brève"}}, "category": "Ll"}, {
      "key": "1E2F",
      "mappings": {"default": {"default": "i tréma et accent aigu"}},
      "category": "Ll"
    }, {
      "key": "1E39",
      "mappings": {"default": {"default": "l point souscrit et macron"}},
      "category": "Ll"
    }, {
      "key": "1E4D",
      "mappings": {"default": {"default": "o tilde et accent aigu"}},
      "category": "Ll"
    }, {"key": "1E4F", "mappings": {"default": {"default": "o tilde et tréma"}}, "category": "Ll"}, {
      "key": "1E51",
      "mappings": {"default": {"default": "o macron et accent grave"}},
      "category": "Ll"
    }, {
      "key": "1E53",
      "mappings": {"default": {"default": "o macron et accent aigu"}},
      "category": "Ll"
    }, {
      "key": "1E5D",
      "mappings": {"default": {"default": "r point souscrit et macron"}},
      "category": "Ll"
    }, {
      "key": "1E65",
      "mappings": {"default": {"default": "s accent aigu et point en chef"}},
      "category": "Ll"
    }, {
      "key": "1E67",
      "mappings": {"default": {"default": "s caron et point en chef"}},
      "category": "Ll"
    }, {
      "key": "1E69",
      "mappings": {"default": {"default": "s point souscrit et point en chef"}},
      "category": "Ll"
    }, {
      "key": "1E79",
      "mappings": {"default": {"default": "u tilde et accent aigu"}},
      "category": "Ll"
    }, {"key": "1E7B", "mappings": {"default": {"default": "u macron et tréma"}}, "category": "Ll"}, {
      "key": "1EA5",
      "mappings": {"default": {"default": "a accent circonflexe et accent aigu"}},
      "category": "Ll"
    }, {
      "key": "1EA7",
      "mappings": {"default": {"default": "a accent circonflexe et accent grave"}},
      "category": "Ll"
    }, {
      "key": "1EA9",
      "mappings": {"default": {"default": "a accent circonflexe et crochet en chef"}},
      "category": "Ll"
    }, {
      "key": "1EAB",
      "mappings": {"default": {"default": "a accent circonflexe et tilde"}},
      "category": "Ll"
    }, {
      "key": "1EAD",
      "mappings": {"default": {"default": "a accent circonflexe et point souscrit"}},
      "category": "Ll"
    }, {
      "key": "1EAF",
      "mappings": {"default": {"default": "a brève et accent aigu"}},
      "category": "Ll"
    }, {
      "key": "1EB1",
      "mappings": {"default": {"default": "a brève et accent grave"}},
      "category": "Ll"
    }, {
      "key": "1EB3",
      "mappings": {"default": {"default": "a brève et crochet en chef"}},
      "category": "Ll"
    }, {"key": "1EB5", "mappings": {"default": {"default": "a brève et tilde"}}, "category": "Ll"}, {
      "key": "1EB7",
      "mappings": {"default": {"default": "a brève et point souscrit"}},
      "category": "Ll"
    }, {
      "key": "1EBF",
      "mappings": {"default": {"default": "e accent circonflexe et accent aigu"}},
      "category": "Ll"
    }, {
      "key": "1EC1",
      "mappings": {"default": {"default": "e accent circonflexe et accent grave"}},
      "category": "Ll"
    }, {
      "key": "1EC3",
      "mappings": {"default": {"default": "e accent circonflexe et crochet en chef"}},
      "category": "Ll"
    }, {
      "key": "1EC5",
      "mappings": {"default": {"default": "e accent circonflexe et tilde"}},
      "category": "Ll"
    }, {
      "key": "1EC7",
      "mappings": {"default": {"default": "e accent circonflexe et point souscrit"}},
      "category": "Ll"
    }, {
      "key": "1ED1",
      "mappings": {"default": {"default": "o accent circonflexe et accent aigu"}},
      "category": "Ll"
    }, {
      "key": "1ED3",
      "mappings": {"default": {"default": "o accent circonflexe et accent grave"}},
      "category": "Ll"
    }, {
      "key": "1ED5",
      "mappings": {"default": {"default": "o accent circonflexe et crochet en chef"}},
      "category": "Ll"
    }, {
      "key": "1ED7",
      "mappings": {"default": {"default": "o accent circonflexe et tilde"}},
      "category": "Ll"
    }, {
      "key": "1ED9",
      "mappings": {"default": {"default": "o accent circonflexe et point souscrit"}},
      "category": "Ll"
    }, {"key": "1EDB", "mappings": {"default": {"default": "o cornu accent aigu"}}, "category": "Ll"}, {
      "key": "1EDD",
      "mappings": {"default": {"default": "o cornu accent grave"}},
      "category": "Ll"
    }, {
      "key": "1EDF",
      "mappings": {"default": {"default": "o cornu crochet en chef"}},
      "category": "Ll"
    }, {"key": "1EE1", "mappings": {"default": {"default": "o cornu tilde"}}, "category": "Ll"}, {
      "key": "1EE3",
      "mappings": {"default": {"default": "o cornu point souscrit"}},
      "category": "Ll"
    }, {"key": "1EE9", "mappings": {"default": {"default": "u cornu accent aigu"}}, "category": "Ll"}, {
      "key": "1EEB",
      "mappings": {"default": {"default": "u cornu accent grave"}},
      "category": "Ll"
    }, {
      "key": "1EED",
      "mappings": {"default": {"default": "u cornu crochet en chef"}},
      "category": "Ll"
    }, {"key": "1EEF", "mappings": {"default": {"default": "u cornu tilde"}}, "category": "Ll"}, {
      "key": "1EF1",
      "mappings": {"default": {"default": "u cornu point souscrit"}},
      "category": "Ll"
    }]
  ,
  "fr/symbols/latin-lower-phonetic.js":
    [{"locale": "fr"}, {
      "key": "00F8",
      "mappings": {"default": {"default": "o barré"}},
      "category": "Ll"
    }, {"key": "0111", "mappings": {"default": {"default": "d barré"}}, "category": "Ll"}, {
      "key": "0127",
      "mappings": {"default": {"default": "h barré"}},
      "category": "Ll"
    }, {"key": "0142", "mappings": {"default": {"default": "l barré"}}, "category": "Ll"}, {
      "key": "0167",
      "mappings": {"default": {"default": "t barré"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0180", "mappings": {"default": {"default": "b barré"}}}, {
      "key": "019B",
      "mappings": {"default": {"default": "lambda barré"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "01B6", "mappings": {"default": {"default": "z barré"}}}, {
      "category": "Ll",
      "key": "01BE",
      "mappings": {"default": {"default": "coup de glotte barré culbuté"}}
    }, {"category": "Ll", "key": "01E5", "mappings": {"default": {"default": "g barré"}}}, {
      "category": "Ll",
      "key": "01FF",
      "mappings": {"default": {"default": "o barré accent aigu"}}
    }, {"category": "Ll", "key": "023C", "mappings": {"default": {"default": "c barré"}}}, {
      "category": "Ll",
      "key": "0247",
      "mappings": {"default": {"default": "e barré"}}
    }, {"category": "Ll", "key": "0249", "mappings": {"default": {"default": "j barré"}}}, {
      "category": "Ll",
      "key": "024D",
      "mappings": {"default": {"default": "r barré"}}
    }, {"category": "Ll", "key": "024F", "mappings": {"default": {"default": "y barré"}}}, {
      "category": "Ll",
      "key": "025F",
      "mappings": {"default": {"default": "j sans point barré"}}
    }, {"category": "Ll", "key": "0268", "mappings": {"default": {"default": "i barré"}}}, {
      "category": "Ll",
      "key": "0284",
      "mappings": {"default": {"default": "j sans point barré crosse"}}
    }, {
      "category": "Ll",
      "key": "02A1",
      "mappings": {"default": {"default": "coup de glotte barré"}}
    }, {
      "category": "Ll",
      "key": "02A2",
      "mappings": {"default": {"default": "coup de glotte barré réfléchi"}}
    }, {
      "category": "Ll",
      "key": "1D13",
      "mappings": {"default": {"default": "o couché barré obliquement"}}
    }, {"category": "Ll", "key": "1D7C", "mappings": {"default": {"default": "iota barré"}}}, {
      "category": "Ll",
      "key": "1D7D",
      "mappings": {"default": {"default": "p barré"}}
    }, {"category": "Ll", "key": "1D7F", "mappings": {"default": {"default": "upsilon barré"}}}, {
      "category": "Ll",
      "key": "1E9C",
      "mappings": {"default": {"default": "s long à barre diagonale"}}
    }, {
      "category": "Ll",
      "key": "1E9D",
      "mappings": {"default": {"default": "s long à barre haute"}}
    }, {"category": "Ll", "key": "018D", "mappings": {"default": {"default": "delta culbutée"}}}, {
      "key": "1E9B",
      "mappings": {"default": {"default": "s long point en chef"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "1E9F", "mappings": {"default": {"default": "delta"}}}, {
      "key": "0138",
      "mappings": {"default": {"default": "kra"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "017F", "mappings": {"default": {"default": "s long"}}}, {
      "category": "Ll",
      "key": "0183",
      "mappings": {"default": {"default": "b potence"}}
    }, {"category": "Ll", "key": "0185", "mappings": {"default": {"default": "sixiéme ton"}}}, {
      "category": "Ll",
      "key": "0188",
      "mappings": {"default": {"default": "c crosse"}}
    }, {"category": "Ll", "key": "018C", "mappings": {"default": {"default": "d potence"}}}, {
      "key": "0192",
      "mappings": {"default": {"default": "f cursif"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0195", "mappings": {"default": {"default": "hv (Hwair)"}}}, {
      "category": "Ll",
      "key": "0199",
      "mappings": {"default": {"default": "k crosse"}}
    }, {"key": "019A", "mappings": {"default": {"default": "l rayé"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "019E",
      "mappings": {"default": {"default": "n à long jambage de droite"}}
    }, {"category": "Ll", "key": "01A1", "mappings": {"default": {"default": "o cornu"}}}, {
      "category": "Ll",
      "key": "01A3",
      "mappings": {"default": {"default": "gha"}}
    }, {"category": "Ll", "key": "01A5", "mappings": {"default": {"default": "p crosse"}}}, {
      "category": "Ll",
      "key": "01A8",
      "mappings": {"default": {"default": "deuxiéme ton"}}
    }, {
      "category": "Ll",
      "key": "01AA",
      "mappings": {"default": {"default": "ech réfléchi bouclé"}}
    }, {"category": "Ll", "key": "01AB", "mappings": {"default": {"default": "t hameçon palatal"}}}, {
      "category": "Ll",
      "key": "01AD",
      "mappings": {"default": {"default": "t crosse"}}
    }, {"category": "Ll", "key": "01B0", "mappings": {"default": {"default": "u cornu"}}}, {
      "category": "Ll",
      "key": "01B4",
      "mappings": {"default": {"default": "y crosse"}}
    }, {"category": "Ll", "key": "01B9", "mappings": {"default": {"default": "ej réfléchi"}}}, {
      "category": "Ll",
      "key": "01BA",
      "mappings": {"default": {"default": "ej hameçon"}}
    }, {"category": "Ll", "key": "01BD", "mappings": {"default": {"default": "cinquiéme tun"}}}, {
      "category": "Ll",
      "key": "01BF",
      "mappings": {"default": {"default": "wynn"}}
    }, {"category": "Ll", "key": "01C6", "mappings": {"default": {"default": "dz caron"}}}, {
      "category": "Ll",
      "key": "01C9",
      "mappings": {"default": {"default": "lj"}}
    }, {"category": "Ll", "key": "01CC", "mappings": {"default": {"default": "nj"}}}, {
      "category": "Ll",
      "key": "01E3",
      "mappings": {"default": {"default": "ae macron"}}
    }, {"category": "Ll", "key": "01EF", "mappings": {"default": {"default": "ej caron"}}}, {
      "category": "Ll",
      "key": "01F3",
      "mappings": {"default": {"default": "dz"}}
    }, {"category": "Ll", "key": "021D", "mappings": {"default": {"default": "yogh"}}}, {
      "category": "Ll",
      "key": "026E",
      "mappings": {"default": {"default": "lej"}}
    }, {"category": "Ll", "key": "0292", "mappings": {"default": {"default": "ej"}}}, {
      "category": "Ll",
      "key": "0293",
      "mappings": {"default": {"default": "ej bouclé"}}
    }, {"category": "Ll", "key": "02A4", "mappings": {"default": {"default": "digramme Dej"}}}, {
      "category": "Ll",
      "key": "01DD",
      "mappings": {"default": {"default": "e culbuté"}}
    }, {
      "category": "Ll",
      "key": "01FD",
      "mappings": {"default": {"default": "ae accent aigu (ash)"}}
    }, {"category": "Ll", "key": "0221", "mappings": {"default": {"default": "d bouclé"}}}, {
      "category": "Ll",
      "key": "0223",
      "mappings": {"default": {"default": "ou"}}
    }, {"category": "Ll", "key": "0225", "mappings": {"default": {"default": "z crochet"}}}, {
      "category": "Ll",
      "key": "0234",
      "mappings": {"default": {"default": "l bouclé"}}
    }, {"category": "Ll", "key": "0235", "mappings": {"default": {"default": "n bouclé"}}}, {
      "category": "Ll",
      "key": "0236",
      "mappings": {"default": {"default": "t bouclé"}}
    }, {
      "category": "Ll",
      "key": "0238",
      "mappings": {"default": {"default": "Ligature minuscule latine Db"}}
    }, {
      "category": "Ll",
      "key": "0239",
      "mappings": {"default": {"default": "Ligature minuscule latine Qp"}}
    }, {"category": "Ll", "key": "023F", "mappings": {"default": {"default": "s à paraphe"}}}, {
      "category": "Ll",
      "key": "0240",
      "mappings": {"default": {"default": "z à paraphe"}}
    }, {"category": "Ll", "key": "0242", "mappings": {"default": {"default": "coup de glotte"}}}, {
      "category": "Ll",
      "key": "024B",
      "mappings": {"default": {"default": "q avec hameçon"}}
    }, {"category": "Ll", "key": "0250", "mappings": {"default": {"default": "a culbuté"}}}, {
      "category": "Ll",
      "key": "0251",
      "mappings": {"default": {"default": "alpha"}}
    }, {"category": "Ll", "key": "0252", "mappings": {"default": {"default": "alpha culbuté"}}}, {
      "category": "Ll",
      "key": "0253",
      "mappings": {"default": {"default": "b crosse"}}
    }, {"category": "Ll", "key": "0254", "mappings": {"default": {"default": "o ouvert"}}}, {
      "category": "Ll",
      "key": "0255",
      "mappings": {"default": {"default": "c bouclé"}}
    }, {
      "category": "Ll",
      "key": "0256",
      "mappings": {"default": {"default": "d hameçon rétroflexe"}}
    }, {"category": "Ll", "key": "0257", "mappings": {"default": {"default": "d crosse"}}}, {
      "category": "Ll",
      "key": "0258",
      "mappings": {"default": {"default": "e réfléchi"}}
    }, {"category": "Ll", "key": "0259", "mappings": {"default": {"default": "schwa"}}}, {
      "category": "Ll",
      "key": "025A",
      "mappings": {"default": {"default": "schwa crochet"}}
    }, {"key": "025B", "mappings": {"default": {"default": "e ouvert"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "025C",
      "mappings": {"default": {"default": "epsilon réfléchi"}}
    }, {
      "category": "Ll",
      "key": "025D",
      "mappings": {"default": {"default": "epsilon réfléchi crochet"}}
    }, {
      "category": "Ll",
      "key": "025E",
      "mappings": {"default": {"default": "epsilon réfléchi fermé"}}
    }, {"category": "Ll", "key": "0260", "mappings": {"default": {"default": "g crosse"}}}, {
      "category": "Ll",
      "key": "0261",
      "mappings": {"default": {"default": "g cursif"}}
    }, {"category": "Ll", "key": "0263", "mappings": {"default": {"default": "gamma"}}}, {
      "category": "Ll",
      "key": "0264",
      "mappings": {"default": {"default": "petit gamma"}}
    }, {"category": "Ll", "key": "0265", "mappings": {"default": {"default": "h culbuté"}}}, {
      "category": "Ll",
      "key": "0266",
      "mappings": {"default": {"default": "h crosse"}}
    }, {"category": "Ll", "key": "0267", "mappings": {"default": {"default": "eng crosse"}}}, {
      "category": "Ll",
      "key": "0269",
      "mappings": {"default": {"default": "iota"}}
    }, {"category": "Ll", "key": "026B", "mappings": {"default": {"default": "l tilde médian"}}}, {
      "category": "Ll",
      "key": "026C",
      "mappings": {"default": {"default": "l sanglé"}}
    }, {
      "category": "Ll",
      "key": "026D",
      "mappings": {"default": {"default": "l hameçon rétroflexe"}}
    }, {"category": "Ll", "key": "026F", "mappings": {"default": {"default": "m culbuté"}}}, {
      "category": "Ll",
      "key": "0270",
      "mappings": {"default": {"default": "m hampé culbuté"}}
    }, {"category": "Ll", "key": "0271", "mappings": {"default": {"default": "m hameçon"}}}, {
      "category": "Ll",
      "key": "0272",
      "mappings": {"default": {"default": "n hameçon à gauche"}}
    }, {
      "category": "Ll",
      "key": "0273",
      "mappings": {"default": {"default": "n hameçon rétroflexe"}}
    }, {"category": "Ll", "key": "0275", "mappings": {"default": {"default": "o barré"}}}, {
      "category": "Ll",
      "key": "0277",
      "mappings": {"default": {"default": "oméga fermé"}}
    }, {"category": "Ll", "key": "0278", "mappings": {"default": {"default": "phi"}}}, {
      "category": "Ll",
      "key": "0279",
      "mappings": {"default": {"default": "r culbuté"}}
    }, {"category": "Ll", "key": "027A", "mappings": {"default": {"default": "r prolongé culbuté"}}}, {
      "category": "Ll",
      "key": "027B",
      "mappings": {"default": {"default": "r crosse culbuté"}}
    }, {"category": "Ll", "key": "027C", "mappings": {"default": {"default": "r prolongé"}}}, {
      "category": "Ll",
      "key": "027D",
      "mappings": {"default": {"default": "r hameçon rétroflexe"}}
    }, {"category": "Ll", "key": "027E", "mappings": {"default": {"default": "r sans obit"}}}, {
      "category": "Ll",
      "key": "027F",
      "mappings": {"default": {"default": "r sans obit réfléchi"}}
    }, {
      "category": "Ll",
      "key": "0282",
      "mappings": {"default": {"default": "s hameçon rétroflexe"}}
    }, {"category": "Ll", "key": "0283", "mappings": {"default": {"default": "ech"}}}, {
      "category": "Ll",
      "key": "0285",
      "mappings": {"default": {"default": "r sans obit réfléchi hameçon rétroflexe"}}
    }, {"category": "Ll", "key": "0286", "mappings": {"default": {"default": "ech bouclé"}}}, {
      "category": "Ll",
      "key": "0287",
      "mappings": {"default": {"default": "t culbuté"}}
    }, {
      "category": "Ll",
      "key": "0288",
      "mappings": {"default": {"default": "t hameçon rétroflexe"}}
    }, {"category": "Ll", "key": "0289", "mappings": {"default": {"default": "u barré"}}}, {
      "category": "Ll",
      "key": "028A",
      "mappings": {"default": {"default": "upsilon"}}
    }, {"category": "Ll", "key": "028B", "mappings": {"default": {"default": "v de ronde"}}}, {
      "category": "Ll",
      "key": "028C",
      "mappings": {"default": {"default": "v culbuté"}}
    }, {"category": "Ll", "key": "028D", "mappings": {"default": {"default": "w culbuté"}}}, {
      "category": "Ll",
      "key": "028E",
      "mappings": {"default": {"default": "y culbuté"}}
    }, {
      "category": "Ll",
      "key": "0290",
      "mappings": {"default": {"default": "z hameçon rétroflexe"}}
    }, {"category": "Ll", "key": "0291", "mappings": {"default": {"default": "z bouclé"}}}, {
      "category": "Ll",
      "key": "0295",
      "mappings": {"default": {"default": "coup de glotte réfléchi"}}
    }, {
      "category": "Ll",
      "key": "0296",
      "mappings": {"default": {"default": "coup de glotte culbuté"}}
    }, {"category": "Ll", "key": "0297", "mappings": {"default": {"default": "C étiré"}}}, {
      "category": "Ll",
      "key": "0298",
      "mappings": {"default": {"default": "clic bilabial"}}
    }, {"category": "Ll", "key": "029A", "mappings": {"default": {"default": "epsilon fermé"}}}, {
      "category": "Ll",
      "key": "029E",
      "mappings": {"default": {"default": "k culbuté"}}
    }, {"category": "Ll", "key": "02A0", "mappings": {"default": {"default": "q crosse"}}}, {
      "category": "Ll",
      "key": "02A3",
      "mappings": {"default": {"default": "digramme Dz"}}
    }, {"category": "Ll", "key": "02A5", "mappings": {"default": {"default": "digramme Dz bouclé"}}}, {
      "category": "Ll",
      "key": "02A6",
      "mappings": {"default": {"default": "digramme Ts"}}
    }, {"category": "Ll", "key": "02A7", "mappings": {"default": {"default": "digramme Tech"}}}, {
      "category": "Ll",
      "key": "02A8",
      "mappings": {"default": {"default": "digramme Tc bouclé"}}
    }, {"category": "Ll", "key": "02A9", "mappings": {"default": {"default": "digramme Feng"}}}, {
      "category": "Ll",
      "key": "02AA",
      "mappings": {"default": {"default": "digramme Ls"}}
    }, {"category": "Ll", "key": "02AB", "mappings": {"default": {"default": "digramme Lz"}}}, {
      "category": "Ll",
      "key": "02AC",
      "mappings": {"default": {"default": "percussion bilabiale"}}
    }, {
      "category": "Ll",
      "key": "02AD",
      "mappings": {"default": {"default": "percussion bidentale"}}
    }, {
      "category": "Ll",
      "key": "02AE",
      "mappings": {"default": {"default": "h culbuté crosse réfléchie"}}
    }, {
      "category": "Ll",
      "key": "02AF",
      "mappings": {"default": {"default": "h culbuté crosse réfléchie et hameçon rétroflexe"}}
    }, {"category": "Ll", "key": "1D02", "mappings": {"default": {"default": "ae culbuté"}}}, {
      "category": "Ll",
      "key": "1D08",
      "mappings": {"default": {"default": "epsilon culbuté"}}
    }, {"category": "Ll", "key": "1D09", "mappings": {"default": {"default": "i culbuté"}}}, {
      "category": "Ll",
      "key": "1D11",
      "mappings": {"default": {"default": "o couché"}}
    }, {"category": "Ll", "key": "1D12", "mappings": {"default": {"default": "o ouvert couché"}}}, {
      "category": "Ll",
      "key": "1D14",
      "mappings": {"default": {"default": "oe culbuté"}}
    }, {
      "category": "Ll",
      "key": "1D16",
      "mappings": {"default": {"default": "moitié supérieure de o"}}
    }, {
      "category": "Ll",
      "key": "1D17",
      "mappings": {"default": {"default": "moitié inférieure de o"}}
    }, {"category": "Ll", "key": "1D1D", "mappings": {"default": {"default": "u couché"}}}, {
      "category": "Ll",
      "key": "1D1E",
      "mappings": {"default": {"default": "u tréma couché"}}
    }, {"category": "Ll", "key": "1D1F", "mappings": {"default": {"default": "m couché"}}}, {
      "category": "Ll",
      "key": "1D24",
      "mappings": {"default": {"default": "fricative pharyngale voisée"}}
    }, {"category": "Ll", "key": "1D25", "mappings": {"default": {"default": "aïn"}}}, {
      "category": "Ll",
      "key": "1D6B",
      "mappings": {"default": {"default": "ue"}}
    }, {"category": "Ll", "key": "1D6C", "mappings": {"default": {"default": "b tilde médian"}}}, {
      "category": "Ll",
      "key": "1D6D",
      "mappings": {"default": {"default": "d tilde médian"}}
    }, {"category": "Ll", "key": "1D6E", "mappings": {"default": {"default": "f tilde médian"}}}, {
      "category": "Ll",
      "key": "1D6F",
      "mappings": {"default": {"default": "m tilde médian"}}
    }, {"category": "Ll", "key": "1D70", "mappings": {"default": {"default": "n tilde médian"}}}, {
      "category": "Ll",
      "key": "1D71",
      "mappings": {"default": {"default": "p tilde médian"}}
    }, {"category": "Ll", "key": "1D72", "mappings": {"default": {"default": "r tilde médian"}}}, {
      "category": "Ll",
      "key": "1D73",
      "mappings": {"default": {"default": "r sans obit tilde médian"}}
    }, {"category": "Ll", "key": "1D74", "mappings": {"default": {"default": "s tilde médian"}}}, {
      "category": "Ll",
      "key": "1D75",
      "mappings": {"default": {"default": "t tilde médian"}}
    }, {"category": "Ll", "key": "1D76", "mappings": {"default": {"default": "z tilde médian"}}}, {
      "category": "Ll",
      "key": "1D77",
      "mappings": {"default": {"default": "g culbuté"}}
    }, {"category": "Ll", "key": "1D79", "mappings": {"default": {"default": "g insulaire"}}}, {
      "category": "Ll",
      "key": "1D7A",
      "mappings": {"default": {"default": "th barré diagonalement"}}
    }, {"category": "Ll", "key": "1D80", "mappings": {"default": {"default": "b hameçon palatal"}}}, {
      "category": "Ll",
      "key": "1D81",
      "mappings": {"default": {"default": "d hameçon palatal"}}
    }, {"category": "Ll", "key": "1D82", "mappings": {"default": {"default": "f hameçon palatal"}}}, {
      "category": "Ll",
      "key": "1D83",
      "mappings": {"default": {"default": "g hameçon palatal"}}
    }, {"category": "Ll", "key": "1D84", "mappings": {"default": {"default": "k hameçon palatal"}}}, {
      "category": "Ll",
      "key": "1D85",
      "mappings": {"default": {"default": "l hameçon palatal"}}
    }, {"category": "Ll", "key": "1D86", "mappings": {"default": {"default": "m hameçon palatal"}}}, {
      "category": "Ll",
      "key": "1D87",
      "mappings": {"default": {"default": "n hameçon palatal"}}
    }, {"category": "Ll", "key": "1D88", "mappings": {"default": {"default": "p hameçon palatal"}}}, {
      "category": "Ll",
      "key": "1D89",
      "mappings": {"default": {"default": "r hameçon palatal"}}
    }, {"category": "Ll", "key": "1D8A", "mappings": {"default": {"default": "s hameçon palatal"}}}, {
      "category": "Ll",
      "key": "1D8B",
      "mappings": {"default": {"default": "ech hameçon palatal"}}
    }, {"category": "Ll", "key": "1D8C", "mappings": {"default": {"default": "v hameçon palatal"}}}, {
      "category": "Ll",
      "key": "1D8D",
      "mappings": {"default": {"default": "x hameçon palatal"}}
    }, {"category": "Ll", "key": "1D8E", "mappings": {"default": {"default": "z hameçon palatal"}}}, {
      "category": "Ll",
      "key": "1D8F",
      "mappings": {"default": {"default": "a hameçon rétroflexe"}}
    }, {
      "category": "Ll",
      "key": "1D90",
      "mappings": {"default": {"default": "alpha hameçon rétroflexe"}}
    }, {
      "category": "Ll",
      "key": "1D91",
      "mappings": {"default": {"default": "d crosse et hameçon rétroflexe"}}
    }, {
      "category": "Ll",
      "key": "1D92",
      "mappings": {"default": {"default": "e hameçon rétroflexe"}}
    }, {
      "category": "Ll",
      "key": "1D93",
      "mappings": {"default": {"default": "epsilon hameçon rétroflexe"}}
    }, {
      "category": "Ll",
      "key": "1D94",
      "mappings": {"default": {"default": "epsilon réfléchi hameçon rétroflexe"}}
    }, {
      "category": "Ll",
      "key": "1D95",
      "mappings": {"default": {"default": "schwa hameçon rétroflexe"}}
    }, {
      "category": "Ll",
      "key": "1D96",
      "mappings": {"default": {"default": "i hameçon rétroflexe"}}
    }, {
      "category": "Ll",
      "key": "1D97",
      "mappings": {"default": {"default": "o ouvert hameçon rétroflexe"}}
    }, {
      "category": "Ll",
      "key": "1D98",
      "mappings": {"default": {"default": "ech hameçon rétroflexe"}}
    }, {
      "category": "Ll",
      "key": "1D99",
      "mappings": {"default": {"default": "u hameçon rétroflexe"}}
    }, {"category": "Ll", "key": "1D9A", "mappings": {"default": {"default": "ej hameçon rétroflexe"}}}, {
      "key": "0149",
      "mappings": {"default": {"default": "n précédée d'une apostrophe"}},
      "category": "Ll"
    }, {"key": "014B", "mappings": {"default": {"default": "eng"}}, "category": "Ll"}]
  ,
  "fr/symbols/latin-lower-single-accent.js":
    [{"locale": "fr"}, {
      "key": "00E0",
      "mappings": {"default": {"default": "a accent grave"}},
      "category": "Ll"
    }, {"key": "00E1", "mappings": {"default": {"default": "a accent aigu"}}, "category": "Ll"}, {
      "key": "00E2",
      "mappings": {"default": {"default": "a accent circonflexe"}},
      "category": "Ll"
    }, {"key": "00E3", "mappings": {"default": {"default": "a tilde"}}, "category": "Ll"}, {
      "key": "00E4",
      "mappings": {"default": {"default": "a tréma"}},
      "category": "Ll"
    }, {"key": "00E5", "mappings": {"default": {"default": "a rond en chef"}}, "category": "Ll"}, {
      "key": "00E7",
      "mappings": {"default": {"default": "c cédille"}},
      "category": "Ll"
    }, {"key": "00E8", "mappings": {"default": {"default": "e accent grave"}}, "category": "Ll"}, {
      "key": "00E9",
      "mappings": {"default": {"default": "e accent aigu"}},
      "category": "Ll"
    }, {"key": "00EA", "mappings": {"default": {"default": "e accent circonflexe"}}, "category": "Ll"}, {
      "key": "00EB",
      "mappings": {"default": {"default": "e tréma"}},
      "category": "Ll"
    }, {"key": "00EC", "mappings": {"default": {"default": "i accent grave"}}, "category": "Ll"}, {
      "key": "00ED",
      "mappings": {"default": {"default": "i accent aigu"}},
      "category": "Ll"
    }, {"key": "00EE", "mappings": {"default": {"default": "i accent circonflexe"}}, "category": "Ll"}, {
      "key": "00EF",
      "mappings": {"default": {"default": "i tréma"}},
      "category": "Ll"
    }, {"key": "00F1", "mappings": {"default": {"default": "n tilde"}}, "category": "Ll"}, {
      "key": "00F2",
      "mappings": {"default": {"default": "o accent grave"}},
      "category": "Ll"
    }, {"key": "00F3", "mappings": {"default": {"default": "o accent aigu"}}, "category": "Ll"}, {
      "key": "00F4",
      "mappings": {"default": {"default": "o accent circonflexe"}},
      "category": "Ll"
    }, {"key": "00F5", "mappings": {"default": {"default": "o tilde"}}, "category": "Ll"}, {
      "key": "00F6",
      "mappings": {"default": {"default": "o tréma"}},
      "category": "Ll"
    }, {"key": "00F9", "mappings": {"default": {"default": "u accent grave"}}, "category": "Ll"}, {
      "key": "00FA",
      "mappings": {"default": {"default": "u accent aigu"}},
      "category": "Ll"
    }, {"key": "00FB", "mappings": {"default": {"default": "u accent circonflexe"}}, "category": "Ll"}, {
      "key": "00FC",
      "mappings": {"default": {"default": "u tréma"}},
      "category": "Ll"
    }, {"key": "00FD", "mappings": {"default": {"default": "y accent aigu"}}, "category": "Ll"}, {
      "key": "00FF",
      "mappings": {"default": {"default": "y tréma"}},
      "category": "Ll"
    }, {"key": "0101", "mappings": {"default": {"default": "a macron"}}, "category": "Ll"}, {
      "key": "0103",
      "mappings": {"default": {"default": "a brève"}},
      "category": "Ll"
    }, {"key": "0105", "mappings": {"default": {"default": "a ogonek"}}, "category": "Ll"}, {
      "key": "0107",
      "mappings": {"default": {"default": "c accent aigu"}},
      "category": "Ll"
    }, {"key": "0109", "mappings": {"default": {"default": "c accent circonflexe"}}, "category": "Ll"}, {
      "key": "010B",
      "mappings": {"default": {"default": "c point en chef"}},
      "category": "Ll"
    }, {"key": "010D", "mappings": {"default": {"default": "c caron"}}, "category": "Ll"}, {
      "key": "010F",
      "mappings": {"default": {"default": "d caron"}},
      "category": "Ll"
    }, {"key": "0113", "mappings": {"default": {"default": "e macron"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "0115",
      "mappings": {"default": {"default": "e brève"}}
    }, {"key": "0117", "mappings": {"default": {"default": "e point en chef"}}, "category": "Ll"}, {
      "key": "0119",
      "mappings": {"default": {"default": "e ogonek"}},
      "category": "Ll"
    }, {"key": "011B", "mappings": {"default": {"default": "e caron"}}, "category": "Ll"}, {
      "key": "011D",
      "mappings": {"default": {"default": "g accent circonflexe"}},
      "category": "Ll"
    }, {"key": "011F", "mappings": {"default": {"default": "g brève"}}, "category": "Ll"}, {
      "key": "0121",
      "mappings": {"default": {"default": "g point en chef"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "0123", "mappings": {"default": {"default": "g cédille"}}}, {
      "key": "0125",
      "mappings": {"default": {"default": "h accent circonflexe"}},
      "category": "Ll"
    }, {"key": "0129", "mappings": {"default": {"default": "i tilde"}}, "category": "Ll"}, {
      "key": "012B",
      "mappings": {"default": {"default": "i macron"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "012D", "mappings": {"default": {"default": "i brève"}}}, {
      "key": "012F",
      "mappings": {"default": {"default": "i ogonek"}},
      "category": "Ll"
    }, {"key": "0131", "mappings": {"default": {"default": "i sans point"}}, "category": "Ll"}, {
      "key": "0135",
      "mappings": {"default": {"default": "j accent circonflexe"}},
      "category": "Ll"
    }, {"key": "0137", "mappings": {"default": {"default": "k cédille"}}, "category": "Ll"}, {
      "key": "013A",
      "mappings": {"default": {"default": "l accent aigu"}},
      "category": "Ll"
    }, {"key": "013C", "mappings": {"default": {"default": "l cédille"}}, "category": "Ll"}, {
      "key": "013E",
      "mappings": {"default": {"default": "l caron"}},
      "category": "Ll"
    }, {"key": "0140", "mappings": {"default": {"default": "l point médian"}}, "category": "Ll"}, {
      "key": "0144",
      "mappings": {"default": {"default": "n accent aigu"}},
      "category": "Ll"
    }, {"key": "0146", "mappings": {"default": {"default": "n cédille"}}, "category": "Ll"}, {
      "key": "0148",
      "mappings": {"default": {"default": "n caron"}},
      "category": "Ll"
    }, {"key": "014D", "mappings": {"default": {"default": "o macron"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "014F",
      "mappings": {"default": {"default": "o brève"}}
    }, {
      "key": "0151",
      "mappings": {"default": {"default": "o avec double accent aigu"}},
      "category": "Ll"
    }, {"key": "0155", "mappings": {"default": {"default": "r accent aigu"}}, "category": "Ll"}, {
      "key": "0157",
      "mappings": {"default": {"default": "r cédille"}},
      "category": "Ll"
    }, {"key": "0159", "mappings": {"default": {"default": "r caron"}}, "category": "Ll"}, {
      "key": "015B",
      "mappings": {"default": {"default": "s accent aigu"}},
      "category": "Ll"
    }, {"key": "015D", "mappings": {"default": {"default": "s accent circonflexe"}}, "category": "Ll"}, {
      "key": "015F",
      "mappings": {"default": {"default": "s cédille"}},
      "category": "Ll"
    }, {"key": "0161", "mappings": {"default": {"default": "s caron"}}, "category": "Ll"}, {
      "key": "0163",
      "mappings": {"default": {"default": "t cédille"}},
      "category": "Ll"
    }, {"key": "0165", "mappings": {"default": {"default": "t caron"}}, "category": "Ll"}, {
      "key": "0169",
      "mappings": {"default": {"default": "u tilde"}},
      "category": "Ll"
    }, {"key": "016B", "mappings": {"default": {"default": "u macron"}}, "category": "Ll"}, {
      "key": "016D",
      "mappings": {"default": {"default": "u brève"}},
      "category": "Ll"
    }, {"key": "016F", "mappings": {"default": {"default": "u rond en chef"}}, "category": "Ll"}, {
      "key": "0171",
      "mappings": {"default": {"default": "u avec double accent aigu"}},
      "category": "Ll"
    }, {"key": "0173", "mappings": {"default": {"default": "u ogonek"}}, "category": "Ll"}, {
      "key": "0175",
      "mappings": {"default": {"default": "w accent circonflexe"}},
      "category": "Ll"
    }, {"key": "0177", "mappings": {"default": {"default": "y accent circonflexe"}}, "category": "Ll"}, {
      "key": "017A",
      "mappings": {"default": {"default": "z accent aigu"}},
      "category": "Ll"
    }, {"key": "017C", "mappings": {"default": {"default": "z point en chef"}}, "category": "Ll"}, {
      "key": "017E",
      "mappings": {"default": {"default": "z caron"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "01CE", "mappings": {"default": {"default": "a caron"}}}, {
      "category": "Ll",
      "key": "01D0",
      "mappings": {"default": {"default": "i caron"}}
    }, {"category": "Ll", "key": "01D2", "mappings": {"default": {"default": "o caron"}}}, {
      "category": "Ll",
      "key": "01D4",
      "mappings": {"default": {"default": "u caron"}}
    }, {"category": "Ll", "key": "01E7", "mappings": {"default": {"default": "g caron"}}}, {
      "category": "Ll",
      "key": "01E9",
      "mappings": {"default": {"default": "k caron"}}
    }, {"category": "Ll", "key": "01EB", "mappings": {"default": {"default": "o ogonek"}}}, {
      "category": "Ll",
      "key": "01F0",
      "mappings": {"default": {"default": "j caron"}}
    }, {"key": "01F5", "mappings": {"default": {"default": "g accent aigu"}}, "category": "Ll"}, {
      "category": "Ll",
      "key": "01F9",
      "mappings": {"default": {"default": "n grave"}}
    }, {
      "category": "Ll",
      "key": "0201",
      "mappings": {"default": {"default": "a double accent grave"}}
    }, {"category": "Ll", "key": "0203", "mappings": {"default": {"default": "a brève renversée"}}}, {
      "category": "Ll",
      "key": "0205",
      "mappings": {"default": {"default": "e double accent grave"}}
    }, {"category": "Ll", "key": "0207", "mappings": {"default": {"default": "e brève renversée"}}}, {
      "category": "Ll",
      "key": "0209",
      "mappings": {"default": {"default": "i double accent grave"}}
    }, {"category": "Ll", "key": "020B", "mappings": {"default": {"default": "i brève renversée"}}}, {
      "category": "Ll",
      "key": "020D",
      "mappings": {"default": {"default": "o double accent grave"}}
    }, {"category": "Ll", "key": "020F", "mappings": {"default": {"default": "o brève renversée"}}}, {
      "category": "Ll",
      "key": "0211",
      "mappings": {"default": {"default": "r double accent grave"}}
    }, {"category": "Ll", "key": "0213", "mappings": {"default": {"default": "r brève renversée"}}}, {
      "category": "Ll",
      "key": "0215",
      "mappings": {"default": {"default": "u double accent grave"}}
    }, {"category": "Ll", "key": "0217", "mappings": {"default": {"default": "u brève renversée"}}}, {
      "category": "Ll",
      "key": "0219",
      "mappings": {"default": {"default": "s virgule souscrite *"}}
    }, {
      "category": "Ll",
      "key": "021B",
      "mappings": {"default": {"default": "t virgule souscrite *"}}
    }, {"category": "Ll", "key": "021F", "mappings": {"default": {"default": "h caron"}}}, {
      "category": "Ll",
      "key": "0227",
      "mappings": {"default": {"default": "a point en chef"}}
    }, {"category": "Ll", "key": "0229", "mappings": {"default": {"default": "e cédille"}}}, {
      "category": "Ll",
      "key": "022F",
      "mappings": {"default": {"default": "o point en chef"}}
    }, {"category": "Ll", "key": "0233", "mappings": {"default": {"default": "y macron"}}}, {
      "category": "Ll",
      "key": "0237",
      "mappings": {"default": {"default": "j sans point"}}
    }, {"key": "1E01", "mappings": {"default": {"default": "a à rond souscrit"}}, "category": "Ll"}, {
      "key": "1E03",
      "mappings": {"default": {"default": "b point en chef"}},
      "category": "Ll"
    }, {"key": "1E05", "mappings": {"default": {"default": "b point souscrit"}}, "category": "Ll"}, {
      "key": "1E07",
      "mappings": {"default": {"default": "b ligne souscrite"}},
      "category": "Ll"
    }, {"key": "1E0B", "mappings": {"default": {"default": "d point en chef"}}, "category": "Ll"}, {
      "key": "1E0D",
      "mappings": {"default": {"default": "d point souscrit"}},
      "category": "Ll"
    }, {"key": "1E0F", "mappings": {"default": {"default": "d ligne souscrite"}}, "category": "Ll"}, {
      "key": "1E11",
      "mappings": {"default": {"default": "d cédille"}},
      "category": "Ll"
    }, {
      "key": "1E13",
      "mappings": {"default": {"default": "d accent circonflexe souscrit"}},
      "category": "Ll"
    }, {
      "key": "1E19",
      "mappings": {"default": {"default": "e accent circonflexe souscrit"}},
      "category": "Ll"
    }, {"key": "1E1B", "mappings": {"default": {"default": "e tilde souscrit"}}, "category": "Ll"}, {
      "key": "1E1F",
      "mappings": {"default": {"default": "f point en chef"}},
      "category": "Ll"
    }, {"key": "1E21", "mappings": {"default": {"default": "g macron"}}, "category": "Ll"}, {
      "key": "1E23",
      "mappings": {"default": {"default": "h point en chef"}},
      "category": "Ll"
    }, {"key": "1E25", "mappings": {"default": {"default": "h point souscrit"}}, "category": "Ll"}, {
      "key": "1E27",
      "mappings": {"default": {"default": "h tréma"}},
      "category": "Ll"
    }, {"key": "1E29", "mappings": {"default": {"default": "h cédille"}}, "category": "Ll"}, {
      "key": "1E2B",
      "mappings": {"default": {"default": "h brève souscrit"}},
      "category": "Ll"
    }, {"key": "1E2D", "mappings": {"default": {"default": "i tilde souscrit"}}, "category": "Ll"}, {
      "key": "1E31",
      "mappings": {"default": {"default": "k accent aigu"}},
      "category": "Ll"
    }, {"key": "1E33", "mappings": {"default": {"default": "k point souscrit"}}, "category": "Ll"}, {
      "key": "1E35",
      "mappings": {"default": {"default": "k ligne souscrite"}},
      "category": "Ll"
    }, {"key": "1E37", "mappings": {"default": {"default": "l point souscrit"}}, "category": "Ll"}, {
      "key": "1E3B",
      "mappings": {"default": {"default": "l ligne souscrite"}},
      "category": "Ll"
    }, {
      "key": "1E3D",
      "mappings": {"default": {"default": "l accent circonflexe souscrit"}},
      "category": "Ll"
    }, {"key": "1E3F", "mappings": {"default": {"default": "m accent aigu"}}, "category": "Ll"}, {
      "key": "1E41",
      "mappings": {"default": {"default": "m point en chef"}},
      "category": "Ll"
    }, {"key": "1E43", "mappings": {"default": {"default": "m point souscrit"}}, "category": "Ll"}, {
      "key": "1E45",
      "mappings": {"default": {"default": "n point en chef"}},
      "category": "Ll"
    }, {"key": "1E47", "mappings": {"default": {"default": "n point souscrit"}}, "category": "Ll"}, {
      "key": "1E49",
      "mappings": {"default": {"default": "n ligne souscrite"}},
      "category": "Ll"
    }, {
      "key": "1E4B",
      "mappings": {"default": {"default": "n accent circonflexe souscrit"}},
      "category": "Ll"
    }, {"key": "1E55", "mappings": {"default": {"default": "p accent aigu"}}, "category": "Ll"}, {
      "key": "1E57",
      "mappings": {"default": {"default": "p point en chef"}},
      "category": "Ll"
    }, {"key": "1E59", "mappings": {"default": {"default": "r point en chef"}}, "category": "Ll"}, {
      "key": "1E5B",
      "mappings": {"default": {"default": "r point souscrit"}},
      "category": "Ll"
    }, {"key": "1E5F", "mappings": {"default": {"default": "r ligne souscrite"}}, "category": "Ll"}, {
      "key": "1E61",
      "mappings": {"default": {"default": "s point en chef"}},
      "category": "Ll"
    }, {"key": "1E63", "mappings": {"default": {"default": "s point souscrit"}}, "category": "Ll"}, {
      "key": "1E6B",
      "mappings": {"default": {"default": "t point en chef"}},
      "category": "Ll"
    }, {"key": "1E6D", "mappings": {"default": {"default": "t point souscrit"}}, "category": "Ll"}, {
      "key": "1E6F",
      "mappings": {"default": {"default": "t ligne souscrite"}},
      "category": "Ll"
    }, {
      "key": "1E71",
      "mappings": {"default": {"default": "t accent circonflexe souscrit"}},
      "category": "Ll"
    }, {"key": "1E73", "mappings": {"default": {"default": "u tréma souscrit"}}, "category": "Ll"}, {
      "key": "1E75",
      "mappings": {"default": {"default": "u tilde souscrit"}},
      "category": "Ll"
    }, {
      "key": "1E77",
      "mappings": {"default": {"default": "u accent circonflexe souscrit"}},
      "category": "Ll"
    }, {"key": "1E7D", "mappings": {"default": {"default": "v tilde"}}, "category": "Ll"}, {
      "key": "1E7F",
      "mappings": {"default": {"default": "v point souscrit"}},
      "category": "Ll"
    }, {"key": "1E81", "mappings": {"default": {"default": "w accent grave"}}, "category": "Ll"}, {
      "key": "1E83",
      "mappings": {"default": {"default": "w accent aigu"}},
      "category": "Ll"
    }, {"key": "1E85", "mappings": {"default": {"default": "w tréma"}}, "category": "Ll"}, {
      "key": "1E87",
      "mappings": {"default": {"default": "w point en chef"}},
      "category": "Ll"
    }, {"key": "1E89", "mappings": {"default": {"default": "w point souscrit"}}, "category": "Ll"}, {
      "key": "1E8B",
      "mappings": {"default": {"default": "x point en chef"}},
      "category": "Ll"
    }, {"key": "1E8D", "mappings": {"default": {"default": "x tréma"}}, "category": "Ll"}, {
      "key": "1E8F",
      "mappings": {"default": {"default": "y point en chef"}},
      "category": "Ll"
    }, {"key": "1E91", "mappings": {"default": {"default": "z accent circonflexe"}}, "category": "Ll"}, {
      "key": "1E93",
      "mappings": {"default": {"default": "z point souscrit"}},
      "category": "Ll"
    }, {"key": "1E95", "mappings": {"default": {"default": "z ligne souscrite"}}, "category": "Ll"}, {
      "key": "1E96",
      "mappings": {"default": {"default": "h ligne souscrite"}},
      "category": "Ll"
    }, {"key": "1E97", "mappings": {"default": {"default": "t tréma"}}, "category": "Ll"}, {
      "key": "1E98",
      "mappings": {"default": {"default": "w rond en chef"}},
      "category": "Ll"
    }, {"key": "1E99", "mappings": {"default": {"default": "y rond en chef"}}, "category": "Ll"}, {
      "key": "1E9A",
      "mappings": {"default": {"default": "a demi rond à droite"}},
      "category": "Ll"
    }, {"key": "1EA1", "mappings": {"default": {"default": "a point souscrit"}}, "category": "Ll"}, {
      "key": "1EA3",
      "mappings": {"default": {"default": "a crochet en chef"}},
      "category": "Ll"
    }, {"key": "1EB9", "mappings": {"default": {"default": "e point souscrit"}}, "category": "Ll"}, {
      "key": "1EBB",
      "mappings": {"default": {"default": "e crochet en chef"}},
      "category": "Ll"
    }, {"key": "1EBD", "mappings": {"default": {"default": "e tilde"}}, "category": "Ll"}, {
      "key": "1EC9",
      "mappings": {"default": {"default": "i crochet en chef"}},
      "category": "Ll"
    }, {"key": "1ECB", "mappings": {"default": {"default": "i point souscrit"}}, "category": "Ll"}, {
      "key": "1ECD",
      "mappings": {"default": {"default": "o point souscrit"}},
      "category": "Ll"
    }, {"key": "1ECF", "mappings": {"default": {"default": "o crochet en chef"}}, "category": "Ll"}, {
      "key": "1EE5",
      "mappings": {"default": {"default": "u point souscrit"}},
      "category": "Ll"
    }, {"key": "1EE7", "mappings": {"default": {"default": "u crochet en chef"}}, "category": "Ll"}, {
      "key": "1EF3",
      "mappings": {"default": {"default": "y accent grave"}},
      "category": "Ll"
    }, {"key": "1EF5", "mappings": {"default": {"default": "y point souscrit"}}, "category": "Ll"}, {
      "key": "1EF7",
      "mappings": {"default": {"default": "y crochet en chef"}},
      "category": "Ll"
    }, {"key": "1EF9", "mappings": {"default": {"default": "y tilde"}}, "category": "Ll"}]
  ,
  "fr/symbols/latin-rest.js":
    [{"locale": "fr"}, {
      "category": "Ll",
      "key": "210E",
      "mappings": {"default": {"default": "constante de planck"}}
    }, {"category": "Mn", "key": "0363", "mappings": {"default": {"default": "diacritique a"}}}, {
      "category": "Mn",
      "key": "0364",
      "mappings": {"default": {"default": "diacritique e"}}
    }, {"category": "Mn", "key": "0365", "mappings": {"default": {"default": "diacritique i"}}}, {
      "category": "Mn",
      "key": "0366",
      "mappings": {"default": {"default": "diacritique o"}}
    }, {"category": "Mn", "key": "0367", "mappings": {"default": {"default": "diacritique u"}}}, {
      "category": "Mn",
      "key": "0368",
      "mappings": {"default": {"default": "diacritique c"}}
    }, {"category": "Mn", "key": "0369", "mappings": {"default": {"default": "diacritique d"}}}, {
      "category": "Mn",
      "key": "036A",
      "mappings": {"default": {"default": "diacritique h"}}
    }, {"category": "Mn", "key": "036B", "mappings": {"default": {"default": "diacritique m"}}}, {
      "category": "Mn",
      "key": "036C",
      "mappings": {"default": {"default": "diacritique r"}}
    }, {"category": "Mn", "key": "036D", "mappings": {"default": {"default": "diacritique t"}}}, {
      "category": "Mn",
      "key": "036E",
      "mappings": {"default": {"default": "diacritique v"}}
    }, {"category": "Mn", "key": "036F", "mappings": {"default": {"default": "diacritique x"}}}, {
      "category": "Lm",
      "key": "1D62",
      "mappings": {"default": {"default": "i souscrit"}}
    }, {"category": "Lm", "key": "1D63", "mappings": {"default": {"default": "r souscrit"}}}, {
      "category": "Lm",
      "key": "1D64",
      "mappings": {"default": {"default": "u souscrit"}}
    }, {"category": "Lm", "key": "1D65", "mappings": {"default": {"default": "v souscrit"}}}, {
      "category": "Mn",
      "key": "1DCA",
      "mappings": {"default": {"default": "diacritique r souscrite"}}
    }, {
      "category": "Mn",
      "key": "1DD3",
      "mappings": {"default": {"default": "diacritique a ouvert aplati suscrit"}}
    }, {"category": "Mn", "key": "1DD4", "mappings": {"default": {"default": "diacritique ae"}}}, {
      "category": "Mn",
      "key": "1DD5",
      "mappings": {"default": {"default": "diacritique ao"}}
    }, {"category": "Mn", "key": "1DD6", "mappings": {"default": {"default": "diacritique av"}}}, {
      "category": "Mn",
      "key": "1DD7",
      "mappings": {"default": {"default": "diacritique c cédille"}}
    }, {
      "category": "Mn",
      "key": "1DD8",
      "mappings": {"default": {"default": "diacritique d insulaire"}}
    }, {"category": "Mn", "key": "1DD9", "mappings": {"default": {"default": "diacritique eth"}}}, {
      "category": "Mn",
      "key": "1DDA",
      "mappings": {"default": {"default": "diacritique g"}}
    }, {
      "category": "Mn",
      "key": "1DDB",
      "mappings": {"default": {"default": "diacritique petite g majuscule"}}
    }, {"category": "Mn", "key": "1DDC", "mappings": {"default": {"default": "diacritique k"}}}, {
      "category": "Mn",
      "key": "1DDD",
      "mappings": {"default": {"default": "diacritique l"}}
    }, {
      "category": "Mn",
      "key": "1DDE",
      "mappings": {"default": {"default": "diacritique petite l majuscule"}}
    }, {
      "category": "Mn",
      "key": "1DDF",
      "mappings": {"default": {"default": "diacritique petite m majuscule"}}
    }, {"category": "Mn", "key": "1DE0", "mappings": {"default": {"default": "diacritique n"}}}, {
      "category": "Mn",
      "key": "1DE1",
      "mappings": {"default": {"default": "diacritique petite n majuscule"}}
    }, {
      "category": "Mn",
      "key": "1DE2",
      "mappings": {"default": {"default": "diacritique petite r majuscule"}}
    }, {
      "category": "Mn",
      "key": "1DE3",
      "mappings": {"default": {"default": "diacritique r rotunda"}}
    }, {"category": "Mn", "key": "1DE4", "mappings": {"default": {"default": "diacritique s"}}}, {
      "category": "Mn",
      "key": "1DE5",
      "mappings": {"default": {"default": "diacritique s long"}}
    }, {"category": "Mn", "key": "1DE6", "mappings": {"default": {"default": "diacritique z"}}}, {
      "category": "Lm",
      "key": "2071",
      "mappings": {"default": {"default": "exposant i"}}
    }, {"category": "Lm", "key": "207F", "mappings": {"default": {"default": "exposant n"}}}, {
      "category": "Lm",
      "key": "2090",
      "mappings": {"default": {"default": "indice a"}}
    }, {"category": "Lm", "key": "2091", "mappings": {"default": {"default": "indice e"}}}, {
      "category": "Lm",
      "key": "2092",
      "mappings": {"default": {"default": "indice o"}}
    }, {"category": "Lm", "key": "2093", "mappings": {"default": {"default": "indice x"}}}, {
      "category": "Lm",
      "key": "2094",
      "mappings": {"default": {"default": "indice schwa"}}
    }, {"category": "Lm", "key": "2095", "mappings": {"default": {"default": "souscrite h"}}}, {
      "category": "Lm",
      "key": "2096",
      "mappings": {"default": {"default": "souscrite k"}}
    }, {"category": "Lm", "key": "2097", "mappings": {"default": {"default": "souscrite l"}}}, {
      "category": "Lm",
      "key": "2098",
      "mappings": {"default": {"default": "souscrite m"}}
    }, {"category": "Lm", "key": "2099", "mappings": {"default": {"default": "souscrite n"}}}, {
      "category": "Lm",
      "key": "209A",
      "mappings": {"default": {"default": "souscrite p"}}
    }, {"category": "Lm", "key": "209B", "mappings": {"default": {"default": "souscrite s"}}}, {
      "category": "Lm",
      "key": "209C",
      "mappings": {"default": {"default": "souscrite t"}}
    }, {"category": "Lm", "key": "2C7C", "mappings": {"default": {"default": "j souscrite"}}}, {
      "category": "So",
      "key": "1F12A",
      "mappings": {"default": {"default": "s majuscule en écaille crochetée"}}
    }, {
      "category": "So",
      "key": "1F12B",
      "mappings": {"default": {"default": "c majuscule en italique cerclée"}}
    }, {
      "category": "So",
      "key": "1F12C",
      "mappings": {"default": {"default": "r majuscule en italique cerclée"}}
    }, {
      "category": "So",
      "key": "1F18A",
      "mappings": {"default": {"default": "P majuscule encadrée et en inversion avec croix"}}
    }]
  ,
  "fr/symbols/latin-upper-double-accent.js":
    [{"locale": "fr"}, {
      "category": "Lu",
      "key": "01D5",
      "mappings": {"default": {"default": "U majuscule trémaa et macron"}}
    }, {
      "category": "Lu",
      "key": "01D7",
      "mappings": {"default": {"default": "U majuscule trémaa et accent aigu"}}
    }, {
      "category": "Lu",
      "key": "01D9",
      "mappings": {"default": {"default": "U majuscule trémaa et caron"}}
    }, {
      "category": "Lu",
      "key": "01DB",
      "mappings": {"default": {"default": "U majuscule trémaa et accent grave"}}
    }, {
      "category": "Lu",
      "key": "01DE",
      "mappings": {"default": {"default": "A majuscule trémaa et macron"}}
    }, {
      "category": "Lu",
      "key": "01E0",
      "mappings": {"default": {"default": "A majuscule point en chef et macron"}}
    }, {
      "category": "Lu",
      "key": "01EC",
      "mappings": {"default": {"default": "O majuscule ogonek et macron"}}
    }, {
      "category": "Lu",
      "key": "01FA",
      "mappings": {"default": {"default": "A majuscule rond en chef et accent aigu"}}
    }, {
      "category": "Lu",
      "key": "022A",
      "mappings": {"default": {"default": "O majuscule tréma et macron"}}
    }, {
      "category": "Lu",
      "key": "022C",
      "mappings": {"default": {"default": "O majuscule tilde et macron"}}
    }, {
      "category": "Lu",
      "key": "0230",
      "mappings": {"default": {"default": "O majuscule point en chef et macron"}}
    }, {
      "key": "1E08",
      "mappings": {"default": {"default": "C cédille et accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E14",
      "mappings": {"default": {"default": "E macron et accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E16",
      "mappings": {"default": {"default": "E macron et accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E1C",
      "mappings": {"default": {"default": "E cédille et brève majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E2E",
      "mappings": {"default": {"default": "I tréma et accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E38",
      "mappings": {"default": {"default": "L point souscrit et macron majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E4C",
      "mappings": {"default": {"default": "O tilde et accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E4E",
      "mappings": {"default": {"default": "O tilde et tréma majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E50",
      "mappings": {"default": {"default": "O macron et accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E52",
      "mappings": {"default": {"default": "O macron et accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E5C",
      "mappings": {"default": {"default": "R point souscrit et macron majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E64",
      "mappings": {"default": {"default": "S accent aigu et point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E66",
      "mappings": {"default": {"default": "S caron et point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E68",
      "mappings": {"default": {"default": "S point souscrit et point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E78",
      "mappings": {"default": {"default": "U tilde et accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E7A",
      "mappings": {"default": {"default": "U macron et tréma majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EA4",
      "mappings": {"default": {"default": "A accent circonflexe et accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EA6",
      "mappings": {"default": {"default": "A accent circonflexe et accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EA8",
      "mappings": {"default": {"default": "A accent circonflexe et crochet en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EAA",
      "mappings": {"default": {"default": "A accent circonflexe et tilde majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EAC",
      "mappings": {"default": {"default": "A accent circonflexe et point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EAE",
      "mappings": {"default": {"default": "A brève et accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EB0",
      "mappings": {"default": {"default": "A brève et accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EB2",
      "mappings": {"default": {"default": "A brève et crochet en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EB4",
      "mappings": {"default": {"default": "A brève et tilde majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EB6",
      "mappings": {"default": {"default": "A brève et point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EBE",
      "mappings": {"default": {"default": "E accent circonflexe et accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EC0",
      "mappings": {"default": {"default": "E accent circonflexe et accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EC2",
      "mappings": {"default": {"default": "E accent circonflexe et crochet en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EC4",
      "mappings": {"default": {"default": "E accent circonflexe et tilde majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EC6",
      "mappings": {"default": {"default": "E accent circonflexe et point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1ED0",
      "mappings": {"default": {"default": "O accent circonflexe et accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1ED2",
      "mappings": {"default": {"default": "O accent circonflexe et accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "1ED4",
      "mappings": {"default": {"default": "O accent circonflexe et crochet en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1ED6",
      "mappings": {"default": {"default": "O accent circonflexe et tilde majuscule"}},
      "category": "Lu"
    }, {
      "key": "1ED8",
      "mappings": {"default": {"default": "O accent circonflexe et point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EDA",
      "mappings": {"default": {"default": "O cornu accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EDC",
      "mappings": {"default": {"default": "O cornu accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EDE",
      "mappings": {"default": {"default": "O cornu crochet en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EE0",
      "mappings": {"default": {"default": "O cornu tilde majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EE2",
      "mappings": {"default": {"default": "O cornu point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EE8",
      "mappings": {"default": {"default": "U cornu accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EEA",
      "mappings": {"default": {"default": "U cornu accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EEC",
      "mappings": {"default": {"default": "U cornu crochet en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EEE",
      "mappings": {"default": {"default": "U cornu tilde majuscule"}},
      "category": "Lu"
    }, {"key": "1EF0", "mappings": {"default": {"default": "U cornu point souscrit majuscule"}}, "category": "Lu"}]
  ,
  "fr/symbols/latin-upper-single-accent.js":
    [{"locale": "fr"}, {
      "key": "00C0",
      "mappings": {"default": {"default": "A accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "00C1",
      "mappings": {"default": {"default": "A accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "00C2",
      "mappings": {"default": {"default": "A accent circonflexe majuscule"}},
      "category": "Lu"
    }, {"key": "00C3", "mappings": {"default": {"default": "A tilde majuscule"}}, "category": "Lu"}, {
      "key": "00C4",
      "mappings": {"default": {"default": "A tréma majuscule"}},
      "category": "Lu"
    }, {"key": "00C5", "mappings": {"default": {"default": "angstrom"}}, "category": "Lu"}, {
      "key": "00C7",
      "mappings": {"default": {"default": "C cédille majuscule"}},
      "category": "Lu"
    }, {
      "key": "00C8",
      "mappings": {"default": {"default": "E accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "00C9",
      "mappings": {"default": {"default": "E accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "00CA",
      "mappings": {"default": {"default": "E accent circonflexe majuscule"}},
      "category": "Lu"
    }, {"key": "00CB", "mappings": {"default": {"default": "E tréma majuscule"}}, "category": "Lu"}, {
      "key": "00CC",
      "mappings": {"default": {"default": "I accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "00CD",
      "mappings": {"default": {"default": "I accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "00CE",
      "mappings": {"default": {"default": "I accent circonflexe majuscule"}},
      "category": "Lu"
    }, {"key": "00CF", "mappings": {"default": {"default": "I tréma majuscule"}}, "category": "Lu"}, {
      "key": "00D1",
      "mappings": {"default": {"default": "N tilde majuscule"}},
      "category": "Lu"
    }, {
      "key": "00D2",
      "mappings": {"default": {"default": "O accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "00D3",
      "mappings": {"default": {"default": "O accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "00D4",
      "mappings": {"default": {"default": "O accent circonflexe majuscule"}},
      "category": "Lu"
    }, {"key": "00D5", "mappings": {"default": {"default": "O tilde majuscule"}}, "category": "Lu"}, {
      "key": "00D6",
      "mappings": {"default": {"default": "O tréma majuscule"}},
      "category": "Lu"
    }, {
      "key": "00D9",
      "mappings": {"default": {"default": "U accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "00DA",
      "mappings": {"default": {"default": "U accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "00DB",
      "mappings": {"default": {"default": "U accent circonflexe majuscule"}},
      "category": "Lu"
    }, {"key": "00DC", "mappings": {"default": {"default": "U tréma majuscule"}}, "category": "Lu"}, {
      "key": "00DD",
      "mappings": {"default": {"default": "Y accent aigu majuscule"}},
      "category": "Lu"
    }, {"key": "0100", "mappings": {"default": {"default": "A macron majuscule"}}, "category": "Lu"}, {
      "key": "0102",
      "mappings": {"default": {"default": "A brève majuscule"}},
      "category": "Lu"
    }, {"key": "0104", "mappings": {"default": {"default": "A ogonek majuscule"}}, "category": "Lu"}, {
      "key": "0106",
      "mappings": {"default": {"default": "C accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "0108",
      "mappings": {"default": {"default": "C accent circonflexe majuscule"}},
      "category": "Lu"
    }, {
      "key": "010A",
      "mappings": {"default": {"default": "C point en chef majuscule"}},
      "category": "Lu"
    }, {"key": "010C", "mappings": {"default": {"default": "C caron majuscule"}}, "category": "Lu"}, {
      "key": "010E",
      "mappings": {"default": {"default": "D caron majuscule"}},
      "category": "Lu"
    }, {"key": "0112", "mappings": {"default": {"default": "E macron majuscule"}}, "category": "Lu"}, {
      "category": "Lu",
      "key": "0114",
      "mappings": {"default": {"default": "E brève majuscule"}}
    }, {
      "key": "0116",
      "mappings": {"default": {"default": "E point en chef majuscule"}},
      "category": "Lu"
    }, {"key": "0118", "mappings": {"default": {"default": "E ogonek majuscule"}}, "category": "Lu"}, {
      "key": "011A",
      "mappings": {"default": {"default": "E caron majuscule"}},
      "category": "Lu"
    }, {
      "key": "011C",
      "mappings": {"default": {"default": "G accent circonflexe majuscule"}},
      "category": "Lu"
    }, {"key": "011E", "mappings": {"default": {"default": "G brève majuscule"}}, "category": "Lu"}, {
      "key": "0120",
      "mappings": {"default": {"default": "G point en chef majuscule"}},
      "category": "Lu"
    }, {"key": "0122", "mappings": {"default": {"default": "G cédille majuscule"}}, "category": "Lu"}, {
      "key": "0124",
      "mappings": {"default": {"default": "H accent circonflexe majuscule"}},
      "category": "Lu"
    }, {"key": "0128", "mappings": {"default": {"default": "I tilde majuscule"}}, "category": "Lu"}, {
      "key": "012A",
      "mappings": {"default": {"default": "I macron majuscule"}},
      "category": "Lu"
    }, {"category": "Lu", "key": "012C", "mappings": {"default": {"default": "I brève majuscule"}}}, {
      "key": "012E",
      "mappings": {"default": {"default": "I ogonek majuscule"}},
      "category": "Lu"
    }, {
      "key": "0130",
      "mappings": {"default": {"default": "I point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "0134",
      "mappings": {"default": {"default": "J accent circonflexe majuscule"}},
      "category": "Lu"
    }, {"key": "0136", "mappings": {"default": {"default": "K cédille majuscule"}}, "category": "Lu"}, {
      "key": "0139",
      "mappings": {"default": {"default": "L accent aigu majuscule"}},
      "category": "Lu"
    }, {"key": "013B", "mappings": {"default": {"default": "L cédille majuscule"}}, "category": "Lu"}, {
      "key": "013D",
      "mappings": {"default": {"default": "L caron majuscule"}},
      "category": "Lu"
    }, {
      "key": "013F",
      "mappings": {"default": {"default": "L point médian majuscule"}},
      "category": "Lu"
    }, {
      "key": "0143",
      "mappings": {"default": {"default": "N accent aigu majuscule"}},
      "category": "Lu"
    }, {"key": "0145", "mappings": {"default": {"default": "N cédille majuscule"}}, "category": "Lu"}, {
      "key": "0147",
      "mappings": {"default": {"default": "N caron majuscule"}},
      "category": "Lu"
    }, {"key": "014C", "mappings": {"default": {"default": "O macron majuscule"}}, "category": "Lu"}, {
      "category": "Lu",
      "key": "014E",
      "mappings": {"default": {"default": "O brève majuscule"}}
    }, {
      "key": "0150",
      "mappings": {"default": {"default": "O avec double accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "0154",
      "mappings": {"default": {"default": "R accent aigu majuscule"}},
      "category": "Lu"
    }, {"key": "0156", "mappings": {"default": {"default": "R cédille majuscule"}}, "category": "Lu"}, {
      "key": "0158",
      "mappings": {"default": {"default": "R caron majuscule"}},
      "category": "Lu"
    }, {
      "key": "015A",
      "mappings": {"default": {"default": "S accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "015C",
      "mappings": {"default": {"default": "S accent circonflexe majuscule"}},
      "category": "Lu"
    }, {"key": "015E", "mappings": {"default": {"default": "S cédille majuscule"}}, "category": "Lu"}, {
      "key": "0160",
      "mappings": {"default": {"default": "S caron majuscule"}},
      "category": "Lu"
    }, {"key": "0162", "mappings": {"default": {"default": "T cédille majuscule"}}, "category": "Lu"}, {
      "key": "0164",
      "mappings": {"default": {"default": "T caron majuscule"}},
      "category": "Lu"
    }, {"key": "0168", "mappings": {"default": {"default": "U tilde majuscule"}}, "category": "Lu"}, {
      "key": "016A",
      "mappings": {"default": {"default": "U macron majuscule"}},
      "category": "Lu"
    }, {"key": "016C", "mappings": {"default": {"default": "U brève majuscule"}}, "category": "Lu"}, {
      "key": "016E",
      "mappings": {"default": {"default": "U rond en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "0170",
      "mappings": {"default": {"default": "U avec double accent aigu majuscule"}},
      "category": "Lu"
    }, {"key": "0172", "mappings": {"default": {"default": "U ogonek majuscule"}}, "category": "Lu"}, {
      "key": "0174",
      "mappings": {"default": {"default": "W accent circonflexe majuscule"}},
      "category": "Lu"
    }, {
      "key": "0176",
      "mappings": {"default": {"default": "Y accent circonflexe majuscule"}},
      "category": "Lu"
    }, {"key": "0178", "mappings": {"default": {"default": "Y tréma majuscule"}}, "category": "Lu"}, {
      "key": "0179",
      "mappings": {"default": {"default": "Z accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "017B",
      "mappings": {"default": {"default": "Z point en chef majuscule"}},
      "category": "Lu"
    }, {"key": "017D", "mappings": {"default": {"default": "Z caron majuscule"}}, "category": "Lu"}, {
      "category": "Lu",
      "key": "01CD",
      "mappings": {"default": {"default": "A caron majuscule"}}
    }, {"category": "Lu", "key": "01CF", "mappings": {"default": {"default": "I caron majuscule"}}}, {
      "category": "Lu",
      "key": "01D1",
      "mappings": {"default": {"default": "O caron majuscule"}}
    }, {"category": "Lu", "key": "01D3", "mappings": {"default": {"default": "U caron majuscule"}}}, {
      "category": "Lu",
      "key": "01E6",
      "mappings": {"default": {"default": "G caron majuscule"}}
    }, {"category": "Lu", "key": "01E8", "mappings": {"default": {"default": "K caron majuscule"}}}, {
      "category": "Lu",
      "key": "01EA",
      "mappings": {"default": {"default": "O ogonek majuscule"}}
    }, {
      "category": "Lu",
      "key": "01F4",
      "mappings": {"default": {"default": "G accent aigu majuscule"}}
    }, {"category": "Lu", "key": "01F8", "mappings": {"default": {"default": "N grave majuscule"}}}, {
      "category": "Lu",
      "key": "0200",
      "mappings": {"default": {"default": "A double accent grave majuscule"}}
    }, {
      "category": "Lu",
      "key": "0202",
      "mappings": {"default": {"default": "A brève renversée majuscule"}}
    }, {
      "category": "Lu",
      "key": "0204",
      "mappings": {"default": {"default": "E double accent grave majuscule"}}
    }, {
      "category": "Lu",
      "key": "0206",
      "mappings": {"default": {"default": "E brève renversée majuscule"}}
    }, {
      "category": "Lu",
      "key": "0208",
      "mappings": {"default": {"default": "I double accent grave majuscule"}}
    }, {
      "category": "Lu",
      "key": "020A",
      "mappings": {"default": {"default": "I brève renversée majuscule"}}
    }, {
      "category": "Lu",
      "key": "020C",
      "mappings": {"default": {"default": "O double accent grave majuscule"}}
    }, {
      "category": "Lu",
      "key": "020E",
      "mappings": {"default": {"default": "O brève renversée majuscule"}}
    }, {
      "category": "Lu",
      "key": "0210",
      "mappings": {"default": {"default": "R double accent grave majuscule"}}
    }, {
      "category": "Lu",
      "key": "0212",
      "mappings": {"default": {"default": "R brève renversée majuscule"}}
    }, {
      "category": "Lu",
      "key": "0214",
      "mappings": {"default": {"default": "U double accent grave majuscule"}}
    }, {
      "category": "Lu",
      "key": "0216",
      "mappings": {"default": {"default": "U brève renversée majuscule"}}
    }, {
      "category": "Lu",
      "key": "0218",
      "mappings": {"default": {"default": "S virgule souscrite * majuscule"}}
    }, {
      "category": "Lu",
      "key": "021A",
      "mappings": {"default": {"default": "T virgule souscrite * majuscule"}}
    }, {"category": "Lu", "key": "021E", "mappings": {"default": {"default": "H caron majuscule"}}}, {
      "category": "Lu",
      "key": "0226",
      "mappings": {"default": {"default": "A point en chef majuscule"}}
    }, {
      "category": "Lu",
      "key": "0228",
      "mappings": {"default": {"default": "E cédille majuscule"}}
    }, {
      "category": "Lu",
      "key": "022E",
      "mappings": {"default": {"default": "O point en chef majuscule"}}
    }, {"category": "Lu", "key": "0232", "mappings": {"default": {"default": "Y macron majuscule"}}}, {
      "key": "1E00",
      "mappings": {"default": {"default": "A à rond souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E02",
      "mappings": {"default": {"default": "B point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E04",
      "mappings": {"default": {"default": "B point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E06",
      "mappings": {"default": {"default": "B ligne souscrite majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E0A",
      "mappings": {"default": {"default": "D point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E0C",
      "mappings": {"default": {"default": "D point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E0E",
      "mappings": {"default": {"default": "D ligne souscrite majuscule"}},
      "category": "Lu"
    }, {"key": "1E10", "mappings": {"default": {"default": "D cédille majuscule"}}, "category": "Lu"}, {
      "key": "1E12",
      "mappings": {"default": {"default": "D accent circonflexe souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E18",
      "mappings": {"default": {"default": "E accent circonflexe souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E1A",
      "mappings": {"default": {"default": "E tilde souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E1E",
      "mappings": {"default": {"default": "F point en chef majuscule"}},
      "category": "Lu"
    }, {"key": "1E20", "mappings": {"default": {"default": "G macron majuscule"}}, "category": "Lu"}, {
      "key": "1E22",
      "mappings": {"default": {"default": "H point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E24",
      "mappings": {"default": {"default": "H point souscrit majuscule"}},
      "category": "Lu"
    }, {"key": "1E26", "mappings": {"default": {"default": "H tréma majuscule"}}, "category": "Lu"}, {
      "key": "1E28",
      "mappings": {"default": {"default": "H cédille majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E2A",
      "mappings": {"default": {"default": "H brève souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E2C",
      "mappings": {"default": {"default": "I tilde souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E30",
      "mappings": {"default": {"default": "K accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E32",
      "mappings": {"default": {"default": "K point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E34",
      "mappings": {"default": {"default": "K ligne souscrite majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E36",
      "mappings": {"default": {"default": "L point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E3A",
      "mappings": {"default": {"default": "L ligne souscrite majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E3C",
      "mappings": {"default": {"default": "L accent circonflexe souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E3E",
      "mappings": {"default": {"default": "M accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E40",
      "mappings": {"default": {"default": "M point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E42",
      "mappings": {"default": {"default": "M point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E44",
      "mappings": {"default": {"default": "N point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E46",
      "mappings": {"default": {"default": "N point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E48",
      "mappings": {"default": {"default": "N ligne souscrite majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E4A",
      "mappings": {"default": {"default": "N accent circonflexe souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E54",
      "mappings": {"default": {"default": "P accent aigu majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E56",
      "mappings": {"default": {"default": "P point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E58",
      "mappings": {"default": {"default": "R point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E5A",
      "mappings": {"default": {"default": "R point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E5E",
      "mappings": {"default": {"default": "R ligne souscrite majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E60",
      "mappings": {"default": {"default": "S point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E62",
      "mappings": {"default": {"default": "S point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E6A",
      "mappings": {"default": {"default": "T point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E6C",
      "mappings": {"default": {"default": "T point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E6E",
      "mappings": {"default": {"default": "T ligne souscrite majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E70",
      "mappings": {"default": {"default": "T accent circonflexe souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E72",
      "mappings": {"default": {"default": "U tréma souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E74",
      "mappings": {"default": {"default": "U tilde souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E76",
      "mappings": {"default": {"default": "U accent circonflexe souscrit majuscule"}},
      "category": "Lu"
    }, {"key": "1E7C", "mappings": {"default": {"default": "V tilde majuscule"}}, "category": "Lu"}, {
      "key": "1E7E",
      "mappings": {"default": {"default": "V point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E80",
      "mappings": {"default": {"default": "W accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E82",
      "mappings": {"default": {"default": "W accent aigu majuscule"}},
      "category": "Lu"
    }, {"key": "1E84", "mappings": {"default": {"default": "W tréma majuscule"}}, "category": "Lu"}, {
      "key": "1E86",
      "mappings": {"default": {"default": "W point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E88",
      "mappings": {"default": {"default": "W point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E8A",
      "mappings": {"default": {"default": "X point en chef majuscule"}},
      "category": "Lu"
    }, {"key": "1E8C", "mappings": {"default": {"default": "X tréma majuscule"}}, "category": "Lu"}, {
      "key": "1E8E",
      "mappings": {"default": {"default": "Y point en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E90",
      "mappings": {"default": {"default": "Z accent circonflexe majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E92",
      "mappings": {"default": {"default": "Z point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1E94",
      "mappings": {"default": {"default": "Z ligne souscrite majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EA0",
      "mappings": {"default": {"default": "A point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EA2",
      "mappings": {"default": {"default": "A crochet en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EB8",
      "mappings": {"default": {"default": "E point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EBA",
      "mappings": {"default": {"default": "E crochet en chef majuscule"}},
      "category": "Lu"
    }, {"key": "1EBC", "mappings": {"default": {"default": "E tilde majuscule"}}, "category": "Lu"}, {
      "key": "1EC8",
      "mappings": {"default": {"default": "I crochet en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1ECA",
      "mappings": {"default": {"default": "I point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1ECC",
      "mappings": {"default": {"default": "O point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1ECE",
      "mappings": {"default": {"default": "O crochet en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EE4",
      "mappings": {"default": {"default": "U point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EE6",
      "mappings": {"default": {"default": "U crochet en chef majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EF2",
      "mappings": {"default": {"default": "Y accent grave majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EF4",
      "mappings": {"default": {"default": "Y point souscrit majuscule"}},
      "category": "Lu"
    }, {
      "key": "1EF6",
      "mappings": {"default": {"default": "Y crochet en chef majuscule"}},
      "category": "Lu"
    }, {"key": "1EF8", "mappings": {"default": {"default": "Y tilde majuscule"}}, "category": "Lu"}]
  ,
  "fr/symbols/math_angles.js":
    [{"locale": "fr"}, {
      "key": "22BE",
      "mappings": {"default": {"default": "angle droit avec arc"}},
      "category": "Sm"
    }, {
      "key": "237C",
      "mappings": {"default": {"default": "angle droit avec flèche en zigzag vers le bas"}},
      "category": "Sm"
    }, {
      "key": "27C0",
      "mappings": {"default": {"default": "angle tridimensionnel"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "299B",
      "mappings": {"default": {"default": "angle mesuré s'ouvrant vers la gauche"}}
    }, {
      "category": "Sm",
      "key": "299C",
      "mappings": {"default": {"default": "variante d'angle avec carré"}}
    }, {
      "key": "299D",
      "mappings": {"default": {"default": "angle droit mesuré pointé"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "299E",
      "mappings": {"default": {"default": "angle avec s inscrit"}}
    }, {"category": "Sm", "key": "299F", "mappings": {"default": {"default": "angle aigu"}}}, {
      "category": "Sm",
      "key": "29A0",
      "mappings": {"default": {"default": "angle sphérique s'ouvrant vers la gauche"}}
    }, {
      "category": "Sm",
      "key": "29A1",
      "mappings": {"default": {"default": "angle sphérique s'ouvrant vers le haut"}}
    }, {"category": "Sm", "key": "29A2", "mappings": {"default": {"default": "angle culbuté"}}}, {
      "category": "Sm",
      "key": "29A3",
      "mappings": {"default": {"default": "angle réfléchi"}}
    }, {
      "key": "29A4",
      "mappings": {"default": {"default": "angle à barre souscrite"}},
      "category": "Sm"
    }, {
      "key": "29A5",
      "mappings": {"default": {"default": "angle à barre souscrite réfléchie"}},
      "category": "Sm"
    }, {
      "key": "29A6",
      "mappings": {"default": {"default": "angle oblique s'ouvrant vers le haut"}},
      "category": "Sm"
    }, {
      "key": "29A7",
      "mappings": {"default": {"default": "angle oblique s'ouvrant vers le bas"}},
      "category": "Sm"
    }, {
      "key": "29A8",
      "mappings": {"default": {"default": "angle calculé à bras vers l'est et bras fléché vers le nord est"}},
      "category": "Sm"
    }, {
      "key": "29A9",
      "mappings": {"default": {"default": "angle calculé à bras vers l'ouest et bras fléché vers le nord ouest"}},
      "category": "Sm"
    }, {
      "key": "29AA",
      "mappings": {"default": {"default": "angle calculé à bras vers l'est et bras fléché vers le sud est"}},
      "category": "Sm"
    }, {
      "key": "29AB",
      "mappings": {"default": {"default": "angle calculé à bras vers l'ouest et bras fléché vers le sud ouest"}},
      "category": "Sm"
    }, {
      "key": "29AC",
      "mappings": {"default": {"default": "angle calculé à bras vers le nord et bras fléché vers le nord est"}},
      "category": "Sm"
    }, {
      "key": "29AD",
      "mappings": {"default": {"default": "angle calculé à bras vers le nord et bras fléché vers le nord ouest"}},
      "category": "Sm"
    }, {
      "key": "29AE",
      "mappings": {"default": {"default": "angle calculé à bras vers le sud et bras fléché vers le sud est"}},
      "category": "Sm"
    }, {
      "key": "29AF",
      "mappings": {"default": {"default": "angle calculé à bras vers le sud et bras fléché vers le sud ouest"}},
      "category": "Sm"
    }]
  ,
  "fr/symbols/math_arrows.js":
    [{"locale": "fr"}, {
      "key": "2190",
      "mappings": {"default": {"default": "flèche gauche"}},
      "category": "Sm"
    }, {"key": "2191", "mappings": {"default": {"default": "flèche vers le haut"}}, "category": "Sm"}, {
      "key": "2192",
      "mappings": {"default": {"default": "flèche droite"}},
      "category": "Sm"
    }, {"key": "2193", "mappings": {"default": {"default": "flèche vers le bas"}}, "category": "Sm"}, {
      "key": "2194",
      "mappings": {"default": {"default": "flèche bilatérale"}},
      "category": "Sm"
    }, {"key": "2195", "mappings": {"default": {"default": "flèche haut et bas"}}, "category": "So"}, {
      "key": "2196",
      "mappings": {"default": {"default": "flèche nord ouest"}},
      "category": "So"
    }, {"key": "2197", "mappings": {"default": {"default": "flèche nord est"}}, "category": "So"}, {
      "key": "2198",
      "mappings": {"default": {"default": "flèche sud est"}},
      "category": "So"
    }, {"key": "2199", "mappings": {"default": {"default": "flèche sud ouest"}}, "category": "So"}, {
      "key": "219A",
      "mappings": {"default": {"default": "flèche barrée gauche"}},
      "category": "Sm"
    }, {"key": "219B", "mappings": {"default": {"default": "flèche barrée droite"}}, "category": "Sm"}, {
      "key": "219C",
      "mappings": {"default": {"default": "flèche ondulée gauche"}},
      "category": "So"
    }, {"key": "219D", "mappings": {"default": {"default": "flèche ondulée droite"}}, "category": "So"}, {
      "key": "219E",
      "mappings": {"default": {"default": "flèche à deux pointes gauche"}},
      "category": "So"
    }, {
      "key": "219F",
      "mappings": {"default": {"default": "flèche à deux pointes vers le haut"}},
      "category": "So"
    }, {
      "key": "21A0",
      "mappings": {"default": {"default": "flèche à deux pointes droite"}},
      "category": "Sm"
    }, {
      "key": "21A1",
      "mappings": {"default": {"default": "flèche à deux pointes vers le bas"}},
      "category": "So"
    }, {
      "key": "21A2",
      "mappings": {"default": {"default": "flèche empennée gauche"}},
      "category": "So"
    }, {
      "key": "21A3",
      "mappings": {"default": {"default": "flèche empennée droite"}},
      "category": "Sm"
    }, {
      "key": "21A4",
      "mappings": {"default": {"default": "flèche d'un taquet gauche"}},
      "category": "So"
    }, {
      "key": "21A5",
      "mappings": {"default": {"default": "flèche d'un taquet vers le haut"}},
      "category": "So"
    }, {
      "key": "21A6",
      "mappings": {"default": {"default": "flèche d'un taquet droite"}},
      "category": "Sm"
    }, {
      "key": "21A7",
      "mappings": {"default": {"default": "flèche d'un taquet vers le bas"}},
      "category": "So"
    }, {
      "key": "21A8",
      "mappings": {"default": {"default": "flèche haut bas sur base"}},
      "category": "So"
    }, {
      "key": "21A9",
      "mappings": {"default": {"default": "flèche avec crochet gauche"}},
      "category": "So"
    }, {
      "key": "21AA",
      "mappings": {"default": {"default": "flèche avec crochet droite"}},
      "category": "So"
    }, {
      "key": "21AB",
      "mappings": {"default": {"default": "flèche avec boucle gauche"}},
      "category": "So"
    }, {
      "key": "21AC",
      "mappings": {"default": {"default": "flèche avec boucle droite"}},
      "category": "So"
    }, {
      "key": "21AD",
      "mappings": {"default": {"default": "flèche ondulée bilatérale"}},
      "category": "So"
    }, {
      "key": "21AE",
      "mappings": {"default": {"default": "flèche barrée bilatérale"}},
      "category": "Sm"
    }, {
      "key": "21AF",
      "mappings": {"default": {"default": "flèche vers le bas en zigzag"}},
      "category": "So"
    }, {
      "key": "21B0",
      "mappings": {"default": {"default": "flèche vers le haut avec pointe gauche"}},
      "category": "So"
    }, {
      "key": "21B1",
      "mappings": {"default": {"default": "flèche vers le haut avec pointe droite"}},
      "category": "So"
    }, {
      "key": "21B2",
      "mappings": {"default": {"default": "flèche vers le bas avec pointe gauche"}},
      "category": "So"
    }, {
      "key": "21B3",
      "mappings": {"default": {"default": "flèche vers le bas avec pointe droite"}},
      "category": "So"
    }, {
      "key": "21B4",
      "mappings": {"default": {"default": "flèche vers le bas avec coin droit"}},
      "category": "So"
    }, {
      "key": "21B5",
      "mappings": {"default": {"default": "flèche vers le bas avec coin gauche"}},
      "category": "So"
    }, {
      "key": "21B6",
      "mappings": {"default": {"default": "flèche semi circulaire en sens positif"}},
      "category": "So"
    }, {
      "key": "21B7",
      "mappings": {"default": {"default": "flèche semi circulaire en sens négatif"}},
      "category": "So"
    }, {
      "key": "21B8",
      "mappings": {"default": {"default": "flèche nord ouest vers une barre horizontale longue"}},
      "category": "So"
    }, {
      "key": "21B9",
      "mappings": {"default": {"default": "flèche gauche vers un taquet sur flèche droite vers un taquet"}},
      "category": "So"
    }, {
      "key": "21BA",
      "mappings": {"default": {"default": "flèche en sens positif à cercle ouvert"}},
      "category": "So"
    }, {
      "key": "21BB",
      "mappings": {"default": {"default": "flèche en sens négatif à cercle ouvert"}},
      "category": "So"
    }, {
      "key": "21C4",
      "mappings": {"default": {"default": "flèche vers le haut par dessus flèche gauche"}},
      "category": "So"
    }, {
      "key": "21C5",
      "mappings": {"default": {"default": "flèche vers le haut à gauche d'une flèche vers le bas"}},
      "category": "So"
    }, {
      "key": "21C6",
      "mappings": {"default": {"default": "flèche gauche par dessus flèche droite"}},
      "category": "So"
    }, {
      "key": "21C7",
      "mappings": {"default": {"default": "paire de flèches gauche"}},
      "category": "So"
    }, {
      "key": "21C8",
      "mappings": {"default": {"default": "paire de flèches vers le haut"}},
      "category": "So"
    }, {
      "key": "21C9",
      "mappings": {"default": {"default": "paire de flèches droite"}},
      "category": "So"
    }, {
      "key": "21CA",
      "mappings": {"default": {"default": "paire de flèches vers le bas"}},
      "category": "So"
    }, {
      "key": "21CD",
      "mappings": {"default": {"default": "double flèche barrée gauche"}},
      "category": "So"
    }, {
      "key": "21CE",
      "mappings": {"default": {"default": "double flèche barrée bilatérale"}},
      "category": "Sm"
    }, {
      "key": "21CF",
      "mappings": {"default": {"default": "double flèche barrée droite"}},
      "category": "Sm"
    }, {"key": "21D0", "mappings": {"default": {"default": "double flèche gauche"}}, "category": "So"}, {
      "key": "21D1",
      "mappings": {"default": {"default": "double flèche vers le haut"}},
      "category": "So"
    }, {"key": "21D2", "mappings": {"default": {"default": "double flèche droite"}}, "category": "Sm"}, {
      "key": "21D3",
      "mappings": {"default": {"default": "double flèche vers le bas"}},
      "category": "So"
    }, {
      "key": "21D4",
      "mappings": {"default": {"default": "double flèche bilatérale"}},
      "category": "Sm"
    }, {
      "key": "21D5",
      "mappings": {"default": {"default": "double flèche haut bas"}},
      "category": "So"
    }, {
      "key": "21D6",
      "mappings": {"default": {"default": "double flèche nord ouest"}},
      "category": "So"
    }, {
      "key": "21D7",
      "mappings": {"default": {"default": "double flèche nord est"}},
      "category": "So"
    }, {"key": "21D8", "mappings": {"default": {"default": "double flèche sud est"}}, "category": "So"}, {
      "key": "21D9",
      "mappings": {"default": {"default": "double flèche sud ouest"}},
      "category": "So"
    }, {"key": "21DA", "mappings": {"default": {"default": "triple flèche gauche"}}, "category": "So"}, {
      "key": "21DB",
      "mappings": {"default": {"default": "triple flèche droite"}},
      "category": "So"
    }, {
      "key": "21DC",
      "mappings": {"default": {"default": "flèche en tire bouchon gauche"}},
      "category": "So"
    }, {
      "key": "21DD",
      "mappings": {"default": {"default": "flèche en tire bouchon droite"}},
      "category": "So"
    }, {
      "key": "21DE",
      "mappings": {"default": {"default": "flèche vers le haut avec double barre"}},
      "category": "So"
    }, {
      "key": "21DF",
      "mappings": {"default": {"default": "flèche vers le bas avec double barre"}},
      "category": "So"
    }, {
      "key": "21E0",
      "mappings": {"default": {"default": "flèche pointillée gauche"}},
      "category": "So"
    }, {
      "key": "21E1",
      "mappings": {"default": {"default": "flèche pointillée vers le haut"}},
      "category": "So"
    }, {
      "key": "21E2",
      "mappings": {"default": {"default": "flèche pointillée droite"}},
      "category": "So"
    }, {
      "key": "21E3",
      "mappings": {"default": {"default": "flèche pointillée vers le bas"}},
      "category": "So"
    }, {
      "key": "21E4",
      "mappings": {"default": {"default": "flèche gauche vers un taquet"}},
      "category": "So"
    }, {
      "key": "21E5",
      "mappings": {"default": {"default": "flèche droite vers un taquet"}},
      "category": "So"
    }, {"key": "21E6", "mappings": {"default": {"default": "flèche blanche gauche"}}, "category": "So"}, {
      "key": "21E7",
      "mappings": {"default": {"default": "flèche blanche vers le haut"}},
      "category": "So"
    }, {"key": "21E8", "mappings": {"default": {"default": "flèche blanche droite"}}, "category": "So"}, {
      "key": "21E9",
      "mappings": {"default": {"default": "flèche blanche vers le bas"}},
      "category": "So"
    }, {
      "key": "21EA",
      "mappings": {"default": {"default": "flèche blanche vers le haut à partir d'un taquet"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "21EB",
      "mappings": {"default": {"default": "flèche blanche vers le haut sur socle"}}
    }, {
      "category": "So",
      "key": "21EC",
      "mappings": {"default": {"default": "flèche blanche vers le haut sur socle et trait horizontal"}}
    }, {
      "category": "So",
      "key": "21ED",
      "mappings": {"default": {"default": "flèche blanche vers le haut sur socle et barre verticale"}}
    }, {
      "category": "So",
      "key": "21EE",
      "mappings": {"default": {"default": "double flèche blanche vers le haut"}}
    }, {
      "category": "So",
      "key": "21EF",
      "mappings": {"default": {"default": "double flèche blanche vers le haut sur socle"}}
    }, {
      "category": "So",
      "key": "21F0",
      "mappings": {"default": {"default": "flèche blanche vers la droite appuyée sur mur"}}
    }, {
      "category": "So",
      "key": "21F1",
      "mappings": {"default": {"default": "flèche nord ouest en encoignure"}}
    }, {
      "category": "So",
      "key": "21F2",
      "mappings": {"default": {"default": "flèche sud est en encoignure"}}
    }, {
      "category": "So",
      "key": "21F3",
      "mappings": {"default": {"default": "flèche blanche vers le haut et le bas"}}
    }, {
      "category": "Sm",
      "key": "21F4",
      "mappings": {"default": {"default": "flèche vers la droite à petit cercle"}}
    }, {
      "key": "21F5",
      "mappings": {"default": {"default": "flèche descendante à gauche d'une flèche montante"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "21F6",
      "mappings": {"default": {"default": "trois flèches vers la droite"}}
    }, {
      "category": "Sm",
      "key": "21F7",
      "mappings": {"default": {"default": "flèche vers la gauche barrée verticalement"}}
    }, {
      "category": "Sm",
      "key": "21F8",
      "mappings": {"default": {"default": "flèche vers la droite barrée verticalement"}}
    }, {
      "category": "Sm",
      "key": "21F9",
      "mappings": {"default": {"default": "flèche vers la droite et la gauche barrée verticalement"}}
    }, {
      "category": "Sm",
      "key": "21FA",
      "mappings": {"default": {"default": "flèche vers la gauche barrée deux fois verticalement"}}
    }, {
      "category": "Sm",
      "key": "21FB",
      "mappings": {"default": {"default": "flèche vers la droite barrée deux fois verticalement"}}
    }, {
      "category": "Sm",
      "key": "21FC",
      "mappings": {"default": {"default": "flèche bilatérale barrée deux fois verticalement"}}
    }, {
      "key": "21FD",
      "mappings": {"default": {"default": "flèche à tête creuse gauche"}},
      "category": "Sm"
    }, {
      "key": "21FE",
      "mappings": {"default": {"default": "flèche à tête creuse droite"}},
      "category": "Sm"
    }, {
      "key": "21FF",
      "mappings": {"default": {"default": "flèche à tête creuse bilatérale"}},
      "category": "Sm"
    }, {"category": "So", "key": "2301", "mappings": {"default": {"default": "flèche électrique"}}}, {
      "category": "So",
      "key": "2303",
      "mappings": {"default": {"default": "pointe de flèche vers le haut"}}
    }, {
      "category": "So",
      "key": "2304",
      "mappings": {"default": {"default": "pointe de flèche vers le bas"}}
    }, {
      "category": "So",
      "key": "2324",
      "mappings": {"default": {"default": "pointe de flèche vers le haut entre deux traits horizontaux"}}
    }, {
      "category": "So",
      "key": "238B",
      "mappings": {"default": {"default": "cercle brisé à flèche nord ouest (échappement)"}}
    }, {
      "category": "So",
      "key": "2794",
      "mappings": {"default": {"default": "flèche grasse vers la droite à pointe large"}}
    }, {
      "category": "So",
      "key": "2798",
      "mappings": {"default": {"default": "flèche grasse sud est"}}
    }, {
      "category": "So",
      "key": "2799",
      "mappings": {"default": {"default": "flèche grasse vers la droite grasse"}}
    }, {
      "category": "So",
      "key": "279A",
      "mappings": {"default": {"default": "flèche grasse nord est"}}
    }, {
      "category": "So",
      "key": "279B",
      "mappings": {"default": {"default": "flèche de traçage vers la droite"}}
    }, {
      "category": "So",
      "key": "279C",
      "mappings": {"default": {"default": "flèche grasse à pointe arrondie vers la droite"}}
    }, {
      "category": "So",
      "key": "279D",
      "mappings": {"default": {"default": "flèche à pointe en triangle vers la droite"}}
    }, {
      "category": "So",
      "key": "279E",
      "mappings": {"default": {"default": "flèche grasse à pointe en triangle vers la droite"}}
    }, {
      "category": "So",
      "key": "279F",
      "mappings": {"default": {"default": "flèche avec pointillés à pointe en triangle vers la droite"}}
    }, {
      "category": "So",
      "key": "27A0",
      "mappings": {"default": {"default": "flèche grasse avec pointillés à pointe en triangle vers la droite"}}
    }, {
      "category": "So",
      "key": "27A1",
      "mappings": {"default": {"default": "flèche noire vers la droite"}}
    }, {
      "category": "So",
      "key": "27A2",
      "mappings": {"default": {"default": "pointe de flèche vers la droite en trompe l'œil éclairée par le haut"}}
    }, {
      "category": "So",
      "key": "27A3",
      "mappings": {"default": {"default": "pointe de flèche vers la droite en trompe l'œil éclairée par le bas"}}
    }, {
      "category": "So",
      "key": "27A4",
      "mappings": {"default": {"default": "pointe de flèche noire vers la droite"}}
    }, {
      "category": "So",
      "key": "27A5",
      "mappings": {"default": {"default": "flèche noire grasse courbée vers le bas et vers la droite"}}
    }, {
      "category": "So",
      "key": "27A6",
      "mappings": {"default": {"default": "flèche noire courbée vers le haut et vers la droite"}}
    }, {
      "category": "So",
      "key": "27A7",
      "mappings": {"default": {"default": "flèche noire trapue vers la droite"}}
    }, {
      "category": "So",
      "key": "27A8",
      "mappings": {"default": {"default": "flèche noire grasse à pointe concave vers la droite"}}
    }, {
      "category": "So",
      "key": "27A9",
      "mappings": {"default": {"default": "flèche blanche vers la droite ombrée à l'avant"}}
    }, {
      "category": "So",
      "key": "27AA",
      "mappings": {"default": {"default": "flèche blanche vers la droite ombrée à l'arrière"}}
    }, {
      "category": "So",
      "key": "27AB",
      "mappings": {"default": {"default": "flèche blanche vers la droite penchée vers l'arrière et ombrée"}}
    }, {
      "category": "So",
      "key": "27AC",
      "mappings": {"default": {"default": "flèche blanche vers la droite penchée vers l'avant et ombrée"}}
    }, {
      "category": "So",
      "key": "27AD",
      "mappings": {"default": {"default": "flèche blanche grasse vers la droite à ombre inférieure droite"}}
    }, {
      "category": "So",
      "key": "27AE",
      "mappings": {"default": {"default": "flèche blanche grasse vers la droite à ombre supérieure droite"}}
    }, {
      "category": "So",
      "key": "27AF",
      "mappings": {"default": {"default": "flèche encochée blanche vers la droite ombrée à l'avant et par dessous"}}
    }, {
      "category": "So",
      "key": "27B1",
      "mappings": {"default": {"default": "flèche encochée blanche vers la droite ombrée à l'avant et par dessus"}}
    }, {
      "category": "So",
      "key": "27B2",
      "mappings": {"default": {"default": "flèche blanche grasse vers la droite à contour circulaire"}}
    }, {
      "category": "So",
      "key": "27B3",
      "mappings": {"default": {"default": "flèche empennée blanche vers la droite"}}
    }, {
      "category": "So",
      "key": "27B4",
      "mappings": {"default": {"default": "flèche empennée noir sud est"}}
    }, {
      "category": "So",
      "key": "27B5",
      "mappings": {"default": {"default": "flèche empennée noir vers la droite"}}
    }, {
      "category": "So",
      "key": "27B6",
      "mappings": {"default": {"default": "flèche empennée noir nord est"}}
    }, {
      "category": "So",
      "key": "27B7",
      "mappings": {"default": {"default": "flèche empennée gras noire sud est"}}
    }, {
      "category": "So",
      "key": "27B8",
      "mappings": {"default": {"default": "flèche empennée gras noire vers la droite"}}
    }, {
      "category": "So",
      "key": "27B9",
      "mappings": {"default": {"default": "flèche empennée gras noire nord est"}}
    }, {
      "category": "So",
      "key": "27BA",
      "mappings": {"default": {"default": "flèche vers la droite à pointe larmée"}}
    }, {
      "category": "So",
      "key": "27BB",
      "mappings": {"default": {"default": "flèche vers la droite à hampe larmée"}}
    }, {
      "category": "So",
      "key": "27BC",
      "mappings": {"default": {"default": "flèche vers la droite à queue en forme de coin"}}
    }, {
      "category": "So",
      "key": "27BD",
      "mappings": {"default": {"default": "flèche grasse vers la droite à queue en forme de coin"}}
    }, {
      "category": "So",
      "key": "27BE",
      "mappings": {"default": {"default": "flèche vers la droite à contour ouvert"}}
    }, {
      "category": "Sm",
      "key": "27F0",
      "mappings": {"default": {"default": "quadruple flèche vers le haut"}}
    }, {
      "category": "Sm",
      "key": "27F1",
      "mappings": {"default": {"default": "quadruple flèche vers le bas"}}
    }, {
      "category": "Sm",
      "key": "27F2",
      "mappings": {"default": {"default": "flèche sens antihoraire entrouverte"}}
    }, {
      "category": "Sm",
      "key": "27F3",
      "mappings": {"default": {"default": "flèche sens horaire entrouverte"}}
    }, {
      "category": "Sm",
      "key": "27F4",
      "mappings": {"default": {"default": "flèche vers la droite à signe plus cerclé"}}
    }, {"key": "27F5", "mappings": {"default": {"default": "longue flèche gauche"}}, "category": "Sm"}, {
      "key": "27F6",
      "mappings": {"default": {"default": "longue flèche droite"}},
      "category": "Sm"
    }, {
      "key": "27F7",
      "mappings": {"default": {"default": "longue flèche bilatérale"}},
      "category": "Sm"
    }, {
      "key": "27F8",
      "mappings": {"default": {"default": "longue double flèche gauche"}},
      "category": "Sm"
    }, {
      "key": "27F9",
      "mappings": {"default": {"default": "longue double flèche droite"}},
      "category": "Sm"
    }, {
      "key": "27FA",
      "mappings": {"default": {"default": "longue double flèche bilatérale"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "27FB",
      "mappings": {"default": {"default": "longue flèche d'un taquet vers la gauche"}}
    }, {
      "key": "27FC",
      "mappings": {"default": {"default": "longue flèche d'un taquet droite"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "27FD",
      "mappings": {"default": {"default": "longue double flèche d'un taquet vers la gauche"}}
    }, {
      "category": "Sm",
      "key": "27FE",
      "mappings": {"default": {"default": "longue double flèche d'un taquet vers la droite"}}
    }, {
      "key": "27FF",
      "mappings": {"default": {"default": "longue flèche en tire bouchon droite"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2900",
      "mappings": {"default": {"default": "flèche à deux pointes vers la droite barrée verticalement"}}
    }, {
      "category": "Sm",
      "key": "2901",
      "mappings": {"default": {"default": "flèche à deux pointes vers la droite barrée deux fois verticalement"}}
    }, {
      "category": "Sm",
      "key": "2902",
      "mappings": {"default": {"default": "double flèche vers la gauche barrée verticalement"}}
    }, {
      "category": "Sm",
      "key": "2903",
      "mappings": {"default": {"default": "double flèche vers la droite barrée verticalement"}}
    }, {
      "category": "Sm",
      "key": "2904",
      "mappings": {"default": {"default": "flèche bilatérale double barrée verticalement"}}
    }, {
      "key": "2905",
      "mappings": {"default": {"default": "flèche à deux têtes d'un taquet droite"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2906",
      "mappings": {"default": {"default": "double flèche d'un taquet vers la gauche"}}
    }, {
      "category": "Sm",
      "key": "2907",
      "mappings": {"default": {"default": "double flèche d'un taquet vers la droite"}}
    }, {
      "category": "Sm",
      "key": "2908",
      "mappings": {"default": {"default": "flèche vers le bas barrée horizontalement"}}
    }, {
      "category": "Sm",
      "key": "2909",
      "mappings": {"default": {"default": "flèche vers le haut barrée horizontalement"}}
    }, {
      "category": "Sm",
      "key": "290A",
      "mappings": {"default": {"default": "triple flèche vers le haut"}}
    }, {
      "category": "Sm",
      "key": "290B",
      "mappings": {"default": {"default": "triple flèche vers le bas"}}
    }, {
      "key": "290C",
      "mappings": {"default": {"default": "flèche à deux traits gauche"}},
      "category": "Sm"
    }, {
      "key": "290D",
      "mappings": {"default": {"default": "flèche à deux traits droite"}},
      "category": "Sm"
    }, {
      "key": "290E",
      "mappings": {"default": {"default": "flèche à trois traits gauche"}},
      "category": "Sm"
    }, {
      "key": "290F",
      "mappings": {"default": {"default": "flèche à trois traits droite"}},
      "category": "Sm"
    }, {
      "key": "2910",
      "mappings": {"default": {"default": "flèche à trois traits et à deux têtes droite"}},
      "category": "Sm"
    }, {
      "key": "2911",
      "mappings": {"default": {"default": "flèche à fût pointillé droite"}},
      "category": "Sm"
    }, {
      "key": "2912",
      "mappings": {"default": {"default": "flèche jusqu'à taquet vers le haut"}},
      "category": "Sm"
    }, {
      "key": "2913",
      "mappings": {"default": {"default": "flèche jusqu'à taquet vers le bas"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2914",
      "mappings": {"default": {"default": "flèche empennée vers la droite barrée verticalement"}}
    }, {
      "category": "Sm",
      "key": "2915",
      "mappings": {"default": {"default": "flèche empennée vers la droite barrée deux fois verticalement"}}
    }, {
      "key": "2916",
      "mappings": {"default": {"default": "flèche empennée à deux têtes droite"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2917",
      "mappings": {"default": {"default": "flèche empennée à deux têtes vers la droite barrée verticalement"}}
    }, {
      "category": "Sm",
      "key": "2918",
      "mappings": {"default": {"default": "flèche empennée à deux têtes vers la droite barrée deux fois verticalement"}}
    }, {"key": "2919", "mappings": {"default": {"default": "empenne gauche"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "291A",
      "mappings": {"default": {"default": "empenne vers la droite"}}
    }, {"key": "291B", "mappings": {"default": {"default": "double empenne gauche"}}, "category": "Sm"}, {
      "key": "291C",
      "mappings": {"default": {"default": "double empenne droite"}},
      "category": "Sm"
    }, {
      "key": "291D",
      "mappings": {"default": {"default": "flèche vers un losange noir à gauche"}},
      "category": "Sm"
    }, {
      "key": "291E",
      "mappings": {"default": {"default": "flèche vers un losange noir à droite"}},
      "category": "Sm"
    }, {
      "key": "291F",
      "mappings": {"default": {"default": "flèche d'un taquet vers un losange noir à gauche"}},
      "category": "Sm"
    }, {
      "key": "2920",
      "mappings": {"default": {"default": "flèche d'un taquet vers un losange noir à droite"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2921",
      "mappings": {"default": {"default": "flèche nord ouest et sud est"}}
    }, {
      "category": "Sm",
      "key": "2922",
      "mappings": {"default": {"default": "flèche nord est et sud ouest"}}
    }, {
      "key": "2923",
      "mappings": {"default": {"default": "flèche nord ouest à crochet"}},
      "category": "Sm"
    }, {
      "key": "2924",
      "mappings": {"default": {"default": "flèche nord est à crochet"}},
      "category": "Sm"
    }, {
      "key": "2925",
      "mappings": {"default": {"default": "flèche sud est à crochet"}},
      "category": "Sm"
    }, {
      "key": "2926",
      "mappings": {"default": {"default": "flèche sud ouest à crochet"}},
      "category": "Sm"
    }, {
      "key": "2927",
      "mappings": {"default": {"default": "flèches nord ouest et nord est"}},
      "category": "Sm"
    }, {
      "key": "2928",
      "mappings": {"default": {"default": "flèches nord est et sud est"}},
      "category": "Sm"
    }, {
      "key": "2929",
      "mappings": {"default": {"default": "flèches sud est et sud ouest"}},
      "category": "Sm"
    }, {
      "key": "292A",
      "mappings": {"default": {"default": "flèches sud ouest et nord ouest"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "292D",
      "mappings": {"default": {"default": "flèche sud est sur flèche nord est"}}
    }, {
      "category": "Sm",
      "key": "292E",
      "mappings": {"default": {"default": "flèche nord est sur flèche sud est"}}
    }, {
      "category": "Sm",
      "key": "292F",
      "mappings": {"default": {"default": "diagonale descendante sur flèche nord est"}}
    }, {
      "category": "Sm",
      "key": "2930",
      "mappings": {"default": {"default": "diagonale montante sur flèche sud est"}}
    }, {
      "category": "Sm",
      "key": "2931",
      "mappings": {"default": {"default": "flèche nord est sur flèche nord ouest"}}
    }, {
      "category": "Sm",
      "key": "2932",
      "mappings": {"default": {"default": "flèche nord ouest sur flèche nord est"}}
    }, {
      "key": "2933",
      "mappings": {"default": {"default": "flèche bossue droite"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2934",
      "mappings": {"default": {"default": "flèche courbe vers la droite puis le haut"}}
    }, {
      "key": "2935",
      "mappings": {"default": {"default": "flèche courbe vers la droite puis le bas"}},
      "category": "Sm"
    }, {
      "key": "2936",
      "mappings": {"default": {"default": "flèche courbe vers le bas puis la gauche"}},
      "category": "Sm"
    }, {
      "key": "2937",
      "mappings": {"default": {"default": "flèche courbe vers le bas puis la droite"}},
      "category": "Sm"
    }, {
      "key": "2938",
      "mappings": {"default": {"default": "côté droit d'arc fléché sens horaire"}},
      "category": "Sm"
    }, {
      "key": "2939",
      "mappings": {"default": {"default": "côté gauche d'arc fléché sens antihoraire"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "293A",
      "mappings": {"default": {"default": "arc supérieur fléché sens antihoraire"}}
    }, {
      "category": "Sm",
      "key": "293B",
      "mappings": {"default": {"default": "arc inférieur fléché sens antihoraire"}}
    }, {
      "key": "293C",
      "mappings": {"default": {"default": "arc fléché sens horaire et signe moins souscrit"}},
      "category": "Sm"
    }, {
      "key": "293D",
      "mappings": {"default": {"default": "arc fléché sens antihoraire et signe plus souscrit"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "293E",
      "mappings": {"default": {"default": "demi cercle fléché inférieur droit sens horaire"}}
    }, {
      "category": "Sm",
      "key": "293F",
      "mappings": {"default": {"default": "demi cercle fléché inférieur gauche sens antihoraire"}}
    }, {
      "category": "Sm",
      "key": "2940",
      "mappings": {"default": {"default": "cercle fléché sens antihoraire"}}
    }, {
      "category": "Sm",
      "key": "2941",
      "mappings": {"default": {"default": "cercle fléché sens horaire"}}
    }, {
      "category": "Sm",
      "key": "2942",
      "mappings": {"default": {"default": "flèche vers la droite surmontant fléchette vers la gauche"}}
    }, {
      "category": "Sm",
      "key": "2943",
      "mappings": {"default": {"default": "flèche vers la gauche surmontant fléchette vers la droite"}}
    }, {
      "category": "Sm",
      "key": "2944",
      "mappings": {"default": {"default": "fléchette vers la droite surmontant flèche vers la gauche"}}
    }, {
      "key": "2945",
      "mappings": {"default": {"default": "flèche droite surmontant signe moins"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2946",
      "mappings": {"default": {"default": "flèche vers la gauche surmontant signe plus"}}
    }, {
      "category": "Sm",
      "key": "2947",
      "mappings": {"default": {"default": "flèche vers la droite au travers un x"}}
    }, {
      "key": "2948",
      "mappings": {"default": {"default": "petit cercle traversé d'une flèche bilatérale"}},
      "category": "Sm"
    }, {
      "key": "2949",
      "mappings": {"default": {"default": "petit cercle surmonté d'une flèche montante à deux têtes"}},
      "category": "Sm"
    }, {
      "key": "2970",
      "mappings": {"default": {"default": "flèche en épingle à cheveu droite"}},
      "category": "Sm"
    }, {
      "key": "2971",
      "mappings": {"default": {"default": "flèche droite surmontée d'un égal"}},
      "category": "Sm"
    }, {
      "key": "2972",
      "mappings": {"default": {"default": "flèche droite surmontée d'un tilde"}},
      "category": "Sm"
    }, {
      "key": "2973",
      "mappings": {"default": {"default": "tilde surmonté d'une flèche gauche"}},
      "category": "Sm"
    }, {
      "key": "2974",
      "mappings": {"default": {"default": "tilde surmonté d'une flèche droite"}},
      "category": "Sm"
    }, {
      "key": "2975",
      "mappings": {"default": {"default": "presque égal à surmonté d'une flèche droite"}},
      "category": "Sm"
    }, {
      "key": "2976",
      "mappings": {"default": {"default": "inférieur à surmonté d'une flèche gauche"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2977",
      "mappings": {"default": {"default": "flèche vers la gauche à travers un inférieur à"}}
    }, {
      "key": "2978",
      "mappings": {"default": {"default": "flèche droite surmontée d'un supérieur à"}},
      "category": "Sm"
    }, {
      "key": "2979",
      "mappings": {"default": {"default": "flèche droite surmontée d'un sous ensemble de"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "297A",
      "mappings": {"default": {"default": "flèche vers la gauche au travers d'un sous ensemble de"}}
    }, {
      "key": "297B",
      "mappings": {"default": {"default": "flèche gauche surmontée d'un sur ensemble de"}},
      "category": "Sm"
    }, {
      "key": "29B3",
      "mappings": {"default": {"default": "ensemble vide flèche droite en chef"}},
      "category": "Sm"
    }, {
      "key": "29B4",
      "mappings": {"default": {"default": "ensemble vide flèche gauche en chef"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29BD",
      "mappings": {"default": {"default": "cercle traversé d'une flèche montante"}}
    }, {
      "category": "Sm",
      "key": "29EA",
      "mappings": {"default": {"default": "losange noir à flèche vers le bas"}}
    }, {
      "category": "Sm",
      "key": "29EC",
      "mappings": {"default": {"default": "cercle blanc à flèche vers le bas"}}
    }, {
      "category": "Sm",
      "key": "29ED",
      "mappings": {"default": {"default": "cercle noir à flèche vers le bas"}}
    }, {
      "key": "2A17",
      "mappings": {"default": {"default": "intégrale à crosse fléchée gauche"}},
      "category": "Sm"
    }, {
      "category": "So",
      "key": "2B00",
      "mappings": {"default": {"default": "flèche blanche nord est"}}
    }, {
      "category": "So",
      "key": "2B01",
      "mappings": {"default": {"default": "flèche blanche nord ouest"}}
    }, {
      "category": "So",
      "key": "2B02",
      "mappings": {"default": {"default": "flèche blanche sud est"}}
    }, {
      "category": "So",
      "key": "2B03",
      "mappings": {"default": {"default": "flèche blanche sud ouest"}}
    }, {
      "category": "So",
      "key": "2B04",
      "mappings": {"default": {"default": "flèche blanche bilatérale"}}
    }, {
      "category": "So",
      "key": "2B05",
      "mappings": {"default": {"default": "flèche noire vers la gauche"}}
    }, {
      "category": "So",
      "key": "2B06",
      "mappings": {"default": {"default": "flèche noire vers le haut"}}
    }, {
      "category": "So",
      "key": "2B07",
      "mappings": {"default": {"default": "flèche noire vers le bas"}}
    }, {
      "category": "So",
      "key": "2B08",
      "mappings": {"default": {"default": "flèche noire nord est"}}
    }, {
      "category": "So",
      "key": "2B09",
      "mappings": {"default": {"default": "flèche noire nord ouest"}}
    }, {
      "category": "So",
      "key": "2B0A",
      "mappings": {"default": {"default": "flèche noire sud est"}}
    }, {
      "category": "So",
      "key": "2B0B",
      "mappings": {"default": {"default": "flèche noire sud ouest"}}
    }, {
      "category": "So",
      "key": "2B0C",
      "mappings": {"default": {"default": "flèche noire bilatérale"}}
    }, {
      "category": "So",
      "key": "2B0D",
      "mappings": {"default": {"default": "flèche noire haut et bas"}}
    }, {
      "category": "So",
      "key": "2B0E",
      "mappings": {"default": {"default": "flèche vers la droite à pointe pliée vers le bas"}}
    }, {
      "category": "So",
      "key": "2B0F",
      "mappings": {"default": {"default": "flèche vers la droite à pointe pliée vers le haut"}}
    }, {
      "category": "So",
      "key": "2B10",
      "mappings": {"default": {"default": "flèche vers la gauche à pointe pliée vers le bas"}}
    }, {
      "category": "So",
      "key": "2B11",
      "mappings": {"default": {"default": "flèche vers la gauche à pointe pliée vers le haut"}}
    }, {
      "category": "Sm",
      "key": "2B30",
      "mappings": {"default": {"default": "flèche vers la gauche à petit cercle"}}
    }, {
      "category": "Sm",
      "key": "2B31",
      "mappings": {"default": {"default": "trois flèches vers la gauche"}}
    }, {
      "category": "Sm",
      "key": "2B32",
      "mappings": {"default": {"default": "flèche vers la gauche à signe plus cerclé"}}
    }, {
      "category": "Sm",
      "key": "2B33",
      "mappings": {"default": {"default": "longue flèche vers la gauche en tire bouchon"}}
    }, {
      "category": "Sm",
      "key": "2B34",
      "mappings": {"default": {"default": "flèche à deux pointes vers la gauche barrée verticalement"}}
    }, {
      "category": "Sm",
      "key": "2B35",
      "mappings": {"default": {"default": "flèche à deux pointes vers la gauche barrée deux fois verticalement"}}
    }, {
      "category": "Sm",
      "key": "2B36",
      "mappings": {"default": {"default": "flèche à deux têtes d'un taquet vers la gauche"}}
    }, {
      "category": "Sm",
      "key": "2B37",
      "mappings": {"default": {"default": "flèche à trois traits et à deux têtes vers la gauche"}}
    }, {
      "category": "Sm",
      "key": "2B38",
      "mappings": {"default": {"default": "flèche vers la gauche à fût pointillé"}}
    }, {
      "category": "Sm",
      "key": "2B39",
      "mappings": {"default": {"default": "flèche empennée vers la gauche barrée verticalement"}}
    }, {
      "category": "Sm",
      "key": "2B3A",
      "mappings": {"default": {"default": "flèche empennée vers la gauche barrée deux fois verticalement"}}
    }, {
      "category": "Sm",
      "key": "2B3B",
      "mappings": {"default": {"default": "flèche empennée à deux têtes vers la gauche"}}
    }, {
      "category": "Sm",
      "key": "2B3C",
      "mappings": {"default": {"default": "flèche empennée à deux têtes vers la gauche barrée verticalement"}}
    }, {
      "category": "Sm",
      "key": "2B3D",
      "mappings": {"default": {"default": "flèche empennée à deux têtes vers la gauche barrée deux fois verticalement"}}
    }, {
      "category": "Sm",
      "key": "2B3E",
      "mappings": {"default": {"default": "flèche vers la gauche au travers un x"}}
    }, {
      "category": "Sm",
      "key": "2B3F",
      "mappings": {"default": {"default": "flèche vers la gauche bossue"}}
    }, {
      "category": "Sm",
      "key": "2B40",
      "mappings": {"default": {"default": "flèche vers la gauche surmontée d'un égal"}}
    }, {
      "category": "Sm",
      "key": "2B41",
      "mappings": {"default": {"default": "flèche vers la gauche surmontée d'un tilde"}}
    }, {
      "category": "Sm",
      "key": "2B42",
      "mappings": {"default": {"default": "presque égal à réfléchi surmonté d'une flèche vers la gauche"}}
    }, {
      "category": "Sm",
      "key": "2B43",
      "mappings": {"default": {"default": "flèche vers la droite à travers un inférieur à"}}
    }, {
      "category": "Sm",
      "key": "2B44",
      "mappings": {"default": {"default": "flèche vers la droite au travers d'un sous ensemble de"}}
    }, {
      "category": "So",
      "key": "2B45",
      "mappings": {"default": {"default": "quadruple flèche vers la gauche"}}
    }, {
      "category": "So",
      "key": "2B46",
      "mappings": {"default": {"default": "quadruple flèche vers la droite"}}
    }, {
      "category": "Sm",
      "key": "2B47",
      "mappings": {"default": {"default": "flèche vers la droite surmontée d'un opérateur tilde réfléchi"}}
    }, {
      "category": "Sm",
      "key": "2B48",
      "mappings": {"default": {"default": "presque égal à réfléchi surmonté d'une flèche vers la droite"}}
    }, {
      "category": "Sm",
      "key": "2B49",
      "mappings": {"default": {"default": "flèche vers la gauche surmontée de l'opérateur tilde"}}
    }, {
      "category": "Sm",
      "key": "2B4A",
      "mappings": {"default": {"default": "flèche vers la gauche par dessus presque égal à"}}
    }, {
      "category": "Sm",
      "key": "2B4B",
      "mappings": {"default": {"default": "tilde réfléchi surmonté d'une flèche vers la gauche"}}
    }, {
      "category": "Sm",
      "key": "2B4C",
      "mappings": {"default": {"default": "ilde surmonté d'une flèche vers la droite"}}
    }, {
      "category": "Sm",
      "key": "FFE9",
      "mappings": {"default": {"default": "flèche vers la gauche demi chasse"}}
    }, {
      "category": "Sm",
      "key": "FFEA",
      "mappings": {"default": {"default": "flèche vers le haut demi chasse"}}
    }, {
      "category": "Sm",
      "key": "FFEB",
      "mappings": {"default": {"default": "flèche vers la droite demi chasse"}}
    }, {"category": "Sm", "key": "FFEC", "mappings": {"default": {"default": "flèche vers le bas demi chasse"}}}]
  ,
  "fr/symbols/math_characters.js":
    [{"locale": "fr"}, {
      "key": "2113",
      "mappings": {"default": {"default": "l minuscule en script"}},
      "category": "Ll"
    }, {
      "key": "2118",
      "mappings": {"default": {"default": "fonction elliptique de weierstrass"}},
      "category": "Sm"
    }, {"category": "Ll", "key": "213C", "mappings": {"default": {"default": "pi ajouré"}}}, {
      "category": "Ll",
      "key": "213D",
      "mappings": {"default": {"default": "gamma ajouré"}}
    }, {
      "category": "Lu",
      "key": "213E",
      "mappings": {"default": {"default": "Gamma majuscule ajouré"}}
    }, {
      "category": "Lu",
      "key": "213F",
      "mappings": {"default": {"default": "Pi majuscule ajouré"}}
    }, {
      "category": "Sm",
      "key": "2140",
      "mappings": {"default": {"default": "sommation de la famille ajouré"}}
    }, {"key": "2145", "mappings": {"default": {"default": "D majuscule"}}, "category": "Lu"}, {
      "key": "2146",
      "mappings": {"default": {"default": "d ajouré en italique"}},
      "category": "Ll"
    }, {"key": "2147", "mappings": {"default": {"default": "e ajouré en italique"}}, "category": "Ll"}, {
      "key": "2148",
      "mappings": {"default": {"default": "i ajouré en italique"}},
      "category": "Ll"
    }, {"category": "Ll", "key": "2149", "mappings": {"default": {"default": "j ajouré en italique"}}}, {
      "key": "1D6A4",
      "mappings": {"default": {"default": "dotless i"}},
      "category": "Ll"
    }, {"key": "1D6A5", "mappings": {"default": {"default": "dotless j"}}, "category": "Ll"}]
  ,
  "fr/symbols/math_delimiters.js":
    [{"locale": "fr"}, {
      "key": "0028",
      "mappings": {"default": {"default": "parenthèse gauche", "alternative": "parenthèse ouvrante"}},
      "category": "Ps"
    }, {
      "key": "0029",
      "mappings": {"default": {"default": "parenthèse droite", "alternative": "parenthèse fermante"}},
      "category": "Pe"
    }, {
      "key": "005B",
      "mappings": {"default": {"default": "crochet gauche", "alternative": "crochet ouvrant"}},
      "category": "Ps"
    }, {
      "key": "005D",
      "mappings": {"default": {"default": "crochet droit", "alternative": "crochet fermant"}},
      "category": "Pe"
    }, {
      "key": "007B",
      "mappings": {"default": {"default": "accolade gauche", "alternative": "accolade ouvrante"}},
      "category": "Ps"
    }, {
      "key": "007D",
      "mappings": {"default": {"default": "accolade droite", "alternative": "accolade fermante"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "2045",
      "mappings": {"default": {"default": "crochet gauche avec pique"}}
    }, {
      "category": "Pe",
      "key": "2046",
      "mappings": {"default": {"default": "crochet droit avec pique"}}
    }, {"key": "2308", "mappings": {"default": {"default": "plafond à gauche"}}, "category": "Sm"}, {
      "key": "2309",
      "mappings": {"default": {"default": "plafond à droite"}},
      "category": "Sm"
    }, {"key": "230A", "mappings": {"default": {"default": "plancher à gauche"}}, "category": "Sm"}, {
      "key": "230B",
      "mappings": {"default": {"default": "plancher à droite"}},
      "category": "Sm"
    }, {"key": "230C", "mappings": {"default": {"default": "repère sud est"}}, "category": "So"}, {
      "key": "230D",
      "mappings": {"default": {"default": "repère sud ouest"}},
      "category": "So"
    }, {"key": "230E", "mappings": {"default": {"default": "repère nord est"}}, "category": "So"}, {
      "key": "230F",
      "mappings": {"default": {"default": "repère nord ouest"}},
      "category": "So"
    }, {"key": "231C", "mappings": {"default": {"default": "coin nord ouest"}}, "category": "So"}, {
      "key": "231D",
      "mappings": {"default": {"default": "coin nord est"}},
      "category": "So"
    }, {"key": "231E", "mappings": {"default": {"default": "coin sud ouest"}}, "category": "So"}, {
      "key": "231F",
      "mappings": {"default": {"default": "coin sud est"}},
      "category": "So"
    }, {
      "key": "2320",
      "mappings": {"default": {"default": "moitié supérieure d'intégrale"}},
      "category": "Sm"
    }, {
      "key": "2321",
      "mappings": {"default": {"default": "moitié inférieure d'intégrale"}},
      "category": "Sm"
    }, {"key": "2329", "mappings": {"default": {"default": "chevron gauche"}}, "category": "Ps"}, {
      "key": "232A",
      "mappings": {"default": {"default": "chevron droite"}},
      "category": "Pe"
    }, {
      "category": "Sm",
      "key": "239B",
      "mappings": {"default": {"default": "arc supérieur de parenthèse gauche"}}
    }, {
      "category": "Sm",
      "key": "239C",
      "mappings": {"default": {"default": "rallonge de parenthèse gauche"}}
    }, {
      "category": "Sm",
      "key": "239D",
      "mappings": {"default": {"default": "arc inférieur de parenthèse gauche"}}
    }, {
      "category": "Sm",
      "key": "239E",
      "mappings": {"default": {"default": "arc supérieur de parenthèse droite"}}
    }, {
      "category": "Sm",
      "key": "239F",
      "mappings": {"default": {"default": "rallonge de parenthèse droite"}}
    }, {
      "category": "Sm",
      "key": "23A0",
      "mappings": {"default": {"default": "arc inférieur de parenthèse droite"}}
    }, {
      "category": "Sm",
      "key": "23A1",
      "mappings": {"default": {"default": "coin supérieur de crochet gauche"}}
    }, {
      "category": "Sm",
      "key": "23A2",
      "mappings": {"default": {"default": "rallonge de crochet gauche"}}
    }, {
      "category": "Sm",
      "key": "23A3",
      "mappings": {"default": {"default": "coin inférieur de crochet gauche"}}
    }, {
      "category": "Sm",
      "key": "23A4",
      "mappings": {"default": {"default": "coin supérieur de crochet droit"}}
    }, {
      "category": "Sm",
      "key": "23A5",
      "mappings": {"default": {"default": "rallonge de crochet droit"}}
    }, {
      "category": "Sm",
      "key": "23A6",
      "mappings": {"default": {"default": "coin inférieur de crochet droit"}}
    }, {
      "category": "Sm",
      "key": "23A7",
      "mappings": {"default": {"default": "arc supérieur d'accolade gauche"}}
    }, {
      "category": "Sm",
      "key": "23A8",
      "mappings": {"default": {"default": "milieu d'accolade gauche"}}
    }, {
      "category": "Sm",
      "key": "23A9",
      "mappings": {"default": {"default": "arc inférieur d'accolade gauche"}}
    }, {
      "category": "Sm",
      "key": "23AA",
      "mappings": {"default": {"default": "rallonge d'accolade"}}
    }, {
      "category": "Sm",
      "key": "23AB",
      "mappings": {"default": {"default": "arc supérieur d'accolade droite"}}
    }, {
      "category": "Sm",
      "key": "23AC",
      "mappings": {"default": {"default": "milieu d'accolade droite"}}
    }, {
      "category": "Sm",
      "key": "23AD",
      "mappings": {"default": {"default": "arc inférieur d'accolade droite"}}
    }, {
      "category": "Sm",
      "key": "23AE",
      "mappings": {"default": {"default": "rallonge d'intégrale"}}
    }, {
      "category": "Sm",
      "key": "23AF",
      "mappings": {"default": {"default": "rallonge de ligne horizontale"}}
    }, {
      "key": "23B0",
      "mappings": {"default": {"default": "moitié supérieure gauche ou inférieure droite d'accolade"}},
      "category": "Sm"
    }, {
      "key": "23B1",
      "mappings": {"default": {"default": "moitié supérieure droite ou inférieure gauche d'accolade"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "23B2",
      "mappings": {"default": {"default": "haut du signe de somme"}}
    }, {"category": "Sm", "key": "23B3", "mappings": {"default": {"default": "bas du signe de somme"}}}, {
      "key": "23B4",
      "mappings": {"default": {"default": "crochet vertical supérieur"}},
      "category": "So"
    }, {
      "key": "23B5",
      "mappings": {"default": {"default": "crochet vertical inférieur"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "23B6",
      "mappings": {"default": {"default": "crochet verticaux en défilé"}}
    }, {"category": "So", "key": "23B7", "mappings": {"default": {"default": "bas du radical"}}}, {
      "category": "So",
      "key": "23B8",
      "mappings": {"default": {"default": "filet vertical gauche"}}
    }, {"category": "So", "key": "23B9", "mappings": {"default": {"default": "filet vertical droit"}}}, {
      "key": "23DC",
      "mappings": {"default": {"default": "parenthèse supérieure"}},
      "category": "Sm"
    }, {"key": "23DD", "mappings": {"default": {"default": "parenthèse inférieure"}}, "category": "Sm"}, {
      "key": "23DE",
      "mappings": {"default": {"default": "accolade supérieure"}},
      "category": "Sm"
    }, {"key": "23DF", "mappings": {"default": {"default": "accolade inférieure"}}, "category": "Sm"}, {
      "key": "23E0",
      "mappings": {"default": {"default": "crochet en écaille supérieur"}},
      "category": "Sm"
    }, {
      "key": "23E1",
      "mappings": {"default": {"default": "crochet en écaille inférieur"}},
      "category": "Sm"
    }, {
      "category": "Ps",
      "key": "2768",
      "mappings": {"default": {"default": "parenthèse de fantaisie gauche demi grasse"}}
    }, {
      "category": "Pe",
      "key": "2769",
      "mappings": {"default": {"default": "parenthèse de fantaisie droite demi grasse"}}
    }, {
      "category": "Ps",
      "key": "276A",
      "mappings": {"default": {"default": "parenthèse de fantaisie gauche demi grasse aplatie"}}
    }, {
      "category": "Pe",
      "key": "276B",
      "mappings": {"default": {"default": "parenthèse de fantaisie droite demi grasse aplatie"}}
    }, {
      "category": "Ps",
      "key": "276C",
      "mappings": {"default": {"default": "chevron de fantaisie vers la gauche demi gras"}}
    }, {
      "category": "Pe",
      "key": "276D",
      "mappings": {"default": {"default": "chevron de fantaisie vers la droite demi gras"}}
    }, {
      "category": "Ps",
      "key": "276E",
      "mappings": {"default": {"default": "guillemet de fantaisie vers la gauche gras"}}
    }, {
      "category": "Pe",
      "key": "276F",
      "mappings": {"default": {"default": "guillemet de fantaisie vers la droite gras"}}
    }, {
      "category": "Ps",
      "key": "2770",
      "mappings": {"default": {"default": "crochet de fantaisie vers la gauche gras"}}
    }, {
      "category": "Pe",
      "key": "2771",
      "mappings": {"default": {"default": "crochet de fantaisie vers la droite gras"}}
    }, {
      "category": "Ps",
      "key": "2772",
      "mappings": {"default": {"default": "crochet de fantaisie gauche maigre en écaille"}}
    }, {
      "category": "Pe",
      "key": "2773",
      "mappings": {"default": {"default": "crochet de fantaisie droit maigre en écaille"}}
    }, {
      "category": "Ps",
      "key": "2774",
      "mappings": {"default": {"default": "accolade de fantaisie gauche moyenne"}}
    }, {
      "category": "Pe",
      "key": "2775",
      "mappings": {"default": {"default": "accolade de fantaisie droite moyenne"}}
    }, {
      "key": "27C5",
      "mappings": {"default": {"default": "délimiteur de sac en s gauche"}},
      "category": "Ps"
    }, {
      "key": "27C6",
      "mappings": {"default": {"default": "délimiteur de sac en s droite"}},
      "category": "Pe"
    }, {"key": "27E6", "mappings": {"default": {"default": "crochet blanc gauche"}}, "category": "Ps"}, {
      "key": "27E7",
      "mappings": {"default": {"default": "crochet blanc droit"}},
      "category": "Pe"
    }, {
      "key": "27E8",
      "mappings": {"default": {"default": "chevron mathématique gauche"}},
      "category": "Ps"
    }, {"key": "27E9", "mappings": {"default": {"default": "chevron droit"}}, "category": "Pe"}, {
      "key": "27EA",
      "mappings": {"default": {"default": "double chevron gauche"}},
      "category": "Ps"
    }, {"key": "27EB", "mappings": {"default": {"default": "double chevron droit"}}, "category": "Pe"}, {
      "key": "27EC",
      "mappings": {"default": {"default": "crochet gauche en écaille blanche"}},
      "category": "Ps"
    }, {
      "key": "27ED",
      "mappings": {"default": {"default": "crochet droit en écaille blanche"}},
      "category": "Pe"
    }, {
      "key": "27EE",
      "mappings": {"default": {"default": "parenthèse gauche aplatie"}},
      "category": "Ps"
    }, {
      "key": "27EF",
      "mappings": {"default": {"default": "parenthèse droite aplatie"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "2983",
      "mappings": {"default": {"default": "accolade gauche ajourée"}}
    }, {
      "category": "Pe",
      "key": "2984",
      "mappings": {"default": {"default": "accolade droite ajourée"}}
    }, {
      "key": "2985",
      "mappings": {"default": {"default": "parenthèse gauche ajourée"}},
      "category": "Ps"
    }, {
      "key": "2986",
      "mappings": {"default": {"default": "parenthèse droite ajourée"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "2987",
      "mappings": {"default": {"default": "parenthèse image de gauche en notation z"}}
    }, {
      "category": "Pe",
      "key": "2988",
      "mappings": {"default": {"default": "parenthèse image de droite en notation z"}}
    }, {
      "key": "2989",
      "mappings": {"default": {"default": "z notation left binding bracket"}},
      "category": "Ps"
    }, {
      "key": "298A",
      "mappings": {"default": {"default": "z notation right binding bracket"}},
      "category": "Pe"
    }, {
      "key": "298B",
      "mappings": {"default": {"default": "crochet gauche avec barre souscrite"}},
      "category": "Ps"
    }, {
      "key": "298C",
      "mappings": {"default": {"default": "crochet droit avec barre souscrite"}},
      "category": "Pe"
    }, {
      "key": "298D",
      "mappings": {"default": {"default": "crochet gauche avec trait dans le coin supérieur"}},
      "category": "Ps"
    }, {
      "key": "298E",
      "mappings": {"default": {"default": "crochet droit avec trait dans le coin inférieur"}},
      "category": "Pe"
    }, {
      "key": "298F",
      "mappings": {"default": {"default": "crochet gauche avec trait dans le coin inférieur"}},
      "category": "Ps"
    }, {
      "key": "2990",
      "mappings": {"default": {"default": "crochet droit avec trait dans le coin supérieur"}},
      "category": "Pe"
    }, {"key": "2991", "mappings": {"default": {"default": "chevron gauche pointé"}}, "category": "Ps"}, {
      "key": "2992",
      "mappings": {"default": {"default": "chevron droit pointé"}},
      "category": "Pe"
    }, {
      "key": "2993",
      "mappings": {"default": {"default": "parenthèse arc gauche et inférieur à"}},
      "category": "Ps"
    }, {
      "key": "2994",
      "mappings": {"default": {"default": "parenthèse arc droite et supérieur à"}},
      "category": "Pe"
    }, {
      "key": "2995",
      "mappings": {"default": {"default": "parenthèse double arc droite et supérieur à"}},
      "category": "Ps"
    }, {
      "key": "2996",
      "mappings": {"default": {"default": "parenthèse double arc gauche et inférieur à"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "2997",
      "mappings": {"default": {"default": "crochet noir gauche en écaille"}}
    }, {
      "category": "Pe",
      "key": "2998",
      "mappings": {"default": {"default": "crochet noir droite en écaille"}}
    }, {
      "category": "Ps",
      "key": "29D8",
      "mappings": {"default": {"default": "clôture dentelée gauche"}}
    }, {
      "category": "Pe",
      "key": "29D9",
      "mappings": {"default": {"default": "clôture dentelée droite"}}
    }, {
      "key": "29DA",
      "mappings": {"default": {"default": "double clôture dentelée gauche"}},
      "category": "Ps"
    }, {
      "key": "29DB",
      "mappings": {"default": {"default": "right double wiggly fence"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "29FC",
      "mappings": {"default": {"default": "chevron vers la gauche courbé"}}
    }, {
      "category": "Pe",
      "key": "29FD",
      "mappings": {"default": {"default": "chevron vers la droite courbé"}}
    }, {
      "category": "Ps",
      "key": "2E22",
      "mappings": {"default": {"default": "anglet supérieur gauche"}}
    }, {
      "category": "Pe",
      "key": "2E23",
      "mappings": {"default": {"default": "anglet supérieur droit"}}
    }, {
      "category": "Ps",
      "key": "2E24",
      "mappings": {"default": {"default": "anglet inférieur gauche"}}
    }, {
      "category": "Pe",
      "key": "2E25",
      "mappings": {"default": {"default": "anglet inférieur droit"}}
    }, {
      "category": "Ps",
      "key": "2E26",
      "mappings": {"default": {"default": "crampillon couché gauche"}}
    }, {
      "category": "Pe",
      "key": "2E27",
      "mappings": {"default": {"default": "crampillon couché droit"}}
    }, {
      "category": "Ps",
      "key": "2E28",
      "mappings": {"default": {"default": "double parenthèse gauche"}}
    }, {
      "category": "Pe",
      "key": "2E29",
      "mappings": {"default": {"default": "double parenthèse droite"}}
    }, {"category": "Ps", "key": "3008", "mappings": {"default": {"default": "chevron gauche"}}}, {
      "category": "Pe",
      "key": "3009",
      "mappings": {"default": {"default": "chevron droit"}}
    }, {"key": "300A", "mappings": {"default": {"default": "double chevron gauche"}}, "category": "Ps"}, {
      "key": "300B",
      "mappings": {"default": {"default": "double chevron droit"}},
      "category": "Pe"
    }, {"category": "Ps", "key": "300C", "mappings": {"default": {"default": "anglet gauche"}}}, {
      "category": "Pe",
      "key": "300D",
      "mappings": {"default": {"default": "anglet droit"}}
    }, {
      "category": "Ps",
      "key": "300E",
      "mappings": {"default": {"default": "anglet ajouré gauche"}}
    }, {
      "category": "Pe",
      "key": "300F",
      "mappings": {"default": {"default": "anglet ajouré droit"}}
    }, {
      "category": "Ps",
      "key": "3010",
      "mappings": {"default": {"default": "crochet noir lenticulaire gauche"}}
    }, {
      "category": "Pe",
      "key": "3011",
      "mappings": {"default": {"default": "crochet noir lenticulaire droit"}}
    }, {
      "key": "3014",
      "mappings": {"default": {"default": "crochet gauche en écaille"}},
      "category": "Ps"
    }, {
      "key": "3015",
      "mappings": {"default": {"default": "crochet droit en écaille"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "3016",
      "mappings": {"default": {"default": "crochet blanc lenticulaire gauche"}}
    }, {
      "category": "Pe",
      "key": "3017",
      "mappings": {"default": {"default": "crochet blanc lenticulaire droit"}}
    }, {
      "category": "Ps",
      "key": "3018",
      "mappings": {"default": {"default": "crochet blanc gauche en écaille"}}
    }, {
      "category": "Pe",
      "key": "3019",
      "mappings": {"default": {"default": "crochet blanc droit en écaille"}}
    }, {"key": "301A", "mappings": {"default": {"default": "crochet blanc gauche"}}, "category": "Ps"}, {
      "key": "301B",
      "mappings": {"default": {"default": "crochet blanc droit"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "301D",
      "mappings": {"default": {"default": "guillemet double prime réfléchi"}}
    }, {
      "key": "301E",
      "mappings": {"default": {"default": "guillemet double prime"}},
      "category": "Pe"
    }, {
      "category": "Pe",
      "key": "301F",
      "mappings": {"default": {"default": "guillemet double prime inférieur"}}
    }, {
      "category": "Ps",
      "key": "FD3E",
      "mappings": {"default": {"default": "parenthèse gauche ornée"}}
    }, {
      "category": "Pe",
      "key": "FD3F",
      "mappings": {"default": {"default": "parenthèse droite ornée"}}
    }, {
      "category": "Ps",
      "key": "FE17",
      "mappings": {"default": {"default": "forme de présentation de crochet blanc lenticulaire gauche vertical"}}
    }, {
      "category": "Pe",
      "key": "FE18",
      "mappings": {"default": {"default": "forme de présentation de crochet blanc lenticulaire droit vertical"}}
    }, {
      "key": "FE35",
      "mappings": {"default": {"default": "forme de présentation de parenthèse gauche verticale"}},
      "category": "Ps"
    }, {
      "key": "FE36",
      "mappings": {"default": {"default": "forme de présentation de parenthèse droite verticale"}},
      "category": "Pe"
    }, {
      "key": "FE37",
      "mappings": {"default": {"default": "forme de présentation d'accolade gauche verticale"}},
      "category": "Ps"
    }, {
      "key": "FE38",
      "mappings": {"default": {"default": "forme de présentation d'accolade droite verticale"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "FE39",
      "mappings": {"default": {"default": "forme de présentation de crochet gauche vertical en écaille"}}
    }, {
      "category": "Pe",
      "key": "FE3A",
      "mappings": {"default": {"default": "forme de présentation de crochet droit vertical en écaille"}}
    }, {
      "category": "Ps",
      "key": "FE3B",
      "mappings": {"default": {"default": "forme de présentation de crochet gauche lenticulaire noir vertical"}}
    }, {
      "category": "Pe",
      "key": "FE3C",
      "mappings": {"default": {"default": "forme de présentation de crochet droit lenticulaire noir vertical"}}
    }, {
      "category": "Ps",
      "key": "FE3D",
      "mappings": {"default": {"default": "forme de présentation de crochet double gauche vertical"}}
    }, {
      "category": "Pe",
      "key": "FE3E",
      "mappings": {"default": {"default": "forme de présentation de crochet double droit vertical"}}
    }, {"key": "FE3F", "mappings": {"default": {"default": "over angle bracket"}}, "category": "Ps"}, {
      "key": "FE40",
      "mappings": {"default": {"default": "under angle bracket"}},
      "category": "Pe"
    }, {
      "category": "Ps",
      "key": "FE41",
      "mappings": {"default": {"default": "forme de présentation de crochet vertical gauche en coin"}}
    }, {
      "category": "Pe",
      "key": "FE42",
      "mappings": {"default": {"default": "forme de présentation de crochet vertical droit en coin"}}
    }, {
      "category": "Ps",
      "key": "FE43",
      "mappings": {"default": {"default": "forme de présentation de crochet blanc vertical gauche en coin"}}
    }, {
      "category": "Pe",
      "key": "FE44",
      "mappings": {"default": {"default": "forme de présentation de crochet blanc vertical droit en coin"}}
    }, {
      "category": "Ps",
      "key": "FE47",
      "mappings": {"default": {"default": "forme de présentation de crochet gauche vertical"}}
    }, {
      "category": "Pe",
      "key": "FE48",
      "mappings": {"default": {"default": "forme de présentation de crochet droit vertical"}}
    }, {
      "category": "Ps",
      "key": "FE59",
      "mappings": {"default": {"default": "parenthèse gauche minuscule"}}
    }, {
      "category": "Pe",
      "key": "FE5A",
      "mappings": {"default": {"default": "parenthèse droite minuscule"}}
    }, {
      "category": "Ps",
      "key": "FE5B",
      "mappings": {"default": {"default": "accolade gauche minuscule"}}
    }, {
      "category": "Pe",
      "key": "FE5C",
      "mappings": {"default": {"default": "accolade droite minuscule"}}
    }, {
      "category": "Ps",
      "key": "FE5D",
      "mappings": {"default": {"default": "crochet gauche en écaille minuscule"}}
    }, {
      "category": "Pe",
      "key": "FE5E",
      "mappings": {"default": {"default": "crochet droit en écaille minuscule"}}
    }, {
      "category": "Ps",
      "key": "FF08",
      "mappings": {"default": {"default": "parenthèse gauche pleine chasse"}}
    }, {
      "category": "Pe",
      "key": "FF09",
      "mappings": {"default": {"default": "parenthèse droite pleine chasse"}}
    }, {
      "category": "Ps",
      "key": "FF3B",
      "mappings": {"default": {"default": "crochet gauche pleine chasse"}}
    }, {
      "category": "Pe",
      "key": "FF3D",
      "mappings": {"default": {"default": "crochet droit pleine chasse"}}
    }, {
      "category": "Ps",
      "key": "FF5B",
      "mappings": {"default": {"default": "accolade gauche pleine chasse"}}
    }, {
      "category": "Pe",
      "key": "FF5D",
      "mappings": {"default": {"default": "accolade droite pleine chasse"}}
    }, {
      "category": "Ps",
      "key": "FF5F",
      "mappings": {"default": {"default": "parenthèse gauche ajourée pleine chasse *"}}
    }, {
      "category": "Pe",
      "key": "FF60",
      "mappings": {"default": {"default": "parenthèse droite ajourée pleine chasse *"}}
    }, {
      "category": "Ps",
      "key": "FF62",
      "mappings": {"default": {"default": "crochet gauche en coin demi chasse"}}
    }, {"category": "Pe", "key": "FF63", "mappings": {"default": {"default": "crochet droit en coin demi chasse"}}}]
  ,
  "fr/symbols/math_geometry.js":
    [{"locale": "fr"}, {
      "key": "2500",
      "mappings": {"default": {"default": "filet horizontal fin"}},
      "category": "So"
    }, {"category": "So", "key": "2501", "mappings": {"default": {"default": "filet horizontal gras"}}}, {
      "key": "2502",
      "mappings": {"default": {"default": "filet vertical fin"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2503",
      "mappings": {"default": {"default": "filet vertical gras"}}
    }, {
      "category": "So",
      "key": "2504",
      "mappings": {"default": {"default": "filet brisé triple horizontal fin"}}
    }, {
      "category": "So",
      "key": "2505",
      "mappings": {"default": {"default": "filet brisé triple horizontal gras"}}
    }, {
      "category": "So",
      "key": "2506",
      "mappings": {"default": {"default": "filet brisé triple vertical fin"}}
    }, {
      "category": "So",
      "key": "2507",
      "mappings": {"default": {"default": "filet brisé triple vertical gras"}}
    }, {
      "category": "So",
      "key": "2508",
      "mappings": {"default": {"default": "filet brisé quadruple horizontal fin"}}
    }, {
      "category": "So",
      "key": "2509",
      "mappings": {"default": {"default": "filet brisé quadruple horizontal gras"}}
    }, {
      "category": "So",
      "key": "250A",
      "mappings": {"default": {"default": "filet brisé quadruple vertical fin"}}
    }, {
      "category": "So",
      "key": "250B",
      "mappings": {"default": {"default": "filet brisé quadruple vertical gras"}}
    }, {
      "key": "250C",
      "mappings": {"default": {"default": "filet fin vers le bas et vers la droite"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "250D",
      "mappings": {"default": {"default": "filet fin vers le bas et gras vers la droite"}}
    }, {
      "category": "So",
      "key": "250E",
      "mappings": {"default": {"default": "filet gras vers le bas et fin vers la droite"}}
    }, {
      "category": "So",
      "key": "250F",
      "mappings": {"default": {"default": "filet gras vers le bas et vers la droite"}}
    }, {
      "key": "2510",
      "mappings": {"default": {"default": "filet fin vers le bas et vers la gauche"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2511",
      "mappings": {"default": {"default": "filet fin vers le bas et gras vers la gauche"}}
    }, {
      "category": "So",
      "key": "2512",
      "mappings": {"default": {"default": "filet gras vers le bas et fin vers la gauche"}}
    }, {
      "category": "So",
      "key": "2513",
      "mappings": {"default": {"default": "filet gras vers le bas et vers la gauche"}}
    }, {
      "key": "2514",
      "mappings": {"default": {"default": "filet fin vers le haut et vers la droite"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2515",
      "mappings": {"default": {"default": "filet fin vers le haut et gras vers la droite"}}
    }, {
      "category": "So",
      "key": "2516",
      "mappings": {"default": {"default": "filet gras vers le haut et fin vers la droite"}}
    }, {
      "category": "So",
      "key": "2517",
      "mappings": {"default": {"default": "filet gras vers le haut et vers la droite"}}
    }, {
      "key": "2518",
      "mappings": {"default": {"default": "filet fin vers le haut et vers la gauche"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2519",
      "mappings": {"default": {"default": "filet fin vers le haut et gras vers la gauche"}}
    }, {
      "category": "So",
      "key": "251A",
      "mappings": {"default": {"default": "filet gras vers le haut et fin vers la gauche"}}
    }, {
      "category": "So",
      "key": "251B",
      "mappings": {"default": {"default": "filet gras vers le haut et vers la gauche"}}
    }, {
      "key": "251C",
      "mappings": {"default": {"default": "filet fin vertical et vers la droite"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "251D",
      "mappings": {"default": {"default": "filet fin vertical et gras vers la droite"}}
    }, {
      "category": "So",
      "key": "251E",
      "mappings": {"default": {"default": "filet gras vers le haut et fin vers la droite et vers le bas"}}
    }, {
      "category": "So",
      "key": "251F",
      "mappings": {"default": {"default": "filet gras vers le bas et fin vers la droite et vers le haut"}}
    }, {
      "category": "So",
      "key": "2520",
      "mappings": {"default": {"default": "filet gras vertical et fin vers la droite"}}
    }, {
      "category": "So",
      "key": "2521",
      "mappings": {"default": {"default": "filet fin vers le bas et gras vers la droite et vers le haut"}}
    }, {
      "category": "So",
      "key": "2522",
      "mappings": {"default": {"default": "filet fin vers le haut et gras vers la droite et vers le bas"}}
    }, {
      "category": "So",
      "key": "2523",
      "mappings": {"default": {"default": "filet gras vertical et vers la droite"}}
    }, {
      "key": "2524",
      "mappings": {"default": {"default": "filet fin vertical et vers la gauche"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2525",
      "mappings": {"default": {"default": "filet fin vertical et gras vers la gauche"}}
    }, {
      "category": "So",
      "key": "2526",
      "mappings": {"default": {"default": "filet gras vers le haut et fin vers la gauche et vers le bas"}}
    }, {
      "category": "So",
      "key": "2527",
      "mappings": {"default": {"default": "filet gras vers le bas et fin vers la gauche et vers le haut"}}
    }, {
      "category": "So",
      "key": "2528",
      "mappings": {"default": {"default": "filet gras vertical et fin vers la gauche"}}
    }, {
      "category": "So",
      "key": "2529",
      "mappings": {"default": {"default": "filet fin vers le bas et gras vers la gauche et vers le haut"}}
    }, {
      "category": "So",
      "key": "252A",
      "mappings": {"default": {"default": "filet fin vers le haut et gras vers la gauche et vers le bas"}}
    }, {
      "category": "So",
      "key": "252B",
      "mappings": {"default": {"default": "filet gras vertical et vers la gauche"}}
    }, {
      "key": "252C",
      "mappings": {"default": {"default": "filet fin vers le bas et horizontal"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "252D",
      "mappings": {"default": {"default": "filet gras vers la gauche et fin vers la droite et vers le bas"}}
    }, {
      "category": "So",
      "key": "252E",
      "mappings": {"default": {"default": "filet gras vers la droite et fin vers la gauche et vers le bas"}}
    }, {
      "category": "So",
      "key": "252F",
      "mappings": {"default": {"default": "filet fin vers le bas et gras horizontal"}}
    }, {
      "category": "So",
      "key": "2530",
      "mappings": {"default": {"default": "filet gras vers le bas et fin horizontal"}}
    }, {
      "category": "So",
      "key": "2531",
      "mappings": {"default": {"default": "filet fin vers la droite et gras vers la gauche et vers le bas"}}
    }, {
      "category": "So",
      "key": "2532",
      "mappings": {"default": {"default": "filet fin vers la gauche et gras vers la droite et vers le bas"}}
    }, {
      "category": "So",
      "key": "2533",
      "mappings": {"default": {"default": "filet gras vers le bas et horizontal"}}
    }, {
      "key": "2534",
      "mappings": {"default": {"default": "filet fin vers le haut et horizontal"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2535",
      "mappings": {"default": {"default": "filet gras vers la gauche et fin vers la droite et vers le haut"}}
    }, {
      "category": "So",
      "key": "2536",
      "mappings": {"default": {"default": "filet gras vers la droite et fin vers la gauche et vers le haut"}}
    }, {
      "category": "So",
      "key": "2537",
      "mappings": {"default": {"default": "filet fin vers le haut et gras horizontal"}}
    }, {
      "category": "So",
      "key": "2538",
      "mappings": {"default": {"default": "filet gras vers le haut et fin horizontal"}}
    }, {
      "category": "So",
      "key": "2539",
      "mappings": {"default": {"default": "filet fin vers la droite et gras vers la gauche et vers le haut"}}
    }, {
      "category": "So",
      "key": "253A",
      "mappings": {"default": {"default": "filet fin vers la gauche et gras vers la droite et vers le haut"}}
    }, {
      "category": "So",
      "key": "253B",
      "mappings": {"default": {"default": "filet gras vers le haut et horizontal"}}
    }, {
      "key": "253C",
      "mappings": {"default": {"default": "filet fin vertical et horizontal"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "253D",
      "mappings": {"default": {"default": "filet gras vers la gauche et fin vers la droite et vertical"}}
    }, {
      "category": "So",
      "key": "253E",
      "mappings": {"default": {"default": "filet gras vers la droite et fin vers la gauche et vertical"}}
    }, {
      "category": "So",
      "key": "253F",
      "mappings": {"default": {"default": "filet vertical fin et horizontal gras"}}
    }, {
      "category": "So",
      "key": "2540",
      "mappings": {"default": {"default": "filet gras vers le haut et fin vers le bas et horizontal"}}
    }, {
      "category": "So",
      "key": "2541",
      "mappings": {"default": {"default": "filet gras vers le bas et fin vers le haut et horizontal"}}
    }, {
      "category": "So",
      "key": "2542",
      "mappings": {"default": {"default": "filet vertical gras et horizontal fin"}}
    }, {
      "category": "So",
      "key": "2543",
      "mappings": {"default": {"default": "filet gras vers la gauche et vers le haut et fin vers la droite et vers le bas"}}
    }, {
      "category": "So",
      "key": "2544",
      "mappings": {"default": {"default": "filet gras vers la droite et vers le haut et fin vers la gauche et vers le bas"}}
    }, {
      "category": "So",
      "key": "2545",
      "mappings": {"default": {"default": "filet gras vers la gauche et vers le bas et fin vers la droite et vers le haut"}}
    }, {
      "category": "So",
      "key": "2546",
      "mappings": {"default": {"default": "filet gras vers la droite et vers le bas et fin vers la gauche et vers le haut"}}
    }, {
      "category": "So",
      "key": "2547",
      "mappings": {"default": {"default": "filet fin vers le bas et gras vers le haut et horizontal"}}
    }, {
      "category": "So",
      "key": "2548",
      "mappings": {"default": {"default": "filet fin vers le haut et gras vers le bas et horizontal"}}
    }, {
      "category": "So",
      "key": "2549",
      "mappings": {"default": {"default": "filet fin vers la droite et gras vers la gauche et vertical"}}
    }, {
      "category": "So",
      "key": "254A",
      "mappings": {"default": {"default": "filet fin vers la gauche et gras vers la droite et vertical"}}
    }, {
      "category": "So",
      "key": "254B",
      "mappings": {"default": {"default": "filet vertical et horizontal gras"}}
    }, {
      "category": "So",
      "key": "254C",
      "mappings": {"default": {"default": "filet brisé double horizontal fin"}}
    }, {
      "category": "So",
      "key": "254D",
      "mappings": {"default": {"default": "filet brisé double horizontal gras"}}
    }, {
      "category": "So",
      "key": "254E",
      "mappings": {"default": {"default": "filet brisé double vertical fin"}}
    }, {
      "category": "So",
      "key": "254F",
      "mappings": {"default": {"default": "filet brisé double vertical gras"}}
    }, {
      "key": "2550",
      "mappings": {"default": {"default": "filet double horizontal"}},
      "category": "So"
    }, {"key": "2551", "mappings": {"default": {"default": "filet double vertical"}}, "category": "So"}, {
      "key": "2552",
      "mappings": {"default": {"default": "filet simple le bas et double vers la droite"}},
      "category": "So"
    }, {
      "key": "2553",
      "mappings": {"default": {"default": "filet double vers le bas et simple vers la droite"}},
      "category": "So"
    }, {
      "key": "2554",
      "mappings": {"default": {"default": "filet double vers le bas et vers la droite"}},
      "category": "So"
    }, {
      "key": "2555",
      "mappings": {"default": {"default": "filet simple vers le bas et double vers la gauche"}},
      "category": "So"
    }, {
      "key": "2556",
      "mappings": {"default": {"default": "filet double vers le bas et simple vers la gauche"}},
      "category": "So"
    }, {
      "key": "2557",
      "mappings": {"default": {"default": "filet double vers le bas et vers la gauche"}},
      "category": "So"
    }, {
      "key": "2558",
      "mappings": {"default": {"default": "filet simple vers le haut et double vers la droite"}},
      "category": "So"
    }, {
      "key": "2559",
      "mappings": {"default": {"default": "filet double vers le haut et simple vers la droite"}},
      "category": "So"
    }, {
      "key": "255A",
      "mappings": {"default": {"default": "filet double vers le haut et vers la droite"}},
      "category": "So"
    }, {
      "key": "255B",
      "mappings": {"default": {"default": "filet simple vers le haut et double vers la gauche"}},
      "category": "So"
    }, {
      "key": "255C",
      "mappings": {"default": {"default": "filet double vers le haut et simple vers la gauche"}},
      "category": "So"
    }, {
      "key": "255D",
      "mappings": {"default": {"default": "filet double vers le haut et vers la gauche"}},
      "category": "So"
    }, {
      "key": "255E",
      "mappings": {"default": {"default": "filet vertical simple et droit double"}},
      "category": "So"
    }, {
      "key": "255F",
      "mappings": {"default": {"default": "filet vertical double et droit simple"}},
      "category": "So"
    }, {
      "key": "2560",
      "mappings": {"default": {"default": "filet double vertical et vers la droite"}},
      "category": "So"
    }, {
      "key": "2561",
      "mappings": {"default": {"default": "filet vertical simple et gauche double"}},
      "category": "So"
    }, {
      "key": "2562",
      "mappings": {"default": {"default": "filet vertical double et gauche simple"}},
      "category": "So"
    }, {
      "key": "2563",
      "mappings": {"default": {"default": "filet double vertical et vers la gauche"}},
      "category": "So"
    }, {
      "key": "2564",
      "mappings": {"default": {"default": "filet vers le bas simple et horizontal double"}},
      "category": "So"
    }, {
      "key": "2565",
      "mappings": {"default": {"default": "filet vers le bas double et horizontal simple"}},
      "category": "So"
    }, {
      "key": "2566",
      "mappings": {"default": {"default": "filet double vers le bas et horizontal"}},
      "category": "So"
    }, {
      "key": "2567",
      "mappings": {"default": {"default": "filet vers le haut simple et horizontal double"}},
      "category": "So"
    }, {
      "key": "2568",
      "mappings": {"default": {"default": "filet vers le haut double et horizontal simple"}},
      "category": "So"
    }, {
      "key": "2569",
      "mappings": {"default": {"default": "filet double vers le haut et horizontal"}},
      "category": "So"
    }, {
      "key": "256A",
      "mappings": {"default": {"default": "filet vertical simple et horizontal double"}},
      "category": "So"
    }, {
      "key": "256B",
      "mappings": {"default": {"default": "filet vertical double et horizontal simple"}},
      "category": "So"
    }, {
      "key": "256C",
      "mappings": {"default": {"default": "filet double vertical et horizontall"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "256D",
      "mappings": {"default": {"default": "filet fin arc vers le bas et vers la droite"}}
    }, {
      "category": "So",
      "key": "256E",
      "mappings": {"default": {"default": "filet fin arc vers le bas et vers la gauche"}}
    }, {
      "category": "So",
      "key": "256F",
      "mappings": {"default": {"default": "filet fin arc vers le haut et vers la gauche"}}
    }, {
      "category": "So",
      "key": "2570",
      "mappings": {"default": {"default": "filet fin arc vers le haut et vers la droite"}}
    }, {
      "category": "So",
      "key": "2571",
      "mappings": {"default": {"default": "filet diagonal fin du coin supérieur droit au coin inférieur gauche"}}
    }, {
      "category": "So",
      "key": "2572",
      "mappings": {"default": {"default": "filet diagonal fin du coin supérieur gauche vers le coin inférieur droit"}}
    }, {
      "category": "So",
      "key": "2573",
      "mappings": {"default": {"default": "filet diagonal fin en croix"}}
    }, {
      "category": "So",
      "key": "2574",
      "mappings": {"default": {"default": "filet fin vers la gauche"}}
    }, {
      "category": "So",
      "key": "2575",
      "mappings": {"default": {"default": "filet fin vers le haut"}}
    }, {
      "category": "So",
      "key": "2576",
      "mappings": {"default": {"default": "filet fin vers la droite"}}
    }, {
      "category": "So",
      "key": "2577",
      "mappings": {"default": {"default": "filet fin vers le bas"}}
    }, {
      "category": "So",
      "key": "2578",
      "mappings": {"default": {"default": "filet gras vers la gauche"}}
    }, {
      "category": "So",
      "key": "2579",
      "mappings": {"default": {"default": "filet gras vers le haut"}}
    }, {
      "category": "So",
      "key": "257A",
      "mappings": {"default": {"default": "filet gras vers la droite"}}
    }, {
      "category": "So",
      "key": "257B",
      "mappings": {"default": {"default": "filet gras vers le bas"}}
    }, {
      "category": "So",
      "key": "257C",
      "mappings": {"default": {"default": "filet fin vers la gauche et gras vers la droite"}}
    }, {
      "category": "So",
      "key": "257D",
      "mappings": {"default": {"default": "filet fin vers le haut et gras vers le bas"}}
    }, {
      "category": "So",
      "key": "257E",
      "mappings": {"default": {"default": "filet gras vers la gauche et fin vers la droite"}}
    }, {
      "category": "So",
      "key": "257F",
      "mappings": {"default": {"default": "filet gras vers le haut et fin vers le bas"}}
    }, {
      "key": "2580",
      "mappings": {"default": {"default": "moitié supérieure de pavé"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2581",
      "mappings": {"default": {"default": "huitième inférieur de pavé"}}
    }, {
      "category": "So",
      "key": "2582",
      "mappings": {"default": {"default": "quart inférieur de pavé"}}
    }, {
      "category": "So",
      "key": "2583",
      "mappings": {"default": {"default": "trois huitièmes inférieurs de pavé"}}
    }, {
      "key": "2584",
      "mappings": {"default": {"default": "moitié inférieure de pavé"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2585",
      "mappings": {"default": {"default": "cinq huitièmes inférieurs de pavé"}}
    }, {
      "category": "So",
      "key": "2586",
      "mappings": {"default": {"default": "trois quarts inférieurs de pavé"}}
    }, {
      "category": "So",
      "key": "2587",
      "mappings": {"default": {"default": "sept huitièmes inférieurs de pavé"}}
    }, {"key": "2588", "mappings": {"default": {"default": "pavé plein"}}, "category": "So"}, {
      "category": "So",
      "key": "2589",
      "mappings": {"default": {"default": "sept huitièmes gauches de pavé"}}
    }, {
      "category": "So",
      "key": "258A",
      "mappings": {"default": {"default": "trois quarts gauches de pavé"}}
    }, {
      "category": "So",
      "key": "258B",
      "mappings": {"default": {"default": "cinq huitièmes gauches de pavé"}}
    }, {
      "category": "So",
      "key": "258C",
      "mappings": {"default": {"default": "moitié gauche de pavé"}}
    }, {
      "category": "So",
      "key": "258D",
      "mappings": {"default": {"default": "trois huitièmes gauches de pavé"}}
    }, {
      "category": "So",
      "key": "258E",
      "mappings": {"default": {"default": "un quart gauche de pavé"}}
    }, {
      "category": "So",
      "key": "258F",
      "mappings": {"default": {"default": "un huitième gauche de pavé"}}
    }, {"category": "So", "key": "2590", "mappings": {"default": {"default": "moitié droite de pavé"}}}, {
      "key": "2591",
      "mappings": {"default": {"default": "ombre légère"}},
      "category": "So"
    }, {"key": "2592", "mappings": {"default": {"default": "ombre moyenne"}}, "category": "So"}, {
      "key": "2593",
      "mappings": {"default": {"default": "ombre foncée"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2594",
      "mappings": {"default": {"default": "huitième supérieur de pavé"}}
    }, {
      "category": "So",
      "key": "2595",
      "mappings": {"default": {"default": "huitième droit de pavé"}}
    }, {
      "category": "So",
      "key": "2596",
      "mappings": {"default": {"default": "quadrant inférieur gauche"}}
    }, {
      "category": "So",
      "key": "2597",
      "mappings": {"default": {"default": "quadrant inférieur droit"}}
    }, {
      "category": "So",
      "key": "2598",
      "mappings": {"default": {"default": "quadrant supérieur gauche"}}
    }, {
      "category": "So",
      "key": "2599",
      "mappings": {"default": {"default": "quadrant supérieur gauche inférieur gauche et inférieur droit"}}
    }, {
      "category": "So",
      "key": "259A",
      "mappings": {"default": {"default": "quadrant supérieur gauche et inférieur droit"}}
    }, {
      "category": "So",
      "key": "259B",
      "mappings": {"default": {"default": "quadrant supérieur gauche supérieur droit et inférieur gauche"}}
    }, {
      "category": "So",
      "key": "259C",
      "mappings": {"default": {"default": "quadrant supérieur gauche supérieur droit et inférieur droit"}}
    }, {
      "category": "So",
      "key": "259D",
      "mappings": {"default": {"default": "quadrant supérieur droit"}}
    }, {
      "category": "So",
      "key": "259E",
      "mappings": {"default": {"default": "quadrant supérieur droit et inférieur gauche"}}
    }, {
      "category": "So",
      "key": "259F",
      "mappings": {"default": {"default": "quadrant supérieur droit inférieur gauche et inférieur droit"}}
    }, {"key": "25A0", "mappings": {"default": {"default": "petit carré noir"}}, "category": "So"}, {
      "key": "25A1",
      "mappings": {"default": {"default": "carré blanc"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "25A2",
      "mappings": {"default": {"default": "carré blanc aux coins arrondis"}}
    }, {
      "category": "So",
      "key": "25A3",
      "mappings": {"default": {"default": "petit carré noir inscrit dans un carré blanc"}}
    }, {
      "category": "So",
      "key": "25A4",
      "mappings": {"default": {"default": "carré hachuré horizontalement"}}
    }, {
      "category": "So",
      "key": "25A5",
      "mappings": {"default": {"default": "carré hachuré verticalement"}}
    }, {"category": "So", "key": "25A6", "mappings": {"default": {"default": "carré quadrillé"}}}, {
      "category": "So",
      "key": "25A7",
      "mappings": {"default": {"default": "carré hachuré du haut à gauche vers le bas à droite"}}
    }, {
      "category": "So",
      "key": "25A8",
      "mappings": {"default": {"default": "carré hachuré du haut à droite vers le bas à gauche"}}
    }, {
      "category": "So",
      "key": "25A9",
      "mappings": {"default": {"default": "carré quadrillé en diagonale"}}
    }, {"key": "25AA", "mappings": {"default": {"default": "petit carré noir"}}, "category": "So"}, {
      "key": "25AB",
      "mappings": {"default": {"default": "petit carré blanc"}},
      "category": "So"
    }, {"category": "So", "key": "25AC", "mappings": {"default": {"default": "rectangle noir"}}}, {
      "key": "25AD",
      "mappings": {"default": {"default": "rectangle blanc"}},
      "category": "So"
    }, {
      "key": "25AE",
      "mappings": {"default": {"default": "rectangle vertical noir"}},
      "category": "So"
    }, {
      "key": "25AF",
      "mappings": {"default": {"default": "white vertical rectangle"}},
      "category": "So"
    }, {"category": "So", "key": "25B0", "mappings": {"default": {"default": "parallélogramme noir"}}}, {
      "key": "25B1",
      "mappings": {"default": {"default": "parallélogramme blanc"}},
      "category": "So"
    }, {
      "key": "25B2",
      "mappings": {"default": {"default": "petit triangle noir pointant vers le haut"}},
      "category": "So"
    }, {
      "key": "25B3",
      "mappings": {"default": {"default": "triangle blanc pointant vers le haut"}},
      "category": "So"
    }, {
      "key": "25B4",
      "mappings": {"default": {"default": "petit triangle noir pointant vers le haut"}},
      "category": "So"
    }, {
      "key": "25B5",
      "mappings": {"default": {"default": "petit triangle blanc pointant vers le haut"}},
      "category": "So"
    }, {
      "key": "25B6",
      "mappings": {"default": {"default": "petit triangle noir pointant vers la droite"}},
      "category": "So"
    }, {
      "key": "25B7",
      "mappings": {"default": {"default": "triangle blanc pointant vers la droite"}},
      "category": "Sm"
    }, {
      "key": "25B8",
      "mappings": {"default": {"default": "petit triangle noir pointant vers la droite"}},
      "category": "So"
    }, {
      "key": "25B9",
      "mappings": {"default": {"default": "petit triangle blanc pointant vers la droite"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "25BA",
      "mappings": {"default": {"default": "pointeur noir vers la droite"}}
    }, {
      "category": "So",
      "key": "25BB",
      "mappings": {"default": {"default": "pointeur blanc vers la droite"}}
    }, {
      "key": "25BC",
      "mappings": {"default": {"default": "petit triangle noir pointant vers le bas"}},
      "category": "So"
    }, {
      "key": "25BD",
      "mappings": {"default": {"default": "triangle blanc pointant vers le bas"}},
      "category": "So"
    }, {
      "key": "25BE",
      "mappings": {"default": {"default": "petit triangle noir pointant vers le bas"}},
      "category": "So"
    }, {
      "key": "25BF",
      "mappings": {"default": {"default": "petit triangle blanc pointant vers le bas"}},
      "category": "So"
    }, {
      "key": "25C0",
      "mappings": {"default": {"default": "petit triangle noir pointant vers la gauche"}},
      "category": "So"
    }, {
      "key": "25C1",
      "mappings": {"default": {"default": "petit triangle blanc pointant vers la gauche"}},
      "category": "Sm"
    }, {
      "key": "25C2",
      "mappings": {"default": {"default": "petit triangle noir pointant vers la gauche"}},
      "category": "So"
    }, {
      "key": "25C3",
      "mappings": {"default": {"default": "petit triangle blanc pointant vers la gauche"}},
      "category": "So"
    }, {
      "key": "25C4",
      "mappings": {"default": {"default": "pointeur noir vers la gauche"}},
      "category": "So"
    }, {
      "key": "25C5",
      "mappings": {"default": {"default": "pointeur blanc vers la gauche"}},
      "category": "So"
    }, {"key": "25C6", "mappings": {"default": {"default": "losange noir"}}, "category": "So"}, {
      "key": "25C7",
      "mappings": {"default": {"default": "losange blanc"}},
      "category": "So"
    }, {"key": "25C8", "mappings": {"default": {"default": "losange blanc évidé"}}, "category": "So"}, {
      "key": "25C9",
      "mappings": {"default": {"default": "cercle blanc évidé"}},
      "category": "So"
    }, {"key": "25CA", "mappings": {"default": {"default": "losange"}}, "category": "So"}, {
      "key": "25CB",
      "mappings": {"default": {"default": "cercle blanc"}},
      "category": "So"
    }, {"key": "25CC", "mappings": {"default": {"default": "cercle en pointillés"}}, "category": "So"}, {
      "key": "25CD",
      "mappings": {"default": {"default": "cercle hachuré verticalement"}},
      "category": "So"
    }, {
      "key": "25CE",
      "mappings": {"default": {"default": "deux cercles concentriques"}},
      "category": "So"
    }, {"key": "25CF", "mappings": {"default": {"default": "cercle noir"}}, "category": "So"}, {
      "key": "25D0",
      "mappings": {"default": {"default": "cercle avec moitié gauche noire"}},
      "category": "So"
    }, {
      "key": "25D1",
      "mappings": {"default": {"default": "cercle avec moitié droite noire"}},
      "category": "So"
    }, {
      "key": "25D2",
      "mappings": {"default": {"default": "cercle avec moitié inférieure"}},
      "category": "So"
    }, {
      "key": "25D3",
      "mappings": {"default": {"default": "cercle avec moitié supérieure"}},
      "category": "So"
    }, {
      "key": "25D4",
      "mappings": {"default": {"default": "cercle avec quadrant supérieur droit noir"}},
      "category": "So"
    }, {
      "key": "25D5",
      "mappings": {"default": {"default": "disque avec quadrant supérieur gauche blanc"}},
      "category": "So"
    }, {
      "key": "25D6",
      "mappings": {"default": {"default": "demi disque gauche noir"}},
      "category": "So"
    }, {
      "key": "25D7",
      "mappings": {"default": {"default": "demi disque droite noir"}},
      "category": "So"
    }, {"key": "25D8", "mappings": {"default": {"default": "point noir au blanc"}}, "category": "So"}, {
      "key": "25D9",
      "mappings": {"default": {"default": "cercle noir au blanc"}},
      "category": "So"
    }, {
      "key": "25DA",
      "mappings": {"default": {"default": "demi cercle supérieur noir au  blanc"}},
      "category": "So"
    }, {
      "key": "25DB",
      "mappings": {"default": {"default": "demi cercle inférieur noir au  blanc"}},
      "category": "So"
    }, {
      "key": "25DC",
      "mappings": {"default": {"default": "arc de cercle dans le quadrant supérieur gauchearc de cercle dans le quadrant supérieur gauche"}},
      "category": "So"
    }, {
      "key": "25DD",
      "mappings": {"default": {"default": "arc de cercle dans le quadrant supérieur droit"}},
      "category": "So"
    }, {
      "key": "25DE",
      "mappings": {"default": {"default": "arc de cercle dans le quadrant inférieur droit"}},
      "category": "So"
    }, {
      "key": "25DF",
      "mappings": {"default": {"default": "arc de cercle dans le quadrant inférieur gauche"}},
      "category": "So"
    }, {"key": "25E0", "mappings": {"default": {"default": "demi cercle supérieur"}}, "category": "So"}, {
      "key": "25E1",
      "mappings": {"default": {"default": "demi cercle inférieur"}},
      "category": "So"
    }, {
      "key": "25E2",
      "mappings": {"default": {"default": "coin triangulaire noir inférieur droit"}},
      "category": "So"
    }, {
      "key": "25E3",
      "mappings": {"default": {"default": "coin triangulaire noir inférieur gauche"}},
      "category": "So"
    }, {
      "key": "25E4",
      "mappings": {"default": {"default": "coin triangulaire noir supérieur gauche"}},
      "category": "So"
    }, {
      "key": "25E5",
      "mappings": {"default": {"default": "coin triangulaire noir supérieur droit"}},
      "category": "So"
    }, {"key": "25E6", "mappings": {"default": {"default": "opérateur rond"}}, "category": "So"}, {
      "key": "25E7",
      "mappings": {"default": {"default": "carré à moitié gauche noire"}},
      "category": "So"
    }, {
      "key": "25E8",
      "mappings": {"default": {"default": "carré à moitié droite noire"}},
      "category": "So"
    }, {
      "key": "25E9",
      "mappings": {"default": {"default": "carré à moitié supérieure gauche noire"}},
      "category": "So"
    }, {
      "key": "25EA",
      "mappings": {"default": {"default": "carré à moitié inférieure droite noire"}},
      "category": "So"
    }, {
      "key": "25EB",
      "mappings": {"default": {"default": "carré blanc à ligne bissectrice verticale"}},
      "category": "So"
    }, {
      "key": "25EC",
      "mappings": {"default": {"default": "triangle blanc pointant vers le haut pointé"}},
      "category": "So"
    }, {
      "key": "25ED",
      "mappings": {"default": {"default": "triangle pointe vers le haut À moitié gauche noire"}},
      "category": "So"
    }, {
      "key": "25EE",
      "mappings": {"default": {"default": "triangle pointe vers le haut À moitié droite noire"}},
      "category": "So"
    }, {"key": "25EF", "mappings": {"default": {"default": "grand cercle"}}, "category": "So"}, {
      "category": "So",
      "key": "25F0",
      "mappings": {"default": {"default": "carré blanc à quartier supérieur gauche"}}
    }, {
      "category": "So",
      "key": "25F1",
      "mappings": {"default": {"default": "carré blanc à quartier inférieur gauche"}}
    }, {
      "category": "So",
      "key": "25F2",
      "mappings": {"default": {"default": "carré blanc à quartier inférieur droit"}}
    }, {
      "category": "So",
      "key": "25F3",
      "mappings": {"default": {"default": "carré blanc à quartier supérieur droit"}}
    }, {
      "category": "So",
      "key": "25F4",
      "mappings": {"default": {"default": "cercle blanc à quartier supérieur gauche"}}
    }, {
      "category": "So",
      "key": "25F5",
      "mappings": {"default": {"default": "cercle blanc à quartier inférieur gauche"}}
    }, {
      "category": "So",
      "key": "25F6",
      "mappings": {"default": {"default": "cercle blanc à quartier inférieur droit"}}
    }, {
      "category": "So",
      "key": "25F7",
      "mappings": {"default": {"default": "cercle blanc à quartier supérieur droit"}}
    }, {
      "key": "25F8",
      "mappings": {"default": {"default": "triangle rectangle supérieur gauche"}},
      "category": "Sm"
    }, {
      "key": "25F9",
      "mappings": {"default": {"default": "triangle rectangle supérieur droit"}},
      "category": "Sm"
    }, {
      "key": "25FA",
      "mappings": {"default": {"default": "triangle rectangle inférieur gauche"}},
      "category": "Sm"
    }, {"key": "25FB", "mappings": {"default": {"default": "carré moyen blanc"}}, "category": "Sm"}, {
      "key": "25FC",
      "mappings": {"default": {"default": "carré moyen noir"}},
      "category": "Sm"
    }, {"key": "25FD", "mappings": {"default": {"default": "carré moyen blanc"}}, "category": "Sm"}, {
      "key": "25FE",
      "mappings": {"default": {"default": "carré moyen noir"}},
      "category": "Sm"
    }, {
      "key": "25FF",
      "mappings": {"default": {"default": "triangle rectangle inférieur droit"}},
      "category": "Sm"
    }, {
      "category": "So",
      "key": "2B12",
      "mappings": {"default": {"default": "carré à moitié supérieure noire"}}
    }, {
      "category": "So",
      "key": "2B13",
      "mappings": {"default": {"default": "carré à moitié inférieure noire"}}
    }, {
      "category": "So",
      "key": "2B14",
      "mappings": {"default": {"default": "carré à moitié diagonale supérieure droite noire"}}
    }, {
      "category": "So",
      "key": "2B15",
      "mappings": {"default": {"default": "carré à moitié diagonale inférieure droite noire"}}
    }, {
      "category": "So",
      "key": "2B16",
      "mappings": {"default": {"default": "losange à moitié gauche noire"}}
    }, {
      "category": "So",
      "key": "2B17",
      "mappings": {"default": {"default": "losange à moitié droite noire"}}
    }, {
      "category": "So",
      "key": "2B18",
      "mappings": {"default": {"default": "losange à moitié supérieure noire"}}
    }, {
      "category": "So",
      "key": "2B19",
      "mappings": {"default": {"default": "losange à moitié inférieure noire"}}
    }, {
      "category": "So",
      "key": "2B1A",
      "mappings": {"default": {"default": "carré en pointillés"}}
    }, {"category": "So", "key": "2B1B", "mappings": {"default": {"default": "grand carré noir"}}}, {
      "category": "So",
      "key": "2B1C",
      "mappings": {"default": {"default": "grand carré blanc"}}
    }, {
      "category": "So",
      "key": "2B1D",
      "mappings": {"default": {"default": "très petit carré noir"}}
    }, {
      "category": "So",
      "key": "2B1E",
      "mappings": {"default": {"default": "très petit carré blanc"}}
    }, {"category": "So", "key": "2B1F", "mappings": {"default": {"default": "pentagone noir"}}}, {
      "category": "So",
      "key": "2B20",
      "mappings": {"default": {"default": "pentagone blanc"}}
    }, {"category": "So", "key": "2B21", "mappings": {"default": {"default": "hexagone blanc"}}}, {
      "category": "So",
      "key": "2B22",
      "mappings": {"default": {"default": "hexagone noir"}}
    }, {
      "category": "So",
      "key": "2B23",
      "mappings": {"default": {"default": "hexagone noir posé sur un côté"}}
    }, {"category": "So", "key": "2B24", "mappings": {"default": {"default": "grand cercle noir"}}}, {
      "category": "So",
      "key": "2B25",
      "mappings": {"default": {"default": "losange moyen noir"}}
    }, {
      "category": "So",
      "key": "2B26",
      "mappings": {"default": {"default": "losange moyen blanc"}}
    }, {"category": "So", "key": "2B27", "mappings": {"default": {"default": "rhombe moyen noir"}}}, {
      "category": "So",
      "key": "2B28",
      "mappings": {"default": {"default": "rhombe moyen blanc"}}
    }, {"category": "So", "key": "2B29", "mappings": {"default": {"default": "petit losange noir"}}}, {
      "category": "So",
      "key": "2B2A",
      "mappings": {"default": {"default": "petit rhombe noir"}}
    }, {"category": "So", "key": "2B2B", "mappings": {"default": {"default": "petit rhombe blanc"}}}, {
      "category": "So",
      "key": "2B2C",
      "mappings": {"default": {"default": "ellipse horizontale noire"}}
    }, {
      "category": "So",
      "key": "2B2D",
      "mappings": {"default": {"default": "ellipse horizontale blanche"}}
    }, {
      "category": "So",
      "key": "2B2E",
      "mappings": {"default": {"default": "ellipse verticale noire"}}
    }, {
      "category": "So",
      "key": "2B2F",
      "mappings": {"default": {"default": "ellipse verticale blanche"}}
    }, {
      "key": "2B50",
      "mappings": {"default": {"default": "étoile blanche moyenne"}},
      "category": "So"
    }, {"key": "2B51", "mappings": {"default": {"default": "petite étoile noire"}}, "category": "So"}, {
      "key": "2B52",
      "mappings": {"default": {"default": "petite étoile blanche"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2B53",
      "mappings": {"default": {"default": "pentagone noir pointant vers la droite"}}
    }, {
      "category": "So",
      "key": "2B54",
      "mappings": {"default": {"default": "pentagone blanc pointant vers la droite"}}
    }, {"category": "So", "key": "2B55", "mappings": {"default": {"default": "grand cercle gras"}}}, {
      "category": "So",
      "key": "2B56",
      "mappings": {"default": {"default": "ovale gras avec ovale inscrit"}}
    }, {
      "category": "So",
      "key": "2B57",
      "mappings": {"default": {"default": "cercle gras avec cercle inscrit"}}
    }, {"category": "So", "key": "2B58", "mappings": {"default": {"default": "cercle gras"}}}, {
      "category": "So",
      "key": "2B59",
      "mappings": {"default": {"default": "sautoir cerclé gras"}}
    }]
  ,
  "fr/symbols/math_harpoons.js":
    [{"locale": "fr"}, {
      "key": "21BC",
      "mappings": {"default": {"default": "harpon gauche avec dent dressée"}},
      "category": "So"
    }, {
      "key": "21BD",
      "mappings": {"default": {"default": "harpon gauche avec dent baissée"}},
      "category": "So"
    }, {
      "key": "21BE",
      "mappings": {"default": {"default": "harpon vers le haut avec dent droite"}},
      "category": "So"
    }, {
      "key": "21BF",
      "mappings": {"default": {"default": "harpon vers le haut avec dent gauche"}},
      "category": "So"
    }, {
      "key": "21C0",
      "mappings": {"default": {"default": "harpon droit avec dent dressée"}},
      "category": "So"
    }, {
      "key": "21C1",
      "mappings": {"default": {"default": "harpon droit avec dent baissée"}},
      "category": "So"
    }, {
      "key": "21C2",
      "mappings": {"default": {"default": "harpon vers le bas avec dent droite"}},
      "category": "So"
    }, {
      "key": "21C3",
      "mappings": {"default": {"default": "harpon vers le bas avec dent gauche"}},
      "category": "So"
    }, {
      "key": "21CB",
      "mappings": {"default": {"default": "harpon gauche par dessus harpon droit"}},
      "category": "So"
    }, {
      "key": "21CC",
      "mappings": {"default": {"default": "harpon droit par dessus harpon gauche"}},
      "category": "So"
    }, {
      "key": "294A",
      "mappings": {"default": {"default": "javelot à dents gauche dressée et droite baissée"}},
      "category": "Sm"
    }, {
      "key": "294B",
      "mappings": {"default": {"default": "javelot à dents gauche baissée et droite dressée"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "294C",
      "mappings": {"default": {"default": "crampon à dents supérieure droite et inférieure gauche"}}
    }, {
      "category": "Sm",
      "key": "294D",
      "mappings": {"default": {"default": "crampon à dents supérieure gauche et inférieure droite"}}
    }, {
      "key": "294E",
      "mappings": {"default": {"default": "javelot à dents droite et gauche dressées"}},
      "category": "Sm"
    }, {
      "key": "294F",
      "mappings": {"default": {"default": "javelot à dents supérieure et inférieure droites"}},
      "category": "Sm"
    }, {
      "key": "2950",
      "mappings": {"default": {"default": "javelot à dents droite et gauche baissées"}},
      "category": "Sm"
    }, {
      "key": "2951",
      "mappings": {"default": {"default": "javelot à dents supérieure et inférieure gauches"}},
      "category": "Sm"
    }, {
      "key": "2952",
      "mappings": {"default": {"default": "harpon gauche à dent dressée contre taquet"}},
      "category": "Sm"
    }, {
      "key": "2953",
      "mappings": {"default": {"default": "harpon droit à dent dressée contre taquet"}},
      "category": "Sm"
    }, {
      "key": "2954",
      "mappings": {"default": {"default": "harpon vers le haut à dent droite contre taquet"}},
      "category": "Sm"
    }, {
      "key": "2955",
      "mappings": {"default": {"default": "harpon vers le bas à dent droite contre taquet"}},
      "category": "Sm"
    }, {
      "key": "2956",
      "mappings": {"default": {"default": "harpon gauche à dent baissée contre taquet"}},
      "category": "Sm"
    }, {
      "key": "2957",
      "mappings": {"default": {"default": "harpon droit à dent baissée contre taquet"}},
      "category": "Sm"
    }, {
      "key": "2958",
      "mappings": {"default": {"default": "harpon vers le haut à dent gauche contre taquet"}},
      "category": "Sm"
    }, {
      "key": "2959",
      "mappings": {"default": {"default": "harpon vers le bas à dent gauche contre taquet"}},
      "category": "Sm"
    }, {
      "key": "295A",
      "mappings": {"default": {"default": "harpon gauche à dent dressée depuis taquet"}},
      "category": "Sm"
    }, {
      "key": "295B",
      "mappings": {"default": {"default": "harpon droit à dent dressée depuis taquet"}},
      "category": "Sm"
    }, {
      "key": "295C",
      "mappings": {"default": {"default": "harpon vers le haut à dent droite depuis taquet"}},
      "category": "Sm"
    }, {
      "key": "295D",
      "mappings": {"default": {"default": "harpon vers le bas à dent droite depuis taquet"}},
      "category": "Sm"
    }, {
      "key": "295E",
      "mappings": {"default": {"default": "harpon gauche à dent baissée depuis taquet"}},
      "category": "Sm"
    }, {
      "key": "295F",
      "mappings": {"default": {"default": "harpon droit à dent baissée depuis taquet"}},
      "category": "Sm"
    }, {
      "key": "2960",
      "mappings": {"default": {"default": "harpon vers le haut à dent gauche depuis taquet"}},
      "category": "Sm"
    }, {
      "key": "2961",
      "mappings": {"default": {"default": "harpon vers le bas à dent gauche depuis taquet"}},
      "category": "Sm"
    }, {
      "key": "2962",
      "mappings": {"default": {"default": "harpon gauche à dent dressée sur harpon gauche à dent baissée"}},
      "category": "Sm"
    }, {
      "key": "2963",
      "mappings": {"default": {"default": "harpon vers le haut à dent gauche à côté de harpon vers le haut à dent droite"}},
      "category": "Sm"
    }, {
      "key": "2964",
      "mappings": {"default": {"default": "harpon droit à dent dressée par dessus harpon droit à dent baissée"}},
      "category": "Sm"
    }, {
      "key": "2965",
      "mappings": {"default": {"default": "harpon vers le bas à dent gauche à côté de harpon vers le bas à dent droite"}},
      "category": "Sm"
    }, {
      "key": "2966",
      "mappings": {"default": {"default": "harpon à dent gauche dressée par dessus harpon à dent droite dressée"}},
      "category": "Sm"
    }, {
      "key": "2967",
      "mappings": {"default": {"default": "harpon à dent gauche baissée par dessus harpon à dent droite baissée"}},
      "category": "Sm"
    }, {
      "key": "2968",
      "mappings": {"default": {"default": "harpon à dent droite dressée par dessus harpon à dent gauche dressée"}},
      "category": "Sm"
    }, {
      "key": "2969",
      "mappings": {"default": {"default": "harpon à dent droite baissée par dessus harpon à dent gauche baissée"}},
      "category": "Sm"
    }, {
      "key": "296A",
      "mappings": {"default": {"default": "harpon à dent gauche dressée par dessus long trait"}},
      "category": "Sm"
    }, {
      "key": "296B",
      "mappings": {"default": {"default": "long trait par dessus harpon à dent gauche baissée"}},
      "category": "Sm"
    }, {
      "key": "296C",
      "mappings": {"default": {"default": "harpon à dent droite dressée par dessus long trait"}},
      "category": "Sm"
    }, {
      "key": "296D",
      "mappings": {"default": {"default": "long trait par dessus harpon à dent droite baissée"}},
      "category": "Sm"
    }, {
      "key": "296E",
      "mappings": {"default": {"default": "harpon vers le haut à dent gauche à côté de harpon vers le bas à dent droite"}},
      "category": "Sm"
    }, {
      "key": "296F",
      "mappings": {"default": {"default": "harpon vers le bas à dent gauche à côté de harpon vers le haut à dent droite"}},
      "category": "Sm"
    }, {"key": "297C", "mappings": {"default": {"default": "ancre gauche"}}, "category": "Sm"}, {
      "key": "297D",
      "mappings": {"default": {"default": "ancre droite"}},
      "category": "Sm"
    }, {"key": "297E", "mappings": {"default": {"default": "ancre vers le haut"}}, "category": "Sm"}, {
      "key": "297F",
      "mappings": {"default": {"default": "ancre vers le bas"}},
      "category": "Sm"
    }]
  ,
  "fr/symbols/math_non_characters.js":
    [{"locale": "fr"}, {
      "key": "210F",
      "mappings": {"default": {"default": "constante de planck sur deux pi"}},
      "category": "Ll"
    }, {"category": "So", "key": "2114", "mappings": {"default": {"default": "symbole l b barré"}}}, {
      "key": "2116",
      "mappings": {"default": {"default": "symbole numéro"}},
      "category": "So"
    }, {
      "key": "2117",
      "mappings": {"default": {"default": "copyright de programme sonore"}},
      "category": "So"
    }, {"key": "211E", "mappings": {"default": {"default": "ordonnances"}}, "category": "So"}, {
      "category": "So",
      "key": "211F",
      "mappings": {"default": {"default": "répons"}}
    }, {
      "category": "So",
      "key": "2120",
      "mappings": {"default": {"default": "symbole anglais marque de service"}}
    }, {"category": "So", "key": "2121", "mappings": {"default": {"default": "signe de téléphone"}}}, {
      "key": "2122",
      "mappings": {"default": {"default": "symbole anglais marque de commerce"}},
      "category": "So"
    }, {"category": "So", "key": "2123", "mappings": {"default": {"default": "versicule"}}}, {
      "key": "2125",
      "mappings": {"default": {"default": "symbole once"}},
      "category": "So"
    }, {"key": "2126", "mappings": {"default": {"default": "ohm"}}, "category": "Lu"}, {
      "key": "2127",
      "mappings": {"default": {"default": "ohm culbuté"}},
      "category": "So"
    }, {"key": "212A", "mappings": {"default": {"default": "degré kelvin"}}, "category": "Lu"}, {
      "key": "212B",
      "mappings": {"default": {"default": "angstroms"}},
      "category": "Lu"
    }, {"category": "So", "key": "212E", "mappings": {"default": {"default": "symbole estimé"}}}, {
      "key": "2132",
      "mappings": {"default": {"default": "f culbuté majuscule"}},
      "category": "Lu"
    }, {
      "category": "Ll",
      "key": "2139",
      "mappings": {"default": {"default": "bureau d'information"}}
    }, {"category": "So", "key": "213A", "mappings": {"default": {"default": "Q majuscule couché"}}}, {
      "category": "So",
      "key": "213B",
      "mappings": {"default": {"default": "symbole télécopie"}}
    }, {
      "category": "Sm",
      "key": "2141",
      "mappings": {"default": {"default": "G culbuté majuscule sans empattement"}}
    }, {
      "category": "Sm",
      "key": "2142",
      "mappings": {"default": {"default": "L culbuté majuscule sans empattement"}}
    }, {
      "category": "Sm",
      "key": "2143",
      "mappings": {"default": {"default": "L réfléchi majuscule sans empattement"}}
    }, {"category": "Sm", "key": "2144", "mappings": {"default": {"default": "Y culbuté majuscule sans empattement"}}}]
  ,
  "fr/symbols/math_symbols.js":
    [{"locale": "fr"}, {
      "key": "0021",
      "mappings": {"default": {"default": "factorielle"}},
      "category": "Po"
    }, {"key": "0022", "mappings": {"default": {"default": "petit guillemet"}}, "category": "Po"}, {
      "key": "0023",
      "mappings": {"default": {"default": "dièse"}},
      "category": "Po"
    }, {"key": "0024", "mappings": {"default": {"default": "dollars"}}, "category": "Sc"}, {
      "key": "0025",
      "mappings": {"default": {"default": "pourcent"}},
      "category": "Po"
    }, {"key": "0026", "mappings": {"default": {"default": "esperluette"}}, "category": "Po"}, {
      "key": "0027",
      "mappings": {"default": {"default": "prime", "alternative": "apostrophe"}},
      "category": "Po"
    }, {"key": "002A", "mappings": {"default": {"default": "astérisque"}}, "category": "Po"}, {
      "key": "002B",
      "mappings": {"default": {"default": "plus"}},
      "category": "Sm"
    }, {"key": "002C", "mappings": {"default": {"default": "virgule"}}, "category": "Po"}, {
      "key": "002D",
      "mappings": {"default": {"default": "moins"}},
      "category": "Pd"
    }, {"key": "002E", "mappings": {"default": {"default": "point"}}, "category": "Po"}, {
      "key": "002F",
      "mappings": {"default": {"default": "barre oblique"}},
      "category": "Po"
    }, {"key": "003A", "mappings": {"default": {"default": "deux points"}}, "category": "Po"}, {
      "key": "003B",
      "mappings": {"default": {"default": "point virgule"}},
      "category": "Po"
    }, {"key": "003C", "mappings": {"default": {"default": "inférieur à"}}, "category": "Sm"}, {
      "key": "003D",
      "mappings": {"default": {"default": "égale"}},
      "category": "Sm"
    }, {"key": "003E", "mappings": {"default": {"default": "supérieur à"}}, "category": "Sm"}, {
      "key": "003F",
      "mappings": {"default": {"default": "point d'interrogation"}},
      "category": "Po"
    }, {"key": "0040", "mappings": {"default": {"default": "arobase"}}, "category": "Po"}, {
      "key": "005C",
      "mappings": {"default": {"default": "barre oblique inversée"}},
      "category": "Po"
    }, {"key": "005E", "mappings": {"default": {"default": "circonflexe"}}, "category": "Sk"}, {
      "key": "005F",
      "mappings": {"default": {"default": "tiret bas"}},
      "category": "Pc"
    }, {"key": "0060", "mappings": {"default": {"default": "accent grave"}}, "category": "Sk"}, {
      "key": "007C",
      "mappings": {"default": {"default": "barre verticale"}},
      "category": "Sm"
    }, {"key": "007E", "mappings": {"default": {"default": "tilde"}}, "category": "Sm"}, {
      "key": "00A1",
      "mappings": {"default": {"default": "point d'exclamation renversé"}},
      "category": "Po"
    }, {"key": "00A2", "mappings": {"default": {"default": "centimes"}}, "category": "Sc"}, {
      "key": "00A3",
      "mappings": {"default": {"default": "livres"}},
      "category": "Sc"
    }, {"key": "00A4", "mappings": {"default": {"default": "symbole monétaire"}}, "category": "Sc"}, {
      "key": "00A5",
      "mappings": {"default": {"default": "yen"}},
      "category": "Sc"
    }, {"key": "00A6", "mappings": {"default": {"default": "barre déjointe"}}, "category": "So"}, {
      "key": "00A7",
      "mappings": {"default": {"default": "paragraphe"}},
      "category": "Po"
    }, {"key": "00A8", "mappings": {"default": {"default": "tréma"}}, "category": "Sk"}, {
      "key": "00A9",
      "mappings": {"default": {"default": "copyright"}},
      "category": "So"
    }, {
      "key": "00AA",
      "mappings": {"default": {"default": "indicateur ordinal féminin"}},
      "category": "Lo"
    }, {
      "key": "00AB",
      "mappings": {"default": {"default": "guillemet chevron pointant gauche"}},
      "category": "Pi"
    }, {"key": "00AC", "mappings": {"default": {"default": "négation"}}, "category": "Sm"}, {
      "key": "00AE",
      "mappings": {"default": {"default": "symbole marque déposée"}},
      "category": "So"
    }, {"key": "00AF", "mappings": {"default": {"default": "macron"}}, "category": "Sk"}, {
      "key": "00B0",
      "mappings": {"default": {"default": "degrés"}},
      "category": "So"
    }, {"key": "00B1", "mappings": {"default": {"default": "plus ou moins"}}, "category": "Sm"}, {
      "key": "00B4",
      "mappings": {"default": {"default": "accent aigu"}},
      "category": "Sk"
    }, {"key": "00B5", "mappings": {"default": {"default": "symbole micro"}}, "category": "Ll"}, {
      "key": "00B6",
      "mappings": {"default": {"default": "fin de paragraphe"}},
      "category": "Po"
    }, {"key": "00B7", "mappings": {"default": {"default": "point médian"}}, "category": "Po"}, {
      "key": "00B8",
      "mappings": {"default": {"default": "cédille"}},
      "category": "Sk"
    }, {
      "key": "00BA",
      "mappings": {"default": {"default": "indicateur ordinal masculin"}},
      "category": "Lo"
    }, {
      "key": "00BB",
      "mappings": {"default": {"default": "guillemet chevron pointant droite"}},
      "category": "Pf"
    }, {
      "key": "00BF",
      "mappings": {"default": {"default": "point d'interrogation renversé"}},
      "category": "Po"
    }, {"key": "00D7", "mappings": {"default": {"default": "multiplié par"}}, "category": "Sm"}, {
      "key": "00F7",
      "mappings": {"default": {"default": "divisé par"}},
      "category": "Sm"
    }, {"key": "02D8", "mappings": {"default": {"default": "brève"}}, "category": "Sk"}, {
      "key": "02D9",
      "mappings": {"default": {"default": "point en chef"}},
      "category": "Sk"
    }, {"key": "02DA", "mappings": {"default": {"default": "rond en chef"}}, "category": "Sk"}, {
      "key": "02DB",
      "mappings": {"default": {"default": "ogonek"}},
      "category": "Sk"
    }, {"key": "02DC", "mappings": {"default": {"default": "tilde"}}, "category": "Sk"}, {
      "key": "02DD",
      "mappings": {"default": {"default": "double accent aigu"}},
      "category": "Sk"
    }, {"key": "2010", "mappings": {"default": {"default": "trait d'union"}}, "category": "Pd"}, {
      "category": "Pd",
      "key": "2011",
      "mappings": {"default": {"default": "trait d'union insécable"}}
    }, {"category": "Pd", "key": "2012", "mappings": {"default": {"default": "tiret numérique"}}}, {
      "key": "2013",
      "mappings": {"default": {"default": "trait d'union insécable"}},
      "category": "Pd"
    }, {"key": "2014", "mappings": {"default": {"default": "tiret numérique"}}, "category": "Pd"}, {
      "key": "2015",
      "mappings": {"default": {"default": "barre horizontalle"}},
      "category": "Pd"
    }, {
      "key": "2016",
      "mappings": {"default": {"default": "double ligne verticale"}},
      "category": "Po"
    }, {"category": "Po", "key": "2017", "mappings": {"default": {"default": "double trait souscrit"}}}, {
      "key": "2018",
      "mappings": {"default": {"default": "guillemet apostrophe culbuté"}},
      "category": "Pi"
    }, {"key": "2019", "mappings": {"default": {"default": "guillemet apostrophe"}}, "category": "Pf"}, {
      "key": "201A",
      "mappings": {"default": {"default": "guillemet virgule inférieur"}},
      "category": "Ps"
    }, {
      "category": "Pi",
      "key": "201B",
      "mappings": {"default": {"default": "guillemet virgule supérieur culbuté"}}
    }, {
      "key": "201C",
      "mappings": {"default": {"default": "guillemet apostrophe double culbuté"}},
      "category": "Pi"
    }, {
      "key": "201D",
      "mappings": {"default": {"default": "guillemet apostrophe double"}},
      "category": "Pf"
    }, {
      "key": "201E",
      "mappings": {"default": {"default": "guillemet double prime"}},
      "category": "Ps"
    }, {
      "category": "Pi",
      "key": "201F",
      "mappings": {"default": {"default": "guillemet virgule double supérieur culbuté"}}
    }, {"key": "2020", "mappings": {"default": {"default": "obèle"}}, "category": "Po"}, {
      "key": "2021",
      "mappings": {"default": {"default": "double obèle"}},
      "category": "Po"
    }, {"key": "2022", "mappings": {"default": {"default": "puce"}}, "category": "Po"}, {
      "category": "Po",
      "key": "2023",
      "mappings": {"default": {"default": "puce triangulaire"}}
    }, {
      "category": "Po",
      "key": "2024",
      "mappings": {"default": {"default": "point de conduite simple"}}
    }, {
      "key": "2025",
      "mappings": {"default": {"default": "point de conduite double"}},
      "category": "Po"
    }, {
      "key": "2026",
      "mappings": {"default": {"default": "points de suspension"}},
      "category": "Po"
    }, {
      "category": "Po",
      "key": "2027",
      "mappings": {"default": {"default": "point de coupure de mot"}}
    }, {"key": "2030", "mappings": {"default": {"default": "symbole pour mille"}}, "category": "Po"}, {
      "key": "2031",
      "mappings": {"default": {"default": "symbole pour dix mille"}},
      "category": "Po"
    }, {"key": "2032", "mappings": {"default": {"default": "prime"}}, "category": "Po"}, {
      "key": "2033",
      "mappings": {"default": {"default": "double prime"}},
      "category": "Po"
    }, {"key": "2034", "mappings": {"default": {"default": "triple prime"}}, "category": "Po"}, {
      "key": "2035",
      "mappings": {"default": {"default": "prime réfléchi"}},
      "category": "Po"
    }, {
      "key": "2036",
      "mappings": {"default": {"default": "double prime réfléchi"}},
      "category": "Po"
    }, {
      "category": "Po",
      "key": "2037",
      "mappings": {"default": {"default": "triple prime réfléchi"}}
    }, {"category": "Po", "key": "2038", "mappings": {"default": {"default": "chevron d'insertion"}}}, {
      "key": "2039",
      "mappings": {"default": {"default": "guillemet simple gauche"}},
      "category": "Pi"
    }, {
      "key": "203A",
      "mappings": {"default": {"default": "guillemet simple droite"}},
      "category": "Pf"
    }, {
      "category": "Po",
      "key": "203B",
      "mappings": {"default": {"default": "marque de référence"}}
    }, {
      "category": "Po",
      "key": "203C",
      "mappings": {"default": {"default": "double point d'exclamation"}}
    }, {"category": "Po", "key": "203D", "mappings": {"default": {"default": "point exclarrogatif"}}}, {
      "key": "203E",
      "mappings": {"default": {"default": "tiret en chef"}},
      "category": "Po"
    }, {"category": "Pc", "key": "203F", "mappings": {"default": {"default": "tirant souscrit"}}}, {
      "key": "2040",
      "mappings": {"default": {"default": "tirant en chef"}},
      "category": "Pc"
    }, {"key": "2041", "mappings": {"default": {"default": "lambda d'insertion"}}, "category": "Po"}, {
      "category": "Po",
      "key": "2042",
      "mappings": {"default": {"default": "astérisme"}}
    }, {"key": "2043", "mappings": {"default": {"default": "puce trait d'union"}}, "category": "Po"}, {
      "key": "2044",
      "mappings": {"default": {"default": "barre de fraction"}},
      "category": "Sm"
    }, {
      "category": "Po",
      "key": "2047",
      "mappings": {"default": {"default": "double point d'interrogation"}}
    }, {
      "category": "Po",
      "key": "2048",
      "mappings": {"default": {"default": "point d'interrogation exclamation"}}
    }, {
      "category": "Po",
      "key": "2049",
      "mappings": {"default": {"default": "point d'exclamation interrogation"}}
    }, {
      "category": "Po",
      "key": "204B",
      "mappings": {"default": {"default": "pied de mouche réfléchi"}}
    }, {
      "category": "Po",
      "key": "204C",
      "mappings": {"default": {"default": "puce noire tronquée à droite"}}
    }, {
      "category": "Po",
      "key": "204D",
      "mappings": {"default": {"default": "puce noire tronquée à gauche"}}
    }, {"category": "Po", "key": "204E", "mappings": {"default": {"default": "astérisque baissé"}}}, {
      "key": "204F",
      "mappings": {"default": {"default": "point virgule réfléchi"}},
      "category": "Po"
    }, {"key": "2050", "mappings": {"default": {"default": "resserrer"}}, "category": "Po"}, {
      "category": "Po",
      "key": "2051",
      "mappings": {"default": {"default": "deux astérisques alignés verticalement"}}
    }, {"category": "Sm", "key": "2052", "mappings": {"default": {"default": "moins commercial"}}}, {
      "category": "Po",
      "key": "2053",
      "mappings": {"default": {"default": "tiret ondé"}}
    }, {"category": "Pc", "key": "2054", "mappings": {"default": {"default": "dos d'âne souscrit"}}}, {
      "category": "Po",
      "key": "2055",
      "mappings": {"default": {"default": "point fleur"}}
    }, {"category": "Po", "key": "2056", "mappings": {"default": {"default": "trois points"}}}, {
      "key": "2057",
      "mappings": {"default": {"default": "quadruple prime"}},
      "category": "Po"
    }, {
      "category": "Po",
      "key": "2058",
      "mappings": {"default": {"default": "quatre points en losange"}}
    }, {
      "category": "Po",
      "key": "2059",
      "mappings": {"default": {"default": "cinq points en quinconce"}}
    }, {
      "category": "Po",
      "key": "205A",
      "mappings": {"default": {"default": "deux points en capitale"}}
    }, {
      "category": "Po",
      "key": "205B",
      "mappings": {"default": {"default": "quatre points en croix"}}
    }, {"category": "Po", "key": "205C", "mappings": {"default": {"default": "croix pointée"}}}, {
      "category": "Po",
      "key": "205D",
      "mappings": {"default": {"default": "trois points vertical"}}
    }, {
      "category": "Po",
      "key": "205E",
      "mappings": {"default": {"default": "quatre points vertical"}}
    }, {
      "category": "Sm",
      "key": "207A",
      "mappings": {"default": {"default": "exposant signe plus"}}
    }, {
      "category": "Sm",
      "key": "207B",
      "mappings": {"default": {"default": "exposant signe moins"}}
    }, {
      "category": "Sm",
      "key": "207C",
      "mappings": {"default": {"default": "exposant signe égal"}}
    }, {
      "category": "Ps",
      "key": "207D",
      "mappings": {"default": {"default": "exposant parenthèse gauche"}}
    }, {
      "category": "Pe",
      "key": "207E",
      "mappings": {"default": {"default": "exposant parenthèse droite"}}
    }, {"category": "Sm", "key": "208A", "mappings": {"default": {"default": "indice signe plus"}}}, {
      "category": "Sm",
      "key": "208B",
      "mappings": {"default": {"default": "indice signe moins"}}
    }, {"category": "Sm", "key": "208C", "mappings": {"default": {"default": "indice signe égal"}}}, {
      "category": "Ps",
      "key": "208D",
      "mappings": {"default": {"default": "indice parenthèse gauche"}}
    }, {
      "category": "Pe",
      "key": "208E",
      "mappings": {"default": {"default": "indice parenthèse droite"}}
    }, {
      "category": "So",
      "key": "214A",
      "mappings": {"default": {"default": "limite de propriété"}}
    }, {"category": "Sm", "key": "214B", "mappings": {"default": {"default": "perluète culbutée"}}}, {
      "category": "So",
      "key": "214C",
      "mappings": {"default": {"default": "symbole per"}}
    }, {"category": "So", "key": "214D", "mappings": {"default": {"default": "aktieselskab"}}}, {
      "category": "Ll",
      "key": "214E",
      "mappings": {"default": {"default": "minuscule f culbuté"}}
    }, {"key": "2200", "mappings": {"default": {"default": "pour tous"}}, "category": "Sm"}, {
      "key": "2201",
      "mappings": {"default": {"default": "complément"}},
      "category": "Sm"
    }, {"key": "2203", "mappings": {"default": {"default": "il existe"}}, "category": "Sm"}, {
      "key": "2204",
      "mappings": {"default": {"default": "il n'existe pas"}},
      "category": "Sm"
    }, {"key": "2205", "mappings": {"default": {"default": "ensemble vide"}}, "category": "Sm"}, {
      "key": "2206",
      "mappings": {"default": {"default": "incrément"}},
      "category": "Sm"
    }, {"key": "2208", "mappings": {"default": {"default": "appartient à"}}, "category": "Sm"}, {
      "key": "2209",
      "mappings": {"default": {"default": "n'appartient pas à"}},
      "category": "Sm"
    }, {"key": "220A", "mappings": {"default": {"default": "appartient à"}}, "category": "Sm"}, {
      "key": "220B",
      "mappings": {"default": {"default": "contient comme élément"}},
      "category": "Sm"
    }, {
      "key": "220C",
      "mappings": {"default": {"default": "ne contient pas comme élément"}},
      "category": "Sm"
    }, {
      "key": "220D",
      "mappings": {"default": {"default": "contient comme élément"}},
      "category": "Sm"
    }, {
      "key": "220E",
      "mappings": {"default": {"default": "ce qu'il fallait démontrer"}},
      "category": "Sm"
    }, {"key": "220F", "mappings": {"default": {"default": "produit"}}, "category": "Sm"}, {
      "key": "2210",
      "mappings": {"default": {"default": "coproduit"}},
      "category": "Sm"
    }, {"key": "2211", "mappings": {"default": {"default": "sommation"}}, "category": "Sm"}, {
      "key": "2212",
      "mappings": {"default": {"default": "moins"}},
      "category": "Sm"
    }, {"key": "2213", "mappings": {"default": {"default": "moins ou plus"}}, "category": "Sm"}, {
      "key": "2214",
      "mappings": {"default": {"default": "plus pointé"}},
      "category": "Sm"
    }, {"key": "2215", "mappings": {"default": {"default": "division"}}, "category": "Sm"}, {
      "key": "2216",
      "mappings": {"default": {"default": "différence d'ensembles"}},
      "category": "Sm"
    }, {"key": "2217", "mappings": {"default": {"default": "opérateur astérisque"}}, "category": "Sm"}, {
      "key": "2218",
      "mappings": {"default": {"default": "opérateur rond"}},
      "category": "Sm"
    }, {"key": "2219", "mappings": {"default": {"default": "point médian"}}, "category": "Sm"}, {
      "key": "221A",
      "mappings": {"default": {"default": "racine carrée"}},
      "category": "Sm"
    }, {"key": "221B", "mappings": {"default": {"default": "racine cubique"}}, "category": "Sm"}, {
      "key": "221C",
      "mappings": {"default": {"default": "racine quatrième"}},
      "category": "Sm"
    }, {"key": "221D", "mappings": {"default": {"default": "proportionnel à"}}, "category": "Sm"}, {
      "key": "221E",
      "mappings": {"default": {"default": "infini"}},
      "category": "Sm"
    }, {"key": "221F", "mappings": {"default": {"default": "angle droit"}}, "category": "Sm"}, {
      "key": "2220",
      "mappings": {"default": {"default": "angle"}},
      "category": "Sm"
    }, {"key": "2221", "mappings": {"default": {"default": "angle mesuré"}}, "category": "Sm"}, {
      "key": "2222",
      "mappings": {"default": {"default": "angle sphérique"}},
      "category": "Sm"
    }, {"key": "2223", "mappings": {"default": {"default": "est un diviseur de"}}, "category": "Sm"}, {
      "key": "2224",
      "mappings": {"default": {"default": "n'est pas un diviseur de"}},
      "category": "Sm"
    }, {"key": "2225", "mappings": {"default": {"default": "parallèle à"}}, "category": "Sm"}, {
      "key": "2226",
      "mappings": {"default": {"default": "non parallèle à"}},
      "category": "Sm"
    }, {"key": "2227", "mappings": {"default": {"default": "et logique"}}, "category": "Sm"}, {
      "key": "2228",
      "mappings": {"default": {"default": "ou logique"}},
      "category": "Sm"
    }, {"key": "2229", "mappings": {"default": {"default": "intersection"}}, "category": "Sm"}, {
      "key": "222A",
      "mappings": {"default": {"default": "union"}},
      "category": "Sm"
    }, {"key": "222B", "mappings": {"default": {"default": "intégrale"}}, "category": "Sm"}, {
      "key": "222C",
      "mappings": {"default": {"default": "intégrale double"}},
      "category": "Sm"
    }, {"key": "222D", "mappings": {"default": {"default": "intégrale triple"}}, "category": "Sm"}, {
      "key": "222E",
      "mappings": {"default": {"default": "intégrale de contour"}},
      "category": "Sm"
    }, {"key": "222F", "mappings": {"default": {"default": "intégrale de surface"}}, "category": "Sm"}, {
      "key": "2230",
      "mappings": {"default": {"default": "intégrale de volume"}},
      "category": "Sm"
    }, {
      "key": "2231",
      "mappings": {"default": {"default": "intégrale en sens négatif"}},
      "category": "Sm"
    }, {
      "key": "2232",
      "mappings": {"default": {"default": "intégrale de contour en sens négatif"}},
      "category": "Sm"
    }, {
      "key": "2233",
      "mappings": {"default": {"default": "intégrale de contour en sens positif"}},
      "category": "Sm"
    }, {"key": "2234", "mappings": {"default": {"default": "par conséquent"}}, "category": "Sm"}, {
      "key": "2235",
      "mappings": {"default": {"default": "parce que"}},
      "category": "Sm"
    }, {"key": "2236", "mappings": {"default": {"default": "rapport"}}, "category": "Sm"}, {
      "key": "2237",
      "mappings": {"default": {"default": "proportion"}},
      "category": "Sm"
    }, {"key": "2238", "mappings": {"default": {"default": "moins pointé"}}, "category": "Sm"}, {
      "key": "2239",
      "mappings": {"default": {"default": "excès"}},
      "category": "Sm"
    }, {
      "key": "223A",
      "mappings": {"default": {"default": "proportion géométrique"}},
      "category": "Sm"
    }, {"key": "223B", "mappings": {"default": {"default": "homothétique"}}, "category": "Sm"}, {
      "key": "223C",
      "mappings": {"default": {"default": "opérateur tilde"}},
      "category": "Sm"
    }, {
      "key": "223D",
      "mappings": {"default": {"default": "opérateur tilde renversé"}},
      "category": "Sm"
    }, {"key": "223E", "mappings": {"default": {"default": "s couché renversé"}}, "category": "Sm"}, {
      "key": "223F",
      "mappings": {"default": {"default": "sinusoïde"}},
      "category": "Sm"
    }, {"key": "2240", "mappings": {"default": {"default": "produit couronne"}}, "category": "Sm"}, {
      "key": "2241",
      "mappings": {"default": {"default": "non tilde"}},
      "category": "Sm"
    }, {"key": "2242", "mappings": {"default": {"default": "moins tilde"}}, "category": "Sm"}, {
      "key": "2243",
      "mappings": {"default": {"default": "asymptotiquement égal à"}},
      "category": "Sm"
    }, {
      "key": "2244",
      "mappings": {"default": {"default": "non asymptotiquement égal à"}},
      "category": "Sm"
    }, {
      "key": "2245",
      "mappings": {"default": {"default": "approximativement égal à"}},
      "category": "Sm"
    }, {
      "key": "2246",
      "mappings": {"default": {"default": "approximativement mais non strictement égal à"}},
      "category": "Sm"
    }, {
      "key": "2247",
      "mappings": {"default": {"default": "ni approximativement ni strictement égal à"}},
      "category": "Sm"
    }, {"key": "2248", "mappings": {"default": {"default": "presque égal à"}}, "category": "Sm"}, {
      "key": "2249",
      "mappings": {"default": {"default": "non presque égal à"}},
      "category": "Sm"
    }, {
      "key": "224A",
      "mappings": {"default": {"default": "presque égal ou égal à"}},
      "category": "Sm"
    }, {"key": "224B", "mappings": {"default": {"default": "triple tilde"}}, "category": "Sm"}, {
      "key": "224C",
      "mappings": {"default": {"default": "entièrement égal à"}},
      "category": "Sm"
    }, {"key": "224D", "mappings": {"default": {"default": "équivalent à"}}, "category": "Sm"}, {
      "key": "224E",
      "mappings": {"default": {"default": "géométriquement équivalent à"}},
      "category": "Sm"
    }, {"key": "224F", "mappings": {"default": {"default": "différence entre"}}, "category": "Sm"}, {
      "key": "2250",
      "mappings": {"default": {"default": "tend vers la limite"}},
      "category": "Sm"
    }, {
      "key": "2251",
      "mappings": {"default": {"default": "géométriquement égal à"}},
      "category": "Sm"
    }, {
      "key": "2252",
      "mappings": {"default": {"default": "approximativement égal à ou image de"}},
      "category": "Sm"
    }, {
      "key": "2253",
      "mappings": {"default": {"default": "image de ou approximativement égal à"}},
      "category": "Sm"
    }, {"key": "2254", "mappings": {"default": {"default": "deux points égal"}}, "category": "Sm"}, {
      "key": "2255",
      "mappings": {"default": {"default": "égal deux points"}},
      "category": "Sm"
    }, {"key": "2256", "mappings": {"default": {"default": "rond dans égal"}}, "category": "Sm"}, {
      "key": "2257",
      "mappings": {"default": {"default": "égal avec rond en chef"}},
      "category": "Sm"
    }, {"key": "2258", "mappings": {"default": {"default": "correspond à"}}, "category": "Sm"}, {
      "key": "2259",
      "mappings": {"default": {"default": "estime"}},
      "category": "Sm"
    }, {"key": "225A", "mappings": {"default": {"default": "équiangulaire à"}}, "category": "Sm"}, {
      "key": "225B",
      "mappings": {"default": {"default": "égal avec étoile en chef"}},
      "category": "Sm"
    }, {"key": "225C", "mappings": {"default": {"default": "égal delta"}}, "category": "Sm"}, {
      "key": "225D",
      "mappings": {"default": {"default": "égal par définition à"}},
      "category": "Sm"
    }, {"key": "225E", "mappings": {"default": {"default": "mesuré par"}}, "category": "Sm"}, {
      "key": "225F",
      "mappings": {"default": {"default": "égalité en doute"}},
      "category": "Sm"
    }, {"key": "2260", "mappings": {"default": {"default": "pas égal à"}}, "category": "Sm"}, {
      "key": "2261",
      "mappings": {"default": {"default": "identique à"}},
      "category": "Sm"
    }, {"key": "2262", "mappings": {"default": {"default": "non identique à"}}, "category": "Sm"}, {
      "key": "2263",
      "mappings": {"default": {"default": "strictement équivalent à"}},
      "category": "Sm"
    }, {"key": "2264", "mappings": {"default": {"default": "plus petit ou égal à"}}, "category": "Sm"}, {
      "key": "2265",
      "mappings": {"default": {"default": "plus grand ou égal à"}},
      "category": "Sm"
    }, {
      "key": "2266",
      "mappings": {"default": {"default": "plus petit que par dessus égal à"}},
      "category": "Sm"
    }, {
      "key": "2267",
      "mappings": {"default": {"default": "plus grand que par dessus égal à"}},
      "category": "Sm"
    }, {
      "key": "2268",
      "mappings": {"default": {"default": "plus petit mais pas égal à"}},
      "category": "Sm"
    }, {
      "key": "2269",
      "mappings": {"default": {"default": "plus grand mais pas égal à"}},
      "category": "Sm"
    }, {
      "key": "226A",
      "mappings": {"default": {"default": "beaucoup plus petit que"}},
      "category": "Sm"
    }, {
      "key": "226B",
      "mappings": {"default": {"default": "beaucoup plus grand que"}},
      "category": "Sm"
    }, {"key": "226C", "mappings": {"default": {"default": "entre"}}, "category": "Sm"}, {
      "key": "226D",
      "mappings": {"default": {"default": "non équivalent à"}},
      "category": "Sm"
    }, {"key": "226E", "mappings": {"default": {"default": "pas plus petit que"}}, "category": "Sm"}, {
      "key": "226F",
      "mappings": {"default": {"default": "pas plus grand que"}},
      "category": "Sm"
    }, {
      "key": "2270",
      "mappings": {"default": {"default": "ni plus petit ni égal à"}},
      "category": "Sm"
    }, {
      "key": "2271",
      "mappings": {"default": {"default": "ni plus grand ni égal à"}},
      "category": "Sm"
    }, {
      "key": "2272",
      "mappings": {"default": {"default": "plus petit ou équivalent à"}},
      "category": "Sm"
    }, {
      "key": "2273",
      "mappings": {"default": {"default": "plus grand ou équivalent à"}},
      "category": "Sm"
    }, {
      "key": "2274",
      "mappings": {"default": {"default": "ni plus petit ni équivalent à"}},
      "category": "Sm"
    }, {
      "key": "2275",
      "mappings": {"default": {"default": "ni plus grand ni équivalent à"}},
      "category": "Sm"
    }, {
      "key": "2276",
      "mappings": {"default": {"default": "plus petit ou plus grand que"}},
      "category": "Sm"
    }, {
      "key": "2277",
      "mappings": {"default": {"default": "plus grand ou plus petit que"}},
      "category": "Sm"
    }, {
      "key": "2278",
      "mappings": {"default": {"default": "ni plus petit ni plus grand que"}},
      "category": "Sm"
    }, {
      "key": "2279",
      "mappings": {"default": {"default": "ni plus grand ni plus petit que"}},
      "category": "Sm"
    }, {"key": "227A", "mappings": {"default": {"default": "précède"}}, "category": "Sm"}, {
      "key": "227B",
      "mappings": {"default": {"default": "suit"}},
      "category": "Sm"
    }, {"key": "227C", "mappings": {"default": {"default": "précède ou est égal à"}}, "category": "Sm"}, {
      "key": "227D",
      "mappings": {"default": {"default": "suit ou est égal à"}},
      "category": "Sm"
    }, {
      "key": "227E",
      "mappings": {"default": {"default": "précède ou est équivalent à"}},
      "category": "Sm"
    }, {
      "key": "227F",
      "mappings": {"default": {"default": "suit ou est équivalent à"}},
      "category": "Sm"
    }, {"key": "2280", "mappings": {"default": {"default": "ne précède pas"}}, "category": "Sm"}, {
      "key": "2281",
      "mappings": {"default": {"default": "ne suit pas"}},
      "category": "Sm"
    }, {"key": "2282", "mappings": {"default": {"default": "sous ensemble de"}}, "category": "Sm"}, {
      "key": "2283",
      "mappings": {"default": {"default": "sur ensemble de"}},
      "category": "Sm"
    }, {
      "key": "2284",
      "mappings": {"default": {"default": "pas un sous ensemble de"}},
      "category": "Sm"
    }, {
      "key": "2285",
      "mappings": {"default": {"default": "pas un sur ensemble de"}},
      "category": "Sm"
    }, {
      "key": "2286",
      "mappings": {"default": {"default": "sous ensemble ou égal à"}},
      "category": "Sm"
    }, {
      "key": "2287",
      "mappings": {"default": {"default": "sur ensemble ou égal à"}},
      "category": "Sm"
    }, {
      "key": "2288",
      "mappings": {"default": {"default": "ni un sous ensemble ni égal à"}},
      "category": "Sm"
    }, {
      "key": "2289",
      "mappings": {"default": {"default": "ni un sur ensemble ni égal à"}},
      "category": "Sm"
    }, {
      "key": "228A",
      "mappings": {"default": {"default": "sous ensemble mais pas égal à"}},
      "category": "Sm"
    }, {
      "key": "228B",
      "mappings": {"default": {"default": "sur ensemble mais pas égal à"}},
      "category": "Sm"
    }, {"key": "228C", "mappings": {"default": {"default": "multiensemble"}}, "category": "Sm"}, {
      "key": "228D",
      "mappings": {"default": {"default": "multiplication de multiensemble"}},
      "category": "Sm"
    }, {
      "key": "228E",
      "mappings": {"default": {"default": "union de multiensemble"}},
      "category": "Sm"
    }, {"key": "228F", "mappings": {"default": {"default": "image carrée de"}}, "category": "Sm"}, {
      "key": "2290",
      "mappings": {"default": {"default": "original carré de"}},
      "category": "Sm"
    }, {
      "key": "2291",
      "mappings": {"default": {"default": "image carrée ou égal à"}},
      "category": "Sm"
    }, {
      "key": "2292",
      "mappings": {"default": {"default": "original carré ou égal à"}},
      "category": "Sm"
    }, {"key": "2293", "mappings": {"default": {"default": "chapeau carré"}}, "category": "Sm"}, {
      "key": "2294",
      "mappings": {"default": {"default": "coupe carrée"}},
      "category": "Sm"
    }, {"key": "2295", "mappings": {"default": {"default": "plus cerclé"}}, "category": "Sm"}, {
      "key": "2296",
      "mappings": {"default": {"default": "moins cerclé"}},
      "category": "Sm"
    }, {"key": "2297", "mappings": {"default": {"default": "multiplié par cerclé"}}, "category": "Sm"}, {
      "key": "2298",
      "mappings": {"default": {"default": "barre oblique de division cerclée"}},
      "category": "Sm"
    }, {
      "key": "2299",
      "mappings": {"default": {"default": "opérateur point cerclé"}},
      "category": "Sm"
    }, {"key": "229A", "mappings": {"default": {"default": "opérateur rond cerclé"}}, "category": "Sm"}, {
      "key": "229B",
      "mappings": {"default": {"default": "opérateur astérisque cerclé"}},
      "category": "Sm"
    }, {"key": "229C", "mappings": {"default": {"default": "égal cerclé"}}, "category": "Sm"}, {
      "key": "229D",
      "mappings": {"default": {"default": "tiret cerclé"}},
      "category": "Sm"
    }, {"key": "229E", "mappings": {"default": {"default": "plus encadré"}}, "category": "Sm"}, {
      "key": "229F",
      "mappings": {"default": {"default": "moins encadré"}},
      "category": "Sm"
    }, {"key": "22A0", "mappings": {"default": {"default": "multiplié par encadré"}}, "category": "Sm"}, {
      "key": "22A1",
      "mappings": {"default": {"default": "opérateur point encadré"}},
      "category": "Sm"
    }, {"key": "22A2", "mappings": {"default": {"default": "taquet droit"}}, "category": "Sm"}, {
      "key": "22A3",
      "mappings": {"default": {"default": "taquet gauche"}},
      "category": "Sm"
    }, {"key": "22A4", "mappings": {"default": {"default": "taquet vers le bas"}}, "category": "Sm"}, {
      "key": "22A5",
      "mappings": {"default": {"default": "taquet vers le haut"}},
      "category": "Sm"
    }, {"key": "22A6", "mappings": {"default": {"default": "assertion"}}, "category": "Sm"}, {
      "key": "22A7",
      "mappings": {"default": {"default": "modélise"}},
      "category": "Sm"
    }, {"key": "22A8", "mappings": {"default": {"default": "vrai"}}, "category": "Sm"}, {
      "key": "22A9",
      "mappings": {"default": {"default": "oblige"}},
      "category": "Sm"
    }, {
      "key": "22AA",
      "mappings": {"default": {"default": "barre verticale triple avec tourniquet à droite"}},
      "category": "Sm"
    }, {
      "key": "22AB",
      "mappings": {"default": {"default": "barre verticale double avec tourniquet à droite double"}},
      "category": "Sm"
    }, {"key": "22AC", "mappings": {"default": {"default": "ne prouve pas"}}, "category": "Sm"}, {
      "key": "22AD",
      "mappings": {"default": {"default": "pas vrai"}},
      "category": "Sm"
    }, {"key": "22AE", "mappings": {"default": {"default": "n'oblige pas"}}, "category": "Sm"}, {
      "key": "22AF",
      "mappings": {"default": {"default": "non barre verticale double avec tourniquet à droite double"}},
      "category": "Sm"
    }, {"key": "22B0", "mappings": {"default": {"default": "précède sous relation"}}, "category": "Sm"}, {
      "key": "22B1",
      "mappings": {"default": {"default": "suit sous relation"}},
      "category": "Sm"
    }, {"key": "22B2", "mappings": {"default": {"default": "sous groupe normal de"}}, "category": "Sm"}, {
      "key": "22B3",
      "mappings": {"default": {"default": "contient comme sous groupe normal"}},
      "category": "Sm"
    }, {
      "key": "22B4",
      "mappings": {"default": {"default": "sous groupe normal ou égal à"}},
      "category": "Sm"
    }, {
      "key": "22B5",
      "mappings": {"default": {"default": "contient comme sous groupe normal ou égal à"}},
      "category": "Sm"
    }, {"key": "22B6", "mappings": {"default": {"default": "original de"}}, "category": "Sm"}, {
      "key": "22B7",
      "mappings": {"default": {"default": "image de"}},
      "category": "Sm"
    }, {"key": "22B8", "mappings": {"default": {"default": "multijection"}}, "category": "Sm"}, {
      "key": "22B9",
      "mappings": {"default": {"default": "matrice hermitienne conjuguée"}},
      "category": "Sm"
    }, {"key": "22BA", "mappings": {"default": {"default": "intercale"}}, "category": "Sm"}, {
      "key": "22BB",
      "mappings": {"default": {"default": "ou exclusif"}},
      "category": "Sm"
    }, {"key": "22BC", "mappings": {"default": {"default": "non et"}}, "category": "Sm"}, {
      "key": "22BD",
      "mappings": {"default": {"default": "non ou"}},
      "category": "Sm"
    }, {"key": "22BF", "mappings": {"default": {"default": "triangle rectangle"}}, "category": "Sm"}, {
      "key": "22C0",
      "mappings": {"default": {"default": "et logique de la famille"}},
      "category": "Sm"
    }, {
      "key": "22C1",
      "mappings": {"default": {"default": "ou logique de la famille"}},
      "category": "Sm"
    }, {
      "key": "22C2",
      "mappings": {"default": {"default": "intersection de la famille"}},
      "category": "Sm"
    }, {"key": "22C3", "mappings": {"default": {"default": "réunion de la famille"}}, "category": "Sm"}, {
      "key": "22C4",
      "mappings": {"default": {"default": "opérateur losange"}},
      "category": "Sm"
    }, {"key": "22C5", "mappings": {"default": {"default": "opérateur point"}}, "category": "Sm"}, {
      "key": "22C6",
      "mappings": {"default": {"default": "opérateur étoile"}},
      "category": "Sm"
    }, {"key": "22C7", "mappings": {"default": {"default": "divisé multiplié"}}, "category": "Sm"}, {
      "key": "22C8",
      "mappings": {"default": {"default": "nœud papillon"}},
      "category": "Sm"
    }, {
      "key": "22C9",
      "mappings": {"default": {"default": "produit semi direct à gauche de facteur normal"}},
      "category": "Sm"
    }, {
      "key": "22CA",
      "mappings": {"default": {"default": "produit semi direct à droite de facteur normal"}},
      "category": "Sm"
    }, {
      "key": "22CB",
      "mappings": {"default": {"default": "produit semi direct à gauche"}},
      "category": "Sm"
    }, {
      "key": "22CC",
      "mappings": {"default": {"default": "produit semi direct à droite"}},
      "category": "Sm"
    }, {"key": "22CD", "mappings": {"default": {"default": "moins tilde renversé"}}, "category": "Sm"}, {
      "key": "22CE",
      "mappings": {"default": {"default": "ou logique recourbé"}},
      "category": "Sm"
    }, {"key": "22CF", "mappings": {"default": {"default": "et logique recourbé"}}, "category": "Sm"}, {
      "key": "22D0",
      "mappings": {"default": {"default": "sous ensemble double"}},
      "category": "Sm"
    }, {"key": "22D1", "mappings": {"default": {"default": "sur ensemble double"}}, "category": "Sm"}, {
      "key": "22D2",
      "mappings": {"default": {"default": "intersection double"}},
      "category": "Sm"
    }, {"key": "22D3", "mappings": {"default": {"default": "union double"}}, "category": "Sm"}, {
      "key": "22D4",
      "mappings": {"default": {"default": "fourche"}},
      "category": "Sm"
    }, {"key": "22D5", "mappings": {"default": {"default": "égal et parallèle à"}}, "category": "Sm"}, {
      "key": "22D6",
      "mappings": {"default": {"default": "plus petit que pointé"}},
      "category": "Sm"
    }, {"key": "22D7", "mappings": {"default": {"default": "plus grand que pointé"}}, "category": "Sm"}, {
      "key": "22D8",
      "mappings": {"default": {"default": "considérablement plus petit que"}},
      "category": "Sm"
    }, {
      "key": "22D9",
      "mappings": {"default": {"default": "considérablement plus grand que"}},
      "category": "Sm"
    }, {
      "key": "22DA",
      "mappings": {"default": {"default": "plus petit ou égal ou plus grand que"}},
      "category": "Sm"
    }, {
      "key": "22DB",
      "mappings": {"default": {"default": "plus grand ou égal ou plus petit que"}},
      "category": "Sm"
    }, {
      "key": "22DC",
      "mappings": {"default": {"default": "égal ou plus petit que"}},
      "category": "Sm"
    }, {
      "key": "22DD",
      "mappings": {"default": {"default": "égal à deux lignes ou supérieur à"}},
      "category": "Sm"
    }, {"key": "22DE", "mappings": {"default": {"default": "égal ou précède"}}, "category": "Sm"}, {
      "key": "22DF",
      "mappings": {"default": {"default": "égal ou suit"}},
      "category": "Sm"
    }, {
      "key": "22E0",
      "mappings": {"default": {"default": "ni précédant ni égal à"}},
      "category": "Sm"
    }, {"key": "22E1", "mappings": {"default": {"default": "ni suivant ni égal à"}}, "category": "Sm"}, {
      "key": "22E2",
      "mappings": {"default": {"default": "ni image carrée ni égal à"}},
      "category": "Sm"
    }, {
      "key": "22E3",
      "mappings": {"default": {"default": "ni original carré ni égal à"}},
      "category": "Sm"
    }, {
      "key": "22E4",
      "mappings": {"default": {"default": "image carré ou différent de"}},
      "category": "Sm"
    }, {
      "key": "22E5",
      "mappings": {"default": {"default": "original carré ou différent de"}},
      "category": "Sm"
    }, {
      "key": "22E6",
      "mappings": {"default": {"default": "plus petit mais non équivalent à"}},
      "category": "Sm"
    }, {
      "key": "22E7",
      "mappings": {"default": {"default": "plus grand mais non équivalent à"}},
      "category": "Sm"
    }, {
      "key": "22E8",
      "mappings": {"default": {"default": "précédant mais non équivalent à"}},
      "category": "Sm"
    }, {
      "key": "22E9",
      "mappings": {"default": {"default": "suivant mais non équivalent à"}},
      "category": "Sm"
    }, {
      "key": "22EA",
      "mappings": {"default": {"default": "pas un sous groupe normal de"}},
      "category": "Sm"
    }, {
      "key": "22EB",
      "mappings": {"default": {"default": "ne contient pas comme sous groupe normal"}},
      "category": "Sm"
    }, {
      "key": "22EC",
      "mappings": {"default": {"default": "ni sous groupe normal ni égal à"}},
      "category": "Sm"
    }, {
      "key": "22ED",
      "mappings": {"default": {"default": "ni égal ni contenant comme sous  groupe normal"}},
      "category": "Sm"
    }, {
      "key": "22EE",
      "mappings": {"default": {"default": "trois points suspendus"}},
      "category": "Sm"
    }, {"key": "22EF", "mappings": {"default": {"default": "trois points médians"}}, "category": "Sm"}, {
      "key": "22F0",
      "mappings": {"default": {"default": "trois points diagonaux vers le coin haut à droite"}},
      "category": "Sm"
    }, {
      "key": "22F1",
      "mappings": {"default": {"default": "trois points diagonaux vers le coin bas à droite"}},
      "category": "Sm"
    }, {
      "key": "22F2",
      "mappings": {"default": {"default": "appartient à avec long trait horizontal"}},
      "category": "Sm"
    }, {
      "key": "22F3",
      "mappings": {"default": {"default": "appartient à avec barre verticale au bout du trait horizontal"}},
      "category": "Sm"
    }, {
      "key": "22F4",
      "mappings": {"default": {"default": "petit appartient à avec barre verticale au bout du trait horizontal"}},
      "category": "Sm"
    }, {
      "key": "22F5",
      "mappings": {"default": {"default": "appartient à avec point en chef"}},
      "category": "Sm"
    }, {
      "key": "22F6",
      "mappings": {"default": {"default": "appartient à avec trait en chef"}},
      "category": "Sm"
    }, {
      "key": "22F7",
      "mappings": {"default": {"default": "petit appartient à avec trait en chef"}},
      "category": "Sm"
    }, {
      "key": "22F8",
      "mappings": {"default": {"default": "appartient à avec trait souscrit"}},
      "category": "Sm"
    }, {
      "key": "22F9",
      "mappings": {"default": {"default": "appartient à avec deux traits horizontaux"}},
      "category": "Sm"
    }, {
      "key": "22FA",
      "mappings": {"default": {"default": "contient avec long trait horizontal"}},
      "category": "Sm"
    }, {
      "key": "22FB",
      "mappings": {"default": {"default": "contient avec barre verticale au bout du trait horizontal"}},
      "category": "Sm"
    }, {
      "key": "22FC",
      "mappings": {"default": {"default": "petit contient avec barre verticale au bout du trait horizontal"}},
      "category": "Sm"
    }, {
      "key": "22FD",
      "mappings": {"default": {"default": "contient avec trait en chef"}},
      "category": "Sm"
    }, {
      "key": "22FE",
      "mappings": {"default": {"default": "petit contient avec trait en chef"}},
      "category": "Sm"
    }, {
      "key": "22FF",
      "mappings": {"default": {"default": "appartient à un sac en notation z"}},
      "category": "Sm"
    }, {"key": "2300", "mappings": {"default": {"default": "diamètre"}}, "category": "So"}, {
      "key": "2302",
      "mappings": {"default": {"default": "maison"}},
      "category": "So"
    }, {"key": "2305", "mappings": {"default": {"default": "projective"}}, "category": "So"}, {
      "key": "2306",
      "mappings": {"default": {"default": "perspective"}},
      "category": "So"
    }, {"category": "So", "key": "2307", "mappings": {"default": {"default": "ligne ondulée"}}}, {
      "key": "2310",
      "mappings": {"default": {"default": "négation réfléchi"}},
      "category": "So"
    }, {"category": "So", "key": "2311", "mappings": {"default": {"default": "pastille carrée"}}}, {
      "key": "2312",
      "mappings": {"default": {"default": "arc"}},
      "category": "So"
    }, {"key": "2313", "mappings": {"default": {"default": "segment"}}, "category": "So"}, {
      "category": "So",
      "key": "2314",
      "mappings": {"default": {"default": "secteur"}}
    }, {"category": "So", "key": "2795", "mappings": {"default": {"default": "plus gras"}}}, {
      "category": "So",
      "key": "2796",
      "mappings": {"default": {"default": "moins gras"}}
    }, {"category": "So", "key": "2797", "mappings": {"default": {"default": "division gras"}}}, {
      "category": "So",
      "key": "27B0",
      "mappings": {"default": {"default": "boucle recourbée"}}
    }, {
      "category": "So",
      "key": "27BF",
      "mappings": {"default": {"default": "double boucle recourbée"}}
    }, {
      "key": "27C1",
      "mappings": {"default": {"default": "petit triangle blanc inscrit dans triangle blanc"}},
      "category": "Sm"
    }, {"key": "27C2", "mappings": {"default": {"default": "perpendiculaire à"}}, "category": "Sm"}, {
      "key": "27C3",
      "mappings": {"default": {"default": "sous ensemble ouvert"}},
      "category": "Sm"
    }, {"key": "27C4", "mappings": {"default": {"default": "sur ensemble ouvert"}}, "category": "Sm"}, {
      "key": "27C7",
      "mappings": {"default": {"default": "ou avec point inscrit"}},
      "category": "Sm"
    }, {
      "key": "27C8",
      "mappings": {"default": {"default": "sous ensemble précédé d'une barre oblique inversée"}},
      "category": "Sm"
    }, {
      "key": "27C9",
      "mappings": {"default": {"default": "sur ensemble suivi d'une barre oblique"}},
      "category": "Sm"
    }, {
      "key": "27CA",
      "mappings": {"default": {"default": "barre verticale À une traverse"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "27CB",
      "mappings": {"default": {"default": "diagonale mathématique ascendante"}}
    }, {"key": "27CC", "mappings": {"default": {"default": "grande division"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "27CD",
      "mappings": {"default": {"default": "diagonale mathématique descendante"}}
    }, {"category": "Sm", "key": "27CE", "mappings": {"default": {"default": "et logique encadré"}}}, {
      "category": "Sm",
      "key": "27CF",
      "mappings": {"default": {"default": "ou logique encadré"}}
    }, {
      "category": "Sm",
      "key": "27D0",
      "mappings": {"default": {"default": "losange blanc à point centré"}}
    }, {"key": "27D1", "mappings": {"default": {"default": "et pointé"}}, "category": "Sm"}, {
      "key": "27D2",
      "mappings": {"default": {"default": "appartient À ouvert vers le haut"}},
      "category": "Sm"
    }, {
      "key": "27D3",
      "mappings": {"default": {"default": "coin inférieur droit pointé"}},
      "category": "Sm"
    }, {
      "key": "27D4",
      "mappings": {"default": {"default": "coin supérieur gauche pointé"}},
      "category": "Sm"
    }, {
      "key": "27D5",
      "mappings": {"default": {"default": "jointure externe gauche"}},
      "category": "Sm"
    }, {
      "key": "27D6",
      "mappings": {"default": {"default": "jointure externe droite"}},
      "category": "Sm"
    }, {
      "key": "27D7",
      "mappings": {"default": {"default": "jointure externe complète"}},
      "category": "Sm"
    }, {
      "key": "27D8",
      "mappings": {"default": {"default": "grand taquet vers le haut"}},
      "category": "Sm"
    }, {
      "key": "27D9",
      "mappings": {"default": {"default": "grand taquet vers le bas"}},
      "category": "Sm"
    }, {
      "key": "27DA",
      "mappings": {"default": {"default": "double tourniquet bilatéral"}},
      "category": "Sm"
    }, {"key": "27DB", "mappings": {"default": {"default": "tourniquet bilatéral"}}, "category": "Sm"}, {
      "key": "27DC",
      "mappings": {"default": {"default": "multijection gauche"}},
      "category": "Sm"
    }, {"key": "27DD", "mappings": {"default": {"default": "long taquet droit"}}, "category": "Sm"}, {
      "key": "27DE",
      "mappings": {"default": {"default": "long taquet gauche"}},
      "category": "Sm"
    }, {
      "key": "27DF",
      "mappings": {"default": {"default": "taquet vers le haut surmonté d'un cercle"}},
      "category": "Sm"
    }, {"key": "27E0", "mappings": {"default": {"default": "losange coupé"}}, "category": "Sm"}, {
      "key": "27E1",
      "mappings": {"default": {"default": "losange concave blanc"}},
      "category": "Sm"
    }, {
      "key": "27E2",
      "mappings": {"default": {"default": "losange concave blanc avec trait à gauche"}},
      "category": "Sm"
    }, {
      "key": "27E3",
      "mappings": {"default": {"default": "losange concave blanc avec trait à droite"}},
      "category": "Sm"
    }, {
      "key": "27E4",
      "mappings": {"default": {"default": "carré blanc avec trait à gauche"}},
      "category": "Sm"
    }, {
      "key": "27E5",
      "mappings": {"default": {"default": "carré blanc avec trait à droite"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "292B",
      "mappings": {"default": {"default": "diagonale montante sur diagonale descendante"}}
    }, {
      "category": "Sm",
      "key": "292C",
      "mappings": {"default": {"default": "diagonale descendante sur diagonale montante"}}
    }, {
      "category": "Sm",
      "key": "2980",
      "mappings": {"default": {"default": "délimiteur triple barre verticale"}}
    }, {"category": "Sm", "key": "2981", "mappings": {"default": {"default": "boulet"}}}, {
      "category": "Sm",
      "key": "2982",
      "mappings": {"default": {"default": "deux points de la notation z"}}
    }, {"category": "Sm", "key": "2999", "mappings": {"default": {"default": "clôture pointillée"}}}, {
      "key": "299A",
      "mappings": {"default": {"default": "ligne verticale en zigzag"}},
      "category": "Sm"
    }, {
      "key": "29B0",
      "mappings": {"default": {"default": "ensemble vide réfléchi"}},
      "category": "Sm"
    }, {
      "key": "29B1",
      "mappings": {"default": {"default": "ensemble vide barre en chef"}},
      "category": "Sm"
    }, {
      "key": "29B2",
      "mappings": {"default": {"default": "ensemble vide petit cercle en chef"}},
      "category": "Sm"
    }, {
      "key": "29B5",
      "mappings": {"default": {"default": "cercle à barre horizontale"}},
      "category": "Sm"
    }, {
      "key": "29B6",
      "mappings": {"default": {"default": "barre verticale cerclée"}},
      "category": "Sm"
    }, {
      "key": "29B7",
      "mappings": {"default": {"default": "parallèles cerclées"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29B8",
      "mappings": {"default": {"default": "barre oblique inversée cerclée"}}
    }, {
      "key": "29B9",
      "mappings": {"default": {"default": "perpendiculaire cerclé"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29BA",
      "mappings": {"default": {"default": "cercle divisé par une barre horizontale et une barre verticale dans la moitié supérieure"}}
    }, {
      "key": "29BB",
      "mappings": {"default": {"default": "cercle superposé d'un x"}},
      "category": "Sm"
    }, {
      "key": "29BC",
      "mappings": {"default": {"default": "division pivoté en sens antihoraire cerclé"}},
      "category": "Sm"
    }, {"key": "29BE", "mappings": {"default": {"default": "puce blanche cerclée"}}, "category": "Sm"}, {
      "key": "29BF",
      "mappings": {"default": {"default": "puce cerclée"}},
      "category": "Sm"
    }, {"key": "29C0", "mappings": {"default": {"default": "inférieur à cerclé"}}, "category": "Sm"}, {
      "key": "29C1",
      "mappings": {"default": {"default": "supérieur à cerclé"}},
      "category": "Sm"
    }, {
      "key": "29C2",
      "mappings": {"default": {"default": "cercle avec petit cercle à droite"}},
      "category": "Sm"
    }, {
      "key": "29C3",
      "mappings": {"default": {"default": "cercle avec deux petits traits horizontaux à droite"}},
      "category": "Sm"
    }, {"key": "29C4", "mappings": {"default": {"default": "oblique encadrée"}}, "category": "Sm"}, {
      "key": "29C5",
      "mappings": {"default": {"default": "oblique inversée encadrée"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "29C6", "mappings": {"default": {"default": "astérisque encadré"}}}, {
      "category": "Sm",
      "key": "29C7",
      "mappings": {"default": {"default": "petite cercle encadré"}}
    }, {"category": "Sm", "key": "29C8", "mappings": {"default": {"default": "carré encadré"}}}, {
      "key": "29C9",
      "mappings": {"default": {"default": "carrés se chevauchant"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29CA",
      "mappings": {"default": {"default": "triangle pointé en chef"}}
    }, {
      "category": "Sm",
      "key": "29CB",
      "mappings": {"default": {"default": "triangle à barre souscrite"}}
    }, {
      "category": "Sm",
      "key": "29CC",
      "mappings": {"default": {"default": "s inscrit dans triangle"}}
    }, {
      "key": "29CD",
      "mappings": {"default": {"default": "triangle à empattements inférieurs"}},
      "category": "Sm"
    }, {
      "key": "29CE",
      "mappings": {"default": {"default": "triangle droit par dessus triangle gauche"}},
      "category": "Sm"
    }, {
      "key": "29CF",
      "mappings": {"default": {"default": "triangle gauche à gauche d'une barre verticale"}},
      "category": "Sm"
    }, {
      "key": "29D0",
      "mappings": {"default": {"default": "triangle droit à droite d'une barre verticale"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29D1",
      "mappings": {"default": {"default": "nœud papillon à aile gauche noire"}}
    }, {
      "category": "Sm",
      "key": "29D2",
      "mappings": {"default": {"default": "nœud papillon à aile droite noire"}}
    }, {"category": "Sm", "key": "29D3", "mappings": {"default": {"default": "nœud papillon noir"}}}, {
      "category": "Sm",
      "key": "29D4",
      "mappings": {"default": {"default": "multiplication à moitié gauche noircie"}}
    }, {
      "category": "Sm",
      "key": "29D5",
      "mappings": {"default": {"default": "multiplication à moitié droite noircie"}}
    }, {"category": "Sm", "key": "29D6", "mappings": {"default": {"default": "sablier blanc"}}}, {
      "category": "Sm",
      "key": "29D7",
      "mappings": {"default": {"default": "sablier noir"}}
    }, {"key": "29DC", "mappings": {"default": {"default": "infini incomplet"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "29DD",
      "mappings": {"default": {"default": "lorgnon"}}
    }, {
      "key": "29DE",
      "mappings": {"default": {"default": "infini nié par une barre verticale"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29DF",
      "mappings": {"default": {"default": "multijection à deux têtes"}}
    }, {"category": "Sm", "key": "29E0", "mappings": {"default": {"default": "carré ombré"}}}, {
      "category": "Sm",
      "key": "29E1",
      "mappings": {"default": {"default": "augmente avec"}}
    }, {
      "category": "Sm",
      "key": "29E2",
      "mappings": {"default": {"default": "produit de permutation"}}
    }, {
      "key": "29E3",
      "mappings": {"default": {"default": "égale à et parallèle à incliné"}},
      "category": "Sm"
    }, {
      "key": "29E4",
      "mappings": {"default": {"default": "égale à et parallèle à incliné surmontés d'un tilde"}},
      "category": "Sm"
    }, {
      "key": "29E5",
      "mappings": {"default": {"default": "identique à et parallèle à incliné"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "29E6", "mappings": {"default": {"default": "gleich stark"}}}, {
      "category": "Sm",
      "key": "29E7",
      "mappings": {"default": {"default": "thermodynamique"}}
    }, {
      "category": "Sm",
      "key": "29E8",
      "mappings": {"default": {"default": "triangle pointe vers le bas à moitié gauche noire"}}
    }, {
      "category": "Sm",
      "key": "29E9",
      "mappings": {"default": {"default": "triangle pointe vers le bas à moitié droite noire"}}
    }, {
      "key": "29EB",
      "mappings": {"default": {"default": "losange fuselé noir"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29EE",
      "mappings": {"default": {"default": "barre d'erreur à carré blanc"}}
    }, {
      "category": "Sm",
      "key": "29EF",
      "mappings": {"default": {"default": "barre d'erreur à carré noir"}}
    }, {
      "category": "Sm",
      "key": "29F0",
      "mappings": {"default": {"default": "barre d'erreur à losange blanc"}}
    }, {
      "category": "Sm",
      "key": "29F1",
      "mappings": {"default": {"default": "barre d'erreur à losange noir"}}
    }, {
      "category": "Sm",
      "key": "29F2",
      "mappings": {"default": {"default": "barre d'erreur à cercle blanc"}}
    }, {
      "category": "Sm",
      "key": "29F3",
      "mappings": {"default": {"default": "barre d'erreur à cercle noir"}}
    }, {"key": "29F4", "mappings": {"default": {"default": "règle différée"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "29F5",
      "mappings": {"default": {"default": "opérateur barre oblique inversée"}}
    }, {
      "key": "29F6",
      "mappings": {"default": {"default": "barre oblique avec barre en chef"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "29F7",
      "mappings": {"default": {"default": "barre oblique inversée à traverse horizontale"}}
    }, {
      "category": "Sm",
      "key": "29F8",
      "mappings": {"default": {"default": "grande barre oblique"}}
    }, {
      "category": "Sm",
      "key": "29F9",
      "mappings": {"default": {"default": "grande barre oblique inversée"}}
    }, {"category": "Sm", "key": "29FA", "mappings": {"default": {"default": "double plus"}}}, {
      "category": "Sm",
      "key": "29FB",
      "mappings": {"default": {"default": "triple plus"}}
    }, {"category": "Sm", "key": "29FE", "mappings": {"default": {"default": "tiny"}}}, {
      "category": "Sm",
      "key": "29FF",
      "mappings": {"default": {"default": "miny"}}
    }, {
      "category": "Sm",
      "key": "2A00",
      "mappings": {"default": {"default": "opérateur point cerclé n aire"}}
    }, {
      "category": "Sm",
      "key": "2A01",
      "mappings": {"default": {"default": "opérateur plus cerclé n aire"}}
    }, {
      "category": "Sm",
      "key": "2A02",
      "mappings": {"default": {"default": "opérateur multiplié par cerclé n aire"}}
    }, {
      "category": "Sm",
      "key": "2A03",
      "mappings": {"default": {"default": "opérateur union n aire pointé"}}
    }, {
      "category": "Sm",
      "key": "2A04",
      "mappings": {"default": {"default": "opérateur union n aire avec plus"}}
    }, {
      "category": "Sm",
      "key": "2A05",
      "mappings": {"default": {"default": "opérateur intersection carré n aire"}}
    }, {
      "category": "Sm",
      "key": "2A06",
      "mappings": {"default": {"default": "opérateur union carré n aire"}}
    }, {
      "category": "Sm",
      "key": "2A07",
      "mappings": {"default": {"default": "opérateur deux et logiques"}}
    }, {
      "category": "Sm",
      "key": "2A08",
      "mappings": {"default": {"default": "opérateur deux ou logiques"}}
    }, {
      "category": "Sm",
      "key": "2A09",
      "mappings": {"default": {"default": "opérateur multiplié par n aire"}}
    }, {
      "category": "Sm",
      "key": "2A0A",
      "mappings": {"default": {"default": "sommation modulo deux"}}
    }, {
      "category": "Sm",
      "key": "2A0B",
      "mappings": {"default": {"default": "sommation avec intégrale"}}
    }, {"key": "2A0C", "mappings": {"default": {"default": "intégrale quadruple"}}, "category": "Sm"}, {
      "key": "2A0D",
      "mappings": {"default": {"default": "intégrale à partie finie"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A0E",
      "mappings": {"default": {"default": "intégrale à deux barres horizontales"}}
    }, {
      "category": "Sm",
      "key": "2A0F",
      "mappings": {"default": {"default": "moyenne d'intégrale à barre oblique inversée"}}
    }, {
      "key": "2A10",
      "mappings": {"default": {"default": "fonction de circulation"}},
      "category": "Sm"
    }, {
      "key": "2A11",
      "mappings": {"default": {"default": "intégrale en sens positif"}},
      "category": "Sm"
    }, {
      "key": "2A12",
      "mappings": {"default": {"default": "intégrale de ligne à chemin rectangulaire autour du pôle"}},
      "category": "Sm"
    }, {
      "key": "2A13",
      "mappings": {"default": {"default": "intégrale de ligne à chemin semi circulaire autour du pôle"}},
      "category": "Sm"
    }, {
      "key": "2A14",
      "mappings": {"default": {"default": "intégrale de ligne évitant le pôle"}},
      "category": "Sm"
    }, {
      "key": "2A15",
      "mappings": {"default": {"default": "intégrale autour d'un opérateur point"}},
      "category": "Sm"
    }, {
      "key": "2A16",
      "mappings": {"default": {"default": "intégrale quaternion"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A18",
      "mappings": {"default": {"default": "intégrale au signe multiplié par"}}
    }, {
      "category": "Sm",
      "key": "2A19",
      "mappings": {"default": {"default": "intégrale à signe intersection"}}
    }, {
      "category": "Sm",
      "key": "2A1A",
      "mappings": {"default": {"default": "intégrale à signe union"}}
    }, {
      "category": "Sm",
      "key": "2A1B",
      "mappings": {"default": {"default": "intégrale à barre en chef"}}
    }, {
      "category": "Sm",
      "key": "2A1C",
      "mappings": {"default": {"default": "intégrale à barre souscrite"}}
    }, {"category": "Sm", "key": "2A1D", "mappings": {"default": {"default": "jointure"}}}, {
      "category": "Sm",
      "key": "2A1E",
      "mappings": {"default": {"default": "gros opérateur triangle gauche"}}
    }, {
      "category": "Sm",
      "key": "2A1F",
      "mappings": {"default": {"default": "composition de schémas en notation z"}}
    }, {
      "category": "Sm",
      "key": "2A20",
      "mappings": {"default": {"default": "tubage de schémas en notation z"}}
    }, {
      "category": "Sm",
      "key": "2A21",
      "mappings": {"default": {"default": "projection de schémas en notation z"}}
    }, {
      "key": "2A22",
      "mappings": {"default": {"default": "plus surmonté d'un petit cercle"}},
      "category": "Sm"
    }, {
      "key": "2A23",
      "mappings": {"default": {"default": "plus surmonté d'un circonflexe"}},
      "category": "Sm"
    }, {
      "key": "2A24",
      "mappings": {"default": {"default": "plus surmonté d'un tilde"}},
      "category": "Sm"
    }, {"key": "2A25", "mappings": {"default": {"default": "plus à point souscrit"}}, "category": "Sm"}, {
      "key": "2A26",
      "mappings": {"default": {"default": "plus à tilde souscrit"}},
      "category": "Sm"
    }, {
      "key": "2A27",
      "mappings": {"default": {"default": "plus à chiffre deux souscrit"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "2A28", "mappings": {"default": {"default": "plus à triangle noir"}}}, {
      "key": "2A29",
      "mappings": {"default": {"default": "moins surmonté d'une virgule"}},
      "category": "Sm"
    }, {
      "key": "2A2A",
      "mappings": {"default": {"default": "moins à point souscrit"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A2B",
      "mappings": {"default": {"default": "moins à points descendants"}}
    }, {
      "category": "Sm",
      "key": "2A2C",
      "mappings": {"default": {"default": "moins à points ascendants"}}
    }, {
      "key": "2A2D",
      "mappings": {"default": {"default": "plus dans demi cercle gauche"}},
      "category": "Sm"
    }, {
      "key": "2A2E",
      "mappings": {"default": {"default": "plus dans demi cercle droit"}},
      "category": "Sm"
    }, {"key": "2A2F", "mappings": {"default": {"default": "produit vectoriel"}}, "category": "Sm"}, {
      "key": "2A30",
      "mappings": {"default": {"default": "multiplication à point en chef"}},
      "category": "Sm"
    }, {
      "key": "2A31",
      "mappings": {"default": {"default": "multiplication à barre souscrite"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A32",
      "mappings": {"default": {"default": "produit semi direct fermé en bas"}}
    }, {"key": "2A33", "mappings": {"default": {"default": "produit fusionné"}}, "category": "Sm"}, {
      "key": "2A34",
      "mappings": {"default": {"default": "multiplication dans demi cercle gauche"}},
      "category": "Sm"
    }, {
      "key": "2A35",
      "mappings": {"default": {"default": "multiplication dans demi cercle droit"}},
      "category": "Sm"
    }, {
      "key": "2A36",
      "mappings": {"default": {"default": "multiplication cerclé circonflexe"}},
      "category": "Sm"
    }, {
      "key": "2A37",
      "mappings": {"default": {"default": "multiplication doublement cerclé"}},
      "category": "Sm"
    }, {"key": "2A38", "mappings": {"default": {"default": "division cerclé"}}, "category": "Sm"}, {
      "key": "2A39",
      "mappings": {"default": {"default": "plus dans triangle"}},
      "category": "Sm"
    }, {"key": "2A3A", "mappings": {"default": {"default": "moins dans triangle"}}, "category": "Sm"}, {
      "key": "2A3B",
      "mappings": {"default": {"default": "multiplication dans triangle"}},
      "category": "Sm"
    }, {"key": "2A3C", "mappings": {"default": {"default": "produit interieur"}}, "category": "Sm"}, {
      "category": "Sm",
      "key": "2A3D",
      "mappings": {"default": {"default": "produit intérieur à droite"}}
    }, {
      "category": "Sm",
      "key": "2A3E",
      "mappings": {"default": {"default": "composition relationnelle en notation z"}}
    }, {
      "key": "2A3F",
      "mappings": {"default": {"default": "amalgamation ou coproduit"}},
      "category": "Sm"
    }, {
      "key": "2A40",
      "mappings": {"default": {"default": "intersection pointée"}},
      "category": "Sm"
    }, {"category": "Sm", "key": "2A41", "mappings": {"default": {"default": "union à signe moins"}}}, {
      "key": "2A42",
      "mappings": {"default": {"default": "union à barre en chef"}},
      "category": "Sm"
    }, {
      "key": "2A43",
      "mappings": {"default": {"default": "intersection à barre en chef"}},
      "category": "Sm"
    }, {
      "key": "2A44",
      "mappings": {"default": {"default": "intersection avec et logique"}},
      "category": "Sm"
    }, {"key": "2A45", "mappings": {"default": {"default": "union avec ou logique"}}, "category": "Sm"}, {
      "key": "2A46",
      "mappings": {"default": {"default": "union par dessus intersection"}},
      "category": "Sm"
    }, {
      "key": "2A47",
      "mappings": {"default": {"default": "intersection par dessus union"}},
      "category": "Sm"
    }, {
      "key": "2A48",
      "mappings": {"default": {"default": "union par dessus bar par dessus intersection"}},
      "category": "Sm"
    }, {
      "key": "2A49",
      "mappings": {"default": {"default": "intersection par dessus trait par dessus union"}},
      "category": "Sm"
    }, {
      "key": "2A4A",
      "mappings": {"default": {"default": "deux unions soudées côte à côte"}},
      "category": "Sm"
    }, {
      "key": "2A4B",
      "mappings": {"default": {"default": "deux intersections soudées côte à côte"}},
      "category": "Sm"
    }, {
      "key": "2A4C",
      "mappings": {"default": {"default": "union fermée à empattements"}},
      "category": "Sm"
    }, {
      "key": "2A4D",
      "mappings": {"default": {"default": "intersection fermée à empattements"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A4E",
      "mappings": {"default": {"default": "deux intersections carrées emboîtées"}}
    }, {
      "category": "Sm",
      "key": "2A4F",
      "mappings": {"default": {"default": "deux unions carrées emboîtées"}}
    }, {
      "key": "2A50",
      "mappings": {"default": {"default": "union fermée à empattements et produit fusionné"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A51",
      "mappings": {"default": {"default": "et logique à point en chef"}}
    }, {
      "category": "Sm",
      "key": "2A52",
      "mappings": {"default": {"default": "ou logique à point en chef"}}
    }, {"key": "2A53", "mappings": {"default": {"default": "double et logique"}}, "category": "Sm"}, {
      "key": "2A54",
      "mappings": {"default": {"default": "double ou logique"}},
      "category": "Sm"
    }, {
      "key": "2A55",
      "mappings": {"default": {"default": "deux et logiques entrecroisés"}},
      "category": "Sm"
    }, {
      "key": "2A56",
      "mappings": {"default": {"default": "deux ou logiques entrecroisés"}},
      "category": "Sm"
    }, {"key": "2A57", "mappings": {"default": {"default": "grand ou pentu"}}, "category": "Sm"}, {
      "key": "2A58",
      "mappings": {"default": {"default": "grand et pentu"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A59",
      "mappings": {"default": {"default": "et et ou logiques entrecroisés"}}
    }, {
      "key": "2A5A",
      "mappings": {"default": {"default": "et logique à hampe médiane"}},
      "category": "Sm"
    }, {
      "key": "2A5B",
      "mappings": {"default": {"default": "ou logique à hampe médiane"}},
      "category": "Sm"
    }, {
      "key": "2A5C",
      "mappings": {"default": {"default": "et logique à tiret horizontal"}},
      "category": "Sm"
    }, {
      "key": "2A5D",
      "mappings": {"default": {"default": "ou logique à tiret horizontal"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A5E",
      "mappings": {"default": {"default": "et logique à deux barres en chef"}}
    }, {
      "key": "2A5F",
      "mappings": {"default": {"default": "et logique à barre souscrite"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A60",
      "mappings": {"default": {"default": "et logique à deux barres souscrites"}}
    }, {
      "category": "Sm",
      "key": "2A61",
      "mappings": {"default": {"default": "petit v à barre souscrite"}}
    }, {
      "category": "Sm",
      "key": "2A62",
      "mappings": {"default": {"default": "ou logique à deux barres en chef"}}
    }, {
      "category": "Sm",
      "key": "2A63",
      "mappings": {"default": {"default": "ou logique à deux barres souscrites"}}
    }, {
      "category": "Sm",
      "key": "2A64",
      "mappings": {"default": {"default": "antirestriction de domaine en notation z"}}
    }, {
      "category": "Sm",
      "key": "2A65",
      "mappings": {"default": {"default": "antirestriction de codomaine en notation z"}}
    }, {
      "key": "2A66",
      "mappings": {"default": {"default": "égal point souscrit"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A67",
      "mappings": {"default": {"default": "identique à surmonté d'un point"}}
    }, {
      "category": "Sm",
      "key": "2A68",
      "mappings": {"default": {"default": "deux barres verticales à trois traverses"}}
    }, {
      "category": "Sm",
      "key": "2A69",
      "mappings": {"default": {"default": "trois barres verticales à trois traverses"}}
    }, {
      "key": "2A6A",
      "mappings": {"default": {"default": "opérateur tilde point en chef"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A6B",
      "mappings": {"default": {"default": "opérateur tilde à points ascendants"}}
    }, {
      "category": "Sm",
      "key": "2A6C",
      "mappings": {"default": {"default": "similaire à moins similaire à"}}
    }, {
      "key": "2A6D",
      "mappings": {"default": {"default": "congruent point en chef"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A6E",
      "mappings": {"default": {"default": "Égal astérisque en chef"}}
    }, {
      "key": "2A6F",
      "mappings": {"default": {"default": "presque égal à circonflexe"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A70",
      "mappings": {"default": {"default": "approximativement égal à ou égal à"}}
    }, {
      "key": "2A71",
      "mappings": {"default": {"default": "plus signe égal en chef"}},
      "category": "Sm"
    }, {
      "key": "2A72",
      "mappings": {"default": {"default": "égal à signe plus en chef"}},
      "category": "Sm"
    }, {
      "key": "2A73",
      "mappings": {"default": {"default": "tilde signe égal en chef"}},
      "category": "Sm"
    }, {
      "key": "2A74",
      "mappings": {"default": {"default": "double deux points égal à"}},
      "category": "Sm"
    }, {
      "key": "2A75",
      "mappings": {"default": {"default": "deux signes égal à consécutifs"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A76",
      "mappings": {"default": {"default": "trois signes égal à consécutifs"}}
    }, {
      "key": "2A77",
      "mappings": {"default": {"default": "égal à deux points en chef deux points souscrits"}},
      "category": "Sm"
    }, {
      "key": "2A78",
      "mappings": {"default": {"default": "équivalent quatre points en chef"}},
      "category": "Sm"
    }, {
      "key": "2A79",
      "mappings": {"default": {"default": "inférieur à avec cercle inscrit"}},
      "category": "Sm"
    }, {
      "key": "2A7A",
      "mappings": {"default": {"default": "supérieur à avec cercle inscrit"}},
      "category": "Sm"
    }, {
      "key": "2A7B",
      "mappings": {"default": {"default": "inférieur à point d'interrogation en chef"}},
      "category": "Sm"
    }, {
      "key": "2A7C",
      "mappings": {"default": {"default": "supérieur à point d'interrogation en chef"}},
      "category": "Sm"
    }, {
      "key": "2A7D",
      "mappings": {"default": {"default": "plus petit ou égal à pentu"}},
      "category": "Sm"
    }, {
      "key": "2A7E",
      "mappings": {"default": {"default": "supérieur à ou égal à pentu"}},
      "category": "Sm"
    }, {
      "key": "2A7F",
      "mappings": {"default": {"default": "inférieur à ou égal à pentu point inscrit"}},
      "category": "Sm"
    }, {
      "key": "2A80",
      "mappings": {"default": {"default": "supérieur à ou égal à pentu point inscrit"}},
      "category": "Sm"
    }, {
      "key": "2A81",
      "mappings": {"default": {"default": "inférieur à ou égal à pentu point en chef"}},
      "category": "Sm"
    }, {
      "key": "2A82",
      "mappings": {"default": {"default": "supérieur à ou égal à pentu point en chef"}},
      "category": "Sm"
    }, {
      "key": "2A83",
      "mappings": {"default": {"default": "inférieur à ou égal à pentu point en chef droit"}},
      "category": "Sm"
    }, {
      "key": "2A84",
      "mappings": {"default": {"default": "supérieur à ou égal à pentu point en chef gauche"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A85",
      "mappings": {"default": {"default": "inférieur à ou approximatif"}}
    }, {
      "category": "Sm",
      "key": "2A86",
      "mappings": {"default": {"default": "supérieur à ou approximatif"}}
    }, {
      "category": "Sm",
      "key": "2A87",
      "mappings": {"default": {"default": "inférieur à et pas égal à une ligne"}}
    }, {
      "category": "Sm",
      "key": "2A88",
      "mappings": {"default": {"default": "supérieur à et pas égal à une ligne"}}
    }, {
      "key": "2A89",
      "mappings": {"default": {"default": "inférieur à et non approximatif"}},
      "category": "Sm"
    }, {
      "key": "2A8A",
      "mappings": {"default": {"default": "supérieur à et non approximatif"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A8B",
      "mappings": {"default": {"default": "inférieur à par dessus égal à deux lignes par dessus supérieur à"}}
    }, {
      "category": "Sm",
      "key": "2A8C",
      "mappings": {"default": {"default": "supérieur à par dessus égal à deux lignes par dessus inférieur à"}}
    }, {
      "key": "2A8D",
      "mappings": {"default": {"default": "inférieur à par dessus similaire à ou égal à"}},
      "category": "Sm"
    }, {
      "key": "2A8E",
      "mappings": {"default": {"default": "supérieur à par dessus similaire à ou égal à"}},
      "category": "Sm"
    }, {
      "key": "2A8F",
      "mappings": {"default": {"default": "inférieur à par dessus similaire à par dessus supérieur à"}},
      "category": "Sm"
    }, {
      "key": "2A90",
      "mappings": {"default": {"default": "supérieur à par dessus similaire à par dessus inférieur à"}},
      "category": "Sm"
    }, {
      "key": "2A91",
      "mappings": {"default": {"default": "inférieur à par dessus supérieur à par dessus égal à deux lignes"}},
      "category": "Sm"
    }, {
      "key": "2A92",
      "mappings": {"default": {"default": "supérieur à par dessus inférieur à par dessus égal à deux lignes"}},
      "category": "Sm"
    }, {
      "key": "2A93",
      "mappings": {"default": {"default": "inférieur à par dessus égal à pentu par dessus supérieur à par dessus égal à pentu"}},
      "category": "Sm"
    }, {
      "key": "2A94",
      "mappings": {"default": {"default": "supérieur à par dessus égal à pentu par dessus inférieur à par dessus égal à pentu"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A95",
      "mappings": {"default": {"default": "Égal à incliné ou inférieur à"}}
    }, {
      "category": "Sm",
      "key": "2A96",
      "mappings": {"default": {"default": "Égal à incliné ou supérieur à"}}
    }, {
      "key": "2A97",
      "mappings": {"default": {"default": "égal à pentu ou inférieur à point inscrit"}},
      "category": "Sm"
    }, {
      "key": "2A98",
      "mappings": {"default": {"default": "égal à pentu ou supérieur à point inscrit"}},
      "category": "Sm"
    }, {
      "key": "2A99",
      "mappings": {"default": {"default": "égal à deux lignes ou inférieur à"}},
      "category": "Sm"
    }, {
      "key": "2A9A",
      "mappings": {"default": {"default": "égal à deux lignes ou supérieur à"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2A9B",
      "mappings": {"default": {"default": "inférieur à ou égal à à deux lignes inclinées"}}
    }, {
      "category": "Sm",
      "key": "2A9C",
      "mappings": {"default": {"default": "supérieur à ou égal à à deux lignes inclinées"}}
    }, {
      "key": "2A9D",
      "mappings": {"default": {"default": "similaire à ou inférieur à"}},
      "category": "Sm"
    }, {
      "key": "2A9E",
      "mappings": {"default": {"default": "similaire à ou supérieur à"}},
      "category": "Sm"
    }, {
      "key": "2A9F",
      "mappings": {"default": {"default": "similaire à par dessus inférieur à par dessus signe égal"}},
      "category": "Sm"
    }, {
      "key": "2AA0",
      "mappings": {"default": {"default": "similaire à par dessus supérieur à par dessus signe égal"}},
      "category": "Sm"
    }, {
      "key": "2AA1",
      "mappings": {"default": {"default": "deux inférieur à emboîtés"}},
      "category": "Sm"
    }, {
      "key": "2AA2",
      "mappings": {"default": {"default": "deux supérieur à emboîtés"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AA3",
      "mappings": {"default": {"default": "deux inférieur à emboîtés avec barre souscrite"}}
    }, {
      "key": "2AA4",
      "mappings": {"default": {"default": "deux supérieur à entrecroisés"}},
      "category": "Sm"
    }, {
      "key": "2AA5",
      "mappings": {"default": {"default": "supérieur à à côté de inférieur à"}},
      "category": "Sm"
    }, {
      "key": "2AA6",
      "mappings": {"default": {"default": "inférieur à fermé d'un arc"}},
      "category": "Sm"
    }, {
      "key": "2AA7",
      "mappings": {"default": {"default": "supérieur à fermé d'un arc"}},
      "category": "Sm"
    }, {
      "key": "2AA8",
      "mappings": {"default": {"default": "inférieur à fermé d'un arc par dessus égal à pentu"}},
      "category": "Sm"
    }, {
      "key": "2AA9",
      "mappings": {"default": {"default": "supérieur à fermé d'un arc par dessus égal à pentu"}},
      "category": "Sm"
    }, {
      "key": "2AAA",
      "mappings": {"default": {"default": "plus petit que en fourche"}},
      "category": "Sm"
    }, {
      "key": "2AAB",
      "mappings": {"default": {"default": "plus grand que en fourche"}},
      "category": "Sm"
    }, {
      "key": "2AAC",
      "mappings": {"default": {"default": "plus petit ou égal que en fourche"}},
      "category": "Sm"
    }, {
      "key": "2AAD",
      "mappings": {"default": {"default": "plus grand ou égal que en fourche"}},
      "category": "Sm"
    }, {"key": "2AAE", "mappings": {"default": {"default": "égal et dos d'âne"}}, "category": "Sm"}, {
      "key": "2AAF",
      "mappings": {"default": {"default": "précède par dessus signe égal à une ligne"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AB0",
      "mappings": {"default": {"default": "suit par dessus signe égal à une ligne"}}
    }, {
      "category": "Sm",
      "key": "2AB1",
      "mappings": {"default": {"default": "précède par dessus pas égal à une ligne"}}
    }, {
      "category": "Sm",
      "key": "2AB2",
      "mappings": {"default": {"default": "suit par dessus pas égal à une ligne"}}
    }, {
      "category": "Sm",
      "key": "2AB3",
      "mappings": {"default": {"default": "précède par dessus signe égal"}}
    }, {
      "category": "Sm",
      "key": "2AB4",
      "mappings": {"default": {"default": "suit par dessus signe égal"}}
    }, {
      "key": "2AB5",
      "mappings": {"default": {"default": "précède par dessus pas égal à"}},
      "category": "Sm"
    }, {
      "key": "2AB6",
      "mappings": {"default": {"default": "suit par dessus pas égal à"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AB7",
      "mappings": {"default": {"default": "précède par dessus presque égal à"}}
    }, {
      "category": "Sm",
      "key": "2AB8",
      "mappings": {"default": {"default": "suit par dessus presque égal à"}}
    }, {
      "category": "Sm",
      "key": "2AB9",
      "mappings": {"default": {"default": "précède par dessus non presque égal à"}}
    }, {
      "category": "Sm",
      "key": "2ABA",
      "mappings": {"default": {"default": "suit par dessus non presque égal à"}}
    }, {"key": "2ABB", "mappings": {"default": {"default": "double précède"}}, "category": "Sm"}, {
      "key": "2ABC",
      "mappings": {"default": {"default": "double suit"}},
      "category": "Sm"
    }, {"key": "2ABD", "mappings": {"default": {"default": "sous ensemble pointé"}}, "category": "Sm"}, {
      "key": "2ABE",
      "mappings": {"default": {"default": "sur ensemble pointé"}},
      "category": "Sm"
    }, {
      "key": "2ABF",
      "mappings": {"default": {"default": "sous ensemble plus souscrit"}},
      "category": "Sm"
    }, {
      "key": "2AC0",
      "mappings": {"default": {"default": "sur ensemble plus souscrit"}},
      "category": "Sm"
    }, {
      "key": "2AC1",
      "mappings": {"default": {"default": "sous ensemble à signe de multiplication souscrit"}},
      "category": "Sm"
    }, {
      "key": "2AC2",
      "mappings": {"default": {"default": "sur ensemble à signe de multiplication souscrit"}},
      "category": "Sm"
    }, {
      "key": "2AC3",
      "mappings": {"default": {"default": "sous ensemble de ou égal à point en chef"}},
      "category": "Sm"
    }, {
      "key": "2AC4",
      "mappings": {"default": {"default": "sur ensemble de ou égal à point en chef"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AC5",
      "mappings": {"default": {"default": "sous ensemble de par dessus signe égal"}}
    }, {
      "category": "Sm",
      "key": "2AC6",
      "mappings": {"default": {"default": "sur ensemble de par dessus signe égal"}}
    }, {
      "key": "2AC7",
      "mappings": {"default": {"default": "sous ensemble de par dessus opérateur tilde"}},
      "category": "Sm"
    }, {
      "key": "2AC8",
      "mappings": {"default": {"default": "sur ensemble de par dessus opérateur tilde"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AC9",
      "mappings": {"default": {"default": "sous ensemble de par dessus presque égal à"}}
    }, {
      "category": "Sm",
      "key": "2ACA",
      "mappings": {"default": {"default": "sur ensemble de par dessus presque égal à"}}
    }, {
      "category": "Sm",
      "key": "2ACB",
      "mappings": {"default": {"default": "sous ensemble de par dessus pas égal à"}}
    }, {
      "category": "Sm",
      "key": "2ACC",
      "mappings": {"default": {"default": "sur ensemble de par dessus pas égal à"}}
    }, {
      "category": "Sm",
      "key": "2ACD",
      "mappings": {"default": {"default": "opérateur boîte carrée ouverte gauche"}}
    }, {
      "category": "Sm",
      "key": "2ACE",
      "mappings": {"default": {"default": "opérateur boîte carrée ouverte droite"}}
    }, {"key": "2ACF", "mappings": {"default": {"default": "sous ensemble clos"}}, "category": "Sm"}, {
      "key": "2AD0",
      "mappings": {"default": {"default": "sur ensemble clos"}},
      "category": "Sm"
    }, {
      "key": "2AD1",
      "mappings": {"default": {"default": "sous ensemble clos ou égal à"}},
      "category": "Sm"
    }, {
      "key": "2AD2",
      "mappings": {"default": {"default": "sur ensemble clos ou égal à"}},
      "category": "Sm"
    }, {
      "key": "2AD3",
      "mappings": {"default": {"default": "sous ensemble par dessus sur ensemble"}},
      "category": "Sm"
    }, {
      "key": "2AD4",
      "mappings": {"default": {"default": "sur ensemble par dessus sous ensemble"}},
      "category": "Sm"
    }, {
      "key": "2AD5",
      "mappings": {"default": {"default": "sous ensemble par dessus sous ensemble"}},
      "category": "Sm"
    }, {
      "key": "2AD6",
      "mappings": {"default": {"default": "sur ensemble par dessus sur ensemble"}},
      "category": "Sm"
    }, {
      "key": "2AD7",
      "mappings": {"default": {"default": "sur ensemble à côté de sous ensemble"}},
      "category": "Sm"
    }, {
      "key": "2AD8",
      "mappings": {"default": {"default": "sur ensemble chaîné à sous ensemble"}},
      "category": "Sm"
    }, {
      "key": "2AD9",
      "mappings": {"default": {"default": "appartient à ouvert vers le bas"}},
      "category": "Sm"
    }, {
      "key": "2ADA",
      "mappings": {"default": {"default": "fourche surmontée d'un t"}},
      "category": "Sm"
    }, {
      "key": "2ADB",
      "mappings": {"default": {"default": "intersection transversale"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2ADC",
      "mappings": {"default": {"default": "bifurcation (non indépendant)"}}
    }, {
      "category": "Sm",
      "key": "2ADD",
      "mappings": {"default": {"default": "non bifurcation (indépendant)"}}
    }, {
      "category": "Sm",
      "key": "2ADE",
      "mappings": {"default": {"default": "taquet gauche court"}}
    }, {
      "category": "Sm",
      "key": "2ADF",
      "mappings": {"default": {"default": "taquet court vers le bas"}}
    }, {
      "category": "Sm",
      "key": "2AE0",
      "mappings": {"default": {"default": "taquet court vers le haut"}}
    }, {
      "category": "Sm",
      "key": "2AE1",
      "mappings": {"default": {"default": "perpendiculaire à s"}}
    }, {
      "category": "Sm",
      "key": "2AE2",
      "mappings": {"default": {"default": "triple tourniquet droit à barre verticale"}}
    }, {
      "category": "Sm",
      "key": "2AE3",
      "mappings": {"default": {"default": "tourniquet gauche à double barre verticale"}}
    }, {
      "key": "2AE4",
      "mappings": {"default": {"default": "double tourniquet gauche à barre verticale"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AE5",
      "mappings": {"default": {"default": "double tourniquet gauche à double barre verticale"}}
    }, {
      "key": "2AE6",
      "mappings": {"default": {"default": "double barre verticale gauche à traverse"}},
      "category": "Sm"
    }, {
      "key": "2AE7",
      "mappings": {"default": {"default": "taquet court vers le bas avec barre en chef"}},
      "category": "Sm"
    }, {
      "key": "2AE8",
      "mappings": {"default": {"default": "taquet court vers le haut avec barre souscrite"}},
      "category": "Sm"
    }, {
      "key": "2AE9",
      "mappings": {"default": {"default": "taquet court vers le haut par dessus taquet court vers le bas"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AEA",
      "mappings": {"default": {"default": "double taquet vers le bas"}}
    }, {
      "key": "2AEB",
      "mappings": {"default": {"default": "double taquet vers le haut"}},
      "category": "Sm"
    }, {
      "key": "2AEC",
      "mappings": {"default": {"default": "négation à deux traits"}},
      "category": "Sm"
    }, {
      "key": "2AED",
      "mappings": {"default": {"default": "négation à deux traits réfléchi"}},
      "category": "Sm"
    }, {
      "key": "2AEE",
      "mappings": {"default": {"default": "n'est pas un diviseur de à barre de négation réfléchie"}},
      "category": "Sm"
    }, {
      "key": "2AEF",
      "mappings": {"default": {"default": "ligne verticale cercle en chef"}},
      "category": "Sm"
    }, {
      "key": "2AF0",
      "mappings": {"default": {"default": "ligne verticale cercle souscrit"}},
      "category": "Sm"
    }, {
      "key": "2AF1",
      "mappings": {"default": {"default": "taquet vers le bas surmontant un cercle"}},
      "category": "Sm"
    }, {
      "key": "2AF2",
      "mappings": {"default": {"default": "parallèlle à vertical à une traverse"}},
      "category": "Sm"
    }, {
      "key": "2AF3",
      "mappings": {"default": {"default": "parallèle à avec opérateur tilde"}},
      "category": "Sm"
    }, {
      "category": "Sm",
      "key": "2AF4",
      "mappings": {"default": {"default": "relation binaire triple barre verticale"}}
    }, {
      "category": "Sm",
      "key": "2AF5",
      "mappings": {"default": {"default": "triple barre verticale à une traverse"}}
    }, {
      "category": "Sm",
      "key": "2AF6",
      "mappings": {"default": {"default": "opérateur trois points"}}
    }, {
      "category": "Sm",
      "key": "2AF7",
      "mappings": {"default": {"default": "trois inférieur à emboîtés"}}
    }, {
      "category": "Sm",
      "key": "2AF8",
      "mappings": {"default": {"default": "trois supérieur à emboîtés"}}
    }, {
      "category": "Sm",
      "key": "2AF9",
      "mappings": {"default": {"default": "inférieur à ou égal à et deux lignes inclinées"}}
    }, {
      "category": "Sm",
      "key": "2AFA",
      "mappings": {"default": {"default": "supérieur à ou égal à et deux lignes inclinées"}}
    }, {
      "category": "Sm",
      "key": "2AFB",
      "mappings": {"default": {"default": "relation binaire triple oblique"}}
    }, {
      "category": "Sm",
      "key": "2AFC",
      "mappings": {"default": {"default": "grand opérateur à trois barres verticales"}}
    }, {
      "category": "Sm",
      "key": "2AFD",
      "mappings": {"default": {"default": "opérateur double oblique"}}
    }, {
      "category": "Sm",
      "key": "2AFE",
      "mappings": {"default": {"default": "barre verticale blanche"}}
    }, {
      "category": "Sm",
      "key": "2AFF",
      "mappings": {"default": {"default": "barre verticale blanche n aire"}}
    }, {
      "category": "Pd",
      "key": "301C",
      "mappings": {"default": {"default": "trait d'union en esse"}}
    }, {
      "category": "Po",
      "key": "FE10",
      "mappings": {"default": {"default": "forme de présentation de virgule verticale"}}
    }, {
      "category": "Po",
      "key": "FE13",
      "mappings": {"default": {"default": "forme de présentation de deux points vertical"}}
    }, {
      "category": "Po",
      "key": "FE14",
      "mappings": {"default": {"default": "forme de présentation de point virgule vertical"}}
    }, {
      "category": "Po",
      "key": "FE15",
      "mappings": {"default": {"default": "forme de présentation de point d'exclamation vertical"}}
    }, {
      "category": "Po",
      "key": "FE16",
      "mappings": {"default": {"default": "forme de présentation de point d'interrogation vertical"}}
    }, {
      "category": "Po",
      "key": "FE19",
      "mappings": {"default": {"default": "forme de présentation de points de suspension verticaux"}}
    }, {
      "category": "Po",
      "key": "FE30",
      "mappings": {"default": {"default": "forme de présentation de point de conduite double vertical"}}
    }, {
      "category": "Pd",
      "key": "FE31",
      "mappings": {"default": {"default": "forme de présentation de tiret cadratin vertical"}}
    }, {
      "category": "Pd",
      "key": "FE32",
      "mappings": {"default": {"default": "forme de présentation de tiret demi cadratin vertical"}}
    }, {
      "category": "Pc",
      "key": "FE33",
      "mappings": {"default": {"default": "forme de présentation de tiret bas vertical"}}
    }, {
      "category": "Pc",
      "key": "FE34",
      "mappings": {"default": {"default": "forme de présentation de tiret bas ondulé vertical"}}
    }, {"category": "Po", "key": "FE45", "mappings": {"default": {"default": "point sésame"}}}, {
      "category": "Po",
      "key": "FE46",
      "mappings": {"default": {"default": "point sésame ajouré"}}
    }, {
      "category": "Po",
      "key": "FE49",
      "mappings": {"default": {"default": "tiret haut en pointillés"}}
    }, {
      "category": "Po",
      "key": "FE4A",
      "mappings": {"default": {"default": "tiret haut à point central"}}
    }, {"category": "Po", "key": "FE4B", "mappings": {"default": {"default": "tiret haut ondulé"}}}, {
      "category": "Po",
      "key": "FE4C",
      "mappings": {"default": {"default": "tiret haut double ondulé"}}
    }, {
      "category": "Pc",
      "key": "FE4D",
      "mappings": {"default": {"default": "tiret bas avec pointillés"}}
    }, {
      "category": "Pc",
      "key": "FE4E",
      "mappings": {"default": {"default": "tiret bas à point central"}}
    }, {"category": "Pc", "key": "FE4F", "mappings": {"default": {"default": "tiret bas ondulé"}}}, {
      "category": "Po",
      "key": "FE50",
      "mappings": {"default": {"default": "virgule minuscule"}}
    }, {"category": "Po", "key": "FE52", "mappings": {"default": {"default": "point minuscule"}}}, {
      "category": "Po",
      "key": "FE54",
      "mappings": {"default": {"default": "point virgule minuscule"}}
    }, {
      "category": "Po",
      "key": "FE55",
      "mappings": {"default": {"default": "deux points minuscule"}}
    }, {
      "category": "Po",
      "key": "FE56",
      "mappings": {"default": {"default": "point d'interrogation minuscule"}}
    }, {
      "category": "Po",
      "key": "FE57",
      "mappings": {"default": {"default": "point d'exclamation minuscule"}}
    }, {
      "category": "Pd",
      "key": "FE58",
      "mappings": {"default": {"default": "trait d'union cadratin minuscule"}}
    }, {
      "category": "Po",
      "key": "FE5F",
      "mappings": {"default": {"default": "croisillon minuscule"}}
    }, {"category": "Po", "key": "FE60", "mappings": {"default": {"default": "perluète minuscule"}}}, {
      "category": "Po",
      "key": "FE61",
      "mappings": {"default": {"default": "astérisque minuscule"}}
    }, {"category": "Sm", "key": "FE62", "mappings": {"default": {"default": "plus minuscule"}}}, {
      "category": "Pd",
      "key": "FE63",
      "mappings": {"default": {"default": "tiret minuscule"}}
    }, {
      "category": "Sm",
      "key": "FE64",
      "mappings": {"default": {"default": "inférieur à minuscule"}}
    }, {
      "category": "Sm",
      "key": "FE65",
      "mappings": {"default": {"default": "supérieur à minuscule"}}
    }, {"category": "Sm", "key": "FE66", "mappings": {"default": {"default": "égal à minuscule"}}}, {
      "key": "FE68",
      "mappings": {"default": {"default": "integer divide"}},
      "category": "Po"
    }, {
      "category": "Sc",
      "key": "FE69",
      "mappings": {"default": {"default": "symbole dollar minuscule"}}
    }, {
      "category": "Po",
      "key": "FE6A",
      "mappings": {"default": {"default": "symbole pour cent minuscule"}}
    }, {"category": "Po", "key": "FE6B", "mappings": {"default": {"default": "arrobe minuscule"}}}, {
      "category": "Po",
      "key": "FF01",
      "mappings": {"default": {"default": "point d'exclamation"}}
    }, {"category": "Po", "key": "FF02", "mappings": {"default": {"default": "guillemet"}}}, {
      "category": "Po",
      "key": "FF03",
      "mappings": {"default": {"default": "croisillon"}}
    }, {"category": "Sc", "key": "FF04", "mappings": {"default": {"default": "symbole dollar"}}}, {
      "category": "Po",
      "key": "FF05",
      "mappings": {"default": {"default": "symbole pour cent"}}
    }, {"category": "Po", "key": "FF06", "mappings": {"default": {"default": "perluète"}}}, {
      "category": "Po",
      "key": "FF07",
      "mappings": {"default": {"default": "apostrophe"}}
    }, {"category": "Po", "key": "FF0A", "mappings": {"default": {"default": "astérisque"}}}, {
      "category": "Sm",
      "key": "FF0B",
      "mappings": {"default": {"default": "plus"}}
    }, {"category": "Po", "key": "FF0C", "mappings": {"default": {"default": "virgule"}}}, {
      "category": "Pd",
      "key": "FF0D",
      "mappings": {"default": {"default": "tiret"}}
    }, {"category": "Po", "key": "FF0E", "mappings": {"default": {"default": "point"}}}, {
      "category": "Po",
      "key": "FF0F",
      "mappings": {"default": {"default": "barre oblique"}}
    }, {"category": "Po", "key": "FF1A", "mappings": {"default": {"default": "deux points colon"}}}, {
      "category": "Po",
      "key": "FF1B",
      "mappings": {"default": {"default": "point virgule"}}
    }, {"category": "Sm", "key": "FF1C", "mappings": {"default": {"default": "inférieur à"}}}, {
      "category": "Sm",
      "key": "FF1D",
      "mappings": {"default": {"default": "égal à"}}
    }, {"category": "Sm", "key": "FF1E", "mappings": {"default": {"default": "supérieur à"}}}, {
      "category": "Po",
      "key": "FF1F",
      "mappings": {"default": {"default": "point d'interrogation"}}
    }, {"category": "Po", "key": "FF20", "mappings": {"default": {"default": "arrobe"}}}, {
      "category": "Po",
      "key": "FF3C",
      "mappings": {"default": {"default": "barre oblique inversée"}}
    }, {"category": "Sk", "key": "FF3E", "mappings": {"default": {"default": "accent circonflexe"}}}, {
      "category": "Pc",
      "key": "FF3F",
      "mappings": {"default": {"default": "tiret bas"}}
    }, {"category": "Sk", "key": "FF40", "mappings": {"default": {"default": "accent grave"}}}, {
      "category": "Sm",
      "key": "FF5C",
      "mappings": {"default": {"default": "barre verticale"}}
    }, {"category": "Sm", "key": "FF5E", "mappings": {"default": {"default": "tilde"}}}, {
      "category": "Sc",
      "key": "FFE0",
      "mappings": {"default": {"default": "symbole centime"}}
    }, {"category": "Sc", "key": "FFE1", "mappings": {"default": {"default": "symbole livre"}}}, {
      "category": "Sm",
      "key": "FFE2",
      "mappings": {"default": {"default": "négation"}}
    }, {"category": "Sk", "key": "FFE3", "mappings": {"default": {"default": "macron *"}}}, {
      "category": "So",
      "key": "FFE4",
      "mappings": {"default": {"default": "ligne brisée"}}
    }, {"category": "Sc", "key": "FFE5", "mappings": {"default": {"default": "symbole yen"}}}, {
      "category": "Sc",
      "key": "FFE6",
      "mappings": {"default": {"default": "symbole won"}}
    }, {
      "category": "So",
      "key": "FFE8",
      "mappings": {"default": {"default": "ligne verticale mince"}}
    }, {"category": "So", "key": "FFED", "mappings": {"default": {"default": "carré noir"}}}, {
      "category": "So",
      "key": "FFEE",
      "mappings": {"default": {"default": "cercle blanc"}}
    }]
  ,
  "fr/symbols/math_whitespace.js":
    [{"locale": "fr"}, {
      "category": "Zs",
      "key": "0020",
      "mappings": {"default": {"default": "espace"}}
    }, {"key": "00A0", "mappings": {"default": {"default": " "}}, "category": "Zs"}, {
      "key": "00AD",
      "mappings": {"default": {"default": "trait d'union conditionnel"}},
      "category": "Cf"
    }, {"category": "Zs", "key": "2000", "mappings": {"default": {"default": "demi cadratin"}}}, {
      "category": "Zs",
      "key": "2001",
      "mappings": {"default": {"default": "cadratin"}}
    }, {"key": "2002", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
      "key": "2003",
      "mappings": {"default": {"default": ""}},
      "category": "Zs"
    }, {"key": "2004", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
      "key": "2005",
      "mappings": {"default": {"default": ""}},
      "category": "Zs"
    }, {"category": "Zs", "key": "2006", "mappings": {"default": {"default": "sixième de cadratin"}}}, {
      "key": "2007",
      "mappings": {"default": {"default": ""}},
      "category": "Zs"
    }, {"key": "2008", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
      "key": "2009",
      "mappings": {"default": {"default": ""}},
      "category": "Zs"
    }, {"key": "200A", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
      "key": "200B",
      "mappings": {"default": {"default": ""}},
      "category": "Cf"
    }, {"key": "200C", "mappings": {"default": {"default": "antiliant sans chasse"}}, "category": "Cf"}, {
      "key": "200D",
      "mappings": {"default": {"default": "liant sans chasse"}},
      "category": "Cf"
    }, {
      "key": "200E",
      "mappings": {"default": {"default": "marque gauche à droite"}},
      "category": "Cf"
    }, {
      "key": "200F",
      "mappings": {"default": {"default": "marque droite à gauche"}},
      "category": "Cf"
    }, {
      "category": "Zl",
      "key": "2028",
      "mappings": {"default": {"default": "séparateur de lignes"}}
    }, {
      "category": "Zp",
      "key": "2029",
      "mappings": {"default": {"default": "séparateur de paragraphes"}}
    }, {
      "category": "Cf",
      "key": "202A",
      "mappings": {"default": {"default": "enchâssement gauche à droite"}}
    }, {
      "category": "Cf",
      "key": "202B",
      "mappings": {"default": {"default": "enchâssement droite à gauche"}}
    }, {
      "category": "Cf",
      "key": "202C",
      "mappings": {"default": {"default": "dépilement de formatage directionnel"}}
    }, {
      "category": "Cf",
      "key": "202D",
      "mappings": {"default": {"default": "forçage gauche à droite"}}
    }, {
      "category": "Cf",
      "key": "202E",
      "mappings": {"default": {"default": "forçage droite à gauche"}}
    }, {
      "category": "Zs",
      "key": "202F",
      "mappings": {"default": {"default": "espace insécable étroite"}}
    }, {
      "key": "205F",
      "mappings": {"default": {"default": "espace moyenne mathématique"}},
      "category": "Zs"
    }, {"key": "2060", "mappings": {"default": {"default": ""}}, "category": "Cf"}, {
      "key": "2061",
      "mappings": {"default": {"default": "de"}},
      "category": "Cf"
    }, {"key": "2062", "mappings": {"default": {"default": " "}}, "category": "Cf"}, {
      "key": "2063",
      "mappings": {"default": {"default": ""}},
      "category": "Cf"
    }, {"category": "Cf", "key": "2064", "mappings": {"default": {"default": "plus invisible"}}}, {
      "category": "Cf",
      "key": "206A",
      "mappings": {"default": {"default": "inhibiteur d'échange symétrique"}}
    }, {
      "category": "Cf",
      "key": "206B",
      "mappings": {"default": {"default": "activateur d'échange symétrique"}}
    }, {
      "category": "Cf",
      "key": "206E",
      "mappings": {"default": {"default": "sélecteur de formes numérales nationales"}}
    }, {
      "category": "Cf",
      "key": "206F",
      "mappings": {"default": {"default": "sélecteur de formes numérales de référence"}}
    }, {"key": "FEFF", "mappings": {"default": {"default": "espace ultrafine"}}, "category": "Cf"}, {
      "category": "Cf",
      "key": "FFF9",
      "mappings": {"default": {"default": "ancre d'annotation interlinéaire"}}
    }, {
      "category": "Cf",
      "key": "FFFA",
      "mappings": {"default": {"default": "séparateur d'annotation interlinéaire"}}
    }, {
      "category": "Cf",
      "key": "FFFB",
      "mappings": {"default": {"default": "terminateur d'annotation interlinéaire"}}
    }]
  ,
  "fr/symbols/other_stars.js":
    [{"locale": "fr"}, {
      "category": "So",
      "key": "23E8",
      "mappings": {"default": {"default": "symbole exposant décimal"}}
    }, {"key": "2605", "mappings": {"default": {"default": "étoile noire"}}, "category": "So"}, {
      "key": "2606",
      "mappings": {"default": {"default": "étoile blanche"}},
      "category": "So"
    }, {"category": "So", "key": "26AA", "mappings": {"default": {"default": "cercle moyen blanc"}}}, {
      "category": "So",
      "key": "26AB",
      "mappings": {"default": {"default": "cercle moyen noir"}}
    }, {
      "category": "So",
      "key": "2705",
      "mappings": {"default": {"default": "signe de vérification blanc gras"}}
    }, {
      "key": "2713",
      "mappings": {"default": {"default": "signe de vérification"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2714",
      "mappings": {"default": {"default": "gros signe de vérification"}}
    }, {
      "category": "So",
      "key": "2715",
      "mappings": {"default": {"default": "x de multiplication"}}
    }, {
      "category": "So",
      "key": "2716",
      "mappings": {"default": {"default": "gros x de multiplication"}}
    }, {
      "key": "2717",
      "mappings": {"default": {"default": "x de bulletin de vote"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2718",
      "mappings": {"default": {"default": "gros x de bulletin de vote"}}
    }, {
      "category": "So",
      "key": "271B",
      "mappings": {"default": {"default": "croix percée d'un carré"}}
    }, {
      "category": "So",
      "key": "271C",
      "mappings": {"default": {"default": "grosse croix percée d'un carré"}}
    }, {"key": "2720", "mappings": {"default": {"default": "croix de malte"}}, "category": "So"}, {
      "category": "So",
      "key": "2721",
      "mappings": {"default": {"default": "Étoile de david"}}
    }, {"category": "So", "key": "2722", "mappings": {"default": {"default": "croix larmée"}}}, {
      "category": "So",
      "key": "2723",
      "mappings": {"default": {"default": "croix pommetée"}}
    }, {
      "category": "So",
      "key": "2724",
      "mappings": {"default": {"default": "grosse croix pommetée"}}
    }, {"category": "So", "key": "2725", "mappings": {"default": {"default": "croix tréflée"}}}, {
      "category": "So",
      "key": "2726",
      "mappings": {"default": {"default": "Étoile noire à quatre branches"}}
    }, {
      "category": "So",
      "key": "2727",
      "mappings": {"default": {"default": "Étoile blanche à quatre branches"}}
    }, {"category": "So", "key": "2728", "mappings": {"default": {"default": "scintillements"}}}, {
      "category": "So",
      "key": "2729",
      "mappings": {"default": {"default": "Étoile blanche à contour accentué"}}
    }, {
      "category": "So",
      "key": "272A",
      "mappings": {"default": {"default": "Étoile blanche cerclée"}}
    }, {"category": "So", "key": "272B", "mappings": {"default": {"default": "molette noire"}}}, {
      "category": "So",
      "key": "272C",
      "mappings": {"default": {"default": "molette blanche"}}
    }, {
      "category": "So",
      "key": "272D",
      "mappings": {"default": {"default": "Étoile noire avec contour"}}
    }, {
      "category": "So",
      "key": "272E",
      "mappings": {"default": {"default": "Étoile noire avec gros contour"}}
    }, {"category": "So", "key": "272F", "mappings": {"default": {"default": "Étoile rayonnante"}}}, {
      "category": "So",
      "key": "2730",
      "mappings": {"default": {"default": "Étoile blanche ombrée"}}
    }, {"category": "So", "key": "2731", "mappings": {"default": {"default": "gros astérisque"}}}, {
      "category": "So",
      "key": "2732",
      "mappings": {"default": {"default": "astérisque percé"}}
    }, {
      "category": "So",
      "key": "2733",
      "mappings": {"default": {"default": "astérisque à huit branches"}}
    }, {
      "category": "So",
      "key": "2734",
      "mappings": {"default": {"default": "Étoile noire à huit branches"}}
    }, {
      "category": "So",
      "key": "2735",
      "mappings": {"default": {"default": "Étoile rayonnante à huit branches"}}
    }, {
      "key": "2736",
      "mappings": {"default": {"default": "étoile noire à six branches"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "2739",
      "mappings": {"default": {"default": "Étoile noire à douze branches"}}
    }, {
      "category": "So",
      "key": "273A",
      "mappings": {"default": {"default": "astérisque à seize branches"}}
    }, {"category": "So", "key": "273B", "mappings": {"default": {"default": "astérisque larmé"}}}, {
      "category": "So",
      "key": "273C",
      "mappings": {"default": {"default": "astérisque larmé percé"}}
    }, {
      "key": "273D",
      "mappings": {"default": {"default": "gros astérisque larmé"}},
      "category": "So"
    }, {
      "category": "So",
      "key": "273E",
      "mappings": {"default": {"default": "sixtefeuille blanche et noire"}}
    }, {
      "category": "So",
      "key": "273F",
      "mappings": {"default": {"default": "quintefeuille noire"}}
    }, {
      "category": "So",
      "key": "2740",
      "mappings": {"default": {"default": "quintefeuille blanche"}}
    }, {
      "category": "So",
      "key": "2741",
      "mappings": {"default": {"default": "double quartefeuille noire avec contour"}}
    }, {
      "category": "So",
      "key": "2742",
      "mappings": {"default": {"default": "Étoile cerclée percée à huit branches"}}
    }, {
      "category": "So",
      "key": "2743",
      "mappings": {"default": {"default": "gros astérisque rayonnant larmé"}}
    }, {"category": "So", "key": "2744", "mappings": {"default": {"default": "flocon de neige"}}}, {
      "category": "So",
      "key": "2745",
      "mappings": {"default": {"default": "flocon de neige à trois folioles transpercé"}}
    }, {
      "category": "So",
      "key": "2746",
      "mappings": {"default": {"default": "gros flocon de neige à chevrons"}}
    }, {"category": "So", "key": "2747", "mappings": {"default": {"default": "Étincellement"}}}, {
      "category": "So",
      "key": "2748",
      "mappings": {"default": {"default": "gros étincellement"}}
    }, {"category": "So", "key": "2749", "mappings": {"default": {"default": "astérisque pommeté"}}}, {
      "category": "So",
      "key": "274A",
      "mappings": {"default": {"default": "astérisque hélice à huit branches larmées"}}
    }, {
      "category": "So",
      "key": "274B",
      "mappings": {"default": {"default": "gros astérisque hélice à huit branches larmées"}}
    }, {"category": "So", "key": "274C", "mappings": {"default": {"default": "croix grasse"}}}, {
      "category": "So",
      "key": "274D",
      "mappings": {"default": {"default": "cercle blanc ombré"}}
    }]
  ,
  "fr/units/energy.js":
    [{"locale": "fr"}, {
      "key": "W",
      "mappings": {"default": {"default": "watts", "singular": "watt", "dual": ""}},
      "category": "energy",
      "names": ["W", "w"]
    }, {
      "key": "J",
      "mappings": {"default": {"default": "joules", "singular": "joule", "dual": ""}},
      "category": "energy",
      "names": ["J"]
    }, {
      "key": "N",
      "mappings": {"default": {"default": "newtons", "singular": "newton", "dual": ""}},
      "category": "energy",
      "names": ["N"]
    }, {
      "key": "A",
      "mappings": {"default": {"default": "ampères", "singular": "ampère", "dual": ""}},
      "category": "energy",
      "names": ["A"]
    }, {
      "key": "V",
      "mappings": {"default": {"default": "volts", "singular": "volt", "dual": ""}},
      "category": "energy",
      "names": ["V"]
    }, {
      "key": "Ω",
      "mappings": {"default": {"default": "ohms", "singular": "ohm", "dual": ""}},
      "category": "energy",
      "names": ["Ω", "Ohm"]
    }]
  ,
  "fr/units/length.js":
    [{"locale": "fr"}, {
      "key": "m",
      "mappings": {"default": {"default": "mètres", "singular": "mètre", "dual": ""}},
      "category": "Length",
      "names": ["m"]
    }, {
      "key": "ft",
      "mappings": {"default": {"default": "pieds", "singular": "pied", "dual": ""}},
      "category": "Length",
      "names": ["ft", "ft."]
    }, {
      "key": "in",
      "mappings": {"default": {"default": "pouces", "singular": "pouce", "dual": ""}},
      "category": "Length",
      "names": ["in", "in."]
    }, {
      "key": "mi",
      "mappings": {"default": {"default": "miles", "singular": "mile", "dual": ""}},
      "category": "Length",
      "names": ["mi", "mi."]
    }, {
      "key": "yd",
      "mappings": {"default": {"default": "yards", "singular": "yard", "dual": ""}},
      "category": "Length",
      "names": ["yd", "yd."]
    }]
  ,
  "fr/units/memory.js":
    [{"locale": "fr"}]
  ,
  "fr/units/other.js":
    [{"locale": "fr"}]
  ,
  "fr/units/speed.js":
    [{"locale": "fr"}, {
      "key": "mph",
      "mappings": {"default": {"default": "miles par heure", "singular": "mile par heure", "dual": ""}},
      "names": ["mph"]
    }]
  ,
  "fr/units/temperature.js":
    [{"locale": "fr"}, {
      "key": "F",
      "mappings": {"default": {"default": "farads", "singular": "farad", "dual": ""}},
      "names": ["F", "F.", "°F"]
    }, {
      "key": "C",
      "mappings": {"default": {"default": "coulombs", "singular": "coulomb", "dual": ""}},
      "names": ["C", "°C"]
    }, {
      "key": "K",
      "mappings": {"default": {"default": "kelvins", "singular": "kelvin", "dual": ""}},
      "names": ["K", "°K"]
    }]
  ,
  "fr/units/time.js":
    [{"locale": "fr"}, {
      "key": "s",
      "mappings": {"default": {"default": "secondes", "singular": "seconde", "dual": ""}},
      "category": "time",
      "names": ["s"]
    }]
  ,
  "fr/units/volume.js":
    [{"locale": "fr"}, {
      "key": "bbl",
      "mappings": {"default": {"default": "barils", "singular": "baril", "dual": ""}},
      "category": "volume",
      "names": ["bbl.", "bbl"]
    }, {
      "key": "gal",
      "mappings": {"default": {"default": "gallons", "singular": "gallon", "dual": ""}},
      "category": "volume",
      "names": ["gal", "gal."]
    }, {
      "key": "pt",
      "mappings": {"default": {"default": "pintes", "singular": "pinte", "dual": ""}},
      "category": "volume",
      "names": ["pt", "pt."]
    }, {
      "key": "qt",
      "mappings": {"default": {"default": "quarts", "singular": "quart", "dual": ""}},
      "category": "volume",
      "names": ["qt", "qt."]
    }, {
      "key": "tbsp",
      "mappings": {"default": {"default": "cuillères de table", "singular": "cuillèe de table", "dual": ""}},
      "category": "volume",
      "names": ["tbsp", "tbsp."]
    }, {
      "key": "tsp",
      "mappings": {"default": {"default": "cuillèes à thé", "singular": "cuillère à thé", "dual": ""}},
      "category": "volume",
      "names": ["tsp", "tsp."]
    }, {
      "key": "l",
      "mappings": {"default": {"default": "litres", "singular": "litre", "dual": ""}},
      "category": "volume",
      "names": ["l"]
    }]
  ,
  "fr/units/weight.js":
    [{"locale": "fr"}, {
      "key": "lb",
      "mappings": {"default": {"default": "livres", "singular": "livre", "dual": ""}},
      "names": ["lb", "lb."]
    }, {
      "key": "oz",
      "mappings": {"default": {"default": "onces", "singular": "oce", "dual": ""}},
      "names": ["oz", "oz."]
    }, {
      "key": "gr",
      "mappings": {"default": {"default": "grains", "singular": "grain", "dual": ""}},
      "names": ["g", "gr"]
    }, {
      "key": "t",
      "mappings": {"default": {"default": "tonnes", "singular": "tonne", "dual": ""}},
      "names": ["t", "T"]
    }]
}

