{
  "es/functions/algebra.js"
:
  [{"locale": "es"}, {
    "key": "deg",
    "names": ["deg"],
    "mappings": {"default": {"default": "grados"}},
    "category": "Algebra"
  }, {
    "key": "det",
    "names": ["det"],
    "mappings": {"default": {"default": "determinante"}},
    "category": "Algebra"
  }, {
    "key": "dim",
    "names": ["dim"],
    "mappings": {"default": {"default": "dimensión"}},
    "category": "Algebra"
  }, {
    "key": "hom",
    "names": ["hom", "Hom"],
    "mappings": {"default": {"default": "homomorfismo"}},
    "category": "Algebra"
  }, {
    "key": "ker",
    "names": ["ker"],
    "mappings": {"default": {"default": "kernel"}},
    "category": "Algebra"
  }, {"key": "Tr", "names": ["Tr", "tr"], "mappings": {"default": {"default": "traza"}}, "category": "Algebra"}]
    ,
    "es/functions/elementary.js"
:
  [{"locale": "es"}, {
    "key": "log",
    "names": ["log"],
    "mappings": {"default": {"default": "logaritmo"}},
    "category": "Logarithm"
  }, {
    "key": "ln",
    "names": ["ln"],
    "mappings": {"default": {"default": "logaritmo neperiano"}},
    "category": "Logarithm"
  }, {
    "key": "lg",
    "names": ["lg"],
    "mappings": {"default": {"default": "logaritmo base 10"}},
    "category": "Logarithm"
  }, {
    "key": "exp",
    "names": ["exp", "expt"],
    "mappings": {"default": {"default": "exponente"}},
    "category": "Elementary"
  }, {
    "key": "gcd",
    "names": ["gcd", "mcd", "MCD"],
    "mappings": {"default": {"default": "MCD"}},
    "category": "Elementary"
  }, {
    "key": "lcm",
    "names": ["lcm", "mcm", "MCM"],
    "mappings": {"default": {"default": "mcm"}},
    "category": "Elementary"
  }, {
    "key": "arg",
    "names": ["arg"],
    "mappings": {"default": {"default": "argumento"}},
    "category": "Complex"
  }, {
    "key": "im",
    "names": ["im"],
    "mappings": {"default": {"default": "parte imaginaria"}},
    "category": "Complex"
  }, {
    "key": "re",
    "names": ["re"],
    "mappings": {"default": {"default": "residuo"}},
    "category": "Complex"
  }, {
    "key": "inf",
    "names": ["inf"],
    "mappings": {"default": {"default": "extremo inferior"}},
    "category": "Limits"
  }, {
    "key": "lim",
    "names": ["lim"],
    "mappings": {"default": {"default": "límite"}},
    "category": "Limits"
  }, {
    "key": "max",
    "names": ["max"],
    "mappings": {"default": {"default": "máximo"}},
    "category": "Limits"
  }, {
    "key": "min",
    "names": ["min"],
    "mappings": {"default": {"default": "mínimo"}},
    "category": "Limits"
  }, {
    "key": "sup",
    "names": ["sup"],
    "mappings": {"default": {"default": "superior"}},
    "category": "Limits"
  }, {
    "key": "lim inf",
    "names": ["lim inf", "liminf"],
    "mappings": {"default": {"default": "límite inferior"}},
    "category": "Limits"
  }, {
    "key": "lim sup",
    "names": ["lim sup", "limsup"],
    "mappings": {"default": {"default": "límite superior"}},
    "category": "Limits"
  }, {
    "key": "injlim",
    "names": ["injlim", "inj lim"],
    "mappings": {"default": {"default": "límite directo"}},
    "category": "Limits"
  }, {
    "key": "projlim",
    "names": ["projlim", "proj lim"],
    "mappings": {"default": {"default": "límite inverso"}},
    "category": "Limits"
  }, {
    "key": "mod",
    "names": ["mod"],
    "mappings": {"default": {"default": "módulo"}},
    "category": "Elementary"
  }, {"key": "Pr", "names": ["Pr"], "mappings": {"default": {"default": "probabilidad"}}, "category": "Probability"}]
    ,
    "es/functions/hyperbolic.js"
:
  [{"locale": "es"}, {
    "key": "cosh",
    "names": ["cosh"],
    "mappings": {"default": {"default": "coseno hiperbólico"}},
    "category": "Hyperbolic"
  }, {
    "key": "coth",
    "names": ["coth"],
    "mappings": {"default": {"default": "cotangente hiperbólica"}},
    "category": "Hyperbolic"
  }, {
    "key": "csch",
    "names": ["csch"],
    "mappings": {"default": {"default": "cosecante hiperbólica"}},
    "category": "Hyperbolic"
  }, {
    "key": "sech",
    "names": ["sech"],
    "mappings": {"default": {"default": "secante hiperbólica"}},
    "category": "Hyperbolic"
  }, {
    "key": "sinh",
    "names": ["sinh"],
    "mappings": {"default": {"default": "seno hiperbólico"}},
    "category": "Hyperbolic"
  }, {
    "key": "tanh",
    "names": ["tanh"],
    "mappings": {"default": {"default": "tangente hiperbólica"}},
    "category": "Hyperbolic"
  }, {
    "key": "arcosh",
    "names": ["arcosh", "arccosh"],
    "mappings": {"default": {"default": "area coseno hiperbólico"}},
    "category": "Area"
  }, {
    "key": "arcoth",
    "names": ["arcoth", "arccoth"],
    "mappings": {"default": {"default": "area cotangente hiperbólica"}},
    "category": "Area"
  }, {
    "key": "arcsch",
    "names": ["arcsch", "arccsch"],
    "mappings": {"default": {"default": "area cosecante hiperbólica"}},
    "category": "Area"
  }, {
    "key": "arsech",
    "names": ["arsech", "arcsech"],
    "mappings": {"default": {"default": "area secante hiperbólica"}},
    "category": "Area"
  }, {
    "key": "arsinh",
    "names": ["arsinh", "arcsinh"],
    "mappings": {"default": {"default": "area seno hiperbólico"}},
    "category": "Area"
  }, {
    "key": "artanh",
    "names": ["artanh", "arctanh"],
    "mappings": {"default": {"default": "area tangente hiperbólica"}},
    "category": "Area"
  }]
    ,
    "es/functions/trigonometry.js"
:
  [{"locale": "es"}, {
    "key": "cos",
    "names": ["cos", "cosine"],
    "mappings": {"default": {"default": "coseno"}},
    "category": "Trigonometric"
  }, {
    "key": "cot",
    "names": ["cot"],
    "mappings": {"default": {"default": "cotangente"}},
    "category": "Trigonometric"
  }, {
    "key": "csc",
    "names": ["csc"],
    "mappings": {"default": {"default": "cosecante"}},
    "category": "Trigonometric"
  }, {
    "key": "sec",
    "names": ["sec"],
    "mappings": {"default": {"default": "secant"}},
    "category": "Trigonometric"
  }, {
    "key": "sin",
    "names": ["sin", "sine", "sen"],
    "mappings": {"default": {"default": "seno"}},
    "category": "Trigonometric"
  }, {
    "key": "tan",
    "names": ["tan"],
    "mappings": {"default": {"default": "tangente"}},
    "category": "Trigonometric"
  }, {
    "key": "arccos",
    "names": ["arccos"],
    "mappings": {"default": {"default": "arco coseno"}},
    "category": "Cyclometric"
  }, {
    "key": "arccot",
    "names": ["arccot"],
    "mappings": {"default": {"default": "arco cotangente"}},
    "category": "Cyclometric"
  }, {
    "key": "arccsc",
    "names": ["arccsc"],
    "mappings": {"default": {"default": "arco cosecante"}},
    "category": "Cyclometric"
  }, {
    "key": "arcsec",
    "names": ["arcsec"],
    "mappings": {"default": {"default": "arco secante"}},
    "category": "Cyclometric"
  }, {
    "key": "arcsin",
    "names": ["arcsin"],
    "mappings": {"default": {"default": "arco seno"}},
    "category": "Cyclometric"
  }, {
    "key": "arctan",
    "names": ["arctan"],
    "mappings": {"default": {"default": "arco tangente"}},
    "category": "Cyclometric"
  }]
    ,
    "es/symbols/digits_rest.js"
:
  [{"locale": "es"}, {
    "category": "No",
    "mappings": {
      "default": {"default": "al cuadrado"},
      "mathspeak": {"default": "al cuadrado"},
      "clearspeak": {"default": "al cuadrado"}
    },
    "key": "00B2"
  }, {
    "category": "No",
    "mappings": {
      "default": {"default": "al cubo"},
      "mathspeak": {"default": "al cubo"},
      "clearspeak": {"default": "al cubo"}
    },
    "key": "00B3"
  }, {"category": "No", "mappings": {"default": {"default": "un cuarto"}}, "key": "00BC"}, {
    "category": "No",
    "mappings": {"default": {"default": "un medio"}},
    "key": "00BD"
  }, {"category": "No", "mappings": {"default": {"default": "tres cuartos"}}, "key": "00BE"}, {
    "category": "No",
    "key": "2150",
    "mappings": {"default": {"default": "Vulgar Fracción Un Séptimo"}}
  }, {
    "category": "No",
    "key": "2151",
    "mappings": {"default": {"default": "Vulgar Fracción Un Noveno"}}
  }, {
    "category": "No",
    "key": "2152",
    "mappings": {"default": {"default": "Vulgar Fracción Un Décimo"}}
  }, {
    "category": "No",
    "key": "2153",
    "mappings": {"default": {"default": "Vulgar Fracción Un Tercero"}}
  }, {
    "category": "No",
    "key": "2154",
    "mappings": {"default": {"default": "Fracción vulgar dos tercios"}}
  }, {
    "category": "No",
    "key": "2155",
    "mappings": {"default": {"default": "Vulgar Fraction One Fifth"}}
  }, {
    "category": "No",
    "key": "2156",
    "mappings": {"default": {"default": "Fracción vulgar dos quintos"}}
  }, {
    "category": "No",
    "key": "2157",
    "mappings": {"default": {"default": "Fracción vulgar tres quintos"}}
  }, {
    "category": "No",
    "key": "2158",
    "mappings": {"default": {"default": "Fracción vulgar cuatro quintos"}}
  }, {
    "category": "No",
    "key": "2159",
    "mappings": {"default": {"default": "Vulgar Fracción Un Sexto"}}
  }, {
    "category": "No",
    "key": "215A",
    "mappings": {"default": {"default": "Vulgar fracción cinco sextos"}}
  }, {
    "category": "No",
    "key": "215B",
    "mappings": {"default": {"default": "Vulgar Fracción Un Octavo"}}
  }, {
    "category": "No",
    "key": "215C",
    "mappings": {"default": {"default": "Fracción vulgar tres octavos"}}
  }, {
    "category": "No",
    "key": "215D",
    "mappings": {"default": {"default": "Fracción vulgar cinco octavos"}}
  }, {
    "category": "No",
    "key": "215E",
    "mappings": {"default": {"default": "Fracción vulgar siete octavos"}}
  }, {
    "category": "No",
    "key": "215F",
    "mappings": {"default": {"default": "Numerador de fracciones uno"}}
  }, {
    "category": "No",
    "key": "2189",
    "mappings": {"default": {"default": "Vulgar Fraction Zero Thirds"}}
  }, {
    "category": "No",
    "key": "3248",
    "mappings": {"default": {"default": "Número diez en un círculo en la Plaza Negra"}}
  }, {
    "category": "No",
    "key": "3249",
    "mappings": {"default": {"default": "Número veinte en un círculo en la Plaza Negra"}}
  }, {
    "category": "No",
    "key": "324A",
    "mappings": {"default": {"default": "Número circundado treinta en cuadrado negro"}}
  }, {
    "category": "No",
    "key": "324B",
    "mappings": {"default": {"default": "Número circundado Cuarenta en cuadrado negro"}}
  }, {
    "category": "No",
    "key": "324C",
    "mappings": {"default": {"default": "Círculo número cincuenta en la plaza negra"}}
  }, {
    "category": "No",
    "key": "324D",
    "mappings": {"default": {"default": "Círculo número sesenta en la plaza negra"}}
  }, {
    "category": "No",
    "key": "324E",
    "mappings": {"default": {"default": "Número setenta en un círculo en la Plaza Negra"}}
  }, {
    "category": "No",
    "key": "324F",
    "mappings": {"default": {"default": "Círculo número ochenta en cuadrado negro"}}
  }]
    ,
    "es/symbols/greek-rest.js"
:
  [{"locale": "es"}, {
    "category": "Lu",
    "key": "0394",
    "mappings": {"clearspeak": {"default": "triangle", "TriangleSymbol_Delta": "cap Delta"}}
  }]
    ,
    "es/symbols/greek-scripts.js"
:
  [{"locale": "es"}, {
    "category": "Ll",
    "key": "1D26",
    "mappings": {"default": {"default": "pequeño mayúscula gamma"}}
  }, {
    "category": "Ll",
    "key": "1D27",
    "mappings": {"default": {"default": "pequeña mayúscula lamda"}}
  }, {"category": "Ll", "key": "1D28", "mappings": {"default": {"default": "pequeña mayúscula Pi"}}}, {
    "category": "Ll",
    "key": "1D29",
    "mappings": {"default": {"default": "pequeña mayúscula Rho"}}
  }, {
    "category": "Ll",
    "key": "1D2A",
    "mappings": {"default": {"default": "pequeña mayúscula Psi"}}
  }, {
    "category": "Lm",
    "key": "1D5E",
    "mappings": {"default": {"default": "modificador de pequeño gamma"}}
  }, {
    "category": "Lm",
    "key": "1D60",
    "mappings": {"default": {"default": "modificador de pequeño phi"}}
  }, {
    "category": "Lm",
    "key": "1D66",
    "mappings": {"default": {"default": "subscripción pequeña beta"}}
  }, {
    "category": "Lm",
    "key": "1D67",
    "mappings": {"default": {"default": "subíndice pequeña gamma"}}
  }, {"category": "Lm", "key": "1D68", "mappings": {"default": {"default": "subíndice rho"}}}, {
    "category": "Lm",
    "key": "1D69",
    "mappings": {"default": {"default": "subíndice phi"}}
  }, {"category": "Lm", "key": "1D6A", "mappings": {"default": {"default": "subscripción chi"}}}]
    ,
    "es/symbols/greek-symbols.js"
:
  [{"locale": "es"}, {
    "key": "03D0",
    "mappings": {"default": {"default": "símbolo beta"}},
    "category": "Ll"
  }, {"key": "03D7", "mappings": {"default": {"default": "símbolo kai griego"}}, "category": "Ll"}, {
    "key": "03F6",
    "mappings": {"default": {"default": "épsilon invertido"}},
    "category": "Sm"
  }, {
    "category": "Lu",
    "key": "1D7CA",
    "mappings": {"default": {"default": "Matemática Bold Capital Digamma"}}
  }, {"category": "Ll", "key": "1D7CB", "mappings": {"default": {"default": "Matemática negrita pequeño digamma"}}}]
    ,
    "es/symbols/hebrew_letters.js"
:
  [{"locale": "es"}, {"key": "2135", "mappings": {"default": {"default": "alef"}}, "category": "Lo"}, {
    "key": "2136",
    "mappings": {"default": {"default": "bet"}},
    "category": "Lo"
  }, {"key": "2137", "mappings": {"default": {"default": "guímel"}}, "category": "Lo"}, {
    "key": "2138",
    "mappings": {"default": {"default": "dálet"}},
    "category": "Lo"
  }]
    ,
    "es/symbols/latin-lower-double-accent.js"
:
  [{"locale": "es"}, {
    "category": "Ll",
    "key": "01D6",
    "mappings": {"default": {"default": "u con diéresis y macron"}}
  }, {
    "category": "Ll",
    "key": "01D8",
    "mappings": {"default": {"default": "u con diéresis y acento agudo"}}
  }, {
    "category": "Ll",
    "key": "01DA",
    "mappings": {"default": {"default": "u con diéresis y acento"}}
  }, {
    "category": "Ll",
    "key": "01DC",
    "mappings": {"default": {"default": "u con diéresis y grave"}}
  }, {
    "category": "Ll",
    "key": "01DF",
    "mappings": {"default": {"default": "a con diéresis y macron"}}
  }, {
    "category": "Ll",
    "key": "01E1",
    "mappings": {"default": {"default": "a con punto arriba y macron"}}
  }, {
    "category": "Ll",
    "key": "01ED",
    "mappings": {"default": {"default": "o con pecíolo y macron"}}
  }, {
    "category": "Ll",
    "key": "01FB",
    "mappings": {"default": {"default": "a con ring above y acento agudo"}}
  }, {
    "category": "Ll",
    "key": "022B",
    "mappings": {"default": {"default": "o con diéresis y macron"}}
  }, {"category": "Ll", "key": "022D", "mappings": {"default": {"default": "o con tilde y macron"}}}, {
    "category": "Ll",
    "key": "0231",
    "mappings": {"default": {"default": "o con Dot Above y Macron"}}
  }, {"key": "1E09", "mappings": {"default": {"default": "c con cedilla y agudo"}}, "category": "Ll"}, {
    "key": "1E15",
    "mappings": {"default": {"default": "e con macrón y grave"}},
    "category": "Ll"
  }, {"key": "1E17", "mappings": {"default": {"default": "e con macrón y agudo"}}, "category": "Ll"}, {
    "key": "1E1D",
    "mappings": {"default": {"default": "e con cedilla y breve"}},
    "category": "Ll"
  }, {"key": "1E2F", "mappings": {"default": {"default": "i con diéresis y agudo"}}, "category": "Ll"}, {
    "key": "1E39",
    "mappings": {"default": {"default": "l con punto debajo y macrón"}},
    "category": "Ll"
  }, {"key": "1E4D", "mappings": {"default": {"default": "o con tilde y acute"}}, "category": "Ll"}, {
    "key": "1E4F",
    "mappings": {"default": {"default": "o con tilde y diéresis"}},
    "category": "Ll"
  }, {"key": "1E51", "mappings": {"default": {"default": "o con macrón y grave"}}, "category": "Ll"}, {
    "key": "1E53",
    "mappings": {"default": {"default": "o con macrón y agudo"}},
    "category": "Ll"
  }, {
    "key": "1E5D",
    "mappings": {"default": {"default": "r con punto debajo y macrón"}},
    "category": "Ll"
  }, {
    "key": "1E65",
    "mappings": {"default": {"default": "s con agudo y punto arriba"}},
    "category": "Ll"
  }, {
    "key": "1E67",
    "mappings": {"default": {"default": "s con carón y punto arriba"}},
    "category": "Ll"
  }, {
    "key": "1E69",
    "mappings": {"default": {"default": "s con punto debajo y punto arriba"}},
    "category": "Ll"
  }, {"key": "1E79", "mappings": {"default": {"default": "u con tilde y agudo"}}, "category": "Ll"}, {
    "key": "1E7B",
    "mappings": {"default": {"default": "u con macrón y diéresis"}},
    "category": "Ll"
  }, {
    "key": "1EA5",
    "mappings": {"default": {"default": "a con acento circunflejo y agudo"}},
    "category": "Ll"
  }, {
    "key": "1EA7",
    "mappings": {"default": {"default": "a con acento circunflejo y grave"}},
    "category": "Ll"
  }, {
    "key": "1EA9",
    "mappings": {"default": {"default": "a con acento circunflejo y gancho arriba"}},
    "category": "Ll"
  }, {
    "key": "1EAB",
    "mappings": {"default": {"default": "a con acento circunflejo y tilde"}},
    "category": "Ll"
  }, {
    "key": "1EAD",
    "mappings": {"default": {"default": "a con acento circunflejo y punto debajo"}},
    "category": "Ll"
  }, {"key": "1EAF", "mappings": {"default": {"default": "a con breve y agudo"}}, "category": "Ll"}, {
    "key": "1EB1",
    "mappings": {"default": {"default": "a con breve y grave"}},
    "category": "Ll"
  }, {
    "key": "1EB3",
    "mappings": {"default": {"default": "a con breve y gancho arriba"}},
    "category": "Ll"
  }, {"key": "1EB5", "mappings": {"default": {"default": "a con breve y tilde"}}, "category": "Ll"}, {
    "key": "1EB7",
    "mappings": {"default": {"default": "a con breve y punto debajo"}},
    "category": "Ll"
  }, {
    "key": "1EBF",
    "mappings": {"default": {"default": "e con acento circunflejo y agudo"}},
    "category": "Ll"
  }, {
    "key": "1EC1",
    "mappings": {"default": {"default": "e con acento circunflejo y grave"}},
    "category": "Ll"
  }, {
    "key": "1EC3",
    "mappings": {"default": {"default": "e con acento circunflejo y gancho arriba"}},
    "category": "Ll"
  }, {
    "key": "1EC5",
    "mappings": {"default": {"default": "e con acento circunflejo y tilde"}},
    "category": "Ll"
  }, {
    "key": "1EC7",
    "mappings": {"default": {"default": "e con acento circunflejo y punto debajo"}},
    "category": "Ll"
  }, {
    "key": "1ED1",
    "mappings": {"default": {"default": "o con acento circunflejo y agudo"}},
    "category": "Ll"
  }, {
    "key": "1ED3",
    "mappings": {"default": {"default": "o con acento circunflejo y grave"}},
    "category": "Ll"
  }, {
    "key": "1ED5",
    "mappings": {"default": {"default": "o con acento circunflejo y gancho arriba"}},
    "category": "Ll"
  }, {
    "key": "1ED7",
    "mappings": {"default": {"default": "o con acento circunflejo y tilde"}},
    "category": "Ll"
  }, {
    "key": "1ED9",
    "mappings": {"default": {"default": "o con acento circunflejo y punto debajo"}},
    "category": "Ll"
  }, {"key": "1EDB", "mappings": {"default": {"default": "o with horn and acute"}}, "category": "Ll"}, {
    "key": "1EDD",
    "mappings": {"default": {"default": "o with horn and grave"}},
    "category": "Ll"
  }, {
    "key": "1EDF",
    "mappings": {"default": {"default": "o with horn and hook above"}},
    "category": "Ll"
  }, {"key": "1EE1", "mappings": {"default": {"default": "o with horn and tilde"}}, "category": "Ll"}, {
    "key": "1EE3",
    "mappings": {"default": {"default": "o con cuerno y punto debajo"}},
    "category": "Ll"
  }, {"key": "1EE9", "mappings": {"default": {"default": "u con cuerno y agudo"}}, "category": "Ll"}, {
    "key": "1EEB",
    "mappings": {"default": {"default": "u con cuerno y grave"}},
    "category": "Ll"
  }, {
    "key": "1EED",
    "mappings": {"default": {"default": "u con cuerno y gancho arriba"}},
    "category": "Ll"
  }, {"key": "1EEF", "mappings": {"default": {"default": "u con cuerno y tilde"}}, "category": "Ll"}, {
    "key": "1EF1",
    "mappings": {"default": {"default": "u con cuerno y punto debajo"}},
    "category": "Ll"
  }]
    ,
    "es/symbols/latin-lower-phonetic.js"
:
  [{"locale": "es"}, {
    "key": "00F8",
    "mappings": {"default": {"default": "o barrada"}},
    "category": "Ll"
  }, {"key": "0111", "mappings": {"default": {"default": "d con barra"}}, "category": "Ll"}, {
    "key": "0127",
    "mappings": {"default": {"default": "h con guión"}},
    "category": "Ll"
  }, {"key": "0142", "mappings": {"default": {"default": "l con guión"}}, "category": "Ll"}, {
    "key": "0167",
    "mappings": {"default": {"default": "t con guión"}},
    "category": "Ll"
  }, {"category": "Ll", "key": "0180", "mappings": {"default": {"default": "b con línea"}}}, {
    "key": "019B",
    "mappings": {"default": {"default": "lambda tachada"}},
    "category": "Ll"
  }, {"category": "Ll", "key": "01B6", "mappings": {"default": {"default": "z con línea"}}}, {
    "category": "Ll",
    "key": "01BE",
    "mappings": {"default": {"default": "parada glotal invertida con línea"}}
  }, {"category": "Ll", "key": "01E5", "mappings": {"default": {"default": "g con línea"}}}, {
    "category": "Ll",
    "key": "01FF",
    "mappings": {"default": {"default": "o con línea y acento agudo"}}
  }, {"category": "Ll", "key": "023C", "mappings": {"default": {"default": "c con trazo"}}}, {
    "category": "Ll",
    "key": "0247",
    "mappings": {"default": {"default": "pequeña e con trazo"}}
  }, {"category": "Ll", "key": "0249", "mappings": {"default": {"default": "j con trazo"}}}, {
    "category": "Ll",
    "key": "024D",
    "mappings": {"default": {"default": "con trazo"}}
  }, {"category": "Ll", "key": "024F", "mappings": {"default": {"default": "y con trazo"}}}, {
    "category": "Ll",
    "key": "025F",
    "mappings": {"default": {"default": "pequeña sin puntos j con trazo"}}
  }, {"category": "Ll", "key": "0268", "mappings": {"default": {"default": "i con trazo"}}}, {
    "category": "Ll",
    "key": "0284",
    "mappings": {"default": {"default": "pequeña sin puntos j con trazo y gancho"}}
  }, {
    "category": "Ll",
    "key": "02A1",
    "mappings": {"default": {"default": "parada glotal con trazo"}}
  }, {
    "category": "Ll",
    "key": "02A2",
    "mappings": {"default": {"default": "parada invertida glotal con trazo"}}
  }, {"category": "Ll", "key": "1D13", "mappings": {"default": {"default": "de lado O con trazo"}}}, {
    "category": "Ll",
    "key": "1D7C",
    "mappings": {"default": {"default": "iota con trazo"}}
  }, {"category": "Ll", "key": "1D7D", "mappings": {"default": {"default": "p con trazo"}}}, {
    "category": "Ll",
    "key": "1D7F",
    "mappings": {"default": {"default": "upsilon con trazo"}}
  }, {
    "category": "Ll",
    "key": "1E9C",
    "mappings": {"default": {"default": "larga s con trazo diagonal"}}
  }, {
    "category": "Ll",
    "key": "1E9D",
    "mappings": {"default": {"default": "pequeña s larga con trazo alto"}}
  }, {"category": "Ll", "key": "018D", "mappings": {"default": {"default": "delta convertido"}}}, {
    "key": "1E9B",
    "mappings": {"default": {"default": "s larga con punto arriba"}},
    "category": "Ll"
  }, {"category": "Ll", "key": "1E9F", "mappings": {"default": {"default": "delta pequeña latina"}}}, {
    "key": "0138",
    "mappings": {"default": {"default": "kra"}},
    "category": "Ll"
  }, {"category": "Ll", "key": "017F", "mappings": {"default": {"default": "s larga"}}}, {
    "category": "Ll",
    "key": "0183",
    "mappings": {"default": {"default": "b con barra encima"}}
  }, {"category": "Ll", "key": "0185", "mappings": {"default": {"default": "tono seis"}}}, {
    "category": "Ll",
    "key": "0188",
    "mappings": {"default": {"default": "c con gancho"}}
  }, {"category": "Ll", "key": "018C", "mappings": {"default": {"default": "d con barra encima"}}}, {
    "key": "0192",
    "mappings": {"default": {"default": "f con gancho"}},
    "category": "Ll"
  }, {"category": "Ll", "key": "0195", "mappings": {"default": {"default": "hv"}}}, {
    "category": "Ll",
    "key": "0199",
    "mappings": {"default": {"default": "k con gancho"}}
  }, {"key": "019A", "mappings": {"default": {"default": "l con barra"}}, "category": "Ll"}, {
    "category": "Ll",
    "key": "019E",
    "mappings": {"default": {"default": "n con pierna derecha larga"}}
  }, {"category": "Ll", "key": "01A1", "mappings": {"default": {"default": "o con cuerno"}}}, {
    "category": "Ll",
    "key": "01A3",
    "mappings": {"default": {"default": "oi"}}
  }, {"category": "Ll", "key": "01A5", "mappings": {"default": {"default": "p con gancho"}}}, {
    "category": "Ll",
    "key": "01A8",
    "mappings": {"default": {"default": "dos tonos"}}
  }, {
    "category": "Ll",
    "key": "01AA",
    "mappings": {"default": {"default": "latin letter invertido Esh bucle"}}
  }, {"category": "Ll", "key": "01AB", "mappings": {"default": {"default": "t con palatal gancho"}}}, {
    "category": "Ll",
    "key": "01AD",
    "mappings": {"default": {"default": "t con gancho"}}
  }, {"category": "Ll", "key": "01B0", "mappings": {"default": {"default": "u con horn"}}}, {
    "category": "Ll",
    "key": "01B4",
    "mappings": {"default": {"default": "y con gancho"}}
  }, {"category": "Ll", "key": "01B9", "mappings": {"default": {"default": "ezh invertido"}}}, {
    "category": "Ll",
    "key": "01BA",
    "mappings": {"default": {"default": "ezh con cola"}}
  }, {"category": "Ll", "key": "01BD", "mappings": {"default": {"default": "tono cinco"}}}, {
    "category": "Ll",
    "key": "01BF",
    "mappings": {"default": {"default": "wynn"}}
  }, {"category": "Ll", "key": "01C6", "mappings": {"default": {"default": "dz con acento"}}}, {
    "category": "Ll",
    "key": "01C9",
    "mappings": {"default": {"default": "lj"}}
  }, {"category": "Ll", "key": "01CC", "mappings": {"default": {"default": "nj"}}}, {
    "category": "Ll",
    "key": "01E3",
    "mappings": {"default": {"default": "ae con macron"}}
  }, {"category": "Ll", "key": "01EF", "mappings": {"default": {"default": "ezh con acento"}}}, {
    "category": "Ll",
    "key": "01F3",
    "mappings": {"default": {"default": "dz"}}
  }, {
    "category": "Ll",
    "key": "021D",
    "mappings": {"default": {"default": "latín letra pequeña yogh"}}
  }, {
    "category": "Ll",
    "key": "026E",
    "mappings": {"default": {"default": "lezh letra pequeña latina"}}
  }, {"category": "Ll", "key": "0292", "mappings": {"default": {"default": "ezh"}}}, {
    "category": "Ll",
    "key": "0293",
    "mappings": {"default": {"default": "pequeña Ezh con Curl"}}
  }, {
    "category": "Ll",
    "key": "02A4",
    "mappings": {"default": {"default": "latín letra pequeña Dezh Digraph"}}
  }, {"category": "Ll", "key": "01DD", "mappings": {"default": {"default": "e convertido"}}}, {
    "category": "Ll",
    "key": "01FD",
    "mappings": {"default": {"default": "ae con acento agudo"}}
  }, {"category": "Ll", "key": "0221", "mappings": {"default": {"default": "d con rizo"}}}, {
    "category": "Ll",
    "key": "0223",
    "mappings": {"default": {"default": "ou"}}
  }, {"category": "Ll", "key": "0225", "mappings": {"default": {"default": "z con gancho"}}}, {
    "category": "Ll",
    "key": "0234",
    "mappings": {"default": {"default": "l con rizo"}}
  }, {"category": "Ll", "key": "0235", "mappings": {"default": {"default": "n con rizo"}}}, {
    "category": "Ll",
    "key": "0236",
    "mappings": {"default": {"default": "t con rizo"}}
  }, {
    "category": "Ll",
    "key": "0238",
    "mappings": {"default": {"default": "letra pequeña latina db digraph"}}
  }, {
    "category": "Ll",
    "key": "0239",
    "mappings": {"default": {"default": "latín letra pequeña Qp Digraph"}}
  }, {"category": "Ll", "key": "023F", "mappings": {"default": {"default": "s con cola cruzada"}}}, {
    "category": "Ll",
    "key": "0240",
    "mappings": {"default": {"default": "z con cola swash"}}
  }, {
    "category": "Ll",
    "key": "0242",
    "mappings": {"default": {"default": "latín letra pequeña parada glotal"}}
  }, {"category": "Ll", "key": "024B", "mappings": {"default": {"default": "q con cola de gancho"}}}, {
    "category": "Ll",
    "key": "0250",
    "mappings": {"default": {"default": "convertida en una"}}
  }, {
    "category": "Ll",
    "key": "0251",
    "mappings": {"default": {"default": "alfa minúscula latina"}}
  }, {"category": "Ll", "key": "0252", "mappings": {"default": {"default": "convertida en alfa"}}}, {
    "category": "Ll",
    "key": "0253",
    "mappings": {"default": {"default": "b con gancho"}}
  }, {"category": "Ll", "key": "0254", "mappings": {"default": {"default": "abierta o"}}}, {
    "category": "Ll",
    "key": "0255",
    "mappings": {"default": {"default": "c con rizo"}}
  }, {"category": "Ll", "key": "0256", "mappings": {"default": {"default": "d con cola"}}}, {
    "category": "Ll",
    "key": "0257",
    "mappings": {"default": {"default": "d con gancho"}}
  }, {"category": "Ll", "key": "0258", "mappings": {"default": {"default": "invertida E"}}}, {
    "category": "Ll",
    "key": "0259",
    "mappings": {"default": {"default": "schwa"}}
  }, {
    "category": "Ll",
    "key": "025A",
    "mappings": {"default": {"default": "pequeña Schwa con gancho"}}
  }, {"key": "025B", "mappings": {"default": {"default": "e abierta"}}, "category": "Ll"}, {
    "category": "Ll",
    "key": "025C",
    "mappings": {"default": {"default": "invertida abierta E"}}
  }, {
    "category": "Ll",
    "key": "025D",
    "mappings": {"default": {"default": "letra pequeña latina invertida E abierta con gancho"}}
  }, {
    "category": "Ll",
    "key": "025E",
    "mappings": {"default": {"default": "cerrada invertida abierta E"}}
  }, {"category": "Ll", "key": "0260", "mappings": {"default": {"default": "g con gancho"}}}, {
    "category": "Ll",
    "key": "0261",
    "mappings": {"default": {"default": "escritura latina de letras pequeñas G"}}
  }, {
    "category": "Ll",
    "key": "0263",
    "mappings": {"default": {"default": "latín letra pequeña gamma"}}
  }, {
    "category": "Ll",
    "key": "0264",
    "mappings": {"default": {"default": "cuerno de carnero de letra pequeña latina"}}
  }, {"category": "Ll", "key": "0265", "mappings": {"default": {"default": "girada h"}}}, {
    "category": "Ll",
    "key": "0266",
    "mappings": {"default": {"default": "h con gancho"}}
  }, {
    "category": "Ll",
    "key": "0267",
    "mappings": {"default": {"default": "pequeña letra latina Heng con gancho"}}
  }, {
    "category": "Ll",
    "key": "0269",
    "mappings": {"default": {"default": "latín letra minúscula Iota"}}
  }, {"category": "Ll", "key": "026B", "mappings": {"default": {"default": "l con tilde medio"}}}, {
    "category": "Ll",
    "key": "026C",
    "mappings": {"default": {"default": "l con cinturón"}}
  }, {
    "category": "Ll",
    "key": "026D",
    "mappings": {"default": {"default": "letra L latina pequeña con gancho Retroflex"}}
  }, {"category": "Ll", "key": "026F", "mappings": {"default": {"default": "girada m"}}}, {
    "category": "Ll",
    "key": "0270",
    "mappings": {"default": {"default": "vuelta M con pierna larga"}}
  }, {"category": "Ll", "key": "0271", "mappings": {"default": {"default": "m con gancho"}}}, {
    "category": "Ll",
    "key": "0272",
    "mappings": {"default": {"default": "n con gancho izquierdo"}}
  }, {
    "category": "Ll",
    "key": "0273",
    "mappings": {"default": {"default": "n con gancho retroflex"}}
  }, {
    "category": "Ll",
    "key": "0275",
    "mappings": {"default": {"default": "letra minúscula omega"}}
  }, {
    "category": "Ll",
    "key": "0277",
    "mappings": {"default": {"default": "latin Small Letter Closed Omega"}}
  }, {"category": "Ll", "key": "0278", "mappings": {"default": {"default": "Letra pequeña latina"}}}, {
    "category": "Ll",
    "key": "0279",
    "mappings": {"default": {"default": "vuelta r"}}
  }, {
    "category": "Ll",
    "key": "027A",
    "mappings": {"default": {"default": "vuelta R con pierna larga"}}
  }, {"category": "Ll", "key": "027B", "mappings": {"default": {"default": "vuelta R con gancho"}}}, {
    "category": "Ll",
    "key": "027C",
    "mappings": {"default": {"default": "R con pierna larga"}}
  }, {"category": "Ll", "key": "027D", "mappings": {"default": {"default": "R con cola"}}}, {
    "category": "Ll",
    "key": "027E",
    "mappings": {"default": {"default": "R con anzuelo"}}
  }, {
    "category": "Ll",
    "key": "027F",
    "mappings": {"default": {"default": "invertida R con anzuelo"}}
  }, {"category": "Ll", "key": "0282", "mappings": {"default": {"default": "S con gancho"}}}, {
    "category": "Ll",
    "key": "0283",
    "mappings": {"default": {"default": "Pequeña letra latina esh"}}
  }, {
    "category": "Ll",
    "key": "0285",
    "mappings": {"default": {"default": "Latín letra pequeña en cuclillas invertida Esh"}}
  }, {
    "category": "Ll",
    "key": "0286",
    "mappings": {"default": {"default": "Esh minúscula latina con rizo"}}
  }, {"category": "Ll", "key": "0287", "mappings": {"default": {"default": "vuelta t"}}}, {
    "category": "Ll",
    "key": "0288",
    "mappings": {"default": {"default": "T con gancho retroflex"}}
  }, {
    "category": "Ll",
    "key": "0289",
    "mappings": {"default": {"default": "Letra de letra pequeña latina U"}}
  }, {"category": "Ll", "key": "028A", "mappings": {"default": {"default": "Upsilon"}}}, {
    "category": "Ll",
    "key": "028B",
    "mappings": {"default": {"default": "V con gancho"}}
  }, {"category": "Ll", "key": "028C", "mappings": {"default": {"default": "convertida V"}}}, {
    "category": "Ll",
    "key": "028D",
    "mappings": {"default": {"default": "girada w"}}
  }, {"category": "Ll", "key": "028E", "mappings": {"default": {"default": "vuelta y"}}}, {
    "category": "Ll",
    "key": "0290",
    "mappings": {"default": {"default": "Z con gancho retroflex"}}
  }, {"category": "Ll", "key": "0291", "mappings": {"default": {"default": "z con rizo"}}}, {
    "category": "Ll",
    "key": "0295",
    "mappings": {"default": {"default": "faríngea con voz fricativa"}}
  }, {
    "category": "Ll",
    "key": "0296",
    "mappings": {"default": {"default": "invertida parada glotal"}}
  }, {"category": "Ll", "key": "0297", "mappings": {"default": {"default": "estirada c"}}}, {
    "category": "Ll",
    "key": "0298",
    "mappings": {"default": {"default": "Carta Bilabial Click"}}
  }, {"category": "Ll", "key": "029A", "mappings": {"default": {"default": "cerrada abierta e"}}}, {
    "category": "Ll",
    "key": "029E",
    "mappings": {"default": {"default": "vuelta k"}}
  }, {"category": "Ll", "key": "02A0", "mappings": {"default": {"default": "q con gancho"}}}, {
    "category": "Ll",
    "key": "02A3",
    "mappings": {"default": {"default": "pequeña letra Dz Digraph"}}
  }, {
    "category": "Ll",
    "key": "02A5",
    "mappings": {"default": {"default": "Letra pequeña latina Dz Digraph con Curl"}}
  }, {
    "category": "Ll",
    "key": "02A6",
    "mappings": {"default": {"default": "Latín letra pequeña Ts Digraph"}}
  }, {
    "category": "Ll",
    "key": "02A7",
    "mappings": {"default": {"default": "Letra pequeña latina Tesh Digraph"}}
  }, {
    "category": "Ll",
    "key": "02A8",
    "mappings": {"default": {"default": "Latín letra pequeña Tc Digraph con Curl"}}
  }, {
    "category": "Ll",
    "key": "02A9",
    "mappings": {"default": {"default": "Letra pequeña latina Feng Digraph"}}
  }, {
    "category": "Ll",
    "key": "02AA",
    "mappings": {"default": {"default": "pequeña letra Ls Digraph"}}
  }, {
    "category": "Ll",
    "key": "02AB",
    "mappings": {"default": {"default": "pequeña letra Lz Digraph"}}
  }, {"category": "Ll", "key": "02AC", "mappings": {"default": {"default": "percusion bilabial"}}}, {
    "category": "Ll",
    "key": "02AD",
    "mappings": {"default": {"default": "de percusión bidental"}}
  }, {
    "category": "Ll",
    "key": "02AE",
    "mappings": {"default": {"default": "convertida en H con anzuelo"}}
  }, {
    "category": "Ll",
    "key": "02AF",
    "mappings": {"default": {"default": "convertida en H con anzuelo y cola"}}
  }, {"category": "Ll", "key": "1D02", "mappings": {"default": {"default": "vuelta ae"}}}, {
    "category": "Ll",
    "key": "1D08",
    "mappings": {"default": {"default": "convertida abierta E"}}
  }, {"category": "Ll", "key": "1D09", "mappings": {"default": {"default": "convertida i"}}}, {
    "category": "Ll",
    "key": "1D11",
    "mappings": {"default": {"default": "de lado O"}}
  }, {"category": "Ll", "key": "1D12", "mappings": {"default": {"default": "de lado abierto O"}}}, {
    "category": "Ll",
    "key": "1D14",
    "mappings": {"default": {"default": "vuelta oe"}}
  }, {"category": "Ll", "key": "1D16", "mappings": {"default": {"default": "mitad superior O"}}}, {
    "category": "Ll",
    "key": "1D17",
    "mappings": {"default": {"default": "Letra latina, letra inferior, mitad inferior O"}}
  }, {"category": "Ll", "key": "1D1D", "mappings": {"default": {"default": "hacia los lados U"}}}, {
    "category": "Ll",
    "key": "1D1E",
    "mappings": {"default": {"default": "de lado diaeresizada U"}}
  }, {
    "category": "Ll",
    "key": "1D1F",
    "mappings": {"default": {"default": "girada hacia los lados M"}}
  }, {
    "category": "Ll",
    "key": "1D24",
    "mappings": {"default": {"default": "Carta latina expresada espirante laríngeo"}}
  }, {"category": "Ll", "key": "1D25", "mappings": {"default": {"default": "ain"}}}, {
    "category": "Ll",
    "key": "1D6B",
    "mappings": {"default": {"default": "ue"}}
  }, {"category": "Ll", "key": "1D6C", "mappings": {"default": {"default": "B con tilde medio"}}}, {
    "category": "Ll",
    "key": "1D6D",
    "mappings": {"default": {"default": "D con tilde medio"}}
  }, {"category": "Ll", "key": "1D6E", "mappings": {"default": {"default": "F con tilde medio"}}}, {
    "category": "Ll",
    "key": "1D6F",
    "mappings": {"default": {"default": "M con tilde medio"}}
  }, {"category": "Ll", "key": "1D70", "mappings": {"default": {"default": "N con tilde medio"}}}, {
    "category": "Ll",
    "key": "1D71",
    "mappings": {"default": {"default": "P con tilde medio"}}
  }, {"category": "Ll", "key": "1D72", "mappings": {"default": {"default": "R con tilde medio"}}}, {
    "category": "Ll",
    "key": "1D73",
    "mappings": {"default": {"default": "R con anzuelo y tilde medio"}}
  }, {"category": "Ll", "key": "1D74", "mappings": {"default": {"default": "S con tilde medio"}}}, {
    "category": "Ll",
    "key": "1D75",
    "mappings": {"default": {"default": "T con tilde medio"}}
  }, {"category": "Ll", "key": "1D76", "mappings": {"default": {"default": "z con tilde media"}}}, {
    "category": "Ll",
    "key": "1D77",
    "mappings": {"default": {"default": "girada g"}}
  }, {"category": "Ll", "key": "1D79", "mappings": {"default": {"default": "insular g"}}}, {
    "category": "Ll",
    "key": "1D7A",
    "mappings": {"default": {"default": "th con tachado"}}
  }, {"category": "Ll", "key": "1D80", "mappings": {"default": {"default": "B con gancho palatal"}}}, {
    "category": "Ll",
    "key": "1D81",
    "mappings": {"default": {"default": "D con gancho palatal"}}
  }, {
    "category": "Ll",
    "key": "1D82",
    "mappings": {"default": {"default": "F con gancho palatino"}}
  }, {"category": "Ll", "key": "1D83", "mappings": {"default": {"default": "G con gancho palatal"}}}, {
    "category": "Ll",
    "key": "1D84",
    "mappings": {"default": {"default": "K con gancho palatino"}}
  }, {
    "category": "Ll",
    "key": "1D85",
    "mappings": {"default": {"default": "L con gancho palatino"}}
  }, {
    "category": "Ll",
    "key": "1D86",
    "mappings": {"default": {"default": "M con gancho palatino"}}
  }, {"category": "Ll", "key": "1D87", "mappings": {"default": {"default": "N con gancho palatal"}}}, {
    "category": "Ll",
    "key": "1D88",
    "mappings": {"default": {"default": "P con gancho palatino"}}
  }, {"category": "Ll", "key": "1D89", "mappings": {"default": {"default": "R con gancho palatal"}}}, {
    "category": "Ll",
    "key": "1D8A",
    "mappings": {"default": {"default": "S con gancho palatal"}}
  }, {
    "category": "Ll",
    "key": "1D8B",
    "mappings": {"default": {"default": "Pequeña letra latina Esh con gancho palatal"}}
  }, {"category": "Ll", "key": "1D8C", "mappings": {"default": {"default": "V con gancho palatal"}}}, {
    "category": "Ll",
    "key": "1D8D",
    "mappings": {"default": {"default": "X con gancho palatal"}}
  }, {"category": "Ll", "key": "1D8E", "mappings": {"default": {"default": "Z con gancho palatal"}}}, {
    "category": "Ll",
    "key": "1D8F",
    "mappings": {"default": {"default": "a con gancho retroflex"}}
  }, {
    "category": "Ll",
    "key": "1D90",
    "mappings": {"default": {"default": "Alfa latina pequeña letra con gancho retroflex"}}
  }, {"category": "Ll", "key": "1D91", "mappings": {"default": {"default": "D con gancho y cola"}}}, {
    "category": "Ll",
    "key": "1D92",
    "mappings": {"default": {"default": "pequeña E con gancho retroflex"}}
  }, {
    "category": "Ll",
    "key": "1D93",
    "mappings": {"default": {"default": "E Pequeña Open con Gancho Retroflex"}}
  }, {
    "category": "Ll",
    "key": "1D94",
    "mappings": {"default": {"default": "E abierta invertido con gancho retroflex"}}
  }, {
    "category": "Ll",
    "key": "1D95",
    "mappings": {"default": {"default": "pequeña Schwa con gancho retroflex"}}
  }, {
    "category": "Ll",
    "key": "1D96",
    "mappings": {"default": {"default": "I con gancho retroflex"}}
  }, {
    "category": "Ll",
    "key": "1D97",
    "mappings": {"default": {"default": "pequeña latina Open O con gancho retroflex"}}
  }, {
    "category": "Ll",
    "key": "1D98",
    "mappings": {"default": {"default": "Esh de letra pequeña latina con gancho retroflex"}}
  }, {
    "category": "Ll",
    "key": "1D99",
    "mappings": {"default": {"default": "U con gancho retroflex"}}
  }, {
    "category": "Ll",
    "key": "1D9A",
    "mappings": {"default": {"default": "pequeña Ezh con gancho retroflex"}}
  }, {
    "key": "0149",
    "mappings": {"default": {"default": "n precedida por apóstrofe"}},
    "category": "Ll"
  }, {"key": "014B", "mappings": {"default": {"default": "eng"}}, "category": "Ll"}]
    ,
    "es/symbols/latin-lower-single-accent.js"
:
  [{"locale": "es"}, {"key": "00E0", "mappings": {"default": {"default": "a grave"}}, "category": "Ll"}, {
    "key": "00E1",
    "mappings": {"default": {"default": "a aguda"}},
    "category": "Ll"
  }, {"key": "00E2", "mappings": {"default": {"default": "a acento circunflejo"}}, "category": "Ll"}, {
    "key": "00E3",
    "mappings": {"default": {"default": "a con tilde"}},
    "category": "Ll"
  }, {"key": "00E4", "mappings": {"default": {"default": "a diéresis"}}, "category": "Ll"}, {
    "key": "00E5",
    "mappings": {"default": {"default": "a con anillo"}},
    "category": "Ll"
  }, {"key": "00E7", "mappings": {"default": {"default": "c cedilla"}}, "category": "Ll"}, {
    "key": "00E8",
    "mappings": {"default": {"default": "e grave"}},
    "category": "Ll"
  }, {"key": "00E9", "mappings": {"default": {"default": "e aguda"}}, "category": "Ll"}, {
    "key": "00EA",
    "mappings": {"default": {"default": "e acento circunflejo"}},
    "category": "Ll"
  }, {"key": "00EB", "mappings": {"default": {"default": "e diéresis"}}, "category": "Ll"}, {
    "key": "00EC",
    "mappings": {"default": {"default": "i grave"}},
    "category": "Ll"
  }, {"key": "00ED", "mappings": {"default": {"default": "i aguda"}}, "category": "Ll"}, {
    "key": "00EE",
    "mappings": {"default": {"default": "i acento circunflejo"}},
    "category": "Ll"
  }, {"key": "00EF", "mappings": {"default": {"default": "i diéresis"}}, "category": "Ll"}, {
    "key": "00F1",
    "mappings": {"default": {"default": "eñe"}},
    "category": "Ll"
  }, {"key": "00F2", "mappings": {"default": {"default": "o grave"}}, "category": "Ll"}, {
    "key": "00F3",
    "mappings": {"default": {"default": "o aguda"}},
    "category": "Ll"
  }, {"key": "00F4", "mappings": {"default": {"default": "o acento circunflejo"}}, "category": "Ll"}, {
    "key": "00F5",
    "mappings": {"default": {"default": "o con tilde"}},
    "category": "Ll"
  }, {"key": "00F6", "mappings": {"default": {"default": "o diéresis"}}, "category": "Ll"}, {
    "key": "00F9",
    "mappings": {"default": {"default": "u grave"}},
    "category": "Ll"
  }, {"key": "00FA", "mappings": {"default": {"default": "u aguda"}}, "category": "Ll"}, {
    "key": "00FB",
    "mappings": {"default": {"default": "u acento circunflejo"}},
    "category": "Ll"
  }, {"key": "00FC", "mappings": {"default": {"default": "u diéresis"}}, "category": "Ll"}, {
    "key": "00FD",
    "mappings": {"default": {"default": "Y aguda", "defaultMP": "ye aguda"}},
    "category": "Ll"
  }, {
    "key": "00FF",
    "mappings": {"default": {"default": "y diéresis", "defaultMP": "ye diéresis"}},
    "category": "Ll"
  }, {"key": "0101", "mappings": {"default": {"default": "a barra"}}, "category": "Ll"}, {
    "key": "0103",
    "mappings": {"default": {"default": "a con breve"}},
    "category": "Ll"
  }, {"key": "0105", "mappings": {"default": {"default": "a con ogonek"}}, "category": "Ll"}, {
    "key": "0107",
    "mappings": {"default": {"default": "c con agudo"}},
    "category": "Ll"
  }, {
    "key": "0109",
    "mappings": {"default": {"default": "c con acento circunflejo"}},
    "category": "Ll"
  }, {"key": "010B", "mappings": {"default": {"default": "c con punto arriba"}}, "category": "Ll"}, {
    "key": "010D",
    "mappings": {"default": {"default": "c con carón"}},
    "category": "Ll"
  }, {"key": "010F", "mappings": {"default": {"default": "d con carón"}}, "category": "Ll"}, {
    "key": "0113",
    "mappings": {"default": {"default": "e barra"}},
    "category": "Ll"
  }, {"category": "Ll", "key": "0115", "mappings": {"default": {"default": "e con breve"}}}, {
    "key": "0117",
    "mappings": {"default": {"default": "e con punto arriba"}},
    "category": "Ll"
  }, {"key": "0119", "mappings": {"default": {"default": "e con ogonek"}}, "category": "Ll"}, {
    "key": "011B",
    "mappings": {"default": {"default": "e con carón"}},
    "category": "Ll"
  }, {
    "key": "011D",
    "mappings": {"default": {"default": "g con acento circunflejo"}},
    "category": "Ll"
  }, {"key": "011F", "mappings": {"default": {"default": "g con breve"}}, "category": "Ll"}, {
    "key": "0121",
    "mappings": {"default": {"default": "g con punto arriba"}},
    "category": "Ll"
  }, {"category": "Ll", "key": "0123", "mappings": {"default": {"default": "g con cedilla"}}}, {
    "key": "0125",
    "mappings": {"default": {"default": "h con acento circunflejo"}},
    "category": "Ll"
  }, {"key": "0129", "mappings": {"default": {"default": "i con tilde"}}, "category": "Ll"}, {
    "key": "012B",
    "mappings": {"default": {"default": "i barra"}},
    "category": "Ll"
  }, {"category": "Ll", "key": "012D", "mappings": {"default": {"default": "i con breve"}}}, {
    "key": "012F",
    "mappings": {"default": {"default": "i con ogonek"}},
    "category": "Ll"
  }, {"key": "0131", "mappings": {"default": {"default": "i sin punto"}}, "category": "Ll"}, {
    "key": "0135",
    "mappings": {"default": {"default": "j con acento circunflejo"}},
    "category": "Ll"
  }, {"key": "0137", "mappings": {"default": {"default": "k con cedilla"}}, "category": "Ll"}, {
    "key": "013A",
    "mappings": {"default": {"default": "l con agudo"}},
    "category": "Ll"
  }, {"key": "013C", "mappings": {"default": {"default": "l con cedilla"}}, "category": "Ll"}, {
    "key": "013E",
    "mappings": {"default": {"default": "l con carón"}},
    "category": "Ll"
  }, {"key": "0140", "mappings": {"default": {"default": "l con punto mediano"}}, "category": "Ll"}, {
    "key": "0144",
    "mappings": {"default": {"default": "n con agudo"}},
    "category": "Ll"
  }, {"key": "0146", "mappings": {"default": {"default": "n con cedilla"}}, "category": "Ll"}, {
    "key": "0148",
    "mappings": {"default": {"default": "n con carón"}},
    "category": "Ll"
  }, {"key": "014D", "mappings": {"default": {"default": "o barra"}}, "category": "Ll"}, {
    "category": "Ll",
    "key": "014F",
    "mappings": {"default": {"default": "o con breve"}}
  }, {
    "key": "0151",
    "mappings": {"default": {"default": "o con doble acento agudo"}},
    "category": "Ll"
  }, {"key": "0155", "mappings": {"default": {"default": "r con agudo"}}, "category": "Ll"}, {
    "key": "0157",
    "mappings": {"default": {"default": "r con cedilla"}},
    "category": "Ll"
  }, {"key": "0159", "mappings": {"default": {"default": "r con carón"}}, "category": "Ll"}, {
    "key": "015B",
    "mappings": {"default": {"default": "s con agudo"}},
    "category": "Ll"
  }, {
    "key": "015D",
    "mappings": {"default": {"default": "s con acento circunflejo"}},
    "category": "Ll"
  }, {"key": "015F", "mappings": {"default": {"default": "s con cedilla"}}, "category": "Ll"}, {
    "key": "0161",
    "mappings": {"default": {"default": "s con carón"}},
    "category": "Ll"
  }, {"key": "0163", "mappings": {"default": {"default": "t con cedilla"}}, "category": "Ll"}, {
    "key": "0165",
    "mappings": {"default": {"default": "t con carón"}},
    "category": "Ll"
  }, {"key": "0169", "mappings": {"default": {"default": "u con tilde"}}, "category": "Ll"}, {
    "key": "016B",
    "mappings": {"default": {"default": "u barra"}},
    "category": "Ll"
  }, {"key": "016D", "mappings": {"default": {"default": "u con breve"}}, "category": "Ll"}, {
    "key": "016F",
    "mappings": {"default": {"default": "u con anillo arriba"}},
    "category": "Ll"
  }, {
    "key": "0171",
    "mappings": {"default": {"default": "u con doble acento agudo"}},
    "category": "Ll"
  }, {"key": "0173", "mappings": {"default": {"default": "u con ogonek"}}, "category": "Ll"}, {
    "key": "0175",
    "mappings": {"default": {"default": "w con circunflejo"}},
    "category": "Ll"
  }, {"key": "0177", "mappings": {"default": {"default": "y con circunflejo"}}, "category": "Ll"}, {
    "key": "017A",
    "mappings": {"default": {"default": "z con agudo"}},
    "category": "Ll"
  }, {"key": "017C", "mappings": {"default": {"default": "z con punto"}}, "category": "Ll"}, {
    "key": "017E",
    "mappings": {"default": {"default": "z con carón"}},
    "category": "Ll"
  }, {"category": "Ll", "key": "01CE", "mappings": {"default": {"default": "a con acento"}}}, {
    "category": "Ll",
    "key": "01D0",
    "mappings": {"default": {"default": "i con acento"}}
  }, {"category": "Ll", "key": "01D2", "mappings": {"default": {"default": "o con acento"}}}, {
    "category": "Ll",
    "key": "01D4",
    "mappings": {"default": {"default": "u con acento"}}
  }, {"category": "Ll", "key": "01E7", "mappings": {"default": {"default": "g con acento"}}}, {
    "category": "Ll",
    "key": "01E9",
    "mappings": {"default": {"default": "k con acento"}}
  }, {"category": "Ll", "key": "01EB", "mappings": {"default": {"default": "o con pecíolo"}}}, {
    "category": "Ll",
    "key": "01F0",
    "mappings": {"default": {"default": "j con acento"}}
  }, {"key": "01F5", "mappings": {"default": {"default": "g con agudo"}}, "category": "Ll"}, {
    "category": "Ll",
    "key": "01F9",
    "mappings": {"default": {"default": "n con grave"}}
  }, {"category": "Ll", "key": "0201", "mappings": {"default": {"default": "a con doble tumba"}}}, {
    "category": "Ll",
    "key": "0203",
    "mappings": {"default": {"default": "a con letras invertidas"}}
  }, {"category": "Ll", "key": "0205", "mappings": {"default": {"default": "e con doble tumba"}}}, {
    "category": "Ll",
    "key": "0207",
    "mappings": {"default": {"default": "e con letras invertidas"}}
  }, {"category": "Ll", "key": "0209", "mappings": {"default": {"default": "i con doble tumba"}}}, {
    "category": "Ll",
    "key": "020B",
    "mappings": {"default": {"default": "i con Breve invertida"}}
  }, {"category": "Ll", "key": "020D", "mappings": {"default": {"default": "o con doble tumba"}}}, {
    "category": "Ll",
    "key": "020F",
    "mappings": {"default": {"default": "o con letras invertidas"}}
  }, {"category": "Ll", "key": "0211", "mappings": {"default": {"default": "r con doble sepulcro"}}}, {
    "category": "Ll",
    "key": "0213",
    "mappings": {"default": {"default": "r con letras invertidas"}}
  }, {"category": "Ll", "key": "0215", "mappings": {"default": {"default": "u con doble tumba"}}}, {
    "category": "Ll",
    "key": "0217",
    "mappings": {"default": {"default": "u con letras invertidas"}}
  }, {"category": "Ll", "key": "0219", "mappings": {"default": {"default": "s con coma debajo"}}}, {
    "category": "Ll",
    "key": "021B",
    "mappings": {"default": {"default": "t con coma debajo"}}
  }, {"category": "Ll", "key": "021F", "mappings": {"default": {"default": "h con caron"}}}, {
    "category": "Ll",
    "key": "0227",
    "mappings": {"default": {"default": "con un punto arriba"}}
  }, {"category": "Ll", "key": "0229", "mappings": {"default": {"default": "e con cedilla"}}}, {
    "category": "Ll",
    "key": "022F",
    "mappings": {"default": {"default": "o con punto arriba"}}
  }, {"category": "Ll", "key": "0233", "mappings": {"default": {"default": "y con Macron"}}}, {
    "category": "Ll",
    "key": "0237",
    "mappings": {"default": {"default": "s in punto j"}}
  }, {"key": "1E01", "mappings": {"default": {"default": "a con anillo debajo"}}, "category": "Ll"}, {
    "key": "1E03",
    "mappings": {"default": {"default": "b con punto arriba"}},
    "category": "Ll"
  }, {"key": "1E05", "mappings": {"default": {"default": "b con punto debajo"}}, "category": "Ll"}, {
    "key": "1E07",
    "mappings": {"default": {"default": "b con línea debajo"}},
    "category": "Ll"
  }, {"key": "1E0B", "mappings": {"default": {"default": "d con punto arriba"}}, "category": "Ll"}, {
    "key": "1E0D",
    "mappings": {"default": {"default": "d con punto debajo"}},
    "category": "Ll"
  }, {"key": "1E0F", "mappings": {"default": {"default": "d con línea debajo"}}, "category": "Ll"}, {
    "key": "1E11",
    "mappings": {"default": {"default": "d con cedilla"}},
    "category": "Ll"
  }, {
    "key": "1E13",
    "mappings": {"default": {"default": "d con acento circunflejo debajo"}},
    "category": "Ll"
  }, {
    "key": "1E19",
    "mappings": {"default": {"default": "e con acento circunflejo debajo"}},
    "category": "Ll"
  }, {"key": "1E1B", "mappings": {"default": {"default": "e con tilde debajo"}}, "category": "Ll"}, {
    "key": "1E1F",
    "mappings": {"default": {"default": "f con punto arriba"}},
    "category": "Ll"
  }, {"key": "1E21", "mappings": {"default": {"default": "g con macrón"}}, "category": "Ll"}, {
    "key": "1E23",
    "mappings": {"default": {"default": "h con punto arriba"}},
    "category": "Ll"
  }, {"key": "1E25", "mappings": {"default": {"default": "h con punto debajo"}}, "category": "Ll"}, {
    "key": "1E27",
    "mappings": {"default": {"default": "h con diéresis"}},
    "category": "Ll"
  }, {"key": "1E29", "mappings": {"default": {"default": "h con cedilla"}}, "category": "Ll"}, {
    "key": "1E2B",
    "mappings": {"default": {"default": "h con breve debajo"}},
    "category": "Ll"
  }, {"key": "1E2D", "mappings": {"default": {"default": "i con tilde debajo"}}, "category": "Ll"}, {
    "key": "1E31",
    "mappings": {"default": {"default": "k con agudo"}},
    "category": "Ll"
  }, {"key": "1E33", "mappings": {"default": {"default": "k con punto debajo"}}, "category": "Ll"}, {
    "key": "1E35",
    "mappings": {"default": {"default": "k con línea debajo"}},
    "category": "Ll"
  }, {"key": "1E37", "mappings": {"default": {"default": "l con punto debajo"}}, "category": "Ll"}, {
    "key": "1E3B",
    "mappings": {"default": {"default": "l con línea debajo"}},
    "category": "Ll"
  }, {
    "key": "1E3D",
    "mappings": {"default": {"default": "l con acento circunflejo debajo"}},
    "category": "Ll"
  }, {"key": "1E3F", "mappings": {"default": {"default": "m con agudo"}}, "category": "Ll"}, {
    "key": "1E41",
    "mappings": {"default": {"default": "m con punto arriba"}},
    "category": "Ll"
  }, {"key": "1E43", "mappings": {"default": {"default": "m con punto debajo"}}, "category": "Ll"}, {
    "key": "1E45",
    "mappings": {"default": {"default": "n con punto arriba"}},
    "category": "Ll"
  }, {"key": "1E47", "mappings": {"default": {"default": "n con punto debajo"}}, "category": "Ll"}, {
    "key": "1E49",
    "mappings": {"default": {"default": "n con línea debajo"}},
    "category": "Ll"
  }, {
    "key": "1E4B",
    "mappings": {"default": {"default": "n con acento circunflejo debajo"}},
    "category": "Ll"
  }, {"key": "1E55", "mappings": {"default": {"default": "p con agudo"}}, "category": "Ll"}, {
    "key": "1E57",
    "mappings": {"default": {"default": "p con punto arriba"}},
    "category": "Ll"
  }, {"key": "1E59", "mappings": {"default": {"default": "r con punto arriba"}}, "category": "Ll"}, {
    "key": "1E5B",
    "mappings": {"default": {"default": "r con punto debajo"}},
    "category": "Ll"
  }, {"key": "1E5F", "mappings": {"default": {"default": "r con línea debajo"}}, "category": "Ll"}, {
    "key": "1E61",
    "mappings": {"default": {"default": "s con punto arriba"}},
    "category": "Ll"
  }, {"key": "1E63", "mappings": {"default": {"default": "s con punto debajo"}}, "category": "Ll"}, {
    "key": "1E6B",
    "mappings": {"default": {"default": "t con punto arriba"}},
    "category": "Ll"
  }, {"key": "1E6D", "mappings": {"default": {"default": "t con punto debajo"}}, "category": "Ll"}, {
    "key": "1E6F",
    "mappings": {"default": {"default": "t con línea debajo"}},
    "category": "Ll"
  }, {
    "key": "1E71",
    "mappings": {"default": {"default": "t con acento circunflejo debajo"}},
    "category": "Ll"
  }, {"key": "1E73", "mappings": {"default": {"default": "u con diéresis debajo"}}, "category": "Ll"}, {
    "key": "1E75",
    "mappings": {"default": {"default": "u con tilde debajo"}},
    "category": "Ll"
  }, {
    "key": "1E77",
    "mappings": {"default": {"default": "u con acento circunflejo debajo"}},
    "category": "Ll"
  }, {"key": "1E7D", "mappings": {"default": {"default": "v con tilde"}}, "category": "Ll"}, {
    "key": "1E7F",
    "mappings": {"default": {"default": "v con punto debajo"}},
    "category": "Ll"
  }, {"key": "1E81", "mappings": {"default": {"default": "w con grave"}}, "category": "Ll"}, {
    "key": "1E83",
    "mappings": {"default": {"default": "w con agudo"}},
    "category": "Ll"
  }, {"key": "1E85", "mappings": {"default": {"default": "w con diéresis"}}, "category": "Ll"}, {
    "key": "1E87",
    "mappings": {"default": {"default": "w con punto arriba"}},
    "category": "Ll"
  }, {"key": "1E89", "mappings": {"default": {"default": "w con punto debajo"}}, "category": "Ll"}, {
    "key": "1E8B",
    "mappings": {"default": {"default": "x con punto arriba"}},
    "category": "Ll"
  }, {"key": "1E8D", "mappings": {"default": {"default": "x con diéresis"}}, "category": "Ll"}, {
    "key": "1E8F",
    "mappings": {"default": {"default": "y con punto arriba"}},
    "category": "Ll"
  }, {
    "key": "1E91",
    "mappings": {"default": {"default": "z con acento circunflejo"}},
    "category": "Ll"
  }, {"key": "1E93", "mappings": {"default": {"default": "z con punto debajo"}}, "category": "Ll"}, {
    "key": "1E95",
    "mappings": {"default": {"default": "z con línea debajo"}},
    "category": "Ll"
  }, {"key": "1E96", "mappings": {"default": {"default": "h con línea debajo"}}, "category": "Ll"}, {
    "key": "1E97",
    "mappings": {"default": {"default": "t con diéresis"}},
    "category": "Ll"
  }, {"key": "1E98", "mappings": {"default": {"default": "w con anillo arriba"}}, "category": "Ll"}, {
    "key": "1E99",
    "mappings": {"default": {"default": "y con anillo arriba"}},
    "category": "Ll"
  }, {
    "key": "1E9A",
    "mappings": {"default": {"default": "a con medio derecho de anillo"}},
    "category": "Ll"
  }, {"key": "1EA1", "mappings": {"default": {"default": "a con punto debajo"}}, "category": "Ll"}, {
    "key": "1EA3",
    "mappings": {"default": {"default": "a con gancho arriba"}},
    "category": "Ll"
  }, {"key": "1EB9", "mappings": {"default": {"default": "e con punto debajo"}}, "category": "Ll"}, {
    "key": "1EBB",
    "mappings": {"default": {"default": "e con gancho arriba"}},
    "category": "Ll"
  }, {"key": "1EBD", "mappings": {"default": {"default": "e con tilde"}}, "category": "Ll"}, {
    "key": "1EC9",
    "mappings": {"default": {"default": "i con gancho arriba"}},
    "category": "Ll"
  }, {"key": "1ECB", "mappings": {"default": {"default": "i con punto debajo"}}, "category": "Ll"}, {
    "key": "1ECD",
    "mappings": {"default": {"default": "o con punto debajo"}},
    "category": "Ll"
  }, {"key": "1ECF", "mappings": {"default": {"default": "o con gancho arriba"}}, "category": "Ll"}, {
    "key": "1EE5",
    "mappings": {"default": {"default": "u con punto debajo"}},
    "category": "Ll"
  }, {"key": "1EE7", "mappings": {"default": {"default": "u con gancho arriba"}}, "category": "Ll"}, {
    "key": "1EF3",
    "mappings": {"default": {"default": "y con grave"}},
    "category": "Ll"
  }, {"key": "1EF5", "mappings": {"default": {"default": "y con punto debajo"}}, "category": "Ll"}, {
    "key": "1EF7",
    "mappings": {"default": {"default": "y con gancho arriba"}},
    "category": "Ll"
  }, {"key": "1EF9", "mappings": {"default": {"default": "y con tilde"}}, "category": "Ll"}]
    ,
    "es/symbols/latin-rest.js"
:
  [{"locale": "es"}, {
    "category": "Ll",
    "key": "210E",
    "mappings": {"default": {"default": "constante de planck"}}
  }, {"category": "Mn", "key": "0363", "mappings": {"default": {"default": "combinando a"}}}, {
    "category": "Mn",
    "key": "0364",
    "mappings": {"default": {"default": "combinando e"}}
  }, {"category": "Mn", "key": "0365", "mappings": {"default": {"default": "combinando i"}}}, {
    "category": "Mn",
    "key": "0366",
    "mappings": {"default": {"default": "combinando o"}}
  }, {"category": "Mn", "key": "0367", "mappings": {"default": {"default": "combinando u"}}}, {
    "category": "Mn",
    "key": "0368",
    "mappings": {"default": {"default": "combinando c"}}
  }, {"category": "Mn", "key": "0369", "mappings": {"default": {"default": "combinando d"}}}, {
    "category": "Mn",
    "key": "036A",
    "mappings": {"default": {"default": "combinando h"}}
  }, {"category": "Mn", "key": "036B", "mappings": {"default": {"default": "combinando m"}}}, {
    "category": "Mn",
    "key": "036C",
    "mappings": {"default": {"default": "combinando r"}}
  }, {"category": "Mn", "key": "036D", "mappings": {"default": {"default": "combinando t"}}}, {
    "category": "Mn",
    "key": "036E",
    "mappings": {"default": {"default": "combinando v"}}
  }, {"category": "Mn", "key": "036F", "mappings": {"default": {"default": "combinando x"}}}, {
    "category": "Lm",
    "key": "1D62",
    "mappings": {"default": {"default": "subíndice i"}}
  }, {"category": "Lm", "key": "1D63", "mappings": {"default": {"default": "subíndice r"}}}, {
    "category": "Lm",
    "key": "1D64",
    "mappings": {"default": {"default": "subíndice u"}}
  }, {
    "category": "Lm",
    "key": "1D65",
    "mappings": {"default": {"default": "subíndice latino letra pequeña V"}}
  }, {"category": "Mn", "key": "1DCA", "mappings": {"default": {"default": "combinando r abajo"}}}, {
    "category": "Mn",
    "key": "1DD3",
    "mappings": {"default": {"default": "combinando letra minúscula en latín, aplanado, abierto arriba"}}
  }, {"category": "Mn", "key": "1DD4", "mappings": {"default": {"default": "combinando ae"}}}, {
    "category": "Mn",
    "key": "1DD5",
    "mappings": {"default": {"default": "combinando ao"}}
  }, {"category": "Mn", "key": "1DD6", "mappings": {"default": {"default": "combinando av"}}}, {
    "category": "Mn",
    "key": "1DD7",
    "mappings": {"default": {"default": "combinando Letra Pequeña Latina C Cedilla"}}
  }, {"category": "Mn", "key": "1DD8", "mappings": {"default": {"default": "combinando insular d"}}}, {
    "category": "Mn",
    "key": "1DD9",
    "mappings": {"default": {"default": "combinando la eth"}}
  }, {"category": "Mn", "key": "1DDA", "mappings": {"default": {"default": "combinando g"}}}, {
    "category": "Mn",
    "key": "1DDB",
    "mappings": {"default": {"default": "combinando pequeña mayúscula G"}}
  }, {"category": "Mn", "key": "1DDC", "mappings": {"default": {"default": "combinando k"}}}, {
    "category": "Mn",
    "key": "1DDD",
    "mappings": {"default": {"default": "combinando l"}}
  }, {
    "category": "Mn",
    "key": "1DDE",
    "mappings": {"default": {"default": "combinando pequeña mayúscula L"}}
  }, {
    "category": "Mn",
    "key": "1DDF",
    "mappings": {"default": {"default": "combinando pequeña mayúscula M"}}
  }, {"category": "Mn", "key": "1DE0", "mappings": {"default": {"default": "combinando n"}}}, {
    "category": "Mn",
    "key": "1DE1",
    "mappings": {"default": {"default": "combinando pequeña mayúscula N"}}
  }, {
    "category": "Mn",
    "key": "1DE2",
    "mappings": {"default": {"default": "combinando pequeña mayúscula R"}}
  }, {
    "category": "Mn",
    "key": "1DE3",
    "mappings": {"default": {"default": "combinando pequeña r rotunda"}}
  }, {"category": "Mn", "key": "1DE4", "mappings": {"default": {"default": "combinando s"}}}, {
    "category": "Mn",
    "key": "1DE5",
    "mappings": {"default": {"default": "combinando larga s"}}
  }, {"category": "Mn", "key": "1DE6", "mappings": {"default": {"default": "combinando z"}}}, {
    "category": "Lm",
    "key": "2071",
    "mappings": {"default": {"default": "superíndice i"}}
  }, {"category": "Lm", "key": "207F", "mappings": {"default": {"default": "superíndice n"}}}, {
    "category": "Lm",
    "key": "2090",
    "mappings": {"default": {"default": "subíndice a"}}
  }, {"category": "Lm", "key": "2091", "mappings": {"default": {"default": "subíndice e"}}}, {
    "category": "Lm",
    "key": "2092",
    "mappings": {"default": {"default": "subíndice o"}}
  }, {"category": "Lm", "key": "2093", "mappings": {"default": {"default": "subíndice x"}}}, {
    "category": "Lm",
    "key": "2094",
    "mappings": {"default": {"default": "subíndice schwa"}}
  }, {"category": "Lm", "key": "2095", "mappings": {"default": {"default": "subíndice h"}}}, {
    "category": "Lm",
    "key": "2096",
    "mappings": {"default": {"default": "subíndice k"}}
  }, {"category": "Lm", "key": "2097", "mappings": {"default": {"default": "subíndice l"}}}, {
    "category": "Lm",
    "key": "2098",
    "mappings": {"default": {"default": "subíndice m"}}
  }, {"category": "Lm", "key": "2099", "mappings": {"default": {"default": "subíndice n"}}}, {
    "category": "Lm",
    "key": "209A",
    "mappings": {"default": {"default": "subíndice p"}}
  }, {"category": "Lm", "key": "209B", "mappings": {"default": {"default": "subíndice s"}}}, {
    "category": "Lm",
    "key": "209C",
    "mappings": {"default": {"default": "subíndice t"}}
  }, {"category": "Lm", "key": "2C7C", "mappings": {"default": {"default": "subíndice j"}}}, {
    "category": "So",
    "key": "1F12A",
    "mappings": {"default": {"default": "concha de tortuga con corchetes mayúscula S"}}
  }, {
    "category": "So",
    "key": "1F12B",
    "mappings": {"default": {"default": "círculo itálico mayúscula C"}}
  }, {
    "category": "So",
    "key": "1F12C",
    "mappings": {"default": {"default": "círculo itálico mayúscula R"}}
  }, {"category": "So", "key": "1F18A", "mappings": {"default": {"default": "cruzada negativa cuadrada Mayúscula P"}}}]
    ,
    "es/symbols/latin-upper-double-accent.js"
:
  [{"locale": "es"}, {
    "category": "Lu",
    "key": "01D5",
    "mappings": {"default": {"default": "mayúscula U con diéresis y macron"}}
  }, {
    "category": "Lu",
    "key": "01D7",
    "mappings": {"default": {"default": "mayúscula U con diéresis y acento agudo"}}
  }, {
    "category": "Lu",
    "key": "01D9",
    "mappings": {"default": {"default": "mayúscula U con diéresis y acento"}}
  }, {
    "category": "Lu",
    "key": "01DB",
    "mappings": {"default": {"default": "mayúscula U con diéresis y grave"}}
  }, {
    "category": "Lu",
    "key": "01DE",
    "mappings": {"default": {"default": "mayúscula A con diéresis y macron"}}
  }, {
    "category": "Lu",
    "key": "01E0",
    "mappings": {"default": {"default": "mayúscula A con punto arriba y macron"}}
  }, {
    "category": "Lu",
    "key": "01EC",
    "mappings": {"default": {"default": "mayúscula O con pecíolo y macron"}}
  }, {
    "category": "Lu",
    "key": "01FA",
    "mappings": {"default": {"default": "mayúscula A con ring above y acento agudo"}}
  }, {
    "category": "Lu",
    "key": "022A",
    "mappings": {"default": {"default": "mayúscula O con diaéresis y macron"}}
  }, {
    "category": "Lu",
    "key": "022C",
    "mappings": {"default": {"default": "mayúscula O con tilde y macron"}}
  }, {
    "category": "Lu",
    "key": "0230",
    "mappings": {"default": {"default": "mayúscula O con dot above y macron"}}
  }, {
    "key": "1E08",
    "mappings": {"default": {"default": "mayúscula C con cedilla y agudo"}},
    "category": "Lu"
  }, {
    "key": "1E14",
    "mappings": {"default": {"default": "mayúscula E con macrón y grave"}},
    "category": "Lu"
  }, {
    "key": "1E16",
    "mappings": {"default": {"default": "mayúscula E con macrón y agudo"}},
    "category": "Lu"
  }, {
    "key": "1E1C",
    "mappings": {"default": {"default": "mayúscula E with cedilla and breve"}},
    "category": "Lu"
  }, {
    "key": "1E2E",
    "mappings": {"default": {"default": "mayúscula I con diéresis y agudo"}},
    "category": "Lu"
  }, {
    "key": "1E38",
    "mappings": {"default": {"default": "mayúscula L con punto debajo y macrón"}},
    "category": "Lu"
  }, {
    "key": "1E4C",
    "mappings": {"default": {"default": "mayúscula O con tilde y acute"}},
    "category": "Lu"
  }, {
    "key": "1E4E",
    "mappings": {"default": {"default": "mayúscula O con tilde y diéresis"}},
    "category": "Lu"
  }, {
    "key": "1E50",
    "mappings": {"default": {"default": "mayúscula O con macrón y grave"}},
    "category": "Lu"
  }, {
    "key": "1E52",
    "mappings": {"default": {"default": "mayúscula O con macrón y agudo"}},
    "category": "Lu"
  }, {
    "key": "1E5C",
    "mappings": {"default": {"default": "mayúscula R con punto debajo y macrón"}},
    "category": "Lu"
  }, {
    "key": "1E64",
    "mappings": {"default": {"default": "mayúscula S con agudo y punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E66",
    "mappings": {"default": {"default": "mayúscula S con carón y punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E68",
    "mappings": {"default": {"default": "mayúscula S con punto debajo y punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E78",
    "mappings": {"default": {"default": "mayúscula U con tilde y agudo"}},
    "category": "Lu"
  }, {
    "key": "1E7A",
    "mappings": {"default": {"default": "mayúscula U con macrón y diéresis"}},
    "category": "Lu"
  }, {
    "key": "1EA4",
    "mappings": {"default": {"default": "mayúscula A con acento circunflejo y agudo"}},
    "category": "Lu"
  }, {
    "key": "1EA6",
    "mappings": {"default": {"default": "mayúscula A con acento circunflejo y grave"}},
    "category": "Lu"
  }, {
    "key": "1EA8",
    "mappings": {"default": {"default": "mayúscula A con acento circunflejo y gancho arriba"}},
    "category": "Lu"
  }, {
    "key": "1EAA",
    "mappings": {"default": {"default": "mayúscula A con acento circunflejo y tilde"}},
    "category": "Lu"
  }, {
    "key": "1EAC",
    "mappings": {"default": {"default": "mayúscula A con acento circunflejo y punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1EAE",
    "mappings": {"default": {"default": "mayúscula A con breve y agudo"}},
    "category": "Lu"
  }, {
    "key": "1EB0",
    "mappings": {"default": {"default": "mayúscula A con breve y grave"}},
    "category": "Lu"
  }, {
    "key": "1EB2",
    "mappings": {"default": {"default": "mayúscula A con breve y gancho arriba"}},
    "category": "Lu"
  }, {
    "key": "1EB4",
    "mappings": {"default": {"default": "mayúscula A con breve y tilde"}},
    "category": "Lu"
  }, {
    "key": "1EB6",
    "mappings": {"default": {"default": "mayúscula A con breve y punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1EBE",
    "mappings": {"default": {"default": "mayúscula E con acento circunflejo y agudo"}},
    "category": "Lu"
  }, {
    "key": "1EC0",
    "mappings": {"default": {"default": "mayúscula E con acento circunflejo y grave"}},
    "category": "Lu"
  }, {
    "key": "1EC2",
    "mappings": {"default": {"default": "mayúscula E con acento circunflejo y gancho arriba"}},
    "category": "Lu"
  }, {
    "key": "1EC4",
    "mappings": {"default": {"default": "mayúscula E con acento circunflejo y tilde"}},
    "category": "Lu"
  }, {
    "key": "1EC6",
    "mappings": {"default": {"default": "mayúscula E con acento circunflejo y punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1ED0",
    "mappings": {"default": {"default": "mayúscula O con acento circunflejo y agudo"}},
    "category": "Lu"
  }, {
    "key": "1ED2",
    "mappings": {"default": {"default": "mayúscula O con acento circunflejo y grave"}},
    "category": "Lu"
  }, {
    "key": "1ED4",
    "mappings": {"default": {"default": "mayúscula O con acento circunflejo y gancho arriba"}},
    "category": "Lu"
  }, {
    "key": "1ED6",
    "mappings": {"default": {"default": "mayúscula O con acento circunflejo y tilde"}},
    "category": "Lu"
  }, {
    "key": "1ED8",
    "mappings": {"default": {"default": "mayúscula O con acento circunflejo y punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1EDA",
    "mappings": {"default": {"default": "mayúscula O with horn and acute"}},
    "category": "Lu"
  }, {
    "key": "1EDC",
    "mappings": {"default": {"default": "mayúscula O with horn and grave"}},
    "category": "Lu"
  }, {
    "key": "1EDE",
    "mappings": {"default": {"default": "mayúscula O with horn and hook above"}},
    "category": "Lu"
  }, {
    "key": "1EE0",
    "mappings": {"default": {"default": "mayúscula O with horn and tilde"}},
    "category": "Lu"
  }, {
    "key": "1EE2",
    "mappings": {"default": {"default": "mayúscula O con cuerno y punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1EE8",
    "mappings": {"default": {"default": "mayúscula U con cuerno y agudo"}},
    "category": "Lu"
  }, {
    "key": "1EEA",
    "mappings": {"default": {"default": "mayúscula U con cuerno y grave"}},
    "category": "Lu"
  }, {
    "key": "1EEC",
    "mappings": {"default": {"default": "mayúscula U con cuerno y gancho arriba"}},
    "category": "Lu"
  }, {
    "key": "1EEE",
    "mappings": {"default": {"default": "mayúscula U con cuerno y tilde"}},
    "category": "Lu"
  }, {"key": "1EF0", "mappings": {"default": {"default": "mayúscula U con cuerno y punto debajo"}}, "category": "Lu"}]
    ,
    "es/symbols/latin-upper-single-accent.js"
:
  [{"locale": "es"}, {
    "key": "00C0",
    "mappings": {"default": {"default": "mayúscula A grave"}},
    "category": "Lu"
  }, {"key": "00C1", "mappings": {"default": {"default": "mayúscula A aguda"}}, "category": "Lu"}, {
    "key": "00C2",
    "mappings": {"default": {"default": "mayúscula A acento circunflejo"}},
    "category": "Lu"
  }, {"key": "00C3", "mappings": {"default": {"default": "mayúscula A con tilde"}}, "category": "Lu"}, {
    "key": "00C4",
    "mappings": {"default": {"default": "mayúscula A diéresis"}},
    "category": "Lu"
  }, {"key": "00C5", "mappings": {"default": {"default": "mayúscula A con anillo"}}, "category": "Lu"}, {
    "key": "00C7",
    "mappings": {"default": {"default": "mayúscula C cedilla"}},
    "category": "Lu"
  }, {"key": "00C8", "mappings": {"default": {"default": "mayúscula E grave"}}, "category": "Lu"}, {
    "key": "00C9",
    "mappings": {"default": {"default": "mayúscula E aguda"}},
    "category": "Lu"
  }, {
    "key": "00CA",
    "mappings": {"default": {"default": "mayúscula E acento circunflejo"}},
    "category": "Lu"
  }, {"key": "00CB", "mappings": {"default": {"default": "mayúscula E diéresis"}}, "category": "Lu"}, {
    "key": "00CC",
    "mappings": {"default": {"default": "mayúscula I grave"}},
    "category": "Lu"
  }, {"key": "00CD", "mappings": {"default": {"default": "mayúscula I aguda"}}, "category": "Lu"}, {
    "key": "00CE",
    "mappings": {"default": {"default": "mayúscula I acento circumflejo"}},
    "category": "Lu"
  }, {"key": "00CF", "mappings": {"default": {"default": "mayúscula I diéresis"}}, "category": "Lu"}, {
    "key": "00D1",
    "mappings": {"default": {"default": "mayúscula Ñ"}},
    "category": "Lu"
  }, {"key": "00D2", "mappings": {"default": {"default": "mayúscula O grave"}}, "category": "Lu"}, {
    "key": "00D3",
    "mappings": {"default": {"default": "mayúscula O aguda"}},
    "category": "Lu"
  }, {
    "key": "00D4",
    "mappings": {"default": {"default": "mayúscula O acento circunflejo"}},
    "category": "Lu"
  }, {"key": "00D5", "mappings": {"default": {"default": "mayúscula O con tilde"}}, "category": "Lu"}, {
    "key": "00D6",
    "mappings": {"default": {"default": "mayúscula O diéresis"}},
    "category": "Lu"
  }, {"key": "00D9", "mappings": {"default": {"default": "mayúscula U grave"}}, "category": "Lu"}, {
    "key": "00DA",
    "mappings": {"default": {"default": "mayúscula U aguda"}},
    "category": "Lu"
  }, {
    "key": "00DB",
    "mappings": {"default": {"default": "mayúscula U acento circunflejo"}},
    "category": "Lu"
  }, {"key": "00DC", "mappings": {"default": {"default": "mayúscula U diéresis"}}, "category": "Lu"}, {
    "key": "00DD",
    "mappings": {"default": {"default": "mayúscula Y aguda"}},
    "category": "Lu"
  }, {"key": "0100", "mappings": {"default": {"default": "mayúscula A barra"}}, "category": "Lu"}, {
    "key": "0102",
    "mappings": {"default": {"default": "mayúscula A con breve"}},
    "category": "Lu"
  }, {"key": "0104", "mappings": {"default": {"default": "mayúscula A con ogonek"}}, "category": "Lu"}, {
    "key": "0106",
    "mappings": {"default": {"default": "mayúscula C con agudo"}},
    "category": "Lu"
  }, {
    "key": "0108",
    "mappings": {"default": {"default": "mayúscula C con acento circunflejo"}},
    "category": "Lu"
  }, {
    "key": "010A",
    "mappings": {"default": {"default": "mayúscula C con punto arriba"}},
    "category": "Lu"
  }, {"key": "010C", "mappings": {"default": {"default": "mayúscula C con carón"}}, "category": "Lu"}, {
    "key": "010E",
    "mappings": {"default": {"default": "mayúscula D con carón"}},
    "category": "Lu"
  }, {"key": "0112", "mappings": {"default": {"default": "mayúscula E barra"}}, "category": "Lu"}, {
    "category": "Lu",
    "key": "0114",
    "mappings": {"default": {"default": "mayúscula E con breve"}}
  }, {
    "key": "0116",
    "mappings": {"default": {"default": "mayúscula E con punto arriba"}},
    "category": "Lu"
  }, {"key": "0118", "mappings": {"default": {"default": "mayúscula E con ogonek"}}, "category": "Lu"}, {
    "key": "011A",
    "mappings": {"default": {"default": "mayúscula E con carón"}},
    "category": "Lu"
  }, {
    "key": "011C",
    "mappings": {"default": {"default": "mayúscula G con acento circunflejo"}},
    "category": "Lu"
  }, {"key": "011E", "mappings": {"default": {"default": "mayúscula G con breve"}}, "category": "Lu"}, {
    "key": "0120",
    "mappings": {"default": {"default": "mayúscula G con punto arriba"}},
    "category": "Lu"
  }, {"key": "0122", "mappings": {"default": {"default": "mayúscula G con cedilla"}}, "category": "Lu"}, {
    "key": "0124",
    "mappings": {"default": {"default": "mayúscula H con acento circunflejo"}},
    "category": "Lu"
  }, {"key": "0128", "mappings": {"default": {"default": "mayúscula I con tilde"}}, "category": "Lu"}, {
    "key": "012A",
    "mappings": {"default": {"default": "mayúscula I barra"}},
    "category": "Lu"
  }, {"category": "Lu", "key": "012C", "mappings": {"default": {"default": "mayúscula I con breve"}}}, {
    "key": "012E",
    "mappings": {"default": {"default": "mayúscula I con ogonek"}},
    "category": "Lu"
  }, {
    "key": "0130",
    "mappings": {"default": {"default": "mayúscula I con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "0134",
    "mappings": {"default": {"default": "mayúscula J con acento circunflejo"}},
    "category": "Lu"
  }, {"key": "0136", "mappings": {"default": {"default": "mayúscula K con cedilla"}}, "category": "Lu"}, {
    "key": "0139",
    "mappings": {"default": {"default": "mayúscula L con agudo"}},
    "category": "Lu"
  }, {"key": "013B", "mappings": {"default": {"default": "mayúscula L con cedilla"}}, "category": "Lu"}, {
    "key": "013D",
    "mappings": {"default": {"default": "mayúscula L con carón"}},
    "category": "Lu"
  }, {
    "key": "013F",
    "mappings": {"default": {"default": "mayúscula L con punto mediano"}},
    "category": "Lu"
  }, {"key": "0143", "mappings": {"default": {"default": "mayúscula N con agudo"}}, "category": "Lu"}, {
    "key": "0145",
    "mappings": {"default": {"default": "mayúscula N con cedilla"}},
    "category": "Lu"
  }, {"key": "0147", "mappings": {"default": {"default": "mayúscula n con carón"}}, "category": "Lu"}, {
    "key": "014C",
    "mappings": {"default": {"default": "mayúscula O barra"}},
    "category": "Lu"
  }, {"category": "Lu", "key": "014E", "mappings": {"default": {"default": "mayúscula O con breve"}}}, {
    "key": "0150",
    "mappings": {"default": {"default": "mayúscula O con doble acento agudo "}},
    "category": "Lu"
  }, {"key": "0154", "mappings": {"default": {"default": "mayúscula R con agudo"}}, "category": "Lu"}, {
    "key": "0156",
    "mappings": {"default": {"default": "mayúscula R con cedilla"}},
    "category": "Lu"
  }, {"key": "0158", "mappings": {"default": {"default": "mayúscula R con carón"}}, "category": "Lu"}, {
    "key": "015A",
    "mappings": {"default": {"default": "mayúscula S con agudo"}},
    "category": "Lu"
  }, {
    "key": "015C",
    "mappings": {"default": {"default": "mayúscula S con acento circunflejo"}},
    "category": "Lu"
  }, {"key": "015E", "mappings": {"default": {"default": "mayúscula S con cedilla"}}, "category": "Lu"}, {
    "key": "0160",
    "mappings": {"default": {"default": "mayúscula S con carón"}},
    "category": "Lu"
  }, {"key": "0162", "mappings": {"default": {"default": "mayúscula T con cedilla"}}, "category": "Lu"}, {
    "key": "0164",
    "mappings": {"default": {"default": "mayúscula T con carón"}},
    "category": "Lu"
  }, {"key": "0168", "mappings": {"default": {"default": "mayúscula U con tilde"}}, "category": "Lu"}, {
    "key": "016A",
    "mappings": {"default": {"default": "mayúscula U barra"}},
    "category": "Lu"
  }, {"key": "016C", "mappings": {"default": {"default": "mayúscula U con breve"}}, "category": "Lu"}, {
    "key": "016E",
    "mappings": {"default": {"default": "mayúscula U con anillo arriba"}},
    "category": "Lu"
  }, {
    "key": "0170",
    "mappings": {"default": {"default": "mayúscula U con doble acento agudo"}},
    "category": "Lu"
  }, {"key": "0172", "mappings": {"default": {"default": "mayúscula U con ogonek"}}, "category": "Lu"}, {
    "key": "0174",
    "mappings": {"default": {"default": "mayúscula W con acento circunflejo"}},
    "category": "Lu"
  }, {
    "key": "0176",
    "mappings": {"default": {"default": "mayúscula Y con circunflejo"}},
    "category": "Lu"
  }, {"key": "0178", "mappings": {"default": {"default": "mayúscula Y diéresis"}}, "category": "Lu"}, {
    "key": "0179",
    "mappings": {"default": {"default": "mayúscula Z con agudo"}},
    "category": "Lu"
  }, {"key": "017B", "mappings": {"default": {"default": "mayúscula Z con punto"}}, "category": "Lu"}, {
    "key": "017D",
    "mappings": {"default": {"default": "mayúscula Z con carón"}},
    "category": "Lu"
  }, {
    "category": "Lu",
    "key": "01CD",
    "mappings": {"default": {"default": "mayúscula A con acento"}}
  }, {
    "category": "Lu",
    "key": "01CF",
    "mappings": {"default": {"default": "mayúscula I con acento"}}
  }, {
    "category": "Lu",
    "key": "01D1",
    "mappings": {"default": {"default": "mayúscula O con acento"}}
  }, {
    "category": "Lu",
    "key": "01D3",
    "mappings": {"default": {"default": "mayúscula U con acento"}}
  }, {
    "category": "Lu",
    "key": "01E6",
    "mappings": {"default": {"default": "mayúscula G con acento"}}
  }, {
    "category": "Lu",
    "key": "01E8",
    "mappings": {"default": {"default": "mayúscula K con acento"}}
  }, {
    "category": "Lu",
    "key": "01EA",
    "mappings": {"default": {"default": "mayúscula O con pecíolo"}}
  }, {
    "category": "Lu",
    "key": "01F4",
    "mappings": {"default": {"default": "mayúscula G con acento agudo"}}
  }, {
    "category": "Lu",
    "key": "01F8",
    "mappings": {"default": {"default": "mayúscula N con grave"}}
  }, {
    "category": "Lu",
    "key": "0200",
    "mappings": {"default": {"default": "mayúscula con doble tumba"}}
  }, {
    "category": "Lu",
    "key": "0202",
    "mappings": {"default": {"default": "mayúscula a con breve invertido"}}
  }, {
    "category": "Lu",
    "key": "0204",
    "mappings": {"default": {"default": "mayúscula E con doble tumba"}}
  }, {
    "category": "Lu",
    "key": "0206",
    "mappings": {"default": {"default": "mayúscula E con Breve invertido"}}
  }, {
    "category": "Lu",
    "key": "0208",
    "mappings": {"default": {"default": "mayúscula I con doble tumba"}}
  }, {
    "category": "Lu",
    "key": "020A",
    "mappings": {"default": {"default": "mayúscula I con Breve invertido"}}
  }, {
    "category": "Lu",
    "key": "020C",
    "mappings": {"default": {"default": "mayúscula O con doble tumba"}}
  }, {
    "category": "Lu",
    "key": "020E",
    "mappings": {"default": {"default": "mayúscula O con Breve invertido"}}
  }, {
    "category": "Lu",
    "key": "0210",
    "mappings": {"default": {"default": "mayúscula R con doble tumba"}}
  }, {
    "category": "Lu",
    "key": "0212",
    "mappings": {"default": {"default": "mayúscula R con breve invertido"}}
  }, {
    "category": "Lu",
    "key": "0214",
    "mappings": {"default": {"default": "mayúscula U con doble tumba"}}
  }, {
    "category": "Lu",
    "key": "0216",
    "mappings": {"default": {"default": "mayúscula U con breve invertido"}}
  }, {
    "category": "Lu",
    "key": "0218",
    "mappings": {"default": {"default": "mayúscula S con coma debajo"}}
  }, {
    "category": "Lu",
    "key": "021A",
    "mappings": {"default": {"default": "mayúscula T con coma debajo"}}
  }, {
    "category": "Lu",
    "key": "021E",
    "mappings": {"default": {"default": "mayúscula H con Caron"}}
  }, {
    "category": "Lu",
    "key": "0226",
    "mappings": {"default": {"default": "mayúscula con un punto arriba"}}
  }, {
    "category": "Lu",
    "key": "0228",
    "mappings": {"default": {"default": "mayúscula E con cedilla"}}
  }, {
    "category": "Lu",
    "key": "022E",
    "mappings": {"default": {"default": "mayúscula O con punto arriba"}}
  }, {"category": "Lu", "key": "0232", "mappings": {"default": {"default": "Mayúscula Y con Macron"}}}, {
    "key": "1E00",
    "mappings": {"default": {"default": "mayúscula A con anillo debajo"}},
    "category": "Lu"
  }, {
    "key": "1E02",
    "mappings": {"default": {"default": "mayúscula B con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E04",
    "mappings": {"default": {"default": "mayúscula B con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1E06",
    "mappings": {"default": {"default": "mayúscula B con línea debajo"}},
    "category": "Lu"
  }, {
    "key": "1E0A",
    "mappings": {"default": {"default": "mayúscula D con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E0C",
    "mappings": {"default": {"default": "mayúscula D con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1E0E",
    "mappings": {"default": {"default": "mayúscula D con línea debajo"}},
    "category": "Lu"
  }, {"key": "1E10", "mappings": {"default": {"default": "mayúscula D con cedilla"}}, "category": "Lu"}, {
    "key": "1E12",
    "mappings": {"default": {"default": "mayúscula D con acento circunflejo debajo"}},
    "category": "Lu"
  }, {
    "key": "1E18",
    "mappings": {"default": {"default": "mayúscula E con acento circunflejo debajo"}},
    "category": "Lu"
  }, {
    "key": "1E1A",
    "mappings": {"default": {"default": "mayúscula E con tilde debajo"}},
    "category": "Lu"
  }, {
    "key": "1E1E",
    "mappings": {"default": {"default": "mayúscula F con punto arriba"}},
    "category": "Lu"
  }, {"key": "1E20", "mappings": {"default": {"default": "mayúscula G con macrón"}}, "category": "Lu"}, {
    "key": "1E22",
    "mappings": {"default": {"default": "mayúscula H con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E24",
    "mappings": {"default": {"default": "mayúscula H con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1E26",
    "mappings": {"default": {"default": "mayúscula H con diéresis"}},
    "category": "Lu"
  }, {"key": "1E28", "mappings": {"default": {"default": "mayúscula H con cedilla"}}, "category": "Lu"}, {
    "key": "1E2A",
    "mappings": {"default": {"default": "mayúscula H con breve debajo"}},
    "category": "Lu"
  }, {
    "key": "1E2C",
    "mappings": {"default": {"default": "mayúscula I con tilde debajo"}},
    "category": "Lu"
  }, {"key": "1E30", "mappings": {"default": {"default": "mayúscula K con agudo"}}, "category": "Lu"}, {
    "key": "1E32",
    "mappings": {"default": {"default": "mayúscula K con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1E34",
    "mappings": {"default": {"default": "mayúscula K con línea debajo"}},
    "category": "Lu"
  }, {
    "key": "1E36",
    "mappings": {"default": {"default": "mayúscula L con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1E3A",
    "mappings": {"default": {"default": "mayúscula L con línea debajo"}},
    "category": "Lu"
  }, {
    "key": "1E3C",
    "mappings": {"default": {"default": "mayúscula L con acento circunflejo debajo"}},
    "category": "Lu"
  }, {"key": "1E3E", "mappings": {"default": {"default": "mayúscula M con agudo"}}, "category": "Lu"}, {
    "key": "1E40",
    "mappings": {"default": {"default": "mayúscula M con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E42",
    "mappings": {"default": {"default": "mayúscula M con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1E44",
    "mappings": {"default": {"default": "mayúscula N con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E46",
    "mappings": {"default": {"default": "mayúscula N con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1E48",
    "mappings": {"default": {"default": "mayúscula N con línea debajo"}},
    "category": "Lu"
  }, {
    "key": "1E4A",
    "mappings": {"default": {"default": "mayúscula N con acento circunflejo debajo"}},
    "category": "Lu"
  }, {"key": "1E54", "mappings": {"default": {"default": "mayúscula P con agudo"}}, "category": "Lu"}, {
    "key": "1E56",
    "mappings": {"default": {"default": "mayúscula P con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E58",
    "mappings": {"default": {"default": "mayúscula R con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E5A",
    "mappings": {"default": {"default": "mayúscula R con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1E5E",
    "mappings": {"default": {"default": "mayúscula R con línea debajo"}},
    "category": "Lu"
  }, {
    "key": "1E60",
    "mappings": {"default": {"default": "mayúscula S con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E62",
    "mappings": {"default": {"default": "mayúscula S con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1E6A",
    "mappings": {"default": {"default": "mayúscula T con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E6C",
    "mappings": {"default": {"default": "mayúscula T con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1E6E",
    "mappings": {"default": {"default": "mayúscula T con línea debajo"}},
    "category": "Lu"
  }, {
    "key": "1E70",
    "mappings": {"default": {"default": "mayúscula T con acento circunflejo debajo"}},
    "category": "Lu"
  }, {
    "key": "1E72",
    "mappings": {"default": {"default": "mayúscula U con diéresis debajo"}},
    "category": "Lu"
  }, {
    "key": "1E74",
    "mappings": {"default": {"default": "mayúscula U con tilde debajo"}},
    "category": "Lu"
  }, {
    "key": "1E76",
    "mappings": {"default": {"default": "mayúscula U con acento circunflejo debajo"}},
    "category": "Lu"
  }, {"key": "1E7C", "mappings": {"default": {"default": "mayúscula V con tilde"}}, "category": "Lu"}, {
    "key": "1E7E",
    "mappings": {"default": {"default": "mayúscula V con punto debajo"}},
    "category": "Lu"
  }, {"key": "1E80", "mappings": {"default": {"default": "mayúscula W con grave"}}, "category": "Lu"}, {
    "key": "1E82",
    "mappings": {"default": {"default": "mayúscula W con agudo"}},
    "category": "Lu"
  }, {
    "key": "1E84",
    "mappings": {"default": {"default": "mayúscula W con diéresis"}},
    "category": "Lu"
  }, {
    "key": "1E86",
    "mappings": {"default": {"default": "mayúscula W con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E88",
    "mappings": {"default": {"default": "mayúscula W con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1E8A",
    "mappings": {"default": {"default": "mayúscula X con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E8C",
    "mappings": {"default": {"default": "mayúscula X con diéresis"}},
    "category": "Lu"
  }, {
    "key": "1E8E",
    "mappings": {"default": {"default": "mayúscula Y con punto arriba"}},
    "category": "Lu"
  }, {
    "key": "1E90",
    "mappings": {"default": {"default": "mayúscula Z con acento circunflejo"}},
    "category": "Lu"
  }, {
    "key": "1E92",
    "mappings": {"default": {"default": "mayúscula Z con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1E94",
    "mappings": {"default": {"default": "mayúscula Z with line below"}},
    "category": "Lu"
  }, {
    "key": "1EA0",
    "mappings": {"default": {"default": "mayúscula A con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1EA2",
    "mappings": {"default": {"default": "mayúscula A con gancho arriba"}},
    "category": "Lu"
  }, {
    "key": "1EB8",
    "mappings": {"default": {"default": "mayúscula E con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1EBA",
    "mappings": {"default": {"default": "mayúscula E con gancho arriba"}},
    "category": "Lu"
  }, {"key": "1EBC", "mappings": {"default": {"default": "mayúscula E con tilde"}}, "category": "Lu"}, {
    "key": "1EC8",
    "mappings": {"default": {"default": "mayúscula I con gancho arriba"}},
    "category": "Lu"
  }, {
    "key": "1ECA",
    "mappings": {"default": {"default": "mayúscula I con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1ECC",
    "mappings": {"default": {"default": "mayúscula O con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1ECE",
    "mappings": {"default": {"default": "mayúscula O con gancho arriba"}},
    "category": "Lu"
  }, {
    "key": "1EE4",
    "mappings": {"default": {"default": "mayúscula U con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1EE6",
    "mappings": {"default": {"default": "mayúscula U con gancho arriba"}},
    "category": "Lu"
  }, {"key": "1EF2", "mappings": {"default": {"default": "mayúscula Y con grave"}}, "category": "Lu"}, {
    "key": "1EF4",
    "mappings": {"default": {"default": "mayúscula Y con punto debajo"}},
    "category": "Lu"
  }, {
    "key": "1EF6",
    "mappings": {"default": {"default": "mayúscula Y con gancho arriba"}},
    "category": "Lu"
  }, {"key": "1EF8", "mappings": {"default": {"default": "mayúscula Y con tilde"}}, "category": "Lu"}]
    ,
    "es/symbols/math_angles.js"
:
  [{"locale": "es"}, {
    "key": "22BE",
    "mappings": {"default": {"default": "ángulo recto con arco"}},
    "category": "Sm"
  }, {
    "key": "237C",
    "mappings": {"default": {"default": "ángulo recto con flecha zigzag hacia abajo"}},
    "category": "Sm"
  }, {"key": "27C0", "mappings": {"default": {"default": "ángulo sólido"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "299B",
    "mappings": {"default": {"default": "Ángulo medido de apertura a la izquierda"}}
  }, {
    "category": "Sm",
    "key": "299C",
    "mappings": {"default": {"default": "Variante de ángulo recto con cuadrado"}}
  }, {
    "key": "299D",
    "mappings": {"default": {"default": "ángulo recto medido con punto"}},
    "category": "Sm"
  }, {"category": "Sm", "key": "299E", "mappings": {"default": {"default": "Ángulo con S dentro"}}}, {
    "category": "Sm",
    "key": "299F",
    "mappings": {"default": {"default": "Ángulo agudo"}}
  }, {
    "category": "Sm",
    "key": "29A0",
    "mappings": {"default": {"default": "Ángulo esférico de apertura a la izquierda"}}
  }, {
    "category": "Sm",
    "key": "29A1",
    "mappings": {"default": {"default": "Apertura de ángulo esférico"}}
  }, {"category": "Sm", "key": "29A2", "mappings": {"default": {"default": "Ángulo de giro"}}}, {
    "category": "Sm",
    "key": "29A3",
    "mappings": {"default": {"default": "Ángulo invertido"}}
  }, {
    "key": "29A4",
    "mappings": {"default": {"default": "ángulo con barra inferior"}},
    "category": "Sm"
  }, {
    "key": "29A5",
    "mappings": {"default": {"default": "ángulo inverso con barra inferior"}},
    "category": "Sm"
  }, {
    "key": "29A6",
    "mappings": {"default": {"default": "ángulo oblicuo hacia arriba"}},
    "category": "Sm"
  }, {
    "key": "29A7",
    "mappings": {"default": {"default": "ángulo oblicuo hacia abajo"}},
    "category": "Sm"
  }, {
    "key": "29A8",
    "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing up and to the right"}},
    "category": "Sm"
  }, {
    "key": "29A9",
    "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing up and to the left"}},
    "category": "Sm"
  }, {
    "key": "29AA",
    "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing down and to the right"}},
    "category": "Sm"
  }, {
    "key": "29AB",
    "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing down and to the left"}},
    "category": "Sm"
  }, {
    "key": "29AC",
    "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing right and up"}},
    "category": "Sm"
  }, {
    "key": "29AD",
    "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing left and up"}},
    "category": "Sm"
  }, {
    "key": "29AE",
    "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing right and down"}},
    "category": "Sm"
  }, {
    "key": "29AF",
    "mappings": {"default": {"default": "measured angle with open arm ending in arrow pointing left and down"}},
    "category": "Sm"
  }]
    ,
    "es/symbols/math_arrows.js"
:
  [{"locale": "es"}, {
    "key": "2190",
    "mappings": {"default": {"default": "flecha izquierda"}},
    "category": "Sm"
  }, {"key": "2191", "mappings": {"default": {"default": "flecha arriba"}}, "category": "Sm"}, {
    "key": "2192",
    "mappings": {"default": {"default": "flecha derecha", "defaultMP": "flecha"}},
    "category": "Sm"
  }, {"key": "2193", "mappings": {"default": {"default": "flecha abajo"}}, "category": "Sm"}, {
    "key": "2194",
    "mappings": {"default": {"default": "flecha izquierda y derecha"}},
    "category": "Sm"
  }, {"key": "2195", "mappings": {"default": {"default": "flecha arriba y abajo"}}, "category": "So"}, {
    "key": "2196",
    "mappings": {"default": {"default": "flecha a noroeste"}},
    "category": "So"
  }, {"key": "2197", "mappings": {"default": {"default": "flecha a nordeste"}}, "category": "So"}, {
    "key": "2198",
    "mappings": {"default": {"default": "flecha a sureste"}},
    "category": "So"
  }, {"key": "2199", "mappings": {"default": {"default": "flecha a suroeste"}}, "category": "So"}, {
    "key": "219A",
    "mappings": {"default": {"default": "flecha izquierda tachada"}},
    "category": "Sm"
  }, {"key": "219B", "mappings": {"default": {"default": "flecha tachada"}}, "category": "Sm"}, {
    "key": "219C",
    "mappings": {"default": {"default": "flecha ondulada izquierda"}},
    "category": "So"
  }, {"key": "219D", "mappings": {"default": {"default": "flecha ondulada"}}, "category": "So"}, {
    "key": "219E",
    "mappings": {"default": {"default": "flecha izquierda con doble punta"}},
    "category": "So"
  }, {
    "key": "219F",
    "mappings": {"default": {"default": "flecha con doble punta hacia arriba"}},
    "category": "So"
  }, {
    "key": "21A0",
    "mappings": {"default": {"default": "flecha derechaa con doble punta"}},
    "category": "Sm"
  }, {
    "key": "21A1",
    "mappings": {"default": {"default": "flecha con doble punta hacia abajo"}},
    "category": "So"
  }, {
    "key": "21A2",
    "mappings": {"default": {"default": "flecha izquierda con cola"}},
    "category": "So"
  }, {"key": "21A3", "mappings": {"default": {"default": "flecha derecha con cola"}}, "category": "Sm"}, {
    "key": "21A4",
    "mappings": {"default": {"default": "flecha izquierda de barra"}},
    "category": "So"
  }, {"key": "21A5", "mappings": {"default": {"default": "barra con flecha"}}, "category": "So"}, {
    "key": "21A6",
    "mappings": {"default": {"default": "flecha derecha de barra"}},
    "category": "Sm"
  }, {"key": "21A7", "mappings": {"default": {"default": "flecha con barra"}}, "category": "So"}, {
    "key": "21A8",
    "mappings": {"default": {"default": "flecha arriba y abajo con base"}},
    "category": "So"
  }, {
    "key": "21A9",
    "mappings": {"default": {"default": "flecha izquierda con gancho"}},
    "category": "So"
  }, {"key": "21AA", "mappings": {"default": {"default": "flecha con gancho"}}, "category": "So"}, {
    "key": "21AB",
    "mappings": {"default": {"default": "flecha izquierda con lazo"}},
    "category": "So"
  }, {"key": "21AC", "mappings": {"default": {"default": "flecha con lazo"}}, "category": "So"}, {
    "key": "21AD",
    "mappings": {"default": {"default": "flecha ondulada izquierda y derecha"}},
    "category": "So"
  }, {
    "key": "21AE",
    "mappings": {"default": {"default": "flecha izquierda y derecha tachada"}},
    "category": "Sm"
  }, {"key": "21AF", "mappings": {"default": {"default": "flecha zigzag abajo"}}, "category": "So"}, {
    "key": "21B0",
    "mappings": {"default": {"default": "flecha arriba con punta izquierda"}},
    "category": "So"
  }, {
    "key": "21B1",
    "mappings": {"default": {"default": "flecha arriba con punta derecha"}},
    "category": "So"
  }, {
    "key": "21B2",
    "mappings": {"default": {"default": "flecha abajo con punta izquierda"}},
    "category": "So"
  }, {
    "key": "21B3",
    "mappings": {"default": {"default": "flecha abajo con punta derecha"}},
    "category": "So"
  }, {
    "key": "21B4",
    "mappings": {"default": {"default": "flecha derecha desde esquina inferior"}},
    "category": "So"
  }, {
    "key": "21B5",
    "mappings": {"default": {"default": "flecha hacia abajo desde esquina izquierda"}},
    "category": "So"
  }, {
    "key": "21B6",
    "mappings": {"default": {"default": "flecha semicircular superior en sentido antihorario"}},
    "category": "So"
  }, {
    "key": "21B7",
    "mappings": {"default": {"default": "flecha semicircular superior en sentido horario"}},
    "category": "So"
  }, {
    "key": "21B8",
    "mappings": {"default": {"default": "flecha noroeste a barra larga"}},
    "category": "So"
  }, {
    "key": "21B9",
    "mappings": {"default": {"default": "flecha izquierda a barra arriba de flecha derecha a barra"}},
    "category": "So"
  }, {
    "key": "21BA",
    "mappings": {"default": {"default": "flecha circular abierta en sentido antihorario"}},
    "category": "So"
  }, {
    "key": "21BB",
    "mappings": {"default": {"default": "flecha circular abierta en sentido horario"}},
    "category": "So"
  }, {
    "key": "21C4",
    "mappings": {"default": {"default": "flecha derecha arriba de flecha izquierda"}},
    "category": "So"
  }, {
    "key": "21C5",
    "mappings": {"default": {"default": "flecha arriba a la izquierda de flecha abajo"}},
    "category": "So"
  }, {
    "key": "21C6",
    "mappings": {"default": {"default": "flecha izquierda arriba de flecha derecha"}},
    "category": "So"
  }, {
    "key": "21C7",
    "mappings": {"default": {"default": "dos flechas hacia la izquierda"}},
    "category": "So"
  }, {
    "key": "21C8",
    "mappings": {"default": {"default": "dos flechas hacia arriba"}},
    "category": "So"
  }, {"key": "21C9", "mappings": {"default": {"default": "dos flechas"}}, "category": "So"}, {
    "key": "21CA",
    "mappings": {"default": {"default": "dos flechas hacia abajo"}},
    "category": "So"
  }, {
    "key": "21CD",
    "mappings": {"default": {"default": "flecha doble izquierda tachada"}},
    "category": "So"
  }, {
    "key": "21CE",
    "mappings": {"default": {"default": "flecha doble izquierda y derecha tachada"}},
    "category": "Sm"
  }, {"key": "21CF", "mappings": {"default": {"default": "flecha doble tachada"}}, "category": "Sm"}, {
    "key": "21D0",
    "mappings": {"default": {"default": "flecha doble hacia la izquierda"}},
    "category": "So"
  }, {
    "key": "21D1",
    "mappings": {"default": {"default": "flecha doble hacia arriba"}},
    "category": "So"
  }, {"key": "21D2", "mappings": {"default": {"default": "flecha doble"}}, "category": "Sm"}, {
    "key": "21D3",
    "mappings": {"default": {"default": "flecha doble hacia abajo"}},
    "category": "So"
  }, {
    "key": "21D4",
    "mappings": {"default": {"default": "flecha doble izquierda-derecha"}},
    "category": "Sm"
  }, {"key": "21D5", "mappings": {"default": {"default": "flecha doble vertical"}}, "category": "So"}, {
    "key": "21D6",
    "mappings": {"default": {"default": "flecha doble hacia noroeste"}},
    "category": "So"
  }, {
    "key": "21D7",
    "mappings": {"default": {"default": "flecha doble hacia nordeste"}},
    "category": "So"
  }, {
    "key": "21D8",
    "mappings": {"default": {"default": "flecha doble hacia sudeste"}},
    "category": "So"
  }, {
    "key": "21D9",
    "mappings": {"default": {"default": "flecha doble hacia sudoeste"}},
    "category": "So"
  }, {"key": "21DA", "mappings": {"default": {"default": "flecha triple izquierda"}}, "category": "So"}, {
    "key": "21DB",
    "mappings": {"default": {"default": "flecha triple"}},
    "category": "So"
  }, {
    "key": "21DC",
    "mappings": {"default": {"default": "flecha ondulada izquierda"}},
    "category": "So"
  }, {"key": "21DD", "mappings": {"default": {"default": "flecha ondulada"}}, "category": "So"}, {
    "key": "21DE",
    "mappings": {"default": {"default": "flecha hacia arriba con doble tachado"}},
    "category": "So"
  }, {
    "key": "21DF",
    "mappings": {"default": {"default": "flecha hacia abajo con doble tachado"}},
    "category": "So"
  }, {
    "key": "21E0",
    "mappings": {"default": {"default": "flecha de puntos hacia la izquierda"}},
    "category": "So"
  }, {
    "key": "21E1",
    "mappings": {"default": {"default": "flecha de puntos hacia arriba"}},
    "category": "So"
  }, {"key": "21E2", "mappings": {"default": {"default": "flecha de puntos"}}, "category": "So"}, {
    "key": "21E3",
    "mappings": {"default": {"default": "flecha de puntos hacia abajo"}},
    "category": "So"
  }, {
    "key": "21E4",
    "mappings": {"default": {"default": "flecha izquierda a barra"}},
    "category": "So"
  }, {"key": "21E5", "mappings": {"default": {"default": "flecha derecha a barra"}}, "category": "So"}, {
    "key": "21E6",
    "mappings": {"default": {"default": "flecha vacía hacia la izquierda"}},
    "category": "So"
  }, {
    "key": "21E7",
    "mappings": {"default": {"default": "flecha vacía hacia arriba"}},
    "category": "So"
  }, {"key": "21E8", "mappings": {"default": {"default": "flecha vacía"}}, "category": "So"}, {
    "key": "21E9",
    "mappings": {"default": {"default": "flecha vacía hacia abajo"}},
    "category": "So"
  }, {
    "key": "21EA",
    "mappings": {"default": {"default": "flecha vacía hacia arriba desde barra"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "21EB",
    "mappings": {"default": {"default": "flecha blanca hacia arriba en el pedestal"}}
  }, {
    "category": "So",
    "key": "21EC",
    "mappings": {"default": {"default": "flecha blanca hacia arriba sobre pedestal con barra horizontal"}}
  }, {
    "category": "So",
    "key": "21ED",
    "mappings": {"default": {"default": "flecha blanca hacia arriba sobre pedestal con barra vertical"}}
  }, {
    "category": "So",
    "key": "21EE",
    "mappings": {"default": {"default": "flecha doble hacia arriba blanca"}}
  }, {
    "category": "So",
    "key": "21EF",
    "mappings": {"default": {"default": "flecha doble hacia arriba blanca en pedestal"}}
  }, {
    "category": "So",
    "key": "21F0",
    "mappings": {"default": {"default": "flecha blanca hacia la derecha de la pared"}}
  }, {
    "category": "So",
    "key": "21F1",
    "mappings": {"default": {"default": "flecha del noroeste a la esquina"}}
  }, {
    "category": "So",
    "key": "21F2",
    "mappings": {"default": {"default": "flecha del sudeste a la esquina"}}
  }, {
    "category": "So",
    "key": "21F3",
    "mappings": {"default": {"default": "arriba abajo flecha blanca"}}
  }, {
    "category": "Sm",
    "key": "21F4",
    "mappings": {"default": {"default": "flecha derecha con círculo pequeño"}}
  }, {
    "key": "21F5",
    "mappings": {"default": {"default": "upwards arrow to the right of downwards arrow"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "21F6",
    "mappings": {"default": {"default": "tres flechas a la derecha"}}
  }, {
    "category": "Sm",
    "key": "21F7",
    "mappings": {"default": {"default": "flecha hacia la izquierda con trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "21F8",
    "mappings": {"default": {"default": "flecha hacia la derecha con trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "21F9",
    "mappings": {"default": {"default": "flecha izquierda derecha con trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "21FA",
    "mappings": {"default": {"default": "flecha hacia la izquierda con doble trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "21FB",
    "mappings": {"default": {"default": "flecha hacia la derecha con doble trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "21FC",
    "mappings": {"default": {"default": "flecha izquierda derecha con doble trazo vertical"}}
  }, {
    "key": "21FD",
    "mappings": {"default": {"default": "flecha izquierda con punta vacía"}},
    "category": "Sm"
  }, {
    "key": "21FE",
    "mappings": {"default": {"default": "flecha izquierda con punta vacía"}},
    "category": "Sm"
  }, {
    "key": "21FF",
    "mappings": {"default": {"default": "flecha izquierda derecha con punta vacía"}},
    "category": "Sm"
  }, {"category": "So", "key": "2301", "mappings": {"default": {"default": "flecha electrica"}}}, {
    "category": "So",
    "key": "2303",
    "mappings": {"default": {"default": "flecha arriba"}}
  }, {"category": "So", "key": "2304", "mappings": {"default": {"default": "flecha hacia abajo"}}}, {
    "category": "So",
    "key": "2324",
    "mappings": {"default": {"default": "flecha arriba entre dos barras horizontales"}}
  }, {
    "category": "So",
    "key": "238B",
    "mappings": {"default": {"default": "círculo roto con la flecha del noroeste"}}
  }, {
    "category": "So",
    "key": "2794",
    "mappings": {"default": {"default": "flecha hacia la derecha con cabeza ancha"}}
  }, {
    "category": "So",
    "key": "2798",
    "mappings": {"default": {"default": "flecha Sureste pesada"}}
  }, {
    "category": "So",
    "key": "2799",
    "mappings": {"default": {"default": "flecha hacia la derecha pesada"}}
  }, {
    "category": "So",
    "key": "279A",
    "mappings": {"default": {"default": "flecha pesada del noreste"}}
  }, {
    "category": "So",
    "key": "279B",
    "mappings": {"default": {"default": "punto de dibujo Flecha hacia la derecha"}}
  }, {
    "category": "So",
    "key": "279C",
    "mappings": {"default": {"default": "flecha hacia la derecha con punta redonda redonda"}}
  }, {
    "category": "So",
    "key": "279D",
    "mappings": {"default": {"default": "flecha hacia la derecha con forma de triángulo"}}
  }, {
    "category": "So",
    "key": "279E",
    "mappings": {"default": {"default": "flecha hacia la derecha con forma de triángulo pesado"}}
  }, {
    "category": "So",
    "key": "279F",
    "mappings": {"default": {"default": "triángulo punteado hacia la flecha hacia la derecha"}}
  }, {
    "category": "So",
    "key": "27A0",
    "mappings": {"default": {"default": "flecha hacia la derecha con forma de triángulo de trazo pesado"}}
  }, {
    "category": "So",
    "key": "27A1",
    "mappings": {"default": {"default": "flecha hacia la derecha negra"}}
  }, {
    "category": "So",
    "key": "27A2",
    "mappings": {"default": {"default": "punta de flecha tridimensional hacia la derecha hacia arriba"}}
  }, {
    "category": "So",
    "key": "27A3",
    "mappings": {"default": {"default": "punta de flecha hacia la derecha con iluminación inferior en tres dimensiones"}}
  }, {
    "category": "So",
    "key": "27A4",
    "mappings": {"default": {"default": "punta de flecha negra hacia la derecha"}}
  }, {
    "category": "So",
    "key": "27A5",
    "mappings": {"default": {"default": "pesado negro curvado hacia abajo y hacia la derecha flecha"}}
  }, {
    "category": "So",
    "key": "27A6",
    "mappings": {"default": {"default": "pesado negro curvado hacia arriba y hacia la derecha flecha"}}
  }, {
    "category": "So",
    "key": "27A7",
    "mappings": {"default": {"default": "squat Black Arrow hacia la derecha"}}
  }, {
    "category": "So",
    "key": "27A8",
    "mappings": {"default": {"default": "flecha hacia la derecha de color negro cóncava puntiaguda pesada"}}
  }, {
    "category": "So",
    "key": "27A9",
    "mappings": {"default": {"default": "sombra derecha flecha blanca hacia la derecha"}}
  }, {
    "category": "So",
    "key": "27AA",
    "mappings": {"default": {"default": "sombra hacia la izquierda blanca a la izquierda"}}
  }, {
    "category": "So",
    "key": "27AB",
    "mappings": {"default": {"default": "back-Tilted Shadowed White Right Arrow"}}
  }, {
    "category": "So",
    "key": "27AC",
    "mappings": {"default": {"default": "flecha hacia la derecha blanca sombreada inclinada hacia delante"}}
  }, {
    "category": "So",
    "key": "27AD",
    "mappings": {"default": {"default": "flecha hacia la derecha pesada inferior, sombreada a la derecha, blanca"}}
  }, {
    "category": "So",
    "key": "27AE",
    "mappings": {"default": {"default": "flecha hacia la derecha blanca superior derecha sombreada pesada"}}
  }, {
    "category": "So",
    "key": "27AF",
    "mappings": {"default": {"default": "con muesca, flecha derecha hacia abajo, a la derecha, hacia la derecha"}}
  }, {
    "category": "So",
    "key": "27B1",
    "mappings": {"default": {"default": "con muescas, flecha derecha hacia arriba, a la derecha, hacia la derecha"}}
  }, {
    "category": "So",
    "key": "27B2",
    "mappings": {"default": {"default": "círculo pesado blanco flecha hacia la derecha"}}
  }, {
    "category": "So",
    "key": "27B3",
    "mappings": {"default": {"default": "flecha hacia la derecha con plumas blancas"}}
  }, {
    "category": "So",
    "key": "27B4",
    "mappings": {"default": {"default": "flecha suroriental de plumas negras"}}
  }, {
    "category": "So",
    "key": "27B5",
    "mappings": {"default": {"default": "flecha hacia la derecha con plumas negras"}}
  }, {
    "category": "So",
    "key": "27B6",
    "mappings": {"default": {"default": "flecha del noreste con plumas negras"}}
  }, {
    "category": "So",
    "key": "27B7",
    "mappings": {"default": {"default": "flecha suroriental con plumas negras pesadas"}}
  }, {
    "category": "So",
    "key": "27B8",
    "mappings": {"default": {"default": "flecha derecha pesada con plumas negras"}}
  }, {
    "category": "So",
    "key": "27B9",
    "mappings": {"default": {"default": "flecha del noreste con plumas negras pesadas"}}
  }, {
    "category": "So",
    "key": "27BA",
    "mappings": {"default": {"default": "lágrima-flecha de flecha hacia la derecha"}}
  }, {
    "category": "So",
    "key": "27BB",
    "mappings": {"default": {"default": "flecha hacia la derecha con forma de lágrima pesada"}}
  }, {
    "category": "So",
    "key": "27BC",
    "mappings": {"default": {"default": "flecha hacia la derecha de cola de cuña"}}
  }, {
    "category": "So",
    "key": "27BD",
    "mappings": {"default": {"default": "flecha hacia la derecha de cola de cuña pesada"}}
  }, {
    "category": "So",
    "key": "27BE",
    "mappings": {"default": {"default": "open-Outlined Outward Arrow"}}
  }, {
    "key": "27F0",
    "mappings": {"default": {"default": "flecha cuádruple hacia arriba"}},
    "category": "Sm"
  }, {
    "key": "27F1",
    "mappings": {"default": {"default": "flecha cuádruple hacia abajo"}},
    "category": "Sm"
  }, {
    "key": "27F2",
    "mappings": {"default": {"default": "flecha hacia la izquierda desde círculo"}},
    "category": "Sm"
  }, {
    "key": "27F3",
    "mappings": {"default": {"default": "flecha hacia la derecha desde círculo"}},
    "category": "Sm"
  }, {
    "key": "27F4",
    "mappings": {"default": {"default": "flecha con más en círculo"}},
    "category": "Sm"
  }, {
    "key": "27F5",
    "mappings": {"default": {"default": "flecha larga hacia la izquierda"}},
    "category": "Sm"
  }, {"key": "27F6", "mappings": {"default": {"default": "flecha larga"}}, "category": "Sm"}, {
    "key": "27F7",
    "mappings": {"default": {"default": "flecha larga hacia izquierda y derecha"}},
    "category": "Sm"
  }, {
    "key": "27F8",
    "mappings": {"default": {"default": "doble flecha larga hacia la izquierda"}},
    "category": "Sm"
  }, {"key": "27F9", "mappings": {"default": {"default": "doble flecha larga"}}, "category": "Sm"}, {
    "key": "27FA",
    "mappings": {"default": {"default": "doble flecha larga hacia izquierda y derecha"}},
    "category": "Sm"
  }, {
    "key": "27FB",
    "mappings": {"default": {"default": "flecha larga hacia la izquierda, desde barra"}},
    "category": "Sm"
  }, {
    "key": "27FC",
    "mappings": {"default": {"default": "flecha larga desde barra"}},
    "category": "Sm"
  }, {
    "key": "27FD",
    "mappings": {"default": {"default": "doble flecha larga hacia la izquierda desde barra"}},
    "category": "Sm"
  }, {
    "key": "27FE",
    "mappings": {"default": {"default": "doble flecha larga desde barra"}},
    "category": "Sm"
  }, {
    "key": "27FF",
    "mappings": {"default": {"default": "flecha larga en zigzag"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2900",
    "mappings": {"default": {"default": "flecha de dos cabezas hacia la derecha con trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "2901",
    "mappings": {"default": {"default": "flecha de dos cabezas hacia la derecha con doble movimiento vertical"}}
  }, {
    "category": "Sm",
    "key": "2902",
    "mappings": {"default": {"default": "doble flecha hacia la izquierda con trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "2903",
    "mappings": {"default": {"default": "doble flecha hacia la derecha con trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "2904",
    "mappings": {"default": {"default": "izquierda Derecha Doble Flecha con Trazo Vertical"}}
  }, {
    "key": "2905",
    "mappings": {"default": {"default": "rightwards two headed arrow from bar"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2906",
    "mappings": {"default": {"default": "doble flecha hacia la izquierda de la barra"}}
  }, {
    "category": "Sm",
    "key": "2907",
    "mappings": {"default": {"default": "flecha doble hacia la derecha de la barra"}}
  }, {
    "category": "Sm",
    "key": "2908",
    "mappings": {"default": {"default": "flecha hacia abajo con trazo horizontal"}}
  }, {
    "category": "Sm",
    "key": "2909",
    "mappings": {"default": {"default": "flecha hacia arriba con trazo horizontal"}}
  }, {
    "category": "Sm",
    "key": "290A",
    "mappings": {"default": {"default": "flecha triple hacia arriba"}}
  }, {
    "category": "Sm",
    "key": "290B",
    "mappings": {"default": {"default": "triple flecha hacia abajo"}}
  }, {
    "key": "290C",
    "mappings": {"default": {"default": "leftwards double dash arrow"}},
    "category": "Sm"
  }, {
    "key": "290D",
    "mappings": {"default": {"default": "rightwards double dash arrow"}},
    "category": "Sm"
  }, {
    "key": "290E",
    "mappings": {"default": {"default": "leftwards triple dash arrow"}},
    "category": "Sm"
  }, {
    "key": "290F",
    "mappings": {"default": {"default": "rightwards triple dash arrow"}},
    "category": "Sm"
  }, {
    "key": "2910",
    "mappings": {"default": {"default": "rightwards two headed triple dash arrow"}},
    "category": "Sm"
  }, {
    "key": "2911",
    "mappings": {"default": {"default": "rightwards arrow with dotted stem"}},
    "category": "Sm"
  }, {"key": "2912", "mappings": {"default": {"default": "upwards arrow to bar"}}, "category": "Sm"}, {
    "key": "2913",
    "mappings": {"default": {"default": "downwards arrow to bar"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2914",
    "mappings": {"default": {"default": "flecha hacia la derecha con cola con trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "2915",
    "mappings": {"default": {"default": "flecha hacia la derecha con la cola con doble trazo vertical"}}
  }, {
    "key": "2916",
    "mappings": {"default": {"default": "rightwards two headed arrow with tail"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2917",
    "mappings": {"default": {"default": "flecha de dos cabezas hacia la derecha con cola con trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "2918",
    "mappings": {"default": {"default": "flecha de dos cabezas hacia la derecha con cola con doble movimiento vertical"}}
  }, {
    "key": "2919",
    "mappings": {"default": {"default": "cola de flecha hacia la izquierda"}},
    "category": "Sm"
  }, {"category": "Sm", "key": "291A", "mappings": {"default": {"default": "flecha hacia la derecha"}}}, {
    "key": "291B",
    "mappings": {"default": {"default": "cola de doble flecha hacia la izquierda"}},
    "category": "Sm"
  }, {
    "key": "291C",
    "mappings": {"default": {"default": "cola de doble flecha hacia la derecha"}},
    "category": "Sm"
  }, {
    "key": "291D",
    "mappings": {"default": {"default": "flecha hacia diamante relleno a la izquierda"}},
    "category": "Sm"
  }, {
    "key": "291E",
    "mappings": {"default": {"default": "flecha hacia diamante a la derecha"}},
    "category": "Sm"
  }, {
    "key": "291F",
    "mappings": {"default": {"default": "flecha desde barra hacia diamante a la izquierda"}},
    "category": "Sm"
  }, {
    "key": "2920",
    "mappings": {"default": {"default": "flecha desde barra hacia diamante a la derecha"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2921",
    "mappings": {"default": {"default": "flechas del noroeste y sureste"}}
  }, {
    "category": "Sm",
    "key": "2922",
    "mappings": {"default": {"default": "flecha noreste y sudoeste"}}
  }, {
    "key": "2923",
    "mappings": {"default": {"default": "flecha a noroeste con garfio"}},
    "category": "Sm"
  }, {
    "key": "2924",
    "mappings": {"default": {"default": "flecha a nordeste con garfio"}},
    "category": "Sm"
  }, {
    "key": "2925",
    "mappings": {"default": {"default": "flecha a sureste con garfio"}},
    "category": "Sm"
  }, {
    "key": "2926",
    "mappings": {"default": {"default": "flecha a suroeste con garfio"}},
    "category": "Sm"
  }, {
    "key": "2927",
    "mappings": {"default": {"default": "flechas a noroeste y nordeste"}},
    "category": "Sm"
  }, {
    "key": "2928",
    "mappings": {"default": {"default": "flechas a nordeste y sudeste"}},
    "category": "Sm"
  }, {
    "key": "2929",
    "mappings": {"default": {"default": "flechas a suddeste y sudoeste"}},
    "category": "Sm"
  }, {
    "key": "292A",
    "mappings": {"default": {"default": "flechas a sudoeste y noroeste"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "292D",
    "mappings": {"default": {"default": "flecha Sureste Cruzando Flecha Noreste"}}
  }, {
    "category": "Sm",
    "key": "292E",
    "mappings": {"default": {"default": "north East Arrow Crossing South East Arrow"}}
  }, {
    "category": "Sm",
    "key": "292F",
    "mappings": {"default": {"default": "caída en diagonal que cruza la flecha noreste"}}
  }, {
    "category": "Sm",
    "key": "2930",
    "mappings": {"default": {"default": "creciente Cruce de la Cruz del Sureste de la Flecha"}}
  }, {
    "category": "Sm",
    "key": "2931",
    "mappings": {"default": {"default": "north East Arrow Crossing North West Arrow"}}
  }, {
    "category": "Sm",
    "key": "2932",
    "mappings": {"default": {"default": "north West Arrow Crossing North East Arrow"}}
  }, {"key": "2933", "mappings": {"default": {"default": "flecha ondulada"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "2934",
    "mappings": {"default": {"default": "flecha que apunta hacia la derecha y luego curva hacia arriba"}}
  }, {
    "key": "2935",
    "mappings": {"default": {"default": "flecha hacia la derecha que gira hacia abajo"}},
    "category": "Sm"
  }, {
    "key": "2936",
    "mappings": {"default": {"default": "flecha hacia abajo que gira hacia la izquierda"}},
    "category": "Sm"
  }, {
    "key": "2937",
    "mappings": {"default": {"default": "flecha hacia abajo que gira hacia la derecha"}},
    "category": "Sm"
  }, {
    "key": "2938",
    "mappings": {"default": {"default": "flecha semicircular a la derecha en sentido horario"}},
    "category": "Sm"
  }, {
    "key": "2939",
    "mappings": {"default": {"default": "flecha semicircular a la izquierda en sentido antihorario"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "293A",
    "mappings": {"default": {"default": "arco superior flecha hacia la izquierda"}}
  }, {
    "category": "Sm",
    "key": "293B",
    "mappings": {"default": {"default": "arco inferior flecha hacia la izquierda"}}
  }, {"key": "293C", "mappings": {"default": {"default": "giro negativo"}}, "category": "Sm"}, {
    "key": "293D",
    "mappings": {"default": {"default": "giro positivo"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "293E",
    "mappings": {"default": {"default": "flecha hacia la derecha semicircular inferior derecha"}}
  }, {
    "category": "Sm",
    "key": "293F",
    "mappings": {"default": {"default": "flecha inferior izquierda semicircular en sentido antihorario"}}
  }, {
    "category": "Sm",
    "key": "2940",
    "mappings": {"default": {"default": "círculo cerrado en sentido contrario a las agujas del reloj"}}
  }, {
    "category": "Sm",
    "key": "2941",
    "mappings": {"default": {"default": "círculo cerrado en el sentido de las agujas del reloj"}}
  }, {
    "category": "Sm",
    "key": "2942",
    "mappings": {"default": {"default": "flecha hacia la derecha sobre la flecha corta hacia la izquierda"}}
  }, {
    "category": "Sm",
    "key": "2943",
    "mappings": {"default": {"default": "flecha hacia la izquierda sobre la flecha corta hacia la derecha"}}
  }, {
    "category": "Sm",
    "key": "2944",
    "mappings": {"default": {"default": "flecha corta hacia la derecha arriba flecha hacia la izquierda"}}
  }, {
    "key": "2945",
    "mappings": {"default": {"default": "flecha con más suscrito"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2946",
    "mappings": {"default": {"default": "flecha hacia la izquierda con más abajo"}}
  }, {
    "category": "Sm",
    "key": "2947",
    "mappings": {"default": {"default": "flecha hacia la derecha a través de X"}}
  }, {
    "key": "2948",
    "mappings": {"default": {"default": "flecha hacia la izquierda a través de círculo"}},
    "category": "Sm"
  }, {
    "key": "2949",
    "mappings": {"default": {"default": "dos cabezas de flecha hacia arriba desde círculo"}},
    "category": "Sm"
  }, {"key": "2970", "mappings": {"default": {"default": "round implies"}}, "category": "Sm"}, {
    "key": "2971",
    "mappings": {"default": {"default": "flecha con igual"}},
    "category": "Sm"
  }, {"key": "2972", "mappings": {"default": {"default": "flecha con tilde"}}, "category": "Sm"}, {
    "key": "2973",
    "mappings": {"default": {"default": "tilde con flecha hacia la izquierda"}},
    "category": "Sm"
  }, {"key": "2974", "mappings": {"default": {"default": "tilde con flecha"}}, "category": "Sm"}, {
    "key": "2975",
    "mappings": {"default": {"default": "flecha con casi igual a"}},
    "category": "Sm"
  }, {
    "key": "2976",
    "mappings": {"default": {"default": "flecha hacia la izquierda con menor que"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2977",
    "mappings": {"default": {"default": "flecha hacia la izquierda a través de menos de"}}
  }, {"key": "2978", "mappings": {"default": {"default": "flecha con mayor que"}}, "category": "Sm"}, {
    "key": "2979",
    "mappings": {"default": {"default": "flecha hacia la izquierda con incluido"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "297A",
    "mappings": {"default": {"default": "flecha hacia la izquierda a través del subconjunto"}}
  }, {
    "key": "297B",
    "mappings": {"default": {"default": "flecha hacia la izquierda con contiene"}},
    "category": "Sm"
  }, {
    "key": "29B3",
    "mappings": {"default": {"default": "conjunto vacío con flecha"}},
    "category": "Sm"
  }, {
    "key": "29B4",
    "mappings": {"default": {"default": "conjunto vacío con flecha inversa"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "29BD",
    "mappings": {"default": {"default": "flecha arriba a través del círculo"}}
  }, {
    "category": "Sm",
    "key": "29EA",
    "mappings": {"default": {"default": "diamante negro con flecha hacia abajo"}}
  }, {
    "category": "Sm",
    "key": "29EC",
    "mappings": {"default": {"default": "círculo blanco con flecha hacia abajo"}}
  }, {
    "category": "Sm",
    "key": "29ED",
    "mappings": {"default": {"default": "círculo negro con flecha hacia abajo"}}
  }, {
    "key": "2A17",
    "mappings": {"default": {"default": "integral con flecha hacia la izquierda con garfio"}},
    "category": "Sm"
  }, {
    "category": "So",
    "key": "2B00",
    "mappings": {"default": {"default": "flecha blanca del noreste"}}
  }, {
    "category": "So",
    "key": "2B01",
    "mappings": {"default": {"default": "flecha blanca del noroeste"}}
  }, {
    "category": "So",
    "key": "2B02",
    "mappings": {"default": {"default": "flecha blanca del sureste"}}
  }, {
    "category": "So",
    "key": "2B03",
    "mappings": {"default": {"default": "sudoeste flecha blanca"}}
  }, {
    "category": "So",
    "key": "2B04",
    "mappings": {"default": {"default": "izquierda derecha flecha blanca"}}
  }, {
    "category": "So",
    "key": "2B05",
    "mappings": {"default": {"default": "flecha negra hacia la izquierda"}}
  }, {
    "category": "So",
    "key": "2B06",
    "mappings": {"default": {"default": "flecha negra hacia arriba"}}
  }, {
    "category": "So",
    "key": "2B07",
    "mappings": {"default": {"default": "flecha negra hacia abajo"}}
  }, {
    "category": "So",
    "key": "2B08",
    "mappings": {"default": {"default": "flecha negra del noreste"}}
  }, {
    "category": "So",
    "key": "2B09",
    "mappings": {"default": {"default": "flecha negra del noroeste"}}
  }, {
    "category": "So",
    "key": "2B0A",
    "mappings": {"default": {"default": "flecha negra sur este"}}
  }, {
    "category": "So",
    "key": "2B0B",
    "mappings": {"default": {"default": "sudoeste flecha negra"}}
  }, {
    "category": "So",
    "key": "2B0C",
    "mappings": {"default": {"default": "izquierda derecha flecha negra"}}
  }, {
    "category": "So",
    "key": "2B0D",
    "mappings": {"default": {"default": "arriba abajo flecha negra"}}
  }, {
    "category": "So",
    "key": "2B0E",
    "mappings": {"default": {"default": "flecha hacia la derecha con la punta hacia abajo"}}
  }, {
    "category": "So",
    "key": "2B0F",
    "mappings": {"default": {"default": "flecha hacia la derecha con la punta hacia arriba"}}
  }, {
    "category": "So",
    "key": "2B10",
    "mappings": {"default": {"default": "flecha hacia la izquierda con la punta hacia abajo"}}
  }, {
    "category": "So",
    "key": "2B11",
    "mappings": {"default": {"default": "flecha hacia la izquierda con la punta hacia arriba"}}
  }, {
    "category": "Sm",
    "key": "2B30",
    "mappings": {"default": {"default": "flecha izquierda con círculo pequeño"}}
  }, {
    "category": "Sm",
    "key": "2B31",
    "mappings": {"default": {"default": "tres flechas a la izquierda"}}
  }, {
    "category": "Sm",
    "key": "2B32",
    "mappings": {"default": {"default": "flecha izquierda con un círculo más"}}
  }, {
    "category": "Sm",
    "key": "2B33",
    "mappings": {"default": {"default": "flecha de flecha larga hacia la izquierda"}}
  }, {
    "category": "Sm",
    "key": "2B34",
    "mappings": {"default": {"default": "flecha de dos cabezas hacia la izquierda con trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "2B35",
    "mappings": {"default": {"default": "flecha de dos cabezas hacia la izquierda con doble movimiento vertical"}}
  }, {
    "category": "Sm",
    "key": "2B36",
    "mappings": {"default": {"default": "flecha de dos cabezas hacia la izquierda de la barra"}}
  }, {
    "category": "Sm",
    "key": "2B37",
    "mappings": {"default": {"default": "flecha triple de dos cabezas hacia la izquierda"}}
  }, {
    "category": "Sm",
    "key": "2B38",
    "mappings": {"default": {"default": "flecha hacia la izquierda con tallo punteado"}}
  }, {
    "category": "Sm",
    "key": "2B39",
    "mappings": {"default": {"default": "flecha hacia la izquierda con cola con trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "2B3A",
    "mappings": {"default": {"default": "flecha hacia la izquierda con cola con doble trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "2B3B",
    "mappings": {"default": {"default": "flecha de dos cabezas hacia la izquierda con cola"}}
  }, {
    "category": "Sm",
    "key": "2B3C",
    "mappings": {"default": {"default": "flecha de dos cabezas hacia la izquierda con cola con trazo vertical"}}
  }, {
    "category": "Sm",
    "key": "2B3D",
    "mappings": {"default": {"default": "flecha de dos cabezas hacia la izquierda con cola con doble movimiento vertical"}}
  }, {
    "category": "Sm",
    "key": "2B3E",
    "mappings": {"default": {"default": "flecha hacia la izquierda a través de X"}}
  }, {
    "category": "Sm",
    "key": "2B3F",
    "mappings": {"default": {"default": "ola flecha apuntando directamente a la izquierda"}}
  }, {
    "category": "Sm",
    "key": "2B40",
    "mappings": {"default": {"default": "signo de igual flecha arriba a la izquierda"}}
  }, {
    "category": "Sm",
    "key": "2B41",
    "mappings": {"default": {"default": "operador de tilde inverso sobre la flecha hacia la izquierda"}}
  }, {
    "category": "Sm",
    "key": "2B42",
    "mappings": {"default": {"default": "flecha hacia la izquierda sobre el reverso casi igual a"}}
  }, {
    "category": "Sm",
    "key": "2B43",
    "mappings": {"default": {"default": "flecha hacia la derecha a través de mayor-que"}}
  }, {
    "category": "Sm",
    "key": "2B44",
    "mappings": {"default": {"default": "flecha hacia la derecha a través de Superset"}}
  }, {
    "category": "So",
    "key": "2B45",
    "mappings": {"default": {"default": "flecha cuádruple hacia la izquierda"}}
  }, {
    "category": "So",
    "key": "2B46",
    "mappings": {"default": {"default": "flecha cuádruple hacia la derecha"}}
  }, {
    "category": "Sm",
    "key": "2B47",
    "mappings": {"default": {"default": "operador de tilde inverso sobre la flecha hacia la derecha"}}
  }, {
    "category": "Sm",
    "key": "2B48",
    "mappings": {"default": {"default": "flecha hacia la derecha sobre el reverso casi igual a"}}
  }, {
    "category": "Sm",
    "key": "2B49",
    "mappings": {"default": {"default": "operador de tilde sobre la flecha hacia la izquierda"}}
  }, {
    "category": "Sm",
    "key": "2B4A",
    "mappings": {"default": {"default": "flecha hacia la izquierda por encima de casi igual a"}}
  }, {
    "category": "Sm",
    "key": "2B4B",
    "mappings": {"default": {"default": "flecha hacia la izquierda sobre el operador de tilde inverso"}}
  }, {
    "category": "Sm",
    "key": "2B4C",
    "mappings": {"default": {"default": "flecha hacia la derecha sobre el operador de tilde inverso"}}
  }, {
    "category": "Sm",
    "key": "FFE9",
    "mappings": {"default": {"default": "flecha hacia la izquierda de medio ancho"}}
  }, {
    "category": "Sm",
    "key": "FFEA",
    "mappings": {"default": {"default": "flecha ascendente de medio ancho"}}
  }, {
    "category": "Sm",
    "key": "FFEB",
    "mappings": {"default": {"default": "flecha de medio ancho hacia la derecha"}}
  }, {"category": "Sm", "key": "FFEC", "mappings": {"default": {"default": "flecha hacia abajo de medio ancho"}}}]
    ,
    "es/symbols/math_characters.js"
:
  [{"locale": "es"}, {
    "key": "2113",
    "mappings": {"default": {"default": "script l"}},
    "category": "Ll"
  }, {"key": "2118", "mappings": {"default": {"default": "p de weierstrass"}}, "category": "Sm"}, {
    "category": "Ll",
    "key": "213C",
    "mappings": {"default": {"default": "negrita de pizarra pi"}}
  }, {
    "category": "Ll",
    "key": "213D",
    "mappings": {"default": {"default": "negrita de pizarra gamma"}}
  }, {
    "category": "Lu",
    "key": "213E",
    "mappings": {"default": {"default": "negrita de pizarra mayúscula Gamma"}}
  }, {
    "category": "Lu",
    "key": "213F",
    "mappings": {"default": {"default": "negrita de pizarra mayúscula Pi"}}
  }, {
    "category": "Sm",
    "key": "2140",
    "mappings": {"default": {"default": "negrita de pizarra suma de n-ary"}}
  }, {"key": "2145", "mappings": {"default": {"default": "mayúscula D"}}, "category": "Lu"}, {
    "key": "2146",
    "mappings": {"default": {"default": "d"}},
    "category": "Ll"
  }, {"key": "2147", "mappings": {"default": {"default": "e"}}, "category": "Ll"}, {
    "key": "2148",
    "mappings": {"default": {"default": "i"}},
    "category": "Ll"
  }, {
    "category": "Ll",
    "key": "2149",
    "mappings": {"default": {"default": "negrita de pizarra cursiva j"}}
  }, {"category": "Ll", "mappings": {"default": {"default": "cursiva sin punto i"}}, "key": "1D6A4"}, {
    "category": "Ll",
    "mappings": {"default": {"default": "cursiva sin punto j"}},
    "key": "1D6A5"
  }]
    ,
    "es/symbols/math_delimiters.js"
:
  [{"locale": "es"}, {
    "key": "0028",
    "mappings": {"default": {"default": "paréntesis izquierdo", "defaultMP": "paréntesis"}},
    "category": "Ps"
  }, {
    "key": "0029",
    "mappings": {"default": {"default": "paréntesis derecho", "defaultMP": "cierra paréntesis"}},
    "category": "Pe"
  }, {
    "key": "005B",
    "mappings": {"default": {"default": "corchete izquierdo", "defaultMP": "abre corchetes"}},
    "category": "Ps"
  }, {
    "key": "005D",
    "mappings": {"default": {"default": "corchete derecho", "defaultMP": "cierra corchetes"}},
    "category": "Pe"
  }, {
    "key": "007B",
    "mappings": {"default": {"default": "llave izquierda", "defaultMP": "abre llaves"}},
    "category": "Ps"
  }, {
    "key": "007D",
    "mappings": {"default": {"default": "llaves derecha", "defaultMP": "cierra llaves"}},
    "category": "Pe"
  }, {
    "category": "Ps",
    "key": "2045",
    "mappings": {"default": {"default": "soporte cuadrado izquierdo con pluma"}}
  }, {
    "category": "Pe",
    "key": "2046",
    "mappings": {"default": {"default": "soporte cuadrado derecho con pluma"}}
  }, {
    "key": "2308",
    "mappings": {"default": {"default": "esquina superior izquierda"}},
    "category": "Sm"
  }, {
    "key": "2309",
    "mappings": {"default": {"default": "esquina superior derecha"}},
    "category": "Sm"
  }, {
    "key": "230A",
    "mappings": {"default": {"default": "esquina inferior izquierda"}},
    "category": "Sm"
  }, {
    "key": "230B",
    "mappings": {"default": {"default": "esquina inferior derecha"}},
    "category": "Sm"
  }, {"key": "230C", "mappings": {"default": {"default": "corte inferior derecho"}}, "category": "So"}, {
    "key": "230D",
    "mappings": {"default": {"default": "corte inferior izquierdo"}},
    "category": "So"
  }, {"key": "230E", "mappings": {"default": {"default": "corte superior derecho "}}, "category": "So"}, {
    "key": "230F",
    "mappings": {"default": {"default": "corte superior izquierdo"}},
    "category": "So"
  }, {
    "key": "231C",
    "mappings": {"default": {"default": "ángulo superior izquierdo"}},
    "category": "So"
  }, {"key": "231D", "mappings": {"default": {"default": "ángulo superior derecho"}}, "category": "So"}, {
    "key": "231E",
    "mappings": {"default": {"default": "ángulo inferior izquierdo"}},
    "category": "So"
  }, {"key": "231F", "mappings": {"default": {"default": "ángulo inferior derecho"}}, "category": "So"}, {
    "key": "2320",
    "mappings": {"default": {"default": "mitad superior de integral"}},
    "category": "Sm"
  }, {
    "key": "2321",
    "mappings": {"default": {"default": "mitad inferior de integral"}},
    "category": "Sm"
  }, {
    "key": "2329",
    "mappings": {"default": {"default": "paréntesis angular izquierdo"}},
    "category": "Ps"
  }, {
    "key": "232A",
    "mappings": {"default": {"default": "paréntesis angular derecho"}},
    "category": "Pe"
  }, {
    "category": "Sm",
    "key": "239B",
    "mappings": {"default": {"default": "paréntesis izquierdo gancho superior"}}
  }, {
    "category": "Sm",
    "key": "239C",
    "mappings": {"default": {"default": "extensión del paréntesis izquierdo"}}
  }, {
    "category": "Sm",
    "key": "239D",
    "mappings": {"default": {"default": "paréntesis inferior gancho inferior"}}
  }, {
    "category": "Sm",
    "key": "239E",
    "mappings": {"default": {"default": "paréntesis derecho gancho superior"}}
  }, {
    "category": "Sm",
    "key": "239F",
    "mappings": {"default": {"default": "extensión del paréntesis derecho"}}
  }, {
    "category": "Sm",
    "key": "23A0",
    "mappings": {"default": {"default": "paréntesis inferior gancho inferior"}}
  }, {
    "category": "Sm",
    "key": "23A1",
    "mappings": {"default": {"default": "esquina cuadrada izquierda esquina superior"}}
  }, {
    "category": "Sm",
    "key": "23A2",
    "mappings": {"default": {"default": "extensión del soporte cuadrado izquierdo"}}
  }, {
    "category": "Sm",
    "key": "23A3",
    "mappings": {"default": {"default": "esquina cuadrada izquierda esquina inferior"}}
  }, {
    "category": "Sm",
    "key": "23A4",
    "mappings": {"default": {"default": "esquina cuadrada derecha esquina superior"}}
  }, {
    "category": "Sm",
    "key": "23A5",
    "mappings": {"default": {"default": "extensión de soporte cuadrado derecho"}}
  }, {
    "category": "Sm",
    "key": "23A6",
    "mappings": {"default": {"default": "esquina cuadrada derecha esquina inferior"}}
  }, {
    "category": "Sm",
    "key": "23A7",
    "mappings": {"default": {"default": "gancho superior rizado izquierdo"}}
  }, {
    "category": "Sm",
    "key": "23A8",
    "mappings": {"default": {"default": "pedazo medio rizado izquierdo"}}
  }, {
    "category": "Sm",
    "key": "23A9",
    "mappings": {"default": {"default": "soporte izquierdo rizado gancho inferior"}}
  }, {
    "category": "Sm",
    "key": "23AA",
    "mappings": {"default": {"default": "extensión de soporte rizado"}}
  }, {
    "category": "Sm",
    "key": "23AB",
    "mappings": {"default": {"default": "gancho superior rizado derecho"}}
  }, {
    "category": "Sm",
    "key": "23AC",
    "mappings": {"default": {"default": "pedazo medio de soporte rizado derecho"}}
  }, {
    "category": "Sm",
    "key": "23AD",
    "mappings": {"default": {"default": "abrazadera inferior derecha rizado"}}
  }, {"category": "Sm", "key": "23AE", "mappings": {"default": {"default": "extensión integral"}}}, {
    "category": "Sm",
    "key": "23AF",
    "mappings": {"default": {"default": "extensión de línea horizontal"}}
  }, {
    "key": "23B0",
    "mappings": {"default": {"default": "izquierda arriba o derecho debajo sección llave"}},
    "category": "Sm"
  }, {
    "key": "23B1",
    "mappings": {"default": {"default": "derecha arriba o izquierdo debajo sección llave"}},
    "category": "Sm"
  }, {"category": "Sm", "key": "23B2", "mappings": {"default": {"default": "suma superior"}}}, {
    "category": "Sm",
    "key": "23B3",
    "mappings": {"default": {"default": "parte inferior de la suma"}}
  }, {"key": "23B4", "mappings": {"default": {"default": "corchete superior"}}, "category": "So"}, {
    "key": "23B5",
    "mappings": {"default": {"default": "corchete inferior"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "23B6",
    "mappings": {"default": {"default": "soporte cuadrado inferior sobre soporte cuadrado superior"}}
  }, {
    "category": "So",
    "key": "23B7",
    "mappings": {"default": {"default": "parte inferior del símbolo radical"}}
  }, {
    "category": "So",
    "key": "23B8",
    "mappings": {"default": {"default": "línea de caja vertical izquierda"}}
  }, {
    "category": "So",
    "key": "23B9",
    "mappings": {"default": {"default": "línea de caja vertical derecha"}}
  }, {"key": "23DC", "mappings": {"default": {"default": "paréntesis superior"}}, "category": "Sm"}, {
    "key": "23DD",
    "mappings": {"default": {"default": "paréntesis inferior"}},
    "category": "Sm"
  }, {
    "key": "23DE",
    "mappings": {"default": {"default": "corchete rizado superior"}},
    "category": "Sm"
  }, {
    "key": "23DF",
    "mappings": {"default": {"default": "corchete rizado inferior"}},
    "category": "Sm"
  }, {
    "key": "23E0",
    "mappings": {"default": {"default": "corchete superior en tortuga"}},
    "category": "Sm"
  }, {
    "key": "23E1",
    "mappings": {"default": {"default": "corchete inferior en tortuga"}},
    "category": "Sm"
  }, {
    "category": "Ps",
    "key": "2768",
    "mappings": {"default": {"default": "ornamento de paréntesis izquierdo medio"}}
  }, {
    "category": "Pe",
    "key": "2769",
    "mappings": {"default": {"default": "adorno de paréntesis medio derecho"}}
  }, {
    "category": "Ps",
    "key": "276A",
    "mappings": {"default": {"default": "adorno paréntesis izquierdo aplanado medio"}}
  }, {
    "category": "Pe",
    "key": "276B",
    "mappings": {"default": {"default": "adorno paréntesis derecho medio aplanado"}}
  }, {
    "category": "Ps",
    "key": "276C",
    "mappings": {"default": {"default": "adorno de soporte de ángulo medio apuntando hacia la izquierda"}}
  }, {
    "category": "Pe",
    "key": "276D",
    "mappings": {"default": {"default": "adorno de soporte de ángulo medio apuntando hacia la derecha"}}
  }, {
    "category": "Ps",
    "key": "276E",
    "mappings": {"default": {"default": "ornamento de comillas en el ángulo que apunta hacia la izquierda"}}
  }, {
    "category": "Pe",
    "key": "276F",
    "mappings": {"default": {"default": "ornamento de comillas en ángulo recto que apunta hacia la derecha"}}
  }, {
    "category": "Ps",
    "key": "2770",
    "mappings": {"default": {"default": "ornamento pesado del soporte del ángulo que señala a la izquierda"}}
  }, {
    "category": "Pe",
    "key": "2771",
    "mappings": {"default": {"default": "ornamento pesado del soporte del ángulo que señala hacia la derecha"}}
  }, {
    "category": "Ps",
    "key": "2772",
    "mappings": {"default": {"default": "ornamento de soporte de concha de tortuga izquierda ligera"}}
  }, {
    "category": "Pe",
    "key": "2773",
    "mappings": {"default": {"default": "ornamento de corchete derecho tortuga derecha"}}
  }, {
    "category": "Ps",
    "key": "2774",
    "mappings": {"default": {"default": "ornamento de soporte rizado izquierdo medio"}}
  }, {
    "category": "Pe",
    "key": "2775",
    "mappings": {"default": {"default": "ornamento de soporte rizado derecho medio"}}
  }, {
    "key": "27C5",
    "mappings": {"default": {"default": "símbolo delimitador izquierdo de bolsa de s-formó"}},
    "category": "Ps"
  }, {
    "key": "27C6",
    "mappings": {"default": {"default": "símbolo delimitador derecho de bolsa de s-formó"}},
    "category": "Pe"
  }, {
    "key": "27E6",
    "mappings": {"default": {"default": "corchete vacío izquierdo"}},
    "category": "Ps"
  }, {"key": "27E7", "mappings": {"default": {"default": "corchete vacío derecho"}}, "category": "Pe"}, {
    "key": "27E8",
    "mappings": {"default": {"default": "corchete agudo izquierdo"}},
    "category": "Ps"
  }, {"key": "27E9", "mappings": {"default": {"default": "corchete agudo derecho"}}, "category": "Pe"}, {
    "key": "27EA",
    "mappings": {"default": {"default": "doble corchete agudo izquierdo"}},
    "category": "Ps"
  }, {
    "key": "27EB",
    "mappings": {"default": {"default": "doble corchete agudo derecho"}},
    "category": "Pe"
  }, {
    "key": "27EC",
    "mappings": {"default": {"default": "corchete de tortuga vacío izquierdo"}},
    "category": "Ps"
  }, {
    "key": "27ED",
    "mappings": {"default": {"default": "corchete de tortuga vacío derecho "}},
    "category": "Pe"
  }, {
    "key": "27EE",
    "mappings": {"default": {"default": "paréntesis plano izquierdo"}},
    "category": "Ps"
  }, {
    "key": "27EF",
    "mappings": {"default": {"default": "paréntesis plano derecho"}},
    "category": "Pe"
  }, {
    "category": "Ps",
    "key": "2983",
    "mappings": {"default": {"default": "soporte rizado blanco izquierdo"}}
  }, {
    "category": "Pe",
    "key": "2984",
    "mappings": {"default": {"default": "derecha rizado blanco soporte"}}
  }, {"key": "2985", "mappings": {"default": {"default": "left white paren"}}, "category": "Ps"}, {
    "key": "2986",
    "mappings": {"default": {"default": "right white paren"}},
    "category": "Pe"
  }, {
    "category": "Ps",
    "key": "2987",
    "mappings": {"default": {"default": "notación Z izquierda soporte de imagen"}}
  }, {
    "category": "Pe",
    "key": "2988",
    "mappings": {"default": {"default": "corchete de imagen derecha de notación Z"}}
  }, {
    "key": "2989",
    "mappings": {"default": {"default": "z notation left binding bracket"}},
    "category": "Ps"
  }, {
    "key": "298A",
    "mappings": {"default": {"default": "z notation right binding bracket"}},
    "category": "Pe"
  }, {
    "key": "298B",
    "mappings": {"default": {"default": "left bracket with underbar"}},
    "category": "Ps"
  }, {
    "key": "298C",
    "mappings": {"default": {"default": "right bracket with underbar"}},
    "category": "Pe"
  }, {
    "key": "298D",
    "mappings": {"default": {"default": "left bracket with tick in top corner"}},
    "category": "Ps"
  }, {
    "key": "298E",
    "mappings": {"default": {"default": "right bracket with tick in bottom corner"}},
    "category": "Pe"
  }, {
    "key": "298F",
    "mappings": {"default": {"default": "left bracket with tick in bottom corner"}},
    "category": "Ps"
  }, {
    "key": "2990",
    "mappings": {"default": {"default": "right bracket with tick in top corner"}},
    "category": "Pe"
  }, {
    "key": "2991",
    "mappings": {"default": {"default": "left angle bracket with dot"}},
    "category": "Ps"
  }, {
    "key": "2992",
    "mappings": {"default": {"default": "right angle bracket with dot"}},
    "category": "Pe"
  }, {
    "key": "2993",
    "mappings": {"default": {"default": "left arc less than bracket"}},
    "category": "Ps"
  }, {
    "key": "2994",
    "mappings": {"default": {"default": "right arc greater than bracket"}},
    "category": "Pe"
  }, {
    "key": "2995",
    "mappings": {"default": {"default": "double left arc greater than bracket"}},
    "category": "Ps"
  }, {
    "key": "2996",
    "mappings": {"default": {"default": "double right arc less than bracket"}},
    "category": "Pe"
  }, {
    "category": "Ps",
    "key": "2997",
    "mappings": {"default": {"default": "soporte de concha de tortuga negra izquierda"}}
  }, {
    "category": "Pe",
    "key": "2998",
    "mappings": {"default": {"default": "abrazadera de concha de tortuga negra derecha"}}
  }, {
    "category": "Ps",
    "key": "29D8",
    "mappings": {"default": {"default": "valla izquierda ondulada"}}
  }, {"category": "Pe", "key": "29D9", "mappings": {"default": {"default": "valla derecha ondulada"}}}, {
    "key": "29DA",
    "mappings": {"default": {"default": "left double wiggly fence"}},
    "category": "Ps"
  }, {
    "key": "29DB",
    "mappings": {"default": {"default": "right double wiggly fence"}},
    "category": "Pe"
  }, {
    "category": "Ps",
    "key": "29FC",
    "mappings": {"default": {"default": "abrazadera de ángulo curvo a la izquierda"}}
  }, {
    "category": "Pe",
    "key": "29FD",
    "mappings": {"default": {"default": "abrazadera de ángulo curvo orientada hacia la derecha"}}
  }, {
    "category": "Ps",
    "key": "2E22",
    "mappings": {"default": {"default": "soporte superior izquierdo superior"}}
  }, {
    "category": "Pe",
    "key": "2E23",
    "mappings": {"default": {"default": "medio soporte superior derecho"}}
  }, {
    "category": "Ps",
    "key": "2E24",
    "mappings": {"default": {"default": "abrazadera inferior izquierda"}}
  }, {
    "category": "Pe",
    "key": "2E25",
    "mappings": {"default": {"default": "medio soporte inferior derecho"}}
  }, {
    "category": "Ps",
    "key": "2E26",
    "mappings": {"default": {"default": "soporte U lateral izquierdo"}}
  }, {
    "category": "Pe",
    "key": "2E27",
    "mappings": {"default": {"default": "lado derecho U soporte"}}
  }, {
    "category": "Ps",
    "key": "2E28",
    "mappings": {"default": {"default": "doble paréntesis izquierdo"}}
  }, {
    "category": "Pe",
    "key": "2E29",
    "mappings": {"default": {"default": "paréntesis doble derecho"}}
  }, {
    "category": "Ps",
    "key": "3008",
    "mappings": {"default": {"default": "soporte de ángulo izquierdo"}}
  }, {"category": "Pe", "key": "3009", "mappings": {"default": {"default": "soporte de ángulo recto"}}}, {
    "key": "300A",
    "mappings": {"default": {"default": "paréntesis angular doble"}},
    "category": "Ps"
  }, {
    "key": "300B",
    "mappings": {
      "default": {
        "default": "paréntesis angular doble derecho",
        "defaultMP": "cierra paréntesis angular doble"
      }
    },
    "category": "Pe"
  }, {
    "category": "Ps",
    "key": "300C",
    "mappings": {"default": {"default": "soporte de la esquina izquierda"}}
  }, {
    "category": "Pe",
    "key": "300D",
    "mappings": {"default": {"default": "soporte de esquina derecha"}}
  }, {
    "category": "Ps",
    "key": "300E",
    "mappings": {"default": {"default": "soporte de esquina blanco izquierdo"}}
  }, {
    "category": "Pe",
    "key": "300F",
    "mappings": {"default": {"default": "soporte de esquina blanco derecho"}}
  }, {
    "category": "Ps",
    "key": "3010",
    "mappings": {"default": {"default": "soporte Lenticular Izquierdo Negro"}}
  }, {
    "category": "Pe",
    "key": "3011",
    "mappings": {"default": {"default": "soporte Lenticular Negro Derecho"}}
  }, {
    "key": "3014",
    "mappings": {"default": {"default": "left tortoise shell bracket"}},
    "category": "Ps"
  }, {
    "key": "3015",
    "mappings": {"default": {"default": "right tortoise shell bracket"}},
    "category": "Pe"
  }, {
    "category": "Ps",
    "key": "3016",
    "mappings": {"default": {"default": "soporte lenticular blanco izquierdo"}}
  }, {
    "category": "Pe",
    "key": "3017",
    "mappings": {"default": {"default": "soporte lenticular blanco derecho"}}
  }, {
    "category": "Ps",
    "key": "3018",
    "mappings": {"default": {"default": "soporte de caparazón de tortuga blanca izquierda"}}
  }, {
    "category": "Pe",
    "key": "3019",
    "mappings": {"default": {"default": "soporte de concha de tortuga blanca derecha"}}
  }, {"key": "301A", "mappings": {"default": {"default": "corchete vacío"}}, "category": "Ps"}, {
    "key": "301B",
    "mappings": {"default": {"default": "corchete vacío derecho", "defaultMP": "cierra corchete vacío"}},
    "category": "Pe"
  }, {"category": "Ps", "key": "301D", "mappings": {"default": {"default": "comilla doble invertida"}}}, {
    "key": "301E",
    "mappings": {"default": {"default": "segunda"}},
    "category": "Pe"
  }, {
    "category": "Pe",
    "key": "301F",
    "mappings": {"default": {"default": "comillas bajas dobles"}}
  }, {
    "category": "Ps",
    "key": "FD3E",
    "mappings": {"default": {"default": "paréntesis izquierdo adornado"}}
  }, {
    "category": "Pe",
    "key": "FD3F",
    "mappings": {"default": {"default": "paréntesis derecho adornado"}}
  }, {
    "category": "Ps",
    "key": "FE17",
    "mappings": {"default": {"default": "formulario de presentación para soporte lenticular blanco izquierdo vertical"}}
  }, {
    "category": "Pe",
    "key": "FE18",
    "mappings": {"default": {"default": "formulario de presentación para soporte lenticular blanco vertical derecho"}}
  }, {"key": "FE35", "mappings": {"default": {"default": "paréntesis superior"}}, "category": "Ps"}, {
    "key": "FE36",
    "mappings": {"default": {"default": "paréntesis inferior"}},
    "category": "Pe"
  }, {"key": "FE37", "mappings": {"default": {"default": "llave superior"}}, "category": "Ps"}, {
    "key": "FE38",
    "mappings": {"default": {"default": "llave inferior"}},
    "category": "Pe"
  }, {
    "category": "Ps",
    "key": "FE39",
    "mappings": {"default": {"default": "formulario de presentación para el soporte vertical de concha de tortuga izquierda"}}
  }, {
    "category": "Pe",
    "key": "FE3A",
    "mappings": {"default": {"default": "formulario de presentación para el soporte vertical de concha de tortuga derecha"}}
  }, {
    "category": "Ps",
    "key": "FE3B",
    "mappings": {"default": {"default": "formulario de presentación para el soporte lenticular negro izquierdo vertical"}}
  }, {
    "category": "Pe",
    "key": "FE3C",
    "mappings": {"default": {"default": "formulario de presentación para el soporte lenticular negro derecho vertical"}}
  }, {
    "category": "Ps",
    "key": "FE3D",
    "mappings": {"default": {"default": "formulario de presentación para soporte vertical doble ángulo izquierdo"}}
  }, {
    "category": "Pe",
    "key": "FE3E",
    "mappings": {"default": {"default": "formulario de presentación para el soporte de ángulo recto doble vertical"}}
  }, {
    "key": "FE3F",
    "mappings": {"default": {"default": "paréntesis angular superior"}},
    "category": "Ps"
  }, {
    "key": "FE40",
    "mappings": {"default": {"default": "paréntesis angular inferior"}},
    "category": "Pe"
  }, {
    "category": "Ps",
    "key": "FE41",
    "mappings": {"default": {"default": "formulario de presentación para soporte de esquina vertical izquierda"}}
  }, {
    "category": "Pe",
    "key": "FE42",
    "mappings": {"default": {"default": "formulario de presentación para soporte de esquina vertical derecha"}}
  }, {
    "category": "Ps",
    "key": "FE43",
    "mappings": {"default": {"default": "formulario de presentación para soporte de esquina blanco vertical izquierdo"}}
  }, {
    "category": "Pe",
    "key": "FE44",
    "mappings": {"default": {"default": "formulario de presentación para soporte de esquina vertical derecha blanca"}}
  }, {
    "category": "Ps",
    "key": "FE47",
    "mappings": {"default": {"default": "formulario de presentación para soporte cuadrado vertical izquierdo"}}
  }, {
    "category": "Pe",
    "key": "FE48",
    "mappings": {"default": {"default": "formulario de presentación para soporte cuadrado vertical derecho"}}
  }, {
    "category": "Ps",
    "key": "FE59",
    "mappings": {"default": {"default": "pequeño paréntesis izquierdo"}}
  }, {
    "category": "Pe",
    "key": "FE5A",
    "mappings": {"default": {"default": "paréntesis derecho pequeño"}}
  }, {
    "category": "Ps",
    "key": "FE5B",
    "mappings": {"default": {"default": "corchete Rizado Izquierdo Pequeño"}}
  }, {
    "category": "Pe",
    "key": "FE5C",
    "mappings": {"default": {"default": "corchete derecho rizado pequeño"}}
  }, {
    "category": "Ps",
    "key": "FE5D",
    "mappings": {"default": {"default": "soporte de concha pequeña tortuga izquierda"}}
  }, {
    "category": "Pe",
    "key": "FE5E",
    "mappings": {"default": {"default": "pequeño corchete derecho de concha de tortuga"}}
  }, {
    "category": "Ps",
    "key": "FF08",
    "mappings": {"default": {"default": "paréntesis izquierdo de ancho completo"}}
  }, {
    "category": "Pe",
    "key": "FF09",
    "mappings": {"default": {"default": "paréntesis derecho de ancho completo"}}
  }, {
    "category": "Ps",
    "key": "FF3B",
    "mappings": {"default": {"default": "soporte cuadrado izquierdo de ancho completo"}}
  }, {
    "category": "Pe",
    "key": "FF3D",
    "mappings": {"default": {"default": "soporte cuadrado derecho de ancho completo"}}
  }, {
    "category": "Ps",
    "key": "FF5B",
    "mappings": {"default": {"default": "corchete rizado izquierdo de ancho completo"}}
  }, {
    "category": "Pe",
    "key": "FF5D",
    "mappings": {"default": {"default": "corchete derecho de ancho completo"}}
  }, {
    "category": "Ps",
    "key": "FF5F",
    "mappings": {"default": {"default": "paréntesis blanco izquierdo de ancho completo"}}
  }, {
    "category": "Pe",
    "key": "FF60",
    "mappings": {"default": {"default": "paréntesis blanco derecho de ancho completo"}}
  }, {
    "category": "Ps",
    "key": "FF62",
    "mappings": {"default": {"default": "corchete de esquina izquierda de medio ancho"}}
  }, {
    "category": "Pe",
    "key": "FF63",
    "mappings": {"default": {"default": "soporte de esquina derecha de medio ancho"}}
  }]
    ,
    "es/symbols/math_geometry.js"
:
  [{"locale": "es"}, {
    "key": "2500",
    "mappings": {"default": {"default": "delimitador horizontal"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "2501",
    "mappings": {"default": {"default": "dibujos de cajas pesadas horizontales"}}
  }, {"key": "2502", "mappings": {"default": {"default": "delimitador vertical"}}, "category": "So"}, {
    "category": "So",
    "key": "2503",
    "mappings": {"default": {"default": "dibujos de cajas Pesado Vertical"}}
  }, {
    "category": "So",
    "key": "2504",
    "mappings": {"default": {"default": "dibujos de cajas Light Triple Dash Horizontal"}}
  }, {
    "category": "So",
    "key": "2505",
    "mappings": {"default": {"default": "dibujos de cajas Heavy Triple Dash Horizontal"}}
  }, {
    "category": "So",
    "key": "2506",
    "mappings": {"default": {"default": "dibujos de cajas Light Triple Dash Vertical"}}
  }, {
    "category": "So",
    "key": "2507",
    "mappings": {"default": {"default": "dibujos de cajas Heavy Triple Dash Vertical"}}
  }, {
    "category": "So",
    "key": "2508",
    "mappings": {"default": {"default": "caja Dibujos Ligero Cuadruple Dash Horizontal"}}
  }, {
    "category": "So",
    "key": "2509",
    "mappings": {"default": {"default": "dibujos de cajas Heavy Quadruple Dash Horizontal"}}
  }, {
    "category": "So",
    "key": "250A",
    "mappings": {"default": {"default": "caja Dibujos Luz Cuadruple Tablero Vertical"}}
  }, {
    "category": "So",
    "key": "250B",
    "mappings": {"default": {"default": "dibujos de cajas Heavy Quadruple Dash Vertical"}}
  }, {
    "key": "250C",
    "mappings": {"default": {"default": "esquina inferior derecha"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "250D",
    "mappings": {"default": {"default": "dibujos de cajas Abajo Ligero y Derecho Pesado"}}
  }, {
    "category": "So",
    "key": "250E",
    "mappings": {"default": {"default": "dibujos de cajas abajo pesados ​​y rectos ligeros"}}
  }, {
    "category": "So",
    "key": "250F",
    "mappings": {"default": {"default": "dibujos de cajas pesados ​​abajo y derecha"}}
  }, {
    "key": "2510",
    "mappings": {"default": {"default": "esquina inferior izquierda"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "2511",
    "mappings": {"default": {"default": "dibujos de cajas abajo ligeros y pesados ​​a la izquierda"}}
  }, {
    "category": "So",
    "key": "2512",
    "mappings": {"default": {"default": "dibujos de cajas abajo pesados ​​y luz izquierda"}}
  }, {
    "category": "So",
    "key": "2513",
    "mappings": {"default": {"default": "dibujos de cajas pesados ​​hacia abajo y hacia la izquierda"}}
  }, {
    "key": "2514",
    "mappings": {"default": {"default": "esquina superior derecha"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "2515",
    "mappings": {"default": {"default": "dibujos de cajas hasta ligero y derecho pesado"}}
  }, {
    "category": "So",
    "key": "2516",
    "mappings": {"default": {"default": "dibujos de cajas hasta Pesadas y Ligeras"}}
  }, {
    "category": "So",
    "key": "2517",
    "mappings": {"default": {"default": "dibujos de cajas pesados ​​y derechos"}}
  }, {
    "key": "2518",
    "mappings": {"default": {"default": "esquina superior izquierda"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "2519",
    "mappings": {"default": {"default": "dibujos de cajas hasta livianos y pesados ​​a la izquierda"}}
  }, {
    "category": "So",
    "key": "251A",
    "mappings": {"default": {"default": "dibujos de cajas hasta pesado y luz izquierda"}}
  }, {
    "category": "So",
    "key": "251B",
    "mappings": {"default": {"default": "dibujos de cajas pesados ​​hacia arriba y hacia la izquierda"}}
  }, {
    "key": "251C",
    "mappings": {"default": {"default": "delimitador izquierdo y separador horizontal"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "251D",
    "mappings": {"default": {"default": "cuadros de caja Vertical ligero y derecho pesado"}}
  }, {
    "category": "So",
    "key": "251E",
    "mappings": {"default": {"default": "dibujos de cajas hasta pesados ​​y rectos hacia abajo"}}
  }, {
    "category": "So",
    "key": "251F",
    "mappings": {"default": {"default": "dibujos de cajas abajo pesados ​​y rectos arriba ligeros"}}
  }, {
    "category": "So",
    "key": "2520",
    "mappings": {"default": {"default": "dibujos de cajas verticales pesadas y rectas ligeras"}}
  }, {
    "category": "So",
    "key": "2521",
    "mappings": {"default": {"default": "dibujos de cajas abajo ligeros y rectos arriba pesados"}}
  }, {
    "category": "So",
    "key": "2522",
    "mappings": {"default": {"default": "dibujos de cajas para arriba livianos y rectos para abajo pesados"}}
  }, {
    "category": "So",
    "key": "2523",
    "mappings": {"default": {"default": "dibujos de cajas pesadas verticales y derechas"}}
  }, {
    "key": "2524",
    "mappings": {"default": {"default": "delimitador derecho y separador horizontal"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "2525",
    "mappings": {"default": {"default": "cuadros de caja Vertical ligero y izquierdo pesado"}}
  }, {
    "category": "So",
    "key": "2526",
    "mappings": {"default": {"default": "dibujos de cajas para arriba Pesado y Izquierda Abajo Luz"}}
  }, {
    "category": "So",
    "key": "2527",
    "mappings": {"default": {"default": "dibujos de cajas abajo pesados ​​y luz izquierda arriba"}}
  }, {
    "category": "So",
    "key": "2528",
    "mappings": {"default": {"default": "dibujos de cajas verticales pesadas y luz izquierda"}}
  }, {
    "category": "So",
    "key": "2529",
    "mappings": {"default": {"default": "dibujos de cajas abajo ligeros e izquierdos arriba pesados"}}
  }, {
    "category": "So",
    "key": "252A",
    "mappings": {"default": {"default": "dibujos de cajas para arriba Ligero y Izquierda Abajo Pesado"}}
  }, {
    "category": "So",
    "key": "252B",
    "mappings": {"default": {"default": "dibujos de cajas pesadas verticales e izquierdas"}}
  }, {
    "key": "252C",
    "mappings": {"default": {"default": "delimitador superior y separador vertical"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "252D",
    "mappings": {"default": {"default": "dibujos de cajas izquierda pesados ​​y derecha abajo luz"}}
  }, {
    "category": "So",
    "key": "252E",
    "mappings": {"default": {"default": "dibujos de cajas, derecha, pesada y izquierda abajo, luz"}}
  }, {
    "category": "So",
    "key": "252F",
    "mappings": {"default": {"default": "dibujos de cajas abajo ligeros y horizontales pesados"}}
  }, {
    "category": "So",
    "key": "2530",
    "mappings": {"default": {"default": "dibujos de cajas de luz pesada y horizontal"}}
  }, {
    "category": "So",
    "key": "2531",
    "mappings": {"default": {"default": "dibujos de la caja de luz derecha y izquierda abajo pesado"}}
  }, {
    "category": "So",
    "key": "2532",
    "mappings": {"default": {"default": "cuadros de caja izquierda ligera y derecha abajo pesada"}}
  }, {
    "category": "So",
    "key": "2533",
    "mappings": {"default": {"default": "dibujos de cajas pesados ​​y horizontales"}}
  }, {
    "key": "2534",
    "mappings": {"default": {"default": "delimitador inferior y separador vertical"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "2535",
    "mappings": {"default": {"default": "dibujos de cajas izquierda pesada y derecha arriba ligera"}}
  }, {
    "category": "So",
    "key": "2536",
    "mappings": {"default": {"default": "dibujos de caja derecha pesada y luz izquierda arriba"}}
  }, {
    "category": "So",
    "key": "2537",
    "mappings": {"default": {"default": "dibujos de cajas hasta ligero y horizontal pesado"}}
  }, {
    "category": "So",
    "key": "2538",
    "mappings": {"default": {"default": "dibujos de cajas hasta luz pesada y horizontal"}}
  }, {
    "category": "So",
    "key": "2539",
    "mappings": {"default": {"default": "dibujos de cajas a la derecha, livianos y a la izquierda, pesados"}}
  }, {
    "category": "So",
    "key": "253A",
    "mappings": {"default": {"default": "dibujos de cajas Izquierda Ligera y Derecha Arriba Pesada"}}
  }, {
    "category": "So",
    "key": "253B",
    "mappings": {"default": {"default": "dibujos de cajas pesados ​​y horizontales"}}
  }, {
    "key": "253C",
    "mappings": {"default": {"default": "cruce de separadores sencillos"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "253D",
    "mappings": {"default": {"default": "dibujos de cajas Izquierda Pesada y Derecha Vertical Luz"}}
  }, {
    "category": "So",
    "key": "253E",
    "mappings": {"default": {"default": "dibujos de caja derecha pesada y luz vertical izquierda"}}
  }, {
    "category": "So",
    "key": "253F",
    "mappings": {"default": {"default": "dibujos de cajas Vertical Ligera y Horizontal Pesada"}}
  }, {
    "category": "So",
    "key": "2540",
    "mappings": {"default": {"default": "dibujos de cajas para arriba pesados ​​y abajo luz horizontal"}}
  }, {
    "category": "So",
    "key": "2541",
    "mappings": {"default": {"default": "dibujos de cajas hacia abajo pesados ​​y hacia arriba luz horizontal"}}
  }, {
    "category": "So",
    "key": "2542",
    "mappings": {"default": {"default": "dibujos de cajas verticales pesadas y horizontales ligeras"}}
  }, {
    "category": "So",
    "key": "2543",
    "mappings": {"default": {"default": "dibujos de cajas Izquierda Arriba Pesada y Derecha abajo Luz"}}
  }, {
    "category": "So",
    "key": "2544",
    "mappings": {"default": {"default": "dibujos de caja derecha arriba pesada y izquierda abajo luz"}}
  }, {
    "category": "So",
    "key": "2545",
    "mappings": {"default": {"default": "dibujos de cajas, izquierda abajo, pesada y derecha arriba, ligera"}}
  }, {
    "category": "So",
    "key": "2546",
    "mappings": {"default": {"default": "dibujos de la caja derecha abajo pesada y izquierda arriba ligera"}}
  }, {
    "category": "So",
    "key": "2547",
    "mappings": {"default": {"default": "dibujos de cajas abajo ligeros y ascendentes horizontales pesados"}}
  }, {
    "category": "So",
    "key": "2548",
    "mappings": {"default": {"default": "dibujos de cajas para arriba Ligero y para abajo horizontal pesado"}}
  }, {
    "category": "So",
    "key": "2549",
    "mappings": {"default": {"default": "dibujos de cajas a la derecha, ligeros y a la izquierda, verticales, pesados"}}
  }, {
    "category": "So",
    "key": "254A",
    "mappings": {"default": {"default": "dibujos de cajas Izquierda Ligera y Derecha Vertical Pesada"}}
  }, {
    "category": "So",
    "key": "254B",
    "mappings": {"default": {"default": "dibujos de cajas pesadas verticales y horizontales"}}
  }, {
    "category": "So",
    "key": "254C",
    "mappings": {"default": {"default": "dibujos de cajas Light Double Dash Horizontal"}}
  }, {
    "category": "So",
    "key": "254D",
    "mappings": {"default": {"default": "dibujos de cajas Heavy Double Dash Horizontal"}}
  }, {
    "category": "So",
    "key": "254E",
    "mappings": {"default": {"default": "dibujos de cajas Light Double Dash Vertical"}}
  }, {
    "category": "So",
    "key": "254F",
    "mappings": {"default": {"default": "dibujos de cajas Heavy Double Dash Vertical"}}
  }, {
    "key": "2550",
    "mappings": {"default": {"default": "delimitador horizontal doble"}},
    "category": "So"
  }, {
    "key": "2551",
    "mappings": {"default": {"default": "delimitador vertical doble"}},
    "category": "So"
  }, {
    "key": "2552",
    "mappings": {"default": {"default": "esquina inferior derecha, vertical doble"}},
    "category": "So"
  }, {
    "key": "2553",
    "mappings": {"default": {"default": "esquina inferior derecha, horizontal doble"}},
    "category": "So"
  }, {
    "key": "2554",
    "mappings": {"default": {"default": "esquina inferior derecha doble"}},
    "category": "So"
  }, {
    "key": "2555",
    "mappings": {"default": {"default": "esquina inferior izquierda, vertical doble"}},
    "category": "So"
  }, {
    "key": "2556",
    "mappings": {"default": {"default": "esquina inferior izquierda, horizontal doble"}},
    "category": "So"
  }, {
    "key": "2557",
    "mappings": {"default": {"default": "esquina inferior izquierda doble"}},
    "category": "So"
  }, {
    "key": "2558",
    "mappings": {"default": {"default": "esquina superior derecha, vertical doble"}},
    "category": "So"
  }, {
    "key": "2559",
    "mappings": {"default": {"default": "esquina superior derecha, horizontal doble"}},
    "category": "So"
  }, {
    "key": "255A",
    "mappings": {"default": {"default": "esquina superior derecha doble"}},
    "category": "So"
  }, {
    "key": "255B",
    "mappings": {"default": {"default": "esquina superior izquierda, vertical doble"}},
    "category": "So"
  }, {
    "key": "255C",
    "mappings": {"default": {"default": "esquina superior izquierda, horizontal doble"}},
    "category": "So"
  }, {
    "key": "255D",
    "mappings": {"default": {"default": "esquina superior izquierda doble"}},
    "category": "So"
  }, {
    "key": "255E",
    "mappings": {"default": {"default": "delimitador izquierdo con separador doble"}},
    "category": "So"
  }, {
    "key": "255F",
    "mappings": {"default": {"default": "delimitador izquierdo doble con separador horizontal"}},
    "category": "So"
  }, {
    "key": "2560",
    "mappings": {"default": {"default": "delimitador izquierdo y separador horizontal dobles"}},
    "category": "So"
  }, {
    "key": "2561",
    "mappings": {"default": {"default": "delimitador derecho con separador doble"}},
    "category": "So"
  }, {
    "key": "2562",
    "mappings": {"default": {"default": "delimitador derecho doble con separador horizontal"}},
    "category": "So"
  }, {
    "key": "2563",
    "mappings": {"default": {"default": "delimitador derecho y separador horizontal dobles"}},
    "category": "So"
  }, {
    "key": "2564",
    "mappings": {"default": {"default": "delimitador superior doble con separador vertical"}},
    "category": "So"
  }, {
    "key": "2565",
    "mappings": {"default": {"default": "delimitador superior con separador vertical doble"}},
    "category": "So"
  }, {
    "key": "2566",
    "mappings": {"default": {"default": "delimitador superior y separador vertical dobles"}},
    "category": "So"
  }, {
    "key": "2567",
    "mappings": {"default": {"default": "delimitador inferior doble con separador vertical"}},
    "category": "So"
  }, {
    "key": "2568",
    "mappings": {"default": {"default": "delimitador inferior con separador vertical doble"}},
    "category": "So"
  }, {
    "key": "2569",
    "mappings": {"default": {"default": "delimitador inferior y separador vertical dobles"}},
    "category": "So"
  }, {
    "key": "256A",
    "mappings": {"default": {"default": "cruce de separadores, vertical sencillo y horizontal doble"}},
    "category": "So"
  }, {
    "key": "256B",
    "mappings": {"default": {"default": "cruce de separadores, vertical doble y horizontal sencillo"}},
    "category": "So"
  }, {
    "key": "256C",
    "mappings": {"default": {"default": "cruce de separadores dobles"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "256D",
    "mappings": {"default": {"default": "dibujos de cajas de luz arco abajo y derecha"}}
  }, {
    "category": "So",
    "key": "256E",
    "mappings": {"default": {"default": "dibujos de cajas de luz arco hacia abajo y hacia la izquierda"}}
  }, {
    "category": "So",
    "key": "256F",
    "mappings": {"default": {"default": "dibujos de cajas de luz arco arriba y a la izquierda"}}
  }, {
    "category": "So",
    "key": "2570",
    "mappings": {"default": {"default": "caja de dibujos de luz arco arriba y derecha"}}
  }, {
    "category": "So",
    "key": "2571",
    "mappings": {"default": {"default": "dibujos de cajas Luz Diagonal Superior derecha a inferior izquierda"}}
  }, {
    "category": "So",
    "key": "2572",
    "mappings": {"default": {"default": "dibujos de cajas Luz Diagonal Superior izquierda a inferior derecha"}}
  }, {
    "category": "So",
    "key": "2573",
    "mappings": {"default": {"default": "dibujos de cajas de luz diagonal cruz"}}
  }, {
    "category": "So",
    "key": "2574",
    "mappings": {"default": {"default": "dibujos de cajas luz izquierda"}}
  }, {
    "category": "So",
    "key": "2575",
    "mappings": {"default": {"default": "dibujos de cajas se iluminan"}}
  }, {
    "category": "So",
    "key": "2576",
    "mappings": {"default": {"default": "dibujos de cajas a la derecha"}}
  }, {
    "category": "So",
    "key": "2577",
    "mappings": {"default": {"default": "dibujos de cajas de luz hacia abajo"}}
  }, {
    "category": "So",
    "key": "2578",
    "mappings": {"default": {"default": "dibujos de cajas pesados ​​a la izquierda"}}
  }, {
    "category": "So",
    "key": "2579",
    "mappings": {"default": {"default": "dibujos de cajas pesados"}}
  }, {
    "category": "So",
    "key": "257A",
    "mappings": {"default": {"default": "caja de dibujos pesado derecho"}}
  }, {
    "category": "So",
    "key": "257B",
    "mappings": {"default": {"default": "dibujos de cajas pesados"}}
  }, {
    "category": "So",
    "key": "257C",
    "mappings": {"default": {"default": "dibujos de cajas Ligero Izquierda y Pesada Derecha"}}
  }, {
    "category": "So",
    "key": "257D",
    "mappings": {"default": {"default": "dibujos de cajas iluminados y pesados"}}
  }, {
    "category": "So",
    "key": "257E",
    "mappings": {"default": {"default": "dibujos de cajas Pesado Izquierda y Luz Derecha"}}
  }, {
    "category": "So",
    "key": "257F",
    "mappings": {"default": {"default": "dibujos de cajas pesados ​​y ligeros"}}
  }, {
    "key": "2580",
    "mappings": {"default": {"default": "bloque mitad superior"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "2581",
    "mappings": {"default": {"default": "bajar un octavo bloque"}}
  }, {
    "category": "So",
    "key": "2582",
    "mappings": {"default": {"default": "bloque de un cuarto inferior"}}
  }, {
    "category": "So",
    "key": "2583",
    "mappings": {"default": {"default": "bloque de tres octavos más bajo"}}
  }, {
    "key": "2584",
    "mappings": {"default": {"default": "bloque mitad inferior"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "2585",
    "mappings": {"default": {"default": "bloque inferior de cinco octavos"}}
  }, {
    "category": "So",
    "key": "2586",
    "mappings": {"default": {"default": "bloque de tres cuartos más bajo"}}
  }, {
    "category": "So",
    "key": "2587",
    "mappings": {"default": {"default": "lower Seven Eighths Block"}}
  }, {"key": "2588", "mappings": {"default": {"default": "bloque"}}, "category": "So"}, {
    "category": "So",
    "key": "2589",
    "mappings": {"default": {"default": "bloque de siete octavos a la izquierda"}}
  }, {
    "category": "So",
    "key": "258A",
    "mappings": {"default": {"default": "bloque de tres cuartos a la izquierda"}}
  }, {
    "category": "So",
    "key": "258B",
    "mappings": {"default": {"default": "bloque de cinco octavos a la izquierda"}}
  }, {
    "category": "So",
    "key": "258C",
    "mappings": {"default": {"default": "media cuadra izquierda"}}
  }, {
    "category": "So",
    "key": "258D",
    "mappings": {"default": {"default": "bloque de tres octavos a la izquierda"}}
  }, {
    "category": "So",
    "key": "258E",
    "mappings": {"default": {"default": "bloque de un cuarto a la izquierda"}}
  }, {
    "category": "So",
    "key": "258F",
    "mappings": {"default": {"default": "izquierda un octavo bloque"}}
  }, {"category": "So", "key": "2590", "mappings": {"default": {"default": "media cuadra derecha"}}}, {
    "key": "2591",
    "mappings": {"default": {"default": "sombra ligera"}},
    "category": "So"
  }, {"key": "2592", "mappings": {"default": {"default": "sombra media"}}, "category": "So"}, {
    "key": "2593",
    "mappings": {"default": {"default": "sombra intensa"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "2594",
    "mappings": {"default": {"default": "upper One Eighth Block"}}
  }, {
    "category": "So",
    "key": "2595",
    "mappings": {"default": {"default": "a la derecha un octavo bloque"}}
  }, {
    "category": "So",
    "key": "2596",
    "mappings": {"default": {"default": "cuadrante inferior izquierdo"}}
  }, {
    "category": "So",
    "key": "2597",
    "mappings": {"default": {"default": "cuadrante inferior derecho"}}
  }, {
    "category": "So",
    "key": "2598",
    "mappings": {"default": {"default": "cuadrante superior izquierdo"}}
  }, {
    "category": "So",
    "key": "2599",
    "mappings": {"default": {"default": "cuadrante superior izquierdo e inferior izquierdo e inferior derecho"}}
  }, {
    "category": "So",
    "key": "259A",
    "mappings": {"default": {"default": "cuadrante superior izquierdo e inferior derecho"}}
  }, {
    "category": "So",
    "key": "259B",
    "mappings": {"default": {"default": "cuadrante superior izquierdo y superior derecho e inferior izquierdo"}}
  }, {
    "category": "So",
    "key": "259C",
    "mappings": {"default": {"default": "cuadrante superior izquierdo y superior derecho e inferior derecho"}}
  }, {
    "category": "So",
    "key": "259D",
    "mappings": {"default": {"default": "cuadrante superior derecho"}}
  }, {
    "category": "So",
    "key": "259E",
    "mappings": {"default": {"default": "cuadrante superior derecho e inferior izquierdo"}}
  }, {
    "category": "So",
    "key": "259F",
    "mappings": {"default": {"default": "cuadrante superior derecho e inferior izquierdo e inferior derecho"}}
  }, {"key": "25A0", "mappings": {"default": {"default": "cuadrado relleno"}}, "category": "So"}, {
    "key": "25A1",
    "mappings": {"default": {"default": "cuadrado"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "25A2",
    "mappings": {"default": {"default": "cuadrado blanco con esquinas redondeadas"}}
  }, {
    "category": "So",
    "key": "25A3",
    "mappings": {"default": {"default": "cuadrado blanco que contiene cuadrado pequeño negro"}}
  }, {
    "category": "So",
    "key": "25A4",
    "mappings": {"default": {"default": "cuadrado con relleno horizontal"}}
  }, {
    "category": "So",
    "key": "25A5",
    "mappings": {"default": {"default": "cuadrado con relleno vertical"}}
  }, {
    "category": "So",
    "key": "25A6",
    "mappings": {"default": {"default": "cuadrado con relleno de rayado ortogonal"}}
  }, {
    "category": "So",
    "key": "25A7",
    "mappings": {"default": {"default": "cuadrado con la esquina superior izquierda a la derecha inferior"}}
  }, {
    "category": "So",
    "key": "25A8",
    "mappings": {"default": {"default": "cuadrado con la parte superior derecha hacia abajo izquierda Relleno"}}
  }, {
    "category": "So",
    "key": "25A9",
    "mappings": {"default": {"default": "cuadrado con relleno de rayado diagonal"}}
  }, {"key": "25AA", "mappings": {"default": {"default": "cuadrado relleno"}}, "category": "So"}, {
    "key": "25AB",
    "mappings": {"default": {"default": "cuadrado"}},
    "category": "So"
  }, {"category": "So", "key": "25AC", "mappings": {"default": {"default": "rectángulo negro"}}}, {
    "key": "25AD",
    "mappings": {"default": {"default": "rectángulo"}},
    "category": "So"
  }, {
    "key": "25AE",
    "mappings": {"default": {"default": "rectángulo relleno vertical"}},
    "category": "So"
  }, {"key": "25AF", "mappings": {"default": {"default": "rectángulo vertical"}}, "category": "So"}, {
    "category": "So",
    "key": "25B0",
    "mappings": {"default": {"default": "paralelogramo negro"}}
  }, {"key": "25B1", "mappings": {"default": {"default": "paralelogramo"}}, "category": "So"}, {
    "key": "25B2",
    "mappings": {"default": {"default": "triángulo relleno"}},
    "category": "So"
  }, {"key": "25B3", "mappings": {"default": {"default": "triángulo"}}, "category": "So"}, {
    "key": "25B4",
    "mappings": {"default": {"default": "triángulo relleno"}},
    "category": "So"
  }, {"key": "25B5", "mappings": {"default": {"default": "triángulo"}}, "category": "So"}, {
    "key": "25B6",
    "mappings": {"default": {"default": "triángulo relleno señala derecha"}},
    "category": "So"
  }, {
    "key": "25B7",
    "mappings": {"default": {"default": "triángulo señala derecha"}},
    "category": "Sm"
  }, {
    "key": "25B8",
    "mappings": {"default": {"default": "triángulo relleno señala derecha"}},
    "category": "So"
  }, {
    "key": "25B9",
    "mappings": {"default": {"default": "triángulo señala derecha"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "25BA",
    "mappings": {"default": {"default": "indicador de apuntar hacia la derecha negro"}}
  }, {
    "category": "So",
    "key": "25BB",
    "mappings": {"default": {"default": "puntero de apuntar hacia la derecha blanco"}}
  }, {
    "key": "25BC",
    "mappings": {"default": {"default": "triángulo relleno señala abajo"}},
    "category": "So"
  }, {"key": "25BD", "mappings": {"default": {"default": "triángulo señala abajo"}}, "category": "So"}, {
    "key": "25BE",
    "mappings": {"default": {"default": "triángulo relleno señala abajo"}},
    "category": "So"
  }, {"key": "25BF", "mappings": {"default": {"default": "triángulo señala abajo"}}, "category": "So"}, {
    "key": "25C0",
    "mappings": {"default": {"default": "triángulo relleno señala izquierda"}},
    "category": "So"
  }, {
    "key": "25C1",
    "mappings": {"default": {"default": "triángulo señala izquierda"}},
    "category": "Sm"
  }, {
    "key": "25C2",
    "mappings": {"default": {"default": "triángulo relleno señala izquierda"}},
    "category": "So"
  }, {
    "key": "25C3",
    "mappings": {"default": {"default": "triángulo señala izquierda"}},
    "category": "So"
  }, {
    "key": "25C4",
    "mappings": {"default": {"default": "puntero relleno señala izquierda"}},
    "category": "So"
  }, {
    "key": "25C5",
    "mappings": {"default": {"default": "puntero señala izquierda"}},
    "category": "So"
  }, {"key": "25C6", "mappings": {"default": {"default": "diamante relleno"}}, "category": "So"}, {
    "key": "25C7",
    "mappings": {"default": {"default": "diamante"}},
    "category": "So"
  }, {
    "key": "25C8",
    "mappings": {"default": {"default": "diamante vacío contiene diamante relleno"}},
    "category": "So"
  }, {"key": "25C9", "mappings": {"default": {"default": "ojo de pez"}}, "category": "So"}, {
    "key": "25CA",
    "mappings": {"default": {"default": "rombo"}},
    "category": "So"
  }, {"key": "25CB", "mappings": {"default": {"default": "círculo vacío"}}, "category": "So"}, {
    "key": "25CC",
    "mappings": {"default": {"default": "círculo punteado"}},
    "category": "So"
  }, {
    "key": "25CD",
    "mappings": {"default": {"default": "círculo con relleno vertical"}},
    "category": "So"
  }, {"key": "25CE", "mappings": {"default": {"default": "blanco"}}, "category": "So"}, {
    "key": "25CF",
    "mappings": {"default": {"default": "círculo relleno"}},
    "category": "So"
  }, {
    "key": "25D0",
    "mappings": {"default": {"default": "círculo con mitad izquierda rellena"}},
    "category": "So"
  }, {
    "key": "25D1",
    "mappings": {"default": {"default": "círculo con mitad derecha rellena"}},
    "category": "So"
  }, {
    "key": "25D2",
    "mappings": {"default": {"default": "círculo con mitad inferior rellena"}},
    "category": "So"
  }, {
    "key": "25D3",
    "mappings": {"default": {"default": "círculo con mitad superior rellena"}},
    "category": "So"
  }, {
    "key": "25D4",
    "mappings": {"default": {"default": "círculo con cuadrante superior derecho relleno"}},
    "category": "So"
  }, {
    "key": "25D5",
    "mappings": {"default": {"default": "círculo con cuadrante superior izquierdo relleno"}},
    "category": "So"
  }, {
    "key": "25D6",
    "mappings": {"default": {"default": "semicírculo izquierdo relleno"}},
    "category": "So"
  }, {
    "key": "25D7",
    "mappings": {"default": {"default": "semicírculo derecho relleno"}},
    "category": "So"
  }, {"key": "25D8", "mappings": {"default": {"default": "bullet inversa"}}, "category": "So"}, {
    "key": "25D9",
    "mappings": {"default": {"default": "círculo vacío inverso"}},
    "category": "So"
  }, {
    "key": "25DA",
    "mappings": {"default": {"default": "semicírculo superior vacío inverso"}},
    "category": "So"
  }, {
    "key": "25DB",
    "mappings": {"default": {"default": "semicírculo superior vacío inverso"}},
    "category": "So"
  }, {"key": "25DC", "mappings": {"default": {"default": "arco superior izquierda"}}, "category": "So"}, {
    "key": "25DD",
    "mappings": {"default": {"default": "arco superior derecha"}},
    "category": "So"
  }, {"key": "25DE", "mappings": {"default": {"default": "arco inferior derecha"}}, "category": "So"}, {
    "key": "25DF",
    "mappings": {"default": {"default": "arco inferior izquierda"}},
    "category": "So"
  }, {"key": "25E0", "mappings": {"default": {"default": "semicírculo superior"}}, "category": "So"}, {
    "key": "25E1",
    "mappings": {"default": {"default": "semicírculo inferior"}},
    "category": "So"
  }, {
    "key": "25E2",
    "mappings": {"default": {"default": "triángulo inferior derecha relleno"}},
    "category": "So"
  }, {
    "key": "25E3",
    "mappings": {"default": {"default": "triángulo inferior izquierda relleno"}},
    "category": "So"
  }, {
    "key": "25E4",
    "mappings": {"default": {"default": "triángulo superior izquierda relleno"}},
    "category": "So"
  }, {
    "key": "25E5",
    "mappings": {"default": {"default": "triángulo superior derecha relleno"}},
    "category": "So"
  }, {"key": "25E6", "mappings": {"default": {"default": "composición"}}, "category": "So"}, {
    "key": "25E7",
    "mappings": {"default": {"default": "cuadrado con mitad izquierda rellena"}},
    "category": "So"
  }, {
    "key": "25E8",
    "mappings": {"default": {"default": "cuadrado con mitad derecha rellena"}},
    "category": "So"
  }, {
    "key": "25E9",
    "mappings": {"default": {"default": "cuadrado con mitad superior izquierda rellena"}},
    "category": "So"
  }, {
    "key": "25EA",
    "mappings": {"default": {"default": "cuadrado con mitad inferior derecha rellena"}},
    "category": "So"
  }, {"key": "25EB", "mappings": {"default": {"default": "cuadrado con bisectriz"}}, "category": "So"}, {
    "key": "25EC",
    "mappings": {"default": {"default": "triángulo vacío con punto"}},
    "category": "So"
  }, {
    "key": "25ED",
    "mappings": {"default": {"default": "triángulo apuntando arriba con mitad izquierda rellena"}},
    "category": "So"
  }, {
    "key": "25EE",
    "mappings": {"default": {"default": "triángulo apuntando arriba con mitad derecha rellena"}},
    "category": "So"
  }, {"key": "25EF", "mappings": {"default": {"default": "círculo grande"}}, "category": "So"}, {
    "category": "So",
    "key": "25F0",
    "mappings": {"default": {"default": "cuadrado blanco con cuadrante superior izquierdo"}}
  }, {
    "category": "So",
    "key": "25F1",
    "mappings": {"default": {"default": "cuadrado blanco con cuadrante inferior izquierdo"}}
  }, {
    "category": "So",
    "key": "25F2",
    "mappings": {"default": {"default": "cuadrado blanco con cuadrante inferior derecho"}}
  }, {
    "category": "So",
    "key": "25F3",
    "mappings": {"default": {"default": "cuadrado blanco con cuadrante superior derecho"}}
  }, {
    "category": "So",
    "key": "25F4",
    "mappings": {"default": {"default": "círculo blanco con cuadrante superior izquierdo"}}
  }, {
    "category": "So",
    "key": "25F5",
    "mappings": {"default": {"default": "círculo blanco con cuadrante inferior izquierdo"}}
  }, {
    "category": "So",
    "key": "25F6",
    "mappings": {"default": {"default": "círculo blanco con cuadrante inferior derecho"}}
  }, {
    "category": "So",
    "key": "25F7",
    "mappings": {"default": {"default": "círculo blanco con cuadrante superior derecho"}}
  }, {
    "key": "25F8",
    "mappings": {"default": {"default": "triángulo superior izquierda"}},
    "category": "Sm"
  }, {
    "key": "25F9",
    "mappings": {"default": {"default": "triángulo superior derecha"}},
    "category": "Sm"
  }, {
    "key": "25FA",
    "mappings": {"default": {"default": "triángulo inferior izquierda"}},
    "category": "Sm"
  }, {"key": "25FB", "mappings": {"default": {"default": "cuadrado mediano vacío"}}, "category": "Sm"}, {
    "key": "25FC",
    "mappings": {"default": {"default": "cuadrado mediano relleno"}},
    "category": "Sm"
  }, {"key": "25FD", "mappings": {"default": {"default": "cuadrado mediano vacío"}}, "category": "Sm"}, {
    "key": "25FE",
    "mappings": {"default": {"default": "cuadrado mediano relleno"}},
    "category": "Sm"
  }, {
    "key": "25FF",
    "mappings": {"default": {"default": "triángulo inferior derecha"}},
    "category": "Sm"
  }, {
    "category": "So",
    "key": "2B12",
    "mappings": {"default": {"default": "cuadrado con mitad superior negra"}}
  }, {
    "category": "So",
    "key": "2B13",
    "mappings": {"default": {"default": "cuadrado con mitad inferior negra"}}
  }, {
    "category": "So",
    "key": "2B14",
    "mappings": {"default": {"default": "cuadrado con mitad derecha superior diagonal negra"}}
  }, {
    "category": "So",
    "key": "2B15",
    "mappings": {"default": {"default": "cuadrado con mitad diagonal izquierda inferior negra"}}
  }, {
    "category": "So",
    "key": "2B16",
    "mappings": {"default": {"default": "diamante con Mitad Izquierda Negro"}}
  }, {
    "category": "So",
    "key": "2B17",
    "mappings": {"default": {"default": "diamante con mitad derecha negra"}}
  }, {
    "category": "So",
    "key": "2B18",
    "mappings": {"default": {"default": "diamante con mitad superior negra"}}
  }, {
    "category": "So",
    "key": "2B19",
    "mappings": {"default": {"default": "diamante con mitad inferior negra"}}
  }, {"category": "So", "key": "2B1A", "mappings": {"default": {"default": "cuadrado punteado"}}}, {
    "category": "So",
    "key": "2B1B",
    "mappings": {"default": {"default": "cuadrado grande negro"}}
  }, {"category": "So", "key": "2B1C", "mappings": {"default": {"default": "gran Plaza Blanca"}}}, {
    "category": "So",
    "key": "2B1D",
    "mappings": {"default": {"default": "cuadrado muy pequeño negro"}}
  }, {
    "category": "So",
    "key": "2B1E",
    "mappings": {"default": {"default": "cuadrado muy pequeño blanco"}}
  }, {"category": "So", "key": "2B1F", "mappings": {"default": {"default": "pentágono negro"}}}, {
    "category": "So",
    "key": "2B20",
    "mappings": {"default": {"default": "pentágono blanco"}}
  }, {"category": "So", "key": "2B21", "mappings": {"default": {"default": "hexágono blanco"}}}, {
    "category": "So",
    "key": "2B22",
    "mappings": {"default": {"default": "hexágono negro"}}
  }, {
    "category": "So",
    "key": "2B23",
    "mappings": {"default": {"default": "hexagonal negro horizontal"}}
  }, {"category": "So", "key": "2B24", "mappings": {"default": {"default": "circulo grande negro"}}}, {
    "category": "So",
    "key": "2B25",
    "mappings": {"default": {"default": "diamante Mediano Negro"}}
  }, {
    "category": "So",
    "key": "2B26",
    "mappings": {"default": {"default": "diamante blanco medio"}}
  }, {
    "category": "So",
    "key": "2B27",
    "mappings": {"default": {"default": "lozenge Mediano Negro"}}
  }, {
    "category": "So",
    "key": "2B28",
    "mappings": {"default": {"default": "pastilla blanca mediana"}}
  }, {
    "category": "So",
    "key": "2B29",
    "mappings": {"default": {"default": "pequeño diamante negro"}}
  }, {
    "category": "So",
    "key": "2B2A",
    "mappings": {"default": {"default": "lozenge pequeño negro"}}
  }, {
    "category": "So",
    "key": "2B2B",
    "mappings": {"default": {"default": "lozenge pequeño blanco"}}
  }, {
    "category": "So",
    "key": "2B2C",
    "mappings": {"default": {"default": "elipse horizontal negro"}}
  }, {
    "category": "So",
    "key": "2B2D",
    "mappings": {"default": {"default": "elipse horizontal blanco"}}
  }, {
    "category": "So",
    "key": "2B2E",
    "mappings": {"default": {"default": "elipse Vertical Negro"}}
  }, {"category": "So", "key": "2B2F", "mappings": {"default": {"default": "elipse Vertical Blanco"}}}, {
    "key": "2B50",
    "mappings": {"default": {"default": "estrella media vacía"}},
    "category": "So"
  }, {
    "key": "2B51",
    "mappings": {"default": {"default": "estrella pequeña rellena"}},
    "category": "So"
  }, {
    "key": "2B52",
    "mappings": {"default": {"default": "estrella pequeña vacía"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "2B53",
    "mappings": {"default": {"default": "pentágono negro que señala hacia la derecha"}}
  }, {
    "category": "So",
    "key": "2B54",
    "mappings": {"default": {"default": "pentágono blanco que señala hacia la derecha"}}
  }, {
    "category": "So",
    "key": "2B55",
    "mappings": {"default": {"default": "círculo grande pesado"}}
  }, {
    "category": "So",
    "key": "2B56",
    "mappings": {"default": {"default": "óvalo pesado con interior oval"}}
  }, {
    "category": "So",
    "key": "2B57",
    "mappings": {"default": {"default": "círculo pesado con círculo dentro"}}
  }, {"category": "So", "key": "2B58", "mappings": {"default": {"default": "círculo pesado"}}}, {
    "category": "So",
    "key": "2B59",
    "mappings": {"default": {"default": "saltire en círculo pesado"}}
  }]
    ,
    "es/symbols/math_harpoons.js"
:
  [{"locale": "es"}, {
    "key": "21BC",
    "mappings": {"default": {"default": "flecha izquierda con arpón arriba"}},
    "category": "So"
  }, {
    "key": "21BD",
    "mappings": {"default": {"default": "flecha izquierda con arpón debajo"}},
    "category": "So"
  }, {
    "key": "21BE",
    "mappings": {"default": {"default": "flecha arriba con arpón derecho"}},
    "category": "So"
  }, {
    "key": "21BF",
    "mappings": {"default": {"default": "flecha arriba con arpón izquierdo"}},
    "category": "So"
  }, {
    "key": "21C0",
    "mappings": {"default": {"default": "flecha derecha con arpón arriba"}},
    "category": "So"
  }, {
    "key": "21C1",
    "mappings": {"default": {"default": "flecha derecha con arpón debajo"}},
    "category": "So"
  }, {
    "key": "21C2",
    "mappings": {"default": {"default": "flecha abajo con arpón derecho"}},
    "category": "So"
  }, {
    "key": "21C3",
    "mappings": {"default": {"default": "flecha abajo con arpón izquierdo"}},
    "category": "So"
  }, {
    "key": "21CB",
    "mappings": {"default": {"default": "arpón izquierdo arriba de arpón derecho"}},
    "category": "So"
  }, {
    "key": "21CC",
    "mappings": {"default": {"default": "arpón derecho arriba de arpón izquierdo"}},
    "category": "So"
  }, {
    "category": "Sm",
    "key": "294A",
    "mappings": {"default": {"default": "barra izquierda arriba barra derecha abajo arpón"}}
  }, {
    "category": "Sm",
    "key": "294B",
    "mappings": {"default": {"default": "barra izquierda abajo barra derecha arriba arpón"}}
  }, {
    "category": "Sm",
    "key": "294C",
    "mappings": {"default": {"default": "up barb right down barb left harpoon"}}
  }, {
    "category": "Sm",
    "key": "294D",
    "mappings": {"default": {"default": "arriba barb izquierda abajo barb derecha arpón"}}
  }, {
    "category": "Sm",
    "key": "294E",
    "mappings": {"default": {"default": "barra izquierda arriba barra derecha arriba arpón"}}
  }, {
    "key": "294F",
    "mappings": {"default": {"default": "arpón a la derecha hacia arriba y abajo"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2950",
    "mappings": {"default": {"default": "barra izquierda abajo barra derecha abajo arpón"}}
  }, {
    "key": "2951",
    "mappings": {"default": {"default": "arpón a la izquierda hacia arriba y abajo"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2952",
    "mappings": {"default": {"default": "arpón a la izquierda con púa hasta la barra"}}
  }, {
    "category": "Sm",
    "key": "2953",
    "mappings": {"default": {"default": "arpón a la derecha con púa hasta la barra"}}
  }, {
    "category": "Sm",
    "key": "2954",
    "mappings": {"default": {"default": "arpón ascendente con púa derecha a barra"}}
  }, {
    "category": "Sm",
    "key": "2955",
    "mappings": {"default": {"default": "arpón hacia abajo con púa derecha a barra"}}
  }, {
    "category": "Sm",
    "key": "2956",
    "mappings": {"default": {"default": "arpón a la izquierda con púas abajo a la barra"}}
  }, {
    "category": "Sm",
    "key": "2957",
    "mappings": {"default": {"default": "arpón a la derecha con púa abajo a la barra"}}
  }, {
    "category": "Sm",
    "key": "2958",
    "mappings": {"default": {"default": "arpón ascendente con púa izquierda a barra"}}
  }, {
    "category": "Sm",
    "key": "2959",
    "mappings": {"default": {"default": "arpón hacia abajo con la barra izquierda a la barra"}}
  }, {
    "category": "Sm",
    "key": "295A",
    "mappings": {"default": {"default": "arpón a la izquierda con púa arriba de la barra"}}
  }, {
    "category": "Sm",
    "key": "295B",
    "mappings": {"default": {"default": "arpón a la derecha con púa arriba del bar"}}
  }, {
    "category": "Sm",
    "key": "295C",
    "mappings": {"default": {"default": "arpón ascendente con púa derecha de barra"}}
  }, {
    "category": "Sm",
    "key": "295D",
    "mappings": {"default": {"default": "arpón hacia abajo con púa derecha de barra"}}
  }, {
    "category": "Sm",
    "key": "295E",
    "mappings": {"default": {"default": "arpón a la izquierda con púa abajo de barra"}}
  }, {
    "category": "Sm",
    "key": "295F",
    "mappings": {"default": {"default": "arpón a la derecha con púa abajo de barra"}}
  }, {
    "category": "Sm",
    "key": "2960",
    "mappings": {"default": {"default": "arpón ascendente con púa izquierda de barra"}}
  }, {
    "category": "Sm",
    "key": "2961",
    "mappings": {"default": {"default": "arpón hacia abajo con barra izquierda de barra"}}
  }, {
    "category": "Sm",
    "key": "2962",
    "mappings": {"default": {"default": "arpón a la izquierda con púas arriba arriba arpón a la izquierda con púas abajo"}}
  }, {
    "category": "Sm",
    "key": "2963",
    "mappings": {"default": {"default": "arpón ascendente con púa izquierda al lado arpón ascendente con púa derecha"}}
  }, {
    "category": "Sm",
    "key": "2964",
    "mappings": {"default": {"default": "arpón a la derecha con púas arriba arriba arpón a la derecha con púas abajo"}}
  }, {
    "category": "Sm",
    "key": "2965",
    "mappings": {"default": {"default": "arpón hacia abajo con lengüeta a la izquierda al lado arpón hacia abajo con lengüeta a la derecha"}}
  }, {
    "category": "Sm",
    "key": "2966",
    "mappings": {"default": {"default": "arpón a la izquierda con púa arriba arriba arpón a la derecha con púa arriba"}}
  }, {
    "category": "Sm",
    "key": "2967",
    "mappings": {"default": {"default": "arpón a la izquierda con púa abajo arriba arpón a la derecha con púa abajo"}}
  }, {
    "category": "Sm",
    "key": "2968",
    "mappings": {"default": {"default": "arpón a la derecha con púas arriba arriba arpón a la izquierda con púas arriba"}}
  }, {
    "category": "Sm",
    "key": "2969",
    "mappings": {"default": {"default": "arpón a la derecha con púa abajo arriba arpón a la izquierda con púa abajo"}}
  }, {
    "key": "296A",
    "mappings": {"default": {"default": "barra con arpón hacia la izquierda"}},
    "category": "Sm"
  }, {
    "key": "296B",
    "mappings": {"default": {"default": "arpón hacia la izquierda con barra"}},
    "category": "Sm"
  }, {
    "key": "296C",
    "mappings": {"default": {"default": "barra con arpón hacia la derecha"}},
    "category": "Sm"
  }, {
    "key": "296D",
    "mappings": {"default": {"default": "arpón hacia la derecha con barra"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "296E",
    "mappings": {"default": {"default": "arpón ascendente con púa izquierda al lado arpón hacia abajo con púa derecha"}}
  }, {
    "category": "Sm",
    "key": "296F",
    "mappings": {"default": {"default": "arpón hacia abajo con lengüeta a la izquierda al lado arpón hacia arriba con pica a la derecha"}}
  }, {
    "category": "Sm",
    "key": "297C",
    "mappings": {"default": {"default": "cola de pescado izquierda"}}
  }, {"category": "Sm", "key": "297D", "mappings": {"default": {"default": "cola de pescado derecha"}}}, {
    "key": "297E",
    "mappings": {"default": {"default": "barra con doble gancho"}},
    "category": "Sm"
  }, {"category": "Sm", "key": "297F", "mappings": {"default": {"default": "cola de pez abajo"}}}]
    ,
    "es/symbols/math_non_characters.js"
:
  [{"locale": "es"}, {
    "key": "210F",
    "mappings": {"default": {"default": "h barra"}},
    "category": "Ll"
  }, {"category": "So", "key": "2114", "mappings": {"default": {"default": "L B símbolo de barra"}}}, {
    "key": "2116",
    "mappings": {"default": {"default": "signo de número"}},
    "category": "So"
  }, {
    "key": "2117",
    "mappings": {"default": {"default": "derechos de autor de sonido"}},
    "category": "So"
  }, {"key": "211E", "mappings": {"default": {"default": "receta"}}, "category": "So"}, {
    "category": "So",
    "key": "211F",
    "mappings": {"default": {"default": "respuesta"}}
  }, {"category": "So", "key": "2120", "mappings": {"default": {"default": "marca de servicio"}}}, {
    "category": "So",
    "key": "2121",
    "mappings": {"default": {"default": "señal de telefono"}}
  }, {"key": "2122", "mappings": {"default": {"default": "registrado"}}, "category": "So"}, {
    "category": "So",
    "key": "2123",
    "mappings": {"default": {"default": "versículo"}}
  }, {"key": "2125", "mappings": {"default": {"default": "onza"}}, "category": "So"}, {
    "key": "2126",
    "mappings": {"default": {"default": "ohmio"}},
    "category": "Lu"
  }, {"key": "2127", "mappings": {"default": {"default": "ohmio invertido"}}, "category": "So"}, {
    "key": "212A",
    "mappings": {"default": {"default": "grados kelvin"}},
    "category": "Lu"
  }, {"key": "212B", "mappings": {"default": {"default": "angstrom"}}, "category": "Lu"}, {
    "category": "So",
    "key": "212E",
    "mappings": {"default": {"default": "Símbolo estimado"}}
  }, {
    "key": "2132",
    "mappings": {"default": {"default": "mayúscula F invertida"}},
    "category": "Lu"
  }, {
    "category": "Ll",
    "key": "2139",
    "mappings": {"default": {"default": "fuente de información"}}
  }, {"category": "So", "key": "213A", "mappings": {"default": {"default": "capital rotado Q"}}}, {
    "category": "So",
    "key": "213B",
    "mappings": {"default": {"default": "signo de fax"}}
  }, {
    "category": "Sm",
    "key": "2141",
    "mappings": {"default": {"default": "convertido sans serif mayúscula G"}}
  }, {
    "category": "Sm",
    "key": "2142",
    "mappings": {"default": {"default": "convertido sans serif mayúscula L"}}
  }, {
    "category": "Sm",
    "key": "2143",
    "mappings": {"default": {"default": "invertido sans serif mayúscula L"}}
  }, {"category": "Sm", "key": "2144", "mappings": {"default": {"default": "convertido en sans serif mayúscula Y"}}}]
    ,
    "es/symbols/math_symbols.js"
:
  [{"locale": "es"}, {
    "key": "0021",
    "mappings": {"default": {"default": "factorial"}},
    "category": "Po"
  }, {"key": "0022", "mappings": {"default": {"default": "comillas"}}, "category": "Po"}, {
    "key": "0023",
    "mappings": {"default": {"default": "almuhadilla", "defaultMP": "signo de número"}},
    "category": "Po"
  }, {"key": "0024", "mappings": {"default": {"default": "dólar"}}, "category": "Sc"}, {
    "key": "0025",
    "mappings": {"default": {"default": "porciento"}},
    "category": "Po"
  }, {"key": "0026", "mappings": {"default": {"default": "ampersand"}}, "category": "Po"}, {
    "key": "0027",
    "mappings": {"default": {"default": "prima", "defaultMP": "apóstrofo"}},
    "category": "Po"
  }, {"key": "002A", "mappings": {"default": {"default": "por"}}, "category": "Po"}, {
    "key": "002B",
    "mappings": {"default": {"default": "más"}},
    "category": "Sm"
  }, {"key": "002C", "mappings": {"default": {"default": "coma"}}, "category": "Po"}, {
    "key": "002D",
    "mappings": {"default": {"default": "menos"}},
    "category": "Pd"
  }, {"key": "002E", "mappings": {"default": {"default": "punto"}}, "category": "Po"}, {
    "key": "002F",
    "mappings": {"default": {"default": "barra oblicua"}},
    "category": "Po"
  }, {"key": "003A", "mappings": {"default": {"default": "dos puntos"}}, "category": "Po"}, {
    "key": "003B",
    "mappings": {"default": {"default": "punto y coma"}},
    "category": "Po"
  }, {"key": "003C", "mappings": {"default": {"default": "menor que"}}, "category": "Sm"}, {
    "key": "003D",
    "mappings": {"default": {"default": "igual"}},
    "category": "Sm"
  }, {"key": "003E", "mappings": {"default": {"default": "mayor que"}}, "category": "Sm"}, {
    "key": "003F",
    "mappings": {"default": {"default": "interrogación derecho", "defaultMP": "cierra interrogación"}},
    "category": "Po"
  }, {"key": "0040", "mappings": {"default": {"default": "arroba"}}, "category": "Po"}, {
    "key": "005C",
    "mappings": {"default": {"default": "barra inversa"}},
    "category": "Po"
  }, {"key": "005E", "mappings": {"default": {"default": "circunflejo"}}, "category": "Sk"}, {
    "key": "005F",
    "mappings": {"default": {"default": "subrayado"}},
    "category": "Pc"
  }, {"key": "0060", "mappings": {"default": {"default": "acento grave"}}, "category": "Sk"}, {
    "key": "007C",
    "mappings": {"default": {"default": "barra vertical"}},
    "category": "Sm"
  }, {"key": "007E", "mappings": {"default": {"default": "tilde"}}, "category": "Sm"}, {
    "key": "00A1",
    "mappings": {"default": {"default": "admiración izquierdo", "defaultMP": "abre admiración"}},
    "category": "Po"
  }, {"key": "00A2", "mappings": {"default": {"default": "centavo"}}, "category": "Sc"}, {
    "key": "00A3",
    "mappings": {"default": {"default": "libra esterlina"}},
    "category": "Sc"
  }, {"key": "00A4", "mappings": {"default": {"default": "moneda"}}, "category": "Sc"}, {
    "key": "00A5",
    "mappings": {"default": {"default": "yen"}},
    "category": "Sc"
  }, {"key": "00A6", "mappings": {"default": {"default": "barra vertical partida"}}, "category": "So"}, {
    "key": "00A7",
    "mappings": {"default": {"default": "sección"}},
    "category": "Po"
  }, {"key": "00A8", "mappings": {"default": {"default": "diéresis"}}, "category": "Sk"}, {
    "key": "00A9",
    "mappings": {"default": {"default": "copyright"}},
    "category": "So"
  }, {"key": "00AA", "mappings": {"default": {"default": "ordinal femenino"}}, "category": "Lo"}, {
    "key": "00AB",
    "mappings": {"default": {"default": "comillas angulares izquierda", "defaultMP": "abre comillas angulares"}},
    "category": "Pi"
  }, {"key": "00AC", "mappings": {"default": {"default": "negación"}}, "category": "Sm"}, {
    "key": "00AE",
    "mappings": {"default": {"default": "registrado"}},
    "category": "So"
  }, {"key": "00AF", "mappings": {"default": {"default": "barra"}}, "category": "Sk"}, {
    "key": "00B0",
    "mappings": {"default": {"default": "grado"}},
    "category": "So"
  }, {"key": "00B1", "mappings": {"default": {"default": "más menos"}}, "category": "Sm"}, {
    "key": "00B4",
    "mappings": {"default": {"default": "acento agudo"}},
    "category": "Sk"
  }, {"key": "00B5", "mappings": {"default": {"default": "micro"}}, "category": "Ll"}, {
    "key": "00B6",
    "mappings": {"default": {"default": "párrafo"}},
    "category": "Po"
  }, {"key": "00B7", "mappings": {"default": {"default": "punto medio"}}, "category": "Po"}, {
    "key": "00B8",
    "mappings": {"default": {"default": "cedilla"}},
    "category": "Sk"
  }, {"key": "00BA", "mappings": {"default": {"default": "ordinal masculino"}}, "category": "Lo"}, {
    "key": "00BB",
    "mappings": {"default": {"default": "comillas angulares derecha", "defaultMP": "cierra comillas angulares"}},
    "category": "Pf"
  }, {
    "key": "00BF",
    "mappings": {"default": {"default": "interrogación izquierdo", "defaultMP": "abre interrogación"}},
    "category": "Po"
  }, {"key": "00D7", "mappings": {"default": {"default": "por"}}, "category": "Sm"}, {
    "key": "00F7",
    "mappings": {"default": {"default": "dividido"}},
    "category": "Sm"
  }, {"key": "02D8", "mappings": {"default": {"default": "breve"}}, "category": "Sk"}, {
    "key": "02D9",
    "mappings": {"default": {"default": "punto en superescrito"}},
    "category": "Sk"
  }, {"key": "02DA", "mappings": {"default": {"default": "anillo en superescrito"}}, "category": "Sk"}, {
    "key": "02DB",
    "mappings": {"default": {"default": "ogonek"}},
    "category": "Sk"
  }, {"key": "02DC", "mappings": {"default": {"default": "tilde"}}, "category": "Sk"}, {
    "key": "02DD",
    "mappings": {"default": {"default": "doble acento agudo"}},
    "category": "Sk"
  }, {"key": "2010", "mappings": {"default": {"default": "menos"}}, "category": "Pd"}, {
    "category": "Pd",
    "key": "2011",
    "mappings": {"default": {"default": "guión no rompible"}}
  }, {"category": "Pd", "key": "2012", "mappings": {"default": {"default": "figura Dash"}}}, {
    "key": "2013",
    "mappings": {"default": {"default": "en raya"}},
    "category": "Pd"
  }, {"key": "2014", "mappings": {"default": {"default": "em raya"}}, "category": "Pd"}, {
    "key": "2015",
    "mappings": {"default": {"default": "barra horizontal"}},
    "category": "Pd"
  }, {"key": "2016", "mappings": {"default": {"default": "doble barra vertical"}}, "category": "Po"}, {
    "category": "Po",
    "key": "2017",
    "mappings": {"default": {"default": "línea baja doble"}}
  }, {"key": "2018", "mappings": {"default": {"default": "comilla izquierda"}}, "category": "Pi"}, {
    "key": "2019",
    "mappings": {"default": {"default": "comilla derecha"}},
    "category": "Pf"
  }, {"key": "201A", "mappings": {"default": {"default": "comilla bajo 9"}}, "category": "Ps"}, {
    "category": "Pi",
    "key": "201B",
    "mappings": {"default": {"default": "marca de comillas simple con inversión alta-9"}}
  }, {"key": "201C", "mappings": {"default": {"default": "doble comilla"}}, "category": "Pi"}, {
    "key": "201D",
    "mappings": {"default": {"default": "doble comilla derecha"}},
    "category": "Pf"
  }, {"key": "201E", "mappings": {"default": {"default": "doble comilla bajo 9"}}, "category": "Ps"}, {
    "category": "Pi",
    "key": "201F",
    "mappings": {"default": {"default": "doble alto-invertido-9 comillas"}}
  }, {"key": "2020", "mappings": {"default": {"default": "daga"}}, "category": "Po"}, {
    "key": "2021",
    "mappings": {"default": {"default": "doble daga"}},
    "category": "Po"
  }, {"key": "2022", "mappings": {"default": {"default": "bala"}}, "category": "Po"}, {
    "category": "Po",
    "key": "2023",
    "mappings": {"default": {"default": "bala triangular"}}
  }, {"category": "Po", "key": "2024", "mappings": {"default": {"default": "líder de un punto"}}}, {
    "key": "2025",
    "mappings": {"default": {"default": "dos puntos líder"}},
    "category": "Po"
  }, {"key": "2026", "mappings": {"default": {"default": "puntos suspensivos"}}, "category": "Po"}, {
    "category": "Po",
    "key": "2027",
    "mappings": {"default": {"default": "punto de separación"}}
  }, {"key": "2030", "mappings": {"default": {"default": "tanto por millar"}}, "category": "Po"}, {
    "key": "2031",
    "mappings": {"default": {"default": "tanto por diez mil"}},
    "category": "Po"
  }, {"key": "2032", "mappings": {"default": {"default": "prima"}}, "category": "Po"}, {
    "key": "2033",
    "mappings": {"default": {"default": "segunda"}},
    "category": "Po"
  }, {"key": "2034", "mappings": {"default": {"default": "tercera"}}, "category": "Po"}, {
    "key": "2035",
    "mappings": {"default": {"default": "acento grave"}},
    "category": "Po"
  }, {"key": "2036", "mappings": {"default": {"default": "doble acento grave"}}, "category": "Po"}, {
    "category": "Po",
    "key": "2037",
    "mappings": {"default": {"default": "prime Prime invertido"}}
  }, {"category": "Po", "key": "2038", "mappings": {"default": {"default": "signo de intercalación"}}}, {
    "key": "2039",
    "mappings": {"default": {"default": "comillas izquierda", "defaultMP": "abre comillas"}},
    "category": "Pi"
  }, {
    "key": "203A",
    "mappings": {"default": {"default": "comillas derecha", "defaultMP": "cierra comillas"}},
    "category": "Pf"
  }, {"category": "Po", "key": "203B", "mappings": {"default": {"default": "marca de referencia"}}}, {
    "category": "Po",
    "key": "203C",
    "mappings": {"default": {"default": "doble signo de exclamación"}}
  }, {"category": "Po", "key": "203D", "mappings": {"default": {"default": "interrobang"}}}, {
    "key": "203E",
    "mappings": {"default": {"default": "barra superior"}},
    "category": "Po"
  }, {"category": "Pc", "key": "203F", "mappings": {"default": {"default": "undertie"}}}, {
    "key": "2040",
    "mappings": {"default": {"default": "ligadura"}},
    "category": "Pc"
  }, {"key": "2041", "mappings": {"default": {"default": "punto de inserción"}}, "category": "Po"}, {
    "category": "Po",
    "key": "2042",
    "mappings": {"default": {"default": "asterismo"}}
  }, {"key": "2043", "mappings": {"default": {"default": "guión bala"}}, "category": "Po"}, {
    "key": "2044",
    "mappings": {"default": {"default": "barra de fracción"}},
    "category": "Sm"
  }, {
    "category": "Po",
    "key": "2047",
    "mappings": {"default": {"default": "doble signo de interrogación"}}
  }, {
    "category": "Po",
    "key": "2048",
    "mappings": {"default": {"default": "pregunta Signo de exclamación"}}
  }, {
    "category": "Po",
    "key": "2049",
    "mappings": {"default": {"default": "signo de interrogación de exclamación"}}
  }, {
    "category": "Po",
    "key": "204B",
    "mappings": {"default": {"default": "signo de Pilcrow invertido"}}
  }, {
    "category": "Po",
    "key": "204C",
    "mappings": {"default": {"default": "bala negra hacia la izquierda"}}
  }, {
    "category": "Po",
    "key": "204D",
    "mappings": {"default": {"default": "bala negra hacia la derecha"}}
  }, {"category": "Po", "key": "204E", "mappings": {"default": {"default": "bajo asterisco"}}}, {
    "key": "204F",
    "mappings": {"default": {"default": "punto y coma invertido"}},
    "category": "Po"
  }, {"key": "2050", "mappings": {"default": {"default": "cierre superior"}}, "category": "Po"}, {
    "category": "Po",
    "key": "2051",
    "mappings": {"default": {"default": "dos asteriscos alineados verticalmente"}}
  }, {
    "category": "Sm",
    "key": "2052",
    "mappings": {"default": {"default": "signo de menos comercial"}}
  }, {"category": "Po", "key": "2053", "mappings": {"default": {"default": "swung Dash"}}}, {
    "category": "Pc",
    "key": "2054",
    "mappings": {"default": {"default": "ropa interior invertida"}}
  }, {
    "category": "Po",
    "key": "2055",
    "mappings": {"default": {"default": "signo de puntuación de la flor"}}
  }, {
    "category": "Po",
    "key": "2056",
    "mappings": {"default": {"default": "puntuación de tres puntos"}}
  }, {"key": "2057", "mappings": {"default": {"default": "cuarta"}}, "category": "Po"}, {
    "category": "Po",
    "key": "2058",
    "mappings": {"default": {"default": "puntuación de cuatro puntos"}}
  }, {
    "category": "Po",
    "key": "2059",
    "mappings": {"default": {"default": "puntuación de cinco puntos"}}
  }, {
    "category": "Po",
    "key": "205A",
    "mappings": {"default": {"default": "puntuación de dos puntos"}}
  }, {
    "category": "Po",
    "key": "205B",
    "mappings": {"default": {"default": "marca de cuatro puntos"}}
  }, {"category": "Po", "key": "205C", "mappings": {"default": {"default": "cruz punteada"}}}, {
    "category": "Po",
    "key": "205D",
    "mappings": {"default": {"default": "tricolon"}}
  }, {
    "category": "Po",
    "key": "205E",
    "mappings": {"default": {"default": "cuatro puntos verticales"}}
  }, {
    "category": "Sm",
    "key": "207A",
    "mappings": {"default": {"default": "superscript Plus Sign"}}
  }, {"category": "Sm", "key": "207B", "mappings": {"default": {"default": "superíndice menos"}}}, {
    "category": "Sm",
    "key": "207C",
    "mappings": {"default": {"default": "superscript Equals Sign"}}
  }, {
    "category": "Ps",
    "key": "207D",
    "mappings": {"default": {"default": "paréntesis izquierdo superíndice"}}
  }, {
    "category": "Pe",
    "key": "207E",
    "mappings": {"default": {"default": "paréntesis derecho superíndice"}}
  }, {
    "category": "Sm",
    "key": "208A",
    "mappings": {"default": {"default": "signo de subíndice más"}}
  }, {"category": "Sm", "key": "208B", "mappings": {"default": {"default": "subíndice menos"}}}, {
    "category": "Sm",
    "key": "208C",
    "mappings": {"default": {"default": "signo de subíndice es igual"}}
  }, {
    "category": "Ps",
    "key": "208D",
    "mappings": {"default": {"default": "subíndice paréntesis izquierdo"}}
  }, {
    "category": "Pe",
    "key": "208E",
    "mappings": {"default": {"default": "subíndice de paréntesis derecho"}}
  }, {"category": "So", "key": "214A", "mappings": {"default": {"default": "línea de propiedad"}}}, {
    "category": "Sm",
    "key": "214B",
    "mappings": {"default": {"default": "ampersand convertido"}}
  }, {"category": "So", "key": "214C", "mappings": {"default": {"default": "por signo"}}}, {
    "category": "So",
    "key": "214D",
    "mappings": {"default": {"default": "aktieselskab"}}
  }, {"category": "Ll", "key": "214E", "mappings": {"default": {"default": "torneado pequeño f"}}}, {
    "key": "2200",
    "mappings": {"default": {"default": "para todo"}},
    "category": "Sm"
  }, {"key": "2201", "mappings": {"default": {"default": "complementario"}}, "category": "Sm"}, {
    "key": "2203",
    "mappings": {"default": {"default": "existe"}},
    "category": "Sm"
  }, {"key": "2204", "mappings": {"default": {"default": "no existe"}}, "category": "Sm"}, {
    "key": "2205",
    "mappings": {"default": {"default": "conjunto vacío"}},
    "category": "Sm"
  }, {"key": "2206", "mappings": {"default": {"default": "incremento"}}, "category": "Sm"}, {
    "key": "2208",
    "mappings": {"default": {"default": "perteneciente a"}},
    "category": "Sm"
  }, {"key": "2209", "mappings": {"default": {"default": "no perteneciente a"}}, "category": "Sm"}, {
    "key": "220A",
    "mappings": {"default": {"default": "perteneciente a"}},
    "category": "Sm"
  }, {"key": "220B", "mappings": {"default": {"default": "comprende a"}}, "category": "Sm"}, {
    "key": "220C",
    "mappings": {"default": {"default": "no comprende a"}},
    "category": "Sm"
  }, {"key": "220D", "mappings": {"default": {"default": "comprende a"}}, "category": "Sm"}, {
    "key": "220E",
    "mappings": {"default": {"default": "fin de demostración"}},
    "category": "Sm"
  }, {"key": "220F", "mappings": {"default": {"default": "producto"}}, "category": "Sm"}, {
    "key": "2210",
    "mappings": {"default": {"default": "coproducto"}},
    "category": "Sm"
  }, {"key": "2211", "mappings": {"default": {"default": "sumatorio"}}, "category": "Sm"}, {
    "key": "2212",
    "mappings": {"default": {"default": "menos"}},
    "category": "Sm"
  }, {"key": "2213", "mappings": {"default": {"default": "menos más"}}, "category": "Sm"}, {
    "key": "2214",
    "mappings": {"default": {"default": "punto más"}},
    "category": "Sm"
  }, {"key": "2215", "mappings": {"default": {"default": "barra de división"}}, "category": "Sm"}, {
    "key": "2216",
    "mappings": {"default": {"default": "menos"}},
    "category": "Sm"
  }, {"key": "2217", "mappings": {"default": {"default": "asterisco"}}, "category": "Sm"}, {
    "key": "2218",
    "mappings": {"default": {"default": "composición"}},
    "category": "Sm"
  }, {"key": "2219", "mappings": {"default": {"default": "bala"}}, "category": "Sm"}, {
    "key": "221A",
    "mappings": {"default": {"default": "raíz"}},
    "category": "Sm"
  }, {"key": "221B", "mappings": {"default": {"default": "raíz cúbica"}}, "category": "Sm"}, {
    "key": "221C",
    "mappings": {"default": {"default": "raíz cuarta"}},
    "category": "Sm"
  }, {"key": "221D", "mappings": {"default": {"default": "proporcional a"}}, "category": "Sm"}, {
    "key": "221E",
    "mappings": {"default": {"default": "infinito"}},
    "category": "Sm"
  }, {"key": "221F", "mappings": {"default": {"default": "ángulo recto"}}, "category": "Sm"}, {
    "key": "2220",
    "mappings": {"default": {"default": "ángulo"}},
    "category": "Sm"
  }, {"key": "2221", "mappings": {"default": {"default": "ángulo medido"}}, "category": "Sm"}, {
    "key": "2222",
    "mappings": {"default": {"default": "ángulo esférico"}},
    "category": "Sm"
  }, {"key": "2223", "mappings": {"default": {"default": "divide a"}}, "category": "Sm"}, {
    "key": "2224",
    "mappings": {"default": {"default": "no divide a"}},
    "category": "Sm"
  }, {"key": "2225", "mappings": {"default": {"default": "paralela a"}}, "category": "Sm"}, {
    "key": "2226",
    "mappings": {"default": {"default": "no paralela a"}},
    "category": "Sm"
  }, {"key": "2227", "mappings": {"default": {"default": "y"}}, "category": "Sm"}, {
    "key": "2228",
    "mappings": {"default": {"default": "o"}},
    "category": "Sm"
  }, {"key": "2229", "mappings": {"default": {"default": "intersección"}}, "category": "Sm"}, {
    "key": "222A",
    "mappings": {"default": {"default": "unión"}},
    "category": "Sm"
  }, {"key": "222B", "mappings": {"default": {"default": "integral"}}, "category": "Sm"}, {
    "key": "222C",
    "mappings": {"default": {"default": "integral doble"}},
    "category": "Sm"
  }, {"key": "222D", "mappings": {"default": {"default": "integral triple"}}, "category": "Sm"}, {
    "key": "222E",
    "mappings": {"default": {"default": "integral de contorno"}},
    "category": "Sm"
  }, {"key": "222F", "mappings": {"default": {"default": "integral de superficie"}}, "category": "Sm"}, {
    "key": "2230",
    "mappings": {"default": {"default": "integral de volumen"}},
    "category": "Sm"
  }, {
    "key": "2231",
    "mappings": {"default": {"default": "integral de contorno en sentido horario"}},
    "category": "Sm"
  }, {
    "key": "2232",
    "mappings": {"default": {"default": "integral de contorno en sentido horario"}},
    "category": "Sm"
  }, {
    "key": "2233",
    "mappings": {"default": {"default": "integral de contorno en sentido antihorario"}},
    "category": "Sm"
  }, {"key": "2234", "mappings": {"default": {"default": "por lo tanto"}}, "category": "Sm"}, {
    "key": "2235",
    "mappings": {"default": {"default": "porque"}},
    "category": "Sm"
  }, {"key": "2236", "mappings": {"default": {"default": "razón"}}, "category": "Sm"}, {
    "key": "2237",
    "mappings": {"default": {"default": "proporción"}},
    "category": "Sm"
  }, {"key": "2238", "mappings": {"default": {"default": "punto menos"}}, "category": "Sm"}, {
    "key": "2239",
    "mappings": {"default": {"default": "exceso"}},
    "category": "Sm"
  }, {"key": "223A", "mappings": {"default": {"default": "proporción geométrica"}}, "category": "Sm"}, {
    "key": "223B",
    "mappings": {"default": {"default": "homotecia"}},
    "category": "Sm"
  }, {"key": "223C", "mappings": {"default": {"default": "tilde"}}, "category": "Sm"}, {
    "key": "223D",
    "mappings": {"default": {"default": "tilde invertido"}},
    "category": "Sm"
  }, {"key": "223E", "mappings": {"default": {"default": "s horizontal invertida"}}, "category": "Sm"}, {
    "key": "223F",
    "mappings": {"default": {"default": "sinusoide"}},
    "category": "Sm"
  }, {"key": "2240", "mappings": {"default": {"default": "producto de guirnalda"}}, "category": "Sm"}, {
    "key": "2241",
    "mappings": {"default": {"default": "no tilde"}},
    "category": "Sm"
  }, {"key": "2242", "mappings": {"default": {"default": "menos tilde"}}, "category": "Sm"}, {
    "key": "2243",
    "mappings": {"default": {"default": "asintóticamente igual a"}},
    "category": "Sm"
  }, {
    "key": "2244",
    "mappings": {"default": {"default": "no asintóticamente igual a"}},
    "category": "Sm"
  }, {"key": "2245", "mappings": {"default": {"default": "aproximadamente igual a"}}, "category": "Sm"}, {
    "key": "2246",
    "mappings": {"default": {"default": "aproximado pero no igual a"}},
    "category": "Sm"
  }, {"key": "2247", "mappings": {"default": {"default": "no aproximado a"}}, "category": "Sm"}, {
    "key": "2248",
    "mappings": {"default": {"default": "aproximado", "defaultMP": "casi igual a"}},
    "category": "Sm"
  }, {"key": "2249", "mappings": {"default": {"default": "no es casi igual a"}}, "category": "Sm"}, {
    "key": "224A",
    "mappings": {"default": {"default": "igual o casi igual a"}},
    "category": "Sm"
  }, {"key": "224B", "mappings": {"default": {"default": "triple tilde"}}, "category": "Sm"}, {
    "key": "224C",
    "mappings": {"default": {"default": "todo igual a"}},
    "category": "Sm"
  }, {"key": "224D", "mappings": {"default": {"default": "equivalente a"}}, "category": "Sm"}, {
    "key": "224E",
    "mappings": {"default": {"default": "geométricamente equivalente a"}},
    "category": "Sm"
  }, {"key": "224F", "mappings": {"default": {"default": "diferencia entre"}}, "category": "Sm"}, {
    "key": "2250",
    "mappings": {"default": {"default": "se acerca al límite"}},
    "category": "Sm"
  }, {"key": "2251", "mappings": {"default": {"default": "geométricamente igual a"}}, "category": "Sm"}, {
    "key": "2252",
    "mappings": {"default": {"default": "aproximadamente igual a o imagen de"}},
    "category": "Sm"
  }, {
    "key": "2253",
    "mappings": {"default": {"default": "imagen de o aproximadamente igual a"}},
    "category": "Sm"
  }, {"key": "2254", "mappings": {"default": {"default": "dos puntos igual"}}, "category": "Sm"}, {
    "key": "2255",
    "mappings": {"default": {"default": "igual dos puntos"}},
    "category": "Sm"
  }, {"key": "2256", "mappings": {"default": {"default": "igual incluyendo anillo"}}, "category": "Sm"}, {
    "key": "2257",
    "mappings": {"default": {"default": "igual anillo a"}},
    "category": "Sm"
  }, {"key": "2258", "mappings": {"default": {"default": "corresponde a"}}, "category": "Sm"}, {
    "key": "2259",
    "mappings": {"default": {"default": "estima a"}},
    "category": "Sm"
  }, {"key": "225A", "mappings": {"default": {"default": "equiangular con"}}, "category": "Sm"}, {
    "key": "225B",
    "mappings": {"default": {"default": "igual con estrella"}},
    "category": "Sm"
  }, {"key": "225C", "mappings": {"default": {"default": "igual con delta"}}, "category": "Sm"}, {
    "key": "225D",
    "mappings": {"default": {"default": "igual por definición a"}},
    "category": "Sm"
  }, {"key": "225E", "mappings": {"default": {"default": "medido por"}}, "category": "Sm"}, {
    "key": "225F",
    "mappings": {"default": {"default": "igual con interrogante"}},
    "category": "Sm"
  }, {
    "key": "2260",
    "mappings": {"default": {"default": "no es igual a", "defaultMP": "distinto de"}},
    "category": "Sm"
  }, {"key": "2261", "mappings": {"default": {"default": "idéntico a"}}, "category": "Sm"}, {
    "key": "2262",
    "mappings": {"default": {"default": "no es idéntico a"}},
    "category": "Sm"
  }, {
    "key": "2263",
    "mappings": {"default": {"default": "estrictamente equivalente a"}},
    "category": "Sm"
  }, {"key": "2264", "mappings": {"default": {"default": "menor o igual que"}}, "category": "Sm"}, {
    "key": "2265",
    "mappings": {"default": {"default": "mayor o igual que"}},
    "category": "Sm"
  }, {"key": "2266", "mappings": {"default": {"default": "menor o igual que"}}, "category": "Sm"}, {
    "key": "2267",
    "mappings": {"default": {"default": "mayor o igual que"}},
    "category": "Sm"
  }, {"key": "2268", "mappings": {"default": {"default": "estrictamente menor que"}}, "category": "Sm"}, {
    "key": "2269",
    "mappings": {"default": {"default": "estrictamente mayor que"}},
    "category": "Sm"
  }, {"key": "226A", "mappings": {"default": {"default": "mucho menor que"}}, "category": "Sm"}, {
    "key": "226B",
    "mappings": {"default": {"default": "mucho mayor que"}},
    "category": "Sm"
  }, {"key": "226C", "mappings": {"default": {"default": "entre"}}, "category": "Sm"}, {
    "key": "226D",
    "mappings": {"default": {"default": "no equivalente a"}},
    "category": "Sm"
  }, {"key": "226E", "mappings": {"default": {"default": "no menor que"}}, "category": "Sm"}, {
    "key": "226F",
    "mappings": {"default": {"default": "no mayor que"}},
    "category": "Sm"
  }, {"key": "2270", "mappings": {"default": {"default": "no menor ni igual a"}}, "category": "Sm"}, {
    "key": "2271",
    "mappings": {"default": {"default": "no mayor ni igual a"}},
    "category": "Sm"
  }, {"key": "2272", "mappings": {"default": {"default": "menor o equivalente a"}}, "category": "Sm"}, {
    "key": "2273",
    "mappings": {"default": {"default": "mayor o equivalente a"}},
    "category": "Sm"
  }, {
    "key": "2274",
    "mappings": {"default": {"default": "no menor ni equivalente a"}},
    "category": "Sm"
  }, {
    "key": "2275",
    "mappings": {"default": {"default": "no mayor ni equivalente a"}},
    "category": "Sm"
  }, {"key": "2276", "mappings": {"default": {"default": "menor o mayor que"}}, "category": "Sm"}, {
    "key": "2277",
    "mappings": {"default": {"default": "mayor o menor que"}},
    "category": "Sm"
  }, {"key": "2278", "mappings": {"default": {"default": "no menor ni mayor que"}}, "category": "Sm"}, {
    "key": "2279",
    "mappings": {"default": {"default": "no mayor ni menor que"}},
    "category": "Sm"
  }, {"key": "227A", "mappings": {"default": {"default": "precede a"}}, "category": "Sm"}, {
    "key": "227B",
    "mappings": {"default": {"default": "sigue a"}},
    "category": "Sm"
  }, {"key": "227C", "mappings": {"default": {"default": "precede o es igual a"}}, "category": "Sm"}, {
    "key": "227D",
    "mappings": {"default": {"default": "sigue o es igual a"}},
    "category": "Sm"
  }, {
    "key": "227E",
    "mappings": {"default": {"default": "precede o es equivalente a"}},
    "category": "Sm"
  }, {
    "key": "227F",
    "mappings": {"default": {"default": "sigue o es equivalente a"}},
    "category": "Sm"
  }, {"key": "2280", "mappings": {"default": {"default": "no precede a"}}, "category": "Sm"}, {
    "key": "2281",
    "mappings": {"default": {"default": "no sigue a"}},
    "category": "Sm"
  }, {"key": "2282", "mappings": {"default": {"default": "incluido en"}}, "category": "Sm"}, {
    "key": "2283",
    "mappings": {"default": {"default": "contiene a"}},
    "category": "Sm"
  }, {"key": "2284", "mappings": {"default": {"default": "no incluido en"}}, "category": "Sm"}, {
    "key": "2285",
    "mappings": {"default": {"default": "no contiene a"}},
    "category": "Sm"
  }, {"key": "2286", "mappings": {"default": {"default": "incluido o igual a"}}, "category": "Sm"}, {
    "key": "2287",
    "mappings": {"default": {"default": "contiene o es igual a"}},
    "category": "Sm"
  }, {"key": "2288", "mappings": {"default": {"default": "no incluido ni igual a"}}, "category": "Sm"}, {
    "key": "2289",
    "mappings": {"default": {"default": "no contiene ni es igual a"}},
    "category": "Sm"
  }, {
    "key": "228A",
    "mappings": {"default": {"default": "incluido estrictamente en"}},
    "category": "Sm"
  }, {
    "key": "228B",
    "mappings": {"default": {"default": "contiene estrictamente a"}},
    "category": "Sm"
  }, {"key": "228C", "mappings": {"default": {"default": "familia de conjuntos"}}, "category": "Sm"}, {
    "key": "228D",
    "mappings": {"default": {"default": "producto de familia de conjuntos"}},
    "category": "Sm"
  }, {
    "key": "228E",
    "mappings": {"default": {"default": "unión de familia de conjuntos"}},
    "category": "Sm"
  }, {"key": "228F", "mappings": {"default": {"default": "imagen cuadrada de"}}, "category": "Sm"}, {
    "key": "2290",
    "mappings": {"default": {"default": "original cuadrado de"}},
    "category": "Sm"
  }, {
    "key": "2291",
    "mappings": {"default": {"default": "imagen cuadrada de o igual a"}},
    "category": "Sm"
  }, {
    "key": "2292",
    "mappings": {"default": {"default": "original cuadrado de o igual a"}},
    "category": "Sm"
  }, {"key": "2293", "mappings": {"default": {"default": "intersección cuadrada"}}, "category": "Sm"}, {
    "key": "2294",
    "mappings": {"default": {"default": "unión cuadrada"}},
    "category": "Sm"
  }, {"key": "2295", "mappings": {"default": {"default": "más en círculo"}}, "category": "Sm"}, {
    "key": "2296",
    "mappings": {"default": {"default": "menos en círculo"}},
    "category": "Sm"
  }, {"key": "2297", "mappings": {"default": {"default": "por en círculo"}}, "category": "Sm"}, {
    "key": "2298",
    "mappings": {"default": {"default": "barra en círculo"}},
    "category": "Sm"
  }, {"key": "2299", "mappings": {"default": {"default": "punto en círculo"}}, "category": "Sm"}, {
    "key": "229A",
    "mappings": {"default": {"default": "anillo en círculo"}},
    "category": "Sm"
  }, {"key": "229B", "mappings": {"default": {"default": "asterisco en círculo"}}, "category": "Sm"}, {
    "key": "229C",
    "mappings": {"default": {"default": "igual en círculo"}},
    "category": "Sm"
  }, {"key": "229D", "mappings": {"default": {"default": "menos en círculo"}}, "category": "Sm"}, {
    "key": "229E",
    "mappings": {"default": {"default": "más en cuadrado"}},
    "category": "Sm"
  }, {"key": "229F", "mappings": {"default": {"default": "menos en cuadrado"}}, "category": "Sm"}, {
    "key": "22A0",
    "mappings": {"default": {"default": "veces en cuadrado"}},
    "category": "Sm"
  }, {"key": "22A1", "mappings": {"default": {"default": "punto en cuadrado"}}, "category": "Sm"}, {
    "key": "22A2",
    "mappings": {"default": {"default": "t horizontal hacia la izquierda"}},
    "category": "Sm"
  }, {
    "key": "22A3",
    "mappings": {"default": {"default": "t horizontal hacia la derecha"}},
    "category": "Sm"
  }, {"key": "22A4", "mappings": {"default": {"default": "perpendicular"}}, "category": "Sm"}, {
    "key": "22A5",
    "mappings": {"default": {"default": "perpendicular invertida"}},
    "category": "Sm"
  }, {"key": "22A6", "mappings": {"default": {"default": "afirmación"}}, "category": "Sm"}, {
    "key": "22A7",
    "mappings": {"default": {"default": "modela"}},
    "category": "Sm"
  }, {"key": "22A8", "mappings": {"default": {"default": "verdadero"}}, "category": "Sm"}, {
    "key": "22A9",
    "mappings": {"default": {"default": "obliga"}},
    "category": "Sm"
  }, {
    "key": "22AA",
    "mappings": {"default": {"default": "triple barra vertical torniquete derecho"}},
    "category": "Sm"
  }, {
    "key": "22AB",
    "mappings": {"default": {"default": "doble barra vertical doble torniquete derecho"}},
    "category": "Sm"
  }, {"key": "22AC", "mappings": {"default": {"default": "no prueba"}}, "category": "Sm"}, {
    "key": "22AD",
    "mappings": {"default": {"default": "falso"}},
    "category": "Sm"
  }, {"key": "22AE", "mappings": {"default": {"default": "no obliga"}}, "category": "Sm"}, {
    "key": "22AF",
    "mappings": {"default": {"default": "doble barra vertical doble torniquete derecho negada"}},
    "category": "Sm"
  }, {"key": "22B0", "mappings": {"default": {"default": "precede respecto de"}}, "category": "Sm"}, {
    "key": "22B1",
    "mappings": {"default": {"default": "sigue respecto de"}},
    "category": "Sm"
  }, {"key": "22B2", "mappings": {"default": {"default": "subgrupo normal de"}}, "category": "Sm"}, {
    "key": "22B3",
    "mappings": {"default": {"default": "contiene como subgrupo normal a"}},
    "category": "Sm"
  }, {
    "key": "22B4",
    "mappings": {"default": {"default": "subgrupo normal o es igual a"}},
    "category": "Sm"
  }, {
    "key": "22B5",
    "mappings": {"default": {"default": "contiene como subgrupo normal o es igual a"}},
    "category": "Sm"
  }, {"key": "22B6", "mappings": {"default": {"default": "original de"}}, "category": "Sm"}, {
    "key": "22B7",
    "mappings": {"default": {"default": "imagen de"}},
    "category": "Sm"
  }, {"key": "22B8", "mappings": {"default": {"default": "multifunción"}}, "category": "Sm"}, {
    "key": "22B9",
    "mappings": {"default": {"default": "matriz hermitiana conjugada"}},
    "category": "Sm"
  }, {"key": "22BA", "mappings": {"default": {"default": "interpola"}}, "category": "Sm"}, {
    "key": "22BB",
    "mappings": {"default": {"default": "o excluyente"}},
    "category": "Sm"
  }, {"key": "22BC", "mappings": {"default": {"default": "no y"}}, "category": "Sm"}, {
    "key": "22BD",
    "mappings": {"default": {"default": "no o"}},
    "category": "Sm"
  }, {"key": "22BF", "mappings": {"default": {"default": "triángulo rectángulo"}}, "category": "Sm"}, {
    "key": "22C0",
    "mappings": {"default": {"default": "y"}},
    "category": "Sm"
  }, {"key": "22C1", "mappings": {"default": {"default": "o"}}, "category": "Sm"}, {
    "key": "22C2",
    "mappings": {"default": {"default": "intersección"}},
    "category": "Sm"
  }, {"key": "22C3", "mappings": {"default": {"default": "unión"}}, "category": "Sm"}, {
    "key": "22C4",
    "mappings": {"default": {"default": "operador diamante"}},
    "category": "Sm"
  }, {"key": "22C5", "mappings": {"default": {"default": "punto"}}, "category": "Sm"}, {
    "key": "22C6",
    "mappings": {"default": {"default": "estrella"}},
    "category": "Sm"
  }, {"key": "22C7", "mappings": {"default": {"default": "dividido por"}}, "category": "Sm"}, {
    "key": "22C8",
    "mappings": {"default": {"default": "moño"}},
    "category": "Sm"
  }, {
    "key": "22C9",
    "mappings": {"default": {"default": "factor normal izquierdo producto semidirecto"}},
    "category": "Sm"
  }, {
    "key": "22CA",
    "mappings": {"default": {"default": "factor normal derecho producto semidirecto"}},
    "category": "Sm"
  }, {
    "key": "22CB",
    "mappings": {"default": {"default": "producto semidirecto izquierdo"}},
    "category": "Sm"
  }, {
    "key": "22CC",
    "mappings": {"default": {"default": "producto semidirecto derecho"}},
    "category": "Sm"
  }, {"key": "22CD", "mappings": {"default": {"default": "tilde invertida igual a"}}, "category": "Sm"}, {
    "key": "22CE",
    "mappings": {"default": {"default": "o rizada"}},
    "category": "Sm"
  }, {"key": "22CF", "mappings": {"default": {"default": "y rizada"}}, "category": "Sm"}, {
    "key": "22D0",
    "mappings": {"default": {"default": "doble incluido"}},
    "category": "Sm"
  }, {"key": "22D1", "mappings": {"default": {"default": "doble contiene"}}, "category": "Sm"}, {
    "key": "22D2",
    "mappings": {"default": {"default": "doble intersección"}},
    "category": "Sm"
  }, {"key": "22D3", "mappings": {"default": {"default": "doble unión"}}, "category": "Sm"}, {
    "key": "22D4",
    "mappings": {"default": {"default": "bieldo"}},
    "category": "Sm"
  }, {"key": "22D5", "mappings": {"default": {"default": "paralela o igual a"}}, "category": "Sm"}, {
    "key": "22D6",
    "mappings": {"default": {"default": "menor que con punto"}},
    "category": "Sm"
  }, {"key": "22D7", "mappings": {"default": {"default": "mayor que con punto"}}, "category": "Sm"}, {
    "key": "22D8",
    "mappings": {"default": {"default": "mucho menor que"}},
    "category": "Sm"
  }, {"key": "22D9", "mappings": {"default": {"default": "mucho mayor que"}}, "category": "Sm"}, {
    "key": "22DA",
    "mappings": {"default": {"default": "menor igual o mayor que"}},
    "category": "Sm"
  }, {"key": "22DB", "mappings": {"default": {"default": "mayor igual o menor que"}}, "category": "Sm"}, {
    "key": "22DC",
    "mappings": {"default": {"default": "menor o igual que"}},
    "category": "Sm"
  }, {"key": "22DD", "mappings": {"default": {"default": "mayor o igual que"}}, "category": "Sm"}, {
    "key": "22DE",
    "mappings": {"default": {"default": "precede o es igual a"}},
    "category": "Sm"
  }, {"key": "22DF", "mappings": {"default": {"default": "sigue o es igual a"}}, "category": "Sm"}, {
    "key": "22E0",
    "mappings": {"default": {"default": "no precede ni es igual a"}},
    "category": "Sm"
  }, {"key": "22E1", "mappings": {"default": {"default": "no sigue ni es igual a"}}, "category": "Sm"}, {
    "key": "22E2",
    "mappings": {"default": {"default": "no es imagen cuadrada ni igual a"}},
    "category": "Sm"
  }, {
    "key": "22E3",
    "mappings": {"default": {"default": "no es original cuadrada ni igual a"}},
    "category": "Sm"
  }, {
    "key": "22E4",
    "mappings": {"default": {"default": "imagen cuadrada estricta"}},
    "category": "Sm"
  }, {
    "key": "22E5",
    "mappings": {"default": {"default": "original cuadrada estricta"}},
    "category": "Sm"
  }, {
    "key": "22E6",
    "mappings": {"default": {"default": "menor que no equivalente a"}},
    "category": "Sm"
  }, {
    "key": "22E7",
    "mappings": {"default": {"default": "mayor que no equivalente a"}},
    "category": "Sm"
  }, {
    "key": "22E8",
    "mappings": {"default": {"default": "precede no equivalente a"}},
    "category": "Sm"
  }, {"key": "22E9", "mappings": {"default": {"default": "sigue no equivalente a"}}, "category": "Sm"}, {
    "key": "22EA",
    "mappings": {"default": {"default": "no subgrupo normal de"}},
    "category": "Sm"
  }, {
    "key": "22EB",
    "mappings": {"default": {"default": "no contiene como subgrupo normal a"}},
    "category": "Sm"
  }, {
    "key": "22EC",
    "mappings": {"default": {"default": "no es subgrupo normal ni igual a"}},
    "category": "Sm"
  }, {
    "key": "22ED",
    "mappings": {"default": {"default": "no contiene como subgrupo normal ni es igual a"}},
    "category": "Sm"
  }, {
    "key": "22EE",
    "mappings": {"default": {"default": "puntos suspensivos verticales"}},
    "category": "Sm"
  }, {
    "key": "22EF",
    "mappings": {"default": {"default": "puntos suspensivos altos"}},
    "category": "Sm"
  }, {
    "key": "22F0",
    "mappings": {"default": {"default": "puntos suspensivos diagonales subiendo"}},
    "category": "Sm"
  }, {
    "key": "22F1",
    "mappings": {"default": {"default": "puntos suspensivos diagonales bajando"}},
    "category": "Sm"
  }, {
    "key": "22F2",
    "mappings": {"default": {"default": "pertenece con trazo horizontal"}},
    "category": "Sm"
  }, {
    "key": "22F3",
    "mappings": {"default": {"default": "pertenece con barra vertical al fin de raya horizontal"}},
    "category": "Sm"
  }, {
    "key": "22F4",
    "mappings": {"default": {"default": "pertenece en pequeño con trazo vertical"}},
    "category": "Sm"
  }, {
    "key": "22F5",
    "mappings": {"default": {"default": "pertenece con punto arriba"}},
    "category": "Sm"
  }, {
    "key": "22F6",
    "mappings": {"default": {"default": "pertenece con barra arriba"}},
    "category": "Sm"
  }, {"key": "22F7", "mappings": {"default": {"default": "perteneciente subrayado"}}, "category": "Sm"}, {
    "key": "22F8",
    "mappings": {"default": {"default": "perteneciente subrayado"}},
    "category": "Sm"
  }, {
    "key": "22F9",
    "mappings": {"default": {"default": "pertenece con dos trazos horizontales"}},
    "category": "Sm"
  }, {"key": "22FA", "mappings": {"default": {"default": "contiene o es igual a"}}, "category": "Sm"}, {
    "key": "22FB",
    "mappings": {"default": {"default": "contiene con marca"}},
    "category": "Sm"
  }, {"key": "22FC", "mappings": {"default": {"default": "contiene con marca"}}, "category": "Sm"}, {
    "key": "22FD",
    "mappings": {"default": {"default": "contiene con barra"}},
    "category": "Sm"
  }, {"key": "22FE", "mappings": {"default": {"default": "perteneciente con barra"}}, "category": "Sm"}, {
    "key": "22FF",
    "mappings": {"default": {"default": "z anotación asociación de bolsa"}},
    "category": "Sm"
  }, {"key": "2300", "mappings": {"default": {"default": "diámetro"}}, "category": "So"}, {
    "key": "2302",
    "mappings": {"default": {"default": "casa"}},
    "category": "So"
  }, {"key": "2305", "mappings": {"default": {"default": "en proyectividad con"}}, "category": "So"}, {
    "key": "2306",
    "mappings": {"default": {"default": "en perspectividad con"}},
    "category": "So"
  }, {"category": "So", "key": "2307", "mappings": {"default": {"default": "línea ondulada"}}}, {
    "key": "2310",
    "mappings": {"default": {"default": "no invertido"}},
    "category": "So"
  }, {"category": "So", "key": "2311", "mappings": {"default": {"default": "pastillas cuadradas"}}}, {
    "key": "2312",
    "mappings": {"default": {"default": "arco"}},
    "category": "So"
  }, {"key": "2313", "mappings": {"default": {"default": "segmento"}}, "category": "So"}, {
    "category": "So",
    "key": "2314",
    "mappings": {"default": {"default": "sector"}}
  }, {"category": "So", "key": "2795", "mappings": {"default": {"default": "signo más pesado"}}}, {
    "category": "So",
    "key": "2796",
    "mappings": {"default": {"default": "signo menos pesado"}}
  }, {
    "category": "So",
    "key": "2797",
    "mappings": {"default": {"default": "signo de la división pesada"}}
  }, {"category": "So", "key": "27B0", "mappings": {"default": {"default": "bucle rizado"}}}, {
    "category": "So",
    "key": "27BF",
    "mappings": {"default": {"default": "doble bucle rizado"}}
  }, {
    "key": "27C1",
    "mappings": {"default": {"default": "triángulo encerrando triángulo"}},
    "category": "Sm"
  }, {"key": "27C2", "mappings": {"default": {"default": "perpendicular"}}, "category": "Sm"}, {
    "key": "27C3",
    "mappings": {"default": {"default": "incluido en"}},
    "category": "Sm"
  }, {"key": "27C4", "mappings": {"default": {"default": "contiene a"}}, "category": "Sm"}, {
    "key": "27C7",
    "mappings": {"default": {"default": "punto en círculo"}},
    "category": "Sm"
  }, {
    "key": "27C8",
    "mappings": {"default": {"default": "atrás tajo antes de subconjunto"}},
    "category": "Sm"
  }, {
    "key": "27C9",
    "mappings": {"default": {"default": "sobreconjunto antes de tajo"}},
    "category": "Sm"
  }, {
    "key": "27CA",
    "mappings": {"default": {"default": "barra vertical con golpe horizontal"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "27CB",
    "mappings": {"default": {"default": "diagonal matemático de levantamiento"}}
  }, {"key": "27CC", "mappings": {"default": {"default": "división larga"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "27CD",
    "mappings": {"default": {"default": "diagonal descendente matemática"}}
  }, {"category": "Sm", "key": "27CE", "mappings": {"default": {"default": "cuadrado Lógico Y"}}}, {
    "category": "Sm",
    "key": "27CF",
    "mappings": {"default": {"default": "cuadrado Lógico O"}}
  }, {
    "category": "Sm",
    "key": "27D0",
    "mappings": {"default": {"default": "diamante blanco con punto centrado"}}
  }, {"key": "27D1", "mappings": {"default": {"default": "y con punto interior"}}, "category": "Sm"}, {
    "key": "27D2",
    "mappings": {"default": {"default": "elemento de apertura hacia arriba"}},
    "category": "Sm"
  }, {
    "key": "27D3",
    "mappings": {"default": {"default": "ángulo inferior derecho con punto"}},
    "category": "Sm"
  }, {
    "key": "27D4",
    "mappings": {"default": {"default": "ángulo inferior izquierdo con punto"}},
    "category": "Sm"
  }, {"key": "27D5", "mappings": {"default": {"default": "izquierdo exterior une"}}, "category": "Sm"}, {
    "key": "27D6",
    "mappings": {"default": {"default": "derecho exterior une"}},
    "category": "Sm"
  }, {"key": "27D7", "mappings": {"default": {"default": "lleno exterior une"}}, "category": "Sm"}, {
    "key": "27D8",
    "mappings": {"default": {"default": "perpendicular inversa"}},
    "category": "Sm"
  }, {"key": "27D9", "mappings": {"default": {"default": "perpendicular"}}, "category": "Sm"}, {
    "key": "27DA",
    "mappings": {"default": {"default": "izquierdo y derecho doble torniquete"}},
    "category": "Sm"
  }, {"key": "27DB", "mappings": {"default": {"default": "doble T horizontal"}}, "category": "Sm"}, {
    "key": "27DC",
    "mappings": {"default": {"default": "multifunción por la izquierda"}},
    "category": "Sm"
  }, {
    "key": "27DD",
    "mappings": {"default": {"default": "t horizontal derecha larga"}},
    "category": "Sm"
  }, {
    "key": "27DE",
    "mappings": {"default": {"default": "t horizontal izquierda larga"}},
    "category": "Sm"
  }, {
    "key": "27DF",
    "mappings": {"default": {"default": "perpendicular inversa con círculo arriba"}},
    "category": "Sm"
  }, {
    "key": "27E0",
    "mappings": {"default": {"default": "pastilla dividida por regla horizontal"}},
    "category": "Sm"
  }, {"key": "27E1", "mappings": {"default": {"default": "diamante cóncavo vacío"}}, "category": "Sm"}, {
    "key": "27E2",
    "mappings": {"default": {"default": "diamante cóncavo con tictac hacia izquierda"}},
    "category": "Sm"
  }, {
    "key": "27E3",
    "mappings": {"default": {"default": "diamante cóncavo con tictac hacia derecha"}},
    "category": "Sm"
  }, {
    "key": "27E4",
    "mappings": {"default": {"default": "cuadrado con tictac hacia izquierda"}},
    "category": "Sm"
  }, {
    "key": "27E5",
    "mappings": {"default": {"default": "cuadrado con tictac hacia derecha"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "292B",
    "mappings": {"default": {"default": "cruce en diagonal Cruce en diagonal Caída en diagonal"}}
  }, {
    "category": "Sm",
    "key": "292C",
    "mappings": {"default": {"default": "diagonal descendente Cruce diagonal ascendente"}}
  }, {
    "category": "Sm",
    "key": "2980",
    "mappings": {"default": {"default": "delimitador de barra vertical triple"}}
  }, {"category": "Sm", "key": "2981", "mappings": {"default": {"default": "punto de notación Z"}}}, {
    "category": "Sm",
    "key": "2982",
    "mappings": {"default": {"default": "tipo de notación Z Colon"}}
  }, {"category": "Sm", "key": "2999", "mappings": {"default": {"default": "valla punteada"}}}, {
    "key": "299A",
    "mappings": {"default": {"default": "zigzag en vertical"}},
    "category": "Sm"
  }, {
    "key": "29B0",
    "mappings": {"default": {"default": "conjunto vacío invertido"}},
    "category": "Sm"
  }, {
    "key": "29B1",
    "mappings": {"default": {"default": "conjunto vacío con barra"}},
    "category": "Sm"
  }, {
    "key": "29B2",
    "mappings": {"default": {"default": "conjunto vacío con círculo"}},
    "category": "Sm"
  }, {"key": "29B5", "mappings": {"default": {"default": "círculo con barra"}}, "category": "Sm"}, {
    "key": "29B6",
    "mappings": {"default": {"default": "barra en círculo"}},
    "category": "Sm"
  }, {"key": "29B7", "mappings": {"default": {"default": "paralelas en círculo"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "29B8",
    "mappings": {"default": {"default": "círculo inverso solidus"}}
  }, {
    "key": "29B9",
    "mappings": {"default": {"default": "perpendicular en círculo"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "29BA",
    "mappings": {"default": {"default": "círculo dividido por barra horizontal y mitad superior dividido por barra vertical"}}
  }, {
    "key": "29BB",
    "mappings": {"default": {"default": "x superimpresa en círculo"}},
    "category": "Sm"
  }, {
    "key": "29BC",
    "mappings": {"default": {"default": "circled anticlockwise rotated division"}},
    "category": "Sm"
  }, {"key": "29BE", "mappings": {"default": {"default": "bullet vacía en círculo"}}, "category": "Sm"}, {
    "key": "29BF",
    "mappings": {"default": {"default": "bullet en círculo"}},
    "category": "Sm"
  }, {"key": "29C0", "mappings": {"default": {"default": "menor que en círculo"}}, "category": "Sm"}, {
    "key": "29C1",
    "mappings": {"default": {"default": "mayor que en círculo"}},
    "category": "Sm"
  }, {
    "key": "29C2",
    "mappings": {"default": {"default": "círculo con círculo a la derecha"}},
    "category": "Sm"
  }, {
    "key": "29C3",
    "mappings": {"default": {"default": "circle with two horizontal strokes to the right"}},
    "category": "Sm"
  }, {
    "key": "29C4",
    "mappings": {"default": {"default": "cuadrado con diagonal ascendente"}},
    "category": "Sm"
  }, {
    "key": "29C5",
    "mappings": {"default": {"default": "cuadrado con diagonal descendente"}},
    "category": "Sm"
  }, {"category": "Sm", "key": "29C6", "mappings": {"default": {"default": "asterisco cuadrado"}}}, {
    "category": "Sm",
    "key": "29C7",
    "mappings": {"default": {"default": "pequeño círculo cuadrado"}}
  }, {"category": "Sm", "key": "29C8", "mappings": {"default": {"default": "cuadrado cuadrado"}}}, {
    "key": "29C9",
    "mappings": {"default": {"default": "dos cuadrados unidos"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "29CA",
    "mappings": {"default": {"default": "triángulo con punto arriba"}}
  }, {
    "category": "Sm",
    "key": "29CB",
    "mappings": {"default": {"default": "triángulo con barra inferior"}}
  }, {"category": "Sm", "key": "29CC", "mappings": {"default": {"default": "s en triangulo"}}}, {
    "key": "29CD",
    "mappings": {"default": {"default": "triangle with serifs at bottom"}},
    "category": "Sm"
  }, {
    "key": "29CE",
    "mappings": {"default": {"default": "triángulo hacia la derecha sobre triángulo hacia la izquierda"}},
    "category": "Sm"
  }, {
    "key": "29CF",
    "mappings": {"default": {"default": "subgrupo normal o igual a"}},
    "category": "Sm"
  }, {
    "key": "29D0",
    "mappings": {"default": {"default": "contiene como subgrupo normal o es igual a"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "29D1",
    "mappings": {"default": {"default": "pajarita Con Mitad Izquierda Negra"}}
  }, {
    "category": "Sm",
    "key": "29D2",
    "mappings": {"default": {"default": "pajarita Con Mitad Derecha Negra"}}
  }, {"category": "Sm", "key": "29D3", "mappings": {"default": {"default": "pajarita Negra"}}}, {
    "category": "Sm",
    "key": "29D4",
    "mappings": {"default": {"default": "tiempos con Mitad Izquierda Negra"}}
  }, {
    "category": "Sm",
    "key": "29D5",
    "mappings": {"default": {"default": "tiempos con la mitad derecha negra"}}
  }, {
    "category": "Sm",
    "key": "29D6",
    "mappings": {"default": {"default": "reloj de arena blanco"}}
  }, {"category": "Sm", "key": "29D7", "mappings": {"default": {"default": "reloj de arena negro"}}}, {
    "key": "29DC",
    "mappings": {"default": {"default": "infinito incompleto"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "29DD",
    "mappings": {"default": {"default": "empate sobre el infinito"}}
  }, {"key": "29DE", "mappings": {"default": {"default": "no infinito"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "29DF",
    "mappings": {"default": {"default": "multimapa de doble extremo"}}
  }, {
    "category": "Sm",
    "key": "29E0",
    "mappings": {"default": {"default": "cuadrado con contorno contorneado"}}
  }, {"category": "Sm", "key": "29E1", "mappings": {"default": {"default": "aumenta como"}}}, {
    "category": "Sm",
    "key": "29E2",
    "mappings": {"default": {"default": "shuffle Product"}}
  }, {"key": "29E3", "mappings": {"default": {"default": "igual tachado doble "}}, "category": "Sm"}, {
    "key": "29E4",
    "mappings": {"default": {"default": "igual con tilde, con doble tachado"}},
    "category": "Sm"
  }, {
    "key": "29E5",
    "mappings": {"default": {"default": "idéntico a con doble tachado"}},
    "category": "Sm"
  }, {"category": "Sm", "key": "29E6", "mappings": {"default": {"default": "gleich Stark"}}}, {
    "category": "Sm",
    "key": "29E7",
    "mappings": {"default": {"default": "termodinamica"}}
  }, {
    "category": "Sm",
    "key": "29E8",
    "mappings": {"default": {"default": "triángulo que apunta hacia abajo con la mitad izquierda negra"}}
  }, {
    "category": "Sm",
    "key": "29E9",
    "mappings": {"default": {"default": "triángulo que apunta hacia abajo con la mitad derecha negra"}}
  }, {"key": "29EB", "mappings": {"default": {"default": "rombo relleno"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "29EE",
    "mappings": {"default": {"default": "plaza blanca con error"}}
  }, {
    "category": "Sm",
    "key": "29EF",
    "mappings": {"default": {"default": "error en la Plaza Negra"}}
  }, {
    "category": "Sm",
    "key": "29F0",
    "mappings": {"default": {"default": "error White Barred Diamond"}}
  }, {
    "category": "Sm",
    "key": "29F1",
    "mappings": {"default": {"default": "error Black Barred Diamond"}}
  }, {
    "category": "Sm",
    "key": "29F2",
    "mappings": {"default": {"default": "círculo blanco con error"}}
  }, {
    "category": "Sm",
    "key": "29F3",
    "mappings": {"default": {"default": "círculo negro con barras de error"}}
  }, {"key": "29F4", "mappings": {"default": {"default": "rule delayed"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "29F5",
    "mappings": {"default": {"default": "operador de Solidus Inverso"}}
  }, {"key": "29F6", "mappings": {"default": {"default": "solidus with overbar"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "29F7",
    "mappings": {"default": {"default": "solidus inverso con trazo horizontal"}}
  }, {"category": "Sm", "key": "29F8", "mappings": {"default": {"default": "sólido grande"}}}, {
    "category": "Sm",
    "key": "29F9",
    "mappings": {"default": {"default": "sólido inverso grande"}}
  }, {"category": "Sm", "key": "29FA", "mappings": {"default": {"default": "doble Plus"}}}, {
    "category": "Sm",
    "key": "29FB",
    "mappings": {"default": {"default": "triple Plus"}}
  }, {"category": "Sm", "key": "29FE", "mappings": {"default": {"default": "minúsculo"}}}, {
    "category": "Sm",
    "key": "29FF",
    "mappings": {"default": {"default": "miny"}}
  }, {
    "category": "Sm",
    "key": "2A00",
    "mappings": {"default": {"default": "operador de puntos circulares N-Ary"}}
  }, {
    "category": "Sm",
    "key": "2A01",
    "mappings": {"default": {"default": "operador N-Ary Circled Plus"}}
  }, {
    "category": "Sm",
    "key": "2A02",
    "mappings": {"default": {"default": "operador de los tiempos en círculo de N-Ary"}}
  }, {
    "category": "Sm",
    "key": "2A03",
    "mappings": {"default": {"default": "operador de la unión N-Ary con punto"}}
  }, {
    "category": "Sm",
    "key": "2A04",
    "mappings": {"default": {"default": "n-Ary Union Operator con Plus"}}
  }, {
    "category": "Sm",
    "key": "2A05",
    "mappings": {"default": {"default": "operador de intersecciones cuadradas N-Ary"}}
  }, {
    "category": "Sm",
    "key": "2A06",
    "mappings": {"default": {"default": "n-Ary Square Union Operator"}}
  }, {
    "category": "Sm",
    "key": "2A07",
    "mappings": {"default": {"default": "dos lógico y operador"}}
  }, {
    "category": "Sm",
    "key": "2A08",
    "mappings": {"default": {"default": "dos lógico u operador"}}
  }, {
    "category": "Sm",
    "key": "2A09",
    "mappings": {"default": {"default": "operador de N-Ary Times"}}
  }, {"category": "Sm", "key": "2A0A", "mappings": {"default": {"default": "modulo dos sumas"}}}, {
    "category": "Sm",
    "key": "2A0B",
    "mappings": {"default": {"default": "suma con Integral"}}
  }, {
    "key": "2A0C",
    "mappings": {"default": {"default": "operador integral cuádruple"}},
    "category": "Sm"
  }, {"key": "2A0D", "mappings": {"default": {"default": "finite part integral"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "2A0E",
    "mappings": {"default": {"default": "integral con Doble Trazo"}}
  }, {
    "category": "Sm",
    "key": "2A0F",
    "mappings": {"default": {"default": "promedio Integral con Slash"}}
  }, {"key": "2A10", "mappings": {"default": {"default": "circulation function"}}, "category": "Sm"}, {
    "key": "2A11",
    "mappings": {"default": {"default": "integral en sentido antihorario"}},
    "category": "Sm"
  }, {
    "key": "2A12",
    "mappings": {"default": {"default": "integral lineal para rectángulo en polo"}},
    "category": "Sm"
  }, {
    "key": "2A13",
    "mappings": {"default": {"default": "integral lineal para semicírculo en polo"}},
    "category": "Sm"
  }, {
    "key": "2A14",
    "mappings": {"default": {"default": "integral lineal excluyendo el polo"}},
    "category": "Sm"
  }, {
    "key": "2A15",
    "mappings": {"default": {"default": "integral en torno de un punto operador"}},
    "category": "Sm"
  }, {
    "key": "2A16",
    "mappings": {"default": {"default": "operador integral para cuaterniones"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A18",
    "mappings": {"default": {"default": "integral con Times Sign"}}
  }, {
    "category": "Sm",
    "key": "2A19",
    "mappings": {"default": {"default": "integral con Intersección"}}
  }, {"category": "Sm", "key": "2A1A", "mappings": {"default": {"default": "integral con union"}}}, {
    "category": "Sm",
    "key": "2A1B",
    "mappings": {"default": {"default": "integral con Overbar"}}
  }, {
    "category": "Sm",
    "key": "2A1C",
    "mappings": {"default": {"default": "integral con barra inferior"}}
  }, {"category": "Sm", "key": "2A1D", "mappings": {"default": {"default": "unirse"}}}, {
    "category": "Sm",
    "key": "2A1E",
    "mappings": {"default": {"default": "operador de triángulo izquierdo grande"}}
  }, {
    "category": "Sm",
    "key": "2A1F",
    "mappings": {"default": {"default": "composición de esquemas de notación Z"}}
  }, {
    "category": "Sm",
    "key": "2A20",
    "mappings": {"default": {"default": "esquema de notación Z"}}
  }, {
    "category": "Sm",
    "key": "2A21",
    "mappings": {"default": {"default": "proyección del esquema de notación Z"}}
  }, {"key": "2A22", "mappings": {"default": {"default": "más con círculo"}}, "category": "Sm"}, {
    "key": "2A23",
    "mappings": {"default": {"default": "más con ángulo"}},
    "category": "Sm"
  }, {"key": "2A24", "mappings": {"default": {"default": "tilde con más suscrito"}}, "category": "Sm"}, {
    "key": "2A25",
    "mappings": {"default": {"default": "más con punto inferior"}},
    "category": "Sm"
  }, {"key": "2A26", "mappings": {"default": {"default": "tilde con más"}}, "category": "Sm"}, {
    "key": "2A27",
    "mappings": {"default": {"default": "más con 2 suscrito"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A28",
    "mappings": {"default": {"default": "signo más con triángulo negro"}}
  }, {"key": "2A29", "mappings": {"default": {"default": "menos con coma"}}, "category": "Sm"}, {
    "key": "2A2A",
    "mappings": {"default": {"default": "menos con punto inferior"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A2B",
    "mappings": {"default": {"default": "signo menos con puntos que caen"}}
  }, {
    "category": "Sm",
    "key": "2A2C",
    "mappings": {"default": {"default": "signo menos con puntos crecientes"}}
  }, {
    "key": "2A2D",
    "mappings": {"default": {"default": "más en semicírculo izquierdo"}},
    "category": "Sm"
  }, {
    "key": "2A2E",
    "mappings": {"default": {"default": "más en semicírculo derecho"}},
    "category": "Sm"
  }, {"key": "2A2F", "mappings": {"default": {"default": "producto vectorial"}}, "category": "Sm"}, {
    "key": "2A30",
    "mappings": {"default": {"default": "por con punto"}},
    "category": "Sm"
  }, {"key": "2A31", "mappings": {"default": {"default": "por con subrayado"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "2A32",
    "mappings": {"default": {"default": "producto semidirecto con fondo cerrado"}}
  }, {"key": "2A33", "mappings": {"default": {"default": "smash product"}}, "category": "Sm"}, {
    "key": "2A34",
    "mappings": {"default": {"default": "por en semicírculo izquierdo"}},
    "category": "Sm"
  }, {
    "key": "2A35",
    "mappings": {"default": {"default": "por en semicírculo derecho"}},
    "category": "Sm"
  }, {
    "key": "2A36",
    "mappings": {"default": {"default": "por en círculo con ángulo"}},
    "category": "Sm"
  }, {"key": "2A37", "mappings": {"default": {"default": "por en anillo"}}, "category": "Sm"}, {
    "key": "2A38",
    "mappings": {"default": {"default": "división en círculo"}},
    "category": "Sm"
  }, {"key": "2A39", "mappings": {"default": {"default": "más en triángulo"}}, "category": "Sm"}, {
    "key": "2A3A",
    "mappings": {"default": {"default": "menos en triángulo"}},
    "category": "Sm"
  }, {"key": "2A3B", "mappings": {"default": {"default": "por en triángulo"}}, "category": "Sm"}, {
    "key": "2A3C",
    "mappings": {"default": {"default": "producto interior"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A3D",
    "mappings": {"default": {"default": "producto interior derecho"}}
  }, {
    "category": "Sm",
    "key": "2A3E",
    "mappings": {"default": {"default": "notación Z Composición Relacional"}}
  }, {"key": "2A3F", "mappings": {"default": {"default": "coproducto"}}, "category": "Sm"}, {
    "key": "2A40",
    "mappings": {"default": {"default": "intersección con punto"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A41",
    "mappings": {"default": {"default": "unión con signo de menos"}}
  }, {"key": "2A42", "mappings": {"default": {"default": "unión con barra"}}, "category": "Sm"}, {
    "key": "2A43",
    "mappings": {"default": {"default": "intersección con barra"}},
    "category": "Sm"
  }, {"key": "2A44", "mappings": {"default": {"default": "intersección con and"}}, "category": "Sm"}, {
    "key": "2A45",
    "mappings": {"default": {"default": "unión con or"}},
    "category": "Sm"
  }, {
    "key": "2A46",
    "mappings": {"default": {"default": "unión sobre intersección"}},
    "category": "Sm"
  }, {
    "key": "2A47",
    "mappings": {"default": {"default": "intersección sobre unión"}},
    "category": "Sm"
  }, {
    "key": "2A48",
    "mappings": {"default": {"default": "unión sobre barra, sobre intersección"}},
    "category": "Sm"
  }, {
    "key": "2A49",
    "mappings": {"default": {"default": "intersección sobre barra, sobre unión "}},
    "category": "Sm"
  }, {"key": "2A4A", "mappings": {"default": {"default": "unión y unión"}}, "category": "Sm"}, {
    "key": "2A4B",
    "mappings": {"default": {"default": "intersección e intersección"}},
    "category": "Sm"
  }, {"key": "2A4C", "mappings": {"default": {"default": "unión cerrada"}}, "category": "Sm"}, {
    "key": "2A4D",
    "mappings": {"default": {"default": "intersección cerrada"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A4E",
    "mappings": {"default": {"default": "intersección cuadrada doble"}}
  }, {"category": "Sm", "key": "2A4F", "mappings": {"default": {"default": "unión cuadrada doble"}}}, {
    "key": "2A50",
    "mappings": {"default": {"default": "unión cerrada con por"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A51",
    "mappings": {"default": {"default": "lógico y con punto arriba."}}
  }, {
    "category": "Sm",
    "key": "2A52",
    "mappings": {"default": {"default": "lógico o con punto arriba"}}
  }, {"key": "2A53", "mappings": {"default": {"default": "y"}}, "category": "Sm"}, {
    "key": "2A54",
    "mappings": {"default": {"default": "o"}},
    "category": "Sm"
  }, {
    "key": "2A55",
    "mappings": {"default": {"default": "two intersecting logical and"}},
    "category": "Sm"
  }, {
    "key": "2A56",
    "mappings": {"default": {"default": "two intersecting logical or"}},
    "category": "Sm"
  }, {"key": "2A57", "mappings": {"default": {"default": "sloping large or"}}, "category": "Sm"}, {
    "key": "2A58",
    "mappings": {"default": {"default": "sloping large and"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A59",
    "mappings": {"default": {"default": "lógica o superpuesta, lógica y"}}
  }, {
    "key": "2A5A",
    "mappings": {"default": {"default": "logical and with middle stem"}},
    "category": "Sm"
  }, {
    "key": "2A5B",
    "mappings": {"default": {"default": "logical or with middle stem"}},
    "category": "Sm"
  }, {
    "key": "2A5C",
    "mappings": {"default": {"default": "logical and with horizontal dash"}},
    "category": "Sm"
  }, {
    "key": "2A5D",
    "mappings": {"default": {"default": "logical or with horizontal dash"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A5E",
    "mappings": {"default": {"default": "lógica y con doble Overbar"}}
  }, {
    "key": "2A5F",
    "mappings": {"default": {"default": "logical and with underbar"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A60",
    "mappings": {"default": {"default": "lógica y con doble barra inferior."}}
  }, {
    "category": "Sm",
    "key": "2A61",
    "mappings": {"default": {"default": "pequeño Vee con Underbar"}}
  }, {
    "category": "Sm",
    "key": "2A62",
    "mappings": {"default": {"default": "lógica o con doble barra."}}
  }, {
    "category": "Sm",
    "key": "2A63",
    "mappings": {"default": {"default": "lógica o con doble barra inferior"}}
  }, {
    "category": "Sm",
    "key": "2A64",
    "mappings": {"default": {"default": "notación Z Dominio Antirestricción"}}
  }, {
    "category": "Sm",
    "key": "2A65",
    "mappings": {"default": {"default": "rango de notación Z Antirestricción"}}
  }, {
    "key": "2A66",
    "mappings": {"default": {"default": "igual con punto suscrito"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A67",
    "mappings": {"default": {"default": "idéntico con punto arriba"}}
  }, {
    "category": "Sm",
    "key": "2A68",
    "mappings": {"default": {"default": "triple barra horizontal con doble movimiento vertical"}}
  }, {
    "category": "Sm",
    "key": "2A69",
    "mappings": {"default": {"default": "triple barra horizontal con triple movimiento vertical"}}
  }, {"key": "2A6A", "mappings": {"default": {"default": "tilde con punto"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "2A6B",
    "mappings": {"default": {"default": "operador de tilde con puntos crecientes"}}
  }, {
    "category": "Sm",
    "key": "2A6C",
    "mappings": {"default": {"default": "similares menos similares"}}
  }, {"key": "2A6D", "mappings": {"default": {"default": "congruente con punto"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "2A6E",
    "mappings": {"default": {"default": "igual a Asterisco"}}
  }, {
    "key": "2A6F",
    "mappings": {"default": {"default": "casi igual con ángulo"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A70",
    "mappings": {"default": {"default": "aproximadamente igual o igual a"}}
  }, {"key": "2A71", "mappings": {"default": {"default": "igual con más suscrito"}}, "category": "Sm"}, {
    "key": "2A72",
    "mappings": {"default": {"default": "igual con más"}},
    "category": "Sm"
  }, {
    "key": "2A73",
    "mappings": {"default": {"default": "igual con operador tilde"}},
    "category": "Sm"
  }, {"key": "2A74", "mappings": {"default": {"default": "igual con dos puntos"}}, "category": "Sm"}, {
    "key": "2A75",
    "mappings": {"default": {"default": "igual doble"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A76",
    "mappings": {"default": {"default": "tres signos de igualdad consecutivos"}}
  }, {
    "key": "2A77",
    "mappings": {"default": {"default": "igual entre cuatro puntos"}},
    "category": "Sm"
  }, {
    "key": "2A78",
    "mappings": {"default": {"default": "equivalente con cuatro puntos"}},
    "category": "Sm"
  }, {
    "key": "2A79",
    "mappings": {"default": {"default": "menor que con círculo interior"}},
    "category": "Sm"
  }, {
    "key": "2A7A",
    "mappings": {"default": {"default": "mayor que con círculo interior"}},
    "category": "Sm"
  }, {
    "key": "2A7B",
    "mappings": {"default": {"default": "menor que con interrogante"}},
    "category": "Sm"
  }, {
    "key": "2A7C",
    "mappings": {"default": {"default": "mayor que con interrogante"}},
    "category": "Sm"
  }, {"key": "2A7D", "mappings": {"default": {"default": "estrictamente menor que"}}, "category": "Sm"}, {
    "key": "2A7E",
    "mappings": {"default": {"default": "estrictamente mayor que"}},
    "category": "Sm"
  }, {
    "key": "2A7F",
    "mappings": {"default": {"default": "estrictamente menor que con punto interior"}},
    "category": "Sm"
  }, {
    "key": "2A80",
    "mappings": {"default": {"default": "estrictamente mayor que con punto interior"}},
    "category": "Sm"
  }, {
    "key": "2A81",
    "mappings": {"default": {"default": "estrictamente menor que con punto"}},
    "category": "Sm"
  }, {
    "key": "2A82",
    "mappings": {"default": {"default": "estrictamente mayor que con punto"}},
    "category": "Sm"
  }, {
    "key": "2A83",
    "mappings": {"default": {"default": "estrictamente menor que con punto"}},
    "category": "Sm"
  }, {
    "key": "2A84",
    "mappings": {"default": {"default": "estrictamente mayor que con punto"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A85",
    "mappings": {"default": {"default": "menor que o aproximado"}}
  }, {
    "category": "Sm",
    "key": "2A86",
    "mappings": {"default": {"default": "mayor que o aproximado"}}
  }, {
    "category": "Sm",
    "key": "2A87",
    "mappings": {"default": {"default": "menor que una línea no es igual a"}}
  }, {
    "category": "Sm",
    "key": "2A88",
    "mappings": {"default": {"default": "mayores y una sola línea no son iguales a"}}
  }, {
    "key": "2A89",
    "mappings": {"default": {"default": "menor que, pero no aproximado a"}},
    "category": "Sm"
  }, {
    "key": "2A8A",
    "mappings": {"default": {"default": "mayor que, pero no aproximado a"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A8B",
    "mappings": {"default": {"default": "menos que arriba de la doble línea igual por encima de mayor que"}}
  }, {
    "category": "Sm",
    "key": "2A8C",
    "mappings": {"default": {"default": "más que arriba de la línea doble igual sobre menos que"}}
  }, {
    "key": "2A8D",
    "mappings": {"default": {"default": "menor, similar o igual a"}},
    "category": "Sm"
  }, {
    "key": "2A8E",
    "mappings": {"default": {"default": "mayor, similar o igual a"}},
    "category": "Sm"
  }, {
    "key": "2A8F",
    "mappings": {"default": {"default": "menor, similar o mayor que"}},
    "category": "Sm"
  }, {
    "key": "2A90",
    "mappings": {"default": {"default": "mayor, similar o menor que"}},
    "category": "Sm"
  }, {"key": "2A91", "mappings": {"default": {"default": "menor, mayor o igual a"}}, "category": "Sm"}, {
    "key": "2A92",
    "mappings": {"default": {"default": "mayor, menor o igual a"}},
    "category": "Sm"
  }, {
    "key": "2A93",
    "mappings": {"default": {"default": "estrictamente menor o mayor que"}},
    "category": "Sm"
  }, {
    "key": "2A94",
    "mappings": {"default": {"default": "estrictamente mayor o menor que"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A95",
    "mappings": {"default": {"default": "inclinado igual o menor que"}}
  }, {
    "category": "Sm",
    "key": "2A96",
    "mappings": {"default": {"default": "inclinado igual o mayor que"}}
  }, {
    "key": "2A97",
    "mappings": {"default": {"default": "estrictamente menor que con punto interior"}},
    "category": "Sm"
  }, {
    "key": "2A98",
    "mappings": {"default": {"default": "estrictamente mayor que con punto interior"}},
    "category": "Sm"
  }, {"key": "2A99", "mappings": {"default": {"default": "menor o igual a"}}, "category": "Sm"}, {
    "key": "2A9A",
    "mappings": {"default": {"default": "mayor o igual a"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2A9B",
    "mappings": {"default": {"default": "doble línea inclinada igual o menor que"}}
  }, {
    "category": "Sm",
    "key": "2A9C",
    "mappings": {"default": {"default": "doble línea inclinada igual o mayor que"}}
  }, {"key": "2A9D", "mappings": {"default": {"default": "equivalente o menor que"}}, "category": "Sm"}, {
    "key": "2A9E",
    "mappings": {"default": {"default": "mayor o similar a"}},
    "category": "Sm"
  }, {
    "key": "2A9F",
    "mappings": {"default": {"default": "similar, menor o igual a"}},
    "category": "Sm"
  }, {
    "key": "2AA0",
    "mappings": {"default": {"default": "similar, mayor o igual a"}},
    "category": "Sm"
  }, {"key": "2AA1", "mappings": {"default": {"default": "mucho menor que"}}, "category": "Sm"}, {
    "key": "2AA2",
    "mappings": {"default": {"default": "mucho mayor que"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2AA3",
    "mappings": {"default": {"default": "doble anidada menor que con barra inferior"}}
  }, {"key": "2AA4", "mappings": {"default": {"default": "menor que, mayor que"}}, "category": "Sm"}, {
    "key": "2AA5",
    "mappings": {"default": {"default": "mayor o menor que"}},
    "category": "Sm"
  }, {
    "key": "2AA6",
    "mappings": {"default": {"default": "menor que cerrado con curva"}},
    "category": "Sm"
  }, {
    "key": "2AA7",
    "mappings": {"default": {"default": "mayor que cerrado con curva"}},
    "category": "Sm"
  }, {
    "key": "2AA8",
    "mappings": {"default": {"default": "estrictamente menor que cerrado con curva"}},
    "category": "Sm"
  }, {
    "key": "2AA9",
    "mappings": {"default": {"default": "estrictamente mayor que cerrado con curva"}},
    "category": "Sm"
  }, {"key": "2AAA", "mappings": {"default": {"default": "más reducido que"}}, "category": "Sm"}, {
    "key": "2AAB",
    "mappings": {"default": {"default": "más amplio que"}},
    "category": "Sm"
  }, {"key": "2AAC", "mappings": {"default": {"default": "más reducido o igual a"}}, "category": "Sm"}, {
    "key": "2AAD",
    "mappings": {"default": {"default": "más amplio o igual a"}},
    "category": "Sm"
  }, {"key": "2AAE", "mappings": {"default": {"default": "diferencia entre"}}, "category": "Sm"}, {
    "key": "2AAF",
    "mappings": {"default": {"default": "precede o es igual a"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2AB0",
    "mappings": {"default": {"default": "sucede por encima de la línea de igual signo"}}
  }, {
    "category": "Sm",
    "key": "2AB1",
    "mappings": {"default": {"default": "precede por encima de una línea no es igual a"}}
  }, {
    "category": "Sm",
    "key": "2AB2",
    "mappings": {"default": {"default": "tiene éxito por encima de una línea no es igual a"}}
  }, {
    "category": "Sm",
    "key": "2AB3",
    "mappings": {"default": {"default": "precede arriba del signo igual"}}
  }, {
    "category": "Sm",
    "key": "2AB4",
    "mappings": {"default": {"default": "sucede por encima del signo igual"}}
  }, {"key": "2AB5", "mappings": {"default": {"default": "precede estrictamente a"}}, "category": "Sm"}, {
    "key": "2AB6",
    "mappings": {"default": {"default": "sigue estrictamente a"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2AB7",
    "mappings": {"default": {"default": "precede por encima de casi igual a"}}
  }, {
    "category": "Sm",
    "key": "2AB8",
    "mappings": {"default": {"default": "tiene éxito por encima de casi igual a"}}
  }, {
    "category": "Sm",
    "key": "2AB9",
    "mappings": {"default": {"default": "precede arriba no es igual a"}}
  }, {
    "category": "Sm",
    "key": "2ABA",
    "mappings": {"default": {"default": "tiene éxito por encima de no casi igual a"}}
  }, {"key": "2ABB", "mappings": {"default": {"default": "precede mucho antes"}}, "category": "Sm"}, {
    "key": "2ABC",
    "mappings": {"default": {"default": "sigue mucho después"}},
    "category": "Sm"
  }, {"key": "2ABD", "mappings": {"default": {"default": "subrelación de"}}, "category": "Sm"}, {
    "key": "2ABE",
    "mappings": {"default": {"default": "contiene como subrelación a"}},
    "category": "Sm"
  }, {
    "key": "2ABF",
    "mappings": {"default": {"default": "incluido con más suscrito"}},
    "category": "Sm"
  }, {
    "key": "2AC0",
    "mappings": {"default": {"default": "contiene con más suscrito"}},
    "category": "Sm"
  }, {
    "key": "2AC1",
    "mappings": {"default": {"default": "incluido con por suscrito"}},
    "category": "Sm"
  }, {
    "key": "2AC2",
    "mappings": {"default": {"default": "contiene con por suscrito"}},
    "category": "Sm"
  }, {
    "key": "2AC3",
    "mappings": {"default": {"default": "incluido con punto o es igual a"}},
    "category": "Sm"
  }, {
    "key": "2AC4",
    "mappings": {"default": {"default": "contiene con punto o es igual a"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2AC5",
    "mappings": {"default": {"default": "subconjunto de signo igual arriba"}}
  }, {
    "category": "Sm",
    "key": "2AC6",
    "mappings": {"default": {"default": "superconjunto de signo igual arriba"}}
  }, {
    "key": "2AC7",
    "mappings": {"default": {"default": "incluido aproximadamente en"}},
    "category": "Sm"
  }, {
    "key": "2AC8",
    "mappings": {"default": {"default": "contiene aproximadamente a"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2AC9",
    "mappings": {"default": {"default": "subconjunto de Arriba Casi Igual a"}}
  }, {
    "category": "Sm",
    "key": "2ACA",
    "mappings": {"default": {"default": "superconjunto de arriba casi igual a"}}
  }, {
    "category": "Sm",
    "key": "2ACB",
    "mappings": {"default": {"default": "subconjunto de lo anterior no es igual a"}}
  }, {
    "category": "Sm",
    "key": "2ACC",
    "mappings": {"default": {"default": "superconjunto de arriba no es igual a"}}
  }, {
    "category": "Sm",
    "key": "2ACD",
    "mappings": {"default": {"default": "operador de caja abierta a la izquierda cuadrada"}}
  }, {
    "category": "Sm",
    "key": "2ACE",
    "mappings": {"default": {"default": "operador de caja abierta a la derecha cuadrada"}}
  }, {"key": "2ACF", "mappings": {"default": {"default": "incluido con cierre en"}}, "category": "Sm"}, {
    "key": "2AD0",
    "mappings": {"default": {"default": "contiene al cierre del subconjunto"}},
    "category": "Sm"
  }, {
    "key": "2AD1",
    "mappings": {"default": {"default": "incluido el cierre o es igual a"}},
    "category": "Sm"
  }, {
    "key": "2AD2",
    "mappings": {"default": {"default": "contiene al cierre del subconjunto o es igual a"}},
    "category": "Sm"
  }, {
    "key": "2AD3",
    "mappings": {"default": {"default": "está incluido o contiene a"}},
    "category": "Sm"
  }, {
    "key": "2AD4",
    "mappings": {"default": {"default": "contiene o está incluido en"}},
    "category": "Sm"
  }, {"key": "2AD5", "mappings": {"default": {"default": "incluido doblemente en"}}, "category": "Sm"}, {
    "key": "2AD6",
    "mappings": {"default": {"default": "contiene doblemente a"}},
    "category": "Sm"
  }, {
    "key": "2AD7",
    "mappings": {"default": {"default": "contiene o está incluido en"}},
    "category": "Sm"
  }, {
    "key": "2AD8",
    "mappings": {"default": {"default": "contiene o está incluido en"}},
    "category": "Sm"
  }, {
    "key": "2AD9",
    "mappings": {"default": {"default": "perteneciente dirigido hacia abajo"}},
    "category": "Sm"
  }, {"key": "2ADA", "mappings": {"default": {"default": "pitchfork with tee top"}}, "category": "Sm"}, {
    "key": "2ADB",
    "mappings": {"default": {"default": "transversal intersection"}},
    "category": "Sm"
  }, {"category": "Sm", "key": "2ADC", "mappings": {"default": {"default": "bifurcación"}}}, {
    "category": "Sm",
    "key": "2ADD",
    "mappings": {"default": {"default": "nonforking"}}
  }, {
    "category": "Sm",
    "key": "2ADE",
    "mappings": {"default": {"default": "tachuela Izquierda Corta"}}
  }, {
    "category": "Sm",
    "key": "2ADF",
    "mappings": {"default": {"default": "tachuela corta hacia abajo"}}
  }, {"category": "Sm", "key": "2AE0", "mappings": {"default": {"default": "short Up Tack"}}}, {
    "category": "Sm",
    "key": "2AE1",
    "mappings": {"default": {"default": "perpendicular con s"}}
  }, {
    "category": "Sm",
    "key": "2AE2",
    "mappings": {"default": {"default": "barra vertical Torniquete a la derecha"}}
  }, {
    "category": "Sm",
    "key": "2AE3",
    "mappings": {"default": {"default": "barra vertical doble Torniquete izquierdo"}}
  }, {
    "key": "2AE4",
    "mappings": {"default": {"default": "double left turnstile vertical bar"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2AE5",
    "mappings": {"default": {"default": "doble barra vertical doble torniquete izquierdo"}}
  }, {
    "key": "2AE6",
    "mappings": {"default": {"default": "long dash from left member of double vertical"}},
    "category": "Sm"
  }, {
    "key": "2AE7",
    "mappings": {"default": {"default": "short down tack with overbar"}},
    "category": "Sm"
  }, {"key": "2AE8", "mappings": {"default": {"default": "barra con perpendicular"}}, "category": "Sm"}, {
    "key": "2AE9",
    "mappings": {"default": {"default": "short up tack above short down tack"}},
    "category": "Sm"
  }, {"category": "Sm", "key": "2AEA", "mappings": {"default": {"default": "tachuela doble"}}}, {
    "key": "2AEB",
    "mappings": {"default": {"default": "double up tack"}},
    "category": "Sm"
  }, {"key": "2AEC", "mappings": {"default": {"default": "double stroke not sign"}}, "category": "Sm"}, {
    "key": "2AED",
    "mappings": {"default": {"default": "reversed double stroke not sign"}},
    "category": "Sm"
  }, {
    "key": "2AEE",
    "mappings": {"default": {"default": "does not divide with reversed negation slash"}},
    "category": "Sm"
  }, {
    "key": "2AEF",
    "mappings": {"default": {"default": "barra vertical con círculo"}},
    "category": "Sm"
  }, {
    "key": "2AF0",
    "mappings": {"default": {"default": "barra con círculo inferior"}},
    "category": "Sm"
  }, {
    "key": "2AF1",
    "mappings": {"default": {"default": "down tack with circle below"}},
    "category": "Sm"
  }, {"key": "2AF2", "mappings": {"default": {"default": "paralela con marca"}}, "category": "Sm"}, {
    "key": "2AF3",
    "mappings": {"default": {"default": "paralela con tilde"}},
    "category": "Sm"
  }, {
    "category": "Sm",
    "key": "2AF4",
    "mappings": {"default": {"default": "relación binaria de barra vertical triple"}}
  }, {
    "category": "Sm",
    "key": "2AF5",
    "mappings": {"default": {"default": "barra vertical triple con trazo horizontal"}}
  }, {
    "category": "Sm",
    "key": "2AF6",
    "mappings": {"default": {"default": "operador Triple Colon"}}
  }, {
    "category": "Sm",
    "key": "2AF7",
    "mappings": {"default": {"default": "triple anidada menos que"}}
  }, {
    "category": "Sm",
    "key": "2AF8",
    "mappings": {"default": {"default": "triple anidado mayor que"}}
  }, {
    "category": "Sm",
    "key": "2AF9",
    "mappings": {"default": {"default": "doble línea inclinada menor o igual a"}}
  }, {
    "category": "Sm",
    "key": "2AFA",
    "mappings": {"default": {"default": "doble línea inclinada mayor o igual a"}}
  }, {
    "category": "Sm",
    "key": "2AFB",
    "mappings": {"default": {"default": "relación Binaria Triple Sólida"}}
  }, {
    "category": "Sm",
    "key": "2AFC",
    "mappings": {"default": {"default": "operador de barra vertical triple grande"}}
  }, {
    "category": "Sm",
    "key": "2AFD",
    "mappings": {"default": {"default": "operador Doble Sólido"}}
  }, {
    "category": "Sm",
    "key": "2AFE",
    "mappings": {"default": {"default": "barra vertical blanca"}}
  }, {
    "category": "Sm",
    "key": "2AFF",
    "mappings": {"default": {"default": "barra vertical blanca N-Ary"}}
  }, {"category": "Pd", "key": "301C", "mappings": {"default": {"default": "wave Dash"}}}, {
    "category": "Po",
    "key": "FE10",
    "mappings": {"default": {"default": "formulario de presentación para coma vertical"}}
  }, {
    "category": "Po",
    "key": "FE13",
    "mappings": {"default": {"default": "formulario de presentación para colon vertical"}}
  }, {
    "category": "Po",
    "key": "FE14",
    "mappings": {"default": {"default": "formulario de presentación para punto y coma vertical"}}
  }, {
    "category": "Po",
    "key": "FE15",
    "mappings": {"default": {"default": "formulario de presentación para el signo de exclamación vertical"}}
  }, {
    "category": "Po",
    "key": "FE16",
    "mappings": {"default": {"default": "formulario de presentación para el signo de interrogación vertical"}}
  }, {
    "category": "Po",
    "key": "FE19",
    "mappings": {"default": {"default": "formulario de presentación para elipsis horizontal vertical"}}
  }, {
    "category": "Po",
    "key": "FE30",
    "mappings": {"default": {"default": "formulario de presentación para líder vertical de dos puntos"}}
  }, {
    "category": "Pd",
    "key": "FE31",
    "mappings": {"default": {"default": "formulario de presentación para Vertical Em Dash"}}
  }, {
    "category": "Pd",
    "key": "FE32",
    "mappings": {"default": {"default": "formulario de presentación para Vertical En Dash"}}
  }, {
    "category": "Pc",
    "key": "FE33",
    "mappings": {"default": {"default": "formulario de presentación para línea vertical baja"}}
  }, {
    "category": "Pc",
    "key": "FE34",
    "mappings": {"default": {"default": "formulario de presentación para línea baja ondulada vertical"}}
  }, {"category": "Po", "key": "FE45", "mappings": {"default": {"default": "punto de sésamo"}}}, {
    "category": "Po",
    "key": "FE46",
    "mappings": {"default": {"default": "punto de sésamo blanco"}}
  }, {"category": "Po", "key": "FE49", "mappings": {"default": {"default": "línea discontinua"}}}, {
    "category": "Po",
    "key": "FE4A",
    "mappings": {"default": {"default": "centinela Overline"}}
  }, {"category": "Po", "key": "FE4B", "mappings": {"default": {"default": "ondulado Overline"}}}, {
    "category": "Po",
    "key": "FE4C",
    "mappings": {"default": {"default": "doble Ondulado Ondulado"}}
  }, {
    "category": "Pc",
    "key": "FE4D",
    "mappings": {"default": {"default": "línea baja discontinua"}}
  }, {
    "category": "Pc",
    "key": "FE4E",
    "mappings": {"default": {"default": "línea baja de línea central"}}
  }, {"category": "Pc", "key": "FE4F", "mappings": {"default": {"default": "línea baja ondulada"}}}, {
    "category": "Po",
    "key": "FE50",
    "mappings": {"default": {"default": "comas pequeñas"}}
  }, {
    "category": "Po",
    "key": "FE52",
    "mappings": {"default": {"default": "parada completa pequeña"}}
  }, {"category": "Po", "key": "FE54", "mappings": {"default": {"default": "pequeño punto y coma"}}}, {
    "category": "Po",
    "key": "FE55",
    "mappings": {"default": {"default": "pequeño colon"}}
  }, {
    "category": "Po",
    "key": "FE56",
    "mappings": {"default": {"default": "pequeño signo de interrogación"}}
  }, {
    "category": "Po",
    "key": "FE57",
    "mappings": {"default": {"default": "pequeño signo de exclamación"}}
  }, {"category": "Pd", "key": "FE58", "mappings": {"default": {"default": "pequeño Em Dash"}}}, {
    "category": "Po",
    "key": "FE5F",
    "mappings": {"default": {"default": "signo de número pequeño"}}
  }, {"category": "Po", "key": "FE60", "mappings": {"default": {"default": "ampersand pequeño"}}}, {
    "category": "Po",
    "key": "FE61",
    "mappings": {"default": {"default": "pequeño asterisco"}}
  }, {"category": "Sm", "key": "FE62", "mappings": {"default": {"default": "signo más pequeño"}}}, {
    "category": "Pd",
    "key": "FE63",
    "mappings": {"default": {"default": "pequeño guión-menos"}}
  }, {
    "category": "Sm",
    "key": "FE64",
    "mappings": {"default": {"default": "signo menor que pequeño"}}
  }, {
    "category": "Sm",
    "key": "FE65",
    "mappings": {"default": {"default": "signo mayor que pequeño"}}
  }, {"category": "Sm", "key": "FE66", "mappings": {"default": {"default": "signo pequeño igual"}}}, {
    "key": "FE68",
    "mappings": {"default": {"default": "división entera"}},
    "category": "Po"
  }, {
    "category": "Sc",
    "key": "FE69",
    "mappings": {"default": {"default": "pequeño signo de dólar"}}
  }, {
    "category": "Po",
    "key": "FE6A",
    "mappings": {"default": {"default": "signo de porcentaje pequeño"}}
  }, {"category": "Po", "key": "FE6B", "mappings": {"default": {"default": "pequeño comercial en"}}}, {
    "category": "Po",
    "key": "FF01",
    "mappings": {"default": {"default": "signo de exclamación de ancho completo"}}
  }, {
    "category": "Po",
    "key": "FF02",
    "mappings": {"default": {"default": "comillas de ancho completo"}}
  }, {
    "category": "Po",
    "key": "FF03",
    "mappings": {"default": {"default": "número de signo de ancho completo"}}
  }, {
    "category": "Sc",
    "key": "FF04",
    "mappings": {"default": {"default": "signo de dólar de ancho completo"}}
  }, {
    "category": "Po",
    "key": "FF05",
    "mappings": {"default": {"default": "signo de porcentaje de ancho completo"}}
  }, {
    "category": "Po",
    "key": "FF06",
    "mappings": {"default": {"default": "ampersand de ancho completo"}}
  }, {
    "category": "Po",
    "key": "FF07",
    "mappings": {"default": {"default": "apóstrofe de ancho completo"}}
  }, {
    "category": "Po",
    "key": "FF0A",
    "mappings": {"default": {"default": "asterisco de ancho completo"}}
  }, {
    "category": "Sm",
    "key": "FF0B",
    "mappings": {"default": {"default": "signo de ancho completo"}}
  }, {
    "category": "Po",
    "key": "FF0C",
    "mappings": {"default": {"default": "coma de ancho completo"}}
  }, {
    "category": "Pd",
    "key": "FF0D",
    "mappings": {"default": {"default": "ancho de ancho completo"}}
  }, {"category": "Po", "key": "FF0E", "mappings": {"default": {"default": "fullwidth Full Stop"}}}, {
    "category": "Po",
    "key": "FF0F",
    "mappings": {"default": {"default": "sólido de ancho completo"}}
  }, {
    "category": "Po",
    "key": "FF1A",
    "mappings": {"default": {"default": "ancho completo de colon"}}
  }, {
    "category": "Po",
    "key": "FF1B",
    "mappings": {"default": {"default": "punto y coma de ancho completo"}}
  }, {
    "category": "Sm",
    "key": "FF1C",
    "mappings": {"default": {"default": "ancho completo menos de la señal"}}
  }, {
    "category": "Sm",
    "key": "FF1D",
    "mappings": {"default": {"default": "signo de igual ancho completo"}}
  }, {
    "category": "Sm",
    "key": "FF1E",
    "mappings": {"default": {"default": "ancho completo mayor que el signo"}}
  }, {
    "category": "Po",
    "key": "FF1F",
    "mappings": {"default": {"default": "signo de interrogación de ancho completo"}}
  }, {
    "category": "Po",
    "key": "FF20",
    "mappings": {"default": {"default": "comercial de ancho completo en"}}
  }, {
    "category": "Po",
    "key": "FF3C",
    "mappings": {"default": {"default": "sólido inverso de ancho completo"}}
  }, {
    "category": "Sk",
    "key": "FF3E",
    "mappings": {"default": {"default": "acento circunflejo de ancho completo"}}
  }, {
    "category": "Pc",
    "key": "FF3F",
    "mappings": {"default": {"default": "línea baja de ancho completo"}}
  }, {
    "category": "Sk",
    "key": "FF40",
    "mappings": {"default": {"default": "acento grave de ancho completo"}}
  }, {"key": "FF5C", "mappings": {"default": {"default": "barra vertical"}}, "category": "Sm"}, {
    "category": "Sm",
    "key": "FF5E",
    "mappings": {"default": {"default": "tilde de ancho completo"}}
  }, {
    "category": "Sc",
    "key": "FFE0",
    "mappings": {"default": {"default": "signo de centavo de ancho completo"}}
  }, {
    "category": "Sc",
    "key": "FFE1",
    "mappings": {"default": {"default": "signo de libra de ancho completo"}}
  }, {"category": "Sm", "key": "FFE2", "mappings": {"default": {"default": "fullwidth no firmar"}}}, {
    "category": "Sk",
    "key": "FFE3",
    "mappings": {"default": {"default": "ancho completo macron"}}
  }, {
    "category": "So",
    "key": "FFE4",
    "mappings": {"default": {"default": "barra rota de ancho completo"}}
  }, {
    "category": "Sc",
    "key": "FFE5",
    "mappings": {"default": {"default": "signo de yen de ancho completo"}}
  }, {
    "category": "Sc",
    "key": "FFE6",
    "mappings": {"default": {"default": "signo de ancho completo ganado"}}
  }, {
    "category": "So",
    "key": "FFE8",
    "mappings": {"default": {"default": "formas de ancho medio luz vertical"}}
  }, {
    "category": "So",
    "key": "FFED",
    "mappings": {"default": {"default": "medio ancho cuadrado negro"}}
  }, {"category": "So", "key": "FFEE", "mappings": {"default": {"default": "círculo blanco de medio ancho"}}}]
    ,
    "es/symbols/math_whitespace.js"
:
  [{"locale": "es"}, {"category": "Zs", "key": "0020", "mappings": {"default": {"default": "espacio"}}}, {
    "key": "00A0",
    "mappings": {"default": {"default": " "}},
    "category": "Zs"
  }, {"key": "00AD", "mappings": {"default": {"default": "guión corto"}}, "category": "Cf"}, {
    "category": "Zs",
    "key": "2000",
    "mappings": {"default": {"default": "en quad"}}
  }, {"category": "Zs", "key": "2001", "mappings": {"default": {"default": "em Quad"}}}, {
    "key": "2002",
    "mappings": {"default": {"default": ""}},
    "category": "Zs"
  }, {"key": "2003", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
    "key": "2004",
    "mappings": {"default": {"default": ""}},
    "category": "Zs"
  }, {"key": "2005", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
    "category": "Zs",
    "key": "2006",
    "mappings": {"default": {"default": "espacio Six-Per-Em"}}
  }, {"key": "2007", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
    "key": "2008",
    "mappings": {"default": {"default": ""}},
    "category": "Zs"
  }, {"key": "2009", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
    "key": "200A",
    "mappings": {"default": {"default": ""}},
    "category": "Zs"
  }, {"key": "200B", "mappings": {"default": {"default": ""}}, "category": "Cf"}, {
    "key": "200C",
    "mappings": {"default": {"default": "ancho de cero no ensamblador"}},
    "category": "Cf"
  }, {
    "key": "200D",
    "mappings": {"default": {"default": "ancho de cero ensamblador"}},
    "category": "Cf"
  }, {
    "key": "200E",
    "mappings": {"default": {"default": "marca de izquierda a derecha"}},
    "category": "Cf"
  }, {
    "key": "200F",
    "mappings": {"default": {"default": "marca de derecha a izquierda"}},
    "category": "Cf"
  }, {"category": "Zl", "key": "2028", "mappings": {"default": {"default": "separador de linea"}}}, {
    "category": "Zp",
    "key": "2029",
    "mappings": {"default": {"default": "separador de párrafo"}}
  }, {
    "category": "Cf",
    "key": "202A",
    "mappings": {"default": {"default": "incrustación de izquierda a derecha"}}
  }, {
    "category": "Cf",
    "key": "202B",
    "mappings": {"default": {"default": "incrustación de derecha a izquierda"}}
  }, {
    "category": "Cf",
    "key": "202C",
    "mappings": {"default": {"default": "formato direccional pop"}}
  }, {
    "category": "Cf",
    "key": "202D",
    "mappings": {"default": {"default": "anulación de izquierda a derecha"}}
  }, {
    "category": "Cf",
    "key": "202E",
    "mappings": {"default": {"default": "anulación de derecha a izquierda"}}
  }, {
    "category": "Zs",
    "key": "202F",
    "mappings": {"default": {"default": "espacio estrecho sin descanso"}}
  }, {"key": "205F", "mappings": {"default": {"default": ""}}, "category": "Zs"}, {
    "key": "2060",
    "mappings": {"default": {"default": ""}},
    "category": "Cf"
  }, {"key": "2061", "mappings": {"default": {"default": "de"}}, "category": "Cf"}, {
    "key": "2062",
    "mappings": {"default": {"default": "por"}},
    "category": "Cf"
  }, {"key": "2063", "mappings": {"default": {"default": "coma"}}, "category": "Cf"}, {
    "key": "2064",
    "mappings": {"default": {"default": "más"}},
    "category": "Cf"
  }, {
    "category": "Cf",
    "key": "206A",
    "mappings": {"default": {"default": "inhibir el intercambio simétrico"}}
  }, {
    "category": "Cf",
    "key": "206B",
    "mappings": {"default": {"default": "activar el intercambio simétrico"}}
  }, {
    "category": "Cf",
    "key": "206E",
    "mappings": {"default": {"default": "formas de dígitos nacionales"}}
  }, {
    "category": "Cf",
    "key": "206F",
    "mappings": {"default": {"default": "formas de dígitos nominales"}}
  }, {"key": "FEFF", "mappings": {"default": {"default": ""}}, "category": "Cf"}, {
    "category": "Cf",
    "key": "FFF9",
    "mappings": {"default": {"default": "ancla de anotación interlinear"}}
  }, {
    "category": "Cf",
    "key": "FFFA",
    "mappings": {"default": {"default": "separador de anotación interlinear"}}
  }, {"category": "Cf", "key": "FFFB", "mappings": {"default": {"default": "terminador de anotación interlinear"}}}]
    ,
    "es/symbols/other_stars.js"
:
  [{"locale": "es"}, {
    "category": "So",
    "key": "23E8",
    "mappings": {"default": {"default": "símbolo exponente decimal"}}
  }, {"key": "2605", "mappings": {"default": {"default": "estrella rellena"}}, "category": "So"}, {
    "key": "2606",
    "mappings": {"default": {"default": "estrella vacía"}},
    "category": "So"
  }, {"category": "So", "key": "26AA", "mappings": {"default": {"default": "círculo blanco medio"}}}, {
    "category": "So",
    "key": "26AB",
    "mappings": {"default": {"default": "círculo negro medio"}}
  }, {
    "category": "So",
    "key": "2705",
    "mappings": {"default": {"default": "marca de verificación pesada blanca"}}
  }, {"key": "2713", "mappings": {"default": {"default": "check mark"}}, "category": "So"}, {
    "category": "So",
    "key": "2714",
    "mappings": {"default": {"default": "marca de verificación pesada"}}
  }, {"category": "So", "key": "2715", "mappings": {"default": {"default": "multiplicación x"}}}, {
    "category": "So",
    "key": "2716",
    "mappings": {"default": {"default": "multiplicación pesada X"}}
  }, {"key": "2717", "mappings": {"default": {"default": "ballot x"}}, "category": "So"}, {
    "category": "So",
    "key": "2718",
    "mappings": {"default": {"default": "boleta pesada X"}}
  }, {
    "category": "So",
    "key": "271B",
    "mappings": {"default": {"default": "centro abierto de la cruz"}}
  }, {
    "category": "So",
    "key": "271C",
    "mappings": {"default": {"default": "pesado centro abierto cruz"}}
  }, {"key": "2720", "mappings": {"default": {"default": "cruz de Malta"}}, "category": "So"}, {
    "category": "So",
    "key": "2721",
    "mappings": {"default": {"default": "estrella de David"}}
  }, {
    "category": "So",
    "key": "2722",
    "mappings": {"default": {"default": "cuatro asterisco de lágrima"}}
  }, {
    "category": "So",
    "key": "2723",
    "mappings": {"default": {"default": "cuatro asterisco de globo"}}
  }, {
    "category": "So",
    "key": "2724",
    "mappings": {"default": {"default": "pesado Cuatro asterisco impulsado por globo"}}
  }, {
    "category": "So",
    "key": "2725",
    "mappings": {"default": {"default": "cuatro asterisco de club-spoked"}}
  }, {
    "category": "So",
    "key": "2726",
    "mappings": {"default": {"default": "estrella de cuatro puntas negra"}}
  }, {
    "category": "So",
    "key": "2727",
    "mappings": {"default": {"default": "estrella blanca de cuatro puntas"}}
  }, {"category": "So", "key": "2728", "mappings": {"default": {"default": "destellos"}}}, {
    "category": "So",
    "key": "2729",
    "mappings": {"default": {"default": "estrés subrayado estrella blanca"}}
  }, {
    "category": "So",
    "key": "272A",
    "mappings": {"default": {"default": "estrella blanca circulada"}}
  }, {
    "category": "So",
    "key": "272B",
    "mappings": {"default": {"default": "centro Abierto Estrella Negra"}}
  }, {
    "category": "So",
    "key": "272C",
    "mappings": {"default": {"default": "centro negro estrella blanca"}}
  }, {
    "category": "So",
    "key": "272D",
    "mappings": {"default": {"default": "contorneado negro estrella"}}
  }, {
    "category": "So",
    "key": "272E",
    "mappings": {"default": {"default": "estrella negra contorneada pesada"}}
  }, {"category": "So", "key": "272F", "mappings": {"default": {"default": "pinwheel Star"}}}, {
    "category": "So",
    "key": "2730",
    "mappings": {"default": {"default": "estrella blanca sombreada"}}
  }, {"category": "So", "key": "2731", "mappings": {"default": {"default": "asterisco pesado"}}}, {
    "category": "So",
    "key": "2732",
    "mappings": {"default": {"default": "centro abierto de asterisco"}}
  }, {
    "category": "So",
    "key": "2733",
    "mappings": {"default": {"default": "ocho asterisco hablado"}}
  }, {
    "category": "So",
    "key": "2734",
    "mappings": {"default": {"default": "ocho estrellas negras puntiagudas"}}
  }, {
    "category": "So",
    "key": "2735",
    "mappings": {"default": {"default": "ocho estrellas de molinete en punta"}}
  }, {
    "key": "2736",
    "mappings": {"default": {"default": "estrella rellena de seis puntas"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "2739",
    "mappings": {"default": {"default": "doce estrellas negras puntiagudas"}}
  }, {
    "category": "So",
    "key": "273A",
    "mappings": {"default": {"default": "dieciséis asteriscos en punta"}}
  }, {
    "category": "So",
    "key": "273B",
    "mappings": {"default": {"default": "asterisco con efecto de lágrima"}}
  }, {
    "category": "So",
    "key": "273C",
    "mappings": {"default": {"default": "centro abierto con asterisco con forma de lágrima"}}
  }, {
    "key": "273D",
    "mappings": {"default": {"default": "heavy teardrop spoked asterisk"}},
    "category": "So"
  }, {
    "category": "So",
    "key": "273E",
    "mappings": {"default": {"default": "seis pétalos de florette en blanco y negro"}}
  }, {"category": "So", "key": "273F", "mappings": {"default": {"default": "florette negro"}}}, {
    "category": "So",
    "key": "2740",
    "mappings": {"default": {"default": "florette blanco"}}
  }, {
    "category": "So",
    "key": "2741",
    "mappings": {"default": {"default": "ocho Petalled Outlined Black Florette"}}
  }, {
    "category": "So",
    "key": "2742",
    "mappings": {"default": {"default": "círculo abierto centro ocho estrellas señaladas"}}
  }, {
    "category": "So",
    "key": "2743",
    "mappings": {"default": {"default": "asterisco pesado del molinillo de viento"}}
  }, {"category": "So", "key": "2744", "mappings": {"default": {"default": "copo de nieve"}}}, {
    "category": "So",
    "key": "2745",
    "mappings": {"default": {"default": "copo de nieve trifolio apretado"}}
  }, {
    "category": "So",
    "key": "2746",
    "mappings": {"default": {"default": "copo de nieve pesado de Chevron"}}
  }, {"category": "So", "key": "2747", "mappings": {"default": {"default": "brillar"}}}, {
    "category": "So",
    "key": "2748",
    "mappings": {"default": {"default": "chispa pesada"}}
  }, {"category": "So", "key": "2749", "mappings": {"default": {"default": "asterisco de globo"}}}, {
    "category": "So",
    "key": "274A",
    "mappings": {"default": {"default": "ocho asterisco de hélice con efecto de lágrima"}}
  }, {
    "category": "So",
    "key": "274B",
    "mappings": {"default": {"default": "heavy eight asterisk de hélice en forma de lágrima"}}
  }, {"category": "So", "key": "274C", "mappings": {"default": {"default": "marca de la cruz"}}}, {
    "category": "So",
    "key": "274D",
    "mappings": {"default": {"default": "círculo blanco sombreado"}}
  }]
    ,
    "es/units/energy.js"
:
  [{"locale": "es"}, {
    "key": "W",
    "names": ["W", "w"],
    "mappings": {"default": {"default": "watt"}},
    "category": "energy"
  }, {
    "key": "kW",
    "names": ["kw", "kW"],
    "mappings": {"default": {"default": "kilowatt"}},
    "category": "energy"
  }, {
    "key": "mW",
    "names": ["mw", "mW"],
    "mappings": {"default": {"default": "miliwatt"}},
    "category": "energy"
  }, {
    "key": "kwh",
    "names": ["kwh", "kWh"],
    "mappings": {"default": {"default": "kilowatt hora"}},
    "category": "energy"
  }, {"key": "J", "names": ["J"], "mappings": {"default": {"default": "Joule"}}, "category": "energy"}, {
    "key": "N",
    "names": ["N"],
    "mappings": {"default": {"default": "Newton"}},
    "category": "energy"
  }, {"key": "A", "names": ["A"], "mappings": {"default": {"default": "Ampere"}}, "category": "energy"}, {
    "key": "V",
    "names": ["V"],
    "mappings": {"default": {"default": "Voltio"}},
    "category": "energy"
  }, {
    "key": "µΩ",
    "names": ["µΩ"],
    "mappings": {"default": {"default": "microohmio"}},
    "category": "energy"
  }, {
    "key": "mΩ",
    "names": ["mΩ"],
    "mappings": {"default": {"default": "miliohmio"}},
    "category": "energy"
  }, {
    "key": "Ω",
    "names": ["Ω", "Ohm"],
    "mappings": {"default": {"default": "ohmio"}},
    "category": "energy"
  }, {
    "key": "kΩ",
    "names": ["kΩ", "KΩ"],
    "mappings": {"default": {"default": "kiloohmio"}},
    "category": "energy"
  }, {
    "key": "MΩ",
    "names": ["MΩ"],
    "mappings": {"default": {"default": "megaohmio"}},
    "category": "energy"
  }, {"key": "GΩ", "names": ["GΩ"], "mappings": {"default": {"default": "gigaohmio"}}, "category": "energy"}]
    ,
    "es/units/length.js"
:
  [{"locale": "es"}, {
    "key": "mm",
    "names": ["mm"],
    "mappings": {"default": {"default": "milímetro"}},
    "category": "length"
  }, {
    "key": "cm",
    "names": ["cm"],
    "mappings": {"default": {"default": "centímetro"}},
    "category": "length"
  }, {"key": "m", "names": ["m"], "mappings": {"default": {"default": "metro"}}, "category": "length"}, {
    "key": "km",
    "names": ["km"],
    "mappings": {"default": {"default": "kilómetro"}},
    "category": "length"
  }, {
    "key": "ft",
    "names": ["ft", "ft."],
    "mappings": {"default": {"default": "pies"}},
    "category": "length"
  }, {
    "key": "in",
    "names": ["in", "in."],
    "mappings": {"default": {"default": "pulgada"}},
    "category": "length"
  }, {
    "key": "mi",
    "names": ["mi", "mi."],
    "mappings": {"default": {"default": "milla"}},
    "category": "length"
  }, {
    "key": "yd",
    "names": ["yd", "yd."],
    "mappings": {"default": {"default": "yarda"}},
    "category": "length"
  }, {"key": "n.m.", "names": ["n.m."], "mappings": {"default": {"default": "millas náuticas"}}, "category": "length"}]
    ,
    "es/units/memory.js"
:
  [{"locale": "es"}, {
    "key": "b",
    "names": ["b"],
    "mappings": {"default": {"default": "bit"}},
    "category": "memory"
  }, {"key": "B", "names": ["B"], "mappings": {"default": {"default": "byte"}}, "category": "memory"}, {
    "key": "KB",
    "names": ["KB"],
    "mappings": {"default": {"default": "kilobyte"}},
    "category": "memory"
  }, {
    "key": "MB",
    "names": ["MB"],
    "mappings": {"default": {"default": "megabyte"}},
    "category": "memory"
  }, {
    "key": "GB",
    "names": ["GB"],
    "mappings": {"default": {"default": "gigabyte"}},
    "category": "memory"
  }, {"key": "TB", "names": ["TB"], "mappings": {"default": {"default": "terabyte"}}, "category": "memory"}]
    ,
    "es/units/other.js"
:
  [{"locale": "es"}, {
    "key": "doz",
    "names": ["doz", "doz.", "dz", "dz."],
    "mappings": {"default": {"default": "docena"}},
    "category": "other"
  }, {
    "key": "sq",
    "names": ["sq", "sq."],
    "mappings": {"default": {"default": "al cuadrado"}},
    "category": "other"
  }, {"key": "ha", "names": ["ha"], "mappings": {"default": {"default": "hectarea"}}, "category": "other"}]
    ,
    "es/units/speed.js"
:
  [{"locale": "es"}, {
    "key": "kt",
    "names": ["kt", "kt."],
    "mappings": {"default": {"default": "nudo"}},
    "category": "speed"
  }, {
    "key": "mph",
    "names": ["mph"],
    "mappings": {"default": {"default": "millas por hora"}},
    "category": "speed"
  }, {
    "key": "rpm",
    "names": ["rpm"],
    "mappings": {"default": {"default": "revoluciones por minuto"}},
    "category": "speed"
  }, {"key": "kmh", "names": ["kmh"], "mappings": {"default": {"default": "kilómetros por hora"}}, "category": "speed"}]
    ,
    "es/units/temperature.js"
:
  [{"locale": "es"}, {
    "key": "F",
    "names": ["F", "F.", "°F"],
    "mappings": {"default": {"default": "Fahrenheit"}},
    "category": "temperature"
  }, {
    "key": "C",
    "names": ["C", "°C"],
    "mappings": {"default": {"default": "centígrados"}},
    "category": "temperature"
  }, {"key": "K", "names": ["K", "°K"], "mappings": {"default": {"default": "Kelvin"}}, "category": "temperature"}]
    ,
    "es/units/time.js"
:
  [{"locale": "es"}, {
    "key": "ns",
    "names": ["ns"],
    "mappings": {"default": {"default": "nanosegundos"}},
    "category": "time"
  }, {
    "key": "µs",
    "names": ["µs"],
    "mappings": {"default": {"default": "microsegundos"}},
    "category": "time"
  }, {
    "key": "ms",
    "names": ["ms"],
    "mappings": {"default": {"default": "milisegundos"}},
    "category": "time"
  }, {"key": "s", "names": ["s"], "mappings": {"default": {"default": "segundos"}}, "category": "time"}, {
    "key": "min",
    "names": ["min"],
    "mappings": {"default": {"default": "minutos"}},
    "category": "time"
  }, {"key": "h", "names": ["h", "hr"], "mappings": {"default": {"default": "hora"}}, "category": "time"}]
    ,
    "es/units/volume.js"
:
  [{"locale": "es"}, {
    "key": "bbl",
    "names": ["bbl.", "bbl"],
    "mappings": {"default": {"default": "barrile"}},
    "category": "volume"
  }, {
    "key": "cu",
    "names": ["cu", "cu."],
    "mappings": {"default": {"default": "cúbico"}},
    "category": "volume"
  }, {
    "key": "fl. oz.",
    "names": ["fl. oz."],
    "mappings": {"default": {"default": "onzas líquidas"}},
    "category": "volume"
  }, {
    "key": "gal",
    "names": ["gal", "gal."],
    "mappings": {"default": {"default": "galone"}},
    "category": "volume"
  }, {
    "key": "pt",
    "names": ["pt", "pt."],
    "mappings": {"default": {"default": "pinta"}},
    "category": "volume"
  }, {
    "key": "qt",
    "names": ["qt", "qt."],
    "mappings": {"default": {"default": "quart"}},
    "category": "volume"
  }, {
    "key": "tbsp",
    "names": ["tbsp", "tbsp."],
    "mappings": {"default": {"default": "cuchara"}},
    "category": "volume"
  }, {
    "key": "tsp",
    "names": ["tsp", "tsp."],
    "mappings": {"default": {"default": "cucharilla"}},
    "category": "volume"
  }, {
    "key": "cc",
    "names": ["cc"],
    "mappings": {"default": {"default": "centímetros cúbicos"}},
    "category": "volume"
  }, {
    "key": "kl",
    "names": ["kl"],
    "mappings": {"default": {"default": "kilolitro"}},
    "category": "volume"
  }, {"key": "l", "names": ["l"], "mappings": {"default": {"default": "litro"}}, "category": "volume"}, {
    "key": "ml",
    "names": ["ml"],
    "mappings": {"default": {"default": "mililitro"}},
    "category": "volume"
  }]
    ,
    "es/units/weight.js"
:
  [{"locale": "es"}, {
    "key": "lb",
    "names": ["lb", "lb."],
    "mappings": {"default": {"default": "libra"}},
    "category": "weight"
  }, {
    "key": "LT",
    "names": ["LT", "L.T."],
    "mappings": {"default": {"default": "libras imperiales"}},
    "category": "weight"
  }, {
    "key": "oz",
    "names": ["oz", "oz."],
    "mappings": {"default": {"default": "onza"}},
    "category": "weight"
  }, {
    "key": "µg",
    "names": ["µg", "mcg"],
    "mappings": {"default": {"default": "microgramo"}},
    "category": "weight"
  }, {
    "key": "gr",
    "names": ["g", "gr"],
    "mappings": {"default": {"default": "gramo"}},
    "category": "weight"
  }, {
    "key": "kg",
    "names": ["kg"],
    "mappings": {"default": {"default": "kilogramo"}},
    "category": "weight"
  }, {
    "key": "mg",
    "names": ["mg"],
    "mappings": {"default": {"default": "miligramo"}},
    "category": "weight"
  }, {"key": "t", "names": ["t", "T"], "mappings": {"default": {"default": "tonelada"}}, "category": "weight"}]
}

